(*:Date: Thu May 28 00:11:52 MET DST 1998
*)

(*:Title: GMCUR.m *)

(*:Author: Alfred Gray *)

(* Copyright 1995-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the  Gaussian curvature, mean curvature, principal curvatures, unit normals and surface normals of many parametrized surfaces.*)


BeginPackage["GMCUR`","SURFS`","CSPROGS`","NumericalMath`BesselZeros`"]

    gcurvature::usage="{u,v}->gcurvature[x][u,v] is the Gaussian curvature of the patch x:U->R^3."

    mcurvature::usage="{u,v}->mcurvature[x][u,v] is the mean curvature of the patch x:U->R^3."

    k1::usage="{u,v}->k1[x][u,v] is a principal curvature of a patch x (k1 is greater than or equal to k2)."

    k2::usage="{u,v}->k2[x][u,v] is a principal curvature of a patch x (k1 is greater than or equal to k2)."

    unitnormal::usage="{u,v}->unitnormal[x][u,v] is the unit normal to the patch x:U->R^3. It is the Gauss map of x."

    surfacenormal::usage="{u,v}->surfacenormal[x][u,v] is the cross product of the tangent vectors xu and xv of the patch x:U->R^3."

Begin["`Private`"]

(*astell *)

gcurvature[astell[a_,a_,a_]][u_,v_]:=
Sec[v]^4/(9*a^6*(-1 + Cos[v]^2 - Cos[u]^2*Cos[v]^2 +
       Cos[u]^4*Cos[v]^2)^2)

mcurvature[astell[a_,a_,a_]][u_,v_]:=
(Sqrt[2]*Cos[v]^9*(66 - 2*Cos[4*u] - 4*Cos[4*u - 2*v] - 
      3*Cos[4*(u - v)] - 56*Cos[2*v] + 6*Cos[4*v] - 
      3*Cos[4*(u + v)] - 4*Cos[4*u + 2*v])*Sin[2*u]^2*Sin[v]^2)/
  (3*a^3*(Cos[v]^8*(18 - 2*Cos[4*u] - Cos[4*u - 2*v] - 
         14*Cos[2*v] - Cos[4*u + 2*v])*Sin[2*u]^2*
       Sin[v]^2)^(3/2))

unitnormal[astell[a_,a_,a_]][u_,v_]:=
{4*Sqrt[2]*Sin[u]*Sin[v]/
   Sqrt[18 - 2*Cos[4*u] - Cos[4*u - 2*v] - 14*Cos[2*v] - Cos[4*u + 2*v]],
  4*Sqrt[2]*Cos[u]*Sin[v]/
   Sqrt[18 - 2*Cos[4*u] - Cos[4*u - 2*v] - 14*Cos[2*v] - Cos[4*u + 2*v]],
  2*Sqrt[2]*Cos[v]*Sin[2*u]/
   Sqrt[18 - 2*Cos[4*u] - Cos[4*u - 2*v] - 14*Cos[2*v] - Cos[4*u + 2*v]]}

surfacenormal[astell[a_,a_,a_]][u_,v_]:=
{9*a^6*Cos[u]*Cos[v]^4*Sin[u]^2*Sin[v]^2,
 9*a^6*Cos[u]^2*Cos[v]^4*Sin[u]*Sin[v]^2,
 9*a^6*Cos[v]^5*Sin[2*u]^2*Sin[v]/4}

(*bohdom *)

gcurvature[bohdom[a_,b_,c_]][u_,v_]:=
b*c^2*Cos[v]*Sin[u]/
  (a*(-c^2*Cos[v]^2 - b^2*Sin[v]^2 + b^2*Cos[u]^2*Sin[v]^2)^2)

mcurvature[bohdom[a_,b_,c_]][u_,v_]:=
(-a^3*b*c*Sin[u] - a^2*c*Cos[v]*(c^2*Cos[v]^2 + b^2*Sin[v]^2))/
  (2*(-a^2*b^2*Cos[u]^2*Sin[v]^2 + a^2*(c^2*Cos[v]^2 + b^2*Sin[v]^2))^(3/2))

unitnormal[bohdom[a_,b_,c_]][u_,v_]:=
{a*c*Cos[u]*Cos[v]/
   Sqrt[a^2*c^2*Cos[u]^2*Cos[v]^2 + a^2*c^2*Cos[v]^2*Sin[u]^2 +
        a^2*b^2*Sin[u]^2*Sin[v]^2],
 a*c*Cos[v]*Sin[u]/
   Sqrt[a^2*c^2*Cos[u]^2*Cos[v]^2 + a^2*c^2*Cos[v]^2*Sin[u]^2 +
        a^2*b^2*Sin[u]^2*Sin[v]^2],
  a*b*Sin[u]*Sin[v]/
   Sqrt[a^2*c^2*Cos[u]^2*Cos[v]^2 + a^2*c^2*Cos[v]^2*Sin[u]^2 +
        a^2*b^2*Sin[u]^2*Sin[v]^2]}

surfacenormal[bohdom[a_,b_,c_]][u_,v_]:=
{a*c*Cos[u]*Cos[v],a*c*Cos[v]*Sin[u],a*b*Sin[u]*Sin[v]}


(*bourpolar *)

gcurvature[bourpolar[m_][t_]][r_,theta_]:=
-4*r^(4 - 2*m)/(1 + r^2)^4

gcurvature[bourpolar[m_,n_][t_]][r_,theta_]:=
-4*n^2*r^(2*(1 - m + n))/(1 + r^(2*n))^4

mcurvature[bourpolar[m_][t_]][r_,theta_]:=
0

mcurvature[bourpolar[m_,n_][t_]][r_,theta_]:=
0

unitnormal[bourpolar[m_][t_]][r_,theta_]:=
{2*r*Cos[theta]/(1 + r^2),2*r*Sin[theta]/(1 + r^2),
(-1 + r^2)/(1 + r^2)}

unitnormal[bourpolar[m_,n_][t_]][r_,theta_]:=
{2*r^n*Cos[n*theta]/(1 + r^(2*n)),
2*r^n*Sin[n*theta]/(1 + r^(2*n)),
 (-1 + r^(2*n))/(1 + r^(2*n))}

surfacenormal[bourpolar[m_][t_]][r_,theta_]:=
{2*r^(-2 + 2*m)*(1 + r^2)*Cos[theta],
2*r^(-2 + 2*m)*(1 + r^2)*Sin[theta],
   r^(-3 + 2*m)*(-1 + r^4)}

surfacenormal[bourpolar[m_,n_][t_]][r_,theta_]:=
{2*r^(-3 + 2*m + n)*(1 + r^(2*n))*Cos[n*theta], 
  2*r^(-3 + 2*m + n)*(1 + r^(2*n))*Sin[n*theta], 
  r^(-3 + 2*m)*(-1 + r^(4*n))}

(*catalan *)

gcurvature[catalan[a_]][u_,v_]:=
Sech[v/2]^4/(8*a^2*(Cos[u] - Cosh[v]))

mcurvature[catalan[a_]][u_,v_]:=
0

unitnormal[catalan[a_]][u_,v_]:=
{Cos[u/2]*Sech[v/2],-(Sech[v/2]*Sin[u/2]),-Tanh[v/2]}

surfacenormal[catalan[a_]][u_,v_]:=
{a^2*Cos[(3*u)/2]*Cosh[v/2] - 
    a^2*Cos[u/2]*Cosh[(3*v)/2],
   2*a^2*Cosh[v/2]*(-Cos[u] + Cosh[v])*Sin[u/2],
   a^2*(-Cos[u] + Cosh[v])*Sinh[v]}


(*catenoid *)

gcurvature[catenoid[a_]][u_,v_]:=
-Sech[v/a]^4/a^2

gcurvature[catenoidasym[a_]][p_,q_]:=
-(Sech[(p - q)/2]^4/a^2)

gcurvature[catenoidiso[a_]][u_,v_]:=
-Sech[v/a]^4/a^2

mcurvature[catenoid[a_]][u_,v_]:=
0

mcurvature[catenoidasym[a_]][p_,q_]:=
0

mcurvature[catenoidiso[a_]][p_,q_]:=
0

k1[catenoid[a_]][u_,v_]:=
Sech[v/a]^2/a

k2[catenoid[a_]][u_,v_]:=
-Sech[v/a]^2/a

unitnormal[catenoid[a_]][u_,v_]:=
{Cos[u]*Sech[v/a],Sech[v/a]*Sin[u],-Tanh[v/a]}

unitnormal[catenoidasym[a_]][p_,q_]:=
{Cos[(p + q)/2]*Sech[(p - q)/2],
 Sech[(p - q)/2]*Sin[(p + q)/2],Tanh[(p - q)/2]}

unitnormal[catenoidiso[a_]][u_,v_]:=
{Cos[u/a]*Sech[v/a],Sech[v/a]*Sin[u/a],-Tanh[v/a]}

surfacenormal[catenoid[a_]][u_,v_]:=
{a*Cos[u]*Cosh[v/a],a*Cosh[v/a]*Sin[u],
   -(a*Sinh[2*v/a])/2}

(*circularcone *)

gcurvature[circularcone[a_,b_]][u_,v_]:=
0

mcurvature[circularcone[a_,b_]][u_,v_]:=
-b/(2*a*Sqrt[a^2 + b^2]*v)

k1[circularcone[a_,b_]][u_,v_]:=
0

k2[circularcone[a_,b_]][u_,v_]:=
-b/(2*a*Sqrt[a^2 + b^2]*v)

unitnormal[circularcone[a_,b_]][u_,v_]:=
{(b*Cos[u])/Sqrt[a^2 + b^2],
  (b*Sin[u])/Sqrt[a^2 + b^2],-(a/Sqrt[a^2 + b^2])}

surfacenormal[circularcone[a_,b_]][u_,v_]:=
{a*b*(v^2)^(1/2)*Cos[u],a*b*(v^2)^(1/2)*Sin[u],
   -(a^2*v)}


(*circularmembrane *)

gcurvature[circularmembrane[m_,n_,t_]][r_,theta_]:=
(2*Cos[t*BesselJZeros[m,{n,n}]]^2*
    (r^2*(BesselJ[-2 + m,r*BesselJZeros[m,{n,n}]] - 
         2*BesselJ[m,r*BesselJZeros[m,{n,n}]] + 
         BesselJ[2 + m,r*BesselJZeros[m,{n,n}]])*
       BesselJZeros[m,{n,n}]^2*
       (-2*m^2*BesselJ[m,r*BesselJZeros[m,{n,n}]] + 
         r*(BesselJ[-1 + m,r*BesselJZeros[m,{n,n}]] - 
            BesselJ[1 + m,r*BesselJZeros[m,{n,n}]])*
          BesselJZeros[m,{n,n}])*Cos[m*theta]^2 - 
      2*m^2*(2*BesselJ[m,r*BesselJZeros[m,{n,n}]] + 
          r*(-BesselJ[-1 + m,r*BesselJZeros[m,{n,n}]] + 
             BesselJ[1 + m,r*BesselJZeros[m,{n,n}]])*
           BesselJZeros[m,{n,n}])^2*Sin[m*theta]^2))/
  (4*r^2 + Cos[t*BesselJZeros[m,{n,n}]]^2*
      (r^2*(BesselJ[-1 + m,r*BesselJZeros[m,{n,n}]] - 
            BesselJ[1 + m,r*BesselJZeros[m,{n,n}]])^2*
         BesselJZeros[m,{n,n}]^2*Cos[m*theta]^2 + 
        4*m^2*BesselJ[m,r*BesselJZeros[m,{n,n}]]^2*
        Sin[mm*theta]^2))^2

mcurvature[circularmembrane[m_,n_,t_]][r_,theta_]:=
(Cos[m*theta]*Cos[t*BesselJZeros[m, {n, n}]]*
    (-8*m^2*r*BesselJ[m, r*BesselJZeros[m, {n, n}]] + 
      r^2*(BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
        BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^3*
       BesselJZeros[m, {n, n}]^3*Cos[m*theta]^2*
       Cos[t*BesselJZeros[m, {n, n}]]^2 + 
      4*(BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
         BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])*
       BesselJZeros[m, {n, n}]*
 (r^2 + 2*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
       Cos[t*BesselJZeros[m, {n, n}]]^2*
          Sin[m*theta]^2) + 
      2*r*BesselJZeros[m, {n, n}]^2*
     (-2*r^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
          Cos[theta]^2 + 
       r^2*BesselJ[2 + m, r*BesselJZeros[m, {n, n}]]*
          Cos[theta]^2 - 
     m^2*BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]]^2*
          BesselJ[m, r*BesselJZeros[m, {n, n}]]*
      Cos[m*theta]^2*Cos[t*BesselJZeros[m, {n, n}]]^2
          + 2*m^2*BesselJ[-1 + m, 
           r*BesselJZeros[m, {n, n}]]*
          BesselJ[m, r*BesselJZeros[m, {n, n}]]*
          BesselJ[1 + m, r*BesselJZeros[m, {n, n}]]*
       Cos[m*theta]^2*Cos[t*BesselJZeros[m, {n, n}]]^2
          - m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
          BesselJ[1 + m, r*BesselJZeros[m, {n, n}]]^2*
       Cos[m*theta]^2*Cos[t*BesselJZeros[m, {n, n}]]^2
        - 2*r^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
          Sin[theta]^2 + 
       r^2*BesselJ[2 + m, r*BesselJZeros[m, {n, n}]]*
          Sin[theta]^2 - 
   2*m^2*BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]]^2*
         BesselJ[m, r*BesselJZeros[m, {n, n}]]*
      Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2
      - 2*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^3*
      Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2
          + 4*m^2*BesselJ[-1 + m, 
           r*BesselJZeros[m, {n, n}]]*
          BesselJ[m, r*BesselJZeros[m, {n, n}]]*
       BesselJ[1 + m, r*BesselJZeros[m, {n, n}]]*
       Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2
       - 2*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
       BesselJ[1 + m, r*BesselJZeros[m, {n, n}]]^2*
       Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2
       + m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
       BesselJ[2 + m, r*BesselJZeros[m, {n, n}]]*
       Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2
       + BesselJ[-2 + m, r*BesselJZeros[m, {n, n}]]*
       (r^2*Cos[theta]^2 + r^2*Sin[theta]^2 + 
       m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
             Cos[t*BesselJZeros[m, {n, n}]]^2*
             Sin[m*theta]^2))))/
  (4*(r^2*(BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
      BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^2*
    BesselJZeros[m, {n, n}]^2*Cos[m*theta]^2*
       Cos[t*BesselJZeros[m, {n, n}]]^2 + 
      4*(r^2*Cos[theta]^2 + r^2*Sin[theta]^2 + 
     m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
    Cos[t*BesselJZeros[m, {n, n}]]^2*Sin[m*theta]^2))*
Sqrt[((4 + (BesselJ[-1 + m,r*BesselJZeros[m, {n, n}]] - 
BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^2*
      BesselJZeros[m, {n, n}]^2*Cos[m*theta]^2*
            Cos[t*BesselJZeros[m, {n, n}]]^2)*
         (r^2*Cos[theta]^2 + r^2*Sin[theta]^2 + 
    m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
            Cos[t*BesselJZeros[m, {n, n}]]^2*
            Sin[m*theta]^2))/4 - 
      (m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
    (BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
       BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^2*
         BesselJZeros[m, {n, n}]^2*
Cos[t*BesselJZeros[m, {n, n}]]^4*Sin[2*m*theta]^2)/16])

surfacenormal[circularmembrane[m_,n_,t_]][r_,theta_]:=
{-(Cos[t*BesselJZeros[m,{n,n}]]*
      (r*(BesselJ[-1 + m,r*BesselJZeros[m,{n,n}]] - 
           BesselJ[1 + m,r*BesselJZeros[m,{n,n}]])*
         BesselJZeros[m,{n,n}]*Cos[theta]*Cos[m*theta]
         + 2*m*BesselJ[m,r*BesselJZeros[m,{n,n}]]*
         Sin[theta]*Sin[m*theta]))/2,
  (Cos[t*BesselJZeros[m,{n,n}]]*
     (r*(-BesselJ[-1 + m,r*BesselJZeros[m,{n,n}]] + 
          BesselJ[1 + m,r*BesselJZeros[m,{n,n}]])*
        BesselJZeros[m,{n,n}]*Cos[m*theta]*Sin[theta] + 
       2*m*BesselJ[m,r*BesselJZeros[m,{n,n}]]*
        Cos[theta]*Sin[m*theta]))/2,r}

unitnormal[circularmembrane[m_,n_,t_]][r_,theta_]:=
{-(Cos[t*BesselJZeros[m, {n, n}]]*
      (r*(BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
           BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])*
    BesselJZeros[m, {n, n}]*Cos[theta]*Cos[m*theta]
         + 2*m*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
         Sin[theta]*Sin[m*theta]))/
   (2*Sqrt[r^2 + (Cos[t*BesselJZeros[m, {n, n}]]^2*
          (r^2*(BesselJ[-1 + m, 
                 r*BesselJZeros[m, {n, n}]] - 
          BesselJ[1 + m, r*BesselJZeros[m, {n, n}]]
                )^2*BesselJZeros[m, {n, n}]^2*
             Cos[m*theta]^2 + 
     4*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
             Sin[m*theta]^2))/4]), 
  -(Cos[t*BesselJZeros[m, {n, n}]]*
      (r*(BesselJ[-1 + m, r*BesselJZeros[m, {n, n}]] - 
           BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])*
   BesselJZeros[m, {n, n}]*Cos[m*theta]*Sin[theta]
         - 2*m*BesselJ[m, r*BesselJZeros[m, {n, n}]]*
         Cos[theta]*Sin[m*theta]))/
   (2*Sqrt[r^2 + (Cos[t*BesselJZeros[m, {n, n}]]^2*
          (r^2*(BesselJ[-1 + m, 
                 r*BesselJZeros[m, {n, n}]] - 
 BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^2*
 BesselJZeros[m, {n, n}]^2*
             Cos[m*theta]^2 + 
      4*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
       Sin[m*theta]^2))/4]), 
  r/Sqrt[r^2 + (Cos[t*BesselJZeros[m, {n, n}]]^2*
        (r^2*(BesselJ[-1 + m, 
               r*BesselJZeros[m, {n, n}]] - 
      BesselJ[1 + m, r*BesselJZeros[m, {n, n}]])^2*
            BesselJZeros[m, {n, n}]^2*Cos[m*theta]^2 + 
  4*m^2*BesselJ[m, r*BesselJZeros[m, {n, n}]]^2*
           Sin[m*theta]^2))/4]}

(*circularcylinder *)

gcurvature[circularcylinder[a_]][u_,v_]:=
0

mcurvature[circularcylinder[a_]][u_,v_]:=
-1/(2*a)

k1[circularcylinder[a_]][u_,v_]:=
0

k2[circularcylinder[a_]][u_,v_]:=
-1/(2*a)

unitnormal[circularcylinder[a_]][u_,v_]:=
{Cos[u],Sin[u],0}

surfacenormal[circularcylinder[a_]][u_,v_]:=
{a*Cos[u],a*Sin[u],0}


(*crosscap *)

gcurvature[crosscap[a_]][u_,v_]:=
(256*(-2*Cos[2*u] - Cos[2*(u - v)] + 4*Cos[2*v] -
Cos[2*(u + v)])*Sec[v]^2)/
  (a^4*(-30 + 2*Cos[2*u] + 3*Cos[2*u - 4*v] +
4*Cos[2*(u - v)] + 8*Cos[2*v] + 6*Cos[4*v] +
4*Cos[2*(u + v)] + 3*Cos[2*u + 4*v])^2)

mcurvature[crosscap[a_]][u_,v_]:=
(8*Sqrt[2]*((a^10*Cos[v]^4*
         (18 + 2*Cos[2*u] - Cos[2*(u - v)] + 
           14*Cos[2*v] - Cos[2*(u + v)])*Sin[u])/2 + 
      (a^10*Cos[v]^4*(1 + 3*Cos[2*v])*
      (6 - 2*Cos[2*u] - Cos[2*u - 4*v] - 2*Cos[4*v] - 
           Cos[2*u + 4*v])*Sin[u])/4 + 
  8*a^10*Cos[u]^2*Cos[v]^4*Cos[2*v]*Sin[u]*Sin[v]^2))/
         (a^12*(Cos[v]^4*(30 - 2*Cos[2*u] - 
         3*Cos[2*u - 4*v] - 4*Cos[2*(u - v)] - 
         8*Cos[2*v] - 6*Cos[4*v] - 4*Cos[2*(u + v)] - 
         3*Cos[2*u + 4*v]))^(3/2))

unitnormal[crosscap[a_]][u_,v_]:=
{-4*Sqrt[2]*Sin[2*v]/
   (30 - 2*Cos[2*u] - 3*Cos[2*u - 4*v] - 
      4*Cos[2*(u - v)] - 8*Cos[2*v] - 6*Cos[4*v] - 
      4*Cos[2*(u + v)] - 3*Cos[2*u + 4*v])^(1/2),
  (Sqrt[2]*Cos[u]*(2*Cos[2*u] + Cos[2*(u - v)] - 
       4*Cos[2*v] + Cos[2*(u + v)]))/
   (30 - 2*Cos[2*u] - 3*Cos[2*u - 4*v] - 
      4*Cos[2*(u - v)] - 8*Cos[2*v] - 6*Cos[4*v] - 
      4*Cos[2*(u + v)] - 3*Cos[2*u + 4*v])^(1/2),
  (Sqrt[2]*(-2 - 2*Cos[2*u] - Cos[2*(u - v)] + 
       2*Cos[2*v] - Cos[2*(u + v)])*Sin[u])/
   (30 - 2*Cos[2*u] - 3*Cos[2*u - 4*v] - 
      4*Cos[2*(u - v)] - 8*Cos[2*v] - 6*Cos[4*v] - 
      4*Cos[2*(u + v)] - 3*Cos[2*u + 4*v])^(1/2)}

surfacenormal[crosscap[a_]][u_,v_]:=
{-4*a^4*Cos[v]^3*Sin[v],
   -2*a^4*Cos[v]^2*Cos[2*v]*Sin[u]*Sin[2*u] + 
    (a^4*(Cos[u] + Cos[3*u])*Sin[2*v]^2)/4,
   -2*a^4*Cos[2*u]*Cos[v]^2*Cos[2*v]*Sin[u] - 
    a^4*Cos[u]^2*Sin[u]*Sin[2*v]^2}


(*cnccosurfrev *)

gcurvature[cnccosurfrev[a_,b_]][u_,v_]:=
-a^(-2)

mcurvature[cnccosurfrev[a_,b_]][u_,v_]:=
((a^2 - b^2)^(1/2)*(a^2 - b^2*Cosh[2*v/a])*
((-2*a^2 + b^2 + b^2*Cosh[2*v/a])/(-a^2 + b^2))^(1/2))/
  (Sqrt[2]*b*(-2*a^3 + a*b^2 + a*b^2*Cosh[2*v/a])*
    Sqrt[Sinh[v/a]^2])

k1[cnccosurfrev[a_,b_]][u_,v_]:=
(Sqrt[2]*b*Sinh[v/a])/
  (a*Sqrt[2*a^2 - b^2 - b^2*Cosh[2*v/a]])

k2[cnccosurfrev[a_,b_]][u_,v_]:=
-Sqrt[2*a^2 - b^2 - b^2*Cosh[2*v/a]]/
    (Sqrt[2]*a*b*Sqrt[Sinh[v/a]^2])

unitnormal[cnccosurfrev[a_,b_]][u_,v_]:=
{(Cos[u]*(a^2 - b^2 - b^2*Sinh[v/a]^2)^(1/2))/a,
  (Sin[u]*(a^2 - b^2 - b^2*Sinh[v/a]^2)^(1/2))/a,
  -((b*Cosh[v/a])/a)}

surfacenormal[cnccosurfrev[a_,b_]][u_,v_]:=
{(b*(a^2 - b^2)^(1/2)*Cos[u]*Sinh[v/a]*
      (1 - (b^2*Sinh[v/a]^2)/(a^2 - b^2))^(1/2))/a,
   (b*(a^2 - b^2)^(1/2)*Sin[u]*Sinh[v/a]*
      (1 - (b^2*Sinh[v/a]^2)/(a^2 - b^2))^(1/2))/a,
   -(b^2*Sinh[2*v/a])/(2*a)}

(*Mathematica3.0
mcurvature[cnccosurfrev[a_,b_]][u_,v_]:=
  -(Sqrt[2]*(a^2 - b^2*Cosh[2*v/a]))/
   (2*a*Sqrt[a^2 - b^2]*Sqrt[(-2*a^2 + b^2 + b^2*Cosh[2*v/a])/(-a^2 + b^2)]*
     Sqrt[b^2*Sinh[v/a]^2])

k1[cnccosurfrev[a_,b_]][u_,v_]:=
  (Sqrt[2]*Sqrt[b^2*Sinh[v/a]^2])/
   (a*Sqrt[a^2 - b^2]*Sqrt[-((-2*a^2 + b^2 + b^2*Cosh[2*v/a])/(a^2 - b^2))])

k2[cnccosurfrev[a_,b_]][u_,v_]:=
  -(Sqrt[2]*Sqrt[a^2 - b^2]*Sqrt[(-2*a^2 + b^2 + b^2*Cosh[2*v/a])/
    (-a^2 + b^2)]*Csch[v/a]^2*Sqrt[b^2*Sinh[v/a]^2])/(2*a*b^2)
*)

(*cnchysurfrev *)

gcurvature[cnchysurfrev[a_,b_]][u_,v_]:=
-a^(-2)

mcurvature[cnchysurfrev[a_,b_]][u_,v_]:=
(((2*a^2 + b^2 - b^2*Cosh[2*v/a])/a^2)^(1/2)*
(-a^2 + b^2*Cosh[2*v/a]))/
  (Sqrt[2]*b*(Cosh[v/a]^2)^(1/2)*
  (2*a^2 + b^2 - b^2*Cosh[2*v/a]))

unitnormal[cnchysurfrev[a_,b_]][u_,v_]:=
{Cos[u]*((a^2 - b^2*Sinh[v/a]^2)/a^2)^(1/2),
  Sin[u]*((a^2 - b^2*Sinh[v/a]^2)/a^2)^(1/2),
  -((b*Sinh[v/a])/a)}

surfacenormal[cnchysurfrev[a_,b_]][u_,v_]:=
{b*Cos[u]*Cosh[v/a]*(1 - (b^2*Sinh[v/a]^2)/a^2)^(1/2),
 b*Cosh[v/a]*Sin[u]*(1 - (b^2*Sinh[v/a]^2)/a^2)^(1/2),
   -(b^2*Sinh[2*v/a])/(2*a)}


(*cossurface *)

gcurvature[cossurface[a_]][u_,v_]:=
(Sin[u]^2*Sin[v]^2*Sin[u + v]^2)/
(a^2*(Sin[u]^2*Sin[v]^2 +
Sin[u]^2*Sin[u + v]^2 + Sin[v]^2*Sin[u + v]^2)^2)

mcurvature[cossurface[a_]][u_,v_]:=
(9 - 4*Cos[2*u] + Cos[4*u] - 4*Cos[2*v] +
Cos[4*v] - 4*Cos[2*(u + v)] + Cos[4*(u + v)])/
(16*a*(Sin[u]^2*Sin[v]^2 + Sin[u]^2*Sin[u + v]^2 +
Sin[v]^2*Sin[u + v]^2)^(3/2))

unitnormal[cossurface[a_]][u_,v_]:=
{-((Sin[v]*Sin[u + v])/
(Sin[u]^2*Sin[v]^2 + Sin[u]^2*Sin[u + v]^2 +
Sin[v]^2*Sin[u + v]^2)^(1/2)),
  -((Sin[u]*Sin[u + v])/
(Sin[u]^2*Sin[v]^2 + Sin[u]^2*Sin[u + v]^2 +
Sin[v]^2*Sin[u + v]^2)^(1/2)),
  (Sin[u]*Sin[v])/
(Sin[u]^2*Sin[v]^2 + Sin[u]^2*Sin[u + v]^2 +
Sin[v]^2*Sin[u + v]^2)^(1/2)}


(*cpcsurfrev *)

gcurvature[cpcsurfrev[a_,b_]][u_,v_]:=
a^(-2)

mcurvature[cpcsurfrev[a_,b_]][u_,v_]:=
((a^2 + b^2*Cos[2*v/a])*Sec[v/a])/
(Sqrt[2]*a*b*(2*a^2 - b^2 + b^2*Cos[2*v/a])^(1/2))

unitnormal[cpcsurfrev[a_,b_]][u_,v_]:=
{Cos[u]*((a^2 - b^2*Sin[v/a]^2)/a^2)^(1/2),
  Sin[u]*((a^2 - b^2*Sin[v/a]^2)/a^2)^(1/2),
  (b*Sin[v/a])/a}

surfacenormal[cpcsurfrev[a_,b_]][u_,v_]:=
{b*Cos[u]*Cos[v/a]*Sqrt[1 - b^2*Sin[v/a]^2/a^2],
   b*Cos[v/a]*Sin[u]*Sqrt[1 - b^2*Sin[v/a]^2/a^2],
   (b^2*Sin[2*v/a])/(2*a)}


(*dini *)

gcurvature[dini[a_,b_]][u_,v_]:=
-(a^2 + b^2)^(-1)

gcurvature[dinitcheb[a_,b_]][u_,v_]:=
-a^(-2)

mcurvature[dini[a_,b_]][u_,v_]:=
Cot[2*v]/Sqrt[a^2 + b^2]

mcurvature[dinitcheb[a_,b_]][u_,v_]:=
((a^2*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)^2 + 
        a^2*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^2*
         Tanh[Sec[b]*(u - v*Sin[b])]^2 + 
        a^2*Sech[Sec[b]*(u - v*Sin[b])]^2*Sin[v]^2*
         Tanh[Sec[b]*(u - v*Sin[b])]^2)*
      (a^3*Cos[b]^2*Cos[v]^2*
         Sech[Sec[b]*(u - v*Sin[b])]^2 - 
        a^3*Cos[b]^2*Cos[v]^2*
         Sech[Sec[b]*(u - v*Sin[b])]^4 + 
        a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^4*
         Sin[b]^2 - a^3*Cos[v]^2*
         Sech[Sec[b]*(u - v*Sin[b])]^6*Sin[b]^2 + 
        a^3*Cos[b]^2*Sech[Sec[b]*(u - v*Sin[b])]^2*
         Sin[v]^2 - a^3*Cos[b]^2*
         Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[v]^2 + 
        a^3*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[b]^2*
         Sin[v]^2 - a^3*Sech[Sec[b]*(u - v*Sin[b])]^6*
         Sin[b]^2*Sin[v]^2 + 
        a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^2*
         Sin[b]^2*Tanh[Sec[b]*(u - v*Sin[b])]^2 - 
        a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^4*
         Sin[b]^2*Tanh[Sec[b]*(u - v*Sin[b])]^2 + 
        a^3*Sech[Sec[b]*(u - v*Sin[b])]^2*Sin[b]^2*
        Sin[v]^2*Tanh[Sec[b]*(u - v*Sin[b])]^2 - 
       a^3*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[b]^2*
        Sin[v]^2*Tanh[Sec[b]*(u - v*Sin[b])]^2) + 
    (a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^4 - 
       a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^6 + 
       a^3*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[v]^2 - 
       a^3*Sech[Sec[b]*(u - v*Sin[b])]^6*Sin[v]^2 - 
       a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^2*
        Tanh[Sec[b]*(u - v*Sin[b])]^2 - 
       a^3*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^4*
        Tanh[Sec[b]*(u - v*Sin[b])]^2 - 
       a^3*Sech[Sec[b]*(u - v*Sin[b])]^2*Sin[v]^2*
        Tanh[Sec[b]*(u - v*Sin[b])]^2 - 
       a^3*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[v]^2*
        Tanh[Sec[b]*(u - v*Sin[b])]^2)*
     (a^2*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[b]^2 + 
       (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
          a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
           Tanh[Sec[b]*(u - v*Sin[b])])^2 + 
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
          a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
           Tanh[Sec[b]*(u - v*Sin[b])])^2))/
  (2*(a^2*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)^2 + 
       a^2*Cos[v]^2*Sech[Sec[b]*(u - v*Sin[b])]^2*
        Tanh[Sec[b]*(u - v*Sin[b])]^2 + 
       a^2*Sech[Sec[b]*(u - v*Sin[b])]^2*Sin[v]^2*
        Tanh[Sec[b]*(u - v*Sin[b])]^2)^(3/2)*
    (a^2*Sech[Sec[b]*(u - v*Sin[b])]^4*Sin[b]^2 + 
       (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
          a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
           Tanh[Sec[b]*(u - v*Sin[b])])^2 + 
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
          a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
           Tanh[Sec[b]*(u - v*Sin[b])])^2)^(3/2))

unitnormal[dini[a_,b_]][u_,v_]:=
{(a*Cos[u - v] + a*Cos[u + v] - 2*b*Sin[u])/
(2*Sqrt[a^2 + b^2]),
(2*b*Cos[u] + a*Sin[u - v] + a*Sin[u + v])/
(2*Sqrt[a^2 + b^2]),
-(a*Sin[v]/Sqrt[a^2 + b^2])}

unitnormal[dinitcheb[a_,b_]][u_,v_]:=
{(-(a^2*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*Sin[v]*
        Tanh[Sec[b]*(u - v*Sin[b])]) - 
     a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
      (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
        a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
         Tanh[Sec[b]*(u - v*Sin[b])]))/
   ((a^2*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*
          Tanh[Sec[b]*(u - v*Sin[b])] + 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
            a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
    (-(a^2*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*Sin[v]*
            Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
        (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
       a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
      (a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
       a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
      (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
        a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2)^(1/2),
  (a^2*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*
      Tanh[Sec[b]*(u - v*Sin[b])] + 
     a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
      (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
        a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
         Tanh[Sec[b]*(u - v*Sin[b])]))/
   ((a^2*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*
          Tanh[Sec[b]*(u - v*Sin[b])] + 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
         a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
   (-(a^2*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*Sin[v]*
            Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
        a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
      (a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
        a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
        a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2)^(1/2),
  (a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]*
      Tanh[Sec[b]*(u - v*Sin[b])]*
      (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
        a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
         Tanh[Sec[b]*(u - v*Sin[b])]) - 
     a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*
      Tanh[Sec[b]*(u - v*Sin[b])]*
      (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
        a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
         Tanh[Sec[b]*(u - v*Sin[b])]))/
   ((a^2*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*
          Tanh[Sec[b]*(u - v*Sin[b])] + 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
        a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
      (-(a^2*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*Sin[v]*
            Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
         a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2 + 
      (a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
          (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
               Sin[v]) + 
        a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
             Tanh[Sec[b]*(u - v*Sin[b])]) - 
         a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*
          Tanh[Sec[b]*(u - v*Sin[b])]*
       (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
         a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
             Tanh[Sec[b]*(u - v*Sin[b])]))^2)^(1/2)}

surfacenormal[dini[a_,b_]][u_,v_]:=
{a^2*Cos[u]*Cos[v]^2 - a*b*Cos[v]*Sin[u],
   a*b*Cos[u]*Cos[v] + a^2*Cos[v]^2*Sin[u],
   -a^2*Sin[2*v]/2}

surfacenormal[dinitcheb[a_,b_]][u_,v_]:=
{-(a^2*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*Sin[v]*
       Tanh[Sec[b]*(u - v*Sin[b])]) - 
    a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
     (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
       a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
        Tanh[Sec[b]*(u - v*Sin[b])]),
   a^2*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]^3*Sin[b]*
     Tanh[Sec[b]*(u - v*Sin[b])] + 
    a*(1 - Sech[Sec[b]*(u - v*Sin[b])]^2)*
     (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
       a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
        Tanh[Sec[b]*(u - v*Sin[b])]),
   a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]*
     Tanh[Sec[b]*(u - v*Sin[b])]*
     (-(a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]) + 
       a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*
        Tanh[Sec[b]*(u - v*Sin[b])]) - 
    a*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])]*
     Tanh[Sec[b]*(u - v*Sin[b])]*
     (a*Cos[b]*Cos[v]*Sech[Sec[b]*(u - v*Sin[b])] + 
       a*Sech[Sec[b]*(u - v*Sin[b])]*Sin[b]*Sin[v]*
        Tanh[Sec[b]*(u - v*Sin[b])])}


(*earth*)

gcurvature[earth][u_,v_]:=
1/(4.068213746010656*10^7 - 276007.9734754775*Cos[2*v] + 
    234.0733688694453*Cos[4*v])

mcurvature[earth][u_,v_]:=
(-1.052004061227612*10^19*Cos[v]^5*Sin[u]^2*
     (0.9932383978496035 + Sin[u]^2) - 
  2.104008122455224*10^19*Cos[v]^3*Sin[u]^4*Sin[v]^2 + 
    Cos[u]^4*(-1.052004061227612*10^19*Cos[v]^5 - 
       2.104008122455224*10^19*Cos[v]^3*Sin[v]^2) + 
    Cos[u]^2*(Cos[v]^5*(-1.044890828304989*10^19 - 
          2.104008122455224*10^19*Sin[u]^2) - 
  4.208016244910448*10^19*Cos[v]^3*Sin[u]^2*Sin[v]^2))/
  (2*(1.643788862450035*10^15*Cos[v]^4*Sin[u]^2 + 
  1.654979173186313*10^15*Cos[u]^4*Cos[v]^2*Sin[v]^2 + 
  1.654979173186313*10^15*Cos[v]^2*Sin[u]^4*Sin[v]^2 + 
       Cos[u]^2*(1.643788862450035*10^15*Cos[v]^4 + 
  3.309958346372626*10^15*Cos[v]^2*Sin[u]^2*
  Sin[v]^2))^(3/2))

(*eighteight *)

gcurvature[eighteight][u_,v_]:=
-2*Cos[2*v]*(2 + Cos[2*v])*Sin[u]^3*
    (3*Sin[u] + Sin[3*u])*Sin[v]^4/
(2*Cos[2*v]^2*Sin[v]^2 + Cos[2*u]*Cos[2*v]^2*Sin[v]^2 +
    Cos[4*u]*Cos[2*v]^2*Sin[v]^2 +
  2*Cos[v]^2*Sin[u]^2*Sin[v]^2 +
2*Cos[u]^2*Cos[v]^2*Sin[u]^2*Sin[v]^2 +
Cos[2*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 +
Cos[u]^2*Cos[2*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 +
Cos[4*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 +
Cos[u]^2*Cos[4*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 -
2*Cos[u]^6*Sin[u]^2*Sin[2*v]^2)^2

mcurvature[eighteight][u_,v_]:=
(3*Cos[2*v]^3*Sin[u]*Sin[v]^2 + 
    3*Cos[v]^2*Cos[2*v]*Sin[u]^3*Sin[v]^2 + 
    3*Cos[u]^2*Cos[v]^2*Cos[2*v]*Sin[u]^3*Sin[v]^2 + 
    Cos[2*v]^3*Sin[3*u]*Sin[v]^2 + 
    Cos[v]^2*Cos[2*v]*Sin[u]^2*Sin[3*u]*Sin[v]^2 + 
    Cos[u]^2*Cos[v]^2*Cos[2*v]*Sin[u]^2*Sin[3*u]*
     Sin[v]^2 - 4*Sin[u]^3*Sin[v]^4 - 
    2*Cos[2*u]*Sin[u]^3*Sin[v]^4 - 
    2*Cos[4*u]*Sin[u]^3*Sin[v]^4 - 
    2*Cos[2*v]*Sin[u]^3*Sin[v]^4 - 
    Cos[2*u]*Cos[2*v]*Sin[u]^3*Sin[v]^4 - 
    Cos[4*u]*Cos[2*v]*Sin[u]^3*Sin[v]^4)/
  (Sqrt[2]*(2*Cos[2*v]^2*Sin[v]^2 + 
       Cos[2*u]*Cos[2*v]^2*Sin[v]^2 + 
       Cos[4*u]*Cos[2*v]^2*Sin[v]^2 + 
       2*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
       2*Cos[u]^2*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
       Cos[2*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
       Cos[u]^2*Cos[2*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
       Cos[4*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
       Cos[u]^2*Cos[4*u]*Cos[v]^2*Sin[u]^2*Sin[v]^2 - 
       2*Cos[u]^6*Sin[u]^2*Sin[2*v]^2)^(3/2))

unitnormal[eighteight][u_,v_]:=
{(2*Cos[2*u]*Cos[2*v]*Sin[v])/
   (4*Cos[u]^2*Cos[2*v]^2*Sin[v]^2 + 
      4*Cos[2*u]^2*Cos[2*v]^2*Sin[v]^2 + 
      Sin[u]^6*Sin[2*v]^2)^(1/2),
  (-2*Cos[u]*Cos[2*v]*Sin[v])/
   (4*Cos[u]^2*Cos[2*v]^2*Sin[v]^2 + 
      4*Cos[2*u]^2*Cos[2*v]^2*Sin[v]^2 + 
      Sin[u]^6*Sin[2*v]^2)^(1/2),
  (Sin[u]^3*Sin[2*v])/
   (4*Cos[u]^2*Cos[2*v]^2*Sin[v]^2 + 
      4*Cos[2*u]^2*Cos[2*v]^2*Sin[v]^2 + 
      Sin[u]^6*Sin[2*v]^2)^(1/2)}

surfacenormal[eighteight][u_,v_]:=
{Cos[2*u]*Cos[2*v]*Sin[v],-(Cos[u]*Cos[2*v]*Sin[v]),
   (Sin[u]^3*Sin[2*v])/2}


(*eightsurface *)

gcurvature[eightsurface][u_,v_]:=
4*(2 + Cos[2*v])/(5 + Cos[2*v] + 4*Cos[4*v])^2

mcurvature[eightsurface][u_,v_]:=
((-11 + 3*Cos[2*v] - 2*Cos[4*v])*Csc[v])/
  (2*Sqrt[2]*(5 + Cos[2*v] + 4*Cos[4*v])^(3/2))

unitnormal[eightsurface][u_,v_]:=
{(Sqrt[2]*Cos[u]*Cos[v])/
   (5 + Cos[2*v] + 4*Cos[4*v])^(1/2),
  (Sqrt[2]*Cos[v]*Sin[u])/
   (5 + Cos[2*v] + 4*Cos[4*v])^(1/2),
  (-2*Sqrt[2]*Cos[2*v])/
   (5 + Cos[2*v] + 4*Cos[4*v])^(1/2)}

surfacenormal[eightsurface][u_,v_]:=
{Cos[u]*Cos[v]*Sin[2*v],Cos[v]*Sin[u]*Sin[2*v],
   -Sin[4*v]}


(*ellhypcyclide *)

gcurvature[ellhypcyclide[a_,c_,k_]][u_,v_]:=
Cos[v]/((k - c*Cos[u])*(-a + k*Cos[v]))

mcurvature[ellhypcyclide[a_,c_,k_]][u_,v_]:=
(a - 2*k*Cos[v] + c*Cos[u]*Cos[v])/
  (2*(-k + c*Cos[u])*(-a + k*Cos[v]))

unitnormal[ellhypcyclide[a_,c_,k_]][u_,v_]:=
{(a*Cos[u]*Cos[v] - c)/(a - c*Cos[u]*Cos[v]), 
 Sqrt[a^2 - c^2]*Cos[v]*Sin[u]/(a - c*Cos[u]*Cos[v]), 
       -Sqrt[a^2 - c^2]*Sin[v]/(a - c*Cos[u]*Cos[v])}

(*ellipticalhyperboloidminus *)

gcurvature[ellipticalhyperboloidminus[a_,b_,c_]][u_,v_]:=
-4*a^2*b^2*c^2/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 + a^2*c^2*v^2 +
b^2*c^2*v^2 - a^2*c^2*Cos[2*u] + b^2*c^2*Cos[2*u] +
a^2*c^2*v^2*Cos[2*u] - b^2*c^2*v^2*Cos[2*u] -
2*a^2*c^2*v*Sin[2*u] + 2*b^2*c^2*v*Sin[2*u])^2

gcurvature[ellipticalhyperboloidplus[a_,b_,c_]][u_,v_]:=
-4*a^2*b^2*c^2/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 + a^2*c^2*v^2 +
b^2*c^2*v^2 - a^2*c^2*Cos[2*u] + b^2*c^2*Cos[2*u] +
a^2*c^2*v^2*Cos[2*u] - b^2*c^2*v^2*Cos[2*u] +
2*a^2*c^2*v*Sin[2*u] - 2*b^2*c^2*v*Sin[2*u])^2

mcurvature[ellipticalhyperboloidminus[a_,b_,c_]][u_,v_]:=
(Sqrt[2]*(a^3*b*c + a*b^3*c - a*b*c^3 - a*b*c^3*v^2 - 
      a*b^3*c*Cos[u]^2 - a*b^3*c*v^2*Cos[u]^2 - 
      a^3*b*c*Cos[2*u] + a*b^3*c*Cos[2*u] - 
      a^3*b*c*Sin[u]^2 - a^3*b*c*v^2*Sin[u]^2 - 
      a^3*b*c*v*Sin[2*u] + a*b^3*c*v*Sin[2*u]))/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 + a^2*c^2*v^2 + 
     b^2*c^2*v^2 - a^2*c^2*Cos[2*u] + b^2*c^2*Cos[2*u] + 
     a^2*c^2*v^2*Cos[2*u] - b^2*c^2*v^2*Cos[2*u] - 
     2*a^2*c^2*v*Sin[2*u] + 2*b^2*c^2*v*Sin[2*u])^(3/2)


mcurvature[ellipticalhyperboloidplus[a_,b_,c_]][u_,v_]:=
(4*(a^3*b*c + a*b^3*c - a*b*c^3 - a*b*c^3*v^2 - 
      a*b^3*c*Cos[u]^2 - a*b^3*c*v^2*Cos[u]^2 - 
      a^3*b*c*Cos[2*u] + a*b^3*c*Cos[2*u] - 
      a^3*b*c*Sin[u]^2 - a^3*b*c*v^2*Sin[u]^2 + 
      a^3*b*c*v*Sin[2*u] - a*b^3*c*v*Sin[2*u]))/
  (-a^4 - 2*a^2*b^2 - b^4 + 4*b^2*c^2*Cos[u]^2 + 
     4*a^2*c^2*v^2*Cos[u]^2 + 4*b^4*Cos[u]^4 + 
     4*a^2*b^2*v^2*Cos[u]^4 + 2*a^4*Cos[2*u] - 
     2*b^4*Cos[2*u] - a^4*Cos[2*u]^2 + 
     2*a^2*b^2*Cos[2*u]^2 - b^4*Cos[2*u]^2 + 
     8*a^2*c^2*v*Cos[u]*Sin[u] - 
     8*b^2*c^2*v*Cos[u]*Sin[u] + 
     8*a^2*b^2*v*Cos[u]^3*Sin[u] - 
     8*b^4*v*Cos[u]^3*Sin[u] + 4*a^2*c^2*Sin[u]^2 + 
     4*b^2*c^2*v^2*Sin[u]^2 + 
     8*a^2*b^2*Cos[u]^2*Sin[u]^2 + 
     4*a^4*v^2*Cos[u]^2*Sin[u]^2 + 
     4*b^4*v^2*Cos[u]^2*Sin[u]^2 + 
     8*a^4*v*Cos[u]*Sin[u]^3 - 
     8*a^2*b^2*v*Cos[u]*Sin[u]^3 + 4*a^4*Sin[u]^4 + 
     4*a^2*b^2*v^2*Sin[u]^4 - 2*a^4*v*Sin[2*u] + 
     2*b^4*v*Sin[2*u] + 2*a^4*v*Cos[2*u]*Sin[2*u] - 
     4*a^2*b^2*v*Cos[2*u]*Sin[2*u] + 
     2*b^4*v*Cos[2*u]*Sin[2*u] - a^4*v^2*Sin[2*u]^2 + 
     2*a^2*b^2*v^2*Sin[2*u]^2 -
     b^4*v^2*Sin[2*u]^2)^(3/2)

unitnormal[ellipticalhyperboloidminus[a_,b_,c_]][u_,v_]:=
{(b*c*(Cos[u] + v*Sin[u]))/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
    a^2*c^2*v^2*Cos[u]^2 - 2*a^2*c^2*v*Cos[u]*Sin[u] + 
      2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
      b^2*c^2*v^2*Sin[u]^2)^(1/2),
  (a*c*(-(v*Cos[u]) + Sin[u]))/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
    a^2*c^2*v^2*Cos[u]^2 - 2*a^2*c^2*v*Cos[u]*Sin[u] + 
      2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
      b^2*c^2*v^2*Sin[u]^2)^(1/2),
  -((a*b*v)/
     (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
        a^2*c^2*v^2*Cos[u]^2 - 
        2*a^2*c^2*v*Cos[u]*Sin[u] + 
        2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
        b^2*c^2*v^2*Sin[u]^2)^(1/2))}

unitnormal[ellipticalhyperboloidplus[a_,b_,c_]][u_,v_]:=
{(b*c*(Cos[u] - v*Sin[u]))/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
    a^2*c^2*v^2*Cos[u]^2 + 2*a^2*c^2*v*Cos[u]*Sin[u] - 
      2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
      b^2*c^2*v^2*Sin[u]^2)^(1/2),
  (a*c*(v*Cos[u] + Sin[u]))/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
  a^2*c^2*v^2*Cos[u]^2 + 2*a^2*c^2*v*Cos[u]*Sin[u] - 
      2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
      b^2*c^2*v^2*Sin[u]^2)^(1/2),
  -((a*b*v)/
     (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 + 
        a^2*c^2*v^2*Cos[u]^2 + 
        2*a^2*c^2*v*Cos[u]*Sin[u] - 
        2*b^2*c^2*v*Cos[u]*Sin[u] + a^2*c^2*Sin[u]^2 + 
        b^2*c^2*v^2*Sin[u]^2)^(1/2))}

surfacenormal[ellipticalhyperboloidminus[a_,b_,c_]][u_,v_]:=
{b*c*(Cos[u] + v*Sin[u]),
-(a*c*(v*Cos[u] - Sin[u])),-a*b*v}

surfacenormal[ellipticalhyperboloidplus[a_,b_,c_]][u_,v_]:=
{b*c*(Cos[u] - v*Sin[u]),
a*c*(v*Cos[u] + Sin[u]),-a*b*v}


(*ellipsoid *)

gcurvature[ellipsoid[a_,b_,c_]][u_,v_]:=
(64*a^2*b^2*c^2)/
  (4*a^2*b^2 + 2*a^2*c^2 + 2*b^2*c^2 -
  2*(a^2 - b^2)*c^2*Cos[2*u] - 
     (a^2 - b^2)*c^2*Cos[2*(u - v)] -
     4*a^2*b^2*Cos[2*v] + 
     2*a^2*c^2*Cos[2*v] + 2*b^2*c^2*Cos[2*v] -
     a^2*c^2*Cos[2*(u + v)] + 
     b^2*c^2*Cos[2*(u + v)])^2

mcurvature[ellipsoid[a_,b_,c_]][u_,v_]:=
(16*(-(a^3*b*c*Cos[v]^3) - a*b^3*c*Cos[v]^3 + 
      a^3*b*c*Cos[2*u]*Cos[v]^3 - 
      a*b^3*c*Cos[2*u]*Cos[v]^3 - 2*a*b*c^3*Cos[v]^5 - 
      2*a^3*b*c*Cos[u]^2*Cos[v]^3*Sin[v]^2 - 
      2*a*b^3*c*Cos[v]^3*Sin[u]^2*Sin[v]^2))/
  (16*b^2*c^2*Cos[u]^2*Cos[v]^4 + 
     16*a^2*c^2*Cos[v]^4*Sin[u]^2 + 
     16*a^2*b^2*Cos[u]^4*Cos[v]^2*Sin[v]^2 + 
     16*a^4*Cos[u]^2*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
     16*b^4*Cos[u]^2*Cos[v]^2*Sin[u]^2*Sin[v]^2 + 
     16*a^2*b^2*Cos[v]^2*Sin[u]^4*Sin[v]^2 - 
     a^4*Sin[2*u]^2*Sin[2*v]^2 + 
     2*a^2*b^2*Sin[2*u]^2*Sin[2*v]^2 - 
     b^4*Sin[2*u]^2*Sin[2*v]^2)^(3/2)

unitnormal[ellipsoid[a_,b_,c_]][u_,v_]:=
{(2*b*c*Cos[u]*Cos[v]^2)/
   (4*b^2*c^2*Cos[u]^2*Cos[v]^4 + 
      4*a^2*c^2*Cos[v]^4*Sin[u]^2 +
      a^2*b^2*Sin[2*v]^2)^
    (1/2),(2*a*c*Cos[v]^2*Sin[u])/
   (4*b^2*c^2*Cos[u]^2*Cos[v]^4 + 
      4*a^2*c^2*Cos[v]^4*Sin[u]^2 +
      a^2*b^2*Sin[2*v]^2)^
    (1/2),(a*b*Sin[2*v])/
   (4*b^2*c^2*Cos[u]^2*Cos[v]^4 + 
      4*a^2*c^2*Cos[v]^4*Sin[u]^2 +
      a^2*b^2*Sin[2*v]^2)^(1/2)}

surfacenormal[ellipsoid[a_,b_,c_]][u_,v_]:=
{b*c*Cos[u]*Cos[v]^2,a*c*Cos[v]^2*Sin[u],
a*b*Sin[2*v]/2}


(*enneper *)

gcurvature[enneper][u_,v_]:=
-4/(1 + u^2 + v^2)^4

mcurvature[enneper][u_,v_]:=
0

k1[enneper][u_,v_]:=
2/(1 + u^2 + v^2)^2

k2[enneper][u_,v_]:=
-2/(1 + u^2 + v^2)^2

unitnormal[enneper][u_,v_]:=
{(2*u)/(1 + u^2 + v^2),
(2*v)/(1 + u^2 + v^2),
(-1 + u^2 + v^2)/(1 + u^2 + v^2)}

surfacenormal[enneper][u_,v_]:=
{2*u*(1 + u^2 + v^2),
2*v*(1 + u^2 + v^2),
-1 + u^4 + 2*u^2*v^2 + v^4}

gcurvature[enneperpolar[n_][t_]][r_,theta_]:=
-4*n^2*r^(-2 + 2*n)/(1 + r^(2*n))^4

mcurvature[enneperpolar[n_][t_]][r_,theta_]:=
0

k1[enneperpolar[n_][t_]][r_,theta_]:=
(2*n*r^(-1 + n))/(1 + r^(2*n))^2

k2[enneperpolar[n_][t_]][r_,theta_]:=
-(2*n*r^(-1 + n))/(1 + r^(2*n))^2

unitnormal[enneperpolar[n_][t_]][r_,theta_]:=
{(2*r^n*Cos[n*theta])/(1 + r^(2*n)),
  (2*r^n*Sin[n*theta])/(1 + r^(2*n)),
  (-1 + r^(2*n))/(1 + r^(2*n))}

surfacenormal[enneperpolar[n_][t_]][r_,theta_]:=
{2*r^(n + 1)*(1 + r^(2*n))*Cos[n*theta],
   2*r^(n + 1)*(1 + r^(2*n))*Sin[n*theta],
   r*(-1 + r^(4*n))}

(*ennepercatenoidpolar *)

gcurvature[ennepercatenoidpolar[n_][t_]][r_,theta_]:=
-4*r^4*(1 + n^2*r^(2*(n + 1)) -
2*n*r^(n + 1)*Cos[theta + n*theta])/
(1 + r^2 + r^(2 + 2*n) +
2*r^(n + 1)*Cos[theta + n*theta])^4

gcurvature[ennepercatenoidpolar[n_,a_,b_][t_]][r_,theta_]:=
-4*r^4*(a^2 + b^2*n^2*r^(2*(n + 1)) -
2*a*b*n*r^(n + 1)*Cos[theta + n*theta])/
(a^2 + r^2 + b^2*r^(2 + 2*n) +
2*a*b*r^(n + 1)*Cos[theta + n*theta])^4

mcurvature[ennepercatenoidpolar[n_][t_]][r_,theta_]:=
0

mcurvature[ennepercatenoidpolar[n_,a_,b_][t_]][r_,theta_]:=
0

unitnormal[ennepercatenoidpolar[n_][t_]][r_,theta_]:=
{2*r*((1 + r^2 + 2*r^(2*(n + 1)))*Cos[theta] + 
       r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
           Cos[n*theta] + Cos[(2 + n)*theta] + 
          r^(n + 1)*Cos[theta + 2*n*theta]))/
   Sqrt[(1 - r^4 + 4*r^(2*(n + 1)) + r^(4*(n + 1)) + 
        2*r^(2*(n + 1))*Cos[2*(n + 1)*theta] + 
        4*r^(n + 1)*(1 + r^(2*(n + 1)))*
         Cos[theta + n*theta])^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Cos[theta] + 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Cos[n*theta] + Cos[(2 + n)*theta] + 
            r^(n + 1)*Cos[theta + 2*n*theta]))^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Sin[theta] - 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Sin[n*theta] - Sin[(2 + n)*theta] + 
            r^(n + 1)*Sin[theta + 2*n*theta]))^2], 
  (2*r*(-((1 + r^2 + 2*r^(2*(n + 1)))*Sin[theta]) + 
       r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
           Sin[n*theta] - Sin[(2 + n)*theta] + 
          r^(n + 1)*Sin[theta + 2*n*theta])))/
   Sqrt[(1 - r^4 + 4*r^(2*(n + 1)) + r^(4*(n + 1)) + 
        2*r^(2*(n + 1))*Cos[2*(n + 1)*theta] + 
        4*r^(n + 1)*(1 + r^(2*(n + 1)))*
         Cos[theta + n*theta])^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Cos[theta] + 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Cos[n*theta] + Cos[(2 + n)*theta] + 
            r^(n + 1)*Cos[theta + 2*n*theta]))^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Sin[theta] - 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Sin[n*theta] - Sin[(2 + n)*theta] + 
            r^(n + 1)*Sin[theta + 2*n*theta]))^2], 
  (1 - r^4 + 4*r^(2*(n + 1)) + r^(4*(n + 1)) + 
     2*r^(2*(n + 1))*Cos[2*(n + 1)*theta] + 
     4*r^(n + 1)*(1 + r^(2*(n + 1)))*
     Cos[theta + n*theta])
    /Sqrt[(1 - r^4 + 4*r^(2*(n + 1)) + r^(4*(n + 1)) + 
        2*r^(2*(n + 1))*Cos[2*(n + 1)*theta] + 
        4*r^(n + 1)*(1 + r^(2*(n + 1)))*
         Cos[theta + n*theta])^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Cos[theta] + 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Cos[n*theta] + Cos[(2 + n)*theta] + 
            r^(n + 1)*Cos[theta + 2*n*theta]))^2 + 
     4*r^2*((1 + r^2 + 2*r^(2*(n + 1)))*Sin[theta] - 
         r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
             Sin[n*theta] - Sin[(2 + n)*theta] + 
            r^(n + 1)*Sin[theta + 2*n*theta]))^2]}

unitnormal[ennepercatenoidpolar[n_,a_,b_][t_]][r_,theta_]:=
{(2*r^4*Sqrt[(a^2 + r^2 + b^2*r^(2*(1 + n)) + 
          2*a*b*r^(1 + n)*Cos[(1 + n)*theta])^4/r^6]*
     (a*(a^2 + r^2 + 2*b^2*r^(2*(1 + n)))*Cos[theta] + 
     b*r^(1 + n)*((2*a^2 + r^2 + b^2*r^(2*(1 + n)))*
           Cos[n*theta] + 
          a*(a*Cos[(2 + n)*theta] + 
             b*r^(1 + n)*Cos[theta + 2*n*theta]))))/
   (a^2 + r^2 + b^2*r^(2 + 2*n) + 
      2*a*b*r^(1 + n)*Cos[theta + n*theta])^4, 
  (2*r^4*Sqrt[(a^2 + r^2 + b^2*r^(2*(1 + n)) + 
          2*a*b*r^(1 + n)*Cos[(1 + n)*theta])^4/r^6]*
  (-(a*(a^2 + r^2 + 2*b^2*r^(2*(1 + n)))*Sin[theta]) + 
    b*r^(1 + n)*((2*a^2 + r^2 + b^2*r^(2*(1 + n)))*
           Sin[n*theta] + 
          a*(-(a*Sin[(2 + n)*theta]) + 
             b*r^(1 + n)*Sin[theta + 2*n*theta]))))/
   (a^2 + r^2 + b^2*r^(2 + 2*n) + 
      2*a*b*r^(1 + n)*Cos[theta + n*theta])^4, 
  (r^3*Sqrt[(a^2 + r^2 + b^2*r^(2*(1 + n)) + 
          2*a*b*r^(1 + n)*Cos[(1 + n)*theta])^4/r^6]*
     (a^2 + r^2*(-1 + b^2*r^(2*n)) + 
       2*a*b*r^(1 + n)*Cos[(1 + n)*theta]))/
   (a^2 + r^2 + b^2*r^(2 + 2*n) + 
      2*a*b*r^(1 + n)*Cos[theta + n*theta])^3}

surfacenormal[ennepercatenoidpolar[n_][t_]][r_,theta_]:=
{(2*((1 + r^2 + 2*r^(2*(n + 1)))*Cos[theta] + 
       r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
           Cos[n*theta] + Cos[(2 + n)*theta] + 
          r^(n + 1)*Cos[theta + 2*n*theta])))/r^2, 
  (2*(-((1 + r^2 + 2*r^(2*(n + 1)))*Sin[theta]) + 
       r^(n + 1)*((2 + r^2 + r^(2*(n + 1)))*
           Sin[n*theta] - Sin[(2 + n)*theta] + 
          r^(n + 1)*Sin[theta + 2*n*theta])))/r^2, 
  (1 - r^4 + 4*r^(2*(n + 1)) + r^(4*(n + 1)) + 
     2*r^(2*(n + 1))*Cos[2*(n + 1)*theta] + 
     4*r^(n + 1)*(1 + r^(2*(n + 1)))*Cos[theta + n*theta])/r^3}

surfacenormal[ennepercatenoidpolar[n_,a_,b_][t_]][r_,theta_]:=
{(2*(a*(a^2 + r^2 + 2*b^2*r^(2*(1 + n)))*Cos[theta] + 
       b*r^(1 + n)*((2*a^2 + r^2 + b^2*r^(2*(1 + n)))*
           Cos[n*theta] + 
          a*(a*Cos[(2 + n)*theta] + 
       b*r^(1 + n)*Cos[theta + 2*n*theta]))))/r^2, 
  (2*(-(a*(a^2 + r^2 + 2*b^2*r^(2*(1 + n)))*Sin[theta]) + 
       b*r^(1 + n)*((2*a^2 + r^2 + b^2*r^(2*(1 + n)))*
           Sin[n*theta] + 
          a*(-(a*Sin[(2 + n)*theta]) + 
          b*r^(1 + n)*Sin[theta + 2*n*theta]))))/r^2, 
  (a^4 - r^4 + 4*a^2*b^2*r^(2*(1 + n)) + 
     b^4*r^(4*(1 + n)) + 
     4*a*b*r^(1 + n)*(a^2 + b^2*r^(2*(1 + n)))*
      Cos[(1 + n)*theta] + 
     2*a^2*b^2*r^(2*(1 + n))*Cos[2*(1 + n)*theta])/r^3}

(*funnel *)

gcurvature[funnel[a_,b_,c_]][u_,v_]:=
(-4*a^2*b^2*c^2)/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 - 
     a^2*c^2*Cos[2*u] + b^2*c^2*Cos[2*u])^2

mcurvature[funnel[a_,b_,c_]][u_,v_]:=
(Sqrt[2]*a*b*c*(-c^2 - a^2*v^2*Cos[2*u] + 
      b^2*v^2*Cos[2*u]))/
  (v*(a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 - 
       a^2*c^2*Cos[2*u] + b^2*c^2*Cos[2*u])^(3/2))

unitnormal[funnel[a_,b_,c_]][u_,v_]:=
{(b*c*Cos[u])/
(a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 +
a^2*c^2*Sin[u]^2)^(1/2),
(a*c*Sin[u])/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 +
   a^2*c^2*Sin[u]^2)^(1/2),
   (-(a*b*v*Cos[u]^2) - a*b*v*Sin[u]^2)/
   (a^2*b^2*v^2 + b^2*c^2*Cos[u]^2 +
   a^2*c^2*Sin[u]^2)^(1/2)}

surfacenormal[funnel[a_,b_,c_]][u_,v_]:=
{b*c*Cos[u],a*c*Sin[u],-a*b*v}


(*genhypparab *)

gcurvature[genhypparab[a_,b_]][u_,v_]:=
-4*a^2*b^2/
  (a^2*b^2 + 4*b^2*u^2 + 8*b^2*u*v + 4*a^2*v^2 +
4*b^2*v^2)^2

mcurvature[genhypparab[a_,b_]][u_,v_]:=
(a*b*(-a^2 + b^2 - 4*u^2 - 8*u*v))/
  (a^2*b^2 + 4*b^2*u^2 + 8*b^2*u*v + 4*a^2*v^2 + 
     4*b^2*v^2)^(3/2)

unitnormal[genhypparab[a_,b_]][u_,v_]:=
{-((b*(2*u + 2*v))/
     (a^2*b^2 + 4*a^2*v^2 + 4*b^2*(u + v)^2)^(1/2)),
  (-2*a*u + a*(2*u + 2*v))/
   (a^2*b^2 + 4*a^2*v^2 + 4*b^2*(u + v)^2)^(1/2),
  (a*b)/(a^2*b^2 + 4*a^2*v^2 + 4*b^2*(u + v)^2)^(1/2)}

surfacenormal[genhypparab[a_,b_]][u_,v_]:=
{-2*b*(u + v),2*a*v,a*b}


(*handkerchief *)

gcurvature[handkerchief[a_]][u_,v_]:=
4*(-a^2 + u^2 - v^2)/
  (1 + 4*a^2*u^2 + 4*a*u^3 + u^4 + 4*a^2*v^2 -
4*a*u*v^2 + 6*u^2*v^2 + v^4)^2

mcurvature[handkerchief[a_]][u_,v_]:=
(2*u - 4*a^3*u^2 + 3*a*u^4 + u^5 + 4*a^3*v^2 - 
    6*a*u^2*v^2 + 2*u^3*v^2 + 3*a*v^4 - 3*u*v^4)/
  (1 + 4*a^2*u^2 + 4*a*u^3 + u^4 + 4*a^2*v^2 - 
     4*a*u*v^2 + 6*u^2*v^2 + v^4)^(3/2)

unitnormal[handkerchief[a_]][u_,v_]:=
{(-2*a*u - u^2 - v^2)/
   (1 + 4*a^2*u^2 + 4*a*u^3 + u^4 + 4*a^2*v^2 - 
      4*a*u*v^2 + 6*u^2*v^2 + v^4)^(1/2),
  (2*(a - u)*v)/
   (1 + 4*a^2*u^2 + 4*a*u^3 + u^4 + 4*a^2*v^2 - 
      4*a*u*v^2 + 6*u^2*v^2 + v^4)^(1/2),
  (1 + 4*a^2*u^2 + 4*a*u^3 + u^4 + 4*a^2*v^2 - 
     4*a*u*v^2 + 6*u^2*v^2 + v^4)^(-1/2)}

surfacenormal[handkerchief[a_]][u_,v_]:=
{-2*a*u - u^2 - v^2,2*(a - u)*v,1}


(*heart *)

gcurvature[heart[a_]][u_,v_]:=
(3*(1 + 2*Cos[v])*Sec[v/2]^4)/(64*a^2)

mcurvature[heart[a_]][u_,v_]:=
-((5 + 7*Cos[v])*Tan[v/2]^3)/(32*a*(Sin[v/2]^2)^(3/2))

(*helicoid *)

gcurvature[helicoid[a_,b_,c_]][u_,v_]:=
-4*a^2*b^2*c^2/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 +
a^2*c^2*Cos[2*u] - b^2*c^2*Cos[2*u])^2

mcurvature[helicoid[a_,b_,c_]][u_,v_]:=
(Sqrt[2]*a*b*(a^2 - b^2)*c*v*Sin[2*u])/
  (a^2*c^2 + b^2*c^2 + 2*a^2*b^2*v^2 + 
     a^2*c^2*Cos[2*u] - b^2*c^2*Cos[2*u])^(3/2)

unitnormal[helicoid[a_,b_,c_]][u_,v_]:=
{-((b*c*Sin[u])/
  (a^2*b^2*v^2 + a^2*c^2*Cos[u]^2 +
  b^2*c^2*Sin[u]^2)^(1/2)),
  (a*c*Cos[u])/
   (a^2*b^2*v^2 + a^2*c^2*Cos[u]^2 +
   b^2*c^2*Sin[u]^2)^(1/2),
   -((a*b*v)/(a^2*b^2*v^2 + a^2*c^2*Cos[u]^2 +
   b^2*c^2*Sin[u]^2)^(1/2))}

surfacenormal[helicoid[a_,b_,c_]][u_,v_]:=
{-2*a*u - u^2 - v^2,2*(a - u)*v,1}


(*henneberg *)

gcurvature[henneberg][u_,v_]:=
Sech[u]^4/(8*(Cos[4*v] - Cosh[4*u]))

mcurvature[henneberg][u_,v_]:=
0

unitnormal[henneberg][u_,v_]:=
{Cos[v]*Sech[u],-(Sech[u]*Sin[v]),Tanh[u]}

surfacenormal[henneberg][u_,v_]:=
{8*Cos[v]*Cosh[u]*(Cos[4*v] - Cosh[4*u]),
   8*Cosh[u]*(-Cos[4*v] + Cosh[4*u])*Sin[v],
   4*(Cos[4*v] - Cosh[4*u])*Sinh[2*u]}


(*hyperbolicparaboloid *)

gcurvature[hyperbolicparaboloid][u_,v_]:=
-(1 + u^2 + v^2)^(-2)

mcurvature[hyperbolicparaboloid][u_,v_]:=
-((u*v)/(1 + u^2 + v^2)^(3/2))

unitnormal[hyperbolicparaboloid][u_,v_]:=
{-(v/(1 + u^2 + v^2)^(1/2)),
-(u/(1 + u^2 + v^2)^(1/2)),
  (1 + u^2 + v^2)^(-1/2)}

surfacenormal[hyperbolicparaboloid][u_,v_]:=
{-v,-u,1}


(*hyperboloid *)

gcurvature[hyperboloid[a_,b_,c_]][u_,v_]:=
-256*a^2*b^2*c^2*Cosh[v]^4/
  (16*b^2*c^2*Cos[u]^2*Cosh[v]^4 +
16*a^2*c^2*Cosh[v]^4*Sin[u]^2 +
16*a^2*b^2*Cos[u]^4*Cosh[v]^2*Sinh[v]^2 +
16*a^4*Cos[u]^2*Cosh[v]^2*Sin[u]^2*Sinh[v]^2 +
16*b^4*Cos[u]^2*Cosh[v]^2*Sin[u]^2*Sinh[v]^2 +
16*a^2*b^2*Cosh[v]^2*Sin[u]^4*Sinh[v]^2 -
a^4*Sin[2*u]^2*Sinh[2*v]^2 +
2*a^2*b^2*Sin[2*u]^2*Sinh[2*v]^2 -
b^4*Sin[2*u]^2*Sinh[2*v]^2)^2

mcurvature[hyperboloid[a_,b_,c_]][u_,v_]:=
(16*(a^3*b*c*Cosh[v]^3 + a*b^3*c*Cosh[v]^3 - 
      a^3*b*c*Cos[2*u]*Cosh[v]^3 + 
    a*b^3*c*Cos[2*u]*Cosh[v]^3 - 2*a*b*c^3*Cosh[v]^5 - 
      2*a^3*b*c*Cos[u]^2*Cosh[v]^3*Sinh[v]^2 - 
      2*a*b^3*c*Cosh[v]^3*Sin[u]^2*Sinh[v]^2))/
  (16*b^2*c^2*Cos[u]^2*Cosh[v]^4 + 
     16*a^2*c^2*Cosh[v]^4*Sin[u]^2 + 
     16*a^2*b^2*Cos[u]^4*Cosh[v]^2*Sinh[v]^2 + 
     16*a^4*Cos[u]^2*Cosh[v]^2*Sin[u]^2*Sinh[v]^2 + 
     16*b^4*Cos[u]^2*Cosh[v]^2*Sin[u]^2*Sinh[v]^2 + 
     16*a^2*b^2*Cosh[v]^2*Sin[u]^4*Sinh[v]^2 - 
     a^4*Sin[2*u]^2*Sinh[2*v]^2 + 
     2*a^2*b^2*Sin[2*u]^2*Sinh[2*v]^2 - 
     b^4*Sin[2*u]^2*Sinh[2*v]^2)^(3/2)

unitnormal[hyperboloid[a_,b_,c_]][u_,v_]:=
{(2*b*c*Cos[u]*Cosh[v]^2)/
   (4*b^2*c^2*Cos[u]^2*Cosh[v]^4 + 
 4*a^2*c^2*Cosh[v]^4*Sin[u]^2 +
 a^2*b^2*Sinh[2*v]^2)^(1/2),
 (2*a*c*Cosh[v]^2*Sin[u])/
 (4*b^2*c^2*Cos[u]^2*Cosh[v]^4 + 
  4*a^2*c^2*Cosh[v]^4*Sin[u]^2 +
  a^2*b^2*Sinh[2*v]^2)^(1/2),
  -((a*b*Sinh[2*v])/
     (4*b^2*c^2*Cos[u]^2*Cosh[v]^4 + 
   4*a^2*c^2*Cosh[v]^4*Sin[u]^2 +
   a^2*b^2*Sinh[2*v]^2)^(1/2))}

surfacenormal[hyperboloid[a_,b_,c_]][u_,v_]:=
{b*c*Cos[u]*Cosh[v]^2,
a*c*Cosh[v]^2*Sin[u],
-a*b*Sinh[2*v]/2}


(*invertedcircularcylinder *)

gcurvature[invertedcircularcylinder[{p_,0,0},rho_,a_,b_]][u_,v_]:=
(2*(a^2 - p^2 - v^2)*(a - p*Cos[u]))/(a*rho^4)

mcurvature[invertedcircularcylinder[{p_,0,0},rho_,a_,b_]][u_,v_]:=
(-3*a^2 + p^2 + v^2 + 2*a*p*Cos[u])/(2*a*rho^2)

unitnormal[invertedcircularcylinder[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(-2*a*p + a^2*Cos[u] + p^2*Cos[u] - v^2*Cos[u])/
   (a^2 + p^2 + v^2 - 2*a*p*Cos[u]),
  ((a^2 - p^2 - v^2)*Sin[u])/
   (a^2 + p^2 + v^2 - 2*a*p*Cos[u]),
  (2*v*(a - p*Cos[u]))/(a^2 + p^2 + v^2 - 2*a*p*Cos[u])}

surfacenormal[invertedcircularcylinder[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(a*rho^4*(-2*a*p + a^2*Cos[u] + p^2*Cos[u] - 
       v^2*Cos[u]))/(a^2 + p^2 + v^2 - 2*a*p*Cos[u])^3,
  (a*rho^4*(a^2 - p^2 - v^2)*Sin[u])/
   (a^2 + p^2 + v^2 - 2*a*p*Cos[u])^3,
  (2*a*rho^4*v*(a - p*Cos[u]))/
   (a^2 + p^2 + v^2 - 2*a*p*Cos[u])^3}

(*invertedtorus *)

gcurvature[invertedcircularcone[{p_,0,0},rho_,a_,b_]][u_,v_]:=
(2*b^2*p*(p^2 + a^2*v^2 + b^2*v^2)*Cos[u])/
  (a*(a^2 + b^2)*rho^4*v)

mcurvature[invertedcircularcone[{p_,0,0},rho_,a_,b_]][u_,v_]:=
(b*(p^2 + a^2*v^2 + b^2*v^2 + 2*a*p*v*Cos[u]))/
  (2*a*Sqrt[a^2 + b^2]*rho^2*v)

unitnormal[invertedcircularcone[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(-4*b*p + 4*a*b*Cos[u] + a^2*Cos[u - v] + 
     b^2*Cos[u - v] + p^2*Cos[u - v] - 4*a*p*Cos[v] + 
     a^2*Cos[u + v] + b^2*Cos[u + v] + p^2*Cos[u + v])/
   (2*(a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
       2*a*b*Cos[v] - b*p*Cos[u + v])),
  ((2*a*b + a^2*Cos[v] + b^2*Cos[v] - p^2*Cos[v])*
     Sin[u])/
   (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
     2*a*b*Cos[v] - b*p*Cos[u + v]),
  ((-a^2 + b^2 - p^2 + 2*a*p*Cos[u])*Sin[v])/
   (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
     2*a*b*Cos[v] - b*p*Cos[u + v])}

surfacenormal[invertedcircularcone[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(b*rho^4*(a + b*Cos[v])*
      (-4*b*p + 4*a*b*Cos[u] + a^2*Cos[u - v] + 
        b^2*Cos[u - v] + p^2*Cos[u - v] - 4*a*p*Cos[v] + 
   a^2*Cos[u + v] + b^2*Cos[u + v] + p^2*Cos[u + v]))/
      (2*(a^2 + b^2 + p^2 - 2*a*p*Cos[u] - 
   b*p*Cos[u - v] + 2*a*b*Cos[v] - b*p*Cos[u + v])^3),
      (b*rho^4*(a + b*Cos[v])*
 (-2*a*b - a^2*Cos[v] - b^2*Cos[v] + p^2*Cos[v])*
      Sin[u])/
    (-a^2 - b^2 - p^2 + 2*a*p*Cos[u] + b*p*Cos[u - v] - 
       2*a*b*Cos[v] + b*p*Cos[u + v])^3,
   (b*rho^4*(-a^2 + b^2 - p^2 + 2*a*p*Cos[u])*
      (a + b*Cos[v])*Sin[v])/
    (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
       2*a*b*Cos[v] - b*p*Cos[u + v])^3}

(*invertedtorus *)

gcurvature[invertedtorus[{p_,0,0},rho_,a_,b_]][u_,v_]:=
((a^2 - b^2 + p^2 - 2*a*p*Cos[u])*
    (-2*a*b - a^2*Cos[v] - b^2*Cos[v] + p^2*Cos[v]))/
  (b*rho^4*(a + b*Cos[v]))

mcurvature[invertedtorus[{p_,0,0},rho_,a_,b_]][u_,v_]:=
(a^3 - 3*a*b^2 + a*p^2 - 2*a^2*p*Cos[u] - 
    a*b*p*Cos[u - v] - 2*b^3*Cos[v] + 2*b*p^2*Cos[v] - 
    a*b*p*Cos[u + v])/(2*b*rho^2*(a + b*Cos[v]))

unitnormal[invertedtorus[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(-4*b*p + 4*a*b*Cos[u] + a^2*Cos[u - v] + 
     b^2*Cos[u - v] + p^2*Cos[u - v] - 4*a*p*Cos[v] + 
     a^2*Cos[u + v] + b^2*Cos[u + v] + p^2*Cos[u + v])/
   (2*(a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
       2*a*b*Cos[v] - b*p*Cos[u + v])),
  ((2*a*b + a^2*Cos[v] + b^2*Cos[v] - p^2*Cos[v])*
     Sin[u])/
   (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
     2*a*b*Cos[v] - b*p*Cos[u + v]),
  ((-a^2 + b^2 - p^2 + 2*a*p*Cos[u])*Sin[v])/
   (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
     2*a*b*Cos[v] - b*p*Cos[u + v])}

surfacenormal[invertedtorus[{p_,0,0},rho_,a_,b_]][u_,v_]:=
{(b*rho^4*(a + b*Cos[v])*
      (-4*b*p + 4*a*b*Cos[u] + a^2*Cos[u - v] + 
        b^2*Cos[u - v] + p^2*Cos[u - v] - 4*a*p*Cos[v] + 
        a^2*Cos[u + v] + b^2*Cos[u + v] + p^2*Cos[u + v])
      )/(2*(a^2 + b^2 + p^2 - 2*a*p*Cos[u] - 
         b*p*Cos[u - v] + 2*a*b*Cos[v] - b*p*Cos[u + v])^3),
    (b*rho^4*(a + b*Cos[v])*
      (-2*a*b - a^2*Cos[v] - b^2*Cos[v] + p^2*Cos[v])*
      Sin[u])/
    (-a^2 - b^2 - p^2 + 2*a*p*Cos[u] + b*p*Cos[u - v] - 
       2*a*b*Cos[v] + b*p*Cos[u + v])^3,
   (b*rho^4*(-a^2 + b^2 - p^2 + 2*a*p*Cos[u])*
      (a + b*Cos[v])*Sin[v])/
    (a^2 + b^2 + p^2 - 2*a*p*Cos[u] - b*p*Cos[u - v] + 
       2*a*b*Cos[v] - b*p*Cos[u + v])^3}

(*jorgemeekspolar *)

mcurvature[jorgemeekspolar[n_,a_]][u_,v_]:=
0

(*kuen *)

gcurvature[kuen][u_,v_]:=
-1

gcurvature[kuentcheb[a_]][u_,v_]:=
-1/a^2

mcurvature[kuen][u_,v_]:=
((-8 + 24*u^2 - 3*u^4 - 24*u^2*Cos[2*v] + 
      4*u^4*Cos[2*v] - u^4*Cos[4*v])*Csc[v/2]*Sec[v/2])/
  (32*u*(2 - u^2 + u^2*Cos[2*v]))

mcurvature[kuentcheb[a_]][u_,v_]:=
((-3 + 24*u^2 - 8*u^4 - 4*Cosh[2*v] + 24*u^2*Cosh[2*v] - 
      Cosh[4*v])*Sech[v])/
  (16*a*u*(1 - 2*u^2 + Cosh[2*v]))

unitnormal[kuen][u_,v_]:=
{(4*u*Cos[u] - 2*u*Cos[u - 2*v] - 2*u*Cos[u + 2*v] - 
     4*Sin[u] + 2*u^2*Sin[u] - u^2*Sin[u - 2*v] - 
     u^2*Sin[u + 2*v])/(2*(-2 - u^2 + u^2*Cos[2*v])),
  (4*Cos[u] - 2*u^2*Cos[u] + u^2*Cos[u - 2*v] + 
     u^2*Cos[u + 2*v] + 4*u*Sin[u] - 2*u*Sin[u - 2*v] - 
     2*u*Sin[u + 2*v])/(2*(-2 - u^2 + u^2*Cos[2*v])),
  (2*u*Sin[2*v])/(-2 - u^2 + u^2*Cos[2*v])}

unitnormal[kuentcheb[a_]][u_,v_]:=
{(-4*u*Cos[u] + Sin[u] - 2*u^2*Sin[u] + Cosh[2*v]*Sin[u])/
   (1 + 2*u^2 + Cosh[2*v]),
  (-Cos[u] + 2*u^2*Cos[u] - Cos[u]*Cosh[2*v] - 
     4*u*Sin[u])/(1 + 2*u^2 + Cosh[2*v]),
  (4*u*Sinh[v])/(1 + 2*u^2 + Cosh[2*v])}

surfacenormal[kuen][u_,v_]:=
{(2*u*(2 - u^2 + u^2*Cos[2*v])*
      (-4*u*Cos[u] + 2*u*Cos[u - 2*v] + 
        2*u*Cos[u + 2*v] + 4*Sin[u] - 2*u^2*Sin[u] + 
        u^2*Sin[u - 2*v] + u^2*Sin[u + 2*v]))/
    (2 + u^2 - u^2*Cos[2*v])^3,
   (2*u*(2 - u^2 + u^2*Cos[2*v])*
      (4*Cos[u] - 2*u^2*Cos[u] + u^2*Cos[u - 2*v] + 
        u^2*Cos[u + 2*v] + 4*u*Sin[u] - 
        2*u*Sin[u - 2*v] - 2*u*Sin[u + 2*v]))/
    (-2 - u^2 + u^2*Cos[2*v])^3,
   (8*u^2*(2 - u^2 + u^2*Cos[2*v])*Sin[2*v])/
    (-2 - u^2 + u^2*Cos[2*v])^3}

surfacenormal[kuentcheb[a_]][u_,v_]:=
{(4*a^2*u*Cosh[v]*(1 - 2*u^2 + Cosh[2*v])*
      (-4*u*Cos[u] + Sin[u] - 2*u^2*Sin[u] + 
        Cosh[2*v]*Sin[u]))/(1 + 2*u^2 + Cosh[2*v])^3,
   (4*a^2*u*Cosh[v]*(-1 + 2*u^2 - Cosh[2*v])*
      (Cos[u] - 2*u^2*Cos[u] + Cos[u]*Cosh[2*v] + 
        4*u*Sin[u]))/(1 + 2*u^2 + Cosh[2*v])^3,
   (8*a^2*u^2*(1 - 2*u^2 + Cosh[2*v])*Sinh[2*v])/
    (1 + 2*u^2 + Cosh[2*v])^3}


(*menn *)

gcurvature[menn[a_]][u_,v_]:=
-4*(u^2 + 6*a*u^2 + v)/
  (1 + u^4 + 16*a^2*u^6 - 4*u^2*v + 16*a*u^4*v + 4*v^2 + 4*u^2*v^2)^2

mcurvature[menn[a_]][u_,v_]:=
(-1 + 6*a*u^2 - 2*a*u^6 - 16*a^2*u^6 + v - 3*u^4*v - 
    24*a*u^4*v + 24*a*u^2*v^2 + 4*v^3)/
  (1 + u^4 + 16*a^2*u^6 - 4*u^2*v + 16*a*u^4*v + 4*v^2 + 4*u^2*v^2)^(3/2)

unitnormal[menn[a_]][u_,v_]:=
{(-2*(2*a*u^3 + u*v))/
   (1 + u^4 + 16*a^2*u^6 - 4*u^2*v + 16*a*u^4*v + 4*v^2 + 4*u^2*v^2)^(1/2),
  (-u^2 + 2*v)/
   (1 + u^4 + 16*a^2*u^6 - 4*u^2*v + 16*a*u^4*v + 4*v^2 + 4*u^2*v^2)^(1/2),
   (1 + u^4 + 16*a^2*u^6 - 4*u^2*v + 16*a*u^4*v + 4*v^2 + 4*u^2*v^2)^(-1/2)}

surfacenormal[menn[a_]][u_,v_]:=
{-4*a*u^3 - 2*u*v,-u^2 + 2*v,1}


(*mercatorellipsoid *)

gcurvature[mercatorellipsoid[a_,b_,c_]][u_,v_]:=
4*a^2*b^2*c^2*Cosh[v]^4/
  (-(a^2*b^2) + a^2*c^2 + b^2*c^2 - a^2*c^2*Cos[2*u] +
b^2*c^2*Cos[2*u] + a^2*b^2*Cosh[2*v])^2

gcurvature[mercatorellipsoid[a_,b_,c_]][u_,v_]:=
(4*a^2*b^2*c^2*Cosh[v]^4)/
  (-(a^2*b^2) + a^2*c^2 + b^2*c^2 - a^2*c^2*Cos[2*u] + 
     b^2*c^2*Cos[2*u] + a^2*b^2*Cosh[2*v])^2

unitnormal[mercatorellipsoid[a_,b_,c_]][u_,v_]:=
{(b*c*Cos[u]*Sech[v]^3)/
   (b^2*c^2*Cos[u]^2*Sech[v]^6 + 
      a^2*c^2*Sech[v]^6*Sin[u]^2 + 
      a^2*b^2*Sech[v]^4*Tanh[v]^2)^(1/2),
  (a*c*Sech[v]^3*Sin[u])/
   (b^2*c^2*Cos[u]^2*Sech[v]^6 + 
      a^2*c^2*Sech[v]^6*Sin[u]^2 + 
      a^2*b^2*Sech[v]^4*Tanh[v]^2)^(1/2),
  (a*b*Sech[v]^2*Tanh[v])/
   (b^2*c^2*Cos[u]^2*Sech[v]^6 + 
      a^2*c^2*Sech[v]^6*Sin[u]^2 + 
      a^2*b^2*Sech[v]^4*Tanh[v]^2)^(1/2)}

surfacenormal[mercatorellipsoid[a_,b_,c_]][u_,v_]:=
{b*c*Cos[u]*Sech[v]^3,a*c*Sech[v]^3*Sin[u],
   a*b*Sech[v]^2*Tanh[v]}


(*moebiusstrip *)

gcurvature[moebiusstrip[a_]][u_,v_]:=
-4/(a^2*(4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u])^2)

mcurvature[moebiusstrip[a_]][u_,v_]:=
(-2*(2 + 2*v^2 + 4*v*Cos[u/2] + v^2*Cos[u])*Sin[u/2])/
  (a*(4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u])^(3/2))

unitnormal[moebiusstrip[a_]][u_,v_]:=
{((-(v*Cos[u/2]) + 2*Cos[u] + v*Cos[(3*u)/2])*Sin[u/2])/
   (4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u])^(1/2),
  (v + 2*Cos[u/2] + 2*v*Cos[u] - 2*Cos[(3*u)/2] - 
     v*Cos[2*u])/
   (2*(4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u])^(1/2)),
  (-2*Cos[u/2]*(1 + v*Cos[u/2]))/
   (4 + 3*v^2 + 8*v*Cos[u/2] + 2*v^2*Cos[u])^(1/2)}

surfacenormal[moebiusstrip[a_]][u_,v_]:=
{(a^2*(-(v*Cos[u/2]) + 2*Cos[u] + v*Cos[(3*u)/2])*
      Sin[u/2])/2,(a^2*
      (v + 2*Cos[u/2] + 2*v*Cos[u] - 2*Cos[(3*u)/2] - 
        v*Cos[2*u]))/4,-(a^2*Cos[u/2]*(1 + v*Cos[u/2]))}


(*monkeysaddle *)

gcurvature[monkeysaddle][u_,v_]:=
-36*(u^2 + v^2)/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)^2

mcurvature[monkeysaddle][u_,v_]:=
(27*u*(-u^4 + 2*u^2*v^2 + 3*v^4))/
  (1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)^(3/2)

unitnormal[monkeysaddle][u_,v_]:=
{(3*(-u^2 + v^2))/
   (1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)^(1/2),
  (6*u*v)/(1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)^(1/2),
  (1 + 9*u^4 + 18*u^2*v^2 + 9*v^4)^(-1/2)}

surfacenormal[monkeysaddle][u_,v_]:=
{3*(-u^2 + v^2),6*u*v,1}

gcurvature[monkeysaddlepolar[n_]][r_,theta_]:=
-(n - 1)^2*n^2*r^(2*n)/(r^2 + n^2*r^(2*n))^2

mcurvature[monkeysaddlepolar[n_]][r_,theta_]:=
-(n - 1)*n^3*r^(3*n - 1)*Sin[n*theta]/(2*(r^2 + n^2*r^(2*n))^(3/2))

(*
Simplify[PowerExpand[Simplify[PowerExpand[Factor//@Simplify[
Factor//@Simplify[k1[monkeysaddlepolar[3]][r,theta]]],r]]]]
*)

k1[monkeysaddlepolar[n_]][r_,theta_]:=
(3*r*(Sqrt[16 + 144*r^4 + 162*r^8 - 162*r^8*Cos[6*theta]] -
18*r^4*Sin[3*theta]))/(2*(1 + 9*r^4)^(3/2))

k2[monkeysaddlepolar[n_]][r_,theta_]:=
(-3*r*(Sqrt[16 + 144*r^4 + 162*r^8 - 162*r^8*Cos[6*theta]] +
18*r^4*Sin[3*theta]))/(2*(1 + 9*r^4)^(3/2))

unitnormal[monkeysaddlepolar[n_]][r_,theta_]:=
{-n*r^n*Sin[(-1 + n)*theta]/Sqrt[r^2 + n^2*r^(2*n)],
  -n*r^n*Cos[(-1 + n)*theta]/Sqrt[r^2 + n^2*r^(2*n)],
 r/Sqrt[r^2 + n^2*r^(2*n)]}

unitnormal[monkeysaddlepolar[n_]][r_,theta_]:=
{n*r^n*Sin[theta - n*theta],-n*r^n*Cos[theta - n*theta],r}

(*monge *)

gcurvature[monge[h_]][u_,v_]:=
(-Derivative[1,1][h][u,v]^2 + 
    Derivative[0,2][h][u,v]*Derivative[2,0][h][u,v])/
  (1 + Derivative[0,1][h][u,v]^2 + 
     Derivative[1,0][h][u,v]^2)^2

mcurvature[monge[h_]][u_,v_]:=
(Derivative[0,2][h][u,v]*(1 + Derivative[1,0][h][u,v]^2) - 
    2*Derivative[0,1][h][u,v]*
     Derivative[1,0][h][u,v]*Derivative[1,1][h][u,v] +
    (1 + Derivative[0,1][h][u,v]^2)*Derivative[2,0][h][u,v])/
       (2*(1 + Derivative[0,1][h][u,v]^2 + 
       Derivative[1,0][h][u,v]^2)^(3/2))

unitnormal[monge[h_]][u_,v_]:=
{-(Derivative[1,0][h][u,v]/
     (1 + Derivative[0,1][h][u,v]^2 + 
        Derivative[1,0][h][u,v]^2)^(1/2)),
  -(Derivative[0,1][h][u,v]/
     (1 + Derivative[0,1][h][u,v]^2 + 
        Derivative[1,0][h][u,v]^2)^(1/2)),
  (1 + Derivative[0,1][h][u,v]^2 + 
     Derivative[1,0][h][u,v]^2)^(-1/2)}

surfacenormal[monge[h_]][u_,v_]:=
{-Derivative[1,0][h][u,v],-Derivative[0,1][h][u,v],1}

gcurvature[mongepolar[g_]][r_,theta_]:=
(-Derivative[0,1][g][r,theta]^2 + 
    2*r*Derivative[0,1][g][r,theta]*
     Derivative[1,1][g][r,theta] - 
    r^2*Derivative[1,1][g][r,theta]^2 + 
    r^2*Derivative[0,2][g][r,theta]*
     Derivative[2,0][g][r,theta] + 
    r^3*Derivative[1,0][g][r,theta]*
     Derivative[2,0][g][r,theta])/
  (r^2 + Derivative[0,1][g][r,theta]^2 + 
     r^2*Derivative[1,0][g][r,theta]^2)^2

mcurvature[mongepolar[g_]][r_,theta_]:=
(r*Derivative[0,2][g][r,theta] + 
    r^2*Derivative[1,0][g][r,theta] + 
    2*Derivative[0,1][g][r,theta]^2*
     Derivative[1,0][g][r,theta] + 
    r*Derivative[0,2][g][r,theta]*
     Derivative[1,0][g][r,theta]^2 + 
    r^2*Derivative[1,0][g][r,theta]^3 - 
    2*r*Derivative[0,1][g][r,theta]*
     Derivative[1,0][g][r,theta]*
     Derivative[1,1][g][r,theta] + 
    r^3*Derivative[2,0][g][r,theta] + 
    r*Derivative[0,1][g][r,theta]^2*
     Derivative[2,0][g][r,theta])/
  (2*(r^2 + Derivative[0,1][g][r,theta]^2 + 
       r^2*Derivative[1,0][g][r,theta]^2)^(3/2))

unitnormal[mongepolar[g_]][r_,theta_]:=
{(Sin[theta]*Derivative[0,1][g][r,theta] - 
     r*Cos[theta]*Derivative[1,0][g][r,theta])/
   (r^2 + Derivative[0,1][g][r,theta]^2 + 
      r^2*Derivative[1,0][g][r,theta]^2)^(1/2),
  (-(Cos[theta]*Derivative[0,1][g][r,theta]) - 
     r*Sin[theta]*Derivative[1,0][g][r,theta])/
   (r^2 + Derivative[0,1][g][r,theta]^2 + 
      r^2*Derivative[1,0][g][r,theta]^2)^(1/2),
  (r*Cos[theta]^2 + r*Sin[theta]^2)/
   (r^2 + Derivative[0,1][g][r,theta]^2 + 
      r^2*Derivative[1,0][g][r,theta]^2)^(1/2)}

surfacenormal[mongepolar[g_]][r_,theta_]:=
{Sin[theta]*Derivative[0,1][g][r,theta] - 
   r*Cos[theta]*Derivative[1,0][g][r,theta],
  -(Cos[theta]*Derivative[0,1][g][r,theta]) - 
   r*Sin[theta]*Derivative[1,0][g][r,theta],r}


(*paraboliccyclide *)

gcurvature[paraboliccyclide[a_,k_]][u_,v_]:=
(64*a^2)/((-8*a^2 + k - u^2)*(8*a^2 + k + v^2))

mcurvature[paraboliccyclide[a_,k_]][u_,v_]:=
(4*a*(2*k - u^2 + v^2))/
  ((8*a^2 - k + u^2)*(8*a^2 + k + v^2))

unitnormal[paraboliccyclide[a_,k_]][u_,v_]:=
{(8*a*u)/(16*a^2 + u^2 + v^2), 
 (-8*a*v)/(16*a^2 + u^2 + v^2), 
  -((-16*a^2 + u^2 + v^2)/(16*a^2 + u^2 + v^2))}

(*paraboloid *)

gcurvature[paraboloid[a_,b_,c_]][u_,v_]:=
(4*a*b - c^2)/
  (1 + 4*a^2*u^2 + c^2*u^2 + 4*a*c*u*v + 4*b*c*u*v +
4*b^2*v^2 + c^2*v^2)^2

mcurvature[paraboloid[a_,b_,c_]][u_,v_]:=
(a + b + 4*a^2*b*u^2 - a*c^2*u^2 + 4*a*b*c*u*v - 
    c^3*u*v + 4*a*b^2*v^2 - b*c^2*v^2)/
  (1 + 4*a^2*u^2 + c^2*u^2 + 4*a*c*u*v + 4*b*c*u*v + 
     4*b^2*v^2 + c^2*v^2)^(3/2)

unitnormal[paraboloid[a_,b_,c_]][u_,v_]:=
{(-2*a*u - c*v)/
   (1 + 4*a^2*u^2 + c^2*u^2 + 4*a*c*u*v + 4*b*c*u*v + 
      4*b^2*v^2 + c^2*v^2)^(1/2),
  (-(c*u) - 2*b*v)/
   (1 + 4*a^2*u^2 + c^2*u^2 + 4*a*c*u*v + 4*b*c*u*v + 
      4*b^2*v^2 + c^2*v^2)^(1/2),
  (1 + 4*a^2*u^2 + c^2*u^2 + 4*a*c*u*v + 4*b*c*u*v + 
     4*b^2*v^2 + c^2*v^2)^(-1/2)}

surfacenormal[paraboloid[a_,b_,c_]][u_,v_]:=
{-2*a*u - c*v,-(c*u) - 2*b*v,1}

gcurvature[perturbedmspolar[n_,a_]][r_,theta_]:=
(4*a^2*r^4 - (-1 + n)^2*n^2*r^(2*n))/
  (r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
     4*a*n*r^(2 + n)*Sin[n*theta])^2

mcurvature[perturbedmspolar[n_,a_]][r_,theta_]:=
(2*a*r^2*(2*r^2 + 4*a^2*r^4 - n^2*(-3 + 2*n)*r^(2*n)) - 
    n*r^n*(4*a^2*(-3 + n)*r^4 + (-1 + n)*n^2*r^(2*n))*
     Sin[n*theta])/
  (2*r*(r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
       4*a*n*r^(2 + n)*Sin[n*theta])^(3/2))

k1[perturbedmspolar[n_,a_]][r_,theta_]:=
(4*a*r^4 + 8*a^3*r^6 + 6*a*n^2*r^(2*(n + 1)) - 
    4*a*n^3*r^(2*(n + 1)) + n^3*r^(3*n)*Sin[n*theta] - 
    n^4*r^(3*n)*Sin[n*theta] + 
    12*a^2*n*r^(4 + n)*Sin[n*theta] - 
    4*a^2*n^2*r^(4 + n)*Sin[n*theta] + 
    r*Sqrt[-4*(4*a^2*r^4 - (-1 + n)^2*n^2*r^(2*n))*
        (r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
          4*a*n*r^(2 + n)*Sin[n*theta]) + 
       1/r^2*(-2*a*r^2*(2*r^2 + 4*a^2*r^4 - 
              n^2*(-3 + 2*n)*r^(2*n)) + 
           n*r^n*(4*a^2*(-3 + n)*r^4 + 
              (-1 + n)*n^2*r^(2*n))*Sin[n*theta])^2])/
  (2*r*(r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
       4*a*n*r^(2 + n)*Sin[n*theta])^(3/2))
           
k2[perturbedmspolar[n_,a_]][r_,theta_]:=
(4*a*r^4 + 8*a^3*r^6 + 6*a*n^2*r^(2*(n + 1)) - 
    4*a*n^3*r^(2*(n + 1)) + n^3*r^(3*n)*Sin[n*theta] - 
    n^4*r^(3*n)*Sin[n*theta] + 
    12*a^2*n*r^(4 + n)*Sin[n*theta] - 
    4*a^2*n^2*r^(4 + n)*Sin[n*theta] -
    r*Sqrt[-4*(4*a^2*r^4 - (-1 + n)^2*n^2*r^(2*n))*
        (r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
          4*a*n*r^(2 + n)*Sin[n*theta]) + 
       1/r^2*(-2*a*r^2*(2*r^2 + 4*a^2*r^4 - 
              n^2*(-3 + 2*n)*r^(2*n)) + 
           n*r^n*(4*a^2*(-3 + n)*r^4 + 
              (-1 + n)*n^2*r^(2*n))*Sin[n*theta])^2])/
  (2*r*(r^2 + 4*a^2*r^4 + n^2*r^(2*n) + 
       4*a*n*r^(2 + n)*Sin[n*theta])^(3/2))
       
(*plucker *)

gcurvature[plucker][u_,v_]:=
-(u^2 - v^2)^2*(u^2 + v^2)^2/
    (u^4 + u^6 - 2*u^2*v^2 + 3*u^4*v^2 + v^4 +
3*u^2*v^4 + v^6)^2

mcurvature[plucker][u_,v_]:=
(-2*u*v*(u^2 + v^2)^(5/2))/
  (u^4 + u^6 - 2*u^2*v^2 + 3*u^4*v^2 + v^4 + 3*u^2*v^4 + 
     v^6)^(3/2)

unitnormal[plucker][u_,v_]:=
{2*v*(u^4 - v^4)/
    ((u^2 + v^2)^(3/2)*Sqrt[u^6 + v^4*(4 + v^2) + u^2*v^2*(-8 + 3*v^2) + 
       u^4*(4 + 3*v^2)]),
 2*u*(-u^4 + v^4)/
   ((u^2 + v^2)^(3/2)*Sqrt[u^6 + v^4*(4 + v^2) + u^2*v^2*(-8 + 3*v^2) + 
       u^4*(4 + 3*v^2)]),
(u^2 + v^2)^(3/2)/
   Sqrt[u^6 + v^4*(4 + v^2) + u^2*v^2*(-8 + 3*v^2) + u^4*(4 + 3*v^2)]}

surfacenormal[plucker][u_,v_]:=
{(v*(u^2 - v^2))/(u^2 + v^2)^2,(u*(-u^2 + v^2))/(u^2 + v^2)^2,1}

gcurvature[pluckerpolar[n_]][r_,theta_]:=
-4*n^2*Cos[n*theta]^2/
  (n^2 + 2*r^2 + n^2*Cos[2*n*theta])^2

gcurvature[pluckerpolar[m_,n_,a_]][r_,theta_]:=
2*(1 - m)*r^(2*m)*(-m^2 - n^2 + 2*m*n^2 + m^2*Cos[2*n*theta] 
- n^2*Cos[2*n*theta])/
  a^2*(2*r^2 + m^2*r^(2*m) + n^2*r^(2*m) - m^2*r^(2*m)*Cos[2*n*theta] + 
       n^2*r^(2*m)*Cos[2*n*theta])^2


mcurvature[pluckerpolar[n_]][r_,theta_]:=
-((Sqrt[2]*n^2*r*Sin[n*theta])/
    (n^2 + 2*r^2 + n^2*Cos[2*n*theta])^(3/2))

mcurvature[pluckerpolar[m_,n_,a_]][r_,theta_]:=
a^5*r^(-1 + m)*(2*m^2*r^2 - 2*n^2*r^2 + m^3*r^(2*m) + m*n^2*r^(2*m) - 
2*m^2*n^2*r^(2*m) - m^3*r^(2*m)*Cos[2*n*theta] +
m*n^2*r^(2*m)*Cos[2*n*theta])*Sin[n*theta]/
Sqrt[2]*(a^4*(2*r^2 + m^2*r^(2*m) + n^2*r^(2*m) - m^2*r^(2*m)*Cos[2*n*theta] + 
n^2*r^(2*m)*Cos[2*n*theta]))^(3/2)


unitnormal[pluckerpolar[n_]][r_,theta_]:=
{(Sqrt[2]*n*Cos[n*theta]*Sin[theta])/
   (n^2 + 2*r^2 + n^2*Cos[2*n*theta])^(1/2),
  -((Sqrt[2]*n*Cos[theta]*Cos[n*theta])/
     (n^2 + 2*r^2 + n^2*Cos[2*n*theta])^(1/2)),
  (Sqrt[2]*r)/(n^2 + 2*r^2 + n^2*Cos[2*n*theta])^(1/2)}

unitnormal[pluckerpolar[m_,n_]][r_,theta_]:=
{(Sqrt[2]*r^m*(n*Cos[n*theta]*Sin[theta] - 
       m*Cos[theta]*Sin[n*theta]))/
   (2*r^2 + m^2*r^(2*m) + n^2*r^(2*m) - 
      m^2*r^(2*m)*Cos[2*n*theta] + 
      n^2*r^(2*m)*Cos[2*n*theta])^(1/2),
  -((Sqrt[2]*r^m*(n*Cos[theta]*Cos[n*theta] + 
         m*Sin[theta]*Sin[n*theta]))/
     (2*r^2 + m^2*r^(2*m) + n^2*r^(2*m) - 
        m^2*r^(2*m)*Cos[2*n*theta] + 
        n^2*r^(2*m)*Cos[2*n*theta])^(1/2)),
  (Sqrt[2]*r)/
   (2*r^2 + m^2*r^(2*m) + n^2*r^(2*m) - 
      m^2*r^(2*m)*Cos[2*n*theta] + 
      n^2*r^(2*m)*Cos[2*n*theta])^(1/2)}

surfacenormal[plucker][u_,v_]:=
{(v*(u^2 - v^2))/(u^2 + v^2)^2,
   (u*(-u^2 + v^2))/(u^2 + v^2)^2,1}

surfacenormal[pluckerpolar[n_]][r_,theta_]:=
{n*Cos[n*theta]*Sin[theta],-n*Cos[theta]*Cos[n*theta],r}


surfacenormal[pluckerpolar[m_,n_]][r_,theta_]:=
{n*r^m*Cos[n*theta]*Sin[theta] - m*r^m*Cos[theta]*Sin[n*theta],
 -n*r^m*Cos[theta]*Cos[n*theta] -  m*r^m*Sin[theta]*Sin[n*theta],r}

(*pseudocrosscap *)

gcurvature[pseudocrosscap][u_,v_]:=
-32*(2 + Cos[2*v])*Sin[v]^4/
  ((-1 + u^2)*(-5 - 10*u^2 - Cos[2*v] +
15*u^2*Cos[2*v] - 4*Cos[4*v] - 6*u^2*Cos[4*v] +
u^2*Cos[6*v])^2)

mcurvature[pseudocrosscap][u_,v_]:=
((17 + 19*u^2 - 8*Cos[2*v] + 8*u^2*Cos[2*v] + 
      7*Cos[4*v] - 27*u^2*Cos[4*v] - 4*Cos[6*v])*Sin[v])/
  (Sqrt[2]*(-1 + u^2)*(5 + 10*u^2 + Cos[2*v] - 
       15*u^2*Cos[2*v] + 4*Cos[4*v] + 6*u^2*Cos[4*v] - 
       u^2*Cos[6*v])^(1/2)*
    (-5 - 10*u^2 - Cos[2*v] + 15*u^2*Cos[2*v] - 
      4*Cos[4*v] - 6*u^2*Cos[4*v] + u^2*Cos[6*v]))

unitnormal[pseudocrosscap][u_,v_]:=
{(2*(-1 + u^2)*Cos[2*v])/
   (Cos[v]^2 - 2*u^2*Cos[v]^2 + u^4*Cos[v]^2 + 
      4*Cos[2*v]^2 - 8*u^2*Cos[2*v]^2 + 
      4*u^4*Cos[2*v]^2 + 16*u^2*Sin[v]^6 - 
      32*u^4*Sin[v]^6 + 16*u^6*Sin[v]^6)^(1/2),
  ((1 - u^2)*Cos[v])/
   (Cos[v]^2 - 2*u^2*Cos[v]^2 + u^4*Cos[v]^2 + 
      4*Cos[2*v]^2 - 8*u^2*Cos[2*v]^2 + 
      4*u^4*Cos[2*v]^2 + 16*u^2*Sin[v]^6 - 
      32*u^4*Sin[v]^6 + 16*u^6*Sin[v]^6)^(1/2),
  (4*u*(1 - u^2)*Sin[v]^3)/
   (Cos[v]^2 - 2*u^2*Cos[v]^2 + u^4*Cos[v]^2 + 
      4*Cos[2*v]^2 - 8*u^2*Cos[2*v]^2 + 
      4*u^4*Cos[2*v]^2 + 16*u^2*Sin[v]^6 - 
      32*u^4*Sin[v]^6 + 16*u^6*Sin[v]^6)^(1/2)}

surfacenormal[pseudocrosscap][u_,v_]:=
{2*(-1 + u^2)*Cos[2*v],(1 - u^2)*Cos[v],4*u*(1 - u^2)*Sin[v]^3}

(*pseudosphere *)

gcurvature[pseudosphere[a_]][u_,v_]:=
-a^(-2)

gcurvature[pseudospheretcheb[a_]][u_,v_]:=
-a^(-2)

mcurvature[pseudosphere[a_]][u_,v_]:=
-Cot[2*v]/a


mcurvature[pseudospheretcheb[a_]][u_,v_]:=
(a*(-3 + Cosh[2*u])*Sech[u]^2)/
  (4*Sqrt[a^4*Sech[u]^2*Tanh[u]^2])
unitnormal[pseudosphere[a_]][u_,v_]:=
{Cos[u]*Cos[v],Cos[v]*Sin[u],-Sin[v]}

unitnormal[pseudospheretcheb[a_]][u_,v_]:=
{-Cos[v]*Tanh[u],-Sin[v]*Tanh[u],-Sech[u]}

surfacenormal[pseudosphere[a_]][u_,v_]:=
{a^2*Cos[u]*Cos[v]^2,a^2*Cos[v]^2*Sin[u],-a^2*Sin[2*v]/2}

surfacenormal[pseudospheretcheb[a_]][u_,v_]:=
{-a^2*Cos[v]*Sech[u]*Tanh[u]^2,-a^2*Sech[u]*Sin[v]*Tanh[u]^2,
 -a^2*Sech[u]^2*Tanh[u]}


(*richmondpolar *)

gcurvature[richmondpolar[n_][t_]][r_,theta_]:=
(-16*(n + 1)^2*r^(4 + 2*n))/(1 + r^(2 + 2*n))^4

mcurvature[richmondpolar[n_][t_]][r_,theta_]:=
0


unitnormal[richmondpolar[n_][t_]][r_,theta_]:=
{(2*r^(n + 1)*Cos[(n + 1)*theta])/(1 + r^(2 + 2*n)),
  (2*r^(n + 1)*Sin[(n + 1)*theta])/(1 + r^(2 + 2*n)),
  (-1 + r^(2 + 2*n))/(1 + r^(2 + 2*n))}

surfacenormal[richmondpolar[n_][t_]][r_,theta_]:=
{(r^(-2 + n)*(1 + r^(2 + 2*n))*Cos[(n + 1)*theta])/2,
   (r^(-2 + n)*(1 + r^(2 + 2*n))*Sin[(n + 1)*theta])/2,
   -1/(4*r^3) + r^(1 + 4*n)/4}


(*roman *)

gcurvature[roman[a_]][u_,v_]:=
8192*Sec[v]^4*(-2*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2 +
 Cos[v]^4*Sin[2*u]^2*Sin[v]^2 +
 3*Cos[v]^4*Cos[2*v]*Sin[2*u]^2*Sin[v]^2)/
  (a^4*(52 + 12*Cos[4*u] + 3*Cos[4*u - 6*v] +
13*Cos[4*u - 2*v] + 10*Cos[4*(u - v)] -
26*Cos[2*v] + 44*Cos[4*v] - 6*Cos[6*v] +
10*Cos[4*(u + v)] + 13*Cos[4*u + 2*v] +
3*Cos[4*u + 6*v])^2)

mcurvature[roman[a_]][u_,v_]:=
(16*Sqrt[2]*Cos[v]^2*(-94 - 18*Cos[4*u] - 
      12*Cos[4*u - 2*v] - 3*Cos[4*(u - v)] + 
      24*Cos[2*v] - 42*Cos[4*v] - 3*Cos[4*(u + v)] - 
      12*Cos[4*u + 2*v])*Sin[2*u]*Sin[v])/
  (a^2*(52 + 12*Cos[4*u] + 3*Cos[4*u - 6*v] + 
       13*Cos[4*u - 2*v] + 10*Cos[4*(u - v)] - 
       26*Cos[2*v] + 44*Cos[4*v] - 6*Cos[6*v] + 
       10*Cos[4*(u + v)] + 13*Cos[4*u + 2*v] + 
       3*Cos[4*u + 6*v])^(3/2))

unitnormal[roman[a_]][u_,v_]:=
{a^4*Sin[4*v]/
   (16*a^8*Cos[u]^2*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2 + 
      16*a^8*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2*Sin[u]^2 + 
      4*a^8*Cos[u]^4*Sin[u]^2*Sin[2*v]^4 + 
      4*a^8*Cos[u]^2*Sin[u]^4*Sin[2*v]^4 + a^8*Sin[4*v]^2
      )^(1/2),(2*(-2*a^4*Cos[u]*Cos[2*u]*Cos[v]^2*
        Cos[2*v] + a^4*Cos[u]*Sin[u]^2*Sin[2*v]^2))/
   (16*a^8*Cos[u]^2*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2 + 
      16*a^8*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2*Sin[u]^2 + 
      4*a^8*Cos[u]^4*Sin[u]^2*Sin[2*v]^4 + 
      4*a^8*Cos[u]^2*Sin[u]^4*Sin[2*v]^4 +
    a^8*Sin[4*v]^2)^(1/2),(2*(2*a^4*Cos[2*u]*Cos[v]^2*Cos[2*v]*
        Sin[u] + a^4*Cos[u]^2*Sin[u]*Sin[2*v]^2))/
   (16*a^8*Cos[u]^2*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2 + 
      16*a^8*Cos[2*u]^2*Cos[v]^4*Cos[2*v]^2*Sin[u]^2 + 
      4*a^8*Cos[u]^4*Sin[u]^2*Sin[2*v]^4 + 
      4*a^8*Cos[u]^2*Sin[u]^4*Sin[2*v]^4 + a^8*Sin[4*v]^2)^(1/2)}

surfacenormal[roman[a_]][u_,v_]:=
{a^4*Sin[4*v]/4,-(a^4*Cos[u]*Cos[2*u]*Cos[v]^2*
       Cos[2*v]) + (a^4*Cos[u]*Sin[u]^2*Sin[2*v]^2)/2,
   a^4*Cos[2*u]*Cos[v]^2*Cos[2*v]*Sin[u] + 
    (a^4*Cos[u]^2*Sin[u]*Sin[2*v]^2)/2}


(*scherk *)

gcurvature[scherk[a_]][u_,v_]:=
-a^2*Sec[a*u]^2*Sec[a*v]^2/
    (Sec[a*u]^2*Sec[a*v]^2 - Tan[a*u]^2*Tan[a*v]^2)^2

mcurvature[scherk[a_]][u_,v_]:=
0

unitnormal[scherk[a_]][u_,v_]:=
{-Tan[a*u]/Sqrt[1 + Tan[a*u]^2 + Tan[a*v]^2],
  Tan[a*v]/Sqrt[1 + Tan[a*u]^2 + Tan[a*v]^2],
  (1 + Tan[a*u]^2 + Tan[a*v]^2)^(-1/2)}

surfacenormal[scherk[a_]][u_,v_]:=
{-Tan[a*u],Tan[a*v],1}


(*scherk5 *)

gcurvature[scherk5][u_,v_]:=
-1/(1 + u^2 + v^2 + u^2*v^2)

mcurvature[scherk5][u_,v_]:=
0

unitnormal[scherk5][u_,v_]:=
{-v/Sqrt[1 + v^2],-u/Sqrt[1 + u^2],
  Sqrt[1 + u^2*v^2]/(Sqrt[1 + u^2]*Sqrt[1 + v^2])}

surfacenormal[scherk5][u_,v_]:=
{-v/(Sqrt[1 + v^2]*Sqrt[1 + u^2*v^2]),
 -u/(Sqrt[1 + u^2]*Sqrt[1 + u^2*v^2]),
  1/(Sqrt[1 + u^2]*Sqrt[1 + v^2])}


(*shoe *)

gcurvature[shoe[a_,b_]][u_,v_]:=
(12*a*b*u)/(1 + 9*a^2*u^4 + 4*b^2*v^2)^2

mcurvature[shoe[a_,b_]][u_,v_]:=
(b + 3*a*u + 9*a^2*b*u^4 + 12*a*b^2*u*v^2)/
(1 + 9*a^2*u^4 + 4*b^2*v^2)^(3/2)

unitnormal[shoe[a_,b_]][u_,v_]:=
{-3*a*u^2/Sqrt[1 + 9*a^2*u^4 + 4*b^2*v^2],
 -2*b*v/Sqrt[1 + 9*a^2*u^4 + 4*b^2*v^2],
  1/Sqrt[1 + 9*a^2*u^4 + 4*b^2*v^2]}

surfacenormal[shoe[a_,b_]][u_,v_]:=
{-3*a*u^2,-2*b*v,1}


(*sinsurface *)

gcurvature[sinsurface[a_]][u_,v_]:=
4*(Sin[2*u]*Sin[2*v] - 4*Cos[u]^2*Cos[v]^2*Sin[u + v]^2)/
  (a^2*(-4 - 2*Cos[2*u] - 2*Cos[2*v] - Cos[2*u]*Cos[2*v] + 4*Cos[u + v]^4 - 
       4*Cos[2*(u + v)] - Cos[2*u]*Cos[2*(u + v)] - Cos[2*v]*Cos[2*(u + v)] - 
       Cos[2*(u + v)]^2)^2)

mcurvature[sinsurface[a_]][u_,v_]:=
((-3*Cos[u] - Cos[3*u] + 2*Cos[u + 2*v] + Cos[u + 4*v] +
Cos[3*u + 4*v])*Sin[u])/
  (a*(4 + 2*Cos[2*u] + 2*Cos[2*v] + Cos[2*u]*Cos[2*v] - 4*Cos[u + v]^4 + 4*Cos[2*(u + v)] + 
Cos[2*u]*Cos[2*(u + v)] + Cos[2*v]*Cos[2*(u + v)] + Cos[2*(u + v)]^2)^(3/2))

unitnormal[sinsurface[a_]][u_,v_]:=
{-Cos[v]*Cos[u + v]/
Sqrt[Cos[u]^2*Cos[v]^2 + Cos[u]^2*Cos[u + v]^2 + Cos[v]^2*Cos[u + v]^2],
  -Cos[u]*Cos[u + v]/
Sqrt[Cos[u]^2*Cos[v]^2 + Cos[u]^2*Cos[u + v]^2 + Cos[v]^2*Cos[u + v]^2],
  Cos[u]*Cos[v]/
Sqrt[Cos[u]^2*Cos[v]^2 + Cos[u]^2*Cos[u + v]^2 + Cos[v]^2*Cos[u + v]^2]}


(*sphere *)

gcurvature[sphere[a_]][u_,v_]:=
a^(-2)

mcurvature[sphere[a_]][u_,v_]:=
-1/a

unitnormal[sphere[a_]][u_,v_]:=
{Cos[u]*Cos[v],Cos[v]*Sin[u],Sin[v]}

surfacenormal[sphere[a_]][u_,v_]:=
{a^2*Cos[u]*Cos[v]^2,a^2*Cos[v]^2*Sin[u],a^2*Sin[2*v]/2}


(*stereographicellipsoid *)

gcurvature[stereographicellipsoid[a_,b_,c_]][u_,v_]:=
(a^2*b^2*c^2*(1 + u^2 + v^2)^4)/
  (a^2*b^2 - 2*a^2*b^2*u^2 + 4*b^2*c^2*u^2 + 
     a^2*b^2*u^4 - 2*a^2*b^2*v^2 + 4*a^2*c^2*v^2 + 
     2*a^2*b^2*u^2*v^2 + a^2*b^2*v^4)^2

mcurvature[stereographicellipsoid[a_,b_,c_]][u_,v_]:=
(a*b*c*(1 + u^2 + v^2)*
    (a^2 + b^2 - 2*a^2*u^2 + 2*b^2*u^2 + 4*c^2*u^2 + 
      a^2*u^4 + b^2*u^4 + 2*a^2*v^2 - 2*b^2*v^2 + 
      4*c^2*v^2 + 2*a^2*u^2*v^2 + 2*b^2*u^2*v^2 + 
      a^2*v^4 + b^2*v^4))/
  (2*(a^2*b^2 - 2*a^2*b^2*u^2 + 4*b^2*c^2*u^2 + 
       a^2*b^2*u^4 - 2*a^2*b^2*v^2 + 4*a^2*c^2*v^2 + 
       2*a^2*b^2*u^2*v^2 + a^2*b^2*v^4)^(3/2))

unitnormal[stereographicellipsoid[a_,b_,c_]][u_,v_]:=
{-2*b*c*u/
   ((1 + u^2 + v^2)^3*((a^2*b^2 - 2*a^2*b^2*u^2 + 
          4*b^2*c^2*u^2 + a^2*b^2*u^4 - 2*a^2*b^2*v^2 + 
          4*a^2*c^2*v^2 + 2*a^2*b^2*u^2*v^2 + a^2*b^2*v^4
          )/(1 + u^2 + v^2)^6)^(1/2)),
  -2*a*c*v/
   ((1 + u^2 + v^2)^3*((a^2*b^2 - 2*a^2*b^2*u^2 + 
          4*b^2*c^2*u^2 + a^2*b^2*u^4 - 2*a^2*b^2*v^2 + 
          4*a^2*c^2*v^2 + 2*a^2*b^2*u^2*v^2 + a^2*b^2*v^4
          )/(1 + u^2 + v^2)^6)^(1/2)),
  a*b*(1 - u^2 - v^2)/
   ((1 + u^2 + v^2)^3*((a^2*b^2 - 2*a^2*b^2*u^2 + 
          4*b^2*c^2*u^2 + a^2*b^2*u^4 - 2*a^2*b^2*v^2 + 
          4*a^2*c^2*v^2 + 2*a^2*b^2*u^2*v^2 + a^2*b^2*v^4)/
          (1 + u^2 + v^2)^6)^(1/2))}

surfacenormal[stereographicellipsoid[a_,b_,c_]][u_,v_]:=
{-8*b*c*u/(1 + u^2 + v^2)^3,
 -8*a*c*v/(1 + u^2 + v^2)^3,
  4*a*b*(1 - u^2 - v^2)/(1 + u^2 + v^2)^3}

(*stereographicsphere *)

gcurvature[stereographicsphere[a_]][u_,v_]:=
a^(-2)

mcurvature[stereographicsphere[a_]][u_,v_]:=
a^(-1)

unitnormal[stereographicsphere[a_]][u_,v_]:=
{-2*u/(1 + u^2 + v^2),-2*v/(1 + u^2 + v^2),(1 - u^2 - v^2)/(1 + u^2 + v^2)}

surfacenormal[stereographicsphere[a_]][u_,v_]:=
{-8*a^2*u/(1 + u^2 + v^2)^3,
 -8*a^2*v/(1 + u^2 + v^2)^3,
 -4*a^2*(-1 + u^2 + v^2)/(1 + u^2 + v^2)^3}

(*swallowtail *)

gcurvature[swallowtail][u_,v_]:=
0

mcurvature[swallowtail][u_,v_]:=
(1 + 4*v^2 + v^4)/(4*(u + 6*v^2)*(1 + v^2 + v^4)^(3/2))

unitnormal[swallowtail][u_,v_]:=
{(1 + v^2 + v^4)^(-1/2),v/Sqrt[1 + v^2 + v^4],
  v^2/Sqrt[1 + v^2 + v^4]}

surfacenormal[swallowtail][u_,v_]:=
{2*(u + 6*v^2),2*v*(u + 6*v^2),2*v^2*(u + 6*v^2)}


(*surfrev *)
gcurvature[surfrev[{phi_[#],psi_[#]}&]][u_,v_]:=
(Derivative[1][psi][v]*
    (-(Derivative[1][psi][v]*Derivative[2][phi][v]) + 
      Derivative[1][phi][v]*Derivative[2][psi][v]))/
  (phi[v]*(Derivative[1][phi][v]^2 + 
       Derivative[1][psi][v]^2)^2)

mcurvature[surfrev[{phi_[#],psi_[#]}&]][u_,v_]:=
       -(phi[v]^2*(Derivative[1][phi][v]^2*
        Derivative[1][psi][v] + Derivative[1][psi][v]^3 - 
       phi[v]*Derivative[1][psi][v]*
        Derivative[2][phi][v] + 
       phi[v]*Derivative[1][phi][v]*Derivative[2][psi][v]))/
       (2*(phi[v]^2*(Derivative[1][phi][v]^2 + 
         Derivative[1][psi][v]^2))^(3/2))

unitnormal[surfrev[{phi_[#],psi_[#]}&]][u_,v_]:=
{Cos[u]*Derivative[1][psi][v]/
   Sqrt[Derivative[1][phi][v]^2 + Derivative[1][psi][v]^2],
  Sin[u]*Derivative[1][psi][v]/
   Sqrt[Derivative[1][phi][v]^2 + Derivative[1][psi][v]^2],
  -Derivative[1][phi][v]/
     Sqrt[Derivative[1][phi][v]^2 + Derivative[1][psi][v]^2]}

surfacenormal[surfrev[{phi_[#],psi_[#]}&]][u_,v_]:=
{Cos[u]*phi[v]*Derivative[1][psi][v],phi[v]*Sin[u]*Derivative[1][psi][v],
  -phi[v]*Derivative[1][phi][v]}

(*thomsen *)

gcurvature[thomsen[a_,b_]][u_,v_]:=
-a^4/(b^2 + 2*b*Sqrt[1 + b^2]*Cos[a*v]*Cosh[a*u] + 
       Cos[a*v]^2*Cosh[a*u]^2 + b^2*Cos[a*v]^2*Cosh[a*u]^2 + 
       Cosh[a*u]^2*Sin[a*v]^2 + b^2*Sin[a*v]^2*Sinh[a*u]^2)^2

mcurvature[thomsen[a_,b_]][u_,v_]:=
0

unitnormal[thomsen[a_,b_]][u_,v_]:=
{-Sqrt[2]*Sin[a*v]/Sqrt[1 + 2*b^2 + b^2*Cos[2*a*v] + 
       4*b*Sqrt[1 + b^2]*Cos[a*v]*Cosh[a*u] + (1 + b^2)*Cosh[2*a*u]], 
 -Sqrt[2]*Sinh[a*u]/Sqrt[1 + 2*b^2 + b^2*Cos[2*a*v] + 
       4*b*Sqrt[1 + b^2]*Cos[a*v]*Cosh[a*u] + (1 + b^2)*Cosh[2*a*u]], 
  (2*(1 + 2*b^2)*Cos[a*v]*Cosh[a*u] + 
     b*Sqrt[1 + b^2]*(2 + Cos[2*a*v] + Cosh[2*a*u]))/
   (Sqrt[2]*(b*Cos[a*v] + Sqrt[1 + b^2]*Cosh[a*u])*
     Sqrt[1 + 2*b^2 + b^2*Cos[2*a*v] + 
       4*b*Sqrt[1 + b^2]*Cos[a*v]*Cosh[a*u] + (1 + b^2)*Cosh[2*a*u]])}

surfacenormal[thomsen[a_,b_]][u_,v_]:=
{-(b*Cos[a*v] + Sqrt[1 + b^2]*Cosh[a*u])*Sin[a*v]/a^2,
-(b*Cos[a*v] + Sqrt[1 + b^2]*Cosh[a*u])*Sinh[a*u]/a^2,
  (Sqrt[1 + b^2] + b*Cos[a*v]*Cosh[a*u])*
   (b + Sqrt[1 + b^2]*Cos[a*v]*Cosh[a*u])/a^2 + 
    (b*Sqrt[1 + b^2]*Sin[a*v]^2*Sinh[a*u]^2)/a^2}



(*torus *)

gcurvature[torus[a_,b_]][u_,v_]:=
Cos[v]/(a*b + b^2*Cos[v])

gcurvature[torus[a_,b_,c_]][u_,v_]:=
4*b*c^2*Cos[v]/
((a + b*Cos[v])*(-b^2 - c^2 + b^2*Cos[2*v] - c^2*Cos[2*v])^2)

mcurvature[torus[a_,b_]][u_,v_]:=
-(a + 2*b*Cos[v])/(2*b*Sqrt[(a + b*Cos[v])^2])

mcurvature[torus[a_,b_,c_]][u_,v_]:=
(c*(-4*a*b - 5*b^2*Cos[v] - 3*c^2*Cos[v] + b^2*Cos[3*v] - c^2*Cos[3*v]))/
  (2*Sqrt[2]*(a + b*Cos[v])* (b^2 + c^2 - b^2*Cos[2*v] + c^2*Cos[2*v])^(3/2))

unitnormal[torus[a_,b_]][u_,v_]:=
{Cos[u]*Cos[v],Cos[v]*Sin[u],Sin[v]}

unitnormal[torus[a_,b_,c_]][u_,v_]:=
{Sqrt[2]*c*Cos[u]*Cos[v]/
   Sqrt[b^2 + c^2 - b^2*Cos[2*v] + c^2*Cos[2*v]],
 Sqrt[2]*c*Cos[v]*Sin[u]/
   Sqrt[b^2 + c^2 - b^2*Cos[2*v] + c^2*Cos[2*v]],
  Sqrt[2]*b*Sin[v]/
   Sqrt[b^2 + c^2 - b^2*Cos[2*v] + c^2*Cos[2*v]]}

surfacenormal[torus[a_,b_]][u_,v_]:=
{b*Cos[u]*Cos[v]*(a + b*Cos[v]),
  b*Cos[v]*(a + b*Cos[v])*Sin[u],b*(a + b*Cos[v])*Sin[v]}

surfacenormal[torus[a_,b_,c_]][u_,v_]:=
{c*Cos[u]*Cos[v]*(a + b*Cos[v]),c*Cos[v]*(a + b*Cos[v])*Sin[u],
   b*(a + b*Cos[v])*Sin[v]}


(*twiflat *)

gcurvature[twiflat[pm_,a_,c_][sl_]][u_,v_]:=
(-c^8 + c^6*sl^2 + 3*a^2*c^6*v^2 - 3*a^2*c^4*sl^2*v^2 -
    2*a^4*c^4*v^4 + a^4*c^4*pm^2*v^4 +
   3*a^4*c^2*sl^2*v^4 - a^6*c^2*pm^2*v^6 -
a^6*sl^2*v^6 + 2*a^4*c^2*pm*v^4*(-c^2 + a^2*v^2))/
  (c^4*Sqrt[a^2*v^2 - c^2] -
c^2*sl^2*Sqrt[a^2*v^2 - c^2] -
c^2*v^2*Sqrt[a^2*v^2 - c^2] -
2*a^2*c^2*pm*v^2*Sqrt[a^2*v^2 - c^2] +
a^2*sl^2*v^2*Sqrt[a^2*v^2 - c^2] +
a^2*v^4*Sqrt[a^2*v^2 - c^2] +
a^4*v^4*Sqrt[a^2*v^2 - c^2])^2

mcurvature[twiflat[pm_,a_,c_][sl_]][u_,v_]=
(a*Sqrt[a^2*v^2 - c^2]*
    (3*a*c^6*pm*v - 3*a*c^4*pm*sl^2*v - a*c^4*pm*v^3 - 
      3*a^3*c^4*pm*v^3 + 5*a^3*c^2*pm*sl^2*v^3 + 
      a^3*c^2*pm*v^5 + a^5*c^2*pm^3*v^5 - 
      2*a^5*pm*sl^2*v^5 - a^7*pm^3*v^7 + 
      (c^6*(-c^2 + a^2*v^2))/(a*v) - 
      (c^4*sl^2*(-c^2 + a^2*v^2))/(a*v) + 
      (c^4*v*(-c^2 + a^2*v^2))/a - 
      2*a*c^2*v^3*(-c^2 + a^2*v^2) + 
      3*a^3*c^2*pm^2*v^3*(-c^2 + a^2*v^2) - 
      a*c^4*pm*v^3*Cos[u]^2 + 
      2*a^3*c^2*pm*v^5*Cos[u]^2 - a^5*pm*v^7*Cos[u]^2 - 
      (c^4*v*(-c^2 + a^2*v^2)*Cos[u]^2)/a + 
      a*c^2*v^3*(-c^2 + a^2*v^2)*Cos[u]^2 - 
      a*c^4*pm*v^3*Sin[u]^2 + 
      2*a^3*c^2*pm*v^5*Sin[u]^2 - a^5*pm*v^7*Sin[u]^2 - 
      (c^4*v*(-c^2 + a^2*v^2)*Sin[u]^2)/a + 
      a*c^2*v^3*(-c^2 + a^2*v^2)*Sin[u]^2))/
  (2*(-c^6 + c^4*sl^2 + c^4*v^2 + a^2*c^4*v^2 - 
       2*a^2*c^2*sl^2*v^2 - 2*a^2*c^2*v^4 - 
       a^4*c^2*v^4 + a^4*sl^2*v^4 + a^4*v^6 + a^6*v^6 - 
       2*a^2*c^2*pm*v^2*(-c^2 + a^2*v^2))^(3/2))

unitnormal[twiflat[pm_,a_,c_][sl_]][u_,v_]:=
{(-(c^2*Sqrt[a^2*v^2 - c^2]*Cos[u]) + 
     a^2*pm*v^2*Sqrt[a^2*v^2 - c^2]*Cos[u] - 
     sl*(-c^2 + a^2*v^2)*Sin[u])/
   (-c^6 + c^4*sl^2 + c^4*v^2 + a^2*c^4*v^2 - 
      2*a^2*c^2*sl^2*v^2 - 2*a^2*c^2*v^4 - a^4*c^2*v^4 + 
      a^4*sl^2*v^4 + a^4*v^6 + a^6*v^6 - 
      2*a^2*c^2*pm*v^2*(-c^2 + a^2*v^2))^(1/2),
  (sl*(-c^2 + a^2*v^2)*Cos[u] - 
     c^2*Sqrt[a^2*v^2 - c^2]*Sin[u] + 
     a^2*pm*v^2*Sqrt[a^2*v^2 - c^2]*Sin[u])/
   (-c^6 + c^4*sl^2 + c^4*v^2 + a^2*c^4*v^2 - 
      2*a^2*c^2*sl^2*v^2 - 2*a^2*c^2*v^4 - a^4*c^2*v^4 + 
      a^4*sl^2*v^4 + a^4*v^6 + a^6*v^6 - 
      2*a^2*c^2*pm*v^2*(-c^2 + a^2*v^2))^(1/2),
  -((v*(-c^2 + a^2*v^2))/
     (-c^6 + c^4*sl^2 + c^4*v^2 + a^2*c^4*v^2 - 
        2*a^2*c^2*sl^2*v^2 - 2*a^2*c^2*v^4 - 
        a^4*c^2*v^4 + a^4*sl^2*v^4 + a^4*v^6 + a^6*v^6 - 
        2*a^2*c^2*pm*v^2*(-c^2 + a^2*v^2))^(1/2))}

surfacenormal[twiflat[pm_,a_,c_][sl_]][u_,v_]:=
{v*(-(c^2/(a*(1 - c^2/(a^2*v^2))^(1/2)*v^2)) + 
       (a^2*pm*v)/Sqrt[a^2*v^2 - c^2])*Cos[u] - 
    sl*Sin[u],sl*Cos[u] + 
    v*(-(c^2/(a*(1 - c^2/(a^2*v^2))^(1/2)*v^2)) + 
       (a^2*pm*v)/Sqrt[a^2*v^2 - c^2])*Sin[u],-v}


(*twisphere *)

gcurvature[twisphere[a_,b_]][u_,v_]:=
(a^2*Cos[v]^4 - b^2*Sin[v]^4)/
  (b^2 + a^2*Cos[v]^2 - b^2*Cos[v]^2)^2

mcurvature[twisphere[a_,b_]][u_,v_]:=
-(Cos[v]*(b^2 + 2*a^2*Cos[v]^2 + 2*b^2*Sin[v]^2))/
  (2*(a^2*Cos[v]^2 + b^2*Sin[v]^2)^(3/2))


unitnormal[twisphere[a_,b_]][u_,v_]:=
{(a*Cos[u]*Cos[v]^2 + b*Sin[u]*Sin[v])/
   Sqrt[a^2*Cos[v]^2 + b^2*Sin[v]^2],
  (a*Cos[v]^2*Sin[u] - b*Cos[u]*Sin[v])/
   Sqrt[a^2*Cos[v]^2 + b^2*Sin[v]^2],
  (a*Cos[v]*Sin[v])/Sqrt[a^2*Cos[v]^2 + b^2*Sin[v]^2]}

surfacenormal[twisphere[a_,b_]][u_,v_]:=
{a^2*Cos[u]*Cos[v]^2 + a*b*Sin[u]*Sin[v],
   a^2*Cos[v]^2*Sin[u] - a*b*Cos[u]*Sin[v],
   a^2*Sin[2*v]/2}


(*wallis *)

gcurvature[wallis[a_,b_,c_]][u_,v_]:=
8*b^4*c^2*(-2*a^2 + b^2 + b^2*Cos[2*u])*Sin[2*u]^2/
  (-(b^4*c^2) - 8*a^2*v^2 + 4*b^2*v^2 +
4*b^2*v^2*Cos[2*u] + b^4*c^2*Cos[4*u])^2

mcurvature[wallis[a_,b_,c_]][u_,v_]:=
(b^2*c*v*(-a^2 + b^2*Cos[u]^2)*
    (3*b^2 - 8*a^2*Cos[2*u] + 4*b^2*Cos[2*u] + 
      b^2*Cos[4*u]))/
  (4*Sqrt[2]*(2*a^2 - b^2 - b^2*Cos[2*u])^(1/2)*
    (-2*a^2 + b^2 + b^2*Cos[2*u])*
    (a^2*v^2*Cos[u]^2 - b^2*v^2*Cos[u]^4 + 
      a^2*v^2*Sin[u]^2 + b^4*c^2*Cos[u]^2*Sin[u]^2 - 
      b^2*v^2*Cos[u]^2*Sin[u]^2)*
    ((-(a^2*v^2*Cos[u]^2) + b^2*v^2*Cos[u]^4 - 
         a^2*v^2*Sin[u]^2 - b^4*c^2*Cos[u]^2*Sin[u]^2 + 
  b^2*v^2*Cos[u]^2*Sin[u]^2)/(-a^2 + b^2*Cos[u]^2))^(1/2))

unitnormal[wallis[a_,b_,c_]][u_,v_]:=
{(-2*Sqrt[2]*b^2*c*Cos[u]*Sin[u]^2)/
   (Sqrt[a^2 - b^2*Cos[u]^2]*
     ((-(b^4*c^2) - 8*a^2*v^2 + 8*b^2*v^2*Cos[u]^2 + 
          b^4*c^2*Cos[4*u])/(-a^2 + b^2*Cos[u]^2))^(1/2)),
   (2*Sqrt[2]*b^2*c*Cos[u]^2*Sin[u])/
   (Sqrt[a^2 - b^2*Cos[u]^2]*
     ((-(b^4*c^2) - 8*a^2*v^2 + 8*b^2*v^2*Cos[u]^2 + 
          b^4*c^2*Cos[4*u])/(-a^2 + b^2*Cos[u]^2))^(1/2)),
   (-2*Sqrt[2]*v)/
   ((-(b^4*c^2) - 8*a^2*v^2 + 8*b^2*v^2*Cos[u]^2 + 
        b^4*c^2*Cos[4*u])/(-a^2 + b^2*Cos[u]^2))^(1/2)}

surfacenormal[wallis[a_,b_,c_]][u_,v_]:=
{-((b^2*c*Cos[u]*Sin[u]^2)/Sqrt[a^2 - b^2*Cos[u]^2]),
   (b^2*c*Cos[u]^2*Sin[u])/Sqrt[a^2 - b^2*Cos[u]^2],-v}


(*whitneyumbrella *)

gcurvature[whitneyumbrella][u_,v_]:=
-4*v^2/(u^2 + 4*v^2 + 4*v^4)^2

mcurvature[whitneyumbrella][u_,v_]:=
-((u*(1 + 3*v^2))/(u^2 + 4*v^2 + 4*v^4)^(3/2))

unitnormal[whitneyumbrella][u_,v_]:=
{(2*v)/(u^2 + 4*v^2 + 4*v^4)^(1/2),
  (-2*v^2)/(u^2 + 4*v^2 + 4*v^4)^(1/2),
  -(u/(u^2 + 4*v^2 + 4*v^4)^(1/2))}

surfacenormal[whitneyumbrella][u_,v_]:=
{2*v,-2*v^2,-u}


(*wrinkles *)

gcurvature[wrinkles[a_]][u_,v_]:=
-a^2*u^4*v^4*(-u + v)^2*(u + v)^2/
    (a^2*u^6 - a^2*u^4*v^2 - a^2*u^2*v^4 + u^4*v^4 + a^2*v^6)^2

mcurvature[wrinkles[a_]][u_,v_]:=
(a*u^3*v^3*(u^4 + v^4))/
  (a^2*u^6 - a^2*u^4*v^2 - a^2*u^2*v^4 + u^4*v^4 + a^2*v^6)^(3/2)

unitnormal[wrinkles[a_]][u_,v_]:=
{(a*v*(-u + v)*(u + v))/
   (a^2*u^6 - a^2*u^4*v^2 - a^2*u^2*v^4 + u^4*v^4 + 
      a^2*v^6)^(1/2),(a*u*(u - v)*(u + v))/
   (a^2*u^6 - a^2*u^4*v^2 - a^2*u^2*v^4 + u^4*v^4 + 
      a^2*v^6)^(1/2),(u^2*v^2)/
   (a^2*u^6 - a^2*u^4*v^2 - a^2*u^2*v^4 + u^4*v^4 + 
      a^2*v^6)^(1/2)}

surfacenormal[wrinkles[a_]][u_,v_]:=
{-(a*(v^(-1) - v/u^2)),-(a*(u^(-1) - u/v^2)),1}


End[]

EndPackage[]




