(*:Date: Tue Feb  3 04:40:04 EST 1998
*)

(*:Title: INVERSURF.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the inversion surfaces of many parametrized surfaces.*)

BeginPackage["INVSURF`","SURFS`","CSPROGS`"]

    inversionsurf::usage="{u,v}->inversionsurf[q,rho][surf][u,v] is the inversion of a patch  surf with respect to a sphere in R^n of radius rho centered at q. {u,v}->inversionsurf[rho][surf][u,v] is the inversion of a patch surf with respect to a sphere in R^n of radius rho centered at the origin."


Begin["`Private`"]

inversionsurf[{p1_,p2_,p3_},rho_][astell[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a^3*Cos[u]^3*Cos[v]^3))/
       ((-p1 + a^3*Cos[u]^3*Cos[v]^3)^2 + 
         (-p2 + b^3*Cos[v]^3*Sin[u]^3)^2 + 
         (-p3 + c^3*Sin[v]^3)^2),
     p2 + (rho^2*(-p2 + b^3*Cos[v]^3*Sin[u]^3))/
       ((-p1 + a^3*Cos[u]^3*Cos[v]^3)^2 + 
         (-p2 + b^3*Cos[v]^3*Sin[u]^3)^2 + 
         (-p3 + c^3*Sin[v]^3)^2),
     p3 + (rho^2*(-p3 + c^3*Sin[v]^3))/
       ((-p1 + a^3*Cos[u]^3*Cos[v]^3)^2 + 
         (-p2 + b^3*Cos[v]^3*Sin[u]^3)^2 + 
         (-p3 + c^3*Sin[v]^3)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][bohdom[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[u]))/
       ((-p1 + a*Cos[u])^2 + 
         (-p2 + b*Cos[v] + a*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2),
     p2 + (rho^2*(-p2 + b*Cos[v] + a*Sin[u]))/
       ((-p1 + a*Cos[u])^2 + 
         (-p2 + b*Cos[v] + a*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2),
     p3 + (rho^2*(-p3 + c*Sin[v]))/
       ((-p1 + a*Cos[u])^2 + 
         (-p2 + b*Cos[v] + a*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][bourpolar[n_][t_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 - (u^(1 + n)*Cos[t - v - n*v])/(1 + n) + 
           (u^(-1 + n)*Cos[t + v - n*v])/(-1 + n)))/
       ((-p3 + (2*u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - (u^(1 + n)*Cos[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Cos[t + v - n*v])/(-1 + n))^2 + 
         (-p2 + (u^(1 + n)*Sin[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Sin[t + v - n*v])/(-1 + n))^2),
     p2 + (rho^2*(-p2 + 
           (u^(1 + n)*Sin[t - v - n*v])/(1 + n) + 
           (u^(-1 + n)*Sin[t + v - n*v])/(-1 + n)))/
       ((-p3 + (2*u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - (u^(1 + n)*Cos[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Cos[t + v - n*v])/(-1 + n))^2 + 
         (-p2 + (u^(1 + n)*Sin[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Sin[t + v - n*v])/(-1 + n))^2),
     p3 + (rho^2*(-p3 + (2*u^n*Cos[t - n*v])/n))/
       ((-p3 + (2*u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - (u^(1 + n)*Cos[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Cos[t + v - n*v])/(-1 + n))^2 + 
         (-p2 + (u^(1 + n)*Sin[t - v - n*v])/(1 + n) + 
            (u^(-1 + n)*Sin[t + v - n*v])/(-1 + n))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      bourpolar[m_,n_][t_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           (u^(-1 + m)*Cos[t - (-1 + m)*v])/(-1 + m) - 
           (u^(-1 + m + 2*n)*Cos[t - (-1 + m + 2*n)*v])/
            (-1 + m + 2*n)))/
       ((-p3 + (2*u^(-1 + m + n)*Cos[t - (-1 + m + n)*v])/
             (-1 + m + n))^2 + 
         (-p1 + (u^(-1 + m)*Cos[t - (-1 + m)*v])/
             (-1 + m) - 
            (u^(-1 + m + 2*n)*Cos[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2 + 
         (-p2 + (u^(-1 + m)*Sin[t - (-1 + m)*v])/
             (-1 + m) + 
            (u^(-1 + m + 2*n)*Sin[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2),
     p2 + (rho^2*(-p2 + 
           (u^(-1 + m)*Sin[t - (-1 + m)*v])/(-1 + m) + 
           (u^(-1 + m + 2*n)*Sin[t - (-1 + m + 2*n)*v])/
            (-1 + m + 2*n)))/
       ((-p3 + (2*u^(-1 + m + n)*Cos[t - (-1 + m + n)*v])/
             (-1 + m + n))^2 + 
         (-p1 + (u^(-1 + m)*Cos[t - (-1 + m)*v])/
             (-1 + m) - 
            (u^(-1 + m + 2*n)*Cos[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2 + 
         (-p2 + (u^(-1 + m)*Sin[t - (-1 + m)*v])/
             (-1 + m) + 
            (u^(-1 + m + 2*n)*Sin[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2),
     p3 + (rho^2*(-p3 + 
           (2*u^(-1 + m + n)*Cos[t - (-1 + m + n)*v])/
            (-1 + m + n)))/
       ((-p3 + (2*u^(-1 + m + n)*Cos[t - (-1 + m + n)*v])/
             (-1 + m + n))^2 + 
         (-p1 + (u^(-1 + m)*Cos[t - (-1 + m)*v])/
             (-1 + m) - 
            (u^(-1 + m + 2*n)*Cos[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2 + 
         (-p2 + (u^(-1 + m)*Sin[t - (-1 + m)*v])/
             (-1 + m) + 
            (u^(-1 + m + 2*n)*Sin[t - (-1 + m + 2*n)*v])/
             (-1 + m + 2*n))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][boy[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (a*(-Cos[v]^2 + 2*Cos[u]^2*Sin[v]^2 - 
                Sin[u]^2*Sin[v]^2 - 
                (Sin[4*u]*Sin[v]^4)/4 + 
                Cos[u]*Cos[v]*Sin[v]*
                 (-Cos[v]^2 + Cos[u]^2*Sin[v]^2) + 
                2*Cos[v]*Sin[u]*Sin[v]*
                 (-Cos[v]^2 + Sin[u]^2*Sin[v]^2)))/2))/
       ((-p3 + c*(Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])*
             (-4*(Cos[u] - Sin[u])*Sin[v]*
                (-Cos[v] + Cos[u]*Sin[v])*
                (Cos[v] - Sin[u]*Sin[v]) + 
               (Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])^3)
            )^2 + (-p2 + 
            (3^(1/2)*b*(-Cos[v]^2 + Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (Cos[v]^2 - Cos[u]^2*Sin[v]^2)))/2)^2 + 
         (-p1 + (a*(-Cos[v]^2 + 2*Cos[u]^2*Sin[v]^2 - 
                 Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (-Cos[v]^2 + Cos[u]^2*Sin[v]^2) + 
                 2*Cos[v]*Sin[u]*Sin[v]*
                  (-Cos[v]^2 + Sin[u]^2*Sin[v]^2)))/2)^2),
       p2 + (rho^2*(-p2 + 
           (3^(1/2)*b*(-Cos[v]^2 + Sin[u]^2*Sin[v]^2 - 
                (Sin[4*u]*Sin[v]^4)/4 + 
                Cos[u]*Cos[v]*Sin[v]*
                 (Cos[v]^2 - Cos[u]^2*Sin[v]^2)))/2))/
       ((-p3 + c*(Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])*
             (-4*(Cos[u] - Sin[u])*Sin[v]*
                (-Cos[v] + Cos[u]*Sin[v])*
                (Cos[v] - Sin[u]*Sin[v]) + 
               (Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])^3)
            )^2 + (-p2 + 
            (3^(1/2)*b*(-Cos[v]^2 + Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (Cos[v]^2 - Cos[u]^2*Sin[v]^2)))/2)^2 + 
         (-p1 + (a*(-Cos[v]^2 + 2*Cos[u]^2*Sin[v]^2 - 
                 Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (-Cos[v]^2 + Cos[u]^2*Sin[v]^2) + 
                 2*Cos[v]*Sin[u]*Sin[v]*
                  (-Cos[v]^2 + Sin[u]^2*Sin[v]^2)))/2)^2),
       p3 + (rho^2*(-p3 + 
           c*(Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])*
            (-4*(Cos[u] - Sin[u])*Sin[v]*
               (-Cos[v] + Cos[u]*Sin[v])*
               (Cos[v] - Sin[u]*Sin[v]) + 
              (Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])^3))
         )/((-p3 + c*(Cos[v] + Cos[u]*Sin[v] + 
               Sin[u]*Sin[v])*
             (-4*(Cos[u] - Sin[u])*Sin[v]*
                (-Cos[v] + Cos[u]*Sin[v])*
                (Cos[v] - Sin[u]*Sin[v]) + 
               (Cos[v] + Cos[u]*Sin[v] + Sin[u]*Sin[v])^3)
            )^2 + (-p2 + 
            (3^(1/2)*b*(-Cos[v]^2 + Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (Cos[v]^2 - Cos[u]^2*Sin[v]^2)))/2)^2 + 
         (-p1 + (a*(-Cos[v]^2 + 2*Cos[u]^2*Sin[v]^2 - 
                 Sin[u]^2*Sin[v]^2 - 
                 (Sin[4*u]*Sin[v]^4)/4 + 
                 Cos[u]*Cos[v]*Sin[v]*
                  (-Cos[v]^2 + Cos[u]^2*Sin[v]^2) + 
                 2*Cos[v]*Sin[u]*Sin[v]*
                  (-Cos[v]^2 + Sin[u]^2*Sin[v]^2)))/2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][catalan[a_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a*(u - Cosh[v]*Sin[u])))/
       ((-p2 + a*(1 - Cos[u]*Cosh[v]))^2 + 
         (-p1 + a*(u - Cosh[v]*Sin[u]))^2 + 
         (-p3 - 4*a*Sin[u/2]*Sinh[v/2])^2),
     p2 + (rho^2*(-p2 + a*(1 - Cos[u]*Cosh[v])))/
       ((-p2 + a*(1 - Cos[u]*Cosh[v]))^2 + 
         (-p1 + a*(u - Cosh[v]*Sin[u]))^2 + 
         (-p3 - 4*a*Sin[u/2]*Sinh[v/2])^2),
     p3 + (rho^2*(-p3 - 4*a*Sin[u/2]*Sinh[v/2]))/
       ((-p2 + a*(1 - Cos[u]*Cosh[v]))^2 + 
         (-p1 + a*(u - Cosh[v]*Sin[u]))^2 + 
         (-p3 - 4*a*Sin[u/2]*Sinh[v/2])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][catalandef[a_][t_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a*Cos[t]*(u - Cosh[v]*Sin[u]) + 
           a*Sin[t]*(v - Cos[u]*Sinh[v])))/
       ((-p3 + 4*a*(1 - Cos[u/2]*Cosh[v/2])*Sin[t] - 
            4*a*Cos[t]*Sin[u/2]*Sinh[v/2])^2 + 
         (-p2 + a*Cos[t]*(1 - Cos[u]*Cosh[v]) + 
            a*Sin[t]*Sin[u]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*(u - Cosh[v]*Sin[u]) + 
            a*Sin[t]*(v - Cos[u]*Sinh[v]))^2),
     p2 + (rho^2*(-p2 + a*Cos[t]*(1 - Cos[u]*Cosh[v]) + 
           a*Sin[t]*Sin[u]*Sinh[v]))/
       ((-p3 + 4*a*(1 - Cos[u/2]*Cosh[v/2])*Sin[t] - 
            4*a*Cos[t]*Sin[u/2]*Sinh[v/2])^2 + 
         (-p2 + a*Cos[t]*(1 - Cos[u]*Cosh[v]) + 
            a*Sin[t]*Sin[u]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*(u - Cosh[v]*Sin[u]) + 
            a*Sin[t]*(v - Cos[u]*Sinh[v]))^2),
     p3 + (rho^2*(-p3 + 
           4*a*(1 - Cos[u/2]*Cosh[v/2])*Sin[t] - 
           4*a*Cos[t]*Sin[u/2]*Sinh[v/2]))/
       ((-p3 + 4*a*(1 - Cos[u/2]*Cosh[v/2])*Sin[t] - 
            4*a*Cos[t]*Sin[u/2]*Sinh[v/2])^2 + 
         (-p2 + a*Cos[t]*(1 - Cos[u]*Cosh[v]) + 
            a*Sin[t]*Sin[u]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*(u - Cosh[v]*Sin[u]) + 
            a*Sin[t]*(v - Cos[u]*Sinh[v]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][catenoid[c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + c*Cos[u]*Cosh[v/c]))/
       ((-p3 + v)^2 + (-p1 + c*Cos[u]*Cosh[v/c])^2 + 
         (-p2 + c*Cosh[v/c]*Sin[u])^2),
     p2 + (rho^2*(-p2 + c*Cosh[v/c]*Sin[u]))/
       ((-p3 + v)^2 + (-p1 + c*Cos[u]*Cosh[v/c])^2 + 
         (-p2 + c*Cosh[v/c]*Sin[u])^2),
     p3 + (rho^2*(-p3 + v))/
       ((-p3 + v)^2 + (-p1 + c*Cos[u]*Cosh[v/c])^2 + 
         (-p2 + c*Cosh[v/c]*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      circularcone[a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*(v^2)^(1/2)*Cos[u]))/
       ((-p3 + b*v)^2 + (-p1 + a*(v^2)^(1/2)*Cos[u])^2 + 
         (-p2 + a*(v^2)^(1/2)*Sin[u])^2),
     p2 + (rho^2*(-p2 + a*(v^2)^(1/2)*Sin[u]))/
       ((-p3 + b*v)^2 + (-p1 + a*(v^2)^(1/2)*Cos[u])^2 + 
         (-p2 + a*(v^2)^(1/2)*Sin[u])^2),
     p3 + (rho^2*(-p3 + b*v))/
       ((-p3 + b*v)^2 + (-p1 + a*(v^2)^(1/2)*Cos[u])^2 + 
         (-p2 + a*(v^2)^(1/2)*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      circularcylinder[a_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]))/
       ((-p3 + v)^2 + (-p1 + a*Cos[u])^2 + 
         (-p2 + a*Sin[u])^2),
     p2 + (rho^2*(-p2 + a*Sin[u]))/
       ((-p3 + v)^2 + (-p1 + a*Cos[u])^2 + 
         (-p2 + a*Sin[u])^2),
     p3 + (rho^2*(-p3 + v))/
       ((-p3 + v)^2 + (-p1 + a*Cos[u])^2 + 
         (-p2 + a*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][clothoidmin[a_][t_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a*Pi^(1/2)*Cos[t]*
            Re[FresnelS[(u + I*v)/Pi^(1/2)]] + 
           a*Pi^(1/2)*Im[FresnelS[(u + I*v)/Pi^(1/2)]]*
            Sin[t]))/
       ((-p3 - a*v*Cos[t] + a*u*Sin[t])^2 + 
         (-p2 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelC[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelC[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2 + 
         (-p1 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelS[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelS[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2),
     p2 + (rho^2*(-p2 + 
           a*Pi^(1/2)*Cos[t]*
            Re[FresnelC[(u + I*v)/Pi^(1/2)]] + 
           a*Pi^(1/2)*Im[FresnelC[(u + I*v)/Pi^(1/2)]]*
            Sin[t]))/
       ((-p3 - a*v*Cos[t] + a*u*Sin[t])^2 + 
         (-p2 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelC[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelC[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2 + 
         (-p1 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelS[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelS[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2),
     p3 + (rho^2*(-p3 - a*v*Cos[t] + a*u*Sin[t]))/
       ((-p3 - a*v*Cos[t] + a*u*Sin[t])^2 + 
         (-p2 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelC[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelC[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2 + 
         (-p1 + a*Pi^(1/2)*Cos[t]*
             Re[FresnelS[(u + I*v)/Pi^(1/2)]] + 
            a*Pi^(1/2)*Im[FresnelS[(u + I*v)/Pi^(1/2)]]*
             Sin[t])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      cnccosurfrev[a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + b*Cos[u]*Sinh[v/a]))/
       ((-p3 - I*(a^2 - b^2)^(1/2)*
             EllipticE[(I*v)/a,-(b^2/(a^2 - b^2))])^2 + 
         (-p1 + b*Cos[u]*Sinh[v/a])^2 + 
         (-p2 + b*Sin[u]*Sinh[v/a])^2),
     p2 + (rho^2*(-p2 + b*Sin[u]*Sinh[v/a]))/
       ((-p3 - I*(a^2 - b^2)^(1/2)*
             EllipticE[(I*v)/a,-(b^2/(a^2 - b^2))])^2 + 
         (-p1 + b*Cos[u]*Sinh[v/a])^2 + 
         (-p2 + b*Sin[u]*Sinh[v/a])^2),
     p3 + (rho^2*(-p3 - 
           I*(a^2 - b^2)^(1/2)*
            EllipticE[(I*v)/a,-(b^2/(a^2 - b^2))]))/
       ((-p3 - I*(a^2 - b^2)^(1/2)*
             EllipticE[(I*v)/a,-(b^2/(a^2 - b^2))])^2 + 
         (-p1 + b*Cos[u]*Sinh[v/a])^2 + 
         (-p2 + b*Sin[u]*Sinh[v/a])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      cnchysurfrev[a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + b*Cos[u]*Cosh[v/a]))/
       ((-p1 + b*Cos[u]*Cosh[v/a])^2 + 
         (-p3 - I*a*EllipticE[(I*v)/a,-(b^2/a^2)])^2 + 
         (-p2 + b*Cosh[v/a]*Sin[u])^2),
     p2 + (rho^2*(-p2 + b*Cosh[v/a]*Sin[u]))/
       ((-p1 + b*Cos[u]*Cosh[v/a])^2 + 
         (-p3 - I*a*EllipticE[(I*v)/a,-(b^2/a^2)])^2 + 
         (-p2 + b*Cosh[v/a]*Sin[u])^2),
     p3 + (rho^2*(-p3 - 
           I*a*EllipticE[(I*v)/a,-(b^2/a^2)]))/
       ((-p1 + b*Cos[u]*Cosh[v/a])^2 + 
         (-p3 - I*a*EllipticE[(I*v)/a,-(b^2/a^2)])^2 + 
         (-p2 + b*Cosh[v/a]*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][cpcsurfrev[a_,b_]][u_,v_]:=
{p1 + (rho^2*(-p1 + b*Cos[u]*Cos[v/a]))/
       ((-p1 + b*Cos[u]*Cos[v/a])^2 + 
         (-p3 + a*EllipticE[v/a,b^2/a^2])^2 + 
         (-p2 + b*Cos[v/a]*Sin[u])^2),
     p2 + (rho^2*(-p2 + b*Cos[v/a]*Sin[u]))/
       ((-p1 + b*Cos[u]*Cos[v/a])^2 + 
         (-p3 + a*EllipticE[v/a,b^2/a^2])^2 + 
         (-p2 + b*Cos[v/a]*Sin[u])^2),
     p3 + (rho^2*(-p3 + a*EllipticE[v/a,b^2/a^2]))/
       ((-p1 + b*Cos[u]*Cos[v/a])^2 + 
         (-p3 + a*EllipticE[v/a,b^2/a^2])^2 + 
         (-p2 + b*Cos[v/a]*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][crosscap[a_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (a^2*Sin[u]*Sin[2*v])/2))/
       ((-p3 + a^2*Cos[2*u]*Cos[v]^2)^2 + 
         (-p2 + a^2*Cos[v]^2*Sin[2*u])^2 + 
         (-p1 + (a^2*Sin[u]*Sin[2*v])/2)^2),
     p2 + (rho^2*(-p2 + a^2*Cos[v]^2*Sin[2*u]))/
       ((-p3 + a^2*Cos[2*u]*Cos[v]^2)^2 + 
         (-p2 + a^2*Cos[v]^2*Sin[2*u])^2 + 
         (-p1 + (a^2*Sin[u]*Sin[2*v])/2)^2),
     p3 + (rho^2*(-p3 + a^2*Cos[2*u]*Cos[v]^2))/
       ((-p3 + a^2*Cos[2*u]*Cos[v]^2)^2 + 
         (-p2 + a^2*Cos[v]^2*Sin[2*u])^2 + 
         (-p1 + (a^2*Sin[u]*Sin[2*v])/2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][dini[a_,b_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[u]*Sin[v]))/
       ((-p3 + b*u + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2),
     p2 + (rho^2*(-p2 + a*Sin[u]*Sin[v]))/
       ((-p3 + b*u + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2),
     p3 + (rho^2*(-p3 + b*u + 
           a*(Cos[v] + Log[Tan[v/2]])))/
       ((-p3 + b*u + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][dinitcheb[a_,b_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a*Cos[b]*Cos[v]*
            Sech[Sec[b]*(u - v*Sin[b])]))/
       ((-p1 + a*Cos[b]*Cos[v]*
             Sech[Sec[b]*(u - v*Sin[b])])^2 + 
         (-p2 + a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
             Sin[v])^2 + 
         (-p3 + a*(u - 
               Cos[b]*Tanh[Sec[b]*(u - v*Sin[b])]))^2),
     p2 + (rho^2*(-p2 + 
           a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*Sin[v]))/
       ((-p1 + a*Cos[b]*Cos[v]*
             Sech[Sec[b]*(u - v*Sin[b])])^2 + 
         (-p2 + a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
             Sin[v])^2 + 
         (-p3 + a*(u - 
               Cos[b]*Tanh[Sec[b]*(u - v*Sin[b])]))^2),
     p3 + (rho^2*(-p3 + 
           a*(u - Cos[b]*Tanh[Sec[b]*(u - v*Sin[b])])))/
       ((-p1 + a*Cos[b]*Cos[v]*
             Sech[Sec[b]*(u - v*Sin[b])])^2 + 
         (-p2 + a*Cos[b]*Sech[Sec[b]*(u - v*Sin[b])]*
             Sin[v])^2 + 
         (-p3 + a*(u - 
               Cos[b]*Tanh[Sec[b]*(u - v*Sin[b])]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ducyc[{q1_,q2_,q3_},rho_,a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + q1 + 
           (rho^2*(-q1 + Cos[u]*(a + b*Cos[v])))/
            ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
              (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
              (-q3 + b*Sin[v])^2)))/
       ((-p1 + q1 + (rho^2*(-q1 + Cos[u]*(a + b*Cos[v])))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p2 + q2 + (rho^2*
               (-q2 + (a + b*Cos[v])*Sin[u]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p3 + q3 + (rho^2*(-q3 + b*Sin[v]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2),
     p2 + (rho^2*(-p2 + q2 + 
           (rho^2*(-q2 + (a + b*Cos[v])*Sin[u]))/
            ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
              (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
              (-q3 + b*Sin[v])^2)))/
       ((-p1 + q1 + (rho^2*(-q1 + Cos[u]*(a + b*Cos[v])))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p2 + q2 + (rho^2*
               (-q2 + (a + b*Cos[v])*Sin[u]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p3 + q3 + (rho^2*(-q3 + b*Sin[v]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2),
     p3 + (rho^2*(-p3 + q3 + 
           (rho^2*(-q3 + b*Sin[v]))/
            ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
              (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
              (-q3 + b*Sin[v])^2)))/
       ((-p1 + q1 + (rho^2*(-q1 + Cos[u]*(a + b*Cos[v])))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p2 + q2 + (rho^2*
               (-q2 + (a + b*Cos[v])*Sin[u]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2 + 
         (-p3 + q3 + (rho^2*(-q3 + b*Sin[v]))/
             ((-q1 + Cos[u]*(a + b*Cos[v]))^2 + 
               (-q2 + (a + b*Cos[v])*Sin[u])^2 + 
               (-q3 + b*Sin[v])^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][eighteight][u_,v_]:=
{p1 + (rho^2*(-p1 + Sin[u]*Sin[v]))/
       ((-p3 + Cos[v]*Sin[v])^2 + 
         (-p1 + Sin[u]*Sin[v])^2 + 
         (-p2 + Cos[u]*Sin[u]*Sin[v])^2),
     p2 + (rho^2*(-p2 + Cos[u]*Sin[u]*Sin[v]))/
       ((-p3 + Cos[v]*Sin[v])^2 + 
         (-p1 + Sin[u]*Sin[v])^2 + 
         (-p2 + Cos[u]*Sin[u]*Sin[v])^2),
     p3 + (rho^2*(-p3 + Cos[v]*Sin[v]))/
       ((-p3 + Cos[v]*Sin[v])^2 + 
         (-p1 + Sin[u]*Sin[v])^2 + 
         (-p2 + Cos[u]*Sin[u]*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][eightsurface][u_,v_]:=
{p1 + (rho^2*(-p1 + Cos[u]*Sin[2*v]))/
       ((-p3 + Sin[v])^2 + (-p1 + Cos[u]*Sin[2*v])^2 + 
         (-p2 + Sin[u]*Sin[2*v])^2),
     p2 + (rho^2*(-p2 + Sin[u]*Sin[2*v]))/
       ((-p3 + Sin[v])^2 + (-p1 + Cos[u]*Sin[2*v])^2 + 
         (-p2 + Sin[u]*Sin[2*v])^2),
     p3 + (rho^2*(-p3 + Sin[v]))/
       ((-p3 + Sin[v])^2 + (-p1 + Cos[u]*Sin[2*v])^2 + 
         (-p2 + Sin[u]*Sin[2*v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ellipsemin[a_,b_][t_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[t]*Cos[u]*Cosh[v] - 
           a*Sin[t]*Sin[u]*Sinh[v]))/
       ((-p3 - b*(Cos[t]*
                Im[EllipticE[u + I*v,1 - a^2/b^2]] - 
               Re[EllipticE[u + I*v,1 - a^2/b^2]]*Sin[t])
            )^2 + (-p2 + b*Cos[t]*Cosh[v]*Sin[u] + 
            b*Cos[u]*Sin[t]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*Cos[u]*Cosh[v] - 
            a*Sin[t]*Sin[u]*Sinh[v])^2),
     p2 + (rho^2*(-p2 + b*Cos[t]*Cosh[v]*Sin[u] + 
           b*Cos[u]*Sin[t]*Sinh[v]))/
       ((-p3 - b*(Cos[t]*
                Im[EllipticE[u + I*v,1 - a^2/b^2]] - 
               Re[EllipticE[u + I*v,1 - a^2/b^2]]*Sin[t])
            )^2 + (-p2 + b*Cos[t]*Cosh[v]*Sin[u] + 
            b*Cos[u]*Sin[t]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*Cos[u]*Cosh[v] - 
            a*Sin[t]*Sin[u]*Sinh[v])^2),
     p3 + (rho^2*(-p3 - 
           b*(Cos[t]*Im[EllipticE[u + I*v,
                 1 - a^2/b^2]] - 
              Re[EllipticE[u + I*v,1 - a^2/b^2]]*Sin[t]))
         )/((-p3 - b*(Cos[t]*
                Im[EllipticE[u + I*v,1 - a^2/b^2]] - 
               Re[EllipticE[u + I*v,1 - a^2/b^2]]*Sin[t])
            )^2 + (-p2 + b*Cos[t]*Cosh[v]*Sin[u] + 
            b*Cos[u]*Sin[t]*Sinh[v])^2 + 
         (-p1 + a*Cos[t]*Cos[u]*Cosh[v] - 
            a*Sin[t]*Sin[u]*Sinh[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ellipsoid[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]*Cos[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + b*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2),
     p2 + (rho^2*(-p2 + b*Cos[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + b*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2),
     p3 + (rho^2*(-p3 + c*Sin[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + b*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2)}

 inversionsurf[{p1_,p2_,p3_},rho_][ellipsoid[a_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]*Cos[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2),
     p2 + (rho^2*(-p2 + a*Cos[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2),
     p3 + (rho^2*(-p3 + c*Sin[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + c*Sin[v])^2)}

 inversionsurf[{p1_,p2_,p3_},rho_][
      ellipticalhyperboloidminus[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u] + a*v*Sin[u]))/
       ((-p3 + c*v)^2 + (-p2 - b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] + a*v*Sin[u])^2),
     p2 + (rho^2*(-p2 - b*v*Cos[u] + b*Sin[u]))/
       ((-p3 + c*v)^2 + (-p2 - b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] + a*v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*v))/
       ((-p3 + c*v)^2 + (-p2 - b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] + a*v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ellipticalhyperboloidplus[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u] - a*v*Sin[u]))/
       ((-p3 + c*v)^2 + (-p2 + b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] - a*v*Sin[u])^2),
     p2 + (rho^2*(-p2 + b*v*Cos[u] + b*Sin[u]))/
       ((-p3 + c*v)^2 + (-p2 + b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] - a*v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*v))/
       ((-p3 + c*v)^2 + (-p2 + b*v*Cos[u] + b*Sin[u])^2 + 
         (-p1 + a*Cos[u] - a*v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ellipticcoor[d,e,f][a_,b_,c_][w]][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           d*(((a - u)*(a - v)*(a - w))/
               ((a - b)*(a - c)))^(1/2)))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/
                ((a - b)*(a - c)))^(1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/
                ((-a + b)*(b - c)))^(1/2))^2 + 
         (-p3 + f*(((c - u)*(c - v)*(c - w))/
                ((-a + c)*(-b + c)))^(1/2))^2),
     p2 + (rho^2*(-p2 + 
           e*(((b - u)*(b - v)*(b - w))/
               ((-a + b)*(b - c)))^(1/2)))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/
                ((a - b)*(a - c)))^(1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/
                ((-a + b)*(b - c)))^(1/2))^2 + 
         (-p3 + f*(((c - u)*(c - v)*(c - w))/
                ((-a + c)*(-b + c)))^(1/2))^2),
     p3 + (rho^2*(-p3 + 
           f*(((c - u)*(c - v)*(c - w))/
               ((-a + c)*(-b + c)))^(1/2)))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/
                ((a - b)*(a - c)))^(1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/
                ((-a + b)*(b - c)))^(1/2))^2 + 
         (-p3 + f*(((c - u)*(c - v)*(c - w))/
                ((-a + c)*(-b + c)))^(1/2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      ellipticparaboloid[a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^2/a^2 + v^2/b^2)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^2/a^2 + v^2/b^2)^2),
     p3 + (rho^2*(-p3 + u^2/a^2 + v^2/b^2))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^2/a^2 + v^2/b^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][enneper][u_,v_]:=
    {p1 + (rho^2*(-p1 + u - u^3/3 + u*v^2))/
       ((-p3 + u^2 - v^2)^2 + 
         (-p1 + u - u^3/3 + u*v^2)^2 + 
         (-p2 - v - u^2*v + v^3/3)^2),
     p2 + (rho^2*(-p2 - v - u^2*v + v^3/3))/
       ((-p3 + u^2 - v^2)^2 + 
         (-p1 + u - u^3/3 + u*v^2)^2 + 
         (-p2 - v - u^2*v + v^3/3)^2),
     p3 + (rho^2*(-p3 + u^2 - v^2))/
       ((-p3 + u^2 - v^2)^2 + 
         (-p1 + u - u^3/3 + u*v^2)^2 + 
         (-p2 - v - u^2*v + v^3/3)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      enneperpolar[n_][t_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + u*Cos[t - v] - 
           (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(1 + 2*n)))/
       ((-p3 + (2*u^(1 + n)*Cos[t - (1 + n)*v])/(1 + n))^
          2 + (-p1 + u*Cos[t - v] - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(1 + 2*n))^
          2 + (-p2 + u*Sin[t - v] + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(1 + 2*n))^
          2),p2 + (rho^2*
         (-p2 + u*Sin[t - v] + 
           (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(1 + 2*n)))/
       ((-p3 + (2*u^(1 + n)*Cos[t - (1 + n)*v])/(1 + n))^
          2 + (-p1 + u*Cos[t - v] - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(1 + 2*n))^
          2 + (-p2 + u*Sin[t - v] + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(1 + 2*n))^
          2),p3 + (rho^2*
         (-p3 + (2*u^(1 + n)*Cos[t - (1 + n)*v])/(1 + n)))
        /((-p3 + (2*u^(1 + n)*Cos[t - (1 + n)*v])/
             (1 + n))^2 + 
         (-p1 + u*Cos[t - v] - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(1 + 2*n))^
          2 + (-p2 + u*Sin[t - v] + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(1 + 2*n))^
          2)}

inversionsurf[{p1_,p2_,p3_},rho_][funnel[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*v*Cos[u]))/
       ((-p1 + a*v*Cos[u])^2 + (-p3 + c*Log[v])^2 + 
         (-p2 + b*v*Sin[u])^2),
     p2 + (rho^2*(-p2 + b*v*Sin[u]))/
       ((-p1 + a*v*Cos[u])^2 + (-p3 + c*Log[v])^2 + 
         (-p2 + b*v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*Log[v]))/
       ((-p1 + a*v*Cos[u])^2 + (-p3 + c*Log[v])^2 + 
         (-p2 + b*v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][genhypparab[a_,b_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*(u + v)))/
       ((-p2 + b*v)^2 + (-p3 + u^2 + 2*u*v)^2 + 
         (-p1 + a*(u + v))^2),
     p2 + (rho^2*(-p2 + b*v))/
       ((-p2 + b*v)^2 + (-p3 + u^2 + 2*u*v)^2 + 
         (-p1 + a*(u + v))^2),
     p3 + (rho^2*(-p3 + u^2 + 2*u*v))/
       ((-p2 + b*v)^2 + (-p3 + u^2 + 2*u*v)^2 + 
         (-p1 + a*(u + v))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][handkerchief[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 + u*v^2 + a*(u^2 - v^2))^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 + u*v^2 + a*(u^2 - v^2))^2),
     p3 + (rho^2*(-p3 + u^3/3 + u*v^2 + a*(u^2 - v^2)))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 + u*v^2 + a*(u^2 - v^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][helicoid[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*v*Cos[u]))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + b*v*Sin[u])^2),
     p2 + (rho^2*(-p2 + b*v*Sin[u]))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + b*v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*u))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + b*v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][helicoid[a_,c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*v*Cos[u]))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + a*v*Sin[u])^2),
     p2 + (rho^2*(-p2 + a*v*Sin[u]))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + a*v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*u))/
       ((-p3 + c*u)^2 + (-p1 + a*v*Cos[u])^2 + 
         (-p2 + a*v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      helicoidprin[a_,b_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           b*Cos[(u + v)/2]*Sinh[(-u + v)/2]))/
       ((-p3 + (b*(u + v))/2)^2 + 
         (-p1 + b*Cos[(u + v)/2]*Sinh[(-u + v)/2])^2 + 
         (-p2 + b*Sin[(u + v)/2]*Sinh[(-u + v)/2])^2),
     p2 + (rho^2*(-p2 + 
           b*Sin[(u + v)/2]*Sinh[(-u + v)/2]))/
       ((-p3 + (b*(u + v))/2)^2 + 
         (-p1 + b*Cos[(u + v)/2]*Sinh[(-u + v)/2])^2 + 
         (-p2 + b*Sin[(u + v)/2]*Sinh[(-u + v)/2])^2),
     p3 + (rho^2*(-p3 + (b*(u + v))/2))/
       ((-p3 + (b*(u + v))/2)^2 + 
         (-p1 + b*Cos[(u + v)/2]*Sinh[(-u + v)/2])^2 + 
         (-p2 + b*Sin[(u + v)/2]*Sinh[(-u + v)/2])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][heltocat[t_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + Cos[u]*Cosh[v]*Sin[t] + 
           Cos[t]*Sin[u]*Sinh[v]))/
       ((-p3 + u*Cos[t] + v*Sin[t])^2 + 
         (-p2 + Cosh[v]*Sin[t]*Sin[u] - 
            Cos[t]*Cos[u]*Sinh[v])^2 + 
         (-p1 + Cos[u]*Cosh[v]*Sin[t] + 
            Cos[t]*Sin[u]*Sinh[v])^2),
     p2 + (rho^2*(-p2 + Cosh[v]*Sin[t]*Sin[u] - 
           Cos[t]*Cos[u]*Sinh[v]))/
       ((-p3 + u*Cos[t] + v*Sin[t])^2 + 
         (-p2 + Cosh[v]*Sin[t]*Sin[u] - 
            Cos[t]*Cos[u]*Sinh[v])^2 + 
         (-p1 + Cos[u]*Cosh[v]*Sin[t] + 
            Cos[t]*Sin[u]*Sinh[v])^2),
     p3 + (rho^2*(-p3 + u*Cos[t] + v*Sin[t]))/
       ((-p3 + u*Cos[t] + v*Sin[t])^2 + 
         (-p2 + Cosh[v]*Sin[t]*Sin[u] - 
            Cos[t]*Cos[u]*Sinh[v])^2 + 
         (-p1 + Cos[u]*Cosh[v]*Sin[t] + 
            Cos[t]*Sin[u]*Sinh[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][henneberg][u_,v_]:=
{p1 + (rho^2*
         (-p1 + 2*Cos[v]*Sinh[u] - 
           (2*Cos[3*v]*Sinh[3*u])/3))/
       ((-p3 + 2*Cos[2*v]*Cosh[2*u])^2 + 
         (-p1 + 2*Cos[v]*Sinh[u] - 
            (2*Cos[3*v]*Sinh[3*u])/3)^2 + 
         (-p2 + 2*Sin[v]*Sinh[u] + 
            (2*Sin[3*v]*Sinh[3*u])/3)^2),
     p2 + (rho^2*(-p2 + 2*Sin[v]*Sinh[u] + 
           (2*Sin[3*v]*Sinh[3*u])/3))/
       ((-p3 + 2*Cos[2*v]*Cosh[2*u])^2 + 
         (-p1 + 2*Cos[v]*Sinh[u] - 
            (2*Cos[3*v]*Sinh[3*u])/3)^2 + 
         (-p2 + 2*Sin[v]*Sinh[u] + 
            (2*Sin[3*v]*Sinh[3*u])/3)^2),
     p3 + (rho^2*(-p3 + 2*Cos[2*v]*Cosh[2*u]))/
       ((-p3 + 2*Cos[2*v]*Cosh[2*u])^2 + 
         (-p1 + 2*Cos[v]*Sinh[u] - 
            (2*Cos[3*v]*Sinh[3*u])/3)^2 + 
         (-p2 + 2*Sin[v]*Sinh[u] + 
            (2*Sin[3*v]*Sinh[3*u])/3)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      hyperbolicparaboloid][u_,v_]:=
    {p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + (-p3 + u*v)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + (-p3 + u*v)^2),
     p3 + (rho^2*(-p3 + u*v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + (-p3 + u*v)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      hyperboloid[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]*Cosh[v]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      ,p2 + (rho^2*(-p2 + b*Cosh[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      ,p3 + (rho^2*(-p3 + c*Sinh[v]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      }

 inversionsurf[{p1_,p2_,p3_},rho_][
      hyperboloid[a_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]*Cosh[v]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      ,p2 + (rho^2*(-p2 + a*Cosh[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      ,p3 + (rho^2*(-p3 + c*Sinh[v]))/
       ((-p1 + a*Cos[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sin[u])^2 + (-p3 + c*Sinh[v])^2)
      }

  inversionsurf[{p1_,p2_,p3_},rho_][
      hy2sheet[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cosh[u]*Cosh[v]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         ),p2 + (rho^2*(-p2 + b*Cosh[v]*Sinh[u]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         ),p3 + (rho^2*(-p3 + c*Sinh[v]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + b*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         )}

inversionsurf[{p1_,p2_,p3_},rho_][
      hy2sheet[a_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cosh[u]*Cosh[v]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         ),p2 + (rho^2*(-p2 + a*Cosh[v]*Sinh[u]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         ),p3 + (rho^2*(-p3 + c*Sinh[v]))/
       ((-p1 + a*Cosh[u]*Cosh[v])^2 + 
         (-p2 + a*Cosh[v]*Sinh[u])^2 + (-p3 + c*Sinh[v])^2
         )}

inversionsurf[{p1_,p2_,p3_},rho_][
      kleinbottle[a_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           Cos[u]*(a + Cos[u/2]*Sin[v] - 
              Sin[u/2]*Sin[2*v])))/
       ((-p3 + Sin[u/2]*Sin[v] + Cos[u/2]*Sin[2*v])^2 + 
         (-p1 + Cos[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2 + 
         (-p2 + Sin[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2),
     p2 + (rho^2*(-p2 + 
           Sin[u]*(a + Cos[u/2]*Sin[v] - 
              Sin[u/2]*Sin[2*v])))/
       ((-p3 + Sin[u/2]*Sin[v] + Cos[u/2]*Sin[2*v])^2 + 
         (-p1 + Cos[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2 + 
         (-p2 + Sin[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2),
     p3 + (rho^2*(-p3 + Sin[u/2]*Sin[v] + 
           Cos[u/2]*Sin[2*v]))/
       ((-p3 + Sin[u/2]*Sin[v] + Cos[u/2]*Sin[2*v])^2 + 
         (-p1 + Cos[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2 + 
         (-p2 + Sin[u]*(a + Cos[u/2]*Sin[v] - 
               Sin[u/2]*Sin[2*v]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][kuen][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           (2*(Cos[u] + u*Sin[u])*Sin[v])/
            (1 + u^2*Sin[v]^2)))/
       ((-p3 + Log[Tan[v/2]] + 
            (2*Cos[v])/(1 + u^2*Sin[v]^2))^2 + 
         (-p2 + (2*(-(u*Cos[u]) + Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2 + 
         (-p1 + (2*(Cos[u] + u*Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2),
     p2 + (rho^2*(-p2 + 
           (2*(-(u*Cos[u]) + Sin[u])*Sin[v])/
            (1 + u^2*Sin[v]^2)))/
       ((-p3 + Log[Tan[v/2]] + 
            (2*Cos[v])/(1 + u^2*Sin[v]^2))^2 + 
         (-p2 + (2*(-(u*Cos[u]) + Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2 + 
         (-p1 + (2*(Cos[u] + u*Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2),
     p3 + (rho^2*(-p3 + Log[Tan[v/2]] + 
           (2*Cos[v])/(1 + u^2*Sin[v]^2)))/
       ((-p3 + Log[Tan[v/2]] + 
            (2*Cos[v])/(1 + u^2*Sin[v]^2))^2 + 
         (-p2 + (2*(-(u*Cos[u]) + Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2 + 
         (-p1 + (2*(Cos[u] + u*Sin[u])*Sin[v])/
             (1 + u^2*Sin[v]^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][kuentcheb[a_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (2*a*Cosh[v]*(Cos[u] + u*Sin[u]))/
            (u^2 + Cosh[v]^2)))/
       ((-p2 + (2*a*Cosh[v]*(-(u*Cos[u]) + Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p1 + (2*a*Cosh[v]*(Cos[u] + u*Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p3 + a*(v - (2*Cosh[v]*Sinh[v])/
                (u^2 + Cosh[v]^2)))^2),
     p2 + (rho^2*(-p2 + 
           (2*a*Cosh[v]*(-(u*Cos[u]) + Sin[u]))/
            (u^2 + Cosh[v]^2)))/
       ((-p2 + (2*a*Cosh[v]*(-(u*Cos[u]) + Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p1 + (2*a*Cosh[v]*(Cos[u] + u*Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p3 + a*(v - (2*Cosh[v]*Sinh[v])/
                (u^2 + Cosh[v]^2)))^2),
     p3 + (rho^2*(-p3 + 
           a*(v - (2*Cosh[v]*Sinh[v])/(u^2 + Cosh[v]^2))))
        /((-p2 + (2*a*Cosh[v]*(-(u*Cos[u]) + Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p1 + (2*a*Cosh[v]*(Cos[u] + u*Sin[u]))/
             (u^2 + Cosh[v]^2))^2 + 
         (-p3 + a*(v - (2*Cosh[v]*Sinh[v])/
                (u^2 + Cosh[v]^2)))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      lemniscatemin[a_][t_]][u_,v_]:=
    {p1 + (rho^2*(-p1 - 
           (16*a*Cos[u]*Cosh[v]*Sin[u]*Sinh[v]*
              (Cos[u]*Cosh[v]*Sin[t] + 
                Cos[t]*Sin[u]*Sinh[v]))/
            (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
              Cosh[4*v]) + 
           (4*a*(3 - Cos[2*u]*Cosh[2*v])*
              (Cos[t]*Cos[u]*Cosh[v] - 
                Sin[t]*Sin[u]*Sinh[v]))/
            (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
              Cosh[4*v])))/
       ((-p3 + a*(-(Cos[t]*Im[EllipticF[u + I*v,-1]]) + 
               Re[EllipticF[u + I*v,-1]]*Sin[t]))^2 + 
         (-p1 - (16*a*Cos[u]*Cosh[v]*Sin[u]*Sinh[v]*
               (Cos[u]*Cosh[v]*Sin[t] + 
                 Cos[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]) + 
            (4*a*(3 - Cos[2*u]*Cosh[2*v])*
               (Cos[t]*Cos[u]*Cosh[v] - 
                 Sin[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]))^2 + 
         (-p2 + (a*(-Sin[t - 4*u] + 
                 6*Cosh[2*v]*Sin[t - 2*u] - 
                 6*Cosh[2*v]*Sin[t + 2*u] + 
                 Sin[t + 4*u] - 
                 6*Sin[t - 2*u]*Sinh[2*v] - 
                 6*Sin[t + 2*u]*Sinh[2*v] + 
                 2*Sin[t]*Sinh[4*v]))/
             (2*(-18 - Cos[4*u] + 12*Cos[2*u]*Cosh[2*v] - 
                 Cosh[4*v])))^2),
     p2 + (rho^2*(-p2 + 
           (a*(-Sin[t - 4*u] + 6*Cosh[2*v]*Sin[t - 2*u] - 
                6*Cosh[2*v]*Sin[t + 2*u] + Sin[t + 4*u] - 
                6*Sin[t - 2*u]*Sinh[2*v] - 
                6*Sin[t + 2*u]*Sinh[2*v] + 
                2*Sin[t]*Sinh[4*v]))/
            (2*(-18 - Cos[4*u] + 12*Cos[2*u]*Cosh[2*v] - 
                Cosh[4*v]))))/
       ((-p3 + a*(-(Cos[t]*Im[EllipticF[u + I*v,-1]]) + 
               Re[EllipticF[u + I*v,-1]]*Sin[t]))^2 + 
         (-p1 - (16*a*Cos[u]*Cosh[v]*Sin[u]*Sinh[v]*
               (Cos[u]*Cosh[v]*Sin[t] + 
                 Cos[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]) + 
            (4*a*(3 - Cos[2*u]*Cosh[2*v])*
               (Cos[t]*Cos[u]*Cosh[v] - 
                 Sin[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]))^2 + 
         (-p2 + (a*(-Sin[t - 4*u] + 
                 6*Cosh[2*v]*Sin[t - 2*u] - 
                 6*Cosh[2*v]*Sin[t + 2*u] + 
                 Sin[t + 4*u] - 
                 6*Sin[t - 2*u]*Sinh[2*v] - 
                 6*Sin[t + 2*u]*Sinh[2*v] + 
                 2*Sin[t]*Sinh[4*v]))/
             (2*(-18 - Cos[4*u] + 12*Cos[2*u]*Cosh[2*v] - 
                 Cosh[4*v])))^2),
     p3 + (rho^2*(-p3 + 
           a*(-(Cos[t]*Im[EllipticF[u + I*v,-1]]) + 
              Re[EllipticF[u + I*v,-1]]*Sin[t])))/
       ((-p3 + a*(-(Cos[t]*Im[EllipticF[u + I*v,-1]]) + 
               Re[EllipticF[u + I*v,-1]]*Sin[t]))^2 + 
         (-p1 - (16*a*Cos[u]*Cosh[v]*Sin[u]*Sinh[v]*
               (Cos[u]*Cosh[v]*Sin[t] + 
                 Cos[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]) + 
            (4*a*(3 - Cos[2*u]*Cosh[2*v])*
               (Cos[t]*Cos[u]*Cosh[v] - 
                 Sin[t]*Sin[u]*Sinh[v]))/
             (18 + Cos[4*u] - 12*Cos[2*u]*Cosh[2*v] + 
               Cosh[4*v]))^2 + 
         (-p2 + (a*(-Sin[t - 4*u] + 
                 6*Cosh[2*v]*Sin[t - 2*u] - 
                 6*Cosh[2*v]*Sin[t + 2*u] + 
                 Sin[t + 4*u] - 
                 6*Sin[t - 2*u]*Sinh[2*v] - 
                 6*Sin[t + 2*u]*Sinh[2*v] + 
                 2*Sin[t]*Sinh[4*v]))/
             (2*(-18 - Cos[4*u] + 12*Cos[2*u]*Cosh[2*v] - 
                 Cosh[4*v])))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][menn][u_,v_]:=
    {p1 + (rho^2*(-p1 + menn[u,v]))/
       ((-p1 + menn[u,v])^2 + (-p2 + menn[u,v])^2 + 
         (-p3 + menn[u,v])^2),
     p2 + (rho^2*(-p2 + menn[u,v]))/
       ((-p1 + menn[u,v])^2 + (-p2 + menn[u,v])^2 + 
         (-p3 + menn[u,v])^2),
     p3 + (rho^2*(-p3 + menn[u,v]))/
       ((-p1 + menn[u,v])^2 + (-p2 + menn[u,v])^2 + 
         (-p3 + menn[u,v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      mercatorellipsoid[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a*Cos[u]*Sech[v]))/
       ((-p1 + a*Cos[u]*Sech[v])^2 + 
         (-p2 + b*Sech[v]*Sin[u])^2 + (-p3 + c*Tanh[v])^2)
      ,p2 + (rho^2*(-p2 + b*Sech[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Sech[v])^2 + 
         (-p2 + b*Sech[v]*Sin[u])^2 + (-p3 + c*Tanh[v])^2)
      ,p3 + (rho^2*(-p3 + c*Tanh[v]))/
       ((-p1 + a*Cos[u]*Sech[v])^2 + 
         (-p2 + b*Sech[v]*Sin[u])^2 + (-p3 + c*Tanh[v])^2)
      }

inversionsurf[{p1_,p2_,p3_},rho_][
      moebcirc[a_]][u_,v_]:=
    {p1 + (rho^2*(-p1 - 
           (2*a*Cos[2*v]*Sin[u])/
            (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
              2^(1/2)*Sin[u]*Sin[2*v])))/
       ((-p3 - (2*a*Cos[u]*Cos[v])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p1 - (2*a*Cos[2*v]*Sin[u])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p2 + (2^(1/2)*a*
               (Cos[u]*Sin[v] - Sin[u]*Sin[2*v]))/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2),
     p2 + (rho^2*(-p2 + 
           (2^(1/2)*a*(Cos[u]*Sin[v] - Sin[u]*Sin[2*v]))/
            (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
              2^(1/2)*Sin[u]*Sin[2*v])))/
       ((-p3 - (2*a*Cos[u]*Cos[v])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p1 - (2*a*Cos[2*v]*Sin[u])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p2 + (2^(1/2)*a*
               (Cos[u]*Sin[v] - Sin[u]*Sin[2*v]))/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2),
     p3 + (rho^2*(-p3 - 
           (2*a*Cos[u]*Cos[v])/
            (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
              2^(1/2)*Sin[u]*Sin[2*v])))/
       ((-p3 - (2*a*Cos[u]*Cos[v])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p1 - (2*a*Cos[2*v]*Sin[u])/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2 + 
         (-p2 + (2^(1/2)*a*
               (Cos[u]*Sin[v] - Sin[u]*Sin[2*v]))/
             (-2 + 2^(1/2)*Cos[u]*Sin[v] + 
               2^(1/2)*Sin[u]*Sin[2*v]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][moebiusstrip[a_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + a*(Cos[u] + v*Cos[u/2]*Cos[u])))/
       ((-p1 + a*(Cos[u] + v*Cos[u/2]*Cos[u]))^2 + 
         (-p3 + a*v*Sin[u/2])^2 + 
         (-p2 + a*(Sin[u] + v*Cos[u/2]*Sin[u]))^2),
     p2 + (rho^2*(-p2 + a*(Sin[u] + v*Cos[u/2]*Sin[u])))/
       ((-p1 + a*(Cos[u] + v*Cos[u/2]*Cos[u]))^2 + 
         (-p3 + a*v*Sin[u/2])^2 + 
         (-p2 + a*(Sin[u] + v*Cos[u/2]*Sin[u]))^2),
     p3 + (rho^2*(-p3 + a*v*Sin[u/2]))/
       ((-p1 + a*(Cos[u] + v*Cos[u/2]*Cos[u]))^2 + 
         (-p3 + a*v*Sin[u/2])^2 + 
         (-p2 + a*(Sin[u] + v*Cos[u/2]*Sin[u]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][monkey[n_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Re[(u + I*v)^n])^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Re[(u + I*v)^n])^2),
     p3 + (rho^2*(-p3 + Re[(u + I*v)^n]))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Re[(u + I*v)^n])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][monkeysaddle][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3 - 3*u*v^2)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3 - 3*u*v^2)^2),
     p3 + (rho^2*(-p3 + u^3 - 3*u*v^2))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3 - 3*u*v^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      paraboliccoor[d,e,f][a_,b_,c_][w]][u_,v_]:=
    {p1 + (rho^2*(-p1 + 
           d*(((a - u)*(a - v)*(a - w))/(-a + b))^(1/2)))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/(-a + b))^
              (1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/(a - b))^
              (1/2))^2 + 
         (-p3 + (f*(-a - b + c + u + v + w))/2)^2),
     p2 + (rho^2*(-p2 + 
           e*(((b - u)*(b - v)*(b - w))/(a - b))^(1/2)))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/(-a + b))^
              (1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/(a - b))^
              (1/2))^2 + 
         (-p3 + (f*(-a - b + c + u + v + w))/2)^2),
     p3 + (rho^2*(-p3 + (f*(-a - b + c + u + v + w))/2))/
       ((-p1 + d*(((a - u)*(a - v)*(a - w))/(-a + b))^
              (1/2))^2 + 
         (-p2 + e*(((b - u)*(b - v)*(b - w))/(a - b))^
              (1/2))^2 + 
         (-p3 + (f*(-a - b + c + u + v + w))/2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][paraboloid[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u^2 + v^2))^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u^2 + v^2))^2),
     p3 + (rho^2*(-p3 + a*(u^2 + v^2)))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u^2 + v^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][paraboloid[a_,b_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + b*v^2)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + b*v^2)^2),
     p3 + (rho^2*(-p3 + a*u^2 + b*v^2))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + b*v^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      paraboloid[a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + c*u*v + b*v^2)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + c*u*v + b*v^2)^2),
     p3 + (rho^2*(-p3 + a*u^2 + c*u*v + b*v^2))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*u^2 + c*u*v + b*v^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][perturbedms[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - u*v^2 + a*(u^2 + v^2))^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - u*v^2 + a*(u^2 + v^2))^2),
     p3 + (rho^2*(-p3 + u^3/3 - u*v^2 + a*(u^2 + v^2)))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - u*v^2 + a*(u^2 + v^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      plane[a1_,a2_,b1_,b2_,c1_,c2_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + a1*u + a2*v))/
       ((-p1 + a1*u + a2*v)^2 + (-p2 + b1*u + b2*v)^2 + 
         (-p3 + c1*u + c2*v)^2),
     p2 + (rho^2*(-p2 + b1*u + b2*v))/
       ((-p1 + a1*u + a2*v)^2 + (-p2 + b1*u + b2*v)^2 + 
         (-p3 + c1*u + c2*v)^2),
     p3 + (rho^2*(-p3 + c1*u + c2*v))/
       ((-p1 + a1*u + a2*v)^2 + (-p2 + b1*u + b2*v)^2 + 
         (-p3 + c1*u + c2*v)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][plucker][u_,v_]:=
    {p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + (u*v)/(u^2 + v^2))^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + (u*v)/(u^2 + v^2))^2),
     p3 + (rho^2*(-p3 + (u*v)/(u^2 + v^2)))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + (u*v)/(u^2 + v^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][pluckerpolar[n_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u*Cos[v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + Sin[n*v])^2),
     p2 + (rho^2*(-p2 + u*Sin[v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + Sin[n*v])^2),
     p3 + (rho^2*(-p3 + Sin[n*v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + Sin[n*v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      pluckerpolar[m_,n_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + u*Cos[v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + u^m*Sin[n*v])^2),
     p2 + (rho^2*(-p2 + u*Sin[v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + u^m*Sin[n*v])^2),
     p3 + (rho^2*(-p3 + u^m*Sin[n*v]))/
       ((-p1 + u*Cos[v])^2 + (-p2 + u*Sin[v])^2 + 
         (-p3 + u^m*Sin[n*v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][pseudocrosscap][u_,v_]:=
{p1 + (rho^2*(-p1 + (1 - u^2)*Sin[v]))/
       ((-p3 + u)^2 + (-p1 + (1 - u^2)*Sin[v])^2 + 
         (-p2 + (1 - u^2)*Sin[2*v])^2),
     p2 + (rho^2*(-p2 + (1 - u^2)*Sin[2*v]))/
       ((-p3 + u)^2 + (-p1 + (1 - u^2)*Sin[v])^2 + 
         (-p2 + (1 - u^2)*Sin[2*v])^2),
     p3 + (rho^2*(-p3 + u))/
       ((-p3 + u)^2 + (-p1 + (1 - u^2)*Sin[v])^2 + 
         (-p2 + (1 - u^2)*Sin[2*v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][pseudosphere[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[u]*Sin[v]))/
       ((-p3 + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2),
     p2 + (rho^2*(-p2 + a*Sin[u]*Sin[v]))/
       ((-p3 + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2),
     p3 + (rho^2*(-p3 + a*(Cos[v] + Log[Tan[v/2]])))/
       ((-p3 + a*(Cos[v] + Log[Tan[v/2]]))^2 + 
         (-p1 + a*Cos[u]*Sin[v])^2 + 
         (-p2 + a*Sin[u]*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][pseudospheretcheb[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[v]*Sech[u]))/
       ((-p1 + a*Cos[v]*Sech[u])^2 + 
         (-p2 + a*Sech[u]*Sin[v])^2 + 
         (-p3 + a*(u - Tanh[u]))^2),
     p2 + (rho^2*(-p2 + a*Sech[u]*Sin[v]))/
       ((-p1 + a*Cos[v]*Sech[u])^2 + 
         (-p2 + a*Sech[u]*Sin[v])^2 + 
         (-p3 + a*(u - Tanh[u]))^2),
     p3 + (rho^2*(-p3 + a*(u - Tanh[u])))/
       ((-p1 + a*Cos[v]*Sech[u])^2 + 
         (-p2 + a*Sech[u]*Sin[v])^2 + 
         (-p3 + a*(u - Tanh[u]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][rconoid][u_,v_]:=

    {p1 + (rho^2*(-p1 + v*Cos[u]))/
       ((-p1 + v*Cos[u])^2 + (-p3 + 2*Sin[u])^2 + 
         (-p2 + v*Sin[u])^2),
     p2 + (rho^2*(-p2 + v*Sin[u]))/
       ((-p1 + v*Cos[u])^2 + (-p3 + 2*Sin[u])^2 + 
         (-p2 + v*Sin[u])^2),
     p3 + (rho^2*(-p3 + 2*Sin[u]))/
       ((-p1 + v*Cos[u])^2 + (-p3 + 2*Sin[u])^2 + 
         (-p2 + v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][richmond][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (-3*u - u^5 + 2*u^3*v^2 + 3*u*v^4)/
            (6*(u^2 + v^2))))/
       ((-p3 + u)^2 + (-p1 + 
            (-3*u - u^5 + 2*u^3*v^2 + 3*u*v^4)/
             (6*(u^2 + v^2)))^2 + 
         (-p2 + (-3*v - 3*u^4*v - 2*u^2*v^3 + v^5)/
             (6*(u^2 + v^2)))^2),
     p2 + (rho^2*(-p2 + 
           (-3*v - 3*u^4*v - 2*u^2*v^3 + v^5)/
            (6*(u^2 + v^2))))/
       ((-p3 + u)^2 + (-p1 + 
            (-3*u - u^5 + 2*u^3*v^2 + 3*u*v^4)/
             (6*(u^2 + v^2)))^2 + 
         (-p2 + (-3*v - 3*u^4*v - 2*u^2*v^3 + v^5)/
             (6*(u^2 + v^2)))^2),
     p3 + (rho^2*(-p3 + u))/
       ((-p3 + u)^2 + (-p1 + 
            (-3*u - u^5 + 2*u^3*v^2 + 3*u*v^4)/
             (6*(u^2 + v^2)))^2 + 
         (-p2 + (-3*v - 3*u^4*v - 2*u^2*v^3 + v^5)/
             (6*(u^2 + v^2)))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      richmondpolar[n_][t_]][u_,v_]:=
    {p1 + (rho^2*(-p1 - Cos[t + v]/(2*u) - 
           (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(2 + 4*n)))/
       ((-p3 + (u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - Cos[t + v]/(2*u) - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(2 + 4*n))^
          2 + (-p2 - Sin[t + v]/(2*u) + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(2 + 4*n))^
          2),p2 + (rho^2*
         (-p2 - Sin[t + v]/(2*u) + 
           (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(2 + 4*n)))/
       ((-p3 + (u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - Cos[t + v]/(2*u) - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(2 + 4*n))^
          2 + (-p2 - Sin[t + v]/(2*u) + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(2 + 4*n))^
          2),p3 + (rho^2*(-p3 + (u^n*Cos[t - n*v])/n))/
       ((-p3 + (u^n*Cos[t - n*v])/n)^2 + 
         (-p1 - Cos[t + v]/(2*u) - 
            (u^(1 + 2*n)*Cos[t - (1 + 2*n)*v])/(2 + 4*n))^
          2 + (-p2 - Sin[t + v]/(2*u) + 
            (u^(1 + 2*n)*Sin[t - (1 + 2*n)*v])/(2 + 4*n))^
          2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      roman[a_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + (a^2*Cos[v]^2*Sin[2*u])/2))/
       ((-p1 + (a^2*Cos[v]^2*Sin[2*u])/2)^2 + 
         (-p3 + (a^2*Cos[u]*Sin[2*v])/2)^2 + 
         (-p2 + (a^2*Sin[u]*Sin[2*v])/2)^2),
     p2 + (rho^2*(-p2 + (a^2*Sin[u]*Sin[2*v])/2))/
       ((-p1 + (a^2*Cos[v]^2*Sin[2*u])/2)^2 + 
         (-p3 + (a^2*Cos[u]*Sin[2*v])/2)^2 + 
         (-p2 + (a^2*Sin[u]*Sin[2*v])/2)^2),
     p3 + (rho^2*(-p3 + (a^2*Cos[u]*Sin[2*v])/2))/
       ((-p1 + (a^2*Cos[v]^2*Sin[2*u])/2)^2 + 
         (-p3 + (a^2*Cos[u]*Sin[2*v])/2)^2 + 
         (-p2 + (a^2*Sin[u]*Sin[2*v])/2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][rosemin[n,a][t_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + Cos[n*u]*Cosh[n*v]*
            (a*Cos[t]*Cos[u]*Cosh[v] - 
              a*Sin[t]*Sin[u]*Sinh[v]) + 
           Sin[n*u]*(-(a*Cos[u]*Cosh[v]*Sin[t]) - 
              a*Cos[t]*Sin[u]*Sinh[v])*Sinh[n*v]))/
       ((-p3 - (a*Cos[t]*
               Im[EllipticE[n*(u + I*v),1 - n^2]])/n + 
            (a*Re[EllipticE[n*(u + I*v),1 - n^2]]*
               Sin[t])/n)^2 + 
         (-p1 + Cos[n*u]*Cosh[n*v]*
             (a*Cos[t]*Cos[u]*Cosh[v] - 
               a*Sin[t]*Sin[u]*Sinh[v]) + 
            Sin[n*u]*(-(a*Cos[u]*Cosh[v]*Sin[t]) - 
               a*Cos[t]*Sin[u]*Sinh[v])*Sinh[n*v])^2 + 
         (-p2 - Cos[u]*Sinh[v]*
             (-(a*Cos[n*u]*Cosh[n*v]*Sin[t]) - 
               a*Cos[t]*Sin[n*u]*Sinh[n*v]) + 
            Cosh[v]*Sin[u]*
             (a*Cos[t]*Cos[n*u]*Cosh[n*v] - 
               a*Sin[t]*Sin[n*u]*Sinh[n*v]))^2),
     p2 + (rho^2*(-p2 - 
           Cos[u]*Sinh[v]*
            (-(a*Cos[n*u]*Cosh[n*v]*Sin[t]) - 
              a*Cos[t]*Sin[n*u]*Sinh[n*v]) + 
           Cosh[v]*Sin[u]*
            (a*Cos[t]*Cos[n*u]*Cosh[n*v] - 
              a*Sin[t]*Sin[n*u]*Sinh[n*v])))/
       ((-p3 - (a*Cos[t]*
               Im[EllipticE[n*(u + I*v),1 - n^2]])/n + 
            (a*Re[EllipticE[n*(u + I*v),1 - n^2]]*
               Sin[t])/n)^2 + 
         (-p1 + Cos[n*u]*Cosh[n*v]*
             (a*Cos[t]*Cos[u]*Cosh[v] - 
               a*Sin[t]*Sin[u]*Sinh[v]) + 
            Sin[n*u]*(-(a*Cos[u]*Cosh[v]*Sin[t]) - 
               a*Cos[t]*Sin[u]*Sinh[v])*Sinh[n*v])^2 + 
         (-p2 - Cos[u]*Sinh[v]*
             (-(a*Cos[n*u]*Cosh[n*v]*Sin[t]) - 
               a*Cos[t]*Sin[n*u]*Sinh[n*v]) + 
            Cosh[v]*Sin[u]*
             (a*Cos[t]*Cos[n*u]*Cosh[n*v] - 
               a*Sin[t]*Sin[n*u]*Sinh[n*v]))^2),
     p3 + (rho^2*(-p3 - 
           (a*Cos[t]*Im[EllipticE[n*(u + I*v),1 - n^2]])/
            n + (a*Re[EllipticE[n*(u + I*v),1 - n^2]]*
              Sin[t])/n))/
       ((-p3 - (a*Cos[t]*
               Im[EllipticE[n*(u + I*v),1 - n^2]])/n + 
            (a*Re[EllipticE[n*(u + I*v),1 - n^2]]*
               Sin[t])/n)^2 + 
         (-p1 + Cos[n*u]*Cosh[n*v]*
             (a*Cos[t]*Cos[u]*Cosh[v] - 
               a*Sin[t]*Sin[u]*Sinh[v]) + 
            Sin[n*u]*(-(a*Cos[u]*Cosh[v]*Sin[t]) - 
               a*Cos[t]*Sin[u]*Sinh[v])*Sinh[n*v])^2 + 
         (-p2 - Cos[u]*Sinh[v]*
             (-(a*Cos[n*u]*Cosh[n*v]*Sin[t]) - 
               a*Cos[t]*Sin[n*u]*Sinh[n*v]) + 
            Cosh[v]*Sin[u]*
             (a*Cos[t]*Cos[n*u]*Cosh[n*v] - 
               a*Sin[t]*Sin[n*u]*Sinh[n*v]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][scherk[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Log[Cos[a*v]*Sec[a*u]]/a)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Log[Cos[a*v]*Sec[a*u]]/a)^2),
     p3 + (rho^2*(-p3 + Log[Cos[a*v]*Sec[a*u]]/a))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + Log[Cos[a*v]*Sec[a*u]]/a)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][scherk[a,theta]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + Sec[2*theta]*
            (u*Cos[theta] + v*Sin[theta])))/
       ((-p3 + Log[Cos[a*Sec[2*theta]*
                 (v*Cos[theta] + u*Sin[theta])]*
               Sec[a*Sec[2*theta]*
                 (u*Cos[theta] + v*Sin[theta])]]/a)^2 + 
         (-p2 + Sec[2*theta]*
             (v*Cos[theta] + u*Sin[theta]))^2 + 
         (-p1 + Sec[2*theta]*
             (u*Cos[theta] + v*Sin[theta]))^2),
     p2 + (rho^2*(-p2 + 
           Sec[2*theta]*(v*Cos[theta] + u*Sin[theta])))/
       ((-p3 + Log[Cos[a*Sec[2*theta]*
                 (v*Cos[theta] + u*Sin[theta])]*
               Sec[a*Sec[2*theta]*
                 (u*Cos[theta] + v*Sin[theta])]]/a)^2 + 
         (-p2 + Sec[2*theta]*
             (v*Cos[theta] + u*Sin[theta]))^2 + 
         (-p1 + Sec[2*theta]*
             (u*Cos[theta] + v*Sin[theta]))^2),
     p3 + (rho^2*(-p3 + 
           Log[Cos[a*Sec[2*theta]*
                (v*Cos[theta] + u*Sin[theta])]*
              Sec[a*Sec[2*theta]*
                (u*Cos[theta] + v*Sin[theta])]]/a))/
       ((-p3 + Log[Cos[a*Sec[2*theta]*
                 (v*Cos[theta] + u*Sin[theta])]*
               Sec[a*Sec[2*theta]*
                 (u*Cos[theta] + v*Sin[theta])]]/a)^2 + 
         (-p2 + Sec[2*theta]*
             (v*Cos[theta] + u*Sin[theta]))^2 + 
         (-p1 + Sec[2*theta]*
             (u*Cos[theta] + v*Sin[theta]))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][scherk5[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*ArcSinh[u]))/
       ((-p3 + c*ArcSin[u*v])^2 + 
         (-p1 + a*ArcSinh[u])^2 + (-p2 + b*ArcSinh[v])^2),
       p2 + (rho^2*(-p2 + b*ArcSinh[v]))/
       ((-p3 + c*ArcSin[u*v])^2 + 
         (-p1 + a*ArcSinh[u])^2 + (-p2 + b*ArcSinh[v])^2),
       p3 + (rho^2*(-p3 + c*ArcSin[u*v]))/
       ((-p3 + c*ArcSin[u*v])^2 + 
         (-p1 + a*ArcSinh[u])^2 + (-p2 + b*ArcSinh[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][shoe][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - v^2/2)^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - v^2/2)^2),
     p3 + (rho^2*(-p3 + u^3/3 - v^2/2))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + u^3/3 - v^2/2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][sievert[a_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (2*Cos[u/(1 + a)^(1/2) - 
                ArcTan[(1 + a)^(1/2)*Tan[u]]]*
              ((1 + a)*(1 + a*Sin[u]^2))^(1/2)*Sin[v])/
            (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2))))/
       ((-p1 + (2*Cos[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]]*
               ((1 + a)*(1 + a*Sin[u]^2))^(1/2)*Sin[v])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2 + 
         (-p3 + (Log[Tan[v/2]] + 
               (2*(1 + a)*Cos[v])/
                (1 + a - a*Cos[u]^2*Sin[v]^2))/a^(1/2))^2 
          + (-p2 - (2*((1 + a)*(1 + a*Sin[u]^2))^(1/2)*
               Sin[v]*Sin[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2),
     p2 + (rho^2*(-p2 - 
           (2*((1 + a)*(1 + a*Sin[u]^2))^(1/2)*Sin[v]*
              Sin[u/(1 + a)^(1/2) - 
                ArcTan[(1 + a)^(1/2)*Tan[u]]])/
            (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2))))/
       ((-p1 + (2*Cos[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]]*
               ((1 + a)*(1 + a*Sin[u]^2))^(1/2)*Sin[v])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2 + 
         (-p3 + (Log[Tan[v/2]] + 
               (2*(1 + a)*Cos[v])/
                (1 + a - a*Cos[u]^2*Sin[v]^2))/a^(1/2))^2 
          + (-p2 - (2*((1 + a)*(1 + a*Sin[u]^2))^(1/2)*
               Sin[v]*Sin[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2),
     p3 + (rho^2*(-p3 + 
           (Log[Tan[v/2]] + 
              (2*(1 + a)*Cos[v])/
               (1 + a - a*Cos[u]^2*Sin[v]^2))/a^(1/2)))/
       ((-p1 + (2*Cos[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]]*
               ((1 + a)*(1 + a*Sin[u]^2))^(1/2)*Sin[v])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2 + 
         (-p3 + (Log[Tan[v/2]] + 
               (2*(1 + a)*Cos[v])/
                (1 + a - a*Cos[u]^2*Sin[v]^2))/a^(1/2))^2 
          + (-p2 - (2*((1 + a)*(1 + a*Sin[u]^2))^(1/2)*
               Sin[v]*Sin[u/(1 + a)^(1/2) - 
                 ArcTan[(1 + a)^(1/2)*Tan[u]]])/
             (a^(1/2)*(1 + a - a*Cos[u]^2*Sin[v]^2)))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][sphere[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[u]*Cos[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + a*Sin[v])^2),
     p2 + (rho^2*(-p2 + a*Cos[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + a*Sin[v])^2),
     p3 + (rho^2*(-p3 + a*Sin[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + (-p3 + a*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      stereographicellipsoid[a_,b_,c_]][u_,v_]:=

    {p1 + (rho^2*(-p1 + (2*a*u)/(1 + u^2 + v^2)))/
       ((-p1 + (2*a*u)/(1 + u^2 + v^2))^2 + 
         (-p2 + (2*b*v)/(1 + u^2 + v^2))^2 + 
         (-p3 + (c*(-1 + u^2 + v^2))/(1 + u^2 + v^2))^2),
     p2 + (rho^2*(-p2 + (2*b*v)/(1 + u^2 + v^2)))/
       ((-p1 + (2*a*u)/(1 + u^2 + v^2))^2 + 
         (-p2 + (2*b*v)/(1 + u^2 + v^2))^2 + 
         (-p3 + (c*(-1 + u^2 + v^2))/(1 + u^2 + v^2))^2),
     p3 + (rho^2*(-p3 + 
           (c*(-1 + u^2 + v^2))/(1 + u^2 + v^2)))/
       ((-p1 + (2*a*u)/(1 + u^2 + v^2))^2 + 
         (-p2 + (2*b*v)/(1 + u^2 + v^2))^2 + 
         (-p3 + (c*(-1 + u^2 + v^2))/(1 + u^2 + v^2))^2)}

inversionsurf[{p1_,p2_,p3_},rho_][swallowtail][u_,v_]:=
{p1 + (rho^2*(-p1 + u*v^2 + 3*v^4))/
       ((-p3 + u)^2 + (-p2 - 2*u*v - 4*v^3)^2 + 
         (-p1 + u*v^2 + 3*v^4)^2),
     p2 + (rho^2*(-p2 - 2*u*v - 4*v^3))/
       ((-p3 + u)^2 + (-p2 - 2*u*v - 4*v^3)^2 + 
         (-p1 + u*v^2 + 3*v^4)^2),
     p3 + (rho^2*(-p3 + u))/
       ((-p3 + u)^2 + (-p2 - 2*u*v - 4*v^3)^2 + 
         (-p1 + u*v^2 + 3*v^4)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      tetrahedral[m_,n_][a_,b_,c_][a_,b_,c_]][u_,v_]:=
    {p1 + (rho^2*(-p1 + A*(-a + u)^m*(-a + v)^n))/
       ((-p1 + A*(-a + u)^m*(-a + v)^n)^2 + 
         (-p2 + B*(-b + u)^m*(-b + v)^n)^2 + 
         (-p3 + C*(-c + u)^m*(-c + v)^n)^2),
     p2 + (rho^2*(-p2 + B*(-b + u)^m*(-b + v)^n))/
       ((-p1 + A*(-a + u)^m*(-a + v)^n)^2 + 
         (-p2 + B*(-b + u)^m*(-b + v)^n)^2 + 
         (-p3 + C*(-c + u)^m*(-c + v)^n)^2),
     p3 + (rho^2*(-p3 + C*(-c + u)^m*(-c + v)^n))/
       ((-p1 + A*(-a + u)^m*(-a + v)^n)^2 + 
         (-p2 + B*(-b + u)^m*(-b + v)^n)^2 + 
         (-p3 + C*(-c + u)^m*(-c + v)^n)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][thomsen[a_,b_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + (b*u)/a + 
           ((1 + b^2)^(1/2)*Cos[a*v]*Sinh[a*u])/a^2))/
       ((-p2 + ((1 + b^2)^(1/2)*v)/a + 
            (b*Cosh[a*u]*Sin[a*v])/a^2)^2 + 
         (-p1 + (b*u)/a + 
            ((1 + b^2)^(1/2)*Cos[a*v]*Sinh[a*u])/a^2)^2 + 
         (-p3 + (Sin[a*v]*Sinh[a*u])/a^2)^2),
     p2 + (rho^2*(-p2 + ((1 + b^2)^(1/2)*v)/a + 
           (b*Cosh[a*u]*Sin[a*v])/a^2))/
       ((-p2 + ((1 + b^2)^(1/2)*v)/a + 
            (b*Cosh[a*u]*Sin[a*v])/a^2)^2 + 
         (-p1 + (b*u)/a + 
            ((1 + b^2)^(1/2)*Cos[a*v]*Sinh[a*u])/a^2)^2 + 
         (-p3 + (Sin[a*v]*Sinh[a*u])/a^2)^2),
     p3 + (rho^2*(-p3 + (Sin[a*v]*Sinh[a*u])/a^2))/
       ((-p2 + ((1 + b^2)^(1/2)*v)/a + 
            (b*Cosh[a*u]*Sin[a*v])/a^2)^2 + 
         (-p1 + (b*u)/a + 
            ((1 + b^2)^(1/2)*Cos[a*v]*Sinh[a*u])/a^2)^2 + 
         (-p3 + (Sin[a*v]*Sinh[a*u])/a^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][torus[a_,b_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + Cos[u]*(a + b*Cos[v])))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + b*Sin[v])^2),
     p2 + (rho^2*(-p2 + (a + b*Cos[v])*Sin[u]))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + b*Sin[v])^2),
     p3 + (rho^2*(-p3 + b*Sin[v]))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + b*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][torus[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*
         (-p1 + Cos[u]*(a + b*Cos[v])))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2),
     p2 + (rho^2*(-p2 + (a + b*Cos[v])*Sin[u]))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2),
     p3 + (rho^2*(-p3 + c*Sin[v]))/
       ((-p1 + Cos[u]*(a + b*Cos[v]))^2 + 
         (-p2 + (a + b*Cos[v])*Sin[u])^2 + 
         (-p3 + c*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][
      twiflat[pm,a,c][sl]][u_,v_]:=
    {p1 + (rho^2*(-p1 + v*Cos[u]))/
       ((-p3 + sl*u + pm*(-c^2 + a^2*v^2)^(1/2) + 
            c*ArcSin[c/(a*v)])^2 + (-p1 + v*Cos[u])^2 + 
         (-p2 + v*Sin[u])^2),
     p2 + (rho^2*(-p2 + v*Sin[u]))/
       ((-p3 + sl*u + pm*(-c^2 + a^2*v^2)^(1/2) + 
            c*ArcSin[c/(a*v)])^2 + (-p1 + v*Cos[u])^2 + 
         (-p2 + v*Sin[u])^2),
     p3 + (rho^2*(-p3 + sl*u + 
           pm*(-c^2 + a^2*v^2)^(1/2) + c*ArcSin[c/(a*v)]))
        /((-p3 + sl*u + pm*(-c^2 + a^2*v^2)^(1/2) + 
            c*ArcSin[c/(a*v)])^2 + (-p1 + v*Cos[u])^2 + 
         (-p2 + v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][twisphere[a_,b_]][u_,v_]:=
{p1 + (rho^2*(-p1 + a*Cos[u]*Cos[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + 
         (-p3 + b*u + a*Sin[v])^2),
     p2 + (rho^2*(-p2 + a*Cos[v]*Sin[u]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + 
         (-p3 + b*u + a*Sin[v])^2),
     p3 + (rho^2*(-p3 + b*u + a*Sin[v]))/
       ((-p1 + a*Cos[u]*Cos[v])^2 + 
         (-p2 + a*Cos[v]*Sin[u])^2 + 
         (-p3 + b*u + a*Sin[v])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][wallis[a_,b_,c_]][u_,v_]:=
{p1 + (rho^2*(-p1 + v*Cos[u]))/
       ((-p1 + v*Cos[u])^2 + 
         (-p3 + c*(a^2 - b^2*Cos[u]^2)^(1/2))^2 + 
         (-p2 + v*Sin[u])^2),
     p2 + (rho^2*(-p2 + v*Sin[u]))/
       ((-p1 + v*Cos[u])^2 + 
         (-p3 + c*(a^2 - b^2*Cos[u]^2)^(1/2))^2 + 
         (-p2 + v*Sin[u])^2),
     p3 + (rho^2*(-p3 + c*(a^2 - b^2*Cos[u]^2)^(1/2)))/
       ((-p1 + v*Cos[u])^2 + 
         (-p3 + c*(a^2 - b^2*Cos[u]^2)^(1/2))^2 + 
         (-p2 + v*Sin[u])^2)}

inversionsurf[{p1_,p2_,p3_},rho_][whitneyumbrella][u_,v_]:=
{p1 + (rho^2*(-p1 + u*v))/
       ((-p2 + u)^2 + (-p1 + u*v)^2 + (-p3 + v^2)^2),
     p2 + (rho^2*(-p2 + u))/
       ((-p2 + u)^2 + (-p1 + u*v)^2 + (-p3 + v^2)^2),
     p3 + (rho^2*(-p3 + v^2))/
       ((-p2 + u)^2 + (-p1 + u*v)^2 + (-p3 + v^2)^2)}

inversionsurf[{p1_,p2_,p3_},rho_][wrinkles[a_]][u_,v_]:=
{p1 + (rho^2*(-p1 + u))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u/v + v/u))^2),
     p2 + (rho^2*(-p2 + v))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u/v + v/u))^2),
     p3 + (rho^2*(-p3 + a*(u/v + v/u)))/
       ((-p1 + u)^2 + (-p2 + v)^2 + 
         (-p3 + a*(u/v + v/u))^2)}

End[]

EndPackage[]
