(*:Date: Tue Feb  3 04:41:23 EST 1998
*)

(*:Title: PEDCUR.m *)

(*:Author: Alfred Gray *)

(* Copyright 1996-1998 by Alfred Gray *)

(*:Package Version: 1.0 *)

(*:Mathematica Version: 2.2, 3.0 *)

(*:Summary: 
This package consists of formulas for the  pedals of many plane curves.*)


BeginPackage["PEDCUR`","CURVES`","CSPROGS`"]

    pedal::usage="t->pedal[{a,b},alpha][t] is the pedal curve of the plane curve alpha with respect to the point {a,b}. It is the base of perpendicular line let down from {a,b} to the tangent line to alpha at alpha[t]. t->pedal[alpha][t] is the pedal curve of the plane curve alpha with respect to the origin."

Begin["`Private`"]

   pedal[{p1_,p2_},airyspiral[a_]][t_]:=
   {(p1*AiryAiPrime[t]^2 + 
     AiryAiPrime[t]*(p2 - a*AiryBi[t])*AiryBiPrime[t] + 
     a*AiryAi[t]*AiryBiPrime[t]^2)/
   (AiryAiPrime[t]^2 + AiryBiPrime[t]^2), 
   (a*AiryAiPrime[t]^2*AiryBi[t] + 
     (p1 - a*AiryAi[t])*AiryAiPrime[t]*AiryBiPrime[t] + 
     p2*AiryBiPrime[t]^2)/
   (AiryAiPrime[t]^2 + AiryBiPrime[t]^2)}

   pedal[{p1_,p2_},agnesi[a_]][t_]:=
    {p1 + (4*a*Cos[t]*Sin[t]*
         (2*a*(-p2 + 2*a*Cos[t]^2)*Sec[t]^2 + 
           4*a*Cos[t]*Sin[t]*(-p1 + 2*a*Tan[t])))/
       (4*a^2*Sec[t]^4 + 16*a^2*Cos[t]^2*Sin[t]^2),
     p2 + (2*a*Sec[t]^2*
         (2*a*(-p2 + 2*a*Cos[t]^2)*Sec[t]^2 + 
           4*a*Cos[t]*Sin[t]*(-p1 + 2*a*Tan[t])))/
       (4*a^2*Sec[t]^4 + 16*a^2*Cos[t]^2*Sin[t]^2)}

 pedal[{p1_,p2_},archimedesspiral[n_,a_]][t_]:=
    {p1 + ((-(a*t^n^(-1)*Cos[t]) - 
           (a*t^(-1 + n^(-1))*Sin[t])/n)*
         ((-p1 + a*t^n^(-1)*Cos[t])*
            (-(a*t^n^(-1)*Cos[t]) - 
              (a*t^(-1 + n^(-1))*Sin[t])/n) + 
           ((a*t^(-1 + n^(-1))*Cos[t])/n - 
              a*t^n^(-1)*Sin[t])*
            (-p2 + a*t^n^(-1)*Sin[t])))/
       ((a*t^n^(-1)*Cos[t] + 
            (a*t^(-1 + n^(-1))*Sin[t])/n)^2 + 
         ((a*t^(-1 + n^(-1))*Cos[t])/n - 
            a*t^n^(-1)*Sin[t])^2),
     p2 + (((a*t^(-1 + n^(-1))*Cos[t])/n - 
           a*t^n^(-1)*Sin[t])*
         ((-p1 + a*t^n^(-1)*Cos[t])*
            (-(a*t^n^(-1)*Cos[t]) - 
              (a*t^(-1 + n^(-1))*Sin[t])/n) + 
           ((a*t^(-1 + n^(-1))*Cos[t])/n - 
              a*t^n^(-1)*Sin[t])*
            (-p2 + a*t^n^(-1)*Sin[t])))/
       ((a*t^n^(-1)*Cos[t] + 
            (a*t^(-1 + n^(-1))*Sin[t])/n)^2 + 
         ((a*t^(-1 + n^(-1))*Cos[t])/n - 
            a*t^n^(-1)*Sin[t])^2)}

 pedal[{p1_,p2_},astroid[n_,a_,b_]][t_]:=
    {p1 - (b*n*Cos[t]*Sin[t]^(-1 + n)*
         (-(b*n*Cos[t]*(-p1 + a*Cos[t]^n)*
              Sin[t]^(-1 + n)) - 
           a*n*Cos[t]^(-1 + n)*Sin[t]*(-p2 + b*Sin[t]^n))
         )/(a^2*n^2*Cos[t]^(-2 + 2*n)*Sin[t]^2 + 
         b^2*n^2*Cos[t]^2*Sin[t]^(-2 + 2*n)),
     p2 - (a*n*Cos[t]^(-1 + n)*Sin[t]*
         (-(b*n*Cos[t]*(-p1 + a*Cos[t]^n)*
              Sin[t]^(-1 + n)) - 
           a*n*Cos[t]^(-1 + n)*Sin[t]*(-p2 + b*Sin[t]^n))
         )/(a^2*n^2*Cos[t]^(-2 + 2*n)*Sin[t]^2 + 
         b^2*n^2*Cos[t]^2*Sin[t]^(-2 + 2*n))}

pedal[{p1_,p2_},bellcurve[mean_,variance_]][t_]:=
{(mean^2*t + t^3 + 2*E^((mean - t)^2/variance^2)*p1*Pi*
      variance^6 + t*(variance^2 - 
        E^((mean - t)^2/(2*variance^2))*p2*Sqrt[2*Pi]*
         (variance^2)^(3/2)) - 
     mean*(2*t^2 + variance^2 - 
        E^((mean - t)^2/(2*variance^2))*p2*Sqrt[2*Pi]*
         (variance^2)^(3/2)))/
   (mean^2 - 2*mean*t + t^2 + 
     2*E^((mean - t)^2/variance^2)*Pi*variance^6), 
  p2 + (-p2 + ((-mean + t)*(-p1 + t))/
       (E^((mean - t)^2/(2*variance^2))*Sqrt[2*Pi]*
         (variance^2)^(3/2)) + 
      1/(E^((mean - t)^2/(2*variance^2))*Sqrt[2*Pi]*
         Sqrt[variance^2]))/
    (1 + (mean - t)^2/
       (2*E^((mean - t)^2/variance^2)*Pi*variance^6))}

pedal[{p1_,p2_},bicorn[a_]][t_]:=
{(227*p1 - 304*p1*Cos[t] + 100*p1*Cos[2*t] - 
     16*p1*Cos[3*t] + p1*Cos[4*t] + 114*a*Sin[t] - 
     144*p2*Sin[t] - 28*a*Sin[2*t] + 82*p2*Sin[2*t] - 
     14*a*Sin[3*t] - 16*p2*Sin[3*t] + 2*a*Sin[4*t] + 
     p2*Sin[4*t])/(292 - 320*Cos[t] + 36*Cos[2*t]), 
  (-122*a - 65*p2 + 2*(59*a + 8*p2)*Cos[t] + 
     8*(a + 8*p2)*Cos[2*t] - 14*a*Cos[3*t] - 
     16*p2*Cos[3*t] + 2*a*Cos[4*t] + p2*Cos[4*t] + 
     144*p1*Sin[t] - 82*p1*Sin[2*t] + 16*p1*Sin[3*t] - 
     p1*Sin[4*t])/(-292 + 320*Cos[t] - 36*Cos[2*t])}

pedal[{p1_,p2_},bow[a_]][t_]:=
{-(Sec[t]*(20*a - 42*p1*Cos[t] - 72*a*Cos[2*t] + 
        8*p1*Cos[3*t] + 16*a*Cos[4*t] + 24*p1*Cos[5*t] - 
        24*a*Cos[6*t] + 9*p1*Cos[7*t] - 4*a*Cos[8*t] + 
        p1*Cos[9*t] - 22*p2*Sin[t] - 4*p2*Sin[3*t] + 
        28*p2*Sin[5*t] + 11*p2*Sin[7*t] + p2*Sin[9*t]))/
   (8*(18 - 13*Cos[2*t] + 2*Cos[4*t] + Cos[6*t])), 
  (55*p2 - 60*p2*Cos[2*t] + 24*p2*Cos[4*t] + 
     12*p2*Cos[6*t] + p2*Cos[8*t] + 20*a*Sin[t] + 
     22*p1*Sin[2*t] - 4*a*Sin[3*t] - 18*p1*Sin[4*t] + 
     12*a*Sin[5*t] - 10*p1*Sin[6*t] + 4*a*Sin[7*t] - 
     p1*Sin[8*t])/
   (4*(18 - 13*Cos[2*t] + 2*Cos[4*t] + Cos[6*t]))}

 pedal[{p1_,p2_},cardioid[a_]][t_]:=
    {p1 + ((-2*a*Cos[t]*(1 + Cos[t]) + 2*a*Sin[t]^2)*
         ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])*
            (-p2 + 2*a*(1 + Cos[t])*Sin[t]) + 
           (-p1 + 2*a*Cos[t]*(1 + Cos[t]))*
            (-2*a*Cos[t]*(1 + Cos[t]) + 2*a*Sin[t]^2)))/
       ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])^
          2 + (2*a*Cos[t]*(1 + Cos[t]) - 2*a*Sin[t]^2)^2)
      ,p2 + ((-2*a*Cos[t]*Sin[t] - 
           2*a*(1 + Cos[t])*Sin[t])*
         ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])*
            (-p2 + 2*a*(1 + Cos[t])*Sin[t]) + 
           (-p1 + 2*a*Cos[t]*(1 + Cos[t]))*
            (-2*a*Cos[t]*(1 + Cos[t]) + 2*a*Sin[t]^2)))/
       ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])^
          2 + (2*a*Cos[t]*(1 + Cos[t]) - 2*a*Sin[t]^2)^2)
      }

pedal[{p1_,p2_},catenary[a_]][t_]:=
    {p1 - (p1 - a*Cosh[t/a] + (-p2 + t)*Sinh[t/a])/
       (1 + Sinh[t/a]^2),
     p2 + (Sinh[t/a]*(p1 - a*Cosh[t/a] + 
           (-p2 + t)*Sinh[t/a]))/(1 + Sinh[t/a]^2)}

 pedal[{p1_,p2_},cayleysextic[a_]][t_]:=
    {p1 + ((-6*a*Cos[t/2]^3*Cos[(3*t)/2] + 
           6*a*Cos[t/2]^2*Sin[t/2]*Sin[(3*t)/2])*
         ((-8*a*Cos[t/2]^3*(-1 + 2*Cos[t])*Sin[t/2] - 
              8*a*Cos[t/2]^4*Sin[t])*
            (-p2 + 4*a*Cos[t/2]^3*Sin[(3*t)/2]) + 
           (-p1 + 4*a*Cos[t/2]^4*(-1 + 2*Cos[t]))*
            (-6*a*Cos[t/2]^3*Cos[(3*t)/2] + 
              6*a*Cos[t/2]^2*Sin[t/2]*Sin[(3*t)/2])))/
       ((-8*a*Cos[t/2]^3*(-1 + 2*Cos[t])*Sin[t/2] - 
            8*a*Cos[t/2]^4*Sin[t])^2 + 
         (6*a*Cos[t/2]^3*Cos[(3*t)/2] - 
            6*a*Cos[t/2]^2*Sin[t/2]*Sin[(3*t)/2])^2),
     p2 + ((-8*a*Cos[t/2]^3*(-1 + 2*Cos[t])*Sin[t/2] - 
           8*a*Cos[t/2]^4*Sin[t])*
         ((-8*a*Cos[t/2]^3*(-1 + 2*Cos[t])*Sin[t/2] - 
              8*a*Cos[t/2]^4*Sin[t])*
            (-p2 + 4*a*Cos[t/2]^3*Sin[(3*t)/2]) + 
           (-p1 + 4*a*Cos[t/2]^4*(-1 + 2*Cos[t]))*
            (-6*a*Cos[t/2]^3*Cos[(3*t)/2] + 
              6*a*Cos[t/2]^2*Sin[t/2]*Sin[(3*t)/2])))/
       ((-8*a*Cos[t/2]^3*(-1 + 2*Cos[t])*Sin[t/2] - 
            8*a*Cos[t/2]^4*Sin[t])^2 + 
         (6*a*Cos[t/2]^3*Cos[(3*t)/2] - 
            6*a*Cos[t/2]^2*Sin[t/2]*Sin[(3*t)/2])^2)}

 pedal[{p1_,p2_},circle[a_]][t_]:=
    {p1 - (a*Cos[t]*(-(a*Cos[t]*(-p1 + a*Cos[t])) - 
           a*Sin[t]*(-p2 + a*Sin[t])))/
       (a^2*Cos[t]^2 + a^2*Sin[t]^2),
     p2 - (a*Sin[t]*(-(a*Cos[t]*(-p1 + a*Cos[t])) - 
           a*Sin[t]*(-p2 + a*Sin[t])))/
       (a^2*Cos[t]^2 + a^2*Sin[t]^2)}

 pedal[{p1_,p2_},circleinvolute[1,a_]][t_]:=
    {p1 - (a*t*Sin[t]*(a*t*Cos[t]*
            (-p2 + a*(-(t*Cos[t]) + Sin[t])) - 
           a*t*Sin[t]*(-p1 + a*(Cos[t] + t*Sin[t]))))/
       (a^2*t^2*Cos[t]^2 + a^2*t^2*Sin[t]^2),
     p2 + (a*t*Cos[t]*(a*t*Cos[t]*
            (-p2 + a*(-(t*Cos[t]) + Sin[t])) - 
           a*t*Sin[t]*(-p1 + a*(Cos[t] + t*Sin[t]))))/
       (a^2*t^2*Cos[t]^2 + a^2*t^2*Sin[t]^2)}

 pedal[{p1_,p2_},clothoid[1,a_]][t_]:=
    {p1 - (a*Cos[t^2/2]*
         (-(a*Cos[t^2/2]*
              (-p1 + a*Pi^(1/2)*FresnelS[t/Pi^(1/2)])) + 
           a*(-p2 + a*Pi^(1/2)*FresnelC[t/Pi^(1/2)])*
            Sin[t^2/2]))/
       (a^2*Cos[t^2/2]^2 + a^2*Sin[t^2/2]^2),
     p2 + (a*Sin[t^2/2]*
         (-(a*Cos[t^2/2]*
              (-p1 + a*Pi^(1/2)*FresnelS[t/Pi^(1/2)])) + 
           a*(-p2 + a*Pi^(1/2)*FresnelC[t/Pi^(1/2)])*
            Sin[t^2/2]))/
       (a^2*Cos[t^2/2]^2 + a^2*Sin[t^2/2]^2)}

 pedal[{p1_,p2_},clothoidprime[n_,a_]][t_]:=
    {p1 + (a*t^n*Sin[t^(1 + n)/(1 + n)]*
         (a*t^n*Cos[t^(1 + n)/(1 + n)]*
            (-p2 + a*Cos[t^(1 + n)/(1 + n)]) + 
           a*t^n*Sin[t^(1 + n)/(1 + n)]*
            (-p1 + a*Sin[t^(1 + n)/(1 + n)])))/
       (a^2*t^(2*n)*Cos[t^(1 + n)/(1 + n)]^2 + 
         a^2*t^(2*n)*Sin[t^(1 + n)/(1 + n)]^2),
     p2 + (a*t^n*Cos[t^(1 + n)/(1 + n)]*
         (a*t^n*Cos[t^(1 + n)/(1 + n)]*
            (-p2 + a*Cos[t^(1 + n)/(1 + n)]) + 
           a*t^n*Sin[t^(1 + n)/(1 + n)]*
            (-p1 + a*Sin[t^(1 + n)/(1 + n)])))/
       (a^2*t^(2*n)*Cos[t^(1 + n)/(1 + n)]^2 + 
         a^2*t^(2*n)*Sin[t^(1 + n)/(1 + n)]^2)}

 pedal[{p1_,p2_},cnccoprofile[pm_,a_,b_]][t_]:=
    {p1 - ((a^2 - b^2)^(1/2)*
         (1 - (b^2*Sinh[t/a]^2)/(a^2 - b^2)^(1/2))^(1/2)*
         ((b*pm*Cosh[t/a]*
              (-p2 - I*(a^2 - b^2)^(1/2)*
                 EllipticE[(I*t)/a,
                  -(b^2/(a^2 - b^2)^(1/2))]))/a - 
           ((a^2 - b^2)^(1/2)*(-p1 + b*pm*Sinh[t/a])*
              (1 - (b^2*Sinh[t/a]^2)/(a^2 - b^2)^(1/2))^
               (1/2))/a))/
       (a*((b^2*pm^2*Cosh[t/a]^2)/a^2 + 
           ((a^2 - b^2)*
              (1 - (b^2*Sinh[t/a]^2)/(a^2 - b^2)^(1/2)))/
            a^2)),p2 + 
      (b*pm*Cosh[t/a]*((b*pm*Cosh[t/a]*
              (-p2 - I*(a^2 - b^2)^(1/2)*
                 EllipticE[(I*t)/a,
                  -(b^2/(a^2 - b^2)^(1/2))]))/a - 
           ((a^2 - b^2)^(1/2)*(-p1 + b*pm*Sinh[t/a])*
              (1 - (b^2*Sinh[t/a]^2)/(a^2 - b^2)^(1/2))^
               (1/2))/a))/
       (a*((b^2*pm^2*Cosh[t/a]^2)/a^2 + 
           ((a^2 - b^2)*
              (1 - (b^2*Sinh[t/a]^2)/(a^2 - b^2)^(1/2)))/
            a^2))}
pedal[{p1_,p2_},
      cnchyprofile[pm_,a_,b_]][t_]:=
    {p1 - ((1 - (b^2*Sinh[t/a]^2)/a^2)^(1/2)*
         ((b*pm*(-p2 - 
                I*a*EllipticE[(I*t)/a,-(b^2/a^2)])*
              Sinh[t/a])/a - 
           (-p1 + b*pm*Cosh[t/a])*
            (1 - (b^2*Sinh[t/a]^2)/a^2)^(1/2)))/
       (1 - (b^2*Sinh[t/a]^2)/a^2 + 
         (b^2*pm^2*Sinh[t/a]^2)/a^2),
     p2 + (b*pm*Sinh[t/a]*
         ((b*pm*(-p2 - 
                I*a*EllipticE[(I*t)/a,-(b^2/a^2)])*
              Sinh[t/a])/a - 
           (-p1 + b*pm*Cosh[t/a])*
            (1 - (b^2*Sinh[t/a]^2)/a^2)^(1/2)))/
       (a*(1 - (b^2*Sinh[t/a]^2)/a^2 + 
           (b^2*pm^2*Sinh[t/a]^2)/a^2))}

 pedal[{p1_,p2_},cochleoid[n_,a_]][t_]:=
    {p1 + ((-(a*Cos[t]*(Sin[t]/t)^n) - 
           a*n*Sin[t]*(Sin[t]/t)^(-1 + n)*
            (Cos[t]/t - Sin[t]/t^2))*
         ((-p2 + a*Sin[t]*(Sin[t]/t)^n)*
            (-(a*Sin[t]*(Sin[t]/t)^n) + 
              a*n*Cos[t]*(Sin[t]/t)^(-1 + n)*
               (Cos[t]/t - Sin[t]/t^2)) + 
           (-p1 + a*Cos[t]*(Sin[t]/t)^n)*
            (-(a*Cos[t]*(Sin[t]/t)^n) - 
              a*n*Sin[t]*(Sin[t]/t)^(-1 + n)*
               (Cos[t]/t - Sin[t]/t^2))))/
       ((-(a*Sin[t]*(Sin[t]/t)^n) + 
            a*n*Cos[t]*(Sin[t]/t)^(-1 + n)*
             (Cos[t]/t - Sin[t]/t^2))^2 + 
         (a*Cos[t]*(Sin[t]/t)^n + 
            a*n*Sin[t]*(Sin[t]/t)^(-1 + n)*
             (Cos[t]/t - Sin[t]/t^2))^2),
     p2 + ((-(a*Sin[t]*(Sin[t]/t)^n) + 
           a*n*Cos[t]*(Sin[t]/t)^(-1 + n)*
            (Cos[t]/t - Sin[t]/t^2))*
         ((-p2 + a*Sin[t]*(Sin[t]/t)^n)*
            (-(a*Sin[t]*(Sin[t]/t)^n) + 
              a*n*Cos[t]*(Sin[t]/t)^(-1 + n)*
               (Cos[t]/t - Sin[t]/t^2)) + 
           (-p1 + a*Cos[t]*(Sin[t]/t)^n)*
            (-(a*Cos[t]*(Sin[t]/t)^n) - 
              a*n*Sin[t]*(Sin[t]/t)^(-1 + n)*
               (Cos[t]/t - Sin[t]/t^2))))/
       ((-(a*Sin[t]*(Sin[t]/t)^n) + 
            a*n*Cos[t]*(Sin[t]/t)^(-1 + n)*
             (Cos[t]/t - Sin[t]/t^2))^2 + 
         (a*Cos[t]*(Sin[t]/t)^n + 
            a*n*Sin[t]*(Sin[t]/t)^(-1 + n)*
             (Cos[t]/t - Sin[t]/t^2))^2)}

 pedal[{p1_,p2_},conchoid[a_,b_]][t_]:=
    {p1 + ((-(a*Cos[t]) - b*Sec[t]^2)*
         ((b - p1 + a*Cos[t])*
            (-(a*Cos[t]) - b*Sec[t]^2) - 
           a*Sin[t]*(-p2 + a*Sin[t] + b*Tan[t])))/
       ((a*Cos[t] + b*Sec[t]^2)^2 + a^2*Sin[t]^2),
     p2 - (a*Sin[t]*((b - p1 + a*Cos[t])*
            (-(a*Cos[t]) - b*Sec[t]^2) - 
           a*Sin[t]*(-p2 + a*Sin[t] + b*Tan[t])))/
       ((a*Cos[t] + b*Sec[t]^2)^2 + a^2*Sin[t]^2)}

 pedal[{p1_,p2_},cothspiral[a_]][t_]:=
    {p1 + (((-2*a*Cos[2*a*t])/(Cos[2*a*t] - Cosh[2*t]) + 
           (Sin[2*a*t]*(-2*a*Sin[2*a*t] - 2*Sinh[2*t]))/
            (Cos[2*a*t] - Cosh[2*t])^2)*
         (((-2*a*Cos[2*a*t])/(Cos[2*a*t] - Cosh[2*t]) + 
              (Sin[2*a*t]*
                 (-2*a*Sin[2*a*t] - 2*Sinh[2*t]))/
               (Cos[2*a*t] - Cosh[2*t])^2)*
            (-p1 - Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t])) + 
           (-p2 + Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t]))*
            ((-2*Cosh[2*t])/(Cos[2*a*t] - Cosh[2*t]) + 
              ((-2*a*Sin[2*a*t] - 2*Sinh[2*t])*
                 Sinh[2*t])/(Cos[2*a*t] - Cosh[2*t])^2)))
        /(((2*a*Cos[2*a*t])/(Cos[2*a*t] - Cosh[2*t]) - 
            (Sin[2*a*t]*(-2*a*Sin[2*a*t] - 2*Sinh[2*t]))/
             (Cos[2*a*t] - Cosh[2*t])^2)^2 + 
         ((-2*Cosh[2*t])/(Cos[2*a*t] - Cosh[2*t]) + 
            ((-2*a*Sin[2*a*t] - 2*Sinh[2*t])*Sinh[2*t])/
             (Cos[2*a*t] - Cosh[2*t])^2)^2),
     p2 + (((-2*Cosh[2*t])/(Cos[2*a*t] - Cosh[2*t]) + 
           ((-2*a*Sin[2*a*t] - 2*Sinh[2*t])*Sinh[2*t])/
            (Cos[2*a*t] - Cosh[2*t])^2)*
         (((-2*a*Cos[2*a*t])/(Cos[2*a*t] - Cosh[2*t]) + 
              (Sin[2*a*t]*
                 (-2*a*Sin[2*a*t] - 2*Sinh[2*t]))/
               (Cos[2*a*t] - Cosh[2*t])^2)*
            (-p1 - Sinh[2*t]/(Cos[2*a*t] - Cosh[2*t])) + 
           (-p2 + Sin[2*a*t]/(Cos[2*a*t] - Cosh[2*t]))*
            ((-2*Cosh[2*t])/(Cos[2*a*t] - Cosh[2*t]) + 
              ((-2*a*Sin[2*a*t] - 2*Sinh[2*t])*
                 Sinh[2*t])/(Cos[2*a*t] - Cosh[2*t])^2)))
        /(((2*a*Cos[2*a*t])/(Cos[2*a*t] - Cosh[2*t]) - 
            (Sin[2*a*t]*(-2*a*Sin[2*a*t] - 2*Sinh[2*t]))/
             (Cos[2*a*t] - Cosh[2*t])^2)^2 + 
         ((-2*Cosh[2*t])/(Cos[2*a*t] - Cosh[2*t]) + 
            ((-2*a*Sin[2*a*t] - 2*Sinh[2*t])*Sinh[2*t])/
             (Cos[2*a*t] - Cosh[2*t])^2)^2)}

 pedal[{p1_,p2_},cpcprofile[pm_,a_,b_]][t_]:=
    {p1 - ((1 - (b^2*Sin[t/a]^2)/a^2)^(1/2)*
         (-((b*pm*(-p2 + a*EllipticE[t/a,b^2/a^2])*
                Sin[t/a])/a) - 
           (-p1 + b*pm*Cos[t/a])*
            (1 - (b^2*Sin[t/a]^2)/a^2)^(1/2)))/
       (1 - (b^2*Sin[t/a]^2)/a^2 + 
         (b^2*pm^2*Sin[t/a]^2)/a^2),
     p2 - (b*pm*Sin[t/a]*
         (-((b*pm*(-p2 + a*EllipticE[t/a,b^2/a^2])*
                Sin[t/a])/a) - 
           (-p1 + b*pm*Cos[t/a])*
            (1 - (b^2*Sin[t/a]^2)/a^2)^(1/2)))/
       (a*(1 - (b^2*Sin[t/a]^2)/a^2 + 
           (b^2*pm^2*Sin[t/a]^2)/a^2))}

 pedal[{p1_,p2_},
      cubicparabola[a_,b_,c_,d_]][t_]:=
    {p1 + ((-c - 2*b*t - 3*a*t^2)*
         (d - p2 + c*t + b*t^2 + a*t^3 + 
           (-p1 + t)*(-c - 2*b*t - 3*a*t^2)))/
       (1 + (c + 2*b*t + 3*a*t^2)^2),
     p2 + (d - p2 + c*t + b*t^2 + a*t^3 + 
         (-p1 + t)*(-c - 2*b*t - 3*a*t^2))/
       (1 + (c + 2*b*t + 3*a*t^2)^2)}

 pedal[{p1_,p2_},cycloid[a_,b_]][t_]:=
    {p1 - (b*Sin[t]*((a - b*Cos[t])*
            (a - p2 - b*Cos[t]) - 
           b*Sin[t]*(-p1 + a*t - b*Sin[t])))/
       ((a - b*Cos[t])^2 + b^2*Sin[t]^2),
     p2 + ((a - b*Cos[t])*
         ((a - b*Cos[t])*(a - p2 - b*Cos[t]) - 
           b*Sin[t]*(-p1 + a*t - b*Sin[t])))/
       ((a - b*Cos[t])^2 + b^2*Sin[t]^2)}

 pedal[{p1_,p2_},deltoid[a_]][t_]:=
    {p1 + ((-2*a*(1 - Cos[t])*Cos[t] - 2*a*Sin[t]^2)*
         ((-p2 + 2*a*(1 - Cos[t])*Sin[t])*
            (-2*a*Cos[t]*Sin[t] - 
              2*a*(1 + Cos[t])*Sin[t]) + 
           (-a - p1 + 2*a*Cos[t]*(1 + Cos[t]))*
            (-2*a*(1 - Cos[t])*Cos[t] - 2*a*Sin[t]^2)))/
       ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])^
          2 + (2*a*(1 - Cos[t])*Cos[t] + 2*a*Sin[t]^2)^2)
      ,p2 + ((-2*a*Cos[t]*Sin[t] - 
           2*a*(1 + Cos[t])*Sin[t])*
         ((-p2 + 2*a*(1 - Cos[t])*Sin[t])*
            (-2*a*Cos[t]*Sin[t] - 
              2*a*(1 + Cos[t])*Sin[t]) + 
           (-a - p1 + 2*a*Cos[t]*(1 + Cos[t]))*
            (-2*a*(1 - Cos[t])*Cos[t] - 2*a*Sin[t]^2)))/
       ((-2*a*Cos[t]*Sin[t] - 2*a*(1 + Cos[t])*Sin[t])^
          2 + (2*a*(1 - Cos[t])*Cos[t] + 2*a*Sin[t]^2)^2)}

pedal[{p1_,p2_},diamond[n_,a_,b_]][t_]:=
    {p1 + ((-(b*(-1 + n)*Cos[t]*Sin[t]^2*
              (Sin[t]^2)^(-1 + (-1 + n)/2)) - 
           b*Cos[t]*(Sin[t]^2)^((-1 + n)/2))*
         ((-p1 + a*Cos[t]*(Cos[t]^2)^((-1 + n)/2))*
            (-(b*(-1 + n)*Cos[t]*Sin[t]^2*
                 (Sin[t]^2)^(-1 + (-1 + n)/2)) - 
              b*Cos[t]*(Sin[t]^2)^((-1 + n)/2)) + 
           (-(a*(-1 + n)*Cos[t]^2*
                 (Cos[t]^2)^(-1 + (-1 + n)/2)*Sin[t]) - 
              a*(Cos[t]^2)^((-1 + n)/2)*Sin[t])*
            (-p2 + b*Sin[t]*(Sin[t]^2)^((-1 + n)/2))))/
       ((-(a*(-1 + n)*Cos[t]^2*
               (Cos[t]^2)^(-1 + (-1 + n)/2)*Sin[t]) - 
            a*(Cos[t]^2)^((-1 + n)/2)*Sin[t])^2 + 
         (b*(-1 + n)*Cos[t]*Sin[t]^2*
             (Sin[t]^2)^(-1 + (-1 + n)/2) + 
            b*Cos[t]*(Sin[t]^2)^((-1 + n)/2))^2),
     p2 + ((-(a*(-1 + n)*Cos[t]^2*
              (Cos[t]^2)^(-1 + (-1 + n)/2)*Sin[t]) - 
           a*(Cos[t]^2)^((-1 + n)/2)*Sin[t])*
         ((-p1 + a*Cos[t]*(Cos[t]^2)^((-1 + n)/2))*
            (-(b*(-1 + n)*Cos[t]*Sin[t]^2*
                 (Sin[t]^2)^(-1 + (-1 + n)/2)) - 
              b*Cos[t]*(Sin[t]^2)^((-1 + n)/2)) + 
           (-(a*(-1 + n)*Cos[t]^2*
                 (Cos[t]^2)^(-1 + (-1 + n)/2)*Sin[t]) - 
              a*(Cos[t]^2)^((-1 + n)/2)*Sin[t])*
            (-p2 + b*Sin[t]*(Sin[t]^2)^((-1 + n)/2))))/
       ((-(a*(-1 + n)*Cos[t]^2*
               (Cos[t]^2)^(-1 + (-1 + n)/2)*Sin[t]) - 
            a*(Cos[t]^2)^((-1 + n)/2)*Sin[t])^2 + 
         (b*(-1 + n)*Cos[t]*Sin[t]^2*
             (Sin[t]^2)^(-1 + (-1 + n)/2) + 
            b*Cos[t]*(Sin[t]^2)^((-1 + n)/2))^2)}

 pedal[{p1_,p2_},eight][t_]:=
    {p1 + ((-Cos[t]^2 + Sin[t]^2)*
         (Cos[t]*(-p2 + Cos[t]*Sin[t]) + 
           (-p1 + Sin[t])*(-Cos[t]^2 + Sin[t]^2)))/
       (Cos[t]^2 + (Cos[t]^2 - Sin[t]^2)^2),
     p2 + (Cos[t]*(Cos[t]*(-p2 + Cos[t]*Sin[t]) + 
           (-p1 + Sin[t])*(-Cos[t]^2 + Sin[t]^2)))/
       (Cos[t]^2 + (Cos[t]^2 - Sin[t]^2)^2)}

 pedal[{p1_,p2_},ellipse[a_,b_]][t_]:=
    {p1 - (b*Cos[t]*(-(b*Cos[t]*(-p1 + a*Cos[t])) - 
           a*Sin[t]*(-p2 + b*Sin[t])))/
       (b^2*Cos[t]^2 + a^2*Sin[t]^2),
     p2 - (a*Sin[t]*(-(b*Cos[t]*(-p1 + a*Cos[t])) - 
           a*Sin[t]*(-p2 + b*Sin[t])))/
       (b^2*Cos[t]^2 + a^2*Sin[t]^2)}

 pedal[{p1_,p2_},epicycloid[a_,b_]][t_]:=
    {p1 + ((-((a + b)*Cos[t]) + 
           (a + b)*Cos[((a + b)*t)/b])*
         ((-p1 + (a + b)*Cos[t] - b*Cos[((a + b)*t)/b])*
            (-((a + b)*Cos[t]) + 
              (a + b)*Cos[((a + b)*t)/b]) + 
           (-p2 + (a + b)*Sin[t] - b*Sin[((a + b)*t)/b])*
            (-((a + b)*Sin[t]) + 
              (a + b)*Sin[((a + b)*t)/b])))/
       (((a + b)*Cos[t] - (a + b)*Cos[((a + b)*t)/b])^
          2 + (-((a + b)*Sin[t]) + 
            (a + b)*Sin[((a + b)*t)/b])^2),
     p2 + ((-((a + b)*Sin[t]) + 
           (a + b)*Sin[((a + b)*t)/b])*
         ((-p1 + (a + b)*Cos[t] - b*Cos[((a + b)*t)/b])*
            (-((a + b)*Cos[t]) + 
              (a + b)*Cos[((a + b)*t)/b]) + 
           (-p2 + (a + b)*Sin[t] - b*Sin[((a + b)*t)/b])*
            (-((a + b)*Sin[t]) + 
              (a + b)*Sin[((a + b)*t)/b])))/
       (((a + b)*Cos[t] - (a + b)*Cos[((a + b)*t)/b])^
          2 + (-((a + b)*Sin[t]) + 
            (a + b)*Sin[((a + b)*t)/b])^2)}

 pedal[{p1_,p2_},epitrochoid[a_,b_,h_]][t_]:=
    {p1 + ((-((a + b)*Cos[t]) + 
           ((a + b)*h*Cos[((a + b)*t)/b])/b)*
         ((-p1 + (a + b)*Cos[t] - h*Cos[((a + b)*t)/b])*
            (-((a + b)*Cos[t]) + 
              ((a + b)*h*Cos[((a + b)*t)/b])/b) + 
           (-p2 + (a + b)*Sin[t] - h*Sin[((a + b)*t)/b])*
            (-((a + b)*Sin[t]) + 
              ((a + b)*h*Sin[((a + b)*t)/b])/b)))/
       (((a + b)*Cos[t] - 
            ((a + b)*h*Cos[((a + b)*t)/b])/b)^2 + 
         (-((a + b)*Sin[t]) + 
            ((a + b)*h*Sin[((a + b)*t)/b])/b)^2),
     p2 + ((-((a + b)*Sin[t]) + 
           ((a + b)*h*Sin[((a + b)*t)/b])/b)*
         ((-p1 + (a + b)*Cos[t] - h*Cos[((a + b)*t)/b])*
            (-((a + b)*Cos[t]) + 
              ((a + b)*h*Cos[((a + b)*t)/b])/b) + 
           (-p2 + (a + b)*Sin[t] - h*Sin[((a + b)*t)/b])*
            (-((a + b)*Sin[t]) + 
              ((a + b)*h*Sin[((a + b)*t)/b])/b)))/
       (((a + b)*Cos[t] - 
            ((a + b)*h*Cos[((a + b)*t)/b])/b)^2 + 
         (-((a + b)*Sin[t]) + 
            ((a + b)*h*Sin[((a + b)*t)/b])/b)^2)}

 pedal[{p1_,p2_},cissoid[a_]][t_]:=
    {p1 + (((4*a*t^4)/(1 + t^2)^2 - (6*a*t^2)/(1 + t^2))*
         (((4*a*t^4)/(1 + t^2)^2 - (6*a*t^2)/(1 + t^2))*
            (-p1 + (2*a*t^2)/(1 + t^2)) + 
           ((-4*a*t^3)/(1 + t^2)^2 + (4*a*t)/(1 + t^2))*
            (-p2 + (2*a*t^3)/(1 + t^2))))/
       (((-4*a*t^3)/(1 + t^2)^2 + (4*a*t)/(1 + t^2))^2 + 
         ((-4*a*t^4)/(1 + t^2)^2 + (6*a*t^2)/(1 + t^2))^2
         ),p2 + (((-4*a*t^3)/(1 + t^2)^2 + 
           (4*a*t)/(1 + t^2))*
         (((4*a*t^4)/(1 + t^2)^2 - (6*a*t^2)/(1 + t^2))*
            (-p1 + (2*a*t^2)/(1 + t^2)) + 
           ((-4*a*t^3)/(1 + t^2)^2 + (4*a*t)/(1 + t^2))*
            (-p2 + (2*a*t^3)/(1 + t^2))))/
       (((-4*a*t^3)/(1 + t^2)^2 + (4*a*t)/(1 + t^2))^2 + 
         ((-4*a*t^4)/(1 + t^2)^2 + (6*a*t^2)/(1 + t^2))^2
         )}

pedal[{p1_,p2_},
      fermatspiral[a_]][t_]:=
    {p1 + ((-(a*t^(1/2)*Cos[t]) - 
           (a*Sin[t])/(2*t^(1/2)))*
         ((-p1 + a*t^(1/2)*Cos[t])*
            (-(a*t^(1/2)*Cos[t]) - 
              (a*Sin[t])/(2*t^(1/2))) + 
           ((a*Cos[t])/(2*t^(1/2)) - a*t^(1/2)*Sin[t])*
            (-p2 + a*t^(1/2)*Sin[t])))/
       ((a*t^(1/2)*Cos[t] + (a*Sin[t])/(2*t^(1/2)))^2 + 
         ((a*Cos[t])/(2*t^(1/2)) - a*t^(1/2)*Sin[t])^2),
     p2 + (((a*Cos[t])/(2*t^(1/2)) - a*t^(1/2)*Sin[t])*
         ((-p1 + a*t^(1/2)*Cos[t])*
            (-(a*t^(1/2)*Cos[t]) - 
              (a*Sin[t])/(2*t^(1/2))) + 
           ((a*Cos[t])/(2*t^(1/2)) - a*t^(1/2)*Sin[t])*
            (-p2 + a*t^(1/2)*Sin[t])))/
       ((a*t^(1/2)*Cos[t] + (a*Sin[t])/(2*t^(1/2)))^2 + 
         ((a*Cos[t])/(2*t^(1/2)) - a*t^(1/2)*Sin[t])^2)}

  pedal[{p1_,p2_},folium][t_]:=
    {p1 + (((9*t^4)/(1 + t^3)^2 - (6*t)/(1 + t^3))*
         (((9*t^4)/(1 + t^3)^2 - (6*t)/(1 + t^3))*
            (-p1 + (3*t)/(1 + t^3)) + 
           ((-9*t^3)/(1 + t^3)^2 + 3/(1 + t^3))*
            (-p2 + (3*t^2)/(1 + t^3))))/
       (((-9*t^3)/(1 + t^3)^2 + 3/(1 + t^3))^2 + 
         ((-9*t^4)/(1 + t^3)^2 + (6*t)/(1 + t^3))^2),
     p2 + (((-9*t^3)/(1 + t^3)^2 + 3/(1 + t^3))*
         (((9*t^4)/(1 + t^3)^2 - (6*t)/(1 + t^3))*
            (-p1 + (3*t)/(1 + t^3)) + 
           ((-9*t^3)/(1 + t^3)^2 + 3/(1 + t^3))*
            (-p2 + (3*t^2)/(1 + t^3))))/
       (((-9*t^3)/(1 + t^3)^2 + 3/(1 + t^3))^2 + 
         ((-9*t^4)/(1 + t^3)^2 + (6*t)/(1 + t^3))^2)}

 pedal[{p1_,p2_},hankelspiral[n_,a_]][t_]:=
{p1 - ((BesselJ[1 + n, t]*(p2 - a*BesselY[n, t]) + 
        BesselJ[-1 + n, t]*(-p2 + a*BesselY[n, t]) + 
        (p1 - a*BesselJ[n, t])*
         (BesselY[-1 + n, t] - BesselY[1 + n, t]))*
      (BesselY[-1 + n, t] - BesselY[1 + n, t]))/
    (BesselJ[-1 + n, t]^2 - 
      2*BesselJ[-1 + n, t]*BesselJ[1 + n, t] + 
      BesselJ[1 + n, t]^2 + 
      (BesselY[-1 + n, t] - BesselY[1 + n, t])^2), 
  p2 + ((BesselJ[-1 + n, t] - BesselJ[1 + n, t])*
      (BesselJ[1 + n, t]*(p2 - a*BesselY[n, t]) + 
        BesselJ[-1 + n, t]*(-p2 + a*BesselY[n, t]) + 
        (p1 - a*BesselJ[n, t])*
         (BesselY[-1 + n, t] - BesselY[1 + n, t])))/
    (BesselJ[-1 + n, t]^2 - 
      2*BesselJ[-1 + n, t]*BesselJ[1 + n, t] + 
      BesselJ[1 + n, t]^2 + 
      (BesselY[-1 + n, t] - BesselY[1 + n, t])^2)}

 pedal[{p1_,p2_},hyperbola[a_,b_]][t_]:=
    {p1 - (b*Cosh[t]*(-(b*Cosh[t]*(-p1 + a*Cosh[t])) + 
           a*Sinh[t]*(-p2 + b*Sinh[t])))/
       (b^2*Cosh[t]^2 + a^2*Sinh[t]^2),
     p2 + (a*Sinh[t]*(-(b*Cosh[t]*(-p1 + a*Cosh[t])) + 
           a*Sinh[t]*(-p2 + b*Sinh[t])))/
       (b^2*Cosh[t]^2 + a^2*Sinh[t]^2)}

 pedal[{p1_,p2_},hyperbolicspiral[a_]][t_]:=
    {p1 + ((-((a*Cos[t])/t) + (a*Sin[t])/t^2)*
         ((-p1 + (a*Cos[t])/t)*
            (-((a*Cos[t])/t) + (a*Sin[t])/t^2) + 
           (-((a*Cos[t])/t^2) - (a*Sin[t])/t)*
            (-p2 + (a*Sin[t])/t)))/
       (((a*Cos[t])/t - (a*Sin[t])/t^2)^2 + 
         (-((a*Cos[t])/t^2) - (a*Sin[t])/t)^2),
     p2 + ((-((a*Cos[t])/t^2) - (a*Sin[t])/t)*
         ((-p1 + (a*Cos[t])/t)*
            (-((a*Cos[t])/t) + (a*Sin[t])/t^2) + 
           (-((a*Cos[t])/t^2) - (a*Sin[t])/t)*
            (-p2 + (a*Sin[t])/t)))/
       (((a*Cos[t])/t - (a*Sin[t])/t^2)^2 + 
         (-((a*Cos[t])/t^2) - (a*Sin[t])/t)^2)}

 pedal[{p1_,p2_},hypocycloid[a_,b_]][t_]:=
    {p1 + ((-((a - b)*Cos[t]) + 
           (a - b)*Cos[((a - b)*t)/b])*
         ((-((a - b)*Cos[t]) + 
              (a - b)*Cos[((a - b)*t)/b])*
            (-p1 + (a - b)*Cos[t] + b*Cos[((a - b)*t)/b])
             + (-((a - b)*Sin[t]) - 
              (a - b)*Sin[((a - b)*t)/b])*
            (-p2 + (a - b)*Sin[t] - b*Sin[((a - b)*t)/b])))/
       (((a - b)*Cos[t] - (a - b)*Cos[((a - b)*t)/b])^
          2 + (-((a - b)*Sin[t]) - 
            (a - b)*Sin[((a - b)*t)/b])^2),
     p2 + ((-((a - b)*Sin[t]) - 
           (a - b)*Sin[((a - b)*t)/b])*
         ((-((a - b)*Cos[t]) + 
              (a - b)*Cos[((a - b)*t)/b])*
            (-p1 + (a - b)*Cos[t] + b*Cos[((a - b)*t)/b])
             + (-((a - b)*Sin[t]) - 
              (a - b)*Sin[((a - b)*t)/b])*
            (-p2 + (a - b)*Sin[t] - b*Sin[((a - b)*t)/b])
           ))/
       (((a - b)*Cos[t] - (a - b)*Cos[((a - b)*t)/b])^
          2 + (-((a - b)*Sin[t]) - 
            (a - b)*Sin[((a - b)*t)/b])^2)}

 pedal[{p1_,p2_},hypotrochoid[a_,b_,h_]][t_]:=
    {p1 + ((-((a - b)*Cos[t]) + 
           ((a - b)*h*Cos[((a - b)*t)/b])/b)*
         ((-p1 + (a - b)*Cos[t] + h*Cos[((a - b)*t)/b])*
            (-((a - b)*Cos[t]) + 
              ((a - b)*h*Cos[((a - b)*t)/b])/b) + 
           (-p2 + (a - b)*Sin[t] - h*Sin[((a - b)*t)/b])*
            (-((a - b)*Sin[t]) - 
              ((a - b)*h*Sin[((a - b)*t)/b])/b)))/
       (((a - b)*Cos[t] - 
            ((a - b)*h*Cos[((a - b)*t)/b])/b)^2 + 
         (-((a - b)*Sin[t]) - 
            ((a - b)*h*Sin[((a - b)*t)/b])/b)^2),
     p2 + ((-((a - b)*Sin[t]) - 
           ((a - b)*h*Sin[((a - b)*t)/b])/b)*
         ((-p1 + (a - b)*Cos[t] + h*Cos[((a - b)*t)/b])*
            (-((a - b)*Cos[t]) + 
              ((a - b)*h*Cos[((a - b)*t)/b])/b) + 
           (-p2 + (a - b)*Sin[t] - h*Sin[((a - b)*t)/b])*
            (-((a - b)*Sin[t]) - 
              ((a - b)*h*Sin[((a - b)*t)/b])/b)))/
       (((a - b)*Cos[t] - 
            ((a - b)*h*Cos[((a - b)*t)/b])/b)^2 + 
         (-((a - b)*Sin[t]) - 
            ((a - b)*h*Sin[((a - b)*t)/b])/b)^2)}

 pedal[{p1_,p2_},lemniscate[a_]][t_]:=
    {p1 + (((2*a*Cos[t]^2*Sin[t]^2)/(1 + Sin[t]^2)^2 - 
           (a*Cos[t]^2)/(1 + Sin[t]^2) + 
           (a*Sin[t]^2)/(1 + Sin[t]^2))*
         (((-2*a*Cos[t]^2*Sin[t])/(1 + Sin[t]^2)^2 - 
              (a*Sin[t])/(1 + Sin[t]^2))*
            (-p2 + (a*Cos[t]*Sin[t])/(1 + Sin[t]^2)) + 
           (-p1 + (a*Cos[t])/(1 + Sin[t]^2))*
            ((2*a*Cos[t]^2*Sin[t]^2)/(1 + Sin[t]^2)^2 - 
              (a*Cos[t]^2)/(1 + Sin[t]^2) + 
              (a*Sin[t]^2)/(1 + Sin[t]^2))))/
       (((-2*a*Cos[t]^2*Sin[t])/(1 + Sin[t]^2)^2 - 
            (a*Sin[t])/(1 + Sin[t]^2))^2 + 
         ((-2*a*Cos[t]^2*Sin[t]^2)/(1 + Sin[t]^2)^2 + 
            (a*Cos[t]^2)/(1 + Sin[t]^2) - 
            (a*Sin[t]^2)/(1 + Sin[t]^2))^2),
     p2 + (((-2*a*Cos[t]^2*Sin[t])/(1 + Sin[t]^2)^2 - 
           (a*Sin[t])/(1 + Sin[t]^2))*
         (((-2*a*Cos[t]^2*Sin[t])/(1 + Sin[t]^2)^2 - 
              (a*Sin[t])/(1 + Sin[t]^2))*
            (-p2 + (a*Cos[t]*Sin[t])/(1 + Sin[t]^2)) + 
           (-p1 + (a*Cos[t])/(1 + Sin[t]^2))*
            ((2*a*Cos[t]^2*Sin[t]^2)/(1 + Sin[t]^2)^2 - 
              (a*Cos[t]^2)/(1 + Sin[t]^2) + 
              (a*Sin[t]^2)/(1 + Sin[t]^2))))/
       (((-2*a*Cos[t]^2*Sin[t])/(1 + Sin[t]^2)^2 - 
            (a*Sin[t])/(1 + Sin[t]^2))^2 + 
         ((-2*a*Cos[t]^2*Sin[t]^2)/(1 + Sin[t]^2)^2 + 
            (a*Cos[t]^2)/(1 + Sin[t]^2) - 
            (a*Sin[t]^2)/(1 + Sin[t]^2))^2)}

 pedal[{p1_,p2_},limacon[a_,b_]][t_]:=
    {p1 + ((-(Cos[t]*(b + 2*a*Cos[t])) + 2*a*Sin[t]^2)*
         ((-2*a*Cos[t]*Sin[t] - (b + 2*a*Cos[t])*Sin[t])*
            (-p2 + (b + 2*a*Cos[t])*Sin[t]) + 
           (-p1 + Cos[t]*(b + 2*a*Cos[t]))*
            (-(Cos[t]*(b + 2*a*Cos[t])) + 2*a*Sin[t]^2)))
        /((-2*a*Cos[t]*Sin[t] - (b + 2*a*Cos[t])*Sin[t])^
          2 + (Cos[t]*(b + 2*a*Cos[t]) - 2*a*Sin[t]^2)^2)
      ,p2 + ((-2*a*Cos[t]*Sin[t] - 
           (b + 2*a*Cos[t])*Sin[t])*
         ((-2*a*Cos[t]*Sin[t] - (b + 2*a*Cos[t])*Sin[t])*
            (-p2 + (b + 2*a*Cos[t])*Sin[t]) + 
           (-p1 + Cos[t]*(b + 2*a*Cos[t]))*
            (-(Cos[t]*(b + 2*a*Cos[t])) + 2*a*Sin[t]^2)))
        /((-2*a*Cos[t]*Sin[t] - (b + 2*a*Cos[t])*Sin[t])^
          2 + (Cos[t]*(b + 2*a*Cos[t]) - 2*a*Sin[t]^2)^2)}

pedal[{p1_,p2_},line[a1_,b1_,a2_,b2_]][t_]:=
    {p1 - (b1*(-(b1*(a2 - p1 + a1*t)) + 
           a1*(b2 - p2 + b1*t)))/(a1^2 + b1^2),
     p2 + (a1*(-(b1*(a2 - p1 + a1*t)) + 
           a1*(b2 - p2 + b1*t)))/(a1^2 + b1^2)}

 pedal[{p1_,p2_},lissajous[n_,d_,a_,b_]][t_]:=
    {p1 - (b*Cos[t]*(a*n*Cos[d + n*t]*(-p2 + b*Sin[t]) - 
           b*Cos[t]*(-p1 + a*Sin[d + n*t])))/
       (b^2*Cos[t]^2 + a^2*n^2*Cos[d + n*t]^2),
     p2 + (a*n*Cos[d + n*t]*
         (a*n*Cos[d + n*t]*(-p2 + b*Sin[t]) - 
           b*Cos[t]*(-p1 + a*Sin[d + n*t])))/
       (b^2*Cos[t]^2 + a^2*n^2*Cos[d + n*t]^2)}

 pedal[{p1_,p2_},lituus[a_]][t_]:=
    {p1 + ((-((a*t^2*Cos[(t^2)^(1/2)])/(t^2)^(5/4)) + 
           (3*a*t^2*Sin[(t^2)^(1/2)])/(2*(t^2)^(7/4)) - 
           (a*Sin[(t^2)^(1/2)])/(t^2)^(3/4))*
         ((-p1 + (a*t*Cos[(t^2)^(1/2)])/(t^2)^(3/4))*
            (-((a*t^2*Cos[(t^2)^(1/2)])/(t^2)^(5/4)) + 
              (3*a*t^2*Sin[(t^2)^(1/2)])/
               (2*(t^2)^(7/4)) - 
              (a*Sin[(t^2)^(1/2)])/(t^2)^(3/4)) + 
           ((-3*a*t^2*Cos[(t^2)^(1/2)])/
               (2*(t^2)^(7/4)) + 
              (a*Cos[(t^2)^(1/2)])/(t^2)^(3/4) - 
              (a*t^2*Sin[(t^2)^(1/2)])/(t^2)^(5/4))*
            (-p2 + (a*t*Sin[(t^2)^(1/2)])/(t^2)^(3/4))))/
       (((-3*a*t^2*Cos[(t^2)^(1/2)])/(2*(t^2)^(7/4)) + 
            (a*Cos[(t^2)^(1/2)])/(t^2)^(3/4) - 
            (a*t^2*Sin[(t^2)^(1/2)])/(t^2)^(5/4))^2 + 
         ((a*t^2*Cos[(t^2)^(1/2)])/(t^2)^(5/4) - 
            (3*a*t^2*Sin[(t^2)^(1/2)])/(2*(t^2)^(7/4)) + 
            (a*Sin[(t^2)^(1/2)])/(t^2)^(3/4))^2),
     p2 + (((-3*a*t^2*Cos[(t^2)^(1/2)])/
            (2*(t^2)^(7/4)) + 
           (a*Cos[(t^2)^(1/2)])/(t^2)^(3/4) - 
           (a*t^2*Sin[(t^2)^(1/2)])/(t^2)^(5/4))*
         ((-p1 + (a*t*Cos[(t^2)^(1/2)])/(t^2)^(3/4))*
            (-((a*t^2*Cos[(t^2)^(1/2)])/(t^2)^(5/4)) + 
              (3*a*t^2*Sin[(t^2)^(1/2)])/
               (2*(t^2)^(7/4)) - 
              (a*Sin[(t^2)^(1/2)])/(t^2)^(3/4)) + 
           ((-3*a*t^2*Cos[(t^2)^(1/2)])/
               (2*(t^2)^(7/4)) + 
              (a*Cos[(t^2)^(1/2)])/(t^2)^(3/4) - 
              (a*t^2*Sin[(t^2)^(1/2)])/(t^2)^(5/4))*
            (-p2 + (a*t*Sin[(t^2)^(1/2)])/(t^2)^(3/4))))/
       (((-3*a*t^2*Cos[(t^2)^(1/2)])/(2*(t^2)^(7/4)) + 
            (a*Cos[(t^2)^(1/2)])/(t^2)^(3/4) - 
            (a*t^2*Sin[(t^2)^(1/2)])/(t^2)^(5/4))^2 + 
         ((a*t^2*Cos[(t^2)^(1/2)])/(t^2)^(5/4) - 
            (3*a*t^2*Sin[(t^2)^(1/2)])/(2*(t^2)^(7/4)) + 
            (a*Sin[(t^2)^(1/2)])/(t^2)^(3/4))^2)}

 pedal[{p1_,p2_},logspiral[a_,b_]][t_]:=
    {p1 + ((-(a*E^(b*t)*Cos[t]) - a*b*E^(b*t)*Sin[t])*
         ((a*b*E^(b*t)*Cos[t] - a*E^(b*t)*Sin[t])*
            (-p2 + a*E^(b*t)*Sin[t]) + 
           (-p1 + a*E^(b*t)*Cos[t])*
            (-(a*E^(b*t)*Cos[t]) - a*b*E^(b*t)*Sin[t])))/
       ((a*b*E^(b*t)*Cos[t] - a*E^(b*t)*Sin[t])^2 + 
         (a*E^(b*t)*Cos[t] + a*b*E^(b*t)*Sin[t])^2),
     p2 + ((a*b*E^(b*t)*Cos[t] - a*E^(b*t)*Sin[t])*
         ((a*b*E^(b*t)*Cos[t] - a*E^(b*t)*Sin[t])*
            (-p2 + a*E^(b*t)*Sin[t]) + 
           (-p1 + a*E^(b*t)*Cos[t])*
            (-(a*E^(b*t)*Cos[t]) - a*b*E^(b*t)*Sin[t])))/
       ((a*b*E^(b*t)*Cos[t] - a*E^(b*t)*Sin[t])^2 + 
         (a*E^(b*t)*Cos[t] + a*b*E^(b*t)*Sin[t])^2)}

 pedal[{p1_,p2_},nephroid[a_]][t_]:=
    {p1 - (a*(3*Cos[t] - 3*Cos[3*t])*
         (-(a*(-p1 + a*(3*Cos[t] - Cos[3*t]))*
              (3*Cos[t] - 3*Cos[3*t])) + 
           a*(-p2 + a*(3*Sin[t] - Sin[3*t]))*
            (-3*Sin[t] + 3*Sin[3*t])))/
       (a^2*(3*Cos[t] - 3*Cos[3*t])^2 + 
         a^2*(-3*Sin[t] + 3*Sin[3*t])^2),
     p2 + (a*(-3*Sin[t] + 3*Sin[3*t])*
         (-(a*(-p1 + a*(3*Cos[t] - Cos[3*t]))*
              (3*Cos[t] - 3*Cos[3*t])) + 
           a*(-p2 + a*(3*Sin[t] - Sin[3*t]))*
            (-3*Sin[t] + 3*Sin[3*t])))/
       (a^2*(3*Cos[t] - 3*Cos[3*t])^2 + 
         a^2*(-3*Sin[t] + 3*Sin[3*t])^2)}

 pedal[{p1_,p2_},nielsenspiral[a_]][t_]:=
    {p1 - (a*Sin[t]*(-((a*(-p1 + a*CosIntegral[t])*
                Sin[t])/t) + 
           (a*Cos[t]*(-p2 + a*SinIntegral[t]))/t))/
       (t*((a^2*Cos[t]^2)/t^2 + (a^2*Sin[t]^2)/t^2)),
     p2 + (a*Cos[t]*(-((a*(-p1 + a*CosIntegral[t])*
                Sin[t])/t) + 
           (a*Cos[t]*(-p2 + a*SinIntegral[t]))/t))/
       (t*((a^2*Cos[t]^2)/t^2 + (a^2*Sin[t]^2)/t^2))}

 pedal[{p1_,p2_},parabola[a_]][t_]:=
    {p1 - (2*a*t*(-2*a*t*(-p1 + 2*a*t) + 
           2*a*(-p2 + a*t^2)))/(4*a^2 + 4*a^2*t^2),
     p2 + (2*a*(-2*a*t*(-p1 + 2*a*t) + 
           2*a*(-p2 + a*t^2)))/(4*a^2 + 4*a^2*t^2)}

 pedal[{p1_,p2_},piriform[a_,b_]][t_]:=
    {p1 + ((-(b*Cos[t]^2) + b*Sin[t]*(1 + Sin[t]))*
         (a*Cos[t]*(-p2 + b*Cos[t]*(1 + Sin[t])) + 
           (-p1 + a*(1 + Sin[t]))*
            (-(b*Cos[t]^2) + b*Sin[t]*(1 + Sin[t]))))/
       (a^2*Cos[t]^2 + 
         (b*Cos[t]^2 - b*Sin[t]*(1 + Sin[t]))^2),
     p2 + (a*Cos[t]*(a*Cos[t]*
            (-p2 + b*Cos[t]*(1 + Sin[t])) + 
           (-p1 + a*(1 + Sin[t]))*
            (-(b*Cos[t]^2) + b*Sin[t]*(1 + Sin[t]))))/
       (a^2*Cos[t]^2 + 
         (b*Cos[t]^2 - b*Sin[t]*(1 + Sin[t]))^2)}

 pedal[{p1_,p2_},rose[n_,a_]][t_]:=
    {p1 + ((-(a*Cos[t]*Cos[n*t]) + a*n*Sin[t]*Sin[n*t])*
         ((-p2 + a*Cos[n*t]*Sin[t])*
            (-(a*Cos[n*t]*Sin[t]) - a*n*Cos[t]*Sin[n*t]) 
            + (-p1 + a*Cos[t]*Cos[n*t])*
            (-(a*Cos[t]*Cos[n*t]) + a*n*Sin[t]*Sin[n*t]))
         )/((-(a*Cos[n*t]*Sin[t]) - a*n*Cos[t]*Sin[n*t])^
          2 + (a*Cos[t]*Cos[n*t] - a*n*Sin[t]*Sin[n*t])^2
         ),p2 + ((-(a*Cos[n*t]*Sin[t]) - 
           a*n*Cos[t]*Sin[n*t])*
         ((-p2 + a*Cos[n*t]*Sin[t])*
            (-(a*Cos[n*t]*Sin[t]) - a*n*Cos[t]*Sin[n*t]) 
            + (-p1 + a*Cos[t]*Cos[n*t])*
            (-(a*Cos[t]*Cos[n*t]) + a*n*Sin[t]*Sin[n*t]))
         )/((-(a*Cos[n*t]*Sin[t]) - a*n*Cos[t]*Sin[n*t])^
          2 + (a*Cos[t]*Cos[n*t] - a*n*Sin[t]*Sin[n*t])^2
         )}

pedal[{p1_,p2_},sc][t_]:=
    {p1 - (3*t^2*(-3*t^2*(-p1 + t^2) + 2*t*(-p2 + t^3)))/
       (4*t^2 + 9*t^4),
     p2 + (2*t*(-3*t^2*(-p1 + t^2) + 2*t*(-p2 + t^3)))/
       (4*t^2 + 9*t^4)}

 pedal[{p1_,p2_},strophoid[a_]][t_]:=
    {p1 + (((2*a*t^2*(-1 + t^2))/(1 + t^2)^2 - 
           (2*a*t^2)/(1 + t^2) - (a*(-1 + t^2))/(1 + t^2)
           )*(((2*a*t^2*(-1 + t^2))/(1 + t^2)^2 - 
              (2*a*t^2)/(1 + t^2) - 
              (a*(-1 + t^2))/(1 + t^2))*
            (-p1 + (a*(-1 + t^2))/(1 + t^2)) + 
           ((-2*a*t*(-1 + t^2))/(1 + t^2)^2 + 
              (2*a*t)/(1 + t^2))*
            (-p2 + (a*t*(-1 + t^2))/(1 + t^2))))/
       (((-2*a*t*(-1 + t^2))/(1 + t^2)^2 + 
            (2*a*t)/(1 + t^2))^2 + 
         ((-2*a*t^2*(-1 + t^2))/(1 + t^2)^2 + 
            (2*a*t^2)/(1 + t^2) + 
            (a*(-1 + t^2))/(1 + t^2))^2),
     p2 + (((-2*a*t*(-1 + t^2))/(1 + t^2)^2 + 
           (2*a*t)/(1 + t^2))*
         (((2*a*t^2*(-1 + t^2))/(1 + t^2)^2 - 
              (2*a*t^2)/(1 + t^2) - 
              (a*(-1 + t^2))/(1 + t^2))*
            (-p1 + (a*(-1 + t^2))/(1 + t^2)) + 
           ((-2*a*t*(-1 + t^2))/(1 + t^2)^2 + 
              (2*a*t)/(1 + t^2))*
            (-p2 + (a*t*(-1 + t^2))/(1 + t^2))))/
       (((-2*a*t*(-1 + t^2))/(1 + t^2)^2 + 
            (2*a*t)/(1 + t^2))^2 + 
         ((-2*a*t^2*(-1 + t^2))/(1 + t^2)^2 + 
            (2*a*t^2)/(1 + t^2) + 
            (a*(-1 + t^2))/(1 + t^2))^2)}

 pedal[{p1_,p2_},talbot[a_,b_,f_]][t_]:=
    {(2*b^2*p1 + a*(4*a^2 - f^2)*Cos[t] - 2*b^2*p1*Cos[2*t] +
      a*f^2*Cos[3*t] - 2*a*b*p2*Sin[2*t])/
     (2*(a^2 + b^2 + (a^2 - b^2)*Cos[2*t])), 
     (2*a^2*p2 + 2*a^2*p2*Cos[2*t] + 
     b*(4*a^2 - 3*f^2)*Sin[t] - 2*a*b*p1*Sin[2*t] + 
     b*f^2*Sin[3*t])/
     (2*(a^2 + b^2 + (a^2 - b^2)*Cos[2*t]))}

 pedal[{p1_,p2_},tanhspiral[a_]][t_]:=
    {p1 + (((-2*a*Cos[2*a*t])/(Cos[2*a*t] + Cosh[2*t]) + 
           (Sin[2*a*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
            (Cos[2*a*t] + Cosh[2*t])^2)*
         ((-p1 + Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t]))*
            ((-2*a*Cos[2*a*t])/
               (Cos[2*a*t] + Cosh[2*t]) + 
              (Sin[2*a*t]*
                 (-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
               (Cos[2*a*t] + Cosh[2*t])^2) + 
           (-p2 + Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t]))*
            ((2*Cosh[2*t])/(Cos[2*a*t] + Cosh[2*t]) - 
              (Sinh[2*t]*
                 (-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
               (Cos[2*a*t] + Cosh[2*t])^2)))/
       (((2*a*Cos[2*a*t])/(Cos[2*a*t] + Cosh[2*t]) - 
            (Sin[2*a*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
             (Cos[2*a*t] + Cosh[2*t])^2)^2 + 
         ((2*Cosh[2*t])/(Cos[2*a*t] + Cosh[2*t]) - 
            (Sinh[2*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
             (Cos[2*a*t] + Cosh[2*t])^2)^2),
     p2 + (((2*Cosh[2*t])/(Cos[2*a*t] + Cosh[2*t]) - 
           (Sinh[2*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
            (Cos[2*a*t] + Cosh[2*t])^2)*
         ((-p1 + Sinh[2*t]/(Cos[2*a*t] + Cosh[2*t]))*
            ((-2*a*Cos[2*a*t])/
               (Cos[2*a*t] + Cosh[2*t]) + 
              (Sin[2*a*t]*
                 (-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
               (Cos[2*a*t] + Cosh[2*t])^2) + 
           (-p2 + Sin[2*a*t]/(Cos[2*a*t] + Cosh[2*t]))*
            ((2*Cosh[2*t])/(Cos[2*a*t] + Cosh[2*t]) - 
              (Sinh[2*t]*
                 (-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
               (Cos[2*a*t] + Cosh[2*t])^2)))/
       (((2*a*Cos[2*a*t])/(Cos[2*a*t] + Cosh[2*t]) - 
            (Sin[2*a*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
             (Cos[2*a*t] + Cosh[2*t])^2)^2 + 
         ((2*Cosh[2*t])/(Cos[2*a*t] + Cosh[2*t]) - 
            (Sinh[2*t]*(-2*a*Sin[2*a*t] + 2*Sinh[2*t]))/
             (Cos[2*a*t] + Cosh[2*t])^2)^2)}

 pedal[{p1_,p2_},tractrix[a_]][t_]:=
    {p1 - (a*((Csc[t/2]*Sec[t/2])/2 - Sin[t])*
         (a*Cos[t]*(-p2 + a*(Cos[t] + Log[Tan[t/2]])) - 
           a*((Csc[t/2]*Sec[t/2])/2 - Sin[t])*
            (-p1 + a*Sin[t])))/
       (a^2*Cos[t]^2 + 
         a^2*((Csc[t/2]*Sec[t/2])/2 - Sin[t])^2),
     p2 + (a*Cos[t]*(a*Cos[t]*
            (-p2 + a*(Cos[t] + Log[Tan[t/2]])) - 
           a*((Csc[t/2]*Sec[t/2])/2 - Sin[t])*
            (-p1 + a*Sin[t])))/
       (a^2*Cos[t]^2 + 
         a^2*((Csc[t/2]*Sec[t/2])/2 - Sin[t])^2)}

 pedal[{p1_,p2_},triangle][t_]:=
    {p1 + ({-p1 + If[t < 1,t*{1,0},
             If[t < 2,
              {1,0} + (t - 1)*{-2^(-1),Sqrt[3]/2},
              {1/2,Sqrt[3]/2} + 
               (t - 2)*{-2^(-1),-(Sqrt[3]/2)}]],
           -p2 + If[t < 1,t*{1,0},
             If[t < 2,
              {1,0} + (t - 1)*{-2^(-1),Sqrt[3]/2},
              {1/2,Sqrt[3]/2} + 
               (t - 2)*{-2^(-1),-(Sqrt[3]/2)}]]} . 
          J[If[t < 1,t*{0,0} + 1*{1,0},
            If[t < 2,{0,0} + 
              ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
             {0,0} + ((t - 2)*{0,0} + 
                1*{-2^(-1),-(Sqrt[3]/2)})]]]*
         J[If[t < 1,t*{0,0} + 1*{1,0},
           If[t < 2,{0,0} + 
             ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
            {0,0} + ((t - 2)*{0,0} + 
               1*{-2^(-1),-(Sqrt[3]/2)})]]])/
       If[t < 1,t*{0,0} + 1*{1,0},
         If[t < 2,{0,0} + 
           ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
          {0,0} + ((t - 2)*{0,0} + 
             1*{-2^(-1),-(Sqrt[3]/2)})]] . 
        If[t < 1,t*{0,0} + 1*{1,0},
         If[t < 2,{0,0} + 
           ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
          {0,0} + ((t - 2)*{0,0} + 
             1*{-2^(-1),-(Sqrt[3]/2)})]],
     p2 + ({-p1 + If[t < 1,t*{1,0},
             If[t < 2,
              {1,0} + (t - 1)*{-2^(-1),Sqrt[3]/2},
              {1/2,Sqrt[3]/2} + 
               (t - 2)*{-2^(-1),-(Sqrt[3]/2)}]],
           -p2 + If[t < 1,t*{1,0},
             If[t < 2,
              {1,0} + (t - 1)*{-2^(-1),Sqrt[3]/2},
              {1/2,Sqrt[3]/2} + 
               (t - 2)*{-2^(-1),-(Sqrt[3]/2)}]]} . 
          J[If[t < 1,t*{0,0} + 1*{1,0},
            If[t < 2,{0,0} + 
              ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
             {0,0} + ((t - 2)*{0,0} + 
                1*{-2^(-1),-(Sqrt[3]/2)})]]]*
         J[If[t < 1,t*{0,0} + 1*{1,0},
           If[t < 2,{0,0} + 
             ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
            {0,0} + ((t - 2)*{0,0} + 
               1*{-2^(-1),-(Sqrt[3]/2)})]]])/
       If[t < 1,t*{0,0} + 1*{1,0},
         If[t < 2,{0,0} + 
           ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
          {0,0} + ((t - 2)*{0,0} + 
             1*{-2^(-1),-(Sqrt[3]/2)})]] . 
        If[t < 1,t*{0,0} + 1*{1,0},
         If[t < 2,{0,0} + 
           ((t - 1)*{0,0} + 1*{-2^(-1),Sqrt[3]/2}),
          {0,0} + ((t - 2)*{0,0} + 
             1*{-2^(-1),-(Sqrt[3]/2)})]]}

 pedal[{p1_,p2_},trisectrix[a_]][t_]:=
    {p1 + ((-(a*(1 - 4*Cos[t]^2)*Sec[t]^2) - 
           8*a*Sin[t]^2)*
         ((-p1 + a*(1 - 4*Cos[t]^2))*
            (-(a*(1 - 4*Cos[t]^2)*Sec[t]^2) - 
              8*a*Sin[t]^2) + 
           8*a*Cos[t]*Sin[t]*
            (-p2 + a*(1 - 4*Cos[t]^2)*Tan[t])))/
       (64*a^2*Cos[t]^2*Sin[t]^2 + 
         (a*(1 - 4*Cos[t]^2)*Sec[t]^2 + 8*a*Sin[t]^2)^2),
       p2 + (8*a*Cos[t]*Sin[t]*
         ((-p1 + a*(1 - 4*Cos[t]^2))*
            (-(a*(1 - 4*Cos[t]^2)*Sec[t]^2) - 
              8*a*Sin[t]^2) + 
           8*a*Cos[t]*Sin[t]*
            (-p2 + a*(1 - 4*Cos[t]^2)*Tan[t])))/
       (64*a^2*Cos[t]^2*Sin[t]^2 + 
         (a*(1 - 4*Cos[t]^2)*Sec[t]^2 + 8*a*Sin[t]^2)^2)}
   
    pedal[{p1_,p2_},zcprofile[pm_,a_,c_]][t_]:=
    {p1 + ((c^2/(a*(1 - c^2/(a^2*t^2))^(1/2)*t^2) - 
           (a^2*pm*t)/(-c^2 + a^2*t^2)^(1/2))*
         (-p2 + pm*(-c^2 + a^2*t^2)^(1/2) + 
           (-p1 + t)*(c^2/
               (a*(1 - c^2/(a^2*t^2))^(1/2)*t^2) - 
              (a^2*pm*t)/(-c^2 + a^2*t^2)^(1/2)) + 
           c*ArcSin[c/(a*t)]))/
       (1 + (-(c^2/(a*(1 - c^2/(a^2*t^2))^(1/2)*t^2)) + 
            (a^2*pm*t)/(-c^2 + a^2*t^2)^(1/2))^2),
     p2 + (-p2 + pm*(-c^2 + a^2*t^2)^(1/2) + 
         (-p1 + t)*(c^2/
             (a*(1 - c^2/(a^2*t^2))^(1/2)*t^2) - 
            (a^2*pm*t)/(-c^2 + a^2*t^2)^(1/2)) + 
         c*ArcSin[c/(a*t)])/
       (1 + (-(c^2/(a*(1 - c^2/(a^2*t^2))^(1/2)*t^2)) + 
            (a^2*pm*t)/(-c^2 + a^2*t^2)^(1/2))^2)}

End[]

EndPackage[]
