(* ::Package:: *)

(*:Sat May 23 21:47:33 MET DST 1998
*)

(*:Title: PLTPROGS.m *)

(*:Author: Alfred Gray *)

(* Copyright 1994-1998 by Alfred Gray *)

(* Adapted to 

Mathematica Versions: 9.0, 

 July 10,2013, by Rolf Sulanke. *)

(*:Package Version: 2.0 *)

(* This package has been adapted to Mathematica v. 9 by Rolf Sulanke, June 16,2013. 
Many functions of the original package are obsolete now and commented out, e.g. those related to Acrospin 
or Geomview..  The new symbols introduced in the package are protected. R.S.    *)


(*:Summary: 
This package consists of several plotting enhancements useful for plotting curves and 
surfaces.
*)

(*:Keywords:
borderedpolygon, centerlines, checkerboard, dip, elrot, filledcurve, graphics2dtosurfrev, 
lathe, linetolines, nomesh, orth4d, paramcurveplot1, paramcurveplot2, plotpatch2d, polygonholes, 
primitiveplot3d, selectcolgraphics3d, selectgraphics3d, ShowInNotebook, surfacecurveplot, stereo4d, 
wireframe, WriteAcroLines, WriteAcroPoints, Write2COLMESH, Write3DCMESH, Write3DCNMESH, Write3DMESH
*)

BeginPackage["PLTPROGS`"]

     pltprogs:=Print[Select[Names["PLTPROGS`*"],#!="pltprogs"&]]

(* Miscellaneous Plotting Programs *)

     barycenter::usage="barycenter[Polygon[{pt1,...,ptn}]] subdivides a polygon using its barycenter."

     borderedpolygon::usage="borderedpolygon[intsty,borsty,versty,pts] creates from the list pts a polygonal region whose vertices are the elements of pts. The directives constituting intsty are applied to the interior, the directives constituting borsty are applied to the boundary, and the directives constituting versty are applied to the vertices."

     centerlines::usage="centerlines[graphic] adds lines connecting the center of each quadrilateral to its vertices."

   (* checkerboard::usage=" checkerboard[graphic] gives a checkerboard pattern to a graphic." *)

    dip::usage="DisplayFunction->dip[ins] inserts a Graphics3D primitive at the beginning of a
 Graphics3D list when the list is displayed. For example, DisplayFunction->dip[EdgeForm[]] displays polygons 
with no edges."

    elrot::usage="elrot[p,q][theta][aa_List] is the elementary rotation by an angle theta of pth and qth elements of the list aa."

    filledcurve::usage="filledcurve parametrically plots a plane curve and fills it in. It has the same syntax as ParametricPlot.  filledcurve[graphic] fills in a graphic."

    graphics2dtosurfrev::usage="graphics2dtosurfrev[r,n][list] constructs a surface of revolution of radius r from a Graphics list of lines. Each circle of latitude is approximated by a regular polygon with n vertices."

    lathe::usage="lathe[dots] creates a surface of revolution with no edges from a list of points in the plane."

    linetolines::usage="linetolines[Line[{pt1,...,ptn}]] converts a broken line segment Line[{pt1,...,ptn}] into a list of line segments connecting consecutive pairs of the points pt1,...,ptn."

    (* nomesh::usage="nomesh[graphic] converts a graphic to a graphic with no edges."  *)

    orth4d::usage="orth4d[surf,{u,umin,umax},{v,vmin,vmax},opts] uses GraphicsArray to plot a surface in R^4 via the four orthogonal projections from R^4 to R^3."

    paramcurveplot1::usage="paramcurveplot1 has the same syntax as ParametricPlot3D, except that a PlotPoints specification is obligatory. It plots the parameter lines u->x[u,v0] of a parametric patch surface:U->R^3."

    paramcurveplot2::usage="paramcurveplot2 has the same syntax as
ParametricPlot3D, except that a PlotPoints specification is obligatory. It plots the parameter lines v->x[u0,v] of a parametric patch surface:U->R^3."

    plotpatch2d::usage="plotpatch2d has the same syntax as ParametricPlot. It plots the images of horizontal 
and vertical lines under the function f:U->R^2. plotpatch2d has the same functionality as Maeder's CartesianMap and PolarMap, but draws curved instead of straight lines."

    polygonholes::usage= (*"polygonholes[fac][...] has the same syntax as ParametricPlot3D[...]. It plots a 
parametric patch in R^3 with holes in each polygon, where fac is the ratio of the size of each hole to that of the 
original polygon containing it. "*) "polygonholes[fac][graphic]  puts a hole in each polygon, where fac is 
the ratio of the size of each hole to that of the original polygon containing it."

(*    selectgraphics3d::usage="selectgraphics3d[graphics3dobj,bound,opts] selects from a list of polygons those 
whose vertices have their coordinates bounded by bound, and then displays the results." 

    selectcolgraphics3d::usage="selectcolgraphics3d[graphics3dobj,bound,opts] selects from a list of colored 
polygons those whose vertices have their coordinates bounded by bound, and then displays the results." *)

    stereo4d::usage="stereo4d[{a,b,c,d},surf,{u,umin,umax},{v,vmin,vmax},opts] uses GraphicsArray to plot a surface in R^4 via the four stereographic projections from R^4 to R^3 with respect to the point {a,b,c,d} in R^4."

(*    ShowInNotebook::usage="ShowInNotebook[graphics,nbopts] displays a graphic in a separate notebook." 

    surfacecurveplot::usage="surfacecurveplot[surface,{u,u0,u1},{v,v0,v1}, PlotLines->{ulines,vlines},
PlotPoints->{upoints,vpoints},opts] plots both the u- and v-parameter curves of a patch x:U->R^3. 
The number of curves is specified with PlotLines, and the resolution of the lines is specified with PlotPoints." 
Both functions don't work under Mathematica v. 9.*)

    wireframe::usage="wireframe[...] has the same syntax as ParametricPlot3D[...]. It plots a wireframe of a 
parametric patch in R^3.  wirframe[graphic] converts a solid graphic into a wireframe graphic."

(* Mathematica to Acrospin Programs 

primitiveplot3d::usage="primitiveplot3d[{fx,fy,fz},{t,tmin,tmax}]
produces a list of points of a space curve parametrized by a variable
t which runs from tmin to tmax. primitiveplot3d[{fx,fy,fz},{t,tmin,tmax},{u,umin,umax}] produces a list of points by polygons of a surface parametrized by
t and u."

WriteAcroLines::usage="WriteAcroLines[{color},file,{fx,fy,fz},
{t,tmin,tmax}] outputs to a file in acrospin line format the
parametric plot of a space curve. WriteAcroLines[{color1,color2},file,{fx,fy,fz},{t,tmin,tmax},{u,umin,umax}] outputs to a file in acrospin line format the parametric plot of a surface."

WriteAcroPoints::usage="WriteAcroPoints[{color},file,{fx,fy,fz},
{t,tmin,tmax}] outputs to a file in acrospin point format the
parametric plot of a space curve.
WriteAcroPoints[{color},file,{fx,fy,fz},{t,tmin,tmax},{u,umin,umax}]
outputs to a file in acrospin point format the parametric plot of a surface."

(* Mathematica to Geomview Programs *)

Write3DMESH::usage="Write3DMESH[file,expression,{u,umin,umax},{v,vmin,vmax},
PlotPoints->{udiv,vdiv},opts] outputs to a file in mesh format the
parametric plot of a surface."

Write3DCMESH::usage="Write3DCMESH[file,{expression,color},
{u,umin,umax},{v,vmin,vmax},PlotPoints->{udiv,vdiv},opts] outputs to a file
in mesh format the parametric plot of expression colored by color."

Write3DCNMESH::usage="Write3DCNMESH[file,{expression,unitn,color},
{u,umin,umax},{v,vmin,vmax},
PlotPoints->{udiv,vdiv},opts] outputs to a file in mesh format the
parametric plot of expression colored by color with normal information unitn."

Write2COLMESH::usage="Write2COLMESH[{color1,color2},file,expression,
{u,umin,umax},{v,vmin,vmax},
PlotPoints->{udiv,vdiv},opts] outputs to a file in mesh format the parametric plot of a surface with one side color1 and the other color2."
*)
Begin["`Private`"]

(* Miscellaneous Plotting Programs *)

  barycenter[Polygon[pts_]]:=
        Module[{ptstmp},
        ptstm=Append[pts,First[pts]];
	Table[Polygon[{ptstm[[i]],ptstm[[i + 1]],
        Plus@@pts/Length[ptstm]}],{i,Length[ptstm] - 1}]]

  borderedpolygon[intsty_,borsty_,versty_,pts_]:=
        {Flatten[{intsty,Polygon[pts]}],
         Flatten[{borsty,Line[Append[pts,First[pts]]]}],
         Flatten[{versty,Point/@pts}]}

(* centerlines[surface_,{u_,u0_,u1_},{v_,v0_,v1_},opts___]:=
        Module[{plottmp,grtmp},
                plottmp = ParametricPlot3D[
                surface//Evaluate,{u,u0,u1},{v,v0,v1},
                DisplayFunction->Identity,opts];
                grtmp = plottmp /.
     (Polygon[pts_]:>
     {Polygon[Append[pts[[{1,2}]],Plus@@pts/4]],
      Polygon[Append[pts[[{2,3}]],Plus@@pts/4]],
      Polygon[Append[pts[[{3,4}]],Plus@@pts/4]],
      Polygon[Append[pts[[{4,1}]],Plus@@pts/4]]});
        Show[grtmp,DisplayFunction->$DisplayFunction]]
*)

    centerlines[graphic_]:=Show[graphic /. (Polygon[pts_]:>
     {Polygon[Append[pts[[{1,2}]],Plus@@pts/4]],
      Polygon[Append[pts[[{2,3}]],Plus@@pts/4]],
      Polygon[Append[pts[[{3,4}]],Plus@@pts/4]],
      Polygon[Append[pts[[{4,1}]],Plus@@pts/4]]})]

 (*   checkerboard[surface_,{u_,u0_,u1_},{v_,v0_,v1_},opts___]:=
        Module[{plottmp,grtmp},
                plottmp = ParametricPlot3D[
                surface,{u,u0,u1},{v,v0,v1},
                DisplayFunction->Identity,opts];
                grtmp = plottmp /.(Polygon[pts_]:> 
                Polygon[(pts + RotateRight[pts])/2]);
        Show[grtmp,DisplayFunction->$DisplayFunction]] 

    checkerboard[graphic_]:=Show[graphic /. (Polygon[pts_]:>
        Polygon[(pts + RotateRight[pts])/2])] *)

    dip[ins_][g_]:= $DisplayFunction[Insert[g,ins,{1,1}]]

    elrot[p_,q_][theta_][aa_List]:=ReplacePart[ReplacePart[
        aa,aa[[p]]*Cos[theta] + aa[[q]]*Sin[theta],p],
        -aa[[p]]*Sin[theta] + aa[[q]]*Cos[theta],q]

    filledcurve[curve_,{u_,u0_,u1_},opts___]:=
        Module[{plottmp,grtmp},
            plottmp = ParametricPlot[
            curve//Evaluate,{u,u0,u1},AspectRatio->Automatic,
            DisplayFunction->Identity,opts];
            grtmp = plottmp /. (Line[pts_]:> Polygon[pts]);
        Show[grtmp,DisplayFunction->$DisplayFunction]]
    
    filledcurve[graphic_]:=Show[graphic /. (Line[pts_]:> Polygon[pts])]

    graphics2dtosurfrev[r_,n_][Line[{{a_,b_},{c_,d_}}]]:=
        Table[Polygon[{
        {r*a*Cos[2*Pi*(k - 1)/n],r*a*Sin[2*Pi*(k - 1)/n],b},
        {r*a*Cos[2*Pi*k/n],r*a*Sin[2*Pi*k/n],b},
        {r*c*Cos[2*Pi*k/n],r*c*Sin[2*Pi*k/n],d},
        {r*c*Cos[2*Pi*(k - 1)/n],
                r*c*Sin[2*Pi*(k - 1)/n],d}}],{k,n}]

   lathe[dots_]:=Show[Graphics3D[
      Map[graphics2dtosurfrev[0.5,60],
      linetolines[Line[dots]]]],Boxed->False,
      DisplayFunction->dip[EdgeForm[]]]

    linetolines[Line[pts_]]:= Table[Line[{pts[[i]],
             pts[[i + 1]]}],{i,Length[pts] - 1}]

 (*  nomesh[graphics_]:=
      Show[graphics,DisplayFunction->dip[EdgeForm[]]] *)

 orth4d[surf_,{u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
        Module[{g1,g2,g3,g4},
        g1=ParametricPlot3D[Evaluate[
                Drop[surf,{1}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g2=ParametricPlot3D[Evaluate[
                Drop[surf,{2}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g3=ParametricPlot3D[Evaluate[
                Drop[surf,{3}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g4=ParametricPlot3D[Evaluate[
                Drop[surf,{4}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        Show[GraphicsGrid[{{g1,g2},{g3,g4}},
                Spacings->{0.2,0}],
                DisplayFunction->$DisplayFunction]]

(* Obsolete version, Graphics Array replaced by GraphicsGrid. GraphicsSpacing 
is now the Option Spacings of GraphicsGrid.

    orth4d[surf_,{u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
        Module[{g1,g2,g3,g4},
        g1=ParametricPlot3D[Evaluate[
                Drop[surf,{1}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g2=ParametricPlot3D[Evaluate[
                Drop[surf,{2}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g3=ParametricPlot3D[Evaluate[
                Drop[surf,{3}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g4=ParametricPlot3D[Evaluate[
                Drop[surf,{4}]],{u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        Show[GraphicsArray[{{g1,g2},{g3,g4}}],
                GraphicsSpacing->{0.2,0},
                DisplayFunction->$DisplayFunction]]
*)

    paramcurveplot1[surface_,{u_,u0_,u1_},{v_,v0_,v1_},
        PlotPoints->{upoints_,vpoints_},opts___]:=
        ParametricPlot3D[Table[
        surface,{v,v0,v1,(v1-v0)/(vpoints - 1)}]//Evaluate,
        {u,u0,u1},PlotPoints->upoints,opts]

    paramcurveplot2[surface_,{u_,u0_,u1_},{v_,v0_,v1_},
        PlotPoints->{upoints_,vpoints_},opts___]:=
        ParametricPlot3D[Table[
        surface,{u,u0,u1,(u1-u0)/(upoints - 1)}]//Evaluate,
        {v,v0,v1},PlotPoints->vpoints,opts]

    plotpatch2d[expr_,{x_,xmin_,xmax_},{y_,ymin_,ymax_},
        PlotPoints->{xpoints_,ypoints_},opts___]:=
        Show[ParametricPlot[Evaluate[
        Table[ComplexExpand[
        {Re[expr],Im[expr]}],
        {y,ymin,ymax,(ymax - ymin)/(ypoints - 1)}]],
        {x,xmin,xmax},PlotPoints->xpoints,
        DisplayFunction->Identity],
        ParametricPlot[Evaluate[
        Table[ComplexExpand[
        {Re[expr],Im[expr]}],
        {x,xmin,xmax,(xmax - xmin)/(xpoints -1)}]],
        {y,ymin,ymax},PlotPoints->ypoints,
        DisplayFunction->Identity],
        AspectRatio->Automatic,
        DisplayFunction->$DisplayFunction,opts]

   (*  polygonholes[fac_][surface_,{u_,u0_,u1_},{v_,v0_,v1_},opts___]:=
        Module[{plottmp,grtmp},
                plottmp = ParametricPlot3D[
                surface,{u,u0,u1},{v,v0,v1},
                DisplayFunction->Identity,opts];
                grtmp = plottmp /.(Polygon[pts_]:> 
                (cntr=(Plus@@pts)/Length[pts];
                sp=(fac*(# - cntr) + cntr)&/@pts;
                Polygon/@Transpose[{pts,RotateLeft[pts],
                RotateLeft[sp],sp}]));
        Show[grtmp,DisplayFunction->$DisplayFunction]]
*)

    polygonholes[fac_][graphic_]:=Show[graphic /. (Polygon[pts_]:>
        (cntr=(Plus@@pts)/Length[pts];
                sp=(fac*(# - cntr) + cntr)&/@pts;
                Polygon/@Transpose[{pts,RotateLeft[pts],
                RotateLeft[sp],sp}]))]

(*    selectgraphics3d[graphics3dobj_,bound_,opts___]:=
        Show[Graphics3D[Select[graphics3dobj,
        (Abs[#[[1,1,1]]]<bound && Abs[#[[1,1,2]]]<bound &&
         Abs[#[[1,1,3]]]<bound && Abs[#[[1,2,1]]]<bound &&
         Abs[#[[1,2,2]]]<bound && Abs[#[[1,2,3]]]<bound &&
         Abs[#[[1,3,1]]]<bound && Abs[#[[1,3,2]]]<bound &&
         Abs[#[[1,3,3]]]<bound && Abs[#[[1,4,1]]]<bound &&
         Abs[#[[1,4,2]]]<bound && Abs[#[[1,4,2]]]<bound
         )&]],opts]

    selectcolgraphics3d[graphics3dobj_,bound_,opts___]:=
        Show[Graphics3D[Select[graphics3dobj,
        (Abs[#[[2,1,1,1]]]<bound && Abs[#[[2,1,1,2]]]<bound &&
        Abs[#[[2,1,1,3]]]<bound && Abs[#[[2,1,2,1]]]<bound &&
        Abs[#[[2,1,2,2]]]<bound && Abs[#[[2,1,2,3]]]<bound &&
        Abs[#[[2,1,3,1]]]<bound && Abs[#[[2,1,3,2]]]<bound &&
        Abs[#[[2,1,3,3]]]<bound && Abs[#[[2,1,4,1]]]<bound &&
        Abs[#[[2,1,4,2]]]<bound && Abs[#[[2,1,4,2]]]<bound
         )&]],opts];
Commented out since both functions yield error messages in Mathematica v,9, *)
(* Here I changed GraphicsArea and GapicsSpacing as indicated for orth4d. *)
    stereo4d[{a_,b_,c_,d_},surf_,
        {u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
        Module[{g1,g2,g3,g4},
        g1=ParametricPlot3D[Evaluate[
                Drop[surf,{1}]/(a-surf[[1]])],
                {u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g2=ParametricPlot3D[Evaluate[
                Drop[surf,{2}]/(b-surf[[2]])],
                {u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g3=ParametricPlot3D[Evaluate[
                Drop[surf,{3}]/(c-surf[[3]])],
                {u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        g4=ParametricPlot3D[Evaluate[
                Drop[surf,{4}]/(d-surf[[4]])],
                {u,umin,umax},{v,vmin,vmax},
                DisplayFunction->Identity,opts];
        Show[GraphicsGrid[{{g1,g2},{g3,g4}},
               Spacings->{0.2,0}],
                DisplayFunction->$DisplayFunction]]

 (*   ShowInNotebook[gr_,nbopts___]:=
    Module[{nb},
        nb = NotebookPut[Notebook[{
            Cell[GraphicsData["PostScript",
                              DisplayString[gr]],
                 "Graphics",
                 ImageSize->{350,350},
                 ImageMargins -> {{0,0},{0,0}},
                 CellMargins -> {{0,0},{0,0}}]},
            nbopts,WindowSize->Fit,Visible -> False,
            ShowCellBracket -> False,WindowElements->{},
            WindowFrameElements->"CloseBox"]];
        SetOptions[nb,WindowSize->Fit];
        SetOptions[nb,Visible->True];
        nb];

    surfacecurveplot[surface_,{u_,u0_,u1_},{v_,v0_,v1_},
        PlotLines->{ulines_,vlines_},
        PlotPoints->{upoints_,vpoints_},opts___]:=
            Show[paramcurveplot1[surface,{u,u0,u1},{v,v0,v1},
                PlotPoints->{upoints,ulines},
                DisplayFunction->Identity,opts],
                paramcurveplot2[surface,{u,u0,u1},{v,v0,v1},
                PlotPoints->{vlines,vpoints},
                DisplayFunction->Identity,opts],
                DisplayFunction->$DisplayFunction];
commented out since they don't work in v. 0 of Mathematica. *)

    wireframe[surface_,{u_,u0_,u1_},{v_,v0_,v1_},opts___]:=
        Module[{plottmp,grtmp},
                plottmp = ParametricPlot3D[
                surface,{u,u0,u1},{v,v0,v1},
                DisplayFunction->Identity,opts];
                grtmp = plottmp /.(Polygon[pts_]:> 
                Line[Append[pts,First[pts]]]);
        Show[grtmp,DisplayFunction->$DisplayFunction]]

    wireframe[graphic_]:=Show[graphic /. (Polygon[pts_]:> 
                Line[Append[pts,First[pts]]])]

(* Acrospin is obsolete now. *)
(* Mathematica to Acrospin Programs 

blue             = 1
green            = 2
cyan             = 3
red              = 4
magenta          = 5
brown            = 6
white            = 7
darkgray         = 8
lightblue        = 9
lightgreen       = 10
lightcyan        = 11
lightred         = 12
lightmagenta     = 13
yellow           = 14
intensifiedwhite = 15

primitiveplot3d[exp_,{t_,tmin_,tmax_},opts___]:=
First[ParametricPlot3D[exp,
{t,tmin,tmax},opts,
DisplayFunction->Identity]] /.
(Line[pts_] :> pts)

primitiveplot3d[exp_,{u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
First[ParametricPlot3D[exp,
{u,umin,umax},{v,vmin,vmax},opts,
DisplayFunction->Identity]] /.
(Polygon[pts_] :> pts)

WriteAcroLines[{color_},file_String,curve_,
{t_,tmin_,tmax_},opts___]:=
Module[{tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8},
OpenWrite[file];
WriteString[file,"Set Color"];
WriteString[file,color];
WriteString[file,"\n"];
WriteString[file,"EndPointList name X Y Z\n"];
tmp1=primitiveplot3d[curve//Evaluate,
{t,tmin,tmax},opts];
tmp2=Flatten[tmp1,1];
tmp3=Chop[tmp2,10^-3];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
tmp5=Table[
{StringForm["VERTEX``",j],tmp4[[j]]},
{j,Length[tmp3]}];
tmp6=Map[Flatten[#,1]&,tmp5,{1}];
Scan[
(WriteString[file,First[#]];
Scan[WriteString[file,"   ",#]&,Rest[#]];
WriteString[file,"\n"])&,
tmp6];
WriteString[file,"\nLineList from to\n"];
tmp7=Table[
StringForm["VERTEX`` VERTEX``\n",j,j+1],
{j,Length[tmp3]-1}];
Scan[WriteString[file,#,"\n"]&,tmp7];
Close[file]]

WriteAcroLines[{color1_:12,color2_:14},
file_String,surface_,
{u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
Module[{tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,
tmp7,tmp8,tmp9,tmp10,tmp11,tmp12},
OpenWrite[file];
WriteString[file,"EndPointList name X Y Z\n\n"];
tmp1=primitiveplot3d[surface//Evaluate,
{u,umin,umax},{v,vmin,vmax},opts];
tmp2=Chop[tmp1,10^-3];
tmp3=Map[PaddedForm[#,{4,4}]&,tmp2,{3}];
tmp4=Table[
{StringForm["POLY``VERTEX``",j,k],tmp3[[j,k]]},
{j,Length[tmp3]},{k,4}];
tmp5=Map[Flatten[#,1]&,tmp4,{2}];
tmp6=Flatten[tmp5,1];
Scan[WriteString[file,"   ",#]&,Rest[#]];
Scan[
(WriteString[file,First[#]];
Scan[WriteString[file,"   ",#]&,Rest[#]];
WriteString[file,"\n"])&,
tmp6];
WriteString[file,"\nLineList from to Color Layer\n\n"];
tmp7=Table[
StringForm["POLY``VERTEX`` POLY``VERTEX`` `` ``\n",
j,1,j,2,color1,1],{j,Length[tmp3]}];
tmp8=Table[
StringForm["POLY``VERTEX`` POLY``VERTEX`` `` ``\n",
j,4,j,3,color1,1],{j,Length[tmp3]}];
tmp9=Join[tmp7,tmp8];
Scan[WriteString[file,#,"\n"]&,tmp9];
tmp10=Table[
StringForm["POLY``VERTEX`` POLY``VERTEX`` `` ``\n",
j,1,j,4,color2,2],{j,Length[tmp3]}];
tmp11=Table[
StringForm["POLY``VERTEX`` POLY``VERTEX`` `` ``\n",
j,3,j,2,color2,2],{j,Length[tmp3]}];
tmp12=Join[tmp10,tmp11];
Scan[WriteString[file,#,"\n"]&,tmp12];
Close[file]]

WriteAcroPoints[{color_},file_String,curve_,
{t_,tmin_,tmax_},opts___]:=
Module[{tmp1,tmp2,tmp3,tmp4},
OpenWrite[file];
WriteString[file,"Set Color"];
WriteString[file,color];
WriteString[file,"\n"];
WriteString[file,"PointList X Y Z\n"];
tmp1=primitiveplot3d[curve//Evaluate,
{t,tmin,tmax},opts];
tmp2=Union[Flatten[tmp1,1]];
tmp3=Chop[tmp2,10^-3];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
Scan[
(WriteString[file,First[#]];
Scan[WriteString[file,"   ",#]&,Rest[#]];
WriteString[file,"\n"])&,
tmp4];
Close[file]]

WriteAcroPoints[{color_},file_String,surface_,
{u_,umin_,umax_},{v_,vmin_,vmax_},opts___]:=
Module[{tmp1,tmp2,tmp3,tmp4},
OpenWrite[file];
WriteString[file,"Set Color"];
WriteString[file,color];
WriteString[file,"\n"];
WriteString[file,"PointList X Y Z\n"];
tmp1=primitiveplot3d[surface//Evaluate,
{u,umin,umax},{v,vmin,vmax},opts];
tmp2=Union[Flatten[tmp1,1]];
tmp3=Chop[tmp2,10^-3];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
Scan[
(WriteString[file,First[#]];
Scan[WriteString[file,"   ",#]&,Rest[#]];
WriteString[file,"\n"])&,
tmp4];
Close[file]]
*)

(* Mathematica to Geomview Programs;  
Commented out; usefull for Users of Geomview. 

Write3DMESH[file_String,expression_,
{u_,umin_,umax_},{v_,vmin_,vmax_},
PlotPoints->{udiv_,vdiv_},otmp___]:=
Module[{tmp1,tmp2,tmp3,tmp4},
OpenWrite[file];
WriteString[file,"MESH\n"];
WriteString[file,ToString[vdiv+1]];
WriteString[file," "];
WriteString[file,ToString[udiv+1]];
WriteString[file,"\n"];
tmp1=N[Table[expression,
    {u,umin,umax,(umax-umin)/udiv},
    {v,vmin,vmax,(vmax-vmin)/vdiv}]];
tmp2=Flatten[tmp1,1];
tmp3=Chop[tmp2,10^-4];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
Scan[
   (WriteString[file,First[#]];
    Scan[WriteString[file,"   ",#]&,Rest[#]];
    WriteString[file,"\n"])&,tmp4];
Close[file]]

Write3DCMESH[file_String,{expression_,color_},
{u_,umin_,umax_},{v_,vmin_,vmax_},
PlotPoints->{udiv_,vdiv_},otmp___]:=
Module[{tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7},
OpenWrite[file];
WriteString[file,"CMESH\n"];
WriteString[file,ToString[vdiv+1]];
WriteString[file," "];
WriteString[file,ToString[udiv+1]];
WriteString[file,"\n"];
tmp1=N[Table[{expression,HSBColor[color,1,1]},
    {u,umin,umax,(umax-umin)/udiv},
    {v,vmin,vmax,(vmax-vmin)/vdiv}]];
tmp2=Apply[List,tmp1,{3}];
tmp3=Map[Flatten[#]&,tmp2,{2}];
tmp4=Flatten[tmp3,1];
tmp5=Map[Append[#,1]&,tmp4];
tmp6=Chop[tmp5,10^-4];
tmp7=Map[PaddedForm[#,{4,4}]&,tmp6,{2}];
Scan[
   (WriteString[file,First[#]];
    Scan[WriteString[file,"   ",#]&,Rest[#]];
    WriteString[file,"\n"])&,tmp7];
Close[file]]



Write3DCNMESH[file_String,{expression_,unitn_,color_},
{u_,umin_,umax_},{v_,vmin_,vmax_},
PlotPoints->{udiv_,vdiv_},otmp___]:=
Module[{tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7},
OpenWrite[file];
WriteString[file,"CNMESH\n"];
WriteString[file,ToString[vdiv+1]];
WriteString[file," "];
WriteString[file,ToString[udiv+1]];
WriteString[file,"\n"];
tmp1=N[Table[{expression,unitn,HSBColor[color,1,1]},
    {u,umin,umax,(umax-umin)/udiv},
    {v,vmin,vmax,(vmax-vmin)/vdiv}]];
tmp2=Apply[List,tmp1,{3}];
tmp3=Map[Flatten[#]&,tmp2,{2}];
tmp4=Flatten[tmp3,1];
tmp5=Map[Append[#,1]&,tmp4];
tmp6=Chop[tmp5,10^-4];
tmp7=Map[PaddedForm[#,{4,4}]&,tmp6,{2}];
Scan[
   (WriteString[file,First[#]];
    Scan[WriteString[file,"   ",#]&,Rest[#]];
    WriteString[file,"\n"])&,tmp7];
Close[file]]

Write2COLMESH[file_String,expression_,
{u_,umin_,umax_},{v_,vmin_,vmax_},
PlotPoints->{udiv_,vdiv_},otmp___]:=
Module[{tmp1,tmp2,tmp3,tmp4},
OpenWrite[file];
WriteString[file,"appearance {\n"];
WriteString[file,"material {\n"];
WriteString[file,"emission 1 1 0\n"];
WriteString[file,"diffuse  1 0 0}}\n"];
WriteString[file,"MESH\n"];
WriteString[file,ToString[vdiv+1]];
WriteString[file," "];
WriteString[file,ToString[udiv+1]];
WriteString[file,"\n"];
tmp1=N[Table[expression,
    {u,umin,umax,(umax-umin)/udiv},
    {v,vmin,vmax,(vmax-vmin)/vdiv}]];
tmp2=Flatten[tmp1,1];
tmp3=Chop[tmp2,10^-4];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
Scan[
   (WriteString[file,First[#]];
    Scan[WriteString[file,"   ",#]&,Rest[#]];
    WriteString[file,"\n"])&,tmp4];
Close[file]]

Write2COLMESH[{color1_,color2_},file_String,expression_,
{u_,umin_,umax_},{v_,vmin_,vmax_},
PlotPoints->{udiv_,vdiv_},otmp___]:=
Module[{cc1,cc2,cc3,dd1,dd2,dd3,df,em,tmp1,tmp2,tmp3,tmp4},
OpenWrite[file];
WriteString[file,"appearance {\n"];
WriteString[file,"material {\n"];
{cc1,cc2,cc3}=List@@color1;
em=StringForm["emission `` `` ``\n",cc1,cc2,cc3];
WriteString[file,em];
{dd1,dd2,dd3}=List@@color2;
df=StringForm["diffuse `` `` ``}}\n",dd1,dd2,dd3];
WriteString[file,df];
WriteString[file,"MESH\n"];
WriteString[file,ToString[vdiv+1]];
WriteString[file," "];
WriteString[file,ToString[udiv+1]];
WriteString[file,"\n"];
tmp1=N[Table[expression,
    {u,umin,umax,(umax-umin)/udiv},
    {v,vmin,vmax,(vmax-vmin)/vdiv}]];
tmp2=Flatten[tmp1,1];
tmp3=Chop[tmp2,10^-4];
tmp4=Map[PaddedForm[#,{4,4}]&,tmp3,{2}];
Scan[
   (WriteString[file,First[#]];
    Scan[WriteString[file,"   ",#]&,Rest[#]];
    WriteString[file,"\n"])&,tmp4];
Close[file]]
*)

End[]

Map[Protect,{"barycenter","borderedpolygon","centerlines","dip","elrot","filledcurve",
"graphics2dtosurfrev","lathe","linetolines","orth4d","paramcurveplot1","paramcurveplot2",
"plotpatch2d","pltprogs","polygonholes","stereo4d","wireframe"}]

EndPackage[]




