(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   2685911,      54042]
NotebookOptionsPosition[   2617301,      52026]
NotebookOutlinePosition[   2621326,      52144]
CellTagsIndexPosition[   2620460,      52119]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Alfred Gray`s Curves and Surfaces\
\>", "Title",
 CellChangeTimes->{{3.5801219892955627`*^9, 3.580122023007222*^9}, {
  3.5801220620384903`*^9, 3.5801220813981247`*^9}, {3.58260746043389*^9, 
  3.582607463129044*^9}, {3.582874185666771*^9, 3.582874217400586*^9}},
 TextAlignment->Center],

Cell[TextData[{
 "adapted to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9.0 by"
}], "Text",
 CellChangeTimes->{{3.5826075583534904`*^9, 3.582607592752458*^9}},
 TextAlignment->Center,
 FontSize->24],

Cell["Rolf Sulanke", "Text",
 CellChangeTimes->{{3.582607601952984*^9, 3.582607607488301*^9}},
 TextAlignment->Center,
 FontSize->24],

Cell["Finished July 16, 2013", "Text",
 CellChangeTimes->{{3.582607684800723*^9, 3.5826076981524863`*^9}, 
   3.5828998793092823`*^9}],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 CellChangeTimes->{{3.5826131003901863`*^9, 3.58261310465343*^9}}],

Cell[TextData[{
 "In the ninetieth, till to his sudden and unexpected death in Bilbao 1998, \
Alfred Gray developed intensively  applications of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to Euclidean differential geometry of curves and surfaces. As a result of \
his work he published in 1994 his trend-setting book [",
 ButtonBox["G1",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray1"],
 "] opening completely new approaches to differential geometry in teaching \
and research. Already in  the eightieth and later Alfred Gray visited our \
Institute of Mathematics at Humboldt University several times. He gave \
reports about his work and introduced us already into the first versions of ",
 StyleBox["Mathematica.",
  FontSlant->"Italic"],
 " He cooperated",
 StyleBox[" ",
  FontSlant->"Italic"],
 "with Hubert Gollek who translated his book into German, see [",
 ButtonBox["G2",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray2"],
 "].  Now the third edition  [",
 ButtonBox["G3",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray3"],
 "] of the book appeared accompanied by a series of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks which correspond to the sections of the book. They can be seen \
and downloaded from the website [",
 ButtonBox["G4",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray4"],
 "] of the publishing house. \n\nIn connection with his book Alfred Gray \
wrote about  20 ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Packages and some notebooks which  he left us generously for use in \
teaching and research  (May 1998). This ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Code is almost identical with the packages contained in the zip-file \
Gray.zip which can be downloaded from the Wolfram Library  Archive [",
 ButtonBox["G5",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray5"],
 "].  I have taken the packages [G5]  as the starting point for adapting \
Gray`s code to the actual ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version 9.0.  Also on the website of Mohammad Ghomi [",
 ButtonBox["MG",
  BaseStyle->"Hyperlink",
  ButtonData->"MG"],
 "] one can find applications of these packages in teaching differential \
geometry.  I cite the first paragraphs of the file readme.txt contained in \
[G5] (here printed in italics}:"
}], "Text",
 CellChangeTimes->{{3.582629461255869*^9, 3.5826294805375032`*^9}, 
   3.5826300526942*^9, {3.582630084621026*^9, 3.5826301414772778`*^9}, {
   3.5826301795974584`*^9, 3.5826301845417414`*^9}, {3.582631778365903*^9, 
   3.5826317868853903`*^9}, {3.5826318776535816`*^9, 
   3.5826320044698353`*^9}, {3.582632723494961*^9, 3.5826327602210617`*^9}, {
   3.5826328921976104`*^9, 3.582632933764988*^9}, {3.5826330607332497`*^9, 
   3.5826332027093706`*^9}, {3.582633281933902*^9, 3.5826332941175985`*^9}, {
   3.5826335619659185`*^9, 3.5826338000615373`*^9}, 3.582633836949647*^9, {
   3.5826339105978594`*^9, 3.5826340353739963`*^9}, {3.5826342271739664`*^9, 
   3.582634275069706*^9}, {3.5826351470695815`*^9, 3.582635294798031*^9}, {
   3.5826353275699053`*^9, 3.5826353843821554`*^9}, {3.5826363605939913`*^9, 
   3.582636360607992*^9}, {3.5827231975705547`*^9, 3.5827231985377564`*^9}, {
   3.5827232565698586`*^9, 3.5827232744162893`*^9}, {3.58272338263368*^9, 
   3.582723424488553*^9}, {3.5827235322959433`*^9, 3.582723629203314*^9}, {
   3.582723664958576*^9, 3.582723838399681*^9}, {3.5827242042709665`*^9, 
   3.58272421482257*^9}, {3.5827251839630013`*^9, 3.582725195958688*^9}, {
   3.582725249718763*^9, 3.5827252739591494`*^9}, {3.5827253077390814`*^9, 
   3.5827253077540817`*^9}, {3.5827254207105427`*^9, 3.582725423510703*^9}, {
   3.582725457534649*^9, 3.5827254892144613`*^9}, {3.582725520023223*^9, 
   3.5827255729182487`*^9}, {3.582725774454776*^9, 3.582725869031185*^9}, {
   3.582725911462612*^9, 3.582726039771951*^9}, {3.582726342839286*^9, 
   3.582726360847316*^9}, {3.582784212586396*^9, 3.5827842143491993`*^9}, 
   3.58278426055648*^9, {3.5827846669995937`*^9, 3.582784688699232*^9}, {
   3.5827847761974893`*^9, 3.582784848645633*^9}, 3.5828687559242077`*^9, {
   3.5828742799931664`*^9, 3.58287428442542*^9}}],

Cell[TextData[{
 StyleBox["README (Version 2.0)\n\nThe subdirectory CandS contains all the \
files of the form *.m.\nThey can be used with Mathematica.  For more \
information see the book\n\[OpenCurlyDoubleQuote]Modern Differential Geometry \
of Curves and Surfaces via Mathematica,\nSecond Edition\
\[CloseCurlyDoubleQuote] (CRC Press, 1998).",
  FontSlant->"Italic"],
 StyleBox[" A gallery of surfaces is available:\nhttp://bianchi.umd.edu, from \
which updates are available.  Updates\nare also available via anonymous ftp \
from bianchi.umd.edu.  You can\ncontact me at gray@bianchi.umd.edu.",
  FontSlant->"Italic",
  FontColor->RGBColor[1, 0, 1]],
 StyleBox["\n\n                 Alfred Gray, April, 1998.\n\nInstructions for \
use:\n\n    1. Make a subdirectory of your home directory called CandS\nand \
copy all of files of the form *.m into it.\n\n    2. To access the *.m files \
using version 2.2 of Mathematica,\nenter the following command into a \
Mathematica session:\n\nAppendTo[$Path,\[CloseCurlyDoubleQuote]~/CandS\
\[CloseCurlyDoubleQuote]]\n\nThis command can also be added to your file \
init.m;\nthen the files will be automatically accessible.\nThe *.m files will \
be accessible using version 3.x of Mathematica,\nprovided they are copied \
into the subdirectory ExtraPackages.\n\n    3. To use the main miniprograms, \
enter one of the following commands in a\nMathematica session:\n\nNeeds[\
\[OpenCurlyDoubleQuote]CAS`\[CloseCurlyDoubleQuote]]\nor\n<<CAS.m\n\nThis \
will load the following files:\n\n\tCSPROGS.m   (miniprograms for \
manipulating curves and surfaces)\n\tCURVES.m    (definitions of \
parametrizations and implicit equations of\n                     curves)\n\t\
SURFS.m\t    (definitions of parametrizations and implicit equations of\n     \
                surfaces)\n        PLTPROGS.m  (useful plotting programs \
commands that supplement those\n                     of Mathematica)",
  FontSlant->"Italic"],
 "\n"
}], "Text",
 CellChangeTimes->{3.582726174457655*^9}],

Cell[TextData[{
 "Unfortunately, the part printed in magenta, is not valid anymore. The four \
packages mentioned in the last paragraph are loaded into this notebook by the \
initialization; the start package CAS.m is not necessary. I adapted the \
packages CSPROGS.m and PLTPROGS.m  to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9.01. Many of the miniprograms of these packages are tested in the \
corresponding subsections below.  The packages CURVES.m and SURFACES.m  are \
catalogs of parameter presentations of plane and space curves, resp. surfaces \
in the Euclidean 3-space.  I used some of them, but I didn\[CloseCurlyQuote]t \
feel the necessity to test them all;  I didn\[CloseCurlyQuote]t change any.  \
Also I didn\[CloseCurlyQuote]t change the other packages of the file \
Gray.zip. \n\nI joined the adapted and the unchanged files of ",
 "Gray.zip",
 " with the current notebook into the new zip-file gray1.zip.  This file  and \
other ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebooks and packages can be downloaded from my ",
 StyleBox[ButtonBox["homepage",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/"], None},
  ButtonNote->"http://www-irm.mathematik.hu-berlin.de/~sulanke/"],
  Background->RGBColor[1, 0.9, 0.8]],
 ". Working with this stuff can students help to understand Euclidean \
differential geometry and to acquire experience in applying ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  I will be grateful for comments, criticism,  and corrections. Write to\n\
sulanke@mathematik.hu-berlin.de.\n\nA completely different method of \
programming and applying  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to differential geometry \nis available from DigiArea,  see the software \
package ",
 StyleBox[ButtonBox["atlas\.b2",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://digi-area.com/Mathematica/atlas/"], None},
  ButtonNote->"http://digi-area.com/Mathematica/atlas/"],
  Background->RGBColor[1, 0.9, 0.8]],
 ".\n\n"
}], "Text",
 CellChangeTimes->{{3.5827848705728874`*^9, 3.5827849203887367`*^9}, 
   3.58278497167667*^9, {3.58278522151696*^9, 3.5827852517566895`*^9}, {
   3.582785292813038*^9, 3.582785359252838*^9}, {3.582785485238044*^9, 
   3.5827858732212353`*^9}, {3.5827859552049246`*^9, 3.582786055493661*^9}, 
   3.582786117429203*^9, {3.582793222743047*^9, 3.582793241981148*^9}, {
   3.582793500565938*^9, 3.5827935021170263`*^9}, {3.5827937154382277`*^9, 
   3.5827937872453346`*^9}, {3.582793924478184*^9, 3.5827939585251317`*^9}, {
   3.582794379239195*^9, 3.5827943905328407`*^9}, {3.582794695869305*^9, 
   3.5827947589329123`*^9}, {3.5827948262457623`*^9, 3.582794896175762*^9}, {
   3.5827949713810635`*^9, 3.5827949822616863`*^9}, {3.5827952638217907`*^9, 
   3.5827953402451615`*^9}, {3.582795419909718*^9, 3.5827954264290905`*^9}, {
   3.5827954832453403`*^9, 3.582795507845748*^9}, {3.5827955442698307`*^9, 
   3.5827956186370845`*^9}, 3.582795672917189*^9, {3.5827957876297503`*^9, 
   3.5827958135332317`*^9}, {3.5827959439416904`*^9, 
   3.5827959628457723`*^9}, {3.5827960726700535`*^9, 
   3.5827960758302345`*^9}, {3.5827961445571656`*^9, 3.582796144564166*^9}, {
   3.582868822825034*^9, 3.582868845344322*^9}, {3.5828690740674047`*^9, 
   3.5828691334808025`*^9}, {3.582869255896805*^9, 3.582869497600629*^9}, {
   3.582869586512715*^9, 3.582869680593096*^9}, {3.5828698531049633`*^9, 
   3.582869853115964*^9}, {3.5828740809147797`*^9, 3.5828740914173803`*^9}, 
   3.582874130952642*^9, {3.58289992883937*^9, 3.5828999577306204`*^9}, {
   3.582900087375459*^9, 3.582900182094877*^9}, {3.5829604179486494`*^9, 
   3.5829604217068644`*^9}, {3.582962091726096*^9, 3.5829620917600985`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialization", "Section",
 CellChangeTimes->{3.5801220870453343`*^9}],

Cell[CellGroupData[{

Cell["Loading Gray`s four basic packages", "Subsection",
 CellChangeTimes->{{3.58012704110484*^9, 3.5801270955021353`*^9}, {
  3.5828743042485538`*^9, 3.5828743084407935`*^9}}],

Cell[TextData[{
 "1. Download the package gray1.zip from my ",
 StyleBox[ButtonBox["homepage",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/"], None},
  ButtonNote->"http://www-irm.mathematik.hu-berlin.de/~sulanke/"],
  Background->RGBColor[1, 0.9, 0.8]],
 " and extract all files into your working directory. ",
 StyleBox[ButtonBox["Download",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www-irm.mathematik.hu-berlin.de/~sulanke/Notebooks/gray1.zip"]\
, None},
  ButtonNote->
   "http://www-irm.mathematik.hu-berlin.de/~sulanke/Notebooks/gray1.zip"],
  Background->RGBColor[1, 0.9, 0.8]],
 ".\n2, Open the notebook cands-1.nb and insert the Path of your working \
directory in the shape corresponding to your operating system as argument of \
SetDirectory into the next cell. Here is an example valid in the OS Windows:"
}], "Text",
 CellChangeTimes->{{3.5829562609388824`*^9, 3.5829567019221053`*^9}, 
   3.5829594332753296`*^9, {3.582959464683126*^9, 3.5829594794669714`*^9}, {
   3.5829597138693786`*^9, 3.582959713876379*^9}, {3.5829611567010217`*^9, 
   3.582961156708022*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "SetDirectory", "[", 
   "\"\<E:\\\\mymath\\\\diffgeo\\\\Gray_Alfred\\\\CandS\>\"", "]"}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801222741740637`*^9, 3.5801222820676775`*^9}, 
   3.580122374976241*^9, 3.5801224093431015`*^9, {3.5801224518359766`*^9, 
   3.580122512005282*^9}}],

Cell["\<\
3. Save the notebook under a new name (to preserve the original notebook for \
further applications) 
4.  Activate Evaluation/Evaluate Initialization Cells  from the menu.\
\>", "Text",
 CellChangeTimes->{{3.5829567973545637`*^9, 3.58295690377065*^9}, {
  3.5829569345704117`*^9, 3.582957027618734*^9}, {3.582957135123883*^9, 
  3.582957158690231*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Get", "[", "\"\<CSPROGS.m\>\"", "]"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801225375737267`*^9, 3.580122559554166*^9}, {
  3.580274594996056*^9, 3.580274611555003*^9}, {3.58027471256478*^9, 
  3.5802747188191385`*^9}, {3.582379164511468*^9, 3.582379168317875*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The Packages \"\>", "\[InvisibleSpace]", "\<\" CSPROGS.m\"\>", 
   "\[InvisibleSpace]", "\<\", \"\>", "\[InvisibleSpace]", "\<\"CURVES.m\"\>",
    "\[InvisibleSpace]", "\<\", \"\>", 
   "\[InvisibleSpace]", "\<\" SURFS.m\"\>", "\[InvisibleSpace]", "\<\", \"\>",
    "\[InvisibleSpace]", "\<\" PLTPROGS.m\"\>", 
   "\[InvisibleSpace]", "\<\" are loaded!\"\>"}],
  SequenceForm[
  "The Packages ", " CSPROGS.m", ", ", "CURVES.m", ", ", " SURFS.m", ", ", 
   " PLTPROGS.m", " are loaded!"],
  Editable->False]], "Print",
 CellChangeTimes->{3.5823792128091526`*^9, 3.5823796318726892`*^9, 
  3.582380151026001*^9, 3.582436543753379*^9, 3.5824381569416475`*^9, 
  3.5824423131243677`*^9, 3.5824460183146973`*^9, 3.582447136016626*^9, 
  3.5824474593601203`*^9, 3.5824481260202513`*^9, 3.5825511011187277`*^9, 
  3.5825526736601844`*^9, 3.582554555278551*^9, 3.5826013852357917`*^9, 
  3.5826041924389715`*^9, 3.582604480229432*^9, 3.5828746657192287`*^9, 
  3.58295704031546*^9}]
}, Open  ]],

Cell["\<\
Now the notebook is ready for your own work. You can use all the \
miniproograms listed in the next chapter. To start a session you have to do \
step 4 only; steps 1 - 3 are necessary only once before the first use of the \
notebook.\
\>", "Text",
 CellChangeTimes->{{3.582957204152831*^9, 3.58295722529104*^9}, {
  3.5829573152581863`*^9, 3.5829573348503065`*^9}, {3.582957376074664*^9, 
  3.5829574615385528`*^9}, {3.5829575453783484`*^9, 3.582957725282638*^9}}],

Cell[TextData[{
 "Remarks.1.  If you want to test the original packages of Alfred Gray \
yourself, or if you work with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version 3,  start step 1 with downloading ",
 "Gray.zip",
 " from [",
 ButtonBox["G5",
  BaseStyle->"Hyperlink",
  ButtonData->"Gray5"],
 "].\n2. To avoid error messages I deleted in  Gray`s original packages  the  \
call  of   obsolete ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " packages of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v.2.2; see the ",
 ButtonBox["Appendix",
  BaseStyle->"Hyperlink",
  ButtonData->"Obso"],
 ".\n\n\n"
}], "Text",
 CellChangeTimes->{{3.5801222136771574`*^9, 3.5801222149719596`*^9}, {
   3.5801226996736116`*^9, 3.5801228028053927`*^9}, {3.58026955834613*^9, 
   3.580269634692664*^9}, {3.580270054286601*^9, 3.5802700682330256`*^9}, {
   3.5802701142999067`*^9, 3.5802701143155065`*^9}, {3.582380613332813*^9, 
   3.582380613972414*^9}, {3.5828743367204113`*^9, 3.582874342312731*^9}, {
   3.582957485393917*^9, 3.582957500458779*^9}, {3.582957735858243*^9, 
   3.5829577915864305`*^9}, {3.582957834674895*^9, 3.5829578852987905`*^9}, {
   3.5829579252510757`*^9, 3.5829579469143143`*^9}, {3.5829580302110786`*^9, 
   3.5829580572516255`*^9}, {3.58295812166931*^9, 3.5829581216743107`*^9}, 
   3.582959064648245*^9, {3.582959117467266*^9, 3.5829591184103203`*^9}, {
   3.582959192258544*^9, 3.582959208858494*^9}, 3.5829620907360396`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["The usages",
 FontSize->36,
 FontColor->GrayLevel[0]]], "Subtitle",
 CellChangeTimes->{{3.580122895063955*^9, 3.5801229024115677`*^9}},
 TextAlignment->Center],

Cell[CellGroupData[{

Cell["CSPROGS", "Section",
 CellChangeTimes->{{3.5802697191980124`*^9, 3.5802697258592243`*^9}}],

Cell[TextData[{
 "The package contains 186 of Alfred  Gray\.08`s miniprograms. I couldn\
\[CloseCurlyQuote]t test all of them. The user should try the wanted program \
and adapt it, if necessary, to the actual ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version."
}], "Text",
 CellChangeTimes->{{3.5805256884096603`*^9, 3.5805257297965326`*^9}, {
   3.5805257931658144`*^9, 3.5805258171171846`*^9}, {3.5805258489340043`*^9, 
   3.5805258722773395`*^9}, {3.5805259238212876`*^9, 3.580525994237315*^9}, 
   3.5823514557085667`*^9, 3.5826045842653823`*^9, {3.5826094393210754`*^9, 
   3.582609453896909*^9}, {3.582874371369393*^9, 3.582874377104721*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<CSPROGS`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5805255402874002`*^9, 3.5805255938110943`*^9}, 
   3.5805257143213053`*^9, 3.5823463367935753`*^9, 3.582346456632986*^9, 
   3.582440420692127*^9, 3.5824423773010387`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805256052303143`*^9, 3.5805256149803314`*^9}}],

Cell[BoxData["186"], "Output",
 CellChangeTimes->{3.5805256231079454`*^9, 3.580526007485073*^9, 
  3.5823463265599575`*^9, 3.582351374931625*^9, 3.582378396272519*^9, 
  3.582440486758906*^9, 3.5824423830103655`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "fff", "]"}]], "Input",
 CellChangeTimes->{{3.5824474831124787`*^9, 3.5824474933500643`*^9}, 
   3.582604550010423*^9}],

Cell[BoxData[
 RowBox[{"{", "Protected", "}"}]], "Output",
 CellChangeTimes->{3.5824474972032847`*^9, 3.5824481438052683`*^9, 
  3.5826014130038404`*^9, 3.5826014980083895`*^9, 3.5826045514665065`*^9}]
}, Open  ]],

Cell["\<\
The subsections corresponding to a miniprogram start with the usage, followed \
by a simple example. For more information see or copy the definition of the \
program from the package.. 
\
\>", "Text",
 CellChangeTimes->{{3.582351478125806*^9, 3.582351604142827*^9}, {
   3.582351661410528*^9, 3.582351929153798*^9}, {3.582351959839052*^9, 
   3.582351980758689*^9}, 3.5823520139711475`*^9, 3.582352924310746*^9, 
   3.5823529671484213`*^9, {3.5823530441345563`*^9, 3.582353048486964*^9}, {
   3.582440500011664*^9, 3.582440525978149*^9}, {3.582440557242937*^9, 
   3.582440624882806*^9}, {3.582609490833022*^9, 3.5826095160174627`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CSPROGS`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801228070174007`*^9, 3.580122832507845*^9}, 
   3.582352030756777*^9, 3.5823799427032347`*^9}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["CSPROGS`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["CSPROGS`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["analytictominimal",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"analytictominimal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["ggg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ggg", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plotintrinsicea",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"plotintrinsicea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["analytictominimalpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"analytictominimalpolar", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["gggg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gggg", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plotintrinsicsurfrev",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"plotintrinsicsurfrev", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["approxcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"approxcurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["gmap",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gmap", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plotsurfrevfromgc",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"plotsurfrevfromgc", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["arclength",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"arclength", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["grad",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"grad", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plottl",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"plottl", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["arclengthea",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"arclengthea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["harmonictoanalytic",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"harmonictoanalytic", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plottp",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"plottp", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["arclengtheaprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"arclengtheaprime", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["hcursq",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hcursq", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["polarize",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"polarize", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["arclengthprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"arclengthprime", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["helical",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"helical", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["polarsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"polarsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["asymeq1",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"asymeq1", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["hessian",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hessian", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["prindev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"prindev", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["asymeq2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"asymeq2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["infarea",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"infarea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["reflectgraphics",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"reflectgraphics", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["binormal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"binormal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["inscurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"inscurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["reflection",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"reflection", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["binormalsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"binormalsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["intrinsic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"intrinsic", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["bjorling",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bjorling", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["intrinsicea",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"intrinsicea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rollcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rollcurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["bracket",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bracket", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["inversion",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"inversion", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rotation2d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rotation2d", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["brioschicurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"brioschicurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["inversioncurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"inversioncurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rotation3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rotation3d", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["canalsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"canalsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["inversionsurf",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"inversionsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rt", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["christoffel",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"christoffel", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["involute",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"involute", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["rtoc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rtoc", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["circle3point",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"circle3point", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["involutends",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"involutends", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["ruled",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ruled", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["clairautu",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clairautu", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["J",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"J", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["seashell",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"seashell", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["clairautv",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clairautv", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["jacobian2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"jacobian2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["sff",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sff", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["combescure",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"combescure", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["k1",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"k1", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["singmonge",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"singmonge", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["commutator",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"commutator", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["k2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"k2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["singmongepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"singmongepolar", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["cone",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cone", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["solvegeoeqs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"solvegeoeqs", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["contrapedal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"contrapedal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["stereoinj",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"stereoinj", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["cross",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cross", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa23d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa23d", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["stereoinjcomplex",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"stereoinjcomplex", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["csprogs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"csprogs", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa2c",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa2c", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["stereoproj",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"stereoproj", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["ctor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ctor", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa2ea",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa2ea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["stretch",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"stretch", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["curveonsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"curveonsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa2implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"kappa2implicit", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["studycurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"studycurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["cylinder",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cylinder", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappa2polar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappa2polar", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfacejacobian",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"surfacejacobian", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["darbouxvector",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"darbouxvector", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kappan",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappan", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfacejacobianmatrix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"surfacejacobianmatrix", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["der",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"der", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kmu",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kmu", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfacenormal",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"surfacenormal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["descenttime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"descenttime", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["kpi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kpi", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfast",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"surfast", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["descenttimen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"descenttimen", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["laplacian",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"laplacian", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"surfrev", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["descenttimends",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"descenttimends", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["length",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"length", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["surfrevfromgc",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"surfrevfromgc", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["descenttimeprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"descenttimeprime", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["lengthea",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lengthea", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tandev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tandev", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["directionalD",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"directionalD", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["lengthean",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lengthean", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tangent",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tangent", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["divergence",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"divergence", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["lengtheands",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lengtheands", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tangentkmu",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tangentkmu", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["ds2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ds2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["lengthn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lengthn", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tangentkpi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tangentkpi", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["ee",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ee", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["lengthnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lengthnds", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tangentline",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tangentline", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["eee",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eee", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["mcurvature",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"mcurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tangentplane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tangentplane", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["eeee",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eeee", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["meancurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"meancurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tau",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tau", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["evolute",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"evolute", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["meancurvaturevf",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"meancurvaturevf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tl",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tl", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["evolute3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"evolute3d", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["metricmatrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"metricmatrix", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["totalscalarcurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"totalscalarcurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["ff",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ff", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["monge",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"monge", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["translategraphics",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"translategraphics", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["fff",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"fff", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["mongepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"mongepolar", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["translation",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"translation", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["ffff",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ffff", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["nateq",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nateq", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["transsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"transsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["filament",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"filament", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["nestedevolutes",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"nestedevolutes", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tubecurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tubecurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["focalk1",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"focalk1", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["normal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"normal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["tubeplanecurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"tubeplanecurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["focalk2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"focalk2", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["normalcurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"normalcurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["turningangle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"turningangle", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["focalkmu",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"focalkmu", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["normalfactor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"normalfactor", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["turningnumber",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"turningnumber", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["focalkpi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"focalkpi", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["normalline",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"normalline", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["twist",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"twist", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["frenetframe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"frenetframe", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["normalsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"normalsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["umeqs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"umeqs", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gaussbrioschi",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"gaussbrioschi", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["osculatingcircle",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"osculatingcircle", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["unitnormal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"unitnormal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gaussiancurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"gaussiancurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["oval",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"oval", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["unitspeednds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"unitspeednds", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gcursq",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gcursq", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["paramtoimp",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"paramtoimp", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["wei",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wei", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gcurvature",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gcurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["parcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"parcurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weicurvature",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weicurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["genhel",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"genhel", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["parsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"parsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weidata",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weidata", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gensurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gensurfrev", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["pedal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pedal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weimetric",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weimetric", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["geo",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"geo", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["pedsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pedsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weingarten",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weingarten", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["geodesiccurvature",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"geodesiccurvature", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["perpsurf",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"perpsurf", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weinormal",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weinormal", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["geodesictorsion",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"geodesictorsion", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["planeframe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"planeframe", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weipolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weipolar", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["geomincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"geomincurve", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plotintrinsic",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"plotintrinsic", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["weiprime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"weiprime", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]},
        {
         ButtonBox["gg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gg", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["plotintrinsic3d",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"plotintrinsic3d", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"], 
         ButtonBox["writhe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"writhe", "CSPROGS`"}},
          ButtonNote->"CSPROGS`"]}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.31666666666666665`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.582957040435467*^9}]
}, Open  ]],

Cell["Click the name to get the usage!", "Text",
 CellChangeTimes->{3.582440710730716*^9, 3.5828688637413745`*^9}],

Cell[CellGroupData[{

Cell["Simple Examples", "Subsection",
 CellChangeTimes->{{3.5813421462676697`*^9, 3.5813421544991403`*^9}, {
  3.5818577417431965`*^9, 3.5818577451908026`*^9}}],

Cell[TextData[{
 "In this section we consider examples for Alfred*s miniprograms running \
without problems also in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9."
}], "Text",
 CellChangeTimes->{{3.5814007629688253`*^9, 3.581400866479746*^9}}],

Cell[CellGroupData[{

Cell["approxcurve", "Subsubsection",
 CellChangeTimes->{3.5805263696957903`*^9, 3.5814006275600805`*^9}],

Cell[BoxData[
 RowBox[{"?", "approxcurve"}]], "Input",
 CellChangeTimes->{{3.5823533427502813`*^9, 3.582353351392696*^9}, 
   3.5823800181605673`*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"ci", "[", "t_", "]"}], ":=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Cos", "[", "t", "]"}], ",", 
     RowBox[{"Sin", "[", "t", "]"}]}], "}"}], 
   "                                  "}]}]], "Input",
 CellChangeTimes->{{3.5805260313844395`*^9, 3.5805260363827257`*^9}, {
  3.5805266915031967`*^9, 3.580526701054743*^9}, {3.5805267402809863`*^9, 
  3.580526758992057*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cir1", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5805271053408666`*^9, 3.5805271123832693`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"approxcurve", "[", 
   RowBox[{"0", ",", 
    RowBox[{"2", "*", "Pi"}], ",", "7"}], "]"}], "[", "ci", "]"}]], "Input",
 CellChangeTimes->{{3.580526404907804*^9, 3.580526504912524*^9}, 
   3.5805265421286526`*^9, {3.5805265751525416`*^9, 3.580526593048565*^9}, {
   3.5805266251524014`*^9, 3.5805266581762905`*^9}, 3.5805267271052327`*^9, {
   3.5805267696086636`*^9, 3.580526770103692*^9}, {3.580526945176706*^9, 
   3.580526949863974*^9}, {3.580526984024928*^9, 3.580526984519956*^9}, {
   3.5805272243996763`*^9, 3.5805272724884267`*^9}}],

Cell[BoxData[
 RowBox[{"Line", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Sin", "[", 
       FractionBox[
        RowBox[{"3", " ", "\[Pi]"}], "14"], "]"}], ",", 
      RowBox[{"Cos", "[", 
       FractionBox[
        RowBox[{"3", " ", "\[Pi]"}], "14"], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Pi]", "14"], "]"}]}], ",", 
      RowBox[{"Cos", "[", 
       FractionBox["\[Pi]", "14"], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Pi]", "7"], "]"}]}], ",", 
      RowBox[{"Sin", "[", 
       FractionBox["\[Pi]", "7"], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Pi]", "7"], "]"}]}], ",", 
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Pi]", "7"], "]"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Pi]", "14"], "]"}]}], ",", 
      RowBox[{"-", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Pi]", "14"], "]"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Sin", "[", 
       FractionBox[
        RowBox[{"3", " ", "\[Pi]"}], "14"], "]"}], ",", 
      RowBox[{"-", 
       RowBox[{"Cos", "[", 
        FractionBox[
         RowBox[{"3", " ", "\[Pi]"}], "14"], "]"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{
  3.5805266594063606`*^9, 3.5805267726138353`*^9, 3.580526953537184*^9, 
   3.5805269859940405`*^9, 3.5805271717576656`*^9, {3.580527203634489*^9, 
   3.580527273975512*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Graphics", "[", "%", "]"}], ",", "cir1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.580526993697481*^9, 3.580527011143479*^9}, {
  3.580527180280153*^9, 3.580527208992795*^9}}],

Cell["\<\
Syntax.  Before using approxcurve first give the curve a Name; inserting \
parameter representations yield senseless results:\
\>", "Text",
 CellChangeTimes->{{3.580527954170417*^9, 3.580528088718113*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"approxcurve", "[", 
   RowBox[{"0", ",", 
    RowBox[{"2", "*", "Pi"}], ",", "7"}], "]"}], "[", 
  RowBox[{"ci", "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.580526404907804*^9, 3.580526504912524*^9}, 
   3.5805265421286526`*^9, {3.5805265751525416`*^9, 3.580526593048565*^9}, {
   3.5805266251524014`*^9, 3.5805266581762905`*^9}, 3.5805267271052327`*^9, {
   3.5805267696086636`*^9, 3.580526770103692*^9}, {3.580526945176706*^9, 
   3.580526949863974*^9}, {3.580526984024928*^9, 3.580526984519956*^9}, {
   3.5805272243996763`*^9, 3.5805272724884267`*^9}, {3.580527833127494*^9, 
   3.580527835199612*^9}}],

Cell[BoxData[
 RowBox[{"Line", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", "0", "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"2", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"4", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"6", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"8", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"10", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"12", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     RowBox[{"2", " ", "\[Pi]"}], "]"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.5805278392448435`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"approxcurve", "[", 
   RowBox[{"0", ",", 
    RowBox[{"2", "*", "Pi"}], ",", "7"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", "t", "]"}], ",", " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.580526404907804*^9, 3.580526504912524*^9}, 
   3.5805265421286526`*^9, {3.5805265751525416`*^9, 3.580526593048565*^9}, {
   3.5805266251524014`*^9, 3.5805266581762905`*^9}, 3.5805267271052327`*^9, {
   3.5805267696086636`*^9, 3.580526770103692*^9}, {3.580526945176706*^9, 
   3.580526949863974*^9}, {3.580526984024928*^9, 3.580526984519956*^9}, {
   3.5805272243996763`*^9, 3.5805272724884267`*^9}, {3.580527833127494*^9, 
   3.580527889447715*^9}}],

Cell[BoxData[
 RowBox[{"Line", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", "0", "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"2", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"4", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"6", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"8", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"10", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     FractionBox[
      RowBox[{"12", " ", "\[Pi]"}], "7"], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "t", "]"}], ",", 
       RowBox[{"Sin", "[", "t", "]"}]}], "}"}], "[", 
     RowBox[{"2", " ", "\[Pi]"}], "]"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.580527891122811*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["darbouxvector", "Subsubsection",
 CellChangeTimes->{3.581400498490698*^9}],

Cell[BoxData[
 RowBox[{"?", "darbouxvector"}]], "Input",
 CellChangeTimes->{{3.582353431888838*^9, 3.582353434696842*^9}, 
   3.5823800181761675`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"darbouxvector", "[", 
   RowBox[{"helix", "[", 
    RowBox[{"2", ",", "2"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5813331789887705`*^9, 3.5813332053402777`*^9}, {
  3.5813364812476487`*^9, 3.581336509036238*^9}, {3.5813369288382497`*^9, 
  3.5813369877256174`*^9}, {3.5813371191271334`*^9, 3.58133712396541*^9}, {
  3.581337156622278*^9, 3.5813371628766356`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", 
   FractionBox[
    SqrtBox[
     RowBox[{"4", "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]], 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5813332089594846`*^9, 3.5813365178517427`*^9, 3.5813366258849216`*^9, {
   3.581336945478201*^9, 3.5813369893947134`*^9}, 3.581337125850518*^9, 
   3.581337164771744*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["evolute, evolute3d", "Subsubsection",
 CellChangeTimes->{{3.5814004457126794`*^9, 3.581400472424207*^9}}],

Cell[BoxData[
 RowBox[{"?", "evolute"}]], "Input",
 CellChangeTimes->{{3.582353480529723*^9, 3.582353486691734*^9}, 
   3.5823800181917677`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"evolute", "[", 
   RowBox[{"ellipse", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"2", "Pi"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"evolute", "[", 
   RowBox[{"ellipse", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.581341725066578*^9, 3.5813417327010145`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "3"]}], "2"], ",", 
   RowBox[{
    RowBox[{"-", "3"}], " ", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "t", "]"}], "3"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581341733978088*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"?", "evolute3d"}]], "Input",
 CellChangeTimes->{{3.5823535018081603`*^9, 3.582353521370595*^9}, 
   3.5823800181917677`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"evolute3d", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.581341824293254*^9, 3.581341869701851*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["b", "2"], " ", 
      RowBox[{"Cos", "[", "t", "]"}]}], "a"]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["b", "2"], " ", 
      RowBox[{"Sin", "[", "t", "]"}]}], "a"]}], ",", 
   RowBox[{"b", " ", "t"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5813418398031406`*^9, 3.5813418710879297`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"evolute3d", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"2", ",", "2"}], "]"}], "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813420598567266`*^9, 3.5813420809179316`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{"2", " ", "t"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5813420738205256`*^9, 3.581342081900988*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"evolute3d", "[", 
     RowBox[{"helix", "[", 
      RowBox[{"2", ",", "2"}], "]"}], "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581341906173937*^9, 3.581342001094366*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["filament", "Subsubsection",
 CellChangeTimes->{{3.5814004134898367`*^9, 3.58140042700861*^9}}],

Cell[BoxData[
 RowBox[{"?", "filament"}]], "Input",
 CellChangeTimes->{{3.58235359851273*^9, 3.5823536043939404`*^9}, 
   3.582380018207368*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"filament", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"2", ",", ".5"}], "]"}], "]"}], "[", 
   RowBox[{"1", ",", "1"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5813996454909096`*^9, 3.581399740667353*^9}, 
   3.581400155604086*^9, 3.5814002242520123`*^9, 3.581400306868738*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"Cos", "[", "t", "]"}]}], "+", 
    RowBox[{"1.`", " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
    RowBox[{"0.24253562503633297`", " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "t", "]"}], "2"]}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", 
      RowBox[{"0.757464374963667`", " ", 
       RowBox[{"Cos", "[", "t", "]"}]}]}], ")"}], " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"0.5`", " ", "t"}], "+", 
    RowBox[{"0.9701425001453319`", " ", 
     RowBox[{"Sin", "[", "t", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581399741906424*^9, 3.5814001582182355`*^9, 
  3.5814002256250916`*^9, 3.58140026319724*^9, 3.581400308380824*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5813997661918135`*^9, 3.581399816018663*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic1", " ", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.581399835822796*^9, 3.5813998432492204`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"2", ",", ".5"}], "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581399846798424*^9, 3.5813999645151567`*^9}, {
  3.5814001794524503`*^9, 3.581400190094059*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "pic1"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581399971146536*^9, 3.581400042746631*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["jacobian2. surfacejacobian. surfacejacobianmatrix", "Subsubsection",
 CellChangeTimes->{{3.582083645760811*^9, 3.5820836840120783`*^9}, 
   3.582084735984326*^9}],

Cell[BoxData[
 RowBox[{"?", "surfacejacobianmatrix"}]], "Input",
 CellChangeTimes->{{3.582353684203681*^9, 3.582353686465685*^9}, 
   3.5823800182229676`*^9}],

Cell[BoxData[
 RowBox[{"?", "jacobian2"}]], "Input",
 CellChangeTimes->{{3.582353722891749*^9, 3.5823537257309537`*^9}, 
   3.5823800182385674`*^9}],

Cell["coincides with", "Text",
 CellChangeTimes->{{3.5820846474385705`*^9, 3.582084652071779*^9}}],

Cell[BoxData[
 RowBox[{"?", "surfacejacobian"}]], "Input",
 CellChangeTimes->{{3.5820838612595897`*^9, 3.582083871181207*^9}, {
   3.5820846590449905`*^9, 3.582084660760994*^9}, {3.5823537779754457`*^9, 
   3.582353792639471*^9}, 3.5823800182541676`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ellipsoid", "[", 
   RowBox[{"2", ",", "1", ",", "3"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5820840909543934`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"3", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.582084094058799*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"elljm", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{
   RowBox[{"surfacejacobianmatrix", "[", 
    RowBox[{"ellipsoid", "[", 
     RowBox[{"2", ",", "1", ",", "3"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820839533465514`*^9, 3.5820840364946976`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      RowBox[{"Cos", "[", "v", "]"}], " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "u", "]"}], " ", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      RowBox[{"Cos", "[", "u", "]"}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sin", "[", "u", "]"}]}], " ", 
      RowBox[{"Sin", "[", "v", "]"}]}], ",", 
     RowBox[{"3", " ", 
      RowBox[{"Cos", "[", "v", "]"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820840381795006`*^9, 3.582084524728755*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.582084058927537*^9, 3.5820840603939395`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], 
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Cos", "[", "v", "]"}]}], "0"},
     {
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Sin", "[", "u", "]"}]}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], 
      RowBox[{"3", " ", 
       RowBox[{"Cos", "[", "v", "]"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.5820840615951414`*^9, 3.5820845446655903`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"jacobian2", "[", 
   RowBox[{"ellipsoid", "[", 
    RowBox[{"2", ",", "1", ",", "3"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.582084120641245*^9, 3.5820841479100933`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"9", " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "4"]}], "+", 
  RowBox[{"36", " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "4"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "u", "]"}], "2"]}], "+", 
  SuperscriptBox[
   RowBox[{"Sin", "[", 
    RowBox[{"2", " ", "v"}], "]"}], "2"]}]], "Output",
 CellChangeTimes->{
  3.58208416893893*^9, {3.5820843600236654`*^9, 3.5820843797577004`*^9}, 
   3.582084552044403*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "\[Equal]", 
   RowBox[{"Det", "[", 
    RowBox[{
     RowBox[{"elljm", "[", 
      RowBox[{"u", ",", "v"}], "]"}], ".", 
     RowBox[{"Transpose", "[", 
      RowBox[{"elljm", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820841862081604`*^9, 3.582084213305408*^9}, {
  3.582084318699193*^9, 3.582084372425688*^9}, {3.5820844304733896`*^9, 
  3.582084442641411*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.582084375514493*^9, 3.582084384000908*^9}, 
   3.5820845593296156`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"surfacejacobian", "[", 
     RowBox[{"ellipsoid", "[", 
      RowBox[{"2", ",", "1", ",", "3"}], "]"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820843900069184`*^9, 3.58208439660573*^9}, {
  3.582084450659825*^9, 3.5820844590682397`*^9}, {3.582084495821904*^9, 
  3.582084514760338*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.582084397650932*^9, 3.58208456748843*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["monge. singmonge. singmongepolar", "Subsubsection",
 CellChangeTimes->{{3.582001219238875*^9, 3.582001243652918*^9}}],

Cell[BoxData[
 RowBox[{"?", "monge"}]], "Input",
 CellChangeTimes->{{3.5823538993280582`*^9, 3.5823539031032653`*^9}, 
   3.5823800182697678`*^9}],

Cell[BoxData[
 RowBox[{"?", "mongepolar"}]], "Input",
 CellChangeTimes->{{3.5823539186096926`*^9, 3.582353923632901*^9}, 
   3.5823800182697678`*^9}],

Cell[BoxData[
 RowBox[{"?", "singmonge"}]], "Input",
 CellChangeTimes->{{3.5823539409021316`*^9, 3.5823539477973433`*^9}, 
   3.5823800182853675`*^9}],

Cell[BoxData[
 RowBox[{"?", "singmongepolar"}]], "Input",
 CellChangeTimes->{{3.5823539409021316`*^9, 3.5823539632101707`*^9}, 
   3.5823800183009677`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", 
   RowBox[{"r_", ",", "u_"}], "]"}], " ", ":=", " ", 
  RowBox[{"r", "*", 
   RowBox[{"Cos", "[", "u", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581999945496638*^9, 3.5819999741226883`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"singmongepolar", "[", 
   RowBox[{
    RowBox[{"-", "2"}], ",", "g"}], "]"}], "[", 
  RowBox[{"r", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.581999987195511*^9, 3.582000019128767*^9}, {
   3.582000101730912*^9, 3.5820001204353447`*^9}, 3.5820009487187996`*^9, 
   3.5820010099021072`*^9, {3.5820010861394415`*^9, 3.5820010874654436`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Cos", "[", "u", "]"}], 
    SuperscriptBox["r", "2"]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "u", "]"}], 
    SuperscriptBox["r", "2"]], ",", 
   RowBox[{"r", " ", 
    RowBox[{"Cos", "[", "u", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.58200012871896*^9, 3.582000620853024*^9, 
  3.58200078376411*^9, 3.5820009504660025`*^9, 3.58200101172731*^9, 
  3.5820010890566463`*^9, 3.5820011198823004`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{"%", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"r", "\[Rule]", " ", 
       RowBox[{"Norm", "[", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"Sin", "[", "u", "]"}], " ", "\[Rule]", " ", 
       RowBox[{"y", "/", 
        RowBox[{"Norm", "[", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}], "]"}]}]}], ",", " ", 
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], "\[Rule]", " ", 
       RowBox[{"x", "/", 
        RowBox[{"Norm", "[", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}], "]"}]}]}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.582000418723469*^9, 3.5820005881865664`*^9}, {
   3.58200080647775*^9, 3.582000848067423*^9}, 3.5823750068857656`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["x", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Abs", "[", "x", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"Abs", "[", "y", "]"}], "2"]}], ")"}], 
     RowBox[{"3", "/", "2"}]]], ",", 
   FractionBox["y", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Abs", "[", "x", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"Abs", "[", "y", "]"}], "2"]}], ")"}], 
     RowBox[{"3", "/", "2"}]]], ",", "x"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5820005896529694`*^9, 3.5820006266250343`*^9, 3.5820007746224937`*^9, 
   3.582000852669431*^9, 3.582000958609217*^9, 3.5820010173745203`*^9, {
   3.582001095998658*^9, 3.5820011225499053`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "2"}], ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582000670258311*^9, 3.582000742049637*^9}, {
   3.582001025658135*^9, 3.5820010361257534`*^9}, 3.5820011136578894`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJyEfXc81u/3v1AZDQpRFKWlvafOnSQaor2IhhYtaSdNlUqUUkkoFNnrNq/b
3nvvTfaKVOJXb/d9Xt9ev8fr8ekfj87j9Xi9zvU85zrnec513delcOjMtqP8
fHx8C4X5+AT+/K0xDy+0dwgkNYc31//9e3KHgsPAwADZRo6aOa9lk1EvzRqs
zMOJmZUsS1HjJ1nUN9SV/wabvHYd710rGk1epo2rULf+RmInnjzcOyuQFC2b
d9GmIp7sHjeNrziwheRX5qTvWOtNNvgdcN1xPoXc/u89VSSrXGrZ3Z43ZMvv
e+vbL6QT/7mb+QKNkslqPT7Dx1OtwT70ra9/ZDpx+Fo4oKgRBbVTPY0bl3nA
cl+XOY3LUsnPgb//yuGyuPvim5sDwLRBuH1NYQLJuifMCTRqgmc27dmxqmxw
fJTxxqAvmuT8J++CzWVdnz+cZ8ObwmuHH8kREqE+St5a8QcU7It6JD40CG48
Fru1ozyIfBIda2ZU3A+72nbfDa3yh30Tl4scUfMlGvWiZxU1+Fm39w3XMbTx
Ba0q9ltD+S/c9wxlPVFI5Lc85A1251ydIwqcyRI3sYyBgeEsOLg5SEYgjPze
uv/QJ3424vzJ4lm86ClCsr8Zrj54hIM4LykyjPYK5pCsZ16nGzfHIc5Kuo5v
eoQ5RG3bBWtlu1TE+eqrfLfMJDZR0pc/JmebhTjr24Yd6FJzJruUQrK71HIR
5+csrSR2mB3kBrZVPXySizhHflTXeXk1EDq3ivmbqWYjztsgJMxYiUCL5XQr
Y6V0xLnm7leJoGgO1CkfValxi0ecJwVm95/sI1C7OaHo+PlIxFm2RCkrKCwM
WgPV+DQtghFnuKX0ylwqGMrO3HatzaBwLj85r8X+YwB4N28UzR1L4azaPmaT
pL0vLH4gEPp3fDycgwJsZ4uuiST7xCIr4hwpnH9cdm4t14shi51Hhp5gRyLO
oz5K7dKeG0/OXOlP8YpMRJy1ycV5ZsMTyOiLky2ar2UhzjE1890/S8SQtaMd
N+bIFCLO/b5T6+MX+pJDeaVKH3aUIc7fhdScDgV9gpRJXzasrC1DnMfkbH2z
3yQKZiSueXdbtwhxflt/l39RSDyEK0wSz43NRpwfXV3ZLG4TD+Znq3eciUhG
nLfumddSPRADjx4YKc8eF404B9SYZcwrjgQZzpsC+QshiPONZR8UN1aGg8+N
CVvSvlA471h5WT64iw06T11NnX+4Ic5qQj0JC9r8YXWZ6Pv+tg+I8yLNglEF
HTFk1nXj41ltQYizSDW5unF1Esm8PaV0ESsKca7PnH9pulIGWcOZJMoOS0Gc
DSMPBPjbZZGlK+P7WF/zEOcElsuQSvNMMmboyWmV5pWIc3OHns1ff9qw6rX9
0nX1iHP3PIvh2W1BMHMjWZPVVo84K6/Ozpe7ngFlrYn7f+dVIc4T7u9h12Zk
gZjXoRvmUoWI87bvI9+yozLgcY2em8vjdMQ5psTWT/pUMmTf05ZKaadwXsLy
uH64IRbMFlb3z+yl/Nk2/7pApXkkvKvb9LPC3AdxdnkfPH737jBware/VX37
E+K86ZWiwL6lf/QvNrx1y8sRcRYtGTUO/sS1ER3S6//EOcRZK32MtsH9DDI+
xvfJjjWUP8933x6vsiWPPJS7PnVvWCri3MHnuC7jfCk5ptwbXT+3GHEWbh7C
3xlcQdLZDsuapBoQ56XT60RvVmWT3+rjr92s6kKcwcs6bnZZMjh4vq51qPuG
OEsFfI8pP1gBU9+6OYiPaUacZacHaxmklMGRns9T//xFnANF+dfVuOXDEJnf
m0r9MxDnCaHXg2QeZsK1DcvWL1KgcI45Ey0Xp5IEgoEhN1J2U/HZZK6kv6Ns
DMgFa5p2KnkizkqSBzy9LAjcrM08pn7xI+I8d1iY11yNYGiy4Z/VXGuHOE8p
kGT/zU/L7JYN3fknX/Fw1iwUs2f/zCYV63R3TE4JRZxfxltOm2RRSg4/+Wj8
Ry/E+cGo2aFVzXVEdbnDizXLShDnMzme7j+2dRB1F4GMv395OA+E2yp3KrUS
0zHlrY6yghwezl0K31rTdesgSmnzonTdYRwezmFd2iOPqHUCZ37lxtCqb4jz
i4PqlY1SDTDj9+Scjj/25OG8eamZwuuxFTCseKwA/7o0xPn50/1xM+/kQtGo
xctObaTy4NNqP9bOnWmw8fP+MI8fPohzkNswSNeNA9VZf/VxQZxFRz3Rc5GN
BP898lK7X79DnPVcRZNCe0JArXNL4AqLF4iz0XHnpCui6SQhQ33JyXBHxPnr
Y6+R0h55pO1F80TXx16I855Lmrv6h1SRhyq93rFDOIizg2rsKJdF7eRnhufV
lPZsxHnimex1leZDOF2LXh2vNKdw9t1WFlWbMYYjOjt55QMpGcRZI27Opkpz
cVYEeyK7wlwBce7U9CL5HYKsSd2qcZlt3xFn49JV8+RsO6Fpm8u5zKQCxPnx
MVA0EqyByk7f1b/zohDnjJ2iLa2eBXBWZubZl10+iLOvNvtmlWsGDL+8cbZp
+AfEmWVtOi+rLR46szsmZ7W9Qpxniww47OaPgvRE/f3O0ywRZ86TvoDM0WEw
ZqVK8iH9e4jzeo+yg3x8GYSP+4+Hs95Wdua5wHyU83A2PGfC/2l/Ncp5OJtK
J94+8rQT5TycKy8tKuEzE+Lw5DycN/wkqXLxS1DOw7lD1qCo/6Y6iyfn4Tzs
v/+JoZyH86mlau9k47uBJ+fh/KsJ5mT51aKch3NxQItCQWMhynk4q+ya1TBs
SSbKeTivmsDZz2eWgHIezmnzndsNtkfR3jOUZXBE/Gn8uTCUM/gzMPgzMPgz
MPgzMPgzMPgzi8GfWQz+DAz+DAz+DAz+DAz+DAz+DAz+DAz+DAzxGRjiMzDE
Z2CIz8AQn4EhPrMY4jOLIT4DQ3wGhvgMDPEZGOIzMMRnYIjPwBCfgYFvAAPf
AAa+AQx8Axj4BjDwDWDgG8DAN4CBbwAD3wAGvgEMfAMY+AYw8A1g4BvAwJ+B
gT8DA38GBv4MDPwZGPgzMPBnYODPwMCfgYE/AwN/Bgb+DAz8GRj4MzDwZ2Co
B4GhHgSGehAY6kFgqAeBoR4EhnoQGOpBYKgHgaEeBIZ6EBjqQWCoB4GhHgSG
ehAY+hvA0N8Ahv4GMPQ3gKG/AQz9DWDobwBDfwMY+hvA0N8Ahv4GMPQ3gKG/
AQz9DWDobyDOtH4dMPTrgKFfBwz9OmDo1wFDvw4Y+nXA0K8Dhn4dMPTrgKFf
Bwz9OmDo1wFDvw4Y+nWI8/5Jlo7OtUFkTNnm7ytfphLD/3CuJ3r90TELj3uQ
zE/zNl/TyyTz3s3jWCkWku/Bfj362ReJj27EsCiLLDLjcQ3fZA0n4tCpc8fu
xSd4YNzg9pmTSaZe7QEN6xw49/J2hPyVQKjonrTTj5NGvkYJ6hUH1kKpjcbi
1fOiSH/R1uKGEdn43QSLIA9ZtyAyfHV9t6xbPn53trHzI9vc28Td9u2UgNpC
/G70xhNLMtr9IFW6hH/ZwQL8rsAOl9NZ2pFgFVEiWjcyB7+7cYm6z0aBVMJR
zRz5viCDKBhJzi8O7CByRTW6y2VTSe0isfn2XQWoz9CSdWPrfWLISAXb386t
FajPpR8e6c3zrMh5rzOdzTurUZ9h7MbV1V8i4WiQ4K+qL5Woz+spS9++V02B
5cHLz5H4ItRnrWHMIlOTVBBTMpnaop5FdBWHi2lYt0P8pB9ObrtyyZZ+Dcuh
atmoZwM5sMTkeQG5842jrHmoAvXUcgtsLVmfQ7bWzBBRT21APbeazdJbwHIi
m8b+frWvvQ31XH+i4/wsTiaEXLnX42/XhHqWsLuCFd8WwDzv56wrClWop3Yb
e7n90jy4l3FJwPx8Luo5/G3NPK+fZURqkqCidR+l52W7pe8k3tSSqNoLRhHW
NajnlTH+a6MX1xPP4LpIH+lu1FNtl0zvrX426Zpg8nTamgHUk7XTYYbesVo4
02bzdkHCd9TTT2dCQLR/HQw5sL7Wt64e9Zxz9cDVAx8rYNcrdcUD3/JQz5js
X1ck7TuJkuykCd/UalGfUZNbZ53qGyDmSV+XnDg/hMPTZ+3kljn+YSXkUdSH
7odPJDk8fb4fYzWm6E7jnNlXUZCuq8biye/fmx/uFczHSnc519+weRiLp+dZ
jeUBJ/i64fcLox9rzRuBpycNH2DABxjwAQZ8gAEfYMAHGPwNGPwNGPwNGPwN
GPwNGPwNGPwN9aTNX2CYv8Awf4Fh/gLD/AWG+QsM8xf1pMU3YIhvwBDfgCG+
AUN8Q32iyzTffa+Khs4LbVErZqehPrR4DgzxHBjiOTDEc/yuzNmOeJ8bIaS0
XGyzO38o6e6c2G6l+JtIjtt9YsK8YNIgI3f79SGqf15dMz3aUfAGudnjOPt6
dC7Od+tfzjM1BL3g1pYvafc8cnC+y9qq3soPCgXtbdIXbW5l4nz3qt367vTz
UIg8skl1anACzncaDih/xdct9cYsDJ5de//ZgC+a23fqAa0rRntU1kaDsfaA
5LLgWJQv3F8nofenfoz+pD+71y6MLPovb/6Crb9FO/Yti4TH7Jgf0BSB8nWc
oQUz9rDBNGL3dM6FAPI0UMTWqJiPlew++fIZv3DYVDs1av+0QJSfs7k1w/Ny
ANhNUe02nOtNBKVHqhcHCrBEvjf5ZKgGwxR49k5EjpIX3VF52xjuC0fv2Jiv
+fKJXL0yulfdehjrWFl364TrPpDZsPd4oTO1TqQS9FPS9FwALC2o/FxR5YrP
H9V227+0OYLYj3LMfypbgnFYaPHH6WZiD0m/bu7mjzvK0S6Vt/eJXVgSAqc0
VLU6J5aiXXySnkiXfYuF/IeTQ/1k89Eu6jfG1JgqJMC8B0/O7P6Ygfj/WFcV
rTQnCXzYz5R9ZiYgzpOPFbcPlMeCQdzZURpjOIhn+u9xB2JaI+HjGxHfjVUU
ntN1u/0D9MLhZsnkU76unojPW1XFM7tWsWFjknPgF0kXHG+Z0K3xf/vX1+bG
TfjbL+HxPflY4ZprsX94o3HLNrkXVdj/j0tWN424kUKmXLNNfMSuQXy+yn+T
z7hoSwYCcndmX/yK+CiPPyJk+TIB+F9J3phZXIv4BAiwNDtNs0C+D/Y5TytH
fMSvNq8WFc+Cpds8xG+2ZVN56tnLnBzPTJi+8JCv2JFExOddovLBFfuToTqj
dKHXOoL4zBteNTF6bSy4aMu3jZ7jT/mbtfuQobEccByvMVF/uTvi88K26Qm7
JQRC9zvP8Jb6gPiELVC/9PpQPhFmqT6ZMK8Q+8xenKqVWW35ZKGA8aSCjhrE
bfTbb22nPiaSG8Y2L0wbWxC3NL4mx5PhkWDe3/ryimgbrpuYOZiPn97zhVTp
h892YHcjbhIRPu35GUUwe9ZUP8uKDsRNxKTv4V37Cuh5sOKhfH8N4taqlnf5
9YZiGEj4EGb+nMrvpu8mH6hTzgeJ5j1CF/PiEbfgkc/W5dhkgK7qgmXfx4Yi
breU9imJ3UiAgv2CWkvfeCNuJUoHoru+/KlTNRL5Jrs6I273zxqRRZv/+Fu4
2zRF93eIW39G4pF3Cukk2/K8v8dJNsa9kEsDnl7uucQpXuKRnmIsEZ0//lmg
0XdS0ud+80NJIXE98eP3uv+zPvjw8uMwqWtV5LN9jNrr09R6yt6vw4+L1NST
JrYba2RpC+K/2UxLLeVBBVEOdjri9vQX4k/jIRwGHoJ4ZmztEKx7UA9es7T0
Jl2l/PDhsV9WtqklEGk/6attcCTiebJ9naXmtGxIHRA1UYnyRTxLMl/mh8sn
Q7zm5bvd4i6I59TLzgslZ8TAsOmuOWtE7RDPpgMXtKPHElA9f/xFZaU14qnm
olsjmpxJZvrqbBpPvBHPh+nWrhpy2eRcSEOwbd8rlB8qFHYwbUwlIdsf2Rp+
/Iz5pXePw/khh4vIGLUL7M01EYh/4PQTx7Rm15J3Zy32HjxC8X+1dCuJ20al
ZHlTZ0T6dxd8/knuNjeT+CZSek8rwvAjNS+ydjoreFxqIAKPTkz7rRGC7yGV
7tnmUj/IQc+lw+oyOtFek8peHDVWGiCP7cvHXlAajesyND6JPPNVxqbTxkpt
8F4hWMVYqRzXX+bwe5m/28DHquk4MUbxVDHacY3odKvIhVVQfkp4trhPIq7L
NHocsGoYaIaEPevH1w8QtK/TMl33tNUVsGmSmLaRsxvaV+qnx/gC7xwQzCt0
3T7sLdpX8lXftluOKXAj1ANcRlmifVsHDNpLemPgQceMNxnGd9C+aRHVIwxt
Q0FafDK/w0NLzEeWfi/WWQlyYMK8XyLdXVfQ7jT7As++nmanS/7uj5lz/HGP
xVRrtG+EdmTLhiHVpNTpQ5xPjTfOI5odgWfHDCmZ+WE9P0nTynFzVlrkIP+n
2RH5pPgv9y1KdyQ5hvU7jQ03DmD8l/04075Jaian+/qvuM5gebSj2Ygduyda
yLHOiI9cG6sigvNu2aiI0L/93DFyw01nlSVT+xx27T15VqUWpH98eLuJ35/J
Xsjfvsp0PDrzuhA2njx6b7K9E66v0ewIDHYEBjsCgx2BZ8fNBcOqJPaGgSJk
nV/82oTJjsAwf5nsCAx2xL4Nbf4y2RF5NW1eox1pdR8+T7MvMNiXxWBfrO9o
dSLWg7R5ijyZ5g/A4A/UOtS/8ZnJH7BfRIvbwBC3gSFuA0PcZvIH7CPR8iAw
xGe0+6XWPR+XmlcQwU6+9fbCVF+UljeBIQ5jH4/Wf0C78xPN4w8cm0lCYqrY
8QzK7rR6n8Wz+8TM6UqfCzuJtadoq+KpX+gP7raSV2UE+uDm2m+1rvy92N+j
xWe0Iy0Oox1rKh/s/bo+H76+MhV6eDgY7UXLgziP7iU9c/XvSyQLNlvbtj4I
Qzz5b13uFQlJIq0Gccazx/khnt8XvRZalZ5BWq2WHbbpjkPc0hxaWSoZhUSk
/uSeMf8HH7HQK3VWaypI8NemYpdnVYhP2OuNz572lxLOmR/fthxoRxwOXHl8
66VrPjyeXTfzmtVvYOB1WD9uWfL+y+cpX6E+7Ye26JoOxI3G97CuTMz42A6k
Gl4/MV8QcIFaN6TxOvTni5aTX3kKxxKZU1+/av2pq3j45Ep/G/liRyK575nM
iVOmcPC/tLviwctMsnpN/Oz46VmIQ7LHh3zbpCwCsovnnHEqQRxo/B9xoPFb
xIFWFyAOd0bJ7mTX5sH3IasmsmvrEAdavYA4jLc8wvrEXwCxRzZdkREoQRxo
dQHicK01tHMj4RC+0gBrz1rKTxasie8e1hhF3uwVurhtcwziYJl+b9TFrmjy
INewYLN2CuLwaQZLpaIojuzIFju9LiYXcaDViYgD2XxL3vdPvK69rxj7J34j
DrT6EXH4eMNnf7peAgyZMCq8dnc54kCrKxEHe8uJbXvCUuFT8owXKlvyEAda
nwHHe37YlEOpu8LIWj1ftbe9kTjeH3MtrIJEwshJdsOkkrvxOF5a3xvHW9TM
L5l73ZeI2M1SHjcrB8dL61fguJbMYM/Z8jSIOOV19N0qDQaGvgfOU1r9jvNd
yfL87MUXfeDgphqBW5GUnPZ+5L13lN/wjzPjwJB0K7bt1mTkdeKCAus3WXAg
z1B2t/uBGOR1ivZFe0WvRMAm3ZPJeYXhyOs+xL5KsWkIgVdWl4dJ1VL19bSD
fouEtwXBCzmWmfgKqv9A0xP5m/GN3gj2GH/QXj065N4RSp5XfiZ52/k4mH3z
zGy3oVRdZqifZCpyOhryVFyGV7GpejZtn+jC33v+1K1Tj32xVKL6JElnyraK
bw6FuplvTk9w8UJ9st0mZ0rnBULio9BxMvJUf6NtxnXFL9fSIHz9ey0BAaqO
NmirWjbrQQL4nn/psnMo1WewOePrSF5Ew7Yv+8KfKlM4HLmv4Lf1CgGV30s9
zy/wwO8Wyk93//YmGPTNVvDtifiI391YfGhXol4OOPdt6PwkQX13242vl0rW
p0Fgx1VtP1cK/6q5RcufN8RB3Tkv5f4PvvhdF9OfY05FRcKwr1kPjAM+4Xcr
jQqvOswKg+BvP42JnCN+l5YXkCffeDVNxhs6YS982mS3nA95qdIb9+o549tg
rXyoSP98qh7cM/bJj7nDaiDoQf2+q9JUfd0Sphz4clIRSDZL3WGdofpjUo+r
/B26MkF8pXmK8fkgHNfl5+nJVY2JcHKx2VAluS84rtinMqdVrkeD/iY1q6Qx
TjiuBVICJ1vVIiAhzmzNanNbHJfsvFOdnq+ziITl7fnnd7ui/9P4A/L/I1uL
12YuLyGTjIVzK4cFYh23evb2elm/erLvXsO77IsJWK/9Gl+nkd72naxiSxpl
tU3H+suzpDEoZdEveL7N8Dzrax3iMz9DLn9ucCPI8PuNr7meSq3XKB7KzeUv
B127tHON54IRnwPWYVerpHKgUUH5w2plN8Tn2UWtHVNHpoCBpq1dkJE94rPM
6PWpzk8xsPZugY33zOeIzyS5O+Kf3Qm06rRNW/XpIeJTtDJ57u2N2bhvjYeP
8O93K7YVlqKch8Pk1iP7bBs7yNOm7UdERsZhPXunrPXz7AmN+DwPH5X4Iq0f
2xawkh+Yk95tC3Df4MDt2si4nBbcJ8bD4c7j49+sTCto+816wNsmrLWiNgfl
PBz2zKvOls5JQTkPh4MmuQu6xsainIfDK21trz0SHJQz+AnGYZo/AAMOyCdp
foL54s0+Zc19S//w0OHfS295VWK+oOGD+6NofoV1Ac1/gMF/gMF/gMF/gMF/
kLc/UDkSkheZQZzrLZO2Dw9AfLo+xAV7KhQQtzdxM9/mRlH4+G2TsNSuIkPt
SvNyv1J8iVbXI/8fZVXLtn+RQlayxoxfVReC75/JnvrFdl42idoYd27mRYqP
JfbHl55KLgbV+Zaej/ZS+5RGbe6SkDSOJ6u15gZsMKJ43ShbPqPCNalkbYiN
1NFs6j2fc14Gj/3+py7aqq086Rz1HueGmQIpyVEkaOLw6MZr1HvMN4w0eVwT
S142r15ftJ+qL8I3Wp+r5Esg1t0KPRW9adT610H2987kcLLLMkgGRCl+dfWb
1ALR/RyyjK3hVz81Gt/TbFSUGfuFQ0pTZj0KzEzE96y1zfI30gwnOiody8eT
DPQfWt8e8aTxVcLAV7GPQeP5hIHn4/M0fyAM/kAY/IEw1Fn4fhr/x33FNJ6P
/RDa/ML3bBf0GYg/14dxiSfXPz+aP+zgENxPy4vPtLiNdTctflK86988CAx5
EBjyIM5TWv8WGPq3uM+E1t8Ghv42te75L6/A79LWF4BhfQHfQ1uXQfk0ywG9
uaI+IKwf1BN7sZPMtlg7wDZyB60Jj6zyx4bB3Iad3gVhP4AnlzJ1SxMoqYK2
5DO+MiKdIKb8672iRhGY+U9LerHMBRTULlheim3C5y9FlX4XM8gEsE2wcvGp
w+dnH+fvqpubQzQKVq0tWJNLDFZL7FHUaCemLH3D04tyifTCpnz7+ZXk+3/+
WUu+T5Lku3ukmMwdZaptrllGqgKmaBkVN5L9mrcmPx2WTbRVY5+HTKkl4/UW
yxsVl5G22/yKPqbBREZG+3C1VhOJ0L5n1j9ACE1PHK91XX1wjnE0CbnT7qQg
1onPHxpQPnepIwNkRONlLnW0E5PT1aQ4MBUeCPHdmvexjEhcElk/0zUf9U/S
C94fY1lGNNX5wjJOZZCSd1JC1opdZOiZhOuxj6pJ/QQXkTXK9TiuyQq+WfEh
dWTqdc0t7ccrcVw3yg+qvI1NJ1V3RiT49PSiPjT8CQ9PtpnLzzrzKjAvP+q4
oqeK6Pxn369Q0PCq49XpZpiZeviUumAbOa/P721UXAUHnt7Mb1leAz6eDqLp
l4vJhv/8qhWMh7byq/j+yf8aTq1jJ1E4H262aZ3wu5F4Ch9+oqfSCDz9Q1d9
s7f6UklGR6vvjLGoAN7z+aOWKiX4VJN9G2bdmTiiHXh2GcdXM1Hz0ldQ+zDq
qO+1GuDpmbhvl5VlfTVMFsywkHjUCDw99xjaudx9XAFavT/HhTVkgcxOoa8D
Ax1gpHJtt+/6CpiltD3MQbAYePrT/Bn9kGZH4NlRsLRN4uq4ElBbkWq/s68C
9XFa8sjLwyEHZHLWkXecKtTHSmCl56KF+eA/QvrYwanZqM+l2yIqJqE5kLW8
WPPc5XzUZ9tdoYQ91hlQO3KFZ/biIny/4VgJocrCVOK726xvoZQ/8Z4in6Bu
3U8kJT7nDZcpIbMPuL7cdCaGaA1IswcGuknk+duxBjvriLJTZ5B0BzVfHCS+
jRnm+xU65/o7l48rwu/+uD2r6K1rGUQ0yKhL/ql/B3nCN/h++ke5kHsVuWOc
wOfxIh9471mZKKIn+zkfFB7k1LLvpeDzF9gVoHGeTQ4uqjgW94WNeh5/sOuZ
3C0fuD80YYPSQldSIC52RFFjOL1/hfmFxhNQTutfYd6h8QeU09b1gGFdDxjW
9YBhXQ/zAm28wDBe4I33jsC7ymFmDzi8fMTje1sv7ul3EckmGaNf9SXZU79T
qzdfLdwpkkd+L5I1fz0hE+MDrf+G6w60PhLmU1pcxThA6yNhnpVbc/lyxIVc
kCyfKbGpvRTjw34BlVda+0pg9FXXZvdddRgfLEbf+/28rxBinaZ9GnWzEOPD
hfdzny5dW0F6miOIjHA3US1S0gs0yqb3CXH9l5ZHMN46EZ9bP9fVwxn+Ezt1
J/BxePPRfvv2voIOVU7W/AHNnLZlLF7cOyEYb1ubsY3VrTVz/T2pvSzee2jr
obhvc+KXh7OEhZuI21miOsOhD3h60uIq8N5PW+/GfK3g/FzxrlUJWW3ncGzl
2BqMe7R1c8z7OT7943dp5pLZ9xO9fy0sxnhIWzdHPS03LFY+kpdD7u5Z7Xmt
rA31pOU1YMhrqP8Z5bP2f3/XXeVv7SI+lNpvI8FOfiI0I49MtF03ZPu2RIwn
d4QOePRoZJEGWT5/izSCv0eTnZtz8FJTDnmcvXeFrFYkidObcNyouJfQ7MLB
fPRvHwDn46UIY6uyJ9nw4I28elIctX94yxe+4j7TPLC/Mv4+uRPNnY/fYYnQ
EqkZckEkj+WZ9WkBNe+Kbj+cG97oDV/2WVeeAyrORA3/dFKh9Y9chz3mQgzF
n2nvwXhFex7Xs2j9CmDoVwBDvwLjBg1/qu/3r/4YN2j+zOH5c/Bk8iurvYgo
drZPOtzRgHxGUeXau9cHc2Dz+0fC0x+y4fJ/3/1J31eD/JDWB0N59OIc49gh
GfDhvMDclgtfcF+x8aMenfKBDHg60mr05Ou+0Fok/inQ6DcUlIjdFtFPgYyR
Xb9vDw3A57/K3NHZnZMOXuWdV1sgEJ+n7VdBf3iuxW+7ST4dLvwK8gxNo/bt
jzbp2uspkA6V16YtNz8Vh/5A2xeE79kRO+tU2/AE2Co2KtzgM/Ue39Adwjt6
k+BeqmfqU2nqPSrV18IzQjLh1UI5t/nLKDmtL4fv35Bbb7KxJA3aomqqdNZG
Is60/iE+/9BLUdv5RALUj2iTcyqknh8vMVaOJUzIunMxn3XMCSmdKfdlYOAn
meg85ni9PZsktZDZrfWROB+zfqyeuceGQyZZ3vxS+4XK+xKLW/yjTgYRU03F
aHmROBKdIxEzMNBOXMq+VeU+55CCXRtvLCRJKDf1LQ566udHVNcZX4hQTcJ6
ilaHonzWiDeLnMe7EeWnE49vzU0lH/oXnB0YKCZ7R2n2bVnhTxQvyhk6p2ej
fMRt3amrMixJ11SdXasz0gmbGwf2v8m+W+X1ClbdfrF6jk06eer+9/eZCVBH
O9eij7eOdj/6Db+HP9jzX1965Xwikf6PRzXAk6s/SsZcigDj/hffk8UzUC56
wzgjOTcI7pWr3c+cF0t4vEtIeLuMlD8HrObfT2s3TkS5mMmYCAlHNtw4edFJ
cXIkcbk5omRgoAeUvo9otzrNgdNbm9PNbaNRPjlJZ86TsiDwKjH2u76B2i+k
e7lcWkE1EDo3Dy89+iqA6J0UqRq4yccSGpVknLTVH8os9xrEjvchrv+9h58l
vu/+tLDNvjDN27/JYbU7aX0xKm/g5lBWvmucl0hfNMlbGXNepImD/hA2J/5Z
z/M4krmhYuoMk3i0u7z76I8mMgnk1qsK5YeLMtC+tPUatCM7fZxQplE89Kx3
mJZmm4O4TfPyeWDYFQ+LD45XbzuRhvjk7k6e3jk+DkzlpvOnfI9FHO52TFgc
fieR1P3a5Wi3IRL1FKl6u+JnUhrZW3++StKWyiNXhGyH7qnNJCGz1SRzIRv1
HGeifEC6JAt6Vdc+UTIvQn3GNbwcstgkE4JU+G8phmWiPibjSOfZgFSQatqR
YGkfj/o42J/J3VmcTubPnfHgyFwKtyqjTp2h3jnkZKXv15wkSp/LWkc/vwgp
Iq+0j0aLPctFfbpOtA+PelYOOQ5f3uu5VKA+mjXinlOMC2FM6MvdjYezUZ+7
1+Vv5rzPhukRZzymHaP0uVH+rbpDOps8eWUwR6wsGPXpWeaQnetaRM6GRhtG
TY9DfU7M1TeS06kmV7d1tQzNyUF9lEROr34U30me7xJwXW3ei/Nr0bOjQw36
qomTyNRKrfxhHN78CvUa6aRk3gikxV9/8acq1D953BBZvaZKGNY1PmX+4izU
/5jio/GPiwpAMshI0TCK8nNbZf96z0e55FfepYT3czxQfy9bR+2bseXErK9a
a+vxENSfjPa0ljBvIPeftDjtMElG/RM7H76Y9UCIIz+mS2bXUD4OT/+kivl1
tkljWDe+2OmedpLl8OLAkEvCs8bv/fZn3oU8l3pfgvo/l4u1PT60HpYZ3tP+
dige9e8wr5WQWlEKjvdvjfVdFYT6t4kbGN66kod9IZ7+q3Y/LOpurkA5T3/Z
6DJbmz3NKOfpXxmSf6OyVgJ5O0//Ny8Wrog7txr7Szz9Ofn6rq9W92L/maf/
ZcHZLB/1BpTz9Dd0PyAR0VuGcgb8gQF/YMAfGPBnMeDPYsAfGPAHBvyBwf+B
wf+Bwf9Rf5qfs3h+HpB2LsvQpgnudcg6KAgORf1p/g8M/g8M/g8M8QQY4gkw
xBNgiCfAEE+AIZ6gPt2TRE46+sRDZ8HNcc98/ICXX25bWK9/UxUFS85OslVb
5w68/PK6fe+imy4RMLX5iuDYJx+Al19ocRsY4jYwxG1giNvAELeBIW7juA4b
hW4bmBwD4/ISAmq1AnFckopfjO684cDj4TYGZ7u9cFwbrEZZqL8PBXhjvkvO
1BUY8iYw5E1gyJs4LnHu70+ncn9/yuMhtLwJDHkTGPImjpe2ro38cOHmmMoD
QRzwG7p33KQZQYgDn36v0AjLMJCf0bnu/RwfxOG9t6FGpwEbbrZoPF/f8Rlx
oPFJxIHGGxEHGj9EHKa1euf/DHIClyc3ii2O5ALGN+7vRtu5vxvl4UPjY4gP
jXchPjR+hfjQ9iEgPilbyuaPOxgKearbVozlUH6yJrHJK+NPHXjNYO+a4IsU
Pp77MoZeVA6Ae3JmkdnS7oiPV/j4U4kT2GTqlGdfA6zDgIFvAwOvBgb+DAw8
GRj4MDDwYWDgw8DAh4GB9wIDvwUGHgsMPBYYeCzifNC1IFJcPIKch/KLU+dS
9Uty5ISr9iSCCD742Xn2ezTyBIO23Aln+MOJUUNU453SBOQJikWjVe90h0GF
kpdKwrx0zPsnenYMHHGKAB2LGZsmaSVg3rcYEAgYYhIO46xGz3nwOQrz/piW
VelCutEkutkgYkkaxQO7Cu3yH1bHEO3Xa4M9EinetSXWfbZiVQzZA6aqC+el
oT5OkiaJRlPj4PueD0uCvmWjPqEm8UrHC2Khn+MR80g+FfW5MPFbSlhMNLyV
GH7DbCLFk2Pirip0xSSQjyq3X7/eTPFk7ZAtc7Nvp5B94jNVguZQvPQda9i0
6alpJM/z5+7tE7NQny8ODi7if+rTzRGjdX0CC1EfO0//4v0kHdoSo80WTKZ4
cnX4COOKd0kgIrpSUXYqxUuL7uemXRZJJ20L5J9dM6TwcVjWZausmk2mjnDY
GzI0CfVZ1ax5UHJoPlk8F3bsqaF4qa2NoZrF01LwEvHgO6dM8WRa3xv1yffR
lB4XngkLGj6VHUmh9ImuOKWmEZxFzikUblXrDEF9Pjlet5uiVEi8Qpa8zDSn
6p1nHwWetn2vIMLxX22efqF4+2eRtaFfGr+C27apu6/ZU7yXti6A+rTu0N19
uT4Pqi2vhfm+p+x11Z41S0Azl9iyrs0+u8ob9Ym9cVjzpk8ZmTXl9yanaoL6
pNTmtnw4XE/aLRTTcjdloj6vJxXvjrneBbFGt/O25pWhPvvXRoqY1NSC+hu9
9bEpyajP2XJ5x5CUYni7t2Fq/OEI1Gepo7reR5M8sqOhT7LilCnqs33i1LKr
xRVEYX2QYOJQO9Sn986C7xpNTaRugtnJynuBqM/H/T1arkq9ICsfcH3kgDfq
s2OdeApRaIAV3wXmlpt/Rn1s58XGyweVwWSxfm9rLSfUZ0JS/cgZkbkkZvLM
o9cSPmP8nD/honTM9grivOy79JT9/hg/3QVd+ZzmNRGRoMKwCLUojJ9Vzzwq
d87shqxc2TU3RCn+sEZ/GCv6ez0oPJ+XU6oShXGs/qregLdqGRxLjBiwlPDE
OFalZvYm3CSbPLblnz40kI368NZH5gyuj6A+VrnqV3em15KElY1Xnoym+IzA
CwPpgsXN8Nr1VV2/LcUbbwxboTx3UzXobV7XpD+N4jN6035ngncRpA3XztJT
puJq8fqZW5bsyiBCHIekyelU/qX1V1Ef0WixBNeGMrKKT2eObBbFG7Nd9X/Y
FFfAvbAcQ37RStQnSV9/u1VpMRwuG2Y++yLFG0VkF+46ZZMLIv4SNq734lAf
j1a1pu/rk0iux/wRaaspvsfilOssepdBIvj2nNWpofTp16tO/ZqRQ8bOLFCz
2kjx8EmiClfax+aA2ogPp8v3F6M+s5ecPe2wKBvWS1y8/1iD4tVjgl/v36Mb
B426GybeavTHPFJQNGrZ6J1RcE2gwfie+xfMI8qVyxK6NkbAqB+LYpPkP2K+
/j3qcruBfQy52XQ2KU6Q0n/Wtb2VqscTiJT0qJuTxiWg/pOHf/gdXptM7p3d
fFJybybq31B6d15WagJ8unJQ2F8lF/V/HJE25ItGErzMIBd3Bqaj/quPd0lf
DEoA4andGo/ZFJ7fGw9Jd2ZGQUj7aOm2dIqHbH739PVDQQ4EGf0oHq7vjePi
7Blnc2JFKNRovhYIqKD46pLyGL0oQQ55mcP5OqmM8pOBHY8adeoiieG4WcuF
D8fiuDKHDY94LBhNSBL77oKYFBxXU1ahxARhDgiOZqmdfUbxLsEXl6YUKkeB
t7p5mcmHJBzXQ5bhQbd9USB+ccNDpXsxOC7a71WRD9TqVHOejCRwpTmzZuok
ipdu8dossfZ0KOycvtF2wnaKD6TGj51eMRAEuZH87L5pbhQvHa17ULWCTaRe
/9iQ8o3iXZU/WrK/3AghwiXdLT+fR+F4D6UWJcrahhBr8vKlrnc8jvcZp2QF
2zUA5FNvty2ck4Tjnf+iatbCimAwLS+IqZwTh+Pdn1Z6cezxUDgX7v5yg2gU
jpf2u10c7717zaMLdgcDka9ffV+fsu9l9vKY97OD4HDExpf2htR4u2f3XTrz
2JPMO7/42ql7GdifKe05s6hb2IcclZUzGAjNRLn02Y6kh6FW5NuQ3ZInZTKx
L9pDlphaaL0kmkPkN91ozkb5ndCE5Pe/beF3wSXVpgMZ2C+t5XwtinnrCFXF
RpmjLXL+Zx+VhhvGfxqfx+df239t5Kxmk0t+3y+tUclD/TX1R8Z23AolwVs9
HNymFKB85efbFw/b2xGL78+DJ9zOR/3n+LvfOBTjRPSaDiW9mFaC8tFvxNLu
NH+AjuvRt4M25aH+fMljDUfke8DLb1IVMxVKCUPdgXrS/B/HRavj8HlafMD8
KOPiOnA6OobwK6wccWR/OY7Lz/GJUO2EBPLFbMOSm8srUe5zUZs9VP0zeTzz
kX7o9EocV0uKTuibVz5E6N2sY24HalE+6QzfinsbfeDB1WFhu0ZV4Liyizb8
qFAOg43ptc3v5tehnEl/WhzD8fLOpyrlnu/Ke54W3zC/0+I84mC5p1ZM1y+b
dBXV3PbTrsfxGhoFjt5sk0/WFG7Yvvd3A8obaftneON9wt0nE8zdJ8OTP0ru
EN7cxYFNdc8OOQxtwPEy6U/LL6g/LW+i/jdUxNgyh2tJmvwJ944dnain6eKF
EpzXjSRq3cGJQo7fUM5b563l7p/h6Tl9V4b9kWslZJJHo+v2rfzYX329ljT1
j86BGGGN/Mk/ulF/Ge45bNO5577y9Kfla6pvXJm3Uq/nJ7nQbRfYa9eH+oyd
wie9f9kQjty4V59ZTf0oZy/bvr1hWQsJFzp3qnHZSNSHXW8+dL8JP+dppJ/g
Hd3xKOetw5Zy18d58rLIhS/LD8pzOq9UOowZMxvlvn3bXi6XHcn6+miV7bsu
aeynHRFJs9U+3g4r7+3+ZXx2OMrbuOcoynHPUcS48S/vxfhMuOcBRnHPA+Q9
T7ML1ss0O6KcZhfUn2ZHrK8nbp9RbXGoHmraO4awWvlRf5odse4O4eoZQdOT
VgdhPqLNC9STNo9QTpsXqCdtHgHDfEE9eef+KXLP/ePpSasfUU9aHEN9aHEP
5bQ4hvrQ4h7KaXEM9aTFPWCY76g/U/+NVudifqflI9Sflr9QTstHqD8tf6Gc
lo9Qf1r+Aoa4jfrT+gnAkH/xeVqfAcdL4xU4LhoPQTmNV+C4aDwE5TRegeOi
8ZD/2Q+k9XNwvEx9LRo/xDo3s/OLWfTi18Qu+Nn9jje5GJ9p/cn/yRNo/BDj
MI0nEwbeiPUvjT+jfJ7v1ML54mwoeu8xykgqENd/eXySM8gnUb5D3zZkzcNw
eHtGQLhSIAjlNL6N8vblBnt1VwVASEVrhKcytY5M46UoV8+8W1O2JhhO35B9
/fQiJafxdpQHrhRck+biCzl9kisT1lHr0X23Npl8LPaHVXM6a7tmUnIry3cb
FzgEwMQn8Wc+ClByGv9H+cypOYceJ3HIyaGC4iInizGvHdGyc0nwcyate8w/
5/GVo32vvrkgN2eEN4g9SVombVX2P3kRrV5Du4xTUxE/1RMJ03cZ6O8RpPCk
1XEot5M2b33rHw7zCjQ/3vjhjfjQ6juUx0je0duQxobipK+wN/AzjpdW96H8
Us505959cWRVvs83R8MKxCFIIWp0nLcHee/vVbtUqhpxeNd/ambx0zQSudha
KO1bNT5/ft2E1t5RAeTX7WdH5MS/4vPqK8r3XquLgMTA+6OFLOr/J7/aq/bi
k+ujDJCQbjkvm0f15cKIqrv5lVhwdW6euMOK2s8g/m9dj/IfT+s8muZFwpwO
sTh/AS/Eh1fvXx2s91HeYywz6YRgGDgSMwtTSQofWh8A5R1iVopLanLJl4Bq
k8dvKL56OvVWd595LjF4/cTs5NuviE/G+MPrPdcRcnfjxHNe69oQn3j5YnnP
xDgY1d0ZsfpQK+LzMbS6aa59Ark7w7G0XKYbn++O2lbaEJ0JKkcvjny35Rs+
P452zj8PT1ofBvGsfrGpIPVHAvDflxsuouSLuI08fjThdlIiHNeK3HO71xvl
qTdzRq44FQ2tqq+STXw/I26rpVyc/Y/HgKv97o4saRcKZ9Udz/n5COxcG+AA
XU6I22tXnRVG5zkwLUbX7VHwO5TT+lQYbz8rWei2uuYT+WTnSxXPE7CveI8d
Fu+zqIK4yl29GZpA8W213auH382qJ9cdHj3UP9+F+FtGbvSU/pVDmk4sSB/o
/Y140vgkrvvT+CfKabwRcab167AOovFSfJ7WxyMMfTy0F0fx4Y5H7cnQoTm6
Xn/GZ7SL0XNxN7H7KXDwlNWBq6HOKHc/sNjr5LpYGJMTI9Ba74h2IX6BJ906
YuHMHdXh0gZvUP7U5McM1SAO7LUauVDi0xu0y8iTL/MX7Y0Eo6ImzuHXz1BO
63OivWj9WJTz+pzxg31OtBetT4v2NXb5kTbMro4IRx6ftVOZWneg9W9Rfs4P
rESv9JNJstXH8gt/o927v6lY2lztIR4tuo1mqmM5DHUHU52CcksNuZajse3g
mRd7dbNPJdq3g2vf8TT70vrJ+Lzgf7sqxFiOr//+SwzjPe8S0Sc0zqkGllnb
CNadpepiWv+ZMPSf0U9IXlT8yshUmHdDK5bP7gX6w7kki4c931MBQt6o+H59
gPLHdxQeiZ2KgxE+nyyuzX2G/nAppbP6xOc4aIl67qOVboJygeVs+RemkfBU
g/UsSfgR+kO4s7SxU1Ak5Fif1ZOZooty2noB9tnUbqXnhblWkCEz5JxjvdzR
7rR1BGBYR8A6opmk2eXYiHFG3rWb2jv2F9UXGlAdvnXaWE65W47Suiiq7p7V
vHTrEbWFnMrcK8/Cqhajfb+6uU41eS7Nyt1y2HvLIXmc7wYG1a9CelayPs/p
1F5uIYHy1UuaEt5O/w4eEpMHStWpPk9AhM7OVV5fYYjvC72HY4PRXrR1DeTP
u4t/hAbdLQPfzIrd+9Z9ovY7/bvegX1Cmh2xH0izO8ppdsQ+Ic3uKKfZEful
NLujnLY+xWRftCNt3QoY1q3QvjQ7Yh1Bszsw2BfrcZodse6m2R3ltD4GvofW
90A5bV6v5s1r2joXMMQNrDtofgUMfkX1k/9dv2PyK/Qf2roeyml5BP2Hlnco
f/s3j6D/0PIOymnrqugndvodSnEXSsnPZdlSEQeofjttvRUY8gLVh/93HRbl
tP4V+gmt34Vy2v5GlH/mnqtznXuuDs9etLyA9qLFc7SLrGXOSj6VYnDe6mWX
zOIg/rQ8i/OLlpdRTls3B4Z1c8QtzOvmjvKT5eTxrYWRoXGtOK5736+6RiSX
wm7TuvF39fqAgX9iPa5JOyeHhwONl+J7RHN9wrse1MB2MZHvpiOpdUzeenfN
4Ho3MOxPAIb9CTguWj2C46LxatSHVqfguHi/V+rhnnvDGxetfsH3NO26Jrhx
awH46Wy1P91UiuOi7SsAhn0gwLAPBBj2gaA/0+pWHO9O62synZUBYLH/yBW3
yCrUk1bP4nhdaOfb8MZLq3PxPeoNdeZnLFPh7CxW6qnD+The2n4bHFcCd7+N
wOB+G2DYb4Pjou3bx3HR+i2oP60PgPOC1jdAOW1/O9P+K+SrtP1dhGF/F2HY
34X8h7aPjjDsoyMM++jwPbR9iYRhXyJh2JeI76Ht2yQM+zYJw75NfA9tvyvy
It49O4RWH4lHTFuxsDUJdmkvnb+twAO/e22x8Tmh4hhYX+W0UnvMR/wuZ++G
p942HChqfatfv8QOv0vLF8gbF9ZePyd3XYzTrP8m4XfeZOR1vPprAo2fb5JR
2n57TSpMAefduosdUJ+HZhtMTi2JAwUTBVvPgzaoz7N9Dnv7DSOhT3j3s8kf
n1B1q9zw4Uq2TUQmtKbcyDMc6xGoLXtj7jibpTxrSJxBxlTEZ8M0FzGRcWm4
v5r33RuH7uhp+MShnPddpdogkTx2JMoZvgsM30U+Q8MHeQsNB2DAAfN4is6c
n6mGZVC6fBxZr03t93hfbnC3TTYXngTqjDgcQ+27oOULxJ+WF/6/+og3Xt7z
NDtivKLZBec1zQ/xeVrfABj6Bhh/aH0VfA/TesGYGT7rijKzyG4PU0d1O6rO
9Rl/UCnBNoMEyW5g8/PV/s/1SivuukwYbb2y/uTutZlv0kHpaO3v7vRO7P8E
d6kcW6OdCPpdZlPuqLWjfPyQ6ckWY0qIbY2aZp8f1Scx4a5PVdLWGWn7fNAu
MUJrTmiuqgborRJ9xaHktPUptK8Wg90/LzisKqlbSaa1D60t39JO8at/98th
flm85Ov4wsZakNfsfPRrCyWnrW8i/vRz/HhyGm7AgBvKafsJ8bsLs2NV3dsK
YWvUwiTRBGo9kckfbnF5RTeNV9D2T+J72LoPBN4dSod1akIfLysWoPy10Xzr
JXNrSMnFlHvv91L9RgPDSutXT6rI8somoQmLKX+m7bMiDPusMH7S+n4YB2h9
QpTT+n7Ufpt/+4TU/v9/+344T2l9Qmo/UuLVm0P4tPB3Ljz/lFoqfb6uPpMM
NWvJviFK7Qul8VucXzReR+3T+Je/oT/T9pWhXPaGc0FMch68T1x/iG809Txt
3RPff5fhuzS+jfOUti6M+SI4qD9pzcJKeGbxsqNHbgCf/8TtU5XQ1tNpfU7k
S7T5jnJavxrnI62/jX5F62NT67D/xjdgiG/4/MXAT1F7nmSSe6oK1xKCwql9
tv+OC/N4YJntihtr8uAORFpOnEDtfxu6/TnRWFoAR5tkJ2QZU/vHaHwDGPgG
MPAN9MNt3woLd18vIR2ly9uUxFvQr2j7mfG7a91uLo5TToHdM7q/zLziit/t
ffE0VvdwInwqWm0QYuOD8nmXLkhOm5wJs1a3eV/Ko95DWwdB+V3BEJlXqWkw
XNjJ4WAwJVcJ+JX/yCURXjcMWzi3g8LHNe7dOF3nFJhp/W6l8XXq+WPmGmNU
+rJgj9WGgy9tKLlF6aegAyciiJ6/7NonC6l7zyW05+b//X1pd5XNxidl1O/0
9SZ/Uvx7/4G29i0JvopYPC8ibFdiv7d1BBFrqmyZlp6C57btipfO/XvunpR/
So5DVyaeN57b7TCPb+hHEnVnTW/smRw8H95JdFH4jmFvQatZY1GBdw6ez1AV
8e3VklUBsNrnDDvteRaeL+Fhyv/i7++iHybGXC9Zn4bnSHyUKVTQkeSAz2KV
bwVzqPvQc6+0SbCaIqBff239vmXUfeiL1vic2/YtFLwXPBH106Huj364Rkx7
tTkbOHXLFB/FU/d0W4WvSHm1JgB2vC/tMplM3Ye+7OyPg04y0WTrysaG82ep
+6AFFO8oaR7ikGeXTDVNnlP3oZfsTBvpuzuWSO39lbm1LgHxFHlw+GKhcyzx
G1UpLHs9E/Hc0NVx/ZZXJOE8d1fctzSfuu8gX//W+4deJDfkwSxD22LE88th
peeH9Z2hQz8yNnN0CeL58JZBUMl8DpTcqCnYqFGAeLb1COR1qcXCer7VayXt
sxDPjFEWBzz+1O3rvwocmx+YhHi2Zv9uuGAfDa5KqcEzQ6IQz6Nzn8kPa+HA
Ad+5+1V2UPeef8k/2XXaKQwkflbH2iZRePavf3qwcQIbfg1cVO7WcUc8+0MN
80/u84eEI5vST+77iOch6Dqeqj1WH0/Y2xKqxOdT95s3tfw+c+FONJnS/OaN
9hEK5wuPVny9+SiFNK496fGIJCPOw/elnAi++idOXmrXUJyfizhPHxGoMk7H
F9pOX03IkKxB3CRUrUR2rUyC2Z1a702OVCBuy6bu+1V3Ox3etWU6yv3MQ9x+
BV7hF9NNhRVzt1ybu4q691lLw+KMTkECNCh+eqgfTd2vnd+39OSPnTGgVdk7
5foBCrfPMktt+R5zYLpEv528DIWbQ8St+v1jQ2Gs/uY8b7vPiNup7gNy2gcD
werdUKvoj06Im4XThSujLFPJQlP7EqF+yj/53+psY9fGEc0yST12bSDitoXh
fJhg7rkiQrT7ODwY7uMQ4Z4rcpl2H0cy97ziO/T7OGj5lIfzOO65MTG0c2M+
/XvOHuJs9e95Dohzxm2bh81jE+BN6tu3y3rZiHOY+9NDf+/Zzdpg6yF33Rtx
jpmyq/RJXzgobahiq/u4IM5uBSWFGzXYcGdR3KOS+Q6I88d9sV5phZkk5XNt
wenNVFzVnnNz7t/720bsDXT5ex8Uzvf6Vav5r1QSrwsuGx1XUPdrOOduyT6W
UUumaurf+nuOHQ/PJ6P405pqS8j3S26VczV+IJ5trzYd/3vu/Igq37jQnp+I
52HztNCIgloIfmtT+/RQO+K52WPeh7/naz94N5b99xw+Hp7fvi6JnR1aBE6n
ibPbbQpPVUODgL/3RVXd1Jj09/4oHp6GMV+M9pUkg6L1u+BD7gGIp6Okib+C
YCzMvnB4rqGNO+LZfTJtYaYWB2rfXrzIuueEeD6c4yyf0RsM+1XWS2X0vkY8
vx1bJi8bnUO+R+W8s1ILRjzHHnkuXLe7nIzJMTNO16Pip2dv9pzmHQ1EzJZ1
PHpDMeK5J8fe7W/dQtTs9/+pYxDPLWVStaabu4mXdx0LCoXw3gSBEd7bd65t
B6FL2Yd3nB+B54tm6RcE2Tv0QEKn3a0/fxFPw11bP/ytT4Z36T74wxMRz7r2
NatsZlTC5Ql9z/JVUhBPdaHcCeuN8mBrgevIMRZhiOeDseuq+5amg6Dl/LdX
5nkjnipxuV+08uMgxIbz2KDvA+JZuCwlffehSLhuGG3suvUN4jkv2/38882h
8HFSQduIRdaIp0zjXjXzhjzC3ykU/bHGHfHk3UvePGdtwAuXe2E8Oe8e84nD
A/L5+C6H8eLDPccB/+1bq8iMIzFlR6+FIf4dK13UhALaSY9bPp9wP5W/Lp4x
9/idx89ZFXZ8/MTr7Yg/7x7zBveoAj6JjDCevGBtutEnfmmOyz2Z8+MFZNEu
5sXm17PaxrGGXlO9kd+hiHaRit3eGVQ7jHU9+HxoUC1lF9795qP++1sQxpO/
ND9kaabaBdaT5pfaXKXi9oZ7344cP1IDnIPCoTOSOWivNsXPVj49BdC45Z6k
XawX2mvjk13r1BMy4M6Wi7uPHXOk7MW999xl+MTsdTFXw3jyiIZX6+TEE2CE
2fhnP0bZoB1596FPmaC9j0/ibBhP/iimfruCZBR8E1+1iqVrgfbl3ZPuuvk2
Sy/1SBhPLvzawvPFtDBQtXkw7nDM7f/P7r0d/9kd7yfVbKgy91qXTnTs7N94
rXuP5wJd4dpr+aC98LypCVycLw/ijOel83C4PogD0McrODheoI+rcXBceB9o
N3e+dw3Od9Tzwey2vX9/V/BS+O96nAfqeeDfeY167uLOa//BeY33ZYhy57X4
4LzGc3F58ytwcH4BfX5dGpxfqCcvzicNxnnU0+mujenfc23MZkU0PvWj7kHg
xX/hwfiP90o4ceOk0mCcBHqcrB6Mk0CPk3sG4ySeK/X03/yO+gj+m99RH9r5
b6hP2L/5Hc8N8+fm96W088fGcvP7Ldq5YekM9xHQ11/w3nBufo+n3S/AO0dX
h3aO7guG85qy/83vQM/vGYP5Hej5fcZgfkecPbj5/e5gfkec9bn8M3iQfwKd
fyoM8k/E2eRf/ok4C//LPxHn1K2B5j2zUsjDppG1khlliPOP+KrWBzUhZFdM
4dsfb6sR55lcvto+yFep+9m5fHXOIF9FnFdy+er7Qb6KOPdz+erKQb4KdL76
dZCvIs6FXL6qPchXgc5Xpw3yVaDz1TGDfBVxNvqXryLOy7n1lNZgPUX587/1
FOJczq2npAfrKcR5JLeeChispxDnjdx6KmqwnkKcJbn1VOFgPYU4e3Hrqe7B
egpxfsytpyoG6ynEuYtbT6kP1lOIcza3ntowWE8hzm3ceurTYD2FOBtw6ymd
wXoKcfbi1lOSg/UU4jzAraf6BuspoNdT8YP1FOJM6wMgzrx6X2uw3kc8I7j1
vvhgvU+dJ8yt98cN1vvAUO8jnh+59b72YL2PeFb/W+8jnl7cev/RYL2PeLpw
632/wXof8cz7t94Her3vNVjvA73eJ4P1PtDr/e2D9T7i+apM+nrHDl8YsqHe
V+Ywdb5fu/98KzKcQwosGhctl4olXQbSmxU1usmuMYdYWcMJcXcWK55Wl0zE
1KcXWCm2EZXGcMVLvmyyfuVTwbKHmWRVlJx0cWA12XK0Z49spysxeb1rb/XK
HJJ6eYa3unUGkVn8/rCuhzXc9Hsb5ZeSQy5scoPnisGQayt+SDLIDwK8HYxm
dGQRc/vfW60VSyH0+P5TESM5sCgotr5FMIFcahHaURzYCe/uyJFxiwkcMtW3
7a2MJIe8R/oHGvXCHseE3B26YbBwQ/HykftCyCbjMRLWiv0wPrJJvfQbG6z9
dLoarvuRCW9FQwJbhrDWWLuA5IcAeC7aOM5mqgd51jfypGKiIIuvq09Y7Ysv
TAy/37ZSzoUM1xUbb/1xOGuRgs1Y5fJYYiifrREpl4HnxKp/Nur3XhxNHC2b
xsmm5uL5w7+yfPdv9QwgKsaXt6smFOF5nus+BP4Ys+IZlI1rDi47XILndu4+
n1T5pSsUTgStFT53rxDPHZ3yetNknVUxMOnK1R/RV2II75y9jiPayePsImFk
juWFIcFhhHdO4Mfhn2rmVYXD2u1Dre4e9yfNmqKCGtZDWLUnarNTlIKh2u1B
2UwrD6L/H26CrOy7ySkKKwIgNT+uMKncGc+BjF/8rGnmwzSy6vGFX69HFRO/
/857rCPrNWMu3zKIIZoV5RdLNatI2Ko5YooaeSSj4WzszODzcN1Ov+vq1RrC
/h3cN0fjNuzYEJd7f3EsJFT1XZgwvorYFve2BRrlQ9++lYoG3bEwb8GF6tSX
EeSZivjq4sC+P35+4/Cp8kg47ygrkB/tT8L4RTWLlw5hHdexJK9nhsPnBz0b
hUZ7kIVHRvarjxFkWT/6sWsVHxu+/Ry9/ULbR/IpdrRnYMsw1kFuXdDIrQtu
cePDyXMm/J/2/71fXP/Da5uXYa+48eG6dOLtI087iUlW+fXAHx5he7nxoZTL
82u4PP8uNz6s/ElS5eKXcMZXq35ZOWpseBg3PgjJGRT131Rn9Zj3lfWPnR9u
y40PIlw+L8Xl8z3c+HB6qdo72fhukBV1ll0xyi8sg5fXmmBOll8tjGqZmTcm
/U1YNjc+lAa0KBQ0FoL0BuMclfAHYeHc+KDK5e1uXN7uyo0PwOXn07j8XJ0b
H5h4uMER8afx58IgcFFlXPy5A2GLufGht3d7aNHMP/W+2O64+6nB8I0bH+q3
nJhr69tKlga0/qx6lA4S3PgwPbJ57dBYPk5FQEyC/vImWMOND35Tw2/aL5Xi
WC+UizE/L8nK5saHaYE1rqXrJVgDw7QMN6TOZB3gxofpI38v8+8TZj2+P4/T
8qAPbnHjg850Y5fm8lr4PXeEG9svCi5y48Pyu1NnCR8ugpFrcxL3qvuAHjc+
9Ob7WfU8y4S1EcYtw1ycYCM3Psg1uboYCybC78vDEu5vfwnjufHhwldtD50/
7/XUCLysnPQYLLnx4bkRp/LtQBgo6XztyXl/B4Zx4wOHe4708sFzpOEYNz5c
P6Z6x9Cmk+y/oSNpJN8MP7jxIbKtMyxWtZdYbTZrNOgTYKlz48PumP2jzXuK
IHTB8+ntF0azxnDjg9jdNiPntb1w2HT7MSvz3zCCGx/45xTrNCcUgNnS5+cb
2BwQ4MYHg/b1F5/aJsD86cnjjY+6QBM3DkTf3TLt8pMoWNVUfyBk9DvgxYFv
96vS5reEgZ2rstPKiueQz40DqUaWw6941pDToyOFrX7Xgj83DiRNP3VDM6Ca
2P/adev4im6I4MaBmCIQe7YsHVYVa6bdGRcCltx5bdY3eVWeWjyo608Lrljh
Dbx57bvIztCwKhJqfLYdeHrIGXjzel+f+iX3ZWGwpGfHiRUP3wFvXtPu58Vz
ZVO59/N+HLyfl4uDAMuGez9v+OD9vHCFe66sUJ6Hyli3CMi+cH1bvKcX5CaP
sDbaL8B6YLposdnsYAh1yTGc5OYCG+eMVileOozVKCzLf1cmCOSf5Nmt1/kM
EypHPTYqHsr68v73gdphEaQ+1GQTKziV2NcpzlDUaCb1enmmjs6BEPP/6vry
qJ6z//+okEJIQkyjxtoikt2NKaHGxIQsM8gyKMZOtuxERJYokpEtJEuLvOu+
ta+kRaVF+0IkSpbw85n7uLfzfn1//uqc5+nc930973Nfu+vn6pqm09BBLXJC
VhSTec8m9AztG0sT+xotOV6ZLubAX9+zZZrb1yByyk3zZOLaPLoOc5vTBiya
b6763d9UaWs7t/YpXYs50rIFM094qD6i17SfB6yfnEa3q2qfDVnxlmr7z90a
avuYmg1X6afvm0P77O29KWRFNT1D0tyWf5ePveY0TnWfXUbzk7fIrT3vUdPx
rqmNxXJiWe6efbFDKb2jU++RF/KELBrfUNOwKZX8leDucLt9ATXxVD5gMKmM
nKkYc0TL4jGJ/hS7RCc4gzI815Jo7LufzPbd097YZxEL+BTA9QCvpHOHrj+e
Q3fWy8dMcSyiKzEPisP3AL4c8N8DQl7nW2VSm7J+bSemVlMzzPGeDPjvgA8A
fO2OgfNNLf6l1p2/eM1+U0vtMD/cQAJXApwsq1szUP6EhLnsfX/v7Euqh71O
EwAPAXwA4M/C3t038Mkhg4KOW7j8XEIrsCeCw40AfwH477Vhw33Nn5J9aRuV
96/JwpzwNwK+G/B5gN9VTXmR890+sLuVPu3w1nTqive9tvziH/JtpVQ2xyFs
xeIS2g/vmwI+Xcv4FPOZK2gU+PQ841MaCj5VirHRO9YYT9UjZ9QNeP+ZNoAe
DDVO2ZYGnCTTT51wGrWqiTp8ZXr89uqVpenTi4lR4bKbB7Y20pugk0KTRe9b
9i0jQ6aM1sz2r6fnoN/14v1P+jmVk0UX/GOvz6ymhqCf7rEVqyduKCNd2vh9
mTy8nPq/Ux3uaVBJTAa5tE/fU0SCfOYvma+US5VAVzMTez/pubiQhEV2No/p
lkVlstZbDSbVkYpqFXflRzU0emjWldbf9VIZ3+8AebuEyVvMwy+nwZC3x5m8
lVtA3u6EvL3B5K3cDvLWNP2vyx8GviftpgwzOFnU2mIz+HFh8nJT2xX1ZP/5
Wbc/2deRdeDHW3iXKPYuxADvshV92Y/Rl+2H/GxlNMO/FsM/yQb+HwDPJgzP
5B7wbAB8zmf4JIOAz0HA23WGN8xhriUJ4DtbxneE82Mi4L8DzvnxBfhuF+M7
sgJ8x+G7AXcC3B58Z8f4jgwG300F/A/ADQFfBL6zZXxHRoPvRkvgWoBbge8i
GN+R3uC7SYDLAO8H+HPwnSnjO1IJvuNwE8CrAZ+qyI+E86Md4LsA5/wYoShv
CedHHcjbIUzeEs6PibCfxzL7mQRJ7OepzH4mYeDHIMjnn5h8Jm9ADytfMLt6
O7OriR340Rhy24rJbcL5cbaivU3Ogh+XQJ4vZPKccH7ULfYeul72iKz3tb1b
4fGMcH70hZyPZ3KecH68NLzD0pvXHhE/3Y1Xekd8fwfwo6OiXiOcH7l/5M/8
I1IPfmyCf2TN/CPC+fEv+EelzD8i9uDHq9CP3kw/kuXgx7/gNzkxv4lowS5K
h96sZXqTrAKfhrxqd9n6YAwJn93R6X/zDOZhX0NX56DiLSGxpOHh9pZPzZ4Q
S+xlmAh/dgPzZ0kC7NV+iv4scYO9mgd/NoT5s8Qd9moZ7IFoZg+QYNgDCepB
VlV3H9DWjr7LdJLDaDuZnpnBpG9U6QqdPtx6C72SvrLQrSST8v0Xb3zbj104
7CbRnNrdQr8kg/L9g/9+yv42wyqcxLt1ren0Jk3s1+vxJmGt97hIMta5Uc1t
Tio99J+9VEO64Hvfsu8VeyjaBHqf2NtFTtT+vHXh0pw4atqrraGnQT0ZeW5B
2YnxsUTX7b3FnQ0JlO8xyd65zyDLJ5Isv9atwdtTTpld9JGMvBfb5l5jFFnw
x/5nyT8/pHx++0nEH5RY/EHE+athX/Vi9hXVhX214WmdSc3HcKpcmhegIs8V
+zs+nivVtb2yl8b+cui8l1O+wI+jznN79eQQcn740Cx3r2cCP1PUYxtLN0ST
50oqPn/vzhL4GfByhOOy0lii0uDSZtLdx2L/4AeL/gHZMcnkXH7E/U+RifCj
G8iJx+U7XbrGE+fVV46rvJLTp//ZyZ+JKuzJx8yepE////YktYE9mYx46QEW
LxV5pQbES6ezeKnIk6b8VdHJc3YcfbFeKVA9oVjg4TfI299g/3C4o+v8Gb53
PenXiZn5G9eVCfzYSOwfDu9+4Uzr3N7RxOlOq10x3iUCb+Ml9g+Hn7V/OeS9
ySOyoMbmaV5qnsBnLuStMewfDk+cFpB+7Msj0qus4F/Xu+kCz1Mk9g+HR2M/
bz+2n5dOxJ6Fh4D3Bnwy4L7Yj1DJ9iPQwZhP5QN4MeBmgHM/4hrzI+hh+BGG
gPsD7gF4CvyLy8y/oNy/SALcD3BVid8RyfwOyv2OY4DfB3wL4C7G2TLTM8HU
5X67b1qzmsS79Dyzs13A6xLyz4mlEau03wv8vzHdlxPT9F1+m/99fdrgSoHn
9anOMou7z8nnTP8ij/rmfaOTQjomLnTNJieG6C/xNUiiL0HP400blxgWp5En
C2z7XLodQbNAzxHw44YxP44egx+3FX6cNfPjaAT8uPPDzbTntYsmuXWd1gw/
f4VyfyoI/l0p8+/oEPh3Ca8s33s9jyC7QjVVjfT9BF/Mgt9nzvw+eg1+34d1
ru2V/0ylh9WOtnf0DqV/HPpp7oq8LzQt+2TrZfce0wM79x5WjwmnXrN77QlZ
0UQPXNz6YZhJJo3Q/C3GXzOGLkvo3nTM4APNM15ccPxdFi0fa5YaYBpPb3p1
M8gLeU/1Ye+1Z/ae3An6pRX87j+Y3025fnGEHXiY2YGU65dgU7Mbf3y3Y2xD
hx7qlVJGuX7xxT6pBrZPilpJ9klFsX1SoMN60gd7SaaxvSR0I+TnGsV9IrQW
cT/u7w9h/r6I+8kV/X3ES1Ussj2+3tOfJicPPY6EKJd7Ue7P1inGAUQ8MHiP
ce8LkY/p3qCWWwedC6K3sf/FrN9orRYv06nDxjamNdnBdOR/cZ7PNGlBrIbZ
jDS69+oZlaBgP9oeem0C8vJ6yMsvQx5kH/Kk3ixPKup51qq1dZv+/CkdPTkx
p2J1BI3H3pxLQ26bj/Mopn9eGhu7bnYqLcC+pzPz3ywt7Z9D/3S/o6k0JpAa
fOyROdHzI12AeOBrxANdeZ3bltn9DnmW0m/D7a4cOkjpSPOu61bkvaMrESc0
QZzwJOKEvRGnKmVxKvlW6P3cyOnl18eqWEyelXc6/7v/EQL93hJxPxXE/b4i
7hc3e1mv333qSJsA0z5m+zLoQejlpYgHaiEemIl4oMrq1y1XtyslFxa46npY
RtJB0MsfESdshThhFuKEa+Q6s+9dzib9Ixc49LS6SbtDLzcgDmbJ4mDUFnGw
XMQVOyGuSBFX7Ib4WAuX/+JjIn6+BvGxeyw+JuLnnoiPDWLxMRE/TwU97GL0
QDRAD5ORx1/E8vji3Z2B/4HAP8/jnMP7zmLvS3rjfXsejpyUPfYtndXV2cDE
P1PEYfojDlnP4pCgz1K6A/HeHYj3Tke8t40iPZDhoIcgxC0vsLilPBl2oDXi
wIaIA3N9rYO4pSuLW1I32IFOeN+ueF9Rd4d3bIt35PmgKNDJp2v/0QniXTUk
D++lhffieaKWoBMPRidkMOhkFeihG6MHogt64PHee4j3cnvsAfh9H+N3cgv8
zutnfmL1MyK/tgl8asH4lMSCT3vjXeawdyHeeJdr4F9Hxr8kD/xbB78+gfn1
pAh+RBXqam6yuhpRF1GD/ugM9Ed7ov48WNIHfRd12jngU8L4lNwHn3qhruYQ
q6sRebqJqKt5wOpqRJ7uPOT5a+wHnAh5zuttClm9jcjfNULOh2NvoAfkvD7k
/ETsn3KBnOf1MGNYPYzI032FHjzG9CCZCj3YFXVlHVldmXgXd+hBOdOD5G/o
wduoN2vP6s1EPvQ+4k72LO5EtsHP3W7Xuo+H4xs6x8FaieaUi/ntXdI2h15c
84EOLJ758mDPb4TbnzmoN0ti9WYiH7pasd5M4LkK9WYbWL2ZwPMk1JvZsnoz
gc8s6K8wpr8I9xf4nsRBbE8ieY99iAcAHwp4PeAfsCfxFduTSDSwD5HD3wDe
lu9JDPbePftCAe03sNPQjZ0yxT7ieEm8hcNfntqs8k23jH55fM7g+1+Bt2pJ
vIXDL2s1nQ8IKaPtX2d3vxj7TuBzqiTewuHvUT/ZidVPinz0TtiTO5g9Sbg9
aSeJt3D4UtRbPmD1luK9esP+XMPsT8LtT2tJHEb4WajPdGP1meJ9rWF/ujP7
k7yC/bnoiO4BP61Y2m7i26Wvx8gIt+vM612NrxrH0TOqYa06mVPC7bqSUXc6
bb2RSPd4PXa9ZhlLuF33m/ecG4G5SdQlY/FpW/94wu26Q7WfZZqrHtHs4obB
1PiJeJd/GjLSLn15RL/0OX9jXkauwP8j+FkvmZ8l8LwI/tQ35k8JvPWE3+TM
/CaBhxj4KX2Zn0K4/8L9Gn3Auf9ivd/Jr7bDQzrtxHznjknhhNtXXY5ed/+w
IooutKjNtH9JCbevbnkkn42NjKbW9/uP3qQZTbhd5KD1KnRYYCz1nbJm3Rbj
ZMLtomEbU7KWz46hkzKnvl8+O1N87xb4162Yfy2+9yv86yTmX4vvXQL/+iLz
r8X3xiOOocziGKQD9Ps6r2s9Zsgi6cINeW9SM+TkF+hrzezxt+205bTzm3++
9LoVR0ZBzwYin1LB8inED/pinGK9ARkBPf5wSVfrFStv09fKZkf3LssQ92+h
GD8R99dGn8h71ici6nAkeX/C7ZYuVaa5RfspyXmkMc3hYpLwp3RdGwbZFFKi
XrKiWks1hvI9gONbPe/8/HAEudLm32+FhyPEvrMsA4vTBSScGO7bFFqZEkx5
3mpot19D++eGEI3tLbLl3/0pvg9R12vOwTWyu8R5cXGM15arlO9DHDssr8Ip
MYYEP5hdHuUYJ35XP2NX7gCbKNLdb8m8SXOp+N3bJ7fXbjekZITPu8LWpiHi
d8ectPOxmhBO7HO0h+xsuiV+d16Yz/o/9UOIPtHq0uHgFfG7g7eGz5ENTiam
DjaZPksTxO9GgZ5/gT/O4WPXvHpnvTeO3DFNim///Z35ffgew3L44xx+6rVb
hsbMKFJvXvd2cVQzfowU830CPvZiSo/qiggyK/hVUe70QHH/ZPjjF+GPc3jp
ic1WaifCyIw18xOHx18S38X98XD44xw+5cj+qnl7npIh1S4BiaT5e4tsXJwn
jkwjO8IrL/s9ad5nd9zZeZJKjwSiY+wx9VW/2+KeaXm9/Q6uiyKDzl84dW7j
ZXGf/EuL2lm3jiDRU+S7zhb4it/ldZi7WR2msIc9W7UJbrntA9m8abRjubqy
nPPd2Dl5V+f//Ja8rY86XhP6QsQBel106LQusZw4+J7bp6fWHFdROdvD2mhn
HllY3aG3GY0W35X/T/31/9UtHbO0nDh0VLD4rqtW+Z/qDieRtWN9l08yuya+
y3jbwYhu2dFE07Kp18DufuK71sRk1xy++93uvJ/1p330SfFdLdal7xgYlU6v
FCv5fDjtL/YUnyuPGNp0Kp96HdtVar/vrthHPNg/6HzsyCoauEBVfWdJLOVy
e2gn14q/I1rIL67vUO6ibiLnfN3x3jfVf4KbyG/DVvae17dc4GF8jO/C/CUv
SX7qu0+N65r5t0dBgVnxuuckpFj/5eGTIQIPTnPIV5MJmaTl7uU7/AdeEXjw
Gp23unFkCmm/rY/99mRvgYe2Uw+W33sYQ9TlcxM8c44KPPxzpuzrzQRKhpwd
kxW7f5/Aw7ORyca7JmfQp0edPyrHuco4HtS+nBsxLbeA3kl4YXXn+BEZx8N2
+Cmb4adwO2134etrhj1e0JfHt6jb3bog4/ip02X1J89Rf8LtgW+7yh/GZb4i
S3OHr9HyviTjeNjtvrT+2PYiMm+Uqblj9DEZx0PQSdnrovJM0q205ayz1Ttl
HA8OJqUZOpkp5IhaXIsLe9fJOB7mrc8yfdc5lnS19H7w+eoyGceD19Sptxy0
5CTihkyjSGeBTEoPfoweiJQejjJ6IBwPb1Gf/4bV5wt7dQjo5AqjE6HfZ36a
Zmgz6QP9pYd2Sf6gZvvqLOrwO7A6fFFHPRF0dZ/RlQWnK0vQTyajH8Lx1gv0
E8Toh0jp59Ou/+iHSOlHhdEPkdKPCqMfsQ93SIDzjbO2T+iXxtFDkr7eEfiZ
u8xUxcI2lw4/tq3nowkPBX489RcuVIsupea5DVVrc5vtHN5P4cf6KYQfYbV+
qY/azkf0XN+CFx963RfnczvZDHYyh7fb1uNIoHoWHTguMEHbI1b8bqPEfubw
j4r9OMKedyhwmjPZL5Y6r215LDMwQpyvvO2b5Y1fk+hNYnQ6zKn5nAGvJnc0
pg+p8ifNgyUjm///+PPHZgazYujBOU/8TXNixP9/2eipdrIwlnb8OcHA6EKq
wMPG3IPJBrUympPZ+C4g5YE4J7z+SI8Fhyk1nXJAO/xZlDhHRSkqXPaC0tTO
TnTPiERxzvbiFfR/cv+Q92m3/+kBTlcHYGc+ZXamkFe8/uE31D9wuDPszyZm
f4p9o1WK+VYBD4bf0Z/5HeKcKvgXTcy/EP9vCvr5yuhHyNvZoJ8RjH6EvPUA
/Qxj9CPO3wq/by7z+8T59uCv/oy/BPwPldvf4lc30VOX7FYnrJbJOHzBmg4t
ZfNaWHQbpZr8Yki0jMvnTpDbE5jcJhzO5ecAJj/FPujx0HfvmL4T/68HfTeD
6Tsi1XfzmL4TfKqO/rVA1r8m5Mk19K/1Yf1roh64HvH2XizeLn63SJJ35vDN
inF4cR+ed96D/DKH28HeGMzsDXFP7kf0gb/A4X7Ivyxk+RcizbP0ZHkWcb4h
8lytWJ5LwLXVWb3HHFbvQZaibvPIzdTclRqpJMLs+Lal4Z+JK+Kcxaj3GMnq
Pcgp5Je77OvatMGznJyNOzK9OrqOHES8i/ddqqLvcjbwHCTpu+R1lbwf0wzw
HYC3RL/GZvRjBiD+E4V+jb3oxzwFed5FX/s306QiGjB+VY6sKJd2Rl1izm31
E7c8ymm90QYbX/KC8rrEuD2XKo2Ks6n+EZfwG4s+0Rgna79nIbFUE9/ly76L
7sJ38Xkj4zBvpCvmLajufVf5a78Sur5LwoTGx4XkPPyXaaq19rPKy+lnj84b
XA1rSdJ//loRnXjHTOVZSTlZtK40ZIn+a5EXdmgwVrUoKSY5ed3WTaEFhMeH
U/C9h9Cf4oPvvYF+k/noN+H1og8wr8MK8zq0cU++72kQ2/dEf8ecCk3ABwI+
FfAo1DeOZ/WNdDHqGyMBHwU4r3vk8bE3iI9NQHzsM+JgcsTBeL6D2qftv+md
S9pZ5XTN3PmI8Pj2bsV4DuXxnP2ADwOcx3O4XrvA9JqQew3QUzVMT1ENif5C
nIfyOE9bRX0n5OQ45MsSWb4MdZKfCc9/ZbL8F+H533jkubazPBexRZ5rt/K5
4lY7DsiHq3WNa6V1NILbG36I/6iz+I/I525H/9EH9Bc3ID5T5Fo5yvVUEX0b
H/uoW6sPNB1x6cXg3weMf+kC8O9J9OE2sT5c0ddQAPr/hdE/yQT931XkR7Kb
95uAH7ehf+o+7z+9/Kf12MpsGthCO8r9XRLldcJ/IE4+CnHyG/j/QvgdHszv
EPYS79stYn27Iu7H/ZGVzB8hUn+kHfNHiNQfMWb+iNAj6cD/PuC/LfDP825T
EI/lebePqAfwZfUAhOdPuV9swvxiIZ8Poh+2JeuHFf0a65G/82D5O/IK+bu1
gLsDzvN6vE/5Z9anLM75F350FfOjCc938zx4CeA83y2pWyA8z8vr8I1YHT7h
eV5ln1eLC/5JJONVbItmZcSIOupT6F/bgP41Xn/O/eJtzC8W73ge9ynDfTjc
E+eswTk832ShGDcQ//+T2fic1eMSyPSxg86WREWJ/EVnxMfms/gY+lA+07uK
dT50LvKwgxFXPM7iivQM4oqTET/cyOKHNBDxQ2/sB8nEfskYzJnRQd3URlY3
RS+gbuoi6qMusPoo+gD1UTqYO/0AewALMU+mAnOYM7HXj8tnnledxvKqZDS+
qzPkc3/s6ePymSLvPI3lnckcfK8r5ng7Yh+fDuaePUHefA/LmxOOh1zkx5+z
/Di5BTxoYH+QOfbuxQIPeqjn1GX1nMQPeJiLus0HrG6ThAMP7TDvvR327l3C
XJeP2C8QgT1x8zB/ZhTqjnqxuiOhL0ajvsiR1RcJPj2t5Fz/gsjJ6bRbKlf7
hUGPKFnYYY/DTOxfu4y5OpL6KOIGPcvroNqyOihiAn30DPVOzqzeiXQDHSpd
PxwSeOABiRjxrnqqUxhR8e9cM9HzG0momu5k0z2MtJHVul/ododsHKLRx9O/
pcUJZ7MTyV1D6C/q+ZEms9JpIrcHAo26D3h/nk5bPNPBwSeTpkIOP8eekVLs
L9uDPNewihsn9h6hNE3fP8jmUA7tPt9Mb0VeIXVB3WkS9jpdRd3pv8d8Azqq
NtKcrhNq7J830l74fxnyZbewL+kJ8mVfn5kEGvjoyLWMW+S4/NzbIhr3CcV+
nOvYj8PPH4f7pLP7ED2cPwp7ZByxRyYF58uwt+UL9rZ05/s0gZ8+DD8kGfiZ
oYgfko37PAN+KrCHZTPw0wd7VZ5jr0p38Bd/r0j2XpS/lxfo5xSjH9S/KVnw
d1Rj70j5O04BXc3AfhBOV0q72PxPgj0grzCvSU2y7+CKZN9BK+w74PS/Gfw7
D3P4OX6aDFnfxzbW94G6o0Zi+yLd+jBNIZ4q9sR5+w3K6f8G5h7rYN4+v2dP
yKUUzMPPh1z6jH4fOev3oW+hx6vR7zOJ9fuIfsAa0EkLRifydLxLEeTbE8yZ
5/ifgf6dJta/Qzehf8cM/TuarH9H9Pf99rru5L8aaST8QNaepZZ+4r3Goe/J
H31P16AfN2fr5k8ZGE+iLxn4bCo5Lt7rR/MKVDA//ALmxvP3+lGf1L9VqzZE
j1OV93jivUXTqF7wbxX6xd6gX4z7L7nIU2djfvtu0GcB4nXJiNddhT1vg++9
zb6XtML3uuC7wtl3kU34Ln5/L8w/5/e/jnvqsXuSONxzEuhkP6MTwun8R/Ox
b0BulDK5QbTB142YDzwd84G5H2Gm2C9JuL+wAPN412EeL6e3OvSf5rH+U1IH
enNA/2kg6z8l7UFvnL8sMF+3Bt+bKenfvwR89pD06T9BXlJHEU55/50p7MOq
Nsw+nAD7sAxz1dQwV43zb9yyvMvuryldblO2pWlqjJj7d2dcoY3/HUo3DOnm
dql7kpiLuFUxjiTiJ/7la7dla92lv/YNu52tlSHmuS0L2aNz+a4X3TzJWndw
RKaYh1kh0QtirrK1bGv3wHByum/50DEXm+fx/rRo7bChgaGkaJL549I1zXN9
v/5gb1Hk9oRlO8tj6V9KlvFXqpv3fZshn2jN8oni/leOuFw1nRtBHcZWf1w7
IFfc/7R+UnjfLEpmWC7efLVP85xhvl/JXrJfqTg8e1txuZa8bkSLjueqi2X8
HO8Tg0fErR5tofTVMz52tW6EdG5th962quYl62XSubWbP29orxHgLJPOrdWT
HRk8P2GhjH/vzpN/Z42Yoy53U24wCPr1m5gTaP5mVmVoubb8UL/CqLDyPmKe
4aiwf0xnrkolh/YNXNx46KyYm+d62CS83bQ48lA93i8r2ZNI91Mcl+wpsHlZ
b3W4sJFuH9ZguM26eV+AhleAxjXfZKL5yvDR4oQA8oP9KeL8X2tzV6nVxJI9
uo5eH8c176M3HDifJkY/JFdDttxY+fKm+P/x69MC0o0iyJdnRtpvQi6J+zhm
5GWNXEvJ8JAEsu1w837eLnVn7rR0DifnNbKsu5+4Kv6/UV01dsCZUDIv2TjN
b2bzHtsxY4vMF8ZE0cXWxnrHApv3px+qsQzePy2eGr7V2XBuQvP8T682f/3m
+d0vnPg2yZLUNu/T4XuCNCR7gmx6nChz+VZGlev0/rWKbp5n+yvwuZnhU9Bh
AOyce5K9kJt/sO8mfvjSzWt1npEEmZ+BVd/mvWZ8j0yCZI/MoIyzNy/3KqJv
ZMtLlqwJFfc/vmTp6J9/eUG7dHofuNwnQdxzE+jtEKM3cc9n0Bdpkn0ffG/L
U8nelueLsurN91YS209zogdVxYj7t67qube/bwFJuFHZy937nrh/BeSYumQ+
5Icf7Gt+Dzn/h2QOfBTkwwImH8QczgjYdUoPFffxHVu2o+bMnhjSL13V6+PS
5rn6Bshf67L8tfD7uNyYzuSGuM/vP9inzOe9r5bMe+fyeRmTz0Qqn9cz+Szu
XwW5WibZ6/fLD/b0jSM+i3SaKNmhs//SnqIE8b0a8Us/HwmkpPWu+gP5E6Ob
52GiTuAyqxMQ37sEcvsMk9tEKrefM7ktvvcb5PZoyZx5yTwKMd/18U8zbyxM
uk76J4d9ckzKE/qiZv4ivbK3MUS5bR+7uTMyBF0dlMxd5OeclvjFHG9FqoZj
6mJCSbpDiFvCo1JxvmRusDifzycciPmE/PwORbe+1E14RHISNnhppacJenbC
PNhFkv1lyZj7ulOyvywaexbUJfvLXBFPq5fMvz2HOeHubE64+N303AeLNIak
U8veT84et/0szq+r/1WW9iGXqPw31+ijON8R9VShmF8n9nEU1Pf7qldA8mf5
trJ3at5vW2HVtofH3XfEYV5mn0jLtmIuuo7e+Vlb6l6Rhmkfewxf0TxfOhbx
57Es/izgxxTPF3S4CnhbKpmjmwm87ZfMy00C3jpK9lPs+cEeiiMNR5cPMAii
az5ejBtqVCbOKQU9ZDJ6EOd0VZxLKc7RVHx3cf9M0K0ho1txTi3oVoXRrcDD
Ycl8G36+dE4Lh3P9NZ/pLyrVgyOYHhT2SWfowXNMD4r/dwceVjM8CDoxxhxd
QzZHV8jhcYr6mkr19XWmr8XvWijqa/G7VT0cKm1vRpOHO+wyK59dFf9/66t9
yrgblETdM6l0u3lB/H8+6O13Rm9Cv6ShbjMWcwI5fi72X2GtlS0nnTpqL96m
eUacE2J6T2vM4wxqFH3Cac7mUKEHO4Nu3zO6beZ3zGFzwbw7fj6fq6+GOnkO
53qWIm7Gzw/F7/ZnvyvgXRTzF0LO3//BXqQy6NPHiL+J+e2453rMi+P32a5o
Jwj6HKk8d6Djd7thiIHMUW9bpLiPzkHDVLvKp9T9ktMoh6YEcZ8Oivu4hd7J
VpzTSKT4b8fwL+R8v8LLnu6+KbTPi5TAqw3Ne1u6Yg55KzaHvHlPGeRVGyav
BB/x+s9w1H9K9zOqIV7H6dZd0X4Tv3sG9tskZr+J3w1GvOimZJ/yWNiHS5h9
KM4JgB8xm/kR4v9NEUdaLNlHHP6DufQSf0qcs1zRnxLnSPS4kM9q0OOtmB4X
/OsBe6Yvs2cEvAjxHGXJ/kpuz3dh9vz/2YeoLdmH+DP8Dlfmd8ilfsdJ5ndQ
qd+RwPyO/7MX75xkL15txyXOO12e0pf7Diwe+Wm3jON/1Ey3Zw01RfSU1+uy
gg2eMv6OutGFp0861NC+sdse2rT0lwk7VsXQ4vbEajJ7QdlLvTw/Gceb8/W5
WpEfCklb+7/tZP2Oyjh+TGEnVzE7WdDJKdjJ7Zid3LxHD/bwU8ketGLYvVbM
7iVSu1fO7F5Bt7bwF5SYvyDOj1bUp0JeybMXXPEa/YGYD634YjcvRMbhSfAL
4phfIM7nc+lvSvau/mg/O5/zf0eyp+m8oh0ivqsT5JsR8rMcb30U5YCQw9XQ
C5TpBWE3BkEvPGR6QcgTbUW5Ic7fhPunIJ7M71+teH+BtxzYS5OZvST4650i
fgT+u0JOHmFyUvwul6vmTK6K7+L7F6odMv9SVhoewc8fBD1rJJlX//8A+qRQ
Uw==
   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnHX4FsXXxrd3viCKgIKFgKiEiKDYAWIjBjYqInaiiGIXYmGBKLaChIgN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         "]], Polygon3DBox[CompressedData["
1:eJw1nAf8V+P7xs98zvmmoYlKKlFRqChkVMpWZGQkiQgpZERERUbZI6syMn9m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         "]], Polygon3DBox[CompressedData["
1:eJwl13m4z2Uax/Gf8zvHMdYWZeyHZJlJSJE1hSxH2ddGNVlKligktNjHdc0U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         "]], 
        Polygon3DBox[{{2144, 1454, 1453, 2143, 1138, 1139}, {1436, 1565, 882, 
         1118, 2132, 1435}, {2146, 1441, 1440, 2145, 1144, 1145}, {1694, 1693,
          1028, 812, 813, 1029}, {2140, 1450, 1449, 2139, 1134, 1135}, {2142, 
         1452, 1451, 2141, 1136, 1137}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{711, 847, 2167, 712, 853, 733, 2202, 859, 740, 1306, 2346, 
       1314, 865, 746, 673, 2351, 1100, 871, 752, 2342, 627, 1250, 877, 760, 
       2210, 1120, 1141, 883, 2325, 768, 1142, 1161, 889, 2329, 776, 1162, 
       1184, 2290, 895, 784, 1185, 2296, 901, 792, 1198, 2301, 907, 801, 1207,
        2306, 915, 810, 2219, 1216, 924, 821, 2227, 1225, 934, 833}], 
      Line3DBox[{713, 848, 2168, 714, 854, 2176, 734, 960, 2241, 860, 741, 
       1278, 971, 2338, 1085, 866, 747, 1284, 979, 1101, 872, 753, 1303, 2204,
        1119, 1121, 878, 761, 2211, 1122, 1256, 884, 2326, 769, 1143, 1261, 
       890, 2330, 777, 1163, 1186, 896, 2334, 785, 1304, 1187, 1274, 2297, 
       902, 793, 1005, 2350, 1329, 908, 802, 1012, 916, 811, 2220, 1217, 925, 
       822, 2228, 1226, 935, 834}], 
      Line3DBox[{715, 226, 2169, 716, 231, 2177, 735, 237, 1977, 564, 1978, 
       244, 566, 1980, 251, 570, 1984, 257, 573, 1985, 385, 1986, 574, 404, 
       1949, 537, 406, 1835, 427, 263, 1848, 439, 1849, 270, 450, 1865, 277, 
       461, 1875, 283, 561, 1974, 471, 835}], 
      Line3DBox[{717, 942, 943, 2170, 718, 951, 952, 2178, 1061, 961, 1072, 
       962, 1074, 1279, 972, 2246, 1086, 973, 1340, 1315, 2347, 980, 1102, 
       981, 754, 2205, 987, 1123, 988, 762, 2212, 1144, 1145, 2181, 770, 1164,
        1165, 2186, 778, 1188, 1189, 2191, 1190, 998, 1275, 2339, 999, 1290, 
       1199, 1006, 2259, 1328, 1007, 1331, 1208, 2307, 1013, 1014, 1218, 2309,
        1022, 1023, 1227, 2312, 1032, 1033, 836}], 
      Line3DBox[{719, 944, 945, 2171, 1042, 953, 954, 1062, 963, 1073, 2274, 
       964, 1075, 1280, 974, 2247, 1087, 975, 1281, 1089, 1285, 982, 2251, 
       1103, 983, 755, 2206, 989, 1124, 990, 1251, 2320, 1125, 1146, 1126, 
       1334, 2353, 1313, 1325, 991, 1166, 992, 2187, 1263, 1169, 2291, 1170, 
       786, 1000, 1326, 2349, 1001, 1327, 1201, 1008, 2260, 1330, 1009, 1332, 
       1210, 1015, 2263, 1016, 1220, 2310, 1024, 1025, 1228, 2313, 1034, 1035,
        837}], Line3DBox[{720, 946, 947, 2172, 1043, 955, 2238, 956, 1063, 
       965, 2242, 966, 1076, 976, 2248, 1088, 1283, 1282, 1090, 984, 2252, 
       1322, 1324, 1323, 1245, 1104, 2281, 1105, 763, 2213, 1147, 1257, 1148, 
       2182, 771, 1167, 1262, 2331, 1168, 779, 993, 2255, 994, 787, 1002, 
       1200, 2302, 1003, 1203, 1295, 1296, 2261, 1209, 1010, 1212, 1017, 2264,
        1219, 1018, 1222, 1026, 2267, 1027, 823, 2229, 1036, 1037, 838}], 
      Line3DBox[{721, 948, 2237, 950, 1044, 957, 2240, 959, 1064, 967, 2244, 
       969, 1077, 977, 2249, 1316, 1321, 1320, 1091, 985, 2253, 1268, 1289, 
       1288, 1246, 1106, 2282, 1108, 1252, 1128, 2286, 2287, 1130, 1270, 1171,
        2293, 1173, 1271, 995, 1191, 2298, 997, 1273, 1004, 1202, 1294, 2303, 
       1293, 1276, 1011, 2262, 1211, 1300, 1299, 1277, 1019, 2265, 1221, 1020,
        812, 1028, 2269, 1030, 824, 1038, 2270, 1040, 839}], 
      Line3DBox[{724, 1045, 850, 2314, 725, 1046, 856, 737, 970, 2245, 862, 
       743, 978, 2250, 868, 749, 986, 2254, 874, 757, 2207, 1109, 1131, 880, 
       765, 2215, 1132, 1150, 886, 2327, 773, 1151, 1175, 892, 2332, 781, 
       1176, 1193, 2294, 898, 789, 617, 1266, 2258, 904, 795, 623, 2341, 1267,
        910, 804, 1021, 2266, 918, 814, 2221, 1031, 927, 826, 2231, 937, 
       841}], Line3DBox[{726, 1047, 851, 2315, 727, 1048, 857, 2316, 738, 
       1065, 863, 744, 1078, 2275, 869, 750, 2203, 1092, 875, 758, 2208, 1110,
        881, 766, 2216, 1133, 1152, 887, 2328, 774, 1153, 1177, 893, 2333, 
       782, 1178, 1194, 899, 2337, 790, 1336, 1205, 1335, 2304, 905, 796, 
       1349, 1214, 2354, 1344, 1345, 1346, 805, 1348, 2217, 1341, 1347, 919, 
       815, 2222, 928, 827, 2232, 938, 842}], 
      Line3DBox[{728, 302, 1890, 481, 303, 1895, 487, 317, 1901, 492, 1902, 
       329, 497, 1910, 342, 506, 1919, 356, 518, 1930, 376, 1941, 529, 395, 
       1954, 542, 417, 2052, 677, 639, 2071, 706, 2072, 699, 708, 2074, 702, 
       816, 2223, 929, 828, 2233, 939, 843}], 
      Line3DBox[{729, 1049, 1050, 2173, 1233, 1051, 1052, 2179, 1236, 1066, 
       1067, 1239, 1079, 2276, 1080, 1242, 2317, 1093, 1094, 1247, 2318, 1111,
        1112, 1253, 2321, 1134, 1135, 2183, 1258, 1154, 1155, 2188, 1264, 
       1179, 1342, 1180, 2192, 1337, 1307, 1343, 1308, 797, 911, 2194, 806, 
       2218, 920, 817, 2224, 930, 829, 2234, 940, 844}], 
      Line3DBox[{730, 1053, 1054, 2174, 1234, 1055, 1056, 2180, 1237, 1068, 
       2272, 1069, 1240, 1081, 2277, 1082, 1243, 1095, 2279, 1096, 1248, 2319,
        1113, 1114, 1254, 2322, 1136, 1137, 2184, 1259, 1156, 1157, 2189, 
       1265, 1181, 1182, 2193, 1338, 1309, 2344, 1310, 798, 912, 2195, 807, 
       921, 2197, 818, 2225, 931, 830, 2235, 941, 845}], 
      Line3DBox[{731, 1057, 1058, 2175, 1235, 1059, 2271, 1060, 1238, 1070, 
       2273, 1071, 1241, 1083, 2278, 1084, 1244, 1097, 2280, 1098, 1249, 1115,
        2283, 1116, 1255, 2323, 1138, 1139, 2185, 1260, 1158, 1159, 2190, 
       1272, 1195, 2299, 1196, 1339, 1311, 2345, 1312, 799, 913, 2196, 808, 
       922, 2198, 819, 932, 2199, 831, 2236, 1229, 1230, 1231}], 
      Line3DBox[{832, 933, 1224, 2226, 820, 923, 2308, 1215, 809, 914, 2305, 
       1206, 800, 906, 2300, 1197, 791, 900, 2295, 1183, 783, 894, 2289, 1160,
        775, 888, 2288, 1140, 767, 2324, 882, 1118, 2209, 759, 876, 2284, 
       1117, 751, 870, 1333, 2352, 1099, 745, 864, 2343, 1305, 739, 858, 2201,
        732, 852, 2200, 710, 846, 1041, 1232}], 
      Line3DBox[{840, 936, 1039, 2230, 825, 926, 2268, 1029, 813, 917, 1223, 
       1302, 2311, 1301, 803, 909, 1213, 2340, 1298, 1297, 794, 903, 2257, 
       1204, 1292, 1291, 788, 897, 2256, 1192, 996, 780, 891, 2292, 1174, 
       1172, 772, 2336, 885, 1149, 1129, 2214, 764, 879, 1127, 2285, 1107, 
       756, 873, 1269, 1287, 2335, 1286, 748, 867, 1317, 2348, 1319, 1318, 
       742, 861, 2243, 968, 736, 855, 2239, 958, 723, 849, 949, 722}]}, {
      Line3DBox[{1351, 1700, 846, 1350, 2167, 1535, 1352, 2168, 1536, 1353, 
       1612, 2169, 1537, 1354, 2105, 2170, 1538, 1355, 2106, 2171, 1701, 1539,
        1356, 2107, 2172, 1702, 1540, 1357, 2237, 1613, 1703, 849, 1358, 1704,
        1888, 2314, 1541, 1359, 1706, 1889, 2315, 1542, 1360, 1708, 1890, 
       1543, 1361, 2121, 2173, 1891, 1544, 1362, 2123, 2174, 1892, 1545, 1363,
        2125, 2175, 1893, 1546, 1710}], 
      Line3DBox[{1365, 2076, 2200, 1364, 853, 1366, 2176, 1547, 1367, 1614, 
       2177, 1548, 1368, 2108, 2178, 1712, 1549, 1369, 954, 1713, 1550, 1370, 
       2238, 1615, 1714, 1551, 1371, 2240, 1616, 1715, 2077, 2239, 1372, 1705,
        1716, 856, 1373, 1707, 1894, 2316, 1552, 1374, 1709, 1895, 1553, 1375,
        2122, 2179, 1896, 1554, 1376, 2124, 2180, 1897, 1555, 1377, 2271, 
       1711, 1898, 1556, 1378}], 
      Line3DBox[{1380, 2078, 2201, 1379, 2079, 2202, 1381, 1617, 1975, 2080, 
       2241, 1382, 1618, 1977, 1557, 1721, 962, 1723, 1558, 1722, 2274, 1619, 
       1725, 1559, 1383, 2242, 1620, 1727, 1560, 1384, 2244, 1621, 1729, 2081,
        2243, 1385, 1622, 1731, 2082, 2245, 1386, 1717, 1899, 863, 1387, 1718,
        1901, 1561, 1388, 1067, 1903, 1562, 1389, 2272, 1719, 1905, 1563, 
       1390, 2273, 1720, 1907, 1564, 1391}], 
      Line3DBox[{1393, 2032, 2033, 2343, 1392, 2035, 2036, 2346, 1394, 1623, 
       1976, 1624, 2338, 2008, 1395, 1625, 1978, 1626, 2009, 1396, 2010, 2246,
        1627, 1724, 1628, 1397, 2011, 2247, 1629, 1726, 1630, 2012, 1398, 
       2248, 1631, 1728, 1632, 2043, 1399, 2249, 1633, 1730, 1634, 2348, 2164,
        1400, 1635, 1732, 1636, 2250, 1401, 1733, 1900, 1734, 2275, 1402, 
       1735, 1902, 1736, 1403, 2276, 1737, 1904, 1738, 1404, 2277, 1739, 1906,
        1740, 1405, 2278, 1741, 1908, 1742, 1406}], Line3DBox[CompressedData["
1:eJwVzksug2EUBuCDuA26BjE2YAOWoBJLkFRRWhPamri0qOoOLIUd0LpUUbRq
VGNJJ0LiMXjznvN83//ln17OLWWHImJBaqMRL2MRxfGItv6ciDhjz+YCe9J9
VmWP5jxLTEYsslPWYjvsQVfsSX4u26zJTljDvmW+l5wcsylvfLiTtd/JptTd
O3LW4xv2W8nINS/zX75uv5E1ueLvrOTsR6+ybz3j7Xlnh7xrT/NLvaIPWOf/
30YiLnSK7bNh92d917UPeEfv8Tn2Zv5ir3pX/wGMtTMi
       "]], 
      Line3DBox[{1434, 1770, 1922, 1769, 2283, 1433, 1768, 1921, 2319, 2131, 
       1432, 1767, 1920, 2318, 2130, 1431, 1766, 1919, 1765, 1430, 2129, 2208,
        1918, 1764, 1429, 2128, 2207, 1917, 1763, 1428, 2285, 1762, 1916, 
       1761, 2282, 1427, 1760, 1915, 1759, 2281, 1426, 1652, 2206, 2111, 1425,
        1651, 2205, 2110, 1424, 1650, 1984, 1649, 1423, 2027, 2028, 2204, 
       1983, 1777, 1422, 2025, 2342, 2026, 1982, 1776, 1421, 2284, 1772, 1981,
        1771, 1775}], 
      Line3DBox[{1437, 1774, 1934, 2324, 1565, 1436, 1779, 1935, 2325, 1566, 
       1797, 1781, 1937, 2326, 1567, 1936, 1801, 1986, 1568, 1441, 2146, 2181,
        1569, 1804, 2051, 2353, 2165, 2041, 2057, 1570, 1938, 2148, 2182, 
       1571, 1445, 2287, 1784, 1987, 2336, 1572, 1806, 1786, 1939, 2327, 1573,
        1807, 1788, 1940, 2328, 1574, 1809, 1791, 1941, 1575, 1450, 2140, 
       2183, 1942, 1576, 1452, 2142, 2184, 1943, 1577, 1454, 2144, 2185, 1944,
        1578, 1812}], 
      Line3DBox[{1455, 1794, 1933, 2323, 2143, 1453, 1793, 1932, 2322, 2141, 
       1451, 1792, 1931, 2321, 2139, 1449, 1790, 1930, 1789, 1448, 2138, 2216,
        1929, 1787, 1447, 2137, 2215, 1928, 1785, 1446, 2136, 2214, 1927, 
       1783, 2286, 1444, 1805, 2213, 2147, 1443, 1782, 1926, 2320, 2135, 1442,
        1803, 2212, 2145, 1440, 1800, 1985, 1799, 1439, 2134, 2211, 1925, 
       1780, 1438, 2133, 2210, 1924, 1778, 1435, 2132, 2209, 1923, 1773, 
       1796}], Line3DBox[{1457, 1795, 1945, 2083, 2288, 1456, 1798, 1946, 
       2329, 1579, 1814, 1802, 1948, 2330, 1580, 1947, 1816, 1949, 1581, 1458,
        2152, 2186, 1582, 1819, 2112, 2187, 1951, 1583, 1950, 2331, 1820, 
       1584, 1459, 2293, 1822, 1988, 2084, 2292, 1823, 1808, 1952, 2332, 1585,
        1824, 1810, 1953, 2333, 1586, 1826, 1811, 1954, 1587, 1460, 2149, 
       2188, 1955, 1588, 1461, 2150, 2189, 1956, 1589, 1462, 2151, 2190, 1989,
        1590, 1829}], 
      Line3DBox[{1464, 1813, 1957, 2085, 2289, 1463, 1815, 1958, 2086, 2290, 
       1831, 1817, 1959, 2334, 1591, 1833, 1818, 1835, 1592, 1465, 2155, 2191,
        1836, 1593, 1466, 2291, 1821, 1594, 1467, 2255, 1653, 1595, 1837, 
       2298, 1654, 1990, 2087, 2256, 1838, 2029, 1825, 1991, 2088, 2294, 1839,
        2030, 1827, 1992, 2337, 1596, 1840, 2058, 1828, 2052, 2062, 2063, 
       2061, 2153, 2192, 2054, 1597, 1468, 2154, 2193, 2055, 1598, 1469, 2299,
        1841, 2056, 1599, 1960}], 
      Line3DBox[{1471, 1830, 1961, 2089, 2295, 1470, 1832, 1963, 2090, 2296, 
       1472, 1834, 1846, 2091, 2297, 1993, 2018, 1655, 1848, 1600, 1994, 2019,
        2339, 1656, 1850, 1601, 2046, 2047, 2349, 1657, 1853, 2004, 2005, 
       1852, 2302, 1658, 1856, 2006, 2007, 1855, 2303, 2020, 1659, 1995, 2092,
        2257, 1858, 2021, 1660, 1997, 2093, 2258, 1965, 2031, 1859, 1999, 
       2094, 2304, 2053, 2038, 2071, 1602, 2064, 1308, 1603, 1473, 2344, 2039,
        1604, 1474, 2345, 2040, 1605, 1475}], 
      Line3DBox[{1477, 1842, 1962, 1843, 2300, 1476, 1844, 1964, 1845, 2301, 
       1478, 1661, 1847, 1662, 2350, 1479, 1663, 1849, 1664, 2048, 1480, 2259,
        1665, 1851, 1666, 2049, 1481, 2260, 1667, 1854, 1668, 2022, 1482, 
       2162, 2261, 1669, 1857, 1670, 1483, 2262, 1671, 1996, 1672, 2340, 2163,
        1484, 1673, 1998, 1674, 2341, 2023, 1485, 1869, 2000, 1870, 2354, 
       2070, 1486, 2073, 2065, 2072, 2066, 1487, 2194, 1606, 1488, 2195, 1607,
        1489, 2196, 1608, 1490}], 
      Line3DBox[{1492, 1860, 1966, 1861, 2305, 1491, 1862, 1967, 1863, 2306, 
       1493, 1675, 1864, 1012, 1494, 1676, 1865, 1677, 1495, 2113, 2307, 1866,
        1678, 1496, 2263, 1679, 1867, 1680, 1497, 2264, 1681, 1868, 1682, 
       1498, 2265, 1683, 2001, 1684, 2024, 2311, 1499, 1685, 2002, 1686, 2266,
        1500, 2059, 2060, 2217, 2166, 2069, 1501, 2075, 2067, 2074, 2068, 
       1502, 2095, 2218, 1503, 2197, 1609, 1504, 2198, 1610, 1505}], 
      Line3DBox[{1507, 1871, 1968, 1872, 2308, 1506, 1873, 1969, 2219, 2156, 
       1508, 1874, 1970, 2220, 2157, 1509, 1687, 1875, 1688, 1510, 2114, 2309,
        1876, 1689, 1511, 2115, 2310, 1877, 1690, 1512, 2267, 1691, 1878, 
       1692, 1513, 2269, 1693, 1694, 2268, 1514, 1695, 2003, 2221, 2116, 1515,
        2096, 2222, 1516, 2097, 2223, 1517, 2098, 2224, 1518, 2099, 2225, 
       1519, 2199, 1611, 1520}], 
      Line3DBox[{1534, 1887, 2236, 2161, 1533, 2235, 2104, 1532, 2234, 2103, 
       1531, 2233, 2102, 1530, 2232, 2101, 1529, 2231, 2100, 1528, 2120, 2230,
        1699, 2270, 1527, 1698, 2229, 2119, 1526, 1697, 1885, 2313, 2118, 
       1525, 1696, 1884, 2312, 2117, 1524, 1883, 1974, 1882, 1523, 2160, 2228,
        1973, 1881, 1522, 2159, 2227, 1972, 1880, 1521, 2158, 2226, 1971, 
       1879, 1886}]}}},
   VertexNormals->CompressedData["
1:eJxcnHk0lW0b9pVmjRIJmTKkEBERl4SQIipUxgZFJCmlhFIoCiGFDJmnzGQ6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    "]],
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{309.7654148417275, 447.1277503904881},
  Method->{},
  PlotRange->{{-1.157078669274492, 
   1.1376826162250655`}, {-1.1518017857070748`, 
   1.1487008094447915`}, {-1.999999642857143, 2.999999642857143}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{-0.3551790042993218, -3.322270750358401, -0.5351307655311617},
  ViewVertical->{
   0.662981664636288, -0.34323326752492883`, -0.9394020540168271}]], "Output",
 CellChangeTimes->{3.5820007428920383`*^9, 3.5820008643538513`*^9, 
  3.5820009718068404`*^9, 3.582001038762158*^9, 3.582001100288666*^9, 
  3.5820011357631283`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["plotintrinsicsurfrev", "Subsubsection",
 CellChangeTimes->{3.5818352019002924`*^9}],

Cell[BoxData[
 RowBox[{"?", "plotintrinsicsurfrev"}]], "Input",
 CellChangeTimes->{{3.5823540639551477`*^9, 3.5823540672155533`*^9}, 
   3.5823800183165674`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"Cos", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5818336761084127`*^9, 3.5818337047968626`*^9}}],

Cell[BoxData[
 RowBox[{"plotintrinsicsurfrev", "[", 
  RowBox[{"kk", ",", "1", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "3"}], ",", "3"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581833650789568*^9, 3.5818336620059876`*^9}, {
   3.581833713532878*^9, 3.5818337137044783`*^9}, {3.581833755652952*^9, 
   3.5818337730469823`*^9}, 3.5818338107366486`*^9, {3.581833842888305*^9, 
   3.581833860048335*^9}, {3.581833892964393*^9, 3.581833968624526*^9}, {
   3.5818340267034283`*^9, 3.58183402785783*^9}, {3.581834065266696*^9, 
   3.5818340685427017`*^9}, {3.5818341084319715`*^9, 3.5818341167623863`*^9}, 
   3.5818350133739614`*^9, 3.5818350592068415`*^9, {3.5818351145713387`*^9, 
   3.5818351565510125`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["polarsurf", "Subsubsection",
 CellChangeTimes->{{3.5818526005052304`*^9, 3.581852603640836*^9}}],

Cell[BoxData[
 RowBox[{"?", " ", "polarsurf"}]], "Input",
 CellChangeTimes->{{3.582354109647628*^9, 3.58235411081763*^9}, 
   3.5823800183321676`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"polarsurf", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5818526340452895`*^9, 3.58185268561898*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", "0.25`"}], " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], "+", 
    RowBox[{"0.4472135954999579`", " ", "v", " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "0.4472135954999579`"}], " ", "v", " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], "-", 
    RowBox[{"0.25`", " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{"0.`", "\[VeryThinSpace]", "+", 
    RowBox[{"0.5`", " ", "u"}], "+", 
    RowBox[{"0.8944271909999159`", " ", "v"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5818526870385823`*^9, 3.5818528010279827`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818527115930257`*^9, 3.581852751731896*^9}, {
  3.581852788875561*^9, 3.5818527922139673`*^9}}],

Cell[BoxData[
 RowBox[{"hel", " ", "=", " ", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"helix", "[", 
      RowBox[{"1", ",", ".5"}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"4", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
     RowBox[{"{", 
      RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581852972238283*^9, 3.5818530667432494`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"%", ",", "%%"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5818530825928774`*^9, 3.5818530940432973`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["prindev", "Subsubsection",
 CellChangeTimes->{{3.5818584203911886`*^9, 3.5818584238387947`*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"prindev", "[", "surf_", "]"}], "[", 
    RowBox[{"u_", ",", "v_", ",", "s_"}], "]"}], ":=", " ", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"surf", "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", "+", "\n", "        ", 
     RowBox[{"s", "*", 
      RowBox[{
       RowBox[{"unitnormal", "[", "surf", "]"}], "[", 
       RowBox[{"u", ",", "v"}], "]"}]}]}], "]"}]}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"paraboloid", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5818572624479547`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["u", "2"]}], "+", 
    SuperscriptBox["v", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581857263664757*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"pf", "[", 
   RowBox[{"u_", ",", "v_", ",", "s_"}], "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"prindev", "[", 
    RowBox[{"paraboloid", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v", ",", "s"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581856584299164*^9, 3.581856651457281*^9}, 
   3.58185673614983*^9, {3.5818569641286306`*^9, 3.581856966780635*^9}, {
   3.5818571728569975`*^9, 3.5818571992522435`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"paraboloid", "[", 
     RowBox[{"2", ",", "1"}], "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818570027230988`*^9, 3.5818571189745026`*^9}, {
   3.5818572161158733`*^9, 3.581857232527102*^9}, {3.581857306580432*^9, 
   3.581857326891668*^9}, 3.581857366390937*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"par", " ", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5818574336426554`*^9, 3.5818574382758636`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"pf", "[", 
    RowBox[{"u", ",", ".5", ",", "s"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818570027230988`*^9, 3.5818571189745026`*^9}, {
   3.5818572161158733`*^9, 3.581857232527102*^9}, {3.581857306580432*^9, 
   3.581857326891668*^9}, 3.581857366390937*^9, {3.581857491674757*^9, 
   3.5818575335140305`*^9}, {3.581857575665305*^9, 3.581857627082995*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pf1", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.581857637441413*^9, 3.581857647612631*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pf2", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5818577267983704`*^9, 3.581857732773181*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"par", ",", "pf1", ",", "pf2"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5818577508068123`*^9, 3.5818577679824424`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pff1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"pf", "[", 
   RowBox[{"u", ",", ".5", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5818579092406907`*^9, 3.581857983309621*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussiancurvature", "[", "pff1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.581857863860211*^9, 3.581857875809832*^9}, {
  3.581857990360833*^9, 3.581857996507244*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{"1.`", " ", 
     SuperscriptBox["u", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.125`", "\[VeryThinSpace]", "+", 
        RowBox[{"1.`", " ", 
         SuperscriptBox["u", "2"]}]}], ")"}], "6"]}], ")"}]}], "/", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"0.125`", "\[VeryThinSpace]", "+", 
       SuperscriptBox["u", "2"]}], ")"}], "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"0.0625`", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"0.125`", "\[VeryThinSpace]", "+", 
          SuperscriptBox["u", "2"]}], ")"}], "3"]], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "u"}], "-", 
         FractionBox[
          RowBox[{"16", " ", "u", " ", "v"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"2.`", "\[VeryThinSpace]", "+", 
             RowBox[{"16", " ", 
              SuperscriptBox["u", "2"]}]}], ")"}], 
           RowBox[{"3", "/", "2"}]]]}], ")"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"64", " ", 
           SuperscriptBox["u", "2"], " ", "v"}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"2.`", "\[VeryThinSpace]", "+", 
             RowBox[{"16", " ", 
              SuperscriptBox["u", "2"]}]}], ")"}], 
           RowBox[{"3", "/", "2"}]]], "-", 
         FractionBox[
          RowBox[{"4", " ", "v"}], 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]]]}], ")"}], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"0.015625`", "\[VeryThinSpace]", "+", 
      RowBox[{"5.`", " ", 
       SuperscriptBox["u", "4"]}], "+", 
      RowBox[{"16.`", " ", 
       SuperscriptBox["u", "6"]}], "-", 
      RowBox[{"0.0625`", " ", 
       SqrtBox[
        RowBox[{"2.`", "\[VeryThinSpace]", "+", 
         RowBox[{"16", " ", 
          SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
      RowBox[{"0.125`", " ", 
       SuperscriptBox["v", "2"]}], "+", 
      RowBox[{
       SuperscriptBox["u", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"0.5`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.5`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}]}], ")"}]}]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.5818579978800464`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Together", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.581858030016103*^9, 3.5818580462713313`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{"0.00390625`", " ", 
     SuperscriptBox["u", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.125`", "\[VeryThinSpace]", "+", 
        SuperscriptBox["u", "2"]}], ")"}], "6"]}], ")"}]}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"0.0009765625`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.3125`", " ", 
       SuperscriptBox["u", "4"]}], "+", 
      RowBox[{"1.`", " ", 
       SuperscriptBox["u", "6"]}], "-", 
      RowBox[{"0.00390625`", " ", 
       SqrtBox[
        RowBox[{"2.`", "\[VeryThinSpace]", "+", 
         RowBox[{"16", " ", 
          SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
      RowBox[{"0.0078125`", " ", 
       SuperscriptBox["v", "2"]}], "+", 
      RowBox[{
       SuperscriptBox["u", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"0.03125`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.03125`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}]}], ")"}]}]}], ")"}], 
    " ", 
    RowBox[{"(", 
     RowBox[{"2.384185791015625`*^-7", "+", 
      RowBox[{"0.8125`", " ", 
       SuperscriptBox["u", "12"]}], "+", 
      SuperscriptBox["u", "14"], "-", 
      RowBox[{"9.5367431640625`*^-7", " ", 
       SqrtBox[
        RowBox[{"2.`", "\[VeryThinSpace]", "+", 
         RowBox[{"16", " ", 
          SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
      RowBox[{"1.9073486328125`*^-6", " ", 
       SuperscriptBox["v", "2"]}], "+", 
      RowBox[{
       SuperscriptBox["u", "10"], " ", 
       RowBox[{"(", 
        RowBox[{"0.28125`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.03125`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["u", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"0.0000152587890625`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.00003814697265625`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
         RowBox[{"0.00006103515625`", " ", 
          SuperscriptBox["v", "2"]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["u", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"0.0004119873046875`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.0006103515625`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
         RowBox[{"0.000732421875`", " ", 
          SuperscriptBox["v", "2"]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["u", "6"], " ", 
       RowBox[{"(", 
        RowBox[{"0.006103515625`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.0048828125`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
         RowBox[{"0.00390625`", " ", 
          SuperscriptBox["v", "2"]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["u", "8"], " ", 
       RowBox[{"(", 
        RowBox[{"0.0537109375`", "\[VeryThinSpace]", "-", 
         RowBox[{"0.01953125`", " ", 
          SqrtBox[
           RowBox[{"2.`", "\[VeryThinSpace]", "+", 
            RowBox[{"16", " ", 
             SuperscriptBox["u", "2"]}]}]], " ", "v"}], "+", 
         RowBox[{"0.0078125`", " ", 
          SuperscriptBox["v", "2"]}]}], ")"}]}]}], ")"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5818580475037336`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{"%%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818580931494136`*^9, 3.581858124131068*^9}, {
  3.5818581711183505`*^9, 3.5818581860319767`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["reflection. reflectgraphics. rf", "Subsubsection",
 CellChangeTimes->{{3.5819178316310277`*^9, 3.581917847638943*^9}, {
  3.5819180671264973`*^9, 3.581918077182072*^9}}],

Cell[BoxData[
 RowBox[{"?", "reflection"}]], "Input",
 CellChangeTimes->{{3.5823542121398077`*^9, 3.582354220220622*^9}, 
   3.582380018363368*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"reflection", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", "1"}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "x", ",", "y"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.581917246773576*^9, 3.5819172945843105`*^9}, {
  3.581917409937908*^9, 3.581917414921193*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "x", ",", 
   RowBox[{"2", "-", "y"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581917295770378*^9, 3.581917420970539*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"gr", " ", "=", " ", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"parabola", "[", ".5", "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581917597361628*^9, 3.5819176507946844`*^9}}],

Cell[BoxData[
 RowBox[{"?", " ", "reflectgraphics"}]], "Input",
 CellChangeTimes->{{3.5823542507030754`*^9, 3.5823542524814787`*^9}, 
   3.582380018363368*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"reflectgraphics", "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], "]"}], "[", "gr", "]"}]], "Input",
 CellChangeTimes->{{3.5819176826285048`*^9, 3.581917706282858*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581917766271289*^9, 3.5819177842313166`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"parabola", "[", "1", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{3.5819181862403097`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", "t"}], ",", 
   SuperscriptBox["t", "2"]}], "}"}]], "Output",
 CellChangeTimes->{3.5819181871793633`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rf", "[", 
   RowBox[{"parabola", "[", "1", "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5819180913148804`*^9, 3.5819181417367644`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", "t"}], ",", 
   RowBox[{"-", 
    SuperscriptBox["t", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581918142757823*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["rollcurve", "Subsubsection",
 CellChangeTimes->{{3.5819188734786177`*^9, 3.5819189048594127`*^9}}],

Cell[BoxData[
 RowBox[{"?", "rollcurve"}]], "Input",
 CellChangeTimes->{{3.5823543379072285`*^9, 3.582354345254842*^9}, 
   3.582380018378968*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rollcurve", "[", 
   RowBox[{
    RowBox[{"ellipse", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "1"}]}], "]"}], ",", ".3", ",", ".5", ",", "0"}], "]"}], 
  "[", 
  RowBox[{"0", ",", 
   RowBox[{"2", "*", "Pi"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819185308340197`*^9, 3.5819185318810797`*^9}, {
  3.581918594593666*^9, 3.5819186404262877`*^9}, {3.5819187201938505`*^9, 
  3.581918728384319*^9}, {3.581918795250143*^9, 3.5819187977292852`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"rc", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5819183562260323`*^9, 3.5819183607992945`*^9}}],

Cell[BoxData[
 RowBox[{"ell", " ", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ellipse", "[", 
      RowBox[{"2", ",", 
       RowBox[{"-", "1"}]}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
    RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581918366270607*^9, 3.5819184069099317`*^9}, {
  3.58191866389063*^9, 3.58191866615376*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"rc", ",", "ell"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581918412579256*^9, 3.5819184360055957`*^9}, {
  3.5819186736571884`*^9, 3.5819186744202323`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["rotation2d. rotation3d. rt", "Subsubsection",
 CellChangeTimes->{{3.5819389269514017`*^9, 3.5819389322147026`*^9}, {
  3.5819420726073227`*^9, 3.5819420840949802`*^9}, {3.5819423461109667`*^9, 
  3.5819423474070406`*^9}}],

Cell[BoxData[
 RowBox[{"?", "rotation2d"}]], "Input",
 CellChangeTimes->{{3.5823544144253635`*^9, 3.582354415938566*^9}, 
   3.582380018394568*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rotation2d", "[", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0"}], "}"}]}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "0"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5819191852904525`*^9, 3.581919287145278*^9}, {
  3.581920054985196*^9, 3.581920074441309*^9}, {3.581920203345682*^9, 
  3.5819202042327323`*^9}, {3.5819204593933268`*^9, 3.5819204666247406`*^9}, {
  3.581938377905998*^9, 3.5819383952569904`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", 
    FractionBox[
     RowBox[{
      RowBox[{"-", "1"}], "+", "x"}], 
     SqrtBox["2"]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "1"}], "+", "x"}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{{3.58191927669168*^9, 3.58191928961642*^9}, 
   3.581919571482541*^9, 3.5819196408855104`*^9, 3.5819198454032087`*^9, 
   3.5819200791695795`*^9, 3.581920205681815*^9, 3.5819204745751953`*^9, 
   3.5819383070869474`*^9, 3.5819384002762775`*^9, 3.581938609733258*^9, 
   3.581938749060227*^9, 3.5819388706121793`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ArcCos", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"%", "-", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "-", "1"}], ",", "0"}], "}"}], ")"}]}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Norm", "[", 
      RowBox[{"%", "-", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "]"}], "*", 
     RowBox[{"Norm", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"x", "-", "1"}], ",", "0"}], "}"}], "]"}]}], ")"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5819193473837233`*^9, 3.5819194273933*^9}, {
  3.5819195920657187`*^9, 3.5819196349311705`*^9}, {3.5819196660669513`*^9, 
  3.581919726041382*^9}, {3.5819200887701283`*^9, 3.5819201403220773`*^9}, {
  3.581920519818783*^9, 3.581920600800415*^9}, {3.581938243068286*^9, 
  3.581938298785473*^9}, {3.581938421107469*^9, 3.5819384501441298`*^9}, {
  3.581938491985523*^9, 3.5819386228990107`*^9}, {3.5819388557393284`*^9, 
  3.581938865450884*^9}}],

Cell[BoxData[
 RowBox[{"ArcCos", "[", 
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "x"}], ")"}], "2"], 
   RowBox[{
    SqrtBox["2"], " ", 
    SuperscriptBox[
     RowBox[{"Abs", "[", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "x"}], "]"}], "2"]}]], "]"}]], "Output",
 CellChangeTimes->{3.5819194325195932`*^9, 3.581919730628644*^9, 
  3.5819198508185186`*^9, 3.5819201416021504`*^9, 3.5819202142523055`*^9, 
  3.5819206047656417`*^9, 3.5819383115072002`*^9, 3.5819386246211095`*^9, 
  3.581938754861559*^9, 3.5819388746384096`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"x", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581919457619029*^9, 3.581919464424418*^9}, {
  3.581938341152896*^9, 3.581938344584092*^9}}],

Cell[BoxData[
 FractionBox["\[Pi]", "4"]], "Output",
 CellChangeTimes->{
  3.581919465579484*^9, 3.581920220007635*^9, {3.581938326241043*^9, 
   3.5819383510124598`*^9}, 3.581938731769238*^9, 3.581938887614152*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"?", "rotation3d"}]], "Input",
 CellChangeTimes->{{3.582354457543839*^9, 3.5823544662954545`*^9}, 
   3.5823800184101677`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rotation3d", "[", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"x", ",", "y", ",", "z"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5819415018536777`*^9, 3.5819416260507812`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", 
    FractionBox[
     RowBox[{
      RowBox[{"-", "1"}], "+", "x"}], 
     SqrtBox["2"]], "-", 
    FractionBox["y", 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"-", "1"}], "+", "x"}], 
     SqrtBox["2"]], "+", 
    FractionBox["y", 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    FractionBox["z", 
     SqrtBox["2"]], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       FractionBox["1", 
        SqrtBox["2"]]}], ")"}], " ", "z"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5819416290379524`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rotation3d", "[", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], "]"}], "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0", ",", "0"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5819415018536777`*^9, 3.5819416648820024`*^9}, {
  3.581941701122075*^9, 3.581941707017412*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "-", 
    FractionBox["1", 
     SqrtBox["2"]]}], ",", 
   RowBox[{"-", 
    FractionBox["1", 
     SqrtBox["2"]]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5819416700542984`*^9, 3.581941741122363*^9, 
  3.5819420466688395`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ArcCos", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"%", "-", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], ")"}], ".", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "0", ",", "0"}], "}"}]}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Norm", "[", 
      RowBox[{"%", "-", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], "]"}], "*", 
     RowBox[{"Norm", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "0", ",", "0"}], "}"}], "]"}]}], ")"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.581939420870652*^9, 3.581939508626672*^9}, {
  3.5819399278826513`*^9, 3.581939950129924*^9}, {3.581941792155282*^9, 
  3.581941802073849*^9}, {3.581941903898673*^9, 3.5819420407785025`*^9}}],

Cell[BoxData[
 FractionBox["\[Pi]", "4"]], "Output",
 CellChangeTimes->{3.581939510067754*^9, 3.5819396748541794`*^9, 
  3.581939952194042*^9, 3.581941803499931*^9, 3.581942050440055*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rt", "[", 
   RowBox[{
    RowBox[{"Pi", "/", "4"}], ",", " ", 
    RowBox[{"ellipse", "[", 
     RowBox[{"3", ",", "1"}], "]"}]}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.581942217531612*^9, 3.5819422464322653`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"3", " ", 
      RowBox[{"Cos", "[", "t", "]"}]}], 
     SqrtBox["2"]], "-", 
    FractionBox[
     RowBox[{"Sin", "[", "t", "]"}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"3", " ", 
      RowBox[{"Cos", "[", "t", "]"}]}], 
     SqrtBox["2"]], "+", 
    FractionBox[
     RowBox[{"Sin", "[", "t", "]"}], 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5819422505575013`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819422577489123`*^9, 3.5819422847854586`*^9}}],

Cell[BoxData[
 RowBox[{"?", "rt"}]], "Input",
 CellChangeTimes->{{3.582354557259214*^9, 3.5823545585852165`*^9}, 
   3.582380018425768*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["ruled", "Subsubsection",
 CellChangeTimes->{{3.5819457752631025`*^9, 3.5819457776872416`*^9}}],

Cell[BoxData[
 RowBox[{"?", "ruled"}]], "Input",
 CellChangeTimes->{{3.5823546165861187`*^9, 3.582354620922926*^9}, 
   3.582380018441368*^9}],

Cell["\<\
Try it replacing rruled by ruled in the next cells! rruled is a slightly \
changed ruled:\
\>", "Text",
 CellChangeTimes->{{3.5819441689752283`*^9, 3.5819442266385264`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rruled", "[", 
    RowBox[{"alpha_", ",", "gamma_"}], "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"v", "*", 
    RowBox[{"alpha", "[", "u", "]"}]}], " ", "+", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "v"}], ")"}], "*", 
    RowBox[{"gamma", "[", "u", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.581943932441699*^9, 3.581943978249319*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"rruled", "[", 
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], ",", 
    RowBox[{"helix", "[", 
     RowBox[{"1.5", ",", "1"}], "]"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.581942887451929*^9, 3.5819429630892553`*^9}, 
   3.581943896985671*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"1.5`", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "v"}], ")"}], " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], "+", 
    RowBox[{"v", " ", 
     RowBox[{"Cos", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"1.5`", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "v"}], ")"}], " ", 
     RowBox[{"Sin", "[", "u", "]"}]}], "+", 
    RowBox[{"v", " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"u", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "v"}], ")"}]}], "+", 
    RowBox[{"0.5`", " ", "u", " ", "v"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.58194296771352*^9, 3.581943055600547*^9, 
  3.581943118492144*^9, 3.581943155801278*^9, 3.5819438220153832`*^9, 
  3.581943996407358*^9, 3.581944112723011*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"3", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", ".2"}], ",", "1.2"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581942974465906*^9, 3.5819430520023413`*^9}, {
  3.581943100658124*^9, 3.58194314989694*^9}, {3.581943813019869*^9, 
  3.5819438169460936`*^9}, {3.5819440921708355`*^9, 3.581944107442709*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.58194321458364*^9, 3.5819432198319407`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"helix", "[", 
       RowBox[{"1", ",", ".5"}], "]"}], "[", "u", "]"}], ",", 
     RowBox[{
      RowBox[{"helix", "[", 
       RowBox[{"1.5", ",", "1"}], "]"}], "[", "u", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"3", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Red", ",", "Thick"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Green", ",", " ", "Thick"}], "}"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.581942974465906*^9, 3.5819430520023413`*^9}, {
   3.581943100658124*^9, 3.58194314989694*^9}, {3.581943242634245*^9, 
   3.581943418979331*^9}, 3.5819434588756127`*^9, {3.5819436570749493`*^9, 
   3.5819436752109866`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic2", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5819436911428976`*^9, 3.5819437032165885`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"pic", ",", "pic2"}], "}"}], ",", " ", 
   RowBox[{"PlotRange", " ", "\[Rule]", "  ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819437070598083`*^9, 3.581943762720992*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"1.5", "*", 
     RowBox[{"(", 
      RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
     RowBox[{"Cos", "[", "u", "]"}]}], " ", "+", " ", 
    RowBox[{"v", "*", 
     RowBox[{"Cos", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
   RowBox[{
    RowBox[{"1.5", "*", 
     RowBox[{"(", 
      RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
     RowBox[{"Sin", "[", "u", "]"}]}], " ", "+", " ", 
    RowBox[{"v", "*", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
   RowBox[{
    RowBox[{"u", "*", 
     RowBox[{"(", 
      RowBox[{"1", " ", "-", " ", "v"}], ")"}]}], " ", "+", " ", 
    RowBox[{"0.5", "*", "u", "*", "v"}]}]}], "}"}]], "Output",
 Evaluatable->True,
 CellChangeTimes->{3.58194296771352*^9, 3.581943055600547*^9, 
  3.581943118492144*^9, 3.581943155801278*^9, 3.5819438220153832`*^9, 
  3.581943996407358*^9, 3.581944112723011*^9, 3.581944697553461*^9},
 FormatType->"InputForm"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic3", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.581944841137674*^9, 3.5819448501041865`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vl", "[", "u_", "]"}], " ", "=", " ", 
  RowBox[{"Line", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"1.5", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
          RowBox[{"Cos", "[", "u", "]"}]}], " ", "+", " ", 
         RowBox[{"v", "*", 
          RowBox[{"Cos", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
        RowBox[{
         RowBox[{"1.5", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
          RowBox[{"Sin", "[", "u", "]"}]}], " ", "+", " ", 
         RowBox[{"v", "*", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
        RowBox[{
         RowBox[{"u", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}]}], " ", "+", " ", 
         RowBox[{"0.5", "*", "u", "*", "v"}]}]}], "}"}], "/.", 
      RowBox[{"v", "\[Rule]", 
       RowBox[{"-", ".2"}]}]}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"1.5", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
          RowBox[{"Cos", "[", "u", "]"}]}], " ", "+", " ", 
         RowBox[{"v", "*", 
          RowBox[{"Cos", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
        RowBox[{
         RowBox[{"1.5", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}], "*", 
          RowBox[{"Sin", "[", "u", "]"}]}], " ", "+", " ", 
         RowBox[{"v", "*", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ",", " ", "\n", "  ", 
        RowBox[{
         RowBox[{"u", "*", 
          RowBox[{"(", 
           RowBox[{"1", " ", "-", " ", "v"}], ")"}]}], " ", "+", " ", 
         RowBox[{"0.5", "*", "u", "*", "v"}]}]}], "}"}], "/.", 
      RowBox[{"v", "\[Rule]", " ", "1.2"}]}]}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5819442930003223`*^9, 3.5819443372728543`*^9}, 
   3.581944403899665*^9, {3.5819444496522818`*^9, 3.581944451652396*^9}, 
   3.5819444843882685`*^9, {3.581944569653146*^9, 3.5819445794287047`*^9}, {
   3.5819447424410286`*^9, 3.5819448109219456`*^9}}],

Cell[BoxData[
 RowBox[{"Line", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1.5999999999999999`", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], ",", 
      RowBox[{"1.5999999999999999`", " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], ",", 
      RowBox[{"1.0999999999999999`", " ", "u"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"0.9`", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], ",", 
      RowBox[{"0.9`", " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], ",", 
      RowBox[{"0.4`", " ", "u"}]}], "}"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.581944812338026*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Animate", "[", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{"{", 
     RowBox[{"pic3", ",", 
      RowBox[{"Graphics3D", "[", 
       RowBox[{"{", 
        RowBox[{"Thick", ",", 
         RowBox[{"vl", "[", "u", "]"}]}], "}"}], "]"}]}], "}"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"3", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819448588956895`*^9, 3.5819449515779905`*^9}, {
   3.581945059866184*^9, 3.5819450706768026`*^9}, {3.581945154095574*^9, 
   3.5819451623220444`*^9}, 3.581945206920595*^9, {3.5819452461098366`*^9, 
   3.581945279467745*^9}, 3.5819453299676332`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["seashell", "Subsubsection",
 CellChangeTimes->{{3.5819464090553536`*^9, 3.5819464126785607`*^9}}],

Cell[BoxData[
 RowBox[{"?", "seashell"}]], "Input",
 CellChangeTimes->{{3.58235468556944*^9, 3.5823546933226533`*^9}, 
   3.5823800184569683`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"seashell", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], "]"}], "[", ".1", "]"}], "[", 
  RowBox[{"t", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.5819459549323792`*^9, 3.5819460319847865`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "t", "]"}], "+", 
    RowBox[{"0.1`", " ", "t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"1.`", " ", 
        RowBox[{"Cos", "[", "t", "]"}], " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "+", 
       RowBox[{"0.4472135954999579`", " ", 
        RowBox[{"Sin", "[", "t", "]"}], " ", 
        RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "t", "]"}], "+", 
    RowBox[{"0.1`", " ", "t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"1.`", " ", 
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"Sin", "[", "t", "]"}]}], "-", 
       RowBox[{"0.4472135954999579`", " ", 
        RowBox[{"Cos", "[", "t", "]"}], " ", 
        RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"0.5`", " ", "t"}], "+", 
    RowBox[{"0.1`", " ", "t", " ", 
     RowBox[{"(", 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.8944271909999159`", " ", 
        RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5819460331928554`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581946039603222*^9, 3.581946096826495*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
sff: Second Fundamental Form.  sff2\
\>", "Subsubsection",
 CellChangeTimes->{{3.5819478876479244`*^9, 3.5819478891910124`*^9}, {
  3.5823548041608477`*^9, 3.5823548193084745`*^9}, {3.5823548928002033`*^9, 
  3.582354899601815*^9}}],

Cell[BoxData[
 RowBox[{"?", "sff"}]], "Input",
 CellChangeTimes->{{3.5823547310279193`*^9, 3.582354737782731*^9}, 
   3.5823547772196007`*^9, 3.582380018472568*^9}],

Cell["\<\
It is not so clear why Alfred put ** in this definition. We compare the \
results with this and the changed definition  sff2 as a quadratic form in du, \
dv, depending on the surface point {u,v}:\
\>", "Text",
 CellChangeTimes->{{3.5819472319354196`*^9, 3.5819473685832357`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"sff2", "[", "x_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], "[", 
   RowBox[{"du_", ",", "dv_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"eee", "[", "x", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{"du", "^", "2"}]}], " ", "+", "\n", "        ", 
   RowBox[{"2", "*", 
    RowBox[{
     RowBox[{"fff", "[", "x", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", "du", "*", "dv"}], " ", "+", 
   RowBox[{
    RowBox[{
     RowBox[{"ggg", "[", "x", "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "*", 
    RowBox[{"dv", "^", "2", " "}]}]}]}]], "Input",
 CellChangeTimes->{{3.5819470612606573`*^9, 3.581947110524475*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sff", "[", 
    RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"du", ",", "dv"}], "]"}]], "Input",
 CellChangeTimes->{{3.5819464756501627`*^9, 3.5819465273781214`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"du", "\[Rule]", "2"}], ",", 
    RowBox[{"dv", "\[Rule]", "1"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5819467450265703`*^9, 3.581946783224755*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["r", "3"], " ", 
       RowBox[{"Cos", "[", "v", "]"}]}], 
      SqrtBox[
       RowBox[{
        SuperscriptBox["r", "4"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"]}]]]}], ")"}], "**", "1"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       SqrtBox[
        RowBox[{
         SuperscriptBox["r", "4"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"]}]]}], "r"]}], ")"}], "**", 
   "4"}]}]], "Output",
 CellChangeTimes->{3.5819467948964224`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sff2", "[", 
    RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"du", ",", "dv"}], "]"}]], "Input",
 CellChangeTimes->{{3.581947124686285*^9, 3.5819471413232365`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["dv", "2"], " ", 
     SuperscriptBox["r", "3"], " ", 
     RowBox[{"Cos", "[", "v", "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["r", "4"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}]]]}], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox["du", "2"], " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    SqrtBox[
     RowBox[{
      SuperscriptBox["r", "4"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}]]}], "r"]}]], "Output",
 CellChangeTimes->{3.58194714259431*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sff2", "[", 
    RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.581947124686285*^9, 3.581947170785922*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["r", "3"], " ", 
     RowBox[{"Cos", "[", "v", "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["r", "4"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}]]]}], "-", 
  FractionBox[
   RowBox[{"4", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    SqrtBox[
     RowBox[{
      SuperscriptBox["r", "4"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}]]}], "r"]}]], "Output",
 CellChangeTimes->{3.5819471758652124`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sff2", "[", 
    RowBox[{
     RowBox[{"seashell", "[", 
      RowBox[{"helix", "[", 
       RowBox[{"1", ",", ".5"}], "]"}], "]"}], "[", ".1", "]"}], "]"}], "[", 
   RowBox[{"t", ",", "u"}], "]"}], "[", 
  RowBox[{"dt", ",", "du"}], "]"}]], "Input",
 CellChangeTimes->{{3.581947410819651*^9, 3.5819474140128336`*^9}, {
  3.5819474606265*^9, 3.5819474849578915`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"0.1414213562373095`", " ", 
      SuperscriptBox["du", "2"], " ", 
      SuperscriptBox["t", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"12.5`", "\[VeryThinSpace]", "+", 
        RowBox[{"1.`", " ", "t", " ", 
         RowBox[{"Cos", "[", "u", "]"}]}], "+", 
        RowBox[{"8.881784197001252`*^-16", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", "u"}], "]"}]}]}], ")"}]}], ")"}]}], "/", 
   RowBox[{"(", 
    RowBox[{"\[Sqrt]", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["t", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"315.`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.9999999999999998`", " ", 
          SuperscriptBox["t", "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "2.710505431213761`*^-14"}], "+", 
            RowBox[{"1.`", " ", 
             SuperscriptBox["t", "2"]}]}], ")"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "u"}], "]"}]}], "-", 
         RowBox[{"2.168404344971009`*^-15", " ", "t", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"3", " ", "u"}], "]"}]}], "-", 
         RowBox[{"4.3368086899420184`*^-17", " ", 
          SuperscriptBox["t", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "u"}], "]"}]}], "+", 
         RowBox[{"t", " ", 
          RowBox[{"Cos", "[", "u", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{"50.`", "\[VeryThinSpace]", "-", 
            RowBox[{"3.878959614448864`*^-16", " ", 
             RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], "+", 
         RowBox[{"3.469446951953614`*^-15", " ", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], ")"}]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"0.1131370849898476`", " ", 
     SuperscriptBox["dt", "2"], " ", "t", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"(", 
         RowBox[{"158.74999999999997`", "\[VeryThinSpace]", "+", 
          RowBox[{"1.`", " ", 
           SuperscriptBox["t", "2"]}], "+", 
          RowBox[{"2.4243497590305397`*^-15", " ", 
           RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], "+", 
       RowBox[{"t", " ", 
        RowBox[{"(", 
         RowBox[{"15.625`", "\[VeryThinSpace]", "+", 
          RowBox[{"12.499999999999998`", " ", 
           RowBox[{"Cos", "[", 
            RowBox[{"2", " ", "u"}], "]"}]}], "+", 
          RowBox[{"0.25000000000000006`", " ", "t", " ", 
           RowBox[{"Cos", "[", 
            RowBox[{"3", " ", "u"}], "]"}]}], "+", 
          RowBox[{"0.5590169943749475`", " ", 
           RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"\[Sqrt]", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["t", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"315.`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.9999999999999998`", " ", 
          SuperscriptBox["t", "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "2.710505431213761`*^-14"}], "+", 
            RowBox[{"1.`", " ", 
             SuperscriptBox["t", "2"]}]}], ")"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "u"}], "]"}]}], "-", 
         RowBox[{"2.168404344971009`*^-15", " ", "t", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"3", " ", "u"}], "]"}]}], "-", 
         RowBox[{"4.3368086899420184`*^-17", " ", 
          SuperscriptBox["t", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "u"}], "]"}]}], "+", 
         RowBox[{"t", " ", 
          RowBox[{"Cos", "[", "u", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{"50.`", "\[VeryThinSpace]", "-", 
            RowBox[{"3.878959614448864`*^-16", " ", 
             RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], "+", 
         RowBox[{"3.469446951953614`*^-15", " ", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], ")"}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"0.12649110640673517`", " ", "dt", " ", "du", " ", 
     SuperscriptBox["t", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"12.5`", "\[VeryThinSpace]", "+", 
       RowBox[{"1.`", " ", "t", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "-", 
       RowBox[{"8.881784197001252`*^-16", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "u"}], "]"}]}], "+", 
       RowBox[{"2.23606797749979`", " ", 
        RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"\[Sqrt]", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["t", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"315.`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.9999999999999998`", " ", 
          SuperscriptBox["t", "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "2.710505431213761`*^-14"}], "+", 
            RowBox[{"1.`", " ", 
             SuperscriptBox["t", "2"]}]}], ")"}], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "u"}], "]"}]}], "-", 
         RowBox[{"2.168404344971009`*^-15", " ", "t", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"3", " ", "u"}], "]"}]}], "-", 
         RowBox[{"4.3368086899420184`*^-17", " ", 
          SuperscriptBox["t", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"4", " ", "u"}], "]"}]}], "+", 
         RowBox[{"t", " ", 
          RowBox[{"Cos", "[", "u", "]"}], " ", 
          RowBox[{"(", 
           RowBox[{"50.`", "\[VeryThinSpace]", "-", 
            RowBox[{"3.878959614448864`*^-16", " ", 
             RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], "+", 
         RowBox[{"3.469446951953614`*^-15", " ", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}], ")"}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.581947487150017*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{"Simplify", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58194751150141*^9, 3.5819475284413786`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"t", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"0.12649110640673517`", " ", "dt", " ", "du", " ", 
          SuperscriptBox["t", "2"]}], "-", 
         RowBox[{"0.1414213562373095`", " ", 
          SuperscriptBox["du", "2"], " ", 
          SuperscriptBox["t", "2"]}], "+", 
         RowBox[{
          SuperscriptBox["dt", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "17.960512242138304`"}], "-", 
            RowBox[{"0.1131370849898476`", " ", 
             SuperscriptBox["t", "2"]}]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], "+", 
      RowBox[{"t", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "1.7677669529663689`"}], " ", 
          SuperscriptBox["dt", "2"]}], "+", 
         RowBox[{"1.5811388300841895`", " ", "dt", " ", "du"}], "-", 
         RowBox[{"1.7677669529663689`", " ", 
          SuperscriptBox["du", "2"]}], "-", 
         RowBox[{"1.414213562373095`", " ", 
          SuperscriptBox["dt", "2"], " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "u"}], "]"}]}], "-", 
         RowBox[{"0.02828427124746191`", " ", 
          SuperscriptBox["dt", "2"], " ", "t", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"3", " ", "u"}], "]"}]}], "-", 
         RowBox[{"0.0632455532033676`", " ", 
          SuperscriptBox["dt", "2"], " ", 
          RowBox[{"Sin", "[", "u", "]"}]}], "+", 
         RowBox[{"0.282842712474619`", " ", "dt", " ", "du", " ", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   SqrtBox[
    RowBox[{
     SuperscriptBox["t", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"315.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.9999999999999998`", " ", 
        SuperscriptBox["t", "2"]}], "+", 
       RowBox[{"50.`", " ", "t", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "+", 
       RowBox[{"1.`", " ", 
        SuperscriptBox["t", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "u"}], "]"}]}]}], ")"}]}]], ")"}]}]], "Output",
 CellChangeTimes->{3.581947530329487*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"dt", "\[Rule]", "0"}], ",", 
     RowBox[{"du", "\[Rule]", "1"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819476506323676`*^9, 3.581947697194031*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"0.1414213562373095`", " ", 
    SuperscriptBox["t", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"12.5`", "\[VeryThinSpace]", "+", 
      RowBox[{"1.`", " ", "t", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}]}], ")"}]}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox["t", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"315.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.9999999999999998`", " ", 
        SuperscriptBox["t", "2"]}], "+", 
       RowBox[{"50.`", " ", "t", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "+", 
       RowBox[{"1.`", " ", 
        SuperscriptBox["t", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "u"}], "]"}]}]}], ")"}]}]]]}]], "Output",
 CellChangeTimes->{3.5819476991661434`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%%", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"dt", "\[Rule]", "1"}], ",", 
     RowBox[{"du", "\[Rule]", "0"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5819476506323676`*^9, 3.581947697194031*^9}, {
  3.581947774023425*^9, 3.5819477952976418`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"t", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "17.960512242138304`"}], "-", 
         RowBox[{"0.1131370849898476`", " ", 
          SuperscriptBox["t", "2"]}]}], ")"}], " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], "+", 
      RowBox[{"t", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1.7677669529663689`"}], "-", 
         RowBox[{"1.414213562373095`", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", " ", "u"}], "]"}]}], "-", 
         RowBox[{"0.02828427124746191`", " ", "t", " ", 
          RowBox[{"Cos", "[", 
           RowBox[{"3", " ", "u"}], "]"}]}], "-", 
         RowBox[{"0.0632455532033676`", " ", 
          RowBox[{"Sin", "[", "u", "]"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
  RowBox[{"(", 
   SqrtBox[
    RowBox[{
     SuperscriptBox["t", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"315.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.9999999999999998`", " ", 
        SuperscriptBox["t", "2"]}], "+", 
       RowBox[{"50.`", " ", "t", " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "+", 
       RowBox[{"1.`", " ", 
        SuperscriptBox["t", "2"], " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", " ", "u"}], "]"}]}]}], ")"}]}]], ")"}]}]], "Output",
 CellChangeTimes->{3.5819477965037107`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["surfast", "Subsubsection",
 CellChangeTimes->{{3.5820867766863103`*^9, 3.5820867787767143`*^9}}],

Cell[BoxData[
 RowBox[{"?", "surfast"}]], "Input",
 CellChangeTimes->{{3.582355092808155*^9, 3.5823550966145616`*^9}, 
   3.582380018488168*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sfp", "[", "n_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"surfast", "[", 
    RowBox[{"n", ",", 
     RowBox[{"ellipse", "[", 
      RowBox[{"2", ",", "1"}], "]"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582085083303336*^9, 3.5820851294482174`*^9}, {
   3.582085228976392*^9, 3.5820852609408484`*^9}, 3.5820853130917397`*^9, 
   3.5820855805854096`*^9, {3.582085824086237*^9, 3.582085857173895*^9}, {
   3.5820860198197813`*^9, 3.5820860304746*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ellipse", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "[", "v", "]"}]], "Input",
 CellChangeTimes->{{3.5820863597599783`*^9, 3.58208639471964*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"Sin", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5820863607271795`*^9, 3.5820863957804413`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sfp", "[", "3", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820860732186747`*^9, 3.582086100050722*^9}, 
   3.582086179844862*^9, 3.5820865116730447`*^9, 3.582086638392068*^9, 
   3.582086683803747*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox[
     RowBox[{"Cos", "[", "u", "]"}], "3"], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    SuperscriptBox[
     RowBox[{"Sin", "[", "u", "]"}], "3"]}], ",", 
   RowBox[{"Sin", "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582086101563925*^9, 3.582086188908478*^9, 
  3.5820865124998465`*^9, 3.5820866423856745`*^9, 3.582086688171755*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sfp", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820853250101604`*^9, 3.582085413867917*^9}, {
   3.582085718084051*^9, 3.5820857646813326`*^9}, {3.5820858789047337`*^9, 
   3.58208600223855*^9}, 3.5820860444210243`*^9, {3.5820862004524984`*^9, 
   3.5820862308881516`*^9}, 3.582086277345033*^9, {3.5820865402366953`*^9, 
   3.582086591981986*^9}, 3.582086695300967*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["surfrev. gensurfref", "Subsubsection",
 CellChangeTimes->{{3.5814165698085346`*^9, 3.5814165713846254`*^9}, 
   3.581416642818711*^9, 3.581417150783765*^9, {3.5814183849383545`*^9, 
   3.58141839219477*^9}, 3.5820013219494553`*^9}],

Cell[BoxData[
 RowBox[{"?", "surfrev"}]], "Input",
 CellChangeTimes->{{3.5823551570958676`*^9, 3.582355161994276*^9}, 
   3.582380018503768*^9}],

Cell[BoxData[
 RowBox[{"?", "gensurfrev"}]], "Input",
 CellChangeTimes->{{3.5823551570958676`*^9, 3.582355161994276*^9}, {
   3.582355210011161*^9, 3.5823552206971793`*^9}, 3.582380018519368*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cg", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{".7", "*", 
      RowBox[{"Cos", "[", "t", "]"}]}]}], ",", "t"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.581417449224835*^9, 3.5814175012908125`*^9}, {
  3.581417668107354*^9, 3.5814176701474705`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"surfrev", "[", "cg", "]"}], "[", 
  RowBox[{"0", ",", " ", 
   RowBox[{"2", "*", "Pi"}], ",", " ", "0", ",", " ", 
   RowBox[{"4", "*", "Pi"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5814175131214895`*^9, 3.5814175366018324`*^9}, {
  3.5814175709377966`*^9, 3.5814175986663823`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ch", "[", "s_", "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"ellipse", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "[", "s", "]"}]}]], "Input",
 CellChangeTimes->{{3.581417756213393*^9, 3.581417794738597*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", "s", "]"}], ",", 
   RowBox[{"2", " ", 
    RowBox[{"Sin", "[", "s", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581417797509755*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gensurfrev", "[", 
   RowBox[{"cg", ",", "ch"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.581417812029586*^9, 3.581417875267203*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"0.7`", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}]}], ")"}], " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"u", " ", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{"2", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581417876244259*^9, 3.5814181141108637`*^9, 
  3.5814181800906377`*^9, 3.5814182486915617`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", " ", 
     RowBox[{"8", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"8", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581417883620681*^9, 3.581417965203347*^9}, {
   3.5814180165802855`*^9, 3.58141803501134*^9}, {3.581418091124549*^9, 
   3.581418109595606*^9}, {3.5814181548851957`*^9, 3.5814181762114162`*^9}, 
   3.5814182415091505`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["surfrevfromgc. plotsurfrevfromgc", "Subsubsection",
 CellChangeTimes->{{3.5818375894064856`*^9, 3.581837619171338*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "surfrevfromgc"}]], "Input",
 CellChangeTimes->{{3.582355285234493*^9, 3.5823552871376963`*^9}, 
   3.582380018534968*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"surfrevfromgc[gc,{a:0,b:0,c:0},{d,e,f},optsnd,{vmin:-10,vmax:\
10}][u,v] is a surface of revolution whose Gaussian curvature is gc.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5823552913341036`*^9},
 CellTags->"Info3582362491-6815425"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{"surfrevfromgc", "[", 
           RowBox[{"gc_", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"a_", ":", "0"}], ",", 
              RowBox[{"b_", ":", "0"}], ",", 
              RowBox[{"c_", ":", "0"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"d_", ",", "e_", ",", "f_"}], "}"}], ",", "optsnd___", 
            ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"vmin_", ":", 
               RowBox[{"-", "10"}]}], ",", 
              RowBox[{"vmax_", ":", "10"}]}], "}"}]}], "]"}], "[", 
          RowBox[{"u_", ",", "v_"}], "]"}], ":=", 
         RowBox[{"Flatten", "[", 
          RowBox[{"Module", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"phi", ",", "psi"}], "}"}], ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"phi", "[", "v", "]"}], " ", 
                RowBox[{"Cos", "[", "u", "]"}]}], ",", 
               RowBox[{
                RowBox[{"phi", "[", "v", "]"}], " ", 
                RowBox[{"Sin", "[", "u", "]"}]}], ",", 
               RowBox[{"psi", "[", "v", "]"}]}], "}"}], "/.", 
             "\[VeryThinSpace]", 
             RowBox[{"NDSolve", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["phi", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "vv", "]"}], "\[Equal]", 
                  RowBox[{
                   RowBox[{"-", 
                    RowBox[{"gc", "[", "vv", "]"}]}], " ", 
                   RowBox[{"phi", "[", "vv", "]"}]}]}], ",", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["psi", "\[Prime]",
                    MultilineFunction->None], "[", "vv", "]"}], "\[Equal]", 
                  SqrtBox[
                   RowBox[{"1", "-", 
                    SuperscriptBox[
                    RowBox[{
                    SuperscriptBox["phi", "\[Prime]\[Prime]",
                    MultilineFunction->None], "[", "vv", "]"}], "2"]}]]}], 
                 ",", 
                 RowBox[{
                  RowBox[{"phi", "[", "a", "]"}], "\[Equal]", "d"}], ",", 
                 RowBox[{
                  RowBox[{
                   SuperscriptBox["phi", "\[Prime]",
                    MultilineFunction->None], "[", "b", "]"}], "\[Equal]", 
                  "e"}], ",", 
                 RowBox[{
                  RowBox[{"psi", "[", "c", "]"}], "\[Equal]", "f"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"phi", ",", "psi"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"vv", ",", "vmin", ",", "vmax"}], "}"}], ",", 
               "optsnd"}], "]"}]}]}], "]"}], "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[CSPROGS`surfrevfromgc],
  Editable->False]], "Print",
 CellChangeTimes->{3.582355291349704*^9},
 CellTags->"Info3582362491-6815425"]
}, Open  ]]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kk", "[", "t_", "]"}], ":=", " ", 
  RowBox[{"1", "+", 
   RowBox[{"1.5", "*", 
    RowBox[{"Sin", "[", "t", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.581835864932657*^9, 3.581835871328668*^9}, {
   3.5818363618091297`*^9, 3.5818363806851625`*^9}, {3.5818366453708277`*^9, 
   3.5818366516108384`*^9}, {3.5818367286749735`*^9, 
   3.5818367315141788`*^9}, {3.581836832321556*^9, 3.581836861743208*^9}, 
   3.581837176785761*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"surfrevfromgc", "[", 
    RowBox[{"kk", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}]}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.581835905430328*^9, 3.5818360050989027`*^9}, {
   3.5818361949668365`*^9, 3.5818362599097505`*^9}, {3.581836304089028*^9, 
   3.5818363224034605`*^9}, {3.581836402416001*^9, 3.581836410356415*^9}, {
   3.5818366225167875`*^9, 3.5818366260423937`*^9}, {3.5818366782868853`*^9, 
   3.5818366907357073`*^9}, 3.5818367628858337`*^9, 3.5818369092452908`*^9, {
   3.581837035090712*^9, 3.581837044669129*^9}, {3.5818371015624285`*^9, 
   3.5818371300480785`*^9}, {3.5818372110434213`*^9, 3.5818372974207726`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "3.141592653589793`"}], ",", "3.141592653589793`"}], 
          "}"}], "}"}], ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "u", "]"}], " ", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "3.141592653589793`"}], ",", "3.141592653589793`"}], 
          "}"}], "}"}], ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "v", "]"}]}], ",", 
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "3.141592653589793`"}], ",", "3.141592653589793`"}], 
         "}"}], "}"}], ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "v", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.581836008718109*^9, {3.5818362000680456`*^9, 3.5818362682089653`*^9}, {
   3.581836307443034*^9, 3.5818363400470915`*^9}, {3.581836385957972*^9, 
   3.5818364135544205`*^9}, {3.581836466079713*^9, 3.581836492443759*^9}, 
   3.5818365243458147`*^9, 3.5818365658574877`*^9, {3.581836606386359*^9, 
   3.5818366295523996`*^9}, {3.581836691968109*^9, 3.581836764726637*^9}, 
   3.5818367994210978`*^9, {3.581836838530367*^9, 3.581836912162496*^9}, 
   3.581836964001387*^9, 3.5818370484599357`*^9, {3.581837109378042*^9, 
   3.581837131483281*^9}, {3.5818371845701747`*^9, 3.5818372985127745`*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818360241777363`*^9, 3.58183603197775*^9}, {
   3.5818361072946825`*^9, 3.5818361595547743`*^9}, {3.58183642502044*^9, 
   3.581836459262501*^9}, 3.5818365148141985`*^9, 3.581836554063867*^9, {
   3.5818365904275312`*^9, 3.581836600177548*^9}, {3.5818367006885247`*^9, 
   3.5818367070221357`*^9}, {3.5818367928846865`*^9, 3.581836795349491*^9}, {
   3.581836949758562*^9, 3.5818369579485765`*^9}}],

Cell[BoxData[
 RowBox[{"?", "plotsurfrevfromgc"}]], "Input",
 CellChangeTimes->{{3.5823553408953905`*^9, 3.582355342408593*^9}, 
   3.582380018550568*^9}],

Cell[BoxData[
 RowBox[{"plotsurfrevfromgc", "[", 
  RowBox[{"kk", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "Pi"}], ",", "Pi"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818373354536395`*^9, 3.581837347668461*^9}, {
  3.5818374186641855`*^9, 3.5818375252435727`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["tandev", "Subsubsection",
 CellChangeTimes->{{3.5820018457983756`*^9, 3.5820018492303815`*^9}}],

Cell[BoxData[
 RowBox[{"?", "tandev"}]], "Input",
 CellChangeTimes->{{3.5820018336147537`*^9, 3.5820018372807603`*^9}, 
   3.5823553948558855`*^9, 3.582380018550568*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"tandev", "[", 
   RowBox[{"viviani", "[", "1", "]"}], "]"}], "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], "Pi"}], ",", 
   RowBox[{"2", "Pi"}], ",", 
   RowBox[{"-", "2"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic1", " ", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.58200153535783*^9, 3.58200155228386*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"viviani", "[", "1", "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{
      RowBox[{"-", "2"}], "*", "Pi"}], ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820016266179905`*^9, 3.582001718190151*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"viv", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.582001725974565*^9, 3.582001731496975*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"pic1", ",", "viv"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820017415901923`*^9, 3.5820017662538357`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["transsurf", "Subsubsection",
 CellChangeTimes->{3.5820923929009705`*^9}],

Cell[BoxData[
 RowBox[{"?", "transsurf"}]], "Input",
 CellChangeTimes->{{3.5820927717008357`*^9, 3.5820927846020584`*^9}, 
   3.582355429909147*^9, 3.5823800185661683`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"transsurf", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "#", "]"}], ",", 
       RowBox[{"Sin", "[", "#", "]"}], ",", "0"}], "}"}], "&"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "#", "]"}], ",", "0", ",", 
       RowBox[{"Sin", "[", "#", "]"}]}], "}"}], "&"}]}], "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"2", "Pi"}], ",", "0", ",", 
   RowBox[{"2", "Pi"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820919990678782`*^9, 3.5820920062750907`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"transsurf", "[", 
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "#", "]"}], ",", 
       RowBox[{"Sin", "[", "#", "]"}], ",", "0"}], "}"}], "&"}]}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.582092451962674*^9, 3.5820925364212227`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], "+", 
    RowBox[{"Cos", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Sin", "[", "u", "]"}], "+", 
    RowBox[{"Sin", "[", "v", "]"}]}], ",", 
   RowBox[{"0.5`", " ", "u"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582092538776827*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"transsurf", "[", 
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], ",", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Cos", "[", "#", "]"}], ",", 
       RowBox[{"Sin", "[", "#", "]"}], ",", "0"}], "}"}], "&"}]}], "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"4", "Pi"}], ",", "0", ",", 
   RowBox[{"2", "Pi"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820919990678782`*^9, 3.5820920062750907`*^9}, {
   3.5820926341865945`*^9, 3.582092642766609*^9}, 3.58209267742987*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["tubecurve. tubeplanecurve", "Subsubsection",
 CellChangeTimes->{{3.5820958104925733`*^9, 3.5820958146733804`*^9}, {
  3.582095932765588*^9, 3.58209593381079*^9}}],

Cell[BoxData[
 RowBox[{"?", "tubecurve"}]], "Input",
 CellChangeTimes->{{3.5823554835420413`*^9, 3.582355490452853*^9}, 
   3.582380018581768*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"tubecurve", "[", 
     RowBox[{
      RowBox[{"deltoid", "[", "1", "]"}], ",", 
      RowBox[{"helix", "[", 
       RowBox[{"6", ",", "1"}], "]"}]}], "]"}], "[", "1", "]"}], "[", 
   RowBox[{"0", ",", 
    RowBox[{"5", "Pi"}], ",", "0", ",", 
    RowBox[{"2", "Pi"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tubecurve", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"6", ",", "1"}], "]"}], "]"}], "[", ".2", "]"}], "[", 
  RowBox[{"0", ",", 
   RowBox[{"5", "Pi"}], ",", "0", ",", 
   RowBox[{"2", "Pi"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582095322757716*^9, 3.5820953432873526`*^9}, {
  3.582095387950231*^9, 3.582095416311081*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tubecurve", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"6", ",", "1"}], "]"}], "]"}], "[", ".2", "]"}], "[", 
  RowBox[{"t", ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.582095322757716*^9, 3.5820953432873526`*^9}, {
   3.582095387950231*^9, 3.582095416311081*^9}, 3.582095478835991*^9, {
   3.5820957131952024`*^9, 3.582095733912039*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"6", " ", 
     RowBox[{"Cos", "[", "t", "]"}]}], "+", 
    RowBox[{"0.2`", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], " ", 
        RowBox[{"Cos", "[", "u", "]"}]}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"Sin", "[", "t", "]"}], " ", 
         RowBox[{"Sin", "[", "u", "]"}]}], 
        SqrtBox["37"]]}], ")"}]}]}], ",", 
   RowBox[{
    RowBox[{"6", " ", 
     RowBox[{"Sin", "[", "t", "]"}]}], "+", 
    RowBox[{"0.2`", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"Sin", "[", "t", "]"}]}], "-", 
       FractionBox[
        RowBox[{
         RowBox[{"Cos", "[", "t", "]"}], " ", 
         RowBox[{"Sin", "[", "u", "]"}]}], 
        SqrtBox["37"]]}], ")"}]}]}], ",", 
   RowBox[{"t", "+", 
    RowBox[{"0.19727878476642877`", " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820957348792405`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"?", "tubeplanecurve"}]], "Input",
 CellChangeTimes->{{3.5823555153816967`*^9, 3.5823555174721007`*^9}, 
   3.5823800185973682`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"tubeplanecurve", "[", 
    RowBox[{"ellipse", "[", 
     RowBox[{"2", ",", "3"}], "]"}], "]"}], "[", "1", "]"}], "[", 
  RowBox[{
   RowBox[{"-", "Pi"}], ",", "Pi", ",", 
   RowBox[{"-", "Pi"}], ",", "Pi"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820958691954765`*^9, 3.5820958784774923`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["turningangle", "Subsubsection",
 CellChangeTimes->{3.5822086154583445`*^9}],

Cell[BoxData[
 RowBox[{"?", "turningangle"}]], "Input",
 CellChangeTimes->{{3.5823555743810005`*^9, 3.5823555753170023`*^9}, 
   3.5823800186129684`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"turningangle", "[", 
   RowBox[{"ellipse", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5822056524997745`*^9, 3.5822056941362476`*^9}}],

Cell[BoxData[
 RowBox[{"ArcSin", "[", 
  FractionBox[
   RowBox[{"Cos", "[", "t", "]"}], 
   SqrtBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "2"], "+", 
     RowBox[{"4", " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]]], "]"}]], "Output",
 CellChangeTimes->{
  3.5822056964450517`*^9, {3.5822067854985647`*^9, 3.5822068088206053`*^9}, 
   3.582207121507554*^9, 3.5822074904609985`*^9, 3.582207575865884*^9, 
   3.582208536276816*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582205730796312*^9, 3.582205765069572*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"turningangle", "[", 
   RowBox[{"parabola", "[", "2", "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5822056524997745`*^9, 3.5822056941362476`*^9}, {
  3.5822076570195255`*^9, 3.5822076707713118`*^9}}],

Cell[BoxData[
 RowBox[{"ArcSin", "[", 
  FractionBox["t", 
   SqrtBox[
    RowBox[{"1", "+", 
     SuperscriptBox["t", "2"]}]]], "]"}]], "Output",
 CellChangeTimes->{3.582207671825372*^9, 3.582208545315333*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582205730796312*^9, 3.582205765069572*^9}, {
  3.5822077066523643`*^9, 3.58220771060259*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"turningangle", "[", 
   RowBox[{"helix", "[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5822056524997745`*^9, 3.5822056941362476`*^9}, {
  3.5822071787128553`*^9, 3.5822072243585353`*^9}, {3.5822075135243177`*^9, 
  3.582207521411769*^9}}],

Cell[BoxData["\<\"The curve must lie in the plane!\"\>"], "Print",
 CellChangeTimes->{{3.582207192518879*^9, 3.582207225481737*^9}, {
   3.5822074980294313`*^9, 3.5822075151364098`*^9}, 3.5822075838713417`*^9, 
   3.582208555521917*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["umeqs", "Subsubsection",
 CellChangeTimes->{{3.5822584443194785`*^9, 3.582258447626684*^9}}],

Cell[BoxData[
 RowBox[{"?", "umeqs"}]], "Input",
 CellChangeTimes->{{3.5822584543970957`*^9, 3.582258458453103*^9}, 
   3.582355705780031*^9, 3.5823800186129684`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"umeqs", "[", "k", "]"}], "[", 
      RowBox[{"ellipsoid", "[", 
       RowBox[{"3", ",", "1", ",", "1"}], "]"}], "]"}], "[", 
     RowBox[{"u", ",", "v"}], "]"}], "\[Equal]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "u", ",", "v"}], "}"}], ",", " ", "Reals"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.582258196981044*^9, 3.582258387784979*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"C", "[", "1", "]"}], "|", 
       RowBox[{"C", "[", "2", "]"}]}], ")"}], "\[Element]", "Integers"}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k", "\[Equal]", "3"}], "&&", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"u", "\[Equal]", 
             RowBox[{"\[Pi]", "+", 
              RowBox[{"2", " ", "\[Pi]", " ", 
               RowBox[{"C", "[", "1", "]"}]}]}]}], "&&", 
            RowBox[{"v", "\[Equal]", 
             RowBox[{
              RowBox[{"-", "\[Pi]"}], "-", 
              RowBox[{"2", " ", "\[Pi]", " ", 
               RowBox[{"C", "[", "2", "]"}]}]}]}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"u", "\[Equal]", 
             RowBox[{"2", " ", "\[Pi]", " ", 
              RowBox[{"C", "[", "2", "]"}]}]}], "&&", 
            RowBox[{"v", "\[Equal]", 
             RowBox[{
              RowBox[{"-", "\[Pi]"}], "-", 
              RowBox[{"2", " ", "\[Pi]", " ", 
               RowBox[{"C", "[", "1", "]"}]}]}]}]}], ")"}], "||", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"u", "\[Equal]", 
             RowBox[{"\[Pi]", " ", 
              RowBox[{"C", "[", "2", "]"}]}]}], "&&", 
            RowBox[{"v", "\[Equal]", 
             RowBox[{
              RowBox[{"-", "\[Pi]"}], "-", 
              RowBox[{"2", " ", "\[Pi]", " ", 
               RowBox[{"C", "[", "1", "]"}]}]}]}]}], ")"}]}], ")"}]}], ")"}], 
      "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"k", "\[Equal]", 
         RowBox[{"-", "3"}]}], "&&", 
        RowBox[{"u", "\[Equal]", 
         RowBox[{"\[Pi]", "+", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           RowBox[{"C", "[", "1", "]"}]}]}]}], "&&", 
        RowBox[{"v", "\[Equal]", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "\[Pi]", " ", 
          RowBox[{"C", "[", "2", "]"}]}]}]}], ")"}]}], ")"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"C", "[", "1", "]"}], "|", 
       RowBox[{"C", "[", "2", "]"}]}], ")"}], "\[Element]", "Integers"}], "&&", 
    RowBox[{"k", "\[Equal]", 
     RowBox[{"-", "3"}]}], "&&", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"u", "\[Equal]", 
         RowBox[{"2", " ", "\[Pi]", " ", 
          RowBox[{"C", "[", "1", "]"}]}]}], "&&", 
        RowBox[{"v", "\[Equal]", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "\[Pi]", " ", 
          RowBox[{"C", "[", "2", "]"}]}]}]}], ")"}], "||", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"u", "\[Equal]", 
         RowBox[{"\[Pi]", " ", 
          RowBox[{"C", "[", "1", "]"}]}]}], "&&", 
        RowBox[{"v", "\[Equal]", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "\[Pi]", " ", 
          RowBox[{"C", "[", "2", "]"}]}]}]}], ")"}]}], ")"}]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{{3.582258322998065*^9, 3.5822583498301125`*^9}, 
   3.5822584033538065`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["unitspeednds", "Subsubsection",
 CellChangeTimes->{3.5822603431392136`*^9}],

Cell[BoxData[
 RowBox[{"?", "unitspeednds"}]], "Input",
 CellChangeTimes->{{3.5822603161355658`*^9, 3.582260322656378*^9}, 
   3.5823558078042107`*^9, 3.5823800186285686`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"unitspeednds", "[", 
   RowBox[{
    RowBox[{"ellipse", "[", 
     RowBox[{"2", ",", "1"}], "]"}], ",", "0", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}], "[", "s", "]"}]], "Input",
 CellChangeTimes->{{3.582259744987363*^9, 3.5822597870606365`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "s", "]"}], "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "s", "]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582259789447441*^9, 3.58226024432864*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{"%", ",", "s"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822598158270874`*^9, 3.582259833330318*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "s", "]"}], "]"}], " ", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "s", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False], "[", "s", "]"}], "]"}], " ", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "[", "s", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5822598347811203`*^9, 3.58226024435984*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.58225984003833*^9, 3.582259853922354*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "2"]}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "5"}], "+", 
    RowBox[{"3", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"2", " ", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
           ",", "\<\"<>\"\>"}], "]"}],
         False,
         Editable->False], "[", "s", "]"}]}], "]"}]}]}], ")"}], " ", 
  SuperscriptBox[
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "s", "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.582259854951956*^9, 3.58226024439104*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5822598733755884`*^9, 3.5822599308928895`*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Chop", "[", 
    RowBox[{
     RowBox[{"%%", "-", "1"}], ",", 
     RowBox[{"10", "^", 
      RowBox[{"(", 
       RowBox[{"-", "5"}], ")"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5822598733755884`*^9, 3.5822599308928895`*^9}, {
  3.5822599629665456`*^9, 3.5822599878329897`*^9}, {3.582260101057988*^9, 
  3.5822601780285234`*^9}, {3.5822602177929935`*^9, 3.5822602235338035`*^9}, {
  3.582260257276663*^9, 3.5822602787579*^9}}],

Cell[BoxData["                     "], "Input",
 CellChangeTimes->{{3.582259308404996*^9, 3.582259309512598*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["arclength", "Subsection",
 CellChangeTimes->{
  3.580528544591187*^9, {3.5814170414845133`*^9, 3.581417069380109*^9}}],

Cell[BoxData[
 RowBox[{"?", "arclengthprime"}]], "Input",
 CellChangeTimes->{{3.582355868737918*^9, 3.582355875133929*^9}, 
   3.5823800186441684`*^9}],

Cell[BoxData[
 RowBox[{"?", "arclength"}]], "Input",
 CellChangeTimes->{{3.582355868737918*^9, 3.582355894509163*^9}, 
   3.582380018659768*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"arclength", "[", "ci", "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5805296284421797`*^9, 3.580529644055073*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ellipse", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5805297396225386`*^9, 3.580529750991189*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", 
   RowBox[{"b", " ", 
    RowBox[{"Sin", "[", "t", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.580529753299321*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"arclength", "[", 
   RowBox[{"ellipse", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", 
  RowBox[{"2", "*", "Pi"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805297693302383`*^9, 3.580529820296153*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["binormal, cross", "Subsection",
 CellChangeTimes->{{3.5805300343553967`*^9, 3.5805300477341623`*^9}, {
  3.5805309219581647`*^9, 3.5805309262374096`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "binormal"}]], "Input",
 CellChangeTimes->{{3.582355981182915*^9, 3.582355988405728*^9}, 
   3.5823800186753683`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"t->binormal[alpha][t] is the binormal of the curve \
alpha:(a,b)->R^3.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.582364594542251*^9},
 CellTags->"Info3582371794-2942315"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{"binormal", "[", "alpha_", "]"}], "[", "t_", "]"}], ":=", 
         RowBox[{
          FractionBox[
           RowBox[{"Simplify", "[", 
            RowBox[{
             RowBox[{
              SubscriptBox["\[PartialD]", "tt"], 
              RowBox[{"alpha", "[", "tt", "]"}]}], "\[Cross]", 
             RowBox[{
              SubscriptBox["\[PartialD]", 
               RowBox[{"{", 
                RowBox[{"tt", ",", "2"}], "}"}]], 
              RowBox[{"alpha", "[", "tt", "]"}]}]}], "]"}], 
           RowBox[{"Simplify", "[", 
            SqrtBox[
             RowBox[{"Factor", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 SubscriptBox["\[PartialD]", "tt"], 
                 RowBox[{"alpha", "[", "tt", "]"}]}], "\[Cross]", 
                RowBox[{
                 SubscriptBox["\[PartialD]", 
                  RowBox[{"{", 
                   RowBox[{"tt", ",", "2"}], "}"}]], 
                 RowBox[{"alpha", "[", "tt", "]"}]}]}], ".", 
               RowBox[{
                RowBox[{
                 SubscriptBox["\[PartialD]", "tt"], 
                 RowBox[{"alpha", "[", "tt", "]"}]}], "\[Cross]", 
                RowBox[{
                 SubscriptBox["\[PartialD]", 
                  RowBox[{"{", 
                   RowBox[{"tt", ",", "2"}], "}"}]], 
                 RowBox[{"alpha", "[", "tt", "]"}]}]}]}], "]"}]], "]"}]], "/.",
           "\[VeryThinSpace]", 
          RowBox[{"tt", "\[Rule]", "t"}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[CSPROGS`binormal],
  Editable->False]], "Print",
 CellChangeTimes->{3.5823645945578513`*^9},
 CellTags->"Info3582371794-2942315"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"binormal", "[", 
   RowBox[{"helix", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5805303617301216`*^9, 3.580530373055769*^9}, {
   3.580530475752643*^9, 3.5805304861682386`*^9}, 3.582356139538793*^9, 
   3.582364658502363*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"a", " ", "b", " ", 
     RowBox[{"Sin", "[", "t", "]"}]}], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["a", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["a", "2"], "+", 
        SuperscriptBox["b", "2"]}], ")"}]}]]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"a", " ", "b", " ", 
      RowBox[{"Cos", "[", "t", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox["a", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["a", "2"], "+", 
         SuperscriptBox["b", "2"]}], ")"}]}]]]}], ",", 
   FractionBox[
    SuperscriptBox["a", "2"], 
    SqrtBox[
     RowBox[{
      SuperscriptBox["a", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["a", "2"], "+", 
        SuperscriptBox["b", "2"]}], ")"}]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.580530375420904*^9, 3.5805304895154305`*^9, 
  3.580531688229993*^9, 3.5814017069778194`*^9, 3.582356067076666*^9, 
  3.5823561454044037`*^9, 3.58235617779006*^9, 3.582364663556772*^9}]
}, Open  ]],

Cell[BoxData["arclengthprime"], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ".", "%"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805305244444284`*^9, 3.580530554911171*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.5805305579703455`*^9, 3.5823560725210752`*^9}]
}, Open  ]],

Cell["\<\
Remark. cross has been replaced by the now built - in Cross product.\
\>", "Text",
 CellChangeTimes->{{3.5805305887451057`*^9, 3.5805306467104216`*^9}, {
  3.582364715972864*^9, 3.582364719685671*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Cross"}]], "Input",
 CellChangeTimes->{{3.5805307733536654`*^9, 3.5805307771098795`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\!\(\*RowBox[{\\\"Cross\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"a\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"b\\\", \
\\\"TI\\\"]}], \\\"]\\\"}]\) gives the vector cross product of \
\!\(\*StyleBox[\\\"a\\\", \\\"TI\\\"]\) and \!\(\*StyleBox[\\\"b\\\", \
\\\"TI\\\"]\). \"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Cross"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.5823647321812925`*^9},
 CellTags->"Info3582371932-2942315"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{"cross", "[", 
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"helix", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "tt", "]"}], ",", "tt"}], "]"}],
      ",", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"helix", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "tt", "]"}], ",", "\n", 
       "        ", 
       RowBox[{"{", 
        RowBox[{"tt", ",", "2"}], "}"}]}], "]"}]}], "]"}], "  ", "/.", " ", 
   RowBox[{"tt", "->", "t"}]}]}]], "Input",
 CellChangeTimes->{
  3.5805308378253527`*^9, {3.5823663350373077`*^9, 3.5823663603717527`*^9}, 
   3.5823664138798466`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "b", " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "a"}], " ", "b", " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["a", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox["a", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5805308449287586`*^9, 3.5823647411825085`*^9, 
  3.5823663659721622`*^9, 3.58236641567385*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "cross"}]], "Input",
 CellChangeTimes->{{3.5823647611193438`*^9, 3.5823647653313513`*^9}}],

Cell[BoxData[
 StyleBox["\<\"cross[v,w] is the cross product of the vectors v={v1,v2,v3} \
and w={w1,w2,w3}; \\ncross is obsolete now, it may be replaced by the \
built-in Cross.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5823647669693537`*^9},
 CellTags->"Info3582371966-2942315"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{"Cross", "[", 
    RowBox[{
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"helix", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "tt", "]"}], ",", "tt"}], "]"}],
      ",", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"helix", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "tt", "]"}], ",", "\n", 
       "        ", 
       RowBox[{"{", 
        RowBox[{"tt", ",", "2"}], "}"}]}], "]"}]}], "]"}], "  ", "/.", " ", 
   RowBox[{"tt", "->", "t"}]}]}]], "Input",
 CellChangeTimes->{{3.5805308378253527`*^9, 3.5805308733373833`*^9}, {
  3.5823664266874695`*^9, 3.5823664349242835`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", "b", " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "a"}], " ", "b", " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox["a", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox["a", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.580530876261551*^9, 3.582364806765024*^9, 
  3.582366439619892*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["binormalsurf", "Subsection",
 CellChangeTimes->{3.5805366946024723`*^9}],

Cell[BoxData[
 RowBox[{"?", "binormalsurf"}]], "Input",
 CellChangeTimes->{{3.5823665538120923`*^9, 3.582366554935294*^9}, 
   3.5823800186909685`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a", ",", "b"}], "]"}], ";", 
  RowBox[{
   RowBox[{"binormalsurf", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5805368279515066`*^9, 3.5805368841896057`*^9}, {
  3.5805369182444654`*^9, 3.5805369270896807`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"a", " ", 
     RowBox[{"Cos", "[", "u", "]"}]}], "+", 
    FractionBox[
     RowBox[{"a", " ", "b", " ", "v", " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox["a", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["a", "2"], "+", 
         SuperscriptBox["b", "2"]}], ")"}]}]]]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"a", " ", "b", " ", "v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], 
      SqrtBox[
       RowBox[{
        SuperscriptBox["a", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["a", "2"], "+", 
          SuperscriptBox["b", "2"]}], ")"}]}]]]}], "+", 
    RowBox[{"a", " ", 
     RowBox[{"Sin", "[", "u", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"b", " ", "u"}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["a", "2"], " ", "v"}], 
     SqrtBox[
      RowBox[{
       SuperscriptBox["a", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["a", "2"], "+", 
         SuperscriptBox["b", "2"]}], ")"}]}]]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5805368857340083`*^9, 3.580536928212883*^9, 
  3.5805372187009935`*^9, 3.580538365100207*^9, 3.580539103730304*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"binormalsurf", "[", 
   RowBox[{"helix", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805370053394184`*^9, 3.580537014169034*^9}, {
  3.580537060937916*^9, 3.580537067817528*^9}, {3.580537124710828*^9, 
  3.580537129266036*^9}, {3.580537183912932*^9, 3.5805372013537626`*^9}, {
  3.58053724545504*^9, 3.580537278636298*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], "+", 
    FractionBox[
     RowBox[{"2", " ", "v", " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], 
     SqrtBox["5"]]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", "v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}], 
      SqrtBox["5"]]}], "+", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"2", " ", "u"}], "+", 
    FractionBox[
     RowBox[{"v", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}], ")"}]}], 
     SqrtBox["5"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5805370317190647`*^9, 3.580537070453933*^9, 
  3.580537132854042*^9, 3.580537202539365*^9, 3.580537281725104*^9, 
  3.580537677060998*^9, 3.5805377159830666`*^9, 3.5805382050283256`*^9, 
  3.5805382460563974`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{
      RowBox[{"-", "2"}], "*", "Pi"}], ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "3"}], ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5805375788588257`*^9, 3.5805376583565655`*^9}, {
  3.580537693924628*^9, 3.580537700149039*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJxcXXlUzd8XbVYpzRqluUzJlNm9psxkSkISQubM85Q5MoTIkIT4JiFTyb1I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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnQX0FdX3xee9mXlvZl5iYIuomICNLRaCioHxs1vBDuwGUcFE7MbuLhQT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         "]], Polygon3DBox[CompressedData["
1:eJwtnQn8FdMbxueuc++de+fesrULiShEFLJUiOyJotAiaylklyVJFNnF375H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         "]], Polygon3DBox[CompressedData["
1:eJwt13m4zmUaB/A3yznnfXnPESGRXWixTWFkPdaiC6ehvRNdkZKsqRw1lhY1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         "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {Line3DBox[CompressedData["
1:eJwVzjssA1AYhuFfhGqlWovEYDBQCbrZbFWJMhkkFgODS0RjNJIUMYpBrNTm
MttcNmoQZpdNMRls4jG8eb///845Od2z5cmVhoiYw0FzRDoZ0YaHRESWM3iS
+3kAdTnPO9mIQf4yj3EJ0RJxYT8hj6PRPM8LWLXvMC/Ji+iUT1IRv/annMY5
zpDT3XJTe8Qd9+EeNeR1u+68y3UU8IkPZJwf0idafQUzdimuO5/kYV0P92Lt
/13+1uW4oCvwCPZ0Xd4alYso6Sr+vIl156fM2/IWpuVDrtkf8bL5mKsoy5f8
qLviDfMNX6MiV+2f5Rfsm9/4FT/4A0cFNBM=
       "]], Line3DBox[CompressedData["
1:eJwVz70rRQEch/Ef5fUKV5Ey2VBcRTbbLeqi3EkZlGx0sZqVt9gs/AEUC8Vs
QQwoVxYborxFBln43OHp+X17Tqdzmsens7miiJjAZmlEsiKiDvmyiAZerI2o
51s7xZ14dXfzitbFH/YgD6G4PCLLq9owl9iTfGRPcaM9w9Nocu9VRvxp+1yN
QxygRbvk0mTEFbcij2uktDeu1d45jS88es8n9+iJREQVxgrP8YtWw71aK7dh
Tuvgb62d01of92NdG+BfLcMZbck3n9rLPGKvFf4To+4tvtC2OWfv8g5m3cd8
o53wvH3OZ1hw3/Od9sAb9jM/4Qf/KmM3xg==
       "]], 
      Line3DBox[{1206, 882, 2014, 884, 4278, 2017, 922, 2036, 4279, 924, 2039,
        961, 2058, 4283, 963, 2061, 1000, 4211, 2080, 1002, 2083, 1449, 4214, 
       2586, 1451, 2090, 4239, 1491, 2613, 1493, 2097, 4244, 1533, 2640, 1535,
        4373, 2104, 1575, 2667, 1577, 4376, 2111, 1617, 2694, 4378, 1619, 
       2118, 1659, 2721, 4383, 1661, 2125, 1039, 4227, 2144, 1041, 2147, 1078,
        4230, 2166, 1080, 2169, 4257, 1117, 2188, 1119, 2191, 4264, 1158, 
       2208, 1160, 1191}], Line3DBox[CompressedData["
1:eJwVzzkvhGEUhuFDgUGExhoM0ekkBrFEp5NYE0s1lEIjwdQSjYjGUopObals
f8FSSCwRf2BQyAwKl+LO/ZzzvHnf72tJL40uFkTEHA6KIhoTEU14LY5IcjPe
5F7uw5c8wJuVEf2cM0/xNMpLIs7tZ+UZVJhXeBXL9q3mjLyGNvmiNKKwKuKS
G3CNK7TrHjmhe+IUXvCMDt2Ou/LyN0bwix9UO9+jrymLqMWCXT1nna/jQV0n
p7Ch6+acrouHdGM8jkNd0l2T8gSG/9/0zbtYdz5t3pf3MC8f8439yf+/ms/4
FBn5hh90t7xlvuc7bMtH9ln5HYfmT/5AHn8FcTfs
       "]], Line3DBox[CompressedData["
1:eJwV0L0vQ1EYgPG3vlsROhiEDpUYDAaDoAv/gaRmBmIwqFnqD2C0NGGQUItB
xISIoSKRjiIknepjELEQgxiIX4cn7znPc8/NPTe7sJovJCJiEYn2iEl8d0Rs
4SEZcY9aT8RPW8SY9s4Xccbv86fmpzaq1fkl7HC72rb5po1od/wMrvl1vogn
bVjbTEWs8RtmrtM3oKYNaR98gV/BAN+PWy2rvWqzfB5/+EVVK/EZfcJ+HI+o
I5WOqOh9WqbxPlzh2fMV81zr1ZqsEzhonNXK5omW1m7coRuX/Lw7zOFQ69Iu
+OXGv7Ke5qewpyXtj/hj/cV6kM+ipLXal/mq2cI14wv/Ci84AQ==
       "]], Line3DBox[CompressedData["
1:eJwV0L0rxHEcwPGPp8Nd6RgMyslDBoPBoDws/AUSM1EGg7MpYVVslhuYhMEk
m0sGD6WzXYpu4jLJJIOkeP2Gd5/f5/3+9a3vt3shP71SExGLqG2MGMV3U8Qe
npsjnlDJRvykIoa1D34DRf4CZe1TG9Je+CU8cAfaPt61Qe2Rn8KdtsVvoqoN
aLvpiHV+xxzPRIyhovVrq9yaljdzfCfK2gHXo8/yMwj+zyxpBS2X3MU+gipe
kWmNuNY7tK7kPNziBnVaUWvX6pMdv845MY9xrrVp9+6QxZU27w5zONVatEt+
OXkr35P8BA61tP1MezP7uF4U+G3/prgjrWSm+AZ84R/vdzY3
       "]], Line3DBox[CompressedData["
1:eJwV0D8oxGEcgPGvyOHi7jqlGCSDckoYDGaKhYwmZOTY3WxhJjFYKKQUNxpu
EMsNFGW4QTqTv8ONyueGp+/7Ps/v/dX79i2vz+cbImIFjYmICfy1RBzgvTWi
ikqaa44Y1375LdzzJ/ydWdPGtCq/gTPuSDs1P7Vh7YVfqJ/jd/jt+r+1nLbX
FlHgd82ZZMQ0KtqAVuML/CaG+ByetH7tQ1viF9HBt6Os7fO9+hQ/iR98I5OJ
uNV7tJz9IB7x5vsH80br0tLWKVzjVbsyi1pWK7tDN0p83h3WcKGltBK/yjeZ
c/wsjrWkdskX+S/rUX4Eh1rC/rz+VtqzdSefrb8p/gG9qzk4
       "]], Line3DBox[CompressedData["
1:eJwVzy1IA1AYRuFPkG3+hG0ILiwoGAwTF1QMdlc0qNG0ZRWXLDNr0qjiqkFw
cZoMC4IoMznEsDjQIKKyKD6Gw8s958LlTpZ31rYHIqKCwWTEEn5TEed4H4p4
QzfNJSIWtS/+AI/8AzpaX5vTenwVT1xDu8KHVtRe+U3ca8f8EXpaQTsdjqjx
J3Z1JGIFXW1a2+f2tJot8rPoaHVuSq/wZWT5DNramTahlfhl/OD7/04m4k7P
azPOBXTwjKR2q+W0rHMGwd3YazS1Ma3tD3m0vFH1h100tLTW4rf4hN3g13Gh
jWpN7tMucPOo84fuprhL7cXm+HH08Qc6ejbw
       "]], Line3DBox[CompressedData["
1:eJwVz78rxHEcx/H3De78GtxfwCl/BTcZzCiUQVwyKBcmSQarsxjcYpAwXClk
urhBFGcTpZu4lBIlE5fyMDx7fT7P57dv329mKj80l4iIHKqpiFRLRNbeYq85
Yh/FZEQPP8C94Ig77Ig4trtavzbINzDLn/HnKGmT2jhf4dPcNW5woh20Rgzz
03qTcxIplLVLO6rltS7nDLpxobWnI+rOS1qv7UMWRc9X9URbxJo28v8OvPJj
9k7r1AravPsCPrVFW9Oy2pa27r7MF+wG6tqq/9jRrvhB33+PB7xp29oEX9Jn
uCc840srazntVFvh3vGBhlbjH/WKtsl94we/3B+VSz1Z
       "]], Line3DBox[CompressedData["
1:eJwV0LErhHEYwPGHgXO3yF9Acn/GRRmsLgOhdF2YMIgyyGAmMrAYpFy6Mtxl
umKQxTEpks2bUopkkUs+7/Dt+f2ez/W+d9dXWiwutEVEWbedEZmuiIJ5p5NM
xGl3RMU86IjIsxH7V9XszljdPGbDrGjf0oVdjV2aVVZio+5TrMfuRk3VWSUb
McZm03c7Z5VTg12b42yJ9TsPKK8rlpiTbI0VnAeVuA+ZTd6ei9hkE+nn9Ja+
37xnvWyLLbuv6Iutms+swPbZtvuOfthu+my24XdMux/xou//oEe9s0M2w6ps
3u5Fib5Zg5XZOVu3+9CnWuyJzaX/Fduz+033+rP/B6FePsc=
       "]], Line3DBox[CompressedData["
1:eJwVzytLQ3Ech/GfxXkpyjQYDL4BL8kXYFIRQRAxGLzMIswwrE7joticsy4o
jKUpBsMUQZDZhhdYnCAi4i3Oz8LD9/9/nnPgnJG17YV0R0Ss4zERMdQdMWWf
cNoVcYaTzohJfo77RIUr90Wc26K2qC3xCfc0X7XXKGkZbVWr8sNcDQ+oaBc9
3uW39KTzAAZxpdXtsrajjTqPYRy3WrI/4sc5q03bGcwi7/lau/dG5LQNLoV3
ftPWtQntUMu67+FX27cNbV4raEfuu3zeHqOpHfiPonbHr/j+Z7zgQyu1/5Ev
6xmuiVf8afdaSrvUctwXvtHSGvybfqMV2g7h3sI/FXc+3g==
       "]], Line3DBox[CompressedData["
1:eJwV0L0rBHAYwPHnDBwWg4RB+RO8DP4Ao45ukHTdcMlbYrnMEsuNsroz3oCu
y8BlsJBJmK68ZBKSpBxl8TF8e37P8+k6rj+3kl5ORMSMblsielsjRs077Scj
9joiDsxSc8QIS7l/6titympmmU2yKfek/UyH7NyssDxL23O8z+1aNzpitTaf
ZUusy7tbPTpldXOarbIB7yEN64I1zAxbY2PeKT3Zx80r3tkeUWBz9gW9s0Wz
zgbZNlu3b+ibbZqPbIIV2Y69pF+2az6zLf9H1l7mWX//vR70wSosx6os7/ai
V/2wSzbLTljB7UsNhf2Nzf//Vt5Ft4S9SeE7/wDnrD0+
       "]], Line3DBox[CompressedData["
1:eJwVz78rxHEcx/G3XHEnRTKZTsRgs1mcgRxJ+bEoBq6QclEm6jrF6haJhToL
xeoWioVBSUZW5F9QF48b3j1f7+fr8/18v9/0Yn5qvS4icuaoMeIjGfHQEHEo
3+O7vdwSUbTfyRWuIN/gG3/LrdnP8Jpbla/wWXfKzdp38Zyblgt8WS5yo/ZM
U8SJvaM1opSKOJYHuSFdJ046X+K2dAeY5gZ0CZzQ7XNzuj2s5/p1ze764nZq
d+E2fmLJ+T79k7zJdeEGJpx/lLt1l7jCJXEZq565kNt1Od+8wLXJeX5eXuL+
cARnsMJXa/9qhrkf7MVxfkz+NlnTw71iCrO6jPxS+97au7lf8w/y2DSR
       "]], Line3DBox[CompressedData["
1:eJwVz70rxAEcx/HvDR7vEqUuZRERZbOZ3IKY5GESEeXKQ6EsdHXEpFBKlAgL
xWI4C8XCpMysrvwJSl43fHu/P5/P79fdr2lqcWghERHT7qgy4qsq4qUi4pA/
46d8URuRl5/4gy7HC/gon9rm5XN+p8vyLd0tP9ONyZt4JQ/znO2S53X9ciYZ
cSI31kUcVEcc8x5dxtaC+3KDbc22x5t13ba0rpzv6MZt21gmd9lqbEXdhi6D
6/hdet9vd9rf+IquDZfxFXdtrbYbntWlcFU3x6952jbrP0/K9XzJNsFndAm5
D0flgv4PR1yv7gc7cBCLbsC1yx9479kUz/D30ne7pPzr/gFyfzNm
       "]], Line3DBox[CompressedData["
1:eJwVz7svg2EYhvHHpNWFxGEgpKNNkAiDrQZxqC4Oi1CDRSUSalVjmYrRYRI2
h81hZdNNSMRfwCLCYPDr8OS6n+t+837fm15czRXqIiJvThIRn8mIx/qII/kB
P+ynjRG79qp8z5XlO3zmn7ii/RKvuXX5Cqu6C27Bvofn3Lxc4s/kCjdmz6Ui
ju2dTRGHDb4rT3EZXT9mnT/gtnX72McN61pxQrfDLevK2MIN6Jrd9cuVuHHc
wh+sON+jf5U3uV4sYsL5F7lbd4sFrg1XsE53I6d1G/55ieuS19yVr72XS9ln
cM6elGdx2v6NQ7V3OpuVv8ykGeTesQNHdSPym8mYdu7P/AMQvzTE
       "]]}, {Line3DBox[CompressedData["
1:eJwVz0cuhXEUhvFjIK4g1w7EchiaGEosgDVot+u9MzHAlES5ei+36NZgE37f
4M15nvecL/l/nf2DPQMNEdEnvS0RR20Rx5JtjRgyG5siMvgEn0oeDyf79oiK
+xwv8zMp4p1URFVfwOe6CxnHu+5HcM2uxC/xlXTpJ/korttN4Gt8oE/JNB/j
b3ZTON0cccNnccZ818/gnNtbfifzPGv+pSP29HP8nj/IYnKb/If+w7cL/JE/
yTLe9/5P/RJ+1r3IatK7z+MvuxX+iivSrV/nBfxtt4ar+FBfMzd50fyx28Ad
3l/n27hk/uq38D9gB0K4
       "]], Line3DBox[CompressedData["
1:eJwVz0VSA1EUheHLCKqwbIH1kBEjprAA2AauwT0JDjEk2CpwXwJMcCu+DE7d
/7/ndffrpvbOlo6qiGiT1tqIRGPEi5mpi+ivjxiQ7UREmp/iV90yHsSFmogs
PsNv9qs45+wQH5YVfm6+69ZxXjfCR2WNX5hPvtdsv8lTfEw28KWZrPZtPI4n
ZAv3OHuFP7wzzyfxlORwr+4af+qKfBrPVO5pX+A3+Eu3g2fxvvuX8C3+tt/D
RWfn+Lzs8jvzR1fGJd0CX6w8y+/NZ/dP2h/yJZ6WA/xglu1/PXvMMzwrR7jP
/hH/6U54d4P/ki75Bx/ZSfY=
       "]], Line3DBox[CompressedData["
1:eJwV0NkuA2EchvF/qzraWi/BLTnjkCNbXQguw75r7UsIIYQQQghBqe0q+M3B
k/d93vkymW86e0e6ypmI6EN3KaKjLeJP7jdHTLRE9LdHZLHHX3nIQ0zqA/YG
HPA3npHHyNkGMWU74jWZlaeo2Kf5DE74u9xM36OfY5bP4Uz/kFue5fRLzPMF
XOgF+zYa9Wss2pdwpY/b63pev8WyPpSexQ3/5Im8x4o+bM/jjn/xJvmIxFbG
qu2Bf8uCfEbVXuFVPPEf2ZP4R/qavo4X/VfuOFtM74qN9L6o6UX7Lkp6HWOt
vh2j+Aef2kCL
       "]], Line3DBox[CompressedData["
1:eJwVz8dKAwEUhtHrOmViYu+xPJBZ2o1Rd8b2ehqNCIIgCIIgCIq4sSb2Ch4X
hxu+HyYzxXK1tNISERWWUxGFJGIsHXHNKJu5iBxLtt9MxIh2xTAb+o+2aPt2
i9oFQ6zbvrSK7dNtem5JO7cN8qEt2N7dPX1AO6OfN61sS/Q6fdopvbza5m1Z
fZce7YRuXmw72pz92e8u7ZhO1vQMs7YnW4d2RDurelObsTXcNu2QAlXbozZt
e3AbvmNcO7DludembHduTW/V9slxq03a0vo2iVYny41twpbSt/7fTauR5tJ/
5PkDc4E5QQ==
       "]], Line3DBox[CompressedData["
1:eJwV0Mcyg2EYR/EntsxgRu/R0pHo3IV2AcZCTbS4Xiz0migb7Wdx5nn/53yT
RZKblZVyIiK2UG2MKDRHLDdF1LGEvN3QGnGq5bwXuRoWcNYSkeVOtIw7z70i
4ftPO41jLeV+YMP3L/ocxu0jbcx9x7r2zM9i1D7USlz4rRnuCdMY0SrahTbs
XeIeUUTS/uXL+pD3FPeASQzaP9qBNuA9wd2jgCrfz+1rfW6eu8M3/2b3Yk/r
cetY42/1HLrtXa3LrWFVu+Gz6LR3tCL3hQx3jTQ6tG3tnG/3TnFX//8LLu02
/AFTOjDN
       "]], Line3DBox[CompressedData["
1:eJwVzEVSA1AQRdFO4QUUsAQ2gCXB3d0hITjFGGa4M4c9EpaA+2Fw673+t3/X
7uzP7yUiYhe35RFbVRHbSFRGbMoNlNVE3HBr+jqCW5X31RE5ec1l5Qp+KyJK
7Wf0ZVxxi3IJOfs//II+j0tuVs79/+W+uRl9Ghdc2luJW1PmL25STuCcy3Nj
+jg+uVE5gmL7Z/yQPowPblAOoIg75fr0frxzKXfu0Gs+4bplD964QvtdeieO
uXbZgaz9V75Nb8UR1yxbkOFeuLSewiGX9FbgVtL8zDXJRhxwD1y93oAn7lHm
UYc/WL4vhg==
       "]], Line3DBox[CompressedData["
1:eJwVzrkyg2EUgOGjNtlIYi9UXIMxGuMGXIKhoEgaCoVCEQUzEkt2JAjuyb5E
3IYnxZnzfe/zzz/f/EZxvTASEZumNGqnIrZMOhFRTkbU0hHP+qnztr5jMuzM
vc5eWMW5oBfNGFvQG+aVnbNdfc+Ms75+qb2xC3tfT2QilvQsr2nvrGofsCRb
ZjnW0D5Y3V7RUuzQN3nW0j5Z0/5lJf3ITLArral98bbzsX5iJtmNe4t9s2vn
sl4xU2xRb5sf1mFVvWam2UC/1fqsO/yHnvWeVX2G3w/fwe7sDsuxrj3LHrQB
69lrvs+zHptjT9ofe7T/AdgOOUw=
       "]], Line3DBox[CompressedData["
1:eJwVz9kuQ1EchfG/mQ7a8gRewhARQ/oEbrkqQWImQRCEhhatDgQxJHg2nsPv
XHxZe61vJ+fskdL2/FZHRCyjlY74zkX8IJeN+JJP+YgimtyH/olB7l0+2t9k
g3uRr8hyc/Y2nvUHrp3cxaItw7ecm6hzdTlciJjl0lxNv0eNm7ENcVU9xVXk
TeK5aa7AlfUB7kpeIm+748+dL9DPncmW+1O45U70U/Rxk8nbcKxXuUN5hF5u
wt7AgV7h9uQ+Fmw9/K7zTvJP3KbM+PY4181t6Ou45saSt3Frehe3KldQ5ka5
FLekd3J/8hcl/AMFJy9V
       "]], Line3DBox[CompressedData["
1:eJwVzzdSA1EURNFHTOEiiADhzT7IIGAJFDEshhRCQN57ZDCLQQJJuA1wJujq
/vfOn5pJXVyfX81FxKXczEfcL0U8yPpCRGYxYmK/L0ek7Uc7LRtczvkWH7mT
tTN4Vja5EV7Axlxe5/C8pLgxV8I+uKIu4EU5xbf4CvbJlXUJX12JOOG2uVry
PVw1uY+V+YrscA1sytWT+1wVr8ku10qcPcGbdt1uyB7Xcb7DZ+627Sbekn1u
ij9hX1xXt/GOHHAzro99cz3dxZ/kDD/kh9gPN9A9fM1/9PUR94L9cs/Juz0/
wIdyzL1hf9yr/gfKOkhT
       "]], Line3DBox[CompressedData["
1:eJwV0DczBHEcx+GfWi7plIyczcixZLwEo6bkhXhB4jkn55wzjex0ni0+s7ff
53Zm/1s2Njk6kRMR45rJjZgtjGjPi6jMj1gqiEjprShi0XWOdbAqlnY/ZS/X
st/zrJNVs3dbxraqFS2wLlbDPti6bUNrWmRZDdtr+ZZtW5taspcUR3R7to7t
2va0o0//T/EeVs8ObIfa11fyTqyXNbBj24m+7UeuadbHGtmZ+2l7hU6T92X9
rIn92C5slzpXhg2wZvbLrm03ukrOyv40Ym/hd7Z73SbntJc6x6BnW9mj7UkP
yibfgw+xNvZie9Wz/gEwGkck
       "]], Line3DBox[CompressedData["
1:eJwV0MkuQ2EcxuF/zXRibV4b1yIiIlRVVdUNiDVr4124LfN4D+aZmp4ufsn3
vU/PyUn7VzfK64mIWNN+MuIgG5FLRYylI24zEZvtEQO6cT5kc2yc3btv2Qd1
53zE8myCDdm29Wh/0DGbZ5Ps0/5se9GTTlhnR0SBT/E327tedcq62AKbrj1r
+9KHZr2nmxXZDPu2/aha+w078+wiy7E/245tWL/O56zE8izhvGsfUThfaIkV
2KhtT/W2Ol2qzIqsam90b1KDrtTje5Z5ibe4t6pZ1+plFVZmSfeU2mr/tff0
sRVWYRlbVmn9A79INik=
       "]], Line3DBox[CompressedData["
1:eJwV0GcuhFEcRvH/6GWa+t06REREREQE0c1gjM7MEtQ16N1W9CWwAHsgwc+H
J+fNOcl9c29HoTxaSkRE0e4bIz4yEeVkxHgqooSH2Yh2u9Oa0hHb3Ji2hQd8
lrvVMrjJjWgbuK+luRsthd/OzXPD+rqe5K61Rnzj17ghbRUbuCutjX+1FW5Q
W8Z67VJr5V+syA1oS1inPXMXeq3vAtevLeIe32LnWo22wPVp87jLV3NnWhXm
uV4thztaJXeqVeCXe+S4Hn1OT3AnWuATP8t1azP4i8daM/9o01wXN4U/ePT/
pvyDTXKd3AR++se7/QGjYy6i
       "]], Line3DBox[CompressedData["
1:eJwVzjVShFEQAOHZGC10cRbnOgQ4rECMu9tyDIoAd86B55BzCOwjmOp53e+X
VG6yZyIREePmviDioyQiXxgxVhSxh93FEW+lEXdal32Xy2k7uM93crdaB25z
We2VT5h27kZrw3DOmYy+5V4rd6214K9vZrW0tqmluCstyb2YDW5UW8dm7VI7
5Jvsa9yItoqNzs/8hd5gX+GGtWWsd37SzrU6+xI3pC1inq/lzrQaXOAGtUf+
x78luVOtGr+dM/yAPu9eFXeiVeKXltb6tTmtgjv+f457MLNcnzaD5dqRdsCX
2ae5Xm0KP73n3fwB6N8zOg==
       "]], Line3DBox[CompressedData["
1:eJwV0LcyBFAchfG/2ggtKq2cc845rKxkrbzLExkKDyBn7yBXaHkKP8U359zz
zdyZe4tXMol0VkQk8Zgd8ZUX8Y39nIjl3IglfORHPHAL+iL2uHl5aJ+T99yM
nEWGe7cn9GnccZNyCil7mp/Qx3HLjcoxrHG73Ig+jBuuyPaGIecdblAO4Jo7
tvfp/djmemUPXu1XfJfejS2uU3bghbvk2vR2bHKFtgO0Ol9wzbIFG9yzvUlv
xDlXLxuQtK/zdXotzrhqWYNVLsVV6ZU45QpsT6j4fydXLstwwh3ZS/TS/3u5
X3//g0/8ARoBMrg=
       "]], Line3DBox[CompressedData["
1:eJwV0FdOQmEUReHjs9E4ABu2AflqR1RsoDRBEOwORIw6C3t3APap+N2HlX32
Xjf5k5tK58dzHRGRwXtnxHF3xAlaXRFH8rInYgxv3IF+iCa3Ly/se/KVa8ld
7HCj9nM09ReukezI2hp83b2NZ64m/+wjqHNVfQtP3LDtF5Xke64sS3jkhuw/
KOo1riA38W174PPuDVS5nDyzp3DPrenryTvcoK2NVf2OW5YrqHAD9lNk9Vtu
US4l2Mr8gjuDG25eftn7UeLS+hyuuT7bJ2b1Ijcjp3HF9do/MKUXuEk5kfxL
9z+Bfjl8
       "]]}}},
   VertexNormals->CompressedData["
1:eJx0fXdcj9/7f1T2KqPIiqxSkmw6IZGszLKSiOyRQpEQIiEzKWlpG6FSnGRk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    "]],
  Axes->True,
  Method->{},
  PlotRange->{{-2.851436630796726, 2.851436748948321}, {-2.8514366898725956`, 
   2.8514366898725956`}, {-13.908009414000276`, 13.908009414000276`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.580538239785187*^9, 3.5805382492076035`*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"hebi", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5805378086628294`*^9, 3.580537841984488*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", "2"}], "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{
      RowBox[{"-", "2"}], "*", "Pi"}], ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotStyle", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", " ", "Thick"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5805370053394184`*^9, 3.580537014169034*^9}, {
  3.580537060937916*^9, 3.580537067817528*^9}, {3.580537124710828*^9, 
  3.580537129266036*^9}, {3.580537183912932*^9, 3.5805372013537626`*^9}, {
  3.58053724545504*^9, 3.580537278636298*^9}, {3.580537859518919*^9, 
  3.580537973726719*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJxdm3k0Vd/7x4XMmZK5cIlrKJEkZJ+EkExJKrNI0UjIVFJkqEyZZ1KGTJmn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     "]]}},
  Axes->True,
  Method->{},
  PlotRange->{{-0.999999512844876, 0.9999999999999671}, {-0.9999996658276197, 
   0.9999993650500513}, {-12.566370101446086`, 12.566370101446086`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.5805379759887233`*^9, {3.580538046298047*^9, 3.5805380728492937`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"%", ",", "hebi"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5805379850367393`*^9, 3.5805380673580837`*^9}}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {RGBColor[1, 0, 0], Thickness[Large], Line3DBox[CompressedData["
1:eJxdm3k0Vd/7x4XMmZK5cIlrKJEkZJ+EkExJKrNI0UjIVFJkqEyZZ1KGTJmn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      "]]}}, GraphicsComplex3DBox[CompressedData["
1:eJxcXXlUzd8XbVYpzRqluUzJlNm9psxkSkISQubM85Q5MoTIkIT4JiFTyb1I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    "], {{
      {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnQX0FdX3xee9mXlvZl5iYIuomICNLRaCioHxs1vBDuwGUcFE7MbuLhQT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          "]], Polygon3DBox[CompressedData["
1:eJwtnQn8FdMbxueuc++de+fesrULiShEFLJUiOyJotAiaylklyVJFNnF375H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          "]], Polygon3DBox[CompressedData["
1:eJwt13m4zmUaB/A3yznnfXnPESGRXWixTWFkPdaiC6ehvRNdkZKsqRw1lhY1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          "]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {Line3DBox[CompressedData["
1:eJwVzjssA1AYhuFfhGqlWovEYDBQCbrZbFWJMhkkFgODS0RjNJIUMYpBrNTm
MttcNmoQZpdNMRls4jG8eb///845Od2z5cmVhoiYw0FzRDoZ0YaHRESWM3iS
+3kAdTnPO9mIQf4yj3EJ0RJxYT8hj6PRPM8LWLXvMC/Ji+iUT1IRv/annMY5
zpDT3XJTe8Qd9+EeNeR1u+68y3UU8IkPZJwf0idafQUzdimuO5/kYV0P92Lt
/13+1uW4oCvwCPZ0Xd4alYso6Sr+vIl156fM2/IWpuVDrtkf8bL5mKsoy5f8
qLviDfMNX6MiV+2f5Rfsm9/4FT/4A0cFNBM=
        "]], Line3DBox[CompressedData["
1:eJwVz70rRQEch/Ef5fUKV5Ey2VBcRTbbLeqi3EkZlGx0sZqVt9gs/AEUC8Vs
QQwoVxYborxFBln43OHp+X17Tqdzmsens7miiJjAZmlEsiKiDvmyiAZerI2o
51s7xZ14dXfzitbFH/YgD6G4PCLLq9owl9iTfGRPcaM9w9Nocu9VRvxp+1yN
QxygRbvk0mTEFbcij2uktDeu1d45jS88es8n9+iJREQVxgrP8YtWw71aK7dh
Tuvgb62d01of92NdG+BfLcMZbck3n9rLPGKvFf4To+4tvtC2OWfv8g5m3cd8
o53wvH3OZ1hw3/Od9sAb9jM/4Qf/KmM3xg==
        "]], 
       Line3DBox[{1206, 882, 2014, 884, 4278, 2017, 922, 2036, 4279, 924, 
        2039, 961, 2058, 4283, 963, 2061, 1000, 4211, 2080, 1002, 2083, 1449, 
        4214, 2586, 1451, 2090, 4239, 1491, 2613, 1493, 2097, 4244, 1533, 
        2640, 1535, 4373, 2104, 1575, 2667, 1577, 4376, 2111, 1617, 2694, 
        4378, 1619, 2118, 1659, 2721, 4383, 1661, 2125, 1039, 4227, 2144, 
        1041, 2147, 1078, 4230, 2166, 1080, 2169, 4257, 1117, 2188, 1119, 
        2191, 4264, 1158, 2208, 1160, 1191}], Line3DBox[CompressedData["
1:eJwVzzkvhGEUhuFDgUGExhoM0ekkBrFEp5NYE0s1lEIjwdQSjYjGUopObals
f8FSSCwRf2BQyAwKl+LO/ZzzvHnf72tJL40uFkTEHA6KIhoTEU14LY5IcjPe
5F7uw5c8wJuVEf2cM0/xNMpLIs7tZ+UZVJhXeBXL9q3mjLyGNvmiNKKwKuKS
G3CNK7TrHjmhe+IUXvCMDt2Ou/LyN0bwix9UO9+jrymLqMWCXT1nna/jQV0n
p7Ch6+acrouHdGM8jkNd0l2T8gSG/9/0zbtYdz5t3pf3MC8f8439yf+/ms/4
FBn5hh90t7xlvuc7bMtH9ln5HYfmT/5AHn8FcTfs
        "]], Line3DBox[CompressedData["
1:eJwV0L0vQ1EYgPG3vlsROhiEDpUYDAaDoAv/gaRmBmIwqFnqD2C0NGGQUItB
xISIoSKRjiIknepjELEQgxiIX4cn7znPc8/NPTe7sJovJCJiEYn2iEl8d0Rs
4SEZcY9aT8RPW8SY9s4Xccbv86fmpzaq1fkl7HC72rb5po1od/wMrvl1vogn
bVjbTEWs8RtmrtM3oKYNaR98gV/BAN+PWy2rvWqzfB5/+EVVK/EZfcJ+HI+o
I5WOqOh9WqbxPlzh2fMV81zr1ZqsEzhonNXK5omW1m7coRuX/Lw7zOFQ69Iu
+OXGv7Ke5qewpyXtj/hj/cV6kM+ipLXal/mq2cI14wv/Ci84AQ==
        "]], Line3DBox[CompressedData["
1:eJwV0L0rxHEcwPGPp8Nd6RgMyslDBoPBoDws/AUSM1EGg7MpYVVslhuYhMEk
m0sGD6WzXYpu4jLJJIOkeP2Gd5/f5/3+9a3vt3shP71SExGLqG2MGMV3U8Qe
npsjnlDJRvykIoa1D34DRf4CZe1TG9Je+CU8cAfaPt61Qe2Rn8KdtsVvoqoN
aLvpiHV+xxzPRIyhovVrq9yaljdzfCfK2gHXo8/yMwj+zyxpBS2X3MU+gipe
kWmNuNY7tK7kPNziBnVaUWvX6pMdv845MY9xrrVp9+6QxZU27w5zONVatEt+
OXkr35P8BA61tP1MezP7uF4U+G3/prgjrWSm+AZ84R/vdzY3
        "]], Line3DBox[CompressedData["
1:eJwV0D8oxGEcgPGvyOHi7jqlGCSDckoYDGaKhYwmZOTY3WxhJjFYKKQUNxpu
EMsNFGW4QTqTv8ONyueGp+/7Ps/v/dX79i2vz+cbImIFjYmICfy1RBzgvTWi
ikqaa44Y1375LdzzJ/ydWdPGtCq/gTPuSDs1P7Vh7YVfqJ/jd/jt+r+1nLbX
FlHgd82ZZMQ0KtqAVuML/CaG+ByetH7tQ1viF9HBt6Os7fO9+hQ/iR98I5OJ
uNV7tJz9IB7x5vsH80br0tLWKVzjVbsyi1pWK7tDN0p83h3WcKGltBK/yjeZ
c/wsjrWkdskX+S/rUX4Eh1rC/rz+VtqzdSefrb8p/gG9qzk4
        "]], Line3DBox[CompressedData["
1:eJwVzy1IA1AYRuFPkG3+hG0ILiwoGAwTF1QMdlc0qNG0ZRWXLDNr0qjiqkFw
cZoMC4IoMznEsDjQIKKyKD6Gw8s958LlTpZ31rYHIqKCwWTEEn5TEed4H4p4
QzfNJSIWtS/+AI/8AzpaX5vTenwVT1xDu8KHVtRe+U3ca8f8EXpaQTsdjqjx
J3Z1JGIFXW1a2+f2tJot8rPoaHVuSq/wZWT5DNramTahlfhl/OD7/04m4k7P
azPOBXTwjKR2q+W0rHMGwd3YazS1Ma3tD3m0vFH1h100tLTW4rf4hN3g13Gh
jWpN7tMucPOo84fuprhL7cXm+HH08Qc6ejbw
        "]], Line3DBox[CompressedData["
1:eJwVz78rxHEcx/H3De78GtxfwCl/BTcZzCiUQVwyKBcmSQarsxjcYpAwXClk
urhBFGcTpZu4lBIlE5fyMDx7fT7P57dv329mKj80l4iIHKqpiFRLRNbeYq85
Yh/FZEQPP8C94Ig77Ig4trtavzbINzDLn/HnKGmT2jhf4dPcNW5woh20Rgzz
03qTcxIplLVLO6rltS7nDLpxobWnI+rOS1qv7UMWRc9X9URbxJo28v8OvPJj
9k7r1AravPsCPrVFW9Oy2pa27r7MF+wG6tqq/9jRrvhB33+PB7xp29oEX9Jn
uCc840srazntVFvh3vGBhlbjH/WKtsl94we/3B+VSz1Z
        "]], Line3DBox[CompressedData["
1:eJwV0LErhHEYwPGHgXO3yF9Acn/GRRmsLgOhdF2YMIgyyGAmMrAYpFy6Mtxl
umKQxTEpks2bUopkkUs+7/Dt+f2ez/W+d9dXWiwutEVEWbedEZmuiIJ5p5NM
xGl3RMU86IjIsxH7V9XszljdPGbDrGjf0oVdjV2aVVZio+5TrMfuRk3VWSUb
McZm03c7Z5VTg12b42yJ9TsPKK8rlpiTbI0VnAeVuA+ZTd6ei9hkE+nn9Ja+
37xnvWyLLbuv6Iutms+swPbZtvuOfthu+my24XdMux/xou//oEe9s0M2w6ps
3u5Fib5Zg5XZOVu3+9CnWuyJzaX/Fduz+033+rP/B6FePsc=
        "]], Line3DBox[CompressedData["
1:eJwVzytLQ3Ech/GfxXkpyjQYDL4BL8kXYFIRQRAxGLzMIswwrE7joticsy4o
jKUpBsMUQZDZhhdYnCAi4i3Oz8LD9/9/nnPgnJG17YV0R0Ss4zERMdQdMWWf
cNoVcYaTzohJfo77RIUr90Wc26K2qC3xCfc0X7XXKGkZbVWr8sNcDQ+oaBc9
3uW39KTzAAZxpdXtsrajjTqPYRy3WrI/4sc5q03bGcwi7/lau/dG5LQNLoV3
ftPWtQntUMu67+FX27cNbV4raEfuu3zeHqOpHfiPonbHr/j+Z7zgQyu1/5Ev
6xmuiVf8afdaSrvUctwXvtHSGvybfqMV2g7h3sI/FXc+3g==
        "]], Line3DBox[CompressedData["
1:eJwV0L0rBHAYwPHnDBwWg4RB+RO8DP4Ao45ukHTdcMlbYrnMEsuNsroz3oCu
y8BlsJBJmK68ZBKSpBxl8TF8e37P8+k6rj+3kl5ORMSMblsielsjRs077Scj
9joiDsxSc8QIS7l/6titympmmU2yKfek/UyH7NyssDxL23O8z+1aNzpitTaf
ZUusy7tbPTpldXOarbIB7yEN64I1zAxbY2PeKT3Zx80r3tkeUWBz9gW9s0Wz
zgbZNlu3b+ibbZqPbIIV2Y69pF+2az6zLf9H1l7mWX//vR70wSosx6os7/ai
V/2wSzbLTljB7UsNhf2Nzf//Vt5Ft4S9SeE7/wDnrD0+
        "]], Line3DBox[CompressedData["
1:eJwVz78rxHEcx/G3XHEnRTKZTsRgs1mcgRxJ+bEoBq6QclEm6jrF6haJhToL
xeoWioVBSUZW5F9QF48b3j1f7+fr8/18v9/0Yn5qvS4icuaoMeIjGfHQEHEo
3+O7vdwSUbTfyRWuIN/gG3/LrdnP8Jpbla/wWXfKzdp38Zyblgt8WS5yo/ZM
U8SJvaM1opSKOJYHuSFdJ046X+K2dAeY5gZ0CZzQ7XNzuj2s5/p1ze764nZq
d+E2fmLJ+T79k7zJdeEGJpx/lLt1l7jCJXEZq565kNt1Od+8wLXJeX5eXuL+
cARnsMJXa/9qhrkf7MVxfkz+NlnTw71iCrO6jPxS+97au7lf8w/y2DSR
        "]], Line3DBox[CompressedData["
1:eJwVz70rxAEcx/HvDR7vEqUuZRERZbOZ3IKY5GESEeXKQ6EsdHXEpFBKlAgL
xWI4C8XCpMysrvwJSl43fHu/P5/P79fdr2lqcWghERHT7qgy4qsq4qUi4pA/
46d8URuRl5/4gy7HC/gon9rm5XN+p8vyLd0tP9ONyZt4JQ/znO2S53X9ciYZ
cSI31kUcVEcc8x5dxtaC+3KDbc22x5t13ba0rpzv6MZt21gmd9lqbEXdhi6D
6/hdet9vd9rf+IquDZfxFXdtrbYbntWlcFU3x6952jbrP0/K9XzJNsFndAm5
D0flgv4PR1yv7gc7cBCLbsC1yx9479kUz/D30ne7pPzr/gFyfzNm
        "]], Line3DBox[CompressedData["
1:eJwVz7svg2EYhvHHpNWFxGEgpKNNkAiDrQZxqC4Oi1CDRSUSalVjmYrRYRI2
h81hZdNNSMRfwCLCYPDr8OS6n+t+837fm15czRXqIiJvThIRn8mIx/qII/kB
P+ynjRG79qp8z5XlO3zmn7ii/RKvuXX5Cqu6C27Bvofn3Lxc4s/kCjdmz6Ui
ju2dTRGHDb4rT3EZXT9mnT/gtnX72McN61pxQrfDLevK2MIN6Jrd9cuVuHHc
wh+sON+jf5U3uV4sYsL5F7lbd4sFrg1XsE53I6d1G/55ieuS19yVr72XS9ln
cM6elGdx2v6NQ7V3OpuVv8ykGeTesQNHdSPym8mYdu7P/AMQvzTE
        "]]}, {Line3DBox[CompressedData["
1:eJwVz0cuhXEUhvFjIK4g1w7EchiaGEosgDVot+u9MzHAlES5ei+36NZgE37f
4M15nvecL/l/nf2DPQMNEdEnvS0RR20Rx5JtjRgyG5siMvgEn0oeDyf79oiK
+xwv8zMp4p1URFVfwOe6CxnHu+5HcM2uxC/xlXTpJ/korttN4Gt8oE/JNB/j
b3ZTON0cccNnccZ818/gnNtbfifzPGv+pSP29HP8nj/IYnKb/If+w7cL/JE/
yTLe9/5P/RJ+1r3IatK7z+MvuxX+iivSrV/nBfxtt4ar+FBfMzd50fyx28Ad
3l/n27hk/uq38D9gB0K4
        "]], Line3DBox[CompressedData["
1:eJwVz0VSA1EUheHLCKqwbIH1kBEjprAA2AauwT0JDjEk2CpwXwJMcCu+DE7d
/7/ndffrpvbOlo6qiGiT1tqIRGPEi5mpi+ivjxiQ7UREmp/iV90yHsSFmogs
PsNv9qs45+wQH5YVfm6+69ZxXjfCR2WNX5hPvtdsv8lTfEw28KWZrPZtPI4n
ZAv3OHuFP7wzzyfxlORwr+4af+qKfBrPVO5pX+A3+Eu3g2fxvvuX8C3+tt/D
RWfn+Lzs8jvzR1fGJd0CX6w8y+/NZ/dP2h/yJZ6WA/xglu1/PXvMMzwrR7jP
/hH/6U54d4P/ki75Bx/ZSfY=
        "]], Line3DBox[CompressedData["
1:eJwV0NkuA2EchvF/qzraWi/BLTnjkCNbXQguw75r7UsIIYQQQghBqe0q+M3B
k/d93vkymW86e0e6ypmI6EN3KaKjLeJP7jdHTLRE9LdHZLHHX3nIQ0zqA/YG
HPA3npHHyNkGMWU74jWZlaeo2Kf5DE74u9xM36OfY5bP4Uz/kFue5fRLzPMF
XOgF+zYa9Wss2pdwpY/b63pev8WyPpSexQ3/5Im8x4o+bM/jjn/xJvmIxFbG
qu2Bf8uCfEbVXuFVPPEf2ZP4R/qavo4X/VfuOFtM74qN9L6o6UX7Lkp6HWOt
vh2j+Aef2kCL
        "]], Line3DBox[CompressedData["
1:eJwVz8dKAwEUhtHrOmViYu+xPJBZ2o1Rd8b2ehqNCIIgCIIgCIq4sSb2Ch4X
hxu+HyYzxXK1tNISERWWUxGFJGIsHXHNKJu5iBxLtt9MxIh2xTAb+o+2aPt2
i9oFQ6zbvrSK7dNtem5JO7cN8qEt2N7dPX1AO6OfN61sS/Q6fdopvbza5m1Z
fZce7YRuXmw72pz92e8u7ZhO1vQMs7YnW4d2RDurelObsTXcNu2QAlXbozZt
e3AbvmNcO7DludembHduTW/V9slxq03a0vo2iVYny41twpbSt/7fTauR5tJ/
5PkDc4E5QQ==
        "]], Line3DBox[CompressedData["
1:eJwV0Mcyg2EYR/EntsxgRu/R0pHo3IV2AcZCTbS4Xiz0migb7Wdx5nn/53yT
RZKblZVyIiK2UG2MKDRHLDdF1LGEvN3QGnGq5bwXuRoWcNYSkeVOtIw7z70i
4ftPO41jLeV+YMP3L/ocxu0jbcx9x7r2zM9i1D7USlz4rRnuCdMY0SrahTbs
XeIeUUTS/uXL+pD3FPeASQzaP9qBNuA9wd2jgCrfz+1rfW6eu8M3/2b3Yk/r
cetY42/1HLrtXa3LrWFVu+Gz6LR3tCL3hQx3jTQ6tG3tnG/3TnFX//8LLu02
/AFTOjDN
        "]], Line3DBox[CompressedData["
1:eJwVzEVSA1AQRdFO4QUUsAQ2gCXB3d0hITjFGGa4M4c9EpaA+2Fw673+t3/X
7uzP7yUiYhe35RFbVRHbSFRGbMoNlNVE3HBr+jqCW5X31RE5ec1l5Qp+KyJK
7Wf0ZVxxi3IJOfs//II+j0tuVs79/+W+uRl9Ghdc2luJW1PmL25STuCcy3Nj
+jg+uVE5gmL7Z/yQPowPblAOoIg75fr0frxzKXfu0Gs+4bplD964QvtdeieO
uXbZgaz9V75Nb8UR1yxbkOFeuLSewiGX9FbgVtL8zDXJRhxwD1y93oAn7lHm
UYc/WL4vhg==
        "]], Line3DBox[CompressedData["
1:eJwVzrkyg2EUgOGjNtlIYi9UXIMxGuMGXIKhoEgaCoVCEQUzEkt2JAjuyb5E
3IYnxZnzfe/zzz/f/EZxvTASEZumNGqnIrZMOhFRTkbU0hHP+qnztr5jMuzM
vc5eWMW5oBfNGFvQG+aVnbNdfc+Ms75+qb2xC3tfT2QilvQsr2nvrGofsCRb
ZjnW0D5Y3V7RUuzQN3nW0j5Z0/5lJf3ITLArral98bbzsX5iJtmNe4t9s2vn
sl4xU2xRb5sf1mFVvWam2UC/1fqsO/yHnvWeVX2G3w/fwe7sDsuxrj3LHrQB
69lrvs+zHptjT9ofe7T/AdgOOUw=
        "]], Line3DBox[CompressedData["
1:eJwVz9kuQ1EchfG/mQ7a8gRewhARQ/oEbrkqQWImQRCEhhatDgQxJHg2nsPv
XHxZe61vJ+fskdL2/FZHRCyjlY74zkX8IJeN+JJP+YgimtyH/olB7l0+2t9k
g3uRr8hyc/Y2nvUHrp3cxaItw7ecm6hzdTlciJjl0lxNv0eNm7ENcVU9xVXk
TeK5aa7AlfUB7kpeIm+748+dL9DPncmW+1O45U70U/Rxk8nbcKxXuUN5hF5u
wt7AgV7h9uQ+Fmw9/K7zTvJP3KbM+PY4181t6Ou45saSt3Frehe3KldQ5ka5
FLekd3J/8hcl/AMFJy9V
        "]], Line3DBox[CompressedData["
1:eJwVzzdSA1EURNFHTOEiiADhzT7IIGAJFDEshhRCQN57ZDCLQQJJuA1wJujq
/vfOn5pJXVyfX81FxKXczEfcL0U8yPpCRGYxYmK/L0ek7Uc7LRtczvkWH7mT
tTN4Vja5EV7Axlxe5/C8pLgxV8I+uKIu4EU5xbf4CvbJlXUJX12JOOG2uVry
PVw1uY+V+YrscA1sytWT+1wVr8ku10qcPcGbdt1uyB7Xcb7DZ+627Sbekn1u
ij9hX1xXt/GOHHAzro99cz3dxZ/kDD/kh9gPN9A9fM1/9PUR94L9cs/Juz0/
wIdyzL1hf9yr/gfKOkhT
        "]], Line3DBox[CompressedData["
1:eJwV0DczBHEcx+GfWi7plIyczcixZLwEo6bkhXhB4jkn55wzjex0ni0+s7ff
53Zm/1s2Njk6kRMR45rJjZgtjGjPi6jMj1gqiEjprShi0XWOdbAqlnY/ZS/X
st/zrJNVs3dbxraqFS2wLlbDPti6bUNrWmRZDdtr+ZZtW5taspcUR3R7to7t
2va0o0//T/EeVs8ObIfa11fyTqyXNbBj24m+7UeuadbHGtmZ+2l7hU6T92X9
rIn92C5slzpXhg2wZvbLrm03ukrOyv40Ym/hd7Z73SbntJc6x6BnW9mj7UkP
yibfgw+xNvZie9Wz/gEwGkck
        "]], Line3DBox[CompressedData["
1:eJwV0MkuQ2EcxuF/zXRibV4b1yIiIlRVVdUNiDVr4124LfN4D+aZmp4ufsn3
vU/PyUn7VzfK64mIWNN+MuIgG5FLRYylI24zEZvtEQO6cT5kc2yc3btv2Qd1
53zE8myCDdm29Wh/0DGbZ5Ps0/5se9GTTlhnR0SBT/E327tedcq62AKbrj1r
+9KHZr2nmxXZDPu2/aha+w078+wiy7E/245tWL/O56zE8izhvGsfUThfaIkV
2KhtT/W2Ol2qzIqsam90b1KDrtTje5Z5ibe4t6pZ1+plFVZmSfeU2mr/tff0
sRVWYRlbVmn9A79INik=
        "]], Line3DBox[CompressedData["
1:eJwV0GcuhFEcRvH/6GWa+t06REREREQE0c1gjM7MEtQ16N1W9CWwAHsgwc+H
J+fNOcl9c29HoTxaSkRE0e4bIz4yEeVkxHgqooSH2Yh2u9Oa0hHb3Ji2hQd8
lrvVMrjJjWgbuK+luRsthd/OzXPD+rqe5K61Rnzj17ghbRUbuCutjX+1FW5Q
W8Z67VJr5V+syA1oS1inPXMXeq3vAtevLeIe32LnWo22wPVp87jLV3NnWhXm
uV4thztaJXeqVeCXe+S4Hn1OT3AnWuATP8t1azP4i8daM/9o01wXN4U/ePT/
pvyDTXKd3AR++se7/QGjYy6i
        "]], Line3DBox[CompressedData["
1:eJwVzjVShFEQAOHZGC10cRbnOgQ4rECMu9tyDIoAd86B55BzCOwjmOp53e+X
VG6yZyIREePmviDioyQiXxgxVhSxh93FEW+lEXdal32Xy2k7uM93crdaB25z
We2VT5h27kZrw3DOmYy+5V4rd6214K9vZrW0tqmluCstyb2YDW5UW8dm7VI7
5Jvsa9yItoqNzs/8hd5gX+GGtWWsd37SzrU6+xI3pC1inq/lzrQaXOAGtUf+
x78luVOtGr+dM/yAPu9eFXeiVeKXltb6tTmtgjv+f457MLNcnzaD5dqRdsCX
2ae5Xm0KP73n3fwB6N8zOg==
        "]], Line3DBox[CompressedData["
1:eJwV0LcyBFAchfG/2ggtKq2cc845rKxkrbzLExkKDyBn7yBXaHkKP8U359zz
zdyZe4tXMol0VkQk8Zgd8ZUX8Y39nIjl3IglfORHPHAL+iL2uHl5aJ+T99yM
nEWGe7cn9GnccZNyCil7mp/Qx3HLjcoxrHG73Ig+jBuuyPaGIecdblAO4Jo7
tvfp/djmemUPXu1XfJfejS2uU3bghbvk2vR2bHKFtgO0Ol9wzbIFG9yzvUlv
xDlXLxuQtK/zdXotzrhqWYNVLsVV6ZU45QpsT6j4fydXLstwwh3ZS/TS/3u5
X3//g0/8ARoBMrg=
        "]], Line3DBox[CompressedData["
1:eJwV0FdOQmEUReHjs9E4ABu2AflqR1RsoDRBEOwORIw6C3t3APap+N2HlX32
Xjf5k5tK58dzHRGRwXtnxHF3xAlaXRFH8rInYgxv3IF+iCa3Ly/se/KVa8ld
7HCj9nM09ReukezI2hp83b2NZ64m/+wjqHNVfQtP3LDtF5Xke64sS3jkhuw/
KOo1riA38W174PPuDVS5nDyzp3DPrenryTvcoK2NVf2OW5YrqHAD9lNk9Vtu
US4l2Mr8gjuDG25eftn7UeLS+hyuuT7bJ2b1Ijcjp3HF9do/MKUXuEk5kfxL
9z+Bfjl8
        "]]}}},
    VertexNormals->CompressedData["
1:eJx0fXdcj9/7f1T2KqPIiqxSkmw6IZGszLKSiOyRQpEQIiEzKWlpG6FSnGRk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     "]]},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{461.0475855723832, 125.86727311286316`},
  Method->{},
  PlotRange->All,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{3.3303110387592656`, 0.5951025487565658, -0.06986659846894527},
  ViewVertical->{2.1423435793209284`, 
   4.375438448412838, -0.04866520920065459}]], "Output",
 CellChangeTimes->{
  3.5805380198716*^9, {3.580538051726856*^9, 3.5805380770769005`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["brioschicurvature", "Subsection",
 CellChangeTimes->{3.5805399899210606`*^9}],

Cell[BoxData[
 RowBox[{"?", "brioschicurvature"}]], "Input",
 CellChangeTimes->{{3.5823666250418177`*^9, 3.58236662646142*^9}, 
   3.5823800187533684`*^9}],

Cell["\<\
This old definition of Alfred is clumsy, since the deciding functions \
ee,ff,gg depending on the surface must be defined before applying the \
function. Using Alfreds next definition I defined the Module \
gaussbrioschi[x][u,v] calculating with Brioschi\[CloseCurlyQuote]s formula \
the Gauss curvature of the surface x[u,v] in arbitrary dimension n>1, see \
below.\
\>", "Text",
 CellChangeTimes->{{3.5806462141326804`*^9, 3.58064632514003*^9}, {
  3.580646356019796*^9, 3.5806467259479547`*^9}, {3.58064698864498*^9, 
  3.5806469962444143`*^9}, {3.5806470691485844`*^9, 3.5806471185564103`*^9}, {
  3.5823669194767346`*^9, 3.5823669263095465`*^9}}],

Cell["gaussbrioschi is the Module", "Text",
 CellChangeTimes->{{3.5806467602599173`*^9, 3.580646798396098*^9}}],

Cell[BoxData[
 RowBox[{"?", "gaussbrioschi"}]], "Input",
 CellChangeTimes->{{3.582366818794158*^9, 3.5823668202605605`*^9}, 
   3.5823800187533684`*^9}],

Cell[CellGroupData[{

Cell["Example. Sphere of radius r", "Subsubsection",
 CellChangeTimes->{{3.580794129295659*^9, 3.5807941542089024`*^9}, 
   3.5823670204557123`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sphe", "[", "r_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
  RowBox[{"r", "*", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "u", "]"}], "*", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Sin", "[", "u", "]"}], "*", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", " ", 
     RowBox[{"Sin", "[", "v", "]"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.580623878880844*^9, 3.5806239817006245`*^9}, {
  3.580624023165498*^9, 3.5806240361603203`*^9}, {3.5806241054244423`*^9, 
  3.5806241257980776`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussbrioschi", "[", 
   RowBox[{"sphe", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.58062405300835*^9, 3.5806240769699917`*^9}, {
  3.5806241415853057`*^9, 3.5806241957486005`*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sec", "[", "v", "]"}], "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["r", "6"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
     RowBox[{
      SuperscriptBox["r", "6"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], "-", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}], ")"}]}]}], ")"}]}], 
  SuperscriptBox["r", "8"]]], "Output",
 CellChangeTimes->{3.580624197776604*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"r", ">", "0"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.580624210989827*^9, 3.580624228867459*^9}}],

Cell[BoxData[
 FractionBox["1", 
  SuperscriptBox["r", "2"]]], "Output",
 CellChangeTimes->{3.5806242312854633`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "u", "]"}], "*", 
     RowBox[{"Cos", "[", "v", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Sin", "[", "u", "]"}], "*", 
     RowBox[{"Cos", "[", "v", "]"}]}], ",", " ", 
    RowBox[{"Sin", "[", "v", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.58056062579072*^9, 3.5805606866742024`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"e1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", "u"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5805617857460656`*^9, 3.5805618500657444`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"e2", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"u", ",", "v"}], "]"}], ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5805617857460656`*^9, 3.5805618767132683`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ff1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"e1", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"e2", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.580561889095977*^9, 3.580561966553407*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ee1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"e1", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"e1", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.580561889095977*^9, 3.5805619468222785`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gg1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"e2", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ".", 
   RowBox[{"e2", "[", 
    RowBox[{"u", ",", "v"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.580561889095977*^9, 3.5805619468222785`*^9}, {
  3.580561984118412*^9, 3.5805620107289343`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ee1", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805620417767096`*^9, 3.5805620490801277`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "u", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.580562051451263*^9, 3.580562408430681*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805620881513624`*^9, 3.5805620956247897`*^9}}],

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"Cos", "[", "v", "]"}], "2"]], "Output",
 CellChangeTimes->{3.580562096635847*^9, 3.580562408458683*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ff1", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805621088895483`*^9, 3.5805621171920233`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5805621196831656`*^9, 3.580562408474684*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gg1", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805621230443583`*^9, 3.580562130616791*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "v", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.580562134517014*^9, 3.580562408491685*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805621378662057`*^9, 3.580562145408637*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.5805624085086856`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"brioschicurvature", "[", 
   RowBox[{"ee1", ",", "ff1", ",", "gg1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5805405133175797`*^9, 3.5805405528012495`*^9}, {
  3.580559954529326*^9, 3.5805599665130115`*^9}, {3.580559999808916*^9, 
  3.5805600179039507`*^9}, {3.5805604129125443`*^9, 3.5805604178728275`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        RowBox[{"Cos", "[", "v", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cos", "[", "v", "]"}], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}], " ", 
     RowBox[{"Sin", "[", "v", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        RowBox[{"Cos", "[", "v", "]"}], " ", 
        RowBox[{"Sin", "[", "v", "]"}]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"Cos", "[", "v", "]"}], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        RowBox[{"Sin", "[", "v", "]"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "u", "]"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "u", "]"}], "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "4"], "-", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "4"]}], "-", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "4"]}]}], ")"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"]}]}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}], "2"]}], 
   ")"}]}]], "Output",
 CellChangeTimes->{3.5805622238471236`*^9, 3.5805624151860676`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.580562227476331*^9, 3.5805622344807315`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.5805622383719544`*^9, 3.5805624190132866`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"brioschicurvature", "[", "x", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{
  3.5805441869360504`*^9, {3.5805605629691267`*^9, 3.5805605712455997`*^9}, 
   3.5805606094487853`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"Sec", "[", "v", "]"}], "4"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"], "-", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "2"]}], ")"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.5805441895071974`*^9, 3.5805600686068506`*^9, 
  3.5805604633904314`*^9, 3.5805605738227477`*^9, 3.5805607139127603`*^9, 
  3.5805624231925254`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805624299679127`*^9, 3.5805624370493183`*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.580562439218442*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Boy\[CloseCurlyQuote]s surface", "Subsubsection",
 CellChangeTimes->{{3.580794179792948*^9, 3.580794191992169*^9}}],

Cell["\<\
The gauss curvature of Boy\[CloseCurlyQuote]s surface could not be calculated:\
\>", "Text",
 CellChangeTimes->{{3.5806488039168077`*^9, 3.5806488428760357`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"boy", "[", 
   RowBox[{"4", ",", "4", ",", "1"}], "]"}], "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "Pi"}], "/", "2"}], ",", 
   RowBox[{"Pi", "/", "2"}], ",", 
   RowBox[{
    RowBox[{"-", "Pi"}], "/", "2"}], ",", 
   RowBox[{"Pi", "/", "2"}]}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"boy", "[", 
   RowBox[{"4", ",", "4", ",", "1"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806393705900803`*^9, 3.5806393774384923`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "2"]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "2"]}], "-", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"4", " ", "u"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "4"]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"Sin", "[", "u", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{"2", " ", 
    SqrtBox["3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "2"]}], "-", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{"4", " ", "u"}], "]"}], " ", 
       SuperscriptBox[
        RowBox[{"Sin", "[", "v", "]"}], "4"]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], "+", 
      RowBox[{
       RowBox[{"Sin", "[", "u", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "4"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", "u", "]"}], "-", 
         RowBox[{"Sin", "[", "u", "]"}]}], ")"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Cos", "[", "v", "]"}]}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "u", "]"}], " ", 
          RowBox[{"Sin", "[", "v", "]"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", "v", "]"}], "-", 
         RowBox[{
          RowBox[{"Sin", "[", "u", "]"}], " ", 
          RowBox[{"Sin", "[", "v", "]"}]}]}], ")"}]}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", "v", "]"}], "+", 
         RowBox[{
          RowBox[{"Cos", "[", "u", "]"}], " ", 
          RowBox[{"Sin", "[", "v", "]"}]}], "+", 
         RowBox[{
          RowBox[{"Sin", "[", "u", "]"}], " ", 
          RowBox[{"Sin", "[", "v", "]"}]}]}], ")"}], "3"]}], ")"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5806393896221137`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"gaussbrioschi", "[", 
    RowBox[{"boy", "[", 
     RowBox[{"4", ",", "4", ",", "1"}], "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.5805413631914725`*^9, {3.580625030006866*^9, 3.5806250789285517`*^9}}],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   PanelBox[GridBox[{
      {
       StyleBox[
        StyleBox[
         DynamicBox[ToBoxes[
           FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
           StandardForm],
          ImageSizeCache->{794., {6., 24.}}],
         StripOnInput->False,
         DynamicUpdating->True], "Panel",
        StripOnInput->False,
        Background->None]},
      {
       ItemBox[
        TagBox[
         FractionBox[
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "128"]}], " ", 
            RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
            RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"(", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
               " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"256", " ", 
                  SuperscriptBox[
                   RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
                  RowBox[{"(", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                   ")"}], " ", 
                  RowBox[{"(", 
                   RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                   ")"}]}], "-", 
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                ")"}]}], "-", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"8", " ", 
                  RowBox[{"\[LeftSkeleton]", "3", "\[RightSkeleton]"}]}], "+", 
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                ")"}], " ", 
               RowBox[{"(", 
                RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
                ")"}]}]}], ")"}]}], 
           RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], 
           RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], "2"]],
         Short[#, 5]& ],
        Background->GrayLevel[1],
        BaseStyle->{Deployed -> False},
        Frame->True,
        FrameStyle->GrayLevel[0, 0.2],
        StripOnInput->False]},
      {
       RowBox[{
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
             StandardForm],
            ImageSizeCache->{143., {1., 24.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 18, 24454857139365139547, 5/2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
             StandardForm],
            ImageSizeCache->{152., {1., 24.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 18, 24454857139365139547, 5 2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
             StandardForm],
            ImageSizeCache->{234., {6., 24.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 18, 24454857139365139547, Infinity],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
             StandardForm],
            ImageSizeCache->{209., {1., 23.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>FrontEndExecute[{
            FrontEnd`SetOptions[
            FrontEnd`$FrontEnd, 
             FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
            FrontEnd`FrontEndToken["PreferencesDialog"]}],
         Evaluator->None,
         Method->"Preemptive"]}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[1.2]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    DefaultBaseStyle->{},
    FrameMargins->5],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[18]]], "Output",
 CellChangeTimes->{3.5806250951681805`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%%", ",", " ", 
   RowBox[{
    RowBox[{"u", ">", "0"}], "&&", 
    RowBox[{"v", ">", "0"}]}], ",", " ", 
   RowBox[{"TimeConstraint", " ", "\[Rule]", " ", "Infinity"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.580625105386198*^9, 3.5806251429822645`*^9}, {
  3.5806256458335476`*^9, 3.580625703116848*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Simplify", "::", "time"}], "MessageName"], ":", 
  " ", "\<\"Time spent on a transformation exceeded \[NoBreak]\\!\\(300.`\\)\
\[NoBreak] seconds, and the transformation was aborted. Increasing the value \
of TimeConstraint option may improve the result of simplification. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Simplify/time\\\", \
ButtonNote -> \\\"Simplify::time\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.580625452736408*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.5806257571709433`*^9}]
}, Closed]],

Cell["\<\
after 4 h simplification without result. Try\
\>", "Text",
 CellChangeTimes->{{3.5806392437930574`*^9, 3.580639274649912*^9}, 
   3.580646003813651*^9, {3.5823672535981216`*^9, 3.582367256608927*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussiancurvature", "[", 
   RowBox[{"boy", "[", 
    RowBox[{"4", ",", "4", ",", "1"}], "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.582367114289877*^9, 3.5823671461919327`*^9}}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.5823672618505363`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Circular Cylinder", "Subsubsection",
 CellChangeTimes->{{3.580648865597336*^9, 3.5806488719967017`*^9}, {
  3.58079422125782*^9, 3.5807942289018335`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"circularcylinder", "[", "1", "]"}], "[", 
  RowBox[{
   RowBox[{"-", "Pi"}], ",", "Pi", ",", 
   RowBox[{"-", "4"}], ",", "4"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806395380719748`*^9, 3.5806395417223806`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussbrioschi", "[", 
   RowBox[{"circularcylinder", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806396021256866`*^9, 3.580639609098899*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.580639653761778*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["hyperbolicparaboloid", "Subsubsection",
 CellChangeTimes->{3.5807942847031317`*^9}],

Cell[BoxData[
 RowBox[{"hyperbolicparaboloid", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "1", ",", 
   RowBox[{"-", "1"}], ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806397123398805`*^9, 3.5806397142118835`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hyperbolicparaboloid", "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{3.5806398371869*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{"u", " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580639838871703*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussbrioschi", "[", "hyperbolicparaboloid", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806396021256866`*^9, 3.580639609098899*^9}, 
   3.5806397835384054`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["1", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["u", "2"]}], " ", 
       SuperscriptBox["v", "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox["u", "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox["v", "2"]}], ")"}]}]}], ")"}], "2"]]}]], "Output",
 CellChangeTimes->{3.5806398475453176`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5806399870251627`*^9, 3.5806399963851795`*^9}, {
  3.580640029878438*^9, 3.580640073059314*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["handkerchief", "Subsubsection",
 CellChangeTimes->{3.5807943217999973`*^9}],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{"handkerchief", "[", "1", "]"}], "[", 
   RowBox[{
    RowBox[{"-", "1"}], ",", "1", ",", 
    RowBox[{"-", "1"}], ",", "1", ",", 
    RowBox[{"PlotRange", "->", "All"}]}], "]"}],
  FontWeight->"Bold"]], "Input",
 CellChangeTimes->{{3.5806415274275007`*^9, 3.5806415327468047`*^9}},
 FontWeight->"Plain"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"handkerchief", "[", "1", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.580641624611059*^9, 3.580641628506282*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{
    SuperscriptBox["u", "2"], "+", 
    FractionBox[
     SuperscriptBox["u", "3"], "3"], "-", 
    SuperscriptBox["v", "2"], "+", 
    RowBox[{"u", " ", 
     SuperscriptBox["v", "2"]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.580641631078429*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussbrioschi", "[", 
   RowBox[{"handkerchief", "[", "1", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5806396021256866`*^9, 3.580639609098899*^9}, 
   3.5806397835384054`*^9, 3.5806416180766854`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "4"}], "+", 
    RowBox[{"4", " ", 
     SuperscriptBox["u", "2"]}], "-", 
    RowBox[{"4", " ", 
     SuperscriptBox["v", "2"]}], "-", 
    RowBox[{"16", " ", 
     SuperscriptBox["u", "2"], " ", 
     SuperscriptBox["v", "2"]}], "-", 
    RowBox[{"16", " ", 
     SuperscriptBox["u", "3"], " ", 
     SuperscriptBox["v", "2"]}], "-", 
    RowBox[{"4", " ", 
     SuperscriptBox["u", "4"], " ", 
     SuperscriptBox["v", "2"]}], "-", 
    RowBox[{"16", " ", 
     SuperscriptBox["v", "4"]}], "+", 
    RowBox[{"16", " ", "u", " ", 
     SuperscriptBox["v", "4"]}], "-", 
    RowBox[{"24", " ", 
     SuperscriptBox["u", "2"], " ", 
     SuperscriptBox["v", "4"]}], "-", 
    RowBox[{"4", " ", 
     SuperscriptBox["v", "6"]}], "-", 
    RowBox[{"2", " ", "v", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "8"}], " ", 
        SuperscriptBox["u", "2"], " ", "v"}], "-", 
       RowBox[{"8", " ", 
        SuperscriptBox["u", "3"], " ", "v"}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["u", "4"], " ", "v"}], "-", 
       RowBox[{"8", " ", 
        SuperscriptBox["v", "3"]}], "+", 
       RowBox[{"8", " ", "u", " ", 
        SuperscriptBox["v", "3"]}], "-", 
       RowBox[{"12", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "3"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["v", "5"]}]}], ")"}]}]}], ")"}], "/", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "4"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "u"}], ")"}], "2"], " ", 
      SuperscriptBox["v", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "u"}], "+", 
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "2"]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"4", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", "u"}], ")"}], "2"], " ", 
         SuperscriptBox["v", "2"]}]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", "u"}], "+", 
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "2"]}], ")"}]}]}], ")"}], 
   "2"]}]], "Output",
 CellChangeTimes->{3.5806416466673207`*^9, 3.580646110537755*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5806416606141186`*^9, 3.580641668402564*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     SuperscriptBox["u", "2"], "-", 
     SuperscriptBox["v", "2"]}], ")"}]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"4", " ", 
      SuperscriptBox["u", "3"]}], "+", 
     SuperscriptBox["u", "4"], "+", 
     RowBox[{"4", " ", 
      SuperscriptBox["v", "2"]}], "-", 
     RowBox[{"4", " ", "u", " ", 
      SuperscriptBox["v", "2"]}], "+", 
     SuperscriptBox["v", "4"], "+", 
     RowBox[{
      SuperscriptBox["u", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"4", "+", 
        RowBox[{"6", " ", 
         SuperscriptBox["v", "2"]}]}], ")"}]}]}], ")"}], "2"]]], "Output",
 CellChangeTimes->{3.5806416695386286`*^9, 3.5806461146609907`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5806399870251627`*^9, 3.5806399963851795`*^9}, {
   3.580640029878438*^9, 3.580640073059314*^9}, {3.580641728115979*^9, 
   3.58064174578799*^9}, 3.5806460949838657`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Commutator", "Subsection",
 CellChangeTimes->{{3.5812435814829082`*^9, 3.5812435878182707`*^9}}],

Cell["\<\
Define a vector field in an open region of  arbitrary dimension:\
\>", "Text",
 CellChangeTimes->{{3.5810487808878813`*^9, 3.5810487990619135`*^9}, {
  3.5811657050616984`*^9, 3.5811657149209156`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"vectorfield", "[", "f_", "]"}], "[", "x___", "]"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"xx", " ", "=", " ", 
       RowBox[{"{", "x", "}"}]}], ",", "ii", ",", "dd"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dd", "=", " ", 
      RowBox[{"Length", "[", "xx", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "ii", "]"}], "[", "x", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"ii", ",", "1", ",", "dd"}], "}"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5810473499129677`*^9, 3.5810473552481775`*^9}, {
   3.581047386432632*^9, 3.5810474829188013`*^9}, {3.5810475329324894`*^9, 
   3.5810475627909417`*^9}, {3.5810476000750074`*^9, 
   3.5810476237246485`*^9}, {3.581047796853753*^9, 3.581047929594386*^9}, 
   3.581048024052552*^9, 3.581048065470625*^9, {3.5810488550816116`*^9, 
   3.581048902193694*^9}, 3.58105706758348*^9, 3.5810571996540337`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vf", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"vectorfield", "[", "f", "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.581228447883052*^9, 3.581228491500728*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5812284924523296`*^9, 3.5812524572009373`*^9, 
  3.5823678594743857`*^9, 3.582378369222071*^9, 3.5823792132303534`*^9, 
  3.5823796321066895`*^9, 3.582380151260001*^9, 3.5824365439603906`*^9, 
  3.5824381571416593`*^9, 3.5824423133293796`*^9, 3.582446018522709*^9, 
  3.582447136224638*^9, 3.5824474595741324`*^9, 3.582448126226263*^9, 
  3.582551101321528*^9, 3.5825526738473845`*^9, 3.582554555481352*^9, 
  3.582601385454192*^9, 3.5826041926459827`*^9, 3.582604480419443*^9, 
  3.582874665950242*^9, 3.582957040524472*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vg", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"vectorfield", "[", "g", "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.581228447883052*^9, 3.581228513044366*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{
    RowBox[{"g", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{
    RowBox[{"g", "[", "3", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5812285213435802`*^9, 3.58125245725194*^9, 
  3.582367859521186*^9, 3.582378369237671*^9, 3.5823792132459536`*^9, 
  3.5823796321066895`*^9, 3.5823801512756014`*^9, 3.5824365439753914`*^9, 
  3.58243815715466*^9, 3.5824423133423805`*^9, 3.58244601853671*^9, 
  3.5824471362376385`*^9, 3.5824474595871334`*^9, 3.582448126240264*^9, 
  3.5825511013371277`*^9, 3.5825526738629847`*^9, 3.582554555481352*^9, 
  3.582601385469792*^9, 3.582604192660984*^9, 3.5826044804324436`*^9, 
  3.582874665964243*^9, 3.5829570405394726`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["bracket", "Subsubsection",
 CellChangeTimes->{3.5810472056751146`*^9, 3.5812281884389963`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "bracket"}]], "Input",
 CellChangeTimes->{{3.582367312613025*^9, 3.5823673171214333`*^9}, 
   3.5823800188313684`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"bracket[vv,ww][f][x,y] is the bracket of vector fields vv and  \
ww on R^2 applied to the function f. bracket[vv,ww][f][x,y,z] is bracket of \
the vector fields vv and  ww on R^3 applied to the function f. \
bracket[vv,ww][f][coor] is the bracket of vector fields vv and  ww on R^n \
using coordinates coor applied to the function f.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.582367318853036*^9},
 CellTags->"Info3582374518-2942315"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"bracket", "[", 
            RowBox[{"vv_", ",", "ww_"}], "]"}], "[", "f_", "]"}], "[", 
          RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "vv", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "ww", "]"}], "[", "f", "]"}], "]"}], "[", 
           RowBox[{"x", ",", "y"}], "]"}], "-", 
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "ww", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "vv", "]"}], "[", "f", "]"}], "]"}], "[", 
           RowBox[{"x", ",", "y"}], "]"}]}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"bracket", "[", 
            RowBox[{"vv_", ",", "ww_"}], "]"}], "[", "f_", "]"}], "[", 
          RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "vv", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "ww", "]"}], "[", "f", "]"}], "]"}], "[", 
           RowBox[{"x", ",", "y", ",", "z"}], "]"}], "-", 
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "ww", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "vv", "]"}], "[", "f", "]"}], "]"}], "[", 
           RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{
          RowBox[{"bracket", "[", 
           RowBox[{"vv_", ",", "ww_"}], "]"}], "[", 
          RowBox[{"expr_", ",", "coor_"}], "]"}], ":=", 
         RowBox[{
          RowBox[{
           RowBox[{"der", "[", "vv", "]"}], "[", 
           RowBox[{
            RowBox[{
             RowBox[{"der", "[", "ww", "]"}], "[", 
             RowBox[{"expr", ",", "coor"}], "]"}], ",", "coor"}], "]"}], "-", 
          RowBox[{
           RowBox[{"der", "[", "ww", "]"}], "[", 
           RowBox[{
            RowBox[{
             RowBox[{"der", "[", "vv", "]"}], "[", 
             RowBox[{"expr", ",", "coor"}], "]"}], ",", "coor"}], "]"}]}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"bracket", "[", 
            RowBox[{"vv_", ",", "ww_"}], "]"}], "[", "f_", "]"}], "[", 
          "coor_", "]"}], ":=", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "vv", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "ww", "]"}], "[", "f", "]"}], "]"}], "[", 
           "coor", "]"}], "-", 
          RowBox[{
           RowBox[{
            RowBox[{"der", "[", "ww", "]"}], "[", 
            RowBox[{
             RowBox[{"der", "[", "vv", "]"}], "[", "f", "]"}], "]"}], "[", 
           "coor", "]"}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[CSPROGS`bracket],
  Editable->False]], "Print",
 CellChangeTimes->{3.582367318868636*^9},
 CellTags->"Info3582374518-2942315"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example for bracket", "Subsubsection",
 CellChangeTimes->{{3.5812429831466846`*^9, 3.581242986938902*^9}, {
  3.5812436257304387`*^9, 3.58124363187479*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"x", "^", "2"}], "+", 
   RowBox[{"y", "^", "2"}], "+", 
   RowBox[{"z", "^", "3"}]}]}]], "Input",
 CellChangeTimes->{{3.5812405429431133`*^9, 3.581240575251961*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bracket", "[", 
    RowBox[{"vf", ",", "vg"}], "]"}], "[", "f", "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.581239637343316*^9, 3.5812396412635403`*^9}, {
  3.5812396772675996`*^9, 3.581239709827462*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"der", "[", "vf", "]"}], "[", 
    RowBox[{
     RowBox[{"der", "[", "vg", "]"}], "[", "f", "]"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"der", "[", "vg", "]"}], "[", 
    RowBox[{
     RowBox[{"der", "[", "vf", "]"}], "[", "f", "]"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5812397113185472`*^9, 3.5812405864446015`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bracket", "[", 
    RowBox[{
     RowBox[{"vf", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"vg", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "]"}], "[", "f", "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.581239637343316*^9, 3.5812396412635403`*^9}, {
  3.5812396772675996`*^9, 3.581239748101651*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.58123975034678*^9, 3.581240591805908*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bracket", "[", 
    RowBox[{
     RowBox[{"vf", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"vg", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "]"}], "[", 
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.581239637343316*^9, 3.5812396412635403`*^9}, {
  3.5812396772675996`*^9, 3.581239748101651*^9}, {3.581240621314596*^9, 
  3.581240636837484*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"f", "[", "2", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
       RowBox[{
        RowBox[{"g", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"f", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
       RowBox[{
        RowBox[{"g", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", "0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         RowBox[{"f", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
       RowBox[{
        RowBox[{"g", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"], "+", 
           SuperscriptBox["z", "3"]}], ")"}], 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.581240638899602*^9, 3.581241793970668*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"br", " ", "=", 
  RowBox[{"Simplify", "[", "%", "]"}]}]], "Input",
 CellChangeTimes->{{3.5812397719240136`*^9, 3.5812397804515014`*^9}, {
  3.581241742660733*^9, 3.5812417467719684`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"f", "[", "2", "]"}], "[", 
           RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
         RowBox[{
          RowBox[{"g", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"f", "[", "3", "]"}], "[", 
           RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
         RowBox[{
          RowBox[{"g", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], ",", "0", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"f", "[", "3", "]"}], "[", 
           RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
         RowBox[{
          RowBox[{"g", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "1", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          RowBox[{"f", "[", "2", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
         RowBox[{
          RowBox[{"g", "[", "3", "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"], "+", 
          SuperscriptBox["z", "3"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.581239785551793*^9, 3.5812406477611084`*^9, 
  3.581241800490041*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "br", "]"}]], "Input",
 CellChangeTimes->{{3.5812418071014194`*^9, 3.581241816793974*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "3"}], "}"}]], "Output",
 CellChangeTimes->{3.5812418178470335`*^9}]
}, Closed]],

Cell["\<\
This is a matrix; a function would have been expected. Other bracket \
definitions did not work:\
\>", "Text",
 CellChangeTimes->{{3.5812431928266783`*^9, 3.581243268249992*^9}, 
   3.582367410019596*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"bracket", "[", 
    RowBox[{"vf", ",", "vg"}], "]"}], "[", 
   RowBox[{"f", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581242686940743*^9, 3.5812427215727243`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"der", "[", "vf", "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", "vg", "]"}], "[", 
     RowBox[{"f", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], "-", 
  RowBox[{
   RowBox[{"der", "[", "vg", "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", "vf", "]"}], "[", 
     RowBox[{"f", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5812427236268415`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"bracket", "[", 
    RowBox[{
     RowBox[{"vf", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"vg", "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "]"}], "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581242686940743*^9, 3.5812427215727243`*^9}, {
   3.5812427655622396`*^9, 3.5812427795500402`*^9}, 3.581242812325915*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"der", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
      RowBox[{
       RowBox[{"f", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
      RowBox[{
       RowBox[{"f", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}], "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"g", "[", "1", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{
         RowBox[{"g", "[", "2", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{
         RowBox[{"g", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"], "+", 
       SuperscriptBox["z", "3"]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], "-", 
  RowBox[{
   RowBox[{"der", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
      RowBox[{
       RowBox[{"g", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
      RowBox[{
       RowBox[{"g", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}], "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "[", "1", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{
         RowBox[{"f", "[", "2", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
        RowBox[{
         RowBox[{"f", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}], "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"], "+", 
       SuperscriptBox["z", "3"]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.581242781438148*^9, 3.5812428147270517`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Clear", "[", "f", "]"}]], "Input",
 CellChangeTimes->{{3.581243279247621*^9, 3.5812432883161397`*^9}}],

Cell[TextData[{
 "Alfred' s bracket is defined as a commutator of the directional operators \
of two vector fields acting on a function or an expression, but the \
commutator of two vector fields as the basic operation in the Lie algebra of \
smooth vector fields has not been defined neither in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " nor by Alfred. Thus we begin with the definition of this commutator. We \
recomment to use commutator instead of brackets."
}], "Text",
 CellChangeTimes->{{3.5812295081161604`*^9, 3.5812295642803726`*^9}, {
   3.581229649303236*^9, 3.5812298650785775`*^9}, {3.5812299219508305`*^9, 
   3.581229987574584*^9}, {3.581241621139783*^9, 3.5812416223468513`*^9}, 
   3.5812433611783075`*^9, {3.581243395427266*^9, 3.5812434551226807`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Commutator", "Subsubsection",
 CellChangeTimes->{{3.580795811912533*^9, 3.5807958164833403`*^9}, {
  3.5812299983992033`*^9, 3.5812300028464575`*^9}}],

Cell[BoxData[
 RowBox[{"?", "commutator"}]], "Input",
 CellChangeTimes->{{3.582367894777248*^9, 3.582367899878457*^9}, 
   3.582380018862569*^9}],

Cell["We consider two vector fields", "Text",
 CellChangeTimes->{{3.5812284080405817`*^9, 3.58122844134664*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"directionalD", "[", "vf", "]"}], "[", "vg", "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", "-", 
  RowBox[{
   RowBox[{
    RowBox[{"directionalD", "[", "vg", "]"}], "[", "vf", "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5812285366160073`*^9, 3.5812286311209736`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"g", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "3", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "1", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"g", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "3", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "2", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{"g", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "3", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"f", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"f", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"g", "[", "3", "]"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5812286355669813`*^9, 3.5812310259909782`*^9, 
  3.5812524641463346`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{"commutator", "[", 
    RowBox[{"vf", ",", "vg"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5812310340594397`*^9, 3.5812310654482346`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5812310691544466`*^9, 3.581252470177679*^9}]
}, Closed]],

Cell[TextData[{
 "Clearly, the commutator of two vector fields is a vector field again. The \
commutator  is ",
 StyleBox["bilinear",
  FontColor->RGBColor[0, 0, 1]],
 " with respect to multiplication with real constants and ",
 StyleBox["anti-commutative",
  FontColor->RGBColor[0, 0, 1]]
}], "Text",
 CellChangeTimes->{{3.5812311186632786`*^9, 3.5812312480226774`*^9}, {
  3.581243752394684*^9, 3.581243796090183*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"xv", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", ":=", 
  RowBox[{"\[Xi]", "*", 
   RowBox[{"vf", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " "}]}]], "Input",
 CellChangeTimes->{{3.581231383365419*^9, 3.5812314333612785`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"commutator", "[", 
     RowBox[{"xv", ",", "vg"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", "\[Equal]", 
   RowBox[{"\[Xi]", "*", 
    RowBox[{
     RowBox[{"commutator", "[", 
      RowBox[{"vf", ",", "vg"}], "]"}], "[", 
     RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581231308700148*^9, 3.5812313378018126`*^9}, {
  3.5812314463550215`*^9, 3.5812314625939503`*^9}, {3.5812315163860273`*^9, 
  3.581231527209646*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5812313430941153`*^9, 3.5812314735755787`*^9, 
  3.5812315295577803`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"commutator", "[", 
    RowBox[{"vf", ",", "vg"}], "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{"-", 
   RowBox[{
    RowBox[{"commutator", "[", 
     RowBox[{"vg", ",", "vf"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581231479043891*^9, 3.5812314893044777`*^9}, {
  3.5812315734262896`*^9, 3.5812316235691576`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.581231490051521*^9, 3.5812316275263844`*^9}]
}, Closed]],

Cell["With a third vector field", "Text",
 CellChangeTimes->{{3.581231647406521*^9, 3.581231657934123*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vh", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vectorfield", "[", "h", "]"}], "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581231663582446*^9, 3.58123170019254*^9}}],

Cell[TextData[{
 "the commutator satisfies the ",
 StyleBox["Jacobi identity",
  FontColor->RGBColor[0, 0, 1]],
 ":"
}], "Text",
 CellChangeTimes->{{3.581231746014161*^9, 3.581231772270663*^9}, 
   3.5812325037264996`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"commutator", "[", 
     RowBox[{
      RowBox[{"commutator", "[", 
       RowBox[{"vf", ",", "vg"}], "]"}], ",", "vh"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", "+", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"commutator", "[", 
     RowBox[{
      RowBox[{"commutator", "[", 
       RowBox[{"vh", ",", "vf"}], "]"}], ",", "vg"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", "+", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"commutator", "[", 
     RowBox[{
      RowBox[{"commutator", "[", 
       RowBox[{"vg", ",", "vh"}], "]"}], ",", "vf"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", "]"}]], "Input",
 CellChangeTimes->{{3.581231777784978*^9, 3.5812318555294247`*^9}, {
  3.5812319041292048`*^9, 3.581232068267593*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.581231929107633*^9, 3.5812320716017838`*^9, 
  3.5812525019094944`*^9}]
}, Closed]],

Cell[TextData[{
 "We conclude:\n",
 StyleBox["Proposition",
  FontColor->RGBColor[0, 0, 1]],
 ".\nThe set of all smooth vector fields over a region with coordinates \
{x,..} with pointwise addition , multiplication with real numbers and the \
commutator as defined above forms an infinite dimensional real Lie algebra. "
}], "Text",
 CellChangeTimes->{{3.5812325418226786`*^9, 3.581232722710025*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
 CellChangeTimes->{{3.5812439678350058`*^9, 3.5812439737223425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", 
    RowBox[{"commutator", "[", 
     RowBox[{"vf", ",", "vg"}], "]"}], "]"}], "[", "f", "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.5812398048938994`*^9, 3.5812398234819627`*^9}, {
   3.581240309579766*^9, 3.581240338348411*^9}, 3.5812403797337785`*^9, 
   3.581240702005211*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"g", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{
       RowBox[{"g", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.581240381357871*^9, 3.5812407058374305`*^9, 
  3.5812414064745045`*^9, 3.5812439062664843`*^9, 3.5823679961306257`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vf", "]"}], "[", 
     RowBox[{
      RowBox[{"directionalD", "[", "vg", "]"}], "[", "f", "]"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], "-", 
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vg", "]"}], "[", 
     RowBox[{
      RowBox[{"directionalD", "[", "vf", "]"}], "[", "f", "]"}], "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581241064516946*^9, 3.5812411089014845`*^9}, {
  3.5812412352527113`*^9, 3.5812413827351465`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"g", "[", "2", "]"}], "[", 
     RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "3", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"f", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "3", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "2", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"g", "[", "1", "]"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
      RowBox[{
       SuperscriptBox["f", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "-", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "1", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "2", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox["f", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"g", "[", "3", "]"}], 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.5812411105045757`*^9, 3.5812413849842753`*^9, 
  3.581241420013279*^9, 3.581243930857891*^9, 3.582367996239826*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"%", "-", "%%"}], "]"}]], "Input",
 CellChangeTimes->{{3.5812403921404877`*^9, 3.581240400419962*^9}, {
  3.581240747340804*^9, 3.5812407510520163`*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.581240402991109*^9, 3.581240754027187*^9, 
  3.5812413915286493`*^9, 3.5812414255275946`*^9, 3.5812439412404847`*^9, 
  3.5823679962710257`*^9}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["directionalD. der", "Subsection",
 CellChangeTimes->{{3.5807951884730377`*^9, 3.5807951924978447`*^9}, {
  3.5808009952032776`*^9, 3.5808009965760803`*^9}, {3.5810472176403356`*^9, 
  3.581047223537146*^9}, {3.5811709465188904`*^9, 3.581170953342281*^9}, {
  3.5820891505808754`*^9, 3.5820891593012905`*^9}}],

Cell[BoxData[
 RowBox[{"?", "der"}]], "Input",
 CellChangeTimes->{{3.58236804397591*^9, 3.5823680454111123`*^9}, 
   3.582380018878169*^9}],

Cell["\<\
The last definition is the most general  one; it gives the expected \
directional derivative of functions and more general objects if applied with \
correct settings of the variables, see below. The first two definitions \
calculate a matrix of derivatives whose sum is the expected  directional \
derivative.\
\>", "Text",
 CellChangeTimes->{{3.5810486488960495`*^9, 3.5810487476442227`*^9}, {
  3.5811649741380143`*^9, 3.581165078720598*^9}, {3.581165122135474*^9, 
  3.5811651900423937`*^9}}],

Cell[CellGroupData[{

Cell["directionalD", "Subsubsection",
 CellChangeTimes->{{3.581166289974551*^9, 3.581166296333915*^9}}],

Cell[TextData[{
 "The directional derivative der may now, in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 9, eaily expressed using the built-in  derivative D. Assuming that the \
vector field and the expression f depend on the same set of variables x___, \
the \[OpenCurlyDoubleQuote]coordinates\[CloseCurlyDoubleQuote], and the \
vector field has the dimension of the number of the variables,  we define "
}], "Text",
 CellChangeTimes->{{3.581056675278041*^9, 3.5810568528621984`*^9}, {
  3.5810575345911913`*^9, 3.5810575388144326`*^9}, {3.5810576025270767`*^9, 
  3.5810576165258775`*^9}, {3.5810582969667964`*^9, 3.5810583295026574`*^9}, {
  3.5811557462820654`*^9, 3.5811557827705297`*^9}, {3.58116639065431*^9, 
  3.5811664031110225`*^9}}],

Cell[BoxData[
 RowBox[{"?", "directionalD"}]], "Input",
 CellChangeTimes->{{3.582368151210498*^9, 3.582368172020935*^9}, 
   3.582380018893769*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Example.Some Vector Fields", "Subsubsection",
 CellChangeTimes->{{3.5810487563646383`*^9, 3.581048759827844*^9}, {
  3.581165601009516*^9, 3.58116560342752*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "vf", "]"}], ";", 
  RowBox[{
   RowBox[{"vf", "[", 
    RowBox[{"u_", ",", "v_", ",", "w_", ",", "x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"vectorfield", "[", "f", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"vf", "[", 
  RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581165657497215*^9, 3.5811656710380383`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "4", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"f", "[", "5", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.581165672426441*^9, 3.58125218119215*^9}]
}, Closed]],

Cell["The null vector field in arbitrary dimension is", "Text",
 CellChangeTimes->{{3.581165731644145*^9, 3.581165769084211*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nullvf", "[", "x___", "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{"0", ",", 
    RowBox[{"{", 
     RowBox[{"Length", "[", 
      RowBox[{"{", "x", "}"}], "]"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.581057343623268*^9, 3.581057378032236*^9}, {
   3.581057467713366*^9, 3.5810574763448596`*^9}, {3.581058892744873*^9, 
   3.5810589099368563`*^9}, {3.5810589801688733`*^9, 
   3.5810589906804743`*^9}, {3.581059087368005*^9, 3.581059108705225*^9}, {
   3.5810591657944903`*^9, 3.581059171521818*^9}, 3.581165796602659*^9, 
   3.581252199428193*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"v2", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"vectorfield", "[", "v", "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5811571692021646`*^9, 3.5811572340514784`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"v", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"v", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581157236578683*^9, 3.5811658133726883`*^9, 
  3.5812522626008067`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"vr", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "/", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"x", "^", "2"}], "+", 
     RowBox[{"y", "^", "2"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581058428074295*^9, 3.581058428447317*^9}, {
  3.581058478351171*^9, 3.5810585473291163`*^9}}],

Cell["A constant vector field :", "Text",
 CellChangeTimes->{{3.581165835399927*^9, 3.5811658561167636`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"v1", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.581059258247779*^9, 3.581059278831956*^9}}],

Cell["\<\
Alfred\[CloseCurlyQuote]s definition of der in dimension 2 :\
\>", "Text",
 CellChangeTimes->{{3.581157660712228*^9, 3.5811576714606466`*^9}, {
  3.5811658768492002`*^9, 3.5811658940716305`*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"der", "[", "v_List", "]"}], "[", "ww_", "]"}], "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"v", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "*", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"ww", "[", 
          RowBox[{"xx", ",", "yy"}], "]"}], ",", "xx"}], "]"}]}], ",", "\n", 
      "              ", 
      RowBox[{
       RowBox[{"v", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "*", 
       RowBox[{"D", "[", 
        RowBox[{
         RowBox[{"ww", "[", 
          RowBox[{"xx", ",", "yy"}], "]"}], ",", "yy"}], "]"}]}]}], "}"}], 
    " ", "/.", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"xx", "->", "x"}], ",", 
      RowBox[{"yy", "->", "y"}]}], "}"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5810480728182373`*^9, 3.58104809704508*^9}, {
   3.5810489193849244`*^9, 3.5810489678074093`*^9}, {3.581049004701474*^9, 
   3.5810490395831356`*^9}, {3.5810491331365*^9, 3.581049153010935*^9}, 
   3.58105706758848*^9, 3.581057199639033*^9, {3.581157036804732*^9, 
   3.5811570405487385`*^9}, 3.581157080656409*^9}],

Cell["yield", "Text",
 CellChangeTimes->{{3.581157694564287*^9, 3.581157696701491*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"der", "[", "v2", "]"}], "[", "vr", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581157706482708*^9, 3.581157743095972*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"der", "[", "v2", "]"}], "[", "vr", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Output",
 CellChangeTimes->{3.5811577460755777`*^9}]
}, Closed]],

Cell["variables must be set in the first argument", "Text",
 CellChangeTimes->{{3.5811659244760838`*^9, 3.581165968546161*^9}, 
   3.581166507886015*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alfder", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"der", "[", 
     RowBox[{"v2", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], "[", "vr", "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581157706482708*^9, 3.5811577689920177`*^9}, {
  3.5811594803150234`*^9, 3.5811594842618303`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          SuperscriptBox["x", "2"], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["x", "2"], "+", 
             SuperscriptBox["y", "2"]}], ")"}], 
           RowBox[{"3", "/", "2"}]]]}], "+", 
        FractionBox["1", 
         SqrtBox[
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
      RowBox[{
       RowBox[{"v", "[", "1", "]"}], "[", 
       RowBox[{"x", ",", "y"}], "]"}]}], ",", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", " ", "y", " ", 
        RowBox[{
         RowBox[{"v", "[", "1", "]"}], "[", 
         RowBox[{"x", ",", "y"}], "]"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}], ")"}], 
        RowBox[{"3", "/", "2"}]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", " ", "y", " ", 
        RowBox[{
         RowBox[{"v", "[", "2", "]"}], "[", 
         RowBox[{"x", ",", "y"}], "]"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}], ")"}], 
        RowBox[{"3", "/", "2"}]]]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          SuperscriptBox["y", "2"], 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["x", "2"], "+", 
             SuperscriptBox["y", "2"]}], ")"}], 
           RowBox[{"3", "/", "2"}]]]}], "+", 
        FractionBox["1", 
         SqrtBox[
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
      RowBox[{
       RowBox[{"v", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y"}], "]"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581157771878023*^9, 3.581159488551838*^9, 
  3.58116597355377*^9}]
}, Closed]],

Cell["This is a matrix, not a vector field, as expected.", "Text",
 CellChangeTimes->{{3.581166011820637*^9, 3.581166037857083*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5811659840837884`*^9, 3.5811659966574106`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "2"}], "}"}]], "Output",
 CellChangeTimes->{3.5811660019146194`*^9}]
}, Closed]],

Cell["\<\
The directional derivative of the vector field is again a vector field\
\>", "Text",
 CellChangeTimes->{{3.581166546904247*^9, 3.581166572734724*^9}, {
  3.5811666275498595`*^9, 3.581166635926339*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "v2", "]"}], "[", "vr", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5811578564301715`*^9, 3.581157873278201*^9}, 
   3.5811662237867656`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["x", "2"], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["y", "2"]}], ")"}], 
          RowBox[{"3", "/", "2"}]]]}], "+", 
       FractionBox["1", 
        SqrtBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
     RowBox[{
      RowBox[{"v", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}]}], "-", 
    FractionBox[
     RowBox[{"x", " ", "y", " ", 
      RowBox[{
       RowBox[{"v", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y"}], "]"}]}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["y", "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"x", " ", "y", " ", 
       RowBox[{
        RowBox[{"v", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y"}], "]"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["y", "2"], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["y", "2"]}], ")"}], 
          RowBox[{"3", "/", "2"}]]]}], "+", 
       FractionBox["1", 
        SqrtBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
     RowBox[{
      RowBox[{"v", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.581157876632207*^9, 3.5811594513301725`*^9, 3.58115949529105*^9, 
   3.5811665822322674`*^9, {3.5812522872132144`*^9, 3.58125231004352*^9}}]
}, Closed]],

Cell["and this is related to Alfreds der by ", "Text",
 CellChangeTimes->{{3.581166599966282*^9, 3.581166615534172*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{"alfder", "[", 
    RowBox[{"[", "1", "]"}], "]"}], " ", "+", 
   RowBox[{"alfder", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5811594563845816`*^9, 3.581159459925788*^9}, {
  3.5811595006574593`*^9, 3.5811595250559025`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.581159527255506*^9}]
}, Closed]],

Cell["\<\
We compare directionalD with the third definition der of Alfred.  The next \
two cells yield no or an incorrect result:\
\>", "Text",
 CellChangeTimes->{{3.5811667843818297`*^9, 3.5811668576860228`*^9}, 
   3.5811671511988106`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"der", "[", 
  RowBox[{"v2", ",", "vr", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5811646210001945`*^9, 3.581164672776685*^9}, 
   3.581164706176344*^9, 3.581166750624899*^9}],

Cell[BoxData[
 RowBox[{"der", "[", 
  RowBox[{"v2", ",", "vr", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{3.581164674867089*^9, 3.5811647080327473`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"v2", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", "vr", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5811646210001945`*^9, 3.581164672776685*^9}, {
   3.581164706176344*^9, 3.581164724131975*^9}, 3.5811667717691083`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.58116472675278*^9}]
}, Closed]],

Cell["\<\
To obtain the espected derivative one has to insert the variables in all \
three arguments\
\>", "Text",
 CellChangeTimes->{{3.581166867989612*^9, 3.5811669174064384`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"grayder", " ", "=", " ", 
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"v2", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", 
    RowBox[{"vr", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5811646210001945`*^9, 3.581164672776685*^9}, {
  3.581164706176344*^9, 3.5811647393732023`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["x", "2"], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["y", "2"]}], ")"}], 
          RowBox[{"3", "/", "2"}]]]}], "+", 
       FractionBox["1", 
        SqrtBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
     RowBox[{
      RowBox[{"v", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}]}], "-", 
    FractionBox[
     RowBox[{"x", " ", "y", " ", 
      RowBox[{
       RowBox[{"v", "[", "2", "]"}], "[", 
       RowBox[{"x", ",", "y"}], "]"}]}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["y", "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"x", " ", "y", " ", 
       RowBox[{
        RowBox[{"v", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "y"}], "]"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["y", "2"], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["x", "2"], "+", 
            SuperscriptBox["y", "2"]}], ")"}], 
          RowBox[{"3", "/", "2"}]]]}], "+", 
       FractionBox["1", 
        SqrtBox[
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["y", "2"]}]]]}], ")"}], " ", 
     RowBox[{
      RowBox[{"v", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "y"}], "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581164741323206*^9, 3.581166934543419*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"directionalD", "[", "v2", "]"}], "[", "vr", "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581164755597231*^9, 3.581164798076105*^9}, 
   3.581166223792766*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5811647994957075`*^9, 3.5811669415338182`*^9}]
}, Closed]],

Cell["The results coincide.", "Text",
 CellChangeTimes->{{3.581166969159398*^9, 3.581167005134456*^9}}],

Cell["Consider the function", "Text",
 CellChangeTimes->{{3.581056873734392*^9, 3.581056881445833*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"hsin", "[", 
   RowBox[{"u_", ",", "v_", ",", "w_", ",", "x_", ",", "y_"}], "]"}], " ", ":=",
   " ", 
  RowBox[{"Sin", "[", 
   RowBox[{"u", "*", "v", "*", "w", "*", "x", "*", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581051356260811*^9, 3.581051410024886*^9}, 
   3.5810515411373854`*^9}],

Cell[" and the vector field vf defined above", "Text",
 CellChangeTimes->{{3.5810569082943687`*^9, 3.5810569160858145`*^9}, 
   3.5810570668804398`*^9, {3.581057271766158*^9, 3.581057277982514*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "vf", "]"}], "[", "hsin", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5810561292868123`*^9, 3.581056162024685*^9}, 
   3.5811662238047667`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", " ", "w", " ", "x", " ", "y", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", "y"}], "]"}], " ", 
   RowBox[{
    RowBox[{"f", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"u", " ", "w", " ", "x", " ", "y", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", "y"}], "]"}], " ", 
   RowBox[{
    RowBox[{"f", "[", "2", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"u", " ", "v", " ", "x", " ", "y", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", "y"}], "]"}], " ", 
   RowBox[{
    RowBox[{"f", "[", "3", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"u", " ", "v", " ", "w", " ", "y", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", "y"}], "]"}], " ", 
   RowBox[{
    RowBox[{"f", "[", "4", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"u", " ", "v", " ", "w", " ", "x", " ", "y"}], "]"}], " ", 
   RowBox[{
    RowBox[{"f", "[", "5", "]"}], "[", 
    RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellChangeTimes->{3.5810561655898886`*^9, 3.5810562634724874`*^9, 
  3.581057286802018*^9, 3.581155929769588*^9, 3.5811671815235453`*^9, 
  3.5812523419903474`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", " ", 
  RowBox[{"der", "[", 
   RowBox[{
    RowBox[{"vf", "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", " ", 
    RowBox[{"hsin", "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.581167198137495*^9, 3.5811672660333786`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5811672684405165`*^9}]
}, Closed]],

Cell["Again der and directionalD yield the same result.", "Text",
 CellChangeTimes->{{3.581167332630188*^9, 3.5811673573496017`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "nullvf", "]"}], "[", "hsin", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581057386883743*^9, 3.5810574200796413`*^9}, 
   3.581166223809767*^9, 3.581167374521584*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.5810574237408504`*^9, 3.581057481019127*^9, {3.5810589235766363`*^9, 
   3.5810589464079423`*^9}, 3.5810590070954137`*^9, 3.581059114431553*^9, 
   3.581059177520161*^9, 3.5811559801576767`*^9, 3.5811673635819583`*^9, 
   3.581167398593961*^9, 3.5812523676568155`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"vh", "[", 
   RowBox[{"u_", ",", "v_", ",", "w_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"u", "^", "2"}], ",", 
    RowBox[{"v", "^", "2"}], ",", 
    RowBox[{"w", "^", "2"}], ",", 
    RowBox[{"x", "^", "3"}], ",", 
    RowBox[{"y", "^", "3"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5810600515681543`*^9, 3.5810601452565126`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "vf", "]"}], "[", "vh", "]"}], "[", 
  RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581060374176606*^9, 3.5810603748806467`*^9}, 
   3.5811662238157673`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", "u", " ", 
    RowBox[{
     RowBox[{"f", "[", "1", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], ",", 
   RowBox[{"2", " ", "v", " ", 
    RowBox[{
     RowBox[{"f", "[", "2", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], ",", 
   RowBox[{"2", " ", "w", " ", 
    RowBox[{
     RowBox[{"f", "[", "3", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], ",", 
   RowBox[{"3", " ", 
    SuperscriptBox["x", "2"], " ", 
    RowBox[{
     RowBox[{"f", "[", "4", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}], ",", 
   RowBox[{"3", " ", 
    SuperscriptBox["y", "2"], " ", 
    RowBox[{
     RowBox[{"f", "[", "5", "]"}], "[", 
     RowBox[{"u", ",", "v", ",", "w", ",", "x", ",", "y"}], "]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5810603795499134`*^9, 3.581156059265415*^9, 
  3.5811674125787606`*^9, 3.5812523784104304`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"hh", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", " ", 
  RowBox[{"x", "*", "y"}]}]], "Input",
 CellChangeTimes->{{3.581058569004356*^9, 3.5810586172721167`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "vr", "]"}], "[", "hh", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5810586359641857`*^9, 3.5810586499289846`*^9}, 
   3.5811662238207674`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"2", " ", "x", " ", "y"}], 
  SqrtBox[
   RowBox[{
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["y", "2"]}]]]], "Output",
 CellChangeTimes->{3.5810586512700615`*^9, 3.5811560892486677`*^9, 
  3.581167422021301*^9, 3.58125240462593*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "nullvf", "]"}], "[", "hh", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581058837078689*^9, 3.581058851584519*^9}, 
   3.581166223825768*^9, 3.581167432687911*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{
  3.581058853619635*^9, 3.581058930720045*^9, 3.581058970390314*^9, 
   3.581059013096757*^9, 3.581059192184*^9, 3.5811561094507036`*^9, {
   3.5811674273966084`*^9, 3.5811674339909854`*^9}, 3.5812524091681895`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "v1", "]"}], "[", "hh", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5810592911546607`*^9, 3.581059309000682*^9}, 
   3.581166223831768*^9}],

Cell[BoxData["y"], "Output",
 CellChangeTimes->{3.5810593117808404`*^9, 3.5811561181555185`*^9, 
  3.5811674391972833`*^9, 3.581252414106472*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Proposition.",
  FontColor->RGBColor[0, 0, 1]],
 "\nLet diff := diff[coords] denote the Ring of the smooth functions of an \
open region for the coordinates coords ={x,y,...}. Let  \[CapitalAlpha] be a \
Module over the ring diff, whose elements themselves may be functions of \
coords, e.g. the space of sections of a vector bundle. Then the operator \
directionalD is welldefined for the elements of \[CapitalAlpha]; The operator \
is lenear for the real vector space \[CapitalAlpha]: it satisfies"
}], "Text",
 CellChangeTimes->{{3.5811675830225096`*^9, 3.5811676677663565`*^9}, {
  3.581167724909625*^9, 3.581167757886511*^9}, {3.5811678774943523`*^9, 
  3.581168085662259*^9}, {3.5811681267906113`*^9, 3.5811681281416883`*^9}, {
  3.581168195550544*^9, 3.5811681960935755`*^9}, {3.581168230582548*^9, 
  3.581168294502204*^9}, {3.58116852941464*^9, 3.5811686100702534`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"directionalD", "[", "vv", "]"}], "[", 
    RowBox[{"fa", "+", "fb"}], "]"}], "[", 
   RowBox[{"x", ",", "..."}], "]"}], " ", "\[Equal]", " ", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vv", "]"}], "[", "fa", "]"}], "[", 
    RowBox[{"x", ",", ".."}], "]"}], " ", "+", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vv", "]"}], "[", "fb", "]"}], "[", 
    RowBox[{"x", ",", ".."}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5811686181147137`*^9, 3.5811688029372845`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"directionalD", "[", "vv", "]"}], "[", 
    RowBox[{"fa", "*", "\[Xi]"}], "]"}], "[", 
   RowBox[{"x", ",", ".."}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vv", "]"}], "[", "fa", "]"}], "[", 
    RowBox[{"x", ",", ".."}], "]"}], "*", "\[Xi]"}]}]], "Input",
 CellChangeTimes->{{3.581168826672642*^9, 3.5811688821368146`*^9}}],

Cell["\<\
for any vector field vv, elements fa,fb \[Element] \[CapitalAlpha], and real \
constants \[Xi].\
\>", "Text",
 CellChangeTimes->{{3.5811689001508446`*^9, 3.5811689240702133`*^9}, {
  3.5811689950042706`*^9, 3.581169047118251*^9}}],

Cell["\<\
Consider fields of matrices of dimensions 3,2 depending on two variables:\
\>", "Text",
 CellChangeTimes->{{3.581169132390128*^9, 3.5811691782297497`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a32", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", 
  RowBox[{"Array", "[", 
   RowBox[{
    RowBox[{"aa", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5810638986911974`*^9, 3.581063930680027*^9}, {
  3.581156167014805*^9, 3.581156233049721*^9}, {3.5811691928175845`*^9, 
  3.581169193040597*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"b32", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", 
  RowBox[{"Array", "[", 
   RowBox[{
    RowBox[{"bb", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "2"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5810638986911974`*^9, 3.581063930680027*^9}, {
  3.581156167014805*^9, 3.581156233049721*^9}, {3.5811691928175845`*^9, 
  3.581169226272498*^9}, {3.5811693828574543`*^9, 3.581169398049323*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"c32", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"a32", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "+", 
   RowBox[{"b32", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581169303741929*^9, 3.581169414849284*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vr", "]"}], "[", "c32", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "vr", "]"}], "[", "a32", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}], " ", "+", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "vr", "]"}], "[", "b32", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5811562598349676`*^9, 3.581156305777048*^9}, 
   3.581166223836768*^9, {3.581169432560297*^9, 3.5811695157540555`*^9}, {
   3.581169593938527*^9, 3.5811696054261847`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5811563081326523`*^9, 3.5811695207693424`*^9, 
  3.581169607677313*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.581169544320689*^9, 3.5811695521521373`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5811695538632355`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"x32", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"\[Xi]", "*", 
   RowBox[{"a32", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581169767806472*^9, 3.58116981970444*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"x32", "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellChangeTimes->{{3.5811698242847023`*^9, 3.58116984364781*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"1", ",", "1"}], "]"}]}], ",", 
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"1", ",", "2"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"2", ",", "1"}], "]"}]}], ",", 
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"2", ",", "2"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"3", ",", "1"}], "]"}]}], ",", 
     RowBox[{"\[Xi]", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["aa", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"x", ",", "y"}], "]"}], "[", 
       RowBox[{"3", ",", "2"}], "]"}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581169847215014*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "==", 
  RowBox[{"\[Xi]", "*", 
   RowBox[{"D", "[", 
    RowBox[{
     RowBox[{"a32", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", "x"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5811698624308844`*^9, 3.5811698995520077`*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.581169902798193*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "vr", "]"}], "[", "x32", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", "\[Equal]", " ", 
   RowBox[{"\[Xi]", "*", 
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "vr", "]"}], "[", "a32", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5811696569211297`*^9, 3.5811697072500086`*^9}, 
   3.581169795587061*^9, {3.5811699156749296`*^9, 3.5811699169370017`*^9}, 
   3.5811699514849777`*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.5811697130263386`*^9, {3.581169927715618*^9, 3.5811699532550793`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "vr", "]"}], "[", "a25", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5811562598349676`*^9, 3.581156305777048*^9}, {
   3.581156378873576*^9, 3.5811563863771896`*^9}, 3.5811662238417687`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"1", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"1", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}], ",", 
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"1", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"1", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"2", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"2", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}], ",", 
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"2", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"2", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"3", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"3", ",", "1"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}], ",", 
     RowBox[{
      FractionBox[
       RowBox[{"y", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"0", ",", "1"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"3", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]], "+", 
      FractionBox[
       RowBox[{"x", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["aa", 
           TagBox[
            RowBox[{"(", 
             RowBox[{"1", ",", "0"}], ")"}],
            Derivative],
           MultilineFunction->None], "[", 
          RowBox[{"x", ",", "y"}], "]"}], "[", 
         RowBox[{"3", ",", "2"}], "]"}]}], 
       SqrtBox[
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}]]]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581156388373993*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "vr", "]"}], "[", "v1", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5811562598349676`*^9, 3.581156305777048*^9}, {
   3.581156378873576*^9, 3.5811563863771896`*^9}, {3.581156434503274*^9, 
   3.581156446671295*^9}, 3.581166223848769*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.581156448231298*^9, 3.581170043855261*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"directionalD", "[", "v1", "]"}], "[", "vr", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5811562598349676`*^9, 3.581156305777048*^9}, {
   3.581156378873576*^9, 3.5811563863771896`*^9}, {3.581156434503274*^9, 
   3.581156470461337*^9}, 3.5811662238537693`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      SuperscriptBox["x", "2"], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["y", "2"]}], ")"}], 
       RowBox[{"3", "/", "2"}]]]}], "+", 
    FractionBox["1", 
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["y", "2"]}]]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"x", " ", "y"}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["y", "2"]}], ")"}], 
      RowBox[{"3", "/", "2"}]]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.581156473300542*^9, 3.5811700509306655`*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Lemma.",
  FontColor->RGBColor[0, 0, 1]],
 "\ndirectionalD is also linear in the first argument as an element of the \
diff-Module of vector fields over the region coords:"
}], "Text",
 CellChangeTimes->{{3.5811707680866847`*^9, 3.5811708624140797`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"wr", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"vr", "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", "+", 
   RowBox[{"v1", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581170142141883*^9, 3.581170176871869*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "wr", "]"}], "[", "vr", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", "==", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "vr", "]"}], "[", "vr", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "v1", "]"}], "[", "vr", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], " ", "]"}]], "Input",
 CellChangeTimes->{{3.5811701895685954`*^9, 3.581170297370761*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{{3.5811702785596857`*^9, 3.5811702985858307`*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"xr", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"h", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "*", 
   RowBox[{"vr", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581170312005598*^9, 3.581170392063177*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"v2", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"vectorfield", "[", "g", "]"}], "[", 
   RowBox[{"x", ",", "y"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581170404919913*^9, 3.5811704627772217`*^9}, 
   3.5811705669461803`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{
    RowBox[{"g", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5811704638902855`*^9, 3.5811704719197445`*^9}, 
   3.5811705700903597`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"directionalD", "[", "xr", "]"}], "[", "v2", "]"}], "[", 
    RowBox[{"x", ",", "y"}], "]"}], "-", 
   RowBox[{
    RowBox[{"h", "[", 
     RowBox[{"x", ",", "y"}], "]"}], 
    RowBox[{
     RowBox[{
      RowBox[{"directionalD", "[", "vr", "]"}], "[", "v2", "]"}], "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5811705889464383`*^9, 3.5811707149626465`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5811707168787556`*^9}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["First Fundamental Form", "Subsection",
 CellChangeTimes->{{3.581339785771657*^9, 3.5813397862426834`*^9}, 
   3.581339837172597*^9}],

Cell[CellGroupData[{

Cell["Metric (First Fundamental Form) , from CSPROGS", "Subsubsection",
 Evaluatable->False,
 CellChangeTimes->{{3.581339863124081*^9, 3.581339984859044*^9}}],

Cell[BoxData[
 RowBox[{"?", "ds2"}]], "Input",
 CellChangeTimes->{{3.5823683143243847`*^9, 3.582368317725191*^9}, 
   3.582380018971769*^9}],

Cell[BoxData[
 RowBox[{"?", "ee"}]], "Input",
 CellChangeTimes->{{3.5823685231151514`*^9, 3.5823685245035543`*^9}, 
   3.582380018971769*^9}],

Cell[BoxData[
 RowBox[{"?", "ff"}]], "Input",
 CellChangeTimes->{3.5823800189873686`*^9}],

Cell[BoxData[
 RowBox[{"?", "gg"}]], "Input",
 CellChangeTimes->{{3.5823685811472535`*^9, 3.582368582239255*^9}, 
   3.582380019002969*^9}],

Cell[BoxData[
 RowBox[{"?", " ", "infarea"}]], "Input",
 CellChangeTimes->{{3.5823686218165245`*^9, 3.5823686256385317`*^9}, 
   3.582380019002969*^9}],

Cell[BoxData[
 RowBox[{"?", "metricmatrix"}]], "Input",
 CellChangeTimes->{{3.5823686596621914`*^9, 3.582368661752595*^9}, 
   3.582380019018569*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
 CellChangeTimes->{{3.581340870484699*^9, 3.581340890688854*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ds2", "[", 
    RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"du", ",", "dv"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813404284644165`*^9, 3.581340489956934*^9}, {
  3.581340638589435*^9, 3.581340647315934*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["r", "2"], "**", 
   SuperscriptBox["dv", "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["r", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "v", "]"}], "2"]}], ")"}], "**", 
   SuperscriptBox["du", "2"]}]}]], "Output",
 CellChangeTimes->{3.5813406502511024`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ee", "[", 
   RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813407588323126`*^9, 3.581340804325915*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["r", "2"], " ", 
  SuperscriptBox[
   RowBox[{"Cos", "[", "v", "]"}], "2"]}]], "Output",
 CellChangeTimes->{3.581340805363974*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ff", "[", 
   RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813407588323126`*^9, 3.581340816364603*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5813408182017083`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gg", "[", 
   RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813407588323126`*^9, 3.581340830445409*^9}}],

Cell[BoxData[
 SuperscriptBox["r", "2"]], "Output",
 CellChangeTimes->{3.5813408329165497`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"metricmatrix", "[", 
   RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813405439380217`*^9, 3.5813405767648993`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["r", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     SuperscriptBox["r", "2"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581340578028971*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", "v_", "]"}], " ", ":=", " ", 
  RowBox[{"v", "/", "2"}]}]], "Input",
 CellChangeTimes->{{3.5813409770457935`*^9, 3.581341002645258*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"metricmatrix", "[", 
   RowBox[{
    RowBox[{"sphere", "[", "r", "]"}], ",", "h"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5813410155649967`*^9, 3.5813410514300485`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}]}]], " ", 
      SuperscriptBox["r", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}]}]], " ", 
      SuperscriptBox["r", "2"]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.581341056270325*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5813410769805098`*^9, 3.581341087141091*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       SuperscriptBox["r", "2"]], " ", 
      SuperscriptBox["r", "2"], " ", 
      SuperscriptBox[
       RowBox[{"Cos", "[", "v", "]"}], "2"]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       SuperscriptBox["r", "2"]], " ", 
      SuperscriptBox["r", "2"]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.58134108835316*^9}]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["frenetframe", "Subsection",
 CellChangeTimes->{{3.581401192481392*^9, 3.581401217911847*^9}, 
   3.581401266087602*^9}],

Cell[BoxData[
 RowBox[{"?", "frenetframe"}]], "Input",
 CellChangeTimes->{{3.5823687326079197`*^9, 3.5823687362583256`*^9}, 
   3.582380019034169*^9}],

Cell[BoxData[
 RowBox[{"?", "tangent"}]], "Input",
 CellChangeTimes->{{3.582368834694499*^9, 3.582368838703706*^9}, 
   3.582380019049769*^9}],

Cell[BoxData[
 RowBox[{"?", "normal"}]], "Input",
 CellChangeTimes->{{3.582368843102914*^9, 3.582368847314921*^9}, 
   3.5823800190653687`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"frh", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{"frenetframe", "[", 
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], "]"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.5814017574277053`*^9, 3.581401809747698*^9}, {
  3.581401860290589*^9, 3.5814018950185747`*^9}, {3.58140205368265*^9, 
  3.5814020677624555`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.581402288106058*^9, 3.581402293537369*^9}},
 NumberMarks->False],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"1", ",", ".5"}], "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581402300591772*^9, 3.5814023491065474`*^9}, {
  3.5814023962812457`*^9, 3.581402437003575*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic0", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5814030037949934`*^9, 3.5814030145696096`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"frhoffs", "[", "t_", "]"}], ":=", 
  RowBox[{
   RowBox[{"frenetframe", "[", 
    RowBox[{
     RowBox[{"helix", "[", 
      RowBox[{"1", ",", ".5"}], "]"}], ",", ".5"}], "]"}], "[", "t", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5814017574277053`*^9, 3.581401809747698*^9}, {
  3.581401860290589*^9, 3.5814018950185747`*^9}, {3.58140205368265*^9, 
  3.5814020677624555`*^9}, {3.5814028007313786`*^9, 3.5814028031225157`*^9}, {
  3.5814028752356405`*^9, 3.5814028775147705`*^9}, {3.5814029190761476`*^9, 
  3.581402921723299*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Graphics3D", "[", 
     RowBox[{"frhoffs", "[", "t", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"4", "*", "Pi"}], ",", " ", "1"}], "}"}]}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.58140192587634*^9, 3.5814019921151285`*^9}, {
   3.5814020849314375`*^9, 3.581402186763262*^9}, 3.5814022257874937`*^9, {
   3.5814028882913866`*^9, 3.581402893890707*^9}, 3.5814029566512966`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic1", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5814029776424975`*^9, 3.5814029875930667`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"pic0", ",", "pic1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.581403023353112*^9, 3.5814030398180537`*^9}}],

Cell["Instead of Line better use Arrow:", "Text",
 CellChangeTimes->{{3.5814059205608225`*^9, 3.5814059352886653`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Arrow"}]], "Input",
 CellChangeTimes->{{3.5814042296531086`*^9, 3.581404232322261*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Arrow\\\", \\\"[\\\", RowBox[{\\\"{\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \\\
\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\"]\\\"}]\\) is a graphics \
primitive that represents an arrow from \\!\\(\\*SubscriptBox[StyleBox[\\\"pt\
\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]]\\) to \
\\!\\(\\*SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]\\).\\n\\!\\(\\*RowBox[{\\\"Arrow\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\
\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"s\\\", \\\"TI\\\"]}], \\\
\"]\\\"}]\\) represents an arrow with its ends set back from \
\\!\\(\\*SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]]\\) and \\!\\(\\*SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]]\\) by a distance \
\\!\\(\\*StyleBox[\\\"s\\\", \\\"TI\\\"]\\). \
\\n\\!\\(\\*RowBox[{\\\"Arrow\\\", \\\"[\\\", RowBox[{RowBox[{\\\"{\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \\\
\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"s\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"s\\\", \
\\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]]}], \\\"}\\\"}]}], \
\\\"]\\\"}]\\) sets back by \\!\\(\\*SubscriptBox[StyleBox[\\\"s\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]]\\) from \
\\!\\(\\*SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]]\\) and \\!\\(\\*SubscriptBox[StyleBox[\\\"s\\\", \\\"TI\\\"], \
StyleBox[\\\"2\\\", \\\"TR\\\"]]\\) from \
\\!\\(\\*SubscriptBox[StyleBox[\\\"pt\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]\\). \\n\\!\\(\\*RowBox[{\\\"Arrow\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"curve\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\\) represents an arrow following \
the specified \\!\\(\\*StyleBox[\\\"curve\\\", \\\"TI\\\"]\\).\"\>", "MSG"], 
  "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Arrow"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.581404234309375*^9},
 CellTags->"Info3581411434-4296284"]
}, Closed]],

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"frenetarrowframe", "[", 
      RowBox[{"alpha_", ",", "offset_"}], "]"}], "[", "t_", "]"}], ":=", "\n",
     "        ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], ",", 
         RowBox[{
          RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
          RowBox[{
           RowBox[{"tangent", "[", "alpha", "]"}], "[", "t", "]"}]}]}], "}"}],
        "]"}], ",", "\n", "        ", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], ",", 
         RowBox[{
          RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
          RowBox[{
           RowBox[{"normal", "[", "alpha", "]"}], "[", "t", "]"}]}]}], "}"}], 
       "]"}], ",", "\n", "        ", 
      RowBox[{"Arrow", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], ",", 
         RowBox[{
          RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
          RowBox[{
           RowBox[{"binormal", "[", "alpha", "]"}], "[", "t", "]"}]}]}], 
        "}"}], "]"}], ",", "\n", "         ", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<T\>\"", ",", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
         RowBox[{"offset", "*", 
          RowBox[{
           RowBox[{"tangent", "[", "alpha", "]"}], "[", "t", "]"}]}]}]}], 
       "]"}], ",", "\n", "         ", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<N\>\"", ",", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
         RowBox[{"offset", "*", 
          RowBox[{
           RowBox[{"normal", "[", "alpha", "]"}], "[", "t", "]"}]}]}]}], 
       "]"}], ",", "\n", "         ", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<B\>\"", ",", 
        RowBox[{
         RowBox[{"alpha", "[", "t", "]"}], " ", "+", " ", 
         RowBox[{"offset", "*", 
          RowBox[{
           RowBox[{"binormal", "[", "alpha", "]"}], "[", "t", "]"}]}]}]}], 
       "]"}]}], "}"}]}], ";"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5814044752831573`*^9, 3.5814045111002064`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"frarrh", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"frenetarrowframe", "[", 
    RowBox[{
     RowBox[{"helix", "[", 
      RowBox[{"1", ",", ".5"}], "]"}], ",", ".5"}], "]"}], "[", "t", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5814045391638117`*^9, 3.581404606531665*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"frarrh", "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5814046138580837`*^9, 3.5814046209614897`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Arrow", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], ",", 
        RowBox[{"Sin", "[", "t", "]"}], ",", 
        RowBox[{"0.5`", " ", "t"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Cos", "[", "t", "]"}], "-", 
         RowBox[{"0.8944271909999159`", " ", 
          RowBox[{"Sin", "[", "t", "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"0.8944271909999159`", " ", 
          RowBox[{"Cos", "[", "t", "]"}]}], "+", 
         RowBox[{"Sin", "[", "t", "]"}]}], ",", 
        RowBox[{"0.4472135954999579`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Arrow", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], ",", 
        RowBox[{"Sin", "[", "t", "]"}], ",", 
        RowBox[{"0.5`", " ", "t"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.`", ",", "0.`", ",", 
        RowBox[{"0.`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Arrow", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], ",", 
        RowBox[{"Sin", "[", "t", "]"}], ",", 
        RowBox[{"0.5`", " ", "t"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Cos", "[", "t", "]"}], "+", 
         RowBox[{"0.4472135954999579`", " ", 
          RowBox[{"Sin", "[", "t", "]"}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "0.4472135954999579`"}], " ", 
          RowBox[{"Cos", "[", "t", "]"}]}], "+", 
         RowBox[{"Sin", "[", "t", "]"}]}], ",", 
        RowBox[{"0.8944271909999159`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Text", "[", 
    RowBox[{"\<\"T\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], "-", 
        RowBox[{"0.4472135954999579`", " ", 
         RowBox[{"Sin", "[", "t", "]"}]}]}], ",", 
       RowBox[{
        RowBox[{"0.4472135954999579`", " ", 
         RowBox[{"Cos", "[", "t", "]"}]}], "+", 
        RowBox[{"Sin", "[", "t", "]"}]}], ",", 
       RowBox[{"0.22360679774997896`", "\[VeryThinSpace]", "+", 
        RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "]"}], ",", 
   RowBox[{"Text", "[", 
    RowBox[{"\<\"N\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"0.5`", " ", 
        RowBox[{"Cos", "[", "t", "]"}]}], ",", 
       RowBox[{"0.5`", " ", 
        RowBox[{"Sin", "[", "t", "]"}]}], ",", 
       RowBox[{"0.`", "\[VeryThinSpace]", "+", 
        RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "]"}], ",", 
   RowBox[{"Text", "[", 
    RowBox[{"\<\"B\"\>", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "t", "]"}], "+", 
        RowBox[{"0.22360679774997896`", " ", 
         RowBox[{"Sin", "[", "t", "]"}]}]}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "0.22360679774997896`"}], " ", 
         RowBox[{"Cos", "[", "t", "]"}]}], "+", 
        RowBox[{"Sin", "[", "t", "]"}]}], ",", 
       RowBox[{"0.4472135954999579`", "\[VeryThinSpace]", "+", 
        RowBox[{"0.5`", " ", "t"}]}]}], "}"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5814046218605413`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Graphics3D", "[", 
     RowBox[{"frarrh", "[", "t", "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"4", "*", "Pi"}], ",", " ", 
      RowBox[{"Pi", "/", "2"}]}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58140192587634*^9, 3.5814019921151285`*^9}, {
   3.5814020849314375`*^9, 3.581402186763262*^9}, 3.5814022257874937`*^9, {
   3.5814028882913866`*^9, 3.581402893890707*^9}, 3.5814029566512966`*^9, {
   3.581404741347376*^9, 3.581404747245713*^9}, {3.581404781675682*^9, 
   3.581404806187084*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"pic3", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5814048125004454`*^9, 3.581404820465901*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"pic0", ",", "pic3"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5814048354877605`*^9, 3.5814048518736973`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Animate"}]], "Input",
 CellChangeTimes->{{3.5814051794424334`*^9, 3.5814051840276957`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\!\(\*RowBox[{\\\"Animate\\\", \\\"[\\\", RowBox[{StyleBox[\\\
\"expr\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"u\
\\\", \\\"TI\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], \
StyleBox[\\\"min\\\", \\\"TI\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], StyleBox[\\\"max\\\", \
\\\"TI\\\"]]}], \\\"}\\\"}]}], \\\"]\\\"}]\) generates an animation of \
\!\(\*StyleBox[\\\"expr\\\", \\\"TI\\\"]\) in which \!\(\*StyleBox[\\\"u\\\", \
\\\"TI\\\"]\) varies continuously from \!\(\*SubscriptBox[StyleBox[\\\"u\\\", \
\\\"TI\\\"], StyleBox[\\\"min\\\", \\\"TI\\\"]]\) to \
\!\(\*SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], StyleBox[\\\"max\\\", \
\\\"TI\\\"]]\). \\n\!\(\*RowBox[{\\\"Animate\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"expr\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"u\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], StyleBox[\\\"min\\\", \
\\\"TI\\\"]], \\\",\\\", SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], \
StyleBox[\\\"max\\\", \\\"TI\\\"]], \\\",\\\", StyleBox[\\\"du\\\", \
\\\"TI\\\"]}], \\\"}\\\"}]}], \\\"]\\\"}]\) takes \!\(\*StyleBox[\\\"u\\\", \
\\\"TI\\\"]\) to vary in steps \!\(\*StyleBox[\\\"du\\\", \\\"TI\\\"]\). \
\\n\!\(\*RowBox[{\\\"Animate\\\", \\\"[\\\", RowBox[{StyleBox[\\\"expr\\\", \
\\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{StyleBox[\\\"u\\\", \\\"TI\
\\\"], \\\",\\\", RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"u\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"}\\\
\"}]}], \\\"}\\\"}]}], \\\"]\\\"}]\) makes \!\(\*StyleBox[\\\"u\\\", \\\"TI\\\
\"]\) take on discrete values \!\(\*SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\
\"], StyleBox[\\\"1\\\", \\\"TR\\\"]]\), \
\!\(\*SubscriptBox[StyleBox[\\\"u\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]\), \[Ellipsis]. \\n\!\(\*RowBox[{\\\"Animate\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"expr\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{StyleBox[\\\"u\\\", \\\"TI\\\"], \\\",\\\", \
StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"}\\\"}], \\\",\\\", RowBox[{\
\\\"{\\\", RowBox[{StyleBox[\\\"v\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\) varies all the variables \
\!\(\*StyleBox[\\\"u\\\", \\\"TI\\\"]\), \!\(\*StyleBox[\\\"v\\\", \
\\\"TI\\\"]\), \[Ellipsis]. \"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Animate"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.581405192530182*^9},
 CellTags->"Info3581412392-4296284"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"helfr", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"pic0", ",", " ", 
      RowBox[{"Graphics3D", "[", 
       RowBox[{"frarrh", "[", "t", "]"}], "]"}]}], "}"}], ",", " ", 
    RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581405239437865*^9, 3.5814053803149223`*^9}, {
  3.58140565668373*^9, 3.5814056669603176`*^9}}],

Cell[BoxData[
 RowBox[{"helfr", "[", "0", "]"}]], "Input",
 CellChangeTimes->{{3.5814053861052537`*^9, 3.581405391401557*^9}, 
   3.5814054459786787`*^9}],

Cell["\<\
Use Animate to show the \[OpenCurlyDoubleQuote]moving frame\
\[CloseCurlyDoubleQuote]:\
\>", "Text",
 CellChangeTimes->{{3.5814058374000664`*^9, 3.5814058675447903`*^9}}],

Cell[BoxData[
 RowBox[{"Animate", "[", 
  RowBox[{
   RowBox[{"helfr", "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"4", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581405471720151*^9, 3.581405508802272*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["geo. solvegeoeqs: Problem?", "Subsection",
 CellChangeTimes->{{3.5820088458427076`*^9, 3.5820088889455833`*^9}}],

Cell["\<\
I couldn\[CloseCurlyQuote]t neither interpret nor apply these functions \
correctly.\
\>", "Text",
 CellChangeTimes->{{3.5820089089760184`*^9, 3.5820089491460886`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "geo"}]], "Input",
 CellChangeTimes->{{3.582369238938009*^9, 3.5823692411220126`*^9}, 
   3.582380019080969*^9}],

Cell[BoxData[
 StyleBox["\<\"{u,v}->geo[x][p,q,u,v] gives the system of first order \
differential \\nequations for a patch x:U->R^n. geo[ee,ff,gg][p,q,u,v] gives \
the system of first order \\ndifferential equations for the geodesics of a \
metric with coefficients ee,ff,gg.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.582380045663416*^9},
 CellTags->"Info3582387245-1359573"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"?", "solvegeoeqs"}]], "Input",
 CellChangeTimes->{{3.5823692714016657`*^9, 3.582369271557666*^9}, 
   3.5823693690422373`*^9, 3.582380019096569*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"geo", "[", 
   RowBox[{"sphere", "[", "r", "]"}], "]"}], "[", 
  RowBox[{"p", ",", "q", ",", "u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820051730478563`*^9, 3.582005207633117*^9}, {
  3.582005284151252*^9, 3.5820052986436768`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", 
   RowBox[{"2", " ", "p", " ", "q", " ", 
    RowBox[{"Tan", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["p", "2"]}], " ", 
    RowBox[{"Cos", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "v", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820053027152843`*^9, 3.5823488249043455`*^9, 
  3.582349423414997*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{"solvegeoeqs", "[", 
   RowBox[{
    RowBox[{"sphere", "[", "1", "]"}], ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "v"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], ",", 
    RowBox[{"2", "*", "Pi"}], ",", "2"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58200564190668*^9, 3.5820057510912714`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Rule]", 
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False]}], ",", 
     RowBox[{"v", "\[Rule]", 
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Rule]", 
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False]}], ",", 
     RowBox[{"v", "\[Rule]", 
      TagBox[
       RowBox[{"InterpolatingFunction", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
         ",", "\<\"<>\"\>"}], "]"}],
       False,
       Editable->False]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820057548976784`*^9, 3.5820072882711716`*^9, 
  3.582007406488179*^9, 3.5820087126004734`*^9, 3.582348836448366*^9, 
  3.582349465659871*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"solv", "=", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.582005972611661*^9, 3.5820059793820724`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solv", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5823509563048897`*^9, 3.5823509586136937`*^9}, {
  3.5823509887841463`*^9, 3.5823509960693593`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "\[Rule]", 
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False]}], ",", 
   RowBox[{"v", "\[Rule]", 
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5823509613436985`*^9, 3.5823509975669622`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"u", "\[Rule]", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], ",", 
        "\"\<<>\>\""}], "]"}],
      False,
      Editable->False], "[", "t", "]"}]}], ",", 
   RowBox[{"v", "\[Rule]", 
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], ",", 
       "\"\<<>\>\""}], "]"}],
     False,
     Editable->False]}]}], "}"}]], "Input",
 CellChangeTimes->{{3.582351130947196*^9, 3.582351143458418*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], ",", 
     "\"\<<>\>\""}], "]"}],
   False,
   Editable->False], "[", "1", "]"}]], "Input",
 CellChangeTimes->{3.5823511733792706`*^9},
 EmphasizeSyntaxErrors->True],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "noinfo"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Input expression contains insufficient information to \
interpret the result.\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.582351187169695*^9, 3.5823512120205383`*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   RowBox[{"InterpolatingFunction", "[", "]"}],
   False,
   Editable->False], "[", "1", "]"}]], "Output",
 CellChangeTimes->{{3.5823511819436855`*^9, 3.5823512120205383`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sphere", "[", "1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "/.", 
  RowBox[{"solv", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582006044870988*^9, 3.5820060521094007`*^9}, {
   3.582006123885126*^9, 3.582006125273529*^9}, 3.5820061572223845`*^9, {
   3.5820085134037232`*^9, 3.582008514823326*^9}, 3.582349487250309*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}], " ", 
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}], " ", 
    RowBox[{"Sin", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5823488511123915`*^9, 3.5823494885451117`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"igeo", "[", "t_", "]"}], " ", "=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Cos", "[", "u", "]"}], " ", 
      RowBox[{"Cos", "[", "v", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "v", "]"}], " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], ",", 
     RowBox[{"Sin", "[", "v", "]"}]}], "}"}], "/.", "\[VeryThinSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", "\[Rule]", 
      RowBox[{
       TagBox[
        RowBox[{"InterpolatingFunction", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], ",", 
          "\"\<<>\>\""}], "]"}],
        False,
        Editable->False], "[", "t", "]"}]}], ",", 
     RowBox[{"v", "\[Rule]", 
      RowBox[{
       TagBox[
        RowBox[{"InterpolatingFunction", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], ",", 
          "\"\<<>\>\""}], "]"}],
        False,
        Editable->False], "[", "t", "]"}]}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.582008555149397*^9, 3.5820086152875023`*^9}},
 EmphasizeSyntaxErrors->True],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "noinfo"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Input expression contains insufficient information to \
interpret the result.\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.582008626550722*^9, 3.582348913231701*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "noinfo"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Input expression contains insufficient information to \
interpret the result.\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.582008626550722*^9, 3.5823489132941008`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", "]"}],
       False,
       Editable->False], "[", "t", "]"}], "]"}], "2"], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", "]"}],
       False,
       Editable->False], "[", "t", "]"}], "]"}], " ", 
    RowBox[{"Sin", "[", 
     RowBox[{
      TagBox[
       RowBox[{"InterpolatingFunction", "[", "]"}],
       False,
       Editable->False], "[", "t", "]"}], "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{
     TagBox[
      RowBox[{"InterpolatingFunction", "[", "]"}],
      False,
      Editable->False], "[", "t", "]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582008626550722*^9, 3.5823489132941008`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"igeo", " ", "=", " ", "%"}], ";"}]], "Input",
 CellChangeTimes->{{3.5820081188322306`*^9, 3.5820081470214796`*^9}, {
   3.5820081910759573`*^9, 3.5820081987043705`*^9}, 3.5820084043751316`*^9, {
   3.5820084367295885`*^9, 3.582008445340804*^9}}],

Cell[CellGroupData[{

Cell[BoxData["igeo"], "Input",
 CellChangeTimes->{{3.5823495625984416`*^9, 3.582349608072521*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}], " ", 
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cos", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}], " ", 
    RowBox[{"Sin", "[", 
     TagBox[
      RowBox[{"InterpolatingFunction", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
        ",", "\<\"<>\"\>"}], "]"}],
      False,
      Editable->False], "]"}]}], ",", 
   RowBox[{"Sin", "[", 
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "6.283185307179586`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5823495656404467`*^9, {3.5823495998825073`*^9, 3.5823496094141235`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", "]"}],
         False,
         Editable->False], "[", "t", "]"}], "]"}], "2"], ",", 
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", "]"}],
         False,
         Editable->False], "[", "t", "]"}], "]"}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{
        TagBox[
         RowBox[{"InterpolatingFunction", "[", "]"}],
         False,
         Editable->False], "[", "t", "]"}], "]"}]}], ",", 
     RowBox[{"Sin", "[", 
      RowBox[{
       TagBox[
        RowBox[{"InterpolatingFunction", "[", "]"}],
        False,
        Editable->False], "[", "t", "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820062343333206`*^9, 3.5820062452689395`*^9}, {
   3.582006315609463*^9, 3.5820063502883244`*^9}, {3.5820063822371798`*^9, 
   3.5820064182420435`*^9}, {3.582007440121838*^9, 3.5820074402934384`*^9}, {
   3.582008458632027*^9, 3.582008472141651*^9}, 3.582350745423719*^9},
 EmphasizeSyntaxErrors->True],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "noinfo"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Input expression contains insufficient information to \
interpret the result.\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.582350755875737*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "sntxb"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Expression cannot begin with \
\\\"\[NoBreak]\\!\\(\\(\\(\\*TagBox[\\(InterpolatingFunction[]\\), False, \
Rule[Editable, False]]\\)\\)[t]\\)\[NoBreak]\\\"\[NoBreak]\\!\\(\\\"\\\"\\)\
\[NoBreak].\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5823507559693375`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Syntax", "::", "sntxi"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Incomplete expression; more input is needed \
\[NoBreak]\\!\\(\\\"\\\"\\)\[NoBreak].\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5823507560473375`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"sphere", "[", "1", "]"}], "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/."}]}]], "Input",
 CellChangeTimes->{{3.582005860587864*^9, 3.5820058843935056`*^9}, {
  3.582005919462367*^9, 3.582005922800773*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"geo", "[", 
   RowBox[{"ellipsoid", "[", 
    RowBox[{"r", ",", "1", ",", "1"}], "]"}], "]"}], "[", 
  RowBox[{"p", ",", "q", ",", "u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820051730478563`*^9, 3.582005207633117*^9}, {
  3.582005284151252*^9, 3.5820052986436768`*^9}, {3.5820053435249557`*^9, 
  3.5820053717766056`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"p", ",", "q", ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox["r", "2"]}], ")"}], " ", 
        RowBox[{"Cos", "[", "u", "]"}], " ", 
        RowBox[{"Sin", "[", "u", "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox["r", "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}]}], 
    "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["p", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["r", "2"]}], ")"}], " ", 
       RowBox[{"Cos", "[", "u", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "v", "]"}], "2"], " ", 
       RowBox[{"Sin", "[", "u", "]"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["r", "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{"2", " ", "p", " ", "q", " ", 
     RowBox[{"Tan", "[", "v", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["p", "2"], " ", 
        SuperscriptBox["r", "2"], " ", 
        RowBox[{"Cos", "[", "v", "]"}], " ", 
        RowBox[{"Sin", "[", "v", "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "4"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
        RowBox[{
         SuperscriptBox["r", "2"], " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
           RowBox[{"2", " ", 
            SuperscriptBox["r", "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
            SuperscriptBox[
             RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}]}], 
    "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox["r", "2"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
       RowBox[{"Cos", "[", "v", "]"}], " ", 
       RowBox[{"Sin", "[", "v", "]"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "4"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "v", "]"}], "2"]}], "+", 
       RowBox[{
        SuperscriptBox["r", "2"], " ", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "u", "]"}], "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"Cos", "[", "v", "]"}], "2"], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["r", "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "u", "]"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"Sin", "[", "v", "]"}], "2"]}]}], ")"}]}]}], ")"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.58200537421021*^9, 3.5823489696258*^9, 
  3.5823512676034365`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["grad. Grad", "Subsection",
 CellChangeTimes->{{3.5815707430730004`*^9, 3.5815707568946247`*^9}, {
  3.5815727202368746`*^9, 3.5815727272922783`*^9}}],

Cell[BoxData[
 RowBox[{"?", "grad"}]], "Input",
 CellChangeTimes->{{3.5823694176675224`*^9, 3.5823694203039274`*^9}, 
   3.582380019096569*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fq", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"x", "^", "2"}], "+", 
   RowBox[{"y", "^", "3"}]}]}]], "Input",
 CellChangeTimes->{{3.5815709042755685`*^9, 3.581570928596011*^9}, 
   3.5823695744165983`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"grad", "[", "fq", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.581570937800028*^9, 3.5815709486576467`*^9}, 
   3.5823696122154646`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", "x"}], ",", 
   RowBox[{"3", " ", 
    SuperscriptBox["y", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5815709519960527`*^9, 3.5815727602541637`*^9, 
  3.58236947329722*^9, 3.582369613744267*^9}]
}, Open  ]],

Cell["\<\
grad may be applied to more general expressions, e.g. Arrays:\
\>", "Text",
 CellChangeTimes->{{3.581572792668018*^9, 3.581572836924549*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"grad", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581572743562209*^9, 3.581572755437888*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"6", " ", "y"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5815727653624563`*^9, 3.5823696237750845`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"?", "Grad"}]], "Input",
 CellChangeTimes->{{3.5815709631656723`*^9, 3.5815709656772766`*^9}}],

Cell["\<\
grad is obsolete now, it may be replaced by the built-in Grad. Consider that \
for Grad the variables of the function must be entered explicitly.\
\>", "Text",
 CellChangeTimes->{{3.5815722344850917`*^9, 3.581572331612647*^9}, 
   3.581572401092621*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{"fq", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5815710019005404`*^9, 3.58157101314816*^9}, 
   3.5823696605131493`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.581571014364962*^9, 3.582369662775153*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{
   RowBox[{"fq", "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5815710019005404`*^9, 3.5815710505726256`*^9}, 
   3.582369671760769*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", " ", "x"}], ",", 
   RowBox[{"3", " ", 
    SuperscriptBox["y", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5815710514618273`*^9, 3.5815721460240316`*^9, 
  3.582369673367572*^9, 3.5823701614144287`*^9, 3.5823702373397627`*^9}]
}, Open  ]],

Cell["Grad may be applied to Arrays :", "Text",
 CellChangeTimes->{{3.581572417227544*^9, 3.58157243954082*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582370215764924*^9, 3.582370233112155*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"6", " ", "y"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5823702414113693`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grad", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581572149705242*^9, 3.5815721854542875`*^9}, {
   3.582370088874302*^9, 3.582370153614415*^9}, 3.5823702639690094`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "6"}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.582370154628417*^9, 3.582370165376836*^9}, 
   3.582370266870614*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Hessian", "Subsection",
 CellChangeTimes->{{3.5816619667612514`*^9, 3.5816619811370735`*^9}}],

Cell[BoxData[
 RowBox[{"?", "hessian"}]], "Input",
 CellChangeTimes->{{3.582374282186493*^9, 3.582374285103698*^9}, 
   3.5823800191277695`*^9}],

Cell[TextData[{
 "This definition  of \nWeisstein, Eric W. \[OpenCurlyDoubleQuote]Hessian.\
\[CloseCurlyDoubleQuote] From MathWorld--A Wolfram Web Resource. ",
 ButtonBox["http://mathworld.wolfram.com/Hessian.html",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://mathworld.wolfram.com/Hessian.html"], None},
  ButtonNote->"http://mathworld.wolfram.com/Hessian.html"]
}], "Text",
 CellChangeTimes->{{3.581662426352538*^9, 3.581662443978546*^9}, {
  3.581662516557698*^9, 3.581662516559698*^9}, {3.5823744957820683`*^9, 
  3.5823745311005297`*^9}}],

Cell["replaces the definition of Alfred Gray in CSPROGS.m.", "Text",
 CellChangeTimes->{{3.582374545265355*^9, 3.5823745710522003`*^9}, {
   3.582374610255069*^9, 3.5823746174154816`*^9}, 3.5823746634823627`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], " ", "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x", "^", "3"}], "+", 
     RowBox[{"y", "^", "3"}]}], ")"}], "*", 
   RowBox[{"z", "^", "3"}]}]}]], "Input",
 CellChangeTimes->{{3.5816612978329906`*^9, 3.581661317986143*^9}, {
  3.581661489178935*^9, 3.581661526259056*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"hessian", "[", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.581661664324953*^9, 3.5816617672038374`*^9}, {
  3.581663307179919*^9, 3.5816633157254076`*^9}, {3.5816648683322115`*^9, 
  3.5816648830430527`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"6", " ", "x", " ", 
       SuperscriptBox["z", "3"]}], "0", 
      RowBox[{"9", " ", 
       SuperscriptBox["x", "2"], " ", 
       SuperscriptBox["z", "2"]}]},
     {"0", 
      RowBox[{"6", " ", "y", " ", 
       SuperscriptBox["z", "3"]}], 
      RowBox[{"9", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["z", "2"]}]},
     {
      RowBox[{"9", " ", 
       SuperscriptBox["x", "2"], " ", 
       SuperscriptBox["z", "2"]}], 
      RowBox[{"9", " ", 
       SuperscriptBox["y", "2"], " ", 
       SuperscriptBox["z", "2"]}], 
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "3"], "+", 
         SuperscriptBox["y", "3"]}], ")"}], " ", "z"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.5816617522389812`*^9, 3.5816617685079117`*^9}, 
   3.5816633167574663`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["intrinsic. intrinsicea", "Subsection",
 CellChangeTimes->{{3.5816664891049147`*^9, 3.581666501024596*^9}}],

Cell["\<\
These functions calculate numerically and plots the plane curves with metric \
resp. equiaffine natural equation curvature[t] = fc[t] under the wanted start \
and border conditions.\
\>", "Text",
 CellChangeTimes->{{3.5816676589048233`*^9, 3.5816678171208725`*^9}, {
   3.5823747644457397`*^9, 3.5823747817773705`*^9}, 3.5823750443726315`*^9}],

Cell[BoxData[
 RowBox[{"?", "intrinsic"}]], "Input",
 CellChangeTimes->{{3.5823748003102026`*^9, 3.5823748065034137`*^9}, 
   3.5823800191277695`*^9}],

Cell[BoxData[
 RowBox[{"?", "intrinsicea"}]], "Input",
 CellChangeTimes->{{3.5823748003102026`*^9, 3.5823748065034137`*^9}, {
   3.582374854223898*^9, 3.582374862944313*^9}, 3.582380019143369*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", "t_", "]"}], " ", "=", " ", 
  RowBox[{"Exp", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.581665391183117*^9, 3.581665398913559*^9}, 
   3.581666122762961*^9, {3.581666235139388*^9, 3.581666250642275*^9}, 
   3.5823750461354346`*^9}],

Cell[BoxData[
 SuperscriptBox["\[ExponentialE]", "t"]], "Output",
 CellChangeTimes->{3.5816654000206223`*^9, 3.581666124008032*^9, 
  3.5816662519073477`*^9, 3.5823751371771946`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"intrinsic", "[", 
   RowBox[{"fc", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}], "[", "t", 
  "]"}]], "Input",
 CellChangeTimes->{{3.5816654047648935`*^9, 3.58166547760306*^9}, {
   3.581665516386278*^9, 3.5816655274269094`*^9}, 3.5816655774437704`*^9, {
   3.581665612659785*^9, 3.5816656144028845`*^9}, {3.5816657672836285`*^9, 
   3.581665823378837*^9}, {3.5816658811231403`*^9, 3.5816659258036957`*^9}, 
   3.5816659571404877`*^9, {3.5816660597403564`*^9, 3.581666066866764*^9}, {
   3.5816663175721035`*^9, 3.5816663199152374`*^9}, 3.5823751598284345`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}], ",", 
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1.`"}], ",", "1.`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.58166577639215*^9, 3.581665825907982*^9}, {
   3.5816658904366727`*^9, 3.5816659276047983`*^9}, 3.581665967260067*^9, 
   3.5816660684588547`*^9, {3.5816661281102667`*^9, 3.58166615247266*^9}, 
   3.5816662065767546`*^9, 3.581666258612731*^9, {3.5816662935737305`*^9, 
   3.5816663219003506`*^9}, {3.582375141498402*^9, 3.582375161357237*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", "t_", "]"}], " ", ":=", " ", "t"}]], "Input",
 CellChangeTimes->{{3.5816669104560146`*^9, 3.5816669191385107`*^9}, 
   3.5823750477890377`*^9}],

Cell["The next cell will not be evaluatedcorrectly:", "Text",
 CellChangeTimes->{{3.581666933577337*^9, 3.5816669836802025`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"intrinsic", "[", 
   RowBox[{"t", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}]}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5816656271126113`*^9, 3.5816656271566143`*^9}, 
   3.5816668817313714`*^9, {3.5816669586757727`*^9, 3.5816669603308673`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"intrinsic", "[", 
   RowBox[{"fc", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "10"}], ",", "10"}], "}"}]}], "]"}], "[", "t", 
  "]"}]], "Input",
 CellChangeTimes->{{3.5816656271126113`*^9, 3.5816656271566143`*^9}, 
   3.5816668817313714`*^9, {3.5816669586757727`*^9, 3.5816669603308673`*^9}, {
   3.5816669914726486`*^9, 3.5816670314429345`*^9}, 3.5823752026817093`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "10.`"}], ",", "10.`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}], ",", 
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "10.`"}], ",", "10.`"}], "}"}], "}"}], 
       ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5816669998461275`*^9, 3.581667032387988*^9, 
  3.5823752328053627`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", 
     RowBox[{"-", "10"}], ",", "10"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5816670353401575`*^9, 3.581667059650548*^9}}],

Cell[BoxData[
 RowBox[{"?", "plotintrinsic"}]], "Input",
 CellChangeTimes->{{3.5823752680146246`*^9, 3.58237527132183*^9}, 
   3.582380019158969*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"intrinsicea", "[", 
   RowBox[{"fc", ",", "0", ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "5"}], "}"}]}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.5816672232709064`*^9, 3.581667234361541*^9}, {
   3.581667274804854*^9, 3.5816673600267286`*^9}, 3.5816674576373115`*^9, 
   3.581667502739891*^9, 3.582375309573097*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "5.`"}], "}"}], "}"}], ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}], ",", 
   RowBox[{
    TagBox[
     RowBox[{"InterpolatingFunction", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.`", ",", "5.`"}], "}"}], "}"}], ",", "\<\"<>\"\>"}], "]"}],
     False,
     Editable->False], "[", "t", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5816673609927835`*^9, 3.581667462179571*^9, 
  3.581667504021964*^9, 3.582375315688308*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ParametricPlot", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", "5"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581667374238541*^9, 3.581667400899066*^9}, 
   3.5816675150355945`*^9}],

Cell[BoxData[
 RowBox[{"?", "plotintrinsicea"}]], "Input",
 CellChangeTimes->{{3.582375340149151*^9, 3.5823753417403536`*^9}, 
   3.582380019174569*^9}],

Cell[BoxData[
 RowBox[{"plotintrinsicea", "[", 
  RowBox[{"fc", ",", "0", ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "10"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5816672232709064`*^9, 3.581667234361541*^9}, {
   3.581667274804854*^9, 3.5816673600267286`*^9}, 3.5816674576373115`*^9, 
   3.581667502739891*^9, {3.5816675482194924`*^9, 3.5816675492155495`*^9}, {
   3.581667579746296*^9, 3.581667591658977*^9}, {3.582375387916435*^9, 
   3.582375408664471*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["kappa2implicit", "Subsection",
 CellChangeTimes->{3.5805284585672665`*^9}],

Cell[BoxData[
 RowBox[{"?", "kappa2implicit"}]], "Input",
 CellChangeTimes->{{3.5823755648207455`*^9, 3.5823755663807487`*^9}, 
   3.5823800191901693`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"gc", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"x", "*", 
   RowBox[{"y", "^", "2", " "}]}]}]], "Input",
 CellChangeTimes->{{3.5804517433364644`*^9, 3.580451764068901*^9}, {
   3.5804518094805803`*^9, 3.5804518174833946`*^9}, 3.5804539642267656`*^9, 
   3.5804540628033385`*^9, {3.5823756439284844`*^9, 3.5823756768757424`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"kappa2implicit", "[", "gc", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.580451370355409*^9, 3.5804513877026396`*^9}, 
   3.5804514269055085`*^9, {3.580451484032809*^9, 3.5804515097572536`*^9}, {
   3.5804518050345726`*^9, 3.580451839105033*^9}, {3.580452001002117*^9, 
   3.580452004153322*^9}, 3.580453620183761*^9, {3.5823756939889727`*^9, 
   3.582375697124578*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"6", " ", "x", " ", 
    SuperscriptBox["y", "4"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", 
       SuperscriptBox["x", "2"], " ", 
       SuperscriptBox["y", "2"]}], "+", 
      SuperscriptBox["y", "4"]}], ")"}], 
    RowBox[{"3", "/", "2"}]]]}]], "Output",
 CellChangeTimes->{3.580451460429968*^9, 3.5804515127680597`*^9, 
  3.5804518498846517`*^9, 3.5804520081469297`*^9, 3.580453621696964*^9, 
  3.5804539784383907`*^9, 3.5804540776233644`*^9, 3.5804654718232484`*^9, 
  3.58237569821658*^9}]
}, Open  ]],

Cell["\<\
The next cell  yields nonsense since the syntax rule; \
\[OpenCurlyDoubleQuote]g is the name of the functiob only, without the \
arguments\[CloseCurlyDoubleQuote], is hurt.\
\>", "Text",
 CellChangeTimes->{{3.580465550525387*^9, 3.580465564456211*^9}, {
   3.5804656040958805`*^9, 3.5804656986944466`*^9}, {3.5823755120302525`*^9, 
   3.5823755208910685`*^9}, 3.5823757292294345`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"kappa2implicit", "[", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"u", "^", "2"}], "+", 
     RowBox[{"v", "^", "2"}]}], ")"}], "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.580451370355409*^9, 3.5804513877026396`*^9}, 
   3.5804514269055085`*^9, {3.580451484032809*^9, 3.5804515097572536`*^9}, {
   3.5804518050345726`*^9, 3.580451839105033*^9}, {3.580452001002117*^9, 
   3.580452004153322*^9}, 3.580453620183761*^9, {3.580453765685217*^9, 
   3.580453786480053*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "2"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"1", ",", "1"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], 
       TagBox[
        RowBox[{"(", 
         RowBox[{"2", ",", "0"}], ")"}],
        Derivative],
       MultilineFunction->None], "[", 
      RowBox[{"u", ",", "v"}], "]"}]}]}], ")"}], "/", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"], "+", 
     SuperscriptBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], 
        TagBox[
         RowBox[{"(", 
          RowBox[{"1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"u", ",", "v"}], "]"}], "2"]}], ")"}], 
   RowBox[{"3", "/", "2"}]]}]], "Output",
 CellChangeTimes->{3.5804537918464627`*^9, 3.5804655086861134`*^9, 
  3.5823757341590433`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Laplacian", "Subsection",
 CellChangeTimes->{{3.581737417837372*^9, 3.581737421956608*^9}}],

Cell["\<\
laplacian is obsolete now, it may be replaced by the built-in Laplacian:\
\>", "Text",
 CellChangeTimes->{{3.5817375089725847`*^9, 3.581737554588194*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Laplacian"}]], "Input",
 CellChangeTimes->{{3.5817366025037374`*^9, 3.581736618770668*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Laplacian\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TI\\\"], \
\\\",\\\", SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"n\\\", \
\\\"TI\\\"]]}], \\\"}\\\"}]}], \\\"]\\\"}]\\) gives the Laplacian \
\\!\\(\\*RowBox[{RowBox[{RowBox[{SuperscriptBox[\\\"\[PartialD]\\\", \
\\\"2\\\"], StyleBox[\\\"f\\\", \\\"TI\\\"]}], \\\"/\\\", RowBox[{\\\"\
\[PartialD]\\\", SuperscriptBox[SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
\\\"1\\\"], \\\"2\\\"]}]}], \\\"+\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\
\"], \\\"+\\\", RowBox[{RowBox[{SuperscriptBox[\\\"\[PartialD]\\\", \
\\\"2\\\"], StyleBox[\\\"f\\\", \\\"TI\\\"]}], \\\"/\\\", RowBox[{\\\"\
\[PartialD]\\\", SuperscriptBox[SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], \
StyleBox[\\\"n\\\", \\\"TI\\\"]], \\\"2\\\"]}]}]}]\\) .\\n\\!\\(\\*RowBox[{\\\
\"Laplacian\\\", \\\"[\\\", RowBox[{StyleBox[\\\"f\\\", \\\"TI\\\"], \
\\\",\\\", RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\
\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", \
StyleBox[\\\"\[Ellipsis]\\\", \\\"TI\\\"], \\\",\\\", \
SubscriptBox[StyleBox[\\\"x\\\", \\\"TI\\\"], StyleBox[\\\"n\\\", \
\\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", StyleBox[\\\"chart\\\", \
\\\"TI\\\"]}], \\\"]\\\"}]\\) gives the Laplacian in the given coordinates \
\\!\\(\\*StyleBox[\\\"chart\\\", \\\"TI\\\"]\\).\"\>", "MSG"], 
  "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Laplacian"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.581736620779783*^9},
 CellTags->"Info3581743820-8655308"]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"gq", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x", "^", "3"}], "+", 
     RowBox[{"y", "^", "3"}]}], ")"}], "*", 
   RowBox[{"Cos", "[", "z", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.581736779361853*^9, 3.5817368332709365`*^9}, 
   3.5823769512887807`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Laplacian", "[", 
  RowBox[{"gq", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581736845248622*^9, 3.581736871359115*^9}, 
   3.5823769524899826`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.581736872403175*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Laplacian", "[", 
  RowBox[{
   RowBox[{"gq", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581736845248622*^9, 3.5817368921353035`*^9}, 
   3.5823769535195847`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"6", " ", "x", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "+", 
  RowBox[{"6", " ", "y", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "3"], "+", 
     SuperscriptBox["y", "3"]}], ")"}], " ", 
   RowBox[{"Cos", "[", "z", "]"}]}]}]], "Output",
 CellChangeTimes->{3.581736893632389*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"laplacian", "[", "gq", "]"}], "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellChangeTimes->{{3.581736919085845*^9, 3.581736924266141*^9}, {
   3.58173696218231*^9, 3.5817369966312804`*^9}, 3.5823769545647864`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"6", " ", "x", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "+", 
  RowBox[{"6", " ", "y", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "3"], "+", 
     SuperscriptBox["y", "3"]}], ")"}], " ", 
   RowBox[{"Cos", "[", "z", "]"}]}]}]], "Output",
 CellChangeTimes->{3.581736997708342*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"laplacian", "[", 
  RowBox[{
   RowBox[{"gq", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581736919085845*^9, 3.581736924266141*^9}, {
   3.58173696218231*^9, 3.5817369966312804`*^9}, {3.581737347284336*^9, 
   3.5817373894647493`*^9}, 3.5817375827998075`*^9, 3.5823769578251925`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"6", " ", "x", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "+", 
  RowBox[{"6", " ", "y", " ", 
   RowBox[{"Cos", "[", "z", "]"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "3"], "+", 
     SuperscriptBox["y", "3"]}], ")"}], " ", 
   RowBox[{"Cos", "[", "z", "]"}]}]}]], "Output",
 CellChangeTimes->{3.5817376291484585`*^9}]
}, Closed]],

Cell["\<\
In the next cell gq is considered to be constant:\
\>", "Text",
 CellChangeTimes->{{3.5817376655405397`*^9, 3.5817376878608165`*^9}, 
   3.5823769623336*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"laplacian", "[", 
  RowBox[{"gg", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581736919085845*^9, 3.581736924266141*^9}, {
   3.58173696218231*^9, 3.5817369966312804`*^9}, {3.581737347284336*^9, 
   3.5817373894647493`*^9}, 3.581737588183115*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.5817373671174707`*^9, 3.5817373923589144`*^9}, 
   3.5817375893101797`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Minimal Curves. studycurvature. geomincurve", "Subsection",
 CellChangeTimes->{{3.5820333416945305`*^9, 3.5820333495249786`*^9}, {
  3.582036058166904*^9, 3.5820360699085755`*^9}, {3.5820361786287937`*^9, 
  3.582036191036504*^9}}],

Cell[TextData[{
 "A complex curve gamma: \[DoubleStruckCapitalC] \[LongRightArrow]  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "n"], TraditionalForm]]],
 "  is said to be",
 StyleBox[" isotropic",
  FontColor->RGBColor[0, 0, 1]],
 " or ",
 StyleBox["minimal",
  FontColor->RGBColor[0, 0, 1]],
 ", if"
}], "Text",
 CellChangeTimes->{{3.582033724621433*^9, 3.5820337380522013`*^9}, {
  3.582033772684182*^9, 3.582033897228306*^9}}],

Cell["\<\
D[gamma[z], z].D[gamma[z], z] == 0, D[gamma[z], z] != {0, 0, ...}\
\>", "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5820339150773263`*^9, 3.58203404468674*^9}, {
   3.5820340820508766`*^9, 3.582034097094737*^9}, 3.582375922888174*^9},
 FormatType->"TextForm"],

Cell["\<\
The real and the imaginary part of the curve in case n = 3 define minimal \
surfaces by\
\>", "Text",
 CellChangeTimes->{{3.582034141812295*^9, 3.5820342046768904`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"sf", "[", "gamma_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{"gamma", "[", 
     RowBox[{"u", "+", 
      RowBox[{"I", "*", "v"}]}], "]"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"u", "\[Element]", "Reals"}], "&&", 
    RowBox[{"v", "\[Element]", "Reals"}]}]}]}]], "Input",
 CellChangeTimes->{{3.5820342195877433`*^9, 3.582034308639837*^9}, 
   3.5820343739365716`*^9}],

Cell["\<\
For minimal curves the following invariant  defined by Alfred is of interest:\
\>", "Text",
 CellChangeTimes->{{3.582034435245078*^9, 3.5820344764524355`*^9}}],

Cell[BoxData[
 RowBox[{"?", "studycurvature"}]], "Input",
 CellChangeTimes->{{3.5820344867890263`*^9, 3.582034501854888*^9}, 
   3.5823759695790567`*^9, 3.5823800192057695`*^9}],

Cell["The studycurvature makes sense for real curves too", "Text",
 CellChangeTimes->{{3.58203482410032*^9, 3.582034865996716*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studycurvature", "[", 
   RowBox[{"helix", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{{3.582029131111699*^9, 3.5820291488637147`*^9}, {
  3.5820292995683346`*^9, 3.582029333024248*^9}}],

Cell[BoxData["b"], "Output",
 CellChangeTimes->{3.58202933497536*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"helix", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "[", "t", "]"}], ",", "t"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5820294326699476`*^9, 3.5820294493279*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "a"}], " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", "b"}], "}"}]], "Output",
 CellChangeTimes->{3.5820294544911957`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", ".", "%"}]], "Input",
 CellChangeTimes->{{3.5820294605215406`*^9, 3.582029465726838*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["b", "2"], "+", 
  RowBox[{
   SuperscriptBox["a", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "t", "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox["a", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "t", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.5820294670739155`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"helix", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "[", "t", "]"}]], "Input",
 CellChangeTimes->{3.5820295722149286`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "t", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Sin", "[", "t", "]"}]}], ",", 
   RowBox[{"b", " ", "t"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582029575939142*^9}]
}, Closed]],

Cell["We consider the \[OpenCurlyDoubleQuote]complex helix", "Text",
 CellChangeTimes->{{3.582034931493462*^9, 3.582034939124899*^9}, {
  3.582034977157074*^9, 3.582034992515952*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"zhelix", "[", "a_", "]"}], "[", "z_", "]"}], " ", ":=", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", " ", 
     RowBox[{"Cos", "[", "z", "]"}]}], ",", 
    RowBox[{"a", " ", 
     RowBox[{"Sin", "[", "z", "]"}]}], ",", 
    RowBox[{"I", "*", "a", " ", "z"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5820295000578012`*^9, 3.5820295268713355`*^9}, {
  3.5820296001995296`*^9, 3.5820296130722656`*^9}, {3.582034895200386*^9, 
  3.5820349070400634`*^9}}],

Cell["being a minimal curve", "Text",
 CellChangeTimes->{{3.582035007108787*^9, 3.5820350155482693`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"zhelix", "[", "a", "]"}], "[", "z", "]"}], ",", "z"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.582011134304327*^9, 3.5820111458951473`*^9}, {
  3.58202966280011*^9, 3.582029670383544*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "a"}], " ", 
    RowBox[{"Sin", "[", "z", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "z", "]"}]}], ",", 
   RowBox[{"\[ImaginaryI]", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582011147018349*^9, 3.5820283648376045`*^9, 
  3.582028525876687*^9, 3.5820285622091513`*^9, 3.5820296868284845`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", ".", "%"}]], "Input",
 CellChangeTimes->{{3.5820111528059597`*^9, 3.5820111550055633`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   SuperscriptBox["a", "2"]}], "+", 
  RowBox[{
   SuperscriptBox["a", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "z", "]"}], "2"]}], "+", 
  RowBox[{
   SuperscriptBox["a", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "z", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.5820111561755652`*^9, 3.582028369174412*^9, 
  3.582028532537899*^9, 3.582028566202758*^9, 3.582029692247794*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.582029699645218*^9, 3.582029708742738*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5820297099218054`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studycurvature", "[", 
   RowBox[{"zhelix", "[", "a", "]"}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.5820284039468727`*^9, 3.582028428548116*^9}, {
  3.582029724272626*^9, 3.58202972975194*^9}}],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", "a"}]], "Output",
 CellChangeTimes->{3.5820284304981194`*^9, 3.582028570617566*^9, 
  3.5820297326551056`*^9}]
}, Closed]],

Cell["The corresponding minimal surface is", "Text",
 CellChangeTimes->{{3.582035054676508*^9, 3.5820350692603416`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"shelix", "[", "a_", "]"}], "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"zhelix", "[", "a", "]"}], "[", 
      RowBox[{"u", "+", 
       RowBox[{"I", "*", "v"}]}], "]"}], "]"}], ",", 
    RowBox[{
     RowBox[{"a", "\[Element]", "Reals"}], "&&", 
     RowBox[{"u", "\[Element]", "Reals"}], "&&", 
     RowBox[{"v", "\[Element]", "Reals"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5820298092344856`*^9, 3.582029902776836*^9}, {
  3.582029941330041*^9, 3.5820299531457167`*^9}, {3.5820300006244326`*^9, 
  3.5820300315932035`*^9}, {3.582030229298512*^9, 3.582030231220622*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Re", "[", 
     RowBox[{"Cos", "[", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Re", "[", 
     RowBox[{"Sin", "[", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "a"}], " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582030043153865*^9, 3.5820302350648413`*^9, 
  3.5820350816870527`*^9, 3.5820937819898105`*^9, 3.5820938685699625`*^9, 
  3.582093977848154*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Element]", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582093799571041*^9, 3.582093815295869*^9}, 
   3.5820938574471426`*^9, {3.582094552615564*^9, 3.5820945540039663`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{"Cos", "[", 
     RowBox[{"u", "+", 
      RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}], ",", 
   RowBox[{"Re", "[", 
    RowBox[{"Sin", "[", 
     RowBox[{"u", "+", 
      RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}], ",", 
   RowBox[{"-", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820938250614862`*^9, 3.582093876276376*^9, 
  3.5820939833081636`*^9, 3.5820945586839743`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrigSimplify", "[", 
  RowBox[{
   RowBox[{"TrigExpand", "[", "%", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Element]", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582093799571041*^9, 3.582093815295869*^9}, 
   3.5820938574471426`*^9, {3.582094117187599*^9, 3.58209414054084*^9}, {
   3.582094200928546*^9, 3.5820942114273643`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Re", "[", 
     RowBox[{"Cos", "[", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Re", "[", 
     RowBox[{"Sin", "[", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "a"}], " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.582094146796451*^9, 3.5820942168717737`*^9}]
}, Closed]],

Cell[TextData[{
 StyleBox["Unfortunately, Mathematica",
  FontSlant->"Italic"],
 " may not simplify this expression as expected. Direct application without \
simplification yields no result"
}], "Text",
 CellChangeTimes->{{3.5820303183336043`*^9, 3.582030342604993*^9}, {
  3.58203058242971*^9, 3.5820306200768633`*^9}, {3.582035175997447*^9, 
  3.5820352685487404`*^9}, {3.5820355304047174`*^9, 3.5820355333888884`*^9}}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"msf", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582035297825415*^9, 3.5820353082190094`*^9}, {
  3.5820354617267895`*^9, 3.582035520159132*^9}}],

Cell["\<\
Application of TrigExpand to the trigonometric expressions yields the wanted \
simplification:\
\>", "Text",
 CellChangeTimes->{{3.5820946534229407`*^9, 3.582094695589815*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"Re", "[", 
       RowBox[{"TrigExpand", "[", 
        RowBox[{"Cos", "[", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}], "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"Re", "[", 
       RowBox[{"TrigExpand", "[", 
        RowBox[{"Sin", "[", 
         RowBox[{"u", "+", 
          RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}], "]"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", "a"}], " ", "v"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Element]", "Reals"}], ",", 
     RowBox[{"u", "\[Element]", "Reals"}], ",", 
     RowBox[{"v", "\[Element]", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582093799571041*^9, 3.582093815295869*^9}, 
   3.5820938574471426`*^9, {3.582094117187599*^9, 3.58209414054084*^9}, {
   3.582094200928546*^9, 3.5820942114273643`*^9}, {3.5820942872902975`*^9, 
   3.5820943885344753`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
   RowBox[{"a", " ", 
    RowBox[{"Cosh", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", "a"}], " ", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5820943908276796`*^9}]
}, Closed]],

Cell["Thus we redefine", "Text",
 CellChangeTimes->{{3.5820947340594826`*^9, 3.5820947385522904`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "shelix", "]"}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"shelix", "[", "a_", "]"}], "[", 
    RowBox[{"u_", ",", "v_"}], "]"}], " ", ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"Cos", "[", "u", "]"}], " ", 
      RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"Cosh", "[", "v", "]"}], " ", 
      RowBox[{"Sin", "[", "u", "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", "a"}], " ", "v"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.5820298092344856`*^9, 3.582029902776836*^9}, {
  3.582029941330041*^9, 3.5820299531457167`*^9}, {3.5820300006244326`*^9, 
  3.5820300315932035`*^9}, {3.582030229298512*^9, 3.582030231220622*^9}, {
  3.5820350864603257`*^9, 3.5820351103356915`*^9}, {3.5820947441527004`*^9, 
  3.5820948059600086`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"msf", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{
   RowBox[{"shelix", "[", "1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582035342982998*^9, 3.582035420376425*^9}, 
   3.582094455130993*^9, {3.582094824648842*^9, 3.5820948531032915`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"Cosh", "[", "v", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cosh", "[", "v", "]"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"-", "v"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5820354229305706`*^9, 3.582094458328998*^9, {3.5820948447728767`*^9, 
   3.5820948559580965`*^9}, 3.5823767718260655`*^9}]
}, Open  ]],

Cell["msf describes a minimal surface:", "Text",
 CellChangeTimes->{{3.582035853452195*^9, 3.582035867892021*^9}, 
   3.5820360220678396`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meancurvature", "[", "msf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.582030171114184*^9, 3.5820301953425694`*^9}, 
   3.582036022068839*^9}],

Cell[BoxData["0"], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5820301976767035`*^9, 3.5820309357579193`*^9, 
  3.58203602207084*^9, 3.582094891276559*^9, 3.5823767757416725`*^9}]
}, Open  ]],

Cell["namely the catenoid", "Text",
 CellChangeTimes->{{3.5820358746474075`*^9, 3.58203589585262*^9}, 
   3.5820360220728397`*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"msf", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5820309722260046`*^9, 3.5820310252810397`*^9}, 
   3.5820360220728397`*^9}],

Cell["Its Gaussian curvature is", "Text",
 CellChangeTimes->{{3.582035977636298*^9, 3.5820359893399677`*^9}, 
   3.58203602208284*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussiancurvature", "[", "msf", "]"}], "[", 
  RowBox[{"u", ",", "v"}], "]"}]], "Input",
 CellChangeTimes->{{3.5820310858115015`*^9, 3.582031102239441*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  SuperscriptBox[
   RowBox[{"Sech", "[", "v", "]"}], "4"]}]], "Output",
 CellChangeTimes->{3.5820311076647515`*^9, 3.582094915378601*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["geomincurve", "Subsubsection",
 CellChangeTimes->{{3.581418345642107*^9, 3.5814183558436904`*^9}, 
   3.581427101422909*^9}],

Cell[BoxData[
 RowBox[{"?", "geomincurve"}]], "Input",
 CellChangeTimes->{{3.582376241019533*^9, 3.5823762439991384`*^9}, 
   3.5823800192369695`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"cc", "[", "z_", "]"}], " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Cos", "[", "z", "]"}], ",", 
    RowBox[{"Sin", "[", "z", "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.581421256949624*^9, 3.5814212856192636`*^9}, 
   3.5814226446279945`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", "z", "]"}], ",", 
   RowBox[{"Sin", "[", "z", "]"}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.5814212909365683`*^9, {3.581422644629995*^9, 3.581422659807863*^9}, 
   3.582376349034123*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"geomincurve", "[", "cc", "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.581421303872308*^9, 3.5814213223793664`*^9}, 
   3.5814226446329947`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", "z", "]"}], ",", 
   RowBox[{"Sin", "[", "z", "]"}], ",", 
   RowBox[{"\[ImaginaryI]", " ", "z"}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.581421323718443*^9, 3.581421419521923*^9, {3.581422644633995*^9, 
   3.5814226598688664`*^9}, 3.582376365492152*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", " ", 
  RowBox[{"z", "\[Rule]", " ", 
   RowBox[{"x", "+", 
    RowBox[{"I", "*", "y"}]}]}]}]], "Input",
 CellChangeTimes->{{3.581421429540496*^9, 3.5814214508427143`*^9}, 
   3.5814226446369953`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Cos", "[", 
    RowBox[{"x", "+", 
     RowBox[{"\[ImaginaryI]", " ", "y"}]}], "]"}], ",", 
   RowBox[{"Sin", "[", 
    RowBox[{"x", "+", 
     RowBox[{"\[ImaginaryI]", " ", "y"}]}], "]"}], ",", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{"x", "+", 
      RowBox[{"\[ImaginaryI]", " ", "y"}]}], ")"}]}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.581421455373973*^9, {3.5814226446379952`*^9, 3.5814226598838673`*^9}, 
   3.582376378128174*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Re", "[", 
    RowBox[{"TrigExpand", "[", "%", "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"x", ">", "0"}], "&&", 
    RowBox[{"y", ">", "0"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5814215223988066`*^9, 3.5814215738117476`*^9}, 
   3.5814226446399956`*^9, {3.582376421761451*^9, 3.5823764461286936`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "x", "]"}], " ", 
    RowBox[{"Cosh", "[", "y", "]"}]}], ",", 
   RowBox[{
    RowBox[{"Cosh", "[", "y", "]"}], " ", 
    RowBox[{"Sin", "[", "x", "]"}]}], ",", 
   RowBox[{"-", "y"}]}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{
  3.581421576359893*^9, {3.5814226446419954`*^9, 3.581422659898868*^9}, 
   3.582376447548296*^9}]
}, Open  ]],

Cell["The real part of the curve geomincurve[cc][z] is", "Text",
 CellChangeTimes->{{3.581426862737257*^9, 3.5814268999443846`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"min", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "x", "]"}], "*", 
     RowBox[{"Cosh", "[", "y", "]"}]}], ",", 
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], "*", 
     RowBox[{"Cosh", "[", "y", "]"}]}], ",", 
    RowBox[{"-", "y"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5814227932684965`*^9, 3.5814227979027615`*^9}, {
   3.5814228344098496`*^9, 3.581422847866619*^9}, {3.581422928146211*^9, 
   3.5814229358186502`*^9}, {3.5814229775950394`*^9, 
   3.5814230216035566`*^9}, {3.5814244527964163`*^9, 3.581424479713956*^9}, {
   3.5814245164270554`*^9, 3.5814246986934805`*^9}, {3.5814248049505577`*^9, 
   3.5814248101978583`*^9}, {3.5814249102235794`*^9, 3.581424921125203*^9}, {
   3.5814265746857815`*^9, 3.5814265957729874`*^9}, 3.582376559135292*^9}],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"min", "[", 
    RowBox[{"x", ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5814247165525017`*^9, 3.581424770403582*^9}, 
   3.582376595155755*^9}],

Cell["\<\
The result is a minimal surface (a catenoid):\
\>", "Text",
 CellChangeTimes->{{3.5814268104572663`*^9, 3.5814268238890347`*^9}, {
  3.5823766902535224`*^9, 3.5823767072263517`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"meancurvature", "[", "min", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.5814226918606963`*^9, 3.581422719234262*^9}, 
   3.5814248358353243`*^9, 3.582376608478179*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5814227231024833`*^9, 3.581424842155686*^9, 
  3.581424934974995*^9, 3.581426615331106*^9, 3.582376610240982*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"gaussiancurvature", "[", "min", "]"}], "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.582376614109789*^9, 3.5823766374006295`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  SuperscriptBox[
   RowBox[{"Sech", "[", "y", "]"}], "4"]}]], "Output",
 CellChangeTimes->{3.582376641097836*^9}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["oval", "Subsection",
 CellChangeTimes->{{3.581484054552539*^9, 3.581484056315342*^9}}],

Cell[BoxData[
 RowBox[{"?", "oval"}]], "Input",
 CellChangeTimes->{{3.5823770900510244`*^9, 3.582377092359829*^9}, 
   3.5823800192681694`*^9}],

Cell["\<\
As a rule, the result is not an oval, but the envelope of the 1-parametric \
line family\
\>", "Text",
 CellChangeTimes->{{3.5814862558826275`*^9, 3.581486308506637*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"x", "*", 
     RowBox[{"Cos", "[", "t", "]"}]}], " ", "+", 
    RowBox[{"y", "*", "Sin", 
     RowBox[{"{", "t"}]}]}], "]"}], " ", "\[Equal]", " ", 
  RowBox[{"p", "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.581486339119388*^9, 3.5814863659169207`*^9}}],

Cell["The equations of the envelope are", "Text",
 CellChangeTimes->{{3.5814864277784595`*^9, 3.581486429249543*^9}, {
  3.581486806922145*^9, 3.581486838073927*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"envelope", "[", "p_", "]"}], "[", "t_", "]"}], " ", ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"x", "*", 
       RowBox[{"Cos", "[", "t", "]"}]}], " ", "+", 
      RowBox[{"y", "*", 
       RowBox[{"Sin", "[", "t", "]"}]}]}], " ", "\[Equal]", " ", 
     RowBox[{"p", "[", "t", "]"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "x"}], "*", 
       RowBox[{"Sin", "[", "t", "]"}]}], " ", "+", " ", 
      RowBox[{"y", "*", 
       RowBox[{"Cos", "[", "t", "]"}]}]}], "\[Equal]", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"p", "[", "t", "]"}], ",", "t"}], "]"}]}]}], "}"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.58148686435143*^9, 3.5814870430306497`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"envelope", "[", "p", "]"}], "[", "t", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581487062721776*^9, 3.5814870944445906`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Cos", "[", "t", "]"}]}], " ", 
         RowBox[{"p", "[", "t", "]"}]}], "+", 
        RowBox[{
         RowBox[{"Sin", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["p", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}]}], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"Cos", "[", "t", "]"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"Sin", "[", "t", "]"}], "2"]}]]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Csc", "[", "t", "]"}]}], " ", 
         RowBox[{"p", "[", "t", "]"}]}], "-", 
        RowBox[{
         RowBox[{"Cot", "[", "t", "]"}], " ", 
         RowBox[{"Csc", "[", "t", "]"}], " ", 
         RowBox[{
          SuperscriptBox["p", "\[Prime]",
           MultilineFunction->None], "[", "t", "]"}]}]}], 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"Cot", "[", "t", "]"}], "2"]}]]}]}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.581487096615714*^9, 3.5823772056316276`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], "/.", 
   RowBox[{"Flatten", "[", "%", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.581487158263241*^9, 3.5814871899410524`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "t", "]"}], " ", 
     RowBox[{"p", "[", "t", "]"}]}], "-", 
    RowBox[{
     RowBox[{"Sin", "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["p", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"p", "[", "t", "]"}], " ", 
     RowBox[{"Sin", "[", "t", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "t", "]"}], " ", 
     RowBox[{
      SuperscriptBox["p", "\[Prime]",
       MultilineFunction->None], "[", "t", "]"}]}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5814871945323153`*^9, 3.5823772177216487`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "\[Equal]", 
  RowBox[{
   RowBox[{"oval", "[", "p", "]"}], "[", "t", "]"}]}]], "Input",
 CellChangeTimes->{{3.581487229548318*^9, 3.581487261348137*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.58148726438531*^9, 3.5823772365040817`*^9}]
}, Open  ]],

Cell["\<\
Plot the envelopes for distinct support functions fc:\
\>", "Text",
 CellChangeTimes->{{3.5814889819225483`*^9, 3.581488983058613*^9}, {
  3.5814893152346125`*^9, 3.581489363658382*^9}, {3.582374968291298*^9, 
  3.582374972846506*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"t", "^", "2"}]}]], "Input",
 CellChangeTimes->{{3.5814834154350166`*^9, 3.5814834266826363`*^9}, 
   3.5823749836885247`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"oval", "[", 
   RowBox[{
    RowBox[{"fc", "[", "#", "]"}], "&"}], "]"}], "[", 
  RowBox[{
   RowBox[{"-", "10"}], ",", "10", ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{3.582374988196933*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", "t_", "]"}], " ", ":=", " ", "1"}]], "Input",
 CellChangeTimes->{{3.5814847712021976`*^9, 3.581484782824218*^9}, 
   3.5823749921125393`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"oval", "[", 
   RowBox[{
    RowBox[{"fc", "[", "#", "]"}], "&"}], "]"}], "[", 
  RowBox[{
   RowBox[{"-", "10"}], ",", "10", ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5823749936257424`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fc", "[", "t_", "]"}], " ", ":=", " ", 
  RowBox[{"Sqrt", "[", 
   RowBox[{
    RowBox[{"5", "*", 
     RowBox[{
      RowBox[{"Cos", "[", "t", "]"}], "^", "2"}]}], " ", "+", 
    RowBox[{
     RowBox[{"Sin", "[", "t", "]"}], "^", "2"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.581489383127496*^9, 3.5814894339724035`*^9}, {
   3.5814894680133505`*^9, 3.581489472788624*^9}, {3.581489571301258*^9, 
   3.5814895912934017`*^9}, {3.5814896683728104`*^9, 3.5814896765492783`*^9}, 
   3.581489721629857*^9, 3.582374995076545*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"oval", "[", "fc", "]"}], "[", "t", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5814895190012674`*^9, 3.581489526371689*^9}, {
   3.581490700517846*^9, 3.5814907065491905`*^9}, 3.5823773040522003`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"5", " ", 
     RowBox[{"Cos", "[", "t", "]"}]}], 
    SqrtBox[
     RowBox[{"3", "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "t"}], "]"}]}]}]]], ",", 
   FractionBox[
    RowBox[{"Sin", "[", "t", "]"}], 
    SqrtBox[
     RowBox[{"3", "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"2", " ", "t"}], "]"}]}]}]]]}], "}"}]], "Output",
 CellChangeTimes->{3.5814895279767804`*^9, 3.5814895981737957`*^9, 
  3.581489683519677*^9, 3.581490436365737*^9, 3.5814907086523113`*^9, 
  3.5823773064858046`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"oval", "[", 
   RowBox[{
    RowBox[{"fc", "[", "#", "]"}], "&"}], "]"}], "[", 
  RowBox[{
   RowBox[{"-", "10"}], ",", "10", ",", 
   RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5823749964337473`*^9}],

Cell[BoxData[
 RowBox[{"Clear", "[", "fc", "]"}]], "Input",
 CellChangeTimes->{{3.5814907776372566`*^9, 3.581490795996307*^9}, 
   3.582377325486638*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["planeframe", "Subsection",
 CellChangeTimes->{3.5818253442150116`*^9}],

Cell[BoxData[
 RowBox[{"?", "planeframe"}]], "Input",
 CellChangeTimes->{{3.5823774610508757`*^9, 3.582377466604486*^9}, 
   3.5823800192837696`*^9}],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
 CellChangeTimes->{{3.5818280820666203`*^9, 3.581828085732627*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ellframe", "[", "t_", "]"}], " ", ":=", 
  RowBox[{
   RowBox[{"planeframe", "[", 
    RowBox[{
     RowBox[{"ellipse", "[", 
      RowBox[{"4", ",", "2"}], "]"}], ",", ".4"}], "]"}], "[", "t", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5817748947586613`*^9, 3.581774905103253*^9}, {
   3.581775016408619*^9, 3.5817750188227572`*^9}, {3.5817756514879436`*^9, 
   3.5817756524069963`*^9}, {3.5817757974162903`*^9, 3.5817757983593445`*^9}, 
   3.581826132281596*^9, {3.581826308343505*^9, 3.581826318998324*^9}, 
   3.581826830382822*^9, 3.581827046162401*^9, {3.5818271796518354`*^9, 
   3.5818271823506403`*^9}, 3.5818273312841015`*^9, {3.5818279778428373`*^9, 
   3.5818279815712442`*^9}}],

Cell[BoxData[
 RowBox[{"ell", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"ellipse", "[", 
      RowBox[{"4", ",", "2"}], "]"}], "[", "t", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
    RowBox[{"PlotStyle", "->", 
     RowBox[{"{", 
      RowBox[{"Red", ",", "Thick"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5817751472561035`*^9, 3.581775281664791*^9}, 
   3.5817753197689705`*^9, {3.581827951525591*^9, 3.5818279646764145`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"ell", ",", 
    RowBox[{"Graphics", "[", 
     RowBox[{"ellframe", "[", "1", "]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{
  3.581775499999279*^9, {3.581826338794759*^9, 3.5818263546911864`*^9}}],

Cell[BoxData[
 RowBox[{"Animate", "[", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"ell", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{"ellframe", "[", "t", "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"PlotRange", " ", "\[Rule]", " ", "All"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5818263692148123`*^9, 3.5818264315837216`*^9}, {
  3.5818265629203525`*^9, 3.5818265704707656`*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["twist", "Subsection",
 CellChangeTimes->{{3.582251032198389*^9, 3.5822510346163936`*^9}}],

Cell[BoxData[
 RowBox[{"?", "twist"}]], "Input",
 CellChangeTimes->{{3.582251018283165*^9, 3.582251020763569*^9}, 
   3.5823776565348196`*^9, 3.58238001929937*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lg", "[", "v_", "]"}], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{"v", ",", "0"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5822484365254307`*^9, 3.5822484658066816`*^9}, 
   3.5822487985552664`*^9, {3.582249206074782*^9, 3.5822492274468193`*^9}, {
   3.582249302904152*^9, 3.582249331935803*^9}, 3.5822500922499385`*^9, {
   3.582250696048999*^9, 3.5822507160014343`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"tw", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "=", 
  RowBox[{
   RowBox[{"twist", "[", 
    RowBox[{"2", ",", "1", ",", "lg"}], "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5822484752758985`*^9, 3.582248549204428*^9}, {
   3.5822487326607504`*^9, 3.5822487449067717`*^9}, 3.5822500759011097`*^9, 
   3.5822508648412957`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "u", "]"}], " ", 
    RowBox[{"(", 
     RowBox[{"2", "+", 
      RowBox[{"v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", "+", 
      RowBox[{"v", " ", 
       RowBox[{"Cos", "[", "u", "]"}]}]}], ")"}], " ", 
    RowBox[{"Sin", "[", "u", "]"}]}], ",", 
   RowBox[{"v", " ", 
    RowBox[{"Sin", "[", "u", "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.582248536412406*^9, 3.58224855028083*^9}, 
   3.582248752379185*^9, 3.5822488044208765`*^9, 3.5822492337492304`*^9, {
   3.5822493091909633`*^9, 3.5822493381758137`*^9}, 3.5822500990983505`*^9, 
   3.582250724612649*^9, 3.5822508676337004`*^9, 3.5823776840376678`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"tw", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "\[Rule]", " ", 
    RowBox[{"Opacity", "[", ".6", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582248597268113*^9, 3.582248660885024*^9}, 
   3.582248723487934*^9, {3.5822487621760025`*^9, 3.582248768447213*^9}, {
   3.5822492464320526`*^9, 3.5822492491932573`*^9}, {3.5822505985488276`*^9, 
   3.5822506475017138`*^9}, {3.582250734362666*^9, 3.5822507600091114`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Weierstrass Representation", "Subsection",
 CellChangeTimes->{3.5822664980174217`*^9, 3.582270073484548*^9}],

Cell[BoxData[
 RowBox[{"?", "wei"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.5823777948602624`*^9}, 
   3.5823800193149695`*^9}],

Cell[BoxData[
 RowBox[{"?", "weicurvature"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.582377820756308*^9}, 
   3.5823800193149695`*^9}],

Cell[BoxData[
 RowBox[{"?", "weidata"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.582377851987563*^9}, 
   3.5823800193305693`*^9}],

Cell[BoxData[
 RowBox[{"?", "weimetric"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.5823778719243975`*^9}, 
   3.5823800193461695`*^9}],

Cell[BoxData[
 RowBox[{"?", "weinormal"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.5823778932184353`*^9}, 
   3.582378063851535*^9, 3.5823800193461695`*^9}],

Cell[BoxData[
 RowBox[{"?", "weipolar"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.5823778932184353`*^9}, {
   3.5823779235448885`*^9, 3.5823779308925014`*^9}, 3.5823800193617697`*^9}],

Cell[BoxData[
 RowBox[{"?", "weiprime"}]], "Input",
 CellChangeTimes->{{3.5823777925514584`*^9, 3.5823778932184353`*^9}, {
   3.5823779235448885`*^9, 3.582377951094537*^9}, 3.58238001937737*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "/", "#"}], "&"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "#1"], "&"}]], "Output",
 CellChangeTimes->{3.582264041293906*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"1", "/", "#"}], "&"}], "[", "2", "]"}]], "Input",
 CellChangeTimes->{{3.5822640476119175`*^9, 3.582264053056327*^9}}],

Cell[BoxData[
 FractionBox["1", "2"]], "Output",
 CellChangeTimes->{3.5822640560671325`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"wei", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], ",", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.582263853126376*^9, 3.5822638949968495`*^9}, {
  3.5822640187050667`*^9, 3.5822640264582806`*^9}, {3.5822640691555552`*^9, 
  3.582264110308428*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox["z", "4"]], "+", 
      RowBox[{"Log", "[", "z", "]"}]}], ")"}]}], ",", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        SuperscriptBox["z", "4"]]}], "+", 
      RowBox[{"Log", "[", "z", "]"}]}], ")"}]}], ",", 
   RowBox[{"-", 
    FractionBox["1", 
     SuperscriptBox["z", "2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5822641123364315`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"weimetric", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], ",", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.582263853126376*^9, 3.5822638949968495`*^9}, {
  3.5822640187050667`*^9, 3.5822640264582806`*^9}, {3.5822640691555552`*^9, 
  3.5822641413836823`*^9}}],

Cell[BoxData[
 FractionBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}], ")"}], "2"], 
  RowBox[{"4", " ", 
   SuperscriptBox[
    RowBox[{"Abs", "[", "z", "]"}], "2"]}]]], "Output",
 CellChangeTimes->{3.582264144191687*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"weicurvature", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], ",", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.582263853126376*^9, 3.5822638949968495`*^9}, {
  3.5822640187050667`*^9, 3.5822640264582806`*^9}, {3.5822640691555552`*^9, 
  3.5822641413836823`*^9}, {3.5822642354206476`*^9, 3.582264249195472*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["256", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       FractionBox["4", 
        SuperscriptBox[
         RowBox[{"Abs", "[", "z", "]"}], "4"]]}], ")"}], "4"], " ", 
    SuperscriptBox[
     RowBox[{"Abs", "[", "z", "]"}], "4"]}]]}]], "Output",
 CellChangeTimes->{3.5822642505838737`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ffz", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "=", 
  RowBox[{"ctor", "[", 
   RowBox[{
    RowBox[{"wei", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"1", "/", "#"}], "&"}], ",", 
      RowBox[{
       RowBox[{"2", "/", 
        RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", 
    RowBox[{"u", "+", 
     RowBox[{"I", "*", "v"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5822646601937933`*^9, 3.5822646621281967`*^9}, {
   3.582264695980256*^9, 3.582264701377866*^9}, {3.5822647996112385`*^9, 
   3.582264805102448*^9}, {3.5822648889993954`*^9, 3.58226490294582*^9}, 
   3.582264952023506*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      FractionBox[
       SuperscriptBox["u", "4"], 
       RowBox[{"2", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]}]], "-", 
      FractionBox[
       RowBox[{"3", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "+", 
      FractionBox[
       SuperscriptBox["v", "4"], 
       RowBox[{"2", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]}]], "+", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox["u", "3"], " ", "v"}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "+", 
      FractionBox[
       RowBox[{"2", " ", "u", " ", 
        SuperscriptBox["v", "3"]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"Arg", "[", 
        RowBox[{"u", "+", 
         RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox["u", "2"], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "2"]]}], "+", 
      FractionBox[
       SuperscriptBox["v", "2"], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "2"]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         SuperscriptBox["u", "3"], " ", "v"}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "+", 
      FractionBox[
       RowBox[{"2", " ", "u", " ", 
        SuperscriptBox["v", "3"]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"Arg", "[", 
        RowBox[{"u", "+", 
         RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox["u", "4"], 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["u", "2"], "+", 
            SuperscriptBox["v", "2"]}], ")"}], "4"]}]]}], "+", 
      FractionBox[
       RowBox[{"3", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "-", 
      FractionBox[
       SuperscriptBox["v", "4"], 
       RowBox[{"2", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]}]], "+", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"Log", "[", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], "]"}]}]}], ",", 
     FractionBox[
      RowBox[{"2", " ", "u", " ", "v"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "2"]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5822647041234703`*^9, 3.5822648095016556`*^9, 
  3.5822649041470222`*^9, 3.5822649530531077`*^9, 3.582268420712015*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ffz1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"ffz", "[", 
     RowBox[{"u", ",", "v"}], "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582264915566242*^9, 3.582264981086357*^9}, 
   3.5822681947120886`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "4"], "-", 
          RowBox[{"6", " ", 
           SuperscriptBox["u", "2"], " ", 
           SuperscriptBox["v", "2"]}], "+", 
          SuperscriptBox["v", "4"]}], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "+", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], "]"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{"u", "-", "v"}], ")"}], " ", "v", " ", 
       RowBox[{"(", 
        RowBox[{"u", "+", "v"}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"Arg", "[", 
      RowBox[{"u", "+", 
       RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["u", "2"]}], "+", 
     SuperscriptBox["v", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"]}], ")"}], "2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.5822649841283627`*^9, 3.5822650785865283`*^9, 
  3.5822684310696077`*^9}]
}, Closed]],

Cell["\<\
Arg should be replaced by its real expression\
\>", "Text",
 CellChangeTimes->{{3.582266798484229*^9, 3.5822668242837048`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Arg", "[", 
     RowBox[{"u", "+", 
      RowBox[{"I", "*", "v"}]}], "]"}], " ", "-", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"v", "/", "u"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582266833420227*^9, 3.5822669041432724`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"ArcTan", "[", 
    FractionBox["v", "u"], "]"}]}], "+", 
  RowBox[{"Arg", "[", 
   RowBox[{"u", "+", 
    RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.58226690621039*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"%", " ", "+", " ", "1"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582266934344*^9, 3.5822669927743416`*^9}}],

Cell["Since", "Text",
 CellChangeTimes->{{3.582267237107317*^9, 3.5822672400114827`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Arg"}]], "Input",
 CellChangeTimes->{{3.582267128552108*^9, 3.5822671314842753`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\\!\\(\\*RowBox[{\\\"Arg\\\", \\\"[\\\", StyleBox[\\\"z\\\", \
\\\"TI\\\"], \\\"]\\\"}]\\) gives the argument of the complex number \
\\!\\(\\*StyleBox[\\\"z\\\", \\\"TI\\\"]\\). \"\>", "MSG"], 
  "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Arg"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.5822671338154087`*^9},
 CellTags->"Info3582274333-2934694"]
}, Closed]],

Cell[TextData[{
 "Arg[z] is left unevaluated if ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 " is not a numeric quantity. \nThus Arg\.b4 is not defined:"
}], "Text",
 CellChangeTimes->{{3.582267185295353*^9, 3.582267292635493*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Arg", "[", 
    RowBox[{"u", "+", 
     RowBox[{"I", "*", "v"}]}], "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.582267031165538*^9, 3.582267045653366*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["Arg", "\[Prime]",
   MultilineFunction->None], "[", 
  RowBox[{"u", "+", 
   RowBox[{"\[ImaginaryI]", " ", "v"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5822670503316336`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"ArcTan", "[", 
    RowBox[{"v", "/", "u"}], "]"}], ",", "u"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822670637334003`*^9, 3.582267090413926*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["v", 
   RowBox[{
    SuperscriptBox["u", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      FractionBox[
       SuperscriptBox["v", "2"], 
       SuperscriptBox["u", "2"]]}], ")"}]}]]}]], "Output",
 CellChangeTimes->{3.5822670923000345`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ffz1", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], "=", 
  RowBox[{
   RowBox[{"ffz1", "[", 
    RowBox[{"u", ",", "v"}], "]"}], "/.", 
   RowBox[{
    RowBox[{"Arg", "[", 
     RowBox[{"u", "+", 
      RowBox[{"I", "*", "v"}]}], "]"}], "\[Rule]", " ", 
    RowBox[{"ArcTan", "[", 
     RowBox[{"v", "/", "u"}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.58226604723903*^9, 3.5822660997175217`*^9}, 
   3.5822682067187753`*^9, 3.582268251783353*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "4"], "-", 
          RowBox[{"6", " ", 
           SuperscriptBox["u", "2"], " ", 
           SuperscriptBox["v", "2"]}], "+", 
          SuperscriptBox["v", "4"]}], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["u", "2"], "+", 
          SuperscriptBox["v", "2"]}], ")"}], "4"]], "+", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], "]"}]}], ")"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{"u", "-", "v"}], ")"}], " ", "v", " ", 
       RowBox[{"(", 
        RowBox[{"u", "+", "v"}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"ArcTan", "[", 
      FractionBox["v", "u"], "]"}]}]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["u", "2"]}], "+", 
     SuperscriptBox["v", "2"]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"]}], ")"}], "2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.582266103180728*^9, 3.5822684536258974`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"ffz1", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5822650871665435`*^9, 3.5822651663990827`*^9}, 
   3.58226622398734*^9, {3.5822663136874976`*^9, 3.5822663346071343`*^9}, {
   3.5822663750268054`*^9, 3.5822664203760853`*^9}, 3.5822683229434233`*^9}],

Cell["\<\
The real part and the imaginary part define minimal surfaces\
\>", "Text",
 CellChangeTimes->{{3.5822673478526506`*^9, 3.5822673717640185`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"meancurvature", "[", "ffz1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.582265224228384*^9, 3.582265255568839*^9}, {
   3.582265826342642*^9, 3.582265860756302*^9}, {3.5822661150835485`*^9, 
   3.5822661272203703`*^9}, 3.5822683269746532`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5822652678772607`*^9, 3.5822658934539595`*^9, 
  3.5822661291079736`*^9, 3.5822684768532257`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gaussiancurvature", "[", "ffz1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822652979073133`*^9, 3.582265329372569*^9}, {
   3.582266256419797*^9, 3.58226628659025*^9}, 3.582268332614976*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"256", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"]}], ")"}], "6"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"4", "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "2"]}], ")"}], "4"]]}]], "Output",
 CellChangeTimes->{3.582266290615057*^9, 3.5822684810714674`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ffz2", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"ffz", "[", 
       RowBox[{"u", ",", "v"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "/.", 
     RowBox[{
      RowBox[{"Arg", "[", 
       RowBox[{"u", "+", 
        RowBox[{"I", "*", "v"}]}], "]"}], "\[Rule]", " ", 
      RowBox[{"ArcTan", "[", 
       RowBox[{"v", "/", "u"}], "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u", " ", "\[Element]", " ", "Reals"}], ",", " ", 
      RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582264915566242*^9, 3.582264981086357*^9}, {
   3.5822674321834745`*^9, 3.58226749347398*^9}, 3.582267524784771*^9, 
   3.582268263008995*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", "u", " ", 
       RowBox[{"(", 
        RowBox[{"u", "-", "v"}], ")"}], " ", "v", " ", 
       RowBox[{"(", 
        RowBox[{"u", "+", "v"}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"ArcTan", "[", 
      FractionBox["v", "u"], "]"}]}]}], ",", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "4"], "-", 
           RowBox[{"6", " ", 
            SuperscriptBox["u", "2"], " ", 
            SuperscriptBox["v", "2"]}], "+", 
           SuperscriptBox["v", "4"]}], ")"}]}], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["u", "2"], "+", 
           SuperscriptBox["v", "2"]}], ")"}], "4"]]}], "+", 
      RowBox[{"Log", "[", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], "]"}]}], ")"}]}], ",", 
   FractionBox[
    RowBox[{"2", " ", "u", " ", "v"}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"]}], ")"}], "2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.5822675266758795`*^9, 3.5822684859747477`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"ParametricPlot3D", "[", 
  RowBox[{
   RowBox[{"ffz2", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.582267535730397*^9, 3.5822675962948613`*^9}, 
   3.582268277006796*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"meancurvature", "[", "ffz2", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.582265224228384*^9, 3.582265255568839*^9}, {
   3.582265826342642*^9, 3.582265860756302*^9}, {3.5822661150835485`*^9, 
   3.5822661272203703`*^9}, {3.582267645967702*^9, 3.5822676469737597`*^9}, 
   3.582268281663062*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.5822652678772607`*^9, 3.5822658934539595`*^9, 
  3.5822661291079736`*^9, 3.582267649518905*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"gaussiancurvature", "[", "ffz2", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822652979073133`*^9, 3.582265329372569*^9}, {
   3.582266256419797*^9, 3.58226628659025*^9}, {3.5822676565513077`*^9, 
   3.582267658045393*^9}, 3.5822682864073334`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"256", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["u", "2"], "+", 
       SuperscriptBox["v", "2"]}], ")"}], "6"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"4", "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["u", "2"], "+", 
         SuperscriptBox["v", "2"]}], ")"}], "2"]}], ")"}], "4"]]}]], "Output",
 CellChangeTimes->{3.582267660670543*^9, 3.5822685032267346`*^9}]
}, Closed]],

Cell["\<\
weicurvature  coincides with the Gaussian curvature of these minimal surfaces:\
\>", "Text",
 CellChangeTimes->{{3.5822679219884896`*^9, 3.582267954828368*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"weicurvature", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "/", "#"}], "&"}], ",", 
     RowBox[{
      RowBox[{"2", "/", 
       RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]}]], "Input",
 CellChangeTimes->{3.5822678275520883`*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox["256", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       FractionBox["4", 
        SuperscriptBox[
         RowBox[{"Abs", "[", "z", "]"}], "4"]]}], ")"}], "4"], " ", 
    SuperscriptBox[
     RowBox[{"Abs", "[", "z", "]"}], "4"]}]]}]], "Output",
 CellChangeTimes->{3.582267828949168*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5822678618940525`*^9, 3.5822678707975616`*^9}}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"256", " ", 
    SuperscriptBox[
     RowBox[{"Abs", "[", "z", "]"}], "12"]}], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"4", "+", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]}], ")"}], "4"]]}]], "Output",
 CellChangeTimes->{3.5822678731576967`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "weimetric"}]], "Input",
 CellChangeTimes->{{3.582268043629447*^9, 3.5822680476006746`*^9}}],

Cell[BoxData[
 StyleBox["\<\"z->weimetric[f,g][z] is the scaling function of the metric of \
the Weierstrass patch determined by meromorphic functions f and g.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.582268050528842*^9},
 CellTags->"Info3582275250-3508047"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"weimetric", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], ",", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{3.5822680272625113`*^9}],

Cell[BoxData[
 FractionBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}], ")"}], "2"], 
  RowBox[{"4", " ", 
   SuperscriptBox[
    RowBox[{"Abs", "[", "z", "]"}], "2"]}]]], "Output",
 CellChangeTimes->{3.5822680294096336`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ds2", "[", "ffz1", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"du", ",", "dv"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822685173875446`*^9, 3.582268549966408*^9}, {
  3.5822686751905704`*^9, 3.5822687015980806`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"4", "+", 
       SuperscriptBox["u", "4"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], "+", 
       SuperscriptBox["v", "4"]}], ")"}], "2"], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], ")"}], "5"]}]], "**", 
   SuperscriptBox["du", "2"]}], "+", 
  RowBox[{
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"4", "+", 
       SuperscriptBox["u", "4"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], "+", 
       SuperscriptBox["v", "4"]}], ")"}], "2"], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], ")"}], "5"]}]], "**", 
   SuperscriptBox["dv", "2"]}]}]], "Output",
 CellChangeTimes->{
  3.5822685526535616`*^9, {3.5822686786167665`*^9, 3.582268704101224*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        FractionBox["4", 
         SuperscriptBox[
          RowBox[{"Abs", "[", "z", "]"}], "4"]]}], ")"}], "2"], 
     RowBox[{"4", " ", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "2"]}]], "/.", 
    RowBox[{"z", "\[Rule]", " ", 
     RowBox[{"u", "+", 
      RowBox[{"I", "*", "v"}]}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"u", " ", "\[Element]", " ", "Reals"}], ",", " ", 
     RowBox[{"v", "\[Element]", " ", "Reals"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5822687430824537`*^9, 3.5822688054660215`*^9}}],

Cell[BoxData[
 FractionBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"4", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], ")"}], "2"]}], ")"}], "2"], 
  RowBox[{"4", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["u", "2"], "+", 
      SuperscriptBox["v", "2"]}], ")"}], "5"]}]]], "Output",
 CellChangeTimes->{3.58226880718712*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ds2", "[", "ffz2", "]"}], "[", 
   RowBox[{"u", ",", "v"}], "]"}], "[", 
  RowBox[{"du", ",", "dv"}], "]"}]], "Input",
 CellChangeTimes->{{3.5822685173875446`*^9, 3.582268549966408*^9}, {
   3.5822686751905704`*^9, 3.5822687015980806`*^9}, 3.582268866310502*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"4", "+", 
       SuperscriptBox["u", "4"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], "+", 
       SuperscriptBox["v", "4"]}], ")"}], "2"], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], ")"}], "5"]}]], "**", 
   SuperscriptBox["du", "2"]}], "+", 
  RowBox[{
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"4", "+", 
       SuperscriptBox["u", "4"], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["u", "2"], " ", 
        SuperscriptBox["v", "2"]}], "+", 
       SuperscriptBox["v", "4"]}], ")"}], "2"], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["u", "2"], "+", 
        SuperscriptBox["v", "2"]}], ")"}], "5"]}]], "**", 
   SuperscriptBox["dv", "2"]}]}]], "Output",
 CellChangeTimes->{3.5822688697847004`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "weiprime"}]], "Input",
 CellChangeTimes->{{3.5822690038473682`*^9, 3.5822690091786737`*^9}}],

Cell[BoxData[
 StyleBox["\<\"z->weiprime[f,g][z] is integrand of the Weierstrass \
representation.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.58226901086777*^9},
 CellTags->"Info3582276210-7037343"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"weiprime", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], ",", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]], "Input",
 CellChangeTimes->{{3.5822689574657154`*^9, 3.5822689781989017`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"1", "-", 
     FractionBox["4", 
      SuperscriptBox["z", "4"]]}], 
    RowBox[{"2", " ", "z"}]], ",", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       FractionBox["4", 
        SuperscriptBox["z", "4"]]}], ")"}]}], 
    RowBox[{"2", " ", "z"}]], ",", 
   FractionBox["2", 
    SuperscriptBox["z", "3"]]}], "}"}]], "Output",
 CellChangeTimes->{3.582268979500976*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "weipolar"}]], "Input",
 CellChangeTimes->{{3.582269195457328*^9, 3.582269196317377*^9}}],

Cell[BoxData[
 StyleBox["\<\"z->weipolar[f,g][t][r,theta] is the polar parametrization of \
the Weierstrass patch corresponding to the minimal curve \
z->E^(-I*t)*wei[f,g][z] determined by meromorphic functions f and g.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.582269197311434*^9},
 CellTags->"Info3582276397-7037343"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{
    RowBox[{"weipolar", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"1", "/", "#"}], "&"}], ",", 
      RowBox[{
       RowBox[{"2", "/", 
        RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "t", "]"}], "[", 
   RowBox[{"r", ",", "theta"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.582269135150879*^9, 3.5822691591262503`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"t", "+", 
          RowBox[{"4", " ", "theta"}]}], "]"}]}], 
       SuperscriptBox["r", "4"]], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Cos", "[", "t", "]"}], " ", 
       RowBox[{"Log", "[", "r", "]"}]}], "+", 
      RowBox[{"2", " ", "theta", " ", 
       RowBox[{"Sin", "[", "t", "]"}]}]}], ")"}]}], ",", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "theta", " ", 
       RowBox[{"Cos", "[", "t", "]"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"Log", "[", "r", "]"}], " ", 
       RowBox[{"Sin", "[", "t", "]"}]}], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"Sin", "[", 
         RowBox[{"t", "+", 
          RowBox[{"4", " ", "theta"}]}], "]"}]}], 
       SuperscriptBox["r", "4"]]}], ")"}]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Cos", "[", 
      RowBox[{"t", "+", 
       RowBox[{"2", " ", "theta"}]}], "]"}], 
     SuperscriptBox["r", "2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5822691605193295`*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{"?", "weinormal"}]], "Input",
 CellChangeTimes->{{3.582269790022335*^9, 3.58226979430158*^9}, 
   3.582378148809284*^9, 3.5823800194241695`*^9}],

Cell["\<\
??weinormal depends on one function only!\
\>", "Text",
 CellChangeTimes->{{3.5822699828763657`*^9, 3.582270014140154*^9}, 
   3.5823800194397697`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"weinormal", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"1", "/", "#"}], "&"}], ",", 
     RowBox[{
      RowBox[{"2", "/", 
       RowBox[{"#", "^", "2"}]}], "&"}]}], "]"}], "[", "z", "]"}]}]], "Input",
 CellChangeTimes->{{3.5822696868224325`*^9, 3.5822696908136606`*^9}, {
  3.5822699311584077`*^9, 3.582269932831503*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"weinormal", "[", 
   RowBox[{
    RowBox[{
     FractionBox["1", "#1"], "&"}], ",", 
    RowBox[{
     FractionBox["2", 
      SuperscriptBox["#1", "2"]], "&"}]}], "]"}], "[", "z", "]"}]], "Output",
 CellChangeTimes->{3.5822696916807103`*^9, 3.582269935077632*^9, 
  3.582378164112911*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"weinormal", "[", 
    RowBox[{
     RowBox[{"1", "/", "#"}], "&"}], "]"}], "[", "z", "]"}]}]], "Input",
 CellChangeTimes->{{3.5822696868224325`*^9, 3.5822696908136606`*^9}, {
  3.5822699311584077`*^9, 3.5822699567428713`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"Re", "[", 
      FractionBox["1", "z"], "]"}]}], 
    RowBox[{"1", "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "2"]]}]], ",", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"Im", "[", 
      FractionBox["1", "z"], "]"}]}], 
    RowBox[{"1", "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "2"]]}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "2"]]}], 
    RowBox[{"1", "+", 
     FractionBox["1", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "2"]]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.582269959504029*^9, 3.5823781684653187`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"weinormal", "[", 
    RowBox[{
     RowBox[{"2", "/", 
      RowBox[{"#", "^", "2"}]}], "&"}], "]"}], "[", "z", "]"}]}]], "Input",
 CellChangeTimes->{{3.5822696868224325`*^9, 3.5822696908136606`*^9}, {
  3.582269733356094*^9, 3.5822697388544083`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"Re", "[", 
      FractionBox["1", 
       SuperscriptBox["z", "2"]], "]"}]}], 
    RowBox[{"1", "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}]], ",", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"Im", "[", 
      FractionBox["1", 
       SuperscriptBox["z", "2"]], "]"}]}], 
    RowBox[{"1", "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}], 
    RowBox[{"1", "+", 
     FractionBox["4", 
      SuperscriptBox[
       RowBox[{"Abs", "[", "z", "]"}], "4"]]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.582269740038476*^9}]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["CURVES", "Section",
 CellChangeTimes->{{3.580269489347209*^9, 3.580269492046014*^9}}],

Cell["\<\
The package CURVES.m is a library of parameter representations of curves in \
the Euclidean plane or space combined with Plot-commands composed by Alfred \
Gray.  I tested many of the functions; all tests  gave reasonable results.\
\>", "Text",
 CellChangeTimes->{{3.5802692285303507`*^9, 3.5802692346143613`*^9}, {
   3.5802692752212324`*^9, 3.580269438272719*^9}, {3.582436290769916*^9, 
   3.58243629874153*^9}, {3.582609087960979*^9, 3.582609163561303*^9}, {
   3.582609230385125*^9, 3.5826092601768293`*^9}, {3.5826093021292286`*^9, 
   3.582609323417446*^9}, 3.582609591796797*^9, 3.5826096837690573`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<CURVES`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5805255402874002`*^9, 3.5805255938110943`*^9}, 
   3.5805257143213053`*^9, 3.5823463367935753`*^9, 3.582346456632986*^9, 
   3.582440420692127*^9, {3.582446092615947*^9, 3.5824460968791904`*^9}, 
   3.582447045383442*^9, {3.582552418584136*^9, 3.582552422390543*^9}, 
   3.582552599731654*^9, {3.582604301106187*^9, 3.58260430798658*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805256052303143`*^9, 3.5805256149803314`*^9}}],

Cell[BoxData["214"], "Output",
 CellChangeTimes->{3.582446158594721*^9, 3.5824470524288454`*^9, 
  3.58255244324778*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "agnesi", "]"}]], "Input",
 CellChangeTimes->{{3.5824472410666347`*^9, 3.5824472766236687`*^9}}],

Cell[BoxData[
 RowBox[{"{", "Protected", "}"}]], "Output",
 CellChangeTimes->{3.582552088393956*^9, 3.5826014408966894`*^9, 
  3.582601761648853*^9, 3.5826043282707405`*^9, 3.582604533337469*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CURVES`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801229215216017`*^9, 3.58012293200482*^9}}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["CURVES`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["CURVES`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["agnesi",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"agnesi", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"nephroidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["agnesiimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"agnesiimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nephroidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["airyspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"airyspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["deltoidunitspeed",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"nephroidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"alysoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["devilimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"devilimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["newtonphillipsimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"newtonphillipsimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidkappa2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"alysoidkappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["diamond",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"diamond", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ngon",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ngon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"alysoidnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["duererimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"duererimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nielsenspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nielsenspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["alysoidprime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"alysoidprime", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["eight",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eight", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["oneparametersubgroup",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"oneparametersubgroup", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["archimedesspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"archimedesspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["eightknot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eightknot", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pacman",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pacman", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["archimedesspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"archimedesspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["elasticainflect",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"elasticainflect", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"parabola", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["ast3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ast3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["elasticanoninflect",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"elasticanoninflect", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"parabolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["astroid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"astroid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipse",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipse", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["parabolicspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"parabolicspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["astroidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"astroidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipsebis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipsebis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["perseusimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"perseusimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["baseballseam",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"baseballseam", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipseimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipseimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["piriform",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"piriform", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bellcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bellcurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["ellipseinc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipseinc", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["piriformimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"piriformimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bernstein",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bernstein", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epicycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"epicycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["predatorpreyimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"predatorpreyimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["besselcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"besselcurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epicycloidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"epicycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenary",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudocatenary", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["besseln",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"besseln", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epispiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"epispiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenarykappa2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudocatenarykappa2", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["beziercurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"beziercurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epispiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"epispiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenarynds",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudocatenarynds", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bicorn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bicorn", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epitrochoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"epitrochoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudocatenaryprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudocatenaryprime", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bicylinder",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bicylinder", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["epitrochoid3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"epitrochoid3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudospheregeodesic",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudospheregeodesic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["biquadratic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"biquadratic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["fermatspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"fermatspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["pseudosphericalloxodrome",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudosphericalloxodrome", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bow",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bow", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["fermatspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"fermatspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rectangle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rectangle", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bowpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bowpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["folium",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"folium", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["regularpolygon",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"regularpolygon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bowtie",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bowtie", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["foliumimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"foliumimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["reuleauxpolygon",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"reuleauxpolygon", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bulletnose",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bulletnose", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["foliumpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"foliumpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rose",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rose", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["bulletnoseimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"bulletnoseimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["genhelix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"genhelix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["rosepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rosepolar", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cardioid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["genparabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"genparabola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sc", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cardioidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hankelspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hankelspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scarab",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scarab", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cardioidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["helix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"helix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cardioidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cardioidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["helixunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"helixunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["scunitspeed",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cartesianimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cartesianimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hippopede",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hippopede", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["seiffertspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"seiffertspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cartesianoval",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cartesianoval", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hippopedeimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hippopedeimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["semicubic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"semicubic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassini",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cassini", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["horopter",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"horopter", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["semicubicimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"semicubicimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassiniimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cassiniimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hyperbola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["serpentine",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"serpentine", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cassinipolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cassinipolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolabis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hyperbolabis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["serpentineimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"serpentineimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["catenary",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catenary", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperbolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinhspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sinhspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["catenaryunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"catenaryunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolicspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperbolicspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sinn", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cayleysextic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cayleysextic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hyperbolicspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperbolicspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sinoval",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sinoval", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"circle", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hypocycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalcardioid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalcardioid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloidinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hypocycloidinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalellipse",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalellipse", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleinvolute",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleinvolute", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypocycloidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hypocycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalhelix",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalhelix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleinvoluteunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleinvoluteunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["hypotrochoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hypotrochoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalloxodrome",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalloxodrome", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["circleunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kampyle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kampyle", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalloxodromeunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {
            "sphericalloxodromeunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cissoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cissoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kampylepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kampylepolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalnephroid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalnephroid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cissoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cissoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kappacurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kappacurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["sphericalspiral",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sphericalspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clelia",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clelia", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kappacurveimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"kappacurveimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["spring",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"spring", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clothoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"keplerimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["strophoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"strophoid", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clothoidnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerorbit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"keplerorbit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["strophoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"strophoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["clothoidprime",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clothoidprime", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["keplerorbitpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"keplerorbitpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["talbot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"talbot", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cnccoprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnccoprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["kochsnowflake",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kochsnowflake", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tanhspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tanhspiral", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cnchyprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnchyprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lagrangecurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lagrangecurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["teardrop",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"teardrop", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cochleoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cochleoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehr",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lehr", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["teeth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"teeth", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cochleoidpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cochleoidpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehrkappa2",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lehrkappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tennisballseam",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"tennisballseam", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conchoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"conchoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lehrnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lehrnds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tooth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tooth", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conchoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"conchoidimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscate",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lemniscate", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["torusknot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"torusknot", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["conicalhelix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"conicalhelix", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscatebis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lemniscatebis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tractrix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["coshspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"coshspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscateimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"lemniscateimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixminus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tractrixminus", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cosn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cosn", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscatepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"lemniscatepolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixplus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tractrixplus", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cothspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cothspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lemniscateunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"lemniscateunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tractrixunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"tractrixunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cpcprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cpcprofile", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limacon",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"limacon", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["triangle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"triangle", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["crosscurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"crosscurve", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limaconimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"limaconimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["trident",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"trident", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["crosscurveimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"crosscurveimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["limaconpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"limaconpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tridentimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"tridentimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicalellipse",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cubicalellipse", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["line",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"line", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["trisectrix",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"trisectrix", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicparabola",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cubicparabola", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["line3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"line3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["tschirnhausen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tschirnhausen", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cubicparabolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cubicparabolaimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lissajous",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lissajous", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["twicubic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"twicubic", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["curves",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"curves", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lissajous3d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lissajous3d", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["twistedn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"twistedn", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cycloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cycloid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lituus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lituus", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["veryflat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"veryflat", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["cycloidunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cycloidunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["lituuspolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lituuspolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["viviani",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"viviani", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunay",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"delaunay", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logistic",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"logistic", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["watt",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"watt", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunaykappa2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"delaunaykappa2", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"logspiral", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["wattimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wattimplicit", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["delaunaynds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"delaunaynds", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiralpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"logspiralpolar", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["wigglyellipse",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wigglyellipse", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["deltoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"deltoid", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["logspiralunitspeed",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"logspiralunitspeed", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["zcprofile",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"zcprofile", "CURVES`"}},
          ButtonNote->"CURVES`"]},
        {
         ButtonBox["deltoidbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"deltoidbis", "CURVES`"}},
          ButtonNote->"CURVES`"], 
         ButtonBox["nephroid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nephroid", "CURVES`"}},
          ButtonNote->"CURVES`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.31666666666666665`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.5829570406734805`*^9}]
}, Open  ]],

Cell["Click the name to get the usage!", "Text",
 CellChangeTimes->{{3.5802692285303507`*^9, 3.5802692346143613`*^9}, {
   3.5802692752212324`*^9, 3.580269438272719*^9}, {3.582436290769916*^9, 
   3.58243629874153*^9}, {3.582609087960979*^9, 3.582609163561303*^9}, {
   3.582609230385125*^9, 3.5826092601768293`*^9}, {3.5826093021292286`*^9, 
   3.582609323417446*^9}, {3.582609591796797*^9, 3.582609597825142*^9}, 
   3.582868911493106*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["SURFS", "Section",
 CellChangeTimes->{{3.580269459285956*^9, 3.580269461235959*^9}}],

Cell["\<\
The package SURFS.m is a library of parameter representations of surfaces \
combined with Plot-commands composed by Alfred Gray.  I tested many of the \
functions; all tests  gave reasonable, sometimes complex results.\
\>", "Text",
 CellChangeTimes->{{3.5802692285303507`*^9, 3.5802692346143613`*^9}, {
   3.5802692752212324`*^9, 3.580269438272719*^9}, {3.582436290769916*^9, 
   3.58243629874153*^9}, {3.582609087960979*^9, 3.582609163561303*^9}, 
   3.582609764604681*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<SURFS`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5805255402874002`*^9, 3.5805255938110943`*^9}, 
   3.5805257143213053`*^9, 3.5823463367935753`*^9, 3.582346456632986*^9, 
   3.582440420692127*^9, {3.582441781043935*^9, 3.582441782428014*^9}, 
   3.5824470679757347`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5805256052303143`*^9, 3.5805256149803314`*^9}}],

Cell[BoxData["199"], "Output",
 CellChangeTimes->{3.5824418438075247`*^9, 3.5824470842526655`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "apple", "]"}]], "Input",
 CellChangeTimes->{{3.5824476352051783`*^9, 3.5824476524631653`*^9}}],

Cell[BoxData[
 RowBox[{"{", "Protected", "}"}]], "Output",
 CellChangeTimes->{3.5824476554093337`*^9, 3.5824481780842285`*^9, 
  3.5825511557812233`*^9, 3.582601484498766*^9, 3.5826045082300334`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SURFS`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801229779157004`*^9, 3.580122983937311*^9}, 
   3.5823798417398577`*^9, 3.582379915465587*^9}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["SURFS`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["SURFS`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["agnesirev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"agnesirev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom1implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom1implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboliccoor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"paraboliccoor", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["apple",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"apple", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom2implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom2implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboliccyclide",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"paraboliccyclide", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["astell",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"astell", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom3implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom3implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"paraboloid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["astroidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"astroidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom4implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom4implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["paraboloidpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"paraboloidpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bohdom",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bohdom", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom5implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom5implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pear",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pear", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bour",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bour", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["equihom6implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"equihom6implicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pearbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pearbis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bourmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bourmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["exptwist",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"exptwist", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["perturbedms",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"perturbedms", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["bourpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"bourpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["exptwistasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"exptwistasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["perturbedmspolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"perturbedmspolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["boy",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"boy", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["flattorus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"flattorus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pillow",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pillow", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cardioidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cardioidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["funnel",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"funnel", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["plane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"plane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalan",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catalan", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["funnelasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"funnelasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["planepolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"planepolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalandef",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catalandef", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["gencube",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"gencube", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["planetocat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"planetocat", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catalanmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"catalanmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["genhypparab",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"genhypparab", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["plucker",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"plucker", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catenoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["genoctahedron",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"genoctahedron", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pluckerpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pluckerpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catenoidasym", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["geopolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"geopolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["poincare",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"poincare", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidiso",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"catenoidiso", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["goursatimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"goursatimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["poindisk",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"poindisk", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["catenoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"catenoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["handkerchief",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"handkerchief", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudocrosscap",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pseudocrosscap", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cayleysexticmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cayleysexticmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["heart",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"heart", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudosphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pseudosphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circleinvolutemin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["helicoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"helicoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudospherebis",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudospherebis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circleinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circleinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["helicoidprin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"helicoidprin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["pseudospheretcheb",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"pseudospheretcheb", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularcone",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"circularcone", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["heltocat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"heltocat", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rconoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rconoid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularcylinder",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circularcylinder", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["henneberg",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"henneberg", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmond",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"richmond", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembrane",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circularmembrane", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hennebergmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hennebergmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmondmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"richmondmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembranemovie",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circularmembranemovie", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hy2sheet",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hy2sheet", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["richmondpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"richmondpolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["circularmembraneplot",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"circularmembraneplot", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hy2sheetimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hy2sheetimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["roman",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"roman", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cissoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cissoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolamincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperbolamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rosemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rosemin", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["clothoidmin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"clothoidmin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolicparaboloid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperbolicparaboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["rosemincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"rosemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["clothoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"clothoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperbolicparaboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {
            "hyperbolicparaboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scherk", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnccosurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnccosurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperboloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"hyperboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk5",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scherk5", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnccotcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnccotcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hyperboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hyperboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherk5implicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"scherk5implicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchyprofilemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cnchyprofilemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidinvolutemin",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hypocycloidinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherkimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scherkimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchysurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnchysurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {
            "hypocycloidinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scherktowermincurveprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"scherktowermincurveprime", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cnchytcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cnchytcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["hypocycloidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"hypocycloidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["schwarzschildplane",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"schwarzschildplane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cossurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cossurface", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedcircularcone",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"invertedcircularcone", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["scmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"scmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cossurfaceimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cossurfaceimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedcircularcylinder",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"invertedcircularcylinder", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["shoe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"shoe", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["costa",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"costa", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["invertedtorus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"invertedtorus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["shoeasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"shoeasym", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["costamincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"costamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeeksmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"jorgemeeksmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sievert",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sievert", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cpcprofilemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"cpcprofilemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeeksmincurveprime",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"jorgemeeksmincurveprime", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinovaloid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sinovaloid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["cpcsurfrev",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"cpcsurfrev", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["jorgemeekspolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"jorgemeekspolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinsurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sinsurface", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["crosscap",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"crosscap", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kazoolaimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"kazoolaimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sinsurfaceimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"sinsurfaceimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["crosscapimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"crosscapimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kidney",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kidney", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["snail",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"snail", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidinvolutemin",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidinvolutemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["klein4",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"klein4", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidinvolutemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidinvolutemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kleinbottle", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["sphereimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"sphereimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidmin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"deltoidmin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottlebis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kleinbottlebis", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembrane",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"squaremembrane", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["deltoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"deltoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kleinbottlewri",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kleinbottlewri", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembranemovie",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"squaremembranemovie", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dini",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"dini", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kuen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kuen", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["squaremembraneplot",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"squaremembraneplot", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dinihelicoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"dinihelicoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kuentcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kuentcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicellipsoid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"stereographicellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["dinitcheb",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"dinitcheb", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["kummerimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"kummerimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicsphere",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"stereographicsphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["earth",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"earth", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["lemniscatemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lemniscatemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["stereographicspherepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"stereographicspherepolar", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["eighteight",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eighteight", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["lemniscatemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"lemniscatemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["surfs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"surfs", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["eightsurface",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"eightsurface", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["liouville",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"liouville", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["swallowtail",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"swallowtail", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellhypcyclide",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellhypcyclide", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logmincurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"logmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["tetrahedral",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"tetrahedral", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsemin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipsemin", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logminpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"logminpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["thomsen",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"thomsen", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsemincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipsemincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["logspiralmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"logspiralmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torus",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"torus", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["menn",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"menn", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusasym",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"torusasym", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipsoidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipsoidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["mercatorellipsoid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"mercatorellipsoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusbis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"torusbis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticalhyperboloidminus",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipticalhyperboloidminus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["mercatorsphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"mercatorsphere", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["torusimplicit",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"torusimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticalhyperboloidplus",
          BaseStyle->"InformationLink",
          ButtonData:>{
           "Info3582964240-3701296", {"ellipticalhyperboloidplus", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["moebcirc",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"moebcirc", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["tractrixmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"tractrixmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticcoor",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"ellipticcoor", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["moebiusstrip",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"moebiusstrip", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twiflat",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"twiflat", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticparaboloid",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipticparaboloid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["monkeysaddle",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"monkeysaddle", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twisphere",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"twisphere", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ellipticparaboloidimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ellipticparaboloidimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["monkeysaddlepolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"monkeysaddlepolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["twocuspsimplicit",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"twocuspsimplicit", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["enneper",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"enneper", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nephroidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"nephroidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["veronese",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"veronese", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepercatenoidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ennepercatenoidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nielsenspiralmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"nielsenspiralmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["wallis",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wallis", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepercatenoidpolar",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ennepercatenoidpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nodoid",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nodoid", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["whitneyumbrella",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"whitneyumbrella", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["ennepermincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"ennepermincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["nodoidnds",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"nodoidnds", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["wrinkles",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wrinkles", "SURFS`"}},
          ButtonNote->"SURFS`"]},
        {
         ButtonBox["enneperpolar",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"enneperpolar", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["parabolamin",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"parabolamin", "SURFS`"}},
          ButtonNote->"SURFS`"], ""},
        {
         ButtonBox["epicycloidmincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"epicycloidmincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], 
         ButtonBox["parabolamincurve",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"parabolamincurve", "SURFS`"}},
          ButtonNote->"SURFS`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.31666666666666665`]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.5829570408554907`*^9}]
}, Open  ]],

Cell["Click the name to get the usage!", "Text",
 CellChangeTimes->{{3.5802692285303507`*^9, 3.5802692346143613`*^9}, {
   3.5802692752212324`*^9, 3.580269438272719*^9}, {3.582436290769916*^9, 
   3.58243629874153*^9}, {3.582609087960979*^9, 3.582609163561303*^9}, {
   3.582609764604681*^9, 3.5826097687369175`*^9}, 3.582868922436732*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["PLTPROGS", "Section",
 CellChangeTimes->{{3.580181740063805*^9, 3.580181747115018*^9}}],

Cell["\<\
The packagePLTPROGS.m  contains miniprograms useful for graphical \
presentations of curves and surfaces. It starts with the Definition\
\>", "Text",
 CellChangeTimes->{{3.5801832132235928`*^9, 3.58018326276928*^9}, {
   3.5826098908098993`*^9, 3.5826098967772408`*^9}, {3.582609993305762*^9, 
   3.582610042680586*^9}, {3.582610136168933*^9, 3.5826101420012665`*^9}, 
   3.582868930069168*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"pltprogs", ":=", 
   RowBox[{"Print", "[", 
    RowBox[{"Select", "[", 
     RowBox[{
      RowBox[{"Names", "[", "\"\<PLTPROGS`*\>\"", "]"}], ",", 
      RowBox[{
       RowBox[{"#", "!=", "\"\<pltprogs\>\""}], "&"}]}], "]"}], 
    "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["pltprogs"], "Input",
 CellChangeTimes->{{3.580182126024083*^9, 3.58018212965889*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"barycenter\"\>", ",", "\<\"borderedpolygon\"\>", 
   ",", "\<\"centerlines\"\>", ",", "\<\"dip\"\>", ",", "\<\"elrot\"\>", 
   ",", "\<\"filledcurve\"\>", ",", "\<\"graphics2dtosurfrev\"\>", 
   ",", "\<\"lathe\"\>", ",", "\<\"linetolines\"\>", ",", "\<\"orth4d\"\>", 
   ",", "\<\"paramcurveplot1\"\>", ",", "\<\"paramcurveplot2\"\>", 
   ",", "\<\"plotpatch2d\"\>", ",", "\<\"polygonholes\"\>", 
   ",", "\<\"stereo4d\"\>", ",", "\<\"wireframe\"\>"}], "}"}]], "Print",
 CellChangeTimes->{3.580182131577693*^9, 3.5802748500336432`*^9, 
  3.5803661330027623`*^9, 3.582436646066231*^9}]
}, Open  ]],

Cell["\<\
The result is the list of names of the Context PLTPROGS`, with exception of \
\"pltprogs\".  All these programs are defined by Alfred Gray.  I commented \
out Mathematica programs constructed for working with graphics in the \
programs Acrospin or Geomview being  obsolete now. . Other definitions are \
commented out since they don' t work in Mathematica v. 9.\
\>", "Text",
 CellChangeTimes->{{3.5824416744688387`*^9, 3.5824417161872253`*^9}, {
  3.5826100681690435`*^9, 3.5826101082333355`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Names", "[", "\"\<PLTPROGS`*\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5824367166592684`*^9, 3.582436737539463*^9}, 
   3.5824415841966753`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5824415944252605`*^9, 3.5824416026997337`*^9}}],

Cell[BoxData["17"], "Output",
 CellChangeTimes->{3.5824416046318445`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "stereo4d", "]"}]], "Input",
 CellChangeTimes->{{3.5824369338766923`*^9, 3.582436950216627*^9}}],

Cell[BoxData[
 RowBox[{"{", "Protected", "}"}]], "Output",
 CellChangeTimes->{3.582436955999958*^9, 3.582438204267355*^9, 
  3.582447671662263*^9, 3.5824482015815725`*^9, 3.5825511194175596`*^9, 
  3.5826014688831387`*^9, 3.5826044972214036`*^9}]
}, Open  ]],

Cell["\<\
We obtain an interactive  list of the remaining usages in this Context by\
\>", "Text",
 CellChangeTimes->{{3.580183819737058*^9, 3.5801838728239517`*^9}, {
  3.580183912962822*^9, 3.580183960917306*^9}, {3.580184003380581*^9, 
  3.5801840953271422`*^9}, {3.5802719433863926`*^9, 3.5802719476336355`*^9}, {
  3.5802719780743766`*^9, 3.5802720889137163`*^9}, {3.5802749044097533`*^9, 
  3.580274938545706*^9}, {3.580365711084421*^9, 3.580365807726591*^9}, {
  3.5824364361712255`*^9, 3.58243647472243*^9}, {3.5824416571868505`*^9, 
  3.582441665882348*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PLTPROGS`*"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801230896898966`*^9, 3.5801231101259327`*^9}, {
  3.582436528259493*^9, 3.5824365381150565`*^9}, {3.5824365807394943`*^9, 
  3.5824365954273343`*^9}, {3.5824403693681917`*^9, 3.5824403765476017`*^9}}],

Cell[BoxData[
 DynamicModuleBox[{Typeset`open$$ = True}, 
  PaneSelectorBox[{False->
   RowBox[{
    OpenerBox[Dynamic[Typeset`open$$],
     ImageSize->Small], 
    StyleBox["PLTPROGS`", "InfoHeading"]}], True->GridBox[{
     {
      RowBox[{
       OpenerBox[Dynamic[Typeset`open$$],
        ImageSize->Small], 
       StyleBox["PLTPROGS`", "InfoHeading"]}]},
     {GridBox[{
        {
         ButtonBox["barycenter",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"barycenter", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["filledcurve",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"filledcurve", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["paramcurveplot1",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"paramcurveplot1", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["stereo4d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"stereo4d", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"]},
        {
         ButtonBox["borderedpolygon",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"borderedpolygon", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["graphics2dtosurfrev",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"graphics2dtosurfrev", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["paramcurveplot2",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"paramcurveplot2", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["wireframe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"wireframe", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"]},
        {
         ButtonBox["centerlines",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"centerlines", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["lathe",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"lathe", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["plotpatch2d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"plotpatch2d", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], ""},
        {
         ButtonBox["dip",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"dip", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["linetolines",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"linetolines", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["pltprogs",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"pltprogs", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], ""},
        {
         ButtonBox["elrot",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"elrot", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["orth4d",
          BaseStyle->"InformationLink",
          ButtonData:>{"Info3582964240-3701296", {"orth4d", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], 
         ButtonBox["polygonholes",
          BaseStyle->"InformationLink",
          
          ButtonData:>{
           "Info3582964240-3701296", {"polygonholes", "PLTPROGS`"}},
          ButtonNote->"PLTPROGS`"], ""}
       },
       DefaultBaseStyle->"InfoGrid",
       GridBoxItemSize->{"Columns" -> {{
            Scaled[0.2375]}}}]}
    },
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}]}, 
   Dynamic[Typeset`open$$],
   ImageSize->Automatic]]], "Print", "InfoCell",
 CellChangeTimes->{3.5829570410165005`*^9}]
}, Open  ]],

Cell["\<\
Click the name to get the usage!

In the following subsections we consider Alfred\[CloseCurlyQuote]s \
definitions in detail. Each subsection starts with the definition copied as a \
not evaluable cell and finishes with an example. \
\>", "Text",
 CellChangeTimes->{{3.5801856739719152`*^9, 3.5801859014515142`*^9}, {
   3.5803592506694746`*^9, 3.580359402832142*^9}, 3.580366235884943*^9, 
   3.580366322633216*^9, 3.5826071736084843`*^9, 3.5828689426698895`*^9}],

Cell[CellGroupData[{

Cell["barycenter", "Subsection",
 CellChangeTimes->{{3.5801817771138706`*^9, 3.5801817817314787`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"barycenter", "[", 
   RowBox[{"Polygon", "[", "pts_", "]"}], "]"}], ":=", "\n", "        ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "ptstmp", "}"}], ",", "\n", "        ", 
    RowBox[{
     RowBox[{"ptstm", "=", 
      RowBox[{"Append", "[", 
       RowBox[{"pts", ",", 
        RowBox[{"First", "[", "pts", "]"}]}], "]"}]}], ";", "\n", "\t", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Polygon", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"ptstm", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"ptstm", "[", 
           RowBox[{"[", 
            RowBox[{"i", " ", "+", " ", "1"}], "]"}], "]"}], ",", "\n", 
          "        ", 
          RowBox[{
           RowBox[{"Plus", "@@", "pts"}], "/", 
           RowBox[{"Length", "[", "ptstm", "]"}]}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ptstm", "]"}], " ", "-", " ", "1"}]}], 
        "}"}]}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"pts", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5801273710545692`*^9, 3.5801274457888436`*^9}, {
   3.5801295699333377`*^9, 3.580129596684868*^9}, 3.5801845032054586`*^9, {
   3.5801852359387455`*^9, 3.5801852427247577`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"ptstm", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"pts", ",", 
     RowBox[{"First", "[", "pts", "]"}]}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwV0FtI0wEYBfCti5EMIWpmMjaciSSblT60ybT+zcuwRjVdrhHpdMFihBDm
ZgwbeIVRFvWworJclsG8EgglmstEF0y2GjqSjEixf5I1Vs1563wPHz84HM7D
l1JZrbnI5XA4pThuTvmuF26WUdSKtGTUMrjtI1TlekbIzPnVObJoZ+cqORG/
EheEtqHe6mloV+qMG3BhbYdgE2osShe3m2UKW5P5GXCsak2cBXVdUespaPpt
LddAXn5b0lnISYkdqKB+8JaC1NWeN5MbP1uWSbPdVmuAF94y+nraWxzY3QLD
y8I3DspHylgyn7fuCMAtzbkRdQ/LtKbPSRtgJBi98QzKPFrBAMw+czkuBKXG
8cFZWB+YkcSg0vTEubWXZXz+20FSX1qcmgpZx1i/Gj6ODzysg5qDRucjyOvJ
HnoKHf0f1vtgZ2WdyAunzm0veQ9zuvZp/8KlBeFxUR/LCKeqcqVQfMeTqYJ2
3/yiGnpGC242wIq2T37ymzxZ3wgTZ6+2kRF5cJi8+5znJd9ZFHlNUNfeXOWE
s9N8Qwe8by457ILjArlmgvoBefok7X/9zpA1PuMDUpzm/ky6VKM/SE53e54f
HtsjOzQDVWWv7pEdlXwfeXpiMyEE464kOCPU8740/IHhX4pL/+C1RKNsBWZY
k8bIrM7GJdLkDh2N0b7ydYukH//Ye+KkFNq+XD+SCSVchl8AJ4vSagqhSC50
ksU6TZMX7teLh8PwPzoSbho=
  "]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Polygon", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ptstm", "[", 
     RowBox[{"[", "1", "]"}], "]"}], ",", 
    RowBox[{"ptstm", "[", 
     RowBox[{"[", "2", "]"}], "]"}], ",", "\n", "        ", 
    RowBox[{
     RowBox[{"Plus", "@@", "pts"}], "/", 
     RowBox[{"Length", "[", "ptstm", "]"}]}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5801847728363323`*^9, 3.580184777984341*^9}}],

Cell[BoxData[
 RowBox[{"Polygon", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      FractionBox["1", "5"], ",", 
      FractionBox["1", "5"], ",", 
      FractionBox["1", "5"]}], "}"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.5801852746736135`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"barycenter", "[", 
  RowBox[{"Polygon", "[", "pts", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.580129385710801*^9, 3.580129411604282*^9}, {
  3.5801296025012007`*^9, 3.580129611644724*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Polygon", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Polygon", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Polygon", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"]}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Polygon", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"], ",", 
        FractionBox["1", "5"]}], "}"}]}], "}"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5801296131398087`*^9, 3.580129619316162*^9}, 
   3.580185287138036*^9, 3.5801874766330814`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"barypts", " ", "=", " ", 
  RowBox[{"Graphics3D", "[", "%", "]"}]}]], "Input",
 CellChangeTimes->{{3.5801874386470146`*^9, 3.5801874676162653`*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   Polygon3DBox[
    NCache[{{0, 0, 0}, {1, 0, 0}, {
      Rational[1, 5], Rational[1, 5], Rational[1, 5]}}, {{0, 0, 0}, {1, 0, 
      0}, {0.2, 0.2, 0.2}}]], 
   Polygon3DBox[
    NCache[{{1, 0, 0}, {0, 1, 0}, {
      Rational[1, 5], Rational[1, 5], Rational[1, 5]}}, {{1, 0, 0}, {0, 1, 
      0}, {0.2, 0.2, 0.2}}]], 
   Polygon3DBox[
    NCache[{{0, 1, 0}, {0, 0, 1}, {
      Rational[1, 5], Rational[1, 5], Rational[1, 5]}}, {{0, 1, 0}, {0, 0, 
      1}, {0.2, 0.2, 0.2}}]], 
   Polygon3DBox[
    NCache[{{0, 0, 1}, {0, 0, 0}, {
      Rational[1, 5], Rational[1, 5], Rational[1, 5]}}, {{0, 0, 1}, {0, 0, 
      0}, {0.2, 0.2, 0.2}}]]},
  AutomaticImageSize->True,
  ImageSize->{354.9642270577869, 331.52955700286975`},
  ViewPoint->{1.4627659383532565`, -2.918735886585416, 0.8895485562643359},
  ViewVertical->{0.2288793789169243, -0.2870304786294157, 
   0.93017618451795}]], "Output",
 CellChangeTimes->{3.5801874834502935`*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["borderedpolygon", "Subsection",
 CellChangeTimes->{3.580184228847777*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"borderedpolygon", "[", 
   RowBox[{"intsty_", ",", "borsty_", ",", "versty_", ",", "pts_"}], "]"}], ":=",
   "\n", "        ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Flatten", "[", 
     RowBox[{"{", 
      RowBox[{"intsty", ",", 
       RowBox[{"Polygon", "[", "pts", "]"}]}], "}"}], "]"}], ",", "\n", 
    "         ", 
    RowBox[{"Flatten", "[", 
     RowBox[{"{", 
      RowBox[{"borsty", ",", 
       RowBox[{"Line", "[", 
        RowBox[{"Append", "[", 
         RowBox[{"pts", ",", 
          RowBox[{"First", "[", "pts", "]"}]}], "]"}], "]"}]}], "}"}], "]"}], 
    ",", "\n", "         ", 
    RowBox[{"Flatten", "[", 
     RowBox[{"{", 
      RowBox[{"versty", ",", 
       RowBox[{"Point", "/@", "pts"}]}], "}"}], "]"}]}], "}"}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Flatten", "[", 
  RowBox[{"{", 
   RowBox[{"Blue", ",", 
    RowBox[{"Polygon", "[", "pts", "]"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5801280813111935`*^9, 3.580128082316251*^9}, {
  3.58012811324402*^9, 3.5801281162541924`*^9}, {3.580128190685449*^9, 
  3.580128193586615*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"RGBColor", "[", 
    RowBox[{"0", ",", "0", ",", "1"}], "]"}], ",", 
   RowBox[{"Polygon", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580129674755333*^9, 3.5801854067902455`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"RGBColor", "[", 
      RowBox[{"0", ",", "0", ",", "1"}], "]"}], ",", 
     RowBox[{"Polygon", "[", "pts", "]"}]}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5801282144268074`*^9, 3.580128236974097*^9}, {
  3.5801296823987703`*^9, 3.580129710852398*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Flatten", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Thickness", "[", "0.03", "]"}], ",", "Red", ",", 
    RowBox[{"Line", "[", 
     RowBox[{"Append", "[", 
      RowBox[{"pts", ",", 
       RowBox[{"First", "[", "pts", "]"}]}], "]"}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5801298783889804`*^9, 3.580129934173171*^9}, {
  3.580130016062855*^9, 3.580130086102861*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Thickness", "[", "0.03`", "]"}], ",", 
   RowBox[{"RGBColor", "[", 
    RowBox[{"1", ",", "0", ",", "0"}], "]"}], ",", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.580129937250347*^9, {3.5801300379961095`*^9, 3.580130087431937*^9}, 
   3.580185443746711*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58012994919103*^9, 3.580129972692374*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"borderedpolygon", "[", 
  RowBox[{
   RowBox[{"{", "Blue", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Thickness", "[", ".01", "]"}], ",", " ", "Red"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"PointSize", "[", "0.03", "]"}], ",", " ", "Black"}], "}"}], ",",
    "pts"}], "]"}]], "Input",
 CellChangeTimes->{{3.580127358427847*^9, 3.5801273628591003`*^9}, {
   3.5801276069490614`*^9, 3.580127623324998*^9}, {3.580127665220394*^9, 
   3.5801277370445023`*^9}, {3.580130318512154*^9, 3.5801303420044975`*^9}, {
   3.580130524549939*^9, 3.5801305721576614`*^9}, {3.580130642390679*^9, 
   3.5801306687341857`*^9}, 3.5801307037991915`*^9, {3.580131640094744*^9, 
   3.5801316524514513`*^9}, {3.5801317999428873`*^9, 3.580131803966117*^9}, {
   3.580185527035257*^9, 3.5801855302800627`*^9}, {3.5801860022276917`*^9, 
   3.5801860202457232`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"RGBColor", "[", 
      RowBox[{"0", ",", "0", ",", "1"}], "]"}], ",", 
     RowBox[{"Polygon", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Thickness", "[", "0.01`", "]"}], ",", 
     RowBox[{"RGBColor", "[", 
      RowBox[{"1", ",", "0", ",", "0"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"PointSize", "[", "0.03`", "]"}], ",", 
     RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], "]"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580186025237732*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"Flatten", "[", "%", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.580128422281696*^9, 3.5801284552685823`*^9}, {
  3.580130385140965*^9, 3.580130385800003*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["centerline", "Subsection",
 CellChangeTimes->{{3.5801871948653865`*^9, 3.580187198905793*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"centerlines", "[", 
    RowBox[{"surface_", ",", 
     RowBox[{"{", 
      RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v_", ",", "v0_", ",", "v1_"}], "}"}], ",", "opts___"}], "]"}], 
   ":=", "\n", "        ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"plottmp", ",", "grtmp"}], "}"}], ",", "\n", "                ", 
     RowBox[{
      RowBox[{"plottmp", " ", "=", " ", 
       RowBox[{"ParametricPlot3D", "[", "\n", "                ", 
        RowBox[{
         RowBox[{"surface", "//", "Evaluate"}], ",", 
         RowBox[{"{", 
          RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", "v0", ",", "v1"}], "}"}], ",", "\n", 
         "                ", 
         RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
        "]"}]}], ";", "\n", "                ", 
      RowBox[{"grtmp", " ", "=", " ", 
       RowBox[{"plottmp", " ", "/.", "\n", "     ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Polygon", "[", "pts_", "]"}], ":>", "\n", "     ", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Polygon", "[", 
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"pts", "[", 
                RowBox[{"[", 
                 RowBox[{"{", 
                  RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], ",", 
               RowBox[{
                RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], 
            ",", "\n", "      ", 
            RowBox[{"Polygon", "[", 
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"pts", "[", 
                RowBox[{"[", 
                 RowBox[{"{", 
                  RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], ",", 
               RowBox[{
                RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], 
            ",", "\n", "      ", 
            RowBox[{"Polygon", "[", 
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"pts", "[", 
                RowBox[{"[", 
                 RowBox[{"{", 
                  RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], ",", 
               RowBox[{
                RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], 
            ",", "\n", "      ", 
            RowBox[{"Polygon", "[", 
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"pts", "[", 
                RowBox[{"[", 
                 RowBox[{"{", 
                  RowBox[{"4", ",", "1"}], "}"}], "]"}], "]"}], ",", 
               RowBox[{
                RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}]}], 
           "}"}]}], ")"}]}]}], ";", "\n", "        ", 
      RowBox[{"Show", "[", 
       RowBox[{"grtmp", ",", 
        RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
    "]"}]}], "\n"}], "\n", 
 RowBox[{"    ", 
  RowBox[{
   RowBox[{"centerlines", "[", "graphic_", "]"}], ":=", 
   RowBox[{"Show", "[", 
    RowBox[{"graphic", " ", "/.", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Polygon", "[", "pts_", "]"}], ":>", "\n", "     ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Polygon", "[", 
          RowBox[{"Append", "[", 
           RowBox[{
            RowBox[{"pts", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
         "\n", "      ", 
         RowBox[{"Polygon", "[", 
          RowBox[{"Append", "[", 
           RowBox[{
            RowBox[{"pts", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
         "\n", "      ", 
         RowBox[{"Polygon", "[", 
          RowBox[{"Append", "[", 
           RowBox[{
            RowBox[{"pts", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
         "\n", "      ", 
         RowBox[{"Polygon", "[", 
          RowBox[{"Append", "[", 
           RowBox[{
            RowBox[{"pts", "[", 
             RowBox[{"[", 
              RowBox[{"{", 
               RowBox[{"4", ",", "1"}], "}"}], "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}]}], 
        "}"}]}], ")"}]}], "]"}]}]}], "\n"}], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pts", "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{3.580187805684459*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5801878070104613`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Append", "[", 
  RowBox[{
   RowBox[{"pts", "[", 
    RowBox[{"[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], ",", 
   RowBox[{
    RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "4"], ",", 
     FractionBox["1", "4"], ",", 
     FractionBox["1", "4"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5801889131148043`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"Polygon", "[", "pts", "]"}], "]"}]], "Input"],

Cell[BoxData[
 Graphics3DBox[Polygon3DBox[{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}}],
  AutomaticImageSize->True,
  ImageSize->{361.5949132319621, 414.43612671310007`},
  ViewPoint->{0.7260832968784519, -2.875006193250542, 1.6300743647960185`},
  ViewVertical->{-0.059282531395107224`, -0.15116560838068957`, 
   0.9867292132668843}]], "Output",
 CellChangeTimes->{3.580188325399372*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"centerlines", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"Polygon", "[", "pts", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58018753298038*^9, 3.5801875506084113`*^9}, {
  3.5801876294821496`*^9, 3.580187685907449*^9}, {3.580187945928705*^9, 
  3.5801879533387184`*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   Polygon3DBox[
    NCache[{{0, 0, 0}, {1, 0, 0}, {
      Rational[1, 4], Rational[1, 4], Rational[1, 4]}}, {{0, 0, 0}, {1, 0, 
      0}, {0.25, 0.25, 0.25}}]], 
   Polygon3DBox[
    NCache[{{1, 0, 0}, {0, 1, 0}, {
      Rational[1, 4], Rational[1, 4], Rational[1, 4]}}, {{1, 0, 0}, {0, 1, 
      0}, {0.25, 0.25, 0.25}}]], 
   Polygon3DBox[
    NCache[{{0, 1, 0}, {0, 0, 1}, {
      Rational[1, 4], Rational[1, 4], Rational[1, 4]}}, {{0, 1, 0}, {0, 0, 
      1}, {0.25, 0.25, 0.25}}]], 
   Polygon3DBox[
    NCache[{{0, 0, 1}, {0, 0, 0}, {
      Rational[1, 4], Rational[1, 4], Rational[1, 4]}}, {{0, 0, 1}, {0, 0, 
      0}, {0.25, 0.25, 0.25}}]]}]], "Output",
 CellChangeTimes->{3.580188088840556*^9, 3.580188339142996*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pts5", " ", "=", " ", 
  RowBox[{"Append", "[", 
   RowBox[{"pts", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.580190571475717*^9, 3.5801906291490183`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwdzF1Ik2EAxfFJKLsQydlMpjVSw1pOSoTN0MW7SsQ5i01FRtSmKxRRu7Bo
IDHQsWjlyhbNMKNcH6Lmpn0slbWmyZolbChuayasiHgTi5jaTLPzeHH4cW7+
e6qbZGdjGAyGHPO/nIz09dNU6rArcQby7K43RHHt+sLWb3v0l5jSEI2bhacF
1qY5WDpepf4HbS3MtE24a/VoT8wATbGYqWwevPFlPT0XZu6LXjoB7XLNGRmM
P2ZMqYRDt9b2K+Ftwc0C4lP/qXpiuO7KT+JIuOWiCq64xIrL0Fg7nKSHYhnX
aYDbc6voLa9tGHxwOlYUkT6jqfInC/xW6KyIXn8M9ZmVaUPw+WhDXACWOCZf
hYiyQPYa/Nj7wLxtkKYWD3TMEhXlJRkZkDZM2KSw/qrvngb6OtXmbshj5o1Z
YDBpZsMKz4c1XA+8G4iVT8ExFqdiBdrPccVcKzpl6kI+XGKN5xRDSdm371LY
ThW1t0Kl8ZOXOPiao2iDyaELRmKiadZB1EbjPcQdwQKRDv7q1teYIe9wsuoh
tL+TH+qBHV35Mjc0+fKz3pN+NU0Rm6fVXcQlS/9nYqP77Q8iY+C+yAvjTcKD
ftKfG+kkNobZ08ST7s2EAOQsJ5gj0Jn3QrUMLTWFdauk36kWRmGITpkgft2p
WyRKsoJH1uAH16g+20ZTcXckpXxYrNQKcuAfLcU+Duc9e5uLoLdvt5k45ZLp
PFA4n+74Df8DL9lx0g==
  "]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"Polygon", "[", "pts5", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58018753298038*^9, 3.5801875506084113`*^9}, {
   3.5801876294821496`*^9, 3.580187685907449*^9}, {3.580187945928705*^9, 
   3.5801879533387184`*^9}, 3.5801906725014944`*^9, {3.5801907218443813`*^9, 
   3.580190724683586*^9}}],

Cell[BoxData[
 Graphics3DBox[
  Polygon3DBox[{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {1, 1, 1}}],
  AutomaticImageSize->True,
  ImageSize->{374.3994931408639, 402.83475493018705`},
  ViewPoint->{0.9788279091435104, -1.5008378621764564`, 2.870432308161847},
  ViewVertical->{-0.1311914279284848, 0.3533473040111942, 
   0.9262475327827361}]], "Output",
 CellChangeTimes->{3.5801907277411914`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"centerlines", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"Polygon", "[", "pts5", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.58018753298038*^9, 3.5801875506084113`*^9}, {
   3.5801876294821496`*^9, 3.580187685907449*^9}, {3.580187945928705*^9, 
   3.5801879533387184`*^9}, 3.5801906725014944`*^9, 3.58019071568237*^9}],

Cell[BoxData[
 Graphics3DBox[{
   Polygon3DBox[
    NCache[{{0, 0, 0}, {1, 0, 0}, {
      Rational[1, 2], Rational[1, 2], Rational[1, 2]}}, {{0, 0, 0}, {1, 0, 
      0}, {0.5, 0.5, 0.5}}]], 
   Polygon3DBox[
    NCache[{{1, 0, 0}, {0, 1, 0}, {
      Rational[1, 2], Rational[1, 2], Rational[1, 2]}}, {{1, 0, 0}, {0, 1, 
      0}, {0.5, 0.5, 0.5}}]], 
   Polygon3DBox[
    NCache[{{0, 1, 0}, {0, 0, 1}, {
      Rational[1, 2], Rational[1, 2], Rational[1, 2]}}, {{0, 1, 0}, {0, 0, 
      1}, {0.5, 0.5, 0.5}}]], 
   Polygon3DBox[
    NCache[{{0, 0, 1}, {0, 0, 0}, {
      Rational[1, 2], Rational[1, 2], Rational[1, 2]}}, {{0, 0, 1}, {0, 0, 
      0}, {0.5, 0.5, 0.5}}]]},
  AutomaticImageSize->True,
  ImageSize->{324.16347740167055`, 316.0803610545196},
  ViewPoint->{-1.3351389815905097`, -3.0027149655273404`, 0.8069118512177151},
  ViewVertical->{-0.117166648640954, -0.3658597746048617, 
   0.9232651849670966}]], "Output",
 CellChangeTimes->{3.5801907454940224`*^9}]
}, Open  ]],

Cell["\<\
The application to surfaces did not work:\
\>", "Text",
 CellChangeTimes->{{3.5801908304985723`*^9, 3.580190853321412*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"centerlines", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "v", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"u", "^", "2"}], "+", 
        RowBox[{"v", "^", "2"}]}], ")"}], "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5801884439595804`*^9, 3.5801885655773935`*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHu0V1W1x4lABR0OBmIamk9SEQJziIqpc3S9efEBdaXMukoNxUflUSNS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   "], {{
     {EdgeForm[None], 
      GraphicsGroup3DBox[{{
         Polygon3DBox[
          NCache[{{685, 910, 209}, {292, 515, 43}, {
            123598, 138575, Rational[171239, 4]}}, {{685, 910, 209}, {292, 
            515, 43}, {123598, 138575, 42809.75}}]], 
         Polygon3DBox[
          NCache[{{292, 515, 43}, {291, 514, 42}, {
            123598, 138575, Rational[171239, 4]}}, {{292, 515, 43}, {291, 514,
             42}, {123598, 138575, 42809.75}}]], 
         Polygon3DBox[
          NCache[{{291, 514, 42}, {290, 513, 41}, {
            123598, 138575, Rational[171239, 4]}}, {{291, 514, 42}, {290, 513,
             41}, {123598, 138575, 42809.75}}]], 
         Polygon3DBox[
          NCache[{{290, 513, 41}, {685, 910, 209}, {
            123598, 138575, Rational[171239, 4]}}, {{290, 513, 41}, {685, 910,
             209}, {123598, 138575, 42809.75}}]]}, {
         Polygon3DBox[
          NCache[{{820, 1174, 382, 133}, {810, 1165, 371, 123}, {
            Rational[316357, 2], 149196, Rational[68709, 2], 49817}}, {{820, 
            1174, 382, 133}, {810, 1165, 371, 123}, {
            158178.5, 149196, 34354.5, 49817}}]], 
         Polygon3DBox[
          NCache[{{810, 1165, 371, 123}, {752, 536, 328, 80}, {
            Rational[316357, 2], 149196, Rational[68709, 2], 49817}}, {{810, 
            1165, 371, 123}, {752, 536, 328, 80}, {
            158178.5, 149196, 34354.5, 49817}}]], 
         Polygon3DBox[
          NCache[{{752, 536, 328, 80}, {827, 1178, 388, 140}, {
            Rational[316357, 2], 149196, Rational[68709, 2], 49817}}, {{752, 
            536, 328, 80}, {827, 1178, 388, 140}, {
            158178.5, 149196, 34354.5, 49817}}]], 
         Polygon3DBox[
          NCache[{{827, 1178, 388, 140}, {820, 1174, 382, 133}, {
            Rational[316357, 2], 149196, Rational[68709, 2], 49817}}, {{827, 
            1178, 388, 140}, {820, 1174, 382, 133}, {
            158178.5, 149196, 34354.5, 49817}}]]}, {
         Polygon3DBox[
          NCache[{{1218, 1018, 523, 300, 301}, {739, 942, 522, 299, 52}, {
            Rational[86865, 2], 49473, Rational[116201, 4], Rational[
             45449, 2], Rational[21547, 2]}}, {{1218, 1018, 523, 300, 301}, {
            739, 942, 522, 299, 52}, {
            43432.5, 49473, 29050.25, 22724.5, 10773.5}}]], 
         Polygon3DBox[
          NCache[{{739, 942, 522, 299, 52}, {897, 999, 666, 443, 195}, {
            Rational[86865, 2], 49473, Rational[116201, 4], Rational[
             45449, 2], Rational[21547, 2]}}, {{739, 942, 522, 299, 52}, {897,
             999, 666, 443, 195}, {
            43432.5, 49473, 29050.25, 22724.5, 10773.5}}]], 
         Polygon3DBox[
          NCache[{{897, 999, 666, 443, 195}, {865, 993, 634, 411, 163}, {
            Rational[86865, 2], 49473, Rational[116201, 4], Rational[
             45449, 2], Rational[21547, 2]}}, {{897, 999, 666, 443, 195}, {
            865, 993, 634, 411, 163}, {
            43432.5, 49473, 29050.25, 22724.5, 10773.5}}]], 
         Polygon3DBox[
          NCache[{{865, 993, 634, 411, 163}, {1218, 1018, 523, 300, 301}, {
            Rational[86865, 2], 49473, Rational[116201, 4], Rational[
             45449, 2], Rational[21547, 2]}}, {{865, 993, 634, 411, 163}, {
            1218, 1018, 523, 300, 301}, {
            43432.5, 49473, 29050.25, 22724.5, 
             10773.5}}]]}}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{251, 474, 1106, 252, 488, 280, 1211, 503, 295, 1216, 518, 
       310, 1223, 533, 325, 1233, 548, 340, 1244, 563, 1150, 355, 578, 1164, 
       370, 1259, 593, 385, 1263, 608, 400, 1268, 623, 415, 1275, 638, 430, 
       1284, 653, 445, 1295, 668, 460}], 
      Line3DBox[{253, 475, 1107, 254, 489, 1118, 281, 504, 296, 1217, 519, 
       311, 1224, 534, 326, 1234, 549, 341, 1245, 564, 1151, 356, 579, 1165, 
       371, 594, 1176, 386, 609, 401, 1269, 624, 416, 1276, 639, 431, 1285, 
       654, 446, 1296, 669, 461}], 
      Line3DBox[{255, 476, 1108, 256, 490, 1119, 282, 505, 1129, 297, 520, 
       312, 1225, 535, 327, 1235, 550, 342, 1246, 565, 1152, 357, 580, 1166, 
       372, 595, 1177, 387, 610, 1186, 402, 625, 417, 1277, 640, 432, 1286, 
       655, 447, 1297, 670, 462}], 
      Line3DBox[{257, 477, 1109, 258, 491, 1120, 283, 506, 1130, 298, 521, 
       1137, 313, 536, 328, 1236, 551, 343, 1247, 566, 1153, 358, 581, 1167, 
       373, 596, 1178, 388, 611, 1187, 403, 626, 1194, 418, 641, 433, 1287, 
       656, 448, 1298, 671, 463}], 
      Line3DBox[{259, 478, 1110, 260, 492, 1121, 284, 507, 1131, 299, 522, 
       1138, 314, 537, 1143, 329, 552, 344, 1248, 567, 1154, 359, 582, 1168, 
       374, 597, 1179, 389, 612, 1188, 404, 627, 1195, 419, 642, 1200, 434, 
       657, 449, 1299, 672, 464}], 
      Line3DBox[{261, 479, 1111, 263, 493, 1122, 285, 508, 1132, 300, 523, 
       1139, 315, 538, 1144, 330, 553, 1147, 345, 568, 1155, 360, 583, 1169, 
       375, 598, 1180, 390, 613, 1189, 405, 628, 1196, 420, 643, 1201, 435, 
       658, 1204, 450, 1300, 673, 465}], 
      Line3DBox[{265, 481, 1112, 266, 1209, 495, 287, 1213, 510, 302, 1219, 
       525, 317, 1227, 540, 332, 1238, 555, 347, 1250, 570, 1156, 362, 585, 
       377, 1261, 600, 392, 1265, 615, 407, 1271, 630, 422, 1279, 645, 437, 
       1289, 660, 452, 1302, 675, 467}], 
      Line3DBox[{267, 482, 1113, 268, 496, 1123, 288, 1214, 511, 303, 1220, 
       526, 318, 1228, 541, 333, 1239, 556, 348, 1251, 571, 1157, 363, 586, 
       1170, 378, 601, 393, 1266, 616, 408, 1272, 631, 423, 1280, 646, 438, 
       1290, 661, 453, 1303, 676, 468}], 
      Line3DBox[{269, 483, 1114, 270, 497, 1124, 289, 512, 304, 1221, 527, 
       319, 1229, 542, 334, 1240, 557, 349, 1252, 572, 1158, 364, 587, 1171, 
       379, 602, 1181, 394, 617, 409, 1273, 632, 424, 1281, 647, 439, 1291, 
       662, 454, 1304, 677, 469}], 
      Line3DBox[{271, 484, 1115, 272, 498, 1125, 290, 513, 1133, 305, 528, 
       320, 1230, 543, 335, 1241, 558, 350, 1253, 573, 1159, 365, 588, 1172, 
       380, 603, 1182, 395, 618, 1190, 410, 633, 425, 1282, 648, 440, 1292, 
       663, 455, 1305, 678, 470}], 
      Line3DBox[{273, 485, 1116, 274, 499, 1126, 291, 514, 1134, 306, 529, 
       1140, 321, 1231, 544, 336, 1242, 559, 351, 1254, 574, 1160, 366, 589, 
       1173, 381, 604, 1183, 396, 619, 1191, 411, 634, 1197, 426, 649, 441, 
       1293, 664, 456, 1306, 679, 471}], 
      Line3DBox[{275, 486, 1117, 276, 500, 1127, 292, 515, 1135, 307, 530, 
       1141, 322, 545, 1145, 337, 560, 352, 1255, 575, 1161, 367, 590, 1174, 
       382, 605, 1184, 397, 620, 1192, 412, 635, 1198, 427, 650, 1202, 442, 
       665, 457, 1307, 680, 472}], 
      Line3DBox[{277, 682, 1308, 683, 278, 501, 1128, 293, 516, 1136, 308, 
       531, 1142, 323, 546, 1146, 338, 561, 1148, 353, 576, 1162, 1163, 368, 
       591, 1175, 383, 606, 1185, 398, 621, 1193, 413, 636, 1199, 428, 651, 
       1203, 443, 666, 1205, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1294, 444, 652, 1283, 429, 637, 1274, 414, 
       622, 1267, 399, 607, 1262, 384, 592, 1258, 369, 577, 1256, 354, 1149, 
       562, 1243, 339, 547, 1232, 324, 532, 1222, 309, 517, 1215, 294, 502, 
       1210, 279, 487, 1207, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1301, 451, 659, 1288, 436, 644, 1278, 421, 629, 
       1270, 406, 614, 1264, 391, 599, 1260, 376, 584, 1257, 361, 569, 1249, 
       346, 554, 1237, 331, 539, 1226, 316, 524, 1218, 301, 509, 1212, 286, 
       494, 1208, 264, 480, 1206, 262}]}, {
      Line3DBox[{690, 1000, 473, 689, 1106, 912, 691, 1107, 913, 692, 1108, 
       914, 693, 1109, 915, 694, 1110, 916, 695, 1111, 1006, 1206, 696, 1112, 
       917, 697, 1113, 918, 698, 1114, 919, 699, 1115, 920, 700, 1116, 921, 
       701, 1117, 922, 702, 1308, 1001, 923, 1002}], 
      Line3DBox[{704, 1007, 1207, 703, 488, 705, 1118, 924, 706, 1119, 925, 
       707, 1120, 926, 708, 1121, 927, 709, 1122, 1008, 1208, 710, 1009, 1209,
        711, 1123, 928, 712, 1124, 929, 713, 1125, 930, 714, 1126, 931, 715, 
       1127, 932, 716, 1128, 933, 717}], 
      Line3DBox[{719, 1010, 1210, 718, 1011, 1211, 720, 504, 721, 1129, 934, 
       722, 1130, 935, 723, 1131, 936, 724, 1132, 1012, 1212, 725, 1013, 1213,
        726, 1014, 1214, 727, 512, 728, 1133, 937, 729, 1134, 938, 730, 1135, 
       939, 731, 1136, 940, 732}], 
      Line3DBox[{734, 1015, 1215, 733, 1016, 1216, 735, 1017, 1217, 736, 520, 
       737, 1137, 941, 738, 1138, 942, 739, 1139, 1018, 1218, 740, 1019, 1219,
        741, 1020, 1220, 742, 1021, 1221, 743, 528, 744, 1140, 943, 745, 1141,
        944, 746, 1142, 945, 747}], 
      Line3DBox[{749, 1022, 1222, 748, 1023, 1223, 750, 1024, 1224, 751, 1025,
        1225, 752, 536, 753, 1143, 946, 754, 1144, 1026, 1226, 755, 1027, 
       1227, 756, 1028, 1228, 757, 1029, 1229, 758, 1030, 1230, 759, 1031, 
       1231, 760, 1145, 947, 761, 1146, 948, 762}], 
      Line3DBox[{764, 1032, 1232, 763, 1033, 1233, 765, 1034, 1234, 766, 1035,
        1235, 767, 1036, 1236, 768, 552, 769, 1147, 1037, 1237, 770, 1038, 
       1238, 771, 1039, 1239, 772, 1040, 1240, 773, 1041, 1241, 774, 1042, 
       1242, 775, 560, 776, 1148, 949, 777}], 
      Line3DBox[{781, 1149, 950, 779, 1150, 951, 783, 1151, 952, 785, 1152, 
       953, 787, 1153, 954, 789, 1154, 955, 791, 1155, 569, 793, 1156, 956, 
       795, 1157, 957, 797, 1158, 958, 799, 1159, 959, 801, 1160, 960, 803, 
       1161, 961, 805, 1163, 963, 807}], 
      Line3DBox[{806, 962, 1162, 804, 1255, 1054, 802, 1254, 1053, 800, 1253, 
       1052, 798, 1252, 1051, 796, 1251, 1050, 794, 1250, 1049, 792, 1249, 
       568, 790, 1248, 1048, 788, 1247, 1047, 786, 1246, 1046, 784, 1245, 
       1045, 782, 1244, 1044, 778, 1243, 1043, 780}], 
      Line3DBox[{809, 1055, 1256, 808, 1164, 964, 810, 1165, 965, 811, 1166, 
       966, 812, 1167, 967, 813, 1168, 968, 814, 1169, 1056, 1257, 815, 585, 
       816, 1170, 969, 817, 1171, 970, 818, 1172, 971, 819, 1173, 972, 820, 
       1174, 973, 821, 1175, 974, 822}], 
      Line3DBox[{824, 1057, 1258, 823, 1058, 1259, 825, 1176, 975, 826, 1177, 
       976, 827, 1178, 977, 828, 1179, 978, 829, 1180, 1059, 1260, 830, 1060, 
       1261, 831, 601, 832, 1181, 979, 833, 1182, 980, 834, 1183, 981, 835, 
       1184, 982, 836, 1185, 983, 837}], 
      Line3DBox[{839, 1061, 1262, 838, 1062, 1263, 840, 609, 841, 1186, 984, 
       842, 1187, 985, 843, 1188, 986, 844, 1189, 1063, 1264, 845, 1064, 1265,
        846, 1065, 1266, 847, 617, 848, 1190, 987, 849, 1191, 988, 850, 1192, 
       989, 851, 1193, 990, 852}], 
      Line3DBox[{854, 1066, 1267, 853, 1067, 1268, 855, 1068, 1269, 856, 625, 
       857, 1194, 991, 858, 1195, 992, 859, 1196, 1069, 1270, 860, 1070, 1271,
        861, 1071, 1272, 862, 1072, 1273, 863, 633, 864, 1197, 993, 865, 1198,
        994, 866, 1199, 995, 867}], 
      Line3DBox[{869, 1073, 1274, 868, 1074, 1275, 870, 1075, 1276, 871, 1076,
        1277, 872, 641, 873, 1200, 996, 874, 1201, 1077, 1278, 875, 1078, 
       1279, 876, 1079, 1280, 877, 1080, 1281, 878, 1081, 1282, 879, 649, 880,
        1202, 997, 881, 1203, 998, 882}], 
      Line3DBox[{884, 1082, 1283, 883, 1083, 1284, 885, 1084, 1285, 886, 1085,
        1286, 887, 1086, 1287, 888, 657, 889, 1204, 1087, 1288, 890, 1088, 
       1289, 891, 1089, 1290, 892, 1090, 1291, 893, 1091, 1292, 894, 1092, 
       1293, 895, 665, 896, 1205, 999, 897}], 
      Line3DBox[{911, 1005, 685, 910, 1307, 1104, 909, 1306, 1103, 908, 1305, 
       1102, 907, 1304, 1101, 906, 1303, 1100, 905, 1302, 1099, 904, 1301, 
       1300, 1098, 903, 1299, 1097, 902, 1298, 1096, 901, 1297, 1095, 900, 
       1296, 1094, 899, 1295, 1093, 898, 1105, 1294, 1003, 1004}]}}},
   VertexNormals->CompressedData["
1:eJx1fHdczt/7f2QlK7IiK6TsEeLWZWVFCclO9iZbIbJVRu9U9pY9MkMueSOV
9tCue9Tdzs4Iv5tzrvvzdX7vl39eDy/Hua9znetc1/P5PNd9t561fNzcyjo6
Os2r6ujoap5Hj0b1DtygAHp+d244+/dTZ0SH7w39cuDyz+t2zycrIbSg7ubf
zwn79nR0TcuA/psaPNjdQwXu79zyfz8fmU5vX/nbG3jucVAe+k0FC1JTGzzT
PF36fSi7vT8euj4/OjgwMBcabvZY+/tZ5jgpuvrkcHg/2jsxr3MemIXf1VNr
njH3B+0+/jAYkrZYvhm5LQ/2nO1Se5TmqcP+bFm73dnmxKk8qPLh2NbfTz4e
xfF8fhTn5/agaA+3H0X7+XpRXC/3D4r+4X5E0Z80jvxK4y/mTag2fbVc+7Tv
mDGr8XQVyHdYp/7rkQWlBdM81tgqYPnU4TZ7pubCt+/zrRtsSIWKitqro42U
cPxq+/rvRuaBZ8HELueiEsDrz1MJw1yNmhwyUkNZhF7PqDaR0GvS8LymU1QQ
YXDwxaonaijUueBbcuQR3NqysWuPOypYfe1TswOm+Vo/5yxYZD8iQQX9Nvc5
nGiTT+NRHM/nR3F+bg+K9nD7UbSfrxfF9XK/oOgf7k8U/Un7RHFK+0Xzkl9p
fkV9szVnDHO0T4qfgXrmd5tWpEG4dT+DA5k5ENWi+X2/Y2ooTnHKb90gCZrt
/eb3br8c9ta8vXqcXz6UF+9Yt6niNTS7c3tRtLECLBo6VS2dVgCL6xxPc9kf
ArpmZzr8joMOHsMzt2UWaP08ruaZ4C2HFWC5MDfdqGYhjUdxPJ8fxfm5PSja
w+1H0X6+ThTXy/2Don+4P1H0J50TOvd0XmhfKU5pf8kO8ivZM752rw8vXTK0
z/17V4/dp8yHQ7D/SpR9MhhfXrh//rNMrX/0TFYNOhoXzZ9ZsHFVXtMF5YVg
8G/TscUeyJ/ZcKVZpxr7fYq0fu7dcrBdyNNseFD3nfr8qyIaj+J4Pj+K83N7
ULSH242i/Xy9KK6X+wdF/3B/ouhPylOURylf0bmic0/ni+KA4pTigewmv5L9
LoXvemWvfqN97m1wa8KN6UUwp77MbO+gWPBrljbAblYK1FI51gpaWQyWqXbH
ql9+Cskv1i3S/5UCjv+u8Pc2K9H6eVWtFo2/WqTClMrr5GcWltB4FMfz+VGc
n9uBoj3cfhTt5+tFcb3cPyj6h/sTRX9SnaC6RPWC8hrlUcpvdA7p3NN5pLih
OKX4oXWSX2m949yz2/T7GKt9vpliUmXelRIw2O8W02FxKH/GQZUxpnd2uZVq
/RzdQOdl9UNxoE6wUkU9L6XxKI7n86I4P7cHRXu4/Sjaz9eL4nq5f1D0D/cn
iv6kOk11nvIP1RWqS1RfKA9SHqV8SOeWzj2dX4ozilOKN/IL+ZX8s9mrRsHy
4lDtc+WQijbPp5dp/Vz00tW0idEzeHJJ9+Dla2U0DsXxfH4U5+f24P9nD7Mf
Rfv5elFcL/cPiv7h/kTRn4STaB2El6iu03uq71SH6D2dL8qb9J7yJ51zek/n
neKS3lN8kh/pPflT5z//vKXxSG9oPJ9f+57m5/Zo35M93H7te7Kfr1f7ntbL
/aN9T/7h/tS+J38K8YwS8YwS8YwS8YwS8YwS8YwS8YxCPOuQXyXiGSXiGSXi
GSXiGSXiGSXiGSXiWetPIT+jRH5GifyMEvkZJfIzSuRnlMjPKORnHYpTifyM
EvkZJfIzSuRnlMjPKJGfUSI/a/0p4A2UwBsogTdQAm+gBN5ACbyBEngDBbyh
Q+deAm+gBN5ACbyBEngDJfAGSuANlMAbWn8K+Bkl8DNK4GeUwM8ogZ9RAj+j
BH5GAT/rUB6VwM8ogZ9RAj+jBH5GCfyMEvgZJfCz1p8CH0QJPogSfBAl+CBK
8EGU4IMowQdR4IM6VJck+CBK8EGU4IMowQdRgg+iBB9ECT6o9aegb6CEvoES
+gZK6BsooW+ghL6BEvoGCvqGDtV5CX0DJfQNlNA3UELfQAl9AyX0DZTQN1DQ
l7Q6k4RehxJ6HUrodSih16GEXocSeh0Kep0O4SYJvQ4l9DqU0OtQQq9DCb0O
JfQ60Z8o+vNEA32b3Tr/e9onxjXXm60E3Oe9rXWhAoau1L015IAcHhUobv7+
Ox+HEuNRYrz2/0nMjxLza8fTOPocGk+fR/MJ71HiPYjzCPZr32+5Mv30cPg/
zwD5mlHF2r+jxPv//ft/j9e+jxrxo++7Wwo47TxAr0p1BYz5Ejn3//wd6d+F
9/T/kN7T3+n/SYwHifHie5CYB8XPfT8qrHBikBzk8YNPpk2Ww+KR4wrNrFTw
ecCjK10+ZMLRJS2W9vylhL0p+sd/P2Wtm0Y0P50NDWbpNrlvpDnfii8mHVvm
wuqIyouPJaRC3WU1mlfT8InvssAxv5+VDGZsqjw7HdamPL62NV4B+p3Hu/t/
yoVux97FOI5Kgt0/PXfemZgLdndvBv9+NvNffzzrUjLcPZz6NG6DEq7mGg+O
eJAHsvwo3dIZsVB/mm2kOjsXnPMf/fr9XPBlStzcjXGwOc556YoaKpg0/HOt
Lk5qaBh4t8gGXkKLRg+arB6RB4EWrWx/P8cYqp8lJ4TBmZOtgpqsVcEwj+p6
xtlq+DprymG3nfegXoD+tfZ782B1ZwPl7yfLDgdkop5/8t7sxR4dH4DlwvRn
6x6pYPH8s+/1u1He9pWJuvRbNj+K8x9i86A4jwGzH0X7RzL7kewfzu3vx/yD
5J9Z3D9zmX9wE/fPZO6fLsz/KPq/KfM/3hH8v4rtL4r7q8P2F8X9/cTiB8X4
6c/iBw2F+Pl3yvYRHZJy0CY6MnqNlRKKhxVV/H7y+EQxPnfseVtrjpcCDhvp
Pa49NguevTKM1eubC/vvXr8ccz4H8ipMric9z4Z+p3p0rZBp4md7l83WHzNA
1+H57ama+jP0emsddV81lM80dh5aNwXSZy7L7C+Tw8cJsuRWrfNhqutSo3un
46Hp6wE/hkTKoeZq/cVOBflQzXNfaZ114VDyOOLIuH4KmNCn/si07QWwFr61
m64IhlaDwtRNtypgc39nu3fqAqjE42E81y36cby0hI1Hcbwumx9L+fwOfP7J
zB4U7fnE7Eey/xO3vx9bL1YW1ruP+QdF/3B/oujPbRMW5E2YpNR8/vCg7jap
UFq55q3I1nmwIGC3U/ZyOZwLK7cvPpkGhsn7fpXaqCG+5G3uqjVZcKvuEn+T
knSoP7uuk/vyfNDr9S337KJUyF5dJpe1zYQdYdEed90LoGDQ0c86zxLA6tJt
5xs9s8AjOOGO5VIN/9rjdVfZKBIOqFYdO1YjG3zWfRu5x7QIlgZHnlke8Ai2
m84pcTqYDX08irr3Olmk9XMfrgMFc/w/m41HcfxSNj8eFOZXM3sQBHtqMPsx
h9u/k9sfx9aL4nrnM//gecE/HsyfOIX7s4z78237T0u21VTB8u7PdDYFJsC6
j51q1y/Ng4e1B84+1kaT15f7jN7wIhEm+eKCB/3zwfPHpvnu0dnQ/ov31u05
SdCgfaDZ2x0FMO/hnCUXzqfBTF1j0w9FyTDGrO6QdyGFEBB4afaMiESArLVN
3r15A2ezPXsNjSuC1hU9I9D/NVTJ+Oyt9EmBK9aVGjpdKYZdEw0zPbqFQMcf
s6Lk+qnwuUHg3V19S7T5TdTPN7LxKI43ZvMjzX+Vz+/H7EHRntnMfhTt38vW
i6Z8vYZ8vdw/KPqH+xNFf9ZKnVUx2E8FB+aPxVdFEXBkQ3q23T41jDNfV2x3
UwGtDrSZZef5GvJfPbyV+zVfgwveTPJ2yYHpdYw8T5lHQ43x3x6MnlsIUdtb
xH86lw7VLtaoX34vBjKGj1rZ5GkRhOQ+exd9Jgku3E5XVfOKhfGX1Q61PhXD
pVaPSnckRcGtmau+1W0aB5nndKwyikpgSJU4Y7dtT+CI0awx71bHQVT42QMX
vEifCZCJ+nkfNh7F8efY/CjO/5DZg6I9kcx+rCHYz9eLtF49vl7uHyT/FHD/
6DN/4kHBn+WTrHdHl6lgTp8pEW2dH8HsLq6LdD6pYdTS723f6Cth7dzKx0Ma
h8D1tnPbv1xdAKM6dN9zNjIHRholu1XEPoGMJ987ZX7W5AFZtbMXR2dA8DvT
Ey6yp2Bpk90kbGEx1BvS9UajHskwqdqjxRu/PYWAsBDdKfdKYGulf8qHX4oG
j4oN1zr/Ewpn5o22PBBaCi7rxvovm4dQM/amk1F5KOivjVm5bEUZzxsBsmKu
+yLXG+ew8SiOd2Pz41Zh/jrMHhTtWc7sR7K/H7efrxdt+Hoz+Xq5f1D0D/cn
iv50VHx9eKNOrpbHRMREBC5qkg+x1aJG7O2o1L5Pq73YfeLJAlA4Wvf9Xpjz
Pz370IaDO8yKQKHrZWa9K0P7vnr9K7FG54uhfsgYF+c5ydr3Jttb1uv/rQQy
v+0KkH+I1r4vCxzfPKdWGeg9DesdcA+1761Oj2uc97RMG886f/15Cz/wz3gU
x6ew+bXvaf56zB7te7Inh9mvfV+D2y9n69W+r8TXG8P8o32fzv0zkflT+z6S
+1OIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SIZ5SI
Z60/hfyMEvkZJfIzSuRnlMjPKJGfUSI/o0R+Ron8jBL5GSXyM0rkZ5TIzyiR
n1EiP2v9KeANlMAbKIE3UAJvoATeQAm8gRJ4AyXwBkrgDZTAGyiBN1ACb6AE
3kAJvIESeEPrTwE/owR+Rgn8jBL4GSXwM0rgZ5TAzyiBn1ECP6MEfkYJ/IwS
+Bkl8DNK4GeUwM9I+FnggyjBB1GCD6IEH0QJPogSfBAl+CBK8EGU4IMowQdR
gg+iBB9ECT6IEnwQJfig1p+CXoTExwVdCInXC/oPkj4g6DxIOoOg5yDpOYJu
g6TbCDoMSugwWv1c0Fu04wVdBUlXEfQTJP1E0Em09gt6iHa9gu6h9Y+gb2j9
+ZzpITCK6SFIeoigyyHpKoL+hqTPCDobks4j6GlIepqgmyHpToI+hhL6mFY/
F/Qu7XhB19LOL+hXSPqVoFNp7Rf0KO16Bd1J659/AhIz9QPk2mf5rO8L7Vqq
wH5LpZemPbJhUJ3wpSN9FeB6afOPtVkq6Nz88heL52nwz/d5prt2K2HcZ4cI
1525MMp+t29qYRL8G71/wpNFKnjgcWjdE4M86JayxmT6vVj4EDezd+8eudC1
1Y9DY9bnwZRrFmNCNr2EO+ueVE1PzoWzO5fN/nE7DwoGvNvlsPse2D9WyEaO
y4Ma6zF1Vej//Cnqh3NnhZ7f3PkB5r3cdVz1WQULp4Zc8a1Qw8jCgMHqo2G4
6PEMwxslKhjy5aKOoasadBSBmUrzOAyat7H9khAVjLfNf3c1Og+iDaOL5hkk
4+t2E5cPW6qCOtMvPHpVNQ/yzALyB7ilYZKNQ8TNKiqwKUp718M8F7K22E4t
X5iFfUsLvQP3KqHu8MsLWlqrYHrzmWbHXBRQ23fXt5S0LJg721k9xjgXVOub
jYpanqN9du8zw8PgZy78061WVCvHDDCQjTjSdqkcWhq/PVU7Mg/WrDLJ7m33
BvIvPOznPU0Bqirj3G03qMFopPuPe2/iIEhl27x9L018L217Sfen5txdn73g
Y8NXcH1IHf/uhUr4d9AHQ9nYfBgy45fapMsD6ONd0nzpWhVM+9lHvWNRvqT+
dnbHhMLOucFobufSb3GEAsoTjs8oCCqA9p6hiU2swvGdxwuvqVEK2DSq/U95
c03enpBmesw2HntstHZrflEB6+JOrNvkkA8LglYmpq9+g6//PR3VepYCelm+
dvFarIaJwWHbC/QzsP588/6DdRWQvlemTl2SBz0Nzr26qJ+DgdX0lmX6yOFm
s9dW8jm5oN/Jc4xFrBwdTvU95NxYDtOeLNy7dJoKEg/e3PwDlJA6yXaALD8V
pg16X2WfXh58e9yorGyAHEoL3Ou7qdKh1lRzHbmBGkanHp5zSJ6pfQakFylH
18iHlXteTQ4NSgG32nqFn6OyIXTF5jv9s/OhdogJrP0RD0ZBYetGRuRAa/9S
l5W7C8Bp+LGw8w/D4ea+7Mzd++XwtVuSakZ5AWxMue5SKy8YejW0sXLrpICO
VSJ77DAp1Pq5r6C/HWpodXDasUdY6chnnR5tcsB+mVuGbHERbBhmvKc8OwIz
xnVM69gnB2Y427+7GVoIPSeHjHJxScCo/rs7T2ubA7M/Kus1zCyA6ad1ujxN
SsHRg/ZPOiLPhhqHHZP7xObDm8y2epv8M/Hjiw9HnUZkw+j82gPnXFVD4YDq
8gFZOdhm1MOSeM8sGGWZOvmuex64udU65XtRgVsi5hh9fZAJU8c3rXl9Qi4E
uG63NPyshNeyu3f1JiaCX/LYJ9GpeeBzreBjcbkcXJ2nZLbsmQx1Sl1irGvm
w5HwwV+DxmTDhuc13Nc1T4GUla0WjO1UAHOCNvmtqJymfXosNL6f3LUQbLZu
soxqkghREZbZm2PTYfSE2CqllYvAaWXQ9uGTIsF6aY7d62WZMKh7mGWgbxEc
L735c+XxR/Awf3X0ihdZ4NA28OeSlCJtXVst6G/f1k+fFWMRgjXeWNq2UKRC
xwP7TU/plcChJ0Flk+dr+MD+W3sODEgD43sTiobaFUNOM68AR49EDIS3PR/Z
pkEf1eO+NnZFcPf26GEhrdLwPnvCwvyi7SGtCiF0xMDAq3rZeKIDVtIPTYX5
dzceupmSD430a3YKOCPH4b4jhswakAr2T9pHNN+tBpvDDrFLXJSYsaqFxaAz
KVBtll3P2f3z4PIeHAy7VLDf29zCdP5riB2X9cLbXQ331D/8bQ4oQD2r0Mvh
RgzIr3/WN0zKh6aHa9n0HpgD65dkfjL2iYNzO2q80WtdCPpJN7z6tUuHxoVx
n40dE8CwcWXbOfZFENzKcuUPwyTtc+oG/8pRY4vhwc9OpY/iXmtw8oijYXeS
4ZPViCpBGv/cr5hiVqVPCOSue5Ku1uxj1IDPvbYv+h/viBH0t+wJkZnD9j7B
G1+GKxYsjQeXdv0qvs4vhS9RuH3grShsdr313oPVEmBAz2+pNmdLwLG83vjw
RUkYNLnui+MzE6Ah7qoxa18xBBqN7vpYlo5v5texabI1AcLyH+w2GFoEr1nc
Yy92DrTxf8Kk9/jOoxU4KfbCUJUsAWLzZ4faLNXg5NnlKZ9aqPD1+ANuG0rj
4VzjLi30ddWwuLrP4l1qFfgtC5iSpOGve36NbFC/RA2jVrk0PvRVASMOZV3I
zA2FdurkTr1mFsB6sxnz9l/PgXDPDp6nnz+HHf9Yf7COKYTZuxLMPvxMh4wV
AYH14sIAb1Tr19+0GGJu+2zvlpoECSfLW8reR4BBnn3WwLElUMOqTdLXStHa
p91Qt6mXu5VCjdNDP87w1PBov8Ue6vmx8G3VmI33b/2PR5cK/atfAowtvJYh
XuvWuqWixXOYGHB60+ypZdBrmKvlc79o9K/9wm9+6gtQn4rweO5XCptSjLyv
aOqasUPtBFvLMBh3YXnW7vUlsG562ov02hl4b97uPZGWr2B0zyVtS42LoajD
3bqtj+XgrJMvrGfJX0F9h23t950phLcs72MnVge0+X/sRrPO/udUOCH8/ojr
M8OhSr+qp8yPqyE1yNP1V+1caNCmDJ2f7oE9a7y3dGiiqQvpQ/YbmynBwG3N
xcTAo2A0/uf2xccL4MazIcERuTlQe55PX5v+F2C/ff3ICyZFsCq5yfnWazPg
ZUh/vzUnb0CjnjKnPP9i6H7/q/XEMcnQQufhFYuCO5DZ4N1LO3kJ/FKcijv9
LBo63HJ0Muj7EF70WGPtpSwF7wfq972Xo/bZd+qPSnu3ifobPd9CfETTs2sR
MdW8MGJIWx8oHdi9bC2WgdX9F6lVyqOxzHFtrcKK02Az9fyjddXLYH9hcWRT
52Tsetd8WGH/y2DwxNJialkJZAX13TnALQP/fWjh2PjhTZg3qKqFp28xLK7T
s/v47BwcfXvwnKynt6G3xc6alQ2L4OXXwxjaVImt1wY5ZvS+B+4tzA1r7yyA
XIZvcCXDO1qcw9/DEo6D6D2fB4zYPLiZz8M/F6zZ5yJ9rpzZCfeZnUh2+rB1
QVO2LqR1DWV+gFTmByQ/ZDG/wWXmNyS/kf5WIOhvW9l+ID1pX76wfcTmbB+R
9rEb23dswPYds/i+u7A4wZssTrAxjxMeV/hr7p+4Qoqr+SwO8bvrnzhEikMe
t1jc+k/cIsWtPYtzGMfiHCnOCQ+ZcXy0mZ+LYnaOYCY7R0jnyJWdO7jFzh3a
8HPnzs4pNGbnFO35ObVk5xoOsnONdK6rHP6TB+AiywNIeYDi+LWgv/049SfP
oD/LM/id55lKLB8hPSkvRbM8htEsj2F9nsecWd7DZJb38CnPe+tYnsSXLE/i
Lp4nR7K8ikNZXkXKqzwPow/Lw0h5mOdtiGB5Gylvn2R5HhxZnsc4nucJDxE+
orpwidURSGR1BKmOTGF1B26wuoNUd36wOgVNWJ1CqlNFrK7BNVbXkOoa+Xml
oL9dZXUTlaxuItXN26zOoozVWdTWWVaPkZ7TeF2uyeo4NmR1HBvyOs7rPq5j
dR/P87rPcQKqGE5ABccJHFegN8MVSLiC4xBIZzgECYc0ZrgFhjHcgoRbnjGc
A8cYzsF5HOcQHiJ8RLhIyXAUnGM4CntzHHWE4S7IY7gLCXdV3fAHp0E1htOQ
cBrh596C/ubLcCAGMxyIEzkOnMxwIw5huBEJN45iOBM5zsQxHGfOYngU6Um4
9DDDsbie4VhM5TiW417cwHAvEu7lOBkjGE5Gf46TOa4Gd4arkXB1EcPhwHE4
2nAcnspwO7xnuB0JtzsxnA8jGM5HwvkWHA8RPqL438x4BHAegcQjTjDeAb8O
/+EdSLxDSn9byXgNcl6DxGumMh6ENxgPwm+cB+kz3oRNGW/CNpw3uTCehZxn
4TPOs2wYH0N6Hua87Dvjcch5HBKP47wPUxjvw+mc93GeCBMYT0Tiib0YrwTO
K5F4JeehYMB4KBIPXcx4K0Qy3ooWnLdWYTwXujOei8RzCQ8RPiJcdJXxaOjA
eDQSj5bS3ywZT8fejKfjdM7TGzJej5zX43PO65swHQBvMR0A13AdYBXTDVDN
dAPM5bqBD9MZkOsM2IrrDFyPQHr24LoE1zGwFtMxcB7XMVZ0DdnY530OlCgi
ksyqKtEyIKxl/HQlZDM9BPowPQTrcT0kn+knkMj0EyT9JI7pLRDJ9BYkvUWX
6TOa+vhHn0HSZ8YwPQe4noOk5yzneIjwEeEf8rOov8mZvoRcX0LSlyYyPQq5
HoXnuB7VlelXyPUrJP1qBNO7kOtdSHpXJ6aPIdfHkPSxsUxPQ66nIelpgj+B
/MnfY6nwnut0SHrdF67XCToelv/9nsYjjRfm1+6j0G+m1VcF3VXbh0b9Z6R/
k05L4+j/0Xjqo6J7CdLPqY+K7oXoPoj6VOheju6PqE+F7kXp/o76AOhemu5P
6d6a+gKoH4DuWcXxdM8qzk/3WKI9dI8l2k/3BOJ6SdcW/Ub9o3RvQHo49QuS
zk36LfWr0X0O3VNQ/xndp9E9EfUD0X0m3dNR/wrdJ9M9MvVb0H0+3VOTP6mf
gvoo6D5bHE/3r+L8dF8o2kP3f6L9dB8jrpfuVyT8hqLfqC+T7mHoPoL6OOne
gPRz6o+k+zG6J6L+NrqfpHs66j+j+2G6F6b+Ibqfp3t56u+h/gjqiyA/a/tT
eJ8G9U+I4+n+W5yf7qdFe+h+UbSf7v/E9dL9jYTfUPQb9SvTvRbdB1F/Ld3D
0P0F9WvSfSPdM1J/JN330j0v9cPRfTvdU1P/GfU7UJ8A9RtRvwn1mWj9zPt9
qM+H+jnE8dQ/Ic5P9+WiPXQ/LdpP95Hieun+T8JvKPqN+rzpnpDuB6nvme61
6D6L+mXp/pbuSanflO7P6Z6a+hGpf4H6BKh/jvpHqE+D+r2of4f6ZMjP1D9F
fUrUTyOOp/4PcX7qVxDtoft+0X66DxbXS/epEn5D0W/UX073rnTfSv3odE9I
93fUD0334XQPTv271I9AfQLUD0r9INQHQv2L1I9DfTLUz0f9UNSnRH6mfjTq
E6N+KXE89d+I81O/iGgP9U+I9tN9v7heuj+W8BuKfqO+f7rHpvvov7//7isj
/EzfH6D7WLpX/ft79AdkhAOpv5z6EaivgO7DiA8RD6I+aeoToX4P6qum8cRP
qd+X+neoD+fv7937ykg3oL5V6qui/qi/v48fICM9h/ovqd+N+taoX5P0NhS+
p099iH/3H9Kf/+mf1A8nMT+K81Nfl4T9KNpP/UkS/kHRP9RnI+F/FP1P/SIS
+4vi/lLfg0S8ae0R4g0l4g3FeKN4pj4Lmp/ilvoCaB6KT+p/IfspDqn/iPxA
8Ub9X+RPiivqv6N9ofih/kfaX4oE6j+lOKF4EMfTvovz0/6K9tA+ivbTfonr
pX2R8BuKfqP8TH0rw4X8TH0WlGfo+yTUT0R9RPT9B+rnEvMz9dNdFfIz9TNS
/qR+aOonFfMz9fNSfqZ+U3E85WdxfsrPoj3UfybaT/1S4nopP0v4DUW/0fem
qA9osoA3qG9lloA3qD+L6i99n4T640S8Qf2JIt6g/lARb1B/rog3qD9axBvi
eMIb4vyEN0R7qJ9PtJ/whrhe6peS8BuKfqPvm1FflYifqQ9IxM/U70Z9bvT9
HOo33CngZ+r3JLxK33+gflsRP1O/cz8BP1O/eQ0BP4vjqX9XnJ/ws2gP9UeK
9mv7+YT1Uv+ZhN9Q9BvxQepTE/kg9VURr6Hvj1H/oMgHqX+T+BR9P4f6Zw3/
mw9q+5Zt/uaD2r5x8jP171Pfvs3ffFA7XuCD2vmpf1e0h/igaD/1R4rrJT4o
4TcU/Ubfb6S+P1HfoD41Ud+gfkzi+/T9MeqHldA3UELf0PaBC/oGivoGfR8i
/b/1DZTQN7TzC/qG1h7q3xXtJ31DXC/1R0r4DUW/Cbqctn9S0N+0/av0vTvq
Iy77b/1N279N32uiPnoJ/Q1F/Y2+RxL53/qbdjz15YvzC/qb1h7qexbtF/Q3
lNDftP4h3ZN0UNItL03pWy8nLQtcKq35fsJFAXeVNdJGGGv4oKHdJtv8VOhs
buajO1AJzh8bTd6mlwd5R+qtMp+YCPnTzTONPytBaV6gh6l5MPaQ1b3t81/D
hJWNzEbvUsEZ100r3NzVMKk9Dg/c/gSaqyIfH1Kr4J212rlSiRoWS/Q31h7v
UX3UzHB892r0gcBzGj40eUsNk+Nq6HU2qnu70ni8OLl6F8OWKvAbMq60kq4a
Ri+vvVd+OgUvtX+d7+OiBGhvNmta/zy4MzZ73tkHmdirYtTShxcV8PCwX8Ll
Cblw0rn4/eDGckxwqdp9fKwcmhc3bfb7PoV0f9KtSTde1sOhm48qHVLUVzd8
HSCH7eo0ebyBGryqGSU69kwGr6LvX3S+yGGoiaLErKZm3x9XttW/GQOuG4aP
WXhAAVtaX3IsS9Sci2qnwh7khUILsx9H7n9VwKNl9+7Um1kA5kL/Id2zd6rf
w9lNloCFz8/ZTBytgJt7B90ftDQfrJIC9V/JUtHbxWN72hk5rGjdbLHhbjU4
xY9bJPPMwgDTz6t2ZOXAjGoFL4LcNXXB3S3/vI8cX+w0NFfq54Dc0LOeck4u
fLZ+Gei5V4lDHxc37bsoCzY9vlaljbUKutu3OHeteQoEbjs8Wj4mG9R2VWpa
dCqALV+O6/j6xMHDf3QCZg/MgWs1veNSWxVC+/5tghUvnsP9kFa7wq/nQPIa
y371YwphTPVdXh5PU3Gm1/llHWtmw5BFKyZfTcmHj4ldRt4Zno3fo1/c6xSQ
Cd1mHHs396oaFLoG535UVuBKl63NdtTKgG01Tk7KWZIHHqZ2Q7yrqPD9voof
fm5pMOix0/W+5rkwcu+hVdscEyD+xOPtvu3S4VvY1h/97YsgXC/5p3t8GMxd
M3Xo3l/pkNG6hayyaTEU7TZ0NJZnY8zFnw1HJaeAadPubgNi8+HpUefdcmcF
njduVbZ+zRuwbrzE23+xGq4EnB7aZqkKweLg21cGyfD8Y9XguKp5oHxmtHPq
hwioMnj4hjupSVBQrv+88dgSGF9F0fdjoAI7VvoVVdUuHm6W9lm72yEf2g4b
+NE2RIVDgg0tLTvGQdfy2cvuR+dBxoyKaidLVHgqteSDzbEwkHuHfG/mqhZ0
gINaXvY3f/fR8ru+Ejx99V983E/LN2P+4t2+Wt5a+tfv4AXIqC+rQIIXR0nw
35V/8dz/fW7vv/hsgNZOKd4qxU/pPmyZ0C/0Nz/11fJTnaRBPu4RCliTJi+3
UgWjx7G7g7KCCmDZ6QUZs9rkwJ7Tffs6HXmE98a1atVhcRHcrz52ZagiFc5b
R2xO6BGCHb2nRO3SK4FBF2+9t1oWD/Zz6kfc3PkElTqjPqrnl8Ld6YnrD7R6
Dp2vGncwXYw402lZstPvfpV2VXqsmh+Lu38U91i15wlWHlPvavStUtiXdM7Y
vnkKGpyrEz+yVwhu6uRWem1RCRSaqlNLn2fhkJUlVV5r7LHy6rboaEoRLJ7m
8s22k6buvr7xsVwZjIkd5vW5alIIvdOSgidoeI1T5ZAXUZ0e4N2hG8+cXJQP
6GN0HcbloUdoUYMHu+5hYLcll9xD88BuQnlV5z45kLIo7HBJVgQa9Ry28HRo
Idy03jhijVUaDF65a4vD3NfY49LXkd3siiHiqW/PN9UToHjJFYsT16Owmvx8
5KCzJXDqe5fPNTJeQPPun1da+Ebjqg3fp8b4lYJhxaqR1neScfMJm5MnYl/j
05uFXTM1ftvZxfSL5bJM7NYm1SNjYiQahshDVb5F0OHCj1/2v38Hytar0cLg
cOzg9/Cof3kBGPiGVqlfqMSMztGrmzd8hQcSN9vNHpsPn0OHzotOzsVCm64x
DpteYvmeYW/r3smDCTG6m2fbpcGu4in3z25NxJ5eRWuG2hXBiJNOpbHOCaAT
PWnCsYVJ2DPTc/K8fcXQY1jD3J/9w8CnILz/inrJ+GVvp1+n15fAZvOTHdrH
pqOVfbdzC5okYvn1QQca6xaBQ4cDQcYRORipSMut9CMe73csTj+3uwAsjvua
1uylRMvbXS9PfxOHb8edO2f+Uw1662asM++RiwmX2/frei8WI9dl9XRenwdL
rxx7E7U1Af7Vb9ywkSwdw6ov9TQaWgT76r12Se7/CtrE3HX8USsDfb0WrdFr
UQzTPlsNCY/Kxs9G8mDPoBRcOHfuMcfsfCh56aneME2BG2avef7L9g2+S8oa
tXiDGj4/9rS+u0iFpx0urwguTMLA2OCEaIM8MBr+9PZSxStwf2r+ovioBndF
fGp78kwhnGpu2rPZUjnO7eier+eYgS6fPpU2i8yDGx+cRm7drcQ+tn46bZ+n
4Qjjmbd37MwFnc/554f5KtB/vWNoqx7Z+L79rUYbs7R1E8S6KXWfS99vEe9P
S/+eR3tvS78PIuKW0r/rtXb8XPfIbxNe7IH7t3qdrVknF6p0PrS4hQaPcfsh
gNkP77j9Tr75Jn2uHYUxu9Z/6mumBCtL+UGH4wWQ+/OqOrLPPQz+JzTqWlMl
XB9dGZvsLCD/QF/mHxjO/XOS+RO4P4H8edFnae5GuADPK+q8+JabA7OvR63Y
YVIEAT0/+GU9u41njD/+MsnOAat5A482NSyCpmy/kO8X0H7x/YWzbH/hAt9f
Hg/gxuIBKB6ms/iBchY/sIDHT356cmb0uRvQ32n/7strNXZudp/9xL8YLq+K
0DUKuYkVSfYjTrtmwBfDY4ZXfIthD4tPbMviE/7h8bmAxTM+Z/EML3k88/iH
RBb/EMHjn58X6M/OC5Tx8zKRnS+IYucL6HxtYecRBrHzCJ/5eTQrTW/yqegO
9Bzer/e7McnwM/tMx45yTf29kbil05DLuHVjkWL0zGS4vnmYiVtZCXRm5x0P
sfMOdN6tWX5AXZYfgPLDOJZPcC/LJ0D5hPqQLIQ+7XKWl6CU5SX4zPMSz2OQ
w/IY7Od5zJzlPdjP8h6Y8ry3m+VJsGB5EhrwPNmI5VXYxvIqIM+rI8Bq6i/L
hzAqbMA9r3+joUuCcdN1ylLoFX+8TZTuGfQ7OrDt3E/RMO/hpUve1cvgCMvb
2IblbaC8HcbyPL5neR4oz19ndQGHs7oAPXldsGV1BDNYHQGqI09Z3YG9rO7A
BV53+rA6BQtZnQKqU0tYXYNPrK4B1bViVgdhDKuDMIDXwYOsboIxq5tAdVOX
1VnwZXUWKvE6m7vsWK5uZx9s2vV8dbsQhN4jK4/ZjGVwg9Vr7MvqNTjzem3F
6jtOZ/UdVLy+32F4AG8xPACdOB5YyvAD+jD8AIQffiX+wRvozvAGbON4g3jO
PM57FnHc0nj5HzuhLrMT+3A7Mxk+hGMMHyLhQ+IJpkJf7ki273CA7TvSvstY
nOBQFifYlccJx6UwmOFSJFw6geFY6MBwLN7gOHYZO1/gzs4X3uDnqxU7j9id
nUf8xc9jFsPJWJnhZCScfJXhahjAcDX+y3F1KMPhcJbhcCQcXspwO0Qz3I6E
22+zfAXfWb5CylcxLL9hX5bfkPLbc8YLcA7jBUi8YDjjERjHeAR+5zyC8w54
x3gHEu9QMp4CKxhPQQ/OUz4zXgNfGa9B4jW2jAfBDMaDkHjQSZb/4TTL/0j5
/xirF8jrBc7h9aIt41nIeRYSz9rMeJmmTv3hZXid87JujMch53FIPI7zPhjM
eB8S7+M8EZ4znog5nCfOZLwS/BmvROKVnIeCF+OhuJzz0C6MtwLnrXiD89ZS
Vk/hAaunSPV0GKu/yOsvUv09w/gycr6Mjzlfnsn4NXJ+jcSvPRkfR87H0Zrz
8SWMvyPn70j8/RTTASCe6QBIOsA9phtAT6YbIOkGdkxnAK4z4ECuM1gwXQIC
mS6BpEsYMB0DuI6BlbiOsUair28gwy3IcQsSbpnA9BM0ZvoJvuf6iS3TW5Dr
LXiW6y25TJ9BNdNnUMX1mdNMz8FOTM9B0nO4/oNc/0HSf6T67rxz0mM/L8yG
Qd3fnbjlqIDvlvIDer9UkDs4Y9bVLunQuSLW+nBdJSzpc2Fbjbhc6Pe6rp7c
NRl2WDi7P7inBBML59uR+/PgliPoBzjEQdo046elQ1Uw0Kjc0rCHGo4MGfOq
l4YPG4ZdGmEUpAKH43Evzl9Xg7lnRJb9u/vgcayosUG5CmK8t/V1+qGW/J7s
KaMWnu0fhOE3I8cxbTTzVGT3KsrWnEd//9eRZ5zj0H+/U/kATdybmzwz2ddV
DRUdp5Tt3p6MSz/2GTTjvhJOpW1aEO+VB5O9Y18pLdKxZdtlPkfrKSFsQKe0
vKhccGiV9frbsmy83tjMX3+yAmTTNp/79l0Fr8yiwkZ6ZUDoqtxGVdrLodrT
ggs2b/NAP3Cm+ZGbbwCzA9ctipLDXiPz0kPRanBZe6T3vnHxcG1KzY9Dpisg
5p3j0GXe+WDbYP2xpVXCoVPFg5l+rxXweP1gv3qtC2DpwzkWNxYGQ07Ey/m2
DZSgN+xNz7LNBdp6IOpIxaXLjXIWxWO71ZkD6zopIE9+d+GeXfkw/GjMObd/
32D4upxJECuH163XL38WpgbvU2rDyMMZGD5//YjXZnLoYjQ5bFBBHrQzUHaw
XK3B3wHJydNW58CnOSbVJpfngmPB1D1fJylwcAenLJPl2XC939Ra3zV+uJXW
/Ww9RQr8MBgzyqJdNswxXqEIqlEAyrKhR4fbJoCbybCgdi1zIDBt/KSCvAJo
1fzEELelERD0Jsw82ysHsmJebevuWwjjW3+5F696CEs21v36PT4Herk/W1FP
gwN7fYv3n1uRgjNqF3Wo2SsbGqwOiNr2Ix9sTllVdf+Qif66QYq8D5nwrMWd
YF1NnATJ6m46YybHEPNIu45HMsBUt2adIZp1bTxn/GJHPSW2zJ2nY9w7HYbl
j62Xr9nf9rX8jS2tEmHV4xTPR9XTweSf/tOfdSmCe8uf6Nm/ioRH03qaP0hK
h5QD/ifblhfBTotWFl/HPYY+HpjZZkoG6G1v/8l0fTFcaDVgwL89s3F+kGW9
BxUpMK7Ye/wOjZ33dlmHG2nyjn6H1OLMf9+AZ8mdoS80/nes3PyfkfeV+F2v
9fVX25NhylnHfxI1cXh/dEbnxZWiYMa2nTfDXyXB+ilbBkX3LAGbZafGOp0M
gdAF/3RrPCAZDli2u5zwsgTM96Y5fpuhwLG7Bsa3WRwPir53vbw0+167h0Nc
F2sVfjl2+G2acxxEnFiWcUAT/9UixldacOsJPH43LS7gfjQUfJ684FFCKXiv
71erWZAKVxS4jl/4IAyqnjAwVFwVf4fqoOy/f4fKR/bfv0PlK/vv36Hyk/33
71D5yuh+M4v3/ZCO9PY/f4cqQEb3+3//3tT/3qfy+2lxHgN+n1qJ29P2P39v
KkD237835Sv779+b8pXRPYuUfvX371D5yugeweJm7YVWmnPewb764LzAYDzh
cLWwkua8X7b1cDdKyIH964pazW/zCLftf2VQvXIRTDLZa6bWxN+kuPa1oj0f
46TudysbriuGrO3LnPdYJcOhpTXDakWH4NI8x16JL0rg2fvRKyseRcOykYsN
TGKfYHjk7Jff4kvBaPleo5OuCCue17LdtgExac3GST23lUFTj66nO96PRnXD
zUlLbj7BtfoOxi81cXJi1wunJFkypkxpvKH0eAh28dx+I1cTh7PbG64vm5yB
D6c2fbPF/jFOb+R3pJ/mXOitnXK4OD4HN1Y9LzuifIjtrjjntNCc68VT7AcM
b6DE9q5HG6QtCEane9UV5Zr1Ol40SqxTrsIdDjHFBW/vo6pn5/hZmnPtmrpr
Q9i+HPjWfHGT+PMRGLe10tuN+wuhaW9fv+MZ6RC1yPzS4e+RGLfskk+/90Xw
41ndQZMSkuC7T+M6n8yj0NpdvQS6lkBCszrRY1tEw7mZK/eVNI/Gj0WfBlp2
LoVK/mqnkrgncPDGu+I2j6Kx6OCJjpU06+1Q78T0f14lYYXdzmbpOlHY0Wrb
BLnmPMYc6pu9LSkdy6PnrDkYFokX3l/fNUCTHxom1dkW45WD0bU9T4QviUDD
/iPHDNPktw4/W+gceK3A4FNtw4p0w3HllsPLjTV53uTS/vwmmnPn0OqZ3vuj
YdhcZtrmiqYO+o57aleraTrcOrr6l9P0RCy2WF58sm0ROMh+PurRJQlumIbP
2Nk5CZVNToduG14MVrm2r106RcGonYfCr8Qn4Wp1DcVwzXq7j2phXzk2BIbs
Nr/YwioZH8Y/HpWjiYfl7y/uOF49HX9uizhewyoRW11blR6vyXsXz9au2axl
Drpe3XlZ3zYBZ63e3+67Jm871R7lNHC6AhuM3Wg+bVw8hpwdMsJVU7962vq/
0dR1rPe6WbGbQxwOXrJsXlNNfT897eyUyOmJULlK1TbXmqSj/unpe89q7B/6
Km5IeEUkWO5r8KptRjqeNJRdt9bsl69Dqx9bvB9Du05tDvpPyUDXAw2qtNbE
s/OQhGDzdtlo32xT/Bd5Chok+jZ6rKkvV365PlsQJUe93c++b775Butdaf/l
uKbOJlc6aht8T4mXV43vEuOajErX1BUxGhwypXm3a6svRIBuwXufK/tysJb6
XJvtmvhZ4z25k5/JI/AMm2mjm5CDlYd2f15Pc77yW5n20W0vxwiL8xEDvTS8
o0U7IztNfT9i+viDBvfgqKZY41yXdDRseWtHTQ3+2ft5fozrxWAo102vYaGJ
a/tGJwOqaeI56EPOGw1+wuEnj2/5uDAbS55fktXU4KhXQZu/2Vc6IOP5ACgP
8PEwgo2HYj5ev9vFzk91fWWUdynPbGGfi/xzYSz/XG4n2DA7oQG3U83WBXxd
QOsKmTHwoEk1X5nYp72I+Qe9mX+gEvfPROZP5P4E8if3P1xj/gcF9z/fL6jJ
9gtov/j+wni2v1CP7y+EV705vYbGHqGfeS+LEzRjcQIUJ8DiCvuzuIITPK6O
sThEXRaHQHHYg8UtGLC4hUE8bmeyOIeGLM6B4pyfC3Bj5wKc+blYwc4R/GLn
CFrwc3TU6Gzn03q+MrFPuB07j2jNziPQebRk5xdHs/MLdH7HsfOOQey8A513
H5Yf8C7LD0D5QUq/4nkGJrE8A5RneF6CEJaXgPJSI5bHIJHlMWjA81gcy3tQ
wfIenOd5z4zlSdAZ+ydPgjnPk/0uTnR5WDNAJvbxfvL7k2/Rj+VboHwbzfIz
Xmb5GSg/f2f5HH+xfA6Uzxux/I8JLP8D5X9eL/AnqxcQz+sFry+wj9UXoPrC
6xF0ZfUIqB7x+gU7Wf0Cql9zWL2DF6zewTRe706x+ggqVh+B6mNLVk/hI6un
sIbXU53tNpn/VguQ1RT6WmuzuoxurC4D1eXHrI7jelbHgep4Bqv7eJLVfaC6
78BwAjoznACEEy4xXIGHGa4AD44rOA7B7gyHAOEQKV2L7P8p9FtyPAlLGJ7E
KhxPEm/pIOhaFCdiP+H38D94Fe8xvIqFHK9yfAufGL5FwrccD4Mtw8Oo5Hh4
V/C/vffpB8jEPrqBDFcjMlyNhKtvMxyO0xgOxw0ch09iuB2+MtyOhNs5zgc9
hvORcP5FxgtgLuMFSLzAiue3bI43Kb9tZPwCLRi/wJqcX9xifAQfMj6CqZyP
tGP8BVcy/oJtOX/ZxPgOGDO+g8R37jB+BA8ZP0LiR2MYnwI/xqeQ+FRvxr9g
GuNfWJ/zL6oXWlxM3+thPA45j0Picc0Z70PO+5B4n5zxRHRlPBEvcp54g/FK
5LwS53JeyXkoDGI8FImHtme8FYoYb8WPnLfuZzwXOM9F4rkjGC+GV4wXYyTn
xaWMR0NbxqOReLRQf7X9VLMZT0fO05F4+kjG67Ej4/VIvH4F0wHwOtMBMJbr
AHpMN0CuG6An1w3CmM6AT5nOgNW5ziCl2zgyvQKuMb0CB3C9YirTN4DrG0j6
xi+mhwDXQ5D0kGNMP4EApp8g6SfnmN4CXG9B0lukvsfq6tRiw4VKvjKxb6oV
03mQ6zwYy3WeQ0wXwvpMF0JHrgvdYDoSpjIdCQdzHcmS6U7IdSdsy3UnFdOp
kOtUSDoV17VwMNO1sILrWlL3mP8PF8nu2w==
    "]],
  Axes->True,
  DisplayFunction->Identity,
  Method->{},
  PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {-0.9999998571428571,
    0.9999998571428571}, {0., 0.9999997142857346}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5801885703198023`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"FullForm", "[", 
  RowBox[{"ParametricPlot3D", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "v", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", "^", "2"}], "+", 
         RowBox[{"v", "^", "2"}]}], ")"}], "/", "2"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.580189141234005*^9, 3.580189218688141*^9}}],

Cell["\<\
Analyzing the very large Output one sees that the Replace  Rule \
\>", "Text",
 CellChangeTimes->{{3.5801894478213434`*^9, 3.5801894674617777`*^9}, {
  3.5801895049486437`*^9, 3.580189519269469*^9}, {3.580189625427655*^9, 
  3.580189645848091*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"Polygon", "[", "pts_", "]"}], ":>", "\n", "     ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Polygon", "[", 
       RowBox[{"Append", "[", 
        RowBox[{
         RowBox[{"pts", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2"}], "}"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
      "\n", "      ", 
      RowBox[{"Polygon", "[", 
       RowBox[{"Append", "[", 
        RowBox[{
         RowBox[{"pts", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
      "\n", "      ", 
      RowBox[{"Polygon", "[", 
       RowBox[{"Append", "[", 
        RowBox[{
         RowBox[{"pts", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"3", ",", "4"}], "}"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}], ",", 
      "\n", "      ", 
      RowBox[{"Polygon", "[", 
       RowBox[{"Append", "[", 
        RowBox[{
         RowBox[{"pts", "[", 
          RowBox[{"[", 
           RowBox[{"{", 
            RowBox[{"4", ",", "1"}], "}"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Plus", "@@", "pts"}], "/", "4"}]}], "]"}], "]"}]}], 
     "}"}]}], ")"}], ";"}]], "Input",
 Evaluatable->False],

Cell[TextData[{
 "has no valid arguments; very likely the definition of ParametricPlot3D has \
be changed in newer versions of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Thus we restricted centerlines to Polygons."
}], "Text",
 CellChangeTimes->{{3.580189660636917*^9, 3.5801897682303057`*^9}, {
  3.5801898001791625`*^9, 3.580189802768767*^9}, {3.58018985003685*^9, 
  3.580189886197713*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["checkerboard", "Subsection",
 CellChangeTimes->{{3.58019120519563*^9, 3.58019121081164*^9}}],

Cell["\<\
The next fiunction does not work as expected; I commented it  out.\
\>", "Text",
 CellChangeTimes->{{3.580191862226137*^9, 3.580191938892522*^9}, {
  3.580192169396706*^9, 3.5801922143562775`*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"checkerboard", "[", 
     RowBox[{"surface_", ",", 
      RowBox[{"{", 
       RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "v0_", ",", "v1_"}], "}"}], ",", "opts___"}], "]"}],
     ":=", "\n", "        ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"plottmp", ",", "grtmp"}], "}"}], ",", "\n", 
      "                ", 
      RowBox[{
       RowBox[{"plottmp", " ", "=", " ", 
        RowBox[{"ParametricPlot3D", "[", "\n", "                ", 
         RowBox[{"surface", ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "v0", ",", "v1"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "                ", 
       RowBox[{"grtmp", " ", "=", " ", 
        RowBox[{"plottmp", " ", "/.", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Polygon", "[", "pts_", "]"}], ":>", " ", "\n", 
           "                ", 
           RowBox[{"Polygon", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"pts", " ", "+", " ", 
               RowBox[{"RotateRight", "[", "pts", "]"}]}], ")"}], "/", "2"}], 
            "]"}]}], ")"}]}]}], ";", "\n", "        ", 
       RowBox[{"Show", "[", 
        RowBox[{"grtmp", ",", 
         RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
     "]"}]}], "\n", "\n", "    ", 
   RowBox[{
    RowBox[{"checkerboard", "[", "graphic_", "]"}], ":=", 
    RowBox[{"Show", "[", 
     RowBox[{"graphic", " ", "/.", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Polygon", "[", "pts_", "]"}], ":>", "\n", "        ", 
        RowBox[{"Polygon", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"pts", " ", "+", " ", 
            RowBox[{"RotateRight", "[", "pts", "]"}]}], ")"}], "/", "2"}], 
         "]"}]}], ")"}]}], "]"}]}]}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"Polygon", "[", "pts5", "]"}], "]"}]], "Input",
 CellChangeTimes->{3.5801914629548826`*^9}],

Cell[BoxData[
 Graphics3DBox[
  Polygon3DBox[{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {1, 1, 1}}],
  AutomaticImageSize->True,
  ImageSize->{435.6194034961618, 439.6818222172755},
  ViewPoint->{2.044976369893696, 2.333978749531544, 1.3493016131731161`},
  ViewVertical->{0.6092870727865011, -0.12922273414133567`, 
   0.7823495049632836}]], "Output",
 CellChangeTimes->{3.5801914650920863`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkerboard", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"Polygon", "[", "pts5", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5801912532593145`*^9, 3.580191259951726*^9}, {
  3.5801912938349857`*^9, 3.580191362615506*^9}}],

Cell[BoxData[
 Graphics3DBox[
  Polygon3DBox[
   NCache[{{Rational[1, 2], Rational[1, 2], Rational[1, 2]}, {
     Rational[1, 2], 0, 0}, {Rational[1, 2], Rational[1, 2], 0}, {
     0, Rational[1, 2], Rational[1, 2]}, {
     Rational[1, 2], Rational[1, 2], 1}}, {{0.5, 0.5, 0.5}, {0.5, 0, 0}, {
     0.5, 0.5, 0}, {0, 0.5, 0.5}, {0.5, 0.5, 1}}]],
  AutomaticImageSize->True,
  ImageSize->{490.87960932128766`, 283.3829037771022},
  ViewPoint->{2.0523793653614577`, 0.10157539320099243`, 2.688386389664732},
  ViewVertical->{0.662663726679514, 1.777643096514636, 
   0.3165682417572373}]], "Output",
 CellChangeTimes->{{3.580191340713068*^9, 3.58019136470591*^9}, 
   3.5801914843425207`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkerboard", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "v", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"u", "^", "2"}], "+", 
        RowBox[{"v", "^", "2"}]}], ")"}], "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5801884439595804`*^9, 3.5801885655773935`*^9}, {
  3.5801915697994704`*^9, 3.580191581967492*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHu0V1W1x4lABR0OBmIamk9SEQJziIqpc3S9efEBdaXMukoNxUflUSNS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   "], {{
     {EdgeForm[None], 
      GraphicsGroup3DBox[{
        Polygon3DBox[
         NCache[{{Rational[1609, 2], 808, Rational[243, 2]}, {
           Rational[977, 2], Rational[1425, 2], 126}, {
           Rational[583, 2], Rational[1029, 2], Rational[85, 2]}, {
           Rational[581, 2], Rational[1027, 2], Rational[83, 2]}, {
           Rational[579, 2], Rational[1025, 2], Rational[81, 2]}, {
           651, Rational[1725, 2], 275}, {653, Rational[1729, 2], 277}, {
           654, Rational[1731, 2], Rational[561, 2]}, {
           650, Rational[1723, 2], 277}, {427, Rational[1301, 2], 75}, {
           Rational[1583, 2], Rational[2007, 2], 312}, {
           642, Rational[1711, 2], 268}, {Rational[1279, 2], 853, 260}, {633, 
           843, 250}, {260, 485, Rational[27, 2]}, {
           Rational[1223, 2], 648, 70}, {
           Rational[1221, 2], 641, Rational[125, 2]}, {
           260, Rational[957, 2], Rational[13, 2]}, {
           263, 480, Rational[15, 2]}, {426, 649, 69}, {
           801, Rational[2033, 2], 324}, {641, 849, Rational[527, 2]}, {
           Rational[549, 2], 494, 22}, {
           Rational[567, 2], Rational[1013, 2], Rational[71, 2]}, {
           Rational[559, 2], 503, Rational[63, 2]}, {
           275, Rational[999, 2], Rational[55, 2]}, {
           Rational[685, 2], Rational[1133, 2], Rational[189, 2]}, {
           743, 955, Rational[801, 2]}, {
           Rational[2149, 2], Rational[2551, 2], Rational[1277, 2]}, {746, 
           958, 404}, {747, 959, 405}, {743, 955, Rational[801, 2]}, {
           739, 951, Rational[785, 2]}, {
           Rational[2139, 2], Rational[2541, 2], Rational[1255, 2]}, {
           Rational[2143, 2], Rational[2545, 2], Rational[1263, 2]}, {
           Rational[2145, 2], Rational[2547, 2], Rational[1269, 2]}, {
           662, 874, Rational[639, 2]}, {332, 555, 83}, {
           Rational[1463, 2], Rational[1887, 2], Rational[735, 2]}, {664, 876,
            300}, {273, Rational[997, 2], Rational[53, 2]}, {270, 497, 25}, {
           Rational[1283, 2], Rational[1709, 2], Rational[529, 2]}, {
           Rational[2021, 2], Rational[2421, 2], Rational[1005, 2]}, {
           646, Rational[1715, 2], 268}, {
           Rational[1331, 2], Rational[1755, 2], 302}, {
           697, 909, Rational[667, 2]}, {313, 536, 65}, {
           Rational[693, 2], Rational[1139, 2], 98}, {
           Rational[1479, 2], Rational[1903, 2], 393}, {
           1013, Rational[2069, 2], 370}, {
           607, Rational[1277, 2], Rational[121, 2]}, {
           Rational[1301, 2], 861, Rational[569, 2]}, {
           Rational[2093, 2], Rational[2493, 2], Rational[1131, 2]}, {
           Rational[2089, 2], Rational[2489, 2], Rational[1125, 2]}, {
           Rational[2087, 2], Rational[2487, 2], Rational[1123, 2]}, {1043, 
           1243, 561}, {Rational[1999, 2], 1020, 337}, {
           Rational[1915, 2], 798, Rational[231, 2]}, {
           998, 1018, Rational[671, 2]}, {
           Rational[2079, 2], Rational[2479, 2], Rational[1113, 2]}, {1042, 
           1242, 560}, {798, Rational[2013, 2], 330}, {
           Rational[1595, 2], 1006, Rational[659, 2]}, {
           675, Rational[1773, 2], 310}, {
           Rational[843, 2], Rational[1283, 2], Rational[139, 2]}, {
           Rational[1557, 2], 987, 304}, {Rational[1327, 2], 875, 292}, {
           302, 525, Rational[107, 2]}, {Rational[1329, 2], 876, 297}, {
           Rational[1369, 2], 896, Rational[633, 2]}, {
           Rational[675, 2], Rational[1121, 2], Rational[177, 2]}, {330, 553, 
           81}, {462, 689, Rational[219, 2]}, {
           Rational[1635, 2], 1033, 347}, {
           Rational[2067, 2], Rational[2467, 2], Rational[1097, 2]}, {
           Rational[2065, 2], Rational[2465, 2], Rational[1095, 2]}, {
           Rational[1307, 2], 859, 280}, {Rational[531, 2], 488, 16}, {
           255, Rational[979, 2], Rational[37, 2]}, {
           257, Rational[981, 2], Rational[39, 2]}, {
           259, Rational[983, 2], Rational[41, 2]}, {
           Rational[531, 2], 488, 16}, {271, 484, 11}, {
           615, 644, Rational[129, 2]}, {616, Rational[1289, 2], 65}, {
           Rational[523, 2], 486, 14}, {
           Rational[505, 2], Rational[963, 2], 10}, {
           431, 658, Rational[157, 2]}, {424, 535, 191}, {
           284, Rational[781, 2], Rational[309, 2]}, {
           682, Rational[1787, 2], Rational[631, 2]}, {
           Rational[1357, 2], 890, Rational[623, 2]}, {
           679, Rational[1781, 2], 312}, {1037, 1237, 553}, {
           684, Rational[1791, 2], Rational[637, 2]}, {
           673, Rational[1769, 2], 300}, {
           Rational[1313, 2], 868, Rational[567, 2]}, {
           Rational[1345, 2], 884, 308}, {
           Rational[2055, 2], Rational[2455, 2], 527}, {
           656, Rational[1735, 2], Rational[543, 2]}, {
           Rational[1371, 2], Rational[1795, 2], 339}, {734, 946, 388}, {
           Rational[777, 2], Rational[1223, 2], Rational[281, 2]}, {
           Rational[757, 2], Rational[1203, 2], 130}, {
           Rational[1443, 2], Rational[1867, 2], 370}, {
           Rational[1507, 2], 958, 403}, {
           410, Rational[1251, 2], Rational[325, 2]}, {
           Rational[757, 2], Rational[1203, 2], 130}, {
           378, 601, Rational[259, 2]}, {376, 599, Rational[255, 2]}, {
           Rational[1439, 2], Rational[1863, 2], 365}, {
           Rational[2125, 2], Rational[2527, 2], 608}, {
           Rational[1459, 2], Rational[1883, 2], Rational[761, 2]}, {
           392, 615, Rational[287, 2]}, {381, 604, 133}, {
           Rational[743, 2], Rational[1189, 2], Rational[247, 2]}, {
           Rational[1429, 2], Rational[1853, 2], 358}, {
           723, 935, Rational[733, 2]}, {
           Rational[775, 2], Rational[1221, 2], Rational[279, 2]}, {
           Rational[773, 2], Rational[1219, 2], Rational[277, 2]}, {
           725, 937, Rational[753, 2]}, {
           Rational[1453, 2], Rational[1877, 2], 378}, {
           Rational[1451, 2], Rational[1875, 2], Rational[749, 2]}, {
           720, 932, Rational[737, 2]}, {
           Rational[759, 2], Rational[1205, 2], Rational[261, 2]}, {
           Rational[761, 2], Rational[1207, 2], Rational[263, 2]}, {
           Rational[1343, 2], 705, Rational[251, 2]}, {
           Rational[2049, 2], 847, Rational[305, 2]}, {
           Rational[2169, 2], 885, Rational[367, 2]}, {
           Rational[2167, 2], 884, Rational[365, 2]}, {
           Rational[2169, 2], Rational[1771, 2], Rational[367, 2]}, {1083, 
           882, 180}, {1088, Rational[2175, 2], 423}, {
           1005, Rational[1991, 2], Rational[691, 2]}, {
           Rational[1829, 2], Rational[1387, 2], Rational[41, 2]}, {
           Rational[2011, 2], Rational[1595, 2], Rational[221, 2]}, {
           Rational[2179, 2], 893, 191}, {Rational[1437, 2], 731, 144}, {
           Rational[1449, 2], Rational[1477, 2], Rational[305, 2]}, {
           Rational[2181, 2], Rational[2103, 2], 428}, {
           768, Rational[1861, 2], 430}, {448, 656, 201}, {
           Rational[1015, 2], Rational[1445, 2], Rational[297, 2]}, {
           Rational[1657, 2], Rational[1681, 2], 143}, {
           Rational[2179, 2], Rational[1783, 2], Rational[379, 2]}, {
           Rational[2181, 2], Rational[1785, 2], Rational[381, 2]}, {
           1092, Rational[1791, 2], 194}, {
           Rational[2181, 2], 894, Rational[385, 2]}, {1090, 892, 190}, {
           Rational[1543, 2], Rational[1553, 2], Rational[395, 2]}, {
           435, Rational[1301, 2], 187}, {
           Rational[775, 2], Rational[1221, 2], Rational[279, 2]}, {
           Rational[1411, 2], Rational[1835, 2], Rational[685, 2]}, {
           Rational[1489, 2], Rational[1913, 2], Rational[763, 2]}, {
           438, 661, Rational[379, 2]}, {
           Rational[885, 2], Rational[1331, 2], Rational[387, 2]}, {
           439, 662, Rational[381, 2]}, {
           Rational[1521, 2], Rational[1945, 2], Rational[843, 2]}, {
           756, 968, Rational[833, 2]}, {
           Rational[1511, 2], Rational[1935, 2], 416}, {
           Rational[2165, 2], Rational[2567, 2], 651}, {
           Rational[2161, 2], Rational[2563, 2], Rational[1295, 2]}, {
           Rational[2173, 2], Rational[2575, 2], 656}, {
           Rational[2183, 2], Rational[2585, 2], Rational[1327, 2]}, {
           Rational[2181, 2], Rational[2583, 2], Rational[1325, 2]}, {1080, 
           1281, 646}, {
           Rational[1473, 2], Rational[1897, 2], Rational[785, 2]}, {
           Rational[807, 2], Rational[1253, 2], Rational[311, 2]}, {
           Rational[809, 2], Rational[1255, 2], Rational[313, 2]}, {
           Rational[803, 2], Rational[1249, 2], 153}, {
           409, 632, Rational[321, 2]}, {
           Rational[1509, 2], Rational[1933, 2], Rational[833, 2]}, {
           Rational[2177, 2], Rational[2579, 2], Rational[1321, 2]}, {
           Rational[2167, 2], Rational[2569, 2], 653}, {
           Rational[2157, 2], Rational[2559, 2], Rational[1291, 2]}, {
           Rational[1551, 2], 984, 331}, {
           450, Rational[1339, 2], Rational[195, 2]}, {
           Rational[1509, 2], Rational[1933, 2], 415}, {720, 932, 381}, {
           Rational[717, 2], Rational[1163, 2], Rational[221, 2]}, {
           363, 586, Rational[229, 2]}, {362, 585, Rational[227, 2]}, {
           705, 917, Rational[683, 2]}, {
           Rational[2105, 2], Rational[2507, 2], Rational[1147, 2]}, {1053, 
           1254, 574}, {
           Rational[2111, 2], Rational[2513, 2], Rational[1167, 2]}, {1054, 
           1255, 582}, {
           Rational[1419, 2], Rational[1843, 2], Rational[691, 2]}, {
           Rational[721, 2], Rational[1167, 2], Rational[223, 2]}, {
           Rational[713, 2], Rational[1159, 2], 108}, {
           Rational[725, 2], Rational[1171, 2], 114}, {
           724, 936, Rational[769, 2]}, {
           Rational[2167, 2], Rational[2569, 2], Rational[1307, 2]}, {
           756, 968, Rational[833, 2]}, {
           Rational[825, 2], Rational[1271, 2], Rational[327, 2]}, {
           Rational[793, 2], Rational[1239, 2], Rational[295, 2]}, {
           Rational[791, 2], Rational[1237, 2], Rational[293, 2]}, {
           Rational[759, 2], Rational[1205, 2], Rational[261, 2]}, {
           Rational[727, 2], Rational[1173, 2], Rational[229, 2]}, {
           Rational[725, 2], Rational[1171, 2], Rational[227, 2]}, {364, 587, 
           115}, {693, Rational[1809, 2], Rational[643, 2]}, {
           1025, 1225, Rational[1069, 2]}, {Rational[1329, 2], 876, 297}, {
           Rational[625, 2], 528, 65}, {333, 541, 86}, {
           Rational[681, 2], Rational[1097, 2], Rational[187, 2]}, {
           Rational[683, 2], Rational[1099, 2], Rational[189, 2]}, {333, 541, 
           86}, {Rational[633, 2], Rational[1049, 2], Rational[139, 2]}, {
           Rational[611, 2], Rational[1027, 2], 58}, {
           Rational[1241, 2], Rational[1645, 2], Rational[569, 2]}, {
           Rational[1249, 2], Rational[1653, 2], 289}, {
           Rational[653, 2], Rational[1069, 2], Rational[159, 2]}, {
           Rational[687, 2], Rational[1103, 2], Rational[193, 2]}, {
           Rational[683, 2], Rational[1099, 2], Rational[189, 2]}, {
           Rational[667, 2], Rational[1083, 2], Rational[173, 2]}, {
           Rational[655, 2], Rational[1071, 2], Rational[161, 2]}, {326, 534, 
           79}, {Rational[1271, 2], 838, Rational[615, 2]}, {
           641, Rational[1687, 2], Rational[625, 2]}, {
           Rational[1283, 2], Rational[1689, 2], Rational[633, 2]}, {647, 850,
            322}, {669, Rational[1749, 2], Rational[725, 2]}, {
           Rational[1939, 2], 1170, 586}, {944, 1141, 531}, {939, 1135, 
           515}, {Rational[1883, 2], 1138, Rational[1045, 2]}, {
           624, Rational[1653, 2], 293}, {476, Rational[1391, 2], 124}, {
           Rational[1641, 2], Rational[2075, 2], 410}, {
           1009, Rational[2339, 2], Rational[1165, 2]}, {
           Rational[2015, 2], 1168, 579}, {
           Rational[1977, 2], Rational[2383, 2], Rational[1239, 2]}, {
           Rational[1975, 2], Rational[2381, 2], Rational[1237, 2]}, {
           694, Rational[1799, 2], 386}, {
           Rational[1385, 2], Rational[1795, 2], 382}, {
           Rational[1969, 2], Rational[2373, 2], Rational[1221, 2]}, {
           Rational[1975, 2], 1190, 616}, {
           Rational[1407, 2], 909, Rational[791, 2]}, {
           367, 575, Rational[239, 2]}, {318, 526, 70}, {
           315, 523, Rational[135, 2]}, {624, 826, Rational[577, 2]}, {
           Rational[1241, 2], Rational[1645, 2], Rational[569, 2]}, {
           308, 516, Rational[121, 2]}, {
           Rational[1253, 2], Rational[1657, 2], 293}, {
           942, Rational[2277, 2], 525}, {
           Rational[1263, 2], 834, Rational[601, 2]}, {631, 833, 297}, {
           943, Rational[2279, 2], 526}, {631, Rational[1667, 2], 300}, {370, 
           578, 122}, {
           Rational[755, 2], Rational[1171, 2], Rational[259, 2]}, {334, 542, 
           86}, {315, 523, Rational[135, 2]}, {Rational[1225, 2], 814, 273}, {
           932, 1127, Rational[1003, 2]}, {615, Rational[1633, 2], 277}, {295,
            503, 48}, {613, Rational[1629, 2], Rational[547, 2]}, {
           Rational[1865, 2], Rational[2255, 2], Rational[1001, 2]}, {
           Rational[1855, 2], Rational[2245, 2], Rational[987, 2]}, {
           Rational[1853, 2], Rational[2243, 2], Rational[985, 2]}, {930, 
           1125, 498}, {
           Rational[1857, 2], Rational[2247, 2], Rational[993, 2]}, {
           Rational[1215, 2], 809, Rational[535, 2]}, {
           606, 807, Rational[529, 2]}, {638, 839, Rational[593, 2]}, {
           639, 840, Rational[595, 2]}, {
           Rational[1853, 2], Rational[2241, 2], Rational[983, 2]}, {925, 
           1119, 490}, {Rational[1845, 2], 1117, 487}, {
           Rational[1201, 2], 802, 259}, {
           Rational[559, 2], Rational[975, 2], Rational[65, 2]}, {
           Rational[693, 2], Rational[1109, 2], Rational[199, 2]}, {
           667, Rational[1737, 2], Rational[651, 2]}, {
           928, 1123, Rational[991, 2]}, {1019, 1218, Rational[1185, 2]}, {
           Rational[2205, 2], Rational[2611, 2], Rational[1357, 2]}, {
           1009, Rational[2415, 2], 578}, {
           Rational[1831, 2], Rational[2219, 2], Rational[955, 2]}, {
           Rational[1331, 2], Rational[1733, 2], 323}, {
           Rational[1329, 2], Rational[1731, 2], 322}, {
           Rational[1827, 2], Rational[2215, 2], Rational[951, 2]}, {
           924, Rational[2237, 2], Rational[981, 2]}, {
           Rational[1269, 2], 836, 296}, {341, 549, 93}, {
           382, 590, Rational[269, 2]}, {664, 865, Rational[643, 2]}, {622, 
           823, 279}, {625, Rational[1653, 2], 283}, {
           804, Rational[2113, 2], 249}, {
           Rational[1789, 2], 1151, Rational[691, 2]}, {1100, 1303, 676}, {
           Rational[2201, 2], Rational[2607, 2], Rational[1353, 2]}, {
           Rational[2019, 2], Rational[2417, 2], Rational[1159, 2]}, {
           Rational[2015, 2], 1206, 577}, {
           Rational[2193, 2], Rational[2597, 2], Rational[1343, 2]}, {1004, 
           1152, 441}, {Rational[2015, 2], 1156, 445}, {
           1101, Rational[2607, 2], Rational[1353, 2]}, {
           Rational[2015, 2], 1206, 577}, {
           Rational[1903, 2], 1150, Rational[1093, 2]}, {
           Rational[1393, 2], Rational[1803, 2], Rational[771, 2]}, {408, 616,
            160}, {690, Rational[1789, 2], Rational[749, 2]}, {
           Rational[1939, 2], 1170, 585}, {
           Rational[1345, 2], Rational[1753, 2], Rational[711, 2]}, {
           673, Rational[1755, 2], Rational[715, 2]}, {971, 1172, 588}, {
           Rational[2281, 2], Rational[2175, 2], Rational[1273, 2]}, {1140, 
           1087, 636}, {973, 1174, 590}, {679, Rational[1767, 2], 364}, {
           677, Rational[1763, 2], 362}, {
           Rational[1355, 2], 882, Rational[725, 2]}, {
           680, Rational[1769, 2], 366}, {Rational[1367, 2], 888, 369}, {
           843, Rational[1705, 2], 414}, {1136, 1142, Rational[1281, 2]}, {
           Rational[1371, 2], 890, 371}, {686, 891, Rational[747, 2]}, {
           978, Rational[2359, 2], 599}, {
           Rational[1953, 2], Rational[2355, 2], Rational[1191, 2]}, {
           978, Rational[2359, 2], 599}, {
           Rational[1961, 2], Rational[2365, 2], Rational[1207, 2]}, {833, 
           1094, 418}, {651, Rational[1853, 2], Rational[393, 2]}, {
           Rational[1711, 2], 1072, 415}, {
           Rational[2187, 2], Rational[2591, 2], Rational[1337, 2]}, {
           1058, 1259, Rational[1201, 2]}, {
           Rational[2051, 2], Rational[2451, 2], 537}, {
           Rational[2057, 2], Rational[2457, 2], Rational[1083, 2]}, {
           Rational[2051, 2], Rational[2451, 2], 537}, {
           Rational[1977, 2], Rational[2013, 2], 322}, {
           Rational[1263, 2], 661, Rational[171, 2]}, {
           304, 527, Rational[111, 2]}, {
           Rational[1319, 2], 871, Rational[577, 2]}, {
           Rational[1973, 2], 1004, 318}, {955, 792, Rational[225, 2]}, {
           Rational[1923, 2], 981, Rational[695, 2]}, {
           960, 977, Rational[691, 2]}, {
           Rational[1905, 2], 786, Rational[219, 2]}, {951, 782, 108}, {924, 
           942, 152}, {Rational[1849, 2], 944, Rational[305, 2]}, {
           988, 1004, Rational[643, 2]}, {669, Rational[1761, 2], 300}, {
           Rational[627, 2], Rational[1073, 2], Rational[131, 2]}, {
           437, Rational[1313, 2], 85}, {
           Rational[1587, 2], Rational[2003, 2], 322}, {
           671, Rational[1765, 2], Rational[607, 2]}, {
           Rational[1339, 2], 881, Rational[601, 2]}, {
           Rational[2007, 2], Rational[2409, 2], 570}, {
           Rational[1937, 2], 1169, 586}, {
           Rational[1909, 2], Rational[2307, 2], Rational[1133, 2]}, {
           696, Rational[1799, 2], 378}, {
           Rational[881, 2], Rational[1297, 2], 193}, {
           Rational[1069, 2], Rational[1509, 2], Rational[367, 2]}, {
           Rational[1621, 2], Rational[2057, 2], 406}, {
           Rational[1991, 2], Rational[2399, 2], 639}, {
           Rational[1989, 2], Rational[2397, 2], Rational[1271, 2]}, {
           Rational[1423, 2], Rational[1835, 2], Rational[817, 2]}, {
           Rational[853, 2], Rational[1269, 2], 179}, {
           Rational[1417, 2], Rational[1829, 2], 405}, {719, 925, 416}, {438, 
           646, 191}, {Rational[869, 2], Rational[1285, 2], 187}, {
           Rational[1079, 2], Rational[1519, 2], 188}, {
           532, 752, Rational[361, 2]}, {
           Rational[823, 2], Rational[1239, 2], 164}, {
           404, 612, Rational[313, 2]}, {
           Rational[833, 2], Rational[1249, 2], Rational[337, 2]}, {432, 640, 
           184}, {Rational[879, 2], Rational[1295, 2], Rational[383, 2]}, {
           Rational[1097, 2], Rational[1537, 2], 197}, {
           827, Rational[2091, 2], 426}, {
           719, Rational[1851, 2], Rational[843, 2]}, {
           Rational[1439, 2], 926, 422}, {957, 949, Rational[673, 2]}, {
           673, Rational[1333, 2], Rational[205, 2]}, {431, 639, 184}, {
           691, Rational[1789, 2], Rational[747, 2]}, {
           953, Rational[2305, 2], 566}, {
           Rational[1913, 2], Rational[2313, 2], Rational[1141, 2]}, {
           953, Rational[2305, 2], 566}, {
           Rational[1301, 2], 854, Rational[665, 2]}, {659, 863, 342}, {
           959, Rational[2317, 2], 572}, {
           Rational[1905, 2], Rational[2303, 2], Rational[1129, 2]}, {952, 
           1151, 564}, {Rational[1915, 2], 1157, Rational[1141, 2]}, {
           Rational[1397, 2], Rational[1805, 2], 382}, {
           699, 903, Rational[765, 2]}, {667, 871, Rational[701, 2]}, {
           501, Rational[1441, 2], Rational[299, 2]}, {
           516, Rational[1471, 2], Rational[329, 2]}, {
           Rational[749, 2], Rational[1165, 2], 127}, {
           666, 871, Rational[707, 2]}, {
           Rational[1941, 2], Rational[2343, 2], 589}, {
           Rational[1919, 2], Rational[2319, 2], Rational[1147, 2]}, {961, 
           1161, 575}, {
           Rational[1923, 2], Rational[2323, 2], Rational[1151, 2]}, {
           813, Rational[2057, 2], 392}, {
           Rational[1623, 2], 1027, Rational[781, 2]}, {982, 1182, 526}, {
           Rational[2101, 2], 1053, Rational[679, 2]}, {
           722, Rational[1457, 2], 150}, {643, 643, 79}, {938, 730, 58}, {
           Rational[1877, 2], Rational[1461, 2], Rational[117, 2]}, {
           Rational[1937, 2], Rational[1627, 2], 134}, {
           1043, Rational[2185, 2], Rational[869, 2]}, {
           Rational[2133, 2], 1036, Rational[753, 2]}, {
           Rational[2093, 2], 785, Rational[189, 2]}, {
           1022, Rational[1667, 2], Rational[289, 2]}, {1019, 827, 140}, {
           991, Rational[1511, 2], 76}, {993, 760, Rational[159, 2]}, {1044, 
           781, 92}, {Rational[2041, 2], 831, 143}, {1019, 827, 140}, {1041, 
           773, 86}, {1016, 813, 126}, {965, 792, Rational[225, 2]}, {1000, 
           781, 94}, {1056, 814, Rational[229, 2]}, {
           Rational[2043, 2], 828, 136}, {
           Rational[1983, 2], Rational[1717, 2], Rational[343, 2]}, {
           Rational[2043, 2], Rational[1655, 2], 136}, {
           1022, Rational[1661, 2], 139}, {1035, Rational[2143, 2], 411}, {
           Rational[2125, 2], 1033, 370}, {
           Rational[1409, 2], Rational[1357, 2], Rational[209, 2]}, {
           715, Rational[1839, 2], 371}, {1064, 1225, Rational[1227, 2]}, {
           1056, 990, 350}, {Rational[1317, 2], 640, Rational[125, 2]}, {
           635, Rational[1591, 2], 251}, {
           Rational[1919, 2], Rational[1803, 2], 249}, {
           Rational[1967, 2], 747, 61}, {1014, 812, 119}, {
           1009, 800, Rational[227, 2]}, {
           Rational[2083, 2], Rational[1549, 2], 88}, {
           Rational[2047, 2], Rational[1979, 2], 289}, {
           Rational[2063, 2], Rational[2017, 2], 300}, {
           Rational[1999, 2], Rational[1555, 2], 90}, {
           Rational[2007, 2], 1005, 344}, {Rational[1999, 2], 994, 333}, {
           Rational[1923, 2], Rational[1573, 2], 107}, {
           1023, Rational[2109, 2], Rational[761, 2]}, {
           Rational[1985, 2], 984, Rational[635, 2]}, {
           957, 779, Rational[199, 2]}, {984, 842, 155}, {
           Rational[1961, 2], Rational[1669, 2], Rational[295, 2]}, {
           831, Rational[1523, 2], 75}, {833, Rational[1531, 2], 79}, {
           Rational[1971, 2], Rational[1687, 2], Rational[313, 2]}, {
           Rational[1975, 2], Rational[1695, 2], Rational[321, 2]}, {
           Rational[2001, 2], Rational[2063, 2], Rational[673, 2]}, {
           977, Rational[1951, 2], 288}, {
           Rational[1911, 2], 778, Rational[197, 2]}, {
           973, Rational[1645, 2], Rational[271, 2]}, {
           Rational[1955, 2], Rational[1657, 2], Rational[283, 2]}, {
           Rational[1951, 2], 825, 138}, {Rational[1949, 2], 824, 137}, {
           Rational[1951, 2], Rational[1653, 2], Rational[279, 2]}, {
           Rational[1947, 2], 823, 136}, {
           Rational[1943, 2], Rational[1639, 2], Rational[265, 2]}, {
           Rational[1941, 2], Rational[1637, 2], Rational[263, 2]}, {
           Rational[1937, 2], Rational[1631, 2], Rational[257, 2]}, {
           776, 814, Rational[239, 2]}, {
           Rational[1641, 2], 1036, Rational[697, 2]}, {
           1025, Rational[2123, 2], Rational[763, 2]}, {981, 840, 153}, {
           Rational[1933, 2], Rational[1625, 2], Rational[251, 2]}, {
           Rational[1931, 2], Rational[1623, 2], Rational[249, 2]}, {
           992, 1015, Rational[649, 2]}, {
           Rational[1997, 2], Rational[2051, 2], 335}, {
           Rational[1975, 2], Rational[1707, 2], Rational[333, 2]}, {
           Rational[1991, 2], Rational[1741, 2], Rational[367, 2]}, {
           732, 772, Rational[401, 2]}, {465, 673, Rational[435, 2]}, {
           Rational[921, 2], Rational[1337, 2], Rational[427, 2]}, {
           350, Rational[899, 2], Rational[443, 2]}, {
           Rational[693, 2], 446, Rational[435, 2]}, {
           Rational[923, 2], Rational[1339, 2], Rational[427, 2]}, {461, 669, 
           213}, {457, 665, Rational[419, 2]}, {465, 673, Rational[435, 2]}, {
           466, 674, Rational[437, 2]}, {
           Rational[1113, 2], Rational[1551, 2], 197}, {
           Rational[1747, 2], 891, Rational[379, 2]}, {
           Rational[1755, 2], Rational[1791, 2], 194}, {
           562, Rational[1563, 2], Rational[405, 2]}, {357, 457, 117}, {
           444, Rational[1111, 2], Rational[185, 2]}, {
           438, Rational[1111, 2], 199}, {
           Rational[689, 2], Rational[901, 2], 217}, {349, 451, 221}, {386, 
           492, 154}, {524, Rational[1481, 2], Rational[137, 2]}, {
           Rational[765, 2], 489, 151}, {
           Rational[491, 2], Rational[479, 2], 119}, {
           Rational[489, 2], Rational[477, 2], 118}, {357, 459, 229}, {
           Rational[709, 2], Rational[913, 2], Rational[449, 2]}, {237, 237, 
           228}, {346, 452, 219}, {484, 696, 132}, {
           Rational[967, 2], Rational[1391, 2], Rational[263, 2]}, {
           727, 934, Rational[873, 2]}, {728, 935, Rational[875, 2]}, {
           Rational[929, 2], Rational[1345, 2], Rational[433, 2]}, {
           Rational[935, 2], Rational[1351, 2], 220}, {
           Rational[711, 2], Rational[911, 2], Rational[427, 2]}, {
           Rational[485, 2], Rational[481, 2], Rational[443, 2]}, {
           Rational[477, 2], Rational[489, 2], Rational[463, 2]}, {
           Rational[489, 2], Rational[489, 2], 228}, {
           Rational[1261, 2], Rational[1373, 2], Rational[733, 2]}, {
           Rational[1695, 2], 705, Rational[537, 2]}, {
           492, Rational[787, 2], 41}, {271, 379, 143}, {453, 572, 213}, {
           Rational[903, 2], Rational[1141, 2], Rational[427, 2]}, {
           Rational[477, 2], Rational[493, 2], Rational[463, 2]}, {
           644, Rational[1039, 2], Rational[327, 2]}, {
           Rational[1289, 2], 518, Rational[309, 2]}, {
           Rational[479, 2], Rational[487, 2], 113}, {
           Rational[477, 2], Rational[485, 2], 8}, {
           Rational[479, 2], Rational[487, 2], 113}, {
           Rational[793, 2], Rational[1013, 2], 153}, {
           Rational[1011, 2], Rational[1435, 2], Rational[293, 2]}, {
           Rational[1469, 2], 693, Rational[243, 2]}, {
           Rational[2021, 2], Rational[1437, 2], Rational[63, 2]}, {
           Rational[2019, 2], 920, Rational[525, 2]}, {
           733, Rational[1807, 2], 351}, {
           372, Rational[1179, 2], Rational[247, 2]}, {
           Rational[527, 2], Rational[727, 2], 135}, {
           Rational[927, 2], 460, 228}, {
           Rational[963, 2], Rational[1369, 2], 119}, {
           Rational[765, 2], Rational[1387, 2], Rational[47, 2]}, {
           365, Rational[943, 2], 130}, {246, Rational[711, 2], 122}, {
           Rational[1253, 2], Rational[1477, 2], Rational[213, 2]}, {963, 
           1003, 113}, {Rational[1973, 2], 898, Rational[129, 2]}, {
           Rational[1973, 2], Rational[1795, 2], Rational[327, 2]}, {
           Rational[1427, 2], 861, Rational[277, 2]}, {
           Rational[1511, 2], Rational[1425, 2], Rational[65, 2]}, {
           1008, 707, Rational[39, 2]}, {968, Rational[1419, 2], 30}, {
           Rational[1911, 2], Rational[1551, 2], 96}, {
           Rational[1909, 2], Rational[1549, 2], 95}, {
           1000, Rational[1577, 2], Rational[203, 2]}, {
           Rational[2149, 2], 869, Rational[335, 2]}, {
           1075, Rational[1739, 2], 168}, {Rational[2153, 2], 1036, 406}, {
           1075, 1035, Rational[809, 2]}, {
           Rational[2095, 2], 809, Rational[227, 2]}, {
           993, Rational[1559, 2], 92}, {
           Rational[2031, 2], Rational[1663, 2], Rational[275, 2]}, {
           Rational[2133, 2], Rational[1707, 2], Rational[303, 2]}, {1064, 
           846, 144}, {
           Rational[2123, 2], Rational[1677, 2], Rational[273, 2]}, {
           Rational[2093, 2], Rational[1603, 2], 106}, {
           Rational[2111, 2], 820, 125}, {Rational[1515, 2], 760, 181}, {
           Rational[1467, 2], Rational[1403, 2], 130}, {
           Rational[1519, 2], 731, Rational[89, 2]}, {
           Rational[909, 2], 666, 95}, {684, 695, 131}, {
           Rational[1929, 2], Rational[1597, 2], 119}, {
           965, 799, Rational[239, 2]}, {
           Rational[1565, 2], Rational[1609, 2], Rational[235, 2]}, {
           815, 786, Rational[183, 2]}, {
           Rational[1933, 2], Rational[1425, 2], Rational[65, 2]}, {
           Rational[1545, 2], 778, Rational[181, 2]}, {
           Rational[1697, 2], 859, 157}, {
           1070, Rational[1717, 2], Rational[313, 2]}, {
           Rational[2137, 2], Rational[2051, 2], Rational[781, 2]}, {
           1070, Rational[2057, 2], Rational[787, 2]}, {
           Rational[2141, 2], Rational[1721, 2], Rational[317, 2]}, {
           Rational[1703, 2], Rational[1723, 2], Rational[319, 2]}, {
           824, Rational[1597, 2], Rational[207, 2]}, {
           Rational[2029, 2], 730, Rational[85, 2]}, {
           Rational[2047, 2], Rational[1489, 2], 58}, {
           Rational[2067, 2], 764, Rational[155, 2]}, {1068, 836, 142}, {1101,
            906, 205}, {
           Rational[2201, 2], Rational[1811, 2], Rational[409, 2]}, {
           1062, 827, Rational[267, 2]}, {1026, Rational[1505, 2], 66}, {
           Rational[2125, 2], 829, 135}, {781, Rational[1575, 2], 209}, {
           Rational[797, 2], Rational[1213, 2], Rational[301, 2]}, {
           Rational[1249, 2], Rational[1237, 2], 54}, {
           Rational[1835, 2], Rational[1395, 2], Rational[49, 2]}, {
           Rational[1829, 2], 694, 21}, {Rational[1939, 2], 723, 43}, {
           1063, Rational[1659, 2], Rational[271, 2]}, {
           854, 872, Rational[341, 2]}, {822, 803, Rational[217, 2]}, {
           Rational[2139, 2], Rational[1675, 2], Rational[287, 2]}, {
           1064, Rational[1659, 2], Rational[271, 2]}, {
           Rational[2087, 2], Rational[1939, 2], 338}, {
           Rational[2093, 2], Rational[1947, 2], 342}, {739, 707, 136}, {
           427, 635, Rational[359, 2]}, {721, Rational[1381, 2], 119}, {
           1032, Rational[1523, 2], Rational[149, 2]}, {
           Rational[2053, 2], Rational[1507, 2], Rational[133, 2]}, {
           1045, Rational[1597, 2], 104}, {992, Rational[1545, 2], 85}, {
           Rational[1999, 2], 788, Rational[201, 2]}, {
           Rational[1681, 2], 854, 152}, {Rational[1679, 2], 853, 151}, {1071,
            860, 158}, {Rational[2167, 2], Rational[1753, 2], 175}, {
           Rational[2119, 2], Rational[1641, 2], 127}, {
           969, Rational[1435, 2], 38}, {
           Rational[1149, 2], 470, Rational[255, 2]}, {
           Rational[1159, 2], 477, 135}, {
           750, Rational[1965, 2], Rational[613, 2]}, {
           Rational[1501, 2], 983, 307}, {
           Rational[1863, 2], Rational[1431, 2], Rational[87, 2]}, {
           984, 742, Rational[125, 2]}, {
           Rational[1969, 2], Rational[1485, 2], 63}, {
           984, 976, Rational[597, 2]}, {Rational[1565, 2], 990, 305}, {
           Rational[1471, 2], 744, Rational[129, 2]}, {944, 746, 74}, {
           933, 724, Rational[103, 2]}, {Rational[1433, 2], 714, 34}, {
           Rational[1081, 2], 941, 304}, {
           Rational[1073, 2], Rational[1875, 2], Rational[601, 2]}, {
           Rational[1541, 2], Rational[1867, 2], 293}, {993, 962, 329}, {
           Rational[1325, 2], Rational[1361, 2], Rational[233, 2]}, {
           Rational[767, 2], Rational[1183, 2], Rational[271, 2]}, {
           Rational[1447, 2], 884, 363}, {
           Rational[2073, 2], 952, Rational[631, 2]}, {
           Rational[1363, 2], Rational[1289, 2], 73}, {640, 638, 74}, {
           809, Rational[1389, 2], 140}, {852, 704, 142}, {853, 713, 143}, {
           856, Rational[1911, 2], Rational[771, 2]}, {
           Rational[2083, 2], 1025, 330}, {
           869, Rational[1297, 2], Rational[347, 2]}, {
           808, 598, Rational[277, 2]}, {959, 780, Rational[201, 2]}, {
           1002, Rational[1579, 2], Rational[205, 2]}, {
           747, 871, Rational[281, 2]}, {746, 870, Rational[279, 2]}, {
           Rational[2037, 2], Rational[1879, 2], 288}, {
           789, 957, Rational[611, 2]}, {438, Rational[1299, 2], 78}, {
           687, Rational[1347, 2], 95}, {
           797, Rational[1575, 2], Rational[187, 2]}, {
           732, 732, Rational[105, 2]}, {
           Rational[1857, 2], Rational[1425, 2], Rational[81, 2]}, {
           Rational[1875, 2], Rational[1467, 2], Rational[123, 2]}, {
           733, 745, Rational[131, 2]}, {722, 721, Rational[83, 2]}}, {{
           804.5, 808, 121.5}, {488.5, 712.5, 126}, {291.5, 514.5, 42.5}, {
           290.5, 513.5, 41.5}, {289.5, 512.5, 40.5}, {651, 862.5, 275}, {
           653, 864.5, 277}, {654, 865.5, 280.5}, {650, 861.5, 277}, {
           427, 650.5, 75}, {791.5, 1003.5, 312}, {642, 855.5, 268}, {
           639.5, 853, 260}, {633, 843, 250}, {260, 485, 13.5}, {
           611.5, 648, 70}, {610.5, 641, 62.5}, {260, 478.5, 6.5}, {
           263, 480, 7.5}, {426, 649, 69}, {801, 1016.5, 324}, {
           641, 849, 263.5}, {274.5, 494, 22}, {283.5, 506.5, 35.5}, {
           279.5, 503, 31.5}, {275, 499.5, 27.5}, {342.5, 566.5, 94.5}, {
           743, 955, 400.5}, {1074.5, 1275.5, 638.5}, {746, 958, 404}, {747, 
           959, 405}, {743, 955, 400.5}, {739, 951, 392.5}, {1069.5, 1270.5, 
           627.5}, {1071.5, 1272.5, 631.5}, {1072.5, 1273.5, 634.5}, {
           662, 874, 319.5}, {332, 555, 83}, {731.5, 943.5, 367.5}, {664, 876,
            300}, {273, 498.5, 26.5}, {270, 497, 25}, {641.5, 854.5, 264.5}, {
           1010.5, 1210.5, 502.5}, {646, 857.5, 268}, {665.5, 877.5, 302}, {
           697, 909, 333.5}, {313, 536, 65}, {346.5, 569.5, 98}, {
           739.5, 951.5, 393}, {1013, 1034.5, 370}, {607, 638.5, 60.5}, {
           650.5, 861, 284.5}, {1046.5, 1246.5, 565.5}, {1044.5, 1244.5, 
           562.5}, {1043.5, 1243.5, 561.5}, {1043, 1243, 561}, {
           999.5, 1020, 337}, {957.5, 798, 115.5}, {998, 1018, 335.5}, {
           1039.5, 1239.5, 556.5}, {1042, 1242, 560}, {798, 1006.5, 330}, {
           797.5, 1006, 329.5}, {675, 886.5, 310}, {421.5, 641.5, 69.5}, {
           778.5, 987, 304}, {663.5, 875, 292}, {302, 525, 53.5}, {
           664.5, 876, 297}, {684.5, 896, 316.5}, {337.5, 560.5, 88.5}, {330, 
           553, 81}, {462, 689, 109.5}, {817.5, 1033, 347}, {1033.5, 1233.5, 
           548.5}, {1032.5, 1232.5, 547.5}, {653.5, 859, 280}, {
           265.5, 488, 16}, {255, 489.5, 18.5}, {257, 490.5, 19.5}, {
           259, 491.5, 20.5}, {265.5, 488, 16}, {271, 484, 11}, {
           615, 644, 64.5}, {616, 644.5, 65}, {261.5, 486, 14}, {
           252.5, 481.5, 10}, {431, 658, 78.5}, {424, 535, 191}, {
           284, 390.5, 154.5}, {682, 893.5, 315.5}, {678.5, 890, 311.5}, {
           679, 890.5, 312}, {1037, 1237, 553}, {684, 895.5, 318.5}, {
           673, 884.5, 300}, {656.5, 868, 283.5}, {672.5, 884, 308}, {
           1027.5, 1227.5, 527}, {656, 867.5, 271.5}, {685.5, 897.5, 339}, {
           734, 946, 388}, {388.5, 611.5, 140.5}, {378.5, 601.5, 130}, {
           721.5, 933.5, 370}, {753.5, 958, 403}, {410, 625.5, 162.5}, {
           378.5, 601.5, 130}, {378, 601, 129.5}, {376, 599, 127.5}, {
           719.5, 931.5, 365}, {1062.5, 1263.5, 608}, {729.5, 941.5, 380.5}, {
           392, 615, 143.5}, {381, 604, 133}, {371.5, 594.5, 123.5}, {
           714.5, 926.5, 358}, {723, 935, 366.5}, {387.5, 610.5, 139.5}, {
           386.5, 609.5, 138.5}, {725, 937, 376.5}, {726.5, 938.5, 378}, {
           725.5, 937.5, 374.5}, {720, 932, 368.5}, {379.5, 602.5, 130.5}, {
           380.5, 603.5, 131.5}, {671.5, 705, 125.5}, {1024.5, 847, 152.5}, {
           1084.5, 885, 183.5}, {1083.5, 884, 182.5}, {1084.5, 885.5, 
           183.5}, {1083, 882, 180}, {1088, 1087.5, 423}, {
           1005, 995.5, 345.5}, {914.5, 693.5, 20.5}, {1005.5, 797.5, 
           110.5}, {1089.5, 893, 191}, {718.5, 731, 144}, {724.5, 738.5, 
           152.5}, {1090.5, 1051.5, 428}, {768, 930.5, 430}, {448, 656, 
           201}, {507.5, 722.5, 148.5}, {828.5, 840.5, 143}, {1089.5, 891.5, 
           189.5}, {1090.5, 892.5, 190.5}, {1092, 895.5, 194}, {
           1090.5, 894, 192.5}, {1090, 892, 190}, {771.5, 776.5, 197.5}, {
           435, 650.5, 187}, {387.5, 610.5, 139.5}, {705.5, 917.5, 342.5}, {
           744.5, 956.5, 381.5}, {438, 661, 189.5}, {442.5, 665.5, 193.5}, {
           439, 662, 190.5}, {760.5, 972.5, 421.5}, {756, 968, 416.5}, {
           755.5, 967.5, 416}, {1082.5, 1283.5, 651}, {1080.5, 1281.5, 
           647.5}, {1086.5, 1287.5, 656}, {1091.5, 1292.5, 663.5}, {1090.5, 
           1291.5, 662.5}, {1080, 1281, 646}, {736.5, 948.5, 392.5}, {403.5, 
           626.5, 155.5}, {404.5, 627.5, 156.5}, {401.5, 624.5, 153}, {
           409, 632, 160.5}, {754.5, 966.5, 416.5}, {1088.5, 1289.5, 660.5}, {
           1083.5, 1284.5, 653}, {1078.5, 1279.5, 645.5}, {775.5, 984, 331}, {
           450, 669.5, 97.5}, {754.5, 966.5, 415}, {720, 932, 381}, {358.5, 
           581.5, 110.5}, {363, 586, 114.5}, {362, 585, 113.5}, {
           705, 917, 341.5}, {1052.5, 1253.5, 573.5}, {1053, 1254, 574}, {
           1055.5, 1256.5, 583.5}, {1054, 1255, 582}, {709.5, 921.5, 345.5}, {
           360.5, 583.5, 111.5}, {356.5, 579.5, 108}, {362.5, 585.5, 114}, {
           724, 936, 384.5}, {1083.5, 1284.5, 653.5}, {756, 968, 416.5}, {
           412.5, 635.5, 163.5}, {396.5, 619.5, 147.5}, {395.5, 618.5, 
           146.5}, {379.5, 602.5, 130.5}, {363.5, 586.5, 114.5}, {362.5, 
           585.5, 113.5}, {364, 587, 115}, {693, 904.5, 321.5}, {
           1025, 1225, 534.5}, {664.5, 876, 297}, {312.5, 528, 65}, {333, 541,
            86}, {340.5, 548.5, 93.5}, {341.5, 549.5, 94.5}, {333, 541, 86}, {
           316.5, 524.5, 69.5}, {305.5, 513.5, 58}, {620.5, 822.5, 284.5}, {
           624.5, 826.5, 289}, {326.5, 534.5, 79.5}, {343.5, 551.5, 96.5}, {
           341.5, 549.5, 94.5}, {333.5, 541.5, 86.5}, {327.5, 535.5, 80.5}, {
           326, 534, 79}, {635.5, 838, 307.5}, {641, 843.5, 312.5}, {641.5, 
           844.5, 316.5}, {647, 850, 322}, {669, 874.5, 362.5}, {
           969.5, 1170, 586}, {944, 1141, 531}, {939, 1135, 515}, {
           941.5, 1138, 522.5}, {624, 826.5, 293}, {476, 695.5, 124}, {
           820.5, 1037.5, 410}, {1009, 1169.5, 582.5}, {1007.5, 1168, 579}, {
           988.5, 1191.5, 619.5}, {987.5, 1190.5, 618.5}, {694, 899.5, 386}, {
           692.5, 897.5, 382}, {984.5, 1186.5, 610.5}, {987.5, 1190, 616}, {
           703.5, 909, 395.5}, {367, 575, 119.5}, {318, 526, 70}, {
           315, 523, 67.5}, {624, 826, 288.5}, {620.5, 822.5, 284.5}, {
           308, 516, 60.5}, {626.5, 828.5, 293}, {942, 1138.5, 525}, {
           631.5, 834, 300.5}, {631, 833, 297}, {943, 1139.5, 526}, {
           631, 833.5, 300}, {370, 578, 122}, {377.5, 585.5, 129.5}, {334, 
           542, 86}, {315, 523, 67.5}, {612.5, 814, 273}, {
           932, 1127, 501.5}, {615, 816.5, 277}, {295, 503, 48}, {
           613, 814.5, 273.5}, {932.5, 1127.5, 500.5}, {927.5, 1122.5, 
           493.5}, {926.5, 1121.5, 492.5}, {930, 1125, 498}, {928.5, 1123.5, 
           496.5}, {607.5, 809, 267.5}, {606, 807, 264.5}, {
           638, 839, 296.5}, {639, 840, 297.5}, {926.5, 1120.5, 491.5}, {925, 
           1119, 490}, {922.5, 1117, 487}, {600.5, 802, 259}, {279.5, 487.5, 
           32.5}, {346.5, 554.5, 99.5}, {667, 868.5, 325.5}, {
           928, 1123, 495.5}, {1019, 1218, 592.5}, {1102.5, 1305.5, 678.5}, {
           1009, 1207.5, 578}, {915.5, 1109.5, 477.5}, {665.5, 866.5, 323}, {
           664.5, 865.5, 322}, {913.5, 1107.5, 475.5}, {924, 1118.5, 490.5}, {
           634.5, 836, 296}, {341, 549, 93}, {382, 590, 134.5}, {
           664, 865, 321.5}, {622, 823, 279}, {625, 826.5, 283}, {
           804, 1056.5, 249}, {894.5, 1151, 345.5}, {1100, 1303, 676}, {
           1100.5, 1303.5, 676.5}, {1009.5, 1208.5, 579.5}, {
           1007.5, 1206, 577}, {1096.5, 1298.5, 671.5}, {1004, 1152, 441}, {
           1007.5, 1156, 445}, {1101, 1303.5, 676.5}, {1007.5, 1206, 577}, {
           951.5, 1150, 546.5}, {696.5, 901.5, 385.5}, {408, 616, 160}, {
           690, 894.5, 374.5}, {969.5, 1170, 585}, {672.5, 876.5, 355.5}, {
           673, 877.5, 357.5}, {971, 1172, 588}, {1140.5, 1087.5, 636.5}, {
           1140, 1087, 636}, {973, 1174, 590}, {679, 883.5, 364}, {
           677, 881.5, 362}, {677.5, 882, 362.5}, {680, 884.5, 366}, {
           683.5, 888, 369}, {843, 852.5, 414}, {1136, 1142, 640.5}, {
           685.5, 890, 371}, {686, 891, 373.5}, {978, 1179.5, 599}, {976.5, 
           1177.5, 595.5}, {978, 1179.5, 599}, {980.5, 1182.5, 603.5}, {833, 
           1094, 418}, {651, 926.5, 196.5}, {855.5, 1072, 415}, {1093.5, 
           1295.5, 668.5}, {1058, 1259, 600.5}, {1025.5, 1225.5, 537}, {
           1028.5, 1228.5, 541.5}, {1025.5, 1225.5, 537}, {
           988.5, 1006.5, 322}, {631.5, 661, 85.5}, {304, 527, 55.5}, {
           659.5, 871, 288.5}, {986.5, 1004, 318}, {955, 792, 112.5}, {
           961.5, 981, 347.5}, {960, 977, 345.5}, {952.5, 786, 109.5}, {951, 
           782, 108}, {924, 942, 152}, {924.5, 944, 152.5}, {
           988, 1004, 321.5}, {669, 880.5, 300}, {313.5, 536.5, 65.5}, {
           437, 656.5, 85}, {793.5, 1001.5, 322}, {671, 882.5, 303.5}, {
           669.5, 881, 300.5}, {1003.5, 1204.5, 570}, {968.5, 1169, 586}, {
           954.5, 1153.5, 566.5}, {696, 899.5, 378}, {440.5, 648.5, 193}, {
           534.5, 754.5, 183.5}, {810.5, 1028.5, 406}, {995.5, 1199.5, 639}, {
           994.5, 1198.5, 635.5}, {711.5, 917.5, 408.5}, {
           426.5, 634.5, 179}, {708.5, 914.5, 405}, {719, 925, 416}, {438, 
           646, 191}, {434.5, 642.5, 187}, {539.5, 759.5, 188}, {
           532, 752, 180.5}, {411.5, 619.5, 164}, {404, 612, 156.5}, {416.5, 
           624.5, 168.5}, {432, 640, 184}, {439.5, 647.5, 191.5}, {
           548.5, 768.5, 197}, {827, 1045.5, 426}, {719, 925.5, 421.5}, {
           719.5, 926, 422}, {957, 949, 336.5}, {673, 666.5, 102.5}, {431, 
           639, 184}, {691, 894.5, 373.5}, {953, 1152.5, 566}, {956.5, 1156.5,
            570.5}, {953, 1152.5, 566}, {650.5, 854, 332.5}, {659, 863, 
           342}, {959, 1158.5, 572}, {952.5, 1151.5, 564.5}, {952, 1151, 
           564}, {957.5, 1157, 570.5}, {698.5, 902.5, 382}, {
           699, 903, 382.5}, {667, 871, 350.5}, {501, 720.5, 149.5}, {
           516, 735.5, 164.5}, {374.5, 582.5, 127}, {666, 871, 353.5}, {
           970.5, 1171.5, 589}, {959.5, 1159.5, 573.5}, {961, 1161, 575}, {
           961.5, 1161.5, 575.5}, {813, 1028.5, 392}, {811.5, 1027, 390.5}, {
           982, 1182, 526}, {1050.5, 1053, 339.5}, {722, 728.5, 150}, {643, 
           643, 79}, {938, 730, 58}, {938.5, 730.5, 58.5}, {
           968.5, 813.5, 134}, {1043, 1092.5, 434.5}, {1066.5, 1036, 376.5}, {
           1046.5, 785, 94.5}, {1022, 833.5, 144.5}, {1019, 827, 140}, {
           991, 755.5, 76}, {993, 760, 79.5}, {1044, 781, 92}, {
           1020.5, 831, 143}, {1019, 827, 140}, {1041, 773, 86}, {1016, 813, 
           126}, {965, 792, 112.5}, {1000, 781, 94}, {1056, 814, 114.5}, {
           1021.5, 828, 136}, {991.5, 858.5, 171.5}, {1021.5, 827.5, 136}, {
           1022, 830.5, 139}, {1035, 1071.5, 411}, {1062.5, 1033, 370}, {
           704.5, 678.5, 104.5}, {715, 919.5, 371}, {1064, 1225, 613.5}, {
           1056, 990, 350}, {658.5, 640, 62.5}, {635, 795.5, 251}, {
           959.5, 901.5, 249}, {983.5, 747, 61}, {1014, 812, 119}, {
           1009, 800, 113.5}, {1041.5, 774.5, 88}, {1023.5, 989.5, 289}, {
           1031.5, 1008.5, 300}, {999.5, 777.5, 90}, {1003.5, 1005, 344}, {
           999.5, 994, 333}, {961.5, 786.5, 107}, {1023, 1054.5, 380.5}, {
           992.5, 984, 317.5}, {957, 779, 99.5}, {984, 842, 155}, {980.5, 
           834.5, 147.5}, {831, 761.5, 75}, {833, 765.5, 79}, {985.5, 843.5, 
           156.5}, {987.5, 847.5, 160.5}, {1000.5, 1031.5, 336.5}, {
           977, 975.5, 288}, {955.5, 778, 98.5}, {973, 822.5, 135.5}, {977.5, 
           828.5, 141.5}, {975.5, 825, 138}, {974.5, 824, 137}, {975.5, 826.5,
            139.5}, {973.5, 823, 136}, {971.5, 819.5, 132.5}, {970.5, 818.5, 
           131.5}, {968.5, 815.5, 128.5}, {776, 814, 119.5}, {
           820.5, 1036, 348.5}, {1025, 1061.5, 381.5}, {981, 840, 153}, {
           966.5, 812.5, 125.5}, {965.5, 811.5, 124.5}, {992, 1015, 324.5}, {
           998.5, 1025.5, 335}, {987.5, 853.5, 166.5}, {995.5, 870.5, 
           183.5}, {732, 772, 200.5}, {465, 673, 217.5}, {460.5, 668.5, 
           213.5}, {350, 449.5, 221.5}, {346.5, 446, 217.5}, {461.5, 669.5, 
           213.5}, {461, 669, 213}, {457, 665, 209.5}, {465, 673, 217.5}, {
           466, 674, 218.5}, {556.5, 775.5, 197}, {873.5, 891, 189.5}, {
           877.5, 895.5, 194}, {562, 781.5, 202.5}, {357, 457, 117}, {
           444, 555.5, 92.5}, {438, 555.5, 199}, {344.5, 450.5, 217}, {349, 
           451, 221}, {386, 492, 154}, {524, 740.5, 68.5}, {
           382.5, 489, 151}, {245.5, 239.5, 119}, {244.5, 238.5, 118}, {357, 
           459, 229}, {354.5, 456.5, 224.5}, {237, 237, 228}, {346, 452, 
           219}, {484, 696, 132}, {483.5, 695.5, 131.5}, {727, 934, 436.5}, {
           728, 935, 437.5}, {464.5, 672.5, 216.5}, {467.5, 675.5, 220}, {
           355.5, 455.5, 213.5}, {242.5, 240.5, 221.5}, {238.5, 244.5, 
           231.5}, {244.5, 244.5, 228}, {630.5, 686.5, 366.5}, {
           847.5, 705, 268.5}, {492, 393.5, 41}, {271, 379, 143}, {453, 572, 
           213}, {451.5, 570.5, 213.5}, {238.5, 246.5, 231.5}, {
           644, 519.5, 163.5}, {644.5, 518, 154.5}, {239.5, 243.5, 113}, {
           238.5, 242.5, 8}, {239.5, 243.5, 113}, {396.5, 506.5, 153}, {505.5,
            717.5, 146.5}, {734.5, 693, 121.5}, {1010.5, 718.5, 31.5}, {
           1009.5, 920, 262.5}, {733, 903.5, 351}, {372, 589.5, 123.5}, {
           263.5, 363.5, 135}, {463.5, 460, 228}, {481.5, 684.5, 119}, {382.5,
            693.5, 23.5}, {365, 471.5, 130}, {246, 355.5, 122}, {626.5, 738.5,
            106.5}, {963, 1003, 113}, {986.5, 898, 64.5}, {986.5, 897.5, 
           163.5}, {713.5, 861, 138.5}, {755.5, 712.5, 32.5}, {
           1008, 707, 19.5}, {968, 709.5, 30}, {955.5, 775.5, 96}, {
           954.5, 774.5, 95}, {1000, 788.5, 101.5}, {1074.5, 869, 167.5}, {
           1075, 869.5, 168}, {1076.5, 1036, 406}, {1075, 1035, 404.5}, {
           1047.5, 809, 113.5}, {993, 779.5, 92}, {1015.5, 831.5, 137.5}, {
           1066.5, 853.5, 151.5}, {1064, 846, 144}, {1061.5, 838.5, 136.5}, {
           1046.5, 801.5, 106}, {1055.5, 820, 125}, {757.5, 760, 181}, {
           733.5, 701.5, 130}, {759.5, 731, 44.5}, {454.5, 666, 95}, {684, 
           695, 131}, {964.5, 798.5, 119}, {965, 799, 119.5}, {782.5, 804.5, 
           117.5}, {815, 786, 91.5}, {966.5, 712.5, 32.5}, {
           772.5, 778, 90.5}, {848.5, 859, 157}, {1070, 858.5, 156.5}, {
           1068.5, 1025.5, 390.5}, {1070, 1028.5, 393.5}, {1070.5, 860.5, 
           158.5}, {851.5, 861.5, 159.5}, {824, 798.5, 103.5}, {
           1014.5, 730, 42.5}, {1023.5, 744.5, 58}, {1033.5, 764, 77.5}, {
           1068, 836, 142}, {1101, 906, 205}, {1100.5, 905.5, 204.5}, {
           1062, 827, 133.5}, {1026, 752.5, 66}, {1062.5, 829, 135}, {
           781, 787.5, 209}, {398.5, 606.5, 150.5}, {624.5, 618.5, 54}, {
           917.5, 697.5, 24.5}, {914.5, 694, 21}, {969.5, 723, 43}, {
           1063, 829.5, 135.5}, {854, 872, 170.5}, {822, 803, 108.5}, {1069.5,
            837.5, 143.5}, {1064, 829.5, 135.5}, {1043.5, 969.5, 338}, {
           1046.5, 973.5, 342}, {739, 707, 136}, {427, 635, 179.5}, {
           721, 690.5, 119}, {1032, 761.5, 74.5}, {1026.5, 753.5, 66.5}, {
           1045, 798.5, 104}, {992, 772.5, 85}, {999.5, 788, 100.5}, {
           840.5, 854, 152}, {839.5, 853, 151}, {1071, 860, 158}, {
           1083.5, 876.5, 175}, {1059.5, 820.5, 127}, {969, 717.5, 38}, {
           574.5, 470, 127.5}, {579.5, 477, 135}, {750, 982.5, 306.5}, {
           750.5, 983, 307}, {931.5, 715.5, 43.5}, {984, 742, 62.5}, {
           984.5, 742.5, 63}, {984, 976, 298.5}, {782.5, 990, 305}, {
           735.5, 744, 64.5}, {944, 746, 74}, {933, 724, 51.5}, {
           716.5, 714, 34}, {540.5, 941, 304}, {536.5, 937.5, 300.5}, {
           770.5, 933.5, 293}, {993, 962, 329}, {662.5, 680.5, 116.5}, {383.5,
            591.5, 135.5}, {723.5, 884, 363}, {1036.5, 952, 315.5}, {
           681.5, 644.5, 73}, {640, 638, 74}, {809, 694.5, 140}, {852, 704, 
           142}, {853, 713, 143}, {856, 955.5, 385.5}, {1041.5, 1025, 330}, {
           869, 648.5, 173.5}, {808, 598, 138.5}, {959, 780, 100.5}, {
           1002, 789.5, 102.5}, {747, 871, 140.5}, {746, 870, 139.5}, {
           1018.5, 939.5, 288}, {789, 957, 305.5}, {438, 649.5, 78}, {
           687, 673.5, 95}, {797, 787.5, 93.5}, {732, 732, 52.5}, {928.5, 
           712.5, 40.5}, {937.5, 733.5, 61.5}, {733, 745, 65.5}, {
           722, 721, 41.5}}]], 
        Polygon3DBox[
         NCache[{{Rational[2111, 2], 1132, 286, 286}, {
           815, Rational[2339, 2], Rational[753, 2], 128}, {
           781, Rational[1701, 2], Rational[699, 2], Rational[203, 2]}, {
           Rational[1579, 2], 857, 358, 110}, {
           Rational[1655, 2], Rational[2357, 2], Rational[777, 2], Rational[
            281, 2]}, {827, 1178, 388, 140}, {
           Rational[1647, 2], 1176, 385, Rational[273, 2]}, {
           787, 1159, 356, Rational[215, 2]}, {
           789, Rational[2319, 2], Rational[715, 2], Rational[219, 2]}, {
           793, 1161, 362, Rational[227, 2]}, {
           Rational[1529, 2], 849, 341, Rational[185, 2]}, {753, 845, 329, 
           81}, {734, Rational[2273, 2], Rational[621, 2], 62}, {
           Rational[1459, 2], Rational[2269, 2], Rational[613, 2], Rational[
            115, 2]}, {726, Rational[2265, 2], Rational[605, 2], 54}, {
           727, Rational[2267, 2], Rational[607, 2], 55}, {
           738, Rational[2277, 2], 315, 66}, {
           Rational[1473, 2], 1137, Rational[627, 2], Rational[129, 2]}, {
           Rational[1587, 2], Rational[2329, 2], Rational[725, 2], 114}, {
           793, 854, 362, Rational[227, 2]}, {
           941, Rational[1081, 2], Rational[665, 2], Rational[415, 2]}, {
           949, Rational[1097, 2], Rational[681, 2], Rational[431, 2]}, {
           740, Rational[1665, 2], Rational[633, 2], 68}, {
           Rational[1481, 2], Rational[2277, 2], 317, Rational[137, 2]}, {
           794, Rational[2329, 2], 363, Rational[229, 2]}, {
           795, Rational[2331, 2], 364, Rational[231, 2]}, {
           781, 1156, Rational[701, 2], Rational[203, 2]}, {
           Rational[1627, 2], 1168, 375, Rational[253, 2]}, {
           Rational[1571, 2], Rational[2311, 2], Rational[709, 2], 106}, {
           786, 1156, 355, Rational[213, 2]}, {
           815, Rational[2339, 2], Rational[753, 2], 128}, {
           Rational[1631, 2], 1170, 377, Rational[257, 2]}, {
           Rational[1621, 2], 1166, 372, 124}, {
           Rational[1623, 2], Rational[1749, 2], Rational[747, 2], 125}, {
           Rational[1591, 2], Rational[1733, 2], 365, 116}, {
           Rational[1779, 2], 1221, Rational[1037, 2], 167}, {
           Rational[2165, 2], 1049, 394, 403}, {
           1036, Rational[1489, 2], 395, 400}, {
           Rational[1685, 2], Rational[1245, 2], 519, Rational[327, 2]}, {
           796, Rational[1731, 2], Rational[731, 2], Rational[233, 2]}, {818, 
           1172, 380, 131}, {
           Rational[1555, 2], Rational[1693, 2], Rational[693, 2], 98}, {
           716, 816, 289, Rational[87, 2]}, {
           Rational[1585, 2], 1158, 358, Rational[225, 2]}, {824, 1217, 385, 
           137}, {Rational[1517, 2], Rational[2459, 2], Rational[639, 2], 
            Rational[143, 2]}, {
           Rational[1515, 2], Rational[2457, 2], Rational[637, 2], Rational[
            141, 2]}, {762, Rational[2463, 2], Rational[645, 2], 75}, {
           Rational[1535, 2], Rational[2471, 2], Rational[655, 2], Rational[
            161, 2]}, {764, Rational[2467, 2], Rational[649, 2], 77}, {
           763, Rational[2465, 2], Rational[647, 2], 76}, {
           Rational[1629, 2], Rational[1867, 2], Rational[751, 2], Rational[
            255, 2]}, {
           Rational[1615, 2], 929, Rational[737, 2], Rational[241, 2]}, {792, 
           1203, 353, 105}, {
           Rational[1597, 2], 1207, Rational[719, 2], Rational[223, 2]}, {
           764, Rational[2465, 2], Rational[649, 2], Rational[155, 2]}, {
           810, 1213, 371, Rational[247, 2]}, {
           804, 1209, Rational[729, 2], 117}, {760, 888, 320, 73}, {
           Rational[1565, 2], Rational[1803, 2], Rational[677, 2], 91}, {
           797, Rational[2503, 2], Rational[697, 2], Rational[201, 2]}, {
           794, 1250, Rational[693, 2], 99}, {
           792, 1249, Rational[691, 2], 98}, {
           793, Rational[2499, 2], Rational[693, 2], Rational[197, 2]}, {
           Rational[1563, 2], Rational[2487, 2], 339, 91}, {
           Rational[1541, 2], Rational[2475, 2], Rational[663, 2], Rational[
            167, 2]}, {
           Rational[1523, 2], Rational[1773, 2], 322, Rational[149, 2]}, {
           Rational[1503, 2], Rational[1759, 2], Rational[623, 2], Rational[
            129, 2]}, {
           Rational[1505, 2], Rational[2449, 2], 313, Rational[131, 2]}, {
           Rational[1503, 2], 1224, Rational[625, 2], 65}, {
           752, Rational[2449, 2], 313, Rational[131, 2]}, {
           779, Rational[2481, 2], 334, 86}, {
           830, Rational[2449, 2], 385, 137}, {
           Rational[1723, 2], Rational[2393, 2], 423, Rational[349, 2]}, {
           880, Rational[1863, 2], 442, 193}, {871, 641, 433, 184}, {
           Rational[1695, 2], Rational[1807, 2], Rational[819, 2], Rational[
            321, 2]}, {872, Rational[2395, 2], 434, 185}, {
           892, 931, Rational[907, 2], 205}, {
           Rational[1769, 2], 930, 446, Rational[395, 2]}, {
           Rational[1761, 2], Rational[2405, 2], Rational[885, 2], Rational[
            387, 2]}, {784, Rational[1883, 2], 334, Rational[427, 2]}, {
           Rational[1537, 2], 936, Rational[637, 2], 198}, {
           768, Rational[1877, 2], Rational[643, 2], Rational[395, 2]}, {
           783, Rational[1335, 2], Rational[673, 2], Rational[425, 2]}, {
           Rational[1553, 2], Rational[1115, 2], 453, Rational[865, 2]}, {
           769, Rational[1663, 2], Rational[891, 2], 425}, {
           Rational[1715, 2], 1195, Rational[839, 2], Rational[341, 2]}, {
           847, Rational[2381, 2], Rational[817, 2], 160}, {
           Rational[1715, 2], Rational[1829, 2], Rational[837, 2], Rational[
            341, 2]}, {
           Rational[1745, 2], Rational[1841, 2], Rational[867, 2], Rational[
            371, 2]}, {
           Rational[1739, 2], Rational[2399, 2], 431, Rational[365, 2]}, {
           861, 912, Rational[845, 2], 174}, {848, 617, 409, 161}, {
           Rational[1681, 2], Rational[1795, 2], Rational[803, 2], Rational[
            307, 2]}, {
           Rational[1677, 2], Rational[2371, 2], 400, Rational[303, 2]}, {
           839, 1186, Rational[801, 2], 152}, {
           846, Rational[2379, 2], Rational[815, 2], 159}, {
           Rational[1685, 2], 1188, Rational[809, 2], Rational[311, 2]}, {834,
            1183, 396, 147}, {
           Rational[1667, 2], Rational[2365, 2], Rational[791, 2], Rational[
            293, 2]}, {817, 1218, 373, Rational[249, 2]}, {
           794, 1250, Rational[693, 2], 99}, {
           787, Rational[2493, 2], Rational[685, 2], Rational[191, 2]}, {
           750, 1186, Rational[633, 2], Rational[137, 2]}, {
           Rational[1429, 2], Rational[2253, 2], Rational[583, 2], Rational[
            85, 2]}, {
           Rational[1431, 2], Rational[2255, 2], Rational[585, 2], Rational[
            87, 2]}, {708, 1122, Rational[567, 2], Rational[71, 2]}, {
           Rational[1399, 2], Rational[2231, 2], 273, Rational[53, 2]}, {
           Rational[1699, 2], 794, 261, 21}, {
           Rational[1689, 2], Rational[1579, 2], 251, Rational[33, 2]}, {
           Rational[1381, 2], 1107, 254, 18}, {
           Rational[1405, 2], Rational[2233, 2], 273, 30}, {
           Rational[1411, 2], Rational[2239, 2], 280, 33}, {
           Rational[1389, 2], Rational[2221, 2], 262, Rational[43, 2]}, {694, 
           1110, 261, 21}, {
           Rational[1391, 2], Rational[2223, 2], 264, Rational[45, 2]}, {
           Rational[1415, 2], Rational[2239, 2], Rational[557, 2], 35}, {
           Rational[1433, 2], Rational[2253, 2], 293, Rational[89, 2]}, {
           763, Rational[2293, 2], 332, Rational[167, 2]}, {
           768, Rational[2299, 2], Rational[673, 2], Rational[177, 2]}, {
           Rational[1513, 2], Rational[2285, 2], 329, 81}, {
           810, Rational[2335, 2], 375, 127}, {
           768, 1150, Rational[673, 2], Rational[177, 2]}, {
           Rational[1415, 2], Rational[2241, 2], Rational[567, 2], Rational[
            71, 2]}, {707, 1120, 283, 35}, {
           Rational[1417, 2], 1121, 285, Rational[73, 2]}, {
           Rational[1431, 2], Rational[1627, 2], 292, Rational[87, 2]}, {
           Rational[1425, 2], 811, Rational[577, 2], Rational[81, 2]}, {
           703, Rational[2235, 2], Rational[557, 2], Rational[61, 2]}, {
           702, Rational[1605, 2], 278, 30}, {
           698, Rational[1597, 2], 269, 26}, {
           762, 855, Rational[651, 2], 82}, {
           782, 908, Rational[687, 2], Rational[191, 2]}, {
           Rational[1451, 2], Rational[2425, 2], Rational[573, 2], Rational[
            79, 2]}, {
           719, Rational[2421, 2], Rational[559, 2], Rational[65, 2]}, {
           Rational[1441, 2], Rational[1715, 2], Rational[561, 2], Rational[
            67, 2]}, {Rational[1499, 2], Rational[1747, 2], 310, 63}, {
           780, Rational[2487, 2], Rational[679, 2], Rational[185, 2]}, {
           779, 902, Rational[677, 2], 91}, {
           Rational[1551, 2], 901, Rational[673, 2], Rational[177, 2]}, {
           Rational[1559, 2], Rational[2487, 2], Rational[677, 2], 91}, {
           Rational[1495, 2], 1227, Rational[607, 2], 59}, {
           Rational[1485, 2], 1225, Rational[601, 2], Rational[111, 2]}, {773,
            1240, 334, 86}, {
           Rational[1545, 2], Rational[2479, 2], Rational[667, 2], Rational[
            171, 2]}, {739, 864, 293, 52}, {
           704, Rational[1695, 2], 251, Rational[35, 2]}, {
           Rational[1413, 2], Rational[2415, 2], 257, 20}, {
           719, 860, Rational[553, 2], 32}, {
           Rational[1463, 2], 864, 292, Rational[89, 2]}, {
           731, 1215, Rational[583, 2], 44}, {726, 1213, 287, 39}, {
           Rational[1451, 2], Rational[2425, 2], Rational[573, 2], Rational[
            77, 2]}, {800, 1207, 361, 113}, {
           805, 1209, Rational[731, 2], 118}, {
           Rational[1479, 2], 1219, 300, Rational[105, 2]}, {
           Rational[1483, 2], Rational[2439, 2], Rational[605, 2], Rational[
            109, 2]}, {Rational[1477, 2], 869, 299, Rational[103, 2]}, {
           739, Rational[1739, 2], Rational[599, 2], 52}, {
           Rational[1485, 2], Rational[1747, 2], Rational[607, 2], Rational[
            111, 2]}, {743, 874, 304, 56}, {
           719, 1213, Rational[565, 2], Rational[63, 2]}, {
           752, 1231, 308, Rational[115, 2]}, {
           755, 1282, Rational[1037, 2], 68}, {
           Rational[1895, 2], 1149, 424, 221}, {
           Rational[2387, 2], Rational[1981, 2], 168, Rational[831, 2]}, {
           Rational[2383, 2], Rational[1835, 2], Rational[313, 2], Rational[
            1031, 2]}, {
           Rational[2381, 2], Rational[1833, 2], 153, Rational[1025, 2]}, {
           Rational[2383, 2], Rational[1977, 2], Rational[327, 2], Rational[
            823, 2]}, {
           Rational[2381, 2], Rational[1975, 2], Rational[325, 2], Rational[
            821, 2]}, {1206, Rational[1735, 2], Rational[239, 2], 471}, {
           1168, 723, Rational[87, 2], Rational[1015, 2]}, {
           1155, Rational[1557, 2], Rational[167, 2], Rational[1111, 2]}, {
           1156, Rational[1559, 2], Rational[169, 2], Rational[1113, 2]}, {
           Rational[2233, 2], Rational[1401, 2], Rational[25, 2], Rational[
            971, 2]}, {1116, 700, 12, 485}, {
           1155, Rational[1691, 2], Rational[183, 2], Rational[903, 2]}, {
           Rational[2383, 2], 989, Rational[329, 2], Rational[823, 2]}, {
           Rational[2375, 2], Rational[1971, 2], Rational[313, 2], Rational[
            807, 2]}, {1186, 984, 153, Rational[801, 2]}, {
           Rational[2367, 2], Rational[1963, 2], Rational[297, 2], Rational[
            793, 2]}, {1183, 981, 148, 396}, {
           Rational[1893, 2], 1052, 217, 217}, {
           943, 1049, Rational[425, 2], 212}, {
           Rational[2295, 2], Rational[1681, 2], 79, 438}, {
           Rational[2301, 2], 843, 83, Rational[885, 2]}, {1190, 987, 163, 
           411}, {Rational[2383, 2], Rational[1977, 2], Rational[333, 2], 
            414}, {Rational[2367, 2], Rational[1963, 2], Rational[301, 2], 
            398}, {Rational[2361, 2], 904, Rational[273, 2], 496}, {1179, 903,
            134, 493}, {
           Rational[2357, 2], Rational[1955, 2], Rational[283, 2], Rational[
            777, 2]}, {Rational[2291, 2], 837, 75, 435}, {
           Rational[2225, 2], Rational[1393, 2], Rational[17, 2], Rational[
            963, 2]}, {1116, Rational[1403, 2], 14, 486}, {
           1014, 1006, Rational[475, 2], 349}, {
           Rational[1819, 2], Rational[2613, 2], Rational[913, 2], Rational[
            417, 2]}, {907, 1304, 454, 206}, {
           903, Rational[2599, 2], Rational[899, 2], 202}, {
           Rational[1805, 2], Rational[2597, 2], Rational[897, 2], Rational[
            403, 2]}, {1005, 995, 226, Rational[677, 2]}, {
           1056, 688, Rational[461, 2], Rational[685, 2]}, {836, 572, 347, 
           220}, {905, 606, 151, Rational[767, 2]}, {
           Rational[2249, 2], 721, 34, Rational[1011, 2]}, {
           1117, Rational[1405, 2], Rational[31, 2], Rational[975, 2]}, {
           905, Rational[993, 2], 132, Rational[729, 2]}, {
           Rational[1683, 2], Rational[1585, 2], Rational[917, 2], 228}, {
           Rational[2201, 2], 1152, Rational[869, 2], 331}, {
           1250, Rational[2297, 2], Rational[861, 2], Rational[1107, 2]}, {
           Rational[2205, 2], Rational[2603, 2], 563, Rational[877, 2]}, {
           1015, Rational[2015, 2], 239, 351}, {1124, 712, 25, 497}, {
           Rational[2247, 2], Rational[1423, 2], Rational[49, 2], Rational[
            993, 2]}, {1125, 713, 26, 498}, {
           Rational[2253, 2], Rational[1429, 2], Rational[55, 2], Rational[
            999, 2]}, {
           Rational[2033, 2], 1009, Rational[481, 2], Rational[705, 2]}, {
           1013, 1005, Rational[473, 2], 348}, {
           1115, Rational[1401, 2], 13, Rational[969, 2]}, {
           1009, Rational[1999, 2], Rational[461, 2], Rational[685, 2]}, {
           Rational[2029, 2], Rational[2013, 2], 238, Rational[699, 2]}, {
           1014, 1006, Rational[475, 2], 349}, {
           1008, Rational[1997, 2], Rational[459, 2], Rational[683, 2]}, {
           Rational[2217, 2], Rational[1385, 2], Rational[9, 2], Rational[
            953, 2]}, {
           1160, Rational[1417, 2], Rational[57, 2], Rational[985, 2]}, {
           Rational[2425, 2], 869, Rational[91, 2], 398}, {
           Rational[2427, 2], Rational[2027, 2], Rational[77, 2], Rational[
            575, 2]}, {Rational[2367, 2], 984, 75, 323}, {
           1164, 964, 123, Rational[741, 2]}, {
           1171, Rational[1941, 2], Rational[261, 2], 378}, {
           1168, Rational[1781, 2], 119, 478}, {
           Rational[2337, 2], 891, Rational[239, 2], Rational[957, 2]}, {
           1169, Rational[1937, 2], Rational[257, 2], 376}, {
           Rational[2309, 2], 854, 91, Rational[901, 2]}, {1138, 840, 59, 
           418}, {1135, 939, Rational[123, 2], 309}, {
           Rational[2273, 2], 940, 65, 313}, {
           1146, 948, Rational[183, 2], 339}, {
           1145, Rational[1895, 2], Rational[177, 2], Rational[671, 2]}, {
           1137, 941, Rational[127, 2], 311}, {1135, 939, 59, 307}, {
           Rational[2305, 2], 954, Rational[189, 2], Rational[685, 2]}, {1172,
            971, 132, 380}, {
           Rational[2331, 2], 965, Rational[249, 2], Rational[745, 2]}, {
           1156, Rational[1913, 2], 114, Rational[723, 2]}, {
           1157, Rational[1915, 2], 115, Rational[725, 2]}, {
           Rational[2323, 2], Rational[1923, 2], 119, 367}, {
           1161, 961, Rational[237, 2], Rational[733, 2]}, {
           Rational[2325, 2], Rational[1925, 2], 121, Rational[737, 2]}, {
           Rational[2321, 2], Rational[1757, 2], Rational[915, 2], 470}, {
           1156, 874, Rational[905, 2], Rational[931, 2]}, {
           Rational[2329, 2], 964, Rational[247, 2], Rational[743, 2]}, {
           1164, Rational[1927, 2], 123, 371}, {
           Rational[2321, 2], Rational[1921, 2], 119, Rational[733, 2]}, {
           1064, 1064, 643, Rational[1333, 2]}, {
           Rational[2127, 2], Rational[2127, 2], Rational[1285, 2], 666}, {
           Rational[2305, 2], Rational[1709, 2], Rational[181, 2], 450}, {
           1138, 842, Rational[117, 2], Rational[837, 2]}, {
           Rational[2277, 2], 942, 67, 315}, {
           Rational[2345, 2], Rational[1943, 2], 135, 383}, {
           Rational[2411, 2], 854, Rational[233, 2], Rational[937, 2]}, {
           1205, Rational[1707, 2], 116, 468}, {
           Rational[2401, 2], 996, 187, 435}, {
           1200, Rational[1869, 2], 176, Rational[1071, 2]}, {
           Rational[2405, 2], Rational[1763, 2], Rational[359, 2], Rational[
            1301, 2]}, {Rational[2413, 2], 949, 105, 462}, {
           1213, 1013, Rational[71, 2], 281}, {1216, 1016, 47, 295}, {
           Rational[2425, 2], Rational[2025, 2], Rational[77, 2], Rational[
            573, 2]}, {1213, 1013, 39, 287}, {
           1217, 878, Rational[115, 2], Rational[819, 2]}, {
           Rational[2429, 2], Rational[1751, 2], 50, 402}, {
           1176, Rational[1955, 2], 53, 301}, {
           1173, Rational[1943, 2], 142, 390}, {
           Rational[2411, 2], Rational[1695, 2], Rational[233, 2], 469}, {
           1175, 725, 45, 509}, {Rational[2293, 2], 852, 87, 446}, {1150, 951,
            108, 355}, {1147, 949, Rational[191, 2], Rational[685, 2]}, {
           Rational[2293, 2], Rational[1897, 2], 95, 342}, {
           Rational[2357, 2], 979, 62, Rational[605, 2]}, {
           Rational[2355, 2], 978, Rational[121, 2], Rational[603, 2]}, {
           1147, Rational[1897, 2], Rational[195, 2], Rational[691, 2]}, {
           Rational[2365, 2], Rational[1967, 2], Rational[143, 2], 320}, {
           1183, 894, Rational[135, 2], Rational[855, 2]}, {
           Rational[2323, 2], 872, Rational[221, 2], Rational[939, 2]}, {
           Rational[2321, 2], 961, 114, Rational[723, 2]}, {
           Rational[2319, 2], 960, Rational[223, 2], Rational[719, 2]}, {
           1158, Rational[1719, 2], Rational[193, 2], Rational[913, 2]}, {
           1144, Rational[1507, 2], Rational[133, 2], Rational[1077, 2]}, {
           Rational[2291, 2], Rational[1521, 2], Rational[147, 2], Rational[
            1091, 2]}, {
           1154, Rational[1565, 2], Rational[877, 2], Rational[1707, 2]}, {
           Rational[2331, 2], 809, Rational[915, 2], 872}, {
           Rational[1973, 2], 988, Rational[479, 2], Rational[701, 2]}, {
           802, 1161, Rational[733, 2], Rational[235, 2]}, {
           798, Rational[2317, 2], Rational[729, 2], Rational[231, 2]}, {799, 
           1159, 365, 116}, {984, 986, 238, 349}, {1166, 811, 109, 580}, {
           1168, Rational[1627, 2], Rational[223, 2], 583}, {
           Rational[2337, 2], 814, 112, Rational[1167, 2]}, {
           Rational[1959, 2], Rational[1967, 2], Rational[471, 2], Rational[
            693, 2]}, {
           Rational[1957, 2], 982, Rational[469, 2], Rational[691, 2]}, {
           Rational[1959, 2], Rational[1965, 2], 235, 346}, {
           Rational[1943, 2], Rational[1933, 2], 226, Rational[675, 2]}, {
           Rational[1951, 2], Rational[1937, 2], 228, Rational[679, 2]}, {
           1007, Rational[1881, 2], Rational[415, 2], Rational[621, 2]}, {
           1213, Rational[1453, 2], 55, 510}, {
           Rational[2431, 2], 734, Rational[125, 2], Rational[1035, 2]}, {
           1215, 731, 59, Rational[1029, 2]}, {
           Rational[2417, 2], 804, 117, 581}, {
           1208, Rational[1607, 2], Rational[233, 2], Rational[1161, 2]}, {
           1206, 796, 109, 573}, {1208, 801, 114, Rational[1155, 2]}, {
           Rational[2437, 2], 739, 67, Rational[1045, 2]}, {
           1215, 730, Rational[117, 2], 514}, {
           Rational[2431, 2], Rational[1461, 2], 59, Rational[1029, 2]}, {
           1215, 727, 55, 511}, {1214, 726, 54, 510}, {
           1213, 722, Rational[101, 2], 506}, {
           Rational[2409, 2], Rational[1583, 2], 105, Rational[1137, 2]}, {
           Rational[1983, 2], Rational[2029, 2], 266, 378}, {
           785, Rational[2303, 2], Rational[713, 2], Rational[217, 2]}, {
           983, 990, Rational[483, 2], 353}, {
           Rational[2363, 2], Rational[1667, 2], Rational[263, 2], Rational[
            1205, 2]}, {1181, 833, 131, 602}, {1177, 826, 124, 595}, {
           Rational[2357, 2], Rational[1655, 2], Rational[251, 2], Rational[
            1193, 2]}, {1176, Rational[1647, 2], Rational[243, 2], 593}, {
           Rational[2347, 2], Rational[1639, 2], Rational[235, 2], Rational[
            1179, 2]}, {Rational[2355, 2], 826, 124, 596}, {
           Rational[2363, 2], Rational[1665, 2], Rational[261, 2], Rational[
            1205, 2]}, {Rational[2357, 2], 827, 125, 597}, {
           1179, Rational[1655, 2], Rational[251, 2], Rational[1195, 2]}, {
           1184, 835, 133, 605}, {
           Rational[2369, 2], Rational[1671, 2], Rational[267, 2], Rational[
            1211, 2]}, {Rational[1963, 2], Rational[1985, 2], 244, 356}, {
           983, 996, Rational[495, 2], 359}, {
           Rational[2381, 2], Rational[1699, 2], Rational[295, 2], Rational[
            1237, 2]}, {1182, Rational[1673, 2], Rational[269, 2], 606}, {
           1171, 817, 115, 587}, {
           Rational[1957, 2], 985, Rational[473, 2], Rational[697, 2]}, {
           784, Rational[2303, 2], Rational[713, 2], Rational[217, 2]}, {785, 
           1152, 357, 109}, {991, Rational[2011, 2], 257, Rational[737, 2]}, {
           Rational[2393, 2], Rational[1723, 2], Rational[319, 2], 631}, {
           Rational[2389, 2], 857, 155, 627}, {
           1194, Rational[1713, 2], Rational[309, 2], Rational[1253, 2]}, {
           1195, Rational[1715, 2], Rational[311, 2], Rational[1255, 2]}, {
           Rational[2381, 2], 847, 145, Rational[1233, 2]}, {
           1190, Rational[1693, 2], Rational[289, 2], 616}, {1191, 849, 147, 
           619}, {Rational[2351, 2], Rational[1809, 2], Rational[265, 2], 
            Rational[985, 2]}, {
           Rational[2361, 2], 917, 145, Rational[1009, 2]}, {
           1034, Rational[2147, 2], Rational[585, 2], Rational[809, 2]}, {
           Rational[1715, 2], 1270, Rational[807, 2], 156}, {
           Rational[1693, 2], Rational[2531, 2], Rational[785, 2], Rational[
            289, 2]}, {847, 941, 393, 145}, {
           Rational[1679, 2], 939, Rational[771, 2], Rational[275, 2]}, {
           838, Rational[2525, 2], 384, 136}, {
           Rational[1683, 2], 1263, Rational[775, 2], 140}, {
           835, Rational[1863, 2], 381, Rational[267, 2]}, {824, 593, 370, 
           122}, {Rational[1639, 2], Rational[1843, 2], Rational[731, 2], 
            118}, {838, Rational[2525, 2], 384, 137}, {
           839, 1263, Rational[769, 2], Rational[275, 2]}, {
           820, 1258, Rational[731, 2], 118}, {
           Rational[1645, 2], Rational[2517, 2], Rational[737, 2], Rational[
            241, 2]}, {Rational[1685, 2], 1264, 388, Rational[281, 2]}, {
           848, Rational[1877, 2], 393, 146}, {
           870, 952, Rational[831, 2], Rational[337, 2]}, {
           Rational[2029, 2], Rational[2017, 2], 240, 352}, {
           Rational[2261, 2], Rational[1445, 2], Rational[71, 2], Rational[
            1013, 2]}, {1130, 722, 35, 506}, {
           Rational[2269, 2], Rational[1465, 2], Rational[91, 2], 517}, {
           Rational[2273, 2], 736, 49, 521}, {
           Rational[1973, 2], Rational[1991, 2], 213, Rational[651, 2]}, {
           987, 996, Rational[427, 2], 326}, {1135, 730, 43, 515}, {
           1018, Rational[2027, 2], 245, Rational[715, 2]}, {
           Rational[2035, 2], 1013, Rational[489, 2], 357}, {
           1018, Rational[2027, 2], 245, Rational[715, 2]}, {
           Rational[2039, 2], Rational[2035, 2], 249, 361}, {
           Rational[2275, 2], Rational[1475, 2], Rational[101, 2], Rational[
            1043, 2]}, {1171, Rational[1633, 2], 122, Rational[1187, 2]}, {
           1155, 1095, 319, Rational[863, 2]}, {996, 1290, 438, 191}, {
           874, Rational[2557, 2], Rational[839, 2], 172}, {
           Rational[1721, 2], Rational[2541, 2], Rational[813, 2], Rational[
            317, 2]}, {
           Rational[1723, 2], Rational[2543, 2], Rational[815, 2], Rational[
            319, 2]}, {Rational[1719, 2], 1271, 405, Rational[315, 2]}, {
           859, Rational[2541, 2], Rational[809, 2], 157}, {
           869, Rational[2551, 2], 415, 167}, {
           Rational[1733, 2], 952, 412, Rational[329, 2]}, {
           866, Rational[1903, 2], Rational[823, 2], 164}, {
           Rational[1745, 2], Rational[2553, 2], 418, Rational[341, 2]}, {
           867, 954, Rational[825, 2], 165}, {
           Rational[1735, 2], Rational[1909, 2], 413, Rational[331, 2]}, {
           872, Rational[1917, 2], 417, 170}, {
           Rational[1745, 2], 959, Rational[835, 2], Rational[341, 2]}, {879, 
           1281, 424, 177}, {882, 1283, Rational[855, 2], 180}, {
           885, Rational[2571, 2], 431, 183}, {
           Rational[1771, 2], 1286, Rational[863, 2], Rational[367, 2]}, {
           Rational[1759, 2], Rational[1931, 2], Rational[851, 2], Rational[
            355, 2]}, {
           Rational[1757, 2], Rational[1929, 2], Rational[849, 2], Rational[
            353, 2]}, {880, Rational[2563, 2], 426, Rational[357, 2]}, {
           884, Rational[2567, 2], Rational[859, 2], Rational[365, 2]}, {
           888, Rational[2573, 2], Rational[867, 2], 186}, {
           Rational[1779, 2], 1288, 435, Rational[375, 2]}, {
           889, Rational[2575, 2], Rational[869, 2], 187}, {
           Rational[1783, 2], Rational[2579, 2], Rational[875, 2], Rational[
            379, 2]}, {891, 974, Rational[873, 2], 189}, {
           Rational[1783, 2], Rational[1949, 2], 437, Rational[379, 2]}, {
           895, Rational[1957, 2], 441, 193}, {
           Rational[1791, 2], 979, Rational[883, 2], Rational[387, 2]}, {
           Rational[2027, 2], Rational[2017, 2], 239, Rational[701, 2]}, {
           1165, 859, 108, Rational[935, 2]}, {1210, 1008, 120, 368}, {
           1164, 967, Rational[79, 2], Rational[561, 2]}, {
           Rational[2111, 2], Rational[1823, 2], Rational[255, 2], 242}, {
           Rational[2311, 2], 910, 230, Rational[911, 2]}, {
           Rational[2611, 2], Rational[1817, 2], Rational[445, 2], Rational[
            1357, 2]}, {1306, 909, 223, 679}, {
           Rational[2309, 2], Rational[1833, 2], 226, Rational[909, 2]}, {
           1003, 963, 229, 230}, {1135, 925, 195, Rational[847, 2]}, {
           1133, Rational[1537, 2], Rational[387, 2], 421}, {
           1110, 855, Rational[281, 2], Rational[529, 2]}, {
           1262, Rational[1927, 2], Rational[275, 2], 490}, {
           1300, Rational[1807, 2], Rational[435, 2], 673}, {
           Rational[2599, 2], 903, 217, Rational[1345, 2]}, {
           Rational[2605, 2], Rational[1811, 2], Rational[439, 2], Rational[
            1351, 2]}, {
           1205, Rational[1819, 2], Rational[239, 2], Rational[931, 2]}, {
           Rational[2405, 2], Rational[1815, 2], Rational[235, 2], 463}, {
           1298, 902, 216, 671}, {
           Rational[2597, 2], Rational[1805, 2], Rational[433, 2], Rational[
            1343, 2]}, {Rational[2599, 2], 1101, 333, 561}, {
           1298, Rational[2199, 2], Rational[663, 2], Rational[1119, 2]}, {
           1201, 906, 116, 461}, {
           1173, Rational[1685, 2], Rational[119, 2], 405}, {1240, 773, 101, 
           557}, {Rational[2483, 2], Rational[1549, 2], Rational[205, 2], 
            Rational[1117, 2]}, {1237, 769, Rational[195, 2], 553}, {
           Rational[2465, 2], 764, Rational[185, 2], Rational[1095, 2]}, {
           Rational[2467, 2], Rational[1531, 2], Rational[187, 2], Rational[
            1097, 2]}, {
           Rational[2469, 2], Rational[1533, 2], Rational[189, 2], Rational[
            1099, 2]}, {1248, 799, Rational[239, 2], 575}, {
           1262, Rational[1671, 2], Rational[297, 2], 604}, {
           1261, Rational[1665, 2], Rational[291, 2], Rational[1201, 2]}, {
           Rational[2517, 2], 824, Rational[275, 2], Rational[1185, 2]}, {
           1260, Rational[1661, 2], Rational[289, 2], Rational[1199, 2]}, {
           1249, 803, 124, 579}, {
           Rational[2467, 2], 764, 92, Rational[1095, 2]}, {
           Rational[2461, 2], Rational[1519, 2], Rational[175, 2], Rational[
            1087, 2]}, {1234, 765, 93, 549}, {1239, 772, 100, 556}, {
           Rational[2489, 2], Rational[1341, 2], 223, Rational[903, 2]}, {
           1238, 662, Rational[429, 2], 443}, {
           Rational[2455, 2], Rational[1513, 2], Rational[169, 2], Rational[
            1081, 2]}, {
           Rational[2451, 2], Rational[1507, 2], Rational[163, 2], 537}, {
           1226, 754, 82, Rational[1075, 2]}, {
           1227, 755, 83, Rational[1077, 2]}, {
           Rational[2449, 2], Rational[1503, 2], Rational[159, 2], Rational[
            1069, 2]}, {1167, Rational[1667, 2], Rational[101, 2], 397}, {
           1116, Rational[1625, 2], 22, Rational[753, 2]}, {
           Rational[2245, 2], Rational[1637, 2], 31, Rational[781, 2]}, {
           Rational[2247, 2], Rational[1857, 2], Rational[81, 2], Rational[
            577, 2]}, {1207, Rational[1821, 2], 123, 475}, {
           Rational[2579, 2], Rational[1783, 2], Rational[409, 2], Rational[
            1321, 2]}, {
           Rational[2581, 2], Rational[1785, 2], Rational[411, 2], Rational[
            1323, 2]}, {
           Rational[2583, 2], Rational[1787, 2], Rational[413, 2], Rational[
            1325, 2]}, {1288, Rational[1779, 2], Rational[405, 2], 658}, {
           1286, 887, 200, 655}, {
           Rational[2573, 2], Rational[1775, 2], Rational[401, 2], Rational[
            1311, 2]}, {1281, 879, 192, 647}, {
           Rational[2553, 2], Rational[1743, 2], Rational[369, 2], Rational[
            1279, 2]}, {
           Rational[2561, 2], Rational[1757, 2], Rational[383, 2], Rational[
            1293, 2]}, {
           Rational[2563, 2], Rational[1761, 2], Rational[387, 2], 649}, {
           1281, Rational[1759, 2], 193, Rational[1297, 2]}, {
           Rational[2037, 2], Rational[2027, 2], Rational[527, 2], 367}, {739,
            1138, 315, 67}, {
           Rational[1419, 2], Rational[2243, 2], Rational[563, 2], 37}, {
           732, 1129, 304, Rational[119, 2]}, {754, 1143, 330, 82}, {
           Rational[1543, 2], 1197, Rational[667, 2], Rational[171, 2]}, {
           1042, 1066, 271, 375}, {1281, 878, 191, 647}, {
           Rational[2563, 2], Rational[1757, 2], Rational[383, 2], Rational[
            1295, 2]}, {1275, 867, 180, Rational[1271, 2]}, {
           1038, Rational[1819, 2], Rational[1131, 2], 715}, {
           1037, 908, Rational[1129, 2], 714}, {
           1271, Rational[1723, 2], 175, 630}, {
           Rational[2395, 2], 898, Rational[219, 2], Rational[921, 2]}, {
           Rational[2239, 2], Rational[1851, 2], Rational[71, 2], Rational[
            565, 2]}, {
           Rational[2237, 2], Rational[1849, 2], Rational[69, 2], Rational[
            563, 2]}, {
           Rational[2231, 2], 921, Rational[59, 2], Rational[549, 2]}, {
           1120, 925, Rational[69, 2], 280}, {
           1124, Rational[1859, 2], Rational[81, 2], 288}, {
           Rational[2249, 2], 930, 41, Rational[577, 2]}, {
           Rational[2253, 2], Rational[1863, 2], Rational[87, 2], Rational[
            583, 2]}, {
           Rational[2251, 2], Rational[1861, 2], Rational[85, 2], Rational[
            581, 2]}, {1119, 924, Rational[67, 2], Rational[557, 2]}, {
           Rational[2223, 2], 806, Rational[31, 2], Rational[745, 2]}, {
           1110, 805, 14, Rational[737, 2]}, {
           1191, 889, Rational[197, 2], 445}, {1272, 862, 175, 631}, {
           Rational[2545, 2], Rational[1729, 2], 178, Rational[1267, 2]}, {
           1273, 865, Rational[357, 2], 634}, {
           Rational[2565, 2], Rational[1757, 2], Rational[383, 2], Rational[
            1295, 2]}, {1205, Rational[1817, 2], Rational[237, 2], 470}, {
           Rational[2231, 2], Rational[1841, 2], Rational[55, 2], 273}, {1115,
            920, 27, 272}, {
           Rational[2231, 2], Rational[1841, 2], Rational[55, 2], 273}, {
           Rational[2383, 2], Rational[1775, 2], Rational[195, 2], Rational[
            895, 2]}, {1262, Rational[1663, 2], 145, 600}, {
           Rational[2521, 2], 827, Rational[281, 2], 596}, {
           977, Rational[1319, 2], 420, Rational[617, 2]}, {
           Rational[1991, 2], 786, 442, 227}, {
           Rational[2557, 2], Rational[2153, 2], 153, 402}, {
           Rational[2509, 2], Rational[2107, 2], Rational[205, 2], Rational[
            703, 2]}, {1258, 1057, 119, Rational[735, 2]}, {
           Rational[2525, 2], Rational[2123, 2], Rational[273, 2], Rational[
            769, 2]}, {1285, 1083, Rational[345, 2], 421}, {
           Rational[2613, 2], Rational[2207, 2], Rational[415, 2], Rational[
            913, 2]}, {
           Rational[2611, 2], Rational[2205, 2], Rational[413, 2], Rational[
            911, 2]}, {
           Rational[2561, 2], Rational[2157, 2], 156, Rational[809, 2]}, {
           1282, Rational[1757, 2], 60, 518}, {
           Rational[2561, 2], 877, Rational[115, 2], 516}, {
           Rational[2501, 2], 1050, Rational[197, 2], 347}, {
           Rational[2495, 2], Rational[2095, 2], Rational[191, 2], Rational[
            687, 2]}, {Rational[2499, 2], 1049, Rational[195, 2], 346}, {
           Rational[2503, 2], Rational[2101, 2], Rational[199, 2], Rational[
            697, 2]}, {1244, 910, Rational[197, 2], 451}, {
           1015, Rational[1339, 2], Rational[1347, 2], 673}, {
           1027, Rational[1629, 2], Rational[1377, 2], Rational[1169, 2]}, {
           1282, 1080, 166, 415}, {
           Rational[2553, 2], Rational[2149, 2], 151, 400}, {
           1248, Rational[2095, 2], 96, Rational[689, 2]}, {
           1256, Rational[2111, 2], 119, Rational[735, 2]}, {
           1255, Rational[2109, 2], 118, Rational[733, 2]}, {
           1274, Rational[2145, 2], Rational[297, 2], 397}, {
           Rational[2435, 2], Rational[2037, 2], Rational[257, 2], Rational[
            753, 2]}, {Rational[2391, 2], 883, Rational[195, 2], 449}, {
           1262, Rational[1675, 2], Rational[301, 2], Rational[1213, 2]}, {
           1248, Rational[1877, 2], 117, 469}, {
           Rational[2463, 2], Rational[2063, 2], 74, Rational[645, 2]}, {
           1228, 1028, 68, Rational[633, 2]}, {
           1229, 1029, Rational[141, 2], Rational[637, 2]}, {
           Rational[2473, 2], Rational[2073, 2], 81, Rational[659, 2]}, {
           Rational[2481, 2], Rational[2081, 2], Rational[171, 2], Rational[
            669, 2]}, {
           Rational[2479, 2], Rational[2079, 2], Rational[169, 2], Rational[
            667, 2]}, {
           Rational[2477, 2], Rational[2077, 2], Rational[167, 2], Rational[
            665, 2]}, {
           Rational[2473, 2], Rational[2073, 2], 81, Rational[659, 2]}, {
           1229, 1029, Rational[141, 2], Rational[637, 2]}, {
           Rational[2447, 2], Rational[2047, 2], Rational[125, 2], Rational[
            621, 2]}, {
           Rational[2465, 2], Rational[2065, 2], Rational[149, 2], 323}, {
           1234, Rational[1797, 2], 85, Rational[875, 2]}, {
           Rational[2483, 2], 785, Rational[211, 2], Rational[1123, 2]}, {
           Rational[2515, 2], 936, Rational[249, 2], Rational[953, 2]}, {
           Rational[2493, 2], 1046, 100, 348}, {1240, 1040, 86, 334}, {
           1252, Rational[2103, 2], Rational[215, 2], Rational[711, 2]}, {
           1251, Rational[2101, 2], Rational[213, 2], Rational[709, 2]}, {
           1236, 1036, Rational[161, 2], 329}, {
           Rational[2457, 2], Rational[2057, 2], Rational[139, 2], Rational[
            637, 2]}, {
           Rational[2455, 2], Rational[2055, 2], Rational[137, 2], Rational[
            635, 2]}, {
           Rational[2457, 2], Rational[2057, 2], Rational[139, 2], Rational[
            637, 2]}, {Rational[2495, 2], 1047, 107, 356}, {
           Rational[2555, 2], Rational[2153, 2], 166, 415}, {
           1263, Rational[2125, 2], Rational[269, 2], 383}, {
           1264, Rational[2127, 2], Rational[271, 2], 384}, {
           1296, Rational[2189, 2], Rational[393, 2], 445}, {
           Rational[2569, 2], 1083, Rational[355, 2], Rational[851, 2]}, {
           1284, Rational[2165, 2], 177, 425}, {
           1296, Rational[2189, 2], Rational[393, 2], 445}, {
           Rational[2561, 2], Rational[2159, 2], 172, Rational[841, 2]}, {
           1277, 1076, 168, 416}, {
           Rational[2571, 2], Rational[2169, 2], Rational[367, 2], Rational[
            863, 2]}, {
           Rational[2563, 2], Rational[1959, 2], Rational[371, 2], Rational[
            1075, 2]}, {
           Rational[2555, 2], Rational[1951, 2], Rational[357, 2], Rational[
            1061, 2]}, {
           Rational[2575, 2], 1086, Rational[369, 2], Rational[865, 2]}, {
           Rational[2427, 2], 1015, 125, Rational[745, 2]}, {
           1208, 1010, Rational[235, 2], 365}, {
           Rational[2557, 2], 973, 179, 531}, {
           1200, Rational[1795, 2], 112, Rational[927, 2]}, {
           Rational[2351, 2], 978, 53, 301}, {
           1261, Rational[1925, 2], Rational[273, 2], 489}, {
           Rational[2577, 2], Rational[1979, 2], 193, 545}, {
           Rational[2573, 2], 1085, Rational[367, 2], Rational[863, 2]}, {
           1288, Rational[2173, 2], 186, Rational[869, 2]}, {
           1273, 1072, 162, Rational[821, 2]}, {1275, 1074, 166, 414}, {
           1277, 1076, Rational[339, 2], 418}, {
           Rational[2559, 2], 980, Rational[361, 2], 533}, {
           Rational[2569, 2], 985, 189, Rational[1083, 2]}, {
           Rational[2561, 2], Rational[2159, 2], Rational[349, 2], Rational[
            847, 2]}, {1281, 1080, 175, 424}, {
           1283, 1082, 179, Rational[855, 2]}, {
           Rational[2579, 2], 1088, Rational[379, 2], Rational[875, 2]}, {
           Rational[2591, 2], Rational[2187, 2], Rational[395, 2], Rational[
            891, 2]}, {1285, Rational[2167, 2], 182, 430}, {
           1273, 1072, Rational[325, 2], 411}, {
           Rational[2561, 2], Rational[2159, 2], Rational[347, 2], Rational[
            845, 2]}, {1282, 1081, Rational[355, 2], 426}, {
           Rational[2569, 2], Rational[2077, 2], Rational[363, 2], Rational[
            859, 2]}, {
           Rational[2567, 2], Rational[2075, 2], 178, Rational[853, 2]}, {
           1282, 1081, 175, 424}}, {{1055.5, 1132, 286, 286}, {
           815, 1169.5, 376.5, 128}, {781, 850.5, 349.5, 101.5}, {
           789.5, 857, 358, 110}, {827.5, 1178.5, 388.5, 140.5}, {827, 1178, 
           388, 140}, {823.5, 1176, 385, 136.5}, {787, 1159, 356, 107.5}, {
           789, 1159.5, 357.5, 109.5}, {793, 1161, 362, 113.5}, {
           764.5, 849, 341, 92.5}, {753, 845, 329, 81}, {
           734, 1136.5, 310.5, 62}, {729.5, 1134.5, 306.5, 57.5}, {
           726, 1132.5, 302.5, 54}, {727, 1133.5, 303.5, 55}, {
           738, 1138.5, 315, 66}, {736.5, 1137, 313.5, 64.5}, {
           793.5, 1164.5, 362.5, 114}, {793, 854, 362, 113.5}, {
           941, 540.5, 332.5, 207.5}, {949, 548.5, 340.5, 215.5}, {
           740, 832.5, 316.5, 68}, {740.5, 1138.5, 317, 68.5}, {
           794, 1164.5, 363, 114.5}, {795, 1165.5, 364, 115.5}, {
           781, 1156, 350.5, 101.5}, {813.5, 1168, 375, 126.5}, {
           785.5, 1155.5, 354.5, 106}, {786, 1156, 355, 106.5}, {
           815, 1169.5, 376.5, 128}, {815.5, 1170, 377, 128.5}, {
           810.5, 1166, 372, 124}, {811.5, 874.5, 373.5, 125}, {
           795.5, 866.5, 365, 116}, {889.5, 1221, 518.5, 167}, {
           1082.5, 1049, 394, 403}, {1036, 744.5, 395, 400}, {
           842.5, 622.5, 519, 163.5}, {796, 865.5, 365.5, 116.5}, {818, 1172, 
           380, 131}, {777.5, 846.5, 346.5, 98}, {716, 816, 289, 43.5}, {
           792.5, 1158, 358, 112.5}, {824, 1217, 385, 137}, {758.5, 1229.5, 
           319.5, 71.5}, {757.5, 1228.5, 318.5, 70.5}, {
           762, 1231.5, 322.5, 75}, {767.5, 1235.5, 327.5, 80.5}, {
           764, 1233.5, 324.5, 77}, {763, 1232.5, 323.5, 76}, {814.5, 933.5, 
           375.5, 127.5}, {807.5, 929, 368.5, 120.5}, {792, 1203, 353, 105}, {
           798.5, 1207, 359.5, 111.5}, {764, 1232.5, 324.5, 77.5}, {
           810, 1213, 371, 123.5}, {804, 1209, 364.5, 117}, {760, 888, 320, 
           73}, {782.5, 901.5, 338.5, 91}, {797, 1251.5, 348.5, 100.5}, {
           794, 1250, 346.5, 99}, {792, 1249, 345.5, 98}, {
           793, 1249.5, 346.5, 98.5}, {781.5, 1243.5, 339, 91}, {770.5, 
           1237.5, 331.5, 83.5}, {761.5, 886.5, 322, 74.5}, {751.5, 879.5, 
           311.5, 64.5}, {752.5, 1224.5, 313, 65.5}, {
           751.5, 1224, 312.5, 65}, {752, 1224.5, 313, 65.5}, {
           779, 1240.5, 334, 86}, {830, 1224.5, 385, 137}, {
           861.5, 1196.5, 423, 174.5}, {880, 931.5, 442, 193}, {871, 641, 433,
            184}, {847.5, 903.5, 409.5, 160.5}, {872, 1197.5, 434, 185}, {
           892, 931, 453.5, 205}, {884.5, 930, 446, 197.5}, {880.5, 1202.5, 
           442.5, 193.5}, {784, 941.5, 334, 213.5}, {
           768.5, 936, 318.5, 198}, {768, 938.5, 321.5, 197.5}, {
           783, 667.5, 336.5, 212.5}, {776.5, 557.5, 453, 432.5}, {
           769, 831.5, 445.5, 425}, {857.5, 1195, 419.5, 170.5}, {
           847, 1190.5, 408.5, 160}, {857.5, 914.5, 418.5, 170.5}, {872.5, 
           920.5, 433.5, 185.5}, {869.5, 1199.5, 431, 182.5}, {
           861, 912, 422.5, 174}, {848, 617, 409, 161}, {840.5, 897.5, 401.5, 
           153.5}, {838.5, 1185.5, 400, 151.5}, {839, 1186, 400.5, 152}, {
           846, 1189.5, 407.5, 159}, {842.5, 1188, 404.5, 155.5}, {834, 1183, 
           396, 147}, {833.5, 1182.5, 395.5, 146.5}, {
           817, 1218, 373, 124.5}, {794, 1250, 346.5, 99}, {
           787, 1246.5, 342.5, 95.5}, {750, 1186, 316.5, 68.5}, {714.5, 
           1126.5, 291.5, 42.5}, {715.5, 1127.5, 292.5, 43.5}, {
           708, 1122, 283.5, 35.5}, {699.5, 1115.5, 273, 26.5}, {
           849.5, 794, 261, 21}, {844.5, 789.5, 251, 16.5}, {
           690.5, 1107, 254, 18}, {702.5, 1116.5, 273, 30}, {
           705.5, 1119.5, 280, 33}, {694.5, 1110.5, 262, 21.5}, {694, 1110, 
           261, 21}, {695.5, 1111.5, 264, 22.5}, {707.5, 1119.5, 278.5, 35}, {
           716.5, 1126.5, 293, 44.5}, {763, 1146.5, 332, 83.5}, {
           768, 1149.5, 336.5, 88.5}, {756.5, 1142.5, 329, 81}, {
           810, 1167.5, 375, 127}, {768, 1150, 336.5, 88.5}, {707.5, 1120.5, 
           283.5, 35.5}, {707, 1120, 283, 35}, {708.5, 1121, 285, 36.5}, {
           715.5, 813.5, 292, 43.5}, {712.5, 811, 288.5, 40.5}, {
           703, 1117.5, 278.5, 30.5}, {702, 802.5, 278, 30}, {
           698, 798.5, 269, 26}, {762, 855, 325.5, 82}, {
           782, 908, 343.5, 95.5}, {725.5, 1212.5, 286.5, 39.5}, {
           719, 1210.5, 279.5, 32.5}, {720.5, 857.5, 280.5, 33.5}, {
           749.5, 873.5, 310, 63}, {780, 1243.5, 339.5, 92.5}, {
           779, 902, 338.5, 91}, {775.5, 901, 336.5, 88.5}, {
           779.5, 1243.5, 338.5, 91}, {747.5, 1227, 303.5, 59}, {
           742.5, 1225, 300.5, 55.5}, {773, 1240, 334, 86}, {772.5, 1239.5, 
           333.5, 85.5}, {739, 864, 293, 52}, {704, 847.5, 251, 17.5}, {
           706.5, 1207.5, 257, 20}, {719, 860, 276.5, 32}, {
           731.5, 864, 292, 44.5}, {731, 1215, 291.5, 44}, {726, 1213, 287, 
           39}, {725.5, 1212.5, 286.5, 38.5}, {800, 1207, 361, 113}, {
           805, 1209, 365.5, 118}, {739.5, 1219, 300, 52.5}, {741.5, 1219.5, 
           302.5, 54.5}, {738.5, 869, 299, 51.5}, {739, 869.5, 299.5, 52}, {
           742.5, 873.5, 303.5, 55.5}, {743, 874, 304, 56}, {
           719, 1213, 282.5, 31.5}, {752, 1231, 308, 57.5}, {
           755, 1282, 518.5, 68}, {947.5, 1149, 424, 221}, {
           1193.5, 990.5, 168, 415.5}, {1191.5, 917.5, 156.5, 515.5}, {
           1190.5, 916.5, 153, 512.5}, {1191.5, 988.5, 163.5, 411.5}, {1190.5,
            987.5, 162.5, 410.5}, {1206, 867.5, 119.5, 471}, {
           1168, 723, 43.5, 507.5}, {1155, 778.5, 83.5, 555.5}, {
           1156, 779.5, 84.5, 556.5}, {1116.5, 700.5, 12.5, 485.5}, {1116, 
           700, 12, 485}, {1155, 845.5, 91.5, 451.5}, {
           1191.5, 989, 164.5, 411.5}, {1187.5, 985.5, 156.5, 403.5}, {
           1186, 984, 153, 400.5}, {1183.5, 981.5, 148.5, 396.5}, {1183, 981, 
           148, 396}, {946.5, 1052, 217, 217}, {943, 1049, 212.5, 212}, {
           1147.5, 840.5, 79, 438}, {1150.5, 843, 83, 442.5}, {1190, 987, 163,
            411}, {1191.5, 988.5, 166.5, 414}, {1183.5, 981.5, 150.5, 398}, {
           1180.5, 904, 136.5, 496}, {1179, 903, 134, 493}, {1178.5, 977.5, 
           141.5, 388.5}, {1145.5, 837, 75, 435}, {1112.5, 696.5, 8.5, 
           481.5}, {1116, 701.5, 14, 486}, {1014, 1006, 237.5, 349}, {909.5, 
           1306.5, 456.5, 208.5}, {907, 1304, 454, 206}, {
           903, 1299.5, 449.5, 202}, {902.5, 1298.5, 448.5, 201.5}, {
           1005, 995, 226, 338.5}, {1056, 688, 230.5, 342.5}, {836, 572, 347, 
           220}, {905, 606, 151, 383.5}, {1124.5, 721, 34, 505.5}, {
           1117, 702.5, 15.5, 487.5}, {905, 496.5, 132, 364.5}, {
           841.5, 792.5, 458.5, 228}, {1100.5, 1152, 434.5, 331}, {
           1250, 1148.5, 430.5, 553.5}, {1102.5, 1301.5, 563, 438.5}, {
           1015, 1007.5, 239, 351}, {1124, 712, 25, 497}, {1123.5, 711.5, 
           24.5, 496.5}, {1125, 713, 26, 498}, {1126.5, 714.5, 27.5, 499.5}, {
           1016.5, 1009, 240.5, 352.5}, {1013, 1005, 236.5, 348}, {
           1115, 700.5, 13, 484.5}, {1009, 999.5, 230.5, 342.5}, {
           1014.5, 1006.5, 238, 349.5}, {1014, 1006, 237.5, 349}, {
           1008, 998.5, 229.5, 341.5}, {1108.5, 692.5, 4.5, 476.5}, {
           1160, 708.5, 28.5, 492.5}, {1212.5, 869, 45.5, 398}, {1213.5, 
           1013.5, 38.5, 287.5}, {1183.5, 984, 75, 323}, {
           1164, 964, 123, 370.5}, {1171, 970.5, 130.5, 378}, {
           1168, 890.5, 119, 478}, {1168.5, 891, 119.5, 478.5}, {
           1169, 968.5, 128.5, 376}, {1154.5, 854, 91, 450.5}, {1138, 840, 59,
            418}, {1135, 939, 61.5, 309}, {1136.5, 940, 65, 313}, {
           1146, 948, 91.5, 339}, {1145, 947.5, 88.5, 335.5}, {
           1137, 941, 63.5, 311}, {1135, 939, 59, 307}, {
           1152.5, 954, 94.5, 342.5}, {1172, 971, 132, 380}, {
           1165.5, 965, 124.5, 372.5}, {1156, 956.5, 114, 361.5}, {
           1157, 957.5, 115, 362.5}, {1161.5, 961.5, 119, 367}, {
           1161, 961, 118.5, 366.5}, {1162.5, 962.5, 121, 368.5}, {
           1160.5, 878.5, 457.5, 470}, {1156, 874, 452.5, 465.5}, {
           1164.5, 964, 123.5, 371.5}, {1164, 963.5, 123, 371}, {
           1160.5, 960.5, 119, 366.5}, {1064, 1064, 643, 666.5}, {
           1063.5, 1063.5, 642.5, 666}, {1152.5, 854.5, 90.5, 450}, {
           1138, 842, 58.5, 418.5}, {1138.5, 942, 67, 315}, {
           1172.5, 971.5, 135, 383}, {1205.5, 854, 116.5, 468.5}, {
           1205, 853.5, 116, 468}, {1200.5, 996, 187, 435}, {
           1200, 934.5, 176, 535.5}, {1202.5, 881.5, 179.5, 650.5}, {
           1206.5, 949, 105, 462}, {1213, 1013, 35.5, 281}, {1216, 1016, 47, 
           295}, {1212.5, 1012.5, 38.5, 286.5}, {1213, 1013, 39, 287}, {
           1217, 878, 57.5, 409.5}, {1214.5, 875.5, 50, 402}, {
           1176, 977.5, 53, 301}, {1173, 971.5, 142, 390}, {
           1205.5, 847.5, 116.5, 469}, {1175, 725, 45, 509}, {
           1146.5, 852, 87, 446}, {1150, 951, 108, 355}, {
           1147, 949, 95.5, 342.5}, {1146.5, 948.5, 95, 342}, {
           1178.5, 979, 62, 302.5}, {1177.5, 978, 60.5, 301.5}, {
           1147, 948.5, 97.5, 345.5}, {1182.5, 983.5, 71.5, 320}, {
           1183, 894, 67.5, 427.5}, {1161.5, 872, 110.5, 469.5}, {
           1160.5, 961, 114, 361.5}, {1159.5, 960, 111.5, 359.5}, {
           1158, 859.5, 96.5, 456.5}, {1144, 753.5, 66.5, 538.5}, {1145.5, 
           760.5, 73.5, 545.5}, {1154, 782.5, 438.5, 853.5}, {
           1165.5, 809, 457.5, 872}, {986.5, 988, 239.5, 350.5}, {
           802, 1161, 366.5, 117.5}, {798, 1158.5, 364.5, 115.5}, {799, 1159, 
           365, 116}, {984, 986, 238, 349}, {1166, 811, 109, 580}, {
           1168, 813.5, 111.5, 583}, {1168.5, 814, 112, 583.5}, {979.5, 983.5,
            235.5, 346.5}, {978.5, 982, 234.5, 345.5}, {
           979.5, 982.5, 235, 346}, {971.5, 966.5, 226, 337.5}, {
           975.5, 968.5, 228, 339.5}, {1007, 940.5, 207.5, 310.5}, {
           1213, 726.5, 55, 510}, {1215.5, 734, 62.5, 517.5}, {
           1215, 731, 59, 514.5}, {1208.5, 804, 117, 581}, {
           1208, 803.5, 116.5, 580.5}, {1206, 796, 109, 573}, {
           1208, 801, 114, 577.5}, {1218.5, 739, 67, 522.5}, {
           1215, 730, 58.5, 514}, {1215.5, 730.5, 59, 514.5}, {1215, 727, 55, 
           511}, {1214, 726, 54, 510}, {1213, 722, 50.5, 506}, {
           1204.5, 791.5, 105, 568.5}, {991.5, 1014.5, 266, 378}, {
           785, 1151.5, 356.5, 108.5}, {983, 990, 241.5, 353}, {1181.5, 833.5,
            131.5, 602.5}, {1181, 833, 131, 602}, {1177, 826, 124, 595}, {
           1178.5, 827.5, 125.5, 596.5}, {1176, 823.5, 121.5, 593}, {1173.5, 
           819.5, 117.5, 589.5}, {1177.5, 826, 124, 596}, {1181.5, 832.5, 
           130.5, 602.5}, {1178.5, 827, 125, 597}, {
           1179, 827.5, 125.5, 597.5}, {1184, 835, 133, 605}, {1184.5, 835.5, 
           133.5, 605.5}, {981.5, 992.5, 244, 356}, {983, 996, 247.5, 359}, {
           1190.5, 849.5, 147.5, 618.5}, {1182, 836.5, 134.5, 606}, {1171, 
           817, 115, 587}, {978.5, 985, 236.5, 348.5}, {
           784, 1151.5, 356.5, 108.5}, {785, 1152, 357, 109}, {
           991, 1005.5, 257, 368.5}, {1196.5, 861.5, 159.5, 631}, {
           1194.5, 857, 155, 627}, {1194, 856.5, 154.5, 626.5}, {
           1195, 857.5, 155.5, 627.5}, {1190.5, 847, 145, 616.5}, {
           1190, 846.5, 144.5, 616}, {1191, 849, 147, 619}, {1175.5, 904.5, 
           132.5, 492.5}, {1180.5, 917, 145, 504.5}, {
           1034, 1073.5, 292.5, 404.5}, {857.5, 1270, 403.5, 156}, {846.5, 
           1265.5, 392.5, 144.5}, {847, 941, 393, 145}, {
           839.5, 939, 385.5, 137.5}, {838, 1262.5, 384, 136}, {
           841.5, 1263, 387.5, 140}, {835, 931.5, 381, 133.5}, {824, 593, 370,
            122}, {819.5, 921.5, 365.5, 118}, {838, 1262.5, 384, 137}, {
           839, 1263, 384.5, 137.5}, {820, 1258, 365.5, 118}, {822.5, 1258.5, 
           368.5, 120.5}, {842.5, 1264, 388, 140.5}, {848, 938.5, 393, 146}, {
           870, 952, 415.5, 168.5}, {1014.5, 1008.5, 240, 352}, {1130.5, 
           722.5, 35.5, 506.5}, {1130, 722, 35, 506}, {
           1134.5, 732.5, 45.5, 517}, {1136.5, 736, 49, 521}, {
           986.5, 995.5, 213, 325.5}, {987, 996, 213.5, 326}, {1135, 730, 43, 
           515}, {1018, 1013.5, 245, 357.5}, {1017.5, 1013, 244.5, 357}, {
           1018, 1013.5, 245, 357.5}, {1019.5, 1017.5, 249, 361}, {1137.5, 
           737.5, 50.5, 521.5}, {1171, 816.5, 122, 593.5}, {
           1155, 1095, 319, 431.5}, {996, 1290, 438, 191}, {
           874, 1278.5, 419.5, 172}, {860.5, 1270.5, 406.5, 158.5}, {861.5, 
           1271.5, 407.5, 159.5}, {859.5, 1271, 405, 157.5}, {
           859, 1270.5, 404.5, 157}, {869, 1275.5, 415, 167}, {
           866.5, 952, 412, 164.5}, {866, 951.5, 411.5, 164}, {
           872.5, 1276.5, 418, 170.5}, {867, 954, 412.5, 165}, {
           867.5, 954.5, 413, 165.5}, {872, 958.5, 417, 170}, {
           872.5, 959, 417.5, 170.5}, {879, 1281, 424, 177}, {
           882, 1283, 427.5, 180}, {885, 1285.5, 431, 183}, {
           885.5, 1286, 431.5, 183.5}, {879.5, 965.5, 425.5, 177.5}, {878.5, 
           964.5, 424.5, 176.5}, {880, 1281.5, 426, 178.5}, {
           884, 1283.5, 429.5, 182.5}, {888, 1286.5, 433.5, 186}, {
           889.5, 1288, 435, 187.5}, {889, 1287.5, 434.5, 187}, {891.5, 
           1289.5, 437.5, 189.5}, {891, 974, 436.5, 189}, {
           891.5, 974.5, 437, 189.5}, {895, 978.5, 441, 193}, {
           895.5, 979, 441.5, 193.5}, {1013.5, 1008.5, 239, 350.5}, {
           1165, 859, 108, 467.5}, {1210, 1008, 120, 368}, {
           1164, 967, 39.5, 280.5}, {1055.5, 911.5, 127.5, 242}, {
           1155.5, 910, 230, 455.5}, {1305.5, 908.5, 222.5, 678.5}, {1306, 
           909, 223, 679}, {1154.5, 916.5, 226, 454.5}, {1003, 963, 229, 
           230}, {1135, 925, 195, 423.5}, {1133, 768.5, 193.5, 421}, {
           1110, 855, 140.5, 264.5}, {1262, 963.5, 137.5, 490}, {
           1300, 903.5, 217.5, 673}, {1299.5, 903, 217, 672.5}, {1302.5, 
           905.5, 219.5, 675.5}, {1205, 909.5, 119.5, 465.5}, {
           1202.5, 907.5, 117.5, 463}, {1298, 902, 216, 671}, {1298.5, 902.5, 
           216.5, 671.5}, {1299.5, 1101, 333, 561}, {
           1298, 1099.5, 331.5, 559.5}, {1201, 906, 116, 461}, {
           1173, 842.5, 59.5, 405}, {1240, 773, 101, 557}, {1241.5, 774.5, 
           102.5, 558.5}, {1237, 769, 97.5, 553}, {
           1232.5, 764, 92.5, 547.5}, {1233.5, 765.5, 93.5, 548.5}, {1234.5, 
           766.5, 94.5, 549.5}, {1248, 799, 119.5, 575}, {
           1262, 835.5, 148.5, 604}, {1261, 832.5, 145.5, 600.5}, {
           1258.5, 824, 137.5, 592.5}, {1260, 830.5, 144.5, 599.5}, {1249, 
           803, 124, 579}, {1233.5, 764, 92, 547.5}, {1230.5, 759.5, 87.5, 
           543.5}, {1234, 765, 93, 549}, {1239, 772, 100, 556}, {
           1244.5, 670.5, 223, 451.5}, {1238, 662, 214.5, 443}, {1227.5, 
           756.5, 84.5, 540.5}, {1225.5, 753.5, 81.5, 537}, {
           1226, 754, 82, 537.5}, {1227, 755, 83, 538.5}, {1224.5, 751.5, 
           79.5, 534.5}, {1167, 833.5, 50.5, 397}, {1116, 812.5, 22, 376.5}, {
           1122.5, 818.5, 31, 390.5}, {1123.5, 928.5, 40.5, 288.5}, {
           1207, 910.5, 123, 475}, {1289.5, 891.5, 204.5, 660.5}, {1290.5, 
           892.5, 205.5, 661.5}, {1291.5, 893.5, 206.5, 662.5}, {
           1288, 889.5, 202.5, 658}, {1286, 887, 200, 655}, {1286.5, 887.5, 
           200.5, 655.5}, {1281, 879, 192, 647}, {1276.5, 871.5, 184.5, 
           639.5}, {1280.5, 878.5, 191.5, 646.5}, {
           1281.5, 880.5, 193.5, 649}, {1281, 879.5, 193, 648.5}, {
           1018.5, 1013.5, 263.5, 367}, {739, 1138, 315, 67}, {
           709.5, 1121.5, 281.5, 37}, {732, 1129, 304, 59.5}, {754, 1143, 330,
            82}, {771.5, 1197, 333.5, 85.5}, {1042, 1066, 271, 375}, {1281, 
           878, 191, 647}, {1281.5, 878.5, 191.5, 647.5}, {
           1275, 867, 180, 635.5}, {1038, 909.5, 565.5, 715}, {
           1037, 908, 564.5, 714}, {1271, 861.5, 175, 630}, {
           1197.5, 898, 109.5, 460.5}, {1119.5, 925.5, 35.5, 282.5}, {1118.5, 
           924.5, 34.5, 281.5}, {1115.5, 921, 29.5, 274.5}, {
           1120, 925, 34.5, 280}, {1124, 929.5, 40.5, 288}, {
           1124.5, 930, 41, 288.5}, {1126.5, 931.5, 43.5, 291.5}, {1125.5, 
           930.5, 42.5, 290.5}, {1119, 924, 33.5, 278.5}, {
           1111.5, 806, 15.5, 372.5}, {1110, 805, 14, 368.5}, {
           1191, 889, 98.5, 445}, {1272, 862, 175, 631}, {
           1272.5, 864.5, 178, 633.5}, {1273, 865, 178.5, 634}, {1282.5, 
           878.5, 191.5, 647.5}, {1205, 908.5, 118.5, 470}, {
           1115.5, 920.5, 27.5, 273}, {1115, 920, 27, 272}, {
           1115.5, 920.5, 27.5, 273}, {1191.5, 887.5, 97.5, 447.5}, {
           1262, 831.5, 145, 600}, {1260.5, 827, 140.5, 596}, {
           977, 659.5, 420, 308.5}, {995.5, 786, 442, 227}, {
           1278.5, 1076.5, 153, 402}, {1254.5, 1053.5, 102.5, 351.5}, {
           1258, 1057, 119, 367.5}, {1262.5, 1061.5, 136.5, 384.5}, {
           1285, 1083, 172.5, 421}, {1306.5, 1103.5, 207.5, 456.5}, {1305.5, 
           1102.5, 206.5, 455.5}, {1280.5, 1078.5, 156, 404.5}, {
           1282, 878.5, 60, 518}, {1280.5, 877, 57.5, 516}, {
           1250.5, 1050, 98.5, 347}, {1247.5, 1047.5, 95.5, 343.5}, {
           1249.5, 1049, 97.5, 346}, {1251.5, 1050.5, 99.5, 348.5}, {
           1244, 910, 98.5, 451}, {1015, 669.5, 673.5, 673}, {
           1027, 814.5, 688.5, 584.5}, {1282, 1080, 166, 415}, {
           1276.5, 1074.5, 151, 400}, {1248, 1047.5, 96, 344.5}, {
           1256, 1055.5, 119, 367.5}, {1255, 1054.5, 118, 366.5}, {
           1274, 1072.5, 148.5, 397}, {1217.5, 1018.5, 128.5, 376.5}, {
           1195.5, 883, 97.5, 449}, {1262, 837.5, 150.5, 606.5}, {
           1248, 938.5, 117, 469}, {1231.5, 1031.5, 74, 322.5}, {
           1228, 1028, 68, 316.5}, {1229, 1029, 70.5, 318.5}, {
           1236.5, 1036.5, 81, 329.5}, {1240.5, 1040.5, 85.5, 334.5}, {1239.5,
            1039.5, 84.5, 333.5}, {1238.5, 1038.5, 83.5, 332.5}, {
           1236.5, 1036.5, 81, 329.5}, {1229, 1029, 70.5, 318.5}, {1223.5, 
           1023.5, 62.5, 310.5}, {1232.5, 1032.5, 74.5, 323}, {
           1234, 898.5, 85, 437.5}, {1241.5, 785, 105.5, 561.5}, {
           1257.5, 936, 124.5, 476.5}, {1246.5, 1046, 100, 348}, {1240, 1040, 
           86, 334}, {1252, 1051.5, 107.5, 355.5}, {
           1251, 1050.5, 106.5, 354.5}, {1236, 1036, 80.5, 329}, {1228.5, 
           1028.5, 69.5, 318.5}, {1227.5, 1027.5, 68.5, 317.5}, {1228.5, 
           1028.5, 69.5, 318.5}, {1247.5, 1047, 107, 356}, {
           1277.5, 1076.5, 166, 415}, {1263, 1062.5, 134.5, 383}, {
           1264, 1063.5, 135.5, 384}, {1296, 1094.5, 196.5, 445}, {
           1284.5, 1083, 177.5, 425.5}, {1284, 1082.5, 177, 425}, {
           1296, 1094.5, 196.5, 445}, {1280.5, 1079.5, 172, 420.5}, {1277, 
           1076, 168, 416}, {1285.5, 1084.5, 183.5, 431.5}, {1281.5, 979.5, 
           185.5, 537.5}, {1277.5, 975.5, 178.5, 530.5}, {
           1287.5, 1086, 184.5, 432.5}, {1213.5, 1015, 125, 372.5}, {
           1208, 1010, 117.5, 365}, {1278.5, 973, 179, 531}, {
           1200, 897.5, 112, 463.5}, {1175.5, 978, 53, 301}, {
           1261, 962.5, 136.5, 489}, {1288.5, 989.5, 193, 545}, {
           1286.5, 1085, 183.5, 431.5}, {1288, 1086.5, 186, 434.5}, {
           1273, 1072, 162, 410.5}, {1275, 1074, 166, 414}, {
           1277, 1076, 169.5, 418}, {1279.5, 980, 180.5, 533}, {
           1284.5, 985, 189, 541.5}, {1280.5, 1079.5, 174.5, 423.5}, {1281, 
           1080, 175, 424}, {1283, 1082, 179, 427.5}, {
           1289.5, 1088, 189.5, 437.5}, {1295.5, 1093.5, 197.5, 445.5}, {
           1285, 1083.5, 182, 430}, {1273, 1072, 162.5, 411}, {1280.5, 1079.5,
            173.5, 422.5}, {1282, 1081, 177.5, 426}, {1284.5, 1038.5, 181.5, 
           429.5}, {1283.5, 1037.5, 178, 426.5}, {1282, 1081, 175, 424}}]], 
        Polygon3DBox[
         NCache[{{
           1015, 992, Rational[1103, 2], Rational[657, 2], Rational[
            411, 2]}, {
           Rational[1957, 2], 980, Rational[1045, 2], Rational[599, 2], 
            Rational[353, 2]}, {
           818, Rational[1941, 2], 594, 371, Rational[247, 2]}, {881, 996, 
           650, 427, 179}, {
           Rational[1603, 2], 967, Rational[1155, 2], Rational[709, 2], 
            107}, {726, Rational[1871, 2], Rational[1019, 2], 285, 39}, {
           Rational[1423, 2], Rational[1857, 2], 495, 266, Rational[49, 2]}, {
           711, 928, Rational[989, 2], 265, 24}, {
           Rational[1415, 2], Rational[1851, 2], Rational[983, 2], Rational[
            545, 2], 20}, {955, 965, Rational[979, 2], 269, 139}, {
           962, 970, Rational[993, 2], Rational[539, 2], Rational[293, 2]}, {
           Rational[1717, 2], Rational[1855, 2], 265, 277, 356}, {
           Rational[1713, 2], Rational[1851, 2], 263, 273, 354}, {
           Rational[1429, 2], Rational[1861, 2], Rational[997, 2], 273, 
            Rational[55, 2]}, {
           Rational[1585, 2], Rational[2007, 2], Rational[1139, 2], 354, 
            106}, {Rational[1569, 2], Rational[1995, 2], Rational[1123, 2], 
            Rational[703, 2], Rational[195, 2]}, {
           Rational[1401, 2], Rational[1841, 2], Rational[969, 2], 272, 
            Rational[25, 2]}, {
           Rational[1399, 2], Rational[1839, 2], Rational[967, 2], 270, 
            Rational[23, 2]}, {
           Rational[1421, 2], Rational[1853, 2], 494, Rational[551, 2], 23}, {
           723, 935, 506, 283, 36}, {
           Rational[1883, 2], Rational[1735, 2], Rational[1305, 2], 668, 
            305}, {933, 862, 644, 654, Rational[593, 2]}, {
           Rational[1415, 2], Rational[1851, 2], Rational[981, 2], 257, 
            Rational[41, 2]}, {
           Rational[1855, 2], Rational[1963, 2], Rational[1045, 2], 302, 
            183}, {Rational[2307, 2], 919, 677, Rational[1399, 2], Rational[
            919, 2]}, {
           933, Rational[1725, 2], Rational[1289, 2], Rational[1307, 2], 
            Rational[593, 2]}, {
           931, Rational[1717, 2], Rational[1281, 2], Rational[1303, 2], 
            Rational[589, 2]}, {
           Rational[1871, 2], 862, Rational[1291, 2], Rational[1323, 2], 
            299}, {762, 993, 538, Rational[643, 2], Rational[149, 2]}, {
           Rational[1675, 2], Rational[2049, 2], 606, Rational[781, 2], 
            Rational[285, 2]}, {874, 996, 643, 420, 172}, {
           872, 1035, Rational[1283, 2], 426, Rational[355, 2]}, {
           Rational[1723, 2], Rational[2143, 2], Rational[1263, 2], Rational[
            847, 2], Rational[349, 2]}, {
           Rational[1721, 2], Rational[2141, 2], Rational[1261, 2], Rational[
            845, 2], Rational[347, 2]}, {
           Rational[1715, 2], 1069, 627, 419, Rational[341, 2]}, {
           Rational[1701, 2], Rational[2133, 2], 620, 412, Rational[
            327, 2]}, {
           Rational[1699, 2], 1066, Rational[1239, 2], Rational[823, 2], 
            163}, {800, 1006, 577, Rational[723, 2], Rational[227, 2]}, {
           796, Rational[2009, 2], 573, Rational[715, 2], 109}, {
           Rational[2041, 2], Rational[2133, 2], 622, 414, 289}, {
           Rational[2385, 2], 1066, Rational[1243, 2], Rational[827, 2], 
            Rational[825, 2]}, {
           Rational[2467, 2], 1070, Rational[1257, 2], 413, 413}, {
           1054, Rational[2137, 2], Rational[1243, 2], 406, 282}, {
           834, 1061, 604, 396, Rational[295, 2]}, {
           Rational[1677, 2], Rational[2123, 2], Rational[1215, 2], Rational[
            799, 2], Rational[303, 2]}, {
           Rational[2019, 2], 1060, 603, 395, Rational[541, 2]}, {
           1002, 1058, Rational[1191, 2], Rational[775, 2], 263}, {
           Rational[1929, 2], Rational[1955, 2], 402, Rational[1051, 2], 
            410}, {Rational[1979, 2], 947, 427, 543, 428}, {
           Rational[1697, 2], 1027, Rational[1235, 2], 402, Rational[
            309, 2]}, {852, Rational[2055, 2], Rational[1243, 2], 406, 158}, {
           Rational[1735, 2], Rational[2063, 2], 636, Rational[841, 2], 
            Rational[345, 2]}, {
           Rational[2011, 2], Rational[1861, 2], 708, 732, Rational[
            737, 2]}, {
           Rational[2445, 2], 941, 726, Rational[1485, 2], Rational[
            1007, 2]}, {
           1220, 936, 721, Rational[1481, 2], Rational[1001, 2]}, {
           1211, 927, 706, Rational[1451, 2], Rational[971, 2]}, {
           Rational[2129, 2], Rational[2061, 2], Rational[1255, 2], Rational[
            809, 2], Rational[563, 2]}, {
           Rational[1717, 2], Rational[1983, 2], Rational[1253, 2], Rational[
            807, 2], Rational[313, 2]}, {
           Rational[1709, 2], 990, 623, 400, Rational[305, 2]}, {859, 992, 
           628, 405, 157}, {
           1018, Rational[2051, 2], 610, Rational[789, 2], 270}, {
           Rational[2381, 2], 1034, 546, Rational[661, 2], Rational[
            661, 2]}, {1185, Rational[1809, 2], 652, 668, 429}, {
           Rational[2319, 2], 800, 799, Rational[2105, 2], Rational[
            1147, 2]}, {
           Rational[2011, 2], Rational[1791, 2], Rational[1421, 2], 725, 
            Rational[723, 2]}, {
           Rational[1699, 2], Rational[1975, 2], Rational[1237, 2], Rational[
            791, 2], Rational[295, 2]}, {
           Rational[1701, 2], Rational[1977, 2], Rational[1239, 2], Rational[
            793, 2], Rational[297, 2]}, {
           1001, Rational[1769, 2], Rational[1399, 2], 721, Rational[
            713, 2]}, {1152, 784, 783, 1046, 565}, {
           Rational[2307, 2], 788, 787, Rational[2095, 2], Rational[
            1133, 2]}, {1151, 784, 783, 1045, 564}, {
           Rational[1921, 2], Rational[1821, 2], Rational[1337, 2], 696, 
            331}, {Rational[1543, 2], Rational[2079, 2], Rational[1113, 2], 
            Rational[697, 2], Rational[199, 2]}, {
           Rational[1545, 2], Rational[2081, 2], Rational[1115, 2], Rational[
            699, 2], Rational[201, 2]}, {
           Rational[1547, 2], Rational[2083, 2], Rational[1117, 2], Rational[
            701, 2], Rational[203, 2]}, {
           Rational[1543, 2], Rational[2079, 2], Rational[1113, 2], Rational[
            697, 2], Rational[199, 2]}, {
           Rational[1921, 2], Rational[1821, 2], Rational[1337, 2], 696, 
            331}, {937, 859, 644, 664, 300}, {
           807, 1012, Rational[1167, 2], 368, 120}, {
           810, 1013, 587, Rational[743, 2], 123}, {
           742, 982, Rational[1053, 2], 311, Rational[125, 2]}, {
           Rational[1511, 2], Rational[2055, 2], Rational[1081, 2], Rational[
            665, 2], Rational[167, 2]}, {
           Rational[1519, 2], Rational[2061, 2], Rational[1089, 2], Rational[
            673, 2], 88}, {
           Rational[1907, 2], Rational[2059, 2], Rational[1087, 2], Rational[
            671, 2], 211}, {
           Rational[1893, 2], 1024, Rational[1071, 2], Rational[655, 2], 
            203}, {Rational[2009, 2], Rational[2081, 2], 565, Rational[
            699, 2], 226}, {1044, 1018, 597, 374, 251}, {
           825, Rational[1951, 2], Rational[1187, 2], Rational[741, 2], 
            123}, {Rational[1587, 2], 1004, 570, Rational[709, 2], Rational[
            213, 2]}, {
           Rational[1515, 2], 1029, Rational[1083, 2], Rational[667, 2], 
            Rational[171, 2]}, {
           749, Rational[2047, 2], Rational[1067, 2], Rational[651, 2], 
            Rational[155, 2]}, {Rational[1499, 2], 1024, 534, 326, 78}, {
           Rational[1469, 2], 1018, 519, 311, 63}, {
           Rational[1475, 2], 1020, Rational[1045, 2], Rational[629, 2], 
            66}, {793, Rational[2007, 2], Rational[1139, 2], 354, 106}, {
           Rational[1961, 2], 995, 553, Rational[675, 2], Rational[427, 2]}, {
           Rational[1873, 2], 1016, Rational[1035, 2], Rational[619, 2], 
            Rational[369, 2]}, {
           Rational[1487, 2], 982, 528, Rational[625, 2], 64}, {
           736, 979, Rational[1041, 2], 305, Rational[113, 2]}, {
           Rational[1451, 2], Rational[2027, 2], Rational[1021, 2], Rational[
            605, 2], Rational[107, 2]}, {
           Rational[1487, 2], Rational[1961, 2], 528, Rational[625, 2], 64}, {
           752, Rational[1969, 2], Rational[1073, 2], 321, Rational[
            145, 2]}, {723, 1014, 507, 299, Rational[101, 2]}, {
           734, Rational[2039, 2], 517, 309, 61}, {
           Rational[1523, 2], Rational[2063, 2], Rational[1091, 2], Rational[
            675, 2], 89}, {
           Rational[2023, 2], 1047, Rational[1157, 2], 363, 239}, {
           1011, Rational[2093, 2], 578, Rational[725, 2], Rational[
            477, 2]}, {
           Rational[1519, 2], Rational[1977, 2], 544, Rational[657, 2], 80}, {
           740, Rational[1957, 2], Rational[1047, 2], 308, 60}, {
           773, 993, Rational[1097, 2], 333, Rational[171, 2]}, {
           Rational[1949, 2], 992, Rational[1089, 2], 329, 205}, {
           916, Rational[2017, 2], Rational[989, 2], Rational[573, 2], 
            Rational[323, 2]}, {
           Rational[1513, 2], Rational[1969, 2], Rational[1069, 2], Rational[
            1541, 2], 420}, {
           Rational[1623, 2], 966, Rational[1163, 2], 810, 460}, {
           Rational[1621, 2], Rational[1931, 2], 581, Rational[1619, 2], 
            459}, {1029, Rational[2015, 2], 578, Rational[1613, 2], Rational[
            1161, 2]}, {
           Rational[2067, 2], 1010, Rational[1161, 2], Rational[715, 2], 
            Rational[469, 2]}, {
           Rational[1555, 2], Rational[1907, 2], Rational[1107, 2], Rational[
            661, 2], 83}, {775, 952, 552, Rational[1561, 2], 431}, {
           812, 966, Rational[1163, 2], 810, Rational[921, 2]}, {
           815, 968, Rational[1167, 2], Rational[721, 2], 113}, {
           Rational[1629, 2], Rational[1935, 2], 583, 360, Rational[
            225, 2]}, {
           822, Rational[1945, 2], Rational[1181, 2], Rational[735, 2], 
            120}, {Rational[1937, 2], Rational[1981, 2], 537, Rational[
            635, 2], Rational[387, 2]}, {
           955, 982, 526, 758, Rational[1061, 2]}, {
           817, Rational[1939, 2], 588, Rational[1633, 2], Rational[
            931, 2]}, {
           Rational[1669, 2], Rational[1961, 2], Rational[1207, 2], Rational[
            761, 2], Rational[265, 2]}, {
           Rational[1667, 2], Rational[1959, 2], Rational[1205, 2], Rational[
            759, 2], Rational[263, 2]}, {
           838, 982, Rational[1213, 2], Rational[767, 2], 136}, {
           Rational[1687, 2], Rational[1971, 2], Rational[1223, 2], Rational[
            777, 2], Rational[283, 2]}, {
           840, 984, Rational[1217, 2], Rational[771, 2], 138}, {
           839, 983, Rational[1215, 2], Rational[769, 2], 137}, {
           Rational[1663, 2], Rational[1957, 2], 600, 377, Rational[
            259, 2]}, {Rational[1627, 2], Rational[1935, 2], 583, 360, 112}, {
           810, 965, 579, 356, Rational[217, 2]}, {
           Rational[1627, 2], Rational[1935, 2], Rational[1163, 2], Rational[
            717, 2], Rational[223, 2]}, {
           825, 975, Rational[1187, 2], Rational[741, 2], 123}, {
           988, Rational[2001, 2], 565, Rational[699, 2], 225}, {
           Rational[1879, 2], Rational[2037, 2], Rational[1049, 2], Rational[
            633, 2], Rational[383, 2]}, {
           737, 1017, 521, 313, Rational[129, 2]}, {
           Rational[1611, 2], Rational[2095, 2], 582, 374, Rational[
            251, 2]}, {
           1039, Rational[2157, 2], Rational[1291, 2], Rational[875, 2], 
            Rational[625, 2]}, {
           1038, Rational[2155, 2], Rational[1289, 2], Rational[873, 2], 
            Rational[623, 2]}, {
           Rational[1751, 2], Rational[2157, 2], Rational[1291, 2], Rational[
            875, 2], Rational[377, 2]}, {
           Rational[1573, 2], 998, Rational[1127, 2], Rational[703, 2], 99}, {
           757, 943, Rational[1067, 2], 314, 62}, {
           855, 1030, Rational[1249, 2], 409, Rational[321, 2]}, {
           Rational[1585, 2], 1002, 569, 354, 105}, {
           791, Rational[2001, 2], Rational[1135, 2], Rational[705, 2], 
            Rational[207, 2]}, {
           Rational[1581, 2], 1000, 567, Rational[703, 2], 103}, {
           Rational[1599, 2], Rational[2015, 2], Rational[1153, 2], 361, 
            Rational[225, 2]}, {
           Rational[1817, 2], Rational[2207, 2], Rational[1359, 2], Rational[
            943, 2], Rational[445, 2]}, {
           801, 1009, 578, Rational[727, 2], 114}, {
           Rational[1563, 2], Rational[1989, 2], Rational[1115, 2], 343, 
            94}, {874, 1078, Rational[1287, 2], Rational[871, 2], 187}, {
           Rational[1747, 2], 1077, Rational[1285, 2], Rational[869, 2], 
            186}, {800, 1006, 576, Rational[721, 2], Rational[225, 2]}, {
           Rational[1463, 2], Rational[1879, 2], Rational[1031, 2], Rational[
            585, 2], Rational[89, 2]}, {
           715, 929, 499, 280, Rational[55, 2]}, {
           952, 962, Rational[961, 2], 264, 136}, {
           1045, 1044, Rational[1131, 2], Rational[705, 2], 229}, {
           Rational[1767, 2], Rational[2165, 2], Rational[1305, 2], Rational[
            889, 2], Rational[393, 2]}, {
           Rational[2087, 2], 1085, 656, 448, Rational[647, 2]}, {
           Rational[2075, 2], Rational[2163, 2], Rational[1299, 2], Rational[
            883, 2], 317}, {879, 1081, 648, 440, 192}, {886, 1085, 655, 447, 
           199}, {Rational[1771, 2], Rational[2169, 2], Rational[1309, 2], 
            Rational[893, 2], Rational[397, 2]}, {
           Rational[1793, 2], Rational[2187, 2], Rational[1333, 2], Rational[
            917, 2], 210}, {801, 1009, 578, Rational[729, 2], 114}, {
           Rational[1389, 2], Rational[1831, 2], Rational[955, 2], 258, 
            Rational[13, 2]}, {
           Rational[1481, 2], 934, 521, Rational[1503, 2], 396}, {
           791, Rational[1909, 2], Rational[1135, 2], 1248, 790}, {
           790, 954, 567, Rational[2495, 2], 789}, {
           788, Rational[1907, 2], Rational[1131, 2], Rational[2493, 2], 
            787}, {790, Rational[1909, 2], Rational[1133, 2], Rational[
            2495, 2], 789}, {
           793, Rational[1911, 2], Rational[1137, 2], 1249, 792}, {
           Rational[1551, 2], Rational[1903, 2], 554, Rational[1565, 2], 
            Rational[863, 2]}, {
           Rational[1489, 2], Rational[1963, 2], 528, Rational[625, 2], 65}, {
           756, 983, 540, Rational[649, 2], Rational[153, 2]}, {
           755, Rational[1965, 2], Rational[1079, 2], 324, 76}, {
           Rational[1959, 2], 1021, 528, Rational[625, 2], 189}, {
           Rational[2463, 2], Rational[2063, 2], Rational[1091, 2], Rational[
            645, 2], Rational[647, 2]}, {
           1008, Rational[1987, 2], Rational[1115, 2], Rational[1573, 2], 
            Rational[1109, 2]}, {
           781, Rational[1901, 2], Rational[1125, 2], Rational[2487, 2], 
            780}, {843, Rational[2049, 2], 618, Rational[1709, 2], Rational[
            999, 2]}, {
           Rational[1797, 2], 1095, Rational[1337, 2], Rational[921, 2], 
            212}, {893, 1091, 663, 455, 206}, {
           898, Rational[2189, 2], Rational[1337, 2], Rational[921, 2], 
            Rational[423, 2]}, {905, 1100, 676, 468, 219}, {
           Rational[1811, 2], Rational[2201, 2], Rational[1353, 2], Rational[
            937, 2], Rational[439, 2]}, {
           Rational[1635, 2], 1019, 595, Rational[759, 2], 131}, {
           814, Rational[2031, 2], 591, Rational[751, 2], Rational[255, 2]}, {
           Rational[1797, 2], Rational[2187, 2], Rational[1337, 2], Rational[
            921, 2], Rational[425, 2]}, {
           Rational[1665, 2], Rational[2129, 2], Rational[1219, 2], Rational[
            803, 2], Rational[307, 2]}, {833, 1065, 610, 402, 154}, {
           Rational[1667, 2], Rational[2131, 2], Rational[1221, 2], Rational[
            805, 2], Rational[309, 2]}, {
           Rational[1669, 2], Rational[2133, 2], Rational[1223, 2], Rational[
            807, 2], Rational[311, 2]}, {
           Rational[1803, 2], Rational[2193, 2], Rational[1343, 2], Rational[
            927, 2], Rational[431, 2]}, {
           Rational[1713, 2], 1031, Rational[1251, 2], 410, Rational[
            325, 2]}}, {{1015, 992, 551.5, 328.5, 205.5}, {
           978.5, 980, 522.5, 299.5, 176.5}, {818, 970.5, 594, 371, 123.5}, {
           881, 996, 650, 427, 179}, {801.5, 967, 577.5, 354.5, 107}, {
           726, 935.5, 509.5, 285, 39}, {711.5, 928.5, 495, 266, 24.5}, {
           711, 928, 494.5, 265, 24}, {707.5, 925.5, 491.5, 272.5, 20}, {
           955, 965, 489.5, 269, 139}, {962, 970, 496.5, 269.5, 146.5}, {
           858.5, 927.5, 265, 277, 356}, {856.5, 925.5, 263, 273, 354}, {
           714.5, 930.5, 498.5, 273, 27.5}, {
           792.5, 1003.5, 569.5, 354, 106}, {784.5, 997.5, 561.5, 351.5, 
           97.5}, {700.5, 920.5, 484.5, 272, 12.5}, {
           699.5, 919.5, 483.5, 270, 11.5}, {710.5, 926.5, 494, 275.5, 23}, {
           723, 935, 506, 283, 36}, {941.5, 867.5, 652.5, 668, 305}, {
           933, 862, 644, 654, 296.5}, {707.5, 925.5, 490.5, 257, 20.5}, {
           927.5, 981.5, 522.5, 302, 183}, {1153.5, 919, 677, 699.5, 459.5}, {
           933, 862.5, 644.5, 653.5, 296.5}, {
           931, 858.5, 640.5, 651.5, 294.5}, {
           935.5, 862, 645.5, 661.5, 299}, {762, 993, 538, 321.5, 74.5}, {
           837.5, 1024.5, 606, 390.5, 142.5}, {874, 996, 643, 420, 172}, {
           872, 1035, 641.5, 426, 177.5}, {861.5, 1071.5, 631.5, 423.5, 
           174.5}, {860.5, 1070.5, 630.5, 422.5, 173.5}, {
           857.5, 1069, 627, 419, 170.5}, {850.5, 1066.5, 620, 412, 163.5}, {
           849.5, 1066, 619.5, 411.5, 163}, {800, 1006, 577, 361.5, 113.5}, {
           796, 1004.5, 573, 357.5, 109}, {1020.5, 1066.5, 622, 414, 289}, {
           1192.5, 1066, 621.5, 413.5, 412.5}, {
           1233.5, 1070, 628.5, 413, 413}, {1054, 1068.5, 621.5, 406, 282}, {
           834, 1061, 604, 396, 147.5}, {838.5, 1061.5, 607.5, 399.5, 
           151.5}, {1009.5, 1060, 603, 395, 270.5}, {
           1002, 1058, 595.5, 387.5, 263}, {964.5, 977.5, 402, 525.5, 410}, {
           989.5, 947, 427, 543, 428}, {848.5, 1027, 617.5, 402, 154.5}, {
           852, 1027.5, 621.5, 406, 158}, {
           867.5, 1031.5, 636, 420.5, 172.5}, {
           1005.5, 930.5, 708, 732, 368.5}, {
           1222.5, 941, 726, 742.5, 503.5}, {1220, 936, 721, 740.5, 500.5}, {
           1211, 927, 706, 725.5, 485.5}, {1064.5, 1030.5, 627.5, 404.5, 
           281.5}, {858.5, 991.5, 626.5, 403.5, 156.5}, {
           854.5, 990, 623, 400, 152.5}, {859, 992, 628, 405, 157}, {
           1018, 1025.5, 610, 394.5, 270}, {
           1190.5, 1034, 546, 330.5, 330.5}, {1185, 904.5, 652, 668, 429}, {
           1159.5, 800, 799, 1052.5, 573.5}, {
           1005.5, 895.5, 710.5, 725, 361.5}, {849.5, 987.5, 618.5, 395.5, 
           147.5}, {850.5, 988.5, 619.5, 396.5, 148.5}, {
           1001, 884.5, 699.5, 721, 356.5}, {1152, 784, 783, 1046, 565}, {
           1153.5, 788, 787, 1047.5, 566.5}, {1151, 784, 783, 1045, 564}, {
           960.5, 910.5, 668.5, 696, 331}, {771.5, 1039.5, 556.5, 348.5, 
           99.5}, {772.5, 1040.5, 557.5, 349.5, 100.5}, {773.5, 1041.5, 558.5,
            350.5, 101.5}, {771.5, 1039.5, 556.5, 348.5, 99.5}, {
           960.5, 910.5, 668.5, 696, 331}, {937, 859, 644, 664, 300}, {
           807, 1012, 583.5, 368, 120}, {810, 1013, 587, 371.5, 123}, {
           742, 982, 526.5, 311, 62.5}, {755.5, 1027.5, 540.5, 332.5, 83.5}, {
           759.5, 1030.5, 544.5, 336.5, 88}, {
           953.5, 1029.5, 543.5, 335.5, 211}, {
           946.5, 1024, 535.5, 327.5, 203}, {
           1004.5, 1040.5, 565, 349.5, 226}, {1044, 1018, 597, 374, 251}, {
           825, 975.5, 593.5, 370.5, 123}, {793.5, 1004, 570, 354.5, 106.5}, {
           757.5, 1029, 541.5, 333.5, 85.5}, {
           749, 1023.5, 533.5, 325.5, 77.5}, {749.5, 1024, 534, 326, 78}, {
           734.5, 1018, 519, 311, 63}, {737.5, 1020, 522.5, 314.5, 66}, {
           793, 1003.5, 569.5, 354, 106}, {980.5, 995, 553, 337.5, 213.5}, {
           936.5, 1016, 517.5, 309.5, 184.5}, {743.5, 982, 528, 312.5, 64}, {
           736, 979, 520.5, 305, 56.5}, {725.5, 1013.5, 510.5, 302.5, 53.5}, {
           743.5, 980.5, 528, 312.5, 64}, {752, 984.5, 536.5, 321, 72.5}, {
           723, 1014, 507, 299, 50.5}, {734, 1019.5, 517, 309, 61}, {
           761.5, 1031.5, 545.5, 337.5, 89}, {
           1011.5, 1047, 578.5, 363, 239}, {
           1011, 1046.5, 578, 362.5, 238.5}, {759.5, 988.5, 544, 328.5, 80}, {
           740, 978.5, 523.5, 308, 60}, {773, 993, 548.5, 333, 85.5}, {
           974.5, 992, 544.5, 329, 205}, {916, 1008.5, 494.5, 286.5, 161.5}, {
           756.5, 984.5, 534.5, 770.5, 420}, {811.5, 966, 581.5, 810, 460}, {
           810.5, 965.5, 581, 809.5, 459}, {
           1029, 1007.5, 578, 806.5, 580.5}, {
           1033.5, 1010, 580.5, 357.5, 234.5}, {
           777.5, 953.5, 553.5, 330.5, 83}, {775, 952, 552, 780.5, 431}, {
           812, 966, 581.5, 810, 460.5}, {815, 968, 583.5, 360.5, 113}, {
           814.5, 967.5, 583, 360, 112.5}, {822, 972.5, 590.5, 367.5, 120}, {
           968.5, 990.5, 537, 317.5, 193.5}, {955, 982, 526, 758, 530.5}, {
           817, 969.5, 588, 816.5, 465.5}, {834.5, 980.5, 603.5, 380.5, 
           132.5}, {833.5, 979.5, 602.5, 379.5, 131.5}, {
           838, 982, 606.5, 383.5, 136}, {843.5, 985.5, 611.5, 388.5, 
           141.5}, {840, 984, 608.5, 385.5, 138}, {
           839, 983, 607.5, 384.5, 137}, {831.5, 978.5, 600, 377, 129.5}, {
           813.5, 967.5, 583, 360, 112}, {810, 965, 579, 356, 108.5}, {813.5, 
           967.5, 581.5, 358.5, 111.5}, {825, 975, 593.5, 370.5, 123}, {
           988, 1000.5, 565, 349.5, 225}, {939.5, 1018.5, 524.5, 316.5, 
           191.5}, {737, 1017, 521, 313, 64.5}, {
           805.5, 1047.5, 582, 374, 125.5}, {
           1039, 1078.5, 645.5, 437.5, 312.5}, {
           1038, 1077.5, 644.5, 436.5, 311.5}, {875.5, 1078.5, 645.5, 437.5, 
           188.5}, {786.5, 998, 563.5, 351.5, 99}, {
           757, 943, 533.5, 314, 62}, {855, 1030, 624.5, 409, 160.5}, {
           792.5, 1002, 569, 354, 105}, {791, 1000.5, 567.5, 352.5, 103.5}, {
           790.5, 1000, 567, 351.5, 103}, {
           799.5, 1007.5, 576.5, 361, 112.5}, {908.5, 1103.5, 679.5, 471.5, 
           222.5}, {801, 1009, 578, 363.5, 114}, {
           781.5, 994.5, 557.5, 343, 94}, {874, 1078, 643.5, 435.5, 187}, {
           873.5, 1077, 642.5, 434.5, 186}, {800, 1006, 576, 360.5, 112.5}, {
           731.5, 939.5, 515.5, 292.5, 44.5}, {715, 929, 499, 280, 27.5}, {
           952, 962, 480.5, 264, 136}, {1045, 1044, 565.5, 352.5, 229}, {
           883.5, 1082.5, 652.5, 444.5, 196.5}, {
           1043.5, 1085, 656, 448, 323.5}, {
           1037.5, 1081.5, 649.5, 441.5, 317}, {879, 1081, 648, 440, 192}, {
           886, 1085, 655, 447, 199}, {885.5, 1084.5, 654.5, 446.5, 198.5}, {
           896.5, 1093.5, 666.5, 458.5, 210}, {801, 1009, 578, 364.5, 114}, {
           694.5, 915.5, 477.5, 258, 6.5}, {740.5, 934, 521, 751.5, 396}, {
           791, 954.5, 567.5, 1248, 790}, {790, 954, 567, 1247.5, 789}, {
           788, 953.5, 565.5, 1246.5, 787}, {
           790, 954.5, 566.5, 1247.5, 789}, {793, 955.5, 568.5, 1249, 792}, {
           775.5, 951.5, 554, 782.5, 431.5}, {744.5, 981.5, 528, 312.5, 65}, {
           756, 983, 540, 324.5, 76.5}, {755, 982.5, 539.5, 324, 76}, {
           979.5, 1021, 528, 312.5, 189}, {1231.5, 1031.5, 545.5, 322.5, 
           323.5}, {1008, 993.5, 557.5, 786.5, 554.5}, {
           781, 950.5, 562.5, 1243.5, 780}, {
           843, 1024.5, 618, 854.5, 499.5}, {
           898.5, 1095, 668.5, 460.5, 212}, {893, 1091, 663, 455, 206}, {
           898, 1094.5, 668.5, 460.5, 211.5}, {905, 1100, 676, 468, 219}, {
           905.5, 1100.5, 676.5, 468.5, 219.5}, {
           817.5, 1019, 595, 379.5, 131}, {814, 1015.5, 591, 375.5, 127.5}, {
           898.5, 1093.5, 668.5, 460.5, 212.5}, {832.5, 1064.5, 609.5, 401.5, 
           153.5}, {833, 1065, 610, 402, 154}, {833.5, 1065.5, 610.5, 402.5, 
           154.5}, {834.5, 1066.5, 611.5, 403.5, 155.5}, {901.5, 1096.5, 
           671.5, 463.5, 215.5}, {
           856.5, 1031, 625.5, 410, 
            162.5}}]]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{251, 474, 1106, 252, 488, 280, 1211, 503, 295, 1216, 518, 
       310, 1223, 533, 325, 1233, 548, 340, 1244, 563, 1150, 355, 578, 1164, 
       370, 1259, 593, 385, 1263, 608, 400, 1268, 623, 415, 1275, 638, 430, 
       1284, 653, 445, 1295, 668, 460}], 
      Line3DBox[{253, 475, 1107, 254, 489, 1118, 281, 504, 296, 1217, 519, 
       311, 1224, 534, 326, 1234, 549, 341, 1245, 564, 1151, 356, 579, 1165, 
       371, 594, 1176, 386, 609, 401, 1269, 624, 416, 1276, 639, 431, 1285, 
       654, 446, 1296, 669, 461}], 
      Line3DBox[{255, 476, 1108, 256, 490, 1119, 282, 505, 1129, 297, 520, 
       312, 1225, 535, 327, 1235, 550, 342, 1246, 565, 1152, 357, 580, 1166, 
       372, 595, 1177, 387, 610, 1186, 402, 625, 417, 1277, 640, 432, 1286, 
       655, 447, 1297, 670, 462}], 
      Line3DBox[{257, 477, 1109, 258, 491, 1120, 283, 506, 1130, 298, 521, 
       1137, 313, 536, 328, 1236, 551, 343, 1247, 566, 1153, 358, 581, 1167, 
       373, 596, 1178, 388, 611, 1187, 403, 626, 1194, 418, 641, 433, 1287, 
       656, 448, 1298, 671, 463}], 
      Line3DBox[{259, 478, 1110, 260, 492, 1121, 284, 507, 1131, 299, 522, 
       1138, 314, 537, 1143, 329, 552, 344, 1248, 567, 1154, 359, 582, 1168, 
       374, 597, 1179, 389, 612, 1188, 404, 627, 1195, 419, 642, 1200, 434, 
       657, 449, 1299, 672, 464}], 
      Line3DBox[{261, 479, 1111, 263, 493, 1122, 285, 508, 1132, 300, 523, 
       1139, 315, 538, 1144, 330, 553, 1147, 345, 568, 1155, 360, 583, 1169, 
       375, 598, 1180, 390, 613, 1189, 405, 628, 1196, 420, 643, 1201, 435, 
       658, 1204, 450, 1300, 673, 465}], 
      Line3DBox[{265, 481, 1112, 266, 1209, 495, 287, 1213, 510, 302, 1219, 
       525, 317, 1227, 540, 332, 1238, 555, 347, 1250, 570, 1156, 362, 585, 
       377, 1261, 600, 392, 1265, 615, 407, 1271, 630, 422, 1279, 645, 437, 
       1289, 660, 452, 1302, 675, 467}], 
      Line3DBox[{267, 482, 1113, 268, 496, 1123, 288, 1214, 511, 303, 1220, 
       526, 318, 1228, 541, 333, 1239, 556, 348, 1251, 571, 1157, 363, 586, 
       1170, 378, 601, 393, 1266, 616, 408, 1272, 631, 423, 1280, 646, 438, 
       1290, 661, 453, 1303, 676, 468}], 
      Line3DBox[{269, 483, 1114, 270, 497, 1124, 289, 512, 304, 1221, 527, 
       319, 1229, 542, 334, 1240, 557, 349, 1252, 572, 1158, 364, 587, 1171, 
       379, 602, 1181, 394, 617, 409, 1273, 632, 424, 1281, 647, 439, 1291, 
       662, 454, 1304, 677, 469}], 
      Line3DBox[{271, 484, 1115, 272, 498, 1125, 290, 513, 1133, 305, 528, 
       320, 1230, 543, 335, 1241, 558, 350, 1253, 573, 1159, 365, 588, 1172, 
       380, 603, 1182, 395, 618, 1190, 410, 633, 425, 1282, 648, 440, 1292, 
       663, 455, 1305, 678, 470}], 
      Line3DBox[{273, 485, 1116, 274, 499, 1126, 291, 514, 1134, 306, 529, 
       1140, 321, 1231, 544, 336, 1242, 559, 351, 1254, 574, 1160, 366, 589, 
       1173, 381, 604, 1183, 396, 619, 1191, 411, 634, 1197, 426, 649, 441, 
       1293, 664, 456, 1306, 679, 471}], 
      Line3DBox[{275, 486, 1117, 276, 500, 1127, 292, 515, 1135, 307, 530, 
       1141, 322, 545, 1145, 337, 560, 352, 1255, 575, 1161, 367, 590, 1174, 
       382, 605, 1184, 397, 620, 1192, 412, 635, 1198, 427, 650, 1202, 442, 
       665, 457, 1307, 680, 472}], 
      Line3DBox[{277, 682, 1308, 683, 278, 501, 1128, 293, 516, 1136, 308, 
       531, 1142, 323, 546, 1146, 338, 561, 1148, 353, 576, 1162, 1163, 368, 
       591, 1175, 383, 606, 1185, 398, 621, 1193, 413, 636, 1199, 428, 651, 
       1203, 443, 666, 1205, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1294, 444, 652, 1283, 429, 637, 1274, 414, 
       622, 1267, 399, 607, 1262, 384, 592, 1258, 369, 577, 1256, 354, 1149, 
       562, 1243, 339, 547, 1232, 324, 532, 1222, 309, 517, 1215, 294, 502, 
       1210, 279, 487, 1207, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1301, 451, 659, 1288, 436, 644, 1278, 421, 629, 
       1270, 406, 614, 1264, 391, 599, 1260, 376, 584, 1257, 361, 569, 1249, 
       346, 554, 1237, 331, 539, 1226, 316, 524, 1218, 301, 509, 1212, 286, 
       494, 1208, 264, 480, 1206, 262}]}, {
      Line3DBox[{690, 1000, 473, 689, 1106, 912, 691, 1107, 913, 692, 1108, 
       914, 693, 1109, 915, 694, 1110, 916, 695, 1111, 1006, 1206, 696, 1112, 
       917, 697, 1113, 918, 698, 1114, 919, 699, 1115, 920, 700, 1116, 921, 
       701, 1117, 922, 702, 1308, 1001, 923, 1002}], 
      Line3DBox[{704, 1007, 1207, 703, 488, 705, 1118, 924, 706, 1119, 925, 
       707, 1120, 926, 708, 1121, 927, 709, 1122, 1008, 1208, 710, 1009, 1209,
        711, 1123, 928, 712, 1124, 929, 713, 1125, 930, 714, 1126, 931, 715, 
       1127, 932, 716, 1128, 933, 717}], 
      Line3DBox[{719, 1010, 1210, 718, 1011, 1211, 720, 504, 721, 1129, 934, 
       722, 1130, 935, 723, 1131, 936, 724, 1132, 1012, 1212, 725, 1013, 1213,
        726, 1014, 1214, 727, 512, 728, 1133, 937, 729, 1134, 938, 730, 1135, 
       939, 731, 1136, 940, 732}], 
      Line3DBox[{734, 1015, 1215, 733, 1016, 1216, 735, 1017, 1217, 736, 520, 
       737, 1137, 941, 738, 1138, 942, 739, 1139, 1018, 1218, 740, 1019, 1219,
        741, 1020, 1220, 742, 1021, 1221, 743, 528, 744, 1140, 943, 745, 1141,
        944, 746, 1142, 945, 747}], 
      Line3DBox[{749, 1022, 1222, 748, 1023, 1223, 750, 1024, 1224, 751, 1025,
        1225, 752, 536, 753, 1143, 946, 754, 1144, 1026, 1226, 755, 1027, 
       1227, 756, 1028, 1228, 757, 1029, 1229, 758, 1030, 1230, 759, 1031, 
       1231, 760, 1145, 947, 761, 1146, 948, 762}], 
      Line3DBox[{764, 1032, 1232, 763, 1033, 1233, 765, 1034, 1234, 766, 1035,
        1235, 767, 1036, 1236, 768, 552, 769, 1147, 1037, 1237, 770, 1038, 
       1238, 771, 1039, 1239, 772, 1040, 1240, 773, 1041, 1241, 774, 1042, 
       1242, 775, 560, 776, 1148, 949, 777}], 
      Line3DBox[{781, 1149, 950, 779, 1150, 951, 783, 1151, 952, 785, 1152, 
       953, 787, 1153, 954, 789, 1154, 955, 791, 1155, 569, 793, 1156, 956, 
       795, 1157, 957, 797, 1158, 958, 799, 1159, 959, 801, 1160, 960, 803, 
       1161, 961, 805, 1163, 963, 807}], 
      Line3DBox[{806, 962, 1162, 804, 1255, 1054, 802, 1254, 1053, 800, 1253, 
       1052, 798, 1252, 1051, 796, 1251, 1050, 794, 1250, 1049, 792, 1249, 
       568, 790, 1248, 1048, 788, 1247, 1047, 786, 1246, 1046, 784, 1245, 
       1045, 782, 1244, 1044, 778, 1243, 1043, 780}], 
      Line3DBox[{809, 1055, 1256, 808, 1164, 964, 810, 1165, 965, 811, 1166, 
       966, 812, 1167, 967, 813, 1168, 968, 814, 1169, 1056, 1257, 815, 585, 
       816, 1170, 969, 817, 1171, 970, 818, 1172, 971, 819, 1173, 972, 820, 
       1174, 973, 821, 1175, 974, 822}], 
      Line3DBox[{824, 1057, 1258, 823, 1058, 1259, 825, 1176, 975, 826, 1177, 
       976, 827, 1178, 977, 828, 1179, 978, 829, 1180, 1059, 1260, 830, 1060, 
       1261, 831, 601, 832, 1181, 979, 833, 1182, 980, 834, 1183, 981, 835, 
       1184, 982, 836, 1185, 983, 837}], 
      Line3DBox[{839, 1061, 1262, 838, 1062, 1263, 840, 609, 841, 1186, 984, 
       842, 1187, 985, 843, 1188, 986, 844, 1189, 1063, 1264, 845, 1064, 1265,
        846, 1065, 1266, 847, 617, 848, 1190, 987, 849, 1191, 988, 850, 1192, 
       989, 851, 1193, 990, 852}], 
      Line3DBox[{854, 1066, 1267, 853, 1067, 1268, 855, 1068, 1269, 856, 625, 
       857, 1194, 991, 858, 1195, 992, 859, 1196, 1069, 1270, 860, 1070, 1271,
        861, 1071, 1272, 862, 1072, 1273, 863, 633, 864, 1197, 993, 865, 1198,
        994, 866, 1199, 995, 867}], 
      Line3DBox[{869, 1073, 1274, 868, 1074, 1275, 870, 1075, 1276, 871, 1076,
        1277, 872, 641, 873, 1200, 996, 874, 1201, 1077, 1278, 875, 1078, 
       1279, 876, 1079, 1280, 877, 1080, 1281, 878, 1081, 1282, 879, 649, 880,
        1202, 997, 881, 1203, 998, 882}], 
      Line3DBox[{884, 1082, 1283, 883, 1083, 1284, 885, 1084, 1285, 886, 1085,
        1286, 887, 1086, 1287, 888, 657, 889, 1204, 1087, 1288, 890, 1088, 
       1289, 891, 1089, 1290, 892, 1090, 1291, 893, 1091, 1292, 894, 1092, 
       1293, 895, 665, 896, 1205, 999, 897}], 
      Line3DBox[{911, 1005, 685, 910, 1307, 1104, 909, 1306, 1103, 908, 1305, 
       1102, 907, 1304, 1101, 906, 1303, 1100, 905, 1302, 1099, 904, 1301, 
       1300, 1098, 903, 1299, 1097, 902, 1298, 1096, 901, 1297, 1095, 900, 
       1296, 1094, 899, 1295, 1093, 898, 1105, 1294, 1003, 1004}]}}},
   VertexNormals->CompressedData["
1:eJx1fHdczt/7f2QlK7IiK6TsEeLWZWVFCclO9iZbIbJVRu9U9pY9MkMueSOV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    "]],
  Axes->True,
  DisplayFunction->Identity,
  Method->{},
  PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {-0.9999998571428571,
    0.9999998571428571}, {0., 0.9999997142857346}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.580191583777095*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["elrot", "Subsection",
 CellChangeTimes->{{3.5801929734686966`*^9, 3.5801929789250083`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"elrot", "[", 
     RowBox[{"p_", ",", "q_"}], "]"}], "[", "theta_", "]"}], "[", "aa_List", 
   "]"}], ":=", 
  RowBox[{"ReplacePart", "[", 
   RowBox[{
    RowBox[{"ReplacePart", "[", "\n", "        ", 
     RowBox[{"aa", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"aa", "[", 
         RowBox[{"[", "p", "]"}], "]"}], "*", 
        RowBox[{"Cos", "[", "theta", "]"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"aa", "[", 
         RowBox[{"[", "q", "]"}], "]"}], "*", 
        RowBox[{"Sin", "[", "theta", "]"}]}]}], ",", "p"}], "]"}], ",", "\n", 
    "        ", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"aa", "[", 
        RowBox[{"[", "p", "]"}], "]"}]}], "*", 
      RowBox[{"Sin", "[", "theta", "]"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"aa", "[", 
       RowBox[{"[", "q", "]"}], "]"}], "*", 
      RowBox[{"Cos", "[", "theta", "]"}]}]}], ",", "q"}], "]"}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"elrot", "[", 
    RowBox[{"3", ",", "5"}], "]"}], "[", "t", "]"}], "[", "pts5", 
  "]"}]], "Input",
 CellChangeTimes->{{3.580193079815779*^9, 3.5801931210351367`*^9}, 
   3.580193178031397*^9, 3.5801936502184043`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Sin", "[", "t", "]"}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "t", "]"}], "+", 
      RowBox[{"Sin", "[", "t", "]"}]}], ",", 
     RowBox[{"Sin", "[", "t", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Cos", "[", "t", "]"}], ",", 
     RowBox[{
      RowBox[{"Cos", "[", "t", "]"}], "-", 
      RowBox[{"Sin", "[", "t", "]"}]}], ",", 
     RowBox[{"Cos", "[", "t", "]"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5801931226942315`*^9, 3.5801931804945374`*^9, 
  3.5801933940407515`*^9, 3.5801936538816137`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{
   RowBox[{"Point", "[", "%", "]"}], "/.", " ", 
   RowBox[{"t", " ", "\[Rule]", " ", 
    RowBox[{"Pi", "/", "4"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.580193232913536*^9, 3.5801932646393504`*^9}, 
   3.580193381464032*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Point", "[", "pts5", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Point", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.58019361203222*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"Point", "[", "pts5", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.580193232913536*^9, 3.580193317586379*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["filledcurve", "Subsection",
 CellChangeTimes->{{3.580272190482526*^9, 3.580272199489041*^9}}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"filledcurve", "[", 
     RowBox[{"curve_", ",", 
      RowBox[{"{", 
       RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", "opts___"}], "]"}],
     ":=", "\n", "        ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"plottmp", ",", "grtmp"}], "}"}], ",", "\n", "            ", 
      RowBox[{
       RowBox[{"plottmp", " ", "=", " ", 
        RowBox[{"ParametricPlot", "[", "\n", "            ", 
         RowBox[{
          RowBox[{"curve", "//", "Evaluate"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
          RowBox[{"AspectRatio", "->", "Automatic"}], ",", "\n", 
          "            ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "            ", 
       RowBox[{"grtmp", " ", "=", " ", 
        RowBox[{"plottmp", " ", "/.", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Line", "[", "pts_", "]"}], ":>", " ", 
           RowBox[{"Polygon", "[", "pts", "]"}]}], ")"}]}]}], ";", "\n", 
       "        ", 
       RowBox[{"Show", "[", 
        RowBox[{"grtmp", ",", 
         RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
     "]"}]}], "\n", "    ", "\n", "    ", 
   RowBox[{
    RowBox[{"filledcurve", "[", "graphic_", "]"}], ":=", 
    RowBox[{"Show", "[", 
     RowBox[{"graphic", " ", "/.", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Line", "[", "pts_", "]"}], ":>", " ", 
        RowBox[{"Polygon", "[", "pts", "]"}]}], ")"}]}], "]"}]}]}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5802733622585473`*^9, 3.580273379802551*^9}}],

Cell[BoxData[
 RowBox[{"filledcurve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"x", "^", "2"}]}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotStyle", " ", "->", " ", "Red"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.580272412037198*^9, 3.5802724586758657`*^9}, 
   3.580273386866955*^9}],

Cell[BoxData[
 RowBox[{"filledcurve", "[", 
  RowBox[{"Graphics", "[", 
   RowBox[{"{", 
    RowBox[{"Blue", ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.580272412037198*^9, 3.5802724586758657`*^9}, 
   3.5802728264829025`*^9, {3.580272859546794*^9, 3.5802728627709785`*^9}, {
   3.580272925090543*^9, 3.58027303834002*^9}, {3.580273282276973*^9, 
   3.580273304573248*^9}, {3.5802733440775075`*^9, 3.5802733465246477`*^9}, {
   3.580273438092885*^9, 3.5802734412640667`*^9}, {3.580273590261588*^9, 
   3.580273650229018*^9}, {3.5802736921564164`*^9, 3.5802736978607426`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["graphics2dtosurfrev", "Subsection",
 CellChangeTimes->{{3.5802752169376287`*^9, 3.5802752676975327`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"graphics2dtosurfrev", "[", 
    RowBox[{"r_", ",", "n_"}], "]"}], "[", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"a_", ",", "b_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"c_", ",", "d_"}], "}"}]}], "}"}], "]"}], "]"}], ":=", "\n", 
  "        ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Polygon", "[", 
     RowBox[{"{", "\n", "        ", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"r", "*", "a", "*", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"k", " ", "-", " ", "1"}], ")"}], "/", "n"}]}], "]"}]}],
          ",", 
         RowBox[{"r", "*", "a", "*", 
          RowBox[{"Sin", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"k", " ", "-", " ", "1"}], ")"}], "/", "n"}]}], "]"}]}],
          ",", "b"}], "}"}], ",", "\n", "        ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"r", "*", "a", "*", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{"k", "/", "n"}]}], "]"}]}], ",", 
         RowBox[{"r", "*", "a", "*", 
          RowBox[{"Sin", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{"k", "/", "n"}]}], "]"}]}], ",", "b"}], "}"}], ",", "\n", 
       "        ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"r", "*", "c", "*", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{"k", "/", "n"}]}], "]"}]}], ",", 
         RowBox[{"r", "*", "c", "*", 
          RowBox[{"Sin", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{"k", "/", "n"}]}], "]"}]}], ",", "d"}], "}"}], ",", "\n", 
       "        ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"r", "*", "c", "*", 
          RowBox[{"Cos", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"k", " ", "-", " ", "1"}], ")"}], "/", "n"}]}], "]"}]}],
          ",", "\n", "                ", 
         RowBox[{"r", "*", "c", "*", 
          RowBox[{"Sin", "[", 
           RowBox[{"2", "*", "Pi", "*", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"k", " ", "-", " ", "1"}], ")"}], "/", "n"}]}], "]"}]}],
          ",", "d"}], "}"}]}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "n"}], "}"}]}], "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.58027611787816*^9, 3.580276120957336*^9}, 
   3.58027617295731*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"graphics2dtosurfrev", "[", 
    RowBox[{"1.2", ",", "50"}], "]"}], "[", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"3", ",", "3"}], "}"}]}], "}"}], "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5802758141807895`*^9, 3.580275917381692*^9}, 
   3.5802759573409777`*^9, {3.5802762514287987`*^9, 3.5802762621484118`*^9}, 
   3.5802763686135015`*^9, 3.5802764051965933`*^9, 3.580276451229226*^9, 
   3.5802764836280794`*^9, 3.5802765297967205`*^9, 3.5802765694529886`*^9, {
   3.5802766125014505`*^9, 3.580276665268469*^9}, {3.580277035986673*^9, 
   3.58027709834824*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", "%", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5802762773752823`*^9, 3.5802763182356195`*^9}}],

Cell[BoxData[
 Graphics3DBox[{
   Polygon3DBox[{{-1.2, 
      0., -1}, {-1.1905376415773734`, -0.1503998802771651, -1}, {
     3.57161292473212, 0.4511996408314953, 3}, {3.5999999999999996`, 0., 3}}],
    Polygon3DBox[{{-1.1905376415773734`, -0.1503998802771651, -1}, \
{-1.1622997933543573`, -0.2984278645978257, -1}, {
     3.4868993800630714`, 0.8952835937934772, 3}, {
     3.57161292473212, 0.4511996408314953, 3}}], 
   Polygon3DBox[{{-1.1622997933543573`, -0.2984278645978257, -1}, \
{-1.1157317830659017`, -0.4417494632216135, -1}, {
     3.347195349197705, 1.3252483896648404`, 3}, {
     3.4868993800630714`, 0.8952835937934772, 3}}], 
   Polygon3DBox[{{-1.1157317830659017`, -0.4417494632216135, -1}, \
{-1.0515680160526362`, -0.5781044089220584, -1}, {
     3.1547040481579085`, 1.734313226766175, 3}, {
     3.347195349197705, 1.3252483896648404`, 3}}], 
   Polygon3DBox[{{-1.0515680160526362`, -0.5781044089220584, -1}, \
{-0.9708203932499369, -0.7053423027509678, -1}, {
     2.9124611797498106`, 2.1160269082529033`, 3}, {
     3.1547040481579085`, 1.734313226766175, 3}}], 
   Polygon3DBox[{{-0.9708203932499369, -0.7053423027509678, -1}, \
{-0.8747623529056938, -0.8214565271144263, -1}, {
     2.6242870587170812`, 2.464369581343279, 3}, {
     2.9124611797498106`, 2.1160269082529033`, 3}}], 
   Polygon3DBox[{{-0.8747623529056938, -0.8214565271144263, -1}, \
{-0.7649087876984275, -0.924615891330947, -1}, {
     2.2947263630952826`, 2.773847673992841, 3}, {
     2.6242870587170812`, 2.464369581343279, 3}}], 
   Polygon3DBox[{{-0.7649087876984275, -0.924615891330947, -1}, \
{-0.6429921539747959, -1.0131935106024181`, -1}, {
     1.9289764619243879`, 3.039580531807254, 3}, {
     2.2947263630952826`, 2.773847673992841, 3}}], 
   Polygon3DBox[{{-0.6429921539747959, -1.0131935106024181`, -1}, \
{-0.5109351498780872, -1.0857924629592233`, -1}, {
     1.5328054496342616`, 3.2573773888776696`, 3}, {
     1.9289764619243879`, 3.039580531807254, 3}}], 
   Polygon3DBox[{{-0.5109351498780872, -1.0857924629592233`, -1}, \
{-0.37082039324993693`, -1.141267819554184, -1}, {
     1.1124611797498107`, 3.4238034586625523`, 3}, {
     1.5328054496342616`, 3.2573773888776696`, 3}}], 
   Polygon3DBox[{{-0.37082039324993693`, -1.141267819554184, -1}, \
{-0.22485757750286953`, -1.1787447008744265`, -1}, {
     0.6745727325086085, 3.536234102623279, 3}, {
     1.1124611797498107`, 3.4238034586625523`, 3}}], 
   Polygon3DBox[{{-0.22485757750286953`, -1.1787447008744265`, -1}, \
{-0.07534862343517605, -1.197632074113926, -1}, {
     0.2260458703055281, 3.592896222341777, 3}, {
     0.6745727325086085, 3.536234102623279, 3}}], 
   Polygon3DBox[{{-0.07534862343517605, -1.197632074113926, -1}, {
     0.07534862343517605, -1.197632074113926, -1}, {-0.2260458703055281, 
      3.592896222341777, 3}, {0.2260458703055281, 3.592896222341777, 3}}], 
   Polygon3DBox[{{0.07534862343517605, -1.197632074113926, -1}, {
     0.22485757750286953`, -1.1787447008744265`, -1}, {-0.6745727325086085, 
      3.536234102623279, 3}, {-0.2260458703055281, 3.592896222341777, 3}}], 
   Polygon3DBox[{{0.22485757750286953`, -1.1787447008744265`, -1}, {
     0.37082039324993693`, -1.141267819554184, -1}, {-1.1124611797498107`, 
      3.4238034586625523`, 3}, {-0.6745727325086085, 3.536234102623279, 3}}], 
   Polygon3DBox[{{0.37082039324993693`, -1.141267819554184, -1}, {
     0.5109351498780872, -1.0857924629592233`, -1}, {-1.5328054496342616`, 
      3.2573773888776696`, 3}, {-1.1124611797498107`, 3.4238034586625523`, 
      3}}], Polygon3DBox[{{0.5109351498780872, -1.0857924629592233`, -1}, {
     0.6429921539747959, -1.0131935106024181`, -1}, {-1.9289764619243879`, 
      3.039580531807254, 3}, {-1.5328054496342616`, 3.2573773888776696`, 3}}],
    Polygon3DBox[{{0.6429921539747959, -1.0131935106024181`, -1}, {
     0.7649087876984275, -0.924615891330947, -1}, {-2.2947263630952826`, 
      2.773847673992841, 3}, {-1.9289764619243879`, 3.039580531807254, 3}}], 
   Polygon3DBox[{{0.7649087876984275, -0.924615891330947, -1}, {
     0.8747623529056938, -0.8214565271144263, -1}, {-2.6242870587170812`, 
      2.464369581343279, 3}, {-2.2947263630952826`, 2.773847673992841, 3}}], 
   Polygon3DBox[{{0.8747623529056938, -0.8214565271144263, -1}, {
     0.9708203932499369, -0.7053423027509678, -1}, {-2.9124611797498106`, 
      2.1160269082529033`, 3}, {-2.6242870587170812`, 2.464369581343279, 3}}],
    Polygon3DBox[{{0.9708203932499369, -0.7053423027509678, -1}, {
     1.0515680160526362`, -0.5781044089220584, -1}, {-3.1547040481579085`, 
      1.734313226766175, 3}, {-2.9124611797498106`, 2.1160269082529033`, 3}}],
    Polygon3DBox[{{1.0515680160526362`, -0.5781044089220584, -1}, {
     1.1157317830659017`, -0.4417494632216135, -1}, {-3.347195349197705, 
      1.3252483896648404`, 3}, {-3.1547040481579085`, 1.734313226766175, 3}}],
    Polygon3DBox[{{1.1157317830659017`, -0.4417494632216135, -1}, {
     1.1622997933543573`, -0.2984278645978257, -1}, {-3.4868993800630714`, 
      0.8952835937934772, 3}, {-3.347195349197705, 1.3252483896648404`, 3}}], 
   Polygon3DBox[{{1.1622997933543573`, -0.2984278645978257, -1}, {
     1.1905376415773734`, -0.1503998802771651, -1}, {-3.57161292473212, 
      0.4511996408314953, 3}, {-3.4868993800630714`, 0.8952835937934772, 3}}],
    Polygon3DBox[{{1.1905376415773734`, -0.1503998802771651, -1}, {
     1.2, 0., -1}, {-3.5999999999999996`, 0., 3}, {-3.57161292473212, 
      0.4511996408314953, 3}}], 
   Polygon3DBox[{{1.2, 0., -1}, {
     1.1905376415773734`, 
      0.1503998802771651, -1}, {-3.57161292473212, -0.4511996408314953, 
      3}, {-3.5999999999999996`, 0., 3}}], 
   Polygon3DBox[{{1.1905376415773734`, 0.1503998802771651, -1}, {
     1.1622997933543573`, 
      0.2984278645978257, -1}, {-3.4868993800630714`, -0.8952835937934772, 
      3}, {-3.57161292473212, -0.4511996408314953, 3}}], 
   Polygon3DBox[{{1.1622997933543573`, 0.2984278645978257, -1}, {
     1.1157317830659017`, 
      0.4417494632216135, -1}, {-3.347195349197705, -1.3252483896648404`, 
      3}, {-3.4868993800630714`, -0.8952835937934772, 3}}], 
   Polygon3DBox[{{1.1157317830659017`, 0.4417494632216135, -1}, {
     1.0515680160526362`, 
      0.5781044089220584, -1}, {-3.1547040481579085`, -1.734313226766175, 
      3}, {-3.347195349197705, -1.3252483896648404`, 3}}], 
   Polygon3DBox[{{1.0515680160526362`, 0.5781044089220584, -1}, {
     0.9708203932499369, 
      0.7053423027509678, -1}, {-2.9124611797498106`, -2.1160269082529033`, 
      3}, {-3.1547040481579085`, -1.734313226766175, 3}}], 
   Polygon3DBox[{{0.9708203932499369, 0.7053423027509678, -1}, {
     0.8747623529056938, 
      0.8214565271144263, -1}, {-2.6242870587170812`, -2.464369581343279, 
      3}, {-2.9124611797498106`, -2.1160269082529033`, 3}}], 
   Polygon3DBox[{{0.8747623529056938, 0.8214565271144263, -1}, {
     0.7649087876984275, 
      0.924615891330947, -1}, {-2.2947263630952826`, -2.773847673992841, 
      3}, {-2.6242870587170812`, -2.464369581343279, 3}}], 
   Polygon3DBox[{{0.7649087876984275, 0.924615891330947, -1}, {
     0.6429921539747959, 
      1.0131935106024181`, -1}, {-1.9289764619243879`, -3.039580531807254, 
      3}, {-2.2947263630952826`, -2.773847673992841, 3}}], 
   Polygon3DBox[{{0.6429921539747959, 1.0131935106024181`, -1}, {
     0.5109351498780872, 
      1.0857924629592233`, -1}, {-1.5328054496342616`, -3.2573773888776696`, 
      3}, {-1.9289764619243879`, -3.039580531807254, 3}}], 
   Polygon3DBox[{{0.5109351498780872, 1.0857924629592233`, -1}, {
     0.37082039324993693`, 
      1.141267819554184, -1}, {-1.1124611797498107`, -3.4238034586625523`, 
      3}, {-1.5328054496342616`, -3.2573773888776696`, 3}}], 
   Polygon3DBox[{{0.37082039324993693`, 1.141267819554184, -1}, {
     0.22485757750286953`, 
      1.1787447008744265`, -1}, {-0.6745727325086085, -3.536234102623279, 
      3}, {-1.1124611797498107`, -3.4238034586625523`, 3}}], 
   Polygon3DBox[{{0.22485757750286953`, 1.1787447008744265`, -1}, {
     0.07534862343517605, 
      1.197632074113926, -1}, {-0.2260458703055281, -3.592896222341777, 
      3}, {-0.6745727325086085, -3.536234102623279, 3}}], 
   Polygon3DBox[{{
     0.07534862343517605, 1.197632074113926, -1}, {-0.07534862343517605, 
      1.197632074113926, -1}, {
     0.2260458703055281, -3.592896222341777, 
      3}, {-0.2260458703055281, -3.592896222341777, 3}}], 
   Polygon3DBox[{{-0.07534862343517605, 
      1.197632074113926, -1}, {-0.22485757750286953`, 
      1.1787447008744265`, -1}, {0.6745727325086085, -3.536234102623279, 3}, {
     0.2260458703055281, -3.592896222341777, 3}}], 
   Polygon3DBox[{{-0.22485757750286953`, 
      1.1787447008744265`, -1}, {-0.37082039324993693`, 
      1.141267819554184, -1}, {
     1.1124611797498107`, -3.4238034586625523`, 3}, {
     0.6745727325086085, -3.536234102623279, 3}}], 
   Polygon3DBox[{{-0.37082039324993693`, 
      1.141267819554184, -1}, {-0.5109351498780872, 
      1.0857924629592233`, -1}, {
     1.5328054496342616`, -3.2573773888776696`, 3}, {
     1.1124611797498107`, -3.4238034586625523`, 3}}], 
   Polygon3DBox[{{-0.5109351498780872, 
      1.0857924629592233`, -1}, {-0.6429921539747959, 
      1.0131935106024181`, -1}, {
     1.9289764619243879`, -3.039580531807254, 3}, {
     1.5328054496342616`, -3.2573773888776696`, 3}}], 
   Polygon3DBox[{{-0.6429921539747959, 
      1.0131935106024181`, -1}, {-0.7649087876984275, 
      0.924615891330947, -1}, {2.2947263630952826`, -2.773847673992841, 3}, {
     1.9289764619243879`, -3.039580531807254, 3}}], 
   Polygon3DBox[{{-0.7649087876984275, 
      0.924615891330947, -1}, {-0.8747623529056938, 0.8214565271144263, -1}, {
     2.6242870587170812`, -2.464369581343279, 3}, {
     2.2947263630952826`, -2.773847673992841, 3}}], 
   Polygon3DBox[{{-0.8747623529056938, 
      0.8214565271144263, -1}, {-0.9708203932499369, 
      0.7053423027509678, -1}, {
     2.9124611797498106`, -2.1160269082529033`, 3}, {
     2.6242870587170812`, -2.464369581343279, 3}}], 
   Polygon3DBox[{{-0.9708203932499369, 
      0.7053423027509678, -1}, {-1.0515680160526362`, 
      0.5781044089220584, -1}, {3.1547040481579085`, -1.734313226766175, 3}, {
     2.9124611797498106`, -2.1160269082529033`, 3}}], 
   Polygon3DBox[{{-1.0515680160526362`, 
      0.5781044089220584, -1}, {-1.1157317830659017`, 
      0.4417494632216135, -1}, {3.347195349197705, -1.3252483896648404`, 3}, {
     3.1547040481579085`, -1.734313226766175, 3}}], 
   Polygon3DBox[{{-1.1157317830659017`, 
      0.4417494632216135, -1}, {-1.1622997933543573`, 
      0.2984278645978257, -1}, {
     3.4868993800630714`, -0.8952835937934772, 3}, {
     3.347195349197705, -1.3252483896648404`, 3}}], 
   Polygon3DBox[{{-1.1622997933543573`, 
      0.2984278645978257, -1}, {-1.1905376415773734`, 
      0.1503998802771651, -1}, {3.57161292473212, -0.4511996408314953, 3}, {
     3.4868993800630714`, -0.8952835937934772, 3}}], 
   Polygon3DBox[{{-1.1905376415773734`, 0.1503998802771651, -1}, {-1.2, 
      0., -1}, {3.5999999999999996`, 0., 3}, {
     3.57161292473212, -0.4511996408314953, 3}}]},
  AutomaticImageSize->True,
  ImageSize->{456.0881736203397, 146.67909314767354`},
  ViewPoint->{-2.507872404535619, 2.1643117709655644`, 0.6901670526971182},
  ViewVertical->{-0.6094261114671814, 0.6071106681503912, 
   3.6816553968074506`}]], "Output",
 CellChangeTimes->{
  3.580276319663701*^9, 3.580276377068985*^9, 3.5802764188603754`*^9, {
   3.5802764678351765`*^9, 3.5802764895514183`*^9}, {3.5802765358440657`*^9, 
   3.580276574708289*^9}, 3.5802766186808043`*^9, {3.5802766502156076`*^9, 
   3.5802766708777895`*^9}, 3.580276993333233*^9, 3.5802770517005715`*^9, {
   3.580277083810408*^9, 3.580277105050623*^9}}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.580276206830248*^9, 3.5802762088263617`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["lathe", "Subsection",
 CellChangeTimes->{{3.580276905306198*^9, 3.580276906825285*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lathe", "[", "dots_", "]"}], ":=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Graphics3D", "[", "\n", "      ", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{"graphics2dtosurfrev", "[", 
        RowBox[{"0.5", ",", "60"}], "]"}], ",", "\n", "      ", 
       RowBox[{"linetolines", "[", 
        RowBox[{"Line", "[", "dots", "]"}], "]"}]}], "]"}], "]"}], ",", 
    RowBox[{"Boxed", "->", "False"}], ",", "\n", "      ", 
    RowBox[{"DisplayFunction", "->", 
     RowBox[{"dip", "[", 
      RowBox[{"EdgeForm", "[", "]"}], "]"}]}]}], "]"}]}]], "Input",
 Evaluatable->False],

Cell["\<\
We delete the last command in lathe defining\
\>", "Text",
 CellChangeTimes->{{3.5802779899862385`*^9, 3.580278028817459*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lathe1", "[", "dots_", "]"}], ":=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Graphics3D", "[", "\n", "      ", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{"graphics2dtosurfrev", "[", 
        RowBox[{"0.5", ",", "60"}], "]"}], ",", "\n", "      ", 
       RowBox[{"linetolines", "[", 
        RowBox[{"Line", "[", "dots", "]"}], "]"}]}], "]"}], "]"}], ",", 
    RowBox[{"Boxed", "->", "False"}]}], "\n", "   ", "]"}]}]], "Input",
 CellChangeTimes->{{3.5802776726460876`*^9, 3.5802777221019163`*^9}}],

Cell[BoxData[
 RowBox[{"lathe", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "4"}], "}"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5802772087705555`*^9, 3.5802772800196304`*^9}, {
  3.5802773391250114`*^9, 3.580277400052496*^9}, {3.5802774556686773`*^9, 
  3.5802775521011925`*^9}}],

Cell["Now the edges are shown", "Text",
 CellChangeTimes->{{3.580278047081504*^9, 3.5802780565370445`*^9}}],

Cell[BoxData[
 RowBox[{"lathe1", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "4"}], "}"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.5802772087705555`*^9, 3.5802772800196304`*^9}, {
   3.5802773391250114`*^9, 3.580277400052496*^9}, {3.5802774556686773`*^9, 
   3.5802775521011925`*^9}, 3.580277769140607*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["linetolines", "Subsection",
 CellChangeTimes->{{3.580278184825382*^9, 3.5802781910017357`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"linetolines", "[", 
   RowBox[{"Line", "[", "pts_", "]"}], "]"}], ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Line", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"pts", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "\n", "             ", 
       RowBox[{"pts", "[", 
        RowBox[{"[", 
         RowBox[{"i", " ", "+", " ", "1"}], "]"}], "]"}]}], "}"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{
       RowBox[{"Length", "[", "pts", "]"}], " ", "-", " ", "1"}]}], "}"}]}], 
   "]"}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"linetolines", "[", 
  RowBox[{"Line", "[", "pts5", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5802782918425035`*^9, 3.580278318075004*^9}, {
  3.5802783893880825`*^9, 3.580278400635726*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}], "]"}], ",", 
   RowBox[{"Line", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "}"}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5802783194450827`*^9, 3.5802784019778028`*^9, 
  3.5802785815240726`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{"Red", ",", " ", "Thick", ",", "%"}], "}"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.5802784112683344`*^9, 3.5802784308194523`*^9}, {
  3.580278464188361*^9, 3.5802784867066493`*^9}, {3.580278533146305*^9, 
  3.580278569748399*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["nomesh", "Subsection",
 CellChangeTimes->{{3.580278856785816*^9, 3.5802788595529747`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nomesh", "[", "graphics_", "]"}], ":=", "\n", "      ", 
  RowBox[{"Show", "[", 
   RowBox[{"graphics", ",", 
    RowBox[{"DisplayFunction", "->", 
     RowBox[{"dip", "[", 
      RowBox[{"EdgeForm", "[", "]"}], "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5802804520980635`*^9, 3.5802804538341627`*^9}, 
   3.580280589654931*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "dip"}]], "Input",
 CellChangeTimes->{{3.580280339410618*^9, 3.580280341617744*^9}, 
   3.5823800195801697`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox["\<\"DisplayFunction->dip[ins] inserts a Graphics3D primitive at \
the beginning of a Graphics3D list when the list is displayed. For example, \
DisplayFunction->dip[EdgeForm[]] displays polygons with no edges.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5802803433628435`*^9},
 CellTags->"Info3580287543-1090846"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{
          RowBox[{"dip", "[", "PLTPROGS`Private`ins_", "]"}], "[", 
          "PLTPROGS`Private`g_", "]"}], ":=", 
         RowBox[{"$DisplayFunction", "[", 
          RowBox[{"Insert", "[", 
           RowBox[{"PLTPROGS`Private`g", ",", "PLTPROGS`Private`ins", ",", 
            RowBox[{"{", 
             RowBox[{"1", ",", "1"}], "}"}]}], "]"}], "]"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[PLTPROGS`dip],
  Editable->False]], "Print",
 CellChangeTimes->{3.5802803433838453`*^9},
 CellTags->"Info3580287543-1090846"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DisplayFunction", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5802800984478354`*^9, 3.580280120528098*^9}}],

Cell[BoxData[
 RowBox[{"DisplayFunction", "[", "]"}]], "Output",
 CellChangeTimes->{3.5802800901783624`*^9, 3.580280122204194*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$DisplayFunction"], "Input",
 CellChangeTimes->{{3.5802800984478354`*^9, 3.580280120528098*^9}, {
  3.5802801987025695`*^9, 3.5802802115363035`*^9}}],

Cell[BoxData["Identity"], "Output",
 CellChangeTimes->{3.5802802134264116`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dip", "[", 
  RowBox[{"EdgeForm", "[", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"dip", "[", 
  RowBox[{"EdgeForm", "[", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.5802797881950903`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "x", "]"}], "*", 
    RowBox[{"Sin", "[", "y", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"nomesh", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5802804939834585`*^9, 3.580280502063921*^9}}],

Cell["\<\
nomesh doesn' t give the expected result. The syntax of dip isn\
\[CloseCurlyQuote]t clear. The built-in option Mesh -> None yields the wanted \
effect:\
\>", "Text",
 CellChangeTimes->{{3.580280621196735*^9, 3.580280708062703*^9}, 
   3.5802808960634565`*^9, {3.5802809311354623`*^9, 3.5802809720788045`*^9}}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Cos", "[", "x", "]"}], "*", 
    RowBox[{"Sin", "[", "y", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"Mesh", " ", "\[Rule]", " ", "None"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5802808726901197`*^9, 3.580280881828642*^9}}],

Cell["nomesh has been commented out.", "Text",
 CellChangeTimes->{{3.580281002038518*^9, 3.5802810148142486`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["orth4d", "Subsection",
 CellChangeTimes->{3.5801942403181562`*^9}],

Cell["Original version:", "Text",
 CellChangeTimes->{{3.5802002737480755`*^9, 3.58020028187654*^9}}],

Cell[BoxData[
 RowBox[{"   ", 
  RowBox[{
   RowBox[{
    RowBox[{"orth4d", "[", 
     RowBox[{"surf_", ",", 
      RowBox[{"{", 
       RowBox[{"u_", ",", "umin_", ",", "umax_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "vmin_", ",", "vmax_"}], "}"}], ",", "opts___"}], 
     "]"}], ":=", "\n", "        ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"g1", ",", "g2", ",", "g3", ",", "g4"}], "}"}], ",", "\n", 
      "        ", 
      RowBox[{
       RowBox[{"g1", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "1", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g2", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "2", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g3", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "3", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g4", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "4", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"Show", "[", 
        RowBox[{
         RowBox[{"GraphicsArray", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"g1", ",", "g2"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"g3", ",", "g4"}], "}"}]}], "}"}], "]"}], ",", "\n", 
         "                ", 
         RowBox[{"GraphicsSpacing", "->", 
          RowBox[{"{", 
           RowBox[{"0.2", ",", "0"}], "}"}]}], ",", "\n", "                ", 
         RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
     "]"}]}], "\n"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5801945032431946`*^9, 3.580194570256027*^9}, 
   3.580194955388056*^9, 3.580194998243507*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"sf4D", "[", 
   RowBox[{"u_", ",", "v_"}], "]"}], " ", "=", " ", 
  RowBox[{"{", 
   RowBox[{"u", ",", "v", ",", 
    RowBox[{
     RowBox[{"u", "^", "2"}], "+", 
     RowBox[{"v", "^", "2"}]}], ",", " ", 
    RowBox[{
     RowBox[{"u", "^", "2"}], "-", 
     RowBox[{"v", "^", "2"}]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{
    SuperscriptBox["u", "2"], "+", 
    SuperscriptBox["v", "2"]}], ",", 
   RowBox[{
    SuperscriptBox["u", "2"], "-", 
    SuperscriptBox["v", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.580195008019066*^9, 3.5802032419945097`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Drop", "[", 
  RowBox[{
   RowBox[{"sf4D", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5801948751834683`*^9, 3.5801948896152935`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"u", ",", "v", ",", 
   RowBox[{
    SuperscriptBox["u", "2"], "-", 
    SuperscriptBox["v", "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.580194891101379*^9, 3.58019501315936*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orth4d", "[", 
  RowBox[{
   RowBox[{"sf4D", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5801944365613804`*^9, 3.5801944945026946`*^9}, {
  3.580194589230113*^9, 3.5801946305114737`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"GraphicsArray", "::", "obs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"GraphicsArray is obsolete. Switching to GraphicsGrid. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/GraphicsArray\\\", ButtonNote \
-> \\\"GraphicsArray::obs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.580194637321863*^9, 3.580195018563669*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {{InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJy9fXucl+PWd5JD5YmHiNEmhA6EiCKt7WFTVPaWbcemTUS82uzYOpC841Cb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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFm3kcFtPbxp+ZOTNT1uxblsqWLaG0oFSWQpGKSChLCyK7hOwJKUUSytZi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              "]], Polygon3DBox[CompressedData["
1:eJwtm3fgV9Mfxu/9nM85F9n97JkysqnIDKFCIUVSSqmUjBRCKgotI6uydxnZ
W0mhyF5Z2SGRvefvefWcP873e597zh2fe895v5/3837fRr1O6nhirSiKn0NR
1PX/Rf3ZuSqKTtq5lXCl/SO17++yKFoIn5rUJ9xZbZ72Pa//zTX+UI3fQv0/
afyPanurbx+1yRr/qvpP1bhvhV/Q/52ED9P4LTV+vnAL4Y7CTYW/F/5K+Ab9
31j7Zur/1sKttL2O+o/R+aYkn+s/4Y+1/z6N2Ur/3xV+TtvNNP4Q4c2Fz9TY
J4WP1P4Xdcwz+v+r+rZXW139c4T7qn9b4dWEB8ai6CP8nvBrwv/TbzlXY/7T
sfsK/6n9w4X/EN5M+ED1ny9c1/7rhB/X9pY6vqXwGsJX6/oLhE/X/u91zFnC
7wh3E36p9HM/X+1TjW0t3Efb49S3go6foX2/aLu1xg8X7iLcRf1jtK8Sni7c
R/e7VP33aF9T7buw7muvqb41GVN335ba3kXnP17jh+oc7wq/qjEzeHY6fg/h
tYV3FT5a7TV+n9pjnFf9O6n/f+q/WHii2trCXwkfDtb5lvLsNf6S3L9WvocF
/Bb19xDeWfgUbU/QvpWEXxDeXds91F7XsTuoXafnc73a49peS/17qu8yHfOd
tm/SvqPqfnavaPtQtUf1ew7T/b2hMY2F++u8k9W/vv631jGHaPtQtTnqa8vv
0fhOGv+WxjcR/pZ5pu3TNP4QjX9deBfmi/AOwq/xbtXfhTkj/I7w7uo/hucp
vJNwO7VHdK791Fpqu7vaq9peQ+09be+p8b01fjeNf1PXv1P4au3fUPv+0bnb
aPs+jf1X/R9pe3Xt66e+vYTfFd5D43sJ7yo8Qn1tte9Bzs3813Yr9R+n/r2F
3xDeVbir8I7CbwvvJtxTuCW/T//v0r4t9H++8Fu6n2l63qvW/A6vUN+VausK
byr8Oe9a18RItBHeW9tnat+Puv7Fwifk9/WL8OfCL+t8i2teT6syfzX+LI3/
Wf2LhI9Q/ze6nzu0bzON+Uv9N6j9qb6DNeYGnesj9Q9jnQqPU9/Y4HN/oXaV
+pvxvtT/nfqPFh6WvJZ4fg/rnDepbxN+p/Aj2t5c59sxr/dH+e3CzYUbCt/P
s1TbRngPneMh7quyfcAevMR6Eu4svLXwl8J7C58s3E54sfA+woOED8BeMHeF
b9X+Jtr3hf7vJXySttuqfx3136nWkPWg6x2k/vZqM7XdBvuk/VOFN9X/JzT+
8vw+1hP+Gvuj336N2u/a7qDx7+p53q7zrVbzGllD23eorS7cQv2/6th9df0R
wkdiP4V7qb2lvtNp2n+t8Eb631H9c7W9o8YfJNxY+BXhfsJthTcWflZ4B+ED
hTdhPuha7YVn654a6ny7CLcVvlu4Et5d+EDhx4VXEl6b56F2c35fbdR/sPqf
U/+62neAfs8TwouFr1f/PurvIPy08P/Uv6twO+H7hZcX3k34AOGHhRsIb6/j
W2PfsNs6/nj8SfCz/hgbrvH91KZhX9T20vZBOn6WxqzO/er4x4S/EL5Exz+D
L9M5blVfodZU/fer/2XsNvZSuKvwND2Pa4Q7Cx+rvqeFHxOuC9+j/ke17zjh
5rpeG+HbhP9hvar/5mTb9A3zA3ukdn72b+3Ud6jGv6jx62PzhD8QHqwx5wnv
rOPbqO9WXW+KxrcU7sL5ha8SLjS+ldrx+B7BoP4HNP5BbIxwW/UdovHzhdfT
mFrd97qitrvyPjX+SPVP1fmu5nzqv1f9y6nvCLUddfx+6r9O+/5Q99/6f4va
39he9Zd1X2sFbXdRa6bx+2v8Tdr3l8b8p/93qSX1Hc5v1vX21JgB2j4z+/Pb
Nf6iYP/0N3MnX6On8O/Cd6v/CuH2wgfp+G7afkL3e6/w/sKH83uFbxdeT+Mb
qV3Avam1V39PHT8DHqH+dVn/auPy+94AW6I2Js+XFrq3zmrjc38/9X0ZvNbe
V9ta26O1r9T5emGzdP4dNL67+gYKf6a+u3S9SzVuJ+Ej1fduZZ8xSWMaaf8o
iJiOP0z/NmQ9ql2ivs1pGt81eSzX/059iyuv4UO16yhdb4DOMV/HPwWn0Ngu
ahdp7Jq8M22/Bt+pe18HjT9GeKbG348NFu4l/KTwA6xv4aOF7xW+RfgTfm9l
H7uPcHfmO/NT/XPwJ9p/h/Alwfyrm/p7C7+g/tnCP6h/uvDl6t9PeC/1d4e/
qP9m/Knu71i12/L7WR/+p3Zhvv82Gt9D4x/S+DvgB8JHCU8XvlF4Cx17uNqE
fPwe6u/AHFP/Dfhvbe+j803N52vFfMFe4TfUP1bHviI8QGM+4vraPgI+qv7L
mD/aPlzteuFLhZdq3CDhvtlfLxE+SfhY4T2Ft9b5HxR+NXj9faP+k4X7qL9V
9t/4+yOzvy4qr6eg/ScU5ienVOYD+PfvMl/B3++T+cLAyvwEvnKr7v+Typy0
pn1rVV7fqwqPhhNWtrerCI8S/lD/j+c3Zn7zuu73i8p+B38zT/8/Y/3hz4QX
6v8AnoHGbwefEf5c+DL9b5b5E3zpgOwfThA+UW0R61ntIt3fe5X3fSb8lP5v
x2/W+PU1fpbwtsKthdcT/lF4sPBA4f2Fn8RXC+8tvG5h/g6fb5f9E/x++8rX
byS8W822Y0lhTjlO1784+drw4e/V922w72+ldrn6Fur4U/Abpf03/Kdn5nsb
YrvU3hR+UXi2+qcJT9CYDYQH1X3sEh07DD6s812RfC6uAZ86sTIfho8t4j0L
H5358T96/ntofH+NPUN47cr2fzXsfWE+elzl38vv653XSy+NHyTcT8fPVP83
up+ZwhvBjdUS8w7/UpkvT9bx4/EHle1xnfsSjpX9U2QeC3fR+WYIL9H57sY+
VubPU3T8BKarrt1ReJLwOOHVK/v7FXT86cINK/vXBsLD4EeV+cKKwmcLr1bZ
fy8vfJpw3+yfT9bvGSncINmfX1uzP16jMh9YSeOHC6+IPcb/qH8iz0f3u6hy
nEJ8skplvrCc8KnMJ/W35Jnp/IOFL4V/Za67DpxP/ZtmHzCyNH8mfiG+JT49
uGbbhq+epf6tsn06t/TzPVjn3i3YZ4wipsn2e7S2R7B+df6dNaa38Cm8b7hY
ZU40vHS81L9y/Eo8vWJlPlVp/xDep46/T/gJXeN44W2C+cIIHdtb+CTWbfDc
/gJ/jr9VO6U0H9oS/6/jXwn2ryX+W22Q+s8pzH/h78Sk+lf8xHU1/oTM/2dG
28MP1b81PjGZrx2JPxN+KfjdH6ftDUv78hd4HzX79MaV+fHNGtOYmBJfrBZr
5qyra/yj6l+k8zTDRqtvvFoDxmITkudjX/W3JMZS31i15Wrm1BOJs4SvVt9J
3J+291C7jrHMUdax8BThY9T2gU/jG7S9VK05XFhtC21PVNtP2y3ULtP2dLVL
NbZZ3X3M0fHYQeGLS6/B7evmDHAF9uFn8WH4mouyfx9acyx7T+l4G59EP/dE
/AjnxLfj44mPPwv2/fym9ZPX92Dt21V4FeFHhD8R3k54JeGHhT8S3ka4o/Bz
le+T+2PuPY+9qnkOzgueW31K82/Wxnz1X1jzGtla+OnKvA++t5XwnMq8Db62
pf6fo7EHaftnDWuKXiR8oPAPwtvBk4U7C9fVttHxz1TmhfDB54PXZt/S/POF
YFvQrzTfZX1hu3hXcIDldPxDOn6Bxm2KjUA7wfcy17hGMj9nHjL/ngz2pUeX
5vfE68Skiwq/U7jZPI0fWzNH2074WeELhI/ineZ4k3nDPcwJto09S8cjTwXb
zh6l45/tNX6ujh+j8d2EN1H/GXAnfA3xAs9G42cIP1ZaLzmg7m00FXwdPq+z
xjYO5rPjK/MX+M2BwuOE78x85hBd73XhofC5wvOP+Yhtwf50yHxvm7rtD9wV
f7B53Rx2y7o5MVxpQp7/zG9sAfFF+8ynp2d+1Cbz6dtyPAC/Or+yfZ6Y+fbY
yvEI8Ucn4YuEH8j8u7fwlcJzcnx0rPCkyvHS48K9hK8Qni38KPFB3frhEP3v
JNxD/ZcJz1L/w8KHCV8ofL/wNPiu8AXYF+HJwvsJjxG+JcdHhwpPEL6POFv4
GOHLhZ8SfkR4vH7b/nAN/fYv4ZN1xwSsR55Xk7pjWGLXsaX1kKv5bdq+o7Td
al03xn4Re70IH645Bmub7RHcAA2nXbYvyxXWkNBfOqg9WTrmR195uzKPgb+g
FTB/0MbQDNAjO9Q8/rfS6x6ucGrp9Y9dWB3fWto+vB/M/QazttQ+COaGQ0rb
88Oy/8LOYd/O1Pb72j5L27vl39+Y9VjYX+HfVq7b1hKj/Bvcz7PpWNq3vAQ/
rNnHvB7MBQaW1uvwTS/DN2v2UXD3N4Wb1j0fWef4+hNLr/c3g7nDCaX1u245
3pmR45cL8H3a3rf0Gt9U53uqss6EvgRXeSSa+8BZTsd2RseOg4VPE349WatB
sxkm/H30ejxLeDjcA74sfLbwaG2/rfFjhK9gziZrXGhbaDjnCa+ttnnN8eHZ
cCP4s8YP076h+DpiFOEhwl9zf8nHogOdJbw0eu6fqX1nYjsra5VolqyVr5Ln
LmuGub4wee4z51lrS5LXEmuOtfMlNrTwGrpU+IvkWJmYmbWwOHltsSbOE35H
eKzwlTWv9feSuRhrfoC2P6ysCTL/z9D2v2g66j9V/SOTNTG0sAXqfzxZs0ar
RoOckczJ4Sto1A8k5wDQ/tGk4HLTk/VkOB1a9bnRuQA0a2Lf+5PHEgPfo+3L
o7kcGuagZM0VrfVpXeM+4U/x0TVrxMQ6HMNYYh5i23uT+4hxp2r7xuRYaBXt
+7SyvvauztWn7nsbH621c4+nCb8j3KhmPZxcwMhoLRxdfqb6BwtvXXPOAu3+
ieTfvkzDFx4RnTtAc8c2fp1sK7GRD6N/RXMnNE60ymHR2iaa5RLhh5L70EHR
Ps+O1jrRQJ9MjkmIRdA80UKHR2v7aKIThT9X/z3ES3Bd5mcy93+MdyD8WbIt
R1MhFpwrvG3hmBDu+mxy7A+HJVZ5Ojk2I2Yh1pyXHDsScxI7v5QcCxNDExu/
mBw7EyMTGz2frJUQIxFbv8D9F46x0SrmE2MX1iyWaQ3RWgSawzJtQvjpwhoF
uZ2XhF8vnOMh9/NUslaNZt1E27Mq66zoq/Dr0dp/WGlNblvh84Q7abum1ik5
XsDuYe+IxZ9LjqWJyTfT9uzKuhh62M/Y5sp6OjYGbXUO87ewxor9IedDrmez
YO4NB8deNAnWl0bVfC1yBluhcwq31/avhWPL+cmxJDEm2s7z0VoOGg/Xfi2Z
y3MPnP/V5Nj6hMw/XknWZoi58a2fJPtKfCy+82P0mcI+FG7yUTIXgaPg6z9N
9sX4fLjAB8m+F04Ad/kwWduFw6DdPB3NLeAYaI2zo7VUNEe01TnC1xbWWNFq
nonWitBsiP2eiNZiiAHRZmZEazVoNGhF88i5FNaMiF3JAV1YOIZFW3k5WbtB
YyE2JqdEbEuMjJb6VDTXgfOgFT4X7WvwOWhXz0ZrY2hYaGVzhW8trJkRKxEz
XV5YQ5os/E1ybD6vZr0OG4ztRX9EP2aNsbbQt8hbHlF37pH8Jbky4puRhXNm
6PkN6+Yab5TO3RHfoAWQw/u37nxj17r1xNWj9cad69Ynx+T4a2jpnOK4HE+d
UToHSb7pE/xS6RiK/Bs5K/gPOZcJOT5D2yGnN6Dm+32htP3rxjMVnlva/pJ7
6Fl37ogcBPnT27NvQK8irpur/5+Uju/gVoznt8Gprq8758i5sbHkZwdkW/uP
2lXq61j3tbD55Km5Prlr8tVrEDdr//Pa/rZwHIWW82XpeIp8zK/at3LNHJ38
4W/Btn926VwZ+e/fg3NmGwt/jM2uW8MnV3yOWrvSOWPyWOS/yA2SzyJ3fHbd
uR5yyNgfNCzs4y/ZPg3M9uuMYPuHRsT6XDN4faMhsX7XCrZv3fP6Hsg9ZX+8
b9351CHRetbJdedbJyXnD4h7iXfJy5NzINdATv3AZP3wvLrzQ+hZxNDEzmhY
dyXrgeSZ8Xnoe8ToxOZoan2j9b2Bdeev8ZX4TOJ84nu07f2JeTQWgYE8HvkV
cqXk8+DOxDI/5PdN3hy9llw0+XO0dfKl5H7R2G/Rdu+65w8cg/zyGzVrHw2y
v3u/si7CPnQH9Hpyg/jQB5PrDZhXzKfr6s43kwuDg5A/R08hVwpneSw5f3l7
3ZxuWt35JXJ1cD5ysWi6aLkNg3Np5NTwr3/UnDsjh4Y//l14St3589aFOQn5
SPRIahngCGiZaJr4999q1kbRSFcU/qlm/06OE8lgSfbn5DgJWb6umU+Qw9dr
Kb7JfAGNR5vFVzXnJtE40TbXCeYr1DDoZxZLa+Zb1AgsL/x9zbUY1GSsIPxD
zVosmmwD4R9r1nLRdFcS/ll4jM45MDm+Jc7Ff6ORw7fXD86NkyMnt7pycG6V
HCu57nWD6x0eqZk7oxHx3Mm5gHn+8C844oXBGl5f9R1bdy3Ge6VtCxi+xhxB
p3pU+xaW1quot3i25twgGhK6/3F111ag/3Nt8Ielr3l3st6NbgjHpFYAzXsT
jY/BuXdy8I2FU3B+7xQ0Dx37V+lcPJoiWmKD4FoCagqohVkxONdPzp/am5WC
a3GoyUEb5yWTaybnTC3BesG1NNTUoJX/V3MtDTU1aOP/1sxt4bgXa2zzwrU6
1OzAn/+puVaGHNuy3FrNuW5y3mjZpY7pGs0J4YLECPAzchjEKxsE1w/AsbG3
czO/v7PuPDZr4rhon4eve1K4v/AUrik8q7DONFfnWae03oQu9azw2qX1KXSZ
mcS0pfWZi2rWLjcqrZGj06CFrlxar0F3mYEdLK2/nI5v1vUWaP/LhXVIcjGN
SuuRE5P1/O+C9aOh+G7eEdfR+AHCV/HMas6nUT9zdeX82myeh/B1cCbheYVt
N/U1cE9s+CDh6+FY+KjC/P/GynUY2JRT1H8DnAyfhM0Xvo05UXN9zpnCUyvX
61CPdBrcR/gt4ZcK6wqvoTeX1heIs8l9Ny0db2+p643S+DvQrYR/io5BiT2J
YX/L+WDqi6gP+Do6v0pdADZwucoxMLEvHPSX6BiY2BeOinZKrrx5aQ318Jz/
hmvBuY6MjiGIHYgp9sz5OLganK2R7m+E+q/Uvh9ZM5VjXmJdYu6vomNeYl3y
m3BjODKxOfUX6AbvYbdK6wctomNyYnFiVvLh5MeJ9Yn5yZcT0xLLkq/YQNvX
MOeZd9h43c+5wtPQDXW+TYRHCk8W/kX9X0bHuMS2xOB/JmsIaAesib+TY3pi
eWLgxsLnCF+l438TPgkuC+dk3gtvpP4fgmMRYpI/gv0j2jA1HeR+qAFAz0HX
Qb8g58L73is4t0cOH/0GHYfcCzUX6O/o8OQSydGjf6KDUkvGM+TZbR6ce6GG
Ar0V3ZVcEDl69Ex0TWpHqCGhtmWL4NwhNRvoR+hI5HaooUDPQdcht0NNCPoM
Og25Gmo40G/QcYgHyWERH4Xg+I8cKs9qw2A+Ts6TZ7tRcHxEDQrxYi04l0MN
CHoxujHxJjkT4p3lg+NLcprEk6sGx4PdMl8/PrieEU6KvfqzdPxKTpN4c5Xg
+I2cKvHAasH1NcT08Es4LbkxajjQf9GBl+dZJ3MVOAvcEX6HFgeHRJs7uG7u
iEZHLRp4s8I1aa9E5w/RHb8szCvhj3BZ+OUH0fUN6Oro6ZtUrh+DN8AX3o6u
J4DXwmffj86vozujNzdk/qEh8v5rjrf2zPHSwcHx1a45HjogOL5qleOnQ4Lj
xw453jo6OP5pn+O17sHxVZscHx0RHI8fnOOhHsG5K2qCyC+QZyA+oqbgCvV3
DI7fyOGxfvcNjqf2yvHSocHx4C45HmsnvILGdk+2NXeWjs9aZj2vbTA/GRXt
/6gZRQt+X+MnFdaEmc/Xqb9r4Zoo1t9Z2MTSNavYRmwk9Sh3BtsmbBT5DPIa
2D5sIPUv9wTbNmwcvq1fsJaJpok/2yN4rTwQnXtnzaCfoZnfINw+uNZgTDK/
gHuQKyNnhv9qGczHb0t+d9SYYruwYcTT+wfbOmzeMq072NZgc9AD0AWwXdgw
8jPkaag/Q5OEy3BNbBk2jXqyq4NtLTaXerzrg7VbNFz88e7BXABOgO8+Ljh2
JYbFN/YPjvfJAaAPdQ62BfdE52qxCeiTaPLoA52CbcO90bkJbATvCw1/qvBh
wXoDmj16ZM9g+/xtdK4TDYZcFDkpas2mB9vKh6Jrs7CZ6BfkCNBHjwnWN9H8
sb0nBmvzaPTwi12C9TZyEvjiQcH6KTkLfPfJwbE5MTp6XO9g/ZWcBb7/pGD9
g5wHemuvYNt4d3SuGxuJ/koOA19zQnBugxzHiByLYcsfjM71Y9Px3fhw8mfk
0e4V/iu6lo6augDfyniD0rnhN5L5ELyI3BY5LnzPjsH1VeSQyR0zxyD6jaJr
k3gGHzG31VYp7Y//qFufRlvqXpjLwengHkOCuRMciviZOBouBCdaoP5hwdwG
jvOO8PBgbgXH+lB4VDA3g6N9IHxuMFeCM70tfLZw4+jnR+6O+gS4ExwK7fTe
urkQnGih+s4Jrs/j/ZM7pP5gi+jnSW6R+oLNo9cjuU7qAQiM0PfJTR6b+Q01
cv0Kr/Hd8/MdWtinNoyef3CDczKfqqn1LWwzeJfo9WiDvNMDdPzDwhcUzsH8
re0No98V7wyuwZxmLqMr4ovviL53fDJrkZq3noXXZIe8Xsg9UnPQJHp9kXuk
XoN8/6bRtQ3YIPgdNYjUHlKDR26THCfvdvvgtf+Xzt+jsA04SH23RNcmUv9A
LpOcJtxnh+DcLTlcai0fEm4WrW+RO2LOUn+K5k+sRcyF/nS28Mula7apV6GG
l1ifGq9R2l4zORZDg7pA2+smaz9oUNj6K5O1Cmw+WgaaBvU11NnAV6mhIBdF
zeGGGrtBcq5q99L6DPWVxF58w0BuiRwTei66LrkhckTUIt4YrI8dJTy5dA03
+i41d+T/qAnsllxjRr6UMWi7aLw8r1uD61eo+UeroMaUbw/OSOYOaHn4kg+5
RmGfglaDZkM9NTkUap9nJddLox2dnJx/gKvAWYYnfwOBXsMzRdtA40DLf6Bu
7QUNhnoraqx4ntS0Ect+gD3U9vhkretTtW+ia1541/wGjp2Q3Lcojwej5aDJ
4Vs+j7YF+Bi0Rd4x75a6/HPz+3ynNOe4Njn+X1T4N1HfT403Whg147zLS4UX
l36nlyXbE2KxxRnTj3aFjUG75B3xbqiZPD/PF37bwjyfRifH/twDesl50fEz
NWX4wk+FuxX2idT6vpscvxDHwEWoIT63MCc5nXMJzy+tGZKLJSf7ufq/q7k+
oEWeH9QID2buY99Lx/wRWyW8femYdUjyNzRokYxZTduzonOzO5b+VuTN5PpE
tK8+yTWc1IewrzlcIzq3i49Df6dGgNj8qGB9nRoBYq2uwXo/OQdivU2Jk7M9
ojZi2zyfWJ9ow9QowGX7J+fy4LTUCr2VfO/E3dQq9Uiu36ZmqWdyDT56MPuo
16JGmdoBYlBqDag5oHaCOg5qIRckx9/E4cQy1EhSG0lMQz3D49G5Xmqmto3W
48kV40Opj9uYVtrHsXaOT+aCrKHtovV9csNwDnI55HSoTb42OL9NDTj1OdT0
UhuBTcWWUqtDLQX3wLXPQMeLzieQK4fDUU+zU17/y2qIk2u+qLWAI1Hf1T7Z
N+AjhiZ/g4R2zxxKyTVY1F41yvYEe9OrsE1CW0BjIPY/Pbh2/I/oemzqstEi
0CTQJk4LrqUiv0r9MdpmVdmn4EvwZ9RS7RZdG0JtwmfR9hxtBA5ALRE1g9Qy
ooX/Hm3fqc3BZ7A+qIknX3NQsP/aNrk2vH9hvoA/QXsZILxxZRuLbYWjUUs0
Nbp2Da2H2qPbsn9C64Hv3pXfP9oM9pLnhT7cpe7apuaZz1HLQK0Z3wiQj0TL
+yeZ88J1jy5c+9Qs8ydqIagVo4aRfAu5hV+ja5qoZcLHw7+xj/ia/rreTcl6
Pt+8YXPJp7aNzkfdWLe+QY6e3Dz1KmyzD63jtmy/sW/UZ3xbs1ZMPQB6M9ol
3BJ9jG9w4JisJ/wXWkuruvno9OhaSbQdfBn6xNJgn0a+mPw52trddWuR1OA2
KaxJos1Ts0v+GI0e7b1RZd+KBk9+mxpuanUnEb9W5phwyz7Y6Gg+TW1X38La
OPn41oU1crQ7NDy0tFOD19d/yc/+zxwv3JftJ1oU8dD90bXGl2e9cUJ0beMh
WQ8fG62X8k3Jkmh/TK0GnGdx9PolFwtHQV8+P1qfpGYY/4X/Jbd6TGE9enS0
Hrp31s/HROuz1ASjf14Yrb9S04k+Py76+fHNC3rsldH6M994oZdcG62NoZGh
v14jfGDhb37Qq6+O1kdOznr6VdH6ChoV+YAp0bXa1GxjS8hRLgy2Kei9k6L1
XL7xQg+fLLx74W/GsEfXJGsL5EXQey+O1of5hvKP5BiV2BSfBh+pV3525Jzh
N7XKz44aGPjf9dFrH+0UfndDNP9HOyVeuDG69hztlfiqQeW5Qc3fX8kxJbEk
HIDawluj682oMUTfviI6n0DNNLWDK1eeSyQlsG/fJceL87NefVG0/s03oOjn
l0XPz65Zj56ITSr8DRZ6+CXR+YJOWX+/NDrfQg09/HTVymubGkPs5bfJ8Soa
LvN3aXK8jgaNPz8nWv+mxop6EmrseV/X1O2P4XMjhXcN9ofYb7TFPYPtGzWT
aKs7BT8PfAS+oXlwvLZ+5t+PBefe8I/Uz5CDQxuZFh2LoJFg7+t5vj0SrHXd
HnNtdOnaxqY5PqDGkXhvo+xPdg6OZdBrbw+OaYhF0FOnBsck+F/8Ldpnq+DY
Dr1ySnCMRy0Ueuak4HpO7DU1o/iyFsF6NHol3/h+XNg2YiOpNcEGoo0RP1AL
jUZG7PpDtC/qnP0hfAptChv/sbYXRmtbP6n9G82PiU+IQdG+uSbXOi94PmG/
+fbrvmAtFU31LfWfFaydo6F/JDw6WPu8KTp2RAPl+8SHsZOFv7kdkvka2hk5
2xHcf3IumhgGX0AMQeyAT+B7I54Rz4b1xrcuzyTXg1MXzrdKs5O/N+C7g1c1
9oJobW8xNlLbkfdb2qejz6HvkGuFg6K93hwdy/XIfIZ617eDNTquxzc21MMs
F+wb8ZGfBmtqxMdNKvsWcgzE3z8l6xfkGMjtoimhJfGNCPH2j8n6BTkKtEXq
pfgWFI1xhcyvyCVTE41vxcd+HKzRwS3gGJ8H12igPfKND/VqaJBok3zztF9h
jRItk2fIs2ua39/3yXoImjjPj3zU5tirrCf8kKzXkGNhPvyezI/IieAb8ZHP
FNbwiYc2rZxbIefLs+MZwgUX5PnxWzJ/IseC/vBLsp5Bjoh85RP1ZZ/hLKsl
RF/4OZkfkXNBq+Wbj5ULa7boGb8m+0NyRv8HZWHq8w==
              "]], Polygon3DBox[CompressedData["
1:eJwt13nYjlUaAPDX9318VChLdkaIUXS1T7JVNBIpSxpd9lQi+5IQRY1mSlPZ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              "]], 
             Polygon3DBox[{{1795, 1294, 1293, 1794, 929, 930}, {1809, 1307, 
              1306, 1808, 949, 950}, {1792, 1292, 1291, 1791, 925, 926}, {
              1303, 1447, 822, 938, 1801, 1302}, {1804, 1303, 1302, 1803, 941,
               942}, {1287, 1439, 816, 918, 1785, 1286}, {1797, 1296, 1295, 
              1796, 933, 934}, {1290, 1440, 817, 920, 1787, 1289}, {1301, 
              1446, 821, 936, 1799, 1300}, {1292, 1441, 818, 922, 1789, 
              1291}, {1807, 1305, 1304, 1806, 945, 
              946}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{559, 745, 1830, 560, 758, 585, 1896, 771, 599, 1899, 
            784, 613, 1904, 797, 627, 1911, 919, 810, 641, 1920, 920, 817, 
            1990, 648, 956, 824, 1995, 655, 976, 1972, 831, 662, 1925, 838, 
            676, 1928, 852, 690, 1933, 865, 704, 1940, 878, 718, 1949, 891, 
            732}], Line3DBox[{561, 746, 1831, 562, 759, 1840, 586, 772, 600, 
            1900, 785, 614, 1905, 798, 628, 1912, 921, 811, 642, 1921, 922, 
            818, 1991, 649, 957, 825, 1996, 656, 977, 832, 1874, 663, 839, 
            677, 1929, 853, 691, 1934, 866, 705, 1941, 879, 719, 1950, 892, 
            733}], Line3DBox[{563, 747, 1832, 564, 760, 1841, 587, 773, 1849, 
            601, 786, 615, 1906, 799, 629, 1913, 923, 924, 1018, 1983, 925, 
            926, 1861, 1027, 958, 959, 1868, 1035, 978, 979, 1875, 664, 840, 
            1880, 678, 854, 692, 1935, 867, 706, 1942, 880, 720, 1951, 893, 
            734}], Line3DBox[{565, 748, 1833, 566, 761, 1842, 588, 774, 1850, 
            602, 787, 1855, 616, 800, 630, 1914, 927, 928, 1019, 1984, 929, 
            930, 1862, 1028, 960, 961, 1869, 1036, 980, 981, 1876, 665, 841, 
            1881, 679, 855, 1887, 693, 868, 707, 1943, 881, 721, 1952, 894, 
            735}], Line3DBox[{567, 749, 1834, 568, 1165, 2019, 1166, 589, 
            1170, 2021, 1171, 603, 1175, 2023, 1176, 617, 1180, 2024, 1181, 
            631, 931, 932, 1020, 1985, 933, 934, 1863, 1029, 962, 963, 1870, 
            1037, 982, 1973, 983, 666, 842, 1882, 680, 1185, 2026, 1186, 694, 
            1190, 2027, 1191, 708, 1195, 1196, 722, 1953, 1200, 1201, 736}], 
           Line3DBox[CompressedData["
1:eJwVz8sqRAEcx/E/JnbWktQ0LlMuxW42Q9lZzGKSB2BhMWW2PAEbDbkUaigU
IxQW8hZDs5otIxnyBPI5i0+/c77n0jnppXJxtSMiFsh1Rmx0RWwyn4rYZocK
bz0RW3rB8T4H7NHSK3rR8RHHXGuHybP6oj3hUzu1VXb1vJ2hoc/aPobI8Of6
sB2hnyyjvLq3244xzgCT1PUJ28sU0wyyzotra3Yu+QfvfXL+bpedn3PBs3Zm
P/QVe0Vbq9nL5Nv1kr3lW7+zN3zpZdvUHuwj9/zov/wDrRAvlQ==
            "]], 
           Line3DBox[{572, 752, 1835, 573, 764, 1843, 592, 1897, 777, 606, 
            1901, 790, 620, 1907, 803, 634, 1915, 935, 814, 645, 1922, 936, 
            821, 1993, 652, 965, 828, 1997, 659, 985, 835, 669, 1926, 845, 
            683, 1930, 858, 697, 1936, 871, 711, 1944, 884, 725, 1955, 897, 
            739}], Line3DBox[{574, 753, 1836, 575, 765, 1844, 593, 778, 607, 
            1902, 791, 621, 1908, 804, 635, 1916, 937, 815, 646, 1923, 938, 
            822, 1994, 653, 966, 829, 1998, 660, 986, 836, 1877, 670, 846, 
            684, 1931, 859, 698, 1937, 872, 712, 1945, 885, 726, 1956, 898, 
            740}], Line3DBox[{576, 754, 1837, 577, 766, 1845, 594, 779, 1851, 
            608, 792, 622, 1909, 805, 636, 1917, 939, 940, 1023, 1986, 941, 
            942, 1864, 1031, 967, 968, 1871, 1039, 987, 988, 1878, 671, 847, 
            1883, 685, 860, 699, 1938, 873, 713, 1946, 886, 727, 1957, 899, 
            741}], Line3DBox[{578, 755, 1838, 579, 767, 1846, 595, 780, 1852, 
            609, 793, 1856, 623, 1910, 806, 637, 1918, 943, 944, 1024, 1987, 
            945, 946, 1865, 1032, 969, 970, 1872, 1040, 989, 990, 1879, 672, 
            848, 1884, 686, 861, 1888, 700, 874, 714, 1947, 887, 728, 1958, 
            900, 742}], 
           Line3DBox[{580, 756, 1839, 581, 768, 1847, 596, 781, 1853, 610, 
            794, 1857, 624, 807, 1859, 638, 947, 948, 1025, 1988, 949, 950, 
            1866, 1033, 971, 972, 1873, 1041, 991, 1974, 992, 673, 849, 1885, 
            687, 862, 1889, 701, 875, 1891, 715, 888, 729, 1959, 901, 743}], 
           Line3DBox[{582, 907, 1960, 908, 583, 769, 1848, 597, 782, 1854, 
            611, 795, 1858, 625, 808, 1860, 639, 951, 1967, 952, 1026, 953, 
            1968, 954, 1867, 1034, 973, 1970, 974, 1042, 993, 1975, 994, 674, 
            850, 1886, 688, 863, 1890, 702, 876, 1892, 716, 889, 1893, 730, 
            1009, 1010, 1011}], 
           Line3DBox[{731, 890, 1005, 1948, 717, 877, 1939, 703, 864, 1932, 
            689, 851, 1927, 675, 837, 1924, 661, 830, 1971, 975, 654, 823, 
            1969, 955, 647, 1989, 816, 918, 1919, 640, 809, 1966, 917, 626, 
            796, 1903, 612, 783, 1898, 598, 770, 1895, 584, 757, 1894, 558, 
            744, 902, 1012}], Line3DBox[CompressedData["
1:eJwVzDcvBmAUhuFXSfwAbbbbRPswEDWfXqLXIIg+IHrvi9kv8AtsJmaDRBBR
d5uaSFyGO89zzrnfN61von48JoSwhOfYEFbiQiiPD6EAqZjCNO4TQpiUV5xp
zqF+gBSMYRx3nFF5wRng7Ot7SMYwLt1H5BBOOS2cXX0HSXhxH5CD6McJJ8rZ
1p/dtmQietGHHhxzCjnz+gOnWL6a82U38nDEidhdu8+ai3Bjjsgu5GKfk2E3
oxfiXM+Rnbj1LltucNLtN/Uy/OrtsuP/P06bnOOk2W/opfjWW+SZe6tsxggn
yX5dX8OX3iib0PD/n3uC3ar+5N2K/DTXyjrUoJrzg2V9CR/ulbIKUUTc3vDo
/aJ5Ae+cLFmBTJTgDyqmPsA=
            "]], 
           Line3DBox[{738, 896, 1007, 1954, 724, 883, 1981, 1004, 710, 870, 
            1980, 1002, 696, 857, 1979, 1000, 682, 844, 1977, 998, 668, 834, 
            2013, 1139, 1055, 1076, 1075, 658, 827, 1058, 1053, 1072, 1071, 
            651, 1992, 820, 1057, 1051, 1066, 1065, 2004, 644, 813, 1056, 
            1049, 1141, 2000, 1140, 633, 802, 1965, 916, 619, 789, 1964, 914, 
            605, 776, 1962, 912, 591, 763, 1961, 910, 571, 2007, 751, 905, 
            906}]}, {
           Line3DBox[{1206, 1486, 744, 1205, 1830, 1404, 1207, 1831, 1405, 
            1208, 1832, 1406, 1209, 1833, 1407, 1210, 1834, 1408, 1211, 904, 
            1644, 1693, 1727, 2015, 1696, 1694, 1695, 1645, 2007, 1409, 1212, 
            1835, 1410, 1213, 1836, 1411, 1214, 1837, 1412, 1215, 1838, 1413, 
            1216, 1839, 1414, 1217, 1960, 1487, 1415, 1488}], 
           Line3DBox[{1219, 1728, 1894, 1218, 758, 1220, 1840, 1416, 1221, 
            1841, 1417, 1222, 1842, 1418, 1223, 2019, 1708, 1419, 1709, 2020, 
            1489, 1646, 503, 1150, 505, 504, 1647, 1729, 1961, 1224, 1843, 
            1420, 1225, 1844, 1421, 1226, 1845, 1422, 1227, 1846, 1423, 1228, 
            1847, 1424, 1229, 1848, 1425, 1230}], 
           Line3DBox[{1232, 1730, 1895, 1231, 1731, 1896, 1233, 772, 1234, 
            1849, 1426, 1235, 1850, 1427, 1236, 2021, 1710, 1428, 1711, 2022, 
            1490, 1648, 1697, 1703, 1698, 2016, 1568, 1699, 1491, 1650, 1732, 
            1962, 1237, 1733, 1897, 1238, 778, 1239, 1851, 1429, 1240, 1852, 
            1430, 1241, 1853, 1431, 1242, 1854, 1432, 1243}], 
           Line3DBox[{1245, 1734, 1898, 1244, 1735, 1899, 1246, 1736, 1900, 
            1247, 786, 1248, 1855, 1433, 1249, 2023, 1712, 1713, 1250, 1826, 
            1963, 1492, 1649, 1493, 1704, 1108, 1653, 1652, 1494, 1651, 1495, 
            1964, 1251, 1737, 1901, 1252, 1738, 1902, 1253, 792, 1254, 1856, 
            1434, 1255, 1857, 1435, 1256, 1858, 1436, 1257}], 
           Line3DBox[{1259, 1739, 1903, 1258, 1740, 1904, 1260, 1741, 1905, 
            1261, 1742, 1906, 1262, 800, 1263, 2024, 1714, 1715, 1264, 2025, 
            1716, 1496, 1654, 1497, 1705, 1657, 1982, 1658, 1656, 1498, 1655, 
            1499, 1965, 1265, 1743, 1907, 1266, 1744, 1908, 1267, 1745, 1909, 
            1268, 1746, 1910, 1269, 1859, 1437, 1270, 1860, 1438, 1271}], 
           Line3DBox[{1285, 1530, 1529, 1967, 1284, 1527, 947, 1283, 1525, 
            1918, 1805, 1282, 1523, 1917, 1802, 1281, 1800, 1916, 1520, 1280, 
            1798, 1915, 1517, 1279, 2000, 1688, 1689, 1633, 1662, 1661, 1278, 
            2008, 1659, 1660, 1630, 1632, 1631, 2005, 1277, 1515, 931, 1276, 
            1513, 1914, 1793, 1275, 1511, 1913, 1790, 1274, 1788, 1912, 1508, 
            1273, 1786, 1911, 1505, 1272, 1966, 1501, 1500, 1504}], 
           Line3DBox[{1288, 1503, 1583, 1989, 1439, 1287, 1507, 1584, 1990, 
            1440, 1290, 1510, 1585, 1991, 1441, 1292, 1792, 1861, 1586, 1442, 
            1294, 1795, 1862, 1587, 1443, 1296, 1797, 1863, 1588, 1444, 1612, 
            1823, 2003, 1621, 1589, 1634, 819, 1690, 1635, 1636, 1590, 1618, 
            1992, 1445, 1613, 1519, 1591, 1993, 1446, 1301, 1522, 1592, 1994, 
            1447, 1303, 1804, 1864, 1593, 1448, 1305, 1807, 1865, 1594, 1449, 
            1307, 1809, 1866, 1595, 1450, 1309, 1810, 1867, 1596, 1451, 
            1539}], Line3DBox[{1310, 1532, 1582, 1531, 1968, 1308, 1528, 1581,
             1988, 1808, 1306, 1526, 1580, 1987, 1806, 1304, 1524, 1579, 1986,
             1803, 1302, 1801, 1923, 1578, 1521, 1300, 1799, 1922, 1577, 1518,
             1299, 1824, 2004, 1622, 1576, 1628, 1627, 1298, 2009, 1625, 1626,
             1575, 1620, 1061, 1297, 1516, 1574, 1985, 1796, 1295, 1514, 1573,
             1984, 1794, 1293, 1512, 1572, 1983, 1791, 1291, 1789, 1921, 1571,
             1509, 1289, 1787, 1920, 1570, 1506, 1286, 1785, 1919, 1569, 1502,
             1534}], 
           Line3DBox[{1312, 1533, 1597, 1747, 1969, 1311, 1535, 1598, 1995, 
            1452, 1313, 1536, 1599, 1996, 1453, 1314, 1811, 1868, 1600, 1454, 
            1315, 1812, 1869, 1601, 1455, 1316, 1813, 1870, 1602, 1456, 1614, 
            2001, 1623, 1624, 1603, 1637, 1748, 2006, 1691, 1638, 1639, 1604, 
            1619, 827, 1615, 1537, 1605, 1997, 1457, 1317, 1538, 1606, 1998, 
            1458, 1318, 1814, 1871, 1607, 1459, 1319, 1815, 1872, 1608, 1460, 
            1320, 1816, 1873, 1609, 1461, 1321, 1970, 1540, 1610, 1462, 
            1550}], Line3DBox[{1323, 1541, 1749, 1971, 1322, 1542, 1750, 1972,
             1324, 1543, 1874, 1463, 1325, 1817, 1875, 1464, 1326, 1818, 1876,
             1465, 1327, 1973, 1544, 1466, 1616, 2002, 1692, 1545, 1640, 1641,
             1642, 2014, 1552, 1643, 1546, 1629, 1686, 1687, 2013, 1617, 1547,
             835, 1328, 1548, 1877, 1467, 1329, 1819, 1878, 1468, 1330, 1820, 
            1879, 1469, 1331, 1974, 1549, 1470, 1332, 1975, 1551, 1471, 
            1553}], Line3DBox[{1334, 1751, 1924, 1333, 1752, 1925, 1335, 839, 
            1336, 1880, 1472, 1337, 1881, 1473, 1338, 1882, 1474, 1339, 1976, 
            1554, 1663, 1700, 1701, 2017, 1611, 1702, 1555, 1665, 1753, 1977, 
            1340, 1754, 1926, 1341, 846, 1342, 1883, 1475, 1343, 1884, 1476, 
            1344, 1885, 1477, 1345, 1886, 1478, 1346}], 
           Line3DBox[{1348, 1755, 1927, 1347, 1756, 1928, 1349, 1757, 1929, 
            1350, 854, 1351, 1887, 1479, 1352, 2026, 1717, 1718, 1353, 1827, 
            1978, 1556, 1664, 1557, 1706, 2018, 1825, 1668, 1667, 1558, 1666, 
            1559, 1979, 1354, 1758, 1930, 1355, 1759, 1931, 1356, 860, 1357, 
            1888, 1480, 1358, 1889, 1481, 1359, 1890, 1482, 1360}], 
           Line3DBox[{1362, 1760, 1932, 1361, 1761, 1933, 1363, 1762, 1934, 
            1364, 1763, 1935, 1365, 868, 1366, 2027, 1719, 1720, 1367, 2028, 
            1721, 1560, 1669, 1561, 1707, 1672, 1999, 1673, 1671, 1562, 1670, 
            1563, 1980, 1368, 1764, 1936, 1369, 1765, 1937, 1370, 1766, 1938, 
            1371, 874, 1372, 1891, 1483, 1373, 1892, 1484, 1374}], 
           Line3DBox[{1376, 1767, 1939, 1375, 1768, 1940, 1377, 1769, 1941, 
            1378, 1770, 1942, 1379, 1771, 1943, 1380, 1195, 1722, 1381, 2029, 
            1723, 1724, 1674, 1679, 1678, 2010, 1382, 1676, 1677, 1675, 1564, 
            1981, 1383, 1772, 1944, 1384, 1773, 1945, 1385, 1774, 1946, 1386, 
            1775, 1947, 1387, 888, 1388, 1893, 1485, 1389}], 
           Line3DBox[{1403, 1567, 1009, 1402, 1959, 1784, 1401, 1958, 1783, 
            1400, 1957, 1782, 1399, 1956, 1781, 1398, 1955, 1780, 1397, 1822, 
            1954, 1681, 1683, 1682, 1396, 2012, 1684, 1685, 1680, 1726, 2011, 
            1829, 1395, 1725, 1953, 1828, 1394, 1952, 1779, 1393, 1951, 1778, 
            1392, 1950, 1777, 1391, 1949, 1776, 1390, 1821, 1948, 1565, 
            1566}]}}},
        VertexNormals->CompressedData["
1:eJzVXXl8TVe0TghqqLFqJtRUElTMwmqRGotKUfM8hppbQ0u1htLG0KqpWrNW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         "]],
       AutomaticImageSize->True,
       Axes->True,
       DisplayFunction:>Identity,
       ImageSize->{162.83445052179437`, 160.59648813514846`},
       Method->{},
       PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]},
       ViewPoint->{1.0510374007191583`, -2.847917043849156, 
        1.4948875856207378`},
       ViewVertical->{0.06179844145042458, -0.1625532572553326, 
        0.9847627510284752}], {192., -192.}, ImageScaled[{0.5, 0.5}], {360., 360.},
      ContentSelectable->True], InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJztXXt8VdWxBgQR8FJuRaGRKipVHoqKoAQjU6+tghLairXUKlUEjVypFqk8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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFmwnYF9P7xmc5MyNRRLYiCZVoQUIphUrIVrYWiSiJ7ErZSiJpj0pUtiI7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              "]], Polygon3DBox[CompressedData["
1:eJw1mwf8T9X/x++95577IU0lGipCiaSEkoaRNIkGDVkJSVOF0NCilEpCQnsv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              "]], Polygon3DBox[CompressedData["
1:eJwt1nmg11Max/Gfm3sHlRlUQyVSjRatmilKRaW0a58WWhGmDKaFNtq1a9eu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              "]], 
             Polygon3DBox[{{1479, 1690, 832, 663, 662, 981}, {1500, 1499, 
              1000, 692, 693, 1001}, {1449, 1448, 934, 603, 604, 935}, {1434, 
              1433, 920, 573, 574, 921}, {1424, 1656, 745, 559, 558, 908}, {
              1442, 1441, 927, 588, 589, 928}, {1428, 1660, 758, 574, 573, 
              915}, {1485, 1484, 986, 662, 663, 987}, {1493, 1492, 993, 677, 
              678, 994}, {1475, 1686, 819, 648, 647, 
              974}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{527, 729, 1756, 528, 741, 553, 1821, 754, 568, 1823, 
            767, 583, 1827, 780, 598, 1834, 793, 613, 1843, 940, 1104, 449, 
            1599, 452, 451, 1105, 809, 1790, 629, 1852, 815, 642, 1854, 828, 
            657, 1856, 841, 672, 1860, 854, 687, 1866, 867, 702, 1875, 880, 
            717}], Line3DBox[{529, 730, 1757, 530, 742, 1767, 554, 755, 569, 
            1824, 768, 584, 1828, 781, 599, 1835, 794, 614, 1844, 941, 1106, 
            1108, 1109, 1928, 1113, 1112, 942, 1110, 810, 1791, 630, 816, 
            1798, 643, 829, 658, 1857, 842, 673, 1861, 855, 688, 1867, 868, 
            703, 1876, 881, 718}], 
           Line3DBox[{531, 731, 1758, 532, 743, 1768, 555, 756, 1776, 570, 
            769, 585, 1829, 782, 600, 1836, 795, 615, 1845, 943, 1107, 944, 
            1136, 1952, 1037, 1137, 945, 1111, 946, 1792, 631, 817, 1799, 644,
             830, 1806, 659, 843, 674, 1862, 856, 689, 1868, 869, 704, 1877, 
            882, 719}], 
           Line3DBox[{533, 732, 1759, 534, 744, 1769, 556, 757, 1777, 571, 
            770, 1782, 586, 783, 601, 1837, 796, 616, 1846, 947, 1114, 948, 
            1138, 1953, 1038, 1139, 949, 1115, 950, 1793, 632, 818, 1800, 645,
             831, 1807, 660, 844, 1812, 675, 857, 690, 1869, 870, 705, 1878, 
            883, 720}], 
           Line3DBox[{535, 892, 893, 1760, 536, 904, 1889, 905, 557, 911, 
            1892, 912, 572, 918, 1896, 919, 587, 925, 1899, 926, 602, 932, 
            933, 1062, 1941, 1134, 1135, 1085, 1092, 1091, 1947, 1140, 1093, 
            1094, 1086, 1088, 1087, 1945, 1064, 970, 1906, 971, 646, 977, 
            1909, 978, 661, 984, 1913, 985, 676, 991, 1916, 992, 691, 998, 
            999, 706, 1879, 1006, 1007, 1011}], 
           Line3DBox[{537, 894, 1019, 896, 538, 906, 1022, 1920, 908, 558, 
            913, 1025, 1921, 915, 573, 920, 1897, 1028, 922, 588, 927, 1900, 
            1031, 929, 603, 934, 1902, 1034, 936, 1063, 1068, 1069, 1039, 
            1096, 1095, 1948, 1141, 1097, 1098, 967, 1075, 1905, 1074, 1065, 
            972, 1044, 1932, 974, 647, 979, 1047, 1933, 981, 662, 986, 1914, 
            1050, 988, 677, 993, 1917, 1053, 995, 692, 1000, 1919, 1056, 1002,
             707, 1880, 1008, 1059, 1010, 1013}], 
           Line3DBox[{542, 736, 1763, 543, 748, 1771, 562, 761, 577, 1825, 
            774, 592, 1830, 787, 607, 1839, 800, 620, 1847, 953, 1120, 1122, 
            1123, 1931, 1127, 1126, 954, 1124, 813, 1794, 636, 822, 1801, 651,
             835, 666, 1858, 848, 681, 1863, 861, 696, 1871, 874, 711, 1883, 
            887, 724}], 
           Line3DBox[{544, 737, 1764, 545, 749, 1772, 563, 762, 1778, 578, 
            775, 593, 1831, 788, 608, 1840, 801, 621, 1848, 955, 1121, 956, 
            1789, 1142, 1042, 1143, 957, 1125, 958, 1795, 637, 823, 1802, 652,
             836, 1808, 667, 849, 682, 1864, 862, 697, 1872, 875, 712, 1884, 
            888, 725}], 
           Line3DBox[{546, 738, 1765, 547, 750, 1773, 564, 763, 1779, 579, 
            776, 1783, 594, 1832, 789, 609, 1841, 802, 622, 1849, 959, 1128, 
            960, 1144, 1954, 1043, 1145, 961, 1129, 962, 1796, 638, 824, 1803,
             653, 837, 1809, 668, 850, 1813, 683, 863, 698, 1873, 876, 713, 
            1885, 889, 726}], 
           Line3DBox[{548, 739, 1766, 549, 751, 1774, 565, 764, 1780, 580, 
            777, 1784, 595, 790, 1786, 610, 803, 623, 1850, 963, 1130, 1147, 
            1146, 1955, 1150, 1148, 1149, 1131, 964, 1797, 639, 825, 1804, 
            654, 838, 1810, 669, 851, 1814, 684, 864, 1816, 699, 877, 714, 
            1886, 890, 727}], 
           Line3DBox[{550, 902, 1888, 903, 551, 752, 1775, 566, 765, 1781, 
            581, 778, 1785, 596, 791, 1787, 611, 804, 1788, 624, 965, 1904, 
            1132, 1152, 1151, 1956, 1155, 1153, 1154, 1133, 1951, 966, 640, 
            826, 1805, 655, 839, 1811, 670, 852, 1815, 685, 865, 1817, 700, 
            878, 1818, 715, 1015, 1016, 1017}], 
           Line3DBox[{716, 879, 1005, 1874, 701, 866, 1865, 686, 853, 1859, 
            671, 840, 1855, 656, 827, 1853, 641, 814, 1851, 628, 808, 1903, 
            1101, 1103, 1102, 625, 1949, 805, 1100, 1099, 939, 1842, 612, 792,
             1833, 597, 779, 1826, 582, 766, 1822, 567, 753, 1820, 552, 740, 
            1819, 526, 728, 891, 1018}], 
           Line3DBox[{721, 884, 1060, 1940, 1009, 708, 871, 1057, 1938, 1001, 
            693, 858, 1054, 1936, 994, 678, 845, 1051, 1934, 987, 663, 832, 
            1910, 1048, 980, 648, 819, 1907, 1045, 973, 633, 811, 1943, 1066, 
            968, 1078, 1077, 626, 806, 1076, 1040, 1071, 1929, 1070, 617, 797,
             1035, 1926, 935, 604, 784, 1032, 1924, 928, 589, 771, 1029, 1922,
             921, 574, 758, 1893, 1026, 914, 559, 745, 1890, 1023, 907, 539, 
            733, 1887, 1020, 895, 897}], 
           Line3DBox[{722, 885, 1061, 1012, 1881, 709, 872, 1058, 1939, 1003, 
            694, 859, 1055, 1937, 996, 679, 846, 1052, 1935, 989, 664, 833, 
            1911, 1049, 982, 649, 820, 1908, 1046, 975, 634, 812, 1067, 969, 
            1081, 1080, 627, 1950, 807, 1079, 1041, 1073, 1072, 1944, 618, 
            798, 1036, 1927, 937, 605, 785, 1033, 1925, 930, 590, 772, 1030, 
            1923, 923, 575, 759, 1894, 1027, 916, 560, 746, 1891, 1024, 909, 
            540, 1761, 734, 1021, 898, 899}], 
           Line3DBox[{723, 886, 1014, 1882, 710, 873, 1004, 1870, 695, 860, 
            1918, 997, 680, 847, 1915, 990, 665, 834, 1912, 983, 650, 821, 
            976, 635, 1942, 1084, 1083, 1082, 952, 1118, 1119, 1930, 1117, 
            1116, 1089, 951, 1090, 1946, 619, 799, 938, 1838, 606, 786, 1901, 
            931, 591, 773, 1898, 924, 576, 760, 1895, 917, 561, 1770, 747, 
            910, 541, 1762, 735, 900, 901}]}, {
           Line3DBox[{1157, 1418, 728, 1156, 1756, 1335, 1158, 1757, 1336, 
            1159, 1758, 1337, 1160, 1759, 1338, 1161, 1719, 1760, 1339, 1511, 
            896, 1654, 1887, 1512, 1419, 1761, 1340, 1513, 1420, 1762, 1341, 
            1162, 1763, 1342, 1163, 1764, 1343, 1164, 1765, 1344, 1165, 1766, 
            1345, 1166, 1888, 1421, 1346, 1422}], 
           Line3DBox[{1168, 1655, 1819, 1167, 741, 1169, 1767, 1347, 1170, 
            1768, 1348, 1171, 1769, 1349, 1172, 1889, 1423, 1350, 1514, 1920, 
            1424, 1656, 1890, 1515, 1425, 1657, 1891, 1516, 1426, 1770, 1351, 
            1173, 1771, 1352, 1174, 1772, 1353, 1175, 1773, 1354, 1176, 1774, 
            1355, 1177, 1775, 1356, 1178}], 
           Line3DBox[{1180, 1658, 1820, 1179, 1659, 1821, 1181, 755, 1182, 
            1776, 1357, 1183, 1777, 1358, 1184, 1892, 1427, 1359, 1517, 1921, 
            1428, 1660, 1893, 1518, 1429, 1661, 1894, 1519, 1430, 1662, 1895, 
            1185, 761, 1186, 1778, 1360, 1187, 1779, 1361, 1188, 1780, 1362, 
            1189, 1781, 1363, 1190}], 
           Line3DBox[{1192, 1663, 1822, 1191, 1664, 1823, 1193, 1665, 1824, 
            1194, 769, 1195, 1782, 1364, 1196, 1896, 1431, 1432, 1197, 1897, 
            1433, 1434, 1922, 1198, 1435, 1436, 1923, 1199, 1437, 1438, 1898, 
            1200, 1666, 1825, 1201, 775, 1202, 1783, 1365, 1203, 1784, 1366, 
            1204, 1785, 1367, 1205}], 
           Line3DBox[{1207, 1667, 1826, 1206, 1668, 1827, 1208, 1669, 1828, 
            1209, 1670, 1829, 1210, 783, 1211, 1899, 1439, 1440, 1212, 1900, 
            1441, 1442, 1924, 1213, 1443, 1444, 1925, 1214, 1445, 1446, 1901, 
            1215, 1671, 1830, 1216, 1672, 1831, 1217, 1673, 1832, 1218, 1786, 
            1368, 1219, 1787, 1369, 1220}], 
           Line3DBox[{1222, 1674, 1833, 1221, 1675, 1834, 1223, 1676, 1835, 
            1224, 1677, 1836, 1225, 1678, 1837, 1226, 932, 1447, 1227, 1902, 
            1448, 1449, 1926, 1228, 1450, 1451, 1927, 1229, 1452, 1838, 1720, 
            1230, 1679, 1839, 1231, 1680, 1840, 1232, 1681, 1841, 1233, 803, 
            1234, 1788, 1370, 1235}], Line3DBox[CompressedData["
1:eJwV0dkunlEUh/EtpuswRyLmMSUEUWNQlRJDa27NNZw6Fi7BENMNoUK5CTVP
rf72wZNnrf9e+/2+d71powuf5hNCCGs4SwohPSWEfGTgOjmEKv6AndQQBvSV
6mF8w1f8kdVwLQbV1TyCMYziRnbK6+7X85C+jscxgT35Od/KG7kpPl/dwJOY
wr6ZC76Tf+Tm+NvqXD7GlvPvXIAHeSa3xP+ovseJetvMDy7EkyyLW5GdGMKj
vkg9jWK86LO5DbvujeifUaKfQSle9b94w3l7fFd9HpdhFuV4k53xppnOuBN9
B89hAfP4G5F3c0/ci7qLF/ETB+7+5n/yXv4S96b+zJc4dL7MS3iX9/NA3J26
j69wZGaVVxCQE79V3K2ZCv4PLf5Acw==
            "]], 
           Line3DBox[{1250, 1468, 1467, 1904, 1249, 1466, 1850, 1733, 1248, 
            1465, 1849, 1731, 1247, 1464, 1848, 1729, 1246, 1728, 1847, 1462, 
            1245, 1581, 1946, 1744, 1546, 1565, 1564, 1244, 1741, 1944, 1561, 
            1545, 1560, 1559, 1243, 1929, 1557, 1558, 1544, 1556, 1068, 1242, 
            1554, 1555, 1543, 1629, 1941, 1750, 1241, 1461, 1846, 1726, 1240, 
            1460, 1845, 1724, 1239, 1723, 1844, 1458, 1238, 1722, 1843, 1456, 
            1236, 1721, 1842, 1453, 1455}], 
           Line3DBox[{1251, 1454, 1682, 1903, 1594, 1457, 1790, 1384, 1598, 
            1459, 1791, 1385, 1601, 1725, 1792, 1386, 1605, 1727, 1793, 1387, 
            1578, 1743, 1945, 1579, 1547, 1551, 1388, 1470, 1905, 1562, 1563, 
            1548, 1552, 1683, 1943, 1471, 1566, 1567, 1549, 1553, 812, 1472, 
            1568, 1569, 1550, 1742, 1942, 1389, 1576, 1463, 1794, 1390, 1614, 
            1730, 1795, 1391, 1618, 1732, 1796, 1392, 1622, 1734, 1797, 1393, 
            1626, 1951, 1469, 1394, 1473}], 
           Line3DBox[{1253, 1684, 1851, 1252, 1685, 1852, 1254, 1798, 1395, 
            1255, 1799, 1396, 1256, 1800, 1397, 1257, 1906, 1474, 1398, 1537, 
            1932, 1475, 1686, 1907, 1538, 1476, 1687, 1908, 1539, 1477, 821, 
            1258, 1801, 1399, 1259, 1802, 1400, 1260, 1803, 1401, 1261, 1804, 
            1402, 1262, 1805, 1403, 1263}], 
           Line3DBox[{1265, 1688, 1853, 1264, 1689, 1854, 1266, 829, 1267, 
            1806, 1404, 1268, 1807, 1405, 1269, 1909, 1478, 1406, 1540, 1933, 
            1479, 1690, 1910, 1541, 1480, 1691, 1911, 1542, 1481, 1692, 1912, 
            1270, 835, 1271, 1808, 1407, 1272, 1809, 1408, 1273, 1810, 1409, 
            1274, 1811, 1410, 1275}], 
           Line3DBox[{1277, 1693, 1855, 1276, 1694, 1856, 1278, 1695, 1857, 
            1279, 843, 1280, 1812, 1411, 1281, 1913, 1482, 1483, 1282, 1914, 
            1484, 1485, 1934, 1283, 1486, 1487, 1935, 1284, 1488, 1489, 1915, 
            1285, 1696, 1858, 1286, 849, 1287, 1813, 1412, 1288, 1814, 1413, 
            1289, 1815, 1414, 1290}], 
           Line3DBox[{1292, 1697, 1859, 1291, 1698, 1860, 1293, 1699, 1861, 
            1294, 1700, 1862, 1295, 857, 1296, 1916, 1490, 1491, 1297, 1917, 
            1492, 1493, 1936, 1298, 1494, 1495, 1937, 1299, 1496, 1497, 1918, 
            1300, 1701, 1863, 1301, 1702, 1864, 1302, 863, 1303, 1816, 1415, 
            1304, 1817, 1416, 1305}], 
           Line3DBox[{1307, 1703, 1865, 1306, 1704, 1866, 1308, 1705, 1867, 
            1309, 1706, 1868, 1310, 1707, 1869, 1311, 998, 1498, 1312, 1919, 
            1499, 1500, 1938, 1313, 1501, 1502, 1939, 1314, 1503, 1870, 1735, 
            1315, 1708, 1871, 1316, 1709, 1872, 1317, 1710, 1873, 1318, 877, 
            1319, 1818, 1417, 1320}], 
           Line3DBox[{1334, 1510, 1015, 1333, 1886, 1718, 1332, 1885, 1717, 
            1331, 1884, 1716, 1330, 1883, 1715, 1329, 1740, 1882, 1509, 1328, 
            1739, 1881, 1508, 1327, 1940, 1507, 1880, 1738, 1326, 1506, 1879, 
            1737, 1325, 1878, 1714, 1324, 1877, 1713, 1323, 1876, 1712, 1322, 
            1875, 1711, 1321, 1736, 1874, 1504, 1505}]}}},
        VertexNormals->CompressedData["
1:eJztXXd8lUXWDk16hwVCMYgCKwRYAtIZpS0KQghdijQR6YJ0AaUtoDSXFgRE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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {576., -192.}, ImageScaled[{0.5, 0.5}], {360., 360.},
      ContentSelectable->True]}, {InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nX28VlWVx9HUJu3DOI1NfsgUi1AxGSUzwBurslFQqSadpBkyxzTJkbGo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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmnfgjlUbx+9zn/NkZiYKqewRSgrZOyLR0kZTivbQkEpaiNCwKjS1Rctq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              "]], Polygon3DBox[CompressedData["
1:eJwtmnfgV9Mfxu8993ysyE5SNJQoGrSQhMxIe6DSTiqrIQ2lrGSXkRINZDSt
iJA9GkZUMn5I9h7N3/PynD9u3/u6534+fT/3c877ed7P+VbpMbjNoJBl2Rj9
E/WzV5Fl/XQ+Pc+yW8WtxMs1cJ14hLibeIX4bvFE8T7iarr/NHET8QTx/zT+
oniBuI/4A/EM8e3i8eLrdf8L4vniquJFGu8r7iQeLP5E/LB4qvha8VfiZeKF
4ht0/oqOCeJh4j01/o/4FHFDcTWdL9Xxkv6PTeJN+vmDeB+N7y1uo/tfE98g
HiluK35dfKN4lHgv8b/iFuJG4nbiN8QTxaPFe4s3i08VNxb/qvf/SVxZXF58
ocZXie8VTxJ3Fb8nvkt8o7iD+E3xLeKx4k7it8W3ia8R9xa/L75ffJu4tfhV
8fXiq8TldN5Z/+fN4qvFR2j8OPEgcXc+v3ix7ukn7izuKH5LfKt4nLi7eKX4
HvFN4l/02h/Fh4gPEPfV+IfiB8R3iAeKL9E9c8T3iPcVz9N4J/EZ4v3E8/md
xGeKv9e9W3S+n459xFM0vknjK8Uvi28WfyN+XbxEfKf4br3mPfEy8Q6d/6rx
Y8Q1xCWNzxWfKT5OXEFcS/d0FZ8rni3+XuOfiVeJDxIvEHcTtxbvLn5MfK74
JPGHeu3TOr7U+d86PtD5+zr+1PlfOqLuf0T3n6H7jxWXET8qbiVuLt5NfJDu
P0d8oniW+GHxBvHKtB6e0P0dxaen+cL66CA+TXyJeK3GHxHfJ/5TYw+IjxXX
Ev8uvl/cWHy4uJ/u/0j8oPhO8TTxDN3zkfhN8Radz9R4M3E98VbxLPGJ4vri
beLZ4ubio8X99fo14pniyeJB4o/FD4nvTZ+Xz9csvT6IH4pe38z3XLxv8Pxn
fRR8fo2fntb/b9QOcSNxTfEf4hniJuIjxGvFK3X8zmt17S+dP6jx43ReW9cm
6/2+Fa8Qv5TW1zRxQ/Fh4i/FZ5ey7Fn93EW8WT/L6DhB43XFZfX6v3X/yeIG
4uvEX1MPxIvEo8Wfi58Wz+X74VzHcPFF4nV6r0nR85d6sUF8i7i8eF/xp6w9
cbk0vz8X3yauKN5ffKnef514rniauJl4/5LX54XiSuIdGm8jPkV8iDjXeDvx
qcxXjf2Yu17tJd5DfHuqL6zPHrp/tXiq+GZxI/Heev1F4gvS/Gd9tErz/XFx
O42v0LW3qad67V7i/ho/X/yZrt+qaxVYs9S/NP6Crn8rPlCv36ZrZ6f5zvrb
Hj3/WQ9VdX6W7v1Yx1ZxFY0XJc935v9xzBfxxeJu4oMLFrXqrrhFqhd36z3q
iKuI62v85GA9oH7VE++p+/skfbieZy++VNxTfIrGDxBfJu4lPl18oPgKcR/x
meIK4iHivuKWzG/xUHE/6pfe71Ad2/nudDTV+H4aH5jq6aZUz47SUZn5lPTl
E12/RFxH9++h+3tpvIN4o67fpfEjxBXF5TW+VdxS3ExcV1xW9/cWdxSfKi4v
vlzcW/w/vf4O3V816clX4jv5HcUHio/U/WV0fw9xO/E3Gp+i8Zrig8RfiyeL
a4griNvr/nN1/6u6PkbcRLyPeIDGu1LP0/N8WePfiYdr/DNdW6jxWZn1v43G
39T4AHFFaov4Xeq0eJjGN+jaAt0/M9XHv6LXI/VoY2G9fgOdYf5QS/X6t8Q/
i0dp/GqdP6X7H8m8Dvm/fsq8HnfV+J96TdO0vn/Q2D3iuuKq4h/F94rriaul
+cL8GZq+7zHiLzT+jPhR6rPO25c8X/EHXTT+rq7dofEJ4qPEu2u8p7i9eKj4
U43PFz+YWb/R39d0XCa+Qjxc509ofIZ4iHi97p8nfiDVy99T/aIertG9z+n4
Wuf/ZhRB1UyNN0j1bRHrO2nS5+LO4neoARofL64t3k2/34XituLDxbuIL0h6
eJ64rfgdvf7azP4qlqxH6HWt5BfQxzbiyuKg8fZJn8qJt7CmxSeIPwqutV/p
/B8d0zX+ncbXaPwtrvE9io9M66O6xkt6vy7is8WHiXcSnyc+J9XrX6LXU5Wk
L+hJo6R3W1I9r5O+35oa31mvP1/cKrPf2bVk/Uffdxb/ofc7XlyHeopWaXyV
3uOPVE+e0rVhab1PEt/K9yd+NtV76i2eAB98i8Y3Ml917TnxfF38JrdWoVkL
xBt1HnO//wLdf6b+v/XiFZm9zS7Rr8XjUOuoeazVOsFatLOOxZk1aS7fs36G
3OvpleDaso3amFmL1qXf7afctaqD3nNM7ppVLdXf0blf87zOa+l8M7+LjuaF
5/syXe+R2UvvFK2FeGq8einaW+PZ+ew8g8fEtwd7g62FvTgeAW++rbB3waOz
tmL0/awx1gqNBLWANYMWoolo1RXB3md7YW3EA6HFRfTaRJO/1PnVafyaYO+4
m45XMntIvMGu0V4Sj4D3K6NjdWYPyNxkjvL8H0jev2a0lqKpaDcajgz9nHut
7ii8tlmzaP2G1Av9kruWUlPLiP/O7S3RWLS1drAXxhPjFasHe1U8K17mgGDv
iKfCS1UM9oZ4RLzSocHeFI+KN6sU7KXx1HjdGqm2UePQ5u25vSEekdpeCvaq
eFa8Y7nkvfBgaEVM3gsPhjbslLwmnhNt2jnVUmoq3mZbbi1Gk/E2W/E00frB
vGQ+4nXwPDvp/Lfc3gSPQj37I7eXwFPoJdmvubUJjdqdWpFby9C03cR/5dY6
NG9X8Z/iZ3ReKXetoeaUxWuUPK+ZzwdF17e3qSOZdWR5sHajJwdH+535yWNW
jq6nzI3Vqb62LNmv0y+g1Wj2HtTj3NoXdMzOrIFoWx6thWgc3nmL3mNKZg/N
9/lB4dpKjeX7fB8NytxzPM7zER/D75bmy4eFeyV6KL7vj8RnZe6hXhB/Kb5c
fLV4obhier8j0/xaJT4+c0/0HM9bPFg8Mvd8WJnmDz0MvfXHhb0Unor5uDrN
31ZpPq8prAVowot8H+IrxGPFy/i+xEPE43L3sr8X7v3paellfyzcW9LT0mv+
VLiXpedE+/EAwzP37C8xV8RDWd/il5kfhXv58bl74Z8Lax09MV4Xz4t36R3c
G/9dOEugR0a7fy2sdWg4WvlLYS1HM8kq/ijc65JZ0Jv/leoRPTq98J+Fe3V6
4ueZ34W93Sg0gnohHigeIV5CfRAPEl/FfBWvF18svjK3920WraUdUv0/Ptrr
43nR2hOivT6aS5bQMLqXoqciSzgi2uvj+ekNmkZ7ZXoEegd6CLT/qGCtbqSj
aWbNpnZSQ5/NXZPR5lo6Dsms0Xj3Y6O9PB4eb98k2pvi8fGqjaO1H89KtnJM
dC9FxkIvRk+GNy2CvctR0b06HgZtrx3tBdB4eqt1hb0j3o4s6J/C2QKZENq8
Vtw/s0aTXXwi7pI5w3gy2rMwP48Pzib+LdwLk1HQG28u3MvSI+M960d7RTwo
WU6D6GyITAcvVifaa+HJ8Jb1or0DHhMvUZfPlNlT0EvSU/L9HRKcTZBR8Dz2
Cs7Gvi2czZCRTWX+B/eCO3L3xvTI9L776/rxhdcsa5U5dELhOcdcYw3TW6Dp
aDk+AL0mA0NPbxSfXLhGUBvwn/gJelzq00SNtyi8hlm71JBjC89Z5ipz+KzC
a461xho8rfAaZ22z5s8ovEZZm6x5Pl+r9P21Dc6iWqf53i55OTwdz3u35AXx
hDzPXdP8QGx5/nsHzycyO+Zz5eDnjYdjvpUP/j7w/MzHA5N+04Mxnzan75ce
g/lbIfj7JbNjPVUJns/0rHjFw4PnOz0r3viI4O8fT8562yd4vTGfmQ8heD2S
2bB+qwavB3o4vGvNtN7wyP952+DssG2qBx2SXyHzwl/dG1x/qyZ9OEbcuHBN
oZbwDKmXHVO9Py+4/nVK9ff84PpKz8F33SW4vnVO9fiC4PpFpkd22TO4HnZJ
9bVrcP0jsyXr7BGsX/1TvRyY6mnvVL/7B9dHMkyyzl7B67VPqucXpfXXN9Xn
AcHrr1+qtxcH60PXtB4uDPaLg1P9vSzYL45J/nFccBbbJtXX9sFZa7ukZx2D
6wcZDVlEC9Zd4ZpLrWVOUj/qJ307JdjPTkh+7oZgPzo01ZsRwXo+LNWPq4L9
5+jkb8cmvzso6dulwX5geKovI5N/vSTpxeXB/nFI0qMrg+tXtaTvDYL9KpkW
/vqe4P6ADBS/e1uwfyXDxM8+Euzr6S1/yOzv8TnoJvqJ38HnL09ejBoxQq+t
Lo2YhXdjzogriq/NndePFNcQz86d3/LdVCq5VvEdkfcfUnKePJYaKm5K/4gX
yDw32A9A25kj48SHa/wxvChzTnyweFLuPHqsuKb4UfH0zM+W/p7egWdMv39Y
yXkyeSPPjv4ZLecZklWSWR6q87L6uRBvpPs/zZ2vkm2ScVbT+R7B2SQ9JL3j
fsFZKD1kVfHu+GnWnl6/Du+ia/PEp4vX4h2Yk+Kq4vvw/pn7+UNL3v8gf2cu
kH/jdZgT5OFVND4ld54/QFxZPFl8A3NCXE08Lff+yUxqG3lJ7nx7Bt6M/EO8
FA3Dy+Fn8X6Z5wJ5Mr0Nc4L9klol5wsPZJ7b7J/QqzDHmUv0s/ROzKmbxEfp
/idz5yk3iGuLF4kfytz/1ik5byH/vFvcgPwHLc6cxx9T8v4M+ze3i+vjv8Xz
mLPieuLn8L7iW8V1xUvwuuI7xEeLl+be32Huk+/TW7IGJoqP1PhitDFz9ksG
XEnnu+jnNWilxh/PnafcJ25MfpW7Xydvb1byfsFL4qnUcvFyvJj4fvGx5FV4
ucx5eZOS+332G+gNyc/pzekRTy25Z6BXeE/Xb4rODz7BS+raY2hVyXk7ecdc
8cniD8RvsGbFJ4nfF78ufljcXLyadSy+MTovWyV+VfwoXkL8Ye78/l60kf4K
r5RZXzeles2eD3sz7NHgr/Lg3p8MgKyuZXC/+nTh/pCMmlrZULwlc81srfsP
iM7eyODIE54s3N+RcdMPP1W4/yPTPlB8bsF/5gyO7IAMgWyXzKy8xs8R78j8
nvRXi8Wl3Hso1aP9PmPcQ/5MJkNWsjZzltgh+r3J+Wro/LfCtS3Xe9wn/lnX
y4kJPehf3yjc37KHwHx5izmUec8AP/pq4WwNP0eeSybN78szoh9+LdUHMib6
02cK+zMyb/bTyCTpHam5laIzTrKY3zJnlR2j6y29Jv3364X9HHs09NNvFu73
2cNBz75Leswe4xzxdr2uunjP4NpAjaB2Dg2u3dRw1uao4NpDDWLuDwuu3dRw
1sro4FpATWBtXRdci6nJrMUJwbWdGk9t6h5ca6m51AL2eFlbrDHW8rXBWSeZ
Zy9xs+BsmYy5p/iE4GybjJts+sTgLJ6MlWz1pOBsEX9DNn90cNZH5sdeRKPg
LJBMkL2GhsFZJZklewFNgmsTNYrac1Nwv8OeCNlr3eB+ij0FsvL6wdk5GTrZ
fvPgLJBMsDvPPzgrJDMk224c3G+RcZLN1wvWRjSSWt0tWE/eKZy/sGdGvvFs
4fnHng77MevxFZn3DPcULyucRWzMnH2wJ8VeFBkIe1OHRWcX5KZoDxr0qXhR
cFZIZrhBvDhYG9AIssM5wdkEGcXn4qeCtQ8N/Ez8ZLB2oWHrxQuDayE1EW18
KLj2UIM+pNYE1yZq1EfUMvxIcJ6yNNVc1hpr7ujcc5S5ypw9KXdmQe9FDzY4
9/4V3gCPgJbODs42yDha584o6H3ogern1nBqJzX0E53PC6591MCPxU8E11Jq
6hpqRXDWQebRPXfPh56/XTg/Y08SrUQz0e5ZwdqERqGdk4PzuqWF8y72QMh3
Xiz8WdmTIY96oXD+xZ4J+dorhX9X9hDIs5YU7m/pv8mnniucp1VN/d3yws+K
z0h+9nJhP8KeBPnW84XzOfZsyNNeKuxHyLSppfum+v3ffln0nhu1mpqNFqAJ
aMd9wVqOpqNtNwdrJxqKVkwK1mo0e5n4rmBtR+PRyluCtRrNxmvcGazNaDTe
bkqwtqKxaNn0YC+AJ0CrpgZrKxqLtk0L1lI0Fe26P9jL4GnQypnB3giPhNcc
nPTqi8L9C94Wr4RnwqsOSuu/nz7/nMweBS+Bp0C770jf76Do+oYH4fP3j66X
eAg+38Bo74En4XldFF0v8UB42/OivSkel/pxZnQ9YA+Oetk52ivjifGyeFq8
YJ/0PAdE1088E8/7kuj6i0fieQ6Orqd4HvZ6z42uR+z5Uu+7Rb8fnpD1elX0
94WHod53j/aK09PzOV98fWZPyfO8ILpe4TnxqnhWvPWQVP97RzOenPpDPSI/
+j13vT4run6yh8r3fUX0fMJToQd9ovUIz4W+9I3WHzwn+nSh+K7MPQZ6cXr0
fgZ7ZOjFadF5FHuS7GW2idYT9jTJjtmDJE97JdUealDZzJkue7Vto/WFPVv2
ZttH6wt7eMy/y6PnOx6Pvd7W0frEni97ta2i9YQ9W/aaz4mu/+w5M58vxfNk
9risr8ui9QbPhb5XSPOjk36fJTq+QEcyZ9FkyXgYvMv8pDctovME9qDQs1Oi
8wb24NiLbxmtd+zJs1fHnl0Pamdwds0eHXknvR177az/8zLvybM3e3a0frJH
y951u2i9Yw8bPT4jWh/Zc8bfsafN3tbrSd86afzKzD0gfov8hvndJrg3o0dj
vYwP3vulHvH7saeOX+gZvX7npPVDj8j87Jv0aHy0XtGToVcTovWHTB29Gxdd
n+m5qO+jo+sHnhl9GhldX/DI9HvvFtYa/kaFejIsev3TI9HfvVd4v4S/QcFL
48+WZ/bU6OHY6PpDT4e+jImuZ3hs/MmJ0X+LQD5PXvN94XyDv/kib/ihcL7M
33iR160ovJ/D37TgX5rrWsvMe5r4i5Oj8yD2JNG3q6PrIT0C/cTE6F6BPVJ6
Cfwvf3tBT0FvQo8CP5j80knReRJ7pPiDa6L1lT1F6u2Q6Hr+WtK/4dH1nJ4R
Pb4yWv/o2dH7UdH6QU+CvxgRXe/pQfGTPaLrL5nA/wE02Pfq
              "]], Polygon3DBox[CompressedData["
1:eJwt13f8jWUbAPDzO+d3FKVhZIVklYSIzLSUESGjjIxQisiMSEZGNChNpayW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              "]], 
             Polygon3DBox[{{821, 987, 581, 702, 1158, 820}, {835, 993, 588, 
              706, 1160, 834}, {1045, 1111, 600, 397, 396, 
              709}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{275, 494, 1165, 276, 507, 302, 1264, 522, 317, 1269, 
            537, 332, 1276, 552, 347, 1286, 567, 362, 1297, 582, 1208, 377, 
            595, 1220, 391, 1309, 608, 406, 1313, 623, 421, 1318, 638, 436, 
            1325, 653, 451, 1334, 668, 466, 1345, 683, 481}], 
           Line3DBox[{277, 495, 1166, 278, 508, 1176, 303, 523, 318, 1270, 
            538, 333, 1277, 553, 348, 1287, 568, 363, 1298, 583, 1209, 378, 
            596, 1221, 392, 609, 1230, 407, 624, 422, 1319, 639, 437, 1326, 
            654, 452, 1335, 669, 467, 1346, 684, 482}], 
           Line3DBox[{279, 496, 1167, 280, 509, 1177, 304, 524, 1187, 319, 
            539, 334, 1278, 554, 349, 1288, 569, 364, 1299, 584, 1210, 379, 
            597, 1222, 393, 610, 1231, 408, 625, 1240, 423, 640, 438, 1327, 
            655, 453, 1336, 670, 468, 1347, 685, 483}], 
           Line3DBox[{281, 497, 1168, 282, 510, 1178, 305, 525, 1188, 320, 
            540, 1195, 335, 555, 350, 1289, 570, 365, 1300, 585, 1211, 380, 
            598, 1223, 394, 611, 1232, 409, 626, 1241, 424, 641, 1248, 439, 
            656, 454, 1337, 671, 469, 1348, 686, 484}], 
           Line3DBox[{283, 498, 1169, 284, 511, 1179, 306, 526, 1189, 321, 
            541, 1196, 336, 556, 1201, 351, 571, 366, 1301, 586, 1212, 381, 
            599, 1224, 395, 612, 1233, 410, 627, 1242, 425, 642, 1249, 440, 
            657, 1254, 455, 672, 470, 1349, 687, 485}], 
           Line3DBox[{285, 696, 698, 286, 512, 1180, 307, 527, 1190, 322, 542,
             1197, 337, 557, 1202, 352, 572, 1205, 367, 703, 705, 1213, 729, 
            707, 1360, 709, 396, 613, 1234, 411, 628, 1243, 426, 643, 1250, 
            441, 658, 1255, 456, 673, 1258, 471, 1350, 718, 720, 730}], 
           Line3DBox[{289, 501, 1171, 290, 515, 1181, 310, 1267, 530, 325, 
            1273, 545, 340, 1281, 560, 355, 1292, 575, 370, 1303, 589, 1214, 
            384, 602, 1225, 399, 616, 414, 1316, 631, 429, 1322, 646, 444, 
            1330, 661, 459, 1340, 676, 474, 1352, 690, 488}], 
           Line3DBox[{291, 502, 1172, 292, 516, 1182, 311, 531, 326, 1274, 
            546, 341, 1282, 561, 356, 1293, 576, 371, 1304, 590, 1215, 385, 
            603, 1226, 400, 617, 1235, 415, 632, 430, 1323, 647, 445, 1331, 
            662, 460, 1341, 677, 475, 1353, 691, 489}], 
           Line3DBox[{293, 503, 1173, 294, 517, 1183, 312, 532, 1191, 327, 
            547, 342, 1283, 562, 357, 1294, 577, 372, 1305, 591, 1216, 386, 
            604, 1227, 401, 618, 1236, 416, 633, 1244, 431, 648, 446, 1332, 
            663, 461, 1342, 678, 476, 1354, 692, 490}], 
           Line3DBox[{295, 504, 1174, 296, 518, 1184, 313, 533, 1192, 328, 
            548, 1198, 343, 1284, 563, 358, 1295, 578, 373, 1306, 592, 1217, 
            387, 605, 1228, 402, 619, 1237, 417, 634, 1245, 432, 649, 1251, 
            447, 664, 462, 1343, 679, 477, 1355, 693, 491}], 
           Line3DBox[{297, 505, 1175, 298, 519, 1185, 314, 534, 1193, 329, 
            549, 1199, 344, 564, 1203, 359, 579, 374, 1307, 593, 1218, 388, 
            606, 1229, 403, 620, 1238, 418, 635, 1246, 433, 650, 1252, 448, 
            665, 1256, 463, 680, 478, 1356, 694, 492}], 
           Line3DBox[{299, 700, 1358, 701, 300, 520, 1186, 315, 535, 1194, 
            330, 550, 1200, 345, 565, 1204, 360, 580, 1206, 375, 711, 1361, 
            712, 1219, 389, 715, 1363, 716, 404, 621, 1239, 419, 636, 1247, 
            434, 651, 1253, 449, 666, 1257, 464, 681, 1259, 479, 722, 723, 
            724}], Line3DBox[{480, 682, 717, 1344, 465, 667, 1333, 450, 652, 
            1324, 435, 637, 1317, 420, 622, 1312, 405, 607, 1308, 390, 594, 
            1362, 713, 376, 1207, 581, 702, 1296, 361, 566, 1285, 346, 551, 
            1275, 331, 536, 1268, 316, 521, 1263, 301, 506, 1260, 274, 493, 
            695, 725}], 
           Line3DBox[{486, 688, 1364, 719, 472, 674, 1338, 457, 659, 1328, 
            442, 644, 1320, 427, 629, 1314, 412, 614, 1310, 397, 600, 1359, 
            714, 708, 382, 587, 728, 1365, 704, 368, 573, 1290, 353, 558, 
            1279, 338, 543, 1271, 323, 528, 1265, 308, 513, 1261, 287, 499, 
            1357, 697, 726}], 
           Line3DBox[{487, 689, 721, 1351, 473, 675, 1339, 458, 660, 1329, 
            443, 645, 1321, 428, 630, 1315, 413, 615, 1311, 398, 601, 710, 
            383, 1366, 588, 706, 1302, 369, 574, 1291, 354, 559, 1280, 339, 
            544, 1272, 324, 529, 1266, 309, 514, 1262, 288, 1170, 500, 699, 
            727}]}, {
           Line3DBox[{732, 1036, 493, 731, 1165, 949, 733, 1166, 950, 734, 
            1167, 951, 735, 1168, 952, 736, 1169, 953, 737, 698, 1063, 1357, 
            738, 1037, 1170, 954, 739, 1171, 955, 740, 1172, 956, 741, 1173, 
            957, 742, 1174, 958, 743, 1175, 959, 744, 1358, 1038, 960, 1039}],
            Line3DBox[{746, 1064, 1260, 745, 507, 747, 1176, 961, 748, 1177, 
            962, 749, 1178, 963, 750, 1179, 964, 751, 1180, 1065, 1261, 752, 
            1066, 1262, 753, 1181, 965, 754, 1182, 966, 755, 1183, 967, 756, 
            1184, 968, 757, 1185, 969, 758, 1186, 970, 759}], 
           Line3DBox[{761, 1067, 1263, 760, 1068, 1264, 762, 523, 763, 1187, 
            971, 764, 1188, 972, 765, 1189, 973, 766, 1190, 1069, 1265, 767, 
            1070, 1266, 768, 1071, 1267, 769, 531, 770, 1191, 974, 771, 1192, 
            975, 772, 1193, 976, 773, 1194, 977, 774}], 
           Line3DBox[{776, 1072, 1268, 775, 1073, 1269, 777, 1074, 1270, 778, 
            539, 779, 1195, 978, 780, 1196, 979, 781, 1197, 1075, 1271, 782, 
            1076, 1272, 783, 1077, 1273, 784, 1078, 1274, 785, 547, 786, 1198,
             980, 787, 1199, 981, 788, 1200, 982, 789}], 
           Line3DBox[{791, 1079, 1275, 790, 1080, 1276, 792, 1081, 1277, 793, 
            1082, 1278, 794, 555, 795, 1201, 983, 796, 1202, 1083, 1279, 797, 
            1084, 1280, 798, 1085, 1281, 799, 1086, 1282, 800, 1087, 1283, 
            801, 1088, 1284, 802, 1203, 984, 803, 1204, 985, 804}], 
           Line3DBox[{806, 1089, 1285, 805, 1090, 1286, 807, 1091, 1287, 808, 
            1092, 1288, 809, 1093, 1289, 810, 571, 811, 1205, 1094, 1290, 812,
             1095, 1291, 813, 1096, 1292, 814, 1097, 1293, 815, 1098, 1294, 
            816, 1099, 1295, 817, 579, 818, 1206, 986, 819}], 
           Line3DBox[{822, 1041, 1207, 987, 821, 1208, 988, 824, 1209, 989, 
            826, 1210, 990, 828, 1211, 991, 830, 1212, 992, 832, 1159, 1213, 
            1059, 587, 1058, 1044, 1060, 1366, 993, 835, 1214, 994, 837, 1215,
             995, 839, 1216, 996, 841, 1217, 997, 843, 1218, 998, 845, 1161, 
            1219, 999, 1061}], 
           Line3DBox[{846, 1048, 1047, 1361, 844, 1307, 1109, 842, 1306, 1108,
             840, 1305, 1107, 838, 1304, 1106, 836, 1303, 1105, 834, 1160, 
            1302, 1043, 833, 1365, 1042, 703, 831, 1301, 1104, 829, 1300, 
            1103, 827, 1299, 1102, 825, 1298, 1101, 823, 1297, 1100, 820, 
            1158, 1296, 1040, 1057}], 
           Line3DBox[{848, 1049, 1110, 1362, 847, 1220, 1000, 849, 1221, 1001,
             850, 1222, 1002, 851, 1223, 1003, 852, 1224, 1004, 853, 1360, 
            1045, 1111, 1359, 1050, 1046, 601, 854, 1225, 1005, 855, 1226, 
            1006, 856, 1227, 1007, 857, 1228, 1008, 858, 1229, 1009, 859, 
            1363, 1051, 1010, 1062}], 
           Line3DBox[{861, 1112, 1308, 860, 1113, 1309, 862, 1230, 1011, 863, 
            1231, 1012, 864, 1232, 1013, 865, 1233, 1014, 866, 1234, 1114, 
            1310, 867, 1115, 1311, 868, 616, 869, 1235, 1015, 870, 1236, 1016,
             871, 1237, 1017, 872, 1238, 1018, 873, 1239, 1019, 874}], 
           Line3DBox[{876, 1116, 1312, 875, 1117, 1313, 877, 624, 878, 1240, 
            1020, 879, 1241, 1021, 880, 1242, 1022, 881, 1243, 1118, 1314, 
            882, 1119, 1315, 883, 1120, 1316, 884, 632, 885, 1244, 1023, 886, 
            1245, 1024, 887, 1246, 1025, 888, 1247, 1026, 889}], 
           Line3DBox[{891, 1121, 1317, 890, 1122, 1318, 892, 1123, 1319, 893, 
            640, 894, 1248, 1027, 895, 1249, 1028, 896, 1250, 1124, 1320, 897,
             1125, 1321, 898, 1126, 1322, 899, 1127, 1323, 900, 648, 901, 
            1251, 1029, 902, 1252, 1030, 903, 1253, 1031, 904}], 
           Line3DBox[{906, 1128, 1324, 905, 1129, 1325, 907, 1130, 1326, 908, 
            1131, 1327, 909, 656, 910, 1254, 1032, 911, 1255, 1132, 1328, 912,
             1133, 1329, 913, 1134, 1330, 914, 1135, 1331, 915, 1136, 1332, 
            916, 664, 917, 1256, 1033, 918, 1257, 1034, 919}], 
           Line3DBox[{921, 1137, 1333, 920, 1138, 1334, 922, 1139, 1335, 923, 
            1140, 1336, 924, 1141, 1337, 925, 672, 926, 1258, 1142, 1338, 927,
             1143, 1339, 928, 1144, 1340, 929, 1145, 1341, 930, 1146, 1342, 
            931, 1147, 1343, 932, 680, 933, 1259, 1035, 934}], 
           Line3DBox[{948, 1056, 722, 947, 1356, 1157, 946, 1355, 1156, 945, 
            1354, 1155, 944, 1353, 1154, 943, 1352, 1153, 942, 1164, 1351, 
            1055, 941, 1364, 1054, 1350, 1163, 940, 1349, 1152, 939, 1348, 
            1151, 938, 1347, 1150, 937, 1346, 1149, 936, 1345, 1148, 935, 
            1162, 1344, 1052, 1053}]}}},
        VertexNormals->CompressedData["
1:eJx1XGdYFUnTRTGAilnMWcyKWVS0TBjXuItZBFFUzDkrq5gT5oRZBCPmiJYo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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 
        0.9999998571428571}, {-0.9999998571428571, 
        0.9999998571428571}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {192., -576.}, ImageScaled[{0.5, 0.5}], {360., 360.},
      ContentSelectable->True], InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHu0V1W1x4nwATocDsQ0NJ+kIobWEBVD5yhvXnxAXSmzrlJD8VGJGpGC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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmnfgjlUbx+9zn/NkZiYKqewRSgrZOyLR0kZTivbQkEpaiNCwKjS1Rctq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              "]], Polygon3DBox[CompressedData["
1:eJwtmnfgV9Mfxu8993ysyE5SNJQoGrSQhMxIe6DSTiqrIQ2lrGSXkRINZDSt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              "]], Polygon3DBox[CompressedData["
1:eJwt13f8jWUbAPDzO+d3FKVhZIVklYSIzLSUESGjjIxQisiMSEZGNChNpayW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              "]], 
             Polygon3DBox[{{821, 987, 581, 702, 1158, 820}, {835, 993, 588, 
              706, 1160, 834}, {1045, 1111, 600, 397, 396, 
              709}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{275, 494, 1165, 276, 507, 302, 1264, 522, 317, 1269, 
            537, 332, 1276, 552, 347, 1286, 567, 362, 1297, 582, 1208, 377, 
            595, 1220, 391, 1309, 608, 406, 1313, 623, 421, 1318, 638, 436, 
            1325, 653, 451, 1334, 668, 466, 1345, 683, 481}], 
           Line3DBox[{277, 495, 1166, 278, 508, 1176, 303, 523, 318, 1270, 
            538, 333, 1277, 553, 348, 1287, 568, 363, 1298, 583, 1209, 378, 
            596, 1221, 392, 609, 1230, 407, 624, 422, 1319, 639, 437, 1326, 
            654, 452, 1335, 669, 467, 1346, 684, 482}], 
           Line3DBox[{279, 496, 1167, 280, 509, 1177, 304, 524, 1187, 319, 
            539, 334, 1278, 554, 349, 1288, 569, 364, 1299, 584, 1210, 379, 
            597, 1222, 393, 610, 1231, 408, 625, 1240, 423, 640, 438, 1327, 
            655, 453, 1336, 670, 468, 1347, 685, 483}], 
           Line3DBox[{281, 497, 1168, 282, 510, 1178, 305, 525, 1188, 320, 
            540, 1195, 335, 555, 350, 1289, 570, 365, 1300, 585, 1211, 380, 
            598, 1223, 394, 611, 1232, 409, 626, 1241, 424, 641, 1248, 439, 
            656, 454, 1337, 671, 469, 1348, 686, 484}], 
           Line3DBox[{283, 498, 1169, 284, 511, 1179, 306, 526, 1189, 321, 
            541, 1196, 336, 556, 1201, 351, 571, 366, 1301, 586, 1212, 381, 
            599, 1224, 395, 612, 1233, 410, 627, 1242, 425, 642, 1249, 440, 
            657, 1254, 455, 672, 470, 1349, 687, 485}], 
           Line3DBox[{285, 696, 698, 286, 512, 1180, 307, 527, 1190, 322, 542,
             1197, 337, 557, 1202, 352, 572, 1205, 367, 703, 705, 1213, 729, 
            707, 1360, 709, 396, 613, 1234, 411, 628, 1243, 426, 643, 1250, 
            441, 658, 1255, 456, 673, 1258, 471, 1350, 718, 720, 730}], 
           Line3DBox[{289, 501, 1171, 290, 515, 1181, 310, 1267, 530, 325, 
            1273, 545, 340, 1281, 560, 355, 1292, 575, 370, 1303, 589, 1214, 
            384, 602, 1225, 399, 616, 414, 1316, 631, 429, 1322, 646, 444, 
            1330, 661, 459, 1340, 676, 474, 1352, 690, 488}], 
           Line3DBox[{291, 502, 1172, 292, 516, 1182, 311, 531, 326, 1274, 
            546, 341, 1282, 561, 356, 1293, 576, 371, 1304, 590, 1215, 385, 
            603, 1226, 400, 617, 1235, 415, 632, 430, 1323, 647, 445, 1331, 
            662, 460, 1341, 677, 475, 1353, 691, 489}], 
           Line3DBox[{293, 503, 1173, 294, 517, 1183, 312, 532, 1191, 327, 
            547, 342, 1283, 562, 357, 1294, 577, 372, 1305, 591, 1216, 386, 
            604, 1227, 401, 618, 1236, 416, 633, 1244, 431, 648, 446, 1332, 
            663, 461, 1342, 678, 476, 1354, 692, 490}], 
           Line3DBox[{295, 504, 1174, 296, 518, 1184, 313, 533, 1192, 328, 
            548, 1198, 343, 1284, 563, 358, 1295, 578, 373, 1306, 592, 1217, 
            387, 605, 1228, 402, 619, 1237, 417, 634, 1245, 432, 649, 1251, 
            447, 664, 462, 1343, 679, 477, 1355, 693, 491}], 
           Line3DBox[{297, 505, 1175, 298, 519, 1185, 314, 534, 1193, 329, 
            549, 1199, 344, 564, 1203, 359, 579, 374, 1307, 593, 1218, 388, 
            606, 1229, 403, 620, 1238, 418, 635, 1246, 433, 650, 1252, 448, 
            665, 1256, 463, 680, 478, 1356, 694, 492}], 
           Line3DBox[{299, 700, 1358, 701, 300, 520, 1186, 315, 535, 1194, 
            330, 550, 1200, 345, 565, 1204, 360, 580, 1206, 375, 711, 1361, 
            712, 1219, 389, 715, 1363, 716, 404, 621, 1239, 419, 636, 1247, 
            434, 651, 1253, 449, 666, 1257, 464, 681, 1259, 479, 722, 723, 
            724}], Line3DBox[{480, 682, 717, 1344, 465, 667, 1333, 450, 652, 
            1324, 435, 637, 1317, 420, 622, 1312, 405, 607, 1308, 390, 594, 
            1362, 713, 376, 1207, 581, 702, 1296, 361, 566, 1285, 346, 551, 
            1275, 331, 536, 1268, 316, 521, 1263, 301, 506, 1260, 274, 493, 
            695, 725}], 
           Line3DBox[{486, 688, 1364, 719, 472, 674, 1338, 457, 659, 1328, 
            442, 644, 1320, 427, 629, 1314, 412, 614, 1310, 397, 600, 1359, 
            714, 708, 382, 587, 728, 1365, 704, 368, 573, 1290, 353, 558, 
            1279, 338, 543, 1271, 323, 528, 1265, 308, 513, 1261, 287, 499, 
            1357, 697, 726}], 
           Line3DBox[{487, 689, 721, 1351, 473, 675, 1339, 458, 660, 1329, 
            443, 645, 1321, 428, 630, 1315, 413, 615, 1311, 398, 601, 710, 
            383, 1366, 588, 706, 1302, 369, 574, 1291, 354, 559, 1280, 339, 
            544, 1272, 324, 529, 1266, 309, 514, 1262, 288, 1170, 500, 699, 
            727}]}, {
           Line3DBox[{732, 1036, 493, 731, 1165, 949, 733, 1166, 950, 734, 
            1167, 951, 735, 1168, 952, 736, 1169, 953, 737, 698, 1063, 1357, 
            738, 1037, 1170, 954, 739, 1171, 955, 740, 1172, 956, 741, 1173, 
            957, 742, 1174, 958, 743, 1175, 959, 744, 1358, 1038, 960, 1039}],
            Line3DBox[{746, 1064, 1260, 745, 507, 747, 1176, 961, 748, 1177, 
            962, 749, 1178, 963, 750, 1179, 964, 751, 1180, 1065, 1261, 752, 
            1066, 1262, 753, 1181, 965, 754, 1182, 966, 755, 1183, 967, 756, 
            1184, 968, 757, 1185, 969, 758, 1186, 970, 759}], 
           Line3DBox[{761, 1067, 1263, 760, 1068, 1264, 762, 523, 763, 1187, 
            971, 764, 1188, 972, 765, 1189, 973, 766, 1190, 1069, 1265, 767, 
            1070, 1266, 768, 1071, 1267, 769, 531, 770, 1191, 974, 771, 1192, 
            975, 772, 1193, 976, 773, 1194, 977, 774}], 
           Line3DBox[{776, 1072, 1268, 775, 1073, 1269, 777, 1074, 1270, 778, 
            539, 779, 1195, 978, 780, 1196, 979, 781, 1197, 1075, 1271, 782, 
            1076, 1272, 783, 1077, 1273, 784, 1078, 1274, 785, 547, 786, 1198,
             980, 787, 1199, 981, 788, 1200, 982, 789}], 
           Line3DBox[{791, 1079, 1275, 790, 1080, 1276, 792, 1081, 1277, 793, 
            1082, 1278, 794, 555, 795, 1201, 983, 796, 1202, 1083, 1279, 797, 
            1084, 1280, 798, 1085, 1281, 799, 1086, 1282, 800, 1087, 1283, 
            801, 1088, 1284, 802, 1203, 984, 803, 1204, 985, 804}], 
           Line3DBox[{806, 1089, 1285, 805, 1090, 1286, 807, 1091, 1287, 808, 
            1092, 1288, 809, 1093, 1289, 810, 571, 811, 1205, 1094, 1290, 812,
             1095, 1291, 813, 1096, 1292, 814, 1097, 1293, 815, 1098, 1294, 
            816, 1099, 1295, 817, 579, 818, 1206, 986, 819}], 
           Line3DBox[{822, 1041, 1207, 987, 821, 1208, 988, 824, 1209, 989, 
            826, 1210, 990, 828, 1211, 991, 830, 1212, 992, 832, 1159, 1213, 
            1059, 587, 1058, 1044, 1060, 1366, 993, 835, 1214, 994, 837, 1215,
             995, 839, 1216, 996, 841, 1217, 997, 843, 1218, 998, 845, 1161, 
            1219, 999, 1061}], 
           Line3DBox[{846, 1048, 1047, 1361, 844, 1307, 1109, 842, 1306, 1108,
             840, 1305, 1107, 838, 1304, 1106, 836, 1303, 1105, 834, 1160, 
            1302, 1043, 833, 1365, 1042, 703, 831, 1301, 1104, 829, 1300, 
            1103, 827, 1299, 1102, 825, 1298, 1101, 823, 1297, 1100, 820, 
            1158, 1296, 1040, 1057}], 
           Line3DBox[{848, 1049, 1110, 1362, 847, 1220, 1000, 849, 1221, 1001,
             850, 1222, 1002, 851, 1223, 1003, 852, 1224, 1004, 853, 1360, 
            1045, 1111, 1359, 1050, 1046, 601, 854, 1225, 1005, 855, 1226, 
            1006, 856, 1227, 1007, 857, 1228, 1008, 858, 1229, 1009, 859, 
            1363, 1051, 1010, 1062}], 
           Line3DBox[{861, 1112, 1308, 860, 1113, 1309, 862, 1230, 1011, 863, 
            1231, 1012, 864, 1232, 1013, 865, 1233, 1014, 866, 1234, 1114, 
            1310, 867, 1115, 1311, 868, 616, 869, 1235, 1015, 870, 1236, 1016,
             871, 1237, 1017, 872, 1238, 1018, 873, 1239, 1019, 874}], 
           Line3DBox[{876, 1116, 1312, 875, 1117, 1313, 877, 624, 878, 1240, 
            1020, 879, 1241, 1021, 880, 1242, 1022, 881, 1243, 1118, 1314, 
            882, 1119, 1315, 883, 1120, 1316, 884, 632, 885, 1244, 1023, 886, 
            1245, 1024, 887, 1246, 1025, 888, 1247, 1026, 889}], 
           Line3DBox[{891, 1121, 1317, 890, 1122, 1318, 892, 1123, 1319, 893, 
            640, 894, 1248, 1027, 895, 1249, 1028, 896, 1250, 1124, 1320, 897,
             1125, 1321, 898, 1126, 1322, 899, 1127, 1323, 900, 648, 901, 
            1251, 1029, 902, 1252, 1030, 903, 1253, 1031, 904}], 
           Line3DBox[{906, 1128, 1324, 905, 1129, 1325, 907, 1130, 1326, 908, 
            1131, 1327, 909, 656, 910, 1254, 1032, 911, 1255, 1132, 1328, 912,
             1133, 1329, 913, 1134, 1330, 914, 1135, 1331, 915, 1136, 1332, 
            916, 664, 917, 1256, 1033, 918, 1257, 1034, 919}], 
           Line3DBox[{921, 1137, 1333, 920, 1138, 1334, 922, 1139, 1335, 923, 
            1140, 1336, 924, 1141, 1337, 925, 672, 926, 1258, 1142, 1338, 927,
             1143, 1339, 928, 1144, 1340, 929, 1145, 1341, 930, 1146, 1342, 
            931, 1147, 1343, 932, 680, 933, 1259, 1035, 934}], 
           Line3DBox[{948, 1056, 722, 947, 1356, 1157, 946, 1355, 1156, 945, 
            1354, 1155, 944, 1353, 1154, 943, 1352, 1153, 942, 1164, 1351, 
            1055, 941, 1364, 1054, 1350, 1163, 940, 1349, 1152, 939, 1348, 
            1151, 938, 1347, 1150, 937, 1346, 1149, 936, 1345, 1148, 935, 
            1162, 1344, 1052, 1053}]}}},
        VertexNormals->CompressedData["
1:eJx1XHlcjdv3jlAhQ8g8y6zMQqxIxmu+maVEyJR5njNmyBwyazCWzOMSqUia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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 
        0.9999998571428571}, {-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {576., -576.}, ImageScaled[{0.5, 0.5}], {360., 360.},
      ContentSelectable->True]}}, {}},
  GraphicsSpacing -> {0.2, 0},
  ContentSelectable->True,
  DisplayFunction->Identity,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.580194637700885*^9, 
  3.580195018685676*^9},ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["Corrected version:", "Text",
 CellChangeTimes->{{3.5801961358055716`*^9, 3.5801961435330133`*^9}}],

Cell[BoxData[
 RowBox[{"   ", 
  RowBox[{
   RowBox[{
    RowBox[{"orth4d", "[", 
     RowBox[{"surf_", ",", 
      RowBox[{"{", 
       RowBox[{"u_", ",", "umin_", ",", "umax_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "vmin_", ",", "vmax_"}], "}"}], ",", "opts___"}], 
     "]"}], ":=", "\n", "        ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"g1", ",", "g2", ",", "g3", ",", "g4"}], "}"}], ",", "\n", 
      "        ", 
      RowBox[{
       RowBox[{"g1", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "1", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g2", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "2", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g3", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "3", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"g4", "=", 
        RowBox[{"ParametricPlot3D", "[", 
         RowBox[{
          RowBox[{"Evaluate", "[", "\n", "                ", 
           RowBox[{"Drop", "[", 
            RowBox[{"surf", ",", 
             RowBox[{"{", "4", "}"}]}], "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "        ", 
       RowBox[{"Show", "[", 
        RowBox[{
         RowBox[{"GraphicsGrid", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"g1", ",", "g2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"g3", ",", "g4"}], "}"}]}], "}"}], ",", "\n", 
           "                ", 
           RowBox[{"Spacings", "->", 
            RowBox[{"{", 
             RowBox[{"0.2", ",", "0"}], "}"}]}]}], "]"}], ",", "\n", 
         "                ", 
         RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
     "]"}]}], "\n"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.5801945032431946`*^9, 3.580194570256027*^9}, 
   3.580194955388056*^9, 3.580194998243507*^9, {3.5801953064621363`*^9, 
   3.5801953076782055`*^9}, {3.5801953387749844`*^9, 3.580195340643091*^9}, 
   3.580195440301791*^9, 3.5801955258916864`*^9, {3.580195576036555*^9, 
   3.5801955868251715`*^9}, {3.5801957189037266`*^9, 3.580195724600052*^9}, {
   3.580195780230234*^9, 3.580195781302295*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orth4dnew", "[", 
  RowBox[{
   RowBox[{"sf4D", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5801944365613804`*^9, 3.5801944945026946`*^9}, {
  3.580194589230113*^9, 3.5801946305114737`*^9}, {3.580195377592204*^9, 
  3.5801953853516483`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {{InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJy9fXucl+PWd5JD5YmHiNEmhA6EiCKt7WFTVPaWbcemTUS82uzYOpC841Cb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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFm3kcFtPbxp+ZOTNT1uxblsqWLaG0oFSWQpGKSChLCyK7hOwJKUUSytZi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              "]], Polygon3DBox[CompressedData["
1:eJwtm3fgV9Mfxu/9nM85F9n97JkysqnIDKFCIUVSSqmUjBRCKgotI6uydxnZ
W0mhyF5Z2SGRvefvefWcP873e597zh2fe895v5/3837fRr1O6nhirSiKn0NR
1PX/Rf3ZuSqKTtq5lXCl/SO17++yKFoIn5rUJ9xZbZ72Pa//zTX+UI3fQv0/
afyPanurbx+1yRr/qvpP1bhvhV/Q/52ED9P4LTV+vnAL4Y7CTYW/F/5K+Ab9
31j7Zur/1sKttL2O+o/R+aYkn+s/4Y+1/z6N2Ur/3xV+TtvNNP4Q4c2Fz9TY
J4WP1P4Xdcwz+v+r+rZXW139c4T7qn9b4dWEB8ai6CP8nvBrwv/TbzlXY/7T
sfsK/6n9w4X/EN5M+ED1ny9c1/7rhB/X9pY6vqXwGsJX6/oLhE/X/u91zFnC
7wh3E36p9HM/X+1TjW0t3Efb49S3go6foX2/aLu1xg8X7iLcRf1jtK8Sni7c
R/e7VP33aF9T7buw7muvqb41GVN335ba3kXnP17jh+oc7wq/qjEzeHY6fg/h
tYV3FT5a7TV+n9pjnFf9O6n/f+q/WHii2trCXwkfDtb5lvLsNf6S3L9WvocF
/Bb19xDeWfgUbU/QvpWEXxDeXds91F7XsTuoXafnc73a49peS/17qu8yHfOd
tm/SvqPqfnavaPtQtUf1ew7T/b2hMY2F++u8k9W/vv631jGHaPtQtTnqa8vv
0fhOGv+WxjcR/pZ5pu3TNP4QjX9deBfmi/AOwq/xbtXfhTkj/I7w7uo/hucp
vJNwO7VHdK791Fpqu7vaq9peQ+09be+p8b01fjeNf1PXv1P4au3fUPv+0bnb
aPs+jf1X/R9pe3Xt66e+vYTfFd5D43sJ7yo8Qn1tte9Bzs3813Yr9R+n/r2F
3xDeVbir8I7CbwvvJtxTuCW/T//v0r4t9H++8Fu6n2l63qvW/A6vUN+VausK
byr8Oe9a18RItBHeW9tnat+Puv7Fwifk9/WL8OfCL+t8i2teT6syfzX+LI3/
Wf2LhI9Q/ze6nzu0bzON+Uv9N6j9qb6DNeYGnesj9Q9jnQqPU9/Y4HN/oXaV
+pvxvtT/nfqPFh6WvJZ4fg/rnDepbxN+p/Aj2t5c59sxr/dH+e3CzYUbCt/P
s1TbRngPneMh7quyfcAevMR6Eu4svLXwl8J7C58s3E54sfA+woOED8BeMHeF
b9X+Jtr3hf7vJXySttuqfx3136nWkPWg6x2k/vZqM7XdBvuk/VOFN9X/JzT+
8vw+1hP+Gvuj336N2u/a7qDx7+p53q7zrVbzGllD23eorS7cQv2/6th9df0R
wkdiP4V7qb2lvtNp2n+t8Eb631H9c7W9o8YfJNxY+BXhfsJthTcWflZ4B+ED
hTdhPuha7YVn654a6ny7CLcVvlu4Et5d+EDhx4VXEl6b56F2c35fbdR/sPqf
U/+62neAfs8TwouFr1f/PurvIPy08P/Uv6twO+H7hZcX3k34AOGHhRsIb6/j
W2PfsNs6/nj8SfCz/hgbrvH91KZhX9T20vZBOn6WxqzO/er4x4S/EL5Exz+D
L9M5blVfodZU/fer/2XsNvZSuKvwND2Pa4Q7Cx+rvqeFHxOuC9+j/ke17zjh
5rpeG+HbhP9hvar/5mTb9A3zA3ukdn72b+3Ud6jGv6jx62PzhD8QHqwx5wnv
rOPbqO9WXW+KxrcU7sL5ha8SLjS+ldrx+B7BoP4HNP5BbIxwW/UdovHzhdfT
mFrd97qitrvyPjX+SPVP1fmu5nzqv1f9y6nvCLUddfx+6r9O+/5Q99/6f4va
39he9Zd1X2sFbXdRa6bx+2v8Tdr3l8b8p/93qSX1Hc5v1vX21JgB2j4z+/Pb
Nf6iYP/0N3MnX6On8O/Cd6v/CuH2wgfp+G7afkL3e6/w/sKH83uFbxdeT+Mb
qV3Avam1V39PHT8DHqH+dVn/auPy+94AW6I2Js+XFrq3zmrjc38/9X0ZvNbe
V9ta26O1r9T5emGzdP4dNL67+gYKf6a+u3S9SzVuJ+Ej1fduZZ8xSWMaaf8o
iJiOP0z/NmQ9ql2ivs1pGt81eSzX/059iyuv4UO16yhdb4DOMV/HPwWn0Ngu
ahdp7Jq8M22/Bt+pe18HjT9GeKbG348NFu4l/KTwA6xv4aOF7xW+RfgTfm9l
H7uPcHfmO/NT/XPwJ9p/h/Alwfyrm/p7C7+g/tnCP6h/uvDl6t9PeC/1d4e/
qP9m/Knu71i12/L7WR/+p3Zhvv82Gt9D4x/S+DvgB8JHCU8XvlF4Cx17uNqE
fPwe6u/AHFP/Dfhvbe+j803N52vFfMFe4TfUP1bHviI8QGM+4vraPgI+qv7L
mD/aPlzteuFLhZdq3CDhvtlfLxE+SfhY4T2Ft9b5HxR+NXj9faP+k4X7qL9V
9t/4+yOzvy4qr6eg/ScU5ienVOYD+PfvMl/B3++T+cLAyvwEvnKr7v+Typy0
pn1rVV7fqwqPhhNWtrerCI8S/lD/j+c3Zn7zuu73i8p+B38zT/8/Y/3hz4QX
6v8AnoHGbwefEf5c+DL9b5b5E3zpgOwfThA+UW0R61ntIt3fe5X3fSb8lP5v
x2/W+PU1fpbwtsKthdcT/lF4sPBA4f2Fn8RXC+8tvG5h/g6fb5f9E/x++8rX
byS8W822Y0lhTjlO1784+drw4e/V922w72+ldrn6Fur4U/Abpf03/Kdn5nsb
YrvU3hR+UXi2+qcJT9CYDYQH1X3sEh07DD6s812RfC6uAZ86sTIfho8t4j0L
H5358T96/ntofH+NPUN47cr2fzXsfWE+elzl38vv653XSy+NHyTcT8fPVP83
up+ZwhvBjdUS8w7/UpkvT9bx4/EHle1xnfsSjpX9U2QeC3fR+WYIL9H57sY+
VubPU3T8BKarrt1ReJLwOOHVK/v7FXT86cINK/vXBsLD4EeV+cKKwmcLr1bZ
fy8vfJpw3+yfT9bvGSncINmfX1uzP16jMh9YSeOHC6+IPcb/qH8iz0f3u6hy
nEJ8skplvrCc8KnMJ/W35Jnp/IOFL4V/Za67DpxP/ZtmHzCyNH8mfiG+JT49
uGbbhq+epf6tsn06t/TzPVjn3i3YZ4wipsn2e7S2R7B+df6dNaa38Cm8b7hY
ZU40vHS81L9y/Eo8vWJlPlVp/xDep46/T/gJXeN44W2C+cIIHdtb+CTWbfDc
/gJ/jr9VO6U0H9oS/6/jXwn2ryX+W22Q+s8pzH/h78Sk+lf8xHU1/oTM/2dG
28MP1b81PjGZrx2JPxN+KfjdH6ftDUv78hd4HzX79MaV+fHNGtOYmBJfrBZr
5qyra/yj6l+k8zTDRqtvvFoDxmITkudjX/W3JMZS31i15Wrm1BOJs4SvVt9J
3J+291C7jrHMUdax8BThY9T2gU/jG7S9VK05XFhtC21PVNtP2y3ULtP2dLVL
NbZZ3X3M0fHYQeGLS6/B7evmDHAF9uFn8WH4mouyfx9acyx7T+l4G59EP/dE
/AjnxLfj44mPPwv2/fym9ZPX92Dt21V4FeFHhD8R3k54JeGHhT8S3ka4o/Bz
le+T+2PuPY+9qnkOzgueW31K82/Wxnz1X1jzGtla+OnKvA++t5XwnMq8Db62
pf6fo7EHaftnDWuKXiR8oPAPwtvBk4U7C9fVttHxz1TmhfDB54PXZt/S/POF
YFvQrzTfZX1hu3hXcIDldPxDOn6Bxm2KjUA7wfcy17hGMj9nHjL/ngz2pUeX
5vfE68Skiwq/U7jZPI0fWzNH2074WeELhI/ineZ4k3nDPcwJto09S8cjTwXb
zh6l45/tNX6ujh+j8d2EN1H/GXAnfA3xAs9G42cIP1ZaLzmg7m00FXwdPq+z
xjYO5rPjK/MX+M2BwuOE78x85hBd73XhofC5wvOP+Yhtwf50yHxvm7rtD9wV
f7B53Rx2y7o5MVxpQp7/zG9sAfFF+8ynp2d+1Cbz6dtyPAC/Or+yfZ6Y+fbY
yvEI8Ucn4YuEH8j8u7fwlcJzcnx0rPCkyvHS48K9hK8Qni38KPFB3frhEP3v
JNxD/ZcJz1L/w8KHCV8ofL/wNPiu8AXYF+HJwvsJjxG+JcdHhwpPEL6POFv4
GOHLhZ8SfkR4vH7b/nAN/fYv4ZN1xwSsR55Xk7pjWGLXsaX1kKv5bdq+o7Td
al03xn4Re70IH645Bmub7RHcAA2nXbYvyxXWkNBfOqg9WTrmR195uzKPgb+g
FTB/0MbQDNAjO9Q8/rfS6x6ucGrp9Y9dWB3fWto+vB/M/QazttQ+COaGQ0rb
88Oy/8LOYd/O1Pb72j5L27vl39+Y9VjYX+HfVq7b1hKj/Bvcz7PpWNq3vAQ/
rNnHvB7MBQaW1uvwTS/DN2v2UXD3N4Wb1j0fWef4+hNLr/c3g7nDCaX1u245
3pmR45cL8H3a3rf0Gt9U53uqss6EvgRXeSSa+8BZTsd2RseOg4VPE349WatB
sxkm/H30ejxLeDjcA74sfLbwaG2/rfFjhK9gziZrXGhbaDjnCa+ttnnN8eHZ
cCP4s8YP076h+DpiFOEhwl9zf8nHogOdJbw0eu6fqX1nYjsra5VolqyVr5Ln
LmuGub4wee4z51lrS5LXEmuOtfMlNrTwGrpU+IvkWJmYmbWwOHltsSbOE35H
eKzwlTWv9feSuRhrfoC2P6ysCTL/z9D2v2g66j9V/SOTNTG0sAXqfzxZs0ar
RoOckczJ4Sto1A8k5wDQ/tGk4HLTk/VkOB1a9bnRuQA0a2Lf+5PHEgPfo+3L
o7kcGuagZM0VrfVpXeM+4U/x0TVrxMQ6HMNYYh5i23uT+4hxp2r7xuRYaBXt
+7SyvvauztWn7nsbH621c4+nCb8j3KhmPZxcwMhoLRxdfqb6BwtvXXPOAu3+
ieTfvkzDFx4RnTtAc8c2fp1sK7GRD6N/RXMnNE60ymHR2iaa5RLhh5L70EHR
Ps+O1jrRQJ9MjkmIRdA80UKHR2v7aKIThT9X/z3ES3Bd5mcy93+MdyD8WbIt
R1MhFpwrvG3hmBDu+mxy7A+HJVZ5Ojk2I2Yh1pyXHDsScxI7v5QcCxNDExu/
mBw7EyMTGz2frJUQIxFbv8D9F46x0SrmE2MX1iyWaQ3RWgSawzJtQvjpwhoF
uZ2XhF8vnOMh9/NUslaNZt1E27Mq66zoq/Dr0dp/WGlNblvh84Q7abum1ik5
XsDuYe+IxZ9LjqWJyTfT9uzKuhh62M/Y5sp6OjYGbXUO87ewxor9IedDrmez
YO4NB8deNAnWl0bVfC1yBluhcwq31/avhWPL+cmxJDEm2s7z0VoOGg/Xfi2Z
y3MPnP/V5Nj6hMw/XknWZoi58a2fJPtKfCy+82P0mcI+FG7yUTIXgaPg6z9N
9sX4fLjAB8m+F04Ad/kwWduFw6DdPB3NLeAYaI2zo7VUNEe01TnC1xbWWNFq
nonWitBsiP2eiNZiiAHRZmZEazVoNGhF88i5FNaMiF3JAV1YOIZFW3k5WbtB
YyE2JqdEbEuMjJb6VDTXgfOgFT4X7WvwOWhXz0ZrY2hYaGVzhW8trJkRKxEz
XV5YQ5os/E1ybD6vZr0OG4ztRX9EP2aNsbbQt8hbHlF37pH8Jbky4puRhXNm
6PkN6+Yab5TO3RHfoAWQw/u37nxj17r1xNWj9cad69Ynx+T4a2jpnOK4HE+d
UToHSb7pE/xS6RiK/Bs5K/gPOZcJOT5D2yGnN6Dm+32htP3rxjMVnlva/pJ7
6Fl37ogcBPnT27NvQK8irpur/5+Uju/gVoznt8Gprq8758i5sbHkZwdkW/uP
2lXq61j3tbD55Km5Prlr8tVrEDdr//Pa/rZwHIWW82XpeIp8zK/at3LNHJ38
4W/Btn926VwZ+e/fg3NmGwt/jM2uW8MnV3yOWrvSOWPyWOS/yA2SzyJ3fHbd
uR5yyNgfNCzs4y/ZPg3M9uuMYPuHRsT6XDN4faMhsX7XCrZv3fP6Hsg9ZX+8
b9351CHRetbJdedbJyXnD4h7iXfJy5NzINdATv3AZP3wvLrzQ+hZxNDEzmhY
dyXrgeSZ8Xnoe8ToxOZoan2j9b2Bdeev8ZX4TOJ84nu07f2JeTQWgYE8HvkV
cqXk8+DOxDI/5PdN3hy9llw0+XO0dfKl5H7R2G/Rdu+65w8cg/zyGzVrHw2y
v3u/si7CPnQH9Hpyg/jQB5PrDZhXzKfr6s43kwuDg5A/R08hVwpneSw5f3l7
3ZxuWt35JXJ1cD5ysWi6aLkNg3Np5NTwr3/UnDsjh4Y//l14St3589aFOQn5
SPRIahngCGiZaJr4999q1kbRSFcU/qlm/06OE8lgSfbn5DgJWb6umU+Qw9dr
Kb7JfAGNR5vFVzXnJtE40TbXCeYr1DDoZxZLa+Zb1AgsL/x9zbUY1GSsIPxD
zVosmmwD4R9r1nLRdFcS/ll4jM45MDm+Jc7Ff6ORw7fXD86NkyMnt7pycG6V
HCu57nWD6x0eqZk7oxHx3Mm5gHn+8C844oXBGl5f9R1bdy3Ge6VtCxi+xhxB
p3pU+xaW1quot3i25twgGhK6/3F111ag/3Nt8Ielr3l3st6NbgjHpFYAzXsT
jY/BuXdy8I2FU3B+7xQ0Dx37V+lcPJoiWmKD4FoCagqohVkxONdPzp/am5WC
a3GoyUEb5yWTaybnTC3BesG1NNTUoJX/V3MtDTU1aOP/1sxt4bgXa2zzwrU6
1OzAn/+puVaGHNuy3FrNuW5y3mjZpY7pGs0J4YLECPAzchjEKxsE1w/AsbG3
czO/v7PuPDZr4rhon4eve1K4v/AUrik8q7DONFfnWae03oQu9azw2qX1KXSZ
mcS0pfWZi2rWLjcqrZGj06CFrlxar0F3mYEdLK2/nI5v1vUWaP/LhXVIcjGN
SuuRE5P1/O+C9aOh+G7eEdfR+AHCV/HMas6nUT9zdeX82myeh/B1cCbheYVt
N/U1cE9s+CDh6+FY+KjC/P/GynUY2JRT1H8DnAyfhM0Xvo05UXN9zpnCUyvX
61CPdBrcR/gt4ZcK6wqvoTeX1heIs8l9Ny0db2+p643S+DvQrYR/io5BiT2J
YX/L+WDqi6gP+Do6v0pdADZwucoxMLEvHPSX6BiY2BeOinZKrrx5aQ318Jz/
hmvBuY6MjiGIHYgp9sz5OLganK2R7m+E+q/Uvh9ZM5VjXmJdYu6vomNeYl3y
m3BjODKxOfUX6AbvYbdK6wctomNyYnFiVvLh5MeJ9Yn5yZcT0xLLkq/YQNvX
MOeZd9h43c+5wtPQDXW+TYRHCk8W/kX9X0bHuMS2xOB/JmsIaAesib+TY3pi
eWLgxsLnCF+l438TPgkuC+dk3gtvpP4fgmMRYpI/gv0j2jA1HeR+qAFAz0HX
Qb8g58L73is4t0cOH/0GHYfcCzUX6O/o8OQSydGjf6KDUkvGM+TZbR6ce6GG
Ar0V3ZVcEDl69Ex0TWpHqCGhtmWL4NwhNRvoR+hI5HaooUDPQdcht0NNCPoM
Og25Gmo40G/QcYgHyWERH4Xg+I8cKs9qw2A+Ts6TZ7tRcHxEDQrxYi04l0MN
CHoxujHxJjkT4p3lg+NLcprEk6sGx4PdMl8/PrieEU6KvfqzdPxKTpN4c5Xg
+I2cKvHAasH1NcT08Es4LbkxajjQf9GBl+dZJ3MVOAvcEX6HFgeHRJs7uG7u
iEZHLRp4s8I1aa9E5w/RHb8szCvhj3BZ+OUH0fUN6Oro6ZtUrh+DN8AX3o6u
J4DXwmffj86vozujNzdk/qEh8v5rjrf2zPHSwcHx1a45HjogOL5qleOnQ4Lj
xw453jo6OP5pn+O17sHxVZscHx0RHI8fnOOhHsG5K2qCyC+QZyA+oqbgCvV3
DI7fyOGxfvcNjqf2yvHSocHx4C45HmsnvILGdk+2NXeWjs9aZj2vbTA/GRXt
/6gZRQt+X+MnFdaEmc/Xqb9r4Zoo1t9Z2MTSNavYRmwk9Sh3BtsmbBT5DPIa
2D5sIPUv9wTbNmwcvq1fsJaJpok/2yN4rTwQnXtnzaCfoZnfINw+uNZgTDK/
gHuQKyNnhv9qGczHb0t+d9SYYruwYcTT+wfbOmzeMq072NZgc9AD0AWwXdgw
8jPkaag/Q5OEy3BNbBk2jXqyq4NtLTaXerzrg7VbNFz88e7BXABOgO8+Ljh2
JYbFN/YPjvfJAaAPdQ62BfdE52qxCeiTaPLoA52CbcO90bkJbATvCw1/qvBh
wXoDmj16ZM9g+/xtdK4TDYZcFDkpas2mB9vKh6Jrs7CZ6BfkCNBHjwnWN9H8
sb0nBmvzaPTwi12C9TZyEvjiQcH6KTkLfPfJwbE5MTp6XO9g/ZWcBb7/pGD9
g5wHemuvYNt4d3SuGxuJ/koOA19zQnBugxzHiByLYcsfjM71Y9Px3fhw8mfk
0e4V/iu6lo6augDfyniD0rnhN5L5ELyI3BY5LnzPjsH1VeSQyR0zxyD6jaJr
k3gGHzG31VYp7Y//qFufRlvqXpjLwengHkOCuRMciviZOBouBCdaoP5hwdwG
jvOO8PBgbgXH+lB4VDA3g6N9IHxuMFeCM70tfLZw4+jnR+6O+gS4ExwK7fTe
urkQnGih+s4Jrs/j/ZM7pP5gi+jnSW6R+oLNo9cjuU7qAQiM0PfJTR6b+Q01
cv0Kr/Hd8/MdWtinNoyef3CDczKfqqn1LWwzeJfo9WiDvNMDdPzDwhcUzsH8
re0No98V7wyuwZxmLqMr4ovviL53fDJrkZq3noXXZIe8Xsg9UnPQJHp9kXuk
XoN8/6bRtQ3YIPgdNYjUHlKDR26THCfvdvvgtf+Xzt+jsA04SH23RNcmUv9A
LpOcJtxnh+DcLTlcai0fEm4WrW+RO2LOUn+K5k+sRcyF/nS28Mula7apV6GG
l1ifGq9R2l4zORZDg7pA2+smaz9oUNj6K5O1Cmw+WgaaBvU11NnAV6mhIBdF
zeGGGrtBcq5q99L6DPWVxF58w0BuiRwTei66LrkhckTUIt4YrI8dJTy5dA03
+i41d+T/qAnsllxjRr6UMWi7aLw8r1uD61eo+UeroMaUbw/OSOYOaHn4kg+5
RmGfglaDZkM9NTkUap9nJddLox2dnJx/gKvAWYYnfwOBXsMzRdtA40DLf6Bu
7QUNhnoraqx4ntS0Ect+gD3U9vhkretTtW+ia1541/wGjp2Q3Lcojwej5aDJ
4Vs+j7YF+Bi0Rd4x75a6/HPz+3ynNOe4Njn+X1T4N1HfT403Whg147zLS4UX
l36nlyXbE2KxxRnTj3aFjUG75B3xbqiZPD/PF37bwjyfRifH/twDesl50fEz
NWX4wk+FuxX2idT6vpscvxDHwEWoIT63MCc5nXMJzy+tGZKLJSf7ufq/q7k+
oEWeH9QID2buY99Lx/wRWyW8femYdUjyNzRokYxZTduzonOzO5b+VuTN5PpE
tK8+yTWc1IewrzlcIzq3i49Df6dGgNj8qGB9nRoBYq2uwXo/OQdivU2Jk7M9
ojZi2zyfWJ9ow9QowGX7J+fy4LTUCr2VfO/E3dQq9Uiu36ZmqWdyDT56MPuo
16JGmdoBYlBqDag5oHaCOg5qIRckx9/E4cQy1EhSG0lMQz3D49G5Xmqmto3W
48kV40Opj9uYVtrHsXaOT+aCrKHtovV9csNwDnI55HSoTb42OL9NDTj1OdT0
UhuBTcWWUqtDLQX3wLXPQMeLzieQK4fDUU+zU17/y2qIk2u+qLWAI1Hf1T7Z
N+AjhiZ/g4R2zxxKyTVY1F41yvYEe9OrsE1CW0BjIPY/Pbh2/I/oemzqstEi
0CTQJk4LrqUiv0r9MdpmVdmn4EvwZ9RS7RZdG0JtwmfR9hxtBA5ALRE1g9Qy
ooX/Hm3fqc3BZ7A+qIknX3NQsP/aNrk2vH9hvoA/QXsZILxxZRuLbYWjUUs0
Nbp2Da2H2qPbsn9C64Hv3pXfP9oM9pLnhT7cpe7apuaZz1HLQK0Z3wiQj0TL
+yeZ88J1jy5c+9Qs8ydqIagVo4aRfAu5hV+ja5qoZcLHw7+xj/ia/rreTcl6
Pt+8YXPJp7aNzkfdWLe+QY6e3Dz1KmyzD63jtmy/sW/UZ3xbs1ZMPQB6M9ol
3BJ9jG9w4JisJ/wXWkuruvno9OhaSbQdfBn6xNJgn0a+mPw52trddWuR1OA2
KaxJos1Ts0v+GI0e7b1RZd+KBk9+mxpuanUnEb9W5phwyz7Y6Gg+TW1X38La
OPn41oU1crQ7NDy0tFOD19d/yc/+zxwv3JftJ1oU8dD90bXGl2e9cUJ0beMh
WQ8fG62X8k3Jkmh/TK0GnGdx9PolFwtHQV8+P1qfpGYY/4X/Jbd6TGE9enS0
Hrp31s/HROuz1ASjf14Yrb9S04k+Py76+fHNC3rsldH6M994oZdcG62NoZGh
v14jfGDhb37Qq6+O1kdOznr6VdH6ChoV+YAp0bXa1GxjS8hRLgy2Kei9k6L1
XL7xQg+fLLx74W/GsEfXJGsL5EXQey+O1of5hvKP5BiV2BSfBh+pV3525Jzh
N7XKz44aGPjf9dFrH+0UfndDNP9HOyVeuDG69hztlfiqQeW5Qc3fX8kxJbEk
HIDawluj682oMUTfviI6n0DNNLWDK1eeSyQlsG/fJceL87NefVG0/s03oOjn
l0XPz65Zj56ITSr8DRZ6+CXR+YJOWX+/NDrfQg09/HTVymubGkPs5bfJ8Soa
LvN3aXK8jgaNPz8nWv+mxop6EmrseV/X1O2P4XMjhXcN9ofYb7TFPYPtGzWT
aKs7BT8PfAS+oXlwvLZ+5t+PBefe8I/Uz5CDQxuZFh2LoJFg7+t5vj0SrHXd
HnNtdOnaxqY5PqDGkXhvo+xPdg6OZdBrbw+OaYhF0FOnBsck+F/8Ldpnq+DY
Dr1ySnCMRy0Ueuak4HpO7DU1o/iyFsF6NHol3/h+XNg2YiOpNcEGoo0RP1AL
jUZG7PpDtC/qnP0hfAptChv/sbYXRmtbP6n9G82PiU+IQdG+uSbXOi94PmG/
+fbrvmAtFU31LfWfFaydo6F/JDw6WPu8KTp2RAPl+8SHsZOFv7kdkvka2hk5
2xHcf3IumhgGX0AMQeyAT+B7I54Rz4b1xrcuzyTXg1MXzrdKs5O/N+C7g1c1
9oJobW8xNlLbkfdb2qejz6HvkGuFg6K93hwdy/XIfIZ617eDNTquxzc21MMs
F+wb8ZGfBmtqxMdNKvsWcgzE3z8l6xfkGMjtoimhJfGNCPH2j8n6BTkKtEXq
pfgWFI1xhcyvyCVTE41vxcd+HKzRwS3gGJ8H12igPfKND/VqaJBok3zztF9h
jRItk2fIs2ua39/3yXoImjjPj3zU5tirrCf8kKzXkGNhPvyezI/IieAb8ZHP
FNbwiYc2rZxbIefLs+MZwgUX5PnxWzJ/IseC/vBLsp5Bjoh85RP1ZZ/hLKsl
RF/4OZkfkXNBq+Wbj5ULa7boGb8m+0NyRv8HZWHq8w==
              "]], Polygon3DBox[CompressedData["
1:eJwt13nYjlUaAPDX9318VChLdkaIUXS1T7JVNBIpSxpd9lQi+5IQRY1mSlPZ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              "]], 
             Polygon3DBox[{{1795, 1294, 1293, 1794, 929, 930}, {1809, 1307, 
              1306, 1808, 949, 950}, {1792, 1292, 1291, 1791, 925, 926}, {
              1303, 1447, 822, 938, 1801, 1302}, {1804, 1303, 1302, 1803, 941,
               942}, {1287, 1439, 816, 918, 1785, 1286}, {1797, 1296, 1295, 
              1796, 933, 934}, {1290, 1440, 817, 920, 1787, 1289}, {1301, 
              1446, 821, 936, 1799, 1300}, {1292, 1441, 818, 922, 1789, 
              1291}, {1807, 1305, 1304, 1806, 945, 
              946}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{559, 745, 1830, 560, 758, 585, 1896, 771, 599, 1899, 
            784, 613, 1904, 797, 627, 1911, 919, 810, 641, 1920, 920, 817, 
            1990, 648, 956, 824, 1995, 655, 976, 1972, 831, 662, 1925, 838, 
            676, 1928, 852, 690, 1933, 865, 704, 1940, 878, 718, 1949, 891, 
            732}], Line3DBox[{561, 746, 1831, 562, 759, 1840, 586, 772, 600, 
            1900, 785, 614, 1905, 798, 628, 1912, 921, 811, 642, 1921, 922, 
            818, 1991, 649, 957, 825, 1996, 656, 977, 832, 1874, 663, 839, 
            677, 1929, 853, 691, 1934, 866, 705, 1941, 879, 719, 1950, 892, 
            733}], Line3DBox[{563, 747, 1832, 564, 760, 1841, 587, 773, 1849, 
            601, 786, 615, 1906, 799, 629, 1913, 923, 924, 1018, 1983, 925, 
            926, 1861, 1027, 958, 959, 1868, 1035, 978, 979, 1875, 664, 840, 
            1880, 678, 854, 692, 1935, 867, 706, 1942, 880, 720, 1951, 893, 
            734}], Line3DBox[{565, 748, 1833, 566, 761, 1842, 588, 774, 1850, 
            602, 787, 1855, 616, 800, 630, 1914, 927, 928, 1019, 1984, 929, 
            930, 1862, 1028, 960, 961, 1869, 1036, 980, 981, 1876, 665, 841, 
            1881, 679, 855, 1887, 693, 868, 707, 1943, 881, 721, 1952, 894, 
            735}], Line3DBox[{567, 749, 1834, 568, 1165, 2019, 1166, 589, 
            1170, 2021, 1171, 603, 1175, 2023, 1176, 617, 1180, 2024, 1181, 
            631, 931, 932, 1020, 1985, 933, 934, 1863, 1029, 962, 963, 1870, 
            1037, 982, 1973, 983, 666, 842, 1882, 680, 1185, 2026, 1186, 694, 
            1190, 2027, 1191, 708, 1195, 1196, 722, 1953, 1200, 1201, 736}], 
           Line3DBox[CompressedData["
1:eJwVz8sqRAEcx/E/JnbWktQ0LlMuxW42Q9lZzGKSB2BhMWW2PAEbDbkUaigU
IxQW8hZDs5otIxnyBPI5i0+/c77n0jnppXJxtSMiFsh1Rmx0RWwyn4rYZocK
bz0RW3rB8T4H7NHSK3rR8RHHXGuHybP6oj3hUzu1VXb1vJ2hoc/aPobI8Of6
sB2hnyyjvLq3244xzgCT1PUJ28sU0wyyzotra3Yu+QfvfXL+bpedn3PBs3Zm
P/QVe0Vbq9nL5Nv1kr3lW7+zN3zpZdvUHuwj9/zov/wDrRAvlQ==
            "]], 
           Line3DBox[{572, 752, 1835, 573, 764, 1843, 592, 1897, 777, 606, 
            1901, 790, 620, 1907, 803, 634, 1915, 935, 814, 645, 1922, 936, 
            821, 1993, 652, 965, 828, 1997, 659, 985, 835, 669, 1926, 845, 
            683, 1930, 858, 697, 1936, 871, 711, 1944, 884, 725, 1955, 897, 
            739}], Line3DBox[{574, 753, 1836, 575, 765, 1844, 593, 778, 607, 
            1902, 791, 621, 1908, 804, 635, 1916, 937, 815, 646, 1923, 938, 
            822, 1994, 653, 966, 829, 1998, 660, 986, 836, 1877, 670, 846, 
            684, 1931, 859, 698, 1937, 872, 712, 1945, 885, 726, 1956, 898, 
            740}], Line3DBox[{576, 754, 1837, 577, 766, 1845, 594, 779, 1851, 
            608, 792, 622, 1909, 805, 636, 1917, 939, 940, 1023, 1986, 941, 
            942, 1864, 1031, 967, 968, 1871, 1039, 987, 988, 1878, 671, 847, 
            1883, 685, 860, 699, 1938, 873, 713, 1946, 886, 727, 1957, 899, 
            741}], Line3DBox[{578, 755, 1838, 579, 767, 1846, 595, 780, 1852, 
            609, 793, 1856, 623, 1910, 806, 637, 1918, 943, 944, 1024, 1987, 
            945, 946, 1865, 1032, 969, 970, 1872, 1040, 989, 990, 1879, 672, 
            848, 1884, 686, 861, 1888, 700, 874, 714, 1947, 887, 728, 1958, 
            900, 742}], 
           Line3DBox[{580, 756, 1839, 581, 768, 1847, 596, 781, 1853, 610, 
            794, 1857, 624, 807, 1859, 638, 947, 948, 1025, 1988, 949, 950, 
            1866, 1033, 971, 972, 1873, 1041, 991, 1974, 992, 673, 849, 1885, 
            687, 862, 1889, 701, 875, 1891, 715, 888, 729, 1959, 901, 743}], 
           Line3DBox[{582, 907, 1960, 908, 583, 769, 1848, 597, 782, 1854, 
            611, 795, 1858, 625, 808, 1860, 639, 951, 1967, 952, 1026, 953, 
            1968, 954, 1867, 1034, 973, 1970, 974, 1042, 993, 1975, 994, 674, 
            850, 1886, 688, 863, 1890, 702, 876, 1892, 716, 889, 1893, 730, 
            1009, 1010, 1011}], 
           Line3DBox[{731, 890, 1005, 1948, 717, 877, 1939, 703, 864, 1932, 
            689, 851, 1927, 675, 837, 1924, 661, 830, 1971, 975, 654, 823, 
            1969, 955, 647, 1989, 816, 918, 1919, 640, 809, 1966, 917, 626, 
            796, 1903, 612, 783, 1898, 598, 770, 1895, 584, 757, 1894, 558, 
            744, 902, 1012}], Line3DBox[CompressedData["
1:eJwVzDcvBmAUhuFXSfwAbbbbRPswEDWfXqLXIIg+IHrvi9kv8AtsJmaDRBBR
d5uaSFyGO89zzrnfN61von48JoSwhOfYEFbiQiiPD6EAqZjCNO4TQpiUV5xp
zqF+gBSMYRx3nFF5wRng7Ot7SMYwLt1H5BBOOS2cXX0HSXhxH5CD6McJJ8rZ
1p/dtmQietGHHhxzCjnz+gOnWL6a82U38nDEidhdu8+ai3Bjjsgu5GKfk2E3
oxfiXM+Rnbj1LltucNLtN/Uy/OrtsuP/P06bnOOk2W/opfjWW+SZe6tsxggn
yX5dX8OX3iib0PD/n3uC3ar+5N2K/DTXyjrUoJrzg2V9CR/ulbIKUUTc3vDo
/aJ5Ae+cLFmBTJTgDyqmPsA=
            "]], 
           Line3DBox[{738, 896, 1007, 1954, 724, 883, 1981, 1004, 710, 870, 
            1980, 1002, 696, 857, 1979, 1000, 682, 844, 1977, 998, 668, 834, 
            2013, 1139, 1055, 1076, 1075, 658, 827, 1058, 1053, 1072, 1071, 
            651, 1992, 820, 1057, 1051, 1066, 1065, 2004, 644, 813, 1056, 
            1049, 1141, 2000, 1140, 633, 802, 1965, 916, 619, 789, 1964, 914, 
            605, 776, 1962, 912, 591, 763, 1961, 910, 571, 2007, 751, 905, 
            906}]}, {
           Line3DBox[{1206, 1486, 744, 1205, 1830, 1404, 1207, 1831, 1405, 
            1208, 1832, 1406, 1209, 1833, 1407, 1210, 1834, 1408, 1211, 904, 
            1644, 1693, 1727, 2015, 1696, 1694, 1695, 1645, 2007, 1409, 1212, 
            1835, 1410, 1213, 1836, 1411, 1214, 1837, 1412, 1215, 1838, 1413, 
            1216, 1839, 1414, 1217, 1960, 1487, 1415, 1488}], 
           Line3DBox[{1219, 1728, 1894, 1218, 758, 1220, 1840, 1416, 1221, 
            1841, 1417, 1222, 1842, 1418, 1223, 2019, 1708, 1419, 1709, 2020, 
            1489, 1646, 503, 1150, 505, 504, 1647, 1729, 1961, 1224, 1843, 
            1420, 1225, 1844, 1421, 1226, 1845, 1422, 1227, 1846, 1423, 1228, 
            1847, 1424, 1229, 1848, 1425, 1230}], 
           Line3DBox[{1232, 1730, 1895, 1231, 1731, 1896, 1233, 772, 1234, 
            1849, 1426, 1235, 1850, 1427, 1236, 2021, 1710, 1428, 1711, 2022, 
            1490, 1648, 1697, 1703, 1698, 2016, 1568, 1699, 1491, 1650, 1732, 
            1962, 1237, 1733, 1897, 1238, 778, 1239, 1851, 1429, 1240, 1852, 
            1430, 1241, 1853, 1431, 1242, 1854, 1432, 1243}], 
           Line3DBox[{1245, 1734, 1898, 1244, 1735, 1899, 1246, 1736, 1900, 
            1247, 786, 1248, 1855, 1433, 1249, 2023, 1712, 1713, 1250, 1826, 
            1963, 1492, 1649, 1493, 1704, 1108, 1653, 1652, 1494, 1651, 1495, 
            1964, 1251, 1737, 1901, 1252, 1738, 1902, 1253, 792, 1254, 1856, 
            1434, 1255, 1857, 1435, 1256, 1858, 1436, 1257}], 
           Line3DBox[{1259, 1739, 1903, 1258, 1740, 1904, 1260, 1741, 1905, 
            1261, 1742, 1906, 1262, 800, 1263, 2024, 1714, 1715, 1264, 2025, 
            1716, 1496, 1654, 1497, 1705, 1657, 1982, 1658, 1656, 1498, 1655, 
            1499, 1965, 1265, 1743, 1907, 1266, 1744, 1908, 1267, 1745, 1909, 
            1268, 1746, 1910, 1269, 1859, 1437, 1270, 1860, 1438, 1271}], 
           Line3DBox[{1285, 1530, 1529, 1967, 1284, 1527, 947, 1283, 1525, 
            1918, 1805, 1282, 1523, 1917, 1802, 1281, 1800, 1916, 1520, 1280, 
            1798, 1915, 1517, 1279, 2000, 1688, 1689, 1633, 1662, 1661, 1278, 
            2008, 1659, 1660, 1630, 1632, 1631, 2005, 1277, 1515, 931, 1276, 
            1513, 1914, 1793, 1275, 1511, 1913, 1790, 1274, 1788, 1912, 1508, 
            1273, 1786, 1911, 1505, 1272, 1966, 1501, 1500, 1504}], 
           Line3DBox[{1288, 1503, 1583, 1989, 1439, 1287, 1507, 1584, 1990, 
            1440, 1290, 1510, 1585, 1991, 1441, 1292, 1792, 1861, 1586, 1442, 
            1294, 1795, 1862, 1587, 1443, 1296, 1797, 1863, 1588, 1444, 1612, 
            1823, 2003, 1621, 1589, 1634, 819, 1690, 1635, 1636, 1590, 1618, 
            1992, 1445, 1613, 1519, 1591, 1993, 1446, 1301, 1522, 1592, 1994, 
            1447, 1303, 1804, 1864, 1593, 1448, 1305, 1807, 1865, 1594, 1449, 
            1307, 1809, 1866, 1595, 1450, 1309, 1810, 1867, 1596, 1451, 
            1539}], Line3DBox[{1310, 1532, 1582, 1531, 1968, 1308, 1528, 1581,
             1988, 1808, 1306, 1526, 1580, 1987, 1806, 1304, 1524, 1579, 1986,
             1803, 1302, 1801, 1923, 1578, 1521, 1300, 1799, 1922, 1577, 1518,
             1299, 1824, 2004, 1622, 1576, 1628, 1627, 1298, 2009, 1625, 1626,
             1575, 1620, 1061, 1297, 1516, 1574, 1985, 1796, 1295, 1514, 1573,
             1984, 1794, 1293, 1512, 1572, 1983, 1791, 1291, 1789, 1921, 1571,
             1509, 1289, 1787, 1920, 1570, 1506, 1286, 1785, 1919, 1569, 1502,
             1534}], 
           Line3DBox[{1312, 1533, 1597, 1747, 1969, 1311, 1535, 1598, 1995, 
            1452, 1313, 1536, 1599, 1996, 1453, 1314, 1811, 1868, 1600, 1454, 
            1315, 1812, 1869, 1601, 1455, 1316, 1813, 1870, 1602, 1456, 1614, 
            2001, 1623, 1624, 1603, 1637, 1748, 2006, 1691, 1638, 1639, 1604, 
            1619, 827, 1615, 1537, 1605, 1997, 1457, 1317, 1538, 1606, 1998, 
            1458, 1318, 1814, 1871, 1607, 1459, 1319, 1815, 1872, 1608, 1460, 
            1320, 1816, 1873, 1609, 1461, 1321, 1970, 1540, 1610, 1462, 
            1550}], Line3DBox[{1323, 1541, 1749, 1971, 1322, 1542, 1750, 1972,
             1324, 1543, 1874, 1463, 1325, 1817, 1875, 1464, 1326, 1818, 1876,
             1465, 1327, 1973, 1544, 1466, 1616, 2002, 1692, 1545, 1640, 1641,
             1642, 2014, 1552, 1643, 1546, 1629, 1686, 1687, 2013, 1617, 1547,
             835, 1328, 1548, 1877, 1467, 1329, 1819, 1878, 1468, 1330, 1820, 
            1879, 1469, 1331, 1974, 1549, 1470, 1332, 1975, 1551, 1471, 
            1553}], Line3DBox[{1334, 1751, 1924, 1333, 1752, 1925, 1335, 839, 
            1336, 1880, 1472, 1337, 1881, 1473, 1338, 1882, 1474, 1339, 1976, 
            1554, 1663, 1700, 1701, 2017, 1611, 1702, 1555, 1665, 1753, 1977, 
            1340, 1754, 1926, 1341, 846, 1342, 1883, 1475, 1343, 1884, 1476, 
            1344, 1885, 1477, 1345, 1886, 1478, 1346}], 
           Line3DBox[{1348, 1755, 1927, 1347, 1756, 1928, 1349, 1757, 1929, 
            1350, 854, 1351, 1887, 1479, 1352, 2026, 1717, 1718, 1353, 1827, 
            1978, 1556, 1664, 1557, 1706, 2018, 1825, 1668, 1667, 1558, 1666, 
            1559, 1979, 1354, 1758, 1930, 1355, 1759, 1931, 1356, 860, 1357, 
            1888, 1480, 1358, 1889, 1481, 1359, 1890, 1482, 1360}], 
           Line3DBox[{1362, 1760, 1932, 1361, 1761, 1933, 1363, 1762, 1934, 
            1364, 1763, 1935, 1365, 868, 1366, 2027, 1719, 1720, 1367, 2028, 
            1721, 1560, 1669, 1561, 1707, 1672, 1999, 1673, 1671, 1562, 1670, 
            1563, 1980, 1368, 1764, 1936, 1369, 1765, 1937, 1370, 1766, 1938, 
            1371, 874, 1372, 1891, 1483, 1373, 1892, 1484, 1374}], 
           Line3DBox[{1376, 1767, 1939, 1375, 1768, 1940, 1377, 1769, 1941, 
            1378, 1770, 1942, 1379, 1771, 1943, 1380, 1195, 1722, 1381, 2029, 
            1723, 1724, 1674, 1679, 1678, 2010, 1382, 1676, 1677, 1675, 1564, 
            1981, 1383, 1772, 1944, 1384, 1773, 1945, 1385, 1774, 1946, 1386, 
            1775, 1947, 1387, 888, 1388, 1893, 1485, 1389}], 
           Line3DBox[{1403, 1567, 1009, 1402, 1959, 1784, 1401, 1958, 1783, 
            1400, 1957, 1782, 1399, 1956, 1781, 1398, 1955, 1780, 1397, 1822, 
            1954, 1681, 1683, 1682, 1396, 2012, 1684, 1685, 1680, 1726, 2011, 
            1829, 1395, 1725, 1953, 1828, 1394, 1952, 1779, 1393, 1951, 1778, 
            1392, 1950, 1777, 1391, 1949, 1776, 1390, 1821, 1948, 1565, 
            1566}]}}},
        VertexNormals->CompressedData["
1:eJzVXXl8TVe0TghqqLFqJtRUElTMwmqRGotKUfM8hppbQ0u1htLG0KqpWrNW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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {180.1, -180}, ImageScaled[{0.5, 0.5}], {359.99999999999994`, 360},
      ContentSelectable->True], InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJztXXt8VdWxBgQR8FJuRaGRKipVHoqKoAQjU6+tghLairXUKlUEjVypFqk8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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFmwnYF9P7xmc5MyNRRLYiCZVoQUIphUrIVrYWiSiJ7ErZSiJpj0pUtiI7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              "]], Polygon3DBox[CompressedData["
1:eJw1mwf8T9X/x++95577IU0lGipCiaSEkoaRNIkGDVkJSVOF0NCilEpCQnsv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              "]], Polygon3DBox[CompressedData["
1:eJwt1nmg11Max/Gfm3sHlRlUQyVSjRatmilKRaW0a58WWhGmDKaFNtq1a9eu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              "]], 
             Polygon3DBox[{{1479, 1690, 832, 663, 662, 981}, {1500, 1499, 
              1000, 692, 693, 1001}, {1449, 1448, 934, 603, 604, 935}, {1434, 
              1433, 920, 573, 574, 921}, {1424, 1656, 745, 559, 558, 908}, {
              1442, 1441, 927, 588, 589, 928}, {1428, 1660, 758, 574, 573, 
              915}, {1485, 1484, 986, 662, 663, 987}, {1493, 1492, 993, 677, 
              678, 994}, {1475, 1686, 819, 648, 647, 
              974}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{527, 729, 1756, 528, 741, 553, 1821, 754, 568, 1823, 
            767, 583, 1827, 780, 598, 1834, 793, 613, 1843, 940, 1104, 449, 
            1599, 452, 451, 1105, 809, 1790, 629, 1852, 815, 642, 1854, 828, 
            657, 1856, 841, 672, 1860, 854, 687, 1866, 867, 702, 1875, 880, 
            717}], 
           Line3DBox[{529, 730, 1757, 530, 742, 1767, 554, 755, 569, 1824, 
            768, 584, 1828, 781, 599, 1835, 794, 614, 1844, 941, 1106, 1108, 
            1109, 1928, 1113, 1112, 942, 1110, 810, 1791, 630, 816, 1798, 643,
             829, 658, 1857, 842, 673, 1861, 855, 688, 1867, 868, 703, 1876, 
            881, 718}], 
           Line3DBox[{531, 731, 1758, 532, 743, 1768, 555, 756, 1776, 570, 
            769, 585, 1829, 782, 600, 1836, 795, 615, 1845, 943, 1107, 944, 
            1136, 1952, 1037, 1137, 945, 1111, 946, 1792, 631, 817, 1799, 644,
             830, 1806, 659, 843, 674, 1862, 856, 689, 1868, 869, 704, 1877, 
            882, 719}], 
           Line3DBox[{533, 732, 1759, 534, 744, 1769, 556, 757, 1777, 571, 
            770, 1782, 586, 783, 601, 1837, 796, 616, 1846, 947, 1114, 948, 
            1138, 1953, 1038, 1139, 949, 1115, 950, 1793, 632, 818, 1800, 645,
             831, 1807, 660, 844, 1812, 675, 857, 690, 1869, 870, 705, 1878, 
            883, 720}], 
           Line3DBox[{535, 892, 893, 1760, 536, 904, 1889, 905, 557, 911, 
            1892, 912, 572, 918, 1896, 919, 587, 925, 1899, 926, 602, 932, 
            933, 1062, 1941, 1134, 1135, 1085, 1092, 1091, 1947, 1140, 1093, 
            1094, 1086, 1088, 1087, 1945, 1064, 970, 1906, 971, 646, 977, 
            1909, 978, 661, 984, 1913, 985, 676, 991, 1916, 992, 691, 998, 
            999, 706, 1879, 1006, 1007, 1011}], 
           Line3DBox[{537, 894, 1019, 896, 538, 906, 1022, 1920, 908, 558, 
            913, 1025, 1921, 915, 573, 920, 1897, 1028, 922, 588, 927, 1900, 
            1031, 929, 603, 934, 1902, 1034, 936, 1063, 1068, 1069, 1039, 
            1096, 1095, 1948, 1141, 1097, 1098, 967, 1075, 1905, 1074, 1065, 
            972, 1044, 1932, 974, 647, 979, 1047, 1933, 981, 662, 986, 1914, 
            1050, 988, 677, 993, 1917, 1053, 995, 692, 1000, 1919, 1056, 1002,
             707, 1880, 1008, 1059, 1010, 1013}], 
           Line3DBox[{542, 736, 1763, 543, 748, 1771, 562, 761, 577, 1825, 
            774, 592, 1830, 787, 607, 1839, 800, 620, 1847, 953, 1120, 1122, 
            1123, 1931, 1127, 1126, 954, 1124, 813, 1794, 636, 822, 1801, 651,
             835, 666, 1858, 848, 681, 1863, 861, 696, 1871, 874, 711, 1883, 
            887, 724}], 
           Line3DBox[{544, 737, 1764, 545, 749, 1772, 563, 762, 1778, 578, 
            775, 593, 1831, 788, 608, 1840, 801, 621, 1848, 955, 1121, 956, 
            1789, 1142, 1042, 1143, 957, 1125, 958, 1795, 637, 823, 1802, 652,
             836, 1808, 667, 849, 682, 1864, 862, 697, 1872, 875, 712, 1884, 
            888, 725}], 
           Line3DBox[{546, 738, 1765, 547, 750, 1773, 564, 763, 1779, 579, 
            776, 1783, 594, 1832, 789, 609, 1841, 802, 622, 1849, 959, 1128, 
            960, 1144, 1954, 1043, 1145, 961, 1129, 962, 1796, 638, 824, 1803,
             653, 837, 1809, 668, 850, 1813, 683, 863, 698, 1873, 876, 713, 
            1885, 889, 726}], 
           Line3DBox[{548, 739, 1766, 549, 751, 1774, 565, 764, 1780, 580, 
            777, 1784, 595, 790, 1786, 610, 803, 623, 1850, 963, 1130, 1147, 
            1146, 1955, 1150, 1148, 1149, 1131, 964, 1797, 639, 825, 1804, 
            654, 838, 1810, 669, 851, 1814, 684, 864, 1816, 699, 877, 714, 
            1886, 890, 727}], 
           Line3DBox[{550, 902, 1888, 903, 551, 752, 1775, 566, 765, 1781, 
            581, 778, 1785, 596, 791, 1787, 611, 804, 1788, 624, 965, 1904, 
            1132, 1152, 1151, 1956, 1155, 1153, 1154, 1133, 1951, 966, 640, 
            826, 1805, 655, 839, 1811, 670, 852, 1815, 685, 865, 1817, 700, 
            878, 1818, 715, 1015, 1016, 1017}], 
           Line3DBox[{716, 879, 1005, 1874, 701, 866, 1865, 686, 853, 1859, 
            671, 840, 1855, 656, 827, 1853, 641, 814, 1851, 628, 808, 1903, 
            1101, 1103, 1102, 625, 1949, 805, 1100, 1099, 939, 1842, 612, 792,
             1833, 597, 779, 1826, 582, 766, 1822, 567, 753, 1820, 552, 740, 
            1819, 526, 728, 891, 1018}], 
           Line3DBox[{721, 884, 1060, 1940, 1009, 708, 871, 1057, 1938, 1001, 
            693, 858, 1054, 1936, 994, 678, 845, 1051, 1934, 987, 663, 832, 
            1910, 1048, 980, 648, 819, 1907, 1045, 973, 633, 811, 1943, 1066, 
            968, 1078, 1077, 626, 806, 1076, 1040, 1071, 1929, 1070, 617, 797,
             1035, 1926, 935, 604, 784, 1032, 1924, 928, 589, 771, 1029, 1922,
             921, 574, 758, 1893, 1026, 914, 559, 745, 1890, 1023, 907, 539, 
            733, 1887, 1020, 895, 897}], 
           Line3DBox[{722, 885, 1061, 1012, 1881, 709, 872, 1058, 1939, 1003, 
            694, 859, 1055, 1937, 996, 679, 846, 1052, 1935, 989, 664, 833, 
            1911, 1049, 982, 649, 820, 1908, 1046, 975, 634, 812, 1067, 969, 
            1081, 1080, 627, 1950, 807, 1079, 1041, 1073, 1072, 1944, 618, 
            798, 1036, 1927, 937, 605, 785, 1033, 1925, 930, 590, 772, 1030, 
            1923, 923, 575, 759, 1894, 1027, 916, 560, 746, 1891, 1024, 909, 
            540, 1761, 734, 1021, 898, 899}], 
           Line3DBox[{723, 886, 1014, 1882, 710, 873, 1004, 1870, 695, 860, 
            1918, 997, 680, 847, 1915, 990, 665, 834, 1912, 983, 650, 821, 
            976, 635, 1942, 1084, 1083, 1082, 952, 1118, 1119, 1930, 1117, 
            1116, 1089, 951, 1090, 1946, 619, 799, 938, 1838, 606, 786, 1901, 
            931, 591, 773, 1898, 924, 576, 760, 1895, 917, 561, 1770, 747, 
            910, 541, 1762, 735, 900, 901}]}, {
           Line3DBox[{1157, 1418, 728, 1156, 1756, 1335, 1158, 1757, 1336, 
            1159, 1758, 1337, 1160, 1759, 1338, 1161, 1719, 1760, 1339, 1511, 
            896, 1654, 1887, 1512, 1419, 1761, 1340, 1513, 1420, 1762, 1341, 
            1162, 1763, 1342, 1163, 1764, 1343, 1164, 1765, 1344, 1165, 1766, 
            1345, 1166, 1888, 1421, 1346, 1422}], 
           Line3DBox[{1168, 1655, 1819, 1167, 741, 1169, 1767, 1347, 1170, 
            1768, 1348, 1171, 1769, 1349, 1172, 1889, 1423, 1350, 1514, 1920, 
            1424, 1656, 1890, 1515, 1425, 1657, 1891, 1516, 1426, 1770, 1351, 
            1173, 1771, 1352, 1174, 1772, 1353, 1175, 1773, 1354, 1176, 1774, 
            1355, 1177, 1775, 1356, 1178}], 
           Line3DBox[{1180, 1658, 1820, 1179, 1659, 1821, 1181, 755, 1182, 
            1776, 1357, 1183, 1777, 1358, 1184, 1892, 1427, 1359, 1517, 1921, 
            1428, 1660, 1893, 1518, 1429, 1661, 1894, 1519, 1430, 1662, 1895, 
            1185, 761, 1186, 1778, 1360, 1187, 1779, 1361, 1188, 1780, 1362, 
            1189, 1781, 1363, 1190}], 
           Line3DBox[{1192, 1663, 1822, 1191, 1664, 1823, 1193, 1665, 1824, 
            1194, 769, 1195, 1782, 1364, 1196, 1896, 1431, 1432, 1197, 1897, 
            1433, 1434, 1922, 1198, 1435, 1436, 1923, 1199, 1437, 1438, 1898, 
            1200, 1666, 1825, 1201, 775, 1202, 1783, 1365, 1203, 1784, 1366, 
            1204, 1785, 1367, 1205}], 
           Line3DBox[{1207, 1667, 1826, 1206, 1668, 1827, 1208, 1669, 1828, 
            1209, 1670, 1829, 1210, 783, 1211, 1899, 1439, 1440, 1212, 1900, 
            1441, 1442, 1924, 1213, 1443, 1444, 1925, 1214, 1445, 1446, 1901, 
            1215, 1671, 1830, 1216, 1672, 1831, 1217, 1673, 1832, 1218, 1786, 
            1368, 1219, 1787, 1369, 1220}], 
           Line3DBox[{1222, 1674, 1833, 1221, 1675, 1834, 1223, 1676, 1835, 
            1224, 1677, 1836, 1225, 1678, 1837, 1226, 932, 1447, 1227, 1902, 
            1448, 1449, 1926, 1228, 1450, 1451, 1927, 1229, 1452, 1838, 1720, 
            1230, 1679, 1839, 1231, 1680, 1840, 1232, 1681, 1841, 1233, 803, 
            1234, 1788, 1370, 1235}], Line3DBox[CompressedData["
1:eJwV0dkunlEUh/EtpuswRyLmMSUEUWNQlRJDa27NNZw6Fi7BENMNoUK5CTVP
rf72wZNnrf9e+/2+d71powuf5hNCCGs4SwohPSWEfGTgOjmEKv6AndQQBvSV
6mF8w1f8kdVwLQbV1TyCMYziRnbK6+7X85C+jscxgT35Od/KG7kpPl/dwJOY
wr6ZC76Tf+Tm+NvqXD7GlvPvXIAHeSa3xP+ovseJetvMDy7EkyyLW5GdGMKj
vkg9jWK86LO5DbvujeifUaKfQSle9b94w3l7fFd9HpdhFuV4k53xppnOuBN9
B89hAfP4G5F3c0/ci7qLF/ETB+7+5n/yXv4S96b+zJc4dL7MS3iX9/NA3J26
j69wZGaVVxCQE79V3K2ZCv4PLf5Acw==
            "]], 
           Line3DBox[{1250, 1468, 1467, 1904, 1249, 1466, 1850, 1733, 1248, 
            1465, 1849, 1731, 1247, 1464, 1848, 1729, 1246, 1728, 1847, 1462, 
            1245, 1581, 1946, 1744, 1546, 1565, 1564, 1244, 1741, 1944, 1561, 
            1545, 1560, 1559, 1243, 1929, 1557, 1558, 1544, 1556, 1068, 1242, 
            1554, 1555, 1543, 1629, 1941, 1750, 1241, 1461, 1846, 1726, 1240, 
            1460, 1845, 1724, 1239, 1723, 1844, 1458, 1238, 1722, 1843, 1456, 
            1236, 1721, 1842, 1453, 1455}], 
           Line3DBox[{1251, 1454, 1682, 1903, 1594, 1457, 1790, 1384, 1598, 
            1459, 1791, 1385, 1601, 1725, 1792, 1386, 1605, 1727, 1793, 1387, 
            1578, 1743, 1945, 1579, 1547, 1551, 1388, 1470, 1905, 1562, 1563, 
            1548, 1552, 1683, 1943, 1471, 1566, 1567, 1549, 1553, 812, 1472, 
            1568, 1569, 1550, 1742, 1942, 1389, 1576, 1463, 1794, 1390, 1614, 
            1730, 1795, 1391, 1618, 1732, 1796, 1392, 1622, 1734, 1797, 1393, 
            1626, 1951, 1469, 1394, 1473}], 
           Line3DBox[{1253, 1684, 1851, 1252, 1685, 1852, 1254, 1798, 1395, 
            1255, 1799, 1396, 1256, 1800, 1397, 1257, 1906, 1474, 1398, 1537, 
            1932, 1475, 1686, 1907, 1538, 1476, 1687, 1908, 1539, 1477, 821, 
            1258, 1801, 1399, 1259, 1802, 1400, 1260, 1803, 1401, 1261, 1804, 
            1402, 1262, 1805, 1403, 1263}], 
           Line3DBox[{1265, 1688, 1853, 1264, 1689, 1854, 1266, 829, 1267, 
            1806, 1404, 1268, 1807, 1405, 1269, 1909, 1478, 1406, 1540, 1933, 
            1479, 1690, 1910, 1541, 1480, 1691, 1911, 1542, 1481, 1692, 1912, 
            1270, 835, 1271, 1808, 1407, 1272, 1809, 1408, 1273, 1810, 1409, 
            1274, 1811, 1410, 1275}], 
           Line3DBox[{1277, 1693, 1855, 1276, 1694, 1856, 1278, 1695, 1857, 
            1279, 843, 1280, 1812, 1411, 1281, 1913, 1482, 1483, 1282, 1914, 
            1484, 1485, 1934, 1283, 1486, 1487, 1935, 1284, 1488, 1489, 1915, 
            1285, 1696, 1858, 1286, 849, 1287, 1813, 1412, 1288, 1814, 1413, 
            1289, 1815, 1414, 1290}], 
           Line3DBox[{1292, 1697, 1859, 1291, 1698, 1860, 1293, 1699, 1861, 
            1294, 1700, 1862, 1295, 857, 1296, 1916, 1490, 1491, 1297, 1917, 
            1492, 1493, 1936, 1298, 1494, 1495, 1937, 1299, 1496, 1497, 1918, 
            1300, 1701, 1863, 1301, 1702, 1864, 1302, 863, 1303, 1816, 1415, 
            1304, 1817, 1416, 1305}], 
           Line3DBox[{1307, 1703, 1865, 1306, 1704, 1866, 1308, 1705, 1867, 
            1309, 1706, 1868, 1310, 1707, 1869, 1311, 998, 1498, 1312, 1919, 
            1499, 1500, 1938, 1313, 1501, 1502, 1939, 1314, 1503, 1870, 1735, 
            1315, 1708, 1871, 1316, 1709, 1872, 1317, 1710, 1873, 1318, 877, 
            1319, 1818, 1417, 1320}], 
           Line3DBox[{1334, 1510, 1015, 1333, 1886, 1718, 1332, 1885, 1717, 
            1331, 1884, 1716, 1330, 1883, 1715, 1329, 1740, 1882, 1509, 1328, 
            1739, 1881, 1508, 1327, 1940, 1507, 1880, 1738, 1326, 1506, 1879, 
            1737, 1325, 1878, 1714, 1324, 1877, 1713, 1323, 1876, 1712, 1322, 
            1875, 1711, 1321, 1736, 1874, 1504, 1505}]}}},
        VertexNormals->CompressedData["
1:eJztXXd8lUXWDk16hwVCMYgCKwRYAtIZpS0KQghdijQR6YJ0AaUtoDSXFgRE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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {540.3, -180}, ImageScaled[{0.5, 0.5}], {359.99999999999994`, 360},
      ContentSelectable->True]}, {InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nX28VlWVx9HUJu3DOI1NfsgUi1AxGSUzwBurslFQqSadpBkyxzTJkbGo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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmnfgjlUbx+9zn/NkZiYKqewRSgrZOyLR0kZTivbQkEpaiNCwKjS1Rctq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              "]], Polygon3DBox[CompressedData["
1:eJwtmnfgV9Mfxu8993ysyE5SNJQoGrSQhMxIe6DSTiqrIQ2lrGSXkRINZDSt
iJA9GkZUMn5I9h7N3/PynD9u3/u6534+fT/3c877ed7P+VbpMbjNoJBl2Rj9
E/WzV5Fl/XQ+Pc+yW8WtxMs1cJ14hLibeIX4bvFE8T7iarr/NHET8QTx/zT+
oniBuI/4A/EM8e3i8eLrdf8L4vniquJFGu8r7iQeLP5E/LB4qvha8VfiZeKF
4ht0/oqOCeJh4j01/o/4FHFDcTWdL9Xxkv6PTeJN+vmDeB+N7y1uo/tfE98g
HiluK35dfKN4lHgv8b/iFuJG4nbiN8QTxaPFe4s3i08VNxb/qvf/SVxZXF58
ocZXie8VTxJ3Fb8nvkt8o7iD+E3xLeKx4k7it8W3ia8R9xa/L75ffJu4tfhV
8fXiq8TldN5Z/+fN4qvFR2j8OPEgcXc+v3ix7ukn7izuKH5LfKt4nLi7eKX4
HvFN4l/02h/Fh4gPEPfV+IfiB8R3iAeKL9E9c8T3iPcVz9N4J/EZ4v3E8/md
xGeKv9e9W3S+n459xFM0vknjK8Uvi28WfyN+XbxEfKf4br3mPfEy8Q6d/6rx
Y8Q1xCWNzxWfKT5OXEFcS/d0FZ8rni3+XuOfiVeJDxIvEHcTtxbvLn5MfK74
JPGHeu3TOr7U+d86PtD5+zr+1PlfOqLuf0T3n6H7jxWXET8qbiVuLt5NfJDu
P0d8oniW+GHxBvHKtB6e0P0dxaen+cL66CA+TXyJeK3GHxHfJ/5TYw+IjxXX
Ev8uvl/cWHy4uJ/u/0j8oPhO8TTxDN3zkfhN8Radz9R4M3E98VbxLPGJ4vri
beLZ4ubio8X99fo14pniyeJB4o/FD4nvTZ+Xz9csvT6IH4pe38z3XLxv8Pxn
fRR8fo2fntb/b9QOcSNxTfEf4hniJuIjxGvFK3X8zmt17S+dP6jx43ReW9cm
6/2+Fa8Qv5TW1zRxQ/Fh4i/FZ5ey7Fn93EW8WT/L6DhB43XFZfX6v3X/yeIG
4uvEX1MPxIvEo8Wfi58Wz+X74VzHcPFF4nV6r0nR85d6sUF8i7i8eF/xp6w9
cbk0vz8X3yauKN5ffKnef514rniauJl4/5LX54XiSuIdGm8jPkV8iDjXeDvx
qcxXjf2Yu17tJd5DfHuqL6zPHrp/tXiq+GZxI/Heev1F4gvS/Gd9tErz/XFx
O42v0LW3qad67V7i/ho/X/yZrt+qaxVYs9S/NP6Crn8rPlCv36ZrZ6f5zvrb
Hj3/WQ9VdX6W7v1Yx1ZxFY0XJc935v9xzBfxxeJu4oMLFrXqrrhFqhd36z3q
iKuI62v85GA9oH7VE++p+/skfbieZy++VNxTfIrGDxBfJu4lPl18oPgKcR/x
meIK4iHivuKWzG/xUHE/6pfe71Ad2/nudDTV+H4aH5jq6aZUz47SUZn5lPTl
E12/RFxH9++h+3tpvIN4o67fpfEjxBXF5TW+VdxS3ExcV1xW9/cWdxSfKi4v
vlzcW/w/vf4O3V816clX4jv5HcUHio/U/WV0fw9xO/E3Gp+i8Zrig8RfiyeL
a4griNvr/nN1/6u6PkbcRLyPeIDGu1LP0/N8WePfiYdr/DNdW6jxWZn1v43G
39T4AHFFaov4Xeq0eJjGN+jaAt0/M9XHv6LXI/VoY2G9fgOdYf5QS/X6t8Q/
i0dp/GqdP6X7H8m8Dvm/fsq8HnfV+J96TdO0vn/Q2D3iuuKq4h/F94rriaul
+cL8GZq+7zHiLzT+jPhR6rPO25c8X/EHXTT+rq7dofEJ4qPEu2u8p7i9eKj4
U43PFz+YWb/R39d0XCa+Qjxc509ofIZ4iHi97p8nfiDVy99T/aIertG9z+n4
Wuf/ZhRB1UyNN0j1bRHrO2nS5+LO4neoARofL64t3k2/34XituLDxbuIL0h6
eJ64rfgdvf7azP4qlqxH6HWt5BfQxzbiyuKg8fZJn8qJt7CmxSeIPwqutV/p
/B8d0zX+ncbXaPwtrvE9io9M66O6xkt6vy7is8WHiXcSnyc+J9XrX6LXU5Wk
L+hJo6R3W1I9r5O+35oa31mvP1/cKrPf2bVk/Uffdxb/ofc7XlyHeopWaXyV
3uOPVE+e0rVhab1PEt/K9yd+NtV76i2eAB98i8Y3Ml917TnxfF38JrdWoVkL
xBt1HnO//wLdf6b+v/XiFZm9zS7Rr8XjUOuoeazVOsFatLOOxZk1aS7fs36G
3OvpleDaso3amFmL1qXf7afctaqD3nNM7ppVLdXf0blf87zOa+l8M7+LjuaF
5/syXe+R2UvvFK2FeGq8einaW+PZ+ew8g8fEtwd7g62FvTgeAW++rbB3waOz
tmL0/awx1gqNBLWANYMWoolo1RXB3md7YW3EA6HFRfTaRJO/1PnVafyaYO+4
m45XMntIvMGu0V4Sj4D3K6NjdWYPyNxkjvL8H0jev2a0lqKpaDcajgz9nHut
7ii8tlmzaP2G1Av9kruWUlPLiP/O7S3RWLS1drAXxhPjFasHe1U8K17mgGDv
iKfCS1UM9oZ4RLzSocHeFI+KN6sU7KXx1HjdGqm2UePQ5u25vSEekdpeCvaq
eFa8Y7nkvfBgaEVM3gsPhjbslLwmnhNt2jnVUmoq3mZbbi1Gk/E2W/E00frB
vGQ+4nXwPDvp/Lfc3gSPQj37I7eXwFPoJdmvubUJjdqdWpFby9C03cR/5dY6
NG9X8Z/iZ3ReKXetoeaUxWuUPK+ZzwdF17e3qSOZdWR5sHajJwdH+535yWNW
jq6nzI3Vqb62LNmv0y+g1Wj2HtTj3NoXdMzOrIFoWx6thWgc3nmL3mNKZg/N
9/lB4dpKjeX7fB8NytxzPM7zER/D75bmy4eFeyV6KL7vj8RnZe6hXhB/Kb5c
fLV4obhier8j0/xaJT4+c0/0HM9bPFg8Mvd8WJnmDz0MvfXHhb0Unor5uDrN
31ZpPq8prAVowot8H+IrxGPFy/i+xEPE43L3sr8X7v3paellfyzcW9LT0mv+
VLiXpedE+/EAwzP37C8xV8RDWd/il5kfhXv58bl74Z8Lax09MV4Xz4t36R3c
G/9dOEugR0a7fy2sdWg4WvlLYS1HM8kq/ijc65JZ0Jv/leoRPTq98J+Fe3V6
4ueZ34W93Sg0gnohHigeIV5CfRAPEl/FfBWvF18svjK3920WraUdUv0/Ptrr
43nR2hOivT6aS5bQMLqXoqciSzgi2uvj+ekNmkZ7ZXoEegd6CLT/qGCtbqSj
aWbNpnZSQ5/NXZPR5lo6Dsms0Xj3Y6O9PB4eb98k2pvi8fGqjaO1H89KtnJM
dC9FxkIvRk+GNy2CvctR0b06HgZtrx3tBdB4eqt1hb0j3o4s6J/C2QKZENq8
Vtw/s0aTXXwi7pI5w3gy2rMwP48Pzib+LdwLk1HQG28u3MvSI+M960d7RTwo
WU6D6GyITAcvVifaa+HJ8Jb1or0DHhMvUZfPlNlT0EvSU/L9HRKcTZBR8Dz2
Cs7Gvi2czZCRTWX+B/eCO3L3xvTI9L776/rxhdcsa5U5dELhOcdcYw3TW6Dp
aDk+AL0mA0NPbxSfXLhGUBvwn/gJelzq00SNtyi8hlm71JBjC89Z5ipz+KzC
a461xho8rfAaZ22z5s8ovEZZm6x5Pl+r9P21Dc6iWqf53i55OTwdz3u35AXx
hDzPXdP8QGx5/nsHzycyO+Zz5eDnjYdjvpUP/j7w/MzHA5N+04Mxnzan75ce
g/lbIfj7JbNjPVUJns/0rHjFw4PnOz0r3viI4O8fT8562yd4vTGfmQ8heD2S
2bB+qwavB3o4vGvNtN7wyP952+DssG2qBx2SXyHzwl/dG1x/qyZ9OEbcuHBN
oZbwDKmXHVO9Py+4/nVK9ff84PpKz8F33SW4vnVO9fiC4PpFpkd22TO4HnZJ
9bVrcP0jsyXr7BGsX/1TvRyY6mnvVL/7B9dHMkyyzl7B67VPqucXpfXXN9Xn
AcHrr1+qtxcH60PXtB4uDPaLg1P9vSzYL45J/nFccBbbJtXX9sFZa7ukZx2D
6wcZDVlEC9Zd4ZpLrWVOUj/qJ307JdjPTkh+7oZgPzo01ZsRwXo+LNWPq4L9
5+jkb8cmvzso6dulwX5geKovI5N/vSTpxeXB/nFI0qMrg+tXtaTvDYL9KpkW
/vqe4P6ADBS/e1uwfyXDxM8+Euzr6S1/yOzv8TnoJvqJ38HnL09ejBoxQq+t
Lo2YhXdjzogriq/NndePFNcQz86d3/LdVCq5VvEdkfcfUnKePJYaKm5K/4gX
yDw32A9A25kj48SHa/wxvChzTnyweFLuPHqsuKb4UfH0zM+W/p7egWdMv39Y
yXkyeSPPjv4ZLecZklWSWR6q87L6uRBvpPs/zZ2vkm2ScVbT+R7B2SQ9JL3j
fsFZKD1kVfHu+GnWnl6/Du+ia/PEp4vX4h2Yk+Kq4vvw/pn7+UNL3v8gf2cu
kH/jdZgT5OFVND4ld54/QFxZPFl8A3NCXE08Lff+yUxqG3lJ7nx7Bt6M/EO8
FA3Dy+Fn8X6Z5wJ5Mr0Nc4L9klol5wsPZJ7b7J/QqzDHmUv0s/ROzKmbxEfp
/idz5yk3iGuLF4kfytz/1ik5byH/vFvcgPwHLc6cxx9T8v4M+ze3i+vjv8Xz
mLPieuLn8L7iW8V1xUvwuuI7xEeLl+be32Huk+/TW7IGJoqP1PhitDFz9ksG
XEnnu+jnNWilxh/PnafcJ25MfpW7Xydvb1byfsFL4qnUcvFyvJj4fvGx5FV4
ucx5eZOS+332G+gNyc/pzekRTy25Z6BXeE/Xb4rODz7BS+raY2hVyXk7ecdc
8cniD8RvsGbFJ4nfF78ufljcXLyadSy+MTovWyV+VfwoXkL8Ye78/l60kf4K
r5RZXzeles2eD3sz7NHgr/Lg3p8MgKyuZXC/+nTh/pCMmlrZULwlc81srfsP
iM7eyODIE54s3N+RcdMPP1W4/yPTPlB8bsF/5gyO7IAMgWyXzKy8xs8R78j8
nvRXi8Wl3Hso1aP9PmPcQ/5MJkNWsjZzltgh+r3J+Wro/LfCtS3Xe9wn/lnX
y4kJPehf3yjc37KHwHx5izmUec8AP/pq4WwNP0eeSybN78szoh9+LdUHMib6
02cK+zMyb/bTyCTpHam5laIzTrKY3zJnlR2j6y29Jv3364X9HHs09NNvFu73
2cNBz75Leswe4xzxdr2uunjP4NpAjaB2Dg2u3dRw1uao4NpDDWLuDwuu3dRw
1sro4FpATWBtXRdci6nJrMUJwbWdGk9t6h5ca6m51AL2eFlbrDHW8rXBWSeZ
Zy9xs+BsmYy5p/iE4GybjJts+sTgLJ6MlWz1pOBsEX9DNn90cNZH5sdeRKPg
LJBMkL2GhsFZJZklewFNgmsTNYrac1Nwv8OeCNlr3eB+ij0FsvL6wdk5GTrZ
fvPgLJBMsDvPPzgrJDMk224c3G+RcZLN1wvWRjSSWt0tWE/eKZy/sGdGvvFs
4fnHng77MevxFZn3DPcULyucRWzMnH2wJ8VeFBkIe1OHRWcX5KZoDxr0qXhR
cFZIZrhBvDhYG9AIssM5wdkEGcXn4qeCtQ8N/Ez8ZLB2oWHrxQuDayE1EW18
KLj2UIM+pNYE1yZq1EfUMvxIcJ6yNNVc1hpr7ujcc5S5ypw9KXdmQe9FDzY4
9/4V3gCPgJbODs42yDha584o6H3ogern1nBqJzX0E53PC6591MCPxU8E11Jq
6hpqRXDWQebRPXfPh56/XTg/Y08SrUQz0e5ZwdqERqGdk4PzuqWF8y72QMh3
Xiz8WdmTIY96oXD+xZ4J+dorhX9X9hDIs5YU7m/pv8mnniucp1VN/d3yws+K
z0h+9nJhP8KeBPnW84XzOfZsyNNeKuxHyLSppfum+v3ffln0nhu1mpqNFqAJ
aMd9wVqOpqNtNwdrJxqKVkwK1mo0e5n4rmBtR+PRyluCtRrNxmvcGazNaDTe
bkqwtqKxaNn0YC+AJ0CrpgZrKxqLtk0L1lI0Fe26P9jL4GnQypnB3giPhNcc
nPTqi8L9C94Wr4RnwqsOSuu/nz7/nMweBS+Bp0C770jf76Do+oYH4fP3j66X
eAg+38Bo74En4XldFF0v8UB42/OivSkel/pxZnQ9YA+Oetk52ivjifGyeFq8
YJ/0PAdE1088E8/7kuj6i0fieQ6Orqd4HvZ6z42uR+z5Uu+7Rb8fnpD1elX0
94WHod53j/aK09PzOV98fWZPyfO8ILpe4TnxqnhWvPWQVP97RzOenPpDPSI/
+j13vT4run6yh8r3fUX0fMJToQd9ovUIz4W+9I3WHzwn+nSh+K7MPQZ6cXr0
fgZ7ZOjFadF5FHuS7GW2idYT9jTJjtmDJE97JdUealDZzJkue7Vto/WFPVv2
ZttH6wt7eMy/y6PnOx6Pvd7W0frEni97ta2i9YQ9W/aaz4mu/+w5M58vxfNk
9risr8ui9QbPhb5XSPOjk36fJTq+QEcyZ9FkyXgYvMv8pDctovME9qDQs1Oi
8wb24NiLbxmtd+zJs1fHnl0Pamdwds0eHXknvR177az/8zLvybM3e3a0frJH
y951u2i9Yw8bPT4jWh/Zc8bfsafN3tbrSd86afzKzD0gfov8hvndJrg3o0dj
vYwP3vulHvH7saeOX+gZvX7npPVDj8j87Jv0aHy0XtGToVcTovWHTB29Gxdd
n+m5qO+jo+sHnhl9GhldX/DI9HvvFtYa/kaFejIsev3TI9HfvVd4v4S/QcFL
48+WZ/bU6OHY6PpDT4e+jImuZ3hs/MmJ0X+LQD5PXvN94XyDv/kib/ihcL7M
33iR160ovJ/D37TgX5rrWsvMe5r4i5Oj8yD2JNG3q6PrIT0C/cTE6F6BPVJ6
Cfwvf3tBT0FvQo8CP5j80knReRJ7pPiDa6L1lT1F6u2Q6Hr+WtK/4dH1nJ4R
Pb4yWv/o2dH7UdH6QU+CvxgRXe/pQfGTPaLrL5nA/wE02Pfq
              "]], Polygon3DBox[CompressedData["
1:eJwt13f8jWUbAPDzO+d3FKVhZIVklYSIzLSUESGjjIxQisiMSEZGNChNpayW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              "]], 
             Polygon3DBox[{{821, 987, 581, 702, 1158, 820}, {835, 993, 588, 
              706, 1160, 834}, {1045, 1111, 600, 397, 396, 
              709}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{275, 494, 1165, 276, 507, 302, 1264, 522, 317, 1269, 
            537, 332, 1276, 552, 347, 1286, 567, 362, 1297, 582, 1208, 377, 
            595, 1220, 391, 1309, 608, 406, 1313, 623, 421, 1318, 638, 436, 
            1325, 653, 451, 1334, 668, 466, 1345, 683, 481}], 
           Line3DBox[{277, 495, 1166, 278, 508, 1176, 303, 523, 318, 1270, 
            538, 333, 1277, 553, 348, 1287, 568, 363, 1298, 583, 1209, 378, 
            596, 1221, 392, 609, 1230, 407, 624, 422, 1319, 639, 437, 1326, 
            654, 452, 1335, 669, 467, 1346, 684, 482}], 
           Line3DBox[{279, 496, 1167, 280, 509, 1177, 304, 524, 1187, 319, 
            539, 334, 1278, 554, 349, 1288, 569, 364, 1299, 584, 1210, 379, 
            597, 1222, 393, 610, 1231, 408, 625, 1240, 423, 640, 438, 1327, 
            655, 453, 1336, 670, 468, 1347, 685, 483}], 
           Line3DBox[{281, 497, 1168, 282, 510, 1178, 305, 525, 1188, 320, 
            540, 1195, 335, 555, 350, 1289, 570, 365, 1300, 585, 1211, 380, 
            598, 1223, 394, 611, 1232, 409, 626, 1241, 424, 641, 1248, 439, 
            656, 454, 1337, 671, 469, 1348, 686, 484}], 
           Line3DBox[{283, 498, 1169, 284, 511, 1179, 306, 526, 1189, 321, 
            541, 1196, 336, 556, 1201, 351, 571, 366, 1301, 586, 1212, 381, 
            599, 1224, 395, 612, 1233, 410, 627, 1242, 425, 642, 1249, 440, 
            657, 1254, 455, 672, 470, 1349, 687, 485}], 
           Line3DBox[{285, 696, 698, 286, 512, 1180, 307, 527, 1190, 322, 542,
             1197, 337, 557, 1202, 352, 572, 1205, 367, 703, 705, 1213, 729, 
            707, 1360, 709, 396, 613, 1234, 411, 628, 1243, 426, 643, 1250, 
            441, 658, 1255, 456, 673, 1258, 471, 1350, 718, 720, 730}], 
           Line3DBox[{289, 501, 1171, 290, 515, 1181, 310, 1267, 530, 325, 
            1273, 545, 340, 1281, 560, 355, 1292, 575, 370, 1303, 589, 1214, 
            384, 602, 1225, 399, 616, 414, 1316, 631, 429, 1322, 646, 444, 
            1330, 661, 459, 1340, 676, 474, 1352, 690, 488}], 
           Line3DBox[{291, 502, 1172, 292, 516, 1182, 311, 531, 326, 1274, 
            546, 341, 1282, 561, 356, 1293, 576, 371, 1304, 590, 1215, 385, 
            603, 1226, 400, 617, 1235, 415, 632, 430, 1323, 647, 445, 1331, 
            662, 460, 1341, 677, 475, 1353, 691, 489}], 
           Line3DBox[{293, 503, 1173, 294, 517, 1183, 312, 532, 1191, 327, 
            547, 342, 1283, 562, 357, 1294, 577, 372, 1305, 591, 1216, 386, 
            604, 1227, 401, 618, 1236, 416, 633, 1244, 431, 648, 446, 1332, 
            663, 461, 1342, 678, 476, 1354, 692, 490}], 
           Line3DBox[{295, 504, 1174, 296, 518, 1184, 313, 533, 1192, 328, 
            548, 1198, 343, 1284, 563, 358, 1295, 578, 373, 1306, 592, 1217, 
            387, 605, 1228, 402, 619, 1237, 417, 634, 1245, 432, 649, 1251, 
            447, 664, 462, 1343, 679, 477, 1355, 693, 491}], 
           Line3DBox[{297, 505, 1175, 298, 519, 1185, 314, 534, 1193, 329, 
            549, 1199, 344, 564, 1203, 359, 579, 374, 1307, 593, 1218, 388, 
            606, 1229, 403, 620, 1238, 418, 635, 1246, 433, 650, 1252, 448, 
            665, 1256, 463, 680, 478, 1356, 694, 492}], 
           Line3DBox[{299, 700, 1358, 701, 300, 520, 1186, 315, 535, 1194, 
            330, 550, 1200, 345, 565, 1204, 360, 580, 1206, 375, 711, 1361, 
            712, 1219, 389, 715, 1363, 716, 404, 621, 1239, 419, 636, 1247, 
            434, 651, 1253, 449, 666, 1257, 464, 681, 1259, 479, 722, 723, 
            724}], Line3DBox[{480, 682, 717, 1344, 465, 667, 1333, 450, 652, 
            1324, 435, 637, 1317, 420, 622, 1312, 405, 607, 1308, 390, 594, 
            1362, 713, 376, 1207, 581, 702, 1296, 361, 566, 1285, 346, 551, 
            1275, 331, 536, 1268, 316, 521, 1263, 301, 506, 1260, 274, 493, 
            695, 725}], 
           Line3DBox[{486, 688, 1364, 719, 472, 674, 1338, 457, 659, 1328, 
            442, 644, 1320, 427, 629, 1314, 412, 614, 1310, 397, 600, 1359, 
            714, 708, 382, 587, 728, 1365, 704, 368, 573, 1290, 353, 558, 
            1279, 338, 543, 1271, 323, 528, 1265, 308, 513, 1261, 287, 499, 
            1357, 697, 726}], 
           Line3DBox[{487, 689, 721, 1351, 473, 675, 1339, 458, 660, 1329, 
            443, 645, 1321, 428, 630, 1315, 413, 615, 1311, 398, 601, 710, 
            383, 1366, 588, 706, 1302, 369, 574, 1291, 354, 559, 1280, 339, 
            544, 1272, 324, 529, 1266, 309, 514, 1262, 288, 1170, 500, 699, 
            727}]}, {
           Line3DBox[{732, 1036, 493, 731, 1165, 949, 733, 1166, 950, 734, 
            1167, 951, 735, 1168, 952, 736, 1169, 953, 737, 698, 1063, 1357, 
            738, 1037, 1170, 954, 739, 1171, 955, 740, 1172, 956, 741, 1173, 
            957, 742, 1174, 958, 743, 1175, 959, 744, 1358, 1038, 960, 1039}],
            Line3DBox[{746, 1064, 1260, 745, 507, 747, 1176, 961, 748, 1177, 
            962, 749, 1178, 963, 750, 1179, 964, 751, 1180, 1065, 1261, 752, 
            1066, 1262, 753, 1181, 965, 754, 1182, 966, 755, 1183, 967, 756, 
            1184, 968, 757, 1185, 969, 758, 1186, 970, 759}], 
           Line3DBox[{761, 1067, 1263, 760, 1068, 1264, 762, 523, 763, 1187, 
            971, 764, 1188, 972, 765, 1189, 973, 766, 1190, 1069, 1265, 767, 
            1070, 1266, 768, 1071, 1267, 769, 531, 770, 1191, 974, 771, 1192, 
            975, 772, 1193, 976, 773, 1194, 977, 774}], 
           Line3DBox[{776, 1072, 1268, 775, 1073, 1269, 777, 1074, 1270, 778, 
            539, 779, 1195, 978, 780, 1196, 979, 781, 1197, 1075, 1271, 782, 
            1076, 1272, 783, 1077, 1273, 784, 1078, 1274, 785, 547, 786, 1198,
             980, 787, 1199, 981, 788, 1200, 982, 789}], 
           Line3DBox[{791, 1079, 1275, 790, 1080, 1276, 792, 1081, 1277, 793, 
            1082, 1278, 794, 555, 795, 1201, 983, 796, 1202, 1083, 1279, 797, 
            1084, 1280, 798, 1085, 1281, 799, 1086, 1282, 800, 1087, 1283, 
            801, 1088, 1284, 802, 1203, 984, 803, 1204, 985, 804}], 
           Line3DBox[{806, 1089, 1285, 805, 1090, 1286, 807, 1091, 1287, 808, 
            1092, 1288, 809, 1093, 1289, 810, 571, 811, 1205, 1094, 1290, 812,
             1095, 1291, 813, 1096, 1292, 814, 1097, 1293, 815, 1098, 1294, 
            816, 1099, 1295, 817, 579, 818, 1206, 986, 819}], 
           Line3DBox[{822, 1041, 1207, 987, 821, 1208, 988, 824, 1209, 989, 
            826, 1210, 990, 828, 1211, 991, 830, 1212, 992, 832, 1159, 1213, 
            1059, 587, 1058, 1044, 1060, 1366, 993, 835, 1214, 994, 837, 1215,
             995, 839, 1216, 996, 841, 1217, 997, 843, 1218, 998, 845, 1161, 
            1219, 999, 1061}], 
           Line3DBox[{846, 1048, 1047, 1361, 844, 1307, 1109, 842, 1306, 1108,
             840, 1305, 1107, 838, 1304, 1106, 836, 1303, 1105, 834, 1160, 
            1302, 1043, 833, 1365, 1042, 703, 831, 1301, 1104, 829, 1300, 
            1103, 827, 1299, 1102, 825, 1298, 1101, 823, 1297, 1100, 820, 
            1158, 1296, 1040, 1057}], 
           Line3DBox[{848, 1049, 1110, 1362, 847, 1220, 1000, 849, 1221, 1001,
             850, 1222, 1002, 851, 1223, 1003, 852, 1224, 1004, 853, 1360, 
            1045, 1111, 1359, 1050, 1046, 601, 854, 1225, 1005, 855, 1226, 
            1006, 856, 1227, 1007, 857, 1228, 1008, 858, 1229, 1009, 859, 
            1363, 1051, 1010, 1062}], 
           Line3DBox[{861, 1112, 1308, 860, 1113, 1309, 862, 1230, 1011, 863, 
            1231, 1012, 864, 1232, 1013, 865, 1233, 1014, 866, 1234, 1114, 
            1310, 867, 1115, 1311, 868, 616, 869, 1235, 1015, 870, 1236, 1016,
             871, 1237, 1017, 872, 1238, 1018, 873, 1239, 1019, 874}], 
           Line3DBox[{876, 1116, 1312, 875, 1117, 1313, 877, 624, 878, 1240, 
            1020, 879, 1241, 1021, 880, 1242, 1022, 881, 1243, 1118, 1314, 
            882, 1119, 1315, 883, 1120, 1316, 884, 632, 885, 1244, 1023, 886, 
            1245, 1024, 887, 1246, 1025, 888, 1247, 1026, 889}], 
           Line3DBox[{891, 1121, 1317, 890, 1122, 1318, 892, 1123, 1319, 893, 
            640, 894, 1248, 1027, 895, 1249, 1028, 896, 1250, 1124, 1320, 897,
             1125, 1321, 898, 1126, 1322, 899, 1127, 1323, 900, 648, 901, 
            1251, 1029, 902, 1252, 1030, 903, 1253, 1031, 904}], 
           Line3DBox[{906, 1128, 1324, 905, 1129, 1325, 907, 1130, 1326, 908, 
            1131, 1327, 909, 656, 910, 1254, 1032, 911, 1255, 1132, 1328, 912,
             1133, 1329, 913, 1134, 1330, 914, 1135, 1331, 915, 1136, 1332, 
            916, 664, 917, 1256, 1033, 918, 1257, 1034, 919}], 
           Line3DBox[{921, 1137, 1333, 920, 1138, 1334, 922, 1139, 1335, 923, 
            1140, 1336, 924, 1141, 1337, 925, 672, 926, 1258, 1142, 1338, 927,
             1143, 1339, 928, 1144, 1340, 929, 1145, 1341, 930, 1146, 1342, 
            931, 1147, 1343, 932, 680, 933, 1259, 1035, 934}], 
           Line3DBox[{948, 1056, 722, 947, 1356, 1157, 946, 1355, 1156, 945, 
            1354, 1155, 944, 1353, 1154, 943, 1352, 1153, 942, 1164, 1351, 
            1055, 941, 1364, 1054, 1350, 1163, 940, 1349, 1152, 939, 1348, 
            1151, 938, 1347, 1150, 937, 1346, 1149, 936, 1345, 1148, 935, 
            1162, 1344, 1052, 1053}]}}},
        VertexNormals->CompressedData["
1:eJx1XGdYFUnTRTGAilnMWcyKWVS0TBjXuItZBFFUzDkrq5gT5oRZBCPmiJYo
KoqIJMlwLzdwCabFDOp3X7tq9rG/nf0zz87O9q2u6a465/QZao+fOXRiQQsL
i2GFLSwszdeU3FODnJ0NQFdMcw1d5ewcC+Psp2W0XaSDHu1/bMm8bsSz5WeG
ZF5/Bc7PDQHTOmqhdI2AmvtqmHBsiYUj99WIh/Tefxun1UoBY41Lfccty0RX
t8drxy1LgHpT/Br+aXoFFoNtR12/lYWxDeoduH4rERYF3qnX4FIEbOxQr2vB
O9n46e6BVQXvJMGmi56xTTvchw9Xl8wvOSEHZ7ivvVVyQjJY0D8pZ4retDuS
g+u/Hs23O5LMz6P8PI2P8vgUD8rxUPwox0/zRXm+lB+U88N5lPNJzyn/Hz//
198xwR+z9HzFRxH9es+bFA/X0tp2jbyUDqcgsXLeDCMWbNyhlV3TRCi5YVZG
8XdpEOax67XVhwyslL7sfLF/kqCr/eHJ1VwToNuvayY2PD3c9vmBFBiUWqm7
w7VIWDjAzj1mWxZO6PD+bp3vqXDd8KAKdH4AO0bUC0gdnY1fBjnfaZ6dpuT5
gHbR7nIns7FUt1edr9fU8PMoP0/jozw+xYNyPBQ/yvHTfFGeL+dFzg/nXc4n
vSflvfH7onGV3+HxL4x5VGxRbz1flfXTAEcnz92ohX8uzIku/dqAH7Wtive/
nAq2rpeqzuuUDHeX3AvWrM3AxQn90i0XaWD9oL9mfXkdDVPdp2xekGnCNW4V
NneYpYUmWT/mO10OgesnV0S/Ss3EMs6n/smvmq7k+fNQvyd27bMw2sc5Mnxk
Oj+P8vM0PsrjUzwox0Pxoxw/z1OeL+ddzg+vezmftE+UfcP7hd6r8p75/VIc
SlwczxL/crfzO+v4ik3uNenf+ZUGPj1tusF1fhp8TapeJPuLHmMoP36Fh06r
ufEVXY04Z1+JC0k3ddDrtUO5F2ee0DUDO5eYt2reNb2S54tbJh+pWsGEW/3z
xy4qa+DnUX6exkd5fIoH5Xg4bjl+zrs8X173cn647sj5pDql1C2uV7SvlH3G
+4vWgbIueD1Q3Mo8OH5/419Fxs7T8hWHNEkeX3GsHr7lTXIqtzgB8vJt5r2o
osND5+qXfd/XCO+eWbcOrxMGbUb0NlYepcewMtsfzb2XoeRZM9lzSJ9oPXZc
0X5/TH8TP4/y8zQ+yuNzHHI8nHc5fl738ny57sj54bov55P6hNI3uF9QXVPq
HNc32ofKvuT9SOtGWUe8fmieyrx5vn/atMl9PDuZr7ht47zBW3UmsK47t9vB
yBd0TcVlc42VJ3/OUvLcrmb3QXfvp+HNUu8z/EKz+XmUn+dx5fE573I8vO7l
+LnuyPPlui/nh/uunE/q00rf5vpDfUXpM9xfqA4qdZHrIe1bZR/z/qV1pqw7
Xm+UFyVPnJ+hK9PqdPzwkq8YN6puIY+zr5U8vyhn8bjo7kjMiO6iDw95ozwn
P895l8fndS/Hw3VHjp/rvjxf7rtyfhj3yPkknKTMg/ES9XXlPvd36kPKfe5H
VDeV+1w/aZ8r93m/07pU7vP6pDwq9zmf9K9eFr/9846fR/l5Xvfy+Fx35Hi4
7svxc9+V58u4R84P4045n9J6BpX1DCrrGVTWM6isZ1BZz6CynkFaz178HlTW
M6isZ1BZz6CynkFlPYPKegaV9azkU6rPoFKfQaU+g0p9BpX6DCr1GVTqM0j1
2Yv3gUp9BpX6DCr1GVTqM6jUZ1Cpz6BSn5V8SngDVPAGqOANUMEboII3QAVv
gAreAAlveHEdUsEboII3QAVvgAreABW8ASp4A1TwhpJPCT+DCn4GFfwMKvgZ
VPAzqOBnUMHPIOFnL+4DKvgZVPAzqOBnUMHPoIKfQQU/gwp+VvIp8UFQ4YOg
wgdBhQ+CCh8EFT4IKnwQJD7oxX1YhQ+CCh8EFT4IKnwQVPggqPBBUOGDSj4l
fQNU9A1Q0TdARd8AFX0DVPQNUNE3QNI3vBgHqegboKJvgIq+ASr6BqjoG6Ci
b4CKvgGSvqRcVfQ6UNHrQEWvAxW9DlT0OlDR60DS67wYh6rodaCi14GKXgcq
eh2o6HWgotfJ+UQ5n1389a9X2Bn4iht9G7pkP3oFuZ5Omw4WMkJ6UYPdj9l6
XPam2P2DhV6B7bKXDVf/cQn8ng6t41wtB++75kUYnFL5Psr3eVyV8VEen/5/
ZTweZ8Vmq8yZOcF8xTk98uuEjH2r/LvKfeWq8rxyn+OWf1eKB/g+v/fsx0sa
VKryAO6dttx+5vz/GwekPChXlTyDSp5BJc/y+CiPz/nl3+Hx+ff4PfD9L3OH
tZw29CLk9hs+rLNPDt5OfX6gs08y30f5vjS+Mg49p/x//DzzCsobct44j/zf
pfso3+c45PGl3wXpPsr3pTyASh5AJQ//jv/7elbGOTzkY/zuLga+4sGn7kZL
K+U+qtxXrirPK/fLHbKd5H3OAM3X9Kgy2qjHHj+tDnqfi1X+nf67fF/5//g+
P8f/XeV5VHlevg8q46D8ux9/HBnWur4BMpqZBvx1T4/pn/W7xreOB9+LG4b/
fK6Fp+N8jAmhGZj/fHvNxNA4sM5wfXurhw6eGmfPqlXWiGXmGe+/xAToeenK
+SsN0+DUzGOBNSwysYBhc3YNiwQYf32V9n1BLVRxmvVxzJQMdD65uFj3wUkw
fVPn1ZE2iWBx5OE673pZ+Ek79bF3vUS41G/58GZXk+HZ5xuuy66a8Ebcbtft
UcnwxjH8jz39YiBsOJasZZeNR4++GFTLLgkSL2jmfzfjzIKNNzXeY+Z/DWue
qDeqeSrcLRUQ3jclDEqU7lln+ttsrNHOYeL0t0lwZC28h2YRsNT1+7kvL7Iw
bX1o+fBeaWDK2xFYeXoQ2Fr/WfjV7BzsdG9l31dmnqTrNNdqyAeELvlPXIKm
ZWOtm/tKJ//g/r7PUT6/iBPPo/z8DREPcjzVKZ4DIh6U48kR80We7zGab5yY
L/J8G9F8p4p8IufzM+UzUOQTwyifNymf3cX7QpX3hVXpfQ2j90XrAXk9fKf1
YCXWA/J6KEvr4fL2AGfXGzp80rd6csNNRvzS165so02veL2hidabjtbbZ7cO
TV+cMkLKlA6Hpnmno/XBOQP25SQAxtW2mh2rB58i8V6fDTrc3aD+5y1JSZBZ
9fWBI3vToemxyePthxkw3/mzZ6m0FPh5HNccuJEGu5+lLCv+yIjBddvnndak
wZHDWws4NkuAa7cmWdlUMWHyzOze1mkayLCysHu2IxJyL0/Jb9E2E+OgeUlH
Py30CBm86VydB9Au8nT9ObmZ+P6WxfkpjdOhAL33L6TrML9wFM+j/LxBjI8f
aPx4Gv+QiAc5nhSK57uIH/dQ/A8ofpOYLzaj+X6n+VJ+kPOzh/JD+UQ5n7Ny
D7Us0MAEVWcv2XNoXxoWbPXGbqNfEvj73t/Tc64R6vdKqtOtkRaXuH7r2coq
FSpmLer+qZceHvi6tnE/lo4fzxg/tBuggeErelWyWKuFx5an//Spr8eNzRfM
vXdMC62z71Zu2TIZKpcb/fPOYQMWSXt8Tp+VDrPel93lnh0N81z8GujTjegW
GBi1rogeZg/JqbgUQ+D9j62ndz7MwK8DW3vUDtNTno86XiJdbRvxwSnieZSf
ny7Gx/nS+C1EPFhFimeYiB+fUPybKH6aL8rzpfygnB/KJ3I+LSmfmwzJHdps
yYQm2cUf/yibgFVvNm4W0zUFJo7z6h5VwAQdvRyc325IwqX1N09fUFoDdUo/
2jvokwE+PwyZa+qXimV7pE+/fFYLjW8ssXHuroNBQzL6HzmmwUIlShXtOEkH
P/qf/jlqcRpMfTrxSvE66Zhub+F9s4EBLHxeWut2voJJ300n1izT4anJN+a/
v2OESJclfofDn8DE4iOw7wI9Nrau0WNYGusbRx3l85RQ8Tx6SM/nbfs1Psrj
fxPx4DQpnoYifuT4C1P8tcV8UZ4v5Qc7SfnZKPKJTSmf1Sif29omVbC8kQXp
JzRnXiyJxI7RIz9k7kmFbfPa7zw1PBO2lyy5+g/3aMyp6FxV90kDBzre2Pb3
ggx46R9YdtXeWNx169j37fp0aLneJkuTpoefj53nlC4Tj0/urd7zpqQBznS7
ezB0mBbcVnxcbH0xEW2OXVhlNSoD/I6sndfsjwQof6zMkgeNU3D/6G+TPNdk
wu67N+Jv2YSBfyvvDVWs0zDv4ZTFnxtk03o+6diedOVbpNdtFM+j/PwxMT5W
kMY/LeJBOR57ET9aPBHxh1L8NF+MlOa7VeQH5fxQPpHz2YnyOflBSKFtK7Jh
WrOs2Ojvwfjc5mWB1svSoFyfQ182dM8C114uVTwrheCCau0LjN2thTl9jDNK
rzNBauLRzQG7H2OTkClnch/ooOKtWF12aSPUqTh0rNfCpzitfel1Pn2M4OQY
M6bUznSot9Fi91/Hn+PBftO7FCuZCQHLJ4DniiR41kz/sWxwBFb2wYR35rwV
GdF1eten4TBzTZ0hCytH4sFXVytsz2H9OcBRPk/JG/7reZxNz/vS8yfE+CiP
30PEg3YUjy/FU0HEj3YU/3SKf7aYL6ZI86X8oJwfyidyPsMpn7lXd6WVffTv
uUntUkVOO4anwbFMY3bQnn/PTaKW7izZ+oW53s6KtD2Y9e+5ycMrPay7l9eD
5u7Hqd9XGZX7VYt6P6l0zwi163Y+crGETrlftlSNxQOvZ8LsoJej24/9lx9Y
dgy/vCEwR8mnxW//vIPYb+v2aXNfKLzgnf+f1TQl3sI0MY5yvxCNU0v8rnK/
PP1uqohTuV+d4qR5Kfd5XpQH5T7ngfKm3Oe8SesWVNYtqKxbUFm3oLJuQWXd
gsq6BZV1CyrrFlTWLaisW1BZt6CybkFl3YLKugVet1IdBpU6DCp1GFTqMKjU
YVCpw6BSh0GlDoNKHQaVOgwqdRhU6jCo1GFQqcOgUoeB67CEK0AFV4AKrgAV
XAEquAJUcAWo4ApQwRWggitABVeACq4AFVwBKrgCVHAFqOAKYFwh4WRQwcmg
gpNBBSeDCk4GFZwMKjgZVHAyqOBkUMHJoIKTQQUngwpOBhWcDCo4GRgnS7wP
VHgfqPA+UOF9oML7QIX3gQrvAxXeByq8D1R4H6jwPlDhfaDC+0CF94EK71Py
Kek8wDqPpOcA6zmSbgOs20j6DLA+I+kwwDqMpLcA6xuSrgIqugqo6CrK85J+
oowv6STAOomkhwDrIZLuAax7SPoGsL4h6RjAOsYVoXsA6R7wmXQPSU8D1tMk
3QxYh5H0MWB9TNLBgHUwSe8C1rs4n/L5SKrQu5D0LmC9S9KpgHUqSY8C1qMk
3QlYd5L0JWVeko4ErCPtfNcm68tXPV8xr4JLqtfPODhb4apnMUM61BhYZ4TR
jBN7pF+8bTkkAUwVPfosnKuBAR3X9rpW1IT9JhXqkLA2EZadtuxr6JUEOKtW
venzMlF7umVwnx1JUMrPy6N2fgxU/RL5NO1yFpbtuuis55hk6Ox/eMp1l+fm
fTUk0ut8NgYPz90eHJmsqu+t3HJxQ8uriHuS21318snG8FrfWxRdlwaNJ78c
7Hz5BeaEFt500zYbY+M7TNN7psKMhY4389JicXHJo5dujDDj1qTjOad/JoM2
7XK5i95JWH/1sbOrp2fihTdPug6YkQS7vRcGuTXS4C3vvd+dPU1Y5HRs+tio
BHBKq7zzfot0tPji2jvUJQM1dR0mTOwSDx4nJ6yuvNi8b633tdkWlo7rz/0x
MfReApya7dP0xBw9X/HupbCIvElJcNnucSTma+Hvu3Gt69QxYqOOxT5cLJwC
fSc2vlm4ZipYnf/07lFIBkbtql126pJUyPV6MvOPcXGwP9dqzeQumTj3UkPn
WcfTYMOcYYEfwiLALctT4z0rCy13PtW/aqaBkrpNNUZ+RvC9d/0v/27ZGJp6
KnBwH42qbnbz3bmv7vUeYMDSFgn762bhP9Z/Rt/Va8Fi34XggX0jcYdDo+xN
vpm4p6hNxP5aWpiy92KxDhPiMdv3jOHhIxM2S93lPKepBlYkbmztNSYV7y/8
084+OAP7zgvpOrdPKoxfuGn1/FNahEMFl14MMOLX6oVOT2pvXv+rVw0ufEGH
uxKsSsVuMWD0csczYzonQtqtxpvcRxhwycwdn3Yv1WO55o0eD/GIB8/uk5ZW
yMuAWI/Rm5bV1uDtjpYb9y5OAssbtedZljHCKSxoXzlOiw797jeevDsFLp6f
Wiz8lI6vWKt+oe4Rt9Jg06NJurmnNTD3zYdDKUsN2LOMW9s5Gg30Xjtpk3XD
RKhyYN3wJW0zsPuSi5lbbmthWfrtVvrcSNj+6E/HcWdMODEsICZmQDq8f1i4
r6/dAxhQov2Qrlcy0e/joh6Hvf/tU7JuZv9uinPMgxB0nJOX8qq4CUOH5X9o
vlAPS460nOa0Oxp7uVWOGrU0A1M+/lEuZrIOLp3+nDqqXxL2rm/pZrnLiEEd
A3buq5YO/7g8OTP+kQa31svrut/LgB2teo87fE8DZ18v29DSSYc/UwZXqzhO
j9NLlR9ZLTYV3GtP2HKyuwHvXgkZWtuM15ZHXGz23rzv3iWZXN7FG9Ht3Zdj
7dukY8uKc4b51UyE6atsngz3yATnZjtSDIkJiAfHdbGtkgJbl7QLPOpvxvvB
qfeG2aegfbkuukRzvy8aHXv/3hgDrBlpu+/TDA0eDv50w7GrFg4mJPUvOCGd
r2idveeV38l02Nj4xNXwgSlwalaHhwOH6dFYJz61iV4HGz3cb+wbFwNuRW99
nVLOiLXOexb564UehtQd3trlUQhMOjF4+BO3DLw0zPbIKgeDgru0km4WcfuK
7YiYJ+g/vFC5c3o9Zs3Istm4IgMaXfq2w7fNK8x1+RbdzVWPk4vGBf6wMkL0
++ge1Vaa1+31jxbr/9ahJv5NyGcnPRw4GN7Of3E6+tI1362Cu/kKPcP6VNd+
0OOLuNOzQydrsXZWyLBvoRq40jw92jLFiDurTDfud9PgtOyJ9tZrU2Hkt+It
2w8zYZnIMSm2SanY/HzNP2Jjk+Bkt1ZX7PdnQZ0oh27zXKKwgJOP9YzFqdD+
xWvvS6UyYZJVq5lhQ2Ixrs/y7j3CNZCN7ZfbfTJCx4yNEdrb8dj27deWjpvS
4arDcO28qnpYNqN6DwefZGxeqdrL6+56OOr2aZ9uiIav2NnZa8GcxkY4uvP8
wyK2cVDoW+SGeX3S8VH7xT1L7M2A0gEPt7okPIGP9lbXh2p1OPHDogPtR5uU
PHeUdDMLp/ASBauG4ccqVfUjm2gw4HyhkjUfZUGWb/WEH+/isWnMgd5poWnY
7lvYlCGRJuhf12rB3TQNrmyQ1fjc+lSs0evzcv+VRqjplru8eEM9/uP6zdfq
QzLa+Re8m9paD71o3V+lfcDrv+OnkxmPXEw4xtDjxq2LCTggd96Ymub69qJd
74Y3r2ViK78Pa6Ic4nHa4yvFN3umwKWGc+KvTM+GJqsvrLN59xCv9PYb/nmq
Gf82dbyeWDELpu4curLHtFBc67TmUYvlWtAGWvTya2cCjz2Xb/3xKhztVlw8
krVJB4WCPfcmbjFA6yt5/tXtotD16IhnIXEGmNkk8vS3OC2svuU0q0xiLPrE
ZRYdZP7/R43MfzwoNIGveGvfuEfef2RBt9BtRe3rhsGOsnPc3UamYHz2/g3H
DrHedNLxJelmRtIftvo/GRmsCceYL5cDD76NwiPTSlb6fuI12NU6NuBijyS8
6D3wzfLF0Xjj9NetF3tkg91y97j+XdNxj13Tkd1HR+PVhBKH7WeZYPTrsICy
OwzY5vlul4Uvo/BQEcfi7QMN0M5+SlO/1xno5/j8e3DZKLzaZtSThx11sJ3q
viX1Aa7/z+eG2AXEZOFlW73fsvUvcYt7x9yn11LhaWrGsU+PssFl/cSShoHb
8cCG+JGej9Igp8WEQee2Z8HPHQ+q2Pc9jk/Pl7rlYsZhRzL3ty2eYoL1HXs2
Sm8eiJ7lrhxNKKGHaf3m7hvhYYRyg77WOhxzA7t2uBg/McAIpQ/3ftMoMR2C
Fobv7lkzGFue2u5b1SsTii1Obrg5Igl8R5Sv8XXsE1y8MMYnz0HWzQKCWDdz
vPEoodDnF/hj+IISWfnH8I/RfncWFn0LNU4H3g+ekIx/vuz2VN/wEq590zih
6v4cOHM1PemnhQ5rjNm2sfm4G3h/nds6t12ZoF3Xf2awhxG93nx1nDDmDt7q
3GZGoX1GONA8Kyog2IR3tnRJO3rvLtZ7u2nWyTc6+NzbLarGxCwsc7Cfe/16
iB0NJ95/OKMFwjGwQeAaYDxDz4OVeB74eRofAsX4UIfG14l4YIaIB25SPOdE
/FBMxA9I8dcR84UuYr7gTfPlvIVJOtjPznVivxZ4gXSFQT2Xjj7T4g0UEfnH
nSL/sJTyX0q8L7wq3he0ovc1VbxfLCTeL3Sn90vrAeeL9QC8Hmj94BuxfoDX
D61DCBTrEHgdEl6BAmLdAq/b9mKdwwmxzuEKrfMxYl+AvdgXwPuigdhHsEPs
I7hC+6ix2HdwVuw74H13QOxTiBD7FHifFqR8sk/4NtXP9qIO4FZRByCB6sBw
qhd0hTtUN6jOoJeoM7CD6gzVJWwh6hK4UV2iOobjRR0DrmMFRd1DD1H3YB3V
PaqT2FDUSbhKdZLqKtiLugpTqa52EnUYRok6DFyHCa8wfgGu27VFnYfXos5D
ParzA0RfgKWiLwD3hbeij0BD0UeA+0gx0XcgV/QdOE19h+tnmqR3WYm+hh9E
XwMP6mu+og+ipeiD8Jj6IPdL7p9AfZP6LC4RfRbsqc9SX0YH0ZeB+zL1cfQQ
fRy4j1Pfx9qi70NB6vujBE4AwgnAOOGqwBWwQ+AKYFzhJHAIhAscAoxDCK8w
fgHGLXEC58A9gXOAcU5zgYvMuPIXLgLGRXECR4GfwFGQTThKTe/qK3Aaegic
BpcJp60VuA7HCVwHtQnXbRA4EP0EDoQMwoEHCC/SFYoTbiScid4CZ8IRwplb
BC7FEwKXAuNSwrH4l8CxcJ9w7HuBe8FV4F5g3DtB4GQIEjgZlhFOPidwNfwQ
uBoYV+cKHA5bBA4HxuGEV8CJ9gGv/xUC54OTwPnAOL+94AVAvACeEi/gPMt6
V5bgHUi8A04R71gieAoSTwEP4ilOgtcg8RroQbyGeBDOETwInIgHBRJfoivU
Id5EPAuJZ0EH4lnEyzBG8DK4Q7yMeBwsFjwOmMelCd4HOwXvgxjife6CJwLx
RGCeuErwSkDBK6Ef8cppgoea8eYvHgrNiYcSXmH8otT/+4LnAvFc+MA8V0Xv
KiB4NB4UPNqMbwSPXiN4NxLvhkLEu98Jno7E02Ee8fTegtcj8XqIIV5/SegA
uFLoANCYdADWC1g/QNINSGdA0hmAdQaH/c8Cvxt0kNPy/bwhzkbwrWSzf/Hd
V9BL6BXw8/MvvQJYryB9A24KfQNY39AJPQTshB4CgaSHzBH6CZB+AgVJP2kh
9BYgvQVekd4i610bSO/62t3z1T9ng/Dlz8wzds458KKXc/i+tGRwEPoP5gv9
B1j/sRF6EZJeBOVIL1oi9CW8J/Ql0JG+lCH0KCQ9CliPOiP0KyT9CnqSfiXl
DTlvhXr8ihMiRZzIcUrzQp7XFsJtu0mPekH4TZqvMg79LvLvHqTfZd1N1t8k
fKjoXdZdBA6jKw4kPCbhN+X8vbfAvRBOuLc/4V4J1yHjOul54OclXO3IuFqK
BzgeCW8D423O1w6aF+eNeAR2Jx7hSzxC4hfA/KIAxZFD/lkkf6w0DvA40u8C
/66kfwLnX3q/yvNqdUNaP8o6l3RLBedL7135XWmdAK8TydcH7OuTdG9k3Vte
tyr6s+K3lPKJnE+t8Bso/oS35DeQfJuKjs3nHKzT83kHx03xKnHyeQmfn/C5
CZ9X8fkVnwPy+Sudxyr+Lj6v5fNbPgfn83I+P2cfAs+H/QvsW2BfiPw8+3Lk
8dlnJsfDvjQ5fvYFyvNlH7CcNz7/oPMixffLOj7r+nyuwedSfE7F53d8Lsjn
hHw+yOfWfE7L57N8Lk4+AsWnxL4E9imwP4HzyT4R9oew/0Z+nv1P8vjsz5Pj
Yf+fHD/7L+X5si9WJW8o543P2/g8is/d+ByFzosUvy6fg9K5qOKD5XNZPqfl
81k+FydfgOK7Yx8G+xTYn8A+GPLFKL4mzjP7dNifw34v+Xn228njs/9Pjof9
l3L87PeV58v+cpW8oZw3Pr+k80zFZ87nWHT+pvjS+VyZzpkVXzGf69M5v+KD
ZV8C+SwU3yb7WsjnovgM2VdEPiPFF8d5Jj8Xso+L/XPy8+xflMdn/6UcD/t3
5fjZPy3Pl335KnlDOW98/krnw4o/n88R6TxT8fPzuTid2ys+bfZJkG9C8fGy
T4V8K4rvlH055BtSfJLs0yLfluIz5DyTPw7ZF8d+RPl59l/K47MfV46H/dBy
/Oz/lufL/n6VvKGcNz7/pvN25TsCPsel82HlOwj2PZAPQvG9s++EfCiKj5p9
OeQDUny/7LsiH5biU2VfGvngFF8l55n8hsg+Q/Zfys+zv1Yen/3HcjzsL5fj
Z3+/PF/+7kIlbyjnjf0H5F9Qvu/4/e9d7AuS/p6Dcp/xD/tFyG+ifC/AfJvO
8YKYd7OPh3xAiu+ddRDi60Gsh7Afi/xZin/797+ncTSIdSr2vZFvTvEhs0+O
fHNBfP7C/kXyMyr+29///kZAkPT3NywIxyHjN/bvqoyD8jjsk1aJE+U42e+u
kgeU88DfFajkGeU88/ceKu8R5ffI3wWprB+U1w/jY8K5Ck7mdUi+GMUPwziY
8LWCh3m9kV9J8SnxuiK/mOITY38e+fUUnx6vE/JLKj5J9qeSX1XxqbLfWr7P
71Eeh783kH+X34scJ+dfnhfnWc4P10nyDYFcJ8n/ovhe2L9Ffi7Fx8X+OfLT
gVwnyc+o+BjZP0p+UpDrJPl5Qa6T5KeGQlKdlJ/n7wTk8blOyvHwdzJy/Pyd
kjxfrpMqeUM5b9z3yYel+K+475OfSPERcd8nf5zii2M/IvkTFV8i+0HJHwpy
3yd/ruLLZT80+aMVXzTnmfzpwL509v3Lz3Pfl8fn717kePi7Izl+7vvyfLnv
q+QN5byxj418bYqfjXEs+bMUXxb7C8lvqPgM2d9Jfk/F58k4lvy2is+W/c3k
d1Z8zuwvJ7+54jPnPJPfH9jnz99R/L/n6TsWeXzGsXI8/B2XHD9/RyfPl7+H
VMkbynmr+jsvU/yBzMvI76b43NivSf5Nxbcp8TKQeRn5lxXfcuTvvEzxjbNf
n/z7oMLLgHkZf5ciPx/5Oy8DmZfJ8Ui8TImfv0uU51v1d14m5w3lvLHOQL5L
xW8p6QyKb1DSGRQfrKQzKD7kTr/rDCDrDOTHBxWdAVR0BlDRGUDWGeTxO/2u
M4CKzgAqOoMyX9YZVPKGct5Mv+tjin9V0sFARQdT/NvsmycfveKfl3Qw5fsF
SQdTvh+RdDBQ0cGU5yUdTBmfvw+U45F0MCV+SQdT5mv6XQdT8sP6Jumgiv75
R6V9A33C0sHtkM37/52TeBfOvvDwXgJ0de8Z7lVbA338c5rb5mXgoo2nKm5a
nAQRrndb5icmwKjYyvEjPDJxQrkjF35UToGcreNbHHOJgox6x61a7c/Cch0i
wvstTgWL/pXtd717CPsP/FPt1vRsrP9x856IqWmwTvKjsj48687McpXXv8TM
IR3H34vJwtYTSsZfupYKq8pci2noEI8diw7sZ7qWie0GXlg+0zMFIj1f/hyd
mIqmsNE1Fw0zYc9p+0cExyZBScdqvqbW6fh8hnN4ywQjNnnd7aJvzUQYs/Pe
cLelerwys8jwPSMM2LLCw2uDPeKV8wzWj/lcQ/v2qV2NOC2kBlUKKlzGiD4f
Sro7706B/QuW/VhqnwJRlk/m+vln4M1VOvcz5jrt2+dJ7/pDY2H0P298b5fK
xM8NGmgqhWvg8eQj45dMD4XP092+ZVfMwifbp6XlLdPyuZviF91L525bZ2pP
hAQmYMHM7vvKjTPhrb+W3S3eRwM4YlWnd64atM2t139dihEHTv+5KHtNKrS1
sfhnQzcdrhnuNaZFDwPuP7qufOaYZKgztEGFWVsMWGaRX/s5F3SYcGPXNbfO
iVBkzJuRB1wysMQzh7lfW6TjsuvVnkztEg9rrc4kWM3UgI+v2/bQMQY8Hjeq
ac2uWviwIHeX2514sG09wM/xkxEbDnbP/rAxHS5PCMUx8eHQaF7syJftTPjl
5vW0VZt0EOa/60P1yVqskF3fYcFHPR4Oep73LlQDuu+2H0a76HHow84xH5x0
+Gxq1Lk6sakwsVaLYs0DjBj8vmtKeX8t1m19wrC0fTLUGxt/pIqnCT+PPu10
rZEGrZwH9JwelQAjq3aYf90nGcp17tvNt6oeP73vEOrurgfLN726ba8fBUn1
8lq03WrAYTnvW9SIM8B1lyalrLwMaP90y6jSjzVYtKT7qzP3NLD28N33l+9n
4IfJAwJSxqZi0YaNtxzokwp/FTvVuc/0TJxS08V/9pokjF66ZNKkGUmwYlv5
oluTYuFn4UO7V8dr0flw7w672pogP/aYZtEjE3onvfnWYmI8uiS8wXVNNVBx
xvKMjSOycOsfX7YFaGJx3bu8Ac9+JoN+3KWeJ2zNOH9T2I3qV15gi0bt/b57
pkp8fK/Cp9T49e882lfhd1qJL3cgntjxN158VOGbL3/jvyeD2EcXpsJn2/7G
W08FsT8n7Td+ejKoE/3u7zz0uBLn73zTV5nX77zSV9YlFL7J54lDH9XYcbVu
Fmhf5ia61H0AVrHjnf31Wnia4PuieQkTVE4pcqrW/RDILeK8KPd/32+VMhz1
Nuhh/NbGd2winkCCjVWvmisyIOZ5/tbSzTSw9fLjYfPLhoGrw+H5R0OyoF+N
oqNufIiCtrE5S1u+CocNH4qOeXXiNZS1vDenz/AUPHMtLiK/ehhs/1Y79u/D
2bD2i03D9RodNn3uOulr9BPodqare8sxJuhR/mJZd7cMnKEttXp1cAgcKVNt
erMOZvw//c1J2yuZ2GDdQn8nc/wjPcvczzf3s4jF19ru65aNBXt6LdKa+eCs
QlqX5eY6wOejdF6qnJe593KoFLE0A36erm/zeGc02F17WSFgsg6cdKlvDrnp
oeKA2U/ftXgFmoyhM09bGeFQL4vRY1+kQb53uS25WfFwe6zlsXKRJihZusj5
8n9Hw62dq1egYxJUKoOjyvTMhrH7T6e+652OBa6NSvIpHwdxt388DN+bAed3
af6uVM6ImY+rj23mEgOu8y64W0boIT4xZ5njGRN6Xt+dov0nEt49GpPffWA6
DNrs7LB4VhauyEt//vBZBNQz56dUczOebO/VcsX5bBzbbah1K5fnULjG4I45
kcmwYZV37vpVOvi+/9TCp8tSIfNym6YaJz0ETCg2YdKWVGj5XO+9PEUDdQ49
7B600ggj33+Yd9wt2tz3+9o+75wOts9B+3SWCbLnWyc0HKbH5u26HhwyMAVm
NTk83EuvA9PqGqcmts3Agvttl29ukAgry96vGXdbC1MbHD/u1iUTKyfV3Zjr
Egfddy7Ke3I8DbbpHl9LvZyFqVsjXT/mxYDzJJcB28z1tkvu3ZKTPiXDy3ux
9cs20ENta9cmP1rrwa//tgKdY6LgW7mJOyJ8DND5wM/EU4EGaNigll3MUgNO
Xjfb4HZaAzcXDSvqo9GAy4mWQcEhGfiybZMm2hqpUCm/+WDfJanwZnDTldPm
ZWIZ178mRPRKgl2Vq4S57kiC17mdhzmVN/ff1rW3tH2dAYdWOp7M7aiDn2t7
/KxZx4g21qeybudrYdLllNiHhVPAd03Z81eLmtCieZFKc+dq4CTAU8PaRHA1
WL4xrDJiWtOAP60N6fBxuv18qyEJik9C7o/sW+VzYz5fls/lv9H5LOMQ+RyT
z33pHFg595d1SN7vEXQuTuftyrk8n9PL97l+/X59x+f4ynUwnb+zz5XO4f/f
+fsW6fydfQ9y/OzbkPcp40b6Hh34O3TGQ150fv6cxpfyr5xr87mwjOs60/k9
n//zOX7339+Xcl/CP8r4rMvJOp7Fb//8m0/2E9B5eBCfg/O5uHxfruesM7N+
KOuBo3+fl+JzoHULtG6R1+3mRpl9mww8Dl06j5r5cHsWHrR5VK69mU/YGHrc
bVUf8cE/OxcUnpiFj1afGl/CzBNpX0ABsS/wBO0L2kdA+wg9aB9lb1ozfIxD
IFwY5fm5e4oJI6sE31lTQg8lHBu424bcRe33vDIjg034sHkxg/GNDrLFPkXa
p8j7lPY1lBX7GnfQvh4n6gBEiTqAFakOUN2AKaJuINeNrYUbvZybfAPWF172
86WH0YyXC0aF+xshOfPHqOaT7uBdlzKbLM33SzYJc9u8zwhHRV3CPFGXkOtS
R1HHMErUMeQ6RnUP0kTdwz+p7lGdhCqiTmI3qpNUV6GQqKvIdZXqMLQQdRhn
Uh3+YjX+yYN6weB+u8Fw66R0DLnn4dJpZSZUrtDlWOKsG7hm1ID53j/Tsf/C
kQEdd2eCs6jzOEfUeaxAdf6k6AvYWvQF5L6wVvQRLHDgVx9B7iP83cQ16buJ
L6IfgZvoR2hJ/Wiw6F+wWvQvrEv9K070O5gm+h2+pX5H/dG8D371R3Sh/jhO
9FMoLPopxlI//fDnrb+7uD+BAhsG77kZmYSzezfb39QhB267JcwqC5ewYNc5
k7q6JuOw9ff9Cx3IASvRr/Ge6NdYkfr1ftHf0WLNr/6O3N97CDyAVQUeQMYD
4wV+wIJnfuEHrE/44aXAIVBK4BCcSTiEcAs0F7gFRxBuIZwDCwTOQV/COesE
LoLWAhchEC6yFTgKrggchdsIRzkJ3IVdBe5Cxl2RAqfhHoHTkHFaI4HrcKrA
dZhIuC5U4ECsLXAgfiQcOFjgRswQuBGtCTey30iuw+zHkvuCQeB5eCnwPDCe
Z1+9heSrDxfvEX7Ar/cII+g9Zon3jj/W/3rvMJfeO/EI2CR4BDCP+C54B/wt
eAeMJd5hL/YLrBb7Bf6g/fJe7C8cL/YXPKb9tUTwGvwheA0MI15DPAgmCR4E
UcSD1gneBO8EbwLmTTcFz4JmgmcB86xsUX/gjqg/UIrqz1JRr3CdqFfQmepV
QcHjMFnwOBhOPG6Y4H1YVvA++EK8z07wRPgkeCIwT/QQvBLuC14JzCv1gofC
EMFDgXko8VYg3grMWyuLeg4aUc+B63mSqP9I9R+4/gcKXozEi+Er8eJ/BI9G
4tHQiHg08W4k3g0niHcXFTwdigueDszTiddDKcHrgXl9O6EDgLfQAYB1gPtC
N4AKQjeAwaQbbBc6AxQQOgPcJp2hiuiPQP0RHlN/XCH6KVI/hQPUT0OEvoGk
bwDrGweEHoJjhB4CX0gP2Sf0EyT9BG6RfkJ6C6YIvQV2kN4yVug2cFnoNsC6
Dek8ECZ0HmCdJ0roQpAhdCHoQbrQGqEjAelI0J50pLlCdwLSnaAN6U75/X7p
V0j6FbB+lS30LiS9C8qT3hUu9DEkfQwmkj4GQk/D3kJPg8Wkp5H+huOF/gZr
SX/j+rNZ8q+GqZxTFyM/K1/Zj/o7j/5XT5B9m/mEz3dLvs0I+l0fikP2c1Z2
HBVzsLwOqpiS8HE/Ixa+UHXijfUJsKnWuE7e0RrIcfWpcPVwBn7ZHp35wCYJ
pt2sXiNmZDIkDsq1/tNgwvS4WbMWr04Gh74+axo5voI/95YdUr9YFhZw2TF+
RlQKODvNi/h0/QWcPT4/dOTHLLzg0arDrLTU//c9O8dTzetH/1LvXqBpV+K0
8rlZuDBng0P1hFSwaLW1lefQV2g9IWVZWNEsbGJ54EiRZynw8ubQNW2nJKOu
xcCf63UmnO66xX3ykmRY4eER1zpeg5/zusx4dSgDAy4u9q5eOAlGunX8UqSi
Dueti1yvM8/3c7NgP9u/E2D2FuOaL+b9uS9/olNqqgFnOw/PrjooBbyLpT4a
+zoVnn/8c9C8XhnY1917p699GmwdX2JPzIJ4mOR76sW47SZMydvb5uo8DVRz
+qdtduFIWGi9bfqUY5nY0uFFkH1jLWzqcLfXrXXBUGfW+b4Bw7LQ4qqlReMT
6t+bN9pwtfdX/3gc0qf5eKctJsxfurzb4skaOGuzfnLJEmkYPETz8lDPDDzQ
ePVwH9s0aO87oMlXh3T8UO14o5A0A6bsSZ29yiEFurYqGuo/X49FLp93e2++
dsv8cLS8cxK4pHc6F2yef/fDls7NzflYExffvKI5DwWmJH6I0GogoMc5n7IN
Ddi21q3HBcpq4aWzX9ba/EQYnz4ntv4eI14a/a5Nwfbp4ND/Xu/43CjwuRjR
KjMsA/2vn70xs7kOKlyKXRfk/xCeWG50KHXEhL3ffvreKE4HB+pefhL+SYOu
I96WPNvEgK3j3Pue+aKBS7e7Q4NAcxxR42efNl9dXjgYC25Lgy1n7TpNNs/n
702tnnl0SEeflj26epvn9bzhwNSz5veaUnqRZob5PT/7NLRKTfP7nV33z4Xd
HqTAs0FN3FaYdLjvhUv1j2Ye7xda139dnViwzt9WZGhnAyZvPtvp5WIDWFfc
OvpbtcfQ915kxPimRnzfzLPtKnMfc+xltby/Ob7aZQY/7fFZg54ndm44a46z
3PaoEf3Mef95puveQjZpOKHBzFG7zPmHCj8ru5rX34k31ao9MK/HSWXf2041
r8PRxcLrf18aB2eOPmwz7KIWXx9cFd/yvRnPLBy4sm1sKJgWdFrnGpiODRzn
l7nTIRNyThe6aGt+3106GQ6fD4jHDtY545aa33sH5xdBF8zr3jrtXO8c8z5w
D3c6bm1e/7bRnVeP3BMG+/se7oNnk/CL9ccjxwvlwMRig3ZYmfeP6XaHaufN
+6n/zZSyNc376Pe/87Y3iM87jtN5EuuT0f/5d958g/7777wdDfrvv/N2NOi/
/87bySD2X6UTf2Ze+d9/z+1U0H//PbeTQf/999yOB/3333P7N36ar6JbRv/n
33P7Nz9pR24PyvgrCzY2yUn+EhsMR44VKaI9qgWHwlv9Wh4ygWOnsfYBhUNg
r22C38FoHTT8HjbCookRou4bTqVteAwHCt1LqOhmhKrnTrR5G5gOY01xlY29
n8Jt+xtLW7Yy86PDDWzcryWBU+zlv60+hkHVSoNKjs/PhhrXdu/8ciYJ+yRZ
FPXaGQZnRvT3sSicA9OMM7a2Ma+bDgsyG2sjQ+HSTNfSpTtmwpQv9mNdzev4
u63TmD5VHsP8yCEtA83rOXbUxVkVzfuwTPxauxy/h/Dutf1GR/N+nNQ0rWig
uR4d8fu4/+LaYEgdFObSwlyXoh83/9IyNAOW97OyiusaDZt3Rron1tRBYMlF
U+t2McCGhGfV8ubHQoUm8244TTBA3Vu9tnsGaeHqTJtjY5/Hwefg091HxmRA
bOC+MTlFE6FWianZPpaJUOxmj1GtmmfBkEqx+7vnhUHo/Ttf7l1OAlNwZpif
eb5ejYrF9TLvkwVDTR1amPdNjH3xOTPN++Vn9TzPSeZ9+/qdTcN9tWPB+dP8
Hx/N+3fS895fCz/PwNme3eNKmevQ1ryba5eb68++KkemLjfX38s7Zlf3MNfj
7xUeaVuZ63Ct13MOjTf3ofM7GoUsNvcl/9d/L/5fP9owfNGcBe91UHBVs+c1
S6RC9qGl13wszfz28fX3hlkaGDP01EWPmRr4euzMxM3fDfDu/LiCw1/GwVvT
j/PRZj5ccUfxDxvM873RePaxngOewl/eR4+UNb/naRPcY1eY3+/ll8nPN5rr
0cnDO2oVNNenZof2d7BoqIeubvmOXc31NHHWuGH25vp6ZfSKepbmulp5ptZ5
vrmvFF4W0W+duc8YFmx4dtHcXxZkf67c0txfzzVu8Ox7p1dw7faloVPMfdZz
W7UDN2xSocedoc6R73SwbFxEfV9z/AWHVshctigWTr1rVWtEZwOUPF5p5wzz
+9pxaHbCpC2P4cTkjVX2NDZCyFnPBn3M63Oer//caua6/75A37tPzH1g10/H
U9/LaOFQjd26v839r3HhVQW6m/thTttvO7aZ+6B966i9o804YMXAdePvmXHB
rcr1ms4244GmIx6XNnaLhrevXjR4/SQDtBEl72ab18+CNWOH+RQNgTEtb9Uv
at43l1t47Llk3i8hPTsW1Jn77paNezd9Mvdh3eCuF8ua+6+Ndevom2b8kb/N
9pyXGY9MW7653R0zDjndOrvV7VfBsCK+1txE876cU9+uoMm8H9NyDH+Fmvva
vKZNZ/4P13yaWsHmihnP0H2g+8j38/4wXH1ruc+RfftcB06I8XGlGB95fIoH
vot4cCrF80jEDxQ/cvynUiPdplntc5T96jNFHnCsyANyHhqJvOE7kTdMp7w1
F3mGlSLPeIPyfFi8F2gq3gtm0XuZK94j5Ir3iDvpPZbUzYrZWOyoI/u6q1Gd
3CjWA/qJ9YAPaT18H/Jr/aC/WD/I68dDrDd0EusNeb3R+oTzYn3iVVqftJ6h
qFjPyOuZ1j+kiPWPl2n9XxH7BfzFfsGmtF/KhTbx7FfmqKPslz4v9h0OF/sO
PWnfZYp9iu/FPkXep83EvkYXsa+R9/U6UQewqKgDmEN1QE1fonoCl0U9wVNU
T6j+wHVRfzCP6o+HqFcwV9Qr3EL1yqLGr/oG70V9w7+ovlE9hCWiHmI01UPY
hYffVj3pyH5m7o+9RF3FcFFXketqpKjDWF/UYeQ6XEfUbbwp6jZy3b4g6jxu
FXUeuc5Hib6AK0VfwC3UF9T4jofoL3BK9Bfk/kL9CKqIfoRvqR95iv4FlhV/
9S+cR/1rhuh30F30OwykfldH9EcYKvojnqb+yN/1FZf4VAXRZ3Gw6LNYjfps
JdGXcbLoyxhEfbm+6OOYIvo4HqQ+Tn0fu4u+j9z3CSfgdoETkHGCh8BjoBF4
DBiPqelLhkuX36+pHeA4XfJh2gi8h3sE3oNvhPc6CnwIRQQ+BMaHhCehk8CT
wHiy0u3rhQ5VOOko+w9R4FLMELgUGhIuHSFwLBKOhbeEY7sK3AvHBO4Fxr3l
BU6GfIGTgXFyF4GroabA1TCVcLUN1aU0wpVclywFPsc+Ap/DP4TPjwk8j1YC
z0MK4flZAv/jU4H/YT/h/3DBFyBZ8AVgvrBV8AvwEvwCthK/uCz4CHgLPgLM
Rw4K/gLjBH8B5i8hmZv2HCp81FH2v5UUPAgfCx4EfYgHtRW8CbcJ3gTMm8IF
z0LiWXCZeBbxMvQXvAyYlxGPg66CxwHzuG6C90FhwfugK/E+B8ETIVfwREgm
nhgoeCUQr4SDxCubCh4KgwUPBeahUt8E7pveguci8VwoQDy3iuDFSLwYWhMv
3ix4NBKPhlTi0X8L3o3hgncD8+5Zgqcj8XSYSzxd7TvWUYLvA/F9YL6/UugD
QPoAsD4QLfQESBd6ArCeYCn0ByD9ARqT/lBb6BWQIfQKYL1C/n6WdRgv6XtY
vj9Y6CRIOgkEkk7STugqSLoKFCRdZarQYZB0GNCRDrNB6DZIug2wblNJ6DxI
Og8UIZ1nVG6W1cQC+xzl70DVvsvuFDBs9u1iAY7yuSfVWbDo8vs5rNp5K89/
lfT3f/j5VdJ3ylKcUIfi5Hjk7zt+/zvh//LK/wPSeybj
         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 
        0.9999998571428571}, {-0.9999998571428571, 
        0.9999998571428571}, {-0.9999997142857346, 0.9999997142857346}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {180.1, -540}, ImageScaled[{0.5, 0.5}], {359.99999999999994`, 360},
      ContentSelectable->True], InsetBox[
      Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHu0V1W1x4nwATocDsQ0NJ+kIobWEBVD5yhvXnxAXSmzrlJD8VGJGpGC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        "], {{
          {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmnfgjlUbx+9zn/NkZiYKqewRSgrZOyLR0kZTivbQkEpaiNCwKjS1Rctq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              "]], Polygon3DBox[CompressedData["
1:eJwtmnfgV9Mfxu8993ysyE5SNJQoGrSQhMxIe6DSTiqrIQ2lrGSXkRINZDSt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              "]], Polygon3DBox[CompressedData["
1:eJwt13f8jWUbAPDzO+d3FKVhZIVklYSIzLSUESGjjIxQisiMSEZGNChNpayW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              "]], 
             Polygon3DBox[{{821, 987, 581, 702, 1158, 820}, {835, 993, 588, 
              706, 1160, 834}, {1045, 1111, 600, 397, 396, 
              709}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
           Line3DBox[{275, 494, 1165, 276, 507, 302, 1264, 522, 317, 1269, 
            537, 332, 1276, 552, 347, 1286, 567, 362, 1297, 582, 1208, 377, 
            595, 1220, 391, 1309, 608, 406, 1313, 623, 421, 1318, 638, 436, 
            1325, 653, 451, 1334, 668, 466, 1345, 683, 481}], 
           Line3DBox[{277, 495, 1166, 278, 508, 1176, 303, 523, 318, 1270, 
            538, 333, 1277, 553, 348, 1287, 568, 363, 1298, 583, 1209, 378, 
            596, 1221, 392, 609, 1230, 407, 624, 422, 1319, 639, 437, 1326, 
            654, 452, 1335, 669, 467, 1346, 684, 482}], 
           Line3DBox[{279, 496, 1167, 280, 509, 1177, 304, 524, 1187, 319, 
            539, 334, 1278, 554, 349, 1288, 569, 364, 1299, 584, 1210, 379, 
            597, 1222, 393, 610, 1231, 408, 625, 1240, 423, 640, 438, 1327, 
            655, 453, 1336, 670, 468, 1347, 685, 483}], 
           Line3DBox[{281, 497, 1168, 282, 510, 1178, 305, 525, 1188, 320, 
            540, 1195, 335, 555, 350, 1289, 570, 365, 1300, 585, 1211, 380, 
            598, 1223, 394, 611, 1232, 409, 626, 1241, 424, 641, 1248, 439, 
            656, 454, 1337, 671, 469, 1348, 686, 484}], 
           Line3DBox[{283, 498, 1169, 284, 511, 1179, 306, 526, 1189, 321, 
            541, 1196, 336, 556, 1201, 351, 571, 366, 1301, 586, 1212, 381, 
            599, 1224, 395, 612, 1233, 410, 627, 1242, 425, 642, 1249, 440, 
            657, 1254, 455, 672, 470, 1349, 687, 485}], 
           Line3DBox[{285, 696, 698, 286, 512, 1180, 307, 527, 1190, 322, 542,
             1197, 337, 557, 1202, 352, 572, 1205, 367, 703, 705, 1213, 729, 
            707, 1360, 709, 396, 613, 1234, 411, 628, 1243, 426, 643, 1250, 
            441, 658, 1255, 456, 673, 1258, 471, 1350, 718, 720, 730}], 
           Line3DBox[{289, 501, 1171, 290, 515, 1181, 310, 1267, 530, 325, 
            1273, 545, 340, 1281, 560, 355, 1292, 575, 370, 1303, 589, 1214, 
            384, 602, 1225, 399, 616, 414, 1316, 631, 429, 1322, 646, 444, 
            1330, 661, 459, 1340, 676, 474, 1352, 690, 488}], 
           Line3DBox[{291, 502, 1172, 292, 516, 1182, 311, 531, 326, 1274, 
            546, 341, 1282, 561, 356, 1293, 576, 371, 1304, 590, 1215, 385, 
            603, 1226, 400, 617, 1235, 415, 632, 430, 1323, 647, 445, 1331, 
            662, 460, 1341, 677, 475, 1353, 691, 489}], 
           Line3DBox[{293, 503, 1173, 294, 517, 1183, 312, 532, 1191, 327, 
            547, 342, 1283, 562, 357, 1294, 577, 372, 1305, 591, 1216, 386, 
            604, 1227, 401, 618, 1236, 416, 633, 1244, 431, 648, 446, 1332, 
            663, 461, 1342, 678, 476, 1354, 692, 490}], 
           Line3DBox[{295, 504, 1174, 296, 518, 1184, 313, 533, 1192, 328, 
            548, 1198, 343, 1284, 563, 358, 1295, 578, 373, 1306, 592, 1217, 
            387, 605, 1228, 402, 619, 1237, 417, 634, 1245, 432, 649, 1251, 
            447, 664, 462, 1343, 679, 477, 1355, 693, 491}], 
           Line3DBox[{297, 505, 1175, 298, 519, 1185, 314, 534, 1193, 329, 
            549, 1199, 344, 564, 1203, 359, 579, 374, 1307, 593, 1218, 388, 
            606, 1229, 403, 620, 1238, 418, 635, 1246, 433, 650, 1252, 448, 
            665, 1256, 463, 680, 478, 1356, 694, 492}], 
           Line3DBox[{299, 700, 1358, 701, 300, 520, 1186, 315, 535, 1194, 
            330, 550, 1200, 345, 565, 1204, 360, 580, 1206, 375, 711, 1361, 
            712, 1219, 389, 715, 1363, 716, 404, 621, 1239, 419, 636, 1247, 
            434, 651, 1253, 449, 666, 1257, 464, 681, 1259, 479, 722, 723, 
            724}], Line3DBox[{480, 682, 717, 1344, 465, 667, 1333, 450, 652, 
            1324, 435, 637, 1317, 420, 622, 1312, 405, 607, 1308, 390, 594, 
            1362, 713, 376, 1207, 581, 702, 1296, 361, 566, 1285, 346, 551, 
            1275, 331, 536, 1268, 316, 521, 1263, 301, 506, 1260, 274, 493, 
            695, 725}], 
           Line3DBox[{486, 688, 1364, 719, 472, 674, 1338, 457, 659, 1328, 
            442, 644, 1320, 427, 629, 1314, 412, 614, 1310, 397, 600, 1359, 
            714, 708, 382, 587, 728, 1365, 704, 368, 573, 1290, 353, 558, 
            1279, 338, 543, 1271, 323, 528, 1265, 308, 513, 1261, 287, 499, 
            1357, 697, 726}], 
           Line3DBox[{487, 689, 721, 1351, 473, 675, 1339, 458, 660, 1329, 
            443, 645, 1321, 428, 630, 1315, 413, 615, 1311, 398, 601, 710, 
            383, 1366, 588, 706, 1302, 369, 574, 1291, 354, 559, 1280, 339, 
            544, 1272, 324, 529, 1266, 309, 514, 1262, 288, 1170, 500, 699, 
            727}]}, {
           Line3DBox[{732, 1036, 493, 731, 1165, 949, 733, 1166, 950, 734, 
            1167, 951, 735, 1168, 952, 736, 1169, 953, 737, 698, 1063, 1357, 
            738, 1037, 1170, 954, 739, 1171, 955, 740, 1172, 956, 741, 1173, 
            957, 742, 1174, 958, 743, 1175, 959, 744, 1358, 1038, 960, 1039}],
            Line3DBox[{746, 1064, 1260, 745, 507, 747, 1176, 961, 748, 1177, 
            962, 749, 1178, 963, 750, 1179, 964, 751, 1180, 1065, 1261, 752, 
            1066, 1262, 753, 1181, 965, 754, 1182, 966, 755, 1183, 967, 756, 
            1184, 968, 757, 1185, 969, 758, 1186, 970, 759}], 
           Line3DBox[{761, 1067, 1263, 760, 1068, 1264, 762, 523, 763, 1187, 
            971, 764, 1188, 972, 765, 1189, 973, 766, 1190, 1069, 1265, 767, 
            1070, 1266, 768, 1071, 1267, 769, 531, 770, 1191, 974, 771, 1192, 
            975, 772, 1193, 976, 773, 1194, 977, 774}], 
           Line3DBox[{776, 1072, 1268, 775, 1073, 1269, 777, 1074, 1270, 778, 
            539, 779, 1195, 978, 780, 1196, 979, 781, 1197, 1075, 1271, 782, 
            1076, 1272, 783, 1077, 1273, 784, 1078, 1274, 785, 547, 786, 1198,
             980, 787, 1199, 981, 788, 1200, 982, 789}], 
           Line3DBox[{791, 1079, 1275, 790, 1080, 1276, 792, 1081, 1277, 793, 
            1082, 1278, 794, 555, 795, 1201, 983, 796, 1202, 1083, 1279, 797, 
            1084, 1280, 798, 1085, 1281, 799, 1086, 1282, 800, 1087, 1283, 
            801, 1088, 1284, 802, 1203, 984, 803, 1204, 985, 804}], 
           Line3DBox[{806, 1089, 1285, 805, 1090, 1286, 807, 1091, 1287, 808, 
            1092, 1288, 809, 1093, 1289, 810, 571, 811, 1205, 1094, 1290, 812,
             1095, 1291, 813, 1096, 1292, 814, 1097, 1293, 815, 1098, 1294, 
            816, 1099, 1295, 817, 579, 818, 1206, 986, 819}], 
           Line3DBox[{822, 1041, 1207, 987, 821, 1208, 988, 824, 1209, 989, 
            826, 1210, 990, 828, 1211, 991, 830, 1212, 992, 832, 1159, 1213, 
            1059, 587, 1058, 1044, 1060, 1366, 993, 835, 1214, 994, 837, 1215,
             995, 839, 1216, 996, 841, 1217, 997, 843, 1218, 998, 845, 1161, 
            1219, 999, 1061}], 
           Line3DBox[{846, 1048, 1047, 1361, 844, 1307, 1109, 842, 1306, 1108,
             840, 1305, 1107, 838, 1304, 1106, 836, 1303, 1105, 834, 1160, 
            1302, 1043, 833, 1365, 1042, 703, 831, 1301, 1104, 829, 1300, 
            1103, 827, 1299, 1102, 825, 1298, 1101, 823, 1297, 1100, 820, 
            1158, 1296, 1040, 1057}], 
           Line3DBox[{848, 1049, 1110, 1362, 847, 1220, 1000, 849, 1221, 1001,
             850, 1222, 1002, 851, 1223, 1003, 852, 1224, 1004, 853, 1360, 
            1045, 1111, 1359, 1050, 1046, 601, 854, 1225, 1005, 855, 1226, 
            1006, 856, 1227, 1007, 857, 1228, 1008, 858, 1229, 1009, 859, 
            1363, 1051, 1010, 1062}], 
           Line3DBox[{861, 1112, 1308, 860, 1113, 1309, 862, 1230, 1011, 863, 
            1231, 1012, 864, 1232, 1013, 865, 1233, 1014, 866, 1234, 1114, 
            1310, 867, 1115, 1311, 868, 616, 869, 1235, 1015, 870, 1236, 1016,
             871, 1237, 1017, 872, 1238, 1018, 873, 1239, 1019, 874}], 
           Line3DBox[{876, 1116, 1312, 875, 1117, 1313, 877, 624, 878, 1240, 
            1020, 879, 1241, 1021, 880, 1242, 1022, 881, 1243, 1118, 1314, 
            882, 1119, 1315, 883, 1120, 1316, 884, 632, 885, 1244, 1023, 886, 
            1245, 1024, 887, 1246, 1025, 888, 1247, 1026, 889}], 
           Line3DBox[{891, 1121, 1317, 890, 1122, 1318, 892, 1123, 1319, 893, 
            640, 894, 1248, 1027, 895, 1249, 1028, 896, 1250, 1124, 1320, 897,
             1125, 1321, 898, 1126, 1322, 899, 1127, 1323, 900, 648, 901, 
            1251, 1029, 902, 1252, 1030, 903, 1253, 1031, 904}], 
           Line3DBox[{906, 1128, 1324, 905, 1129, 1325, 907, 1130, 1326, 908, 
            1131, 1327, 909, 656, 910, 1254, 1032, 911, 1255, 1132, 1328, 912,
             1133, 1329, 913, 1134, 1330, 914, 1135, 1331, 915, 1136, 1332, 
            916, 664, 917, 1256, 1033, 918, 1257, 1034, 919}], 
           Line3DBox[{921, 1137, 1333, 920, 1138, 1334, 922, 1139, 1335, 923, 
            1140, 1336, 924, 1141, 1337, 925, 672, 926, 1258, 1142, 1338, 927,
             1143, 1339, 928, 1144, 1340, 929, 1145, 1341, 930, 1146, 1342, 
            931, 1147, 1343, 932, 680, 933, 1259, 1035, 934}], 
           Line3DBox[{948, 1056, 722, 947, 1356, 1157, 946, 1355, 1156, 945, 
            1354, 1155, 944, 1353, 1154, 943, 1352, 1153, 942, 1164, 1351, 
            1055, 941, 1364, 1054, 1350, 1163, 940, 1349, 1152, 939, 1348, 
            1151, 938, 1347, 1150, 937, 1346, 1149, 936, 1345, 1148, 935, 
            1162, 1344, 1052, 1053}]}}},
        VertexNormals->CompressedData["
1:eJx1XHlcjdv3jlAhQ8g8y6zMQqxIxmu+maVEyJR5njNmyBwyazCWzOMSqUia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         "]],
       Axes->True,
       DisplayFunction:>Identity,
       Method->{},
       PlotRange->{{-0.9999998571428571, 
        0.9999998571428571}, {-0.9999998571428571, 0.9999998571428571}, {0., 
        1.9999994285714693`}},
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02], 
         Scaled[0.02]}], {540.3, -540}, ImageScaled[{0.5, 0.5}], {359.99999999999994`, 360},
      ContentSelectable->True]}}, {}},
  ContentSelectable->True,
  DisplayFunction->Identity,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.5801953870117435`*^9, 3.5801956008039713`*^9, 
  3.580195733016534*^9, 
  3.5801961749438105`*^9},ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["paramcurveplot", "Subsection",
 CellChangeTimes->{{3.580281488734355*^9, 3.5802815217742453`*^9}, 
   3.580282014302416*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"paramcurveplot1", "[", 
    RowBox[{"surface_", ",", 
     RowBox[{"{", 
      RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v_", ",", "v0_", ",", "v1_"}], "}"}], ",", "\n", "        ", 
     RowBox[{"PlotPoints", "->", 
      RowBox[{"{", 
       RowBox[{"upoints_", ",", "vpoints_"}], "}"}]}], ",", "opts___"}], 
    "]"}], ":=", "\n", "        ", 
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Table", "[", "\n", "        ", 
       RowBox[{"surface", ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", "v0", ",", "v1", ",", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"v1", "-", "v0"}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"vpoints", " ", "-", " ", "1"}], ")"}]}]}], "}"}]}], 
       "]"}], "//", "Evaluate"}], ",", "\n", "        ", 
     RowBox[{"{", 
      RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
     RowBox[{"PlotPoints", "->", "upoints"}], ",", "opts"}], 
    "]"}]}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"paramcurveplot1", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "v", ",", 
      RowBox[{
       RowBox[{"u", "^", "2"}], "+", 
       RowBox[{"v", "^", "2"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "\n", "        ", 
    RowBox[{"PlotPoints", "->", 
     RowBox[{"{", 
      RowBox[{"30", ",", "30"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5802816479224606`*^9, 3.580281784826291*^9}, 
   3.580281874698431*^9}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF021IkwEQB/CZIhMNlmK+9IaECEWCWqlsdbOgiVMpmaVRiqmZI9vMUsm0
UCTLWVYOP9jmCxqzcmJlqLOWhooimRN0jqnbUJe56bO5DTdzSyTPg/twP477
cPAPuMFJyNxDIpECt9p5q/l5VoXDsSolbRchfbc9O2CFVv0gnlhBfyWenRsz
2YHhGuQvUOnRT3vpPjQZN8Gv1Motf72MzujJahxVb4AlNJ9XkLOEbvOTNdL9
bbDAqpsJS9Cin6lMVpY8WgdzauYFqtsCek2kN5XyxwKUhum8WK0GfYPM1qfw
zcAcNnaJalXo15ZTvlHjTBAydCtL2KJEz6OFjracXIP5TPqiPVuO3tJ3TuzO
MsLkZFqNWTaB3l85zbRXGECa3l7r8BpD57i/v55vIODALG2G1jaI3v03Uff8
MQHPcuLacw/3o98VzC16JhBAWCV74o5I0MU+gWrrCQJsuvLuGcFH9KnBh9Gt
3gSQv9erlBHN6FTty9501607BnEnt/w2ephc3ulBImBEvTQcLGqAHQ+VXB1P
dCGAwjcXhqR1oBcUP51SeRJAi7TwmHd6dvevKHJajxMgdAli1HP60G0lX02k
WAIEOtmQr2gAnXOJ9YZ+nwB/31r23NhP9IgQ55gkPQGd4cnF0rYJ9KML1iR6
mQEO7ju03xovRzdGqynaeCOkkqvPCxVK9Dr254tV9DUYiJla8+Gp0IPZ0qaz
DBNQRVFivlyDvpcqCezgmeHtL6dmN/s8Oj+DnP5FYwHHE7+MT75adG7UvcKR
xHUIv8zz8ExbQpf9qGBlU2yQscoibpYuowtrBFWUiQ3IrS/s5Y7r0QMUmhdF
85tQ5tbN7Pq9svvPY6ecivR2+J8v9J18/QOBuIip
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcwHFJ3db1wihGybkO11XuUEnW/Th3Vp4qKaU7WlHykEkP6s7V
cZRuXe7i9qKMSlk5Q9ohD2PlYchid1rLw1oY9/9v9sCYYXmR34vvm8/7r8eV
zJhkWxsbG8/1Nq9Xmm1WWK1zomd1ozJKv0aUWL1syh9fBS2j+EG0TovOnJ6w
nJWtAHOrt2uZUoNOhGt6vYYs4JJnZhe8INBX9CltpzvMYPK7y7mXMYsuOCNr
+7a0CFOxL8f8Y9ToBnH8km2CCRYSk8OCqFPoJ7Kc4kp658G+4ld2pFqFXhec
Rv2bbYQIiaGJz1Wi0/ZeVoj2G8C3J+U6790ouvdt/5l9jnqYTA6eXrshR483
h/Y999PByAirZGFYhk4WCqV2Ui2IrtZyrY5S9LDCC8b5cg3sGmeMMWq60X+n
COy92kl4mhFVm7VHjB71sXOLD4sEnbnFNsq9Bb2bF7lsoZOwTBY0j5V9Qnde
tfTGzxBA6ShXjh6tRJ847lUvlRCg0wuE7IJ09EH3z2892gjo+zMr8eFXwIYP
TDUqGroIsC9dyPFl1aPHpe5US1QEMAJNnIibX9C/uzU6XtpBAs/Om1me2Yme
kJHrse0cCWXkcA+d34XOdKhmF1WR4Ernpk5IB9HlQ5SBJq4GhAEXc0U1MnRB
oGn2yVct7Ka5OZmj5ehVj1QH0g7qIJFSHMpTjKKfeiNMp7nroSv8p9GZo0RX
ve5oF3saIIgfIiiVq9DFt1pDDqcZ4f2PTZXUtUn0k6+oD491zoP1sUtSA12N
7nn/Tkn0IRMEnOdsd2DNohdaitga7SIkzcXqruUR6DEDvA8l9WbIKs9pZQ9p
0Edok9VWsQXyqc0RTTNa9GnWETdr/wr8/wt9469/zDgagQ==
     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmHTEBeBusTmKRELRUNYQstcfGqR6aaImuWF2dSVM82JHVRQ
otBQFgzKdhGbVprTUHE2PAWraOoy5hGbojbN9d90m9vclM3Y8iK/Lt6b5+q9
+YUVlGey3d3c3E7tz2N/zZX2RZdrSxbIGurwfbkpKzBfeuv8tQdGOr8m3WRE
Tz0dJVaqHJDsGREoVBvQjXza/dfzdgh4bOfWP99Ed9L6ORPyXdg584D3sEyH
3icIKQGnDTTZr5ZjMwl0q1d7W22hFWz57MvxJA06c+Cmh7fSAj6tC5VMYg1d
0iCV5lWbgaGwDIoFanQ/TlfDeaoJqGPFt0XtS+jZ9/YiU6hGWGcn/HZyVOjD
dcEGP64e5udZL2wzs+gWcmqkY2EDZIW9Atexyf9/yAbGRpcOglboy/TuUfRV
axInSKGFprK03ooTX9CzzubcCC/Vgsk+4p4WOoL+PWo10xyuBYe+fmhZKEEP
aeSHZWwT4PWpRb10rg1doxgmyecIMJl7pNz6UvS591f++I8T8G1Vp4gRt8KB
z1QwYzunCPBptlVRWX3oeTudIDMQQI/b4THuDqPPdjJKM0K1IDockdxS/hm9
yO5b48rXglA/M0YRy9HTRKCok2ghkCIo+TmpRF/JE57seaMDKS23VtY9i/5B
3pRUp9yAYN8Qf3u6Cj3rWa5kgq2HfC/+RdHiEnrblItKSTGCPPXH9nGeGl1D
HLlzIdoE8eLEnmbVGvrXQU9jL9cM76YOtZGc6+jMtaxo6agFXE8DivopBHqk
TJagiLMCLYd3lMzSoTcmzgUXW21QtJVtuvV4Ez2HrLvqPbALFS1VH7nTBnTV
tZjrNeN2eEIaYgxqjei6jkeK6mkH/OsL/aCvv/RB5gk=
     "]]}, 
   {Hue[0.37820393249936934`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tM0gEYxmHSPM0uVLZSstK2ZGm1SQdN2T7ThVseamY5t5xRitlSPEwl
DxtZVjO2rCS3ClBnypbHylKp0BaZ5tJkU3SesD8igkAoKJSaF/ldvDfPxXv3
873Cjku1I5FIBzZnvzl+rnV8Y8MgFbs/LaIwNdK3pxXTbDcb6OkVhbFGPfoL
7kcCHKwQ6UilCGYW0cNU/W0e9qvgVWrNKnuiRT/HY9TTTWaw0PJ5BRkadDtd
a32r/zKo4p9PHo1TozOSgpX6hyYwJ6cyQl1U6ALXZeDajOBWPZYbrZ5Fd373
iBBEGSCqz9QhrppB/15YdDdBooPA3mtpwpcT6Cve+2sDyheASA2bW09XoF+i
RrBWvs3DyAiz0jwsR+dLsmomaGqQXm2p2iAPohu6yLIB+znYPUWfpDd9Rb8j
SJ9d81VBeUZMS/bez+iWqeJRBxkBRqvELmafBJ0tdxqQFxNg05V1Tgpeo3cH
vn9GiybAuVs0MxFch34/R1nacHjz53dze1bZDfSCeHae1YeAfqWm74i4GrY8
j5Lb+OAgAW58MyeQ2YZOEg111DAIoJ+08KIyu9Dz43JUNA4Bwu3USBG7B92p
NnhJ2UGAQDfc6ymWoRsZrHCmqwoonlXXpwd/oN9y/CKo+KOC9qDEEmmTHD0h
s01ymaoGb/c9O62xCnTLcW5A86d5SHauiBCOT6BH3vR7ZatZANmZ0aVdvBn0
DA79bFKLDkLFp5r5iln0Q5RjF8JDDFA/tK3OZZ1AF55Qp/ktGWHjnlfKG081
eiH5fEkIywRBF3k7PJga9HF+WuKYzzKkGOKNrFItegOr+DH3lxmyRZwPWT8X
0f0beyq1yytw26UzqmNejx7yl+yoXVuF/32hb/X1DzkC1+o=
     "]]}, 
   {Hue[0.6142719099991583, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tIkwEYxnGziCQprchplqWUhZLzOGOrV7FUNI2WiRG4PCzTtLkWKktN
DJFiglCySp1iK0d4QMvSDKaSh1mpaZ7GbN83D3NsznnYdAkuL/Lt4rn5XT7w
P5HIYbKtraysTm5t59bKeGa5xbIo9e+6f8XNUy2NkXyoG09aBz2jlB9t0KMz
pl821catQdhud6dKYgGdCKvN58eawLHQnFn0TItu0LvyxJdWweSTJcjO0KDX
RJfwKPnLMBtTPuXLVKPPSQ81xE4bwMhih9JtZtH5imrHoXI92FVP8i6rVejh
DyPyM5a1EClbbpUICfS9QX7H3RM14N17J0X0RoFOU8npG1Q1zLCD5jZTJ9Ar
dBuWHs4sjI0lPDcOj6CPFpyh101OgzSpUWg5OIjuVfA9vuKdCo78Zkwx6nvQ
vya65v7oI+FpRlQj91gXeoDYnzt6lwSDud06yqUdvfFFB/u1Gwl/dEVtU5XN
6FZrHD/rFQL2dFQRikAxeh/tlXP2LwIMSw0tmUXp6J8cT++X9RLQT2pkZyXV
sO0tSo/Qm0ME2JUZc7wTmtCDkp8wuQsEMM6ZBJH3PqN/dPB4ZO1CgmiXe1gV
pxM9LGVJ0MQioVI33EuRdKNTbV0U1GYSnCjCNOXgAHrnt1zarRoVtNBu5Enr
R9CFvmnXqQPT4Gx/9LA5egI9YCOwO489C6w9pSEiuQK9+ILyYn+4Grojxlcc
BAR6/XnjvGesBuiS4IayCRV6ulKntye18HZoh9hmcwY916c41blED5Zix+T3
FDW6U5ZXRE2bAWixAtsDCRp0/nrIqeAHy5C8GGO4Xaj9/4MsjjUfsArcqpwv
mT8X0Pv3iRKuhZvgsU1bZOu8Hl0ebxhmXl2Df32hb/f1F+5TzNE=
     "]]}, 
   {Hue[0.8503398874989481, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIkwEYxvFpIY6ktNScWSZmiol4AI+r1w5oeQpErKyQmcsDmmczQyWH
RTZKVsML0QlZzEBliuQJp0sTLbblyOb2fW7N6Rxzc6mbrQuXF/VePDe/ywf+
frmlGUxHCoUSsL8D++NW2hR2+6ZQVHHdP2RXK7xEU1Cjlq1gorfWpZtN6MGZ
wsNOMgskOQV6d6iN6OLhr0q5dAdoTbay5tcGdFXOVanj5BZYI2rYD0v06Bzh
gLTmtxlWM9vJyAwd+gdGlWfRmAksOczEeOoq+s0Jg4zdagDXrqXKVJ0G/exd
bkywbh1S5raG+W1qdL1fo2Ilaw3CZwvyO98R6B7TAdaPAVrQMhPW9grl6HXi
xFFWngYWFxlvLAsy9MHcKkuBWA3Ce/1t9mMSdO/p0IiBCyo4sUwn6b2f0a88
VxS7PyahpSStv/yUCD3IxeHGmSkCzLYxxzTfMXSaxTNhMoyAPxvNI2THAPq4
oJqslyjBeZKnJmK60VMj53vzOUow/+obKmsuRj+3u8o59EgJ8z/1c6H8LsB/
WEZKWIsSXLmW2nCGAJ3O5tPuTCuBHmtlpzwYRQ/aNlQ0BBLQeTAwiVc6hX6N
U/uS4BPQsbEw68WfQfchZCHKNBK8vdqKVBIxOtx+tRIbqoKh6Fv1wl4ZerXH
0BGjSA0+bic9belydHeS1RiXrYEc59bLnQoCnXk62P70vBZmkn9sH2er0V/4
JrzVJ69BPP9iH1euQU+eiu4Rf1+H91KHbuqeFj3bfcF/6YkB7M9oeYNeOnQ3
cU+6Q4EJorPYLkcZevSiwkIWz2SGvM1M8/0mA3pUXMN4omALynm142XfjOgj
jZ8mBKIdYFFHUobXTejzEo8wwRcL/OsL/X9ffwH8Cua6
     "]]}, 
   {Hue[0.08640786499873876, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02lI03EYB/BluRxZmUZO8yhTzEibGqm4eOwgNVNJpEMDm0etyLyiRmSJ
V2QbCipLZG6E1irdKtO0GcuhqRHpNv1vk9kOj21M3aZOWcSWL/Lni++bDw98
4YHvweyCtDwnHA4XtJ6t62kssU06HCZh6qGja8dFWmHZi4S1midWWCTXPUwx
LyKnSkP/ZtxbgXh8sDdLvYDcNTp2JLR4GbzKbYVV9UbkPqrur5cpFliNuE9/
kG9Azio9/YgZaYLZ9OapyDQd8sDrFy0ediNYs/LOxRJmkUv6hnknpHpw4yhK
Lui0yDmZxR7jIXOQNLLUw2WqkeccuNbFaJqG8CHqzZY2JfJakZOYQtXATF7c
nP2WHLmhKf5IZooKMIzSYJVIkWeIXnKCApQgzOEzHR6jyHf163XOAwrY/5s8
Re74jjy6uiGAipNDTX4yv8hPtPkHgsCVdk4GZpvAKdlfgHzn0pjVoxuDP/NV
vVOsj8jf847Vi9MxcPnGViujW5HHkSpzBJ4YmC28rsKqO8j9V9gJWfYJ+KEx
jIRxObDh3mWvuyt2Y+DWaKWFUz4gJz3LHuUnYkCOWaUn3f2C3MfcdljSjkHL
tuB4dkE/8pO1EWdJYTJgzUuGiNxB5G4KhjZMKgNvIvO2avQX8sgriU2MTgV0
RV0tFXZIkauCAydCdyjBZ4/vPluKfLP3c/+n4RgVZLnUnWmZVG7e+22fJJVp
YPC8bNmTrkbu7Of7uJ4xDbHcU7xGuRb5gHBvJY04B6/GtrQS7DPIx9359vIB
PTieeuV2EnXIn/+sDnn31ghRl+iu7hQD8mQxX6P3N0GuKd18o9yIvKR3/k1z
mgWK2LS+QvEC8lV7eDs+dxkqCL1JPfpF5PjUSl98/gr83xfyjX39A45S2Kc=

     "]]}, 
   {Hue[0.3224758424985268, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF020wEwAcx/FF13EVqrvapLndYXFyyXVoc39yx2XokrpLHXmWxsh57EpH
u44buUrTOeZKbuk8VTrM3dQ1TgrbnLHb2DBrmZnN1LywvMjfi9+bz7vfiy8l
hRWXbkcgEDx3Zr+z+gKr3GZbEyqlDmMhNSrhQS9C4XTkBhjodWWxRgO6+bam
tI1uhsgDVNcm1So6V2SKKL1gAlKFNY/9fAWdz8ylvfE2wua5Ik5xjg79U0NP
sd83PWjiG5UBcVp0+v0hfXutDixJ6RE0Rw36b1lg+8/UZXBpmS2I1i6g9xWp
ne4OLAJj1NTH56rQy2hj3R4+avAfycpsfqtAjxUsNqyvK2EpPXR5+84MOnHS
k1LFksP0dPILi0SKXvX3EfdquAyEqV1c27EJ9JA/w3MBmik4OUdX0juG0bMG
75FeUqRQnRPTlU/+ih4Fr+zbMiVgtArsYtwFe39Pd+nDZGLY0rP7lU0f0OeX
DlfbCsXgMMRTKYJa0VkZKQm6IDEY1zt789jMvV8+D6HGXQzf1bpRP34L7Pql
z+yOnrNicKm3lPgn96AnCUOHldlioAdvchi5A+gMr8fuhHExNO+nRvJYX9CZ
Ihda4mUJNOklI0S+CB2cmbO3NiTgSuRmz0+Mo6fUUusEU1PQG3jjgbBDiu6U
aPfjpq8M3I6cOm6NnUGPXuW930yQQ5JDXXizXIFuKNC3P6XMgShKZj7BUaG7
BXNKfchqoPHDOutnFtAV8spy53eL0Da5r9Vxewl95Qx5i3RtGWxPSGkfiVr0
11qLx0WaDgKvcw4dTdahp+nJyvJuPaStxRszKlbQbRGtVK2rEfJ5JYN54lX0
Co7OO87XBJWO/Yy+Xwb0Zwpa45XzZvjfF/puX/8ArCz2XQ==
     "]]}, 
   {Hue[0.5585438199983166, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHFU1nCridYQtNKa5avlzAM5aZ/VllPS0jmdS3Mo+CifuaVr
65BGbo0mhg6bE42Zo5polE2zZtbUAB+kjdSAEBEEFRCnjaaSh/wO/8vv/j+R
V57G8SQQCKf28tqruco173bbh36QlBee9GqGTMOEOip9A2wM/r0Uhw19cnup
fjnSCQkHQgPa9Gvod1gb6YPh60CrdVVwBSvoRF/dU0W0DbbO3eVVl1rQi1Uy
gUxshaX0Vm1MmhndIRsmRZWaYTOXcyXOewld5BM7mnrJCOSOuaqrZgN6idQQ
YW9ZAKbc2S8R6tFvmlqepfrqIHqsqFD0QoP+PsKkcknnwMiJN+0Wz6I/gLCs
xjNqUKtZTZvTM+jOwrqpG4QZGMrvEboPT6F3FcgpDLEKAnUMLaN7FP15plXM
+DQBj0uTeyqPf0EvzKFXZyeOg8M16JkcNIieFJWf2BSmhL+r3AFtmwydLwl+
6FOgANLndr3mfCd672t5yOKKHBzr0r4Kbgl6WYyHPWJADooFi/yspAP2nc0k
K6t+yYHcvFkTzXqD3pDzaFt5TQGM2C0es+wDelHkoR4yRQkiYmhCe/kw+qvi
06Zx6ji0rU6PUSUj6Nd3NCcVeRMQQBXe+j01ia7cCf/2tVEFffSs+0PdM+jV
E0Qi2zINR32PUVwps+jvssWLu/5qyCXxL4vmNegZizaDUDcHI0k/N/x5enTx
QkBkjksLcZKL0uZZAzrfj9sqqF+ALpVHp/euEb2DE0QRxRjBXU9jv6Wa0TOD
/sSP0sxAz+Ad9GNZ0GkhwS6vBiuw7emOgtoVdNlI4u20IzaobK/5WPF9DT07
zEpvClyHOu8BZv+yDb2khvFSEOKE/3+h7//1D89v7rc=
     "]]}, 
   {Hue[0.7946117974981064, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIEwAUgGE1TUdSZnmtlJUlFFSmMWuTo0YTvAwaJimWma7UvExGJqIQ
thWshUoO6bKpYLYIpymCumK2eWFim20519hyDpcT59wUF7Nw+ZDn4X/53n/i
nSo6w8fLy+vkTnt24rPceo9nTWovZVuEVJ10f3k68VrWOtgpzXU0hx3d1X3u
FDHVCWl7YyMFplX0V/uSNjbAARGNbibnxQp6DqNaRb5lA9eFGt7DimV0SQx1
5oTKCpbs18Z4+hJ6qv/1qx0vLbBZwKCSCRZ0R9G0j7zSDEEdP1iZS2b0ej+b
cpQ7DxmK9SFRmwndp0kanFCmh7jJknvCtwb0PE1gusFfC4uM5F/bpTr0GCtN
zeKoQastbN1Ua9DPELNb6vuVIC3qbfMcUqG3Rpmik1wKOPKTYqT0TKBbRSS+
d9UEcCuyequjZOhiMbuU+GcMHG6JT1a0BL3lUR85Vy+HLRtn2CjoR59zV9R8
D5NDwGi7yZDYhf77b2iYSCADh1M8yOSUow+yaQtrZTKYWlhWnBV1wK5/eHPz
c+JzGQTxN2vjCj+ia8Uae7O3HCiXXLyMyhH03vq8ztlJOQh9Y9Paq76gr4+w
dC1TYyCwqSfDRePobbTTIU2HJyAyvK1sXqVEN2R2NmbOKmCQlNsg7dGgJ9hy
tnhsJRw9eCzUTdOh3/BT5C8w1VAQ0HxFqDegbz0Lya8jaWE8fW4jjGdCp3JT
uoCuB7IoRczXmdHHvp7vI5TMQ/eMdxdhexHdOdxPP3DbDJ6nEcUD4Uvo71nc
BhLVAqQcXmBw4TL6/dqBiw8kVihey3bcbVxB9400vpMn26C6vfYT89sqOkcd
r52Oc8BjwnDGkNWOzj/+JG36shP+/4W++9c/+n8ASQ==
     "]]}, 
   {Hue[0.030679774997896203`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYBXAzNRcVXkqn3fASQplkZitcfIWhlGkhUg7COS9l5uZEUlFE
M0yTCVKuUeJmorEIXReMzWlbirfEmVtsa236z6ZrXua0NGfm8iE/H87L7+lw
4PilZMenOzo4OBxaz9b1cHNtOrt9TkYaPFP9Sq+S3Q80XC6tmgcLtaYwzmpB
Z9G7Eq6UWCHaJci3nphFD6gUREatWMCnzMYufzSNfl79OPipdgqWjudx8plm
9Bfc20fLLphgIqHOEBZvQj9WUNi45mmERXp6VARpAn308zzdz0yAW8OX3Eum
cXSa6eSD4hI9xAwsiIU8An2MvX3ImqSB0L6Mm/xmPTql7ZrTk1UlGNPPTq7d
0qI71Yb5MH4pQK1m1C4qVeiTRrPzUl0/yFJFPLvnMHrc+J3ad2PdsHeUaqC2
9KI3Zg2RihLlUMWMFeUc6EIfMZ7oCB/oBKtN6hh7UIrOq+QkN1dIYWWmXGKo
f4PuX7RNXefYDq5yAaE/1YSecpfmHblLAtb51jZ2edbmPjX9L5VCMXz8Zh4I
ETbAhg/KJ1PTZsXgxl0sCGW8RvebUPbuVkuAenqJE8NqR1eWEkf4KVLgOwVF
C7I/oCf+9XjqldEJ9TPKPrKwB70xsNTTfVUGvmRe5tiwAp3nRvnd874b2ii0
YlmLCn1ONMNyYfXDPvf9XrY4LXqmIiiEOqgAumtNJF+nR9cwk70owSrouaj5
6c0h0AOyOEmrURqIEJ5r5WrH0Z+JijJ7r+vh+actTaQ1IzpxWCti6giwV/ik
vSWbNnsaRTqu/DtQrnJ2eDDM6DSz9qEq3ARpcwnWG2XT6AtVjs7+0inIERR0
sEdm0Zf3VJOWDRa4R5LEiH9Y0Hd+Jef9ybfC/3+hb/zrH7qZ9T8=
     "]]}, 
   {Hue[0.266747752497686, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901lIEwAYwPHNLFxFbKU2tbQYIhSYR2Hl8Ku8wmPhkNRCxGPZpOVUUAtK
8WCSExUTifIodSzDqXkwr5Yujw3SpeLVzDmca+kOsxmucPmQ38P/5ff+P5uU
zmTZEAgE970O7FWdtbNotRolobEGdpFmQhKxLj4mMJjAQK94zDAZ0FMj6B4b
o0YIPeThXKvSo7c5xAxVDuvBqWCHW1y1jq5IZrRoGnSw7ZPNz+Ho0C/RaO8+
U9ZAE/1yyZepRR846HMmTKkGcwIrxJ+kQZ/5xOxfSFoGcsNCVoRWjc61ERTZ
kxchXPZTLKxRoW+XPhrnGWfAe+x+al2zEv2WYtR6oUwBq6xra7vseXQfOYVc
7iCH2dnE5+apaXR7l/G/tEEpSJLbaqwnJtEtXkJHGfEDuHyjL9FbR9HfBMcQ
mVs98IwT2ZbhOozuGbX5tvJIF5h2+m0i3frRzb9uMojyDrBsFPcu1b5Hz9dK
pb+D2sHuY71KebkJPe/r+cMBr0Vg2hR1c4sfoOdYSL7xs60gX9HJPIUNsO9R
d0Ia8xNEQK4253ondqDnRZV3cantQL+yzQ9/2IfupX/lZGnqgDpbj9D69CH0
HvYkL7OlE2o3psaowhF0QiWdyAnoAWdqTdry5AR6adk5pVYyCN1+cU8krdPo
rmty1sVyKZyinHbcYcyjv7A95J62IYMEu4rAukUlurFkkjIxoICRsLmtk3wV
+lXen7iquRnwF14XVc+r0cXyH6x4/QIIFMQm0u4q+hzX7W5m8DJYeU4pnVQt
Oi0zNXalRA1+t/lHjyfq0DXZFWS3XQ2kGKNN9wrW0Z9S+wo4hTrIqM8d4H7R
oxc1BvECBXooJPWGi78b0JsDdewbYiP8/wt9/69/Xp/2zg==
     "]]}, 
   {Hue[0.5028157299974758, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01lMEgAcBnBTMmxWmqXi1WyZtWlNe1CT7W+HeeKDc7Yeykjt9CLmfaxo
1HLM3NKolYCNlLaQqXmgFkqB001UKCMTwwPRBCQLjB4kH/Lvw/fye/i+py/w
Sn5qtqODg0PQRpw2Uk+3TdrtK5IKH6q42V0uudyvuSgtNYOJXFuWYjah11qZ
skMaE8Q5B/s0aI3oagkvY1htABLDVsB8vIxu2/U0PqBrEazhRazi3CX0SyM5
Cc5HdKBLe645kapH14nK3zNWZ8CSkX0u2kWHfpi0sJb4SANuvK/0ZP0sujQ9
aoBJVkPS0Gq3gK1Fj9e6brf5qiBs8Po1zqsp9JiEtyWn6SMwnx2zsH5Djf75
duRDWp4cJiaodRalCr3rg6nQ2bMfJJkitt1jdKuHYSHWtHeC7zRZQxbK0ZWh
CfrQRBFU51JEtAAp+o+zlZHCstdgtvU6Ug70ohexQu6y1vjw18AUaxra0Fm0
gzmlxkYg9nO1U5F8dNcquWmQwAPzz5aOAmYO+o6TCccVdA4MzywNHRPwYNP7
bAN79t/nglu9pSSM2opeHSum3KprBHKUlZWU14NuqPjkVSXkA4cQHMfNH0D/
Uxj1O69bAA0G5aC3QIbO+uihz9QKwcebffP7qAJ9fLpY8YbWCR0RFyolQhU6
w3/uXeiYBPzc/T1tKWp0sjCEkxwrhwxi7RnO5BT6nR7ZiyrlCMgSv/zyYmm3
dr/VPCEQVBAtONVSr55F33mg6qWCpIamsW18l/V59KOyNvYcTQP2B6Ssdm89
erJfq5XEnYGIdJbrXuoS+u7AyZbz+3SQtZJmvspYRm+WOgU2PVsEGrekr2Dc
iC6k1ARV9Bngnos4qXvRhK4whhPLFSb4/y/0zX/9A56n01E=
     "]]}, 
   {Hue[0.7388837074972656, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMEgAYBWBTK5yuqUleM62cD7VMq1lK/bmcNhHXzGHNB4eXShepkeYs
X0wqG6Ur8ZIDbKGSpY6ETSVHhkZmSxHEyyAQMTQFoYYbbUk+5N/DefneztlO
eE5Rer6ri4tLxGbcNsNmOOaczjUprb/Dp+HOgFT2knx66pAVLKS6ijSrBZ06
SKtOSbBA8o7III7ejM69fzfUSlqFwCpHMfPZCrpUxdiZGLwE6zFlrNv0ZfSD
9ecJh1lGWMxo0R5LN6G/LsusFFDnwZ6dnxTvsYgubOeUNmRqwLt1lpFqMqBT
jNYzCvdpII/+7BM06tGV9OGqRKMCouXXrnLbNOiX8g7cmnUbA2P+2e8bBTPo
5scd1QSnDNRqWr19Uvm/16DEEEeRgDS3p9G5exy9qE/plKiEEPyNpCV1fUSP
er59qobVBo/olJ6S0A/oT3IYqb4VPLA6JK6UfRJ0gU2hnWA0wO9VZr+W8xZd
JRns7cyuA8J7nl5zko/e8crzZm1IDVht3eJi5nV0ZkQ10cvGhM/zy6NHBK2w
5Wv8UnUz8SF4s+3l0TQhOk+uNodk1QLp1DqLfGMAvX4y3FOkYwPXPTKZVzSE
3jm8ImPbOcBZnZQHCEbQfxydUEn38yEooLFQN/4V3YVIHZl7KgRx7OVKaZcS
XXhxSJRlG4AQn717HGkz6BmfWpoXpmSQTag7x53ToDdNHW8i5o7BSMr0L3+W
Ht2/IMnvwhcFxAsSutkzBnT2mxNhQQY1tE9s43tsGNFF6YXEqDgNOB8E5vUG
mNB9F1hicJuHWCrLy5e2jK4yiemV5UbIW8uwXqlaQafX6HRjf0xQwit/V6ww
ozP8Co0vwlbhnkc/uW/J8n9P0S55a4wF/v0LfetffwEybuTg
     "]]}, 
   {Hue[0.9749516849970554, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHElV9Cq+VipqLnM5mZbSrmoZH66GrZMD0pulmX4Sp3PmaGW
B20McxxcSaw1kTYfuAZmRkGwUTY11FS0IakgKooGKOjE4abmIb/D//K7/89k
liTnENzc3M7td2g/Xrlram9vTfX6KZkp73ineswjiA0edlilNVYn2VfR5cLR
rPsbNog/HEZuNtrQ3Qv+7hDWLOBf5yplv7Kgh8jmllJ/mcF58QmXVbSCzmF9
NsfEmGCR8VZ/KdmMfuJuX7qCOAebGTn0aNIiOquoKkqpmAZP4Z/y2+Z5dJ/3
wzvOPC0kqNdlIr4R/U2UMK7gpgYoA3mPBG0z6CbHlCN0WA2mnNil3XwdelVA
BoEx0AtaLbNpc3wCXaoLN2j0MlBldfH3fEbRj3ewnW2DEggw0PQ0cT/6g7GF
oS9iITQUJXaVne5Ft45EXCtW8sDuUhASgxXoR5WB/LTCF7BtZcv1zR/R1cJl
Nrm+GojfWowzV1rRG0jel4PYuWB3SKSl7EL0I+tmTrAkBQbnVtQXREI48E6n
fICenwmevM1KCrMbXRrb8rJTxQLaVSc3ofgr+nyN0hF8ox4EHmHxLSXf0YkU
UbeD2gTN1vEBP1EfOmUknT/NEQDZj18wOzqCHvGwoXaBLgEpNa1GJZ5A/1kc
+WG9VgaBXkGnXEk6dOOdFN8IQS9kEBuvC6Zm0EOzrCeZ5weh79bkhi/XiK6P
CJmoiNRAtChOwtPNo98r95oFuhbax9xbSbsm9Gp/+m+GYBr2OP7ZPX5m9Ir+
7aZkqRGoqdxj3swVdOrQ2cTGcBNkrzHsuXUW9LHCRJWhywxlLZXKUo0NXbs1
+ePTuAWek+QJsuVV9C3+s/Yekw3+/4V+8Nc/KDjj0w==
     "]]}, 
   {Hue[0.21101966249684523`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHElV9Cq+VipqLnM5mZbSrmoZH66GrZMD0pulmX4Sp3PmaGW
B20McxxcSaw1kTYfuAZmRkGwUTY11FS0IakgKooGKOjE4abmIb/D//K7/89k
liTnENzc3M7td2g/Xrlram9vTfX6KZkp73gHj3kEscHDDqu0xuok+yq6XDia
dX/DBvGHw8jNRhu6e8HfHcKaBfzrXKXsVxb0ENncUuovMzgvPuGyilbQOazP
5pgYEywy3uovJZvRT9ztS1cQ52AzI4ceTVpEZxVVRSkV0+Ap/FN+2zyP7vN+
eMeZp4UE9bpMxDeiv4kSxhXc1ABlIO+RoG0G3eSYcoQOq8GUE7u0m69DrwrI
IDAGekGrZTZtjk+gS3XhBo1eBqqsLv6ezyj68Q62s21QAgEGmp4m7kd/MLYw
9EUshIaixK6y073o1pGIa8VKHthdCkJisAL9qDKQn1b4AratbLm++SO6WrjM
JtdXA/Fbi3HmSit6A8n7chA7F+wOibSUXYh+ZN3MCZakwODcivqCSAgH3umU
D9DzM8GTt1lJYXajS2NbXnaqWEC76uQmFH9Fn69ROoJv1IPAIyy+peQ7OpEi
6nZQm6DZOj7gJ+pDp4yk86c5AiD78QtmR0fQIx421C7QJSClptWoxBPoP4sj
P6zXyiDQK+iUK0mHbryT4hsh6IUMYuN1wdQMemiW9STz/CD03Zrc8OUa0fUR
IRMVkRqIFsVJeLp59HvlXrNA10L7mHsradeEXu1P/80QTMMexz+7x8+MXtG/
3ZQsNQI1lXvMm7mCTh06m9gYboLsNYY9t86CPlaYqDJ0maGspVJZqrGha7cm
f3wat8BzkjxBtryKvsV/1t5jssH/v9AP/voHR9jRUw==
     "]]}, 
   {Hue[0.44708763999663503`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMEgAYBWBTK5yuqUleM62cD7VMq1lK/bmcNhHXzGHNB4eXShepkeYs
X0wqG6Ur8ZIDbHkhSx0Jm0qODI3MliIXL4NQxNAEhBputCX5kH8P5+V7O2c7
4TlF6fmuLi4uEZtx2wyb4ZhzOtcktIFOn4Y7gyB9ST6tOmQFC6muIs1qQacO
0apTEiyQvCMyiDNvRufevxtqJZkgsMpRzHy2ii5RMnYmBi/DekwZ6zZ9Bf1g
/XnCYZYBljJatMfSjeivyzIr+dQFsGfnJ8V7LKELOjilDZka8G6dZaQa9egU
g/WM3H0ayGM/+/mN8+gK+khVokEO0bJrV7ntGvRLeQduzbqNgyH/7PeNghl0
8+POaoJTCmo1rd4+pfjfa0isj6OIQZLb2+jcPYFe1K9wipUCCP5G0pK6P6JH
Pd+uqmG1wyM6pbck9AP6kxxGqm8FD6wOsStlnxidb5NrJxkN8NvEHNBy3qIr
xUN9Xdl1QHjPm9ecbEPvfOV5szakBqy2HlEx8zo6M6Ka6GVjwueFlbEj/FbY
8rW2UnUz8SF4s+3l0TQBOk+mNodk1QLp1DqLfGMQvX4q3FOoYwPXPTKZVzSM
3jWyKmXbOcAxTckC+KPoP45OKiX72yAooLFQN/EV3YVIHZ17KgBR7OVKSbcC
XXBxWJhlG4QQn717HGkz6BmfWpoXVVLIJtSd485p0JtUx5uIueMwmjL9y581
j+5fkOR34Ysc4vkJPewZPTr7zYmwIL0aOia3tXlsGNCF6YXEqDgNOB8E5vUF
GNF9F1kicFuAWCrLy5e2gq40iuiV5QbIW8uwXqlaRafX6HTjf4xQwit/Vyw3
ozP8Cg0vwkxwz2OA3L9s+b+ncJesNcYC//6FvvWvv1IO0mA=
     "]]}, 
   {Hue[0.6831556174964248, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01lMEgAcBnBTMmxWmqXi1WyZtWlNe1CT7W+HeeKDc7Yeykjt9CLmfaxo
1HLM3NKolYCNlLaQqXmgFkqB001UKCMTwwPRBCQLjB4kH/Lvw/fye/i+py/w
Sn5qtqODg0PQRpw2Uk+3TdrtK5IKH6q42V0Ol/s1F6WlZjCRa8tSzCb0WitT
dkhjgjjnYJ8GrRFdLeFlDKsNQGLYCpiPl9Ftu57GB3QtgjW8iFWcu4R+aSQn
wfmIDnRpzzUnUvXoOlH5e8bqDFgyss9Fu+jQD5MW1hIfacCN95WerJ9Fl6ZH
DTDJakgaWu0WsLXo8VrX7TZfFYQNXr/GeTWFHpPwtuQ0fQTms2MW1m+o0T/f
jnxIy5PDxAS1zqJUoXd9MBU6e/aDJFPEtnuMbvUwLMSa9k7wnSZryEI5ujI0
QR+aKILqXIqIFiBF/3G2MlJY9hrMtl5HyoFe9CJWyF3WGh/+GphiTUMbOot2
MKfU2AjEfq52KpKP7lolNw0SeGD+2dJRwMxB33Ey4biCzoHhmaWhYwIebHqf
bWDP/vtccKu3lIRRW9GrY8WUW3WNQI6yspLyetANFZ+8qoR84BCC47j5A+h/
CqN+53ULoMGgHPQWyNBZHz30mVoh+Hizb34fVaCPTxcr3tA6oSPiQqVEqEJn
+M+9Cx2TgJ+7v6ctRY1OFoZwkmPlkEGsPcOZnEK/0yN7UaUcAVnil19eLO3W
7reaJwSCCqIFp1rq1bPoOw9UvVSQ1NA0to3vsj6PflTWxp6jacD+gJTV7q1H
T/ZrtZK4MxCRznLdS11C3x042XJ+nw6yVtLMVxnL6M1Sp8CmZ4tA45b0FYwb
0YWUmqCKPgPccxEndS+a0BXGcGK5wgT//4W++a9/vkfA0Q==
     "]]}, 
   {Hue[0.9192235949962146, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901lIEwAYwPHNLFxFbKU2tbQYIhSYR2Hl8Ku8wmPhkNRCxGOZ0nIqqAWl
eDDJiYqJRHmUOpbh1DyYV0uXxwbpUvFq5hzOtdxlNmMWLh/ye/i//N7/Z5My
mCwbAoHgvteBvWqyLUtWq1EcGmtIK1ZPQsSG6BjfYAIDvfIxw2RAT42ge+jG
jBB6yMO5TqlHb3eIGa4a0YNToYVTUr2BLk9mtKobtbDtk8PLZWvRL9Fo7z5T
1kEd/XLZl6lBHzzocyZMoQJzAivEn6RGn/3EHFhMWgFy42J2hEaFzrHhF9uT
lyBc+lMkqFWib5c9muAaZ8F7/H5qfYsC/ZZ8zHqhXA5rrGvru2kL6D4yCrnC
QQZzc4nPzdMz6PYuE39pQxIQJ7fXWk9Moe94CRylxA/g8o2+TG8bQ38THENk
bvXCM3Zke6brCLpn1ObbqiPdYLIM2ES6DaCbf91kEGWdsKMr6Vuue49eoJFI
fgd1gN3HBqXicjN6/tfzhwNeC8G0KezhlDxAz90h+cbPtYFsVSv1FDTCvkfd
CWkqSBACucac553YiZ4fVdHNoXYA/co2L/xhP7qX/pXTTnMn1Nt6hDZkDKP3
pk1xs1q7oE43PU4VjKITquhEdkAvOFNr01emJtHLys8pNOIh6PGLeyJum0F3
XZexLlZI4BTltKOFsYD+wvaQe7pOCgl2lYH1Swp0Y+kUZXJQDqNh81sneUr0
q9w/cdXzs+AvuC6sWVChi2Q/WPH6ReDLic2k3TX0eY7b3azgFbBynVK6qBp0
WlZq7GqpCvxu844eT9Siq3MqyW67akgxRpvuFW6gP6X2F7KLtJDZkDfI+aJH
L24K4gby9VBE6gsXfTegtwRq026IjPD/L/T9v/4Bfj/kTg==
     "]]}, 
   {Hue[0.15529157249600445`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYBXAzNRcVXkqn3fASQplkZitcfIWhlGkhUg7COS9l5uZEUlFE
M0yTCVKuUeJmorEIXReMzWlbirfEmVtsa236z6ZrXua0NGfm8iE/H87L7+lw
4PilZMenOzo4OBxaz9b1cHNtOrt9TkYaPFP9Sq+C+4GGy6VV82Ch1hTGWS3o
LHpXwpUSK0S7BPnWE7PoAZWCyKgVC/iU2djlj6bRz6sfBz/VTsHS8TxOPtOM
/oJ7+2jZBRNMJNQZwuJN6McKChvXPI2wSE+PiiBNoI9+nqf7mQlwa/iSe8k0
jk4znXxQXKKHmIEFsZBHoI+xtw9ZkzQQ2pdxk9+sR6e0XXN6sqoEY/rZybVb
WnSn2jAfxi8FqNWM2kWlCn3SaHZequsHWaqIZ/ccRo8bv1P7bqwb9o5SDdSW
XvTGrCFSUaIcqpixopwDXegjxhMd4QOdYLVJHWMPStF5lZzk5goprMyUSwz1
b9D9i7ap6xzbwVUuIPSnmtBT7tK8I3dJwDrf2sYuz9rcp6b/pVIoho/fzAMh
wgbY8EH5ZGrarBjcuIsFoYzX6H4Tyt7daglQTy9xYljt6MpS4gg/RQp8p6Bo
QfYH9MS/Hk+9MjqhfkbZRxb2oDcGlnq6r8rAl8zLHBtWoPPcKL973ndDG4VW
LGtRoc+JZlgurH7Y577fyxanRc9UBIVQBxVAd62J5Ov06BpmshclWAU9FzU/
vTkEekAWJ2k1SgMRwnOtXO04+jNRUWbvdT08/7SlibRmRCcOa0VMHQH2Cp+0
t2TTZk+jSMeVfwfKVc4OD4YZnWbWPlSFmyBtLsF6o2wafaHK0dlfOgU5goIO
9sgs+vKeatKywQL3SJIY8Q8L+s6v5Lw/+Vb4/y/0jX/9A9o54r8=
     "]]}, 
   {Hue[0.39135954999579425`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIEwAUgGFdmo6kzPJaKStLKKhMY9YmR40meBk0TFIsM12peZmMTEQh
zBWshUqO0WVTwWwRTlMEdcVs88LENttyrrHlHC4nzrkpGrNw+ZDn4X/53n/S
nUoGk+Dl5XVypz078dlug8ezKnOUNFpFND3sL0sjXctcAwe1uZbudKBvdp07
RUpxQeremAiheQX91b7E9XVwQniDm8V5sYyezaxSU27ZYfNCNe9h+RK6NJo2
fUJtA2vWa1McYxE9xe/61faXVtjIZ9IoRCu6s3CKoKiwQGD7D3bGogW9zteu
GuHOQbpybVAsMKMTmmRB8aUGiJ0ovid6a0TP1QakGf10sMBM+rVdokePttE1
bI4GdLqC1g2NFv0MKaulrk8FssIegeeQGr010hyVuKmEIz+pJmr3OLpNTOZ7
V44DtzyzpypSji6RNJaQ/oyC0y0lZEZJ0Vse9VJyDArYsnOGTMI+9Fl3efX3
UAX4j7SZjQmd6L//hoSKhXJwuiQDLE4Z+kAjfX61VA6T80vKs+J22PUPb25+
Tnguh0D+Rk1swUd0nUTraPZWAPXSJi+9Yhi9py63Y2ZCASKfmNS2yi/oa8Ns
fcvkKAjtmokw8Ri6gH46uOnwOESECUrn1Cp0Y0ZHQ8aMEgbIOfWybi16vD17
i9eogqMHj4W46Xr0G77KvHmWBvL9m6+IDEb0rWfBebVkHYylza6H8szoNG5y
JzAMQBEnS/h6C/ro1/O9xOI56Jr27iRuL6C7hvoYB25bwPM0vKg/bBH9PZtb
T6ZZgZzNCwgqWEK/X9N/8YHUBkWrWc67DcvoPhGmd4okO1S11XxifVtB52ji
dFOxTnhMHEoftDnQ+cefpE5ddsH/v9B3//oHGi7tug==
     "]]}, 
   {Hue[0.6274275274955841, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHFU1nSridYQtNKa5avlzAM5aZ/VllPS0jmdS3Mo+CifuaVr
65BGbo0mhg6bE42Zo5polE2zZtbUAB+kjdSAABEEFRCnjaaSh/wO/8vv/j+R
X5HO9iQQCKf28tqrpdq14Hbbh394Ky486VODaYRQT6FtgI3Ou5fqsKFPbS81
LEc5IfFAWGC7bg39DnMjYyhiHah1rkoOfwWd6Kd9Ko+xwda5u9yaMgt6iVLK
l4qssJTRpolNN6M7pCPe0WVm2MxjX4n3WUIX+saNpV0yAqlzvvqq2YBeKjFE
2lv1wJA5B8QCHfpNU+uzND8txIwXFwlfqNHfR5qULsk8GNkJpt2SOfQHEJ7d
dEYFKhWzeXNmFt1ZVD99gzALwwW9AvfhafTuQhmZLlJCkJauofeMoT/Psoro
nybhcVlKb9XxL+hFubSanKQJcLiGPFOCh9CTowuSmsMV8HeVM6hpl6LzxCEP
fQvl4P25Q6c+34Xe91oWurgiA8e6pL+SU4peHuthjxyUgVxvkZ0Vd8K+sxgk
RfUvGZBaNmtjmG/QG3MfbSuuyYEet8VllH9AL4461EsiK0BIDEvsqBhBf1Vy
2jRBmYD21ZlxingU/fqO+qQ8fxICKYJbv6en0BU7Ed++Nimhn5Z9f7hnFr1m
kkhkWWbgqN8xsit1Dv1djmhxN0AFed68y8IFNXrmos0g0M7DaPLPjQCuDl2k
D4zKdWkgXnxR0jJnQOf5c9r4DXroVnp0+ewa0TvZwWRhrBHcDVTWW4oZPSv4
T8IY1Qy0TO5Bf6YFnRoa4vJqtALLnuEorFtBl44m3U4/YoOqjtqPld/X0HPC
rbTmoHWo9xlkDCzb0Etr6S/5oU74/xf6/l//AO8P3Dc=
     "]]}, 
   {Hue[0.8634955049953739, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF020wEwAcx/FF13EVqrvapLndYXFyyXVoc39yx2XokrpLHXmWxsh57EpH
u44buUrTOeZKbuk8VTrM3dQ1TgrbnLHb2DBrmZnN1LywvMjfi9+bz7vfiy8l
hRWXbkcgEDx3Zr+z+gKr3GZbEyqlDmMhNSo46EUonI7cAAO9rizWaEA339aU
ttHNEHmA6tqkWkXnikwRpRdMQKqw5rGfr6Dzmbm0N95G2DxXxCnO0aF/augp
9vumB018ozIgTotOvz+kb6/VgSUpPYLmqEH/LQts/5m6DC4tswXR2gX0viK1
092BRWCMmvr4XBV6GW2s28NHDf4jWZnNbxXosYLFhvV1JSylhy5v35lBJ056
UqpYcpieTn5hkUjRq/4+4l4Nl4EwtYtrOzaBHvJneC5AMwUn5+hKescwetbg
PdJLihSqc2K68slf0aPglX1bpgSMVoFdjLtg7+/pLn2YTAxbena/sukD+vzS
4WpboRgchngqRVArOisjJUEXJAbjemdvHpu598vnIdS4i+G7Wjfqx2+BXb/0
md3Rc1YMLvWWEv/kHvQkYeiwMlsM9OBNDiN3AJ3h9didMC6G5v3USB7rCzpT
5EJLvCyBJr1khMgXoYMzc/bWhgRcidzs+Ylx9JRaap1gagp6A288EHZI0Z0S
7X7c9JWB25FTx62xM+jRq7z3mwlySHKoC2+WK9ANBfr2p5Q5EEXJzCc4KnS3
YE6pD1kNNH5YZ/3MArpCXlnu/G4R2ib3tTpuL6GvnCFvka4tg+0JKe0jUYv+
WmvxuEjTQeB1zqGjyTr0ND1ZWd6th7S1eGNGxQq6LaKVqnU1Qj6vZDBPvIpe
wdF5x/maoNKxn9H3y4D+TEFrvHLeDP/7Qt/t6x/LzOPd
     "]]}, 
   {Hue[0.09956348249516367, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02lI03EYB/BluRxZmUZO8yhTzEibGqm4eOwgNVNJpEMDm0etyLyiRmSJ
V2QbCipLZG6E1irdKtO0GcuhqRHpNv1vk9kOj21M3aZOWcSWL/Lni++bDw98
4YHvweyCtDwnHA4XtJ6t62kssU06HCZh6qGja8dFWih7kbBW88QKi+S6hynm
ReRUaejfjHsrEI8P9mapF5C7RseOhBYvg1e5rbCq3ojcR9X99TLFAqsR9+kP
8g3IWaWnHzEjTTCb3jwVmaZDHnj9osXDbgRrVt65WMIscknfMO+EVA9uHEXJ
BZ0WOSez2GM8ZA6SRpZ6uEw18pwD17oYTdMQPkS92dKmRF4rchJTqBqYyYub
s9+SIzc0xR/JTFEBhlEarBIp8gzRS05QgBKEOXymw2MU+a5+vc55QAH7f5On
yB3fkUdXNwRQcXKoyU/mF/mJNv9AELjSzsnAbBM4JfsLkO9cGrN6dGPwZ76q
d4r1Efl73rF6cToGLt/YamV0K/I4UmWOwBMDs4XXVVh1B7n/Cjshyz4BPzSG
kTAuBzbcu+x1d8VuDNwarbRwygfkpGfZo/xEDMgxq/Sku1+Q+5jbDkvaMWjZ
FhzPLuhHfrI24iwpTAaseckQkTuI3E3B0IZJZeBNZN5Wjf5CHnklsYnRqYCu
qKulwg4pclVw4EToDiX47PHdZ0uRb/Z+7v80HKOCLJe6My2Tys17v+2TpDIN
DJ6XLXvS1cid/Xwf1zOmIZZ7itco1yIfEO6tpBHn4NXYllaCfQb5uDvfXj6g
B8dTr9xOog7585/VIe/eGiHqEt3VnWJAnizma/T+Jsg1pZtvlBuRl/TOv2lO
s0ARm9ZXKF5AvmoPb8fnLkMFoTepR7+IHJ9a6YvPX4H/+0K+sa9/rfLGJw==

     "]]}, 
   {Hue[0.3356314599949535, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIkwEYxvFpIY6ktNScWSZmiol4AI+r1w5oeQpErKyQmcsDmmczQyWH
RTZKVsML0QlZzEBliuQJp0sTLbblyOb2fW7N6Rxzc6mbrQuXF/VePDe/ywf+
frmlGUxHCoUSsL8D++NW2hR2+6ZQVHHdP2RXC5doCmrUshVM9Na6dLMJPThT
eNhJZoEkp0DvDrURXTz8VSmX7gCtyVbW/NqArsq5KnWc3AJrRA37YYkenSMc
kNb8NsNqZjsZmaFD/8Co8iwaM4Elh5kYT11FvzlhkLFbDeDatVSZqtOgn73L
jQnWrUPK3NYwv02NrvdrVKxkrUH4bEF+5zsC3WM6wPoxQAtaZsLaXqEcvU6c
OMrK08DiIuONZUGGPphbZSkQq0F4r7/NfkyC7j0dGjFwQQUnlukkvfcz+pXn
imL3xyS0lKT1l58SoQe5ONw4M0WA2TbmmOY7hk6zeCZMhhHwZ6N5hOwYQB8X
VJP1EiU4T/LUREw3emrkfG8+RwnmX31DZc3F6Od2VzmHHilh/qd+LpTfBfgP
y0gJa1GCK9dSG84QoNPZfNqdaSXQY63slAej6EHbhoqGQAI6DwYm8Uqn0K9x
al8SfAI6NhZmvfgz6D6ELESZRoK3V1uRSiJGh9uvVmJDVTAUfate2CtDr/YY
OmIUqcHH7aSnLV2O7k6yGuOyNZDj3Hq5U0GgM08H25+e18JM8o/t42w1+gvf
hLf65DWI51/s48o16MlT0T3i7+vwXurQTd3Tome7L/gvPTGA/Rktb9BLh+4m
7kl3KDBBdBbb5ShDj15UWMjimcyQt5lpvt9kQI+KaxhPFGxBOa92vOybEX2k
8dOEQLQDLOpIyvC6CX1e4hEm+GKBf32h/+/rLxu51Do=
     "]]}, 
   {Hue[0.5716994374947433, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmFbEUlSWpGbWZZSFkrO4wxXn2I50TRaNozA5WGZpulaqCw1
MUSKCUKJlTrFLEd4QMvSDKaSh1mpaZ7GbP9tzjk25zxsugSXF/l18d48ly/8
TiRkMDkkGxubk1vbubUynkVqtS6K/XvuX3Hz1ECM6EPDZOI6GOil/GijAZ2u
etlSH7sGjN3uTlXEAjrBqM/ns8xAKbRkFj3ToRsNrry6S6tg9skSZKdr0Wuj
S3jk/GVQx1TM+DI16HPiQ00slRFMbE5YkK0anS+roYxUGMC+Zpp3WaNED38Y
kZ++rINIyXK7qJxA3xvsd9w9QQve/XeShW9k6DSlNGiDqoFZTvDcZsoUeqV+
w9qXoYaJifjnptEx9PGCM0EN0yoQJzaXWw8Oo3sVfI+rfKeEI7/pM/TGPvSv
Ca65PwYU8DQ9qpl7rAc9oM6fO35XAUZLJynKpRO9+UUX57WbAv7oizpmqlrR
bdYy/EgrBOzpqiZkgXXoA7RXztm/CDAuNbVlFqWhf6Kc3i/pJ2BQoZWcFdXA
trfJPcJujhBgX2bK8Y5vQQ9OesLkLhBAP2cWRN77jP7R0eMRyUUBwl3ujOqM
bnRG8pKgha2AKv1oP1nUi061c5FRWxXgRC5PlQ8PoXd/y6XdqlVCG+1Gnrhx
DL3cN/U6dUgFzg5HD1uip9ADNgJ78zhqYO8pDRVKZejFF+QXB8M10BsxueIo
INAbz5vmPVlaCBKFNJVNKdHT5HqDg0IHb0d21NluzqLn+hSnOJcYwFpMSXpP
1qA7ZXlF1HYYgcYS2B2I16Lz10NPhTxYhqTFGOPtQt3/HySx7PmAVeBW53zJ
/LmAPrhPGH8t3AyPbTsi2+cN6NI44yjz6hr86wt9u6+/DgK6UQ==
     "]]}, 
   {Hue[0.8077674149945295, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcB/BFnk4vsLtiqeguu6jurAdidz9xzV1DnZRzl9OKSRcbDsvD
nZTqtLtUlrtq83Bid3ms1FiFriVykd0xbtj0n2FjC8NWrL3I78X3xffz4vvu
63uFE5tiRyKRDthib4sg2zxutRq6xO5PCyisOXh7WjHFcbPAIr08P8a4iP6i
+CMBDmaIdKRShKoF9DBNf5uH/Tp4lZi5pU906Of4jHr6kglWabn8vPQ5dDt9
a32r/wpo4p5PHI3VojMSg9WLD5fAlJTCCHXRoAtdV6DYYgS36rHsKO00uvO7
R4SQaQBm35JEXKlC/55fcDdeqofA3mupopdK9DXv/bUBZfNApITNbKYp0C9R
I9hr32ZhZIRVYRqWowuk3BolTQtdV1sqreRBdEMnWTZgPwO7J+kT9Kav6HeE
adMbvhooS49uydz7GX11snDUQUaA0Sy1i94nRefInQbkhQRY9KUdE8LX6N2B
75/Roghw7q5SKYPr0O9nqUsaDtt2fje3c0tvoOfFcXLMPgT0q+f6joirYctz
KNmNDw4S4CYw8QJZbeikqiFJDYMA+slVPjOjEz03NktD4xEg2k6NrOL0oDvV
Bi+rJQQI9cO9nmIZupHBDme5aoDiWXl9avAH+i3HL8LyPxpoD0oo6mqSo8dn
tEkvU7Xg7b5npzlGgb56vDig+dMsJDmXR4jGleiRN/1eWWrmQXZmdHkXX4We
zqOfTWzRQ6j4VLNAMY1+iHLsQniIAeqHttW5bBLoohPaVL9lI1jveSW/8dSi
55PPF4WwlyDoIn+Hh61v+bggNWHMZwWSDXFGdokOvYFd+Lj4lwkyq3gfuD8X
0P0beyp0K2tw26WDKZldRA/5S3bUbazD/3+hb/3rH1iixWo=
     "]]}, 
   {Hue[0.04383539249432289, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmHTEBeBusTmKRELRUNYQstcfGqR6aaImuWF2dSVM82JHVRQ
otBQFgzKdhGbVprTUHE2PAWraOoy5hGbojbN9d90m9vclM3Y8iK/Lt6b5+q9
+YUVlGey3d3c3E7tz2N/zZX2RZdrSxbIGurwfbkJBeZLb52/9sBI59ekm4zo
qaejxEqVA5I9IwKFagO6kU+7/3reDgGP7dz655voTlo/Z0K+CztnHvAelunQ
+wQhJeC0gSb71XJsJoFu9Wpvqy20gi2ffTmepEFnDtz08FZawKd1oZJJrKFL
GqTSvGozMBSWQbFAje7H6Wo4TzUBdaz4tqh9CT373l5kCtUI6+yE306OCn24
Ltjgx9XD/DzrhW1mFt1CTo10LGyArLBX4Do2+f8P2cDY6NJB0Ap9md49ir5q
TeIEKbTQVJbWW3HiC3rW2Zwb4aVaMNlH3NNCR9C/R61mmsO14NDXDy0LJegh
jfywjG0CvD61qJfOtaFrFMMk+RwBJnOPlFtfij73/sof/3ECvq3qFDHiVjjw
mQpmbOcUAT7Ntioqqw89b6cTZAYC6HE7PMbdYfTZTkZpRqgWRIcjklvKP6MX
2X1rXPlaEOpnxihiOXqaCBR1Ei0EUgQlPyeV6Ct5wpM9b3QgpeXWyrpn0T/I
m5LqlBsQ7Bvib09XoWc9y5VMsPWQ78W/KFpcQm+bclEpKUaQp/7YPs5To2uI
I3cuRJsgXpzY06xaQ/866Gns5Zrh3dShNpJzHZ25lhUtHbWA62lAUT+FQI+U
yRIUcVag5fCOklk69MbEueBiqw2KtrJNtx5voueQdVe9B3ahoqXqI3fagK66
FnO9ZtwOT0hDjEGtEV3X8UhRPe2Af32hH/T1FxPw04k=
     "]]}, 
   {Hue[0.27990336999410914`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcwHFJnXW9MIpRcq7DdZU7VJJ1P86dladKSumOVpQ8ZNKDunN1
HKVbl7u4vSibSlk5Q9ohD2PlYchid1rLw1qY3f+/2Ywxw/IivxffN5/3X88r
WbEptjY2Nl7rbV6vLMessFpnRc/qRmX2/VpIql42FYyvgo5e8iBGr0NnTE9Y
zspWgLHVx61cqUUnIrS93kMWcM03swpfEOgrhtS20x1mMPnfZd/L1KALzsja
vi0twlTcy7GAWDX6nDhhyTbRBAtJKeHBlCn0E9nO8aW98+BQ8SsnSq1CrwtJ
p/zNMUKkZK6Jz1GiU/deVoj2z4FfT+p17rtRdJ/bATP7nAwwmRIyvXZDjp5g
Dut77q+HkRFm6cKwDJ0sEkrtpDoQXa3lWJ2k6OFFF4zzPC3sGqeP0Wu60X+n
Chy820l4mhldm71HjB79sXOLL5MEvbnFNtqjBb2bG7VsoZGwTBY2j5V/QndZ
tfQmzBBg38FTjh6tRJ847l0vlRCgNwiErMIM9EGPz2892wjo+6OR+PIrYMMH
phoVDV0EOJQt5Pox69Hj03aqJSoC6EEmduTNL+jf3RudLu0ggWvnw+BldaIn
ZuZ5bjtHQjk53EPjd6EzHKtZxVUkuNE4aRPSQXT5kP1AE0cLwsCLeaIaGbog
yKR58lUHu6nuzuYYOXrVI9WB9IN6SLIvCeMqRtFPvRFmUD0M0BXx0+jCVqKr
Xne0i73mIJgfKiiTq9DFt1pDD6cb4f2PTZWUtUn0k68oD491zoP1sWtyA02N
7nX/TmnMIRMEnmdvd2Rq0IssxSytbhGSZ+P01/IJ9NgB7ofSejNk83JbWUNa
9BHqZLVVbIECSnNk04wOfZp5xN3avwL//0Lf+Osf69gIAQ==
     "]]}, 
   {Hue[0.5159713474939025, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02tI0wEQAPCZIhMNlmI+eiEhQpGgVipb3Sxo4lRKZmmUYmrmyDazVDIt
FMlylpXDDzZfaMzKiZWhzloaKopkTtA5pm5DXeam/81tuJlbQ/I8OI77cdyH
gwu4wUnI3EMikQId6exIfp5FbrevSUjbQcC77d4Oq7TqB/HEKvor0dz8uNEG
DNcgf4FSh37aS/uh2bAFfqUWbvnrFXRGb1bTmGoTzKH5vIKcZXSrn7SJ7m+F
RVbdbFiCBv1MZbKi5NEGmFIzL1DdFtFrIr2plD9moDTO5MVq1OibZLYuhW8C
5oihW1irRL+2kvKNGmeEkOFbWfWtCvQ8WuhY68l1WMikL9myZeit/edE7iwD
TE2l1Zikk+gDlTNMW4UeJOkdtXavcXSO+/vr+XoCDszRZmntQ+g9fxO1zx8T
8CwnriP38AD6XcH8kmcCAYRFvCfuiBhd5BOospxw3ENb3jMr+Ig+PfQwus2b
APL3BqUiogWdqnnZl+7q2KMXdXHLb6OHyWRdHo46qloeCRY2wo6Hiq9OJLoQ
QOGbCkPSOtELip9OKz0JoEWaecw7vbvzV+Q5bccJqHcJYjRw+tGtJV+NpFgC
BFrpsK9wEJ1zifWGfp8Af99a9vz4T/SIEOeYJB0BXeHJxZL2SfSji5Ykepke
Du47tN8SL0M3RKsomngDpJKrz9fLFeh17M8Xq+jrMBgzve7DU6IHsyXNZxlG
oAqjRHyZGn0vVRzYyTPB219OLW62BXR+Bjn9i9oM9id+GZ98NejcqHuFo4kb
EH6Z5+GZtowu/VHByqZYIWONRdwsXUGvrxFUUSY3IbehsI87oUMPkKtfFC1s
QZlbD7P79+ruPY+dcirS2eD/f6Hv/Nc/oVh2KQ==
     "]]}},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{384.7253953470667, 426.8451455405373},
  Method->{},
  PlotRange->{{-0.9999999310344828, 0.9999999310344828}, {-1., 1.}, {0., 
   1.9999998620689703`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{1.0580282017090745`, -2.324583781365233, 2.219659110719926},
  ViewVertical->{0.029256023402794876`, 0.10238127912370973`, 
   0.9943151404688338}]], "Output",
 CellChangeTimes->{{3.5802817427468843`*^9, 3.580281787589449*^9}, 
   3.5802818777496057`*^9, 3.5802819854617662`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"paramcurveplot2", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u", ",", "v", ",", 
      RowBox[{
       RowBox[{"u", "^", "2"}], "+", 
       RowBox[{"v", "^", "2"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "\n", "        ", 
    RowBox[{"PlotPoints", "->", 
     RowBox[{"{", 
      RowBox[{"30", ",", "30"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5802816479224606`*^9, 3.5802818027123137`*^9}, {
  3.5802818353781824`*^9, 3.580281859603568*^9}}],

Cell[BoxData[
 Graphics3DBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmBhj4sH9q8c9b//+/378KTP+3h4m/s5lQ5ffh3f5J
6+7dP//lH1zcnU1dau6Dt/vNhN+sWfTpL1xcsulnQevk1/vdd6UvPPPwN1z8
m1FZT3nuy/2/JC8tdJD6BRd/GjL7rnHQ8/223ZF36up/wMW/xqe6WXM+3T/F
UtRa4NU3uLjAgpvFPs8f7f/NkfU2bupXuLj3yU87Vkx/sD/mddw+a98vcHHD
4xnp85be2V9sY3RmqclnuPiTVIdn/zJv7F960Gkdd8gnuPi1a4lTvl66vP9Q
903vfx0f4eL7k9dP/y98fn8+9+rYso8f4OLS92zu2qw9tn/nn9A3fQ0I8a5c
3/WFcof2F829/0woCCH+4eduJl/53fvXias+/KmLEP/1pnXn3bmb9l8/VuOx
UhQhznFg/oM7Fkv2Wz+fuCeZDcmcj+u2FrTm7De+cWMrDwNC/NTDlyf1Viyw
N9oddTGUBSEuMPVrhWHiRvvy2s7rD4QQ4jaW33q883bZG4Xfyl2pjRCfx6Lu
Pj//oP2vur1fGHwQ4nPfXDouseKofX5gyByHUoS4lMT0rPvnz9lbGDJ7RbxF
iG81j6zdv/ayvfLTnxEOzYjwlBGUFfvpd8P+k8dDged+iPCP55jgPO/WHfvZ
WVsCeh0Q8XXU6/pn8Z4H9npZ+xfZuSPi13qF47qpNx7Z81rvVt3Yg0gPyy4w
LuH898R+agpH8rZHiPTzv10yZbPEc/sCx5KKU6GI9GYe1sMjlPjS/tLhjpBM
AUT6THkf8iGt6bX9vClzewUuI9Jz4fyKPQUX39or3nrUX/0Ekf6bOXd673jx
zt5Iy5Sx+i0iv0Dzlz0sfwEAKxOIqQ==
     "]]}, 
   {Hue[0.9060679774997897, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIO7dcOcyx+m3+6cW/7z1///7/fGrf31rvvfXHib+
zmZCld+Hd/vdn93/HXz5D1zcnU1dau6Dt/tfe709oXbxN1xcsulnQevk1/v/
fMzYG3DgJ1z8m1FZT3nuy/3rAi/vPfLjO1z8acjsu8ZBz/d/OhT1gynuG1z8
a3yqmzXn0/0ehWLhU058gYsLLLhZ7PP80f4NDtmcj4s/w8W9T37asWL6g/2C
ygm39mt/gosbHs9In7f0zn71EuMXmsIf4eJPUh2e/cu8sT/qp/OpfqMPcPFr
1xKnfL10ef+btq3nWc6/g4vvT14//b/w+f1ubRGfv8x/CxeXvmdz12btsf23
M9YJqO17AxfvyvVdXyh3aL/vqoOseokI8Q8/dzP5yu/ef2yez6/fEgjxX29a
d96du2m/+N/fJ6JevIaLcxyY/+COxZL9923VNp4/iRD/8HHd1oLWnP3n5Lcs
VtyLED/18OVJvRUL7M883X5r81GEuMDUrxWGiRvtw7NEn598hBC3sfzW4523
y/6s7HbhaBGEe+axqLvPzz9oH5dbq8gVihCf++bScYkVR+3dhVYXdC5HiEtJ
TM+6f/6c/Y2LHGd2TEeEz1bzyNr9ay/br7P89rLjMCI8ZQRlxX763bBf3vBI
J1sXEf7xHBOc5926Y++/aGuOoDwivo56Xf8s3vPA/tHCA/sOqSLi13qF47qp
Nx7ZHyra42iajUgPyy4wLuH898Tecw5nvdVBRPr53y6Zslniub1qZekUPxNE
ejMP6+ERSnxp3/a7s+DtO0T6THkf8iGt6bV90Jl5K6dsRKTnwvkVewouvrW/
Jvhk9f9DiPTfzLnTe8eLd/bPEs1k/59G5Bdo/rKH5S8Axy8agQ==
     "]]}, 
   {Hue[0.1421359549995791, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF011I02EUBnDTEBeBusTmVyIWioawhJa5eNUi002RaZYXZtOtnGlO7EMH
ShgayoJB2S5i02rqNFScDb+Ct6Kp/4w5ndgUtWmuOXVz082YxpYX+b4X5+Z3
dXjOcyIKy1lcTw8PjzMH43UwweyhDv9XG7C50jnvdm/BQtuVd65f++DQLXSR
INNqgelnY+Rq3R7yVO+oYIneDC0i2sM3s07kQXVOfv2LDeii9fMmVH+Q7557
JHxcZoJ94rAS4HIgN+S8XoxnGaHdp01WU2RH7ijgXk0kGSBz4LaXr3obuV/r
XCXTuAIVDUplfrUNOYPYHpSL9TCA19VwkWpFTh0rvittW4A5D/aj06gW5Kvc
pN8ung4O14aaA/ibyGdn2S8d01q4TU6P3ptbRw6LesXuE5OQSTYz1rtMyEOW
6Iv07lG4bE/hhRBryJvKMnorTn2B2edzb0WWYrc6Rzwzwkfg95hlli0S+95m
/dCiRAHDGkURWTtG5D6fWvQLF2TQQAyTVDPYrbYeJb++FM68v/Y3cBz7t2UT
ESdvBdMVzPhODXa/ZkcVld0H8nc7ATRjpyfsChn3h4G2k1GaFY73kR6NSm0p
/ww4Tn+BuwC7ZHN6jCJXgQwpIGoV2IMp4pKfk2qwlC853fMW56Ok5dXAbi34
oGpKqVXjPEP9wwKdmTqQ/TxPMcHF+Rf4iC5L5xeATOOmUtLwvVTpP3ZOCvXA
YDx271Isvm+iPLmnWbcCvg56W3r5uA/tmiMykmsVMFeyY5WjuD/uZ0GcfooR
REOYRCTgvtFyhcfJbBNoTJ4JLbbjfnK2cqx36jZALtl03XcA97mipeojf8oM
dDfibgrGcf+fkoYYg2sWYOp4QlRP4X/5/1/g8L/+AX5R5gk=
     "]]}, 
   {Hue[0.37820393249936934`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIF4hOK1aKvHl/qnFP2/9//9+/xbXG/fzBX7Zw8Tf
2Uyo8vvwbv+chr1P7Fl/wsXd2dSl5j54u9/h6amNQsw/4OKSTT8LWie/3h/Q
47bM5tNXuPg3o7Ke8tyX+5nebFi2QesLXPxpyOy7xkHP97vFWjx81/8JLv41
PtXNmvPp/rncX+wbfn2AiwssuFns8/zRfo5tE5/M9X4PF/c++WnHiukP9p+u
qm4L3/0GLm54PCN93tI7+7/LKC3S7noFF3+S6vDsX+aN/THqzmnfT7yAi1+7
ljjl66XL+6fuLlh4x+g5XHx/8vrp/4XP73+/S/joGeZncHHpezZ3bdYe298y
N/PRX8WncPGuXN/1hXKH9n+7V3Od9egTuPiHn7uZfOV378+/zH7mcg1C/Neb
1p13527af8Bw+ywjH4Q4x4H5D+5YLNnfUfSwabkukjkf120taM3ZXx6SX/pT
ASF+6uHLk3orFtiXShWv6dZEiAtM/VphmLjRnmH+hR0L3RDiNpbferzzdtmX
BRU9NapAiM9jUXefn3/Qnn2RxeeHOxDic99cOi6x4qj9B7c0p0RuhH+lJKZn
3T9/zr6R7cjcCb8R4lvNI2v3r71sH563cXeCOiI8ZQRlxX763bD/ZtqgvW4f
IvzjOSY4z7t1x969Um31r4WI+Drqdf2zeM8D+9wKG//Y9Yj4tV7huG7qjUf2
OlImoU5WiPSw7ALjEs5/T+znmT1PV/uMSD//2yVTNks8t68SDq61SkOkN/Ow
Hh6hxJf2t6amR95UQKTPlPchH9KaXtsvT6uZ1PAYkZ4L51fsKbj41l5rzcEp
r798h4s3c+703vHinb3VH2G2138R+QKav+xh+QsAeivX6g==
     "]]}, 
   {Hue[0.6142719099991583, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIDY9VOSvrPN8/9Tin7f+/3+/P2TFljXXk3/Yw8Tf
2Uyo8vvwbr/N45kbl0d8h4u7s6lLzX3wdv8D9+V1VWHf4OKSTT8LWie/3v/h
nVLxEtcvcPFvRmU95bkv9y/y6y2WqPsEF38aMvuucdDz/c/2i6wLe/wBLv41
PtXNmvPp/qo7CyQvzH4HFxdYcLPY5/mj/R7VXnW5n17Dxb1PftqxYvqD/dwO
JgrqSS/h4obHM9LnLb2z3/zRLevfBs/h4k9SHZ79y7yxf86b3/+P5T+Fi1+7
ljjl66XL+682aFqvufkYLr4/ef30/8Ln9+s3nImbs+oRXFz6ns1dm7XH9h9J
Uqo5e+IhXLwr13d9odyh/WZLTAuvZiPEP/zczeQrv3v/+hkHUhcrI8R/vWnd
eXfupv0M3/NNmD4/gItzHJj/4I7Fkv0nzGfJlF9BiH/4uG5rQWvO/u2SGvwn
jyPETz18eVJvxQL7rfe13aIvIMQFpn6tMEzcaO+Q0hlU+BYhbmP5rcc7b5f9
NnHteiZ5hHvmsai7z88/aO+e/rFnYzxCfO6bS8clVhy1N+CRv2OwCSEuJTE9
6/75c/YHT9eYJyxChM9W88ja/Wsv2083zgo1OIcITxlBWbGffjfszX5bHK1N
RYR/PMcE53m37ti32913OeWBiK+jXtc/i/c8sF9r+/WFThgifq1XOK6beuOR
fc79N+8EHyLSw7ILjEs4/z2xrzFqz5TpRaSf/+2SKZslnttLlel7LdqJSG/m
YT08Qokv7at+OKs5liDSZ8r7kA9pTa/t3U9GxL8wQ6TnwvkVewouvrU/xTcv
MdgDkf6bOXd673jxzv5W3IdLQYGI/ALNX/aw/AUA5XzM0Q==
     "]]}, 
   {Hue[0.8503398874989481, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmID5U5K+s8/3J/qnFP2/9//9+v5PkLU6ze9/sYeLv
bCZU+X14t18rZD8f2+WvcHF3NnWpuQ/e7j+348ztGxe+wMUlm34WtE5+vf9+
vMcFpgOf4OLfjMp6ynNf7p+0f9OFsh8f4OJPQ2bfNQ56vn9VYolY1u53cPGv
8alu1pxP90fse325Z8JruLjAgpvFPs8f7VeLnWqh9fwFXNz75KcdK6Y/2P9S
sf7W47BncHHD4xnp85be2S96RPXbdtUncPEnqQ7P/mXe2F91zm1Xc8ojuPi1
a4lTvl66vH9zUsnXjHMP4OL7k9dP/y98fr/UET2jTXb34eLS92zu2qw9tt+l
81aOSPVduHhXru/6QrlD+zV4GMNVDt6Bi3/4uZvJV373fsmvYg4HDBDiv960
7rw7d9P+PRtL79aevw0X5zgw/8EdiyX7fYxPrU2fhBD/8HHd1oLWnP3a359O
4q5EiJ96+PKk3ooF9mrNbxkMuhDiAlO/VhgmbrS36VkhGXMEIW5j+a3HO2+X
vcbn10V16gj3zGNRd5+ff9Dec1JF350VCPG5by4dl1hx1F7mzmWd274I/0pJ
TM+6f/6cvX10/2NLPUT4bDWPrN2/9rJ9qehW/reHEOEpIygr9tPvhr3I3eZ6
qyhE+MdzTHCed+uOfaqC1v82W0R8HfW6/lm854F9t7zD4pdeiPi1XuG4buqN
R/ZeB81XnruKSA/LLjAu4fz3xD5K5JLyzUZE+vnfLpmyWeK5veC5lX6MGYj0
Zh7WwyOU+NI+KzOzef47RPpMeR/yIa3ptb2ZVd0et42I9Fw4v2JPwcW39jvr
D+/beAiR/ps5d3rvePHO/tR5UYONpxH5BZq/7GH5CwDQGua6
     "]]}, 
   {Hue[0.08640786499873876, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIPZX1vlucujR/qnFP2/9//9+f8MMj+9d9V/tYeLv
bCZU+X14tz/jsu6fqJIvcHF3NnWpuQ/e7uexsD6pW/QZLi7Z9LOgdfLr/TL3
t+0NT/wIF/9mVNZTnvty/9xap5rpxu/h4k9DZt81Dnq+XyUh8KPwv9dw8a/x
qW7WnE/3X9pzYp3Z5RdwcYEFN4t9nj/avyC6SPiK5jO4uPfJTztWTH+wP1kh
ZmvvzMdwccPjGenzlt7Z33+I6WJixkO4+JNUh2f/Mm/sfznTXSva7z5c/Nq1
xClfL13eH3Vo0QJVpTtw8f3J66f/Fz6/n+/gi+esR27CxaXv2dy1WXtsv0Xb
FKUMhhtw8a5c3/WFcof2y3Du5qlwuw4X//BzN5Ov/O79vJ8ufBXedg0u/utN
6867czft37BOf/LFEIQ4x4H5D+5YLNnvYNCSvFscIf7h47qtBa05++W/zPeI
/3cVLn7q4cuTeisW2Es1LN/WzI9QLzD1a4Vh4kZ7g86k8+s9EeI2lt96vPN2
2ct8WKpxaQ1CfB6Luvv8/IP2tv1GLgZ6CPfPfXPpuMSKo/YCN3sf6V1GiEtJ
TM+6f/6cvXGE58zezYjw2WoeWbt/7WX7++oqV3W5EeEpIygr9tPvhr3M9oNb
Tlgiwj+eY4LzvFt37O/Lsd8yaEDE11Gv65/Fex7Ys8rJ1k3uRcSv9QrHdVNv
PLI/sl+kpUICkR6WXWBcwvnvif0VofX/mo4g0s//dsmUzRLP7bvPtGmuXoVI
b+ZhPTxCiS/tfS+uf/hCHpE+U96HfEhrem1fvPPNytlBiPRcOL9iT8HFt/bf
/hmuYUtBpP9mzp3eO168s2fzb5Fly0XkF2j+soflLwAYYtin
     "]]}, 
   {Hue[0.3224758424985268, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL57meO0be+D/VOLf976///9fm41htJr7l/sYeLv
bCZU+X14t/9zwtPKZTaf4eLubOpScx+83T/96Ce3SqtPcHHJpp8FrZNf71+R
k2e9WPMDXPybUVlPee7L/VtmbCzXO/IGLv40ZPZd46Dn+22qD7xZ1fcSLv41
PtXNmvPp/lfXzVedTX4GFxdYcLPY5/mj/TvKHvJl73oMF/c++WnHiukP9ldZ
n96govUQLm54PCN93tI7+/12P57x8eNduPiTVIdn/zJv7Je4oKrYmX8LLn7t
WuKUr5cu7+/80TA92Pk6XHx/8vrp/4XP77f9fuye8dMrcHHpezZ3bdYe25+x
p0hymuJluHhXru/6QrlD+73sZzIvS78EF//wczeTr/zu/TYa6984Xr8IF//1
pnXn3bmb9t9/wtv1vxQhznFg/oM7Fkv256clRb20QIh/+Lhua0Frzn4/rTr7
XnmE+KmHL0/qrVhg77mtde1GA4S4wNSvFYaJG+3j9zscu5uFELex/NbjnbfL
3lutRZ7hHEJ8Hou6+/z8g/Y5RwWs4/wR7p/75tJxiRVH7e35c27GfEGIS0lM
z7p//px9Up/6hN1XEOGz1Tyydv/ay/Z8cUxnonUQ4SkjKCv20++Gvc/b+au/
RSHCP55jgvO8W3fs3xW/WdWveA8uftTr+mfxngf2MpY9lVpyiPi1XuG4buqN
R/Z3bjXX869EpIdlFxiXcP57Yv9aV+6XZCgi/fxvl0zZLPHcftHzrypO1oj0
Zh7WwyOU+NI+5Y3c3foNiPSZ8j7kQ1rTa/v/bkvUn0sh0nPh/Io9BRff2jf1
vNQM0kGk/2bOnd47Xryzn3THenagKSK/QPOXPSx/AQAUS/Zd
     "]]}, 
   {Hue[0.5585438199983166, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL7Ccdq2d8Od/VOLf976///9/mcHGZolzD/bw8Tf
2Uyo8vvwbv+5P0/bX2h/gou7s6lLzX3wdn9R4ueQ3Zof4eKSTT8LWie/3s8i
eG/SKcN3cPFvRmU95bkv92de2DR506JXcPGnIbPvGgc93/9h00EO/dzncPGv
8alu1pxP98/jtzwW6PQELi6w4Gaxz/NH+3PWPdJ6P+MhXNz75KcdK6Y/2B/3
bMbMQMF7cHHD4xnp85be2b9N69mFn+tuwsWfpDo8+5d5Y3+jvUbkRJ1rcPFr
1xKnfL10ef+n9Obz0QyX4eL7k9dP/y98fv+ytJNiNosuwMWl79nctVl7bP/C
8FeLbPaehYt35fquL5Q7tD891rw8xvMMXPzDz91MvvK793vpJ3tO0TgNF//1
pnXn3bmb9k9YodDCn3YKLs5xYP6DOxZL9m9YfVLl8euTCHM+rtta0JqzP8+Y
8b3WToT4qYcvT+qtWGCf4i1wuvg2Qlxg6tcKw8SN9v2xbX9O+yPMt7H81uOd
t8s+Q5t3vYAYwj3zWNTd5+cftF+VqfbsjATC/XPfXDouseKofcDfO0qnkhD+
lZKYnnX//Dn70381TxyeiAifreaRtfvXXrYvP8vCkvLyElxcRlBW7KffDfst
MYse/xNHhH88xwTnebfu2Ic9fvdo+j1EfB31uv5ZvOeB/aKHUtqxP+/Cxa1X
OK6beuOR/QSh1tmT2xHpYdkFxiWc/57YL0iVF5tnjEg//9slUzZLPLcPl//u
cEwSkd7Mw3p4hBJf2kuqKPxk7kekz5T3IR/Sml7bbzrqmR0kgkjPhfMr9hRc
fGsfo/HKfIo0Iv03c+703vHinX1Ohc3KySqI/ALNX/aw/AUAAA7utw==
     "]]}, 
   {Hue[0.7946117974981064, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIH6X2fJ0ntuN/VOLf976///9fr4cL8VA30/2MPF3
NhOq/D682/9tmb6aotNHuLg7m7rU3Adv98/itv382f4DXFyy6WdB6+TX+8NS
C89bx72Bi38zKuspz325f7eK2wXl8y/g4k9DZt81Dnq+34k91HXBzKdw8a/x
qW7WnE/3f0g+w3Q47xFcXGDBzWKf54/217C+OXeg6z5c3Pvkpx0rpj/Yz9S/
X8gk6xZc3PB4Rvq8pXf2R13m8brDfg0u/iTV4dm/zBv7VV74XSpuvQQXv3Yt
ccrXS5f3ayuGTKzZdA4uvj95/fT/wuf3T5F7IG/77SRcXPqezV2btcf2v1hh
PpUx/xhcvCvXd32h3KH969a1ZCr+PgIX//BzN5Ov/O79Exs2WEfeOgwX//Wm
defduZv2X/+ZW3ZFHCHOcWD+gzsWS/Z//yMmvmLuIYQ5H9dtLWjN2b+1xe/h
+yyE+KmHL0/qrVhgv3pO7F6LXoS4wNSvFYaJG+2vrbv8bgIjwnwby2893nm7
7NfXRC28ehwhPo9F3X1+/kH7T7uKb0w8hXD/3DeXjkusOGo/3U9LtF8E4V8p
ielZ98+fs7/js7DJ5yoifLaaR9buX3vZ3uRN2K+eFkR4ygjKiv30u2EfwXoy
5mEBIvzjOSY4z7t1x/5Xt2hMlTkivo56Xf8s3vPA3q3LcYl9ECJ+rVc4rpt6
45H9kbMGGzgzEOlh2QXGJZz/nth/3LkpiD8BkX7+t0umbJZ4br+yuKvW3A2R
3szDeniEEl/aZ1dsNi3djUifKe9DPqQ1vbZnkbq7/LADIj0Xzq/YU3DxrX3r
JeNrZwwR6b+Zc6f3jhfv7KcqtbmfsULkF2j+soflLwC99gBJ
     "]]}, 
   {Hue[0.030679774997896203`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIOY8bdu74c7l/VOLf976///9/jaVu/4NXR/tYeLv
bCZU+X14tz8v/lBIQP0HuLg7m7rU3Adv9yt3zHd2+/UOLi7Z9LOgdfLr/S7X
punMuvEKLv7NqKynPPfl/pVTs3WbPJ/DxZ+GzL5rHPR8v0FF1aJ/wk/g4l/j
U92sOZ/uv3flY7ziywdwcYEFN4t9nj/aH/ncrLO2/g5c3Pvkpx0rpj/Yf7+A
6+yHuOtwccPjGenzlt7Zb741nGXmn0tw8SepDs/+Zd7YzzLFWDLxyzm4+LVr
iVO+Xrq8/9mTl6zfZp+Ai+9PXj/9v/D5/X6PSqdsu38YLi59z+auzdpj+xfl
nOWsjjgAF+/K9V1fKHdo/8UnJntMT+6Fi3/4uZvJV373/ukdPQlL23fDxX+9
ad15d+6m/UrV7NdmM+2Ci3McmP/gjsWS/UmNkeLOfDsR5nxct7WgNWf/vQkn
Vl9asQMufurhy5N6KxbYnz7wLDnlLUJcYOrXCsPEjfaKTy8dE7mGMMfG8luP
d94u+0sND7TnJSHcM49F3X1+/kH7iL9Cs8QyEO6f++bScYkVR+0XqTQIC/7Z
DxeXkpiedf/8OfvpAubfj+5DhM9W88ja/Wsv279f/yaPLQ8RnjKCsmI//W7Y
Z51T17M5jQj/eI4JzvNu3bG/npsgZq5zGS5+1Ov6Z/GeB/bKOT1xf9wQ8Wu9
wnHd1BuP7Beur846FoNID8suMC7h/PfE/oHWjfW5txDp53+7ZMpmief205+s
vzX1wGO4uHlYD49Q4kv7yJc3Jl02RaTPlPchH9KaXtt/6mJiVdqNSM+F8yv2
FFx8a/9DtJfzx11E+m/m3Om948U7e97bEmW/yxH5BZq/7GH5CwDrKfU/
     "]]}, 
   {Hue[0.266747752497686, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIHaPeJfZ8vTc/qnFP2/9//9+v8/rHXzL3n2wh4m/
s5lQ5ffh3f50Hxv1N8few8Xd2dSl5j54u3+9aPjBiYfewsUlm34WtE5+vf9C
st+qpwtewsW/GZX1lOe+3G+qrLz6rOAzuPjTkNl3jYOe79/DaqTgdecRXPxr
fKqbNefT/VeOBO2+mXQfLi6w4Gaxz/NH+wuYlrWICNyCi3uf/LRjxfQH+791
V55of38FLm54PCN93tI7+/0vHPuv33sBLv4k1eHZv8wb+41OCQr0i56Ci1+7
ljjl66XL+0WkT/xR3nsYLr4/ef30/8Ln9/8yWCF2knEfXFz6ns1dm7XH9i9y
DWcM+rwNLt6V67u+UO7Qfr3Ajysncm+Bi3/4uZvJV373/q9fPPwYT22Ei/96
07rz7txN+xueHz783WUDXJzjwPwHdyyW7K+/rc1lt3AdwpyP67YWtObsL//F
aRx7bS1c/NTDlyf1ViywD4xyW9wQj1AvMPVrhWHiRvv6wP4tBRII820sv/V4
5+2yN3g7R/LXEoR75rGou8/PP2i/LfN8e9GqzXDxuW8uHZdYcdSeYaINY64d
wr9SEtOz7p8/Z9/dq3Xn+f69cPGt5pG1+9detpd7dirVpB8RnjKCsmI//W7Y
z2RhU816cxIuHs8xwXnerTv27zvOC57bg4ivo17XP4v3PLC3av8dOfk6In6t
Vzium3rjkf2OU69SY9/ehIsvu8C4hPPfE/vrBfLRRa6I9PO/XTJls8Rze+Wi
9IiHHYj0Zh7WwyOU+NL+adkEAfl/T+HiKe9DPqQ1vbavk9jVlNuMSM+F8yv2
FFx8a9+y2KXdeRki/Tdz7vTe8eKd/VLnl5lOOxD5BZq/7GH5CwDGr/bO
     "]]}, 
   {Hue[0.5028157299974758, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIK6RSty5XPDY/qnFP2/9//9+f8KBu7GHKj/Yw8Tf
2Uyo8vvwbv+Eb61HVe6+g4u7s6lLzX3wdv+N/QviT914AxeXbPpZ0Dr59f6f
vDM85La/gIt/MyrrKc99uT/ubI4nm8ZTuPjTkNl3jYOe73+6vnpf06eHcPGv
8alu1pxP96tJPvvu1X8XLi6w4Gaxz/NH+w+FWR5stbkBF/c++WnHiukP9ns8
4GH9KX0ZLm54PCN93tI7+x08t1Q4FZ+Fiz9JdXj2L/PG/qtFFp2Fecfg4teu
JU75euny/u2H35WyiR2Ai+9PXj/9v/D5/Q5NXzn6Nm+Di0vfs7lrs/bY/ku6
ns91vdbDxbtyfdcXyh3a/8ql1mJt1Uq4+Iefu5l85XfvL+vRaez5vgQu/utN
6867czft7ylUyql8uxAuznFg/oM7Fkv289Qde3ecZQHCnI/rtha05uxnt/LU
P1c8Dy5+6uHLk3orFtjv+XmQX7RtPiLcpn6tMEzcaN/lutM3ewrCfBvLbz3e
ebvs39RcEa9bi3DPPBZ19/n5B+1/lFp+yduxAi4+982l4xIrjtr3HBF+nvxg
LVxcSmJ61v3z5+wv3is/t6YQET5bzSNr96+9bN8k+3iv7oX9cHEZQVmxn343
7G3W6szzcUWEfzzHBOd5t+7YN+w6OqfuEiK+jnpd/yze88C+53bfNBYWRPxa
r3BcN/XGI3su+bpF5yQR6WHZBcYlnP+e2Gse3TT9cSEi/fxvl0zZLPHc3kdm
4zfJ+Yj0Zh7WwyOU+NKeT/HWunARRPpMeR/yIa3ptf3yQ8yKy2Yi0nPh/Io9
BRff2q/17VOt2YNI/82cO713vHhnf+6tEUf1OUR+geYve1j+AgDf0NNR
     "]]}, 
   {Hue[0.7388837074972656, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMkmEYB3BSK5yupUkeM62cF7VMq1lKvbmcNhHXzGHNC4eHShepkeYs
b0wq21e6Ag85Di1UstSRsInk3gyNzJYiBw+DQMTQFIQabrQleZHvd/Hc/C6e
i//zf6ILyrKLvQgEQszmeG8OXdYV0Hx3EHKY7jmPZw0qXlHOaA87wJbbyU01
WQ47pA3R6zNS7MjTd8SGcU02yHtwL9JBXkUeWucuZz1fgVDD3JkavoR8PaEK
u8NYhofYF4hHMAvyxZx2w/FsK3xTlVsros0jd+UXpyX7LkJxJ7eyOVePfLdg
lplpNUOqxXFW5TONnDL2a0DUYoJqxkhdqkWFPF55/RqvQw8vFx28Pes9jtxS
fO7HRskMtD3pqid6FMh1OjrbNaWGvCG5OYkqRw4L+1o8eyZg2YDaI9eIkYd/
JxvIPZ9g3Ivt2gasA/ljBrWvIvIjfFrAzAys4SN3uOVe1P1yKHKqDJPMZuR/
VlkyA/cd1MiH+rvzm5ATP/BN+lNC2PXa71ZjRAO+x9krLWfdgKyYepK/k4X8
y/zy2FGRAKwJK3VtpEd4bhxXdTxdDPhKnS0irxE5+fQ6Rrk5CNhT0X4SIwc5
zyc2nV82DLpHVhQcFxc5d3VKGSIaBT+PTWrgASHysJCWUuPEN0Ag0UbnnuH5
SBOv1MIeNRBfGpbkOQeRRwTs2+vOmgE5n9vbFrR4/vnEpvO8OT1o1Z5oJRXi
9xrNmP4djJlAcEla0MWv+H2TRSm9nBkz4Lw9GRVm1iHvnNwm9N2wAEl2KSku
Ce+P52FoUX+IFQQuYFLgjfctkYb5B9KXgcYqZdRW4/0sWstxXK1bAYwGo3H8
rxV5Bb/6fbnKBphBpZaXUXj/7/vKKANLdsCW7FIKEvB/+f9fYOu//gHz7+Tg

     "]]}, 
   {Hue[0.9749516849970554, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tM0lEcB3AlV9Cqpa5U1Fxmc7MtpFxUMo+uhi3DByU3yyK8pc4Lzgy1
fNDGMMeDK4m1JtDmBefAzCgItlM2JNS8NiQVREXRRASdONyUfMjzf/i9fB7O
w/d8v2ezy9LycD4+Puf379D+vXlGZKk63kNBhWfK612HTwQ4mdnPCQ7cQW2q
SXU6oEoykvNgcw158uFoYotlDfoW/d3Fra8iD6n3sLmvV2Gkcm4p45cNufvS
Uz6nZAXyOJ9tCQlW5IuMd6bLaTZ44p42S42fQ77FzKPFExYhp6Q6TqOeRn5S
8qfijm0eBnYN7boLDMhT9BtKqdAC38ZJkopujSEn6woei9pmoNU15Yoa0iO3
5iUu7RUaYXUoE8fQ9SE3GFjNW+MTUGGMMY+ZlMhhTrfQGzgCj3dw3W0DcuSh
ZqqJKuuHD0cXBr/IJMgbS+jd5Wf6oH2YdL1UI0Du9Khx9Ag1PKoJE2YWv0S+
Y+eqTC0foV6yzCU21CDHfxNbZq62wkZCwJVwbj72jkuuYHOL4ZENGy9Cno58
YG5Ff1EqAZ1ulY5WmI3lJtiqIrN6gCJR/KoTcpBTr7n5KaVfwXytxhVxswG5
yC86WVz2HeDJ0h4XpRl5i31cFyzVAvJwlnCaJ0JODBYWzY4MA9KjxroFGpaP
gpJZC2UT4Gdp7IeNOizPMP/w055UI7DcTQ8iibD8mfimG6KpGRCVYz/FujCA
XHt7cjOIbwEmUuREZSz2v/HSJLnAOA/uV/jPAhrWh/ZR31bCnhXUhNB+M0RY
f7y8kNzeYBuo7N9pTlNYkFMy+McCWCuAMniO3hSD9TN3neHMr18Fo8V0aO7G
+lwurtKwx9aAYXvyx6dxrP8vCKoU5bIDbAuft/dasb383xc42Nc/1znj0w==

     "]]}, 
   {Hue[0.21101966249684523`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tM0lEcB3AlV9Cqpa5U1Fxmc7MtpFxUMo+uhi3DByU3yyK8pc4Lzgy1
fNDGMMeDK4m1JtDmBefAzCgItlM2JNS8NiQVREXRRASdONyUfMjzf/i9fB7O
w/d8v2ezy9LycD4+Puf379D+vXlGZKk63gNBhWfK612HTwQ4mdnPCQ7cQW2q
SXU6oEoykvNgcw158uFoYotlDfoW/d3Fra8iD6n3sLmvV2Gkcm4p45cNufvS
Uz6nZAXyOJ9tCQlW5IuMd6bLaTZ44p42S42fQ77FzKPFExYhp6Q6TqOeRn5S
8qfijm0eBnYN7boLDMhT9BtKqdAC38ZJkopujSEn6woei9pmoNU15Yoa0iO3
5iUu7RUaYXUoE8fQ9SE3GFjNW+MTUGGMMY+ZlMhhTrfQGzgCj3dw3W0DcuSh
ZqqJKuuHD0cXBr/IJMgbS+jd5Wf6oH2YdL1UI0Du9Khx9Ag1PKoJE2YWv0S+
Y+eqTC0foV6yzCU21CDHfxNbZq62wkZCwJVwbj72jkuuYHOL4ZENGy9Cno58
YG5Ff1EqAZ1ulY5WmI3lJtiqIrN6gCJR/KoTcpBTr7n5KaVfwXytxhVxswG5
yC86WVz2HeDJ0h4XpRl5i31cFyzVAvJwlnCaJ0JODBYWzY4MA9KjxroFGpaP
gpJZC2UT4Gdp7IeNOizPMP/w055UI7DcTQ8iibD8mfimG6KpGRCVYz/FujCA
XHt7cjOIbwEmUuREZSz2v/HSJLnAOA/uV/jPAhrWh/ZR31bCnhXUhNB+M0RY
f7y8kNzeYBuo7N9pTlNYkFMy+McCWCuAMniO3hSD9TN3neHMr18Fo8V0aO7G
+lwurtKwx9aAYXvyx6dxrP8vCKoU5bIDbAuft/dasb383xc42Nc/YtnRUw==

     "]]}, 
   {Hue[0.44708763999663503`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMkmEYB3BSK5yupUkeM62cF7VMq1lKvbmcNhHXzGHNC4eHShepkeYs
b0wq21e6Ag85Di1UstSRsInk3gyNzJYiBw+DQMTQFIQabrQleZHvd/Hc/C6e
i//zf6ILyrKLvQgEQszmeG8OXdYV0Hx3EHCY7jmPZw0qXlHOaA87wJbbyU01
WQ47pA3R6zNS7MjTd8SGcU02yHtwL9JBXkUeWucuZz1fgVDD3JkavoR8PaEK
u8NYhofYF4hHMAvyxZx2w/FsK3xTlVsros0jd+UXpyX7LkJxJ7eyOVePfLdg
lplpNUOqxXFW5TONnDL2a0DUYoJqxkhdqkWFPF55/RqvQw8vFx28Pes9jtxS
fO7HRskMtD3pqid6FMh1OjrbNaWGvCG5OYkqRw4L+1o8eyZg2YDaI9eIkYd/
JxvIPZ9g3Ivt2gasA/ljBrWvIvIjfFrAzAys4SN3uOVe1P1yKHKqDJPMZuR/
VlkyA/cd1MiH+rvzm5ATP/BN+lNC2PXa71ZjRAO+x9krLWfdgKyYepK/k4X8
y/zy2FGRAKwJK3VtpEd4bhxXdTxdDPhKnS0irxE5+fQ6Rrk5CNhT0X4SIwc5
zyc2nV82DLpHVhQcFxc5d3VKGSIaBT+PTWrgASHysJCWUuPEN0Ag0UbnnuH5
SBOv1MIeNRBfGpbkOQeRRwTs2+vOmgE5n9vbFrR4/vnEpvO8OT1o1Z5oJRXi
9xrNmP4djJlAcEla0MWv+H2TRSm9nBkz4Lw9GRVm1iHvnNwm9N2wAEl2KSku
Ce+P52FoUX+IFQQuYFLgjfctkYb5B9KXgcYqZdRW4/0sWstxXK1bAYwGo3H8
rxV5Bb/6fbnKBphBpZaXUXj/7/vKKANLdsCW7FIKEvB/+f9fYOu//gF/j9Jg

     "]]}, 
   {Hue[0.6831556174964248, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIK6RSty5XPCY/dTin7f+/3+/P+HA3dhDlR/sYeLv
bCZU+X14t3/Ct9ajKnffwcXd2dSl5j54u//G/gXxp268gYtLNv0saJ38ev9P
3hkecttfwMW/GZX1lOe+3B93NseTTeMpXPxpyOy7xkHP9z9dX72v6dNDuPjX
+FQ3a86n+9Ukn3336r8LFxdYcLPY5/mj/YfCLA+22tyAi3uf/LRjxfQH+z0e
8LD+lL4MFzc8npE+b+md/Q6eWyqcis/CxZ+kOjz7l3lj/9Uii87CvGNw8WvX
Eqd8vXR5//bD70rZxA7Axfcnr5/+X/j8foemrxx9m7fBxaXv2dy1WXts/yVd
z+e6Xuvh4l25vusL5Q7tf+VSa7G2aiVc/MPP3Uy+8rv3l/XoNPZ8XwIX//Wm
defduZv29xQq5VS+XQgX5zgw/8EdiyX7eeqOvTvOsgBhzsd1Wwtac/azW3nq
nyueBxc/9fDlSb0VC+z3/DzIL9o2HxFuU79WGCZutO9y3embPQVhvo3ltx7v
vF32b2quiNetRbhnHou6+/z8g/Y/Si2/5O1YARef++bScYkVR+17jgg/T36w
Fi4uJTE96/75c/YX75WfW1OICJ+t5pG1+9detm+SfbxX98J+uLiMoKzYT78b
9jZrdeb5uCLCP55jgvO8W3fsG3YdnVN3CRFfR72ufxbveWDfc7tvGgsLIn6t
Vzium3rjkT2XfN2ic5KI9LDsAuMSzn9P7DWPbpr+uBCRfv63S6Zslnhu7yOz
8ZvkfER6Mw/r4RFKfGnPp3hrXbgIIn2mvA/5kNb02n75IWbFZTMR6blwfsWe
gotv7df69qnW7EGk/2bOnd47XryzP/fWiKP6HCK/QPOXPSx/AQBrcMDR
     "]]}, 
   {Hue[0.9192235949962146, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIHaPeJfZ8vSc/dTin7f+/3+/3+f1Dr5l7z7Yw8Tf
2Uyo8vvwbn+6j436m2Pv4eLubOpScx+83b9eNPzgxENv4eKSTT8LWie/3n8h
2W/V0wUv4eLfjMp6ynNf7jdVVl59VvAZXPxpyOy7xkHP9+9hNVLwuvMILv41
PtXNmvPp/itHgnbfTLoPFxdYcLPY5/mj/QVMy1pEBG7Bxb1PftqxYvqD/d+6
K0+0v78CFzc8npE+b+md/f4Xjv3X770AF3+S6vDsX+aN/UanBAX6RU/Bxa9d
S5zy9dLl/SLSJ/4o7z0MF9+fvH76f+Hz+38ZrBA7ybgPLi59z+auzdpj+xe5
hjMGfd4GF+/K9V1fKHdov17gx5UTubfAxT/83M3kK797/9cvHn6MpzbCxX+9
ad15d+6m/Q3PDx/+7rIBLs5xYP6DOxZL9tff1uayW7gOYc7HdVsLWnP2l//i
NI69thYufurhy5N6KxbYB0a5LW6IR6gXmPq1wjBxo319YP+WAgmE+TaW33q8
83bZG7ydI/lrCcI981jU3efnH7Tflnm+vWjVZrj43DeXjkusOGrPMNGGMdcO
4V8pielZ98+fs+/u1brzfP9euPhW88ja/Wsv28s9O5Vq0o8ITxlBWbGffjfs
Z7KwqWa9OQkXj+eY4Dzv1h379x3nBc/tQcTXUa/rn8V7Hthbtf+OnHwdEb/W
KxzXTb3xyH7HqVepsW9vwsWXXWBcwvnvif31AvnoIldE+vnfLpmyWeK5vXJR
esTDDkR6Mw/r4RFKfGn/tGyCgPy/p3DxlPchH9KaXtvXSexqym1GpOfC+RV7
Ci6+tW9Z7NLuvAyR/ps5d3rvePHOfqnzy0ynHYj8As1f9rD8BQBST+RO
     "]]}, 
   {Hue[0.15529157249600445`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIOY8bdu74c5l+6nFP2/9//9+f5vKXf+Gro/2MPF3
NhOq/D68258XfygkoP4DXNydTV1q7oO3+5U75ju7/XoHF5ds+lnQOvn1fpdr
03Rm3XgFF/9mVNZTnvty/8qp2bpNns/h4k9DZt81Dnq+36CiatE/4Sdw8a/x
qW7WnE/337vyMV7x5QO4uMCCm8U+zx/tj3xu1llbfwcu7n3y044V0x/sv1/A
dfZD3HW4uOHxjPR5S+/sN98azjLzzyW4+JNUh2f/Mm/sZ5liLJn45Rxc/Nq1
xClfL13e/+zJS9Zvs0/Axfcnr5/+X/j8fr9HpVO23T8MF5e+Z3PXZu2x/Yty
znJWRxyAi3fl+q4vlDu0/+ITkz2mJ/fCxT/83M3kK797//SOnoSl7bvh4r/e
tO68O3fTfqVq9muzmXbBxTkOzH9wx2LJ/qTGSHFnvp0Icz6u21rQmrP/3oQT
qy+t2AEXP/Xw5Um9FQvsTx94lpzyFiEuMPVrhWHiRnvFp5eOiVxDmGNj+a3H
O2+X/aWGB9rzkhDumcei7j4//6B9xF+hWWIZCPfPfXPpuMSKo/aLVBqEBf/s
h4tLSUzPun/+nP10AfPvR/chwmereWTt/rWX7d+vf5PHlocITxlBWbGffjfs
s86p69mcRoR/PMcE53m37thfz00QM9e5DBc/6nX9s3jPA3vlnJ64P26I+LVe
4bhu6o1H9gvXV2cdi0Gkh2UXGJdw/nti/0DrxvrcW4j0879dMmWzxHP76U/W
35p64DFc3Dysh0co8aV95Msbky6bItJnyvuQD2lNr+0/dTGxKu1GpOfC+RV7
Ci6+tf8h2sv54y4i/Tdz7vTe8eKdPe9tibLf5Yj8As1f9rD8BQB2yeK/
     "]]}, 
   {Hue[0.39135954999579425`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIH6X2fJ0ntsN+6nFP2/9//9+P1+Ol2Kg7yd7mPg7
mwlVfh/e7f+2TF9N0ekjXNydTV1q7oO3+2dx237+bP8BLi7Z9LOgdfLr/WGp
heet497Axb8ZlfWU577cv1vF7YLy+Rdw8achs+8aBz3f78Qe6rpg5lO4+Nf4
VDdrzqf7PySfYTqc9wguLrDgZrHP80f7a1jfnDvQdR8u7n3y044V0x/sZ+rf
L2SSdQsubng8I33e0jv7oy7zeN1hvwYXf5Lq8Oxf5o39Ki/8LhW3XoKLX7uW
OOXrpcv7tRVDJtZsOgcX35+8fvp/4fP7p8g9kLf9dhIuLn3P5q7N2mP7X6ww
n8qYfwwu3pXru75Q7tD+detaMhV/H4GLf/i5m8lXfvf+iQ0brCNvHYaL/3rT
uvPu3E37r//MLbsijhDnODD/wR2LJfu//xETXzH3EMKcj+u2FrTm7N/a4vfw
fRZC/NTDlyf1ViywXz0ndq9FL0JcYOrXCsPEjfbX1l1+N4ERYb6N5bce77xd
9utrohZePY4Qn8ei7j4//6D9p13FNyaeQrh/7ptLxyVWHLWf7qcl2i+C8K+U
xPSs++fP2d/xWdjkcxURPlvNI2v3r71sb/Im7FdPCyI8ZQRlxX763bCPYD0Z
87AAEf7xHBOc5926Y/+rWzSmyhwRX0e9rn8W73lg79bluMQ+CBG/1isc1029
8cj+yFmDDZwZiPSw7ALjEs5/T+w/7twUxJ+ASD//2yVTNks8t19Z3FVr7oZI
b+ZhPTxCiS/tsys2m5buRqTPlPchH9KaXtuzSN1dftgBkZ4L51fsKbj41r71
kvG1M4aI9N/MudN7x4t39lOV2tzPWCHyCzR/2cPyFwBJlu26
     "]]}, 
   {Hue[0.6274275274955841, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL7Ccdq2d8Md+6nFP2/9//9+/7ODDM0S5p/tYeLv
bCZU+X14t//cn6ftL7Q/wcXd2dSl5j54u78o8XPIbs2PcHHJpp8FrZNf72cR
vDfplOE7uPg3o7Ke8tyX+zMvbJq8adEruPjTkNl3jYOe7/+w6SCHfu5zuPjX
+FQ3a86n++fxWx4LdHoCFxdYcLPY5/mj/TnrHmm9n/EQLu598tOOFdMf7I97
NmNmoOA9uLjh8Yz0eUvv7N+m9ezCz3U34eJPUh2e/cu8sb/RXiNyos41uPi1
a4lTvl66vP9TevP5aIbLcPH9yeun/xc+v39Z2kkxm0UX4OLS92zu2qw9tn9h
+KtFNnvPwsW7cn3XF8od2p8ea14e43kGLv7h524mX/nd+730kz2naJyGi/96
07rz7txN+yesUGjhTzsFF+c4MP/BHYsl+zesPqny+PVJhDkf120taM3Zn2fM
+F5rJ0L81MOXJ/VWLLBP8RY4XXwbIS4w9WuFYeJG+/7Ytj+n/RHm21h+6/HO
22Wfoc27XkAM4Z55LOru8/MP2q/KVHt2RgLh/rlvLh2XWHHUPuDvHaVTSQj/
SklMz7p//pz96b+aJw5PRITPVvPI2v1rL9uXn2VhSXl5CS4uIygr9tPvhv2W
mEWP/4kjwj+eY4LzvFt37MMev3s0/R4ivo56Xf8s3vPAftFDKe3Yn3fh4tYr
HNdNvfHIfoJQ6+zJ7Yj0sOwC4xLOf0/sF6TKi80zRqSf/+2SKZslntuHy393
OCaJSG/mYT08Qokv7SVVFH4y9yPSZ8r7kA9pTa/tNx31zA4SQaTnwvkVewou
vrWP0XhlPkUakf6bOXd673jxzj6nwmblZBVEfoHmL3tY/gIAi5/cNw==
     "]]}, 
   {Hue[0.8634955049953739, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL57meO0be8D+6nFP2/9//9+P7caQ+k19y/2MPF3
NhOq/D682/854WnlMpvPcHF3NnWpuQ/e7p9+9JNbpdUnuLhk08+C1smv96/I
ybNerPkBLv7NqKynPPfl/i0zNpbrHXkDF38aMvuucdDz/TbVB96s6nsJF/8a
n+pmzfl0/6vr5qvOJj+DiwssuFns8/zR/h1lD/mydz2Gi3uf/LRjxfQH+6us
T29Q0XoIFzc8npE+b+md/X67H8/4+PEuXPxJqsOzf5k39ktcUFXszL8FF792
LXHK10uX93f+aJge7HwdLr4/ef30/8Ln99t+P3bP+OkVuLj0PZu7NmuP7c/Y
UyQ5TfEyXLwr13d9odyh/V72M5mXpV+Ci3/4uZvJV373fhuN9W8cr1+Ei/96
07rz7txN++8/4e36X4oQ5zgw/8EdiyX789OSol5aIMQ/fFy3taA1Z7+fVp19
rzxC/NTDlyf1Viyw99zWunajAUJcYOrXCsPEjfbx+x2O3c1CiNtYfuvxzttl
763WIs9wDiE+j0XdfX7+QfucowLWcf4I9899c+m4xIqj9vb8OTdjviDEpSSm
Z90/f84+qU99wu4riPDZah5Zu3/tZXu+OKYz0TqI8JQRlBX76XfD3uft/NXf
ohDhH88xwXnerTv274rfrOpXvAcXP+p1/bN4D1CfZU+llhwifq1XOK6beuOR
/Z1bzfX8KxHpYdkFxiWc/57Yv9aV+yUZikg//9slUzZLPLdf9PyripM1Ir2Z
h/XwCCW+tE95I3e3fgMifaa8D/mQ1vTa/r/bEvXnUoj0XDi/Yk/Bxbf2TT0v
NYN0EOm/mXOn944X7+wn3bGeHWiKyC/Q/GUPy18An9zj3Q==
     "]]}, 
   {Hue[0.09956348249516367, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIPZX1vlucuiR/dTin7f+/3+/v2GGx/eu+q/2MPF3
NhOq/D68259xWfdPVMkXuLg7m7rU3Adv9/NYWJ/ULfoMF5ds+lnQOvn1fpn7
2/aGJ36Ei38zKuspz325f26tU8104/dw8achs+8aBz3fr5IQ+FH432u4+Nf4
VDdrzqf7L+05sc7s8gu4uMCCm8U+zx/tXxBdJHxF8xlc3Pvkpx0rpj/Yn6wQ
s7V35mO4uOHxjPR5S+/s7z/EdDEx4yFc/Emqw7N/mTf2v5zprhXtdx8ufu1a
4pSvly7vjzq0aIGq0h24+P7k9dP/C5/fz3fwxXPWIzfh4tL3bO7arD2236Jt
ilIGww24eFeu7/pCuUP7ZTh381S4XYeLf/i5m8lXfvd+3k8XvgpvuwYX//Wm
defduZv2b1inP/liCEKc48D8B3cslux3MGhJ3i2OEP/wcd3Wgtac/fJf5nvE
/7sKFz/18OVJvRUL7KUalm9r5keoF5j6tcIwcaO9QWfS+fWeCHEby2893nm7
7GU+LNW4tAYhPo9F3X1+/kF7234jFwM9hPvnvrl0XGLFUXuBm72P9C4jxKUk
pmfdP3/O3jjCc2bvZkT4bDWPrN2/9rL9fXWVq7rciPCUEZQV++l3w15m+8Et
JywR4R/PMcF53q079vfl2G8ZNCDi66jX9c/iPQ/sWeVk6yb3IuLXeoXjuqk3
Htkf2S/SUiGBSA/LLjAu4fz3xP6K0Pp/TUcQ6ed/u2TKZonn9t1n2jRXr0Kk
N/OwHh6hxJf2vhfXP3whj0ifKe9DPqQ1vbYv3vlm5ewgRHounF+xp+DiW/tv
/wzXsKUg0n8z507vHS/e2bP5t8iy5SLyCzR/2cPyFwCj88Yn
     "]]}, 
   {Hue[0.3356314599949535, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmID5U5K+s8/2J/dTin7f+/3+/30nyFqfZvW/2MPF3
NhOq/D68268Vsp+P7fJXuLg7m7rU3Adv95/bceb2jQtf4OKSTT8LWie/3n8/
3uMC04FPcPFvRmU95bkv90/av+lC2Y8PcPGnIbPvGgc9378qsUQsa/c7uPjX
+FQ3a86n+yP2vb7cM+E1XFxgwc1in+eP9qvFTrXQev4CLu598tOOFdMf7H+p
WH/rcdgzuLjh8Yz0eUvv7Bc9ovptu+oTuPiTVIdn/zJv7K8657arOeURXPza
tcQpXy9d3r85qeRrxrkHcPH9yeun/xc+v1/qiJ7RJrv7cHHpezZ3bdYe2+/S
eStHpPouXLwr13d9odyh/Ro8jOEqB+/AxT/83M3kK797v+RXMYcDBgjxX29a
d96du2n/no2ld2vP34aLcxyY/+COxZL9Psan1qZPQoh/+Lhua0Frzn7t708n
cVcixE89fHlSb8UCe7XmtwwGXQhxgalfKwwTN9rb9KyQjDmCELex/NbjnbfL
XuPz66I6dYR75rGou8/PP2jvOami784KhPjcN5eOS6w4ai9z57LObV+Ef6Uk
pmfdP3/O3j66/7GlHiJ8tppH1u5fe9m+VHQr/9tDiPCUEZQV++l3w17kbnO9
VRQi/OM5JjjPu3XHPlVB63+bLSK+jnpd/yze88C+W95h8UsvRPxar3BcN/XG
I3uvg+Yrz11FpIdlFxiXcP57Yh8lckn5ZiMi/fxvl0zZLPHcXvDcSj/GDER6
Mw/r4RFKfGmflZnZPP8dIn2mvA/5kNb02t7Mqm6P20ZEei6cX7Gn4OJb+531
h/dtPIRI/82cO713vHhnf+q8qMHG04j8As1f9rD8BQBbutQ6
     "]]}, 
   {Hue[0.5716994374947433, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIDY9VOSvrPPcfmrxz1v//7/fH7Jiy5rryT/sYeLv
bCZU+X14t9/m8cyNyyO+w8Xd2dSl5j54u/+B+/K6qrBvcHHJpp8FrZNf7//w
Tql4iesXuPg3o7Ke8tyX+xf59RZL1H2Ciz8NmX3XOOj5/mf7RdaFPf4AF/8a
n+pmzfl0f9WdBZIXZr+DiwssuFns8/zRfo9qr7rcT6/h4t4nP+1YMf3Bfm4H
EwX1pJdwccPjGenzlt7Zb/7olvVvg+dw8SepDs/+Zd7YP+fN7//H8p/Cxa9d
S5zy9dLl/VcbNK3X3HwMF9+fvH76f+Hz+/UbzsTNWfUILi59z+auzdpj+48k
KdWcPfEQLt6V67u+UO7QfrMlpoVXsxHiH37uZvKV371//YwDqYuVEeK/3rTu
vDt3036G7/kmTJ8fwMU5Dsx/cMdiyf4T5rNkyq8gxD98XLe1oDVn/3ZJDf6T
xxHipx6+PKm3YoH91vvabtEXEOICU79WGCZutHdI6QwqfIsQt7H81uOdt8t+
m7h2PZM8wj3zWNTd5+cftHdP/9izMR4hPvfNpeMSK47aG/DI3zHYhBCXkpie
df/8OfuDp2vMExYhwmereWTt/rWX7acbZ4UanEOEp4ygrNhPvxv2Zr8tjtam
IsI/nmOC87xbd+zb7e67nPJAxNdRr+ufxXse2K+1/fpCJwwRv9YrHNdNvfHI
Puf+m3eCDxHpYdkFxiWc/57Y1xi1Z8r0ItLP/3bJlM0SQPeW6Xst2olIb+Zh
PTxCiS/tq344qzmWINJnyvuQD2lNr+3dT0bEvzBDpOfC+RV7Ci6+tT/FNy8x
2AOR/ps5d3rvePHO/lbch0tBgYj8As1f9rD8BQBxHLpR
     "]]}, 
   {Hue[0.8077674149945295, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIF4hOK1aKvGl/dTin7f+/3+/f4vrjfv5Ar/sYeLv
bCZU+X14t39Ow94n9qw/4eLubOpScx+83e/w9NRGIeYfcHHJpp8FrZNf7w/o
cVtm8+krXPybUVlPee7L/UxvNizboPUFLv40ZPZd46Dn+91iLR6+6/8EF/8a
n+pmzfl0/1zuL/YNvz7AxQUW3Cz2ef5oP8e2iU/mer+Hi3uf/LRjxfQH+09X
VbeF734DFzc8npE+b+md/d9llBZpd72Ciz9JdXj2L/PG/hh157TvJ17Axa9d
S5zy9dLl/VN3Fyy8Y/QcLr4/ef30/8Ln97/fJXz0DPMzuLj0PZu7NmuP7W+Z
m/nor+JTuHhXru/6QrlD+7/dq7nOevQJXPzDz91MvvK79+dfZj9zuQYh/utN
6867czftP2C4fZaRD0Kc48D8B3csluzvKHrYtFwXyZyP67YWtObsLw/JL/2p
gBA/9fDlSb0VC+xLpYrXdGsixAWmfq0wTNxozzD/wo6FbghxG8tvPd55u+zL
goqeGlUgxOexqLvPzz9oz77I4vPDHQjxuW8uHZdYcdT+g1uaUyI3wr9SEtOz
7p8/Z9/IdmTuhN8I8a3mkbX71162D8/buDtBHRGeMoKyYj/9bth/M23QXrcP
Ef7xHBOc5926Y+9eqbb610JEfB31uv5ZvOeBfW6FjX/sekT8Wq9wXDf1xiN7
HSmTUCcrRHpYdoFxCee/J/bzzJ6nq31GpJ//7ZIpmyWe21cJB9dapSHSm3lY
D48QkL41NT3ypgIifaa8D/mQ1vTafnlazaSGx4j0XDi/Yk/Bxbf2WmsOTnn9
5TtcvJlzp/eOF+/srf4Is73+i8gX0PxlD8tfAAXLxWo=
     "]]}, 
   {Hue[0.04383539249432289, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIJZK3LlccNpr+6nFP2/9//9+f9JHl8X/Hv+2h4m/
s5lQ5ffh3X4vHa0V5278gou7s6lLzX3wdv+7CealC6/9hItLNv0saJ38ev8/
882Zp49+h4t/MyrrKc99uX/jdNks+39f4eJPQ2bfNQ56vv8Lx9Iltclf4OJf
41PdrDmf7vfZnsDMf+4TXFxgwc1in+eP9m9q27o1tvIjXNz75KcdK6Y/2C+S
uarNyvADXNzweEb6vKV39oeU/NbwNHwHF3+S6vDsX+aN/bvqZN6KFLyBi1+7
ljjl66XL+z8JeWn8uvkKLr4/ef30/8Ln9/sIvfV+teolXFz6ns1dm7XH9j/8
4pQpffIFXLwr13d9odyh/cFmYXHKOQjxDz93M/nK795/Ruth0EdlhPivN607
787dtF+2c4JiwOfncHGOA/Mf3LFYsv/pyV2cR68gxD98XLe1oDVn/5XVHn/E
TiDETz18eVJvxQL7S4U+xisvIMQFpn6tMEzcaB/7baX9/rcIcRvLbz3eebvs
L6/0zgmQR7hnHou6+/z8g/YpPwWr/scjxOe+uXRcYsVRe9959ifrNiHEpSSm
Z90/f87+XuxclXWLEOGz1Tyydv/ay/ZbjnY51Z1DhKeMoKzYT78b9sG9kZtO
pyLCP55jgvO8W3fsl1z4byjhiYivo17XP4v3PLB/+pwr21YbEb/WKxzXTb3x
yP7IDrZ36wsQ6WHZBcYlnP+e2Ps8CtbeegyRfv63S6Zslnhur7F/v8NJS0R6
Mw/r4RFKfGnf6XhFJuMLIn2mvA/5kNb02j5M6GUo/3ZEei6cX7Gn4OJb+xvh
ehFVJxDpv5lzp/eOF+/sXy5vOFl5EZFfoPnLHpa/AAnx04k=
     "]]}, 
   {Hue[0.27990336999410914`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIO7dcOcyx+m39lOLf976///9/vjVv7413/trDxN/
ZzOhyu/Du/3uz+7/Dr78By7uzqYuNffB2/2vvd6eULv4Gy4u2fSzoHXy6/1/
PmbsDTjwEy7+zaispzz35f51gZf3HvnxHS7+NGT2XeOg5/s/HYr6wRT3DS7+
NT7VzZrz6X6PQrHwKSe+wMUFFtws9nn+aP8Gh2zOx8Wf4eLeJz/tWDH9wX5B
5YRb+7U/wcUNj2ekz1t6Z796ifELTeGPcPEnqQ7P/mXe2B/10/lUv9EHuPi1
a4lTvl66vP9N29bzLOffwcX3J6+f/l/4/H63tojPX+a/hYtL37O5a7P22P7b
GesE1Pa9gYt35fquL5Q7tN931UFWvUSE+Iefu5l85XfvPzbP59dvCYT4rzet
O+/O3bRf/O/vE1EvXsPFOQ7Mf3DHYsn++7ZqG8+fRIh/+Lhua0Frzv5z8lsW
K+5FiJ96+PKk3ooF9meebr+1+ShCXGDq1wrDxI324Vmiz08+QojbWH7r8c7b
ZX9WdrtwtAjCPfNY1N3n5x+0j8utVeQKRYjPfXPpuMSKo/buQqsLOpcjxKUk
pmfdP3/O/sZFjjM7piPCZ6t5ZO3+tZft11l+e9lxGBGeMoKyYj/9btgvb3ik
k62LCP94jgnO827dsfdftDVHUB4RX0e9rn8W73lg/2jhgX2HVBHxa73Ccd3U
G4/sDxXtcTTNRqSHZRcYl3D+e2LvOYez3uogIv38b5dM2Szx3F61snSKnwki
vZmH9fAIJb60b/vdWfD2HSJ9prwP+ZDW9No+6My8lVM2ItJz4fyKPQUXgelF
8Mnq/4cQ6b+Zc6f3jhfv7J8lmsn+P43IL9D8ZQ/LXwBSzwgB
     "]]}, 
   {Hue[0.5159713474939025, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmBhj4YD+1+Oet///f718Fpv/bw8Tf2Uyo8vvwbv+k
dffun//yDy7uzqYuNffB2/1mwm/WLPr0Fy4u2fSzoHXy6/3uu9IXnnn4Gy7+
zaispzz35f5fkpcWOkj9gos/DZl91zjo+X7b7sg7dfU/4OJf41PdrDmf7p9i
KWot8OobXFxgwc1in+eP9v/myHobN/UrXNz75KcdK6Y/2B/zOm6fte8XuLjh
8Yz0eUvv7C+2MTqz1OQzXPxJqsOzf5k39i896LSOO+QTXPzatcQpXy9d3n+o
+6b3v46PcPH9yeun/xc+vz+fe3Vs2ccPcHHpezZ3bdYe27/zT+ibvgaEeFeu
7/pCuUP7i+befyYUhBD/8HM3k6/87v3rxFUf/tRFiP9607rz7txN+68fq/FY
KYoQ5zgw/8EdiyX7rZ9P3JPMhmTOx3VbC1pz9hvfuLGVhwEhfurhy5N6KxbY
G+2OuhjKghAXmPq1wjBxo315bef1B0IIcRvLbz3eebvsjcJv5a7URojPY1F3
n59/0P5X3d4vDD4I8blvLh2XWHHUPj8wZI5DKUJcSmJ61v3z5+wtDJm9It4i
xLeaR9buX3vZXvnpzwiHZkR4ygjKiv30u2H/yeOhwHM/RPjHc0xwnnfrjv3s
rC0BvQ6I+Drqdf2zeM8De72s/Yvs3BHxa73Ccd3UG4/sea13q27sQaSHZRcY
l3D+e2I/NYUjedsjRPr53y6ZslniuX2BY0nFqVBEejMP6+ERSnxpf+lwR0im
ACJ9prwP+ZDW9Np+3pS5vQKXEem5cH7FnoKLb+0Vbz3qr36CSP/NnDu9d7x4
Z2+kZcpY/RaRX6D5yx6WvwC2pHYp
     "]]}},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{367.4098157907564, 378.7250818135658},
  Method->{},
  PlotRange->{{-1., 1.}, {-0.9999999310344828, 0.9999999310344828}, {0., 
   1.9999998620689703`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{1.9439757495501204`, -2.0970551059891323`, 1.8092313748125075`},
  ViewVertical->{0.09498762424450648, -0.006837203743368776, 
   0.9954550419641749}]], "Output",
 CellChangeTimes->{{3.580281836932271*^9, 3.580281860773635*^9}, 
   3.5802819424783077`*^9, 3.580281985591774*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"{", 
   RowBox[{"%", ",", "%%"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.580281885040023*^9, 3.580281895239606*^9}, 
   3.580281968864817*^9}],

Cell[BoxData[
 Graphics3DBox[{{{}, {}, 
    {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmBhj4sH9q8c9b//+/378KTP+3h4m/s5lQ5ffh3f5J
6+7dP//lH1zcnU1dau6Dt/vNhN+sWfTpL1xcsulnQevk1/vdd6UvPPPwN1z8
m1FZT3nuy/2/JC8tdJD6BRd/GjL7rnHQ8/223ZF36up/wMW/xqe6WXM+3T/F
UtRa4NU3uLjAgpvFPs8f7f/NkfU2bupXuLj3yU87Vkx/sD/mddw+a98vcHHD
4xnp85be2V9sY3RmqclnuPiTVIdn/zJv7F960Gkdd8gnuPi1a4lTvl66vP9Q
903vfx0f4eL7k9dP/y98fn8+9+rYso8f4OLS92zu2qw9tn/nn9A3fQ0I8a5c
3/WFcof2F829/0woCCH+4eduJl/53fvXias+/KmLEP/1pnXn3bmb9l8/VuOx
UhQhznFg/oM7Fkv2Wz+fuCeZDcmcj+u2FrTm7De+cWMrDwNC/NTDlyf1Viyw
N9oddTGUBSEuMPVrhWHiRvvy2s7rD4QQ4jaW33q883bZG4Xfyl2pjRCfx6Lu
Pj//oP2vur1fGHwQ4nPfXDouseKofX5gyByHUoS4lMT0rPvnz9lbGDJ7RbxF
iG81j6zdv/ayvfLTnxEOzYjwlBGUFfvpd8P+k8dDged+iPCP55jgPO/WHfvZ
WVsCeh0Q8XXU6/pn8Z4H9npZ+xfZuSPi13qF47qpNx7Z81rvVt3Yg0gPyy4w
LuH898R+agpH8rZHiPTzv10yZbPEc/sCx5KKU6GI9GYe1sMjlPjS/tLhjpBM
AUT6THkf8iGt6bX9vClzewUuI9Jz4fyKPQUX39or3nrUX/0Ekf6bOXd673jx
zt5Iy5Sx+i0iv0Dzlz0sfwEAKxOIqQ==
      "]]}, 
    {Hue[0.9060679774997897, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIO7dcOcyx+m3+6cW/7z1///7/fGrf31rvvfXHib+
zmZCld+Hd/vdn93/HXz5D1zcnU1dau6Dt/tfe709oXbxN1xcsulnQevk1/v/
fMzYG3DgJ1z8m1FZT3nuy/3rAi/vPfLjO1z8acjsu8ZBz/d/OhT1gynuG1z8
a3yqmzXn0/0ehWLhU058gYsLLLhZ7PP80f4NDtmcj4s/w8W9T37asWL6g/2C
ygm39mt/gosbHs9In7f0zn71EuMXmsIf4eJPUh2e/cu8sT/qp/OpfqMPcPFr
1xKnfL10ef+btq3nWc6/g4vvT14//b/w+f1ubRGfv8x/CxeXvmdz12btsf23
M9YJqO17AxfvyvVdXyh3aL/vqoOseokI8Q8/dzP5yu/ef2yez6/fEgjxX29a
d96du2m/+N/fJ6JevIaLcxyY/+COxZL9923VNp4/iRD/8HHd1oLWnP3n5Lcs
VtyLED/18OVJvRUL7M883X5r81GEuMDUrxWGiRvtw7NEn598hBC3sfzW4523
y/6s7HbhaBGEe+axqLvPzz9oH5dbq8gVihCf++bScYkVR+3dhVYXdC5HiEtJ
TM+6f/6c/Y2LHGd2TEeEz1bzyNr9ay/br7P89rLjMCI8ZQRlxX763bBf3vBI
J1sXEf7xHBOc5926Y++/aGuOoDwivo56Xf8s3vPA/tHCA/sOqSLi13qF47qp
Nx7ZHyra42iajUgPyy4wLuH898Tecw5nvdVBRPr53y6Zslniub1qZekUPxNE
ejMP6+ERSnxp3/a7s+DtO0T6THkf8iGt6bV90Jl5K6dsRKTnwvkVewouvrW/
Jvhk9f9DiPTfzLnTe8eLd/bPEs1k/59G5Bdo/rKH5S8Axy8agQ==
      "]]}, 
    {Hue[0.1421359549995791, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF011I02EUBnDTEBeBusTmVyIWioawhJa5eNUi002RaZYXZtOtnGlO7EMH
ShgayoJB2S5i02rqNFScDb+Ct6Kp/4w5ndgUtWmuOXVz082YxpYX+b4X5+Z3
dXjOcyIKy1lcTw8PjzMH43UwweyhDv9XG7C50jnvdm/BQtuVd65f++DQLXSR
INNqgelnY+Rq3R7yVO+oYIneDC0i2sM3s07kQXVOfv2LDeii9fMmVH+Q7557
JHxcZoJ94rAS4HIgN+S8XoxnGaHdp01WU2RH7ijgXk0kGSBz4LaXr3obuV/r
XCXTuAIVDUplfrUNOYPYHpSL9TCA19VwkWpFTh0rvittW4A5D/aj06gW5Kvc
pN8ung4O14aaA/ibyGdn2S8d01q4TU6P3ptbRw6LesXuE5OQSTYz1rtMyEOW
6Iv07lG4bE/hhRBryJvKMnorTn2B2edzb0WWYrc6Rzwzwkfg95hlli0S+95m
/dCiRAHDGkURWTtG5D6fWvQLF2TQQAyTVDPYrbYeJb++FM68v/Y3cBz7t2UT
ESdvBdMVzPhODXa/ZkcVld0H8nc7ATRjpyfsChn3h4G2k1GaFY73kR6NSm0p
/ww4Tn+BuwC7ZHN6jCJXgQwpIGoV2IMp4pKfk2qwlC853fMW56Ok5dXAbi34
oGpKqVXjPEP9wwKdmTqQ/TxPMcHF+Rf4iC5L5xeATOOmUtLwvVTpP3ZOCvXA
YDx271Isvm+iPLmnWbcCvg56W3r5uA/tmiMykmsVMFeyY5WjuD/uZ0GcfooR
REOYRCTgvtFyhcfJbBNoTJ4JLbbjfnK2cqx36jZALtl03XcA97mipeojf8oM
dDfibgrGcf+fkoYYg2sWYOp4QlRP4X/5/1/g8L/+AX5R5gk=
      "]]}, 
    {Hue[0.37820393249936934`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIF4hOK1aKvHl/qnFP2/9//9+/xbXG/fzBX7Zw8Tf
2Uyo8vvwbv+chr1P7Fl/wsXd2dSl5j54u9/h6amNQsw/4OKSTT8LWie/3h/Q
47bM5tNXuPg3o7Ke8tyX+5nebFi2QesLXPxpyOy7xkHP97vFWjx81/8JLv41
PtXNmvPp/rncX+wbfn2AiwssuFns8/zRfo5tE5/M9X4PF/c++WnHiukP9p+u
qm4L3/0GLm54PCN93tI7+7/LKC3S7noFF3+S6vDsX+aN/THqzmnfT7yAi1+7
ljjl66XL+6fuLlh4x+g5XHx/8vrp/4XP73+/S/joGeZncHHpezZ3bdYe298y
N/PRX8WncPGuXN/1hXKH9n+7V3Od9egTuPiHn7uZfOV378+/zH7mcg1C/Neb
1p13527af8Bw+ywjH4Q4x4H5D+5YLNnfUfSwabkukjkf120taM3ZXx6SX/pT
ASF+6uHLk3orFtiXShWv6dZEiAtM/VphmLjRnmH+hR0L3RDiNpbferzzdtmX
BRU9NapAiM9jUXefn3/Qnn2RxeeHOxDic99cOi6x4qj9B7c0p0RuhH+lJKZn
3T9/zr6R7cjcCb8R4lvNI2v3r71sH563cXeCOiI8ZQRlxX763bD/ZtqgvW4f
IvzjOSY4z7t1x969Um31r4WI+Drqdf2zeM8D+9wKG//Y9Yj4tV7huG7qjUf2
OlImoU5WiPSw7ALjEs5/T+znmT1PV/uMSD//2yVTNks8t68SDq61SkOkN/Ow
Hh6hxJf2t6amR95UQKTPlPchH9KaXtsvT6uZ1PAYkZ4L51fsKbj41l5rzcEp
r798h4s3c+703vHinb3VH2G2138R+QKav+xh+QsAeivX6g==
      "]]}, 
    {Hue[0.6142719099991583, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIDY9VOSvrPN8/9Tin7f+/3+/P2TFljXXk3/Yw8Tf
2Uyo8vvwbr/N45kbl0d8h4u7s6lLzX3wdv8D9+V1VWHf4OKSTT8LWie/3v/h
nVLxEtcvcPFvRmU95bkv9y/y6y2WqPsEF38aMvuucdDz/c/2i6wLe/wBLv41
PtXNmvPp/qo7CyQvzH4HFxdYcLPY5/mj/R7VXnW5n17Dxb1PftqxYvqD/dwO
JgrqSS/h4obHM9LnLb2z3/zRLevfBs/h4k9SHZ79y7yxf86b3/+P5T+Fi1+7
ljjl66XL+682aFqvufkYLr4/ef30/8Ln9+s3nImbs+oRXFz6ns1dm7XH9h9J
Uqo5e+IhXLwr13d9odyh/WZLTAuvZiPEP/zczeQrv3v/+hkHUhcrI8R/vWnd
eXfupv0M3/NNmD4/gItzHJj/4I7Fkv0nzGfJlF9BiH/4uG5rQWvO/u2SGvwn
jyPETz18eVJvxQL7rfe13aIvIMQFpn6tMEzcaO+Q0hlU+BYhbmP5rcc7b5f9
NnHteiZ5hHvmsai7z88/aO+e/rFnYzxCfO6bS8clVhy1N+CRv2OwCSEuJTE9
6/75c/YHT9eYJyxChM9W88ja/Wsv2083zgo1OIcITxlBWbGffjfszX5bHK1N
RYR/PMcE53m37ti32913OeWBiK+jXtc/i/c8sF9r+/WFThgifq1XOK6beuOR
fc79N+8EHyLSw7ILjEs4/z2xrzFqz5TpRaSf/+2SKZslnttLlel7LdqJSG/m
YT08Qokv7at+OKs5liDSZ8r7kA9pTa/t3U9GxL8wQ6TnwvkVewouvrU/xTcv
MdgDkf6bOXd673jxzv5W3IdLQYGI/ALNX/aw/AUA5XzM0Q==
      "]]}, 
    {Hue[0.8503398874989481, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmID5U5K+s8/3J/qnFP2/9//9+v5PkLU6ze9/sYeLv
bCZU+X14t18rZD8f2+WvcHF3NnWpuQ/e7j+348ztGxe+wMUlm34WtE5+vf9+
vMcFpgOf4OLfjMp6ynNf7p+0f9OFsh8f4OJPQ2bfNQ56vn9VYolY1u53cPGv
8alu1pxP90fse325Z8JruLjAgpvFPs8f7VeLnWqh9fwFXNz75KcdK6Y/2P9S
sf7W47BncHHD4xnp85be2S96RPXbdtUncPEnqQ7P/mXe2F91zm1Xc8ojuPi1
a4lTvl66vH9zUsnXjHMP4OL7k9dP/y98fr/UET2jTXb34eLS92zu2qw9tt+l
81aOSPVduHhXru/6QrlD+zV4GMNVDt6Bi3/4uZvJV373fsmvYg4HDBDiv960
7rw7d9P+PRtL79aevw0X5zgw/8EdiyX7fYxPrU2fhBD/8HHd1oLWnP3a359O
4q5EiJ96+PKk3ooF9mrNbxkMuhDiAlO/VhgmbrS36VkhGXMEIW5j+a3HO2+X
vcbn10V16gj3zGNRd5+ff9Dec1JF350VCPG5by4dl1hx1F7mzmWd274I/0pJ
TM+6f/6cvX10/2NLPUT4bDWPrN2/9rJ9qehW/reHEOEpIygr9tPvhr3I3eZ6
qyhE+MdzTHCed+uOfaqC1v82W0R8HfW6/lm854F9t7zD4pdeiPi1XuG4buqN
R/ZeB81XnruKSA/LLjAu4fz3xD5K5JLyzUZE+vnfLpmyWeK5veC5lX6MGYj0
Zh7WwyOU+NI+KzOzef47RPpMeR/yIa3ptb2ZVd0et42I9Fw4v2JPwcW39jvr
D+/beAiR/ps5d3rvePHO/tR5UYONpxH5BZq/7GH5CwDQGua6
      "]]}, 
    {Hue[0.08640786499873876, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIPZX1vlucujR/qnFP2/9//9+f8MMj+9d9V/tYeLv
bCZU+X14tz/jsu6fqJIvcHF3NnWpuQ/e7uexsD6pW/QZLi7Z9LOgdfLr/TL3
t+0NT/wIF/9mVNZTnvty/9xap5rpxu/h4k9DZt81Dnq+XyUh8KPwv9dw8a/x
qW7WnE/3X9pzYp3Z5RdwcYEFN4t9nj/avyC6SPiK5jO4uPfJTztWTH+wP1kh
ZmvvzMdwccPjGenzlt7Z33+I6WJixkO4+JNUh2f/Mm/sfznTXSva7z5c/Nq1
xClfL13eH3Vo0QJVpTtw8f3J66f/Fz6/n+/gi+esR27CxaXv2dy1WXtsv0Xb
FKUMhhtw8a5c3/WFcof2y3Du5qlwuw4X//BzN5Ov/O79vJ8ufBXedg0u/utN
6867czft37BOf/LFEIQ4x4H5D+5YLNnvYNCSvFscIf7h47qtBa05++W/zPeI
/3cVLn7q4cuTeisW2Es1LN/WzI9QLzD1a4Vh4kZ7g86k8+s9EeI2lt96vPN2
2ct8WKpxaQ1CfB6Luvv8/IP2tv1GLgZ6CPfPfXPpuMSKo/YCN3sf6V1GiEtJ
TM+6f/6cvXGE58zezYjw2WoeWbt/7WX7++oqV3W5EeEpIygr9tPvhr3M9oNb
Tlgiwj+eY4LzvFt37O/Lsd8yaEDE11Gv65/Fex7Ys8rJ1k3uRcSv9QrHdVNv
PLI/sl+kpUICkR6WXWBcwvnvif0VofX/mo4g0s//dsmUzRLP7bvPtGmuXoVI
b+ZhPTxCiS/tfS+uf/hCHpE+U96HfEhrem1fvPPNytlBiPRcOL9iT8HFt/bf
/hmuYUtBpP9mzp3eO168s2fzb5Fly0XkF2j+soflLwAYYtin
      "]]}, 
    {Hue[0.3224758424985268, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL57meO0be+D/VOLf976///9fm41htJr7l/sYeLv
bCZU+X14t/9zwtPKZTaf4eLubOpScx+83T/96Ce3SqtPcHHJpp8FrZNf71+R
k2e9WPMDXPybUVlPee7L/VtmbCzXO/IGLv40ZPZd46Dn+22qD7xZ1fcSLv41
PtXNmvPp/lfXzVedTX4GFxdYcLPY5/mj/TvKHvJl73oMF/c++WnHiukP9ldZ
n96govUQLm54PCN93tI7+/12P57x8eNduPiTVIdn/zJv7Je4oKrYmX8LLn7t
WuKUr5cu7+/80TA92Pk6XHx/8vrp/4XP77f9fuye8dMrcHHpezZ3bdYe25+x
p0hymuJluHhXru/6QrlD+73sZzIvS78EF//wczeTr/zu/TYa6984Xr8IF//1
pnXn3bmb9t9/wtv1vxQhznFg/oM7Fkv256clRb20QIh/+Lhua0Frzn4/rTr7
XnmE+KmHL0/qrVhg77mtde1GA4S4wNSvFYaJG+3j9zscu5uFELex/NbjnbfL
3lutRZ7hHEJ8Hou6+/z8g/Y5RwWs4/wR7p/75tJxiRVH7e35c27GfEGIS0lM
z7p//px9Up/6hN1XEOGz1Tyydv/ay/Z8cUxnonUQ4SkjKCv20++Gvc/b+au/
RSHCP55jgvO8W3fs3xW/WdWveA8uftTr+mfxngf2MpY9lVpyiPi1XuG4buqN
R/Z3bjXX869EpIdlFxiXcP57Yv9aV+6XZCgi/fxvl0zZLPHcftHzrypO1oj0
Zh7WwyOU+NI+5Y3c3foNiPSZ8j7kQ1rTa/v/bkvUn0sh0nPh/Io9BRff2jf1
vNQM0kGk/2bOnd47Xryzn3THenagKSK/QPOXPSx/AQAUS/Zd
      "]]}, 
    {Hue[0.5585438199983166, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL7Ccdq2d8Od/VOLf976///9/mcHGZolzD/bw8Tf
2Uyo8vvwbv+5P0/bX2h/gou7s6lLzX3wdn9R4ueQ3Zof4eKSTT8LWie/3s8i
eG/SKcN3cPFvRmU95bkv92de2DR506JXcPGnIbPvGgc93/9h00EO/dzncPGv
8alu1pxP98/jtzwW6PQELi6w4Gaxz/NH+3PWPdJ6P+MhXNz75KcdK6Y/2B/3
bMbMQMF7cHHD4xnp85be2b9N69mFn+tuwsWfpDo8+5d5Y3+jvUbkRJ1rcPFr
1xKnfL10ef+n9Obz0QyX4eL7k9dP/y98fv+ytJNiNosuwMWl79nctVl7bP/C
8FeLbPaehYt35fquL5Q7tD891rw8xvMMXPzDz91MvvK793vpJ3tO0TgNF//1
pnXn3bmb9k9YodDCn3YKLs5xYP6DOxZL9m9YfVLl8euTCHM+rtta0JqzP8+Y
8b3WToT4qYcvT+qtWGCf4i1wuvg2Qlxg6tcKw8SN9v2xbX9O+yPMt7H81uOd
t8s+Q5t3vYAYwj3zWNTd5+cftF+VqfbsjATC/XPfXDouseKofcDfO0qnkhD+
lZKYnnX//Dn70381TxyeiAifreaRtfvXXrYvP8vCkvLyElxcRlBW7KffDfst
MYse/xNHhH88xwTnebfu2Ic9fvdo+j1EfB31uv5ZvOeB/aKHUtqxP+/Cxa1X
OK6beuOR/QSh1tmT2xHpYdkFxiWc/57YL0iVF5tnjEg//9slUzZLPLcPl//u
cEwSkd7Mw3p4hBJf2kuqKPxk7kekz5T3IR/Sml7bbzrqmR0kgkjPhfMr9hRc
fGsfo/HKfIo0Iv03c+703vHinX1Ohc3KySqI/ALNX/aw/AUAAA7utw==
      "]]}, 
    {Hue[0.7946117974981064, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIH6X2fJ0ntuN/VOLf976///9fr4cL8VA30/2MPF3
NhOq/D682/9tmb6aotNHuLg7m7rU3Adv98/itv382f4DXFyy6WdB6+TX+8NS
C89bx72Bi38zKuspz325f7eK2wXl8y/g4k9DZt81Dnq+34k91HXBzKdw8a/x
qW7WnE/3f0g+w3Q47xFcXGDBzWKf54/217C+OXeg6z5c3Pvkpx0rpj/Yz9S/
X8gk6xZc3PB4Rvq8pXf2R13m8brDfg0u/iTV4dm/zBv7VV74XSpuvQQXv3Yt
ccrXS5f3ayuGTKzZdA4uvj95/fT/wuf3T5F7IG/77SRcXPqezV2btcf2v1hh
PpUx/xhcvCvXd32h3KH969a1ZCr+PgIX//BzN5Ov/O79Exs2WEfeOgwX//Wm
defduZv2X/+ZW3ZFHCHOcWD+gzsWS/Z//yMmvmLuIYQ5H9dtLWjN2b+1xe/h
+yyE+KmHL0/qrVhgv3pO7F6LXoS4wNSvFYaJG+2vrbv8bgIjwnwby2893nm7
7NfXRC28ehwhPo9F3X1+/kH7T7uKb0w8hXD/3DeXjkusOGo/3U9LtF8E4V8p
ielZ98+fs7/js7DJ5yoifLaaR9buX3vZ3uRN2K+eFkR4ygjKiv30u2EfwXoy
5mEBIvzjOSY4z7t1x/5Xt2hMlTkivo56Xf8s3vPA3q3LcYl9ECJ+rVc4rpt6
45H9kbMGGzgzEOlh2QXGJZz/nth/3LkpiD8BkX7+t0umbJZ4br+yuKvW3A2R
3szDeniEEl/aZ1dsNi3djUifKe9DPqQ1vbZnkbq7/LADIj0Xzq/YU3DxrX3r
JeNrZwwR6b+Zc6f3jhfv7KcqtbmfsULkF2j+soflLwC99gBJ
      "]]}, 
    {Hue[0.030679774997896203`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIOY8bdu74c7l/VOLf976///9/jaVu/4NXR/tYeLv
bCZU+X14tz8v/lBIQP0HuLg7m7rU3Adv9yt3zHd2+/UOLi7Z9LOgdfLr/S7X
punMuvEKLv7NqKynPPfl/pVTs3WbPJ/DxZ+GzL5rHPR8v0FF1aJ/wk/g4l/j
U92sOZ/uv3flY7ziywdwcYEFN4t9nj/aH/ncrLO2/g5c3Pvkpx0rpj/Yf7+A
6+yHuOtwccPjGenzlt7Zb741nGXmn0tw8SepDs/+Zd7YzzLFWDLxyzm4+LVr
iVO+Xrq8/9mTl6zfZp+Ai+9PXj/9v/D5/X6PSqdsu38YLi59z+auzdpj+xfl
nOWsjjgAF+/K9V1fKHdo/8UnJntMT+6Fi3/4uZvJV373/ukdPQlL23fDxX+9
ad15d+6m/UrV7NdmM+2Ci3McmP/gjsWS/UmNkeLOfDsR5nxct7WgNWf/vQkn
Vl9asQMufurhy5N6KxbYnz7wLDnlLUJcYOrXCsPEjfaKTy8dE7mGMMfG8luP
d94u+0sND7TnJSHcM49F3X1+/kH7iL9Cs8QyEO6f++bScYkVR+0XqTQIC/7Z
DxeXkpiedf/8OfvpAubfj+5DhM9W88ja/Wsv279f/yaPLQ8RnjKCsmI//W7Y
Z51T17M5jQj/eI4JzvNu3bG/npsgZq5zGS5+1Ov6Z/GeB/bKOT1xf9wQ8Wu9
wnHd1BuP7Beur846FoNID8suMC7h/PfE/oHWjfW5txDp53+7ZMpmief205+s
vzX1wGO4uHlYD49Q4kv7yJc3Jl02RaTPlPchH9KaXtt/6mJiVdqNSM+F8yv2
FFx8a/9DtJfzx11E+m/m3Om948U7e97bEmW/yxH5BZq/7GH5CwDrKfU/
      "]]}, 
    {Hue[0.266747752497686, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIHaPeJfZ8vTc/qnFP2/9//9+v8/rHXzL3n2wh4m/
s5lQ5ffh3f50Hxv1N8few8Xd2dSl5j54u3+9aPjBiYfewsUlm34WtE5+vf9C
st+qpwtewsW/GZX1lOe+3G+qrLz6rOAzuPjTkNl3jYOe79/DaqTgdecRXPxr
fKqbNefT/VeOBO2+mXQfLi6w4Gaxz/NH+wuYlrWICNyCi3uf/LRjxfQH+791
V55of38FLm54PCN93tI7+/0vHPuv33sBLv4k1eHZv8wb+41OCQr0i56Ci1+7
ljjl66XL+0WkT/xR3nsYLr4/ef30/8Ln9/8yWCF2knEfXFz6ns1dm7XH9i9y
DWcM+rwNLt6V67u+UO7Qfr3Ajysncm+Bi3/4uZvJV373/q9fPPwYT22Ei/96
07rz7txN+xueHz783WUDXJzjwPwHdyyW7K+/rc1lt3AdwpyP67YWtObsL//F
aRx7bS1c/NTDlyf1ViywD4xyW9wQj1AvMPVrhWHiRvv6wP4tBRII820sv/V4
5+2yN3g7R/LXEoR75rGou8/PP2i/LfN8e9GqzXDxuW8uHZdYcdSeYaINY64d
wr9SEtOz7p8/Z9/dq3Xn+f69cPGt5pG1+9detpd7dirVpB8RnjKCsmI//W7Y
z2RhU816cxIuHs8xwXnerTv27zvOC57bg4ivo17XP4v3PLC3av8dOfk6In6t
Vzium3rjkf2OU69SY9/ehIsvu8C4hPPfE/vrBfLRRa6I9PO/XTJls8Rze+Wi
9IiHHYj0Zh7WwyOU+NL+adkEAfl/T+HiKe9DPqQ1vbavk9jVlNuMSM+F8yv2
FFx8a9+y2KXdeRki/Tdz7vTe8eKd/VLnl5lOOxD5BZq/7GH5CwDGr/bO
      "]]}, 
    {Hue[0.5028157299974758, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIK6RSty5XPDY/qnFP2/9//9+f8KBu7GHKj/Yw8Tf
2Uyo8vvwbv+Eb61HVe6+g4u7s6lLzX3wdv+N/QviT914AxeXbPpZ0Dr59f6f
vDM85La/gIt/MyrrKc99uT/ubI4nm8ZTuPjTkNl3jYOe73+6vnpf06eHcPGv
8alu1pxP96tJPvvu1X8XLi6w4Gaxz/NH+w+FWR5stbkBF/c++WnHiukP9ns8
4GH9KX0ZLm54PCN93tI7+x08t1Q4FZ+Fiz9JdXj2L/PG/qtFFp2Fecfg4teu
JU75euny/u2H35WyiR2Ai+9PXj/9v/D5/Q5NXzn6Nm+Di0vfs7lrs/bY/ku6
ns91vdbDxbtyfdcXyh3a/8ql1mJt1Uq4+Iefu5l85XfvL+vRaez5vgQu/utN
6867czft7ylUyql8uxAuznFg/oM7Fkv289Qde3ecZQHCnI/rtha05uxnt/LU
P1c8Dy5+6uHLk3orFtjv+XmQX7RtPiLcpn6tMEzcaN/lutM3ewrCfBvLbz3e
ebvs39RcEa9bi3DPPBZ19/n5B+1/lFp+yduxAi4+982l4xIrjtr3HBF+nvxg
LVxcSmJ61v3z5+wv3is/t6YQET5bzSNr96+9bN8k+3iv7oX9cHEZQVmxn343
7G3W6szzcUWEfzzHBOd5t+7YN+w6OqfuEiK+jnpd/yze88C+53bfNBYWRPxa
r3BcN/XGI3su+bpF5yQR6WHZBcYlnP+e2Gse3TT9cSEi/fxvl0zZLPHc3kdm
4zfJ+Yj0Zh7WwyOU+NKeT/HWunARRPpMeR/yIa3ptf3yQ8yKy2Yi0nPh/Io9
BRff2q/17VOt2YNI/82cO713vHhnf+6tEUf1OUR+geYve1j+AgDf0NNR
      "]]}, 
    {Hue[0.7388837074972656, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMkmEYB3BSK5yupUkeM62cF7VMq1lKvbmcNhHXzGHNC4eHShepkeYs
b0wq21e6Ag85Di1UstSRsInk3gyNzJYiBw+DQMTQFIQabrQleZHvd/Hc/C6e
i//zf6ILyrKLvQgEQszmeG8OXdYV0Hx3EHKY7jmPZw0qXlHOaA87wJbbyU01
WQ47pA3R6zNS7MjTd8SGcU02yHtwL9JBXkUeWucuZz1fgVDD3JkavoR8PaEK
u8NYhofYF4hHMAvyxZx2w/FsK3xTlVsros0jd+UXpyX7LkJxJ7eyOVePfLdg
lplpNUOqxXFW5TONnDL2a0DUYoJqxkhdqkWFPF55/RqvQw8vFx28Pes9jtxS
fO7HRskMtD3pqid6FMh1OjrbNaWGvCG5OYkqRw4L+1o8eyZg2YDaI9eIkYd/
JxvIPZ9g3Ivt2gasA/ljBrWvIvIjfFrAzAys4SN3uOVe1P1yKHKqDJPMZuR/
VlkyA/cd1MiH+rvzm5ATP/BN+lNC2PXa71ZjRAO+x9krLWfdgKyYepK/k4X8
y/zy2FGRAKwJK3VtpEd4bhxXdTxdDPhKnS0irxE5+fQ6Rrk5CNhT0X4SIwc5
zyc2nV82DLpHVhQcFxc5d3VKGSIaBT+PTWrgASHysJCWUuPEN0Ag0UbnnuH5
SBOv1MIeNRBfGpbkOQeRRwTs2+vOmgE5n9vbFrR4/vnEpvO8OT1o1Z5oJRXi
9xrNmP4djJlAcEla0MWv+H2TRSm9nBkz4Lw9GRVm1iHvnNwm9N2wAEl2KSku
Ce+P52FoUX+IFQQuYFLgjfctkYb5B9KXgcYqZdRW4/0sWstxXK1bAYwGo3H8
rxV5Bb/6fbnKBphBpZaXUXj/7/vKKANLdsCW7FIKEvB/+f9fYOu//gHz7+Tg

      "]]}, 
    {Hue[0.9749516849970554, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tM0lEcB3AlV9Cqpa5U1Fxmc7MtpFxUMo+uhi3DByU3yyK8pc4Lzgy1
fNDGMMeDK4m1JtDmBefAzCgItlM2JNS8NiQVREXRRASdONyUfMjzf/i9fB7O
w/d8v2ezy9LycD4+Puf379D+vXlGZKk63kNBhWfK612HTwQ4mdnPCQ7cQW2q
SXU6oEoykvNgcw158uFoYotlDfoW/d3Fra8iD6n3sLmvV2Gkcm4p45cNufvS
Uz6nZAXyOJ9tCQlW5IuMd6bLaTZ44p42S42fQ77FzKPFExYhp6Q6TqOeRn5S
8qfijm0eBnYN7boLDMhT9BtKqdAC38ZJkopujSEn6woei9pmoNU15Yoa0iO3
5iUu7RUaYXUoE8fQ9SE3GFjNW+MTUGGMMY+ZlMhhTrfQGzgCj3dw3W0DcuSh
ZqqJKuuHD0cXBr/IJMgbS+jd5Wf6oH2YdL1UI0Du9Khx9Ag1PKoJE2YWv0S+
Y+eqTC0foV6yzCU21CDHfxNbZq62wkZCwJVwbj72jkuuYHOL4ZENGy9Cno58
YG5Ff1EqAZ1ulY5WmI3lJtiqIrN6gCJR/KoTcpBTr7n5KaVfwXytxhVxswG5
yC86WVz2HeDJ0h4XpRl5i31cFyzVAvJwlnCaJ0JODBYWzY4MA9KjxroFGpaP
gpJZC2UT4Gdp7IeNOizPMP/w055UI7DcTQ8iibD8mfimG6KpGRCVYz/FujCA
XHt7cjOIbwEmUuREZSz2v/HSJLnAOA/uV/jPAhrWh/ZR31bCnhXUhNB+M0RY
f7y8kNzeYBuo7N9pTlNYkFMy+McCWCuAMniO3hSD9TN3neHMr18Fo8V0aO7G
+lwurtKwx9aAYXvyx6dxrP8vCKoU5bIDbAuft/dasb383xc42Nc/1znj0w==

      "]]}, 
    {Hue[0.21101966249684523`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tM0lEcB3AlV9Cqpa5U1Fxmc7MtpFxUMo+uhi3DByU3yyK8pc4Lzgy1
fNDGMMeDK4m1JtDmBefAzCgItlM2JNS8NiQVREXRRASdONyUfMjzf/i9fB7O
w/d8v2ezy9LycD4+Puf379D+vXlGZKk63gNBhWfK612HTwQ4mdnPCQ7cQW2q
SXU6oEoykvNgcw158uFoYotlDfoW/d3Fra8iD6n3sLmvV2Gkcm4p45cNufvS
Uz6nZAXyOJ9tCQlW5IuMd6bLaTZ44p42S42fQ77FzKPFExYhp6Q6TqOeRn5S
8qfijm0eBnYN7boLDMhT9BtKqdAC38ZJkopujSEn6woei9pmoNU15Yoa0iO3
5iUu7RUaYXUoE8fQ9SE3GFjNW+MTUGGMMY+ZlMhhTrfQGzgCj3dw3W0DcuSh
ZqqJKuuHD0cXBr/IJMgbS+jd5Wf6oH2YdL1UI0Du9Khx9Ag1PKoJE2YWv0S+
Y+eqTC0foV6yzCU21CDHfxNbZq62wkZCwJVwbj72jkuuYHOL4ZENGy9Cno58
YG5Ff1EqAZ1ulY5WmI3lJtiqIrN6gCJR/KoTcpBTr7n5KaVfwXytxhVxswG5
yC86WVz2HeDJ0h4XpRl5i31cFyzVAvJwlnCaJ0JODBYWzY4MA9KjxroFGpaP
gpJZC2UT4Gdp7IeNOizPMP/w055UI7DcTQ8iibD8mfimG6KpGRCVYz/FujCA
XHt7cjOIbwEmUuREZSz2v/HSJLnAOA/uV/jPAhrWh/ZR31bCnhXUhNB+M0RY
f7y8kNzeYBuo7N9pTlNYkFMy+McCWCuAMniO3hSD9TN3neHMr18Fo8V0aO7G
+lwurtKwx9aAYXvyx6dxrP8vCKoU5bIDbAuft/dasb383xc42Nc/YtnRUw==

      "]]}, 
    {Hue[0.44708763999663503`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMkmEYB3BSK5yupUkeM62cF7VMq1lKvbmcNhHXzGHNC4eHShepkeYs
b0wq21e6Ag85Di1UstSRsInk3gyNzJYiBw+DQMTQFIQabrQleZHvd/Hc/C6e
i//zf6ILyrKLvQgEQszmeG8OXdYV0Hx3EHCY7jmPZw0qXlHOaA87wJbbyU01
WQ47pA3R6zNS7MjTd8SGcU02yHtwL9JBXkUeWucuZz1fgVDD3JkavoR8PaEK
u8NYhofYF4hHMAvyxZx2w/FsK3xTlVsros0jd+UXpyX7LkJxJ7eyOVePfLdg
lplpNUOqxXFW5TONnDL2a0DUYoJqxkhdqkWFPF55/RqvQw8vFx28Pes9jtxS
fO7HRskMtD3pqid6FMh1OjrbNaWGvCG5OYkqRw4L+1o8eyZg2YDaI9eIkYd/
JxvIPZ9g3Ivt2gasA/ljBrWvIvIjfFrAzAys4SN3uOVe1P1yKHKqDJPMZuR/
VlkyA/cd1MiH+rvzm5ATP/BN+lNC2PXa71ZjRAO+x9krLWfdgKyYepK/k4X8
y/zy2FGRAKwJK3VtpEd4bhxXdTxdDPhKnS0irxE5+fQ6Rrk5CNhT0X4SIwc5
zyc2nV82DLpHVhQcFxc5d3VKGSIaBT+PTWrgASHysJCWUuPEN0Ag0UbnnuH5
SBOv1MIeNRBfGpbkOQeRRwTs2+vOmgE5n9vbFrR4/vnEpvO8OT1o1Z5oJRXi
9xrNmP4djJlAcEla0MWv+H2TRSm9nBkz4Lw9GRVm1iHvnNwm9N2wAEl2KSku
Ce+P52FoUX+IFQQuYFLgjfctkYb5B9KXgcYqZdRW4/0sWstxXK1bAYwGo3H8
rxV5Bb/6fbnKBphBpZaXUXj/7/vKKANLdsCW7FIKEvB/+f9fYOu//gF/j9Jg

      "]]}, 
    {Hue[0.6831556174964248, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIK6RSty5XPCY/dTin7f+/3+/P+HA3dhDlR/sYeLv
bCZU+X14t3/Ct9ajKnffwcXd2dSl5j54u//G/gXxp268gYtLNv0saJ38ev9P
3hkecttfwMW/GZX1lOe+3B93NseTTeMpXPxpyOy7xkHP9z9dX72v6dNDuPjX
+FQ3a86n+9Ukn3336r8LFxdYcLPY5/mj/YfCLA+22tyAi3uf/LRjxfQH+z0e
8LD+lL4MFzc8npE+b+md/Q6eWyqcis/CxZ+kOjz7l3lj/9Uii87CvGNw8WvX
Eqd8vXR5//bD70rZxA7Axfcnr5/+X/j8foemrxx9m7fBxaXv2dy1WXts/yVd
z+e6Xuvh4l25vusL5Q7tf+VSa7G2aiVc/MPP3Uy+8rv3l/XoNPZ8XwIX//Wm
defduZv29xQq5VS+XQgX5zgw/8EdiyX7eeqOvTvOsgBhzsd1Wwtac/azW3nq
nyueBxc/9fDlSb0VC+z3/DzIL9o2HxFuU79WGCZutO9y3embPQVhvo3ltx7v
vF32b2quiNetRbhnHou6+/z8g/Y/Si2/5O1YARef++bScYkVR+17jgg/T36w
Fi4uJTE96/75c/YX75WfW1OICJ+t5pG1+9detm+SfbxX98J+uLiMoKzYT78b
9jZrdeb5uCLCP55jgvO8W3fsG3YdnVN3CRFfR72ufxbveWDfc7tvGgsLIn6t
Vzium3rjkT2XfN2ic5KI9LDsAuMSzn9P7DWPbpr+uBCRfv63S6Zslnhu7yOz
8ZvkfER6Mw/r4RFKfGnPp3hrXbgIIn2mvA/5kNb02n75IWbFZTMR6blwfsWe
gotv7df69qnW7EGk/2bOnd47XryzP/fWiKP6HCK/QPOXPSx/AQBrcMDR
      "]]}, 
    {Hue[0.9192235949962146, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIHaPeJfZ8vSc/dTin7f+/3+/3+f1Dr5l7z7Yw8Tf
2Uyo8vvwbn+6j436m2Pv4eLubOpScx+83b9eNPzgxENv4eKSTT8LWie/3n8h
2W/V0wUv4eLfjMp6ynNf7jdVVl59VvAZXPxpyOy7xkHP9+9hNVLwuvMILv41
PtXNmvPp/itHgnbfTLoPFxdYcLPY5/mj/QVMy1pEBG7Bxb1PftqxYvqD/d+6
K0+0v78CFzc8npE+b+md/f4Xjv3X770AF3+S6vDsX+aN/UanBAX6RU/Bxa9d
S5zy9dLl/SLSJ/4o7z0MF9+fvH76f+Hz+38ZrBA7ybgPLi59z+auzdpj+xe5
hjMGfd4GF+/K9V1fKHdov17gx5UTubfAxT/83M3kK797/9cvHn6MpzbCxX+9
ad15d+6m/Q3PDx/+7rIBLs5xYP6DOxZL9tff1uayW7gOYc7HdVsLWnP2l//i
NI69thYufurhy5N6KxbYB0a5LW6IR6gXmPq1wjBxo319YP+WAgmE+TaW33q8
83bZG7ydI/lrCcI981jU3efnH7Tflnm+vWjVZrj43DeXjkusOGrPMNGGMdcO
4V8pielZ98+fs+/u1brzfP9euPhW88ja/Wsv28s9O5Vq0o8ITxlBWbGffjfs
Z7KwqWa9OQkXj+eY4Dzv1h379x3nBc/tQcTXUa/rn8V7Hthbtf+OnHwdEb/W
KxzXTb3xyH7HqVepsW9vwsWXXWBcwvnvif31AvnoIldE+vnfLpmyWeK5vXJR
esTDDkR6Mw/r4RFKfGn/tGyCgPy/p3DxlPchH9KaXtvXSexqym1GpOfC+RV7
Ci6+tW9Z7NLuvAyR/ps5d3rvePHOfqnzy0ynHYj8As1f9rD8BQBST+RO
      "]]}, 
    {Hue[0.15529157249600445`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIOY8bdu74c5l+6nFP2/9//9+f5vKXf+Gro/2MPF3
NhOq/D68258XfygkoP4DXNydTV1q7oO3+5U75ju7/XoHF5ds+lnQOvn1fpdr
03Rm3XgFF/9mVNZTnvty/8qp2bpNns/h4k9DZt81Dnq+36CiatE/4Sdw8a/x
qW7WnE/337vyMV7x5QO4uMCCm8U+zx/tj3xu1llbfwcu7n3y044V0x/sv1/A
dfZD3HW4uOHxjPR5S+/sN98azjLzzyW4+JNUh2f/Mm/sZ5liLJn45Rxc/Nq1
xClfL13e/+zJS9Zvs0/Axfcnr5/+X/j8fr9HpVO23T8MF5e+Z3PXZu2x/Yty
znJWRxyAi3fl+q4vlDu0/+ITkz2mJ/fCxT/83M3kK797//SOnoSl7bvh4r/e
tO68O3fTfqVq9muzmXbBxTkOzH9wx2LJ/qTGSHFnvp0Icz6u21rQmrP/3oQT
qy+t2AEXP/Xw5Um9FQvsTx94lpzyFiEuMPVrhWHiRnvFp5eOiVxDmGNj+a3H
O2+X/aWGB9rzkhDumcei7j4//6B9xF+hWWIZCPfPfXPpuMSKo/aLVBqEBf/s
h4tLSUzPun/+nP10AfPvR/chwmereWTt/rWX7d+vf5PHlocITxlBWbGffjfs
s86p69mcRoR/PMcE53m37thfz00QM9e5DBc/6nX9s3jPA3vlnJ64P26I+LVe
4bhu6o1H9gvXV2cdi0Gkh2UXGJdw/nti/0DrxvrcW4j0879dMmWzxHP76U/W
35p64DFc3Dysh0co8aV95Msbky6bItJnyvuQD2lNr+0/dTGxKu1GpOfC+RV7
Ci6+tf8h2sv54y4i/Tdz7vTe8eKdPe9tibLf5Yj8As1f9rD8BQB2yeK/
      "]]}, 
    {Hue[0.39135954999579425`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIH6X2fJ0ntsN+6nFP2/9//9+P1+Ol2Kg7yd7mPg7
mwlVfh/e7f+2TF9N0ekjXNydTV1q7oO3+2dx237+bP8BLi7Z9LOgdfLr/WGp
heet497Axb8ZlfWU577cv1vF7YLy+Rdw8achs+8aBz3f78Qe6rpg5lO4+Nf4
VDdrzqf7PySfYTqc9wguLrDgZrHP80f7a1jfnDvQdR8u7n3y044V0x/sZ+rf
L2SSdQsubng8I33e0jv7oy7zeN1hvwYXf5Lq8Oxf5o39Ki/8LhW3XoKLX7uW
OOXrpcv7tRVDJtZsOgcX35+8fvp/4fP7p8g9kLf9dhIuLn3P5q7N2mP7X6ww
n8qYfwwu3pXru75Q7tD+detaMhV/H4GLf/i5m8lXfvf+iQ0brCNvHYaL/3rT
uvPu3E37r//MLbsijhDnODD/wR2LJfu//xETXzH3EMKcj+u2FrTm7N/a4vfw
fRZC/NTDlyf1ViywXz0ndq9FL0JcYOrXCsPEjfbX1l1+N4ERYb6N5bce77xd
9utrohZePY4Qn8ei7j4//6D9p13FNyaeQrh/7ptLxyVWHLWf7qcl2i+C8K+U
xPSs++fP2d/xWdjkcxURPlvNI2v3r71sb/Im7FdPCyI8ZQRlxX763bCPYD0Z
87AAEf7xHBOc5926Y/+rWzSmyhwRX0e9rn8W73lg79bluMQ+CBG/1isc1029
8cj+yFmDDZwZiPSw7ALjEs5/T+w/7twUxJ+ASD//2yVTNks8t19Z3FVr7oZI
b+ZhPTxCiS/tsys2m5buRqTPlPchH9KaXtuzSN1dftgBkZ4L51fsKbj41r71
kvG1M4aI9N/MudN7x4t39lOV2tzPWCHyCzR/2cPyFwBJlu26
      "]]}, 
    {Hue[0.6274275274955841, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL7Ccdq2d8Md+6nFP2/9//9+/7ODDM0S5p/tYeLv
bCZU+X14t//cn6ftL7Q/wcXd2dSl5j54u78o8XPIbs2PcHHJpp8FrZNf72cR
vDfplOE7uPg3o7Ke8tyX+zMvbJq8adEruPjTkNl3jYOe7/+w6SCHfu5zuPjX
+FQ3a86n++fxWx4LdHoCFxdYcLPY5/mj/TnrHmm9n/EQLu598tOOFdMf7I97
NmNmoOA9uLjh8Yz0eUvv7N+m9ezCz3U34eJPUh2e/cu8sb/RXiNyos41uPi1
a4lTvl66vP9TevP5aIbLcPH9yeun/xc+v39Z2kkxm0UX4OLS92zu2qw9tn9h
+KtFNnvPwsW7cn3XF8od2p8ea14e43kGLv7h524mX/nd+730kz2naJyGi/96
07rz7txN+yesUGjhTzsFF+c4MP/BHYsl+zesPqny+PVJhDkf120taM3Zn2fM
+F5rJ0L81MOXJ/VWLLBP8RY4XXwbIS4w9WuFYeJG+/7Ytj+n/RHm21h+6/HO
22Wfoc27XkAM4Z55LOru8/MP2q/KVHt2RgLh/rlvLh2XWHHUPuDvHaVTSQj/
SklMz7p//pz96b+aJw5PRITPVvPI2v1rL9uXn2VhSXl5CS4uIygr9tPvhv2W
mEWP/4kjwj+eY4LzvFt37MMev3s0/R4ivo56Xf8s3vPAftFDKe3Yn3fh4tYr
HNdNvfHIfoJQ6+zJ7Yj0sOwC4xLOf0/sF6TKi80zRqSf/+2SKZslntuHy393
OCaJSG/mYT08Qokv7SVVFH4y9yPSZ8r7kA9pTa/tNx31zA4SQaTnwvkVewou
vrWP0XhlPkUakf6bOXd673jxzj6nwmblZBVEfoHmL3tY/gIAi5/cNw==
      "]]}, 
    {Hue[0.8634955049953739, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIL57meO0be8D+6nFP2/9//9+P7caQ+k19y/2MPF3
NhOq/D682/854WnlMpvPcHF3NnWpuQ/e7p9+9JNbpdUnuLhk08+C1smv96/I
ybNerPkBLv7NqKynPPfl/i0zNpbrHXkDF38aMvuucdDz/TbVB96s6nsJF/8a
n+pmzfl0/6vr5qvOJj+DiwssuFns8/zR/h1lD/mydz2Gi3uf/LRjxfQH+6us
T29Q0XoIFzc8npE+b+md/X67H8/4+PEuXPxJqsOzf5k39ktcUFXszL8FF792
LXHK10uX93f+aJge7HwdLr4/ef30/8Ln99t+P3bP+OkVuLj0PZu7NmuP7c/Y
UyQ5TfEyXLwr13d9odyh/V72M5mXpV+Ci3/4uZvJV373fhuN9W8cr1+Ei/96
07rz7txN++8/4e36X4oQ5zgw/8EdiyX789OSol5aIMQ/fFy3taA1Z7+fVp19
rzxC/NTDlyf1Viyw99zWunajAUJcYOrXCsPEjfbx+x2O3c1CiNtYfuvxzttl
763WIs9wDiE+j0XdfX7+QfucowLWcf4I9899c+m4xIqj9vb8OTdjviDEpSSm
Z90/f84+qU99wu4riPDZah5Zu3/tZXu+OKYz0TqI8JQRlBX76XfD3uft/NXf
ohDhH88xwXnerTv274rfrOpXvAcXP+p1/bN4D1CfZU+llhwifq1XOK6beuOR
/Z1bzfX8KxHpYdkFxiWc/57Yv9aV+yUZikg//9slUzZLPLdf9PyripM1Ir2Z
h/XwCCW+tE95I3e3fgMifaa8D/mQ1vTa/r/bEvXnUoj0XDi/Yk/Bxbf2TT0v
NYN0EOm/mXOn944X7+wn3bGeHWiKyC/Q/GUPy18An9zj3Q==
      "]]}, 
    {Hue[0.09956348249516367, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIPZX1vlucuiR/dTin7f+/3+/v2GGx/eu+q/2MPF3
NhOq/D68259xWfdPVMkXuLg7m7rU3Adv9/NYWJ/ULfoMF5ds+lnQOvn1fpn7
2/aGJ36Ei38zKuspz325f26tU8104/dw8achs+8aBz3fr5IQ+FH432u4+Nf4
VDdrzqf7L+05sc7s8gu4uMCCm8U+zx/tXxBdJHxF8xlc3Pvkpx0rpj/Yn6wQ
s7V35mO4uOHxjPR5S+/s7z/EdDEx4yFc/Emqw7N/mTf2v5zprhXtdx8ufu1a
4pSvly7vjzq0aIGq0h24+P7k9dP/C5/fz3fwxXPWIzfh4tL3bO7arD2236Jt
ilIGww24eFeu7/pCuUP7ZTh381S4XYeLf/i5m8lXfvd+3k8XvgpvuwYX//Wm
defduZv2b1inP/liCEKc48D8B3cslux3MGhJ3i2OEP/wcd3Wgtac/fJf5nvE
/7sKFz/18OVJvRUL7KUalm9r5keoF5j6tcIwcaO9QWfS+fWeCHEby2893nm7
7GU+LNW4tAYhPo9F3X1+/kF7234jFwM9hPvnvrl0XGLFUXuBm72P9C4jxKUk
pmfdP3/O3jjCc2bvZkT4bDWPrN2/9rL9fXWVq7rciPCUEZQV++l3w15m+8Et
JywR4R/PMcF53q079vfl2G8ZNCDi66jX9c/iPQ/sWeVk6yb3IuLXeoXjuqk3
Htkf2S/SUiGBSA/LLjAu4fz3xP6K0Pp/TUcQ6ed/u2TKZonn9t1n2jRXr0Kk
N/OwHh6hxJf2vhfXP3whj0ifKe9DPqQ1vbYv3vlm5ewgRHounF+xp+DiW/tv
/wzXsKUg0n8z507vHS/e2bP5t8iy5SLyCzR/2cPyFwCj88Yn
      "]]}, 
    {Hue[0.3356314599949535, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmID5U5K+s8/2J/dTin7f+/3+/30nyFqfZvW/2MPF3
NhOq/D68268Vsp+P7fJXuLg7m7rU3Adv95/bceb2jQtf4OKSTT8LWie/3n8/
3uMC04FPcPFvRmU95bkv90/av+lC2Y8PcPGnIbPvGgc9378qsUQsa/c7uPjX
+FQ3a86n+yP2vb7cM+E1XFxgwc1in+eP9qvFTrXQev4CLu598tOOFdMf7H+p
WH/rcdgzuLjh8Yz0eUvv7Bc9ovptu+oTuPiTVIdn/zJv7K8657arOeURXPza
tcQpXy9d3r85qeRrxrkHcPH9yeun/xc+v1/qiJ7RJrv7cHHpezZ3bdYe2+/S
eStHpPouXLwr13d9odyh/Ro8jOEqB+/AxT/83M3kK797v+RXMYcDBgjxX29a
d96du2n/no2ld2vP34aLcxyY/+COxZL9Psan1qZPQoh/+Lhua0Frzn7t708n
cVcixE89fHlSb8UCe7XmtwwGXQhxgalfKwwTN9rb9KyQjDmCELex/NbjnbfL
XuPz66I6dYR75rGou8/PP2jvOami784KhPjcN5eOS6w4ai9z57LObV+Ef6Uk
pmfdP3/O3j66/7GlHiJ8tppH1u5fe9m+VHQr/9tDiPCUEZQV++l3w17kbnO9
VRQi/OM5JjjPu3XHPlVB63+bLSK+jnpd/yze88C+W95h8UsvRPxar3BcN/XG
I3uvg+Yrz11FpIdlFxiXcP57Yh8lckn5ZiMi/fxvl0zZLPHcXvDcSj/GDER6
Mw/r4RFKfGmflZnZPP8dIn2mvA/5kNb02t7Mqm6P20ZEei6cX7Gn4OJb+531
h/dtPIRI/82cO713vHhnf+q8qMHG04j8As1f9rD8BQBbutQ6
      "]]}, 
    {Hue[0.5716994374947433, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIDY9VOSvrPPcfmrxz1v//7/fH7Jiy5rryT/sYeLv
bCZU+X14t9/m8cyNyyO+w8Xd2dSl5j54u/+B+/K6qrBvcHHJpp8FrZNf7//w
Tql4iesXuPg3o7Ke8tyX+xf59RZL1H2Ciz8NmX3XOOj5/mf7RdaFPf4AF/8a
n+pmzfl0f9WdBZIXZr+DiwssuFns8/zRfo9qr7rcT6/h4t4nP+1YMf3Bfm4H
EwX1pJdwccPjGenzlt7Zb/7olvVvg+dw8SepDs/+Zd7YP+fN7//H8p/Cxa9d
S5zy9dLl/VcbNK3X3HwMF9+fvH76f+Hz+/UbzsTNWfUILi59z+auzdpj+48k
KdWcPfEQLt6V67u+UO7QfrMlpoVXsxHiH37uZvKV371//YwDqYuVEeK/3rTu
vDt3036G7/kmTJ8fwMU5Dsx/cMdiyf4T5rNkyq8gxD98XLe1oDVn/3ZJDf6T
xxHipx6+PKm3YoH91vvabtEXEOICU79WGCZutHdI6QwqfIsQt7H81uOdt8t+
m7h2PZM8wj3zWNTd5+cftHdP/9izMR4hPvfNpeMSK47aG/DI3zHYhBCXkpie
df/8OfuDp2vMExYhwmereWTt/rWX7acbZ4UanEOEp4ygrNhPvxv2Zr8tjtam
IsI/nmOC87xbd+zb7e67nPJAxNdRr+ufxXse2K+1/fpCJwwRv9YrHNdNvfHI
Puf+m3eCDxHpYdkFxiWc/57Y1xi1Z8r0ItLP/3bJlM0SQPeW6Xst2olIb+Zh
PTxCiS/tq344qzmWINJnyvuQD2lNr+3dT0bEvzBDpOfC+RV7Ci6+tT/FNy8x
2AOR/ps5d3rvePHO/lbch0tBgYj8As1f9rD8BQBxHLpR
      "]]}, 
    {Hue[0.8077674149945295, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIF4hOK1aKvGl/dTin7f+/3+/f4vrjfv5Ar/sYeLv
bCZU+X14t39Ow94n9qw/4eLubOpScx+83e/w9NRGIeYfcHHJpp8FrZNf7w/o
cVtm8+krXPybUVlPee7L/UxvNizboPUFLv40ZPZd46Dn+91iLR6+6/8EF/8a
n+pmzfl0/1zuL/YNvz7AxQUW3Cz2ef5oP8e2iU/mer+Hi3uf/LRjxfQH+09X
VbeF734DFzc8npE+b+md/d9llBZpd72Ciz9JdXj2L/PG/hh157TvJ17Axa9d
S5zy9dLl/VN3Fyy8Y/QcLr4/ef30/8Ln97/fJXz0DPMzuLj0PZu7NmuP7W+Z
m/nor+JTuHhXru/6QrlD+7/dq7nOevQJXPzDz91MvvK79+dfZj9zuQYh/utN
6867czftP2C4fZaRD0Kc48D8B3csluzvKHrYtFwXyZyP67YWtObsLw/JL/2p
gBA/9fDlSb0VC+xLpYrXdGsixAWmfq0wTNxozzD/wo6FbghxG8tvPd55u+zL
goqeGlUgxOexqLvPzz9oz77I4vPDHQjxuW8uHZdYcdT+g1uaUyI3wr9SEtOz
7p8/Z9/IdmTuhN8I8a3mkbX71162D8/buDtBHRGeMoKyYj/9bth/M23QXrcP
Ef7xHBOc5926Y+9eqbb610JEfB31uv5ZvOeBfW6FjX/sekT8Wq9wXDf1xiN7
HSmTUCcrRHpYdoFxCee/J/bzzJ6nq31GpJ//7ZIpmyWe21cJB9dapSHSm3lY
D48QkL41NT3ypgIifaa8D/mQ1vTafnlazaSGx4j0XDi/Yk/Bxbf2WmsOTnn9
5TtcvJlzp/eOF+/srf4Is73+i8gX0PxlD8tfAAXLxWo=
      "]]}, 
    {Hue[0.04383539249432289, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIJZK3LlccNpr+6nFP2/9//9+f9JHl8X/Hv+2h4m/
s5lQ5ffh3X4vHa0V5278gou7s6lLzX3wdv+7CealC6/9hItLNv0saJ38ev8/
882Zp49+h4t/MyrrKc99uX/jdNks+39f4eJPQ2bfNQ56vv8Lx9Iltclf4OJf
41PdrDmf7vfZnsDMf+4TXFxgwc1in+eP9m9q27o1tvIjXNz75KcdK6Y/2C+S
uarNyvADXNzweEb6vKV39oeU/NbwNHwHF3+S6vDsX+aN/bvqZN6KFLyBi1+7
ljjl66XL+z8JeWn8uvkKLr4/ef30/8Ln9/sIvfV+teolXFz6ns1dm7XH9j/8
4pQpffIFXLwr13d9odyh/cFmYXHKOQjxDz93M/nK795/Ruth0EdlhPivN607
787dtF+2c4JiwOfncHGOA/Mf3LFYsv/pyV2cR68gxD98XLe1oDVn/5XVHn/E
TiDETz18eVJvxQL7S4U+xisvIMQFpn6tMEzcaB/7baX9/rcIcRvLbz3eebvs
L6/0zgmQR7hnHou6+/z8g/YpPwWr/scjxOe+uXRcYsVRe9959ifrNiHEpSSm
Z90/f87+XuxclXWLEOGz1Tyydv/ay/ZbjnY51Z1DhKeMoKzYT78b9sG9kZtO
pyLCP55jgvO8W3fsl1z4byjhiYivo17XP4v3PLB/+pwr21YbEb/WKxzXTb3x
yP7IDrZ36wsQ6WHZBcYlnP+e2Ps8CtbeegyRfv63S6Zslnhur7F/v8NJS0R6
Mw/r4RFKfGnf6XhFJuMLIn2mvA/5kNb02j5M6GUo/3ZEei6cX7Gn4OJb+xvh
ehFVJxDpv5lzp/eOF+/sXy5vOFl5EZFfoPnLHpa/AAnx04k=
      "]]}, 
    {Hue[0.27990336999410914`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmIO7dcOcyx+m39lOLf976///9/vjVv7413/trDxN/
ZzOhyu/Du/3uz+7/Dr78By7uzqYuNffB2/2vvd6eULv4Gy4u2fSzoHXy6/1/
PmbsDTjwEy7+zaispzz35f51gZf3HvnxHS7+NGT2XeOg5/s/HYr6wRT3DS7+
NT7VzZrz6X6PQrHwKSe+wMUFFtws9nn+aP8Gh2zOx8Wf4eLeJz/tWDH9wX5B
5YRb+7U/wcUNj2ekz1t6Z796ifELTeGPcPEnqQ7P/mXe2B/10/lUv9EHuPi1
a4lTvl66vP9N29bzLOffwcX3J6+f/l/4/H63tojPX+a/hYtL37O5a7P22P7b
GesE1Pa9gYt35fquL5Q7tN931UFWvUSE+Iefu5l85XfvPzbP59dvCYT4rzet
O+/O3bRf/O/vE1EvXsPFOQ7Mf3DHYsn++7ZqG8+fRIh/+Lhua0Frzv5z8lsW
K+5FiJ96+PKk3ooF9meebr+1+ShCXGDq1wrDxI324Vmiz08+QojbWH7r8c7b
ZX9WdrtwtAjCPfNY1N3n5x+0j8utVeQKRYjPfXPpuMSKo/buQqsLOpcjxKUk
pmfdP3/O/sZFjjM7piPCZ6t5ZO3+tZft11l+e9lxGBGeMoKyYj/9btgvb3ik
k62LCP94jgnO827dsfdftDVHUB4RX0e9rn8W73lg/2jhgX2HVBHxa73Ccd3U
G4/sDxXtcTTNRqSHZRcYl3D+e2LvOYez3uogIv38b5dM2Szx3F61snSKnwki
vZmH9fAIJb60b/vdWfD2HSJ9prwP+ZDW9No+6My8lVM2ItJz4fyKPQUXgelF
8Mnq/4cQ6b+Zc6f3jhfv7J8lmsn+P43IL9D8ZQ/LXwBSzwgB
      "]]}, 
    {Hue[0.5159713474939025, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxTTMoPSmViYGBQBWJmBhj4YD+1+Oet///f718Fpv/bw8Tf2Uyo8vvwbv+k
dffun//yDy7uzqYuNffB2/1mwm/WLPr0Fy4u2fSzoHXy6/3uu9IXnnn4Gy7+
zaispzz35f5fkpcWOkj9gos/DZl91zjo+X7b7sg7dfU/4OJf41PdrDmf7p9i
KWot8OobXFxgwc1in+eP9v/myHobN/UrXNz75KcdK6Y/2B/zOm6fte8XuLjh
8Yz0eUvv7C+2MTqz1OQzXPxJqsOzf5k39i896LSOO+QTXPzatcQpXy9d3n+o
+6b3v46PcPH9yeun/xc+vz+fe3Vs2ccPcHHpezZ3bdYe27/zT+ibvgaEeFeu
7/pCuUP7i+befyYUhBD/8HM3k6/87v3rxFUf/tRFiP9607rz7txN+68fq/FY
KYoQ5zgw/8EdiyX7rZ9P3JPMhmTOx3VbC1pz9hvfuLGVhwEhfurhy5N6KxbY
G+2OuhjKghAXmPq1wjBxo315bef1B0IIcRvLbz3eebvsjcJv5a7URojPY1F3
n59/0P5X3d4vDD4I8blvLh2XWHHUPj8wZI5DKUJcSmJ61v3z5+wtDJm9It4i
xLeaR9buX3vZXvnpzwiHZkR4ygjKiv30u2H/yeOhwHM/RPjHc0xwnnfrjv3s
rC0BvQ6I+Drqdf2zeM8De72s/Yvs3BHxa73Ccd3UG4/sea13q27sQaSHZRcY
l3D+e2I/NYUjedsjRPr53y6ZslniuX2BY0nFqVBEejMP6+ERSnxpf+lwR0im
ACJ9prwP+ZDW9Np+3pS5vQKXEem5cH7FnoKLb+0Vbz3qr36CSP/NnDu9d7x4
Z2+kZcpY/RaRX6D5yx6WvwC2pHYp
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF021IkwEQB/CZIhMNlmK+9IaECEWCWqlsdbOgiVMpmaVRiqmZI9vMUsm0
UCTLWVYOP9jmCxqzcmJlqLOWhooimRN0jqnbUJe56bO5DTdzSyTPg/twP477
cPAPuMFJyNxDIpECt9p5q/l5VoXDsSolbRchfbc9O2CFVv0gnlhBfyWenRsz
2YHhGuQvUOnRT3vpPjQZN8Gv1Motf72MzujJahxVb4AlNJ9XkLOEbvOTNdL9
bbDAqpsJS9Cin6lMVpY8WgdzauYFqtsCek2kN5XyxwKUhum8WK0GfYPM1qfw
zcAcNnaJalXo15ZTvlHjTBAydCtL2KJEz6OFjracXIP5TPqiPVuO3tJ3TuzO
MsLkZFqNWTaB3l85zbRXGECa3l7r8BpD57i/v55vIODALG2G1jaI3v03Uff8
MQHPcuLacw/3o98VzC16JhBAWCV74o5I0MU+gWrrCQJsuvLuGcFH9KnBh9Gt
3gSQv9erlBHN6FTty9501607BnEnt/w2ephc3ulBImBEvTQcLGqAHQ+VXB1P
dCGAwjcXhqR1oBcUP51SeRJAi7TwmHd6dvevKHJajxMgdAli1HP60G0lX02k
WAIEOtmQr2gAnXOJ9YZ+nwB/31r23NhP9IgQ55gkPQGd4cnF0rYJ9KML1iR6
mQEO7ju03xovRzdGqynaeCOkkqvPCxVK9Dr254tV9DUYiJla8+Gp0IPZ0qaz
DBNQRVFivlyDvpcqCezgmeHtL6dmN/s8Oj+DnP5FYwHHE7+MT75adG7UvcKR
xHUIv8zz8ExbQpf9qGBlU2yQscoibpYuowtrBFWUiQ3IrS/s5Y7r0QMUmhdF
85tQ5tbN7Pq9svvPY6ecivR2+J8v9J18/QOBuIip
      "]]}, 
    {Hue[0.9060679774997897, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcwHFJ3db1wihGybkO11XuUEnW/Th3Vp4qKaU7WlHykEkP6s7V
cZRuXe7i9qKMSlk5Q9ohD2PlYchid1rLw1oY9/9v9sCYYXmR34vvm8/7r8eV
zJhkWxsbG8/1Nq9Xmm1WWK1zomd1ozJKv0aUWL1syh9fBS2j+EG0TovOnJ6w
nJWtAHOrt2uZUoNOhGt6vYYs4JJnZhe8INBX9CltpzvMYPK7y7mXMYsuOCNr
+7a0CFOxL8f8Y9ToBnH8km2CCRYSk8OCqFPoJ7Kc4kp658G+4ld2pFqFXhec
Rv2bbYQIiaGJz1Wi0/ZeVoj2G8C3J+U6790ouvdt/5l9jnqYTA6eXrshR483
h/Y999PByAirZGFYhk4WCqV2Ui2IrtZyrY5S9LDCC8b5cg3sGmeMMWq60X+n
COy92kl4mhFVm7VHjB71sXOLD4sEnbnFNsq9Bb2bF7lsoZOwTBY0j5V9Qnde
tfTGzxBA6ShXjh6tRJ847lUvlRCg0wuE7IJ09EH3z2892gjo+zMr8eFXwIYP
TDUqGroIsC9dyPFl1aPHpe5US1QEMAJNnIibX9C/uzU6XtpBAs/Om1me2Yme
kJHrse0cCWXkcA+d34XOdKhmF1WR4Ernpk5IB9HlQ5SBJq4GhAEXc0U1MnRB
oGn2yVct7Ka5OZmj5ehVj1QH0g7qIJFSHMpTjKKfeiNMp7nroSv8p9GZo0RX
ve5oF3saIIgfIiiVq9DFt1pDDqcZ4f2PTZXUtUn0k6+oD491zoP1sUtSA12N
7nn/Tkn0IRMEnOdsd2DNohdaitga7SIkzcXqruUR6DEDvA8l9WbIKs9pZQ9p
0Edok9VWsQXyqc0RTTNa9GnWETdr/wr8/wt9469/zDgagQ==
      "]]}, 
    {Hue[0.1421359549995791, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmHTEBeBusTmKRELRUNYQstcfGqR6aaImuWF2dSVM82JHVRQ
otBQFgzKdhGbVprTUHE2PAWraOoy5hGbojbN9d90m9vclM3Y8iK/Lt6b5+q9
+YUVlGey3d3c3E7tz2N/zZX2RZdrSxbIGurwfbkpKzBfeuv8tQdGOr8m3WRE
Tz0dJVaqHJDsGREoVBvQjXza/dfzdgh4bOfWP99Ed9L6ORPyXdg584D3sEyH
3icIKQGnDTTZr5ZjMwl0q1d7W22hFWz57MvxJA06c+Cmh7fSAj6tC5VMYg1d
0iCV5lWbgaGwDIoFanQ/TlfDeaoJqGPFt0XtS+jZ9/YiU6hGWGcn/HZyVOjD
dcEGP64e5udZL2wzs+gWcmqkY2EDZIW9Atexyf9/yAbGRpcOglboy/TuUfRV
axInSKGFprK03ooTX9CzzubcCC/Vgsk+4p4WOoL+PWo10xyuBYe+fmhZKEEP
aeSHZWwT4PWpRb10rg1doxgmyecIMJl7pNz6UvS591f++I8T8G1Vp4gRt8KB
z1QwYzunCPBptlVRWX3oeTudIDMQQI/b4THuDqPPdjJKM0K1IDockdxS/hm9
yO5b48rXglA/M0YRy9HTRKCok2ghkCIo+TmpRF/JE57seaMDKS23VtY9i/5B
3pRUp9yAYN8Qf3u6Cj3rWa5kgq2HfC/+RdHiEnrblItKSTGCPPXH9nGeGl1D
HLlzIdoE8eLEnmbVGvrXQU9jL9cM76YOtZGc6+jMtaxo6agFXE8DivopBHqk
TJagiLMCLYd3lMzSoTcmzgUXW21QtJVtuvV4Ez2HrLvqPbALFS1VH7nTBnTV
tZjrNeN2eEIaYgxqjei6jkeK6mkH/OsL/aCvv/RB5gk=
      "]]}, 
    {Hue[0.37820393249936934`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tM0gEYxmHSPM0uVLZSstK2ZGm1SQdN2T7ThVseamY5t5xRitlSPEwl
DxtZVjO2rCS3ClBnypbHylKp0BaZ5tJkU3SesD8igkAoKJSaF/ldvDfPxXv3
873Cjku1I5FIBzZnvzl+rnV8Y8MgFbs/LaIwNdK3pxXTbDcb6OkVhbFGPfoL
7kcCHKwQ6UilCGYW0cNU/W0e9qvgVWrNKnuiRT/HY9TTTWaw0PJ5BRkadDtd
a32r/zKo4p9PHo1TozOSgpX6hyYwJ6cyQl1U6ALXZeDajOBWPZYbrZ5Fd373
iBBEGSCqz9QhrppB/15YdDdBooPA3mtpwpcT6Cve+2sDyheASA2bW09XoF+i
RrBWvs3DyAiz0jwsR+dLsmomaGqQXm2p2iAPohu6yLIB+znYPUWfpDd9Rb8j
SJ9d81VBeUZMS/bez+iWqeJRBxkBRqvELmafBJ0tdxqQFxNg05V1Tgpeo3cH
vn9GiybAuVs0MxFch34/R1nacHjz53dze1bZDfSCeHae1YeAfqWm74i4GrY8
j5Lb+OAgAW58MyeQ2YZOEg111DAIoJ+08KIyu9Dz43JUNA4Bwu3USBG7B92p
NnhJ2UGAQDfc6ymWoRsZrHCmqwoonlXXpwd/oN9y/CKo+KOC9qDEEmmTHD0h
s01ymaoGb/c9O62xCnTLcW5A86d5SHauiBCOT6BH3vR7ZatZANmZ0aVdvBn0
DA79bFKLDkLFp5r5iln0Q5RjF8JDDFA/tK3OZZ1AF55Qp/ktGWHjnlfKG081
eiH5fEkIywRBF3k7PJga9HF+WuKYzzKkGOKNrFItegOr+DH3lxmyRZwPWT8X
0f0beyq1yytw26UzqmNejx7yl+yoXVuF/32hb/X1DzkC1+o=
      "]]}, 
    {Hue[0.6142719099991583, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tIkwEYxnGziCQprchplqWUhZLzOGOrV7FUNI2WiRG4PCzTtLkWKktN
DJFiglCySp1iK0d4QMvSDKaSh1mpaZ7GbN83D3NsznnYdAkuL/Lt4rn5XT7w
P5HIYbKtraysTm5t59bKeGa5xbIo9e+6f8XNUy2NkXyoG09aBz2jlB9t0KMz
pl821catQdhud6dKYgGdCKvN58eawLHQnFn0TItu0LvyxJdWweSTJcjO0KDX
RJfwKPnLMBtTPuXLVKPPSQ81xE4bwMhih9JtZtH5imrHoXI92FVP8i6rVejh
DyPyM5a1EClbbpUICfS9QX7H3RM14N17J0X0RoFOU8npG1Q1zLCD5jZTJ9Ar
dBuWHs4sjI0lPDcOj6CPFpyh101OgzSpUWg5OIjuVfA9vuKdCo78Zkwx6nvQ
vya65v7oI+FpRlQj91gXeoDYnzt6lwSDud06yqUdvfFFB/u1Gwl/dEVtU5XN
6FZrHD/rFQL2dFQRikAxeh/tlXP2LwIMSw0tmUXp6J8cT++X9RLQT2pkZyXV
sO0tSo/Qm0ME2JUZc7wTmtCDkp8wuQsEMM6ZBJH3PqN/dPB4ZO1CgmiXe1gV
pxM9LGVJ0MQioVI33EuRdKNTbV0U1GYSnCjCNOXgAHrnt1zarRoVtNBu5Enr
R9CFvmnXqQPT4Gx/9LA5egI9YCOwO489C6w9pSEiuQK9+ILyYn+4Grojxlcc
BAR6/XnjvGesBuiS4IayCRV6ulKntye18HZoh9hmcwY916c41blED5Zix+T3
FDW6U5ZXRE2bAWixAtsDCRp0/nrIqeAHy5C8GGO4Xaj9/4MsjjUfsArcqpwv
mT8X0Pv3iRKuhZvgsU1bZOu8Hl0ebxhmXl2Df32hb/f1F+5TzNE=
      "]]}, 
    {Hue[0.8503398874989481, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIkwEYxvFpIY6ktNScWSZmiol4AI+r1w5oeQpErKyQmcsDmmczQyWH
RTZKVsML0QlZzEBliuQJp0sTLbblyOb2fW7N6Rxzc6mbrQuXF/VePDe/ywf+
frmlGUxHCoUSsL8D++NW2hR2+6ZQVHHdP2RXK7xEU1Cjlq1gorfWpZtN6MGZ
wsNOMgskOQV6d6iN6OLhr0q5dAdoTbay5tcGdFXOVanj5BZYI2rYD0v06Bzh
gLTmtxlWM9vJyAwd+gdGlWfRmAksOczEeOoq+s0Jg4zdagDXrqXKVJ0G/exd
bkywbh1S5raG+W1qdL1fo2Ilaw3CZwvyO98R6B7TAdaPAVrQMhPW9grl6HXi
xFFWngYWFxlvLAsy9MHcKkuBWA3Ce/1t9mMSdO/p0IiBCyo4sUwn6b2f0a88
VxS7PyahpSStv/yUCD3IxeHGmSkCzLYxxzTfMXSaxTNhMoyAPxvNI2THAPq4
oJqslyjBeZKnJmK60VMj53vzOUow/+obKmsuRj+3u8o59EgJ8z/1c6H8LsB/
WEZKWIsSXLmW2nCGAJ3O5tPuTCuBHmtlpzwYRQ/aNlQ0BBLQeTAwiVc6hX6N
U/uS4BPQsbEw68WfQfchZCHKNBK8vdqKVBIxOtx+tRIbqoKh6Fv1wl4ZerXH
0BGjSA0+bic9belydHeS1RiXrYEc59bLnQoCnXk62P70vBZmkn9sH2er0V/4
JrzVJ69BPP9iH1euQU+eiu4Rf1+H91KHbuqeFj3bfcF/6YkB7M9oeYNeOnQ3
cU+6Q4EJorPYLkcZevSiwkIWz2SGvM1M8/0mA3pUXMN4omALynm142XfjOgj
jZ8mBKIdYFFHUobXTejzEo8wwRcL/OsL/X9ffwH8Cua6
      "]]}, 
    {Hue[0.08640786499873876, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02lI03EYB/BluRxZmUZO8yhTzEibGqm4eOwgNVNJpEMDm0etyLyiRmSJ
V2QbCipLZG6E1irdKtO0GcuhqRHpNv1vk9kOj21M3aZOWcSWL/Lni++bDw98
4YHvweyCtDwnHA4XtJ6t62kssU06HCZh6qGja8dFWmHZi4S1midWWCTXPUwx
LyKnSkP/ZtxbgXh8sDdLvYDcNTp2JLR4GbzKbYVV9UbkPqrur5cpFliNuE9/
kG9Azio9/YgZaYLZ9OapyDQd8sDrFy0ediNYs/LOxRJmkUv6hnknpHpw4yhK
Lui0yDmZxR7jIXOQNLLUw2WqkeccuNbFaJqG8CHqzZY2JfJakZOYQtXATF7c
nP2WHLmhKf5IZooKMIzSYJVIkWeIXnKCApQgzOEzHR6jyHf163XOAwrY/5s8
Re74jjy6uiGAipNDTX4yv8hPtPkHgsCVdk4GZpvAKdlfgHzn0pjVoxuDP/NV
vVOsj8jf847Vi9MxcPnGViujW5HHkSpzBJ4YmC28rsKqO8j9V9gJWfYJ+KEx
jIRxObDh3mWvuyt2Y+DWaKWFUz4gJz3LHuUnYkCOWaUn3f2C3MfcdljSjkHL
tuB4dkE/8pO1EWdJYTJgzUuGiNxB5G4KhjZMKgNvIvO2avQX8sgriU2MTgV0
RV0tFXZIkauCAydCdyjBZ4/vPluKfLP3c/+n4RgVZLnUnWmZVG7e+22fJJVp
YPC8bNmTrkbu7Of7uJ4xDbHcU7xGuRb5gHBvJY04B6/GtrQS7DPIx9359vIB
PTieeuV2EnXIn/+sDnn31ghRl+iu7hQD8mQxX6P3N0GuKd18o9yIvKR3/k1z
mgWK2LS+QvEC8lV7eDs+dxkqCL1JPfpF5PjUSl98/gr83xfyjX39A45S2Kc=

      "]]}, 
    {Hue[0.3224758424985268, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF020wEwAcx/FF13EVqrvapLndYXFyyXVoc39yx2XokrpLHXmWxsh57EpH
u44buUrTOeZKbuk8VTrM3dQ1TgrbnLHb2DBrmZnN1LywvMjfi9+bz7vfiy8l
hRWXbkcgEDx3Zr+z+gKr3GZbEyqlDmMhNSrhQS9C4XTkBhjodWWxRgO6+bam
tI1uhsgDVNcm1So6V2SKKL1gAlKFNY/9fAWdz8ylvfE2wua5Ik5xjg79U0NP
sd83PWjiG5UBcVp0+v0hfXutDixJ6RE0Rw36b1lg+8/UZXBpmS2I1i6g9xWp
ne4OLAJj1NTH56rQy2hj3R4+avAfycpsfqtAjxUsNqyvK2EpPXR5+84MOnHS
k1LFksP0dPILi0SKXvX3EfdquAyEqV1c27EJ9JA/w3MBmik4OUdX0juG0bMG
75FeUqRQnRPTlU/+ih4Fr+zbMiVgtArsYtwFe39Pd+nDZGLY0rP7lU0f0OeX
DlfbCsXgMMRTKYJa0VkZKQm6IDEY1zt789jMvV8+D6HGXQzf1bpRP34L7Pql
z+yOnrNicKm3lPgn96AnCUOHldlioAdvchi5A+gMr8fuhHExNO+nRvJYX9CZ
Ihda4mUJNOklI0S+CB2cmbO3NiTgSuRmz0+Mo6fUUusEU1PQG3jjgbBDiu6U
aPfjpq8M3I6cOm6NnUGPXuW930yQQ5JDXXizXIFuKNC3P6XMgShKZj7BUaG7
BXNKfchqoPHDOutnFtAV8spy53eL0Da5r9Vxewl95Qx5i3RtGWxPSGkfiVr0
11qLx0WaDgKvcw4dTdahp+nJyvJuPaStxRszKlbQbRGtVK2rEfJ5JYN54lX0
Co7OO87XBJWO/Yy+Xwb0Zwpa45XzZvjfF/puX/8ArCz2XQ==
      "]]}, 
    {Hue[0.5585438199983166, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHFU1nCridYQtNKa5avlzAM5aZ/VllPS0jmdS3Mo+CifuaVr
65BGbo0mhg6bE42Zo5polE2zZtbUAB+kjdSAEBEEFRCnjaaSh/wO/8vv/j+R
V57G8SQQCKf28tqruco173bbh36QlBee9GqGTMOEOip9A2wM/r0Uhw19cnup
fjnSCQkHQgPa9Gvod1gb6YPh60CrdVVwBSvoRF/dU0W0DbbO3eVVl1rQi1Uy
gUxshaX0Vm1MmhndIRsmRZWaYTOXcyXOewld5BM7mnrJCOSOuaqrZgN6idQQ
YW9ZAKbc2S8R6tFvmlqepfrqIHqsqFD0QoP+PsKkcknnwMiJN+0Wz6I/gLCs
xjNqUKtZTZvTM+jOwrqpG4QZGMrvEboPT6F3FcgpDLEKAnUMLaN7FP15plXM
+DQBj0uTeyqPf0EvzKFXZyeOg8M16JkcNIieFJWf2BSmhL+r3AFtmwydLwl+
6FOgANLndr3mfCd672t5yOKKHBzr0r4Kbgl6WYyHPWJADooFi/yspAP2nc0k
K6t+yYHcvFkTzXqD3pDzaFt5TQGM2C0es+wDelHkoR4yRQkiYmhCe/kw+qvi
06Zx6ji0rU6PUSUj6Nd3NCcVeRMQQBXe+j01ia7cCf/2tVEFffSs+0PdM+jV
E0Qi2zINR32PUVwps+jvssWLu/5qyCXxL4vmNegZizaDUDcHI0k/N/x5enTx
QkBkjksLcZKL0uZZAzrfj9sqqF+ALpVHp/euEb2DE0QRxRjBXU9jv6Wa0TOD
/sSP0sxAz+Ad9GNZ0GkhwS6vBiuw7emOgtoVdNlI4u20IzaobK/5WPF9DT07
zEpvClyHOu8BZv+yDb2khvFSEOKE/3+h7//1D89v7rc=
      "]]}, 
    {Hue[0.7946117974981064, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIEwAUgGE1TUdSZnmtlJUlFFSmMWuTo0YTvAwaJimWma7UvExGJqIQ
thWshUoO6bKpYLYIpymCumK2eWFim20519hyDpcT59wUF7Nw+ZDn4X/53n/i
nSo6w8fLy+vkTnt24rPceo9nTWovZVuEVJ10f3k68VrWOtgpzXU0hx3d1X3u
FDHVCWl7YyMFplX0V/uSNjbAARGNbibnxQp6DqNaRb5lA9eFGt7DimV0SQx1
5oTKCpbs18Z4+hJ6qv/1qx0vLbBZwKCSCRZ0R9G0j7zSDEEdP1iZS2b0ej+b
cpQ7DxmK9SFRmwndp0kanFCmh7jJknvCtwb0PE1gusFfC4uM5F/bpTr0GCtN
zeKoQastbN1Ua9DPELNb6vuVIC3qbfMcUqG3Rpmik1wKOPKTYqT0TKBbRSS+
d9UEcCuyequjZOhiMbuU+GcMHG6JT1a0BL3lUR85Vy+HLRtn2CjoR59zV9R8
D5NDwGi7yZDYhf77b2iYSCADh1M8yOSUow+yaQtrZTKYWlhWnBV1wK5/eHPz
c+JzGQTxN2vjCj+ia8Uae7O3HCiXXLyMyhH03vq8ztlJOQh9Y9Paq76gr4+w
dC1TYyCwqSfDRePobbTTIU2HJyAyvK1sXqVEN2R2NmbOKmCQlNsg7dGgJ9hy
tnhsJRw9eCzUTdOh3/BT5C8w1VAQ0HxFqDegbz0Lya8jaWE8fW4jjGdCp3JT
uoCuB7IoRczXmdHHvp7vI5TMQ/eMdxdhexHdOdxPP3DbDJ6nEcUD4Uvo71nc
BhLVAqQcXmBw4TL6/dqBiw8kVihey3bcbVxB9400vpMn26C6vfYT89sqOkcd
r52Oc8BjwnDGkNWOzj/+JG36shP+/4W++9c/+n8ASQ==
      "]]}, 
    {Hue[0.030679774997896203`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYBXAzNRcVXkqn3fASQplkZitcfIWhlGkhUg7COS9l5uZEUlFE
M0yTCVKuUeJmorEIXReMzWlbirfEmVtsa236z6ZrXua0NGfm8iE/H87L7+lw
4PilZMenOzo4OBxaz9b1cHNtOrt9TkYaPFP9Sq+S3Q80XC6tmgcLtaYwzmpB
Z9G7Eq6UWCHaJci3nphFD6gUREatWMCnzMYufzSNfl79OPipdgqWjudx8plm
9Bfc20fLLphgIqHOEBZvQj9WUNi45mmERXp6VARpAn308zzdz0yAW8OX3Eum
cXSa6eSD4hI9xAwsiIU8An2MvX3ImqSB0L6Mm/xmPTql7ZrTk1UlGNPPTq7d
0qI71Yb5MH4pQK1m1C4qVeiTRrPzUl0/yFJFPLvnMHrc+J3ad2PdsHeUaqC2
9KI3Zg2RihLlUMWMFeUc6EIfMZ7oCB/oBKtN6hh7UIrOq+QkN1dIYWWmXGKo
f4PuX7RNXefYDq5yAaE/1YSecpfmHblLAtb51jZ2edbmPjX9L5VCMXz8Zh4I
ETbAhg/KJ1PTZsXgxl0sCGW8RvebUPbuVkuAenqJE8NqR1eWEkf4KVLgOwVF
C7I/oCf+9XjqldEJ9TPKPrKwB70xsNTTfVUGvmRe5tiwAp3nRvnd874b2ii0
YlmLCn1ONMNyYfXDPvf9XrY4LXqmIiiEOqgAumtNJF+nR9cwk70owSrouaj5
6c0h0AOyOEmrURqIEJ5r5WrH0Z+JijJ7r+vh+actTaQ1IzpxWCti6giwV/ik
vSWbNnsaRTqu/DtQrnJ2eDDM6DSz9qEq3ARpcwnWG2XT6AtVjs7+0inIERR0
sEdm0Zf3VJOWDRa4R5LEiH9Y0Hd+Jef9ybfC/3+hb/zrH7qZ9T8=
      "]]}, 
    {Hue[0.266747752497686, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901lIEwAYwPHNLFxFbKU2tbQYIhSYR2Hl8Ku8wmPhkNRCxGPZpOVUUAtK
8WCSExUTifIodSzDqXkwr5Yujw3SpeLVzDmca+kOsxmucPmQ38P/5ff+P5uU
zmTZEAgE970O7FWdtbNotRolobEGdpFmQhKxLj4mMJjAQK94zDAZ0FMj6B4b
o0YIPeThXKvSo7c5xAxVDuvBqWCHW1y1jq5IZrRoGnSw7ZPNz+Ho0C/RaO8+
U9ZAE/1yyZepRR846HMmTKkGcwIrxJ+kQZ/5xOxfSFoGcsNCVoRWjc61ERTZ
kxchXPZTLKxRoW+XPhrnGWfAe+x+al2zEv2WYtR6oUwBq6xra7vseXQfOYVc
7iCH2dnE5+apaXR7l/G/tEEpSJLbaqwnJtEtXkJHGfEDuHyjL9FbR9HfBMcQ
mVs98IwT2ZbhOozuGbX5tvJIF5h2+m0i3frRzb9uMojyDrBsFPcu1b5Hz9dK
pb+D2sHuY71KebkJPe/r+cMBr0Vg2hR1c4sfoOdYSL7xs60gX9HJPIUNsO9R
d0Ia8xNEQK4253ondqDnRZV3cantQL+yzQ9/2IfupX/lZGnqgDpbj9D69CH0
HvYkL7OlE2o3psaowhF0QiWdyAnoAWdqTdry5AR6adk5pVYyCN1+cU8krdPo
rmty1sVyKZyinHbcYcyjv7A95J62IYMEu4rAukUlurFkkjIxoICRsLmtk3wV
+lXen7iquRnwF14XVc+r0cXyH6x4/QIIFMQm0u4q+hzX7W5m8DJYeU4pnVQt
Oi0zNXalRA1+t/lHjyfq0DXZFWS3XQ2kGKNN9wrW0Z9S+wo4hTrIqM8d4H7R
oxc1BvECBXooJPWGi78b0JsDdewbYiP8/wt9/69/Xp/2zg==
      "]]}, 
    {Hue[0.5028157299974758, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01lMEgAcBnBTMmxWmqXi1WyZtWlNe1CT7W+HeeKDc7Yeykjt9CLmfaxo
1HLM3NKolYCNlLaQqXmgFkqB001UKCMTwwPRBCQLjB4kH/Lvw/fye/i+py/w
Sn5qtqODg0PQRpw2Uk+3TdrtK5IKH6q42V0uudyvuSgtNYOJXFuWYjah11qZ
skMaE8Q5B/s0aI3oagkvY1htABLDVsB8vIxu2/U0PqBrEazhRazi3CX0SyM5
Cc5HdKBLe645kapH14nK3zNWZ8CSkX0u2kWHfpi0sJb4SANuvK/0ZP0sujQ9
aoBJVkPS0Gq3gK1Fj9e6brf5qiBs8Po1zqsp9JiEtyWn6SMwnx2zsH5Djf75
duRDWp4cJiaodRalCr3rg6nQ2bMfJJkitt1jdKuHYSHWtHeC7zRZQxbK0ZWh
CfrQRBFU51JEtAAp+o+zlZHCstdgtvU6Ug70ohexQu6y1vjw18AUaxra0Fm0
gzmlxkYg9nO1U5F8dNcquWmQwAPzz5aOAmYO+o6TCccVdA4MzywNHRPwYNP7
bAN79t/nglu9pSSM2opeHSum3KprBHKUlZWU14NuqPjkVSXkA4cQHMfNH0D/
Uxj1O69bAA0G5aC3QIbO+uihz9QKwcebffP7qAJ9fLpY8YbWCR0RFyolQhU6
w3/uXeiYBPzc/T1tKWp0sjCEkxwrhwxi7RnO5BT6nR7ZiyrlCMgSv/zyYmm3
dr/VPCEQVBAtONVSr55F33mg6qWCpIamsW18l/V59KOyNvYcTQP2B6Ssdm89
erJfq5XEnYGIdJbrXuoS+u7AyZbz+3SQtZJmvspYRm+WOgU2PVsEGrekr2Dc
iC6k1ARV9Bngnos4qXvRhK4whhPLFSb4/y/0zX/9A56n01E=
      "]]}, 
    {Hue[0.7388837074972656, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMEgAYBWBTK5yuqUleM62cD7VMq1lK/bmcNhHXzGHNB4eXShepkeYs
X0wqG6Ur8ZIDbKGSpY6ETSVHhkZmSxHEyyAQMTQFoYYbbUk+5N/DefneztlO
eE5Rer6ri4tLxGbcNsNmOOaczjUprb/Dp+HOgFT2knx66pAVLKS6ijSrBZ06
SKtOSbBA8o7III7ejM69fzfUSlqFwCpHMfPZCrpUxdiZGLwE6zFlrNv0ZfSD
9ecJh1lGWMxo0R5LN6G/LsusFFDnwZ6dnxTvsYgubOeUNmRqwLt1lpFqMqBT
jNYzCvdpII/+7BM06tGV9OGqRKMCouXXrnLbNOiX8g7cmnUbA2P+2e8bBTPo
5scd1QSnDNRqWr19Uvm/16DEEEeRgDS3p9G5exy9qE/plKiEEPyNpCV1fUSP
er59qobVBo/olJ6S0A/oT3IYqb4VPLA6JK6UfRJ0gU2hnWA0wO9VZr+W8xZd
JRns7cyuA8J7nl5zko/e8crzZm1IDVht3eJi5nV0ZkQ10cvGhM/zy6NHBK2w
5Wv8UnUz8SF4s+3l0TQhOk+uNodk1QLp1DqLfGMAvX4y3FOkYwPXPTKZVzSE
3jm8ImPbOcBZnZQHCEbQfxydUEn38yEooLFQN/4V3YVIHZl7KgRx7OVKaZcS
XXhxSJRlG4AQn717HGkz6BmfWpoXpmSQTag7x53ToDdNHW8i5o7BSMr0L3+W
Ht2/IMnvwhcFxAsSutkzBnT2mxNhQQY1tE9s43tsGNFF6YXEqDgNOB8E5vUG
mNB9F1hicJuHWCrLy5e2jK4yiemV5UbIW8uwXqlaQafX6HRjf0xQwit/V6ww
ozP8Co0vwlbhnkc/uW/J8n9P0S55a4wF/v0LfetffwEybuTg
      "]]}, 
    {Hue[0.9749516849970554, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHElV9Cq+VipqLnM5mZbSrmoZH66GrZMD0pulmX4Sp3PmaGW
B20McxxcSaw1kTYfuAZmRkGwUTY11FS0IakgKooGKOjE4abmIb/D//K7/89k
liTnENzc3M7td2g/Xrlram9vTfX6KZkp73ineswjiA0edlilNVYn2VfR5cLR
rPsbNog/HEZuNtrQ3Qv+7hDWLOBf5yplv7Kgh8jmllJ/mcF58QmXVbSCzmF9
NsfEmGCR8VZ/KdmMfuJuX7qCOAebGTn0aNIiOquoKkqpmAZP4Z/y2+Z5dJ/3
wzvOPC0kqNdlIr4R/U2UMK7gpgYoA3mPBG0z6CbHlCN0WA2mnNil3XwdelVA
BoEx0AtaLbNpc3wCXaoLN2j0MlBldfH3fEbRj3ewnW2DEggw0PQ0cT/6g7GF
oS9iITQUJXaVne5Ft45EXCtW8sDuUhASgxXoR5WB/LTCF7BtZcv1zR/R1cJl
Nrm+GojfWowzV1rRG0jel4PYuWB3SKSl7EL0I+tmTrAkBQbnVtQXREI48E6n
fICenwmevM1KCrMbXRrb8rJTxQLaVSc3ofgr+nyN0hF8ox4EHmHxLSXf0YkU
UbeD2gTN1vEBP1EfOmUknT/NEQDZj18wOzqCHvGwoXaBLgEpNa1GJZ5A/1kc
+WG9VgaBXkGnXEk6dOOdFN8IQS9kEBuvC6Zm0EOzrCeZ5weh79bkhi/XiK6P
CJmoiNRAtChOwtPNo98r95oFuhbax9xbSbsm9Gp/+m+GYBr2OP7ZPX5m9Ir+
7aZkqRGoqdxj3swVdOrQ2cTGcBNkrzHsuXUW9LHCRJWhywxlLZXKUo0NXbs1
+ePTuAWek+QJsuVV9C3+s/Yekw3+/4V+8Nc/KDjj0w==
      "]]}, 
    {Hue[0.21101966249684523`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHElV9Cq+VipqLnM5mZbSrmoZH66GrZMD0pulmX4Sp3PmaGW
B20McxxcSaw1kTYfuAZmRkGwUTY11FS0IakgKooGKOjE4abmIb/D//K7/89k
liTnENzc3M7td2g/Xrlram9vTfX6KZkp73gHj3kEscHDDqu0xuok+yq6XDia
dX/DBvGHw8jNRhu6e8HfHcKaBfzrXKXsVxb0ENncUuovMzgvPuGyilbQOazP
5pgYEywy3uovJZvRT9ztS1cQ52AzI4ceTVpEZxVVRSkV0+Ap/FN+2zyP7vN+
eMeZp4UE9bpMxDeiv4kSxhXc1ABlIO+RoG0G3eSYcoQOq8GUE7u0m69DrwrI
IDAGekGrZTZtjk+gS3XhBo1eBqqsLv6ezyj68Q62s21QAgEGmp4m7kd/MLYw
9EUshIaixK6y073o1pGIa8VKHthdCkJisAL9qDKQn1b4AratbLm++SO6WrjM
JtdXA/Fbi3HmSit6A8n7chA7F+wOibSUXYh+ZN3MCZakwODcivqCSAgH3umU
D9DzM8GTt1lJYXajS2NbXnaqWEC76uQmFH9Fn69ROoJv1IPAIyy+peQ7OpEi
6nZQm6DZOj7gJ+pDp4yk86c5AiD78QtmR0fQIx421C7QJSClptWoxBPoP4sj
P6zXyiDQK+iUK0mHbryT4hsh6IUMYuN1wdQMemiW9STz/CD03Zrc8OUa0fUR
IRMVkRqIFsVJeLp59HvlXrNA10L7mHsradeEXu1P/80QTMMexz+7x8+MXtG/
3ZQsNQI1lXvMm7mCTh06m9gYboLsNYY9t86CPlaYqDJ0maGspVJZqrGha7cm
f3wat8BzkjxBtryKvsV/1t5jssH/v9AP/voHR9jRUw==
      "]]}, 
    {Hue[0.44708763999663503`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tMEgAYBWBTK5yuqUleM62cD7VMq1lK/bmcNhHXzGHNB4eXShepkeYs
X0wqG6Ur8ZIDbHkhSx0Jm0qODI3MliIXL4NQxNAEhBputCX5kH8P5+V7O2c7
4TlF6fmuLi4uEZtx2wyb4ZhzOtcktIFOn4Y7gyB9ST6tOmQFC6muIs1qQacO
0apTEiyQvCMyiDNvRufevxtqJZkgsMpRzHy2ii5RMnYmBi/DekwZ6zZ9Bf1g
/XnCYZYBljJatMfSjeivyzIr+dQFsGfnJ8V7LKELOjilDZka8G6dZaQa9egU
g/WM3H0ayGM/+/mN8+gK+khVokEO0bJrV7ntGvRLeQduzbqNgyH/7PeNghl0
8+POaoJTCmo1rd4+pfjfa0isj6OIQZLb2+jcPYFe1K9wipUCCP5G0pK6P6JH
Pd+uqmG1wyM6pbck9AP6kxxGqm8FD6wOsStlnxidb5NrJxkN8NvEHNBy3qIr
xUN9Xdl1QHjPm9ecbEPvfOV5szakBqy2HlEx8zo6M6Ka6GVjwueFlbEj/FbY
8rW2UnUz8SF4s+3l0TQBOk+mNodk1QLp1DqLfGMQvX4q3FOoYwPXPTKZVzSM
3jWyKmXbOcAxTckC+KPoP45OKiX72yAooLFQN/EV3YVIHZ17KgBR7OVKSbcC
XXBxWJhlG4QQn717HGkz6BmfWpoXVVLIJtSd485p0JtUx5uIueMwmjL9y581
j+5fkOR34Ysc4vkJPewZPTr7zYmwIL0aOia3tXlsGNCF6YXEqDgNOB8E5vUF
GNF9F1kicFuAWCrLy5e2gq40iuiV5QbIW8uwXqlaRafX6HTjf4xQwit/Vyw3
ozP8Cg0vwkxwz2OA3L9s+b+ncJesNcYC//6FvvWvv1IO0mA=
      "]]}, 
    {Hue[0.6831556174964248, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01lMEgAcBnBTMmxWmqXi1WyZtWlNe1CT7W+HeeKDc7Yeykjt9CLmfaxo
1HLM3NKolYCNlLaQqXmgFkqB001UKCMTwwPRBCQLjB4kH/Lvw/fye/i+py/w
Sn5qtqODg0PQRpw2Uk+3TdrtK5IKH6q42V0Ol/s1F6WlZjCRa8tSzCb0WitT
dkhjgjjnYJ8GrRFdLeFlDKsNQGLYCpiPl9Ftu57GB3QtgjW8iFWcu4R+aSQn
wfmIDnRpzzUnUvXoOlH5e8bqDFgyss9Fu+jQD5MW1hIfacCN95WerJ9Fl6ZH
DTDJakgaWu0WsLXo8VrX7TZfFYQNXr/GeTWFHpPwtuQ0fQTms2MW1m+o0T/f
jnxIy5PDxAS1zqJUoXd9MBU6e/aDJFPEtnuMbvUwLMSa9k7wnSZryEI5ujI0
QR+aKILqXIqIFiBF/3G2MlJY9hrMtl5HyoFe9CJWyF3WGh/+GphiTUMbOot2
MKfU2AjEfq52KpKP7lolNw0SeGD+2dJRwMxB33Ey4biCzoHhmaWhYwIebHqf
bWDP/vtccKu3lIRRW9GrY8WUW3WNQI6yspLyetANFZ+8qoR84BCC47j5A+h/
CqN+53ULoMGgHPQWyNBZHz30mVoh+Hizb34fVaCPTxcr3tA6oSPiQqVEqEJn
+M+9Cx2TgJ+7v6ctRY1OFoZwkmPlkEGsPcOZnEK/0yN7UaUcAVnil19eLO3W
7reaJwSCCqIFp1rq1bPoOw9UvVSQ1NA0to3vsj6PflTWxp6jacD+gJTV7q1H
T/ZrtZK4MxCRznLdS11C3x042XJ+nw6yVtLMVxnL6M1Sp8CmZ4tA45b0FYwb
0YWUmqCKPgPccxEndS+a0BXGcGK5wgT//4W++a9/vkfA0Q==
      "]]}, 
    {Hue[0.9192235949962146, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901lIEwAYwPHNLFxFbKU2tbQYIhSYR2Hl8Ku8wmPhkNRCxGOZ0nIqqAWl
eDDJiYqJRHmUOpbh1DyYV0uXxwbpUvFq5hzOtdxlNmMWLh/ye/i//N7/Z5My
mCwbAoHgvteBvWqyLUtWq1EcGmtIK1ZPQsSG6BjfYAIDvfIxw2RAT42ge+jG
jBB6yMO5TqlHb3eIGa4a0YNToYVTUr2BLk9mtKobtbDtk8PLZWvRL9Fo7z5T
1kEd/XLZl6lBHzzocyZMoQJzAivEn6RGn/3EHFhMWgFy42J2hEaFzrHhF9uT
lyBc+lMkqFWib5c9muAaZ8F7/H5qfYsC/ZZ8zHqhXA5rrGvru2kL6D4yCrnC
QQZzc4nPzdMz6PYuE39pQxIQJ7fXWk9Moe94CRylxA/g8o2+TG8bQ38THENk
bvXCM3Zke6brCLpn1ObbqiPdYLIM2ES6DaCbf91kEGWdsKMr6Vuue49eoJFI
fgd1gN3HBqXicjN6/tfzhwNeC8G0KezhlDxAz90h+cbPtYFsVSv1FDTCvkfd
CWkqSBACucac553YiZ4fVdHNoXYA/co2L/xhP7qX/pXTTnMn1Nt6hDZkDKP3
pk1xs1q7oE43PU4VjKITquhEdkAvOFNr01emJtHLys8pNOIh6PGLeyJum0F3
XZexLlZI4BTltKOFsYD+wvaQe7pOCgl2lYH1Swp0Y+kUZXJQDqNh81sneUr0
q9w/cdXzs+AvuC6sWVChi2Q/WPH6ReDLic2k3TX0eY7b3azgFbBynVK6qBp0
WlZq7GqpCvxu844eT9Siq3MqyW67akgxRpvuFW6gP6X2F7KLtJDZkDfI+aJH
L24K4gby9VBE6gsXfTegtwRq026IjPD/L/T9v/4Bfj/kTg==
      "]]}, 
    {Hue[0.15529157249600445`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYBXAzNRcVXkqn3fASQplkZitcfIWhlGkhUg7COS9l5uZEUlFE
M0yTCVKuUeJmorEIXReMzWlbirfEmVtsa236z6ZrXua0NGfm8iE/H87L7+lw
4PilZMenOzo4OBxaz9b1cHNtOrt9TkYaPFP9Sq+C+4GGy6VV82Ch1hTGWS3o
LHpXwpUSK0S7BPnWE7PoAZWCyKgVC/iU2djlj6bRz6sfBz/VTsHS8TxOPtOM
/oJ7+2jZBRNMJNQZwuJN6McKChvXPI2wSE+PiiBNoI9+nqf7mQlwa/iSe8k0
jk4znXxQXKKHmIEFsZBHoI+xtw9ZkzQQ2pdxk9+sR6e0XXN6sqoEY/rZybVb
WnSn2jAfxi8FqNWM2kWlCn3SaHZequsHWaqIZ/ccRo8bv1P7bqwb9o5SDdSW
XvTGrCFSUaIcqpixopwDXegjxhMd4QOdYLVJHWMPStF5lZzk5goprMyUSwz1
b9D9i7ap6xzbwVUuIPSnmtBT7tK8I3dJwDrf2sYuz9rcp6b/pVIoho/fzAMh
wgbY8EH5ZGrarBjcuIsFoYzX6H4Tyt7daglQTy9xYljt6MpS4gg/RQp8p6Bo
QfYH9MS/Hk+9MjqhfkbZRxb2oDcGlnq6r8rAl8zLHBtWoPPcKL973ndDG4VW
LGtRoc+JZlgurH7Y577fyxanRc9UBIVQBxVAd62J5Ov06BpmshclWAU9FzU/
vTkEekAWJ2k1SgMRwnOtXO04+jNRUWbvdT08/7SlibRmRCcOa0VMHQH2Cp+0
t2TTZk+jSMeVfwfKVc4OD4YZnWbWPlSFmyBtLsF6o2wafaHK0dlfOgU5goIO
9sgs+vKeatKywQL3SJIY8Q8L+s6v5Lw/+Vb4/y/0jX/9A9o54r8=
      "]]}, 
    {Hue[0.39135954999579425`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIEwAUgGFdmo6kzPJaKStLKKhMY9YmR40meBk0TFIsM12peZmMTEQh
zBWshUqO0WVTwWwRTlMEdcVs88LENttyrrHlHC4nzrkpGrNw+ZDn4X/53n/S
nUoGk+Dl5XVypz078dlug8ezKnOUNFpFND3sL0sjXctcAwe1uZbudKBvdp07
RUpxQeremAiheQX91b7E9XVwQniDm8V5sYyezaxSU27ZYfNCNe9h+RK6NJo2
fUJtA2vWa1McYxE9xe/61faXVtjIZ9IoRCu6s3CKoKiwQGD7D3bGogW9zteu
GuHOQbpybVAsMKMTmmRB8aUGiJ0ovid6a0TP1QakGf10sMBM+rVdokePttE1
bI4GdLqC1g2NFv0MKaulrk8FssIegeeQGr010hyVuKmEIz+pJmr3OLpNTOZ7
V44DtzyzpypSji6RNJaQ/oyC0y0lZEZJ0Vse9VJyDArYsnOGTMI+9Fl3efX3
UAX4j7SZjQmd6L//hoSKhXJwuiQDLE4Z+kAjfX61VA6T80vKs+J22PUPb25+
Tnguh0D+Rk1swUd0nUTraPZWAPXSJi+9Yhi9py63Y2ZCASKfmNS2yi/oa8Ns
fcvkKAjtmokw8Ri6gH46uOnwOESECUrn1Cp0Y0ZHQ8aMEgbIOfWybi16vD17
i9eogqMHj4W46Xr0G77KvHmWBvL9m6+IDEb0rWfBebVkHYylza6H8szoNG5y
JzAMQBEnS/h6C/ro1/O9xOI56Jr27iRuL6C7hvoYB25bwPM0vKg/bBH9PZtb
T6ZZgZzNCwgqWEK/X9N/8YHUBkWrWc67DcvoPhGmd4okO1S11XxifVtB52ji
dFOxTnhMHEoftDnQ+cefpE5ddsH/v9B3//oHGi7tug==
      "]]}, 
    {Hue[0.6274275274955841, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw900tMEgAYwHFU1nSridYQtNKa5avlzAM5aZ/VllPS0jmdS3Mo+CifuaVr
65BGbo0mhg6bE42Zo5polE2zZtbUAB+kjdSAABEEFRCnjaaSh/wO/8vv/j+R
X5HO9iQQCKf28tqrpdq14Hbbh394Ky486VODaYRQT6FtgI3Ou5fqsKFPbS81
LEc5IfFAWGC7bg39DnMjYyhiHah1rkoOfwWd6Kd9Ko+xwda5u9yaMgt6iVLK
l4qssJTRpolNN6M7pCPe0WVm2MxjX4n3WUIX+saNpV0yAqlzvvqq2YBeKjFE
2lv1wJA5B8QCHfpNU+uzND8txIwXFwlfqNHfR5qULsk8GNkJpt2SOfQHEJ7d
dEYFKhWzeXNmFt1ZVD99gzALwwW9AvfhafTuQhmZLlJCkJauofeMoT/Psoro
nybhcVlKb9XxL+hFubSanKQJcLiGPFOCh9CTowuSmsMV8HeVM6hpl6LzxCEP
fQvl4P25Q6c+34Xe91oWurgiA8e6pL+SU4peHuthjxyUgVxvkZ0Vd8K+sxgk
RfUvGZBaNmtjmG/QG3MfbSuuyYEet8VllH9AL4461EsiK0BIDEvsqBhBf1Vy
2jRBmYD21ZlxingU/fqO+qQ8fxICKYJbv6en0BU7Ed++Nimhn5Z9f7hnFr1m
kkhkWWbgqN8xsit1Dv1djmhxN0AFed68y8IFNXrmos0g0M7DaPLPjQCuDl2k
D4zKdWkgXnxR0jJnQOf5c9r4DXroVnp0+ewa0TvZwWRhrBHcDVTWW4oZPSv4
T8IY1Qy0TO5Bf6YFnRoa4vJqtALLnuEorFtBl44m3U4/YoOqjtqPld/X0HPC
rbTmoHWo9xlkDCzb0Etr6S/5oU74/xf6/l//AO8P3Dc=
      "]]}, 
    {Hue[0.8634955049953739, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF020wEwAcx/FF13EVqrvapLndYXFyyXVoc39yx2XokrpLHXmWxsh57EpH
u44buUrTOeZKbuk8VTrM3dQ1TgrbnLHb2DBrmZnN1LywvMjfi9+bz7vfiy8l
hRWXbkcgEDx3Zr+z+gKr3GZbEyqlDmMhNSo46EUonI7cAAO9rizWaEA339aU
ttHNEHmA6tqkWkXnikwRpRdMQKqw5rGfr6Dzmbm0N95G2DxXxCnO0aF/augp
9vumB018ozIgTotOvz+kb6/VgSUpPYLmqEH/LQts/5m6DC4tswXR2gX0viK1
092BRWCMmvr4XBV6GW2s28NHDf4jWZnNbxXosYLFhvV1JSylhy5v35lBJ056
UqpYcpieTn5hkUjRq/4+4l4Nl4EwtYtrOzaBHvJneC5AMwUn5+hKescwetbg
PdJLihSqc2K68slf0aPglX1bpgSMVoFdjLtg7+/pLn2YTAxbena/sukD+vzS
4WpboRgchngqRVArOisjJUEXJAbjemdvHpu598vnIdS4i+G7Wjfqx2+BXb/0
md3Rc1YMLvWWEv/kHvQkYeiwMlsM9OBNDiN3AJ3h9didMC6G5v3USB7rCzpT
5EJLvCyBJr1khMgXoYMzc/bWhgRcidzs+Ylx9JRaap1gagp6A288EHZI0Z0S
7X7c9JWB25FTx62xM+jRq7z3mwlySHKoC2+WK9ANBfr2p5Q5EEXJzCc4KnS3
YE6pD1kNNH5YZ/3MArpCXlnu/G4R2ib3tTpuL6GvnCFvka4tg+0JKe0jUYv+
WmvxuEjTQeB1zqGjyTr0ND1ZWd6th7S1eGNGxQq6LaKVqnU1Qj6vZDBPvIpe
wdF5x/maoNKxn9H3y4D+TEFrvHLeDP/7Qt/t6x/LzOPd
      "]]}, 
    {Hue[0.09956348249516367, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02lI03EYB/BluRxZmUZO8yhTzEibGqm4eOwgNVNJpEMDm0etyLyiRmSJ
V2QbCipLZG6E1irdKtO0GcuhqRHpNv1vk9kOj21M3aZOWcSWL/Lni++bDw98
4YHvweyCtDwnHA4XtJ6t62kssU06HCZh6qGja8dFWih7kbBW88QKi+S6hynm
ReRUaejfjHsrEI8P9mapF5C7RseOhBYvg1e5rbCq3ojcR9X99TLFAqsR9+kP
8g3IWaWnHzEjTTCb3jwVmaZDHnj9osXDbgRrVt65WMIscknfMO+EVA9uHEXJ
BZ0WOSez2GM8ZA6SRpZ6uEw18pwD17oYTdMQPkS92dKmRF4rchJTqBqYyYub
s9+SIzc0xR/JTFEBhlEarBIp8gzRS05QgBKEOXymw2MU+a5+vc55QAH7f5On
yB3fkUdXNwRQcXKoyU/mF/mJNv9AELjSzsnAbBM4JfsLkO9cGrN6dGPwZ76q
d4r1Efl73rF6cToGLt/YamV0K/I4UmWOwBMDs4XXVVh1B7n/Cjshyz4BPzSG
kTAuBzbcu+x1d8VuDNwarbRwygfkpGfZo/xEDMgxq/Sku1+Q+5jbDkvaMWjZ
FhzPLuhHfrI24iwpTAaseckQkTuI3E3B0IZJZeBNZN5Wjf5CHnklsYnRqYCu
qKulwg4pclVw4EToDiX47PHdZ0uRb/Z+7v80HKOCLJe6My2Tys17v+2TpDIN
DJ6XLXvS1cid/Xwf1zOmIZZ7itco1yIfEO6tpBHn4NXYllaCfQb5uDvfXj6g
B8dTr9xOog7585/VIe/eGiHqEt3VnWJAnizma/T+Jsg1pZtvlBuRl/TOv2lO
s0ARm9ZXKF5AvmoPb8fnLkMFoTepR7+IHJ9a6YvPX4H/+0K+sa9/rfLGJw==

      "]]}, 
    {Hue[0.3356314599949535, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw901tIkwEYxvFpIY6ktNScWSZmiol4AI+r1w5oeQpErKyQmcsDmmczQyWH
RTZKVsML0QlZzEBliuQJp0sTLbblyOb2fW7N6Rxzc6mbrQuXF/VePDe/ywf+
frmlGUxHCoUSsL8D++NW2hR2+6ZQVHHdP2RXC5doCmrUshVM9Na6dLMJPThT
eNhJZoEkp0DvDrURXTz8VSmX7gCtyVbW/NqArsq5KnWc3AJrRA37YYkenSMc
kNb8NsNqZjsZmaFD/8Co8iwaM4Elh5kYT11FvzlhkLFbDeDatVSZqtOgn73L
jQnWrUPK3NYwv02NrvdrVKxkrUH4bEF+5zsC3WM6wPoxQAtaZsLaXqEcvU6c
OMrK08DiIuONZUGGPphbZSkQq0F4r7/NfkyC7j0dGjFwQQUnlukkvfcz+pXn
imL3xyS0lKT1l58SoQe5ONw4M0WA2TbmmOY7hk6zeCZMhhHwZ6N5hOwYQB8X
VJP1EiU4T/LUREw3emrkfG8+RwnmX31DZc3F6Od2VzmHHilh/qd+LpTfBfgP
y0gJa1GCK9dSG84QoNPZfNqdaSXQY63slAej6EHbhoqGQAI6DwYm8Uqn0K9x
al8SfAI6NhZmvfgz6D6ELESZRoK3V1uRSiJGh9uvVmJDVTAUfate2CtDr/YY
OmIUqcHH7aSnLV2O7k6yGuOyNZDj3Hq5U0GgM08H25+e18JM8o/t42w1+gvf
hLf65DWI51/s48o16MlT0T3i7+vwXurQTd3Tome7L/gvPTGA/Rktb9BLh+4m
7kl3KDBBdBbb5ShDj15UWMjimcyQt5lpvt9kQI+KaxhPFGxBOa92vOybEX2k
8dOEQLQDLOpIyvC6CX1e4hEm+GKBf32h/+/rLxu51Do=
      "]]}, 
    {Hue[0.5716994374947433, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmFbEUlSWpGbWZZSFkrO4wxXn2I50TRaNozA5WGZpulaqCw1
MUSKCUKJlTrFLEd4QMvSDKaSh1mpaZ7GbP9tzjk25zxsugSXF/l18d48ly/8
TiRkMDkkGxubk1vbubUynkVqtS6K/XvuX3Hz1ECM6EPDZOI6GOil/GijAZ2u
etlSH7sGjN3uTlXEAjrBqM/ns8xAKbRkFj3ToRsNrry6S6tg9skSZKdr0Wuj
S3jk/GVQx1TM+DI16HPiQ00slRFMbE5YkK0anS+roYxUGMC+Zpp3WaNED38Y
kZ++rINIyXK7qJxA3xvsd9w9QQve/XeShW9k6DSlNGiDqoFZTvDcZsoUeqV+
w9qXoYaJifjnptEx9PGCM0EN0yoQJzaXWw8Oo3sVfI+rfKeEI7/pM/TGPvSv
Ca65PwYU8DQ9qpl7rAc9oM6fO35XAUZLJynKpRO9+UUX57WbAv7oizpmqlrR
bdYy/EgrBOzpqiZkgXXoA7RXztm/CDAuNbVlFqWhf6Kc3i/pJ2BQoZWcFdXA
trfJPcJujhBgX2bK8Y5vQQ9OesLkLhBAP2cWRN77jP7R0eMRyUUBwl3ujOqM
bnRG8pKgha2AKv1oP1nUi061c5FRWxXgRC5PlQ8PoXd/y6XdqlVCG+1Gnrhx
DL3cN/U6dUgFzg5HD1uip9ADNgJ78zhqYO8pDRVKZejFF+QXB8M10BsxueIo
INAbz5vmPVlaCBKFNJVNKdHT5HqDg0IHb0d21NluzqLn+hSnOJcYwFpMSXpP
1qA7ZXlF1HYYgcYS2B2I16Lz10NPhTxYhqTFGOPtQt3/HySx7PmAVeBW53zJ
/LmAPrhPGH8t3AyPbTsi2+cN6NI44yjz6hr86wt9u6+/DgK6UQ==
      "]]}, 
    {Hue[0.8077674149945295, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcB/BFnk4vsLtiqeguu6jurAdidz9xzV1DnZRzl9OKSRcbDsvD
nZTqtLtUlrtq83Bid3ms1FiFriVykd0xbtj0n2FjC8NWrL3I78X3xffz4vvu
63uFE5tiRyKRDthib4sg2zxutRq6xO5PCyisOXh7WjHFcbPAIr08P8a4iP6i
+CMBDmaIdKRShKoF9DBNf5uH/Tp4lZi5pU906Of4jHr6kglWabn8vPQ5dDt9
a32r/wpo4p5PHI3VojMSg9WLD5fAlJTCCHXRoAtdV6DYYgS36rHsKO00uvO7
R4SQaQBm35JEXKlC/55fcDdeqofA3mupopdK9DXv/bUBZfNApITNbKYp0C9R
I9hr32ZhZIRVYRqWowuk3BolTQtdV1sqreRBdEMnWTZgPwO7J+kT9Kav6HeE
adMbvhooS49uydz7GX11snDUQUaA0Sy1i94nRefInQbkhQRY9KUdE8LX6N2B
75/Roghw7q5SKYPr0O9nqUsaDtt2fje3c0tvoOfFcXLMPgT0q+f6joirYctz
KNmNDw4S4CYw8QJZbeikqiFJDYMA+slVPjOjEz03NktD4xEg2k6NrOL0oDvV
Bi+rJQQI9cO9nmIZupHBDme5aoDiWXl9avAH+i3HL8LyPxpoD0oo6mqSo8dn
tEkvU7Xg7b5npzlGgb56vDig+dMsJDmXR4jGleiRN/1eWWrmQXZmdHkXX4We
zqOfTWzRQ6j4VLNAMY1+iHLsQniIAeqHttW5bBLoohPaVL9lI1jveSW/8dSi
55PPF4WwlyDoIn+Hh61v+bggNWHMZwWSDXFGdokOvYFd+Lj4lwkyq3gfuD8X
0P0beyp0K2tw26WDKZldRA/5S3bUbazD/3+hb/3rH1iixWo=
      "]]}, 
    {Hue[0.04383539249432289, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF01tI0wEYxmHTEBeBusTmKRELRUNYQstcfGqR6aaImuWF2dSVM82JHVRQ
otBQFgzKdhGbVprTUHE2PAWraOoy5hGbojbN9d90m9vclM3Y8iK/Lt6b5+q9
+YUVlGey3d3c3E7tz2N/zZX2RZdrSxbIGurwfbkJBeZLb52/9sBI59ekm4zo
qaejxEqVA5I9IwKFagO6kU+7/3reDgGP7dz655voTlo/Z0K+CztnHvAelunQ
+wQhJeC0gSb71XJsJoFu9Wpvqy20gi2ffTmepEFnDtz08FZawKd1oZJJrKFL
GqTSvGozMBSWQbFAje7H6Wo4TzUBdaz4tqh9CT373l5kCtUI6+yE306OCn24
Ltjgx9XD/DzrhW1mFt1CTo10LGyArLBX4Do2+f8P2cDY6NJB0Ap9md49ir5q
TeIEKbTQVJbWW3HiC3rW2Zwb4aVaMNlH3NNCR9C/R61mmsO14NDXDy0LJegh
jfywjG0CvD61qJfOtaFrFMMk+RwBJnOPlFtfij73/sof/3ECvq3qFDHiVjjw
mQpmbOcUAT7Ntioqqw89b6cTZAYC6HE7PMbdYfTZTkZpRqgWRIcjklvKP6MX
2X1rXPlaEOpnxihiOXqaCBR1Ei0EUgQlPyeV6Ct5wpM9b3QgpeXWyrpn0T/I
m5LqlBsQ7Bvib09XoWc9y5VMsPWQ78W/KFpcQm+bclEpKUaQp/7YPs5To2uI
I3cuRJsgXpzY06xaQ/866Gns5Zrh3dShNpJzHZ25lhUtHbWA62lAUT+FQI+U
yRIUcVag5fCOklk69MbEueBiqw2KtrJNtx5voueQdVe9B3ahoqXqI3fagK66
FnO9ZtwOT0hDjEGtEV3X8UhRPe2Af32hH/T1FxPw04k=
      "]]}, 
    {Hue[0.27990336999410914`, 0.6, 0.6], Line3DBox[CompressedData["
1:eJw9020w0wEcwHFJnXW9MIpRcq7DdZU7VJJ1P86dladKSumOVpQ8ZNKDunN1
HKVbl7u4vSibSlk5Q9ohD2PlYchid1rLw1qY3f+/2Ywxw/IivxffN5/3X88r
WbEptjY2Nl7rbV6vLMessFpnRc/qRmX2/VpIql42FYyvgo5e8iBGr0NnTE9Y
zspWgLHVx61cqUUnIrS93kMWcM03swpfEOgrhtS20x1mMPnfZd/L1KALzsja
vi0twlTcy7GAWDX6nDhhyTbRBAtJKeHBlCn0E9nO8aW98+BQ8SsnSq1CrwtJ
p/zNMUKkZK6Jz1GiU/deVoj2z4FfT+p17rtRdJ/bATP7nAwwmRIyvXZDjp5g
Dut77q+HkRFm6cKwDJ0sEkrtpDoQXa3lWJ2k6OFFF4zzPC3sGqeP0Wu60X+n
Chy820l4mhldm71HjB79sXOLL5MEvbnFNtqjBb2bG7VsoZGwTBY2j5V/QndZ
tfQmzBBg38FTjh6tRJ847l0vlRCgNwiErMIM9EGPz2892wjo+6OR+PIrYMMH
phoVDV0EOJQt5Pox69Hj03aqJSoC6EEmduTNL+jf3RudLu0ggWvnw+BldaIn
ZuZ5bjtHQjk53EPjd6EzHKtZxVUkuNE4aRPSQXT5kP1AE0cLwsCLeaIaGbog
yKR58lUHu6nuzuYYOXrVI9WB9IN6SLIvCeMqRtFPvRFmUD0M0BXx0+jCVqKr
Xne0i73mIJgfKiiTq9DFt1pDD6cb4f2PTZWUtUn0k68oD491zoP1sWtyA02N
7nX/TmnMIRMEnmdvd2Rq0IssxSytbhGSZ+P01/IJ9NgB7ofSejNk83JbWUNa
9BHqZLVVbIECSnNk04wOfZp5xN3avwL//0Lf+Osf69gIAQ==
      "]]}, 
    {Hue[0.5159713474939025, 0.6, 0.6], Line3DBox[CompressedData["
1:eJxF02tI0wEQAPCZIhMNlmI+eiEhQpGgVipb3Sxo4lRKZmmUYmrmyDazVDIt
FMlylpXDDzZfaMzKiZWhzloaKopkTtA5pm5DXeam/81tuJlbQ/I8OI77cdyH
gwu4wUnI3EMikQId6exIfp5FbrevSUjbQcC77d4Oq7TqB/HEKvor0dz8uNEG
DNcgf4FSh37aS/uh2bAFfqUWbvnrFXRGb1bTmGoTzKH5vIKcZXSrn7SJ7m+F
RVbdbFiCBv1MZbKi5NEGmFIzL1DdFtFrIr2plD9moDTO5MVq1OibZLYuhW8C
5oihW1irRL+2kvKNGmeEkOFbWfWtCvQ8WuhY68l1WMikL9myZeit/edE7iwD
TE2l1Zikk+gDlTNMW4UeJOkdtXavcXSO+/vr+XoCDszRZmntQ+g9fxO1zx8T
8CwnriP38AD6XcH8kmcCAYRFvCfuiBhd5BOospxw3ENb3jMr+Ig+PfQwus2b
APL3BqUiogWdqnnZl+7q2KMXdXHLb6OHyWRdHo46qloeCRY2wo6Hiq9OJLoQ
QOGbCkPSOtELip9OKz0JoEWaecw7vbvzV+Q5bccJqHcJYjRw+tGtJV+NpFgC
BFrpsK9wEJ1zifWGfp8Af99a9vz4T/SIEOeYJB0BXeHJxZL2SfSji5Ykepke
Du47tN8SL0M3RKsomngDpJKrz9fLFeh17M8Xq+jrMBgzve7DU6IHsyXNZxlG
oAqjRHyZGn0vVRzYyTPB219OLW62BXR+Bjn9i9oM9id+GZ98NejcqHuFo4kb
EH6Z5+GZtowu/VHByqZYIWONRdwsXUGvrxFUUSY3IbehsI87oUMPkKtfFC1s
QZlbD7P79+ruPY+dcirS2eD/f6Hv/Nc/oVh2KQ==
      "]]}}},
  AutomaticImageSize->True,
  Axes->True,
  ImageSize->{364.12597856404506`, 390.3315418632271},
  Method->{},
  PlotRange->{{-1., 1.}, {-0.9999999310344828, 0.9999999310344828}, {0., 
   1.9999998620689703`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{2.5469209736392413`, 1.3678292696739274`, 1.7584756589329944`},
  ViewVertical->{-0.07051520396973057, 0.4471147161945623, 
   0.8916928696678927}]], "Output",
 CellChangeTimes->{3.5802819528629017`*^9, 3.5802819856547775`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["plotpatch2d", "Subsection",
 CellChangeTimes->{3.5802822252874837`*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"plotpatch2d", "[", 
    RowBox[{"expr_", ",", 
     RowBox[{"{", 
      RowBox[{"x_", ",", "xmin_", ",", "xmax_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y_", ",", "ymin_", ",", "ymax_"}], "}"}], ",", "\n", 
     "        ", 
     RowBox[{"PlotPoints", "->", 
      RowBox[{"{", 
       RowBox[{"xpoints_", ",", "ypoints_"}], "}"}]}], ",", "opts___"}], 
    "]"}], ":=", "\n", "        ", 
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"ParametricPlot", "[", 
      RowBox[{
       RowBox[{"Evaluate", "[", "\n", "        ", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"ComplexExpand", "[", "\n", "        ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Re", "[", "expr", "]"}], ",", 
             RowBox[{"Im", "[", "expr", "]"}]}], "}"}], "]"}], ",", "\n", 
          "        ", 
          RowBox[{"{", 
           RowBox[{"y", ",", "ymin", ",", "ymax", ",", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"ymax", " ", "-", " ", "ymin"}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"ypoints", " ", "-", " ", "1"}], ")"}]}]}], "}"}]}], 
         "]"}], "]"}], ",", "\n", "        ", 
       RowBox[{"{", 
        RowBox[{"x", ",", "xmin", ",", "xmax"}], "}"}], ",", 
       RowBox[{"PlotPoints", "->", "xpoints"}], ",", "\n", "        ", 
       RowBox[{"DisplayFunction", "->", "Identity"}]}], "]"}], ",", "\n", 
     "        ", 
     RowBox[{"ParametricPlot", "[", 
      RowBox[{
       RowBox[{"Evaluate", "[", "\n", "        ", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"ComplexExpand", "[", "\n", "        ", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Re", "[", "expr", "]"}], ",", 
             RowBox[{"Im", "[", "expr", "]"}]}], "}"}], "]"}], ",", "\n", 
          "        ", 
          RowBox[{"{", 
           RowBox[{"x", ",", "xmin", ",", "xmax", ",", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"xmax", " ", "-", " ", "xmin"}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{"xpoints", " ", "-", "1"}], ")"}]}]}], "}"}]}], "]"}], 
        "]"}], ",", "\n", "        ", 
       RowBox[{"{", 
        RowBox[{"y", ",", "ymin", ",", "ymax"}], "}"}], ",", 
       RowBox[{"PlotPoints", "->", "ypoints"}], ",", "\n", "        ", 
       RowBox[{"DisplayFunction", "->", "Identity"}]}], "]"}], ",", "\n", 
     "        ", 
     RowBox[{"AspectRatio", "->", "Automatic"}], ",", "\n", "        ", 
     RowBox[{"DisplayFunction", "->", "$DisplayFunction"}], ",", "opts"}], 
    "]"}]}]}]], "Input",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"plotpatch2d", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Sin", "[", 
       RowBox[{
        RowBox[{"u", "^", "2"}], "+", 
        RowBox[{"v", "^", "2"}]}], "]"}], ",", 
      RowBox[{"Cos", "[", 
       RowBox[{
        RowBox[{"u", "^", "2"}], "-", 
        RowBox[{"v", "^", "2"}]}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"u", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", "0", ",", 
      RowBox[{"2", "*", "Pi"}]}], "}"}], ",", "\n", "        ", 
    RowBox[{"PlotPoints", "->", 
     RowBox[{"{", 
      RowBox[{"30", ",", "30"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.58028237453802*^9, 3.5802824020265923`*^9}, {
  3.580282445291067*^9, 3.580282463577113*^9}, {3.580282494746896*^9, 
  3.580282531449995*^9}, {3.5802825904183683`*^9, 3.580282626538434*^9}, {
  3.580282667939802*^9, 3.58028269903458*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["polygonholes", "Subsection",
 CellChangeTimes->{{3.58035956675723*^9, 3.5803595826068573`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"polygonholes", "[", "fac_", "]"}], "[", 
    RowBox[{"surface_", ",", 
     RowBox[{"{", 
      RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"v_", ",", "v0_", ",", "v1_"}], "}"}], ",", "opts___"}], "]"}], 
   ":=", "\n", "        ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"plottmp", ",", "grtmp"}], "}"}], ",", "\n", "                ", 
     RowBox[{
      RowBox[{"plottmp", " ", "=", " ", 
       RowBox[{"ParametricPlot3D", "[", "\n", "                ", 
        RowBox[{"surface", ",", 
         RowBox[{"{", 
          RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", "v0", ",", "v1"}], "}"}], ",", "\n", 
         "                ", 
         RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
        "]"}]}], ";", "\n", "                ", 
      RowBox[{"grtmp", " ", "=", " ", 
       RowBox[{"plottmp", " ", "/.", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Polygon", "[", "pts_", "]"}], ":>", " ", "\n", 
          "                ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"cntr", "=", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"Plus", "@@", "pts"}], ")"}], "/", 
              RowBox[{"Length", "[", "pts", "]"}]}]}], ";", "\n", 
            "                ", 
            RowBox[{"sp", "=", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"fac", "*", 
                  RowBox[{"(", 
                   RowBox[{"#", " ", "-", " ", "cntr"}], ")"}]}], " ", "+", 
                 " ", "cntr"}], ")"}], "&"}], "/@", "pts"}]}], ";", "\n", 
            "                ", 
            RowBox[{"Polygon", "/@", 
             RowBox[{"Transpose", "[", 
              RowBox[{"{", 
               RowBox[{"pts", ",", 
                RowBox[{"RotateLeft", "[", "pts", "]"}], ",", "\n", 
                "                ", 
                RowBox[{"RotateLeft", "[", "sp", "]"}], ",", "sp"}], "}"}], 
              "]"}]}]}], ")"}]}], ")"}]}]}], ";", "\n", "        ", 
      RowBox[{"Show", "[", 
       RowBox[{"grtmp", ",", 
        RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
    "]"}]}], "\n"}], "\n", 
 RowBox[{"    ", 
  RowBox[{
   RowBox[{
    RowBox[{"polygonholes", "[", "fac_", "]"}], "[", "graphic_", "]"}], ":=", 
   RowBox[{"Show", "[", 
    RowBox[{"graphic", " ", "/.", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Polygon", "[", "pts_", "]"}], ":>", "\n", "        ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"cntr", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Plus", "@@", "pts"}], ")"}], "/", 
           RowBox[{"Length", "[", "pts", "]"}]}]}], ";", "\n", 
         "                ", 
         RowBox[{"sp", "=", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"fac", "*", 
               RowBox[{"(", 
                RowBox[{"#", " ", "-", " ", "cntr"}], ")"}]}], " ", "+", " ", 
              "cntr"}], ")"}], "&"}], "/@", "pts"}]}], ";", "\n", 
         "                ", 
         RowBox[{"Polygon", "/@", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{"pts", ",", 
             RowBox[{"RotateLeft", "[", "pts", "]"}], ",", "\n", 
             "                ", 
             RowBox[{"RotateLeft", "[", "sp", "]"}], ",", "sp"}], "}"}], 
           "]"}]}]}], ")"}]}], ")"}]}], "]"}]}]}]}], "Input",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"polygonholes", "[", ".5", "]"}], "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", 
     RowBox[{
      RowBox[{"Cos", "[", "x", "]"}], "*", 
      RowBox[{"Sin", "[", "y", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.580359932375072*^9, 3.5803600182532225`*^9}}],

Cell["\<\
The result is an error message only; this part of the definition has been \
commented out. For 3D- graphics it works\
\>", "Text",
 CellChangeTimes->{{3.5803601172197967`*^9, 3.5803601629122767`*^9}, {
  3.5803602267787895`*^9, 3.580360242612817*^9}, {3.580360469140815*^9, 
  3.580360562023378*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"polygonholes", "[", ".5", "]"}], "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.5803603690510387`*^9, 3.580360390532277*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[" wireframe", "Subsection",
 CellChangeTimes->{{3.580361943755805*^9, 3.580361978231866*^9}, 
   3.580362068743224*^9, 3.5803636454535933`*^9, 3.5803647564407444`*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"wireframe", "[", 
     RowBox[{"surface_", ",", 
      RowBox[{"{", 
       RowBox[{"u_", ",", "u0_", ",", "u1_"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"v_", ",", "v0_", ",", "v1_"}], "}"}], ",", "opts___"}], "]"}],
     ":=", "\n", "        ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"plottmp", ",", "grtmp"}], "}"}], ",", "\n", 
      "                ", 
      RowBox[{
       RowBox[{"plottmp", " ", "=", " ", 
        RowBox[{"ParametricPlot3D", "[", "\n", "                ", 
         RowBox[{"surface", ",", 
          RowBox[{"{", 
           RowBox[{"u", ",", "u0", ",", "u1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", "v0", ",", "v1"}], "}"}], ",", "\n", 
          "                ", 
          RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], 
         "]"}]}], ";", "\n", "                ", 
       RowBox[{"grtmp", " ", "=", " ", 
        RowBox[{"plottmp", " ", "/.", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Polygon", "[", "pts_", "]"}], ":>", " ", "\n", 
           "                ", 
           RowBox[{"Line", "[", 
            RowBox[{"Append", "[", 
             RowBox[{"pts", ",", 
              RowBox[{"First", "[", "pts", "]"}]}], "]"}], "]"}]}], ")"}]}]}],
        ";", "\n", "        ", 
       RowBox[{"Show", "[", 
        RowBox[{"grtmp", ",", 
         RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
     "]"}]}], "\n", "\n", "    ", 
   RowBox[{
    RowBox[{"wireframe", "[", "graphic_", "]"}], ":=", 
    RowBox[{"Show", "[", 
     RowBox[{"graphic", " ", "/.", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Polygon", "[", "pts_", "]"}], ":>", " ", "\n", 
        "                ", 
        RowBox[{"Line", "[", 
         RowBox[{"Append", "[", 
          RowBox[{"pts", ",", 
           RowBox[{"First", "[", "pts", "]"}]}], "]"}], "]"}]}], ")"}]}], 
     "]"}]}], "\n"}]}]], "Input",
 Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wireframe", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u", ",", "v", ",", 
     RowBox[{
      RowBox[{"Sin", "[", "u", "]"}], "*", 
      RowBox[{"Cos", "[", "v", "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "0", ",", 
     RowBox[{"2", "*", "Pi"}]}], "}"}], ",", " ", 
   RowBox[{"PlotPoints", " ", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"10", ",", "10"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5803650324832296`*^9, 3.5803651436958246`*^9}, {
  3.5803651762530823`*^9, 3.5803652152219505`*^9}}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nXecz8fzx/VO1EP03vkmerv3h6jRewRBBNF7SyQRRCcnRIl2RO9dtPvo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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Line3DBox[CompressedData["
1:eJxNmwccT9X7x+8694oG2kuplIRIw0pStEghhCQ0SMNoGtGgJDRIpUJaGpSS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         "]], Line3DBox[CompressedData["
1:eJxlm3ccl9P7/+953hJCMivtVESEhtBSZFbKroSEVPZsK8nMqJRRUSRl5YOs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         "]], Line3DBox[CompressedData["
1:eJxt1334ztUdB/Db/bvvm/zkcUmEidLDXF2XRmySHqi2Lg0hLZqxIVItC4Ut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         "]], 
        Line3DBox[{{1711, 1172, 1171, 1710, 706, 708}, {1377, 1698, 664, 604, 
         603, 850}, {1202, 1247, 666, 730, 1723, 1201}, {1166, 1232, 643, 699,
          1707, 1165}, {1351, 1350, 814, 576, 577, 815}, {1421, 1420, 895, 
         613, 614, 896}, {1411, 1410, 882, 608, 609, 883}, {1727, 1208, 1207, 
         1726, 737, 739}, {1305, 1689, 638, 568, 567, 766}, {1711, 1172, 1171,
          1710, 706, 708}}]}]}, {}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{551, 756, 925, 1898, 758, 760, 683, 784, 1838, 685, 565, 698,
        1826, 796, 1845, 700, 1059, 1060, 1061, 1852, 1024, 813, 1780, 936, 
       816, 837, 1863, 818, 940, 1906, 838, 857, 1872, 840, 860, 729, 1828, 
       870, 1878, 731, 1054, 1055, 1943, 1056, 1035, 1936, 744, 1038, 1129, 
       1130, 1894, 1072, 909, 910}], 
      Line3DBox[{334, 333, 452, 1618, 450, 928, 1900, 764, 786, 1839, 766, 
       567, 797, 799, 1775, 576, 814, 1853, 935, 458, 1626, 459, 345, 427, 
       1605, 425, 530, 1669, 466, 859, 1874, 843, 1109, 1958, 1107, 1108, 608,
        882, 1884, 1037, 884, 1791, 944, 351, 1612, 438, 435, 439}], 
      Line3DBox[{554, 761, 927, 1899, 763, 1015, 1118, 1961, 686, 1043, 688, 
       1773, 788, 1841, 701, 801, 702, 1776, 803, 712, 820, 713, 1781, 937, 
       822, 842, 1865, 824, 1050, 1940, 1051, 720, 1150, 722, 1786, 1147, 
       1030, 1124, 1964, 732, 1123, 734, 1788, 872, 1880, 745, 886, 746, 1792,
        945, 889, 1888, 912, 891, 913}], 
      Line3DBox[{556, 768, 634, 1833, 557, 969, 1799, 689, 789, 952, 1913, 
       953, 570, 975, 1805, 703, 804, 957, 1916, 958, 579, 1808, 714, 1048, 
       651, 1782, 587, 656, 594, 1811, 723, 861, 959, 1917, 960, 601, 985, 
       1816, 735, 873, 964, 1920, 965, 611, 994, 1821, 747, 892, 966, 1921, 
       967, 621, 1824, 894, 1075, 679, 628}], 
      Line3DBox[{558, 769, 1835, 771, 773, 690, 1825, 790, 971, 1842, 970, 
       793, 976, 1922, 977, 805, 1848, 705, 1022, 1120, 1962, 1121, 1049, 716,
        1783, 1027, 844, 1868, 846, 849, 724, 862, 983, 1875, 982, 865, 986, 
       1924, 987, 874, 989, 1881, 988, 878, 995, 1926, 996, 893, 1889, 749, 
       1041, 1131, 1967, 1132, 1076, 915, 918}], 
      Line3DBox[{560, 1000, 1001, 931, 1901, 772, 932, 1902, 777, 792, 1843, 
       779, 1142, 1143, 1969, 1144, 1100, 1098, 1777, 1126, 1127, 1128, 1858, 
       1026, 1087, 1086, 1951, 1088, 1067, 1068, 848, 1966, 1066, 1065, 1134, 
       1968, 1091, 1092, 864, 1876, 850, 603, 875, 943, 1908, 877, 613, 895, 
       1890, 1040, 897, 1793, 946, 900, 1892, 917, 1080, 1079, 1133}], 
      Line3DBox[{563, 781, 1017, 1928, 782, 1046, 1047, 1939, 695, 1020, 696, 
       1774, 794, 978, 1923, 709, 807, 710, 1779, 979, 811, 1851, 717, 829, 
       718, 1785, 939, 833, 852, 1870, 834, 1052, 1941, 1053, 726, 1032, 727, 
       1787, 868, 990, 1925, 740, 880, 741, 1790, 991, 881, 997, 1927, 753, 
       902, 754, 1795, 998, 948, 906, 920, 907, 922}], 
      Line3DBox[{625, 676, 1071, 908, 1823, 617, 1113, 1960, 1112, 1034, 742, 
       1819, 606, 962, 1918, 961, 869, 728, 1814, 984, 597, 1873, 659, 856, 
       719, 1809, 590, 653, 1862, 836, 835, 583, 1931, 1104, 1103, 1023, 711, 
       1807, 574, 955, 1914, 954, 795, 697, 1803, 972, 564, 950, 1911, 949, 
       783, 682, 1796, 968, 550, 631, 924, 755, 923}], 
      Line3DBox[{626, 677, 1012, 911, 1895, 887, 619, 1909, 672, 885, 1885, 
       883, 609, 667, 1957, 1122, 1141, 1963, 1140, 599, 661, 1148, 1135, 
       1866, 1149, 1028, 1146, 1090, 1145, 1089, 1952, 592, 654, 1008, 1855, 
       841, 1006, 1005, 585, 1953, 649, 1004, 819, 1854, 815, 577, 644, 800, 
       1846, 798, 568, 638, 1832, 1042, 765, 1798, 553, 632, 1831, 926, 1058, 
       1057, 1125}], 
      Line3DBox[{627, 678, 1887, 890, 620, 673, 1830, 888, 993, 1886, 992, 
       610, 668, 1919, 963, 871, 1879, 733, 600, 662, 1959, 1110, 1029, 1934, 
       721, 593, 655, 1857, 823, 586, 650, 821, 981, 1856, 980, 578, 645, 
       1915, 956, 802, 974, 1847, 973, 569, 639, 1912, 951, 787, 1840, 687, 
       555, 633, 1954, 1094, 1014, 762, 767}], 
      Line3DBox[{629, 680, 916, 1896, 914, 622, 1117, 1116, 1039, 748, 1822, 
       612, 1882, 669, 942, 736, 1817, 602, 663, 863, 725, 1812, 595, 657, 
       1867, 847, 845, 588, 1933, 1106, 1105, 1025, 1932, 715, 580, 1102, 
       1956, 1101, 1099, 704, 1806, 1139, 571, 1844, 640, 791, 691, 1800, 559,
        635, 1834, 930, 770, 929}], 
      Line3DBox[{490, 356, 443, 1613, 354, 623, 1910, 674, 898, 1891, 896, 
       614, 670, 876, 1818, 604, 664, 1869, 866, 349, 1636, 484, 348, 1608, 
       432, 343, 479, 1634, 342, 826, 1859, 825, 581, 1965, 646, 1097, 572, 
       641, 1837, 1044, 778, 1801, 561, 453, 1622, 455, 336, 420}], 
      Line3DBox[{630, 681, 919, 1897, 904, 624, 675, 901, 1893, 751, 616, 671,
        1829, 879, 1883, 738, 605, 665, 1111, 1871, 1031, 854, 1813, 596, 658,
        1861, 851, 831, 589, 1905, 652, 828, 1860, 809, 582, 647, 1827, 806, 
       1849, 707, 573, 642, 1955, 1096, 1019, 693, 1802, 562, 636, 1095, 1836,
        1016, 775, 780}], 
      Line3DBox[{759, 999, 757, 1083, 1949, 1081, 1082, 552, 1797, 684, 785, 
       637, 566, 1804, 699, 643, 1944, 575, 812, 1903, 934, 648, 1904, 584, 
       1007, 817, 1064, 1945, 1062, 1063, 591, 1810, 839, 858, 1864, 660, 598,
        1815, 730, 666, 1942, 607, 1820, 743, 1036, 1114, 1115, 1937, 618, 
       1073, 1136, 1137, 1947, 1138, 1074, 1010, 1011, 1093}], 
      Line3DBox[{921, 1077, 905, 1078, 1948, 903, 1013, 947, 1794, 752, 899, 
       750, 615, 1789, 739, 737, 1935, 1033, 855, 1877, 867, 853, 1907, 941, 
       1069, 832, 1946, 1070, 830, 1009, 938, 1784, 810, 827, 1850, 808, 933, 
       1778, 708, 706, 1930, 1021, 694, 1938, 1045, 692, 1119, 1929, 1018, 
       1084, 776, 1950, 1085, 774, 1002, 1003}]}, {
      Line3DBox[{1151, 1297, 1299, 631, 1435, 1898, 1298, 1302, 1758, 1949, 
       1617, 1615, 1436, 1616, 1618, 1301, 1438, 1687, 1831, 1437, 1899, 1304,
        1529, 1639, 1640, 1954, 1528, 1306, 1307, 1833, 1228, 1152, 1835, 
       1309, 1311, 1688, 1834, 1440, 1901, 1310, 1442, 1619, 1622, 1620, 1441,
        1621, 1623, 1950, 1313, 1532, 1641, 1836, 1762, 1531, 1928, 1315, 
       1573, 1229, 1316}], 
      Line3DBox[{1154, 1570, 1256, 1318, 1911, 1472, 1473, 1317, 1838, 1257, 
       637, 1320, 1839, 1305, 1689, 1832, 1571, 1703, 1773, 1322, 1474, 1475, 
       1912, 1321, 1490, 1259, 1325, 1913, 1476, 1477, 1324, 1842, 1491, 1261,
        1328, 1844, 1230, 1327, 1843, 1314, 1681, 1690, 1837, 1572, 1938, 
       1262, 1535, 1642, 1643, 1955, 1534, 1706, 1774, 1330, 1538, 1539, 
       1537}], Line3DBox[{1162, 1263, 1574, 1939, 1576, 1575, 1161, 1705, 
       1802, 1533, 1769, 1929, 1656, 1160, 1739, 1801, 1443, 1902, 1738, 1159,
        1704, 1800, 1312, 1260, 1825, 1158, 1488, 1489, 1799, 1308, 1487, 
       1486, 1157, 1840, 1258, 1530, 1961, 1655, 1654, 1156, 1737, 1798, 1439,
        1900, 1736, 1155, 1702, 1797, 1303, 683, 1153, 1484, 1485, 1796, 1300,
        1653, 1652, 1569}], 
      Line3DBox[{1164, 1578, 1579, 1333, 1914, 1478, 1231, 1332, 1845, 1265, 
       1600, 1944, 1232, 1166, 1740, 1775, 644, 1336, 1709, 1776, 1338, 1479, 
       1691, 1915, 1337, 1492, 1493, 1341, 1916, 1480, 1233, 1340, 1848, 1266,
        1540, 1956, 1645, 1234, 1644, 1763, 1777, 1665, 1965, 1235, 1172, 
       1711, 1778, 1444, 1692, 1827, 1343, 1494, 1495, 1779, 1346, 1543, 1236,
        1542}], 
      Line3DBox[{1174, 398, 1331, 1923, 311, 1173, 1849, 1267, 1536, 1930, 
       1710, 1171, 1097, 1682, 542, 1969, 1170, 539, 1806, 1329, 310, 1922, 
       1169, 308, 1805, 1326, 307, 1168, 1847, 306, 1323, 1841, 1708, 1167, 
       1846, 1335, 797, 1165, 1707, 1804, 1264, 1826, 1163, 305, 1803, 1319, 
       510, 1577}], 
      Line3DBox[{1176, 1269, 1545, 1764, 1931, 1237, 1544, 1742, 1780, 1447, 
       1904, 1238, 1446, 1624, 1626, 1625, 1448, 1518, 1633, 1953, 1239, 1353,
        1713, 1781, 1449, 650, 1355, 1272, 1782, 1240, 1581, 1715, 1783, 1547,
        1765, 1933, 1241, 1546, 1759, 1951, 1628, 1627, 1450, 1519, 1634, 
       1242, 1359, 1741, 1784, 1451, 1905, 1243, 1361, 1717, 1785, 1452, 1244,
        1582}], 
      Line3DBox[{1184, 1583, 1275, 1347, 1851, 1716, 1183, 1860, 1345, 1445, 
       1344, 1850, 1182, 1859, 1358, 1666, 1357, 1858, 1770, 1181, 1932, 1274,
        1541, 1273, 1658, 1962, 1180, 1714, 1808, 1342, 1271, 1497, 1179, 
       1856, 1496, 1270, 1339, 712, 1178, 1854, 1351, 1350, 1853, 1177, 1903, 
       1349, 1601, 1348, 1852, 1755, 1175, 1712, 1807, 1334, 1268, 1657, 
       1580}], Line3DBox[{1185, 1364, 1366, 1693, 1862, 1365, 1863, 1352, 
       1453, 1756, 1945, 1604, 1602, 1368, 1603, 1605, 1354, 1630, 1520, 1606,
        1855, 1752, 1370, 1865, 1356, 1586, 1694, 1857, 1186, 656, 1187, 1868,
        1372, 1374, 1695, 1867, 1373, 1757, 1966, 1667, 1360, 1632, 1521, 
       1608, 1522, 1376, 1607, 1609, 1946, 1362, 1457, 1696, 1861, 1378, 1870,
        1363, 1589, 1245, 1188}], 
      Line3DBox[{1190, 1585, 1277, 1381, 1873, 1246, 1380, 1872, 1369, 1384, 
       1697, 1864, 1383, 1874, 1371, 1646, 1675, 1676, 1866, 1772, 1684, 1548,
        1686, 1683, 1685, 1719, 1786, 1550, 1648, 1649, 1959, 1549, 1280, 
       1387, 1917, 1481, 1482, 1386, 1875, 1498, 1281, 1390, 663, 1389, 1876, 
       1377, 1698, 1869, 1392, 1877, 1379, 1553, 1650, 1871, 1767, 1552, 1722,
        1787, 1393, 1556, 1557, 1555}], 
      Line3DBox[{1198, 1282, 1590, 1754, 1941, 1591, 1197, 1746, 1813, 1458, 
       1907, 1745, 1196, 1523, 1636, 1524, 1456, 1631, 1674, 1968, 1761, 1195,
        1721, 1812, 1375, 724, 1194, 1720, 1811, 1279, 1193, 1934, 1278, 1587,
        1753, 1940, 1588, 1192, 1760, 1952, 1635, 1673, 1672, 1455, 1670, 
       1671, 1669, 1629, 1191, 1744, 1810, 1454, 1906, 1743, 1189, 1718, 1809,
        1367, 1276, 1584}], 
      Line3DBox[{1200, 410, 1396, 1918, 296, 1395, 1878, 1284, 1596, 1942, 
       1247, 1202, 1108, 1699, 1957, 1661, 1724, 1788, 1399, 297, 1919, 1398, 
       319, 1402, 1920, 298, 1401, 1881, 321, 1405, 1882, 1248, 1459, 1908, 
       1404, 670, 1208, 1727, 1789, 1700, 1829, 1407, 323, 1790, 1408, 381, 
       1558}], Line3DBox[{1210, 1593, 1594, 1394, 1925, 1508, 1507, 1209, 
       1883, 1286, 1554, 1935, 1726, 1207, 1747, 1818, 875, 1206, 1725, 1817, 
       1391, 1506, 1505, 1924, 1205, 1503, 1504, 1816, 1388, 1502, 1501, 1204,
        1879, 1285, 1551, 1964, 1663, 1662, 1203, 1963, 1679, 1680, 1647, 
       1958, 1766, 1201, 1723, 1815, 1385, 1283, 1828, 1199, 1499, 1500, 1814,
        1382, 1660, 1659, 1592}], 
      Line3DBox[{1212, 1288, 1560, 1960, 1651, 1249, 1559, 1936, 1290, 1563, 
       1768, 1937, 1250, 1562, 1748, 1791, 1460, 1909, 1251, 1412, 1731, 1792,
        1462, 1701, 1830, 1415, 1512, 1513, 1464, 1921, 1483, 1252, 1418, 
       1889, 1294, 1565, 1117, 1253, 1564, 1750, 1793, 1466, 1910, 1254, 1422,
        1734, 1794, 1468, 675, 1425, 1516, 1517, 1795, 1470, 1568, 1255, 
       1567}], Line3DBox[{1220, 1599, 1296, 1409, 1927, 1735, 1515, 1219, 
       1893, 1295, 750, 1218, 1891, 1421, 1420, 1890, 1217, 1733, 1822, 1406, 
       1293, 1514, 1926, 1216, 1511, 1732, 1821, 1403, 1292, 1510, 1215, 1886,
        1509, 1291, 1400, 1880, 1730, 1214, 1885, 1411, 1410, 1884, 1213, 
       1729, 1820, 1597, 1289, 1598, 1943, 1211, 1728, 1819, 1397, 1287, 1664,
        1595}], 
      Line3DBox[{1227, 1428, 1471, 906, 1226, 1897, 1427, 1469, 1426, 1948, 
       1614, 1527, 1638, 1526, 1613, 1424, 1467, 1423, 1892, 1225, 1896, 1434,
        1566, 1433, 1668, 1967, 1224, 1749, 1824, 1465, 1419, 1223, 1887, 
       1417, 1463, 1416, 1888, 1222, 1895, 1414, 1461, 1413, 1612, 1525, 1637,
        1610, 1611, 1947, 1678, 1432, 1677, 1431, 1894, 1771, 1221, 1751, 
       1823, 1561, 1429, 1430}]}}},
   VertexNormals->CompressedData["
1:eJytfXlUTl30sCKZh6iIhFSGUIqkPKeoJBTxkmSWMfOszLNMJYWigZAMhVCe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    "]],
  AutomaticImageSize->True,
  Axes->True,
  DisplayFunction->Identity,
  ImageSize->{361.9092964921807, 246.7016124090426},
  Method->{},
  PlotRange->{{0., 6.283184609047885}, {0., 
   6.283184609047885}, {-0.999999999999939, 0.999999999999939}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{-0.1974853754623486, -3.0659008805746426`, 1.418185924683406},
  ViewVertical->{0.03503401544920535, -0.20157944565124708`, 
   3.075133220492545}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{{3.580365151121438*^9, 3.580365177313884*^9}, 
   3.5803652174839544`*^9, {3.5803653501621876`*^9, 3.5803653752002316`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graphics3D", "[", 
  RowBox[{"Polygon", "[", "pts", "]"}], "]"}]], "Input"],

Cell[BoxData[
 Graphics3DBox[Polygon3DBox[{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}}],
  AutomaticImageSize->True,
  ImageSize->{295.8978018443685, 303.1180929343556},
  ViewPoint->{0.06566761143153935, -3.2734571576404883`, 0.8544975142744016},
  ViewVertical->{-0.0005228704587434269, -0.3682958662791733, 
   0.9297084927482144}]], "Output",
 CellChangeTimes->{3.580365548610136*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wireframe", "[", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"Polygon", "[", "pts", "]"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5803653986626725`*^9, 3.580365464931589*^9}, 
   3.580365521856089*^9}],

Cell[BoxData[
 Graphics3DBox[
  Line3DBox[{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0, 0}}],
  AutomaticImageSize->True,
  ImageSize->{361.30389820680244`, 393.36182778348297`},
  ViewPoint->{1.2553500860476057`, -2.5942099348831262`, 1.773181032837299},
  ViewVertical->{-0.018526047283885627`, -0.09993736829741351, 
   0.9948212442392967}]], "Output",
 CellChangeTimes->{3.5803654664759912`*^9, 3.5803655236188917`*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["stereo4d", "Subsection",
 CellChangeTimes->{{3.5802029502115974`*^9, 3.5802029660300255`*^9}}],

Cell["Corrected version:", "Text",
 CellChangeTimes->{{3.5802040007330427`*^9, 3.580204007238254*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"stereo4d", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_", ",", "d_"}], "}"}], ",", "surf_", 
    ",", "\n", "        ", 
    RowBox[{"{", 
     RowBox[{"u_", ",", "umin_", ",", "umax_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v_", ",", "vmin_", ",", "vmax_"}], "}"}], ",", "opts___"}], 
   "]"}], ":=", "\n", "        ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"g1", ",", "g2", ",", "g3", ",", "g4"}], "}"}], ",", "\n", 
    "        ", 
    RowBox[{
     RowBox[{"g1", "=", 
      RowBox[{"ParametricPlot3D", "[", 
       RowBox[{
        RowBox[{"Evaluate", "[", "\n", "                ", 
         RowBox[{
          RowBox[{"Drop", "[", 
           RowBox[{"surf", ",", 
            RowBox[{"{", "1", "}"}]}], "]"}], "/", 
          RowBox[{"(", 
           RowBox[{"a", "-", 
            RowBox[{"surf", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}], "]"}], ",", "\n", 
        "                ", 
        RowBox[{"{", 
         RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
        "                ", 
        RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], "]"}]}],
      ";", "\n", "        ", 
     RowBox[{"g2", "=", 
      RowBox[{"ParametricPlot3D", "[", 
       RowBox[{
        RowBox[{"Evaluate", "[", "\n", "                ", 
         RowBox[{
          RowBox[{"Drop", "[", 
           RowBox[{"surf", ",", 
            RowBox[{"{", "2", "}"}]}], "]"}], "/", 
          RowBox[{"(", 
           RowBox[{"b", "-", 
            RowBox[{"surf", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], "]"}], ",", "\n", 
        "                ", 
        RowBox[{"{", 
         RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
        "                ", 
        RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], "]"}]}],
      ";", "\n", "        ", 
     RowBox[{"g3", "=", 
      RowBox[{"ParametricPlot3D", "[", 
       RowBox[{
        RowBox[{"Evaluate", "[", "\n", "                ", 
         RowBox[{
          RowBox[{"Drop", "[", 
           RowBox[{"surf", ",", 
            RowBox[{"{", "3", "}"}]}], "]"}], "/", 
          RowBox[{"(", 
           RowBox[{"c", "-", 
            RowBox[{"surf", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}], ")"}]}], "]"}], ",", "\n", 
        "                ", 
        RowBox[{"{", 
         RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
        "                ", 
        RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], "]"}]}],
      ";", "\n", "        ", 
     RowBox[{"g4", "=", 
      RowBox[{"ParametricPlot3D", "[", 
       RowBox[{
        RowBox[{"Evaluate", "[", "\n", "                ", 
         RowBox[{
          RowBox[{"Drop", "[", 
           RowBox[{"surf", ",", 
            RowBox[{"{", "4", "}"}]}], "]"}], "/", 
          RowBox[{"(", 
           RowBox[{"d", "-", 
            RowBox[{"surf", "[", 
             RowBox[{"[", "4", "]"}], "]"}]}], ")"}]}], "]"}], ",", "\n", 
        "                ", 
        RowBox[{"{", 
         RowBox[{"u", ",", "umin", ",", "umax"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", "vmin", ",", "vmax"}], "}"}], ",", "\n", 
        "                ", 
        RowBox[{"DisplayFunction", "->", "Identity"}], ",", "opts"}], "]"}]}],
      ";", "\n", "        ", 
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"GraphicsGrid", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"g1", ",", "g2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"g3", ",", "g4"}], "}"}]}], "}"}], ",", "\n", 
         RowBox[{"Spacings", "->", 
          RowBox[{"{", 
           RowBox[{"0.2", ",", "0"}], "}"}]}]}], "]"}], ",", "\n", 
       "                ", 
       RowBox[{"DisplayFunction", "->", "$DisplayFunction"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 Evaluatable->False,
 CellChangeTimes->{{3.580203130079913*^9, 3.5802031847112093`*^9}}],

Cell[BoxData[
 RowBox[{"stereo4d", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"sf4D", "[", 
    RowBox[{"u", ",", "v"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"u", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.580203256658536*^9, 3.580203304332219*^9}, 
   3.5802033364214754`*^9, {3.5802034032207932`*^9, 3.580203409476404*^9}}],

Cell[BoxData["          "], "Input",
 CellChangeTimes->{{3.582379256161629*^9, 3.5823792568636303`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Appendix", "Section",
 CellChangeTimes->{{3.580269944353208*^9, 3.580269948955216*^9}}],

Cell[CellGroupData[{

Cell["References", "Subsection",
 CellChangeTimes->{{3.582610253553647*^9, 3.5826102569848433`*^9}}],

Cell["\<\
[G1] Alfred Gray. Modern Differential Geometry of Curves and Surfaces.
CRC Press. 1994\
\>", "Text",
 CellChangeTimes->{{3.5826324110140877`*^9, 3.58263243377339*^9}, {
  3.5826324638531103`*^9, 3.582632479324995*^9}},
 CellTags->"Gray1"],

Cell["\<\
[G2] Alfred Gray. Differentialgeometrie. Klassische Theorie in moderner \
Darstellung. (\[CapitalUDoubleDot]bersetzung aus dem Amerikanischen H. Gollek).
Spektrum Akademischer Verlag, Heidelberg.Berlin.Oxford. 1994.\
\>", "Text",
 CellChangeTimes->{{3.450237198875*^9, 3.45023720096875*^9}, {
   3.450237249703125*^9, 3.450237440828125*^9}, {3.450238940359375*^9, 
   3.45023894278125*^9}, {3.454066758375*^9, 3.4540668540625*^9}, 
   3.45407173278125*^9, 3.541836408394155*^9, 3.5418367181953783`*^9, 
   3.582632359846162*^9},
 CellTags->"Gray2"],

Cell[TextData[{
 "[G3] Alfred Gray, Simon Salamon, Elsa Abbena. Modern Differential Geometry \
of Curves and Surfaces with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".\nThird ed. CRC Press. 2006. "
}], "Text",
 CellChangeTimes->{{3.4481738125625*^9, 3.448173840671875*^9}, {
   3.44817393246875*^9, 3.44817394509375*^9}, {3.448173979828125*^9, 
   3.448173980734375*^9}, {3.4481759745*^9, 3.448176007265625*^9}, 
   3.450237117953125*^9, {3.450237154*^9, 3.450237191546875*^9}, {
   3.450238953984375*^9, 3.450238956734375*^9}, 3.541836408394127*^9, 
   3.5418367181953497`*^9, 3.5826323678696203`*^9},
 CellTags->"Gray3"],

Cell[TextData[{
 StyleBox[ButtonBox["[G4]",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://alpha01.dm.unito.it/personalpages/abbena/gray/"], None},
  ButtonNote->"http://alpha01.dm.unito.it/personalpages/abbena/gray/"],
  FontColor->GrayLevel[0]],
 ButtonBox["     ",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://alpha01.dm.unito.it/personalpages/abbena/gray/"], None},
  ButtonNote->"http://alpha01.dm.unito.it/personalpages/abbena/gray/"],
 StyleBox[ButtonBox["http://alpha01.dm.unito.it/personalpages/abbena/gray/",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://alpha01.dm.unito.it/personalpages/abbena/gray/"], None},
  ButtonNote->"http://alpha01.dm.unito.it/personalpages/abbena/gray/"],
  Background->RGBColor[1, 0.9, 0.8]]
}], "Text",
 CellChangeTimes->{
  3.582634670663333*^9, {3.5826347119046917`*^9, 3.582634711907692*^9}, {
   3.582634815413612*^9, 3.5826348601251693`*^9}, {3.5826349655171976`*^9, 
   3.582635022189439*^9}},
 CellTags->"Gray4"],

Cell[TextData[{
 "\n[G5] Gray.zip.    ",
 StyleBox[ButtonBox["http://library.wolfram.com/infocenter/Books/3759/",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://library.wolfram.com/infocenter/Books/3759/"], None},
  ButtonNote->"http://library.wolfram.com/infocenter/Books/3759/"],
  Background->RGBColor[1, 0.9, 0.8]]
}], "Text",
 CellChangeTimes->{{3.582723909226206*^9, 3.5827239634519005`*^9}, {
  3.582724023934652*^9, 3.5827240527933025`*^9}},
 CellTags->"Gray5"],

Cell[TextData[{
 "[MG] Mohammad Ghomi.    ",
 StyleBox[ButtonBox["http://people.math.gatech.edu/~ghomi/Classes/Math426/\
index.html",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://people.math.gatech.edu/~ghomi/Classes/Math426/index.html"], 
    None},
  ButtonNote->
   "http://people.math.gatech.edu/~ghomi/Classes/Math426/index.html"],
  Background->RGBColor[1, 0.9, 0.8]]
}], "Text",
 CellChangeTimes->{{3.5827243212866592`*^9, 3.582724378781948*^9}, {
  3.5827244205763383`*^9, 3.5827244205803385`*^9}},
 CellTags->"MG"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Obsolete Contexts", "Subsection",
 CellChangeTimes->{{3.5801257356977468`*^9, 3.580125744542962*^9}},
 CellTags->"Obso"],

Cell[TextData[{
 "Alfred Gray working with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v. 2.2 needed the Contexts contained in the next cells. These Contexts don\
\[CloseCurlyQuote]t exist in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " v.9. We deleted them in the BeginPackage[] commands."
}], "Text",
 CellChangeTimes->{{3.5801257853806343`*^9, 3.580125947106118*^9}, {
  3.580186090742247*^9, 3.580186124609906*^9}, {3.580186470244114*^9, 
  3.5801864751737223`*^9}, {3.5801865593982697`*^9, 3.580186575279098*^9}, {
  3.580186617929573*^9, 3.5801866593164454`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Graphics`Graphics`\>\"", "]"}], "\n"}]], "Input",
 CellChangeTimes->{{3.5801249975824504`*^9, 3.5801250381269217`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Get", "::", "noopen"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot open \[NoBreak]\\!\\(\\\"Graphics`Graphics`\\\"\\)\
\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/noopen\\\", ButtonNote -> \
\\\"Get::noopen\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5801250393749237`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Needs", "::", "nocont"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Context \[NoBreak]\\!\\(\\\"Graphics`Graphics`\\\"\\)\
\[NoBreak] was not created when Needs was evaluated. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Needs/nocont\\\", ButtonNote -> \
\\\"Needs::nocont\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.580125039406124*^9}],

Cell[BoxData["$Failed"], "Output",
 CellChangeTimes->{{3.5801250289073057`*^9, 3.580125039406124*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<Graphics`ImplicitPlot`\>\"", "]"}], 
  "\n"}]], "Input",
 CellChangeTimes->{{3.5801249975824504`*^9, 3.5801250545693502`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Get", "::", "noopen"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot open \[NoBreak]\\!\\(\\\"Graphics`ImplicitPlot`\\\"\
\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/noopen\\\", ButtonNote -> \
\\\"Get::noopen\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5801250581573567`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Needs", "::", "nocont"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Context \[NoBreak]\\!\\(\\\"Graphics`ImplicitPlot`\\\"\\)\
\[NoBreak] was not created when Needs was evaluated. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Needs/nocont\\\", ButtonNote -> \
\\\"Needs::nocont\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5801250581885567`*^9}],

Cell[BoxData["$Failed"], "Output",
 CellChangeTimes->{3.5801250581885567`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<NumericalMath`BesselZeros`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.5801254792332964`*^9, 3.5801254863937087`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Get", "::", "noopen"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot open \[NoBreak]\\!\\(\\\"NumericalMath`BesselZeros`\
\\\"\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\
\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/noopen\\\", ButtonNote -> \
\\\"Get::noopen\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5801254944121227`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Needs", "::", "nocont"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Context \[NoBreak]\\!\\(\\\"NumericalMath`BesselZeros`\\\"\
\\)\[NoBreak] was not created when Needs was evaluated. \\!\\(\\*ButtonBox[\\\
\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Needs/nocont\\\", ButtonNote -> \
\\\"Needs::nocont\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.580125494443323*^9}],

Cell[BoxData["$Failed"], "Output",
 CellChangeTimes->{3.580125494443323*^9}]
}, Open  ]],

Cell[BoxData[" "], "Input",
 CellChangeTimes->{3.58012503162171*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Epilog", "Subsection",
 CellChangeTimes->{{3.582604048205722*^9, 3.582604061129461*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"curvlist", " ", "=", " ", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Names", "[", "\"\<CURVES`*\>\"", "]"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.582604092292243*^9, 3.5826041351786957`*^9}, {
   3.582604177010089*^9, 3.5826042274669747`*^9}, 3.5826043543632326`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Map", "[", 
   RowBox[{"Protect", ",", " ", "curvlist"}], "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5826042403397107`*^9, 3.582604269783395*^9}, 
   3.5826043673709764`*^9}]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{1431, 886},
WindowMargins->{{10, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingPageRange->{32000, 32000},
PrintingOptions->{"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{595.1999999999999, 841.8}},
ShowSelection->True,
ShowCellTags->True,
Magnification->1.7999999523162842`,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3582362491-6815425"->{
  Cell[290917, 6510, 282, 5, 144629, "Print",
   CellTags->"Info3582362491-6815425"],
  Cell[291202, 6517, 3636, 92, 270001, "Print",
   CellTags->"Info3582362491-6815425"]},
 "Info3582371794-2942315"->{
  Cell[319942, 7427, 216, 4, 108584, "Print",
   CellTags->"Info3582371794-2942315"],
  Cell[320161, 7433, 2256, 59, 141499, "Print",
   CellTags->"Info3582371794-2942315"]},
 "Info3582371932-2942315"->{
  Cell[324583, 7570, 580, 13, 111562, "Print",
   CellTags->"Info3582371932-2942315"]},
 "Info3582371966-2942315"->{
  Cell[326662, 7639, 297, 5, 144629, "Print",
   CellTags->"Info3582371966-2942315"]},
 "Info3582374518-2942315"->{
  Cell[869180, 17321, 481, 8, 178234, "Print",
   CellTags->"Info3582374518-2942315"],
  Cell[869664, 17331, 3542, 101, 342842, "Print",
   CellTags->"Info3582374518-2942315"]},
 "Info3581411434-4296284"->{
  Cell[986340, 21100, 2676, 43, 2147483647, "Print",
   CellTags->"Info3581411434-4296284"]},
 "Info3581412392-4296284"->{
  Cell[996345, 21358, 2897, 45, 2147483647, "Print",
   CellTags->"Info3581412392-4296284"]},
 "Info3582387245-1359573"->{
  Cell[1000832, 21458, 393, 6, 99019, "Print",
   CellTags->"Info3582387245-1359573"]},
 "Info3581743820-8655308"->{
  Cell[1045362, 22905, 1854, 31, 692986146, "Print",
   CellTags->"Info3581743820-8655308"]},
 "Info3582274333-2934694"->{
  Cell[1099048, 24652, 492, 12, 898043, "Print",
   CellTags->"Info3582274333-2934694"]},
 "Info3582275250-3508047"->{
  Cell[1110817, 25053, 279, 5, 1172953, "Print",
   CellTags->"Info3582275250-3508047"]},
 "Info3582276210-7037343"->{
  Cell[1116004, 25239, 214, 4, 881981, "Print",
   CellTags->"Info3582276210-7037343"]},
 "Info3582276397-7037343"->{
  Cell[1117207, 25285, 339, 6, 1172953, "Print",
   CellTags->"Info3582276397-7037343"]},
 "Info3580287543-1090846"->{
  Cell[1494385, 33076, 356, 6, 2147483647, "Print",
   CellTags->"Info3580287543-1090846"],
  Cell[1494744, 33084, 1101, 29, 2147483647, "Print",
   CellTags->"Info3580287543-1090846"]},
 "Gray1"->{
  Cell[2608499, 51780, 248, 6, 109, "Text",
   CellTags->"Gray1"]},
 "Gray2"->{
  Cell[2608750, 51788, 558, 10, 142, "Text",
   CellTags->"Gray2"]},
 "Gray3"->{
  Cell[2609311, 51800, 633, 13, 143, "Text",
   CellTags->"Gray3"]},
 "Gray4"->{
  Cell[2609947, 51815, 995, 23, 75, "Text",
   CellTags->"Gray4"]},
 "Gray5"->{
  Cell[2610945, 51840, 480, 11, 109, "Text",
   CellTags->"Gray5"]},
 "MG"->{
  Cell[2611428, 51853, 536, 14, 75, "Text",
   CellTags->"MG"]},
 "Obso"->{
  Cell[2612001, 51872, 126, 2, 108, "Subsection",
   CellTags->"Obso"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3582362491-6815425", 2617880, 52045},
 {"Info3582371794-2942315", 2618084, 52050},
 {"Info3582371932-2942315", 2618288, 52055},
 {"Info3582371966-2942315", 2618405, 52058},
 {"Info3582374518-2942315", 2618521, 52061},
 {"Info3581411434-4296284", 2618728, 52066},
 {"Info3581412392-4296284", 2618851, 52069},
 {"Info3582387245-1359573", 2618974, 52072},
 {"Info3581743820-8655308", 2619091, 52075},
 {"Info3582274333-2934694", 2619214, 52078},
 {"Info3582275250-3508047", 2619333, 52081},
 {"Info3582276210-7037343", 2619452, 52084},
 {"Info3582276397-7037343", 2619570, 52087},
 {"Info3580287543-1090846", 2619689, 52090},
 {"Gray1", 2619888, 52095},
 {"Gray2", 2619968, 52098},
 {"Gray3", 2620049, 52101},
 {"Gray4", 2620130, 52104},
 {"Gray5", 2620210, 52107},
 {"MG", 2620288, 52110},
 {"Obso", 2620364, 52113}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 297, 6, 181, "Title"],
Cell[879, 30, 213, 8, 75, "Text"],
Cell[1095, 40, 133, 3, 74, "Text"],
Cell[1231, 45, 134, 2, 53, "Text"],
Cell[CellGroupData[{
Cell[1390, 51, 98, 1, 153, "Section"],
Cell[1491, 54, 4163, 84, 594, "Text"],
Cell[5657, 140, 2018, 32, 1533, "Text"],
Cell[7678, 174, 3762, 67, 625, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11477, 246, 77, 1, 153, "Section"],
Cell[CellGroupData[{
Cell[11579, 251, 176, 2, 86, "Subsection"],
Cell[11758, 255, 1151, 24, 187, "Text"],
Cell[12912, 281, 346, 9, 53, "Input",
 InitializationCell->True],
Cell[13261, 292, 362, 7, 87, "Text"],
Cell[CellGroupData[{
Cell[13648, 303, 312, 5, 53, "Input",
 InitializationCell->True],
Cell[13963, 310, 1023, 18, 38, "Print"]
}, Open  ]],
Cell[15001, 331, 474, 8, 120, "Text"],
Cell[15478, 341, 1471, 34, 257, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16998, 381, 183, 4, 129, "Subtitle"],
Cell[CellGroupData[{
Cell[17206, 389, 96, 1, 153, "Section"],
Cell[17305, 392, 662, 12, 88, "Text"],
Cell[17970, 406, 286, 5, 53, "Input"],
Cell[CellGroupData[{
Cell[18281, 415, 129, 2, 53, "Input"],
Cell[18413, 419, 216, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18666, 427, 161, 3, 53, "Input"],
Cell[18830, 432, 201, 3, 51, "Output"]
}, Open  ]],
Cell[19046, 438, 646, 11, 120, "Text"],
Cell[CellGroupData[{
Cell[19717, 453, 197, 4, 53, "Input",
 InitializationCell->True],
Cell[19917, 459, 36667, 903, 2529, "Print"]
}, Open  ]],
Cell[56599, 1365, 114, 1, 53, "Text"],
Cell[CellGroupData[{
Cell[56738, 1370, 160, 2, 86, "Subsection"],
Cell[56901, 1374, 257, 7, 89, "Text"],
Cell[CellGroupData[{
Cell[57183, 1385, 104, 1, 68, "Subsubsection"],
Cell[57290, 1388, 150, 3, 95, "Input"],
Cell[57443, 1393, 433, 11, 95, "Input"],
Cell[57879, 1406, 150, 3, 95, "Input"],
Cell[CellGroupData[{
Cell[58054, 1413, 575, 10, 95, "Input"],
Cell[58632, 1425, 1829, 58, 819478, "Output"]
}, Closed]],
Cell[60476, 1486, 259, 6, 70, "Input"],
Cell[60738, 1494, 215, 4, 156, "Text"],
Cell[CellGroupData[{
Cell[60978, 1502, 653, 12, 95, "Input"],
Cell[61634, 1516, 1795, 57, 1072616, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[63466, 1578, 740, 15, 70, "Input"],
Cell[64209, 1595, 1793, 57, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[66051, 1658, 80, 1, 54, "Subsubsection"],
Cell[66134, 1661, 150, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[66309, 1668, 424, 8, 60325, "Input"],
Cell[66736, 1678, 574, 17, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[67359, 1701, 111, 1, 54, "Subsubsection"],
Cell[67473, 1704, 144, 3, 33514, "Input"],
Cell[67620, 1709, 191, 6, 33514, "Input"],
Cell[CellGroupData[{
Cell[67836, 1719, 216, 5, 33514, "Input"],
Cell[68055, 1726, 326, 11, 2147483647, "Output"]
}, Closed]],
Cell[68396, 1740, 148, 3, 24632, "Input"],
Cell[CellGroupData[{
Cell[68569, 1747, 252, 6, 33514, "Input"],
Cell[68824, 1755, 435, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[69296, 1774, 256, 6, 24632, "Input"],
Cell[69555, 1782, 327, 10, 2147483647, "Output"]
}, Closed]],
Cell[69897, 1795, 373, 10, 24632, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[70307, 1810, 100, 1, 54, "Subsubsection"],
Cell[70410, 1813, 144, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[70579, 1820, 343, 8, 60325, "Input"],
Cell[70925, 1830, 818, 23, 2147483647, "Output"]
}, Closed]],
Cell[71758, 1856, 246, 6, 44337, "Input"],
Cell[72007, 1864, 143, 3, 60325, "Input"],
Cell[72153, 1869, 567, 14, 102178, "Input"],
Cell[72723, 1885, 253, 6, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[73013, 1896, 168, 2, 54, "Subsubsection"],
Cell[73184, 1900, 158, 3, 60325, "Input"],
Cell[73345, 1905, 148, 3, 60325, "Input"],
Cell[73496, 1910, 98, 1, 60325, "Text"],
Cell[73597, 1913, 254, 4, 60325, "Input"],
Cell[CellGroupData[{
Cell[73876, 1921, 191, 5, 60325, "Input"],
Cell[74070, 1928, 367, 11, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[74474, 1944, 357, 9, 44337, "Input"],
Cell[74834, 1955, 780, 24, 4063215, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[75651, 1984, 127, 2, 44337, "Input"],
Cell[75781, 1988, 1203, 35, 3696229, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[77021, 2028, 253, 6, 44337, "Input"],
Cell[77277, 2036, 555, 17, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[77869, 2058, 480, 12, 44337, "Input"],
Cell[78352, 2072, 125, 2, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[78514, 2079, 433, 10, 44337, "Input"],
Cell[78950, 2091, 94, 1, 2797858, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[79093, 2098, 123, 1, 54, "Subsubsection"],
Cell[79219, 2101, 146, 3, 60325, "Input"],
Cell[79368, 2106, 149, 3, 60325, "Input"],
Cell[79520, 2111, 150, 3, 60325, "Input"],
Cell[79673, 2116, 155, 3, 60325, "Input"],
Cell[79831, 2121, 236, 6, 60325, "Input"],
Cell[CellGroupData[{
Cell[80092, 2131, 382, 8, 60325, "Input"],
Cell[80477, 2141, 482, 13, 28784683, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[80996, 2159, 916, 26, 85111, "Input"],
Cell[81915, 2187, 816, 24, 28784683, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[82768, 2216, 404, 10, 44337, "Input"],
Cell[83175, 2228, 147081, 2390, 252939492, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[230305, 4624, 89, 1, 54, "Subsubsection"],
Cell[230397, 4627, 161, 3, 60325, "Input"],
Cell[230561, 4632, 190, 4, 60325, "Input"],
Cell[230754, 4638, 948, 19, 102178, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[231739, 4662, 102, 1, 54, "Subsubsection"],
Cell[231844, 4665, 150, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[232019, 4672, 278, 7, 60325, "Input"],
Cell[232300, 4681, 664, 18, 136463740, "Output"]
}, Closed]],
Cell[232979, 4702, 387, 10, 44337, "Input"],
Cell[233369, 4714, 496, 13, 102178, "Input"],
Cell[233868, 4729, 175, 4, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[234080, 4738, 102, 1, 54, "Subsubsection"],
Cell[234185, 4741, 477, 14, 52772967, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[234687, 4759, 182, 5, 31982147, "Input"],
Cell[234872, 4766, 227, 7, 34115282, "Output"]
}, Closed]],
Cell[235114, 4776, 476, 11, 31982147, "Input"],
Cell[235593, 4789, 533, 13, 31982147, "Input"],
Cell[236129, 4804, 144, 3, 31982147, "Input"],
Cell[236276, 4809, 599, 13, 31982147, "Input"],
Cell[236878, 4824, 135, 3, 31982147, "Input"],
Cell[237016, 4829, 137, 3, 31982147, "Input"],
Cell[237156, 4834, 190, 4, 31982147, "Input"],
Cell[237349, 4840, 249, 6, 31982147, "Input"],
Cell[CellGroupData[{
Cell[237623, 4850, 239, 5, 31982147, "Input"],
Cell[237865, 4857, 2720, 80, 167382173, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[240622, 4942, 164, 3, 31982147, "Input"],
Cell[240789, 4947, 3769, 100, 340095598, "Output"]
}, Closed]],
Cell[244573, 5050, 413, 10, 31982147, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[245023, 5065, 175, 2, 54, "Subsubsection"],
Cell[245201, 5069, 147, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[245373, 5076, 350, 9, 60325, "Input"],
Cell[245726, 5087, 171, 4, 44776696, "Output"]
}, Closed]],
Cell[245912, 5094, 345, 9, 44337, "Input"],
Cell[246260, 5105, 159, 3, 60325, "Input"],
Cell[246422, 5110, 214, 5, 60325, "Input"],
Cell[246639, 5117, 203, 4, 60325, "Input"],
Cell[CellGroupData[{
Cell[246867, 5125, 133, 3, 60325, "Input"],
Cell[247003, 5130, 166, 5, 47442065, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[247206, 5140, 189, 4, 44337, "Input"],
Cell[247398, 5146, 184, 6, 47442065, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[247631, 5158, 104, 1, 54, "Subsubsection"],
Cell[247738, 5161, 146, 3, 60325, "Input"],
Cell[247887, 5166, 491, 12, 60325, "Input"],
Cell[248381, 5180, 143, 3, 60325, "Input"],
Cell[248527, 5185, 508, 13, 60325, "Input"],
Cell[249038, 5200, 308, 7, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[249383, 5212, 227, 3, 54, "Subsubsection"],
Cell[249613, 5217, 147, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[249785, 5224, 513, 12, 60325, "Input"],
Cell[250301, 5238, 601, 16, 61834222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[250939, 5259, 1074, 29, 44337, "Input"],
Cell[252016, 5290, 591, 16, 64499155, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[252644, 5311, 228, 5, 44337, "Input"],
Cell[252875, 5318, 215, 4, 53837728, "Output"]
}, Closed]],
Cell[253105, 5325, 149, 3, 44337, "Input"],
Cell[CellGroupData[{
Cell[253279, 5332, 405, 11, 60325, "Input"],
Cell[253687, 5345, 617, 25, 63965730, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[254341, 5375, 454, 12, 44337, "Input"],
Cell[254798, 5389, 295, 10, 61834222, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[255130, 5404, 822, 24, 44337, "Input"],
Cell[255955, 5430, 187, 3, 53837728, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[256179, 5438, 268, 7, 44337, "Input"],
Cell[256450, 5447, 503, 19, 62367616, "Output"]
}, Closed]],
Cell[256968, 5469, 246, 6, 44337, "Input"],
Cell[257217, 5477, 139, 3, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[257393, 5485, 100, 1, 54, "Subsubsection"],
Cell[257496, 5488, 142, 3, 60325, "Input"],
Cell[257641, 5493, 183, 4, 60325, "Text"],
Cell[257827, 5499, 444, 13, 60325, "Input"],
Cell[CellGroupData[{
Cell[258296, 5516, 351, 10, 60325, "Input"],
Cell[258650, 5528, 816, 24, 44776696, "Output"]
}, Closed]],
Cell[259481, 5555, 551, 12, 44337, "Input"],
Cell[260035, 5569, 146, 3, 60325, "Input"],
Cell[260184, 5574, 966, 26, 102178, "Input"],
Cell[261153, 5602, 150, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[261328, 5609, 258, 6, 60325, "Input"],
Cell[261589, 5617, 975, 26, 78359343, "Output",
 Evaluatable->True]
}, Closed]],
Cell[262579, 5646, 148, 3, 44337, "Input"],
Cell[CellGroupData[{
Cell[262752, 5653, 2200, 56, 259343, "Input"],
Cell[264955, 5711, 634, 18, 44776696, "Output"]
}, Closed]],
Cell[265604, 5732, 665, 16, 44337, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[266306, 5753, 103, 1, 54, "Subsubsection"],
Cell[266412, 5756, 146, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[266583, 5763, 273, 7, 60325, "Input"],
Cell[266859, 5772, 1148, 32, 84756592, "Output"]
}, Closed]],
Cell[268022, 5807, 340, 9, 44337, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[268399, 5821, 242, 5, 54, "Subsubsection"],
Cell[268644, 5828, 164, 3, 60325, "Input"],
Cell[268811, 5833, 289, 5, 99626, "Text"],
Cell[269103, 5840, 748, 22, 101235, "Input"],
Cell[269854, 5864, 267, 7, 60325, "Input"],
Cell[CellGroupData[{
Cell[270146, 5875, 229, 6, 60325, "Input"],
Cell[270378, 5883, 722, 26, 79958442, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[271137, 5914, 266, 7, 44337, "Input"],
Cell[271406, 5923, 630, 22, 79958442, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[272073, 5950, 262, 7, 44337, "Input"],
Cell[272338, 5959, 568, 20, 79958442, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[272943, 5984, 414, 11, 44337, "Input"],
Cell[273360, 5997, 5914, 150, 331033084, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[279311, 6152, 159, 3, 44337, "Input"],
Cell[279473, 6157, 2227, 58, 147658109, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[281737, 6220, 268, 7, 44337, "Input"],
Cell[282008, 6229, 788, 22, 71430168, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[282833, 6256, 320, 8, 44337, "Input"],
Cell[283156, 6266, 1387, 39, 123137278, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[284592, 6311, 102, 1, 54, "Subsubsection"],
Cell[284697, 6314, 144, 3, 60325, "Input"],
Cell[284844, 6319, 590, 14, 60325, "Input"],
Cell[CellGroupData[{
Cell[285459, 6337, 179, 4, 60325, "Input"],
Cell[285641, 6343, 230, 6, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[285908, 6354, 273, 6, 44337, "Input"],
Cell[286184, 6362, 501, 13, 2964284, "Output"]
}, Closed]],
Cell[286700, 6378, 761, 17, 85111, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[287498, 6400, 237, 3, 54, "Subsubsection"],
Cell[287738, 6405, 144, 3, 60325, "Input"],
Cell[287885, 6410, 195, 3, 60325, "Input"],
Cell[288083, 6415, 342, 9, 60325, "Input"],
Cell[288428, 6426, 320, 7, 60325, "Input"],
Cell[CellGroupData[{
Cell[288773, 6437, 243, 6, 60325, "Input"],
Cell[289019, 6445, 202, 6, 15723971, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[289258, 6456, 206, 5, 44337, "Input"],
Cell[289467, 6463, 484, 14, 15723971, "Output"]
}, Closed]],
Cell[289966, 6480, 589, 13, 44337, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[290592, 6498, 125, 1, 54, "Subsubsection"],
Cell[CellGroupData[{
Cell[290742, 6503, 150, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[290917, 6510, 282, 5, 144629, "Print",
 CellTags->"Info3582362491-6815425"],
Cell[291202, 6517, 3636, 92, 270001, "Print",
 CellTags->"Info3582362491-6815425"]
}, Open  ]]
}, Open  ]],
Cell[294865, 6613, 466, 10, 60325, "Input"],
Cell[CellGroupData[{
Cell[295356, 6627, 917, 19, 60325, "Input"],
Cell[296276, 6648, 1796, 48, 176976812, "Output"]
}, Closed]],
Cell[298087, 6699, 692, 14, 44337, "Input"],
Cell[298782, 6715, 154, 3, 60325, "Input"],
Cell[298939, 6720, 511, 14, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[299487, 6739, 101, 1, 54, "Subsubsection"],
Cell[299591, 6742, 169, 3, 60325, "Input"],
Cell[299763, 6747, 244, 8, 60325, "Input"],
Cell[300010, 6757, 139, 3, 60325, "Input"],
Cell[300152, 6762, 475, 13, 102178, "Input"],
Cell[300630, 6777, 140, 3, 60325, "Input"],
Cell[300773, 6782, 179, 4, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[300989, 6791, 78, 1, 54, "Subsubsection"],
Cell[301070, 6794, 172, 3, 60325, "Input"],
Cell[301245, 6799, 573, 17, 102178, "Input"],
Cell[CellGroupData[{
Cell[301843, 6820, 419, 12, 60325, "Input"],
Cell[302265, 6834, 328, 10, 2797858, "Output"]
}, Closed]],
Cell[302608, 6847, 554, 15, 44337, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[303199, 6867, 168, 2, 54, "Subsubsection"],
Cell[303370, 6871, 146, 3, 60325, "Input"],
Cell[303519, 6876, 353, 11, 60325, "Input"],
Cell[303875, 6889, 380, 10, 60325, "Input"],
Cell[CellGroupData[{
Cell[304280, 6903, 395, 9, 60325, "Input"],
Cell[304678, 6914, 1050, 34, 6595446, "Output"]
}, Closed]],
Cell[305743, 6951, 155, 3, 44337, "Input"],
Cell[305901, 6956, 341, 9, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[306279, 6970, 81, 1, 54, "Subsubsection"],
Cell[306363, 6973, 153, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[306541, 6980, 223, 5, 60325, "Input"],
Cell[306767, 6987, 495, 14, 2163346, "Output"]
}, Closed]],
Cell[307277, 7004, 232, 6, 44337, "Input"],
Cell[CellGroupData[{
Cell[307534, 7014, 252, 5, 60325, "Input"],
Cell[307789, 7021, 210, 6, 1913443, "Output"]
}, Closed]],
Cell[308014, 7030, 276, 7, 44337, "Input"],
Cell[CellGroupData[{
Cell[308315, 7041, 325, 7, 60325, "Input"],
Cell[308643, 7050, 236, 3, 715582, "Print"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[308928, 7059, 98, 1, 54, "Subsubsection"],
Cell[309029, 7062, 166, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[309220, 7069, 522, 15, 102178, "Input"],
Cell[309745, 7086, 3145, 91, 2247100, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[312939, 7183, 81, 1, 54, "Subsubsection"],
Cell[313023, 7186, 175, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[313223, 7193, 346, 9, 60325, "Input"],
Cell[313572, 7204, 807, 26, 1014841, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[314416, 7235, 145, 3, 44337, "Input"],
Cell[314564, 7240, 1448, 48, 1631341, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[316049, 7293, 149, 3, 44337, "Input"],
Cell[316201, 7298, 933, 31, 1197581, "Output"]
}, Closed]],
Cell[317149, 7332, 296, 7, 44337, "Input"],
Cell[317448, 7341, 658, 16, 60325, "Input"],
Cell[318109, 7359, 113, 1, 60325, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[318271, 7366, 124, 2, 86, "Subsection"],
Cell[318398, 7370, 151, 3, 60325, "Input"],
Cell[318552, 7375, 144, 3, 60325, "Input"],
Cell[318699, 7380, 159, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[318883, 7387, 180, 4, 60325, "Input"],
Cell[319066, 7393, 227, 7, 2147483647, "Output"]
}, Closed]],
Cell[319308, 7403, 242, 6, 44337, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[319587, 7414, 160, 2, 69, "Subsection"],
Cell[CellGroupData[{
Cell[319772, 7420, 145, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[319942, 7427, 216, 4, 108584, "Print",
 CellTags->"Info3582371794-2942315"],
Cell[320161, 7433, 2256, 59, 141499, "Print",
 CellTags->"Info3582371794-2942315"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[322466, 7498, 315, 7, 60325, "Input"],
Cell[322784, 7507, 1096, 35, 137584, "Output"]
}, Open  ]],
Cell[323895, 7545, 40, 0, 60325, "Input"],
Cell[CellGroupData[{
Cell[323960, 7549, 152, 3, 60325, "Input"],
Cell[324115, 7554, 96, 1, 85791, "Output"]
}, Open  ]],
Cell[324226, 7558, 211, 4, 60325, "Text"],
Cell[CellGroupData[{
Cell[324462, 7566, 118, 2, 60325, "Input"],
Cell[324583, 7570, 580, 13, 111562, "Print",
 CellTags->"Info3582371932-2942315"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325200, 7588, 700, 22, 102178, "Input"],
Cell[325903, 7612, 601, 18, 91461, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[326541, 7635, 118, 2, 60325, "Input"],
Cell[326662, 7639, 297, 5, 144629, "Print",
 CellTags->"Info3582371966-2942315"]
}, Open  ]],
Cell[CellGroupData[{
Cell[326996, 7649, 698, 21, 102178, "Input"],
Cell[327697, 7672, 574, 18, 124433, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[328320, 7696, 78, 1, 69, "Subsection"],
Cell[328401, 7699, 151, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[328577, 7706, 380, 10, 60325, "Input"],
Cell[328960, 7718, 1316, 43, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[330313, 7766, 436, 9, 44337, "Input"],
Cell[330752, 7777, 972, 30, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[331761, 7812, 436, 12, 44337, "Input"],
Cell[332200, 7826, 230638, 3812, 2147483647, "Output"]
}, Closed]],
Cell[562853, 11641, 148, 3, 44337, "Input"],
Cell[CellGroupData[{
Cell[563026, 11648, 747, 18, 102178, "Input"],
Cell[563776, 11668, 18210, 304, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[582023, 11977, 252, 6, 44337, "Input"],
Cell[582278, 11985, 248640, 4110, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[830967, 16101, 83, 1, 69, "Subsection"],
Cell[831053, 16104, 155, 3, 60325, "Input"],
Cell[831211, 16109, 661, 11, 177550, "Text"],
Cell[831875, 16122, 111, 1, 60325, "Text"],
Cell[831989, 16125, 152, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[832166, 16132, 148, 2, 78064, "Subsubsection"],
Cell[832317, 16136, 622, 17, 60325, "Input"],
Cell[CellGroupData[{
Cell[832964, 16157, 269, 6, 60325, "Input"],
Cell[833236, 16165, 749, 24, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[834022, 16194, 174, 4, 44337, "Input"],
Cell[834199, 16200, 116, 3, 2147483647, "Output"]
}, Closed]],
Cell[834330, 16206, 457, 13, 44337, "Input"],
Cell[834790, 16221, 293, 8, 60325, "Input"],
Cell[835086, 16231, 293, 8, 60325, "Input"],
Cell[835382, 16241, 305, 9, 60325, "Input"],
Cell[835690, 16252, 307, 9, 60325, "Input"],
Cell[836000, 16263, 358, 10, 60325, "Input"],
Cell[CellGroupData[{
Cell[836383, 16277, 149, 3, 60325, "Input"],
Cell[836535, 16282, 388, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[836960, 16299, 131, 2, 44337, "Input"],
Cell[837094, 16303, 145, 3, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[837276, 16311, 149, 3, 44337, "Input"],
Cell[837428, 16316, 94, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[837559, 16322, 147, 3, 44337, "Input"],
Cell[837709, 16327, 453, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[838199, 16346, 129, 2, 44337, "Input"],
Cell[838331, 16350, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[838440, 16356, 383, 7, 44337, "Input"],
Cell[838826, 16365, 3496, 109, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[842359, 16479, 129, 2, 44337, "Input"],
Cell[842491, 16483, 96, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[842624, 16489, 246, 6, 44337, "Input"],
Cell[842873, 16497, 738, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[843648, 16524, 131, 2, 44337, "Input"],
Cell[843782, 16528, 70, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[843901, 16535, 121, 1, 61979, "Subsubsection"],
Cell[844025, 16538, 172, 3, 60325, "Text"],
Cell[844200, 16543, 301, 10, 60325, "Input"],
Cell[CellGroupData[{
Cell[844526, 16557, 211, 5, 60325, "Input"],
Cell[844740, 16564, 4197, 127, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[848974, 16696, 300, 8, 44337, "Input"],
Cell[849277, 16706, 5929, 162, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[855243, 16873, 362, 9, 44337, "Input"],
Cell[855608, 16884, 581, 10, 2147483647, "Message"],
Cell[856192, 16896, 79, 1, 2147483647, "Output"]
}, Closed]],
Cell[856286, 16900, 210, 4, 50426, "Text"],
Cell[CellGroupData[{
Cell[856521, 16908, 255, 6, 60325, "Input"],
Cell[856779, 16916, 79, 1, 85791, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[856907, 16923, 160, 2, 61979, "Subsubsection"],
Cell[857070, 16927, 250, 6, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[857345, 16937, 229, 5, 2147483647, "Input"],
Cell[857577, 16944, 70, 1, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[857696, 16951, 89, 1, 61979, "Subsubsection"],
Cell[857788, 16954, 224, 5, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[858037, 16963, 136, 3, 2147483647, "Input"],
Cell[858176, 16968, 149, 4, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[858362, 16977, 232, 5, 2147483647, "Input"],
Cell[858597, 16984, 498, 17, 2147483647, "Output"]
}, Closed]],
Cell[859110, 17004, 371, 10, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[859518, 17019, 81, 1, 61979, "Subsubsection"],
Cell[859602, 17022, 351, 10, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[859978, 17036, 182, 4, 2147483647, "Input"],
Cell[860163, 17042, 323, 10, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[860523, 17057, 277, 6, 2147483647, "Input"],
Cell[860803, 17065, 2550, 80, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[863390, 17150, 129, 2, 2147483647, "Input"],
Cell[863522, 17154, 791, 25, 2147483647, "Output"]
}, Closed]],
Cell[864328, 17182, 505, 12, 2147483647, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[864882, 17200, 102, 1, 69, "Subsection"],
Cell[864987, 17203, 211, 4, 60325, "Text"],
Cell[865201, 17209, 1117, 28, 140515, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[866343, 17241, 319, 8, 60325, "Input",
 InitializationCell->True],
Cell[866665, 17251, 912, 19, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[867614, 17275, 319, 8, 60325, "Input",
 InitializationCell->True],
Cell[867936, 17285, 911, 19, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[868884, 17309, 100, 1, 78064, "Subsubsection"],
Cell[CellGroupData[{
Cell[869009, 17314, 146, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[869180, 17321, 481, 8, 178234, "Print",
 CellTags->"Info3582374518-2942315"],
Cell[869664, 17331, 3542, 101, 342842, "Print",
 CellTags->"Info3582374518-2942315"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[873267, 17439, 162, 2, 78064, "Subsubsection"],
Cell[873432, 17443, 298, 8, 60325, "Input"],
Cell[CellGroupData[{
Cell[873755, 17455, 296, 7, 60325, "Input"],
Cell[874054, 17464, 494, 14, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[874585, 17483, 432, 11, 44337, "Input"],
Cell[875020, 17496, 323, 9, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[875380, 17510, 545, 14, 44337, "Input"],
Cell[875928, 17526, 8577, 259, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[884542, 17790, 211, 4, 44337, "Input"],
Cell[884756, 17796, 6396, 194, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[891189, 17995, 132, 2, 44337, "Input"],
Cell[891324, 17999, 117, 3, 2147483647, "Output"]
}, Closed]],
Cell[891456, 18005, 214, 5, 50426, "Text"],
Cell[CellGroupData[{
Cell[891695, 18014, 287, 8, 60325, "Input"],
Cell[891985, 18024, 694, 22, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[892716, 18051, 565, 15, 44337, "Input"],
Cell[893284, 18068, 2484, 74, 2147483647, "Output"]
}, Closed]],
Cell[895783, 18145, 126, 2, 44337, "Input"],
Cell[895912, 18149, 784, 13, 178234, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[896733, 18167, 156, 2, 78064, "Subsubsection"],
Cell[896892, 18171, 145, 3, 60325, "Input"],
Cell[897040, 18176, 112, 1, 60325, "Text"],
Cell[CellGroupData[{
Cell[897177, 18181, 392, 10, 60325, "Input"],
Cell[897572, 18193, 7480, 244, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[905089, 18442, 258, 6, 44337, "Input"],
Cell[905350, 18450, 97, 1, 527735063, "Output"]
}, Closed]],
Cell[905462, 18454, 420, 10, 90766, "Text"],
Cell[905885, 18466, 286, 7, 60325, "Input"],
Cell[CellGroupData[{
Cell[906196, 18477, 562, 14, 60325, "Input"],
Cell[906761, 18493, 126, 2, 527735063, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[906924, 18500, 463, 12, 44337, "Input"],
Cell[907390, 18514, 97, 1, 527735063, "Output"]
}, Closed]],
Cell[907502, 18518, 107, 1, 50426, "Text"],
Cell[907612, 18521, 292, 7, 60325, "Input"],
Cell[907907, 18530, 222, 7, 60325, "Text"],
Cell[CellGroupData[{
Cell[908154, 18541, 877, 22, 141499, "Input"],
Cell[909034, 18565, 176, 4, 527735063, "Output"]
}, Closed]],
Cell[909225, 18572, 400, 8, 208297, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[909662, 18585, 102, 1, 61979, "Subsubsection"],
Cell[CellGroupData[{
Cell[909789, 18590, 392, 9, 60325, "Input"],
Cell[910184, 18601, 8535, 273, 498913, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[918756, 18879, 629, 16, 102178, "Input"],
Cell[919388, 18897, 11037, 376, 685139, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[930462, 19278, 204, 4, 60325, "Input"],
Cell[930669, 19284, 194, 3, 85791, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[930924, 19294, 314, 4, 69, "Subsection"],
Cell[931241, 19300, 139, 3, 60325, "Input"],
Cell[931383, 19305, 505, 9, 137584, "Text"],
Cell[CellGroupData[{
Cell[931913, 19318, 103, 1, 78064, "Subsubsection"],
Cell[932019, 19321, 758, 13, 178234, "Text"],
Cell[932780, 19336, 147, 3, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[932964, 19344, 167, 2, 61979, "Subsubsection"],
Cell[933134, 19348, 320, 8, 60325, "Input"],
Cell[CellGroupData[{
Cell[933479, 19360, 176, 3, 60325, "Input"],
Cell[933658, 19365, 739, 19, 403162, "Output"]
}, Closed]],
Cell[934412, 19387, 129, 1, 50426, "Text"],
Cell[934544, 19390, 637, 14, 60325, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[935206, 19408, 275, 7, 60325, "Input"],
Cell[935484, 19417, 334, 10, 244211, "Output"]
}, Closed]],
Cell[935833, 19430, 413, 12, 44337, "Input"],
Cell[936249, 19444, 109, 1, 60325, "Text"],
Cell[936361, 19447, 228, 6, 60325, "Input"],
Cell[936592, 19455, 205, 4, 60325, "Text"],
Cell[936800, 19461, 1249, 35, 140515, "Input",
 Evaluatable->False],
Cell[938052, 19498, 87, 1, 60325, "Text"],
Cell[CellGroupData[{
Cell[938164, 19503, 204, 5, 60325, "Input"],
Cell[938371, 19510, 183, 5, 244211, "Output"]
}, Closed]],
Cell[938569, 19518, 153, 2, 50426, "Text"],
Cell[CellGroupData[{
Cell[938747, 19524, 353, 9, 60325, "Input"],
Cell[939103, 19535, 2187, 72, 976197, "Output"]
}, Closed]],
Cell[941305, 19610, 132, 1, 50426, "Text"],
Cell[CellGroupData[{
Cell[941462, 19615, 133, 2, 60325, "Input"],
Cell[941598, 19619, 117, 3, 244211, "Output"]
}, Closed]],
Cell[941730, 19625, 211, 4, 50426, "Text"],
Cell[CellGroupData[{
Cell[941966, 19633, 243, 6, 60325, "Input"],
Cell[942212, 19641, 2119, 70, 976197, "Output"]
}, Closed]],
Cell[944346, 19714, 120, 1, 50426, "Text"],
Cell[CellGroupData[{
Cell[944491, 19719, 323, 8, 60325, "Input"],
Cell[944817, 19729, 73, 1, 244211, "Output"]
}, Closed]],
Cell[944905, 19733, 241, 5, 89283, "Text"],
Cell[CellGroupData[{
Cell[945171, 19742, 256, 6, 60325, "Input"],
Cell[945430, 19750, 207, 5, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[945674, 19760, 359, 9, 44337, "Input"],
Cell[946036, 19771, 69, 1, 244211, "Output"]
}, Closed]],
Cell[946120, 19775, 182, 4, 50426, "Text"],
Cell[CellGroupData[{
Cell[946327, 19783, 413, 11, 60325, "Input"],
Cell[946743, 19796, 2018, 68, 976197, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[948798, 19869, 277, 7, 44337, "Input"],
Cell[949078, 19878, 99, 1, 244211, "Output"]
}, Closed]],
Cell[949192, 19882, 103, 1, 50426, "Text"],
Cell[949298, 19885, 103, 1, 60325, "Text"],
Cell[949404, 19888, 331, 8, 60325, "Input"],
Cell[949738, 19898, 198, 2, 60325, "Text"],
Cell[CellGroupData[{
Cell[949961, 19904, 275, 6, 60325, "Input"],
Cell[950239, 19912, 1566, 35, 864543, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[951842, 19952, 459, 11, 44337, "Input"],
Cell[952304, 19965, 75, 1, 244211, "Output"]
}, Closed]],
Cell[952394, 19969, 133, 1, 50426, "Text"],
Cell[CellGroupData[{
Cell[952552, 19974, 299, 6, 60325, "Input"],
Cell[952854, 19982, 321, 5, 244211, "Output"]
}, Closed]],
Cell[953190, 19990, 415, 11, 44337, "Input"],
Cell[CellGroupData[{
Cell[953630, 20005, 273, 6, 60325, "Input"],
Cell[953906, 20013, 1030, 27, 439579, "Output"]
}, Closed]],
Cell[954951, 20043, 201, 5, 44337, "Input"],
Cell[CellGroupData[{
Cell[955177, 20052, 245, 6, 60325, "Input"],
Cell[955425, 20060, 283, 8, 435193, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[955745, 20073, 265, 6, 44337, "Input"],
Cell[956013, 20081, 264, 4, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[956314, 20090, 241, 6, 44337, "Input"],
Cell[956558, 20098, 145, 2, 244211, "Output"]
}, Closed]],
Cell[956718, 20103, 906, 14, 205003, "Text"],
Cell[957627, 20119, 605, 17, 101235, "Input"],
Cell[958235, 20138, 439, 12, 60325, "Input"],
Cell[958677, 20152, 240, 5, 60325, "Text"],
Cell[958920, 20159, 165, 3, 60325, "Text"],
Cell[959088, 20164, 443, 12, 60325, "Input"],
Cell[959534, 20178, 491, 12, 60325, "Input"],
Cell[960028, 20192, 317, 9, 60325, "Input"],
Cell[CellGroupData[{
Cell[960370, 20205, 724, 18, 101235, "Input"],
Cell[961097, 20225, 124, 2, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[961258, 20232, 129, 2, 44337, "Input"],
Cell[961390, 20236, 75, 1, 244211, "Output"]
}, Closed]],
Cell[961480, 20240, 264, 7, 44337, "Input"],
Cell[CellGroupData[{
Cell[961769, 20251, 200, 5, 60325, "Input"],
Cell[961972, 20258, 2192, 75, 614608, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[964201, 20338, 261, 7, 44337, "Input"],
Cell[964465, 20347, 73, 1, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[964575, 20353, 565, 14, 44337, "Input"],
Cell[965143, 20369, 126, 2, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[965306, 20376, 292, 6, 44337, "Input"],
Cell[965601, 20384, 6080, 207, 2451227, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[971718, 20596, 339, 7, 44337, "Input"],
Cell[972060, 20605, 137, 3, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[972234, 20613, 341, 7, 44337, "Input"],
Cell[972578, 20622, 770, 27, 452500, "Output"]
}, Closed]],
Cell[973363, 20652, 281, 6, 129932, "Text"],
Cell[973647, 20660, 319, 9, 60325, "Input"],
Cell[CellGroupData[{
Cell[973991, 20673, 584, 16, 101235, "Input"],
Cell[974578, 20691, 101, 1, 244211, "Output"]
}, Closed]],
Cell[974694, 20695, 313, 9, 44337, "Input"],
Cell[CellGroupData[{
Cell[975032, 20708, 301, 8, 60325, "Input"],
Cell[975336, 20718, 339, 10, 244211, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[975712, 20733, 489, 14, 44337, "Input"],
Cell[976204, 20749, 117, 3, 244211, "Output"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[976382, 20759, 138, 2, 69, "Subsection"],
Cell[CellGroupData[{
Cell[976545, 20765, 158, 2, 78064, "Subsubsection",
 Evaluatable->False],
Cell[976706, 20769, 140, 3, 60325, "Input"],
Cell[976849, 20774, 141, 3, 60325, "Input"],
Cell[976993, 20779, 89, 2, 60325, "Input"],
Cell[977085, 20783, 139, 3, 60325, "Input"],
Cell[977227, 20788, 151, 3, 60325, "Input"],
Cell[977381, 20793, 149, 3, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[977567, 20801, 98, 1, 61979, "Subsubsection"],
Cell[CellGroupData[{
Cell[977690, 20806, 314, 8, 60325, "Input"],
Cell[978007, 20816, 356, 12, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[978400, 20833, 208, 5, 44337, "Input"],
Cell[978611, 20840, 171, 5, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[978819, 20850, 208, 5, 44337, "Input"],
Cell[979030, 20857, 72, 1, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[979139, 20863, 208, 5, 44337, "Input"],
Cell[979350, 20870, 95, 2, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[979482, 20877, 220, 5, 44337, "Input"],
Cell[979705, 20884, 357, 12, 2147483647, "Output"]
}, Closed]],
Cell[980077, 20899, 180, 4, 44337, "Input"],
Cell[CellGroupData[{
Cell[980282, 20907, 245, 6, 60325, "Input"],
Cell[980530, 20915, 1644, 49, 2147483647, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[982211, 20969, 129, 2, 44337, "Input"],
Cell[982343, 20973, 535, 17, 2147483647, "Output"]
}, Closed]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[982939, 20997, 125, 2, 69, "Subsection"],
Cell[983067, 21001, 150, 3, 60325, "Input"],
Cell[983220, 21006, 142, 3, 60325, "Input"],
Cell[983365, 21011, 143, 3, 60325, "Input"],
Cell[983511, 21016, 378, 9, 60325, "Input"],
Cell[983892, 21027, 166, 4, 60325, "Input"],
Cell[984061, 21033, 442, 11, 60325, "Input"],
Cell[984506, 21046, 145, 3, 60325, "Input"],
Cell[984654, 21051, 568, 13, 60325, "Input"],
Cell[985225, 21066, 515, 12, 60325, "Input"],
Cell[985743, 21080, 150, 3, 60325, "Input"],
Cell[985896, 21085, 178, 4, 60325, "Input"],
Cell[986077, 21091, 119, 1, 60325, "Text"],
Cell[CellGroupData[{
Cell[986221, 21096, 116, 2, 60325, "Input"],
Cell[986340, 21100, 2676, 43, 2147483647, "Print",
 CellTags->"Info3581411434-4296284"]
}, Closed]],
Cell[989031, 21146, 2242, 62, 281707, "Input",
 InitializationCell->True],
Cell[991276, 21210, 327, 9, 60325, "Input"],
Cell[CellGroupData[{
Cell[991628, 21223, 129, 2, 60325, "Input"],
Cell[991760, 21227, 3449, 95, 2147483647, "Output"]
}, Closed]],
Cell[995224, 21325, 639, 14, 44337, "Input"],
Cell[995866, 21341, 148, 3, 60325, "Input"],
Cell[996017, 21346, 180, 4, 60325, "Input"],
Cell[CellGroupData[{
Cell[996222, 21354, 120, 2, 60325, "Input"],
Cell[996345, 21358, 2897, 45, 2147483647, "Print",
 CellTags->"Info3581412392-4296284"]
}, Closed]],
Cell[999257, 21406, 460, 11, 44337, "Input"],
Cell[999720, 21419, 154, 3, 60325, "Input"],
Cell[999877, 21424, 180, 4, 60325, "Text"],
Cell[1000060, 21430, 268, 7, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1000365, 21442, 118, 1, 69, "Subsection"],
Cell[1000486, 21445, 178, 4, 33514, "Text"],
Cell[CellGroupData[{
Cell[1000689, 21453, 140, 3, 33514, "Input"],
Cell[1000832, 21458, 393, 6, 99019, "Print",
 CellTags->"Info3582387245-1359573"]
}, Open  ]],
Cell[1001240, 21467, 172, 3, 33514, "Input"],
Cell[CellGroupData[{
Cell[1001437, 21474, 280, 6, 33514, "Input"],
Cell[1001720, 21482, 407, 11, 124433, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1002164, 21498, 377, 10, 24632, "Input"],
Cell[1002544, 21510, 1527, 49, 94674, "Output"]
}, Open  ]],
Cell[1004086, 21562, 138, 3, 33514, "Input"],
Cell[CellGroupData[{
Cell[1004249, 21569, 203, 4, 33514, "Input"],
Cell[1004455, 21575, 697, 23, 53575, "Output"]
}, Open  ]],
Cell[1005167, 21601, 733, 24, 56242, "Input"],
Cell[CellGroupData[{
Cell[1005925, 21629, 358, 12, 33514, "Input"],
Cell[1006286, 21643, 294, 7, 24094, "Message"],
Cell[1006583, 21652, 210, 6, 32218, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1006830, 21663, 407, 9, 33514, "Input"],
Cell[1007240, 21674, 1634, 55, 115045, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1008911, 21734, 1235, 38, 78064, "Input"],
Cell[1010149, 21774, 290, 7, 24094, "Message"],
Cell[1010442, 21783, 292, 7, 24094, "Message"],
Cell[1010737, 21792, 848, 29, 81131, "Output"]
}, Open  ]],
Cell[1011600, 21824, 280, 5, 33514, "Input"],
Cell[CellGroupData[{
Cell[1011905, 21833, 98, 1, 33514, "Input"],
Cell[1012006, 21836, 1663, 56, 115045, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1013706, 21897, 1314, 38, 83026, "Input"],
Cell[1015023, 21937, 268, 7, 24094, "Message"],
Cell[1015294, 21946, 387, 9, 24094, "Message"],
Cell[1015684, 21957, 276, 7, 24094, "Message"]
}, Open  ]],
Cell[1015975, 21967, 289, 7, 33514, "Input"],
Cell[CellGroupData[{
Cell[1016289, 21978, 371, 8, 33514, "Input"],
Cell[1016663, 21988, 6409, 190, 232106, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1023121, 22184, 155, 2, 69, "Subsection"],
Cell[1023279, 22188, 143, 3, 60325, "Input"],
Cell[1023425, 22193, 282, 8, 60325, "Input"],
Cell[CellGroupData[{
Cell[1023732, 22205, 205, 5, 60325, "Input"],
Cell[1023940, 22212, 261, 7, 66493, "Output"]
}, Open  ]],
Cell[1024216, 22222, 151, 3, 60325, "Text"],
Cell[CellGroupData[{
Cell[1024392, 22229, 195, 5, 60325, "Input"],
Cell[1024590, 22236, 265, 8, 57992, "Output"]
}, Open  ]],
Cell[1024870, 22247, 117, 2, 60325, "Input"],
Cell[1024990, 22251, 263, 5, 99626, "Text"],
Cell[CellGroupData[{
Cell[1025278, 22260, 225, 6, 60325, "Input"],
Cell[1025506, 22268, 137, 3, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1025680, 22276, 280, 8, 60325, "Input"],
Cell[1025963, 22286, 288, 7, 66493, "Output"]
}, Open  ]],
Cell[1026266, 22296, 112, 1, 60325, "Text"],
Cell[CellGroupData[{
Cell[1026403, 22301, 195, 5, 60325, "Input"],
Cell[1026601, 22308, 241, 8, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1026879, 22321, 271, 6, 60325, "Input"],
Cell[1027153, 22329, 475, 16, 57992, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1027677, 22351, 99, 1, 69, "Subsection"],
Cell[1027779, 22354, 144, 3, 95, "Input"],
Cell[1027926, 22359, 557, 11, 215, "Text"],
Cell[1028486, 22372, 212, 2, 95, "Text"],
Cell[1028701, 22376, 390, 11, 95, "Input"],
Cell[CellGroupData[{
Cell[1029116, 22391, 419, 11, 95, "Input"],
Cell[1029538, 22404, 1383, 41, 1360387465, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1030970, 22451, 112, 1, 69, "Subsection"],
Cell[1031085, 22454, 353, 6, 99626, "Text"],
Cell[1031441, 22462, 150, 3, 60325, "Input"],
Cell[1031594, 22467, 196, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[1031815, 22474, 285, 6, 60325, "Input"],
Cell[1032103, 22482, 182, 3, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1032322, 22490, 723, 15, 60325, "Input"],
Cell[1033048, 22507, 1015, 29, 57992, "Output"]
}, Open  ]],
Cell[1034078, 22539, 188, 4, 60325, "Input"],
Cell[1034269, 22545, 129, 1, 60325, "Text"],
Cell[1034401, 22548, 394, 9, 60325, "Input"],
Cell[CellGroupData[{
Cell[1034820, 22561, 500, 12, 60325, "Input"],
Cell[1035323, 22575, 746, 26, 96434, "Output"]
}, Open  ]],
Cell[1036084, 22604, 240, 6, 60325, "Input"],
Cell[1036327, 22612, 149, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[1036501, 22619, 476, 11, 60325, "Input"],
Cell[1036980, 22632, 694, 22, 57992, "Output"]
}, Open  ]],
Cell[1037689, 22657, 243, 6, 60325, "Input"],
Cell[1037935, 22665, 152, 3, 60325, "Input"],
Cell[1038090, 22670, 575, 12, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1038702, 22687, 80, 1, 69, "Subsection"],
Cell[1038785, 22690, 155, 3, 60325, "Input"],
Cell[1038943, 22695, 391, 8, 60325, "Input"],
Cell[CellGroupData[{
Cell[1039359, 22707, 439, 8, 60325, "Input"],
Cell[1039801, 22717, 588, 16, 105386, "Output"]
}, Open  ]],
Cell[1040404, 22736, 394, 7, 99626, "Text"],
Cell[CellGroupData[{
Cell[1040823, 22747, 537, 12, 60325, "Input"],
Cell[1041363, 22761, 3538, 124, 170413, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1044950, 22891, 97, 1, 69, "Subsection"],
Cell[1045050, 22894, 164, 3, 60325, "Text"],
Cell[CellGroupData[{
Cell[1045239, 22901, 120, 2, 60325, "Input"],
Cell[1045362, 22905, 1854, 31, 692986146, "Print",
 CellTags->"Info3581743820-8655308"]
}, Closed]],
Cell[1047231, 22939, 377, 11, 44337, "Input"],
Cell[CellGroupData[{
Cell[1047633, 22954, 239, 6, 60325, "Input"],
Cell[1047875, 22962, 70, 1, 375278645, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1047982, 22968, 305, 8, 44337, "Input"],
Cell[1048290, 22978, 387, 12, 397666497, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1048714, 22995, 265, 5, 44337, "Input"],
Cell[1048982, 23002, 387, 12, 397666497, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1049406, 23019, 426, 9, 44337, "Input"],
Cell[1049835, 23030, 389, 12, 397666497, "Output"]
}, Closed]],
Cell[1050239, 23045, 167, 4, 50426, "Text"],
Cell[CellGroupData[{
Cell[1050431, 23053, 336, 7, 60325, "Input"],
Cell[1050770, 23062, 126, 2, 375278645, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1050945, 23070, 237, 3, 69, "Subsection"],
Cell[1051185, 23075, 460, 14, 33514, "Text"],
Cell[1051648, 23091, 277, 6, 33514, "Input",
 Evaluatable->False],
Cell[1051928, 23099, 179, 4, 33514, "Text"],
Cell[1052110, 23105, 487, 14, 33514, "Input"],
Cell[1052600, 23121, 169, 3, 33514, "Text"],
Cell[1052772, 23126, 177, 3, 33514, "Input"],
Cell[1052952, 23131, 131, 1, 33514, "Text"],
Cell[CellGroupData[{
Cell[1053108, 23136, 272, 6, 33514, "Input"],
Cell[1053383, 23144, 69, 1, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1053489, 23150, 235, 7, 24632, "Input"],
Cell[1053727, 23159, 259, 8, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1054023, 23172, 117, 2, 24632, "Input"],
Cell[1054143, 23176, 338, 11, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1054518, 23192, 154, 4, 24632, "Input"],
Cell[1054675, 23198, 261, 8, 1965007, "Output"]
}, Closed]],
Cell[1054951, 23209, 183, 2, 28015, "Text"],
Cell[1055137, 23213, 507, 13, 33514, "Input"],
Cell[1055647, 23228, 105, 1, 33514, "Text"],
Cell[CellGroupData[{
Cell[1055777, 23233, 283, 8, 33514, "Input"],
Cell[1056063, 23243, 390, 10, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1056490, 23258, 119, 2, 24632, "Input"],
Cell[1056612, 23262, 448, 13, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1057097, 23280, 127, 2, 24632, "Input"],
Cell[1057227, 23284, 72, 1, 1965007, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1057336, 23290, 245, 5, 24632, "Input"],
Cell[1057584, 23297, 155, 3, 1965007, "Output"]
}, Closed]],
Cell[1057754, 23303, 120, 1, 28015, "Text"],
Cell[CellGroupData[{
Cell[1057899, 23308, 732, 18, 33514, "Input"],
Cell[1058634, 23328, 603, 17, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1059274, 23350, 406, 9, 24632, "Input"],
Cell[1059683, 23361, 473, 13, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1060193, 23379, 493, 11, 24632, "Input"],
Cell[1060689, 23392, 503, 15, 2797858, "Output"]
}, Closed]],
Cell[1061207, 23410, 421, 8, 50426, "Text"],
Cell[1061631, 23420, 486, 13, 33514, "Input"],
Cell[1062120, 23435, 186, 4, 33514, "Text"],
Cell[CellGroupData[{
Cell[1062331, 23443, 1032, 27, 56242, "Input"],
Cell[1063366, 23472, 369, 11, 2797858, "Output"]
}, Closed]],
Cell[1063750, 23486, 102, 1, 28015, "Text"],
Cell[1063855, 23489, 848, 21, 33514, "Input"],
Cell[CellGroupData[{
Cell[1064728, 23514, 341, 8, 33514, "Input"],
Cell[1065072, 23524, 427, 12, 32218, "Output"]
}, Open  ]],
Cell[1065514, 23539, 142, 2, 33514, "Text"],
Cell[CellGroupData[{
Cell[1065681, 23545, 213, 5, 33514, "Input"],
Cell[1065897, 23552, 216, 4, 32218, "Output"]
}, Open  ]],
Cell[1066128, 23559, 130, 2, 33514, "Text"],
Cell[1066261, 23563, 427, 12, 33514, "Input"],
Cell[1066691, 23577, 134, 2, 33514, "Text"],
Cell[CellGroupData[{
Cell[1066850, 23583, 191, 4, 33514, "Input"],
Cell[1067044, 23589, 167, 4, 2797858, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1067248, 23598, 130, 2, 34433, "Subsubsection"],
Cell[1067381, 23602, 150, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[1067556, 23609, 296, 8, 60325, "Input"],
Cell[1067855, 23619, 304, 9, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068196, 23633, 189, 4, 60325, "Input"],
Cell[1068388, 23639, 372, 10, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068797, 23654, 234, 6, 60325, "Input"],
Cell[1069034, 23662, 568, 17, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1069639, 23684, 374, 9, 60325, "Input"],
Cell[1070016, 23695, 449, 14, 57992, "Output"]
}, Open  ]],
Cell[1070480, 23712, 132, 1, 60325, "Text"],
Cell[1070615, 23715, 875, 19, 60325, "Input"],
Cell[1071493, 23736, 418, 12, 60325, "Input"],
Cell[1071914, 23750, 192, 4, 60325, "Text"],
Cell[CellGroupData[{
Cell[1072131, 23758, 237, 5, 60325, "Input"],
Cell[1072371, 23765, 163, 2, 57992, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1072571, 23772, 191, 4, 60325, "Input"],
Cell[1072765, 23778, 143, 4, 60325, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1072969, 23789, 92, 1, 69, "Subsection"],
Cell[1073064, 23792, 143, 3, 60325, "Input"],
Cell[1073210, 23797, 180, 4, 60325, "Text"],
Cell[1073393, 23803, 320, 9, 60325, "Input"],
Cell[1073716, 23814, 166, 2, 60325, "Text"],
Cell[1073885, 23818, 824, 24, 101235, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1074734, 23846, 264, 7, 60325, "Input"],
Cell[1075001, 23855, 1286, 40, 114740, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1076324, 23900, 237, 6, 60325, "Input"],
Cell[1076564, 23908, 679, 21, 83588, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1077280, 23934, 182, 4, 60325, "Input"],
Cell[1077465, 23940, 96, 1, 83588, "Output"]
}, Open  ]],
Cell[1077576, 23944, 245, 5, 60325, "Text"],
Cell[1077824, 23951, 211, 5, 60325, "Input"],
Cell[1078038, 23958, 281, 8, 60325, "Input"],
Cell[1078322, 23968, 186, 4, 60325, "Input"],
Cell[1078511, 23974, 283, 8, 60325, "Input"],
Cell[1078797, 23984, 563, 13, 60325, "Input"],
Cell[CellGroupData[{
Cell[1079385, 24001, 266, 5, 60325, "Input"],
Cell[1079654, 24008, 635, 20, 118310, "Output"]
}, Open  ]],
Cell[1080304, 24031, 283, 8, 60325, "Input"],
Cell[1080590, 24041, 153, 3, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1080780, 24049, 76, 1, 69, "Subsection"],
Cell[1080859, 24052, 149, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[1081033, 24059, 100, 1, 78064, "Subsubsection"],
Cell[1081136, 24062, 727, 15, 60325, "Input"],
Cell[1081866, 24079, 551, 14, 60325, "Input"],
Cell[1082420, 24095, 274, 7, 60325, "Input"],
Cell[1082697, 24104, 537, 14, 60325, "Input"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1083283, 24124, 95, 1, 69, "Subsection"],
Cell[1083381, 24127, 163, 3, 60325, "Input"],
Cell[1083547, 24132, 409, 8, 60325, "Input"],
Cell[CellGroupData[{
Cell[1083981, 24144, 399, 10, 60325, "Input"],
Cell[1084383, 24156, 768, 20, 83588, "Output"]
}, Open  ]],
Cell[1085166, 24179, 714, 16, 60325, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1085917, 24200, 114, 1, 69, "Subsection"],
Cell[1086034, 24203, 144, 3, 60325, "Input"],
Cell[1086181, 24208, 151, 3, 60325, "Input"],
Cell[1086335, 24213, 146, 3, 60325, "Input"],
Cell[1086484, 24218, 150, 3, 60325, "Input"],
Cell[1086637, 24223, 172, 3, 60325, "Input"],
Cell[1086812, 24228, 199, 3, 60325, "Input"],
Cell[1087014, 24233, 194, 3, 60325, "Input"],
Cell[CellGroupData[{
Cell[1087233, 24240, 67, 2, 60325, "Input"],
Cell[1087303, 24244, 109, 3, 881981, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1087449, 24252, 164, 4, 44337, "Input"],
Cell[1087616, 24258, 92, 2, 881981, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1087745, 24265, 399, 11, 44337, "Input"],
Cell[1088147, 24278, 614, 21, 889462, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1088798, 24304, 407, 11, 44337, "Input"],
Cell[1089208, 24317, 330, 11, 1131895, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1089575, 24333, 458, 11, 44337, "Input"],
Cell[1090036, 24346, 367, 12, 1114491, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1090440, 24363, 648, 18, 44337, "Input"],
Cell[1091091, 24383, 4423, 142, 2788199, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1095551, 24530, 361, 10, 44337, "Input"],
Cell[1095915, 24542, 1638, 54, 1572263, "Output"]
}, Closed]],
Cell[1097568, 24599, 137, 3, 50426, "Text"],
Cell[CellGroupData[{
Cell[1097730, 24606, 474, 13, 60325, "Input"],
Cell[1098207, 24621, 257, 8, 856394, "Output"]
}, Closed]],
Cell[1098479, 24632, 335, 9, 44337, "Input"],
Cell[1098817, 24643, 89, 1, 60325, "Text"],
Cell[CellGroupData[{
Cell[1098931, 24648, 114, 2, 60325, "Input"],
Cell[1099048, 24652, 492, 12, 898043, "Print",
 CellTags->"Info3582274333-2934694"]
}, Closed]],
Cell[1099555, 24667, 247, 6, 90766, "Text"],
Cell[CellGroupData[{
Cell[1099827, 24677, 225, 6, 60325, "Input"],
Cell[1100055, 24685, 219, 6, 698011, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1100311, 24696, 204, 5, 44337, "Input"],
Cell[1100518, 24703, 294, 10, 1048381, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1100849, 24718, 485, 14, 44337, "Input"],
Cell[1101337, 24734, 1570, 52, 1572263, "Output"]
}, Closed]],
Cell[1102922, 24789, 500, 11, 44337, "Input"],
Cell[1103425, 24802, 154, 3, 60325, "Text"],
Cell[CellGroupData[{
Cell[1103604, 24809, 351, 7, 60325, "Input"],
Cell[1103958, 24818, 147, 2, 698011, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1104142, 24825, 304, 6, 44337, "Input"],
Cell[1104449, 24833, 518, 17, 964796, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1105004, 24855, 834, 23, 85111, "Input"],
Cell[1105841, 24880, 1542, 50, 1538512, "Output"]
}, Closed]],
Cell[1107398, 24933, 376, 10, 44337, "Input"],
Cell[CellGroupData[{
Cell[1107799, 24947, 401, 8, 60325, "Input"],
Cell[1108203, 24957, 145, 2, 698011, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1108385, 24964, 358, 7, 44337, "Input"],
Cell[1108746, 24973, 518, 17, 964796, "Output"]
}, Closed]],
Cell[1109279, 24993, 170, 3, 50426, "Text"],
Cell[CellGroupData[{
Cell[1109474, 25000, 305, 10, 60325, "Input"],
Cell[1109782, 25012, 365, 12, 1114491, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1110184, 25029, 135, 2, 44337, "Input"],
Cell[1110322, 25033, 335, 11, 964796, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1110694, 25049, 120, 2, 44337, "Input"],
Cell[1110817, 25053, 279, 5, 1172953, "Print",
 CellTags->"Info3582275250-3508047"]
}, Closed]],
Cell[CellGroupData[{
Cell[1111133, 25063, 277, 9, 44337, "Input"],
Cell[1111413, 25074, 332, 11, 1131895, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1111782, 25090, 286, 7, 44337, "Input"],
Cell[1112071, 25099, 1133, 37, 964796, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1113241, 25141, 709, 21, 110109, "Input"],
Cell[1113953, 25164, 459, 16, 964796, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1114449, 25185, 309, 7, 44337, "Input"],
Cell[1114761, 25194, 1082, 36, 964796, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1115880, 25235, 121, 2, 44337, "Input"],
Cell[1116004, 25239, 214, 4, 881981, "Print",
 CellTags->"Info3582276210-7037343"]
}, Closed]],
Cell[CellGroupData[{
Cell[1116255, 25248, 302, 9, 44337, "Input"],
Cell[1116560, 25259, 490, 17, 997549, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[1117087, 25281, 117, 2, 44337, "Input"],
Cell[1117207, 25285, 339, 6, 1172953, "Print",
 CellTags->"Info3582276397-7037343"]
}, Closed]],
Cell[CellGroupData[{
Cell[1117583, 25296, 390, 12, 44337, "Input"],
Cell[1117976, 25310, 1259, 40, 1930708, "Output"]
}, Closed]],
Cell[1119250, 25353, 167, 3, 44337, "Input"],
Cell[1119420, 25358, 161, 4, 60325, "Text"],
Cell[CellGroupData[{
Cell[1119606, 25366, 379, 11, 60325, "Input"],
Cell[1119988, 25379, 324, 10, 110109, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1120349, 25394, 278, 7, 60325, "Input"],
Cell[1120630, 25403, 848, 29, 149446, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1121515, 25437, 303, 8, 60325, "Input"],
Cell[1121821, 25447, 882, 31, 1164068, "Output"]
}, Closed]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1122764, 25485, 91, 1, 153, "Section"],
Cell[1122858, 25488, 621, 9, 120, "Text"],
Cell[1123482, 25499, 452, 7, 53, "Input"],
Cell[CellGroupData[{
Cell[1123959, 25510, 129, 2, 53, "Input"],
Cell[1124091, 25514, 120, 2, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1124248, 25521, 138, 2, 53, "Input"],
Cell[1124389, 25525, 195, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1124621, 25533, 145, 3, 53, "Input",
 InitializationCell->True],
Cell[1124769, 25538, 44407, 1126, 2969, "Print"]
}, Open  ]],
Cell[1169191, 26667, 441, 6, 53, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1169669, 26678, 90, 1, 153, "Section"],
Cell[1169762, 26681, 485, 8, 120, "Text"],
Cell[1170250, 26691, 334, 6, 53, "Input"],
Cell[CellGroupData[{
Cell[1170609, 26701, 129, 2, 53, "Input"],
Cell[1170741, 26705, 98, 1, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1170876, 26711, 137, 2, 53, "Input"],
Cell[1171016, 26715, 199, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171252, 26723, 195, 4, 53, "Input",
 InitializationCell->True],
Cell[1171450, 26729, 40649, 1038, 2729, "Print"]
}, Open  ]],
Cell[1212114, 27770, 339, 4, 53, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1212490, 27779, 93, 1, 153, "Section"],
Cell[1212586, 27782, 404, 7, 87, "Text"],
Cell[1212993, 27791, 293, 9, 53, "Input"],
Cell[CellGroupData[{
Cell[1213311, 27804, 99, 1, 53, "Input"],
Cell[1213413, 27807, 629, 10, 99, "Print"]
}, Open  ]],
Cell[1214057, 27820, 508, 8, 154, "Text"],
Cell[1214568, 27830, 189, 4, 53, "Input"],
Cell[CellGroupData[{
Cell[1214782, 27838, 129, 2, 53, "Input"],
Cell[1214914, 27842, 73, 1, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1215024, 27848, 138, 2, 53, "Input"],
Cell[1215165, 27852, 246, 4, 51, "Output"]
}, Open  ]],
Cell[1215426, 27859, 566, 9, 53, "Text"],
Cell[CellGroupData[{
Cell[1216017, 27872, 304, 5, 53, "Input",
 InitializationCell->True],
Cell[1216324, 27879, 4095, 104, 249, "Print"]
}, Open  ]],
Cell[1220434, 27986, 474, 9, 154, "Text"],
Cell[CellGroupData[{
Cell[1220933, 27999, 102, 1, 86, "Subsection"],
Cell[1221038, 28002, 1095, 31, 67485, "Input",
 Evaluatable->False],
Cell[1222136, 28035, 617, 16, 18619, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[1222778, 28055, 196, 6, 18619, "Input",
 InitializationCell->True],
Cell[1222977, 28063, 1198, 27, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224212, 28095, 420, 11, 31246, "Input"],
Cell[1224635, 28108, 420, 13, 25799, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1225092, 28126, 216, 4, 18619, "Input"],
Cell[1225311, 28132, 1699, 52, 45446, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1227047, 28189, 172, 3, 18619, "Input"],
Cell[1227222, 28194, 966, 23, 2147483647, "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[1228237, 28223, 79, 1, 69, "Subsection"],
Cell[1228319, 28226, 811, 24, 905, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1229155, 28254, 315, 7, 305, "Input"],
Cell[1229473, 28263, 579, 17, 293, "Output"]
}, Open  ]],
Cell[1230067, 28283, 376, 9, 305, "Input"],
Cell[CellGroupData[{
Cell[1230468, 28296, 417, 11, 305, "Input"],
Cell[1230888, 28309, 755, 22, 487, "Output"]
}, Open  ]],
Cell[1231658, 28334, 159, 3, 305, "Input"],
Cell[CellGroupData[{
Cell[1231842, 28341, 895, 18, 305, "Input"],
Cell[1232740, 28361, 1860, 55, 860, "Output"]
}, Open  ]],
Cell[1234615, 28419, 246, 5, 305, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1234898, 28429, 100, 1, 69, "Subsection"],
Cell[1235001, 28432, 4790, 127, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1239816, 28563, 170, 5, 2147483647, "Input"],
Cell[1239989, 28570, 235, 7, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240261, 28582, 246, 8, 2147483647, "Input"],
Cell[1240510, 28592, 376, 12, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240923, 28609, 99, 2, 2147483647, "Input"],
Cell[1241025, 28613, 390, 7, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1241452, 28625, 307, 6, 2147483647, "Input"],
Cell[1241762, 28633, 758, 18, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1242557, 28656, 277, 7, 2147483647, "Input",
 InitializationCell->True],
Cell[1242837, 28665, 1194, 27, 70, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1244068, 28697, 347, 6, 2147483647, "Input"],
Cell[1244418, 28705, 400, 8, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1244855, 28718, 355, 6, 2147483647, "Input"],
Cell[1245213, 28726, 971, 23, 2147483647, "Output"]
}, Open  ]],
Cell[1246199, 28752, 133, 3, 2147483647, "Text"],
Cell[CellGroupData[{
Cell[1246357, 28759, 525, 16, 2147483647, "Input"],
Cell[1246885, 28777, 41771, 685, 2147483647, "Output"]
}, Open  ]],
Cell[1288671, 29465, 575, 17, 2147483647, "Input"],
Cell[1289249, 29484, 258, 5, 2147483647, "Text"],
Cell[1289510, 29491, 1533, 47, 2147483647, "Input",
 Evaluatable->False],
Cell[1291046, 29540, 406, 9, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1291489, 29554, 98, 1, 69, "Subsection"],
Cell[1291590, 29557, 207, 4, 2147483647, "Text"],
Cell[1291800, 29563, 2132, 56, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1293957, 29623, 144, 3, 2147483647, "Input"],
Cell[1294104, 29628, 398, 8, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1294539, 29641, 259, 5, 2147483647, "Input"],
Cell[1294801, 29648, 690, 14, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1295528, 29667, 577, 17, 2147483647, "Input"],
Cell[1296108, 29686, 170465, 2680, 2147483647, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[1466622, 32372, 97, 1, 69, "Subsection"],
Cell[1466722, 32375, 1007, 31, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1467754, 32410, 264, 7, 2147483647, "Input"],
Cell[1468021, 32419, 837, 24, 2147483647, "Output"]
}, Open  ]],
Cell[1468873, 32446, 285, 7, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1469183, 32457, 61, 1, 2147483647, "Input"],
Cell[1469247, 32460, 482, 14, 2147483647, "Output"]
}, Open  ]],
Cell[1469744, 32477, 164, 3, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1469945, 32485, 99, 1, 69, "Subsection"],
Cell[1470047, 32488, 1726, 44, 2147483647, "Input",
 Evaluatable->False],
Cell[1471776, 32534, 396, 11, 2147483647, "Input"],
Cell[1472175, 32547, 875, 20, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1473087, 32572, 111, 1, 69, "Subsection"],
Cell[1473201, 32575, 2667, 76, 2147483647, "Input",
 Evaluatable->False],
Cell[1475871, 32653, 785, 19, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1476681, 32676, 164, 3, 2147483647, "Input"],
Cell[1476848, 32681, 11731, 203, 2147483647, "Output"]
}, Open  ]],
Cell[1488594, 32887, 94, 1, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1488725, 32893, 93, 1, 69, "Subsection"],
Cell[1488821, 32896, 626, 16, 2147483647, "Input",
 Evaluatable->False],
Cell[1489450, 32914, 136, 3, 2147483647, "Text"],
Cell[1489589, 32919, 556, 13, 2147483647, "Input"],
Cell[1490148, 32934, 503, 14, 2147483647, "Input"],
Cell[1490654, 32950, 107, 1, 2147483647, "Text"],
Cell[1490764, 32953, 528, 14, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1491329, 32972, 101, 1, 69, "Subsection"],
Cell[1491433, 32975, 614, 19, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1492072, 32998, 217, 4, 2147483647, "Input"],
Cell[1492292, 33004, 1043, 33, 2147483647, "Output"]
}, Open  ]],
Cell[1493350, 33040, 337, 8, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1493724, 33053, 96, 1, 69, "Subsection"],
Cell[1493823, 33056, 372, 9, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1494220, 33069, 140, 3, 2147483647, "Input"],
Cell[CellGroupData[{
Cell[1494385, 33076, 356, 6, 2147483647, "Print",
 CellTags->"Info3580287543-1090846"],
Cell[1494744, 33084, 1101, 29, 2147483647, "Print",
 CellTags->"Info3580287543-1090846"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1495894, 33119, 131, 2, 2147483647, "Input"],
Cell[1496028, 33123, 130, 2, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496195, 33130, 163, 2, 2147483647, "Input"],
Cell[1496361, 33134, 79, 1, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496477, 33140, 86, 2, 2147483647, "Input"],
Cell[1496566, 33144, 131, 3, 2147483647, "Output"]
}, Open  ]],
Cell[1496712, 33150, 352, 11, 2147483647, "Input"],
Cell[1497067, 33163, 127, 2, 2147483647, "Input"],
Cell[1497197, 33167, 320, 6, 2147483647, "Text"],
Cell[1497520, 33175, 476, 13, 2147483647, "Input"],
Cell[1497999, 33190, 114, 1, 2147483647, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[1498150, 33196, 72, 1, 69, "Subsection"],
Cell[1498225, 33199, 100, 1, 2147483647, "Text"],
Cell[1498328, 33202, 3657, 90, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1502010, 33296, 342, 11, 2147483647, "Input"],
Cell[1502355, 33309, 311, 9, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1502703, 33323, 231, 6, 2147483647, "Input"],
Cell[1502937, 33331, 222, 6, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1503196, 33342, 430, 12, 2147483647, "Input"],
Cell[1503629, 33356, 458, 9, 2147483647, "Message"],
Cell[1504090, 33367, 435049, 7067, 2147483647, 243939, 3933, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[1939154, 40437, 104, 1, 2147483647, "Text"],
Cell[1939261, 40440, 3996, 96, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1943282, 40540, 484, 13, 2147483647, "Input"],
Cell[1943769, 40555, 438641, 7124, 2147483647, 243696, 3927, "CachedBoxData", \
"BoxData", "Output"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[2382459, 47685, 130, 2, 69, "Subsection"],
Cell[2382592, 47689, 1094, 30, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2383711, 47723, 642, 19, 2147483647, "Input"],
Cell[2384356, 47744, 30095, 528, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2414488, 48277, 669, 19, 2147483647, "Input"],
Cell[2415160, 48298, 30188, 530, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2445385, 48833, 197, 5, 2147483647, "Input"],
Cell[2445585, 48840, 59746, 1041, 2147483647, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[2505380, 49887, 77, 1, 69, "Subsection"],
Cell[2505460, 49890, 2681, 67, 2147483647, "Input",
 Evaluatable->False],
Cell[2508144, 49959, 964, 26, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[2509145, 49990, 101, 1, 69, "Subsection"],
Cell[2509249, 49993, 3693, 97, 2147483647, "Input",
 Evaluatable->False],
Cell[2512945, 50092, 520, 15, 2147483647, "Input"],
Cell[2513468, 50109, 310, 6, 2147483647, "Text"],
Cell[2513781, 50117, 162, 3, 2147483647, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[2513980, 50125, 172, 2, 69, "Subsection"],
Cell[2514155, 50129, 2058, 55, 2147483647, "Input",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2516238, 50188, 655, 18, 2147483647, "Input"],
Cell[2516896, 50208, 84878, 1376, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2601811, 51589, 99, 2, 2147483647, "Input"],
Cell[2601913, 51593, 388, 7, 2147483647, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2602338, 51605, 230, 5, 2147483647, "Input"],
Cell[2602571, 51612, 429, 8, 2147483647, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[2603049, 51626, 100, 1, 69, "Subsection"],
Cell[2603152, 51629, 102, 1, 33514, "Text"],
Cell[2603257, 51632, 4330, 115, 493149, "Input",
 Evaluatable->False],
Cell[2607590, 51749, 532, 14, 33514, "Input"],
Cell[2608125, 51765, 104, 1, 33514, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2608278, 51772, 93, 1, 153, "Section"],
Cell[CellGroupData[{
Cell[2608396, 51777, 100, 1, 86, "Subsection"],
Cell[2608499, 51780, 248, 6, 109, "Text",
 CellTags->"Gray1"],
Cell[2608750, 51788, 558, 10, 142, "Text",
 CellTags->"Gray2"],
Cell[2609311, 51800, 633, 13, 143, "Text",
 CellTags->"Gray3"],
Cell[2609947, 51815, 995, 23, 75, "Text",
 CellTags->"Gray4"],
Cell[2610945, 51840, 480, 11, 109, "Text",
 CellTags->"Gray5"],
Cell[2611428, 51853, 536, 14, 75, "Text",
 CellTags->"MG"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2612001, 51872, 126, 2, 108, "Subsection",
 CellTags->"Obso"],
Cell[2612130, 51876, 590, 13, 89, "Text"],
Cell[CellGroupData[{
Cell[2612745, 51893, 172, 3, 88, "Input"],
Cell[2612920, 51898, 450, 10, 38, "Message"],
Cell[2613373, 51910, 487, 10, 38, "Message"],
Cell[2613863, 51922, 102, 1, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2614002, 51928, 179, 4, 88, "Input"],
Cell[2614184, 51934, 454, 10, 38, "Message"],
Cell[2614641, 51946, 493, 10, 38, "Message"],
Cell[2615137, 51958, 78, 1, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2615252, 51964, 161, 2, 53, "Input"],
Cell[2615416, 51968, 458, 10, 38, "Message"],
Cell[2615877, 51980, 495, 10, 38, "Message"],
Cell[2616375, 51992, 76, 1, 51, "Output"]
}, Open  ]],
Cell[2616466, 51996, 68, 1, 53, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[2616571, 52002, 94, 1, 69, "Subsection"],
Cell[2616668, 52005, 339, 7, 3193, "Input",
 InitializationCell->True],
Cell[2617010, 52014, 239, 6, 3193, "Input",
 InitializationCell->True]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
